<?php

/**
 * BillingDefinition controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class BillingDefinitionController extends ApplicationController {

	var $plugin_name = "advanced_billing";
	
	/**
	 * Construct the BillingDefinitionController
	 *
	 * @access public
	 * @param void
	 * @return BillingDefinitionController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
		if (Plugins::instance()->isActivePlugin('multiple_currencies')) {
			Env::useHelper('functions', 'multiple_currencies');
		}
	//	Env::useHelper('functions', $this->plugin_name);
	} // __construct
	
	// ---------------------------------------------------
	//  Index
	// ---------------------------------------------------
	
	function init() {
		require_javascript("AdvancedBillingManager.js", $this->plugin_name);
		ajx_current("panel", "advanced-billing-containerpanel", null, null, true);
		ajx_replace(true);
	}
	
	function get_allowed_dimensions() {
		ajx_current("empty");
		ajx_extra_data(array('allowed_dimensions' => array_filter(config_option('adv_billing_dimensions'))));
	}
	
	function enable_disable_dimension() {
		ajx_current("empty");
		
		if (!can_manage_billing(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$dimension = Dimensions::getDimensionById(array_var($_GET, 'dim_id'));
		if (!$dimension instanceof Dimension) {
			flash_error(lang('dimension dnx'));
			ajx_current('empty');
			return;
		}
		
		$enable = array_var($_GET, 'enable');
		$allowed_dimensions = array_filter(config_option('adv_billing_dimensions'));
		
		if ($enable) {
			if (!in_array($dimension->getId(), $allowed_dimensions)) {
				$allowed_dimensions[] = $dimension->getId();
				set_config_option('adv_billing_dimensions', $allowed_dimensions);
			}
		} else {
			if (in_array($dimension->getId(), $allowed_dimensions)) {
				foreach ($allowed_dimensions as $k => &$d) {
					if ($d == $dimension->getId()) unset($allowed_dimensions[$k]);
				}
				set_config_option('adv_billing_dimensions', $allowed_dimensions);
			}
		}
		
	}
	
	function list_all() {
		ajx_current("empty");
		
		// Get all variables from request
		$start = array_var($_GET, 'start', 0);
		$limit = array_var($_GET, 'limit', config_option('files_per_page'));
		$order = array_var($_GET, 'sort');
		$order_dir = strtoupper(array_var($_GET, 'dir'));
		
		if (!is_numeric($start)) $start = 0;
		if (!is_numeric($limit)) $limit = config_option('files_per_page');
		
		$action = array_var($_GET, 'action');
		$attributes = array(
			"ids" => explode(',', array_var($_GET, 'ids')),
		);
		
		//Resolve actions to perform
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		$context_conditions_array = array();
		$context = active_context();
		foreach ($context as $selection) {
			if ($selection instanceof Member) {
				$all_mem_ids = array($selection->getId());
				$all_mem_ids = array_merge($all_mem_ids, $selection->getAllChildrenIds(true));
				$context_conditions_array[] = "EXISTS (SELECT bdm.member_id FROM ".TABLE_PREFIX."billing_definition_members bdm 
						WHERE bdm.member_id IN (".implode(',',$all_mem_ids).") AND bdm.dimension_id='".$selection->getDimensionId()."' AND bdm.definition_id=d.id)";
			}
		}
		$context_conditions_sql = "";
		if (count($context_conditions_array) > 0) {
			$context_conditions_sql = " AND ".implode(" AND ", $context_conditions_array);
		}


		$joins = array();
		$select_cols = array();
		
		
		if (!in_array($order_dir, array('ASC', 'DESC'))) {
			$order_dir = 'ASC';
		}
		
		switch ($order) {
			case 'hourly_rate':
				$order = "d.enable_rate $order_dir, d.$order";
				break;
			case 'hourly_cost':
				$order = "d.enable_cost $order_dir, d.$order";
				break;
			case 'name':
			case 'updated_on':
			case 'updated_by_id':
				$order = "d.$order";
				break;
			case 'user_id':
				$order = "user_name";
				break;
			default:
				if (str_starts_with($order, 'dim_')) {
					$joins[] = " LEFT JOIN ".TABLE_PREFIX."members mem ON dm".$did.".member_id=mem.id";
					$select_cols[] = "mem.name as order_member_name";
					$order = "order_member_name";
				}
				if (in_array($order, BillingDefinitions::instance()->getColumns())) {
					$order = 'd.'.$order;
				} else {
					$order = 'd.name';
				}
		}
		
		
		// join for category user
		$joins[] = " LEFT JOIN ".TABLE_PREFIX."objects o ON o.id=d.user_id ";
		$select_cols[] = "o.name as user_name";
		
		$select_cols_sql = (count($select_cols) > 0 ? ",".implode(',',$select_cols) : "");
		$joins_sql = (count($joins) > 0 ? implode(' ', $joins) : "");
		$order_sql = "ORDER BY $order $order_dir";
		$limit_sql = "LIMIT $start, $limit";
		
		$conditions_sql = "true $context_conditions_sql";
		
		$sql = "SELECT d.* $select_cols_sql FROM ".TABLE_PREFIX."billing_definitions d $joins_sql WHERE $conditions_sql $order_sql $limit_sql";
		
		$billing_definitions = DB::executeAll($sql);
		
		$count_sql = "SELECT count(d.id) as total FROM ".TABLE_PREFIX."billing_definitions d $joins_sql WHERE $conditions_sql";
		$total_rows = DB::executeAll($count_sql);
		
		// Prepare response object
		$object = $this->prepareObject($billing_definitions, $start, $limit, $total_rows[0]['total']);
		ajx_extra_data($object);
		tpl_assign("listing", $object);
		
	}
	

	private function prepareObject($categories, $start, $limit, $total) {
		$object = array(
			"totalCount" => $total,
			"start" => $start,
			"billing_definitions" => array(),
			"allowed_dimensions" => array_filter(config_option('adv_billing_dimensions')),
		);
		
		for ($i = 0; $i < $limit; $i++){
			$billing_definition_data = array_var($categories, $i);
			
			if (!is_null($billing_definition_data)) {
				
				$def_mem_sql = "SELECT member_id FROM ".TABLE_PREFIX."billing_definition_members WHERE definition_id=".$billing_definition_data['id'];
				$def_member_ids = array_unique(array_filter(array_flat(DB::executeAll($def_mem_sql))));
				
				$mem_path = array();
				foreach ($def_member_ids as $member_id) {
					$mem = Members::getMemberById($member_id);
					if (!$mem instanceof Member) continue;
					
					$member_type_id = $mem->getObjectTypeId();
					$dim = $mem->getDimensionId();
					if (!isset($mem_path[$dim])) $mem_path[$dim] = array();
					if (!isset($mem_path[$dim][$member_type_id])) $mem_path[$dim][$member_type_id] = array();
					$mem_path[$dim][$member_type_id][] = $member_id;
				}
				$billing_definition_data['memPath'] = json_encode($mem_path);

				$rate_currency = Currencies::instance()->findById($billing_definition_data['rate_currency_id']);
				$rate_c_symbol = $rate_currency instanceof Currency ? $rate_currency->getSymbol() : "";
				
				$cost_currency = Currencies::instance()->findById($billing_definition_data['cost_currency_id']);
				$cost_c_symbol = $cost_currency instanceof Currency ? $cost_currency->getSymbol() : "";
				
				$billing_definition_data['user_name'] = escape_character($billing_definition_data['user_name']);
				if ($billing_definition_data['enable_rate']) {
					$billing_definition_data['hourly_rate'] = $rate_c_symbol .' '. number_format($billing_definition_data['hourly_rate'], 2);
				} else {
					$billing_definition_data['hourly_rate'] = '';
				}
				if ($billing_definition_data['enable_cost']) {
					$billing_definition_data['hourly_cost'] = $cost_c_symbol .' '. number_format($billing_definition_data['hourly_cost'], 2);
				} else {
					$billing_definition_data['hourly_cost'] = '';
				}
				
				$updated_on = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $billing_definition_data['updated_on']);
				$billing_definition_data['updated_on'] = format_datetime($updated_on);
				
				$billing_definition_data['updated_by_name'] = lang('n/a');
				$updated_by = Contacts::instance()->findById($billing_definition_data['updated_by_id']);
				if ($updated_by instanceof Contact) {
					$billing_definition_data['updated_by_name'] = clean($updated_by->getObjectName());
				}
				
				$object["billing_definition"][$i] = $billing_definition_data;
				
			}
		}
		
		return $object;
	}
	
	
	/**
	 * Resolve action to perform
	 *
	 * @param string $action
	 * @param array $attributes
	 */
	private function resolveAction($action, $attributes){
		 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$billing_definition = BillingDefinitions::instance()->findById($id);
					if (can_manage_billing(logged_user()) && $billing_definition instanceof BillingDefinition){
						try{
							DB::beginWork();
							
							BillingDefinitionMembers::instance()->delete("definition_id=".$billing_definition->getId());
							$billing_definition->delete();
							
							DB::commit();
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) . ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;
		}
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
		
	}
	
	
	

	/**
	 * View single category
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function view() {
		$this->addHelper('textile');

		$object = BillingDefinitions::instance()->findById(get_id());
		if(!($object instanceof BillingDefinition)) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		} // if

		if(!can_manage_billing(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		} // if
		
		tpl_assign('object', $object);
		
		ajx_extra_data(array("title" => $object->getName()));
		ajx_set_no_toolbar(true);
		
	} // view
	

	/**
	 * Add billing category definition
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function add() {
		
		if (!can_manage_billing(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$billing_definition = new BillingDefinition();
		tpl_assign('billing_definition', $billing_definition);

		$data = array_var($_POST, 'billing_definition');
		if(!is_array($data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			tpl_assign('selected_members', array());
			
		} else {
			foreach ($data as $k => &$v) {
				$v = remove_scripts($v);
			}
			try {
				DB::beginWork();
				
				$this->save_billing_definition($data, $billing_definition);
				
				DB::commit();
				
				flash_success(lang('success add billing definition', $billing_definition->getName()));
				
	          	if (array_var($_REQUEST, 'modal')) {
	          		evt_add("reload current panel");
          		} else {
          			ajx_current("back");
          		}
          		
	          	ajx_add("overview-panel", "reload");

			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
				
			} // try

		} // if
	} // add

	/**
	 * Edit billing category definition
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function edit() {
		$this->setTemplate('add');
		
		$billing_definition = BillingDefinitions::instance()->findById(get_id());
		if (!$billing_definition instanceof BillingDefinition) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		if (!can_manage_billing(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		tpl_assign('billing_definition', $billing_definition);
		
		$data = array_var($_POST, 'billing_definition');
		if(!is_array($data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			$selected_members = array_flat(DB::executeAll("SELECT member_id FROM ".TABLE_PREFIX."billing_definition_members 
					WHERE definition_id=".$billing_definition->getId()));
			tpl_assign('selected_members', $selected_members);
			
		} else {
			foreach ($data as $k => &$v) {
				$v = remove_scripts($v);
			}
			try {
				DB::beginWork();
				
				$this->save_billing_definition($data, $billing_definition);
				
				DB::commit();
				
				flash_success(lang('success edit billing definition', $billing_definition->getName()));
				
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				} else {
					ajx_current("back");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
			} // try
		} // if
	} // edit
	
	
	private function save_billing_definition($data, $billing_definition) {
		
		$billing_definition->setFromAttributes($data);
		$billing_definition->save();
		
		$values_sql = array();
		foreach ($data as $key => $member_data) {
			
			if (str_starts_with($key, 'dim_')) {
				
				$member_id = array_var($member_data, 'member_id', 0);
				if (!is_numeric($member_id)) {
					if (is_string($member_id)) $member_id = json_decode($member_id);
				} else {
					$member_id = array($member_id);
				}
				
				$dim_id = substr($key, 4);
				if (is_numeric($dim_id)) {
					$member_id = array_filter(array_unique($member_id));
					foreach ($member_id as $mid) {
						$values_sql[] = "('".$billing_definition->getId()."','$dim_id','$mid')";
					}
				}
			}
		}
		DB::execute("DELETE FROM ".TABLE_PREFIX."billing_definition_members WHERE definition_id=".$billing_definition->getId());
		if (count($values_sql) > 0) {
			DB::execute("INSERT INTO ".TABLE_PREFIX."billing_definition_members (definition_id, dimension_id, member_id) 
					VALUES ".implode(',', $values_sql)." ON DUPLICATE KEY UPDATE definition_id=definition_id;");
		}
		
	}

	/**
	 * Delete specific analysis
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function delete() {
		
		$billing_definition = BillingDefinitions::instance()->findById(get_id());
		if (!$billing_definition instanceof BillingDefinition) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		if (!can_manage_billing(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		try {
			DB::beginWork();
			
			$name = $billing_definition->getName();
			BillingDefinitionMembers::instance()->delete("definition_id=".$billing_definition->getId());
			$billing_definition->delete();
		
			DB::commit();
		
			flash_success(lang('success delete billing definition', $name));
		
			if (array_var($_REQUEST, 'modal')) {
				evt_add("reload current panel");
			} else {
				ajx_current("back");
			}
		
		} catch(Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			ajx_current("empty");
		}
	} // delete

	
	
	function recalculate_timeslots_billing() {
		ajx_current("empty");
		if (!can_manage_billing(logged_user())) {
			flash_error(lang('no access permissions'));
			return;
		}
		
		$timeslots = get_timeslots_from_grid_parameters();
		
		try {
			$objects_to_log = array();

			DB::beginWork();
			
			$count = 0;
			foreach ($timeslots as $ts) {
				if ($ts instanceof Timeslot && $ts->getColumnValue('invoicing_status') != 'invoiced') {
					// to use when saving the application log
					$old_content_object = $ts->generateOldContentObjectData();

					$ts->setForceRecalculateBilling(true);
					$ts->save();
					$count++;

					$objects_to_log[] = $ts;
				}
			}
			
			DB::commit();
			
			flash_success(lang('success recalculating billing for timeslots', $count));
			ajx_current("reload");

			foreach ($objects_to_log as $timeslot) {
				ApplicationLogs::createLog($timeslot, ApplicationLogs::ACTION_EDIT, false, true);
			}
			
		} catch (Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			ajx_current("empty");
		}
	}

	function get_labor_category_billable_for_task_form(){
		$member_ids = (array) json_decode(array_var($_REQUEST, 'member_ids', '{}'));
		$current_member_ids = (array) json_decode(array_var($_REQUEST, 'current_member_ids', '{}'));
		$current_billable = array_var($_REQUEST, 'current_billable', 1);
		$result = array('has_value' => false);
		if(Plugins::instance()->isActivePlugin('hour_types')){
			$hour_type_dim_id = Dimensions::findByCode('hour_types')->getId();
			$hour_type_ot_id = ObjectTypes::findByName('hour_type')->getId();
			$labor_category_id = array_var($member_ids, $hour_type_dim_id, array(0))[0];
			$current_labor_category_array = array_var($current_member_ids, $hour_type_dim_id, array(0));
			$current_labor_category_id = count($current_labor_category_array) == 2 ? $current_labor_category_array[1] : $current_labor_category_array[0];
			if($labor_category_id > 0 && $labor_category_id != $current_labor_category_id){
				$member_cp = MemberCustomProperties::instance()->findOne(array('conditions' => '`object_type_id`='.$hour_type_ot_id.' AND `code`="is_billable"'));
				if($member_cp instanceof MemberCustomProperty){
					$cp_id = $member_cp->getId();
					$hour_type_cpv = MemberCustomPropertyValues::instance()->findOne(array('conditions' => '`member_id`='.$labor_category_id.' AND `custom_property_id`='.$cp_id));
					if($hour_type_cpv instanceof MemberCustomPropertyValue){
						$is_billable = $hour_type_cpv->getValue() == '-1' ? 0 : 1;
						$result = array('has_value' => true, 'is_billable' => $is_billable);
					} else {
						// Default value is 1
						$is_billable = 1;
						$result = array('has_value' => true, 'is_billable' => $is_billable);
					}
				}
			}
		}
		ajx_current("empty");
		ajx_extra_data($result);
	}


	function get_task_billable_for_time_form() {
		$task_id = array_var($_REQUEST, 'task_id', 0);
		$current_billable = array_var($_REQUEST, 'current_billable', 1);
		$result = array('has_value' => false);
		if(Plugins::instance()->isActivePlugin('hour_types')){
			$task = ProjectTasks::instance()->findById($task_id);
			if($task instanceof ProjectTask){
				$task_billable_value = $task->getColumnValue('is_billable');
				$result = array('has_value' => true, 'is_billable' => $task_billable_value);
				if($current_billable == 0 && $task_billable_value == 1){
					flash_success(lang('billable value is changed to yes'));
				} else if($current_billable == 1 && $task_billable_value == 0){
					flash_success(lang('billable value is changed to no'));
				}
			}
		}
		ajx_current("empty");
		ajx_extra_data($result);
	}

	function get_labor_cat_billable_for_time_form() {
		$member_ids = explode(',', array_var($_REQUEST, 'member_ids', ''));
		$current_member_ids = explode(',', array_var($_REQUEST, 'current_member_ids', ''));
		$current_billable = array_var($_REQUEST, 'current_billable', 1);
		$result = array('has_value' => false);
		if(Plugins::instance()->isActivePlugin('hour_types')){
			$hour_type_dim_id = Dimensions::findByCode('hour_types')->getId();
			$hour_type_ot_id = ObjectTypes::findByName('hour_type')->getId();
			$labor_cat_member_id = 0;
			foreach($member_ids as $member_id){
				$member = Members::instance()->findById($member_id);
				if($member instanceof Member && $member->getObjectTypeId() != $hour_type_ot_id) continue;
				$labor_cat_member_id = $member_id;
			}
			$current_labor_cat_member_id = 0;
			foreach($current_member_ids as $member_id){
				$member = Members::instance()->findById($member_id);
				if($member instanceof Member && $member->getObjectTypeId() != $hour_type_ot_id) continue;
				$current_labor_cat_member_id = $member_id;
			}
			if($labor_cat_member_id > 0 && $labor_cat_member_id != $current_labor_cat_member_id){
				$member_cp = MemberCustomProperties::instance()->findOne(array('conditions' => '`object_type_id`='.$hour_type_ot_id.' AND `code`="is_billable"'));
				if($member_cp instanceof MemberCustomProperty){
					$cp_id = $member_cp->getId();
					$hour_type_cpv = MemberCustomPropertyValues::instance()->findOne(array('conditions' => '`member_id`='.$labor_cat_member_id.' AND `custom_property_id`='.$cp_id));
					if($hour_type_cpv instanceof MemberCustomPropertyValue){
						$is_billable = $hour_type_cpv->getValue() == '-1' ? 0 : 1;
						$result = array('has_value' => true, 'is_billable' => $is_billable);
					} else {
						// Default value is 1
						$is_billable = 1;
						$result = array('has_value' => true, 'is_billable' => $is_billable);
					}
				}
			}	
		}
		ajx_current("empty");
		ajx_extra_data($result);
	}

	function has_quoted_price_in_parent_hierarchy() {
		ajx_current("empty");
		$parent_task_id = array_var($_REQUEST, 'parent_task_id', 0);

		$parent_task = ProjectTasks::instance()->findById($parent_task_id, true);
		if(!$parent_task instanceof ProjectTask) return;

		$manual_estimated_price = $parent_task->getColumnValue('is_calculated_estimated_price') == 0;
		Env::useHelper('additional_tasks_columns_functions', 'advanced_billing');
        $has_parent_with_quoted_price = has_parent_with_quoted_price($parent_task);
		$has_quoted_price = $manual_estimated_price || $has_parent_with_quoted_price;

        ajx_extra_data(array('has_quoted_price_parent' => $has_quoted_price));
	}

	function get_billable_value_of_parent_task() {
		ajx_current("empty");
        
		$parent_task_id = array_var($_REQUEST, 'parent_task_id', 0);

		$parent_task = ProjectTasks::instance()->findById($parent_task_id, true);
		if(!$parent_task instanceof ProjectTask) return;

		Env::useHelper('additional_tasks_columns_functions', 'advanced_billing');
		$has_fixed_fee_parent = has_parent_with_fixed_fee(null, $parent_task_id);

		$parent_is_billable = $parent_task->getColumnValue('is_billable') == 1;

        ajx_extra_data(array('parent_is_billable' => $parent_is_billable, 'has_fixed_fee_parent' => $has_fixed_fee_parent));
	}

} // BillingDefinitionController

?>