<?php

function add_tasks_show_estimated_cost_column($task, &$info){ 
    if (!isset($info['additional_data'])) $info['additional_data'] = array();
    if (user_config_option('tasksShowEstimatedCost')) {

        $estimated_cost = array_var($task, 'estimated_cost', 0);

        // Get currency symbol
        $currency_symbol = get_currency_symbol(null);

        // Format estimated cost
        $formatted_estimated_cost = format_money_amount($estimated_cost, $currency_symbol);

        // Build HTML with estimated cost
        $html = build_html_for_task($formatted_estimated_cost);

        // Add HTML to the task
        $info['additional_data']['tasksShowEstimatedCost'] = array(
            "html" => $html,
            "cls" => "",
        );
    } else {
        $info['additional_data']['tasksShowEstimatedCost'] = array("html" => "");
    }

}

function add_tasks_show_estimated_price_column($task, &$info){
    if (!isset($info['additional_data'])) $info['additional_data'] = array();
    if (user_config_option('tasksShowEstimatedPrice')) {

        $estimated_price = array_var($task, 'estimated_price', 0);

        // Get currency symbol
        $currency_symbol = get_currency_symbol(null);

        // Format estimated price
        $formatted_estimated_price = format_money_amount($estimated_price, $currency_symbol);

        // Build HTML with executed cost
        $html = build_html_for_task($formatted_estimated_price);

        // Add HTML to the task
        $info['additional_data']['tasksShowEstimatedPrice'] = array(
            "html" => $html,
            "cls" => "",
        );
    } else {
        $info['additional_data']['tasksShowEstimatedPrice'] = array("html" => "");
    }

}

function add_tasks_show_executed_cost_column($task, &$info){ 
    if (!isset($info['additional_data'])) $info['additional_data'] = array();
    if (user_config_option('tasksShowExecutedCost')) {
        $executed_cost = array_var($task, 'executed_cost', 0);
        $estimated_cost = array_var($task, 'estimated_cost', 0);
        $overvalued = $executed_cost > $estimated_cost;
        // Get currency symbol
        $currency_symbol = get_currency_symbol(null);

        // Format executed cost
        $formatted_executed_cost = format_money_amount($executed_cost, $currency_symbol);

        // Build HTML with executed cost
        $html = build_html_for_task($formatted_executed_cost, $overvalued);

        // Add HTML to the task
        $info['additional_data']['tasksShowExecutedCost'] = array(
            "html" => $html,
            "cls" => "",
        );
    } else {
        $info['additional_data']['tasksShowExecutedCost'] = array("html" => "");
    }

}

function add_tasks_show_executed_price_column($task, &$info){
    if (!isset($info['additional_data'])) $info['additional_data'] = array();
    if (user_config_option('tasksShowExecutedPrice')) {
        $executed_price = array_var($task, 'earned_value', 0);
        // Get currency symbol
        $currency_symbol = get_currency_symbol(null);
        // Format executed price
        $formatted_executed_price = format_money_amount($executed_price, $currency_symbol);
        // Build HTML with executed price
        $html = build_html_for_task($formatted_executed_price);
        // Add HTML to the task
        $info['additional_data']['tasksShowExecutedPrice'] = array(
            "html" => $html,
            "cls" => "",
        );
    } else {
        $info['additional_data']['tasksShowExecutedPrice'] = array("html" => "");
    }

}

function add_tasks_is_billable_column($task, &$info){
    if (!isset($info['additional_data'])) $info['additional_data'] = array();
    if (user_config_option('tasksShowIsBillable')) {
        $is_billable = array_var($task, 'is_billable', 1);
        $formatted_is_billable = $is_billable == 1 ? lang('yes') : lang('no');
        // Build HTML with is_billable
        $billable_html = '<td class="additional-list-col">'. $formatted_is_billable . '</td>';
        // Add HTML to the task
        $info['additional_data']['tasksShowIsBillable'] = array(
            "html" => $billable_html,
            "cls" => "",
        );
    } else {
        $info['additional_data']['tasksShowIsBillable'] = array("html" => "");
    }
}

/*
* HELPER FUNCTIONS
*/

function get_currency_symbol($billing_definition) {
    $currency_id = $billing_definition instanceof BillingDefinition ? $billing_definition->getRateCurrencyId() : null;
    if($currency_id){
        $currency = Currencies::instance()->getCurrency($currency_id);
        $currency_symbol = $currency->getSymbol();
    } else {
        $currency = Currencies::instance()->getDefaultCurrencyInfo();
        $currency_symbol = $currency['symbol'];
    }

    return $currency_symbol;
}

/*function get_task_dimension_members($info){
    foreach($info['members_data'] as $member_data){
        if($member_data['object_type_name']=='hour_type'){
            $task_labor_id = $member_data['id'];
            $task_hour_type_dimension_id = $member_data['dimension_id'];
        }
        if($member_data['object_type_name']=='rate_schedule'){
            $task_rate_schedule_id = $member_data['id'];
            $task_rate_schedule_dimension_id = $member_data['dimension_id'];
        }
    }
    $dim_members = array(
        $task_rate_schedule_dimension_id => array($task_rate_schedule_id),
        $task_hour_type_dimension_id => array($task_labor_id),
    );

    return $dim_members;
}*/

function get_task_dimension_members($allowed_dimensions, $task_id){
    $dim_members = array();
    $sql = "SELECT m.dimension_id, om.member_id FROM ".TABLE_PREFIX."object_members om
    INNER JOIN ".TABLE_PREFIX."members m ON om.member_id = m.id
    WHERE om.is_optimization=0 AND dimension_id IN (".implode(',', $allowed_dimensions).") AND om.object_id = '$task_id'";

    $dim_member_rows = DB::executeAll($sql);
    if (is_array($dim_member_rows)) {
        foreach ($dim_member_rows as $dmr) {
            if (!isset($dim_members[$dmr['dimension_id']])) $dim_members[$dmr['dimension_id']] = array();
            $dim_members[$dmr['dimension_id']][] = $dmr['member_id'];
        }
    }

    return $dim_members;
}

function get_tasks_timeslots($task_id){
    $extra_conditions = " AND rel_object_id = " . $task_id;
    $timeslots_res = Timeslots::instance()->listing(array(
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "check_sharing_table" => false,
        "extra_conditions" => $extra_conditions 
    ));
    $timeslots = $timeslots_res->objects;
	if (is_null($timeslots)) $timeslots = array();

    return $timeslots;
}

function build_html_for_task($formatted_amount, $overvalued = false){
    $red_text = $overvalued ? 'color: #f00;' : '';
    $html = '<td>';
    $html .= '<div style="font-size: 12px; text-align: right;'.$red_text.'">' . $formatted_amount . '</div>';
    $html .= '</td>';

    return $html;
}


function getTaskEstimatedExecutedValues($task_id){ 
    $result = array(
        'executed_cost' => 0,
        'earned_value' => 0,
        'estimated_cost' => 0,
        'estimated_price' => 0
    );
    $task = ProjectTasks::instance()->findById($task_id);
    if($task instanceof ProjectTask && Plugins::instance()->isActivePlugin('advanced_billing')){
        Env::useHelper('functions', 'advanced_billing');
        $task_children_financials = get_task_children_financials_info($task);

        // Calculate executed_cost and earnded_value
        $timeslots = get_tasks_timeslots($task_id);
        $executed_cost = 0;
        $earned_value = 0;
        if($timeslots){
            foreach($timeslots as $time){
                $executed_cost += $time['fixed_cost'];
                $earned_value += $time['fixed_billing'];
            }
        }
        $result['executed_cost'] = $executed_cost;
        $result['earned_value'] = $earned_value;

        // Assign values to variables to get billing definition
        $allowed_dimensions = array_filter(config_option('adv_billing_dimensions'));
        $cant_decide_between_billing_definitions = null;
        $dim_members = get_task_dimension_members($allowed_dimensions, $task_id);
        $assigned_to_contact = $task->getAssignedToContactId();
        $estimated_time = $task->getTimeEstimate();

        // Calculate estimated cost
        $cost_billing_definition = find_billing_def_by_user_and_members($assigned_to_contact, $dim_members, $allowed_dimensions, 'cost', $cant_decide_between_billing_definitions); 
        $hourly_cost = $cost_billing_definition instanceof BillingDefinition ? $cost_billing_definition->getHourlyCost() : 0;
        $estimated_cost = $estimated_time > 0 ? ($estimated_time / 60) * $hourly_cost : 0;

        // Calculate estimated price
        $price_billing_definition = find_billing_def_by_user_and_members($assigned_to_contact, $dim_members, $allowed_dimensions, 'billing', $cant_decide_between_billing_definitions); 
        $hourly_price = $price_billing_definition instanceof BillingDefinition ? $price_billing_definition->getHourlyRate() : 0;
        $estimated_price = $estimated_time > 0 ? ($estimated_time / 60) * $hourly_price : 0;

        $result['estimated_cost'] = $estimated_cost;
        $result['estimated_price'] = $estimated_price;

        // Add children financials
        foreach($task_children_financials as $k => $v){
            $result[$k] += $v;
        }

    }
    return $result;
}

function has_parent_with_quoted_price($task, $parent_id = null) {
    if(($task instanceof ProjectTask && $task->getParentId() == 0) && ($parent_id == null || $parent_id == 0)) {
        return false;
    }

    if($task instanceof ProjectTask && $task->getParentId() != 0) {
        $parents = $task->getAllParents();
    } else {
        $parents = array(ProjectTasks::instance()->findById($parent_id));
        $more_parents = $parents[0]->getAllParents();
        $parents = array_merge($parents, $more_parents);
    }
    foreach($parents as $parent){
        $is_calculated_estimated_price = $parent->getColumnValue('is_calculated_estimated_price');
        if($is_calculated_estimated_price == 0) return true;
    }

    return false;
}

function has_parent_with_fixed_fee($task, $parent_id = null) {
    if(($task instanceof ProjectTask && $task->getParentId() == 0) && ($parent_id == null || $parent_id == 0)) {
        return false;
    }

    if($task instanceof ProjectTask && $task->getParentId() != 0) {
        $parents = $task->getAllParents();
    } else {
        $parents = array(ProjectTasks::instance()->findById($parent_id));
        $more_parents = $parents[0]->getAllParents();
        $parents = array_merge($parents, $more_parents);
    }
    foreach($parents as $parent){
        $is_fixed_fee = $parent->getColumnValue('is_fixed_fee');
        if($is_fixed_fee == 1) return true;
    }

    return false;
}

function has_subtasks_with_est_price($task) {
    if(!$task instanceof ProjectTask) return false;

    $subtasks = $task->getAllSubtasks(false);
    foreach($subtasks as $subtask){
        $estimated_price = $subtask->getColumnValue('estimated_price');
        if($estimated_price > 0) return true;
    }

    return false;
}

function update_estimated_price_for_subtasks_with_quoted_parent_task($task) {
    if(!$task instanceof ProjectTask) return;

    $all_subtasks = $task->getAllSubtasks(false);
    $update_subtask_ids = array();
    $recalc_subtasks = array();
    // Filter subtasks that have correct values for estimated_price, is_calculated_estimated_price and is_fixed_fee
    foreach($all_subtasks as $subtask) {
        $estimated_price = $subtask->getColumnValue('estimated_price');
        $is_calculated_estimated_price = $subtask->getColumnValue('is_calculated_estimated_price');
        $is_fixed_fee = $subtask->getColumnValue('is_fixed_fee');
        if($estimated_price == 0 && $is_calculated_estimated_price == 0 && $is_fixed_fee == 0) continue;
        $update_subtask_ids[] = $subtask->getId();
        if($is_fixed_fee == 1) $recalc_subtasks[] = $subtask;

    }

    if(count($update_subtask_ids) == 0) return;

    // Set estimated_price, is_calculated_estimated_price and is_fixed_fee to 0 for subtasks that have quoted parent task
    $subtask_ids = implode(',',  $update_subtask_ids);
    $set_sql = "SET `estimated_price`=0";
    $set_sql .= ", `is_calculated_estimated_price`=0"; 
    $set_sql .= ", `is_fixed_fee`=0";
    DB::execute("
        UPDATE ".TABLE_PREFIX."project_tasks 
        $set_sql
        WHERE object_id IN (".$subtask_ids.")"
    );

    if(count($recalc_subtasks) == 0) return;
    // recalculate executed values for subtasks that had fixed fee before change
    Env::useHelper('functions', 'advanced_billing');
    foreach($recalc_subtasks as $subtask) {
        if($subtask instanceof ProjectTask) {
            calculate_task_executed_cost_and_price($subtask);
        }
    }

}

function set_task_to_non_billable_and_update_related_objects($task) {
    if(!$task instanceof ProjectTask) return;

    set_task_to_non_billable_via_db_query($task);
    set_related_time_and_expenses_to_task_to_non_billable($task);
    set_subtasks_related_to_task_to_non_billable($task);
}

function set_related_time_and_expenses_to_task_to_non_billable($task) {
    if(!$task instanceof ProjectTask) return;

    // Update related objects to non-billable
    if(Plugins::instance()->isActivePlugin('income')) {
        set_timeslots_related_to_task_to_non_billable($task);
    }
    if(Plugins::instance()->isActivePlugin('expenses2') && Plugins::instance()->isActivePlugin('income')) {
        set_actual_expenses_related_to_task_to_non_billable($task);
        set_budgeted_expenses_related_to_task_to_non_billable($task);
    }
}

function set_task_to_non_billable_via_db_query($task) {
    DB::execute("
        UPDATE ".TABLE_PREFIX."project_tasks 
        SET `is_billable`=0, 
            `estimated_price`=0, 
            `earned_value`=0,
            `is_calculated_estimated_price`=0, 
            `invoicing_status`='non_billable'
        WHERE `object_id` = ".$task->getObjectId().";"
    );
}

function set_timeslots_related_to_task_to_non_billable($task) {
    DB::execute("
        UPDATE ".TABLE_PREFIX."timeslots 
        SET `invoicing_status`='non_billable', `fixed_billing`=0, `hourly_billing`=0
        WHERE `rel_object_id` = ".$task->getObjectId().";"
    );
}

function set_actual_expenses_related_to_task_to_non_billable($task) {
    if(!Plugins::instance()->isActivePlugin('expenses2')) return;
    DB::execute("
        UPDATE ".TABLE_PREFIX."payment_receipts 
        SET `invoicing_status`='non_billable', 
            `is_billable`=0, 
            `unit_price`=0, 
            `total_price`=0
        WHERE `task_id` = ".$task->getObjectId().";"
    );
}

function set_budgeted_expenses_related_to_task_to_non_billable($task) {
    if(!Plugins::instance()->isActivePlugin('expenses2')) return;
    DB::execute("
        UPDATE 
        ".TABLE_PREFIX."expenses exp, 
        ".TABLE_PREFIX."expense_totals exp_t, 
        ".TABLE_PREFIX."expense_items exp_items
        SET 
            exp.invoicing_status = 'non_billable', 
            exp.is_billable = 0, 
            exp_items.unit_price = 0, 
            exp_items.total_price = 0,
            exp_t.unit_price = 0,
            exp_t.total_price = 0
        WHERE 
            exp.task_id = '".$task->getObjectId()."' AND 
            exp.object_id = exp_items.expense_id AND 
            exp.object_id = exp_t.expense_id;"
    );
}

function set_subtasks_related_to_task_to_non_billable($task) {
    $subtasks = $task->getSubtasks(false, false, true);
    foreach($subtasks as $subtask) {
        if($subtask instanceof ProjectTask) {
            set_task_to_non_billable_and_update_related_objects($subtask);
        }
    }
}

function set_related_objects_to_non_billable_using_task_ids($task_ids) {
    if(!is_array($task_ids)) return;

    set_all_task_ids_to_non_billable_via_db_query($task_ids);
    set_all_timeslots_related_to_task_ids_to_non_billable($task_ids);
    set_all_actual_expenses_related_to_task_ids_to_non_billable($task_ids);
    set_all_budgeted_expenses_related_to_task_ids_to_non_billable($task_ids);
}

function set_all_task_ids_to_non_billable_via_db_query($task_ids) {
    if(!is_array($task_ids)) return;
    
    $task_ids = implode(',', $task_ids);
    DB::execute("
        UPDATE ".TABLE_PREFIX."project_tasks 
        SET `is_billable`=0, 
            `estimated_price`=0, 
            `earned_value`=0,
            `is_calculated_estimated_price`=0, 
            `invoicing_status`='non_billable'
        WHERE `object_id` IN (".$task_ids.");"
    );
}

function set_all_timeslots_related_to_task_ids_to_non_billable($task_ids) {
    if(!is_array($task_ids)) return;

    $task_ids = implode(',', $task_ids);
    DB::execute("
        UPDATE ".TABLE_PREFIX."timeslots 
        SET `invoicing_status`='non_billable', `fixed_billing`=0, `hourly_billing`=0
        WHERE `rel_object_id` IN (".$task_ids.");"
    );
}

function set_all_actual_expenses_related_to_task_ids_to_non_billable($task_ids) {
    if(!is_array($task_ids)) return;
    if(!Plugins::instance()->isActivePlugin('expenses2')) return;

    $task_ids = implode(',', $task_ids);
    DB::execute("
        UPDATE ".TABLE_PREFIX."payment_receipts 
        SET `invoicing_status`='non_billable', 
            `is_billable`=0, 
            `unit_price`=0, 
            `total_price`=0
        WHERE `task_id` IN (".$task_ids.");"
    );
}

function set_all_budgeted_expenses_related_to_task_ids_to_non_billable($task_ids) {
    if(!is_array($task_ids)) return;
    if(!Plugins::instance()->isActivePlugin('expenses2')) return;

    $task_ids = implode(',', $task_ids);
    DB::execute("
        UPDATE 
        ".TABLE_PREFIX."expenses exp, 
        ".TABLE_PREFIX."expense_totals exp_t, 
        ".TABLE_PREFIX."expense_items exp_items
        SET 
            exp.invoicing_status = 'non_billable', 
            exp.is_billable = 0, 
            exp_items.unit_price = 0, 
            exp_items.total_price = 0,
            exp_t.unit_price = 0,
            exp_t.total_price = 0
        WHERE 
            exp.task_id IN (".$task_ids.") AND 
            exp.object_id = exp_items.expense_id AND 
            exp.object_id = exp_t.expense_id;"
    );
}

