<?php


function get_timeslot_person_key_for_grouping(Timeslot $t) {
	$key = $t->getContactId();
	Hook::fire('override_timeslot_person_key_for_grouping', $t, $key);
	return $key;
}

function get_timeslot_person_key_text($key) {
	$exploded = array_filter(explode("-", $key));
	
	// get the person id from the array and remove it
	$person_id = array_shift($exploded);
	// find person object
	$person = Contacts::instance()->findById($person_id);
	// put the line description prefix for generated invoince lines
	$text = trim(income_config_option("generated_invoice_line_prefix_text"));
	// set text as the person name
	$text .= " " . ($person instanceof Contact ? $person->getObjectName() : "");
	
	$text = trim($text);
	
	// look for member ids in the rest of the array
	if (count($exploded) > 0) {
		foreach ($exploded as $member_id) {
			$m = Members::getMemberById($member_id);
			if ($m instanceof Member) {
				$text .= "\n" . $m->getName();
			}
		}
	}
	
	return $text;
}


function generate_invoice_data_from_objects($objects, Contact $user, $group_member_id, &$tasks_without_time) {
	
	$invoice_data = null;
	
	if (Plugins::instance()->isActivePlugin('income')) {
		
		// default invoice type
        $invoice_type = ObjectSubtypes::instance()->findOne(array('conditions' => "code='invoice'"));
        $invoice_type_id = $invoice_type->getId();
		
		// taxes
		$tax_ids = IncomeTaxes::instance()->findAll(array('id' => true, 'conditions' => 'select_default=1 AND removed_by_id = 0'));
		$taxes = array();
		foreach ($tax_ids as $tid) $taxes[$tid] = 1;
		
		// copies
		$copy_ids = IncomeCopies::instance()->findAll(array('id' => true, 'conditions' => 'select_default=1 AND removed_by_id = 0'));
		$copies = array();
		foreach ($copy_ids as $cid) $copies[$cid] = 1;
		
		// only logged user is subscribed
		$subscribers = array('user_' . $user->getId() => 1);

		Env::useHelper('functions', 'income');
		$group_invoice_timeslots_by = income_config_option('group_invoice_timeslots_by');
		//$one_line_per_timeslot = income_config_option('generate_one_inv_line_per_timeslot');
		$one_line_per_timeslot = $group_invoice_timeslots_by == 'Timeslot';
		$show_rate_detailed_lines = income_config_option('generate_inv_line_detailed');
		
		$invoice_lines = array();
		$member_ids = array();
		$invoiced_tasks = array();
		
		if ($group_invoice_timeslots_by == 'Person') {
			// group timeslots by person (and labor category if plugin is installed)
			$all_timeslots = array();
			foreach ($objects as $object) {
				if ($object instanceof Timeslot) {
					$timeslots = array($object);
				} else {
					$timeslots = $object->getTimeslots();
				}
				$pending_timeslots = array();
				foreach($timeslots as $ts){
					if($ts->getColumnValue('invoicing_status') == 'pending'){
						array_push($pending_timeslots, $ts);
					}
				}
				$all_timeslots = array_merge($all_timeslots, $pending_timeslots);
				
				$member_ids = array_unique(array_merge($member_ids, $object->getMemberIds()));
			}
			
			// group them by person and labor cat
			$grouped_timeslots = array();
			foreach ($all_timeslots as $t) {
				$key = get_timeslot_person_key_for_grouping($t);
				if (!isset($grouped_timeslots[$key])) $grouped_timeslots[$key] = array();
				$grouped_timeslots[$key][] = $t;
			}
			
			foreach ($grouped_timeslots as $key => $timeslots) {
				
				// generate invoice lines from timeslots
				$object_total = 0;
				$total_timeslots_minutes = 0;
				$amount = 0;
				
				foreach ($timeslots as $timeslot) {
					if ($timeslot->getColumnValue('invoicing_status') == 'pending') {
						$ts_minutes = $timeslot->getMinutes();
						
						Hook::fire('override_timeslot_column_fields_when_generating_invoice', array('object'=>$timeslot), $timeslot);
				
						if ($timeslot->getIsFixedBilling()) {
							$total = $timeslot->getFixedBilling();
						} else {
							$total = $timeslot->getHourlyBilling() * ($ts_minutes/60);
						}
				
						$total_timeslots_minutes += ($ts_minutes/60);
						$object_total += $total;
				
						if ($one_line_per_timeslot || $object instanceof Timeslot) {
							if ($show_rate_detailed_lines) {
								$amount += ($ts_minutes/60);
							} else {
								$amount += 1;
							}
						} else {
							$amount += ($ts_minutes/60);
						}
						$invoiced_tasks[] = $timeslot;
					}
				}
				
				$line_text = get_timeslot_person_key_text($key);
				$data = array(
						'amount' => $amount,
						'unit_value' => $object_total / $amount,
						'total' => $object_total,
						'subtotal' => $object_total,
						'detail' => $line_text,
				);
				$invoice_lines[] = $data;
				
			}
			
		} else {
			// group the timeslots by tasks or one line per timeslots
			foreach ($objects as $object) {
				
				if ($object instanceof Timeslot) {
					$timeslots = array($object);
				} else {
					$timeslots = $object->getTimeslots();
				}
				// set member ids = task member ids
				$member_ids = array_unique(array_merge($member_ids, $object->getMemberIds()));
				
				// generate invoice lines from timeslots
				$object_total = 0;
				$total_timeslots_minutes = 0;
				$has_timeslots = false;
				
				foreach ($timeslots as $timeslot) {
					if ($timeslot->getColumnValue('invoicing_status') == 'pending') {
						$ts_minutes = $timeslot->getMinutes();
						
						Hook::fire('override_timeslot_column_fields_when_generating_invoice', array('object'=>$timeslot), $timeslot);
						
						if ($timeslot->getIsFixedBilling()) {
							$total = $timeslot->getFixedBilling();
						} else {
							$total = $timeslot->getHourlyBilling() * ($ts_minutes/60);
						}
						
						$total_timeslots_minutes += ($ts_minutes/60);
						$object_total += $total;
						
						if ($one_line_per_timeslot || $object instanceof Timeslot) {
							$amount = 1;
							$unit_value = $total;
							if ($show_rate_detailed_lines) {
								$amount = ($ts_minutes/60);
								$unit_value = $total / $amount;
							}
							$ts_text = trim($timeslot->getDescription());
							if ($ts_text == "") {
								$rel_obj = $timeslot->getRelObject();
								if ($rel_obj instanceof ContentDataObject) {
									$ts_text = lang('hours')." - ".$rel_obj->getObjectName();
								} else {
									$ts_text = lang('hours');
								}
							}
							$data = array(
								'amount' => $amount,
								'unit_value' => $unit_value,
								'total' => $total,
								'subtotal' => $total,
								'detail' => $ts_text,
							);
							$invoice_lines[] = $data;
						}
						
						$has_timeslots = true;
					}
				}
				
				if (!$one_line_per_timeslot && !$object instanceof Timeslot && $object_total > 0) {
					$amount = 1;
					$unit_value = $object_total;
					if ($show_rate_detailed_lines) {
						$amount = $total_timeslots_minutes;
						$unit_value = $object_total / $amount;
					}
					$data = array(
						'amount' => $amount,
						'unit_value' => $unit_value,
						'total' => $object_total,
						'subtotal' => $object_total,
						'detail' => $object->getObjectName(),
					);
					$invoice_lines[] = $data;
				}
				
				if (!$has_timeslots) {
					$objects_without_time[] = $object;
				} else {
					$invoiced_tasks[] = $object;
				}
			}
			
		}
		
		
		if (count($invoice_lines) == 0) {
			//throw new Exception(lang('task has no time records with pending invoicing'));
			return null;
		}
		
		// find customer data
		if (!$group_member_id) {
			$customer_member = null;
			$mem_object_type_ids = ObjectTypes::instance()->findAll(array('id' => true, 'conditions' => "name IN ('project','customer_folder','project_folder')"));
			$cli_ot = ObjectTypes::findByName('customer');
			$project_ot = ObjectTypes::findByName('project');
			$projects_dim = Dimensions::findByCode('customer_project');
			Env::useHelper('functions', 'crpm');
			$clients_dim = get_customers_dimension();
			
			foreach ($member_ids as $member_id) {
				$member = Members::instance()->findById($member_id);
				if ($member instanceof Member) {
					if ($member->getObjectTypeId() == $cli_ot->getId()) {
						
						$customer_member = $member;
						
					} else if (in_array($member->getObjectTypeId(), $mem_object_type_ids)) {
						
						if ($member->getObjectTypeId() == $project_ot->getId() && $projects_dim->getId() != $clients_dim->getId()) {
							
							$customer_member_id = get_associated_status_member_id($member, $clients_dim, $cli_ot, false, 'project_customer');
							if ($customer_member_id > 0) {
								$customer_member = Members::getMemberById($customer_member_id);
							}
							
						} else {
							
							$all_parents = $member->getAllParentMembersInHierarchy();
							foreach ($all_parents as $parent) {
								if ($parent->getObjectTypeId() == $cli_ot->getId()) {
									$customer_member = $parent;
									break;
								}
							}
						}
					}
				}
				if ($customer_member instanceof Member) break;
			}
			
		} else {
			$customer_member = Members::instance()->findById($group_member_id);
		}
		
		if (!$customer_member instanceof Member) {
			throw new Exception(lang('task must be related to a client or a project or folder below a client'));
		}
		$customer = Customers::instance()->findById($customer_member->getObjectId());
		if (!$customer instanceof Customer) {
			throw new Exception(lang('task must be related to a client or a project or folder below a client'));
		}
		
		// invoice general data
		$invoice = array(
			'name' => '',
			'date' => format_date(DateTimeValueLib::now()),
			'customer_name' => $customer->getName(),
			'customer_rut' => $customer->getCustomerRut(),
			'customer_address' => $customer->getCustomerAddress(),
			'customer_number' => $customer->getCustomerNumber(),
			'invoice_type_id' => $invoice_type_id,
			'additional_information' => '',
			'status' => 'pending',
			'invoice_notebook_id' => 0,
			'series' => '',
			'invoice_number' => 0,
		);
		
		$invoice_data = array(
			'invoice' => $invoice,
			'taxes' => $taxes,
			'copies' => $copies,
			'members' => $member_ids,
			'invoice_lines' => $invoice_lines,
			'subscribers' => $subscribers,
			'invoiced_tasks' => $invoiced_tasks,
		);
		
	}
	
	return $invoice_data;

}


/**
 * Groups objects by customer
 */
function group_objects_for_invoices($tasks) {
	
	return group_objects_by_member_of_type($tasks, 'customer');
	
}

function group_objects_by_member_of_type($objects, $object_type_name) {
	$grouped = array();
	$cli_ot = ObjectTypes::findByName($object_type_name);

	foreach ($objects as $object) {
		$member_id = get_object_member_of_type($object, $cli_ot->getId());
		if (!isset($grouped[$member_id])) {
			$grouped[$member_id] = array();
		}
		$grouped[$member_id][] = $object;
	}

	return $grouped;
}

/**
 * Returns the member id that is associated to the object $object and its type is equal to $object_type_id
 */
function get_object_member_of_type($object, $object_type_id) {
	$mem_id = 0;
	if (!$object instanceof ContentDataObject) return $mem_id;
	
	$members = $object->getMembers();
	foreach ($members as $mem) {
		if ($mem->getObjectTypeId() == $object_type_id) {
			
			return $mem->getId();
			
		} else {
			
			$all_parents = $mem->getAllParentMembersInHierarchy();
			foreach ($all_parents as $parent) {
				if ($parent->getObjectTypeId() == $object_type_id) {
					return $parent->getId();
				}
			}
		}
	}
	if ($mem_id == 0) {
		$cli_ot = ObjectTypes::findByName('customer');
		$project_ot = ObjectTypes::findByName('project');
		$projects_dim = Dimensions::findByCode('customer_project');
		Env::useHelper('functions', 'crpm');
		$clients_dim = get_customers_dimension();
		
		if ($object_type_id == $cli_ot->getId() && $projects_dim->getId() != $clients_dim->getId()) {
			foreach ($members as $member) {
				if ($member->getObjectTypeId() == $project_ot->getId()) {
					$mem_id = get_associated_status_member_id($member, $clients_dim, $cli_ot, false, 'project_customer');
				}
			}
		}
	}
	
	return $mem_id;
}



function calculate_timeslot_rate_and_cost($object) {
	// first reset billing category status
	$sql = "UPDATE ".TABLE_PREFIX."timeslots SET billing_category_status='' WHERE object_id=".$object->getId();
	DB::execute($sql);
	
	$allowed_dimensions = array_filter(config_option('adv_billing_dimensions'));
	if (count($allowed_dimensions) == 0) return;
	
	$object_id = $object->getId();
	
	$sql = "SELECT m.dimension_id, om.member_id FROM ".TABLE_PREFIX."object_members om
				INNER JOIN ".TABLE_PREFIX."members m ON om.member_id = m.id
				WHERE om.is_optimization=0 AND dimension_id IN (".implode(',', $allowed_dimensions).") AND om.object_id = '$object_id'";
	
	$dim_member_rows = DB::executeAll($sql);
	$dim_members = array();
	if (is_array($dim_member_rows)) {
		foreach ($dim_member_rows as $dmr) {
			if (!isset($dim_members[$dmr['dimension_id']])) $dim_members[$dmr['dimension_id']] = array();
			$dim_members[$dmr['dimension_id']][] = $dmr['member_id'];
		}
	}
	
	if ($object->getColumnValue('invoicing_status') == 'non_billable') {
		// if it is non billable then 
		$sql = "
			UPDATE ".TABLE_PREFIX."timeslots
			SET is_fixed_billing='0', fixed_billing='0', hourly_billing='0',
				rate_currency_id='1', adv_billing_id=0
			WHERE object_id=".$object->getId();
		
		DB::execute($sql);
		
	} else {
	
		$cant_decide_between_billing_definitions = null;
		
		// GET BILLING DEFINITION FOR BILLING RATE
		$billing_definition = find_billing_def_by_user_and_members($object->getContactId(), $dim_members, $allowed_dimensions, 'billing', $cant_decide_between_billing_definitions);
		
		$bd_id = $billing_definition instanceof BillingDefinition ? $billing_definition->getId() : 0;
		$bd_hourly_billing = $billing_definition instanceof BillingDefinition ? $billing_definition->getHourlyRate() : 0;
		$bd_id_sql = $billing_definition instanceof BillingDefinition ? (", adv_billing_id='".$billing_definition->getId()."'") : "";
		
		$object_data = array_var($_POST, 'timeslot', array());
		if(empty($object_data)) $object_data = $object->getArrayInfo(true);
		// SET BILLING RATE
		if ($billing_definition instanceof BillingDefinition || isset($object_data['hourly_billing']) || $object->getForceRecalculateBilling()) {
			$is_fixed_billing = array_var($object_data, 'is_fixed_billing', false);
			$uses_modified_keys = array_var($object_data, 'uses_modified_keys', false);
			if ($object->getForceRecalculateBilling()) {
				$hourly_billing = $bd_hourly_billing;
			} else {
				$hourly_billing_key = $uses_modified_keys ? 'hourlybilling' : 'hourly_billing';
				$hourly_billing =  array_var($object_data, $hourly_billing_key, $bd_hourly_billing);
			}
			$fixed_billing = 0;
			
			if ($is_fixed_billing){
				$fixed_billing_key = $uses_modified_keys ? 'totalbilling' : 'fixed_billing';
				$fixed_billing = array_var($object_data, $fixed_billing_key, 0);
				$r_cur_id = array_var($object_data, 'rate_currency_id');
			} else {
				$fixed_billing = $hourly_billing * $object->getMinutes() / 60;
				if ($billing_definition instanceof BillingDefinition) {
					$r_cur_id = $billing_definition->getRateCurrencyId();
				} else {
					$c = Currencies::getDefaultCurrencyInfo();
					$r_cur_id = is_array($c) ? $c['id'] : 0;
				}
			}
			
			
			$sql = "
				UPDATE ".TABLE_PREFIX."timeslots
				SET is_fixed_billing='$is_fixed_billing', fixed_billing='$fixed_billing', hourly_billing='$hourly_billing', 
					rate_currency_id='$r_cur_id' $bd_id_sql
				WHERE object_id=".$object->getId();
			
			DB::execute($sql);
			
		} else {
			
			// could not decide => mark timeslot
			if ($cant_decide_between_billing_definitions) {
				$sql = "UPDATE ".TABLE_PREFIX."timeslots SET billing_category_status='conflict' WHERE object_id=".$object->getId();
				DB::execute($sql);
			}
		}
		
	}
	
	$cant_decide_between_cost_definitions = null;
	
	// GET BILLING DEFINITION FOR COST
	$billing_definition = find_billing_def_by_user_and_members($object->getContactId(), $dim_members, $allowed_dimensions, 'cost', $cant_decide_between_cost_definitions);
	
	$bd_id = $billing_definition instanceof BillingDefinition ? $billing_definition->getId() : 0;
	$bd_hourly_cost = $billing_definition instanceof BillingDefinition ? $billing_definition->getHourlyCost() : 0;
	$bd_id_sql = $billing_definition instanceof BillingDefinition ? (", adv_cost_id='".$billing_definition->getId()."'") : "";
	
	// SET BILLING COST
	if ($billing_definition instanceof BillingDefinition || isset($object_data['hourly_cost']) || $object->getForceRecalculateBilling()) {
		$is_fixed_cost = isset($object_data) && array_var($object_data, 'is_fixed_cost', false);  // *** LC 2023-10-03
		if ($object->getForceRecalculateBilling()) {
			$hourly_cost = $bd_hourly_cost;
		} else {
			$hourly_cost = array_var($object_data, 'hourly_cost', $bd_hourly_cost);
		}
		$fixed_cost = 0;
					
		if ($is_fixed_cost){
			$fixed_cost = array_var($object_data, 'fixed_cost', 0);
			$c_cur_id = array_var($object_data, 'cost_currency_id');
		} else {
			$fixed_cost = $hourly_cost * $object->getMinutes() / 60;
			if ($billing_definition instanceof BillingDefinition) {
				$c_cur_id = $billing_definition->getCostCurrencyId();
			} else {
				$c = Currencies::getDefaultCurrencyInfo();
				$c_cur_id = is_array($c) ? $c['id'] : 0;
			}
		}
		
		$sql = "
			UPDATE ".TABLE_PREFIX."timeslots
			SET is_fixed_cost='$is_fixed_cost', fixed_cost='$fixed_cost', hourly_cost='$hourly_cost', 
				cost_currency_id='$c_cur_id' $bd_id_sql
			WHERE object_id=".$object->getId();
		
		DB::execute($sql);
		
	} else {
		
		// could not decide => mark timeslot
		if ($cant_decide_between_cost_definitions) {
			$sql = "UPDATE ".TABLE_PREFIX."timeslots SET billing_category_status='conflict' WHERE object_id=".$object->getId();
			DB::execute($sql);
		}
	}
}

function calculate_rate_and_cost($object,ObjectBilling $object_billing) {

	// sometimes we don't want to calculate these values, example: in task templates they are calculated at the end 
	// so the first times we get into this function are not relevant because object still has not its complete data
	// in that case we set this variable to skip these calculations and improve performance
	if ($object->dont_calculate_financials) return;


    // first reset billing category status
    $sql = "UPDATE ".TABLE_PREFIX."object_billing SET billing_category_status='' WHERE rel_object_id=".$object->getObjectId();
    DB::execute($sql);

    $allowed_dimensions = array_filter(config_option('adv_billing_dimensions'));
    if (count($allowed_dimensions) == 0) return;

    $object_id = $object->getObjectId();

    $sql = "SELECT m.dimension_id, om.member_id FROM ".TABLE_PREFIX."object_members om
				INNER JOIN ".TABLE_PREFIX."members m ON om.member_id = m.id
				WHERE om.is_optimization=0 AND dimension_id IN (".implode(',', $allowed_dimensions).") AND om.object_id = '$object_id'";

    $dim_member_rows = DB::executeAll($sql);
    $dim_members = array();
    if (is_array($dim_member_rows)) {
        foreach ($dim_member_rows as $dmr) {
            if (!isset($dim_members[$dmr['dimension_id']])) $dim_members[$dmr['dimension_id']] = array();
            $dim_members[$dmr['dimension_id']][] = $dmr['member_id'];
        }
    }

    $cant_decide_between_billing_definitions = null;

    // GET BILLING DEFINITION FOR BILLING RATE
    $billing_definition = find_billing_def_by_user_and_members($object_billing->getContactId(), $dim_members, $allowed_dimensions, 'billing', $cant_decide_between_billing_definitions);

    $bd_id = $billing_definition instanceof BillingDefinition ? $billing_definition->getId() : 0;
    $bd_hourly_billing = $billing_definition instanceof BillingDefinition ? $billing_definition->getHourlyRate() : 0;
    $bd_id_sql = $billing_definition instanceof BillingDefinition ? (", adv_billing_id='".$billing_definition->getId()."'") : "";


    // SET BILLING RATE
    if ($billing_definition instanceof BillingDefinition || !is_null($object_billing->getBillingRate())) {


        if ($object_billing->getForceRecalculateBilling()) {
            $hourly_billing = $bd_hourly_billing;
            $is_fixed_billing = false;
        } else {
            $hourly_billing = $object_billing->getBillingRate()? $object_billing->getBillingRate() : $bd_hourly_billing;
            $is_fixed_billing = $object_billing->getIsTotalPrice();
        }

        $fixed_billing = 0;

        if ($is_fixed_billing){
            $fixed_billing = $object_billing->getTotalPrice();
            $r_cur_id = $object_billing->getRateCurrencyId();
        } else {
            $fixed_billing = $hourly_billing * ($object_billing->getTime() / 60);
            if ($billing_definition instanceof BillingDefinition) {
                $r_cur_id = $billing_definition->getRateCurrencyId();
            } else {
                $c = Currencies::getDefaultCurrencyInfo();
                $r_cur_id = is_array($c) ? $c['id'] : 0;
            }
        }

        if ($fixed_billing == 0){
            $bd_id_sql = ', adv_billing_id=0';
        }

        $sql = "
			UPDATE ".TABLE_PREFIX."object_billing
			SET is_total_price='$is_fixed_billing', total_price='$fixed_billing', billing_rate='$hourly_billing', 
				rate_currency_id='$r_cur_id' $bd_id_sql
			WHERE rel_object_id=".$object->getObjectId();

        DB::execute($sql);

    } else {

        // could not decide => mark timeslot
        if ($cant_decide_between_billing_definitions) {
            $sql = "UPDATE ".TABLE_PREFIX."object_billing SET billing_category_status='conflict' WHERE rel_object_id=".$object->getObjectId();
            DB::execute($sql);
        }
    }

    $cant_decide_between_cost_definitions = null;

    // GET BILLING DEFINITION FOR COST
    $billing_definition = find_billing_def_by_user_and_members($object_billing->getContactId(), $dim_members, $allowed_dimensions, 'cost', $cant_decide_between_cost_definitions);

    $bd_id = $billing_definition instanceof BillingDefinition ? $billing_definition->getId() : 0;
    $bd_hourly_cost = $billing_definition instanceof BillingDefinition ? $billing_definition->getHourlyCost() : 0;
    $bd_id_sql = $billing_definition instanceof BillingDefinition ? (", adv_cost_id='".$billing_definition->getId()."'") : "";

    // SET BILLING COST
    if ($billing_definition instanceof BillingDefinition || !is_null($object_billing->getCostRate())) {

        if ($object_billing->getForceRecalculateBilling()) {
            $hourly_cost = $bd_hourly_cost;
            $is_fixed_cost = false;
        } else {
            $hourly_cost = $object_billing->getCostRate() ? $object_billing->getCostRate() : $bd_hourly_cost;
            $is_fixed_cost = $object_billing->getIsTotalCost();
        }

        $fixed_cost = 0;

        if ($is_fixed_cost){
            $fixed_cost = $object_billing->getTotalCost();
            $c_cur_id = $object_billing->getCostCurrencyId();
        } else {
            $fixed_cost = $hourly_cost * ($object_billing->getTime() / 60);
            if ($billing_definition instanceof BillingDefinition) {
                $c_cur_id = $billing_definition->getCostCurrencyId();
            } else {
                $c = Currencies::getDefaultCurrencyInfo();
                $c_cur_id = is_array($c) ? $c['id'] : 0;
            }
        }

        if ($fixed_cost == 0){
            $bd_id_sql = ', adv_cost_id=0';
        }

        $sql = "
			UPDATE ".TABLE_PREFIX."object_billing
			SET is_total_cost='$is_fixed_cost', total_cost='$fixed_cost', cost_rate='$hourly_cost', 
				cost_currency_id='$c_cur_id' $bd_id_sql
			WHERE rel_object_id=".$object->getObjectId();

        DB::execute($sql);

    } else {

        // could not decide => mark object
        if ($cant_decide_between_cost_definitions) {
            $sql = "UPDATE ".TABLE_PREFIX."object_billing SET billing_category_status='conflict' WHERE rel_object_id=".$object->getObjectId();
            DB::execute($sql);
        }
    }



}

function find_billing_def_by_user_and_members($contact_id, $dim_member_rows, $allowed_dimensions, $type, &$cant_decide_between_definitions) {
	
	$billing_definition = null;
	
	$dim_member_conds = array();
	
	foreach ($allowed_dimensions as $dim_id) {
		
		$this_dim_conds = array();
		$member_ids = array_var($dim_member_rows, $dim_id);
		
		if (is_array($member_ids) && count($member_ids) > 0) {
			$all_path_member_ids = array();
			
			foreach ($member_ids as $mem_id) {
				
				$m = Members::getMemberById($mem_id);
				if ($m instanceof Member) {
					$all_path_members = $m->getAllParentMembersInHierarchy(true, false);
					
					foreach ($all_path_members as $pm) {
						$all_path_member_ids[] = $pm->getId();
					}
				}
			}
			
			if (count($all_path_member_ids) > 0) {
				// condition to check that all member_ids of this dimension are included in the timeslot classification
				$all_path_member_ids = array_unique(array_filter($all_path_member_ids));
				$this_dim_conds[] = "(
					(SELECT count(dm1.member_id) FROM `".TABLE_PREFIX."billing_definition_members` dm1 
							where dm1.definition_id=d.id AND dm1.dimension_id=$dim_id) =
					(SELECT count(dm2.member_id) FROM `".TABLE_PREFIX."billing_definition_members` dm2 
							where dm2.definition_id=d.id AND dm2.dimension_id=$dim_id
							AND dm2.member_id IN (".implode(',',$all_path_member_ids)."))
				)";
			}
		}
		
		if (count($this_dim_conds) > 0) {
			$dim_member_conds[$dim_id] = "( " . implode(" OR ", $this_dim_conds) . " )";
		} else {
			$dim_member_conds[$dim_id] = "";
		}
	}

	if ($type == 'cost') {
		$type_cond = "enable_cost > 0 AND ";
	} else {
		$type_cond = "enable_rate > 0 AND ";
	}

	$rows = array();
	$stop = false;
	
	$dimension_ids = array_keys($dim_member_conds);
	// start with the full set of dimension ids
	$set_size = count($dimension_ids);
	
	while (!$stop) {
		
		$dim_subsets = array();
		get_all_subsets($dimension_ids, 0, $set_size, 0, $dim_subsets);
		
		// search for a billing definition using all combinations of dimension
		foreach ($dim_subsets as $subset) {
			$dim_member_not_cond_sql = "";
			$dim_member_cond_sql = "";
			
			foreach ($dimension_ids as $dim_id) {
				if (in_array($dim_id, $subset) && trim(array_var($dim_member_conds, $dim_id, "")) != "") {
					$dim_member_cond_sql .= " AND " . $dim_member_conds[$dim_id];
				} else {
					$dim_member_not_cond_sql .= "
						AND NOT EXISTS (
							SELECT dm.definition_id FROM ".TABLE_PREFIX."billing_definition_members dm WHERE dm.definition_id=d.id AND dimension_id='$dim_id' AND member_id>0
						) ";
				}
			}
			
			// check for specific billing definition for the user and this set of dimension ids
			$sql = "SELECT d.* FROM ".TABLE_PREFIX."billing_definitions d WHERE $type_cond user_id='$contact_id' $dim_member_cond_sql $dim_member_not_cond_sql";
			$rows = DB::executeAll($sql);
			
			// if no billing definition found for the user and this set of dimension ids => check for the same dimension ids and all users (user_id=0)
			if (!isset($rows) || count($rows) == 0) {
				$sql = "SELECT d.* FROM ".TABLE_PREFIX."billing_definitions d WHERE $type_cond user_id=0 $dim_member_cond_sql $dim_member_not_cond_sql";
				$rows = DB::executeAll($sql);
			}
			
			if (isset($rows) && is_array($rows) && count($rows) > 0) {
				// billing definition found for this subset
				$stop = true;
				break;
			} else {
				// decrease set size => try using less dimensions in queries, uses all subsets
				$set_size = $set_size - 1;
			}
		}
		
		if ($set_size <= 0) {
			$stop = true;
		}
	}
	
	$row_data = null;
	if (isset($rows) && is_array($rows) && count($rows) > 0) {
		
		$row_data = get_best_billing_definition($rows, $dim_member_rows, $cant_decide_between_definitions);
		
	} else {

		// try general gategory for the user
		$sql = "SELECT d.* FROM ".TABLE_PREFIX."billing_definitions d WHERE $type_cond user_id='$contact_id' AND NOT EXISTS (
					SELECT dm.definition_id FROM ".TABLE_PREFIX."billing_definition_members dm WHERE dm.definition_id=d.id AND member_id>0
				)";
		$rows = DB::executeAll($sql);
		
		if (!is_array($rows) || count($rows) == 0) {
			// try general gategory for all users
			$sql = "SELECT d.* FROM ".TABLE_PREFIX."billing_definitions d WHERE $type_cond user_id='0' AND NOT EXISTS (
						SELECT dm.definition_id FROM ".TABLE_PREFIX."billing_definition_members dm WHERE dm.definition_id=d.id AND member_id>0
					)";
			$rows = DB::executeAll($sql);
		}
		
		if (is_array($rows) && count($rows) > 0) {
			$row_data = $rows[0];
		}
	}
	
	if (is_array($row_data)) {
		$billing_definition = new BillingDefinition();
		$billing_definition->setFromAttributes($row_data);
		$billing_definition->setId($row_data['id']);
	}
	
	return $billing_definition;
	
}


// returns the definition with more depth for the dimensions in $dim_member_rows
function get_best_billing_definition($billing_definition_rows, $dim_member_rows, &$cant_decide_between_definitions) {
	if (count($billing_definition_rows) <= 1) {
		return array_var($billing_definition_rows, 0);
	}
	
	$dim_ids = array_keys($dim_member_rows);
	$bd_ids = array();
	$bds = array();
	
	foreach ($billing_definition_rows as $row) {
		$bd_ids[] = $row['id'];
		$bds[$row['id']] = $row;
	}
	
	if (count($dim_ids) > 0 && count($bd_ids) > 0) {
		$bd_ids = array_filter($bd_ids);
		$dim_ids = array_filter($dim_ids);
		$sql = "SELECT definition_id, sum(depth) as total_depth FROM ".TABLE_PREFIX."billing_definition_members bdm
			INNER JOIN ".TABLE_PREFIX."members m on m.id=bdm.member_id
			WHERE bdm.definition_id IN (".implode(',', $bd_ids).") AND bdm.dimension_id IN (".implode(',', $dim_ids).")
			GROUP BY definition_id;";
		
		$result = DB::executeAll($sql);
		
		$grouped_by_depth = array();
		if (is_array($result)) {
			foreach ($result as $r) {
				if (!isset($grouped_by_depth[$r['total_depth']])) $grouped_by_depth[$r['total_depth']] = array();
				$grouped_by_depth[$r['total_depth']][] = $r;
			}
		}
		
		if (count($grouped_by_depth)) {
			$max_key = max(array_keys($grouped_by_depth));
			
			$max = null;
			$max_depth_rows = array_var($grouped_by_depth, $max_key);
			if (count($max_depth_rows) <= 1) {
				$max = $max_depth_rows[0];
			} else {
				$cant_decide_between_definitions = true;
				return null;
			}
			
			if ($max) {
				return array_var($bds, $max['definition_id']);
			}
			
		} else {
			return array_var($billing_definition_rows, 0);
		}
		
		
	} else {
		return array_var($billing_definition_rows, 0);
	}
	
	return null;
}


function calculate_task_estimated_financials($object, $params = array()){
	$calculate_parents = array_var($params, 'calculate_parents', true);
	// sometimes we don't want to calculate these values, example: in task templates they are calculated at the end 
	// so the first times we get into this function are not relevant because object still has not its complete data
	// in that case we set this variable to skip these calculations and improve performance
	if ($object->dont_calculate_financials) return;

	Env::useHelper('additional_tasks_columns_functions', 'advanced_billing');
	Env::useHelper('functions', 'advanced_billing');

	$calculate_estimated_price = $object->getColumnValue('is_calculated_estimated_price');
	$is_billable = $object->getColumnValue('is_billable');
	$task_children_financials = get_task_children_financials_info($object);

	// Calculate estimated values
	// Assign values to variables to get billing definition
	$allowed_dimensions = array_filter(config_option('adv_billing_dimensions'));
	$cant_decide_between_billing_definitions = null;
	$task_id = $object->getObjectId();
	$dim_members = get_task_dimension_members($allowed_dimensions, $task_id);
	$assigned_to_contact = $object->getAssignedToContactId();
	$estimated_time = $object->getTimeEstimate();


	$task_expense_totals = array('total_cost' => 0, 'total_price' => 0);
	if (Plugins::instance()->isActivePlugin('expenses2')) {
		$task_expense_totals = Expenses::getBudgetedExpensesTotalsByTasks(array($object->getId()));
	}

	// Calculate estimated cost
	$cost_billing_definition = find_billing_def_by_user_and_members($assigned_to_contact, $dim_members, $allowed_dimensions, 'cost', $cant_decide_between_billing_definitions); 
	$hourly_cost = $cost_billing_definition instanceof BillingDefinition ? $cost_billing_definition->getHourlyCost() : 0;
	// Calculate estimated price for task itself
	$estimated_cost = $estimated_time > 0 ? ($estimated_time / 60) * $hourly_cost : 0;
	// Add estimated cost of subtasks
	$estimated_cost += $task_children_financials['estimated_cost'];
	// Add estimated cost of expenses
	$estimated_cost += $task_expense_totals['total_cost'];
    // Set estimated cost
	$object->setColumnValue('estimated_cost', $estimated_cost);

	// Calculate estimated price
	if($calculate_estimated_price){
		$price_billing_definition = find_billing_def_by_user_and_members($assigned_to_contact, $dim_members, $allowed_dimensions, 'billing', $cant_decide_between_billing_definitions); 
		$hourly_price = $price_billing_definition instanceof BillingDefinition ? $price_billing_definition->getHourlyRate() : 0;
		$estimated_price = $estimated_time > 0 ? ($estimated_time / 60) * $hourly_price : 0;
		$estimated_price += $task_children_financials['estimated_price'];

		$estimated_price += $task_expense_totals['total_price'];

		$object->setColumnValue('estimated_price', $estimated_price);
	}

	//	Check if parent has quoted price => Set estimated price to ZERO
	$has_parent_with_quoted_price = has_parent_with_quoted_price($object);
	if($has_parent_with_quoted_price || !$is_billable) {
		$object->setColumnValue('estimated_price', 0);
	}

	$object->save();

	$parent_id = $object->getParentId();
	$parent_task = ProjectTasks::instance()->findById($parent_id);
	if($parent_task instanceof ProjectTask && $calculate_parents){
		// if we dont want to recalculate project financials for this task then we also don't want it in the parent
		$parent_task->dont_calculate_project_financials = $object->dont_calculate_project_financials;
		calculate_task_estimated_financials($parent_task);
	}
}

function calculate_task_estimated_and_executed_financials($object, $params = array()){
	// Avoid calculating financials
	if(array_var($_SESSION, 'dont_calculate_anything', false)) return;

	$calculate_parents = array_var($params, 'calculate_parents', true);
	// sometimes we don't want to calculate these values, example: in task templates they are calculated at the end 
	// so the first times we get into this function are not relevant because object still has not its complete data
	// in that case we set this variable to skip these calculations and improve performance
	if ($object->dont_calculate_financials) return;

	Env::useHelper('additional_tasks_columns_functions', 'advanced_billing');
	Env::useHelper('functions', 'advanced_billing');

	$calculate_estimated_price = $object->getColumnValue('is_calculated_estimated_price');
	$calculate_earned_value = $object->getColumnValue('is_calculated_earned_value');
	$calculate_earned_value_using_estimated_price = $calculate_estimated_price == 0;
	$is_fixed_fee = $object->getColumnValue('is_fixed_fee');
	$is_billable = $object->getColumnValue('is_billable');


	$task_children_financials = get_task_children_financials_info($object);
	// Calculate estimated values
	// Assign values to variables to get billing definition
	$allowed_dimensions = array_filter(config_option('adv_billing_dimensions'));
	$cant_decide_between_billing_definitions = null;
	$task_id = $object->getObjectId();
	$dim_members = get_task_dimension_members($allowed_dimensions, $task_id);
	$assigned_to_contact = $object->getAssignedToContactId();
	$estimated_time = $object->getTimeEstimate();


	$task_expense_totals = array('total_cost' => 0, 'total_price' => 0);
	if (Plugins::instance()->isActivePlugin('expenses2')) {
		$task_expense_totals = Expenses::getBudgetedExpensesTotalsByTasks(array($object->getId()));
	}

	// Calculate estimated cost
	$cost_billing_definition = find_billing_def_by_user_and_members($assigned_to_contact, $dim_members, $allowed_dimensions, 'cost', $cant_decide_between_billing_definitions); 
	$hourly_cost = $cost_billing_definition instanceof BillingDefinition ? $cost_billing_definition->getHourlyCost() : 0;
	$estimated_cost = $estimated_time > 0 ? ($estimated_time / 60) * $hourly_cost : 0;
	$estimated_cost += $task_children_financials['estimated_cost'];
	$estimated_cost += $task_expense_totals['total_cost'];
	$object->setColumnValue('estimated_cost', $estimated_cost);

	// Calculate estimated price
	if($calculate_estimated_price){
		$price_billing_definition = find_billing_def_by_user_and_members($assigned_to_contact, $dim_members, $allowed_dimensions, 'billing', $cant_decide_between_billing_definitions); 
		$hourly_price = $price_billing_definition instanceof BillingDefinition ? $price_billing_definition->getHourlyRate() : 0;
		$estimated_price = $estimated_time > 0 ? ($estimated_time / 60) * $hourly_price : 0;
		$estimated_price += $task_children_financials['estimated_price'];

		$estimated_price += $task_expense_totals['total_price'];

		$object->setColumnValue('estimated_price', $estimated_price);
	}

	if(!$is_billable) {
		$estimated_price = 0;
		$object->setColumnValue('estimated_price', $estimated_price);
	}
	// Finish calculations estimated values
	

	// Calculate executed values
	Env::useHelper('additional_tasks_columns_functions', 'advanced_billing');
	$task_id = $object->getObjectId();
	$timeslots = get_tasks_timeslots($task_id);

	$executed_cost = 0;
	$earned_value = 0;
	$ev_time_material = 0;
	if(is_array($timeslots)){
		foreach($timeslots as $time){
			$executed_cost += $time['fixed_cost'];
			$ev_time_material += $time['fixed_billing'];
		}
	}

	if (Plugins::instance()->isActivePlugin('expenses2')) {
		$task_actual_expenses = PaymentReceipts::getActualExpensesByTask($object->getId());
		if (is_array($task_actual_expenses)) {
			foreach ($task_actual_expenses as $exp) {
				$executed_cost += $exp->getAmount();
				if ($exp->getIsBillable()) {
					$ev_time_material += $exp->getTotalPrice();
				}
			}
		}
	}

	// Add children financials to executed cost
	$executed_cost += $task_children_financials['executed_cost'];
	$object->setColumnValue('executed_cost', $executed_cost);
	
	
	// Add children financials to earned value
	$ev_time_material += $task_children_financials['earned_value'];

	// Calculate earned value fixed fee
	$percent_completed = $object->getPercentCompleted();
	$estimated_price = $object->getColumnValue('estimated_price');
	$ev_fixed_fee = ($percent_completed / 100) * $estimated_price;

	// Set earned value based on 'is_fixed_fee' value
	$earned_value = $calculate_earned_value_using_estimated_price && $is_fixed_fee ? $ev_fixed_fee : $ev_time_material; 

	//	Check if parent has quoted price => Set estimated price and earned value on fixed fee to ZERO
	$has_parent_with_quoted_price = has_parent_with_quoted_price($object);
	if($has_parent_with_quoted_price) {
		$ev_fixed_fee = 0;
		$object->setColumnValue('estimated_price', 0);
		$earned_value = $ev_time_material;
	}

	if(!$is_billable) {
		$earned_value = 0;
		$ev_fixed_fee = 0;
		$ev_time_material = 0;
	}

	$object->setColumnValue('earned_value', $earned_value);
	$object->setColumnValue('earned_value_fixed_fee', $ev_fixed_fee);
	$object->setColumnValue('earned_value_time_material', $ev_time_material);
	// Finish calculations executed values
	
	$object->save();

	$parent_id = $object->getParentId();
	$parent_task = ProjectTasks::instance()->findById($parent_id);
	if($parent_task instanceof ProjectTask && $calculate_parents){
		// if we dont want to recalculate project financials for this task then we also don't want it in the parent
		$parent_task->dont_calculate_project_financials = $object->dont_calculate_project_financials;

		calculate_task_estimated_and_executed_financials($parent_task);
	}

}

function calculate_task_executed_cost_and_price($task, $params = array()){
	// Avoid calculating financials
	if(array_var($_SESSION, 'dont_calculate_anything', false)) return;

	$calculate_parents = array_var($params, 'calculate_parents', true);
	Env::useHelper('additional_tasks_columns_functions', 'advanced_billing');
	$calculate_earned_value = $task->getColumnValue('is_calculated_earned_value');
	$calculate_executed_cost = $task->getColumnValue('is_calculated_executed_cost');
	$calculate_estimated_price = $task->getColumnValue('is_calculated_estimated_price');
	$calculate_earned_value_using_estimated_price = $calculate_estimated_price == 0;
	$is_fixed_fee = $task->getColumnValue('is_fixed_fee');
	$is_billable = $task->getColumnValue('is_billable');

	if(($calculate_earned_value || $calculate_earned_value_using_estimated_price) || $calculate_executed_cost){
		$task_id = $task->getObjectId();
		$task_children_financials = get_task_children_financials_info($task);
		$timeslots = get_tasks_timeslots($task_id);
		$executed_cost = 0;
		$earned_value = 0;
		$ev_time_material = 0;
		foreach($timeslots as $time){
			$executed_cost += $time['fixed_cost'];
			$ev_time_material += $time['fixed_billing'];
		}

		if (Plugins::instance()->isActivePlugin('expenses2')) {
			$task_actual_expenses = PaymentReceipts::getActualExpensesByTask($task->getId());
			if (is_array($task_actual_expenses)) {
				foreach ($task_actual_expenses as $exp) {
					$executed_cost += $exp->getAmount();
					if ($exp->getIsBillable()) {
						$ev_time_material += $exp->getTotalPrice();
					}
				}
			}
		}

		if($calculate_executed_cost){
			$executed_cost += $task_children_financials['executed_cost'];
			$task->setColumnValue('executed_cost', $executed_cost);
		}

		// Add children financials to earned value
		$ev_time_material += $task_children_financials['earned_value'];

		if($is_billable) {
			// Calculate earned value fixed fee
			$percent_completed = $task->getPercentCompleted();
			$estimated_price = $task->getColumnValue('estimated_price');
			$ev_fixed_fee = ($percent_completed / 100) * $estimated_price;

			// Set earned value based on 'is_fixed_fee' value
			$earned_value = $calculate_earned_value_using_estimated_price && $is_fixed_fee ? $ev_fixed_fee : $ev_time_material; 
		} else {
			$earned_value = 0;
			$ev_fixed_fee = 0;
			$ev_time_material = 0;
		}


		$task->setColumnValue('earned_value', $earned_value);
		$task->setColumnValue('earned_value_fixed_fee', $ev_fixed_fee);
		$task->setColumnValue('earned_value_time_material', $ev_time_material);
		//$task->save();
		
		$set_sql = "SET earned_value = ".$earned_value;
		$set_sql .= ", earned_value_fixed_fee = ".$ev_fixed_fee;
		$set_sql .= ", earned_value_time_material = ".$ev_time_material;
		if($calculate_executed_cost){
			$set_sql .= ", executed_cost = ".$executed_cost;
		}
		DB::execute("UPDATE ".TABLE_PREFIX."project_tasks 
					$set_sql
					WHERE object_id = ".$task->getId());
	}

	$parent_id = $task->getParentId();
	$parent_task = ProjectTasks::instance()->findById($parent_id);
	if($parent_task instanceof ProjectTask && $calculate_parents){
		calculate_task_executed_cost_and_price($parent_task);
	}
}

function get_task_children_financials_info($task){
	$financials = array(
		'executed_cost' => 0,
		'earned_value' => 0,
		'estimated_cost' => 0,
		'estimated_price' => 0
	);
	$subtasks = $task->getSubTasks(false, false, true);
	foreach($subtasks as $t) {
		foreach($financials as $k => $v){
			$financials[$k] += $t->getColumnValue($k);
		}
	}

	return $financials;
}

function get_task_estimated_executed_view_info($object){
	$result = '';

	$currency_sym = config_option('currency_code');
	$can_see_billing_info = true;
	Hook::fire('get_can_see_billing_information', array('user'=>logged_user()), $can_see_billing_info);
	$can_see_cost_info = true;
	Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $can_see_cost_info);


	if($can_see_cost_info){
		$estimated_cost = $object->getColumnValue('estimated_cost');
		$formatted_estimated_cost = format_money_amount($estimated_cost, $currency_sym);
		$executed_cost = $object->getColumnValue('executed_cost');
		$formatted_executed_cost = format_money_amount($object->getColumnValue('executed_cost'), $currency_sym);

		$red_text = $executed_cost > $estimated_cost ? 'style="color: #FF0000;"' : '';

		$result .= "<table><tr><td class='bold task-estimated-label'>".lang('estimated_cost').":&nbsp;</td><td>".$formatted_estimated_cost."</td></tr></table>";
		$result .= "<table><tr><td class='bold task-estimated-label'>".lang('executed_cost').":&nbsp;</td><td ".$red_text.">".$formatted_executed_cost."</td></tr></table>";
	}

	if($can_see_billing_info){
		$estimated_price = format_money_amount($object->getColumnValue('estimated_price'), $currency_sym);
		$earned_value = format_money_amount($object->getColumnValue('earned_value'), $currency_sym);

		
		$result .= "<table><tr><td class='bold task-estimated-label'>".lang('estimated_price').":&nbsp;</td><td>".$estimated_price."</td></tr></table>";
		$result .= "<table><tr><td class='bold task-estimated-label'>".lang('earned_value').":&nbsp;</td><td>".$earned_value."</td></tr></table>";
	}

	return $result;
}


