<?php

function get_member_budgeted_financials($member_id){

    $member_financials = array('budgeted_price' => 0, 'budgeted_cost' => 0);
	$labor_financials = get_labor_budgeted_financials_for_member($member_id);
    $member_financials['budgeted_price'] += $labor_financials['budgeted_price'];
    $member_financials['budgeted_cost'] += $labor_financials['budgeted_cost'];
   
    if(Plugins::instance()->isActivePlugin('expenses2')){
        $expenses_financials = get_expenses_budgeted_financials_for_member($member_id);
        $member_financials['budgeted_price'] += $expenses_financials['budgeted_price'];
        $member_financials['budgeted_cost'] += $expenses_financials['budgeted_cost'];
    }
    
    return $member_financials;
}

function get_labor_budgeted_financials_for_member($member_id){
    $labor_financials = array('budgeted_price' => 0, 'budgeted_cost' => 0);
	$task_ids = "GROUP_CONCAT(task_id) AS task_ids";
    $total_estimated_cost = "SUM(estimated_cost) AS total_estimated_cost ";
    $total_estimated_price = "SUM(estimated_price) AS total_estimated_price ";

    $tasks = ProjectTasks::instance()->listing(array(
        "select_columns" => array("o.id as task_id", "estimated_cost", "estimated_price"),
        "query_wraper_start" => "SELECT $task_ids, $total_estimated_cost, $total_estimated_price FROM (",
        "query_wraper_end" => ") AS pending_calc",
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        "count_results" => false,
        "member_ids" => array($member_id),
        "extra_conditions" => " AND `parent_id`=0"
    ))->objects;  
    
    $labor_financials['budgeted_price'] += $tasks[0]['total_estimated_price'] ? $tasks[0]['total_estimated_price'] : 0;
    $labor_financials['budgeted_cost'] += $tasks[0]['total_estimated_cost'] ? $tasks[0]['total_estimated_cost'] : 0;

	// As tasks financials contains the related expenses amounts, we have to subtract them from its financials to get the labor amounts
	if (Plugins::instance()->isActivePlugin('expenses2')) {
		// get the totals of the expenses related to the involved tasks
		$task_ids = explode(',', $tasks[0]['task_ids']);
		$expense_task_totals = Expenses::getBudgetedExpensesTotalsByTasks($task_ids);

		// subtract the expense totals to get the labor totals
		$labor_financials['budgeted_cost'] -= $expense_task_totals['total_cost'];
		$labor_financials['budgeted_price'] -= $expense_task_totals['total_price'];
	}

    return $labor_financials;
}

function get_expenses_budgeted_financials_for_member($member_id, $exp_extra_conditions = ''){
    $expenses_financials = array('budgeted_price' => 0, 'budgeted_cost' => 0);
    
    $expenses = Expenses::instance()->listing(array(
        "select_columns" => array("e.object_id"),
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        "count_results" => false,
        "extra_conditions" => $exp_extra_conditions,
        "member_ids" => array($member_id)
    ))->objects;

    $expense_ids = is_array($expenses) ? array_column($expenses, 'object_id') : array();

    if(count($expense_ids)){
        $expense_ids_string = '(' . implode(',', $expense_ids) . ')';
        $extra_condition = ' AND expense_id IN '.$expense_ids_string;
        $total_estimated_cost = "SUM(total_cost) AS total_estimated_cost ";
        $total_estimated_price = "SUM(total_price) AS total_estimated_price ";

        $expense_item_totals = ExpenseItems::instance()->listing(array(
            "select_columns" => array("total_price", "total_cost"),
            "query_wraper_start" => "SELECT $total_estimated_cost, $total_estimated_price FROM (",
            "query_wraper_end" => ") AS pending_calc",
            "extra_conditions" => $extra_condition,
            "count_results" => false,
            "raw_data" => true,
            "fire_additional_data_hook" => false,
            "ignore_context" => true
        ))->objects;

        $expenses_financials['budgeted_price'] += $expense_item_totals[0]['total_estimated_price'] ? $expense_item_totals[0]['total_estimated_price'] : 0;
        $expenses_financials['budgeted_cost'] += $expense_item_totals[0]['total_estimated_cost'] ? $expense_item_totals[0]['total_estimated_cost'] : 0;
    }

    return $expenses_financials;
}

function get_additional_calculated_project_financial_fields($member) {

    $member_id = $member->getId();
    $member_object_id = $member->getObjectId();
    $member_data = array();

    $member_totals = MemberTotals::instance()->findOne(array('conditions' => array('`member_id`="'.$member_id.'"')));
    $member_has_totals = $member_totals instanceof MemberTotal;

    $total_budget = 0;
    $project_ot = ObjectTypes::findByName('project');
    $budget_custom_property = CustomProperties::instance()->findOne(array('conditions' => '`object_type_id`='.$project_ot->getId(). ' AND `code`="budget_total"'));
    if ($budget_custom_property instanceof CustomProperty) {
        $cp_val = CustomPropertyValues::getCustomPropertyValue($member_object_id, $budget_custom_property->getId());
        if ($cp_val instanceof CustomPropertyValue) {
            $cp_budget = floatval(str_replace(',','', $cp_val->getValue()));
            $total_budget = $cp_budget > 0 ? $cp_budget : 0;
        }
    }
    $total_earned_velue = $member_has_totals ? $member_totals->getColumnValue('executed_overall_total_price') : 0;
    $total_executed_cost = $member_has_totals ? $member_totals->getColumnValue('executed_overall_total_cost') : 0;
    $total_estimated_revenue = $member_has_totals ? $member_totals->getColumnValue('estimated_overall_total_price') : 0;
    $total_estimated_cost = $member_has_totals ? $member_totals->getColumnValue('estimated_overall_total_cost') : 0;
    
    // Earned value (vs. Contract amount) = Total Earned value (Project) / Contract amount
    if ($total_budget > 0) {
        $earned_value_vs_contract_amount = number_format(100 * ($total_earned_velue / $total_budget), 2) . ' %';
    } else {
        $earned_value_vs_contract_amount = '-';
    }
    
    // Remaining budgeted (vs. Contract amount) = Contract amount - Executed price (Earned value)
    $remaining_budgeted_vs_contract_amount = $total_budget - $total_earned_velue;

    // Earned value (vs. Estimated price) = Total Earned value (Project) / Total Estimated revenue (Project)
    if ($total_estimated_revenue > 0) {
        $earned_value_vs_estimated_price = number_format(100 * ($total_earned_velue / $total_estimated_revenue), 2) . ' %';
    } else {
        $earned_value_vs_estimated_price = '-';
    }

    // Remaining budgeted (vs. Estimated price) = Total Estimated revenue (Project) - Executed price (Earned value) (edited) 
    $remaining_budgeted_vs_estimated_price = $total_estimated_revenue - $total_earned_velue;

    // Budgeted cost consumption = Total Executed cost (Project) / Total Estimated Cost (Project)
    if ($total_estimated_cost > 0) {
        $budgeted_cost_consumption = number_format(100 * ($total_executed_cost / $total_estimated_cost), 2) . ' %';
    } else {
        $budgeted_cost_consumption = '-';
    }
    // Remaining budgeted cost = Total Estimated Cost (Project) - Total Executed cost (Project)
    $remaining_budgeted_cost = $total_estimated_cost - $total_executed_cost;
    
    $csymbol = config_option('currency_code');
    $member_data['earned_value_vs_contract_amount'] = $earned_value_vs_contract_amount;
    $member_data['remaining_budgeted_vs_contract_amount'] = format_money_amount($remaining_budgeted_vs_contract_amount, $csymbol);

    $member_data['earned_value_vs_estimated_price'] = $earned_value_vs_estimated_price;
    $member_data['remaining_budgeted_vs_estimated_price'] = format_money_amount($remaining_budgeted_vs_estimated_price, $csymbol);

    $member_data['budgeted_cost_consumption'] = $budgeted_cost_consumption;
    $member_data['remaining_budgeted_cost'] = format_money_amount($remaining_budgeted_cost, $csymbol);

    return $member_data;

}