<?php
Hook::register('advanced_billing');

function advanced_billing_object_definition($type, &$ret) {
    if ($type == 'Timeslot') {
        $ret['is_fixed_cost'] = DATA_TYPE_BOOLEAN;
        $ret['fixed_cost'] = DATA_TYPE_FLOAT;
        $ret['hourly_cost'] = DATA_TYPE_FLOAT;
        $ret['adv_billing_id'] = DATA_TYPE_INTEGER;
        $ret['adv_cost_id'] = DATA_TYPE_INTEGER;
        $ret['cost_currency_id'] = DATA_TYPE_INTEGER;
        $ret['billing_category_status'] = DATA_TYPE_STRING;
    }

    if ($type == 'ProjectTask') {
        $ret['executed_cost'] = DATA_TYPE_FLOAT;
        $ret['earned_value'] = DATA_TYPE_FLOAT;
        $ret['estimated_cost'] = DATA_TYPE_FLOAT;
        $ret['estimated_price'] = DATA_TYPE_FLOAT;
        $ret['is_calculated_executed_cost'] = DATA_TYPE_BOOLEAN;
        $ret['is_calculated_earned_value'] = DATA_TYPE_BOOLEAN;
        $ret['is_calculated_estimated_cost'] = DATA_TYPE_BOOLEAN;
        $ret['is_calculated_estimated_price'] = DATA_TYPE_BOOLEAN;
        $ret['is_billable'] = DATA_TYPE_BOOLEAN;
        $ret['invoicing_status']= DATA_TYPE_STRING;
        $ret['invoice_id']= DATA_TYPE_INTEGER;
        $ret['is_fixed_fee'] = DATA_TYPE_BOOLEAN;
        $ret['earned_value_time_material'] = DATA_TYPE_FLOAT;
        $ret['earned_value_fixed_fee'] = DATA_TYPE_FLOAT;
	}

    if ($type == 'TemplateTask') {
        $ret['is_billable'] = DATA_TYPE_BOOLEAN;
        $ret['is_calculated_estimated_cost'] = DATA_TYPE_BOOLEAN;
        $ret['is_calculated_estimated_price'] = DATA_TYPE_BOOLEAN;
        $ret['estimated_cost'] = DATA_TYPE_FLOAT;
        $ret['estimated_price'] = DATA_TYPE_FLOAT;
    }
}



/**
 * Add estimated time as a possible variable to assign in templates
 */
function advanced_billing_add_custom_properties_as_template_properties($parameters, &$templateObjectProperties) {
    $templateObjectProperties[] = array(
        'id' => 'estimated_price',
        'name' => lang('estimated_price'),
        'type' => DATA_TYPE_FLOAT,
    );
}

/**
 * If the estimated cost or estimated price was calculated or taken from the template task then set is calculated as false
 */
function advanced_billing_after_task_template_param_assigned($parameters, &$copy)
{
    $template_task = array_var($parameters, 'template_task');
    $property = array_var($parameters, 'property');
    if ($template_task instanceof TemplateTask) {
        if ($property == 'estimated_price') {
            $copy->setColumnValue('is_calculated_estimated_price', 0);
        }
    }
}

function advanced_billing_after_setting_all_task_template_params($parameters, &$copy) {
    if ($copy instanceof ProjectTask) {
        $project_ot = ObjectTypes::findByName('project');
        $contract_type_cp = $project_ot instanceof ObjectType ? CustomProperties::getCustomPropertyByCode($project_ot->getId(), 'contract_type') : null;
        $active_contract_type_cp = $contract_type_cp instanceof CustomProperty && !$contract_type_cp->getIsDisabled();

        if($active_contract_type_cp) {
            // Get the project member
            $task_members = $copy->getMembers();
            $project_member = null;
            foreach ($task_members as $member) {
                if ($member instanceof Member && $member->getObjectTypeId() == $project_ot->getId()) {
                    $project_member = $member;
                    break;
                }
            }
            if(!$project_member instanceof Member) return;

            // Get the contract type cp value
            $contract_type_cp_value = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $contract_type_cp->getId());
            if(!$contract_type_cp_value instanceof CustomPropertyValue) return;

            // Set the is fixed fee and is calculated estimated price
            $contract_type_val = $contract_type_cp_value->getValue();
            if($contract_type_val == 'Fixed fee' || $contract_type_val == 'Percent complete' || $contract_type_val == 'FDEP') {
                $copy->setColumnValue('is_fixed_fee', 1);
                $copy->setColumnValue('is_calculated_estimated_price', 0);
            } else if ($contract_type_val == 'T&M (Not to exceed)') {
                $copy->setColumnValue('is_fixed_fee', 0);
                $copy->setColumnValue('is_calculated_estimated_price', 0);
            } else if ($contract_type_val == 'T&M') {
                $copy->setColumnValue('is_fixed_fee', 0);
                $copy->setColumnValue('is_calculated_estimated_price', 1);
            }
        }
    }
}

function advanced_billing_additional_system_columns($manager, &$columns)
{
    if ($manager instanceof Timeslots) {
        $columns[] = 'is_fixed_cost';
        $columns[] = 'rate_currency_id';
        $columns[] = 'cost_currency_id';
    }
}

function advanced_billing_more_columns_to_aggregate_in_totals($manager, &$columns)
{
    if ($manager instanceof Timeslots) {
        $columns['fixed_cost'] = array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'cost_currency_id', 'group_by' => 'cost_currency_id');
    }

    if ($manager instanceof ProjectTasks) {
        $columns['executed_cost'] = array('operation' => 'sum', 'format' => 'money');
        $columns['earned_value'] = array('operation' => 'sum', 'format' => 'money');
        $columns['estimated_cost'] = array('operation' => 'sum', 'format' => 'money');
        $columns['estimated_price'] = array('operation' => 'sum', 'format' => 'money');
    }

    if ($manager instanceof Projects) {
        $money_format = array('operation' => 'sum', 'format' => 'money');
        $time_format = array('operation' => 'sum', 'format' => 'minutes');

        // Labor
        $columns['estimated_labor_minutes'] = $time_format;
        $columns['estimated_labor_billable_price'] = $money_format;
        $columns['estimated_labor_non_billable_price'] = $money_format;
        $columns['estimated_labor_total_price'] = $money_format;
        $columns['estimated_labor_billable_cost'] = $money_format;
        $columns['estimated_labor_non_billable_cost'] = $money_format;
        $columns['estimated_labor_total_cost'] = $money_format;
        $columns['executed_labor_minutes'] = $time_format;
        $columns['executed_labor_billable_price'] = $money_format;
        $columns['executed_labor_non_billable_price'] = $money_format;
        $columns['executed_labor_total_price'] = $money_format;
        $columns['executed_labor_billable_cost'] = $money_format;
        $columns['executed_labor_non_billable_cost'] = $money_format;
        $columns['executed_labor_total_cost'] = $money_format;
        // Expenses
        $columns['estimated_expense_billable_price'] = $money_format;
        $columns['estimated_expense_non_billable_price'] = $money_format;
        $columns['estimated_expense_total_price'] = $money_format;
        $columns['estimated_expense_billable_cost'] = $money_format;
        $columns['estimated_expense_non_billable_cost'] = $money_format;
        $columns['estimated_expense_total_cost'] = $money_format;
        $columns['executed_expense_billable_price'] = $money_format;
        $columns['executed_expense_non_billable_price'] = $money_format;
        $columns['executed_expense_total_price'] = $money_format;
        $columns['executed_expense_billable_cost'] = $money_format;
        $columns['executed_expense_non_billable_cost'] = $money_format;
        $columns['executed_expense_total_cost'] = $money_format;
        // Overall
        $columns['estimated_overall_billable_price'] = $money_format;
        $columns['estimated_overall_non_billable_price'] = $money_format;
        $columns['estimated_overall_total_price'] = $money_format;
        $columns['estimated_overall_billable_cost'] = $money_format;
        $columns['estimated_overall_non_billable_cost'] = $money_format;
        $columns['estimated_overall_total_cost'] = $money_format;
        $columns['executed_overall_billable_price'] = $money_format;
        $columns['executed_overall_non_billable_price'] = $money_format;
        $columns['executed_overall_total_price'] = $money_format;
        $columns['executed_overall_billable_cost'] = $money_format;
        $columns['executed_overall_non_billable_cost'] = $money_format;
        $columns['executed_overall_total_cost'] = $money_format;
        // Invoiced
        $columns['invoiced_total'] = $money_format;
        $columns['invoiced_subtotal'] = $money_format;
        $columns['invoiced_paid_amount'] = $money_format;
        $columns['invoiced_due_amount'] = $money_format;
    }
}


function advanced_billing_delete_member($member, &$ignored)
{

    if ($member instanceof Member) {
        DB::execute("DELETE FROM " . TABLE_PREFIX . "billing_definition_members WHERE member_id=" . $member->getId());
    }

}



function advanced_billing_timeslot_list_additional_actions($params, &$actions)
{
    if (can_manage_billing(logged_user())) {
        $actions[] = array(
            'url' => '#',
            'onclick' => "og.advanced_billing.recalculate_timeslots_billing('" . array_var($params, 'panel_id') . "');",
            'cls' => 'ico-billing',
            'text' => lang('recalculate billing'),
        );
    }
}


function advanced_billing_custom_reports_override_column_format($parameters, &$value)
{
    $manager = array_var($parameters, 'manager');
    $field = array_var($parameters, 'field');
    $object = array_var($parameters, 'object');
    if ($manager instanceof Timeslots) {
        if ($field == 'fixed_billing' || $field == 'fixed_cost' || $field == 'hourly_billing' || $field == 'hourly_cost') {
            if ($field == 'fixed_billing' || $field == 'hourly_billing') {
                $currency = Currencies::instance()->getCurrency($object->getColumnValue('rate_currency_id'));
            } else if ($field == 'fixed_cost' || $field == 'hourly_cost') {
                $currency = Currencies::instance()->getCurrency($object->getColumnValue('cost_currency_id'));
            }
            $csymbol = null;
            if ($currency instanceof Currency) {
                $csymbol = $currency->getSymbol();
            } else {
                $csymbol = config_option('currency_code');
            }
            $value = format_money_amount($object->getColumnValue($field), $csymbol);
        }

        if ($field == 'bill_total_price' || $field == 'bill_total_cost' || $field == 'bill_billing_rate') {
            $object_billing = ObjectBillings::instance()->findByRelObjectId($object->getRelObjectId());
            if (!is_null($object_billing)) {
                if ($field == 'bill_total_price') {
                    $currency = Currencies::instance()->findById($object_billing->getRateCurrencyId());
                } else if ($field == 'bill_total_cost') {
                    $currency = Currencies::instance()->findById($object_billing->getCostCurrencyId());
                }

                $csymbol = null;
                if (isset($currency) && $currency instanceof Currency) {
                    $csymbol = $currency->getSymbol();
                } else {
                    $csymbol = config_option('currency_code');
                }
                $fieldAux = substr($field, 5);
                $value = format_money_amount($object_billing->getColumnValue($fieldAux), $csymbol);
            }
        }




    }

    if ($manager instanceof ProjectTasks) {

        $object_billing = ObjectBillings::instance()->findByRelObjectId($object->getObjectId());
        if ($field == 'bill_billing_rate' || $field == 'bill_cost_rate') {
            if (!is_null($object_billing)) {
                if ($field == 'bill_billing_rate') {
                    $currency = Currencies::instance()->getCurrency($object_billing->getRateCurrencyId());
                } else if ($field == 'bill_cost_rate') {
                    $currency = Currencies::instance()->getCurrency($object_billing->getCostCurrencyId());
                }
                $csymbol = null;
                if ($currency instanceof Currency) {
                    $csymbol = $currency->getSymbol();
                } else {
                    $csymbol = config_option('currency_code');
                }
                $fieldAux = substr($field, 5);
                $value = format_money_amount($object_billing->getColumnValue($fieldAux), $csymbol);
            }
        }

        if ($field == 'bill_total_price' || $field == 'bill_total_cost') {
            if (!is_null($object_billing)) {
                if ($field == 'bill_total_price') {
                    $currency = Currencies::instance()->findById($object_billing->getRateCurrencyId());
                } else if ($field == 'bill_total_cost') {
                    $currency = Currencies::instance()->findById($object_billing->getCostCurrencyId());
                }

                $csymbol = null;
                if ($currency instanceof Currency) {
                    $csymbol = $currency->getSymbol();
                } else {
                    $csymbol = config_option('currency_code');
                }
                $fieldAux = substr($field, 5);
                $value = format_money_amount($object_billing->getColumnValue($fieldAux), $csymbol);
            }
        }
        if ($field == 'bill_profit_margin') {
            if (!is_null($object_billing)) {
                $currency = Currencies::instance()->findById($object_billing->getRateCurrencyId());
                $csymbol = null;
                if ($currency instanceof Currency) {
                    $csymbol = $currency->getSymbol();
                } else {
                    $csymbol = config_option('currency_code');
                }
                $value = format_money_amount($value, $csymbol);
            }
        }
        if ($field == 'bill_profit_margin_percentage') {
            $percentage = is_numeric($value) && !is_nan($value) ? $value : 0;
            $value = '% ' . round($percentage, 2);
        }
        if (in_array($field, array('executed_cost', 'earned_value', 'estimated_cost', 'estimated_price'))) {
            $csymbol = config_option('currency_code');
            $value = format_money_amount($value, $csymbol);
        }
    }
}

function advanced_billing_custom_reports_get_possible_external_column_values($parameters, &$values)
{
    $ot_id = array_var($parameters, 'ot_id');
    $field = array_var($parameters, 'field');
    $ot = ObjectTypes::instance()->findById($ot_id);
    if ($ot->getName() == 'timeslot' || $ot->getName() == 'task') {
        if ($field == 'adv_billing_id' || $field == 'adv_cost_id') {
            $all_billing_definitions = BillingDefinitions::instance()->findAll();
            foreach ($all_billing_definitions as $def) {
                $values[] = array('id' => $def->getId(), 'name' => $def->getName());
            }
        } else if ($field == 'billing_category_status') {
            $values[] = array('id' => '', 'name' => lang('All'));
            $values[] = array('id' => 'conflict', 'name' => lang('conflict'));
            $values[] = array('id' => 'solved', 'name' => lang('solved'));
        }
    }
}

// remove columns that are not wanted in the group by combo
function advanced_billing_validate_report_group_by_columns($params, &$objectColumns)
{
    $ot = array_var($params, 'ot');

    if ($ot instanceof ObjectType && $ot->getName() == 'timeslot') {
        $cols = array();
        foreach ($objectColumns as $k => &$col) {
            if (!in_array($col, array('adv_billing_id', 'adv_cost_id', 'fixed_billing', 'fixed_cost', 'hourly_billing', 'hourly_cost', 'worked_time', 'subtract', 'description'))) {
                $cols[] = $col;
            }
        }
        $objectColumns = $cols;
    }
}


function advanced_billing_additional_external_columns($manager, &$columns)
{
    if ($manager instanceof Timeslots) {
        $columns[] = 'adv_billing_id';
        $columns[] = 'adv_cost_id';
        $columns[] = 'billing_category_status';
    }
}


function advanced_billing_custom_reports_get_external_column_value($parameters, &$value)
{
    $external_id = array_var($parameters, 'external_id');
    $manager = array_var($parameters, 'manager');
    $field = array_var($parameters, 'field');
    $object = array_var($parameters, 'object');

    if ($manager instanceof Timeslots) {
        if ($field == 'adv_billing_id' || $field == 'adv_cost_id') {
            $billing_def = BillingDefinitions::instance()->getById($external_id);

            if ($billing_def instanceof BillingDefinition) {
                $value = $billing_def->getName();
            } else {
                $value = "";
            }
        } else if ($field == 'billing_category_status') {
            if ($external_id != '') {
                $value = lang($external_id);
            }
        }
    }
}


function advanced_billing_additional_task_timeslot_class($timeslot, &$add_cls)
{
    $cat_status = $timeslot->getColumnValue('billing_category_status');
    if ($cat_status == 'conflict') {
        $add_cls = "adv-billing-conflict";
    }
}

//Revise:
//I couldn't find where this function is ever used
//Parameter "ignored" is not used inside the function
function advanced_billing_timeslots_list_additional_sections($ignored)
{
    tpl_display(get_template_path('timeslots_with_conflict_section', 'timeslots_list', 'advanced_billing'));
}

function advanced_billing_edit_timeslot_form_additional_fields($object, &$categories)
{
    $cat_status = $object->getColumnValue('billing_category_status');
    if ($cat_status == 'conflict') {
        $statuses = array('conflict', 'solved');
        $options = array();
        foreach ($statuses as $st) {
            $attr = $cat_status == $st ? array('selected' => 'selected') : null;
            $options[] = option_tag(lang($st), $st, $attr);
        }
        $before_inputs = "<br />";
        $before_inputs .= '<span class="bold" style="min-width:150px;display:inline-block;">' . lang('billing status') . ':</span>';
        $before_inputs .= select_box('timeslot[billing_category_status]', $options);
        tpl_assign('before_inputs', $before_inputs);
    }

    $sep = '<div style="margin-top:25px; border-bottom:1px solid #ccc;" class="bold">' . lang('billing information') . '</div>';

    tpl_assign('separator', $sep);
    tpl_assign('timeslot', $object);
    $hide_time_billing_tab = config_option('hide_billing_tab_in_timeslot');
    if (!$hide_time_billing_tab) {
        echo tpl_fetch(get_template_path('edit_timestlot_billing_tab', 'billing_definition', 'advanced_billing'));
    }
}

function advanced_billing_object_edit_categories($object, &$categories)
{

    if ($object instanceof Timeslot && can_manage_billing(logged_user())) {

        $cat_status = $object->getColumnValue('billing_category_status');
        if ($cat_status == 'conflict') {
            $statuses = array('conflict', 'solved');
            $options = array();
            foreach ($statuses as $st) {
                $attr = $cat_status == $st ? array('selected' => 'selected') : null;
                $options[] = option_tag(lang($st), $st, $attr);
            }
            $before_inputs = '<label>' . lang('billing status') . ':</label>';
            $before_inputs .= select_box('timeslot[billing_category_status]', $options);
            tpl_assign('before_inputs', $before_inputs);
        }

        tpl_assign('timeslot', $object);
        $hide_time_billing_tab = config_option('hide_billing_tab_in_timeslot');

        if (!$hide_time_billing_tab) {
            $categories[] = array(
                'id' => 'billing_info',
                'name' => lang('billing'),
                'required' => false,
                'visible' => true,
                'content' => tpl_fetch(get_template_path('edit_timestlot_billing_tab', 'billing_definition', 'advanced_billing')),
            );
        }

    }

    if ($object instanceof ProjectTask) {
        $show_financials_tab = config_option('show_financial_tab_in_task_form');
        if ($show_financials_tab) {
            $can_see_billing_info = true;
            Hook::fire('get_can_see_billing_information', array('user' => logged_user()), $can_see_billing_info);
            $can_see_cost_info = true;
            Hook::fire('get_can_see_cost_information', array('user' => logged_user()), $can_see_cost_info);
            if ($can_see_billing_info || $can_see_cost_info) {
                tpl_assign('task', $object);
                $categories[] = array(
                    'id' => 'task_financials_tab',
                    'name' => lang('financials'),
                    'required' => false,
                    'visible' => true,
                    'content' => tpl_fetch(get_template_path('task_form_financials_tab', 'task', 'advanced_billing'))
                );
            }
        }

    } else if ($object instanceof TemplateTask) {

        $show_financials_tab = config_option('show_financial_tab_in_task_form');
        if ($show_financials_tab) {
            tpl_assign('task', $object);
            $categories[] = array(
                'id' => 'template_task_financials_tab',
                'name' => lang('financials'),
                'required' => false,
                'visible' => true,
                'content' => tpl_fetch(get_template_path('template_task_form_financials_tab', 'task', 'advanced_billing'))
            );

        }

    } else {
        /*
            $object_billing = ObjectBillings::findByRelObjectId($object->getObjectId());
            if ($object_billing instanceof ObjectBilling){

                $cat_status = $object_billing->getColumnValue('billing_category_status');
                if ($cat_status == 'conflict') {
                    $statuses = array('conflict','solved');
                    $options = array();
                    foreach ($statuses as $st) {
                        $attr = $cat_status == $st ? array('selected' => 'selected') : null;
                        $options[] = option_tag(lang($st), $st, $attr);
                    }
                    $before_inputs = '<label>'. lang('billing status') .':</label>';
                    $before_inputs .= select_box('timeslot[billing_category_status]', $options);
                    tpl_assign('before_inputs', $before_inputs);
                }
            }else{
                $object_billing = new ObjectBilling();

            }
            tpl_assign('objectBilling', $object_billing);
            $categories[] = array(
                'id' => 'billing_info',
                'name' => lang('billing'),
                'required' => false,
                'visible' => true,
                'content' => tpl_fetch(get_template_path('edit_object_billing_tab', 'billing_definition', 'advanced_billing')),
            );*/
    }
}

function advanced_billing_render_billing_categories_admin_link($ignored, &$ret)
{
    // link to billing categories list

    $onclick = "og.openLink(og.getUrl('billing_definition', 'init')); return false;"; //, {caller:'billing-tab'}

    echo '<div class="configCategory odd">
			<h2><a class="internalLink" href="#" onclick="' . $onclick . '">' . lang("billing categories") . '</a></h2>
			<div class="configCategoryDescription">' . lang("billing categories description") . '</div>
		  </div>';

    // dimensions to apply billing
    echo '<div class="configCategory">
			<h2>' . lang("dimensions that can be applied to billing categories") . '</h2>';

    // get checked dimensions
    $co = ConfigOptions::getByName('adv_billing_dimensions');

    // render checked dimensions
    echo $co->getConfigHandler()->do_render('adv_billing_dimensions', array('onchange_fn' => 'og.onAdvancedBillingDimensionChanged'));

    echo '<div class="clear"></div></div>';

}

function advanced_billing_set_non_billable_property_using_old_task_and_new_task($params, &$ret) {
    $old_task = array_var($params, 'old_task'); 
    $new_task = array_var($params, 'new_task');

    if($old_task instanceof ProjectTask && $new_task instanceof ProjectTask){
        Env::useHelper('additional_tasks_columns_functions', 'advanced_billing');
        $changed_to_non_billable = $old_task->getColumnValue('is_billable') != $new_task->getColumnValue('is_billable') && $new_task->getColumnValue('is_billable') == 0;
        $changed_to_fixed_fee = $old_task->getColumnValue('is_fixed_fee') != $new_task->getColumnValue('is_fixed_fee') && $new_task->getColumnValue('is_fixed_fee') == 1;
        if($changed_to_non_billable){
            set_task_to_non_billable_and_update_related_objects($new_task);
        } else if ($changed_to_fixed_fee) {
            set_related_time_and_expenses_to_task_to_non_billable($new_task);
            $subtasks = $new_task->getSubtasks(false, false, true);
            foreach($subtasks as $subtask) {
                if($subtask instanceof ProjectTask) {
                    set_task_to_non_billable_and_update_related_objects($subtask);
                }
            }
        }
    }
            
}

function advanced_billing_render_more_total_task_times_report_params($report_data, &$ret)
{
    $can_see_cost_info = true;
    Hook::fire('get_can_see_cost_information', array('user' => logged_user()), $can_see_cost_info);
    if (can_manage_billing(logged_user()) && $can_see_cost_info) {
        echo "
			<tr style='height:30px;'>
				<td><span class='bold'>" . lang('show costs information') . "</span></td>
				<td align='left' style='padding-left:10px;'>
					" . checkbox_field('report[show_cost]', array_var($report_data, 'show_cost', false), array("id" => "report[show_cost]")) . "
				</td>
			</tr>
		";
    }
}

function advanced_billing_after_add_to_members($object, &$ret)
{
    if ($object instanceof Timeslot && $object->getColumnValue('invoicing_status') != 'invoiced') {
        Env::useHelper('functions', 'advanced_billing');
        calculate_timeslot_rate_and_cost($object);
    }

    // Calculate and set task's executed cost and value 
    if ($object instanceof Timeslot) {
        $task_id = $object->getRelObjectId();
        $task = ProjectTasks::instance()->findById($task_id);
        if ($task instanceof ProjectTask) {
            Env::useHelper('functions', 'advanced_billing');
            calculate_task_executed_cost_and_price($task);
        }

        Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
        update_executed_labor_info_for_member($object);
    }

    if ($object instanceof ProjectTask) {
        Env::useHelper('functions', 'advanced_billing');
        calculate_task_estimated_and_executed_financials($object);

        Env::useHelper('additional_tasks_columns_functions', 'advanced_billing');
        $is_calculated_estimated_price = $object->getColumnValue('is_calculated_estimated_price');
        $has_parent_with_quoted_price = has_parent_with_quoted_price($object);
        if(!$is_calculated_estimated_price || $has_parent_with_quoted_price){
            Env::useHelper('additional_tasks_columns_functions', 'advanced_billing');
            update_estimated_price_for_subtasks_with_quoted_parent_task($object); 
        }
    }
}

function advanced_billing_after_remove_members_from_object($object, $removedMemebersIds)
{
    if ($object instanceof Timeslot || $object instanceof ProjectTask) {
        Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
        $project_ot_id = ObjectTypes::findByName('project')->getId();

        foreach ($removedMemebersIds as $memberId) {
            $member = Members::instance()->findById($memberId);
            if ($member->getObjectTypeId() == $project_ot_id) {
                if ($object instanceof Timeslot) {
                    update_executed_labor_info_for_member_using_member_id($memberId);
                } else if ($object instanceof ProjectTask) {
                    update_estimated_labor_info_for_member_using_member_id($memberId);
                    if (Plugins::instance()->isActivePlugin('expenses2')) {
                        Env::useHelper('persist_member_expense_totals_functions', 'expenses2');
                        update_estimated_expenses_info_for_member_using_member_id($memberId);
                        update_executed_expenses_info_for_member_by_using_member_id($memberId);
                    }
                }
            }
        }
    }
}


/**
 * After classification a task or a timeslot using drag and drop, recalculate task's financials
 */
function advanced_billing_after_dragdrop_classify($parameters, &$member)
{

    Env::useHelper('functions', 'advanced_billing');

    $objects = array_var($parameters, 'objects');
    $prev_classification = array_var($parameters, 'prev_classification');

    foreach ($objects as $obj) {
        if ($obj instanceof ProjectTask) {

            calculate_task_estimated_and_executed_financials($obj);

        } else if ($obj instanceof Timeslot) {

            if ($obj->getColumnValue('invoicing_status') != 'invoiced') {
                // ensure that billing and cost will be recalculated using the billing category's rate and cost
                $obj->setForceRecalculateBilling(true);
                calculate_timeslot_rate_and_cost($obj);
            }

            $task = $obj->getRelObject();
            if ($task instanceof ProjectTask) {
                calculate_task_executed_cost_and_price($task);
            }

            Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
            update_executed_labor_info_for_member($object);
        }
    }
}

function advanced_billing_calculate_executed_cost_and_price($params, &$object)
{
    if ($object instanceof ProjectTask) {
        Env::useHelper('functions', 'advanced_billing');
        calculate_task_executed_cost_and_price($object);

        Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
        update_estimated_labor_info_for_member($object);
        update_executed_labor_info_for_member($object);
        if (Plugins::instance()->isActivePlugin('expenses2')) {
            Env::useHelper('persist_member_expense_totals_functions', 'expenses2');
            update_estimated_expenses_info_for_member($object);
            update_executed_expenses_info_for_member($object);
        }
    }
}

function advanced_billing_calculate_estimated_executed_financials($params, &$object){
    if($object instanceof ProjectTask){
        Env::useHelper('functions', 'advanced_billing');
        calculate_task_estimated_and_executed_financials($object);
        
        Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
        update_estimated_labor_info_for_member($object);
        update_executed_labor_info_for_member($object);
        if (Plugins::instance()->isActivePlugin('expenses2')) {
            Env::useHelper('persist_member_expense_totals_functions', 'expenses2');
            update_estimated_expenses_info_for_member($object); 
            update_executed_expenses_info_for_member($object);
        }
    }
}

function advanced_billing_after_content_object_untrash($params, $null){
    $object = array_var($params, 'object');
    if ($object instanceof Timeslot) {
        $task_id = $object->getRelObjectId();
        $task = ProjectTasks::instance()->findById($task_id);
        if ($task instanceof ProjectTask) {
            Env::useHelper('functions', 'advanced_billing');
            calculate_task_executed_cost_and_price($task);
        }

        Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
        update_executed_labor_info_for_member($object);
    }
    if ($object instanceof ProjectTask) {
        Env::useHelper('functions', 'advanced_billing');
        calculate_task_estimated_and_executed_financials($object);
    }
}

function advanced_billing_after_content_object_trash($params, &$null)
{
    $object = array_var($params, 'object');
    if ($object instanceof Timeslot) {
        $task_id = $object->getRelObjectId();
        $task = ProjectTasks::instance()->findById($task_id);
        if ($task instanceof ProjectTask) {
            Env::useHelper('functions', 'advanced_billing');
            calculate_task_executed_cost_and_price($task);
        }

        Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
        update_executed_labor_info_for_member($object);
    }
    if ($object instanceof ProjectTask) {
        $parent_task = $object->getParent();
        if ($parent_task instanceof ProjectTask) {
            Env::useHelper('functions', 'advanced_billing');
			$parent_task->dont_calculate_project_financials = $object->dont_calculate_project_financials;
            calculate_task_estimated_and_executed_financials($parent_task);
        } else {
            Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
            update_estimated_labor_info_for_member($object);
            update_executed_labor_info_for_member($object);

            if (Plugins::instance()->isActivePlugin('expenses2')) {
                Env::useHelper('persist_member_expense_totals_functions', 'expenses2');
                update_estimated_expenses_info_for_member($object);
                update_executed_expenses_info_for_member($object);
            }
        }
    }
}

function advanced_billing_calculate_estimated_and_executed_financials($params, $object)
{
    if ($object instanceof ProjectTask) {
        Env::useHelper('functions', 'advanced_billing');
        calculate_task_estimated_and_executed_financials($object);
    }
}

function advanced_billing_after_template_object_instantiation_and_commit($params, $null)
{
    $object = array_var($params, 'object');
    if ($object instanceof ProjectTask) {
        Env::useHelper('functions', 'advanced_billing');
        calculate_task_estimated_and_executed_financials($object);
    }
}

function advanced_billing_after_object_save($object, &$ret)
{
    if ($object instanceof Timeslot && $object->getColumnValue('invoicing_status') != 'invoiced') {

        Env::useHelper('functions', 'advanced_billing');
        calculate_timeslot_rate_and_cost($object);

    }

    if ($object instanceof Timeslot) {
        Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
        update_executed_labor_info_for_member($object);
    }

    if ($object instanceof ProjectTask) {
        $extra_data = array_var($_REQUEST, 'object_billing');
        if (array_var($extra_data, 'invoicing_status') != 'invoiced') {

            $object_billing = ObjectBillings::findByRelObjectId($object->getObjectId());
            if (!$object_billing instanceof ObjectBilling) {
                $object_billing = new ObjectBilling();
            }
            /*if ($object_billing->getContactId() != $object->getAssignedToContactId()) {
                $object_billing->setForceRecalculateBilling(true);
            }*/
            $object_billing->setForceRecalculateBilling(true);

            $object_billing->setRelObjectId($object->getObjectId());
            $object_billing->setTime($object->getTimeEstimate());
            $object_billing->setContactId($object->getAssignedToContactId());
            $object_billing->save();

            // Billing
            $user = Contacts::instance()->findById($object->getAssignedToContactId());
            if ($user instanceof Contact && $user->isUser()) {
                $billing_category_id = $user->getDefaultBillingId();
                $bc = BillingCategories::instance()->findById($billing_category_id);
                if ($bc instanceof BillingCategory) {
                    $object_billing->setAdvBillingId($billing_category_id);
                    $hourly_billing = $bc->getDefaultValue();
                    $object_billing->setBillingRate($hourly_billing);
                    $object_billing->setTotalPrice($hourly_billing * ($object_billing->getTime() / 60));
                    $object_billing->setIsTotalPrice(false);
                }
            }
			$object_billing->save();

			if (Plugins::instance()->isActivePlugin('income')) {
				Env::useHelper('functions', 'income'); 
				$set_objects_non_billable_if_price_is_manual = income_config_option('set_objects_non_billable_if_price_is_manual');
                $is_calculated_estimated_price = $object->getColumnValue('is_calculated_estimated_price');
                $is_fixed_fee = $object->getColumnValue('is_fixed_fee');
				if ($set_objects_non_billable_if_price_is_manual && !$is_calculated_estimated_price && $is_fixed_fee) {
					// set timeslots as non billable
					$timeslots = Timeslots::getTimeslotsByObject($object);
					foreach ($timeslots as $ts) {
						if ($ts->getColumnValue('invoicing_status') != 'invoiced') {
							$ts->changeInvoicingStatus('non_billable');	
						}
					}
					if (Plugins::instance()->isActivePlugin('expenses2')) {
						// set actual expenses as non billable
						$a_expenses = PaymentReceipts::getActualExpensesByTask($object->getId());
						foreach ($a_expenses as $exp) {
							if ($exp->getColumnValue('invoicing_status') != 'invoiced') {
								// do an sql query to prevent infinite loops after doing ->save()
								// the project financial values will be recalculated anyways in the lines below
								DB::execute("
									UPDATE ".TABLE_PREFIX."payment_receipts
									SET is_billable=0, unit_price=0, total_price=0
									WHERE object_id=".$exp->getId()."
								");
							}
						}
						// set budgeted expenses as non billable
						$b_expenses = Expenses::getBudgetedExpensesByTask($object->getId());
						foreach ($b_expenses as $exp) {
							if ($exp->getColumnValue('invoicing_status') != 'invoiced') {
								// do an sql query to prevent infinite loops after doing ->save()
								// the project financial values will be recalculated anyways in the lines below
								DB::execute("
									UPDATE ".TABLE_PREFIX."payment_receipts
									SET is_billable=0
									WHERE object_id=".$exp->getId()."
								");
							}
						}
					}
				}
			}

			Env::useHelper('functions', 'advanced_billing');
			calculate_rate_and_cost($object,$object_billing); 
        }

        Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
        update_estimated_labor_info_for_member($object);
        update_executed_labor_info_for_member($object);

        if (Plugins::instance()->isActivePlugin('expenses2')) {
            Env::useHelper('persist_member_expense_totals_functions', 'expenses2');
            update_estimated_expenses_info_for_member($object);
            update_executed_expenses_info_for_member($object);
        }

    }

}


/**
 * Calculate project financials for projects involved after instantiating a task template
 */
function advanced_billing_after_task_template_instantiation($params, &$null) {
	$template = array_var($params, 'template');
	$objects = array_var($params, 'objects');

	Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
	recalculate_project_financials_of_objects_in_background($objects);
}

/**
 * Calculate project financials for projects involved after deleting a set of tasks
 */
function advanced_billing_after_object_controller_trash($ids, &$ignored) {
	if (!is_array($ids)) {
		$ids = explode(',', $ids);
	}
	$ids = array_filter($ids, 'is_numeric');
	if (count($ids) > 0) {
		$trashed_tasks = ProjectTasks::instance()->findAll(array("conditions" => "id IN (".implode(',', $ids).")"));
		
		Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
		recalculate_project_financials_of_objects_in_background($trashed_tasks);
	}
}

function advanced_billing_save_additional_data_in_related_members($params, $null) {
    $object = array_var($params, 'object');
    if ($object instanceof ProjectTask) {
        Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
        update_percent_completed_for_member($object);
    }
}

function advanced_billing_listing_extra_conditions($ignored, &$extra_conditions)
{
    $controller = array_var($_GET, 'c');
    if ($controller == 'time' && array_var($_GET, 'type_filter') == 'conflict') {
        $extra_conditions .= " AND billing_category_status='conflict' ";
    }
}



function advanced_billing_view_timeslot_render_more_columns($timeslot, &$additional_columns)
{
    $can_see_cost_info = true;
    Hook::fire('get_can_see_cost_information', array('user' => logged_user()), $can_see_cost_info);
    $can_see_billing_info = true;
    Hook::fire('get_can_see_billing_information', array('user' => logged_user()), $can_see_billing_info);
    $show_billing = can_manage_billing(logged_user()) && $can_see_cost_info;

    if ($show_billing) {
        $additional_columns['fixed_cost'] = "";
        if ($timeslot->getColumnValue('fixed_cost') > 0) {
            $c = Currencies::instance()->getCurrency($timeslot->getColumnValue('cost_currency_id'));
            $c_symbol = $c instanceof Currency ? $c->getSymbol() : '';
            $additional_columns['fixed_cost'] = format_money_amount($timeslot->getColumnValue('fixed_cost'), $c_symbol);
        }
    }

    if ($can_see_cost_info) {
        $c = Currencies::instance()->getCurrency($timeslot->getColumnValue('cost_currency_id'));
        $c_symbol = $c instanceof Currency ? $c->getSymbol() : config_option('currency_code');
        $additional_columns['hourly_cost'] = format_money_amount($timeslot->getColumnValue('hourly_cost'), $c_symbol);
    }

    if ($can_see_billing_info) {
        $c = Currencies::instance()->getCurrency($timeslot->getColumnValue('rate_currency_id'));
        $c_symbol = $c instanceof Currency ? $c->getSymbol() : config_option('currency_code');
        $additional_columns['hourly_billing'] = format_money_amount($timeslot->getColumnValue('hourly_billing'), $c_symbol);
    }
}


function advanced_billing_render_view_billings($object, &$additional_html)
{

    // load the default html to print the  billing panel

    $cat_status = $object->getColumnValue('billing_category_status');
    if ($cat_status == 'conflict') {
        $statuses = array('conflict', 'solved');
        $options = array();
        foreach ($statuses as $st) {
            $attr = $cat_status == $st ? array('selected' => 'selected') : null;
            $options[] = option_tag(lang($st), $st, $attr);
        }
        $before_inputs = '<label>' . lang('billing status') . ':</label>';
        $before_inputs .= select_box('object_billing[billing_category_status]', $options);
        tpl_assign('before_inputs', $before_inputs);
    }

    tpl_assign('$object', $object);
    $categories[] = array(
        'id' => 'billing_info',
        'name' => lang('billing'),
        'required' => false,
        'visible' => true,
        'content' => tpl_fetch(get_template_path('edit_object_billing_tab', 'billing_definition', 'advanced_billing')),
    );

}

/**
 * Add currency columns to list in reports
 * @param array $params
 * @param mixed $ret
 */
function advanced_billing_get_allow_columns_for_reports($params, &$ret)
{
    $allowed_columns = $ret[0];
    $object_type = array_var($params, 'object_type');

    if ($object_type == ProjectTasks::instance()->getObjectTypeId() || $object_type == Timeslots::instance()->getObjectTypeId()) {

        $managerCurrencyInstance = ObjectBillings::instance();
        $columnCurr = $managerCurrencyInstance->getReportsColumns();
        foreach ($columnCurr as $key => $value) {

            $allowed_columns[] = array('id' => 'bill_' . $value, 'name' => lang('bill_' . $value), 'type' => $value);
        }
    }

    $can_see_billing_info = true;
    Hook::fire('get_can_see_billing_information', array('user' => logged_user()), $can_see_billing_info);
    $can_see_cost_info = true;
    Hook::fire('get_can_see_cost_information', array('user' => logged_user()), $can_see_cost_info);

    $project_ot = ObjectTypes::findByName('project');
    $client_ot = ObjectTypes::findByName('customer');
    if ($object_type == $client_ot->getId()) {
        if ($can_see_billing_info) {
            $allowed_columns[] = array('id' => 'budgeted_price', 'name' => lang('budgeted revenue'));
        }
        if ($can_see_cost_info) {
            $allowed_columns[] = array('id' => 'budgeted_cost', 'name' => lang('budgeted cost'));
        }
    }

    if ($object_type == $project_ot->getId()) {
        // Labor - estimated
        $allowed_columns[] = array('id' => 'estimated_labor_minutes', 'name' => lang('estimated labor hours'));
        $allowed_columns[] = array('id' => 'percent_completed', 'name' => lang('percent completed'));
        if ($can_see_billing_info) {
            $allowed_columns[] = array('id' => 'estimated_labor_billable_price', 'name' => lang('estimated labor billable price'));
            $allowed_columns[] = array('id' => 'estimated_labor_non_billable_price', 'name' => lang('estimated labor non-billable price'));
            $allowed_columns[] = array('id' => 'estimated_labor_total_price', 'name' => lang('estimated labor total price'));
        }
        if ($can_see_cost_info) {
            $allowed_columns[] = array('id' => 'estimated_labor_billable_cost', 'name' => lang('estimated labor billable cost'));
            $allowed_columns[] = array('id' => 'estimated_labor_non_billable_cost', 'name' => lang('estimated labor non-billable cost'));
            $allowed_columns[] = array('id' => 'estimated_labor_total_cost', 'name' => lang('estimated labor total cost'));
        }
        // Labor - executed
        $allowed_columns[] = array('id' => 'executed_labor_minutes', 'name' => lang('executed labor hours'));
        if ($can_see_billing_info) {
            $allowed_columns[] = array('id' => 'executed_labor_billable_price', 'name' => lang('executed labor billable price'));
            $allowed_columns[] = array('id' => 'executed_labor_non_billable_price', 'name' => lang('executed labor non-billable price'));
            $allowed_columns[] = array('id' => 'executed_labor_total_price', 'name' => lang('executed labor total price'));
        }
        if ($can_see_cost_info) {
            $allowed_columns[] = array('id' => 'executed_labor_billable_cost', 'name' => lang('executed labor billable cost'));
            $allowed_columns[] = array('id' => 'executed_labor_non_billable_cost', 'name' => lang('executed labor non-billable cost'));
            $allowed_columns[] = array('id' => 'executed_labor_total_cost', 'name' => lang('executed labor total cost'));
        }
        // Expenses
        if (Plugins::instance()->isActivePlugin('expenses2')) {
            // estimated
            if ($can_see_billing_info) {
                $allowed_columns[] = array('id' => 'estimated_expense_billable_price', 'name' => lang('estimated expense billable price'));
                $allowed_columns[] = array('id' => 'estimated_expense_non_billable_price', 'name' => lang('estimated expense non-billable price'));
                $allowed_columns[] = array('id' => 'estimated_expense_total_price', 'name' => lang('estimated expense total price'));
            }
            if ($can_see_cost_info) {
                $allowed_columns[] = array('id' => 'estimated_expense_billable_cost', 'name' => lang('estimated expense billable cost'));
                $allowed_columns[] = array('id' => 'estimated_expense_non_billable_cost', 'name' => lang('estimated expense non-billable cost'));
                $allowed_columns[] = array('id' => 'estimated_expense_total_cost', 'name' => lang('estimated expense total cost'));
            }
            // executed
            if ($can_see_billing_info) {
                $allowed_columns[] = array('id' => 'executed_expense_billable_price', 'name' => lang('executed expense billable price'));
                $allowed_columns[] = array('id' => 'executed_expense_non_billable_price', 'name' => lang('executed expense non-billable price'));
                $allowed_columns[] = array('id' => 'executed_expense_total_price', 'name' => lang('executed expense total price'));
            }
            if ($can_see_cost_info) {
                $allowed_columns[] = array('id' => 'executed_expense_billable_cost', 'name' => lang('executed expense billable cost'));
                $allowed_columns[] = array('id' => 'executed_expense_non_billable_cost', 'name' => lang('executed expense non-billable cost'));
                $allowed_columns[] = array('id' => 'executed_expense_total_cost', 'name' => lang('executed expense total cost'));
            }
        }
        // Overall - estimated
        if ($can_see_billing_info) {
            $allowed_columns[] = array('id' => 'estimated_overall_billable_price', 'name' => lang('estimated overall billable price'));
            $allowed_columns[] = array('id' => 'estimated_overall_non_billable_price', 'name' => lang('estimated overall non-billable price'));
            $allowed_columns[] = array('id' => 'estimated_overall_total_price', 'name' => lang('estimated overall total price'));
        }
        if ($can_see_cost_info) {
            $allowed_columns[] = array('id' => 'estimated_overall_billable_cost', 'name' => lang('estimated overall billable cost'));
            $allowed_columns[] = array('id' => 'estimated_overall_non_billable_cost', 'name' => lang('estimated overall non-billable cost'));
            $allowed_columns[] = array('id' => 'estimated_overall_total_cost', 'name' => lang('estimated overall total cost'));
        }
        // Overall - executed
        if ($can_see_billing_info) {
            $allowed_columns[] = array('id' => 'executed_overall_billable_price', 'name' => lang('executed overall billable price'));
            $allowed_columns[] = array('id' => 'executed_overall_non_billable_price', 'name' => lang('executed overall non-billable price'));
            $allowed_columns[] = array('id' => 'executed_overall_total_price', 'name' => lang('executed overall total price'));
        }
        if ($can_see_cost_info) {
            $allowed_columns[] = array('id' => 'executed_overall_billable_cost', 'name' => lang('executed overall billable cost'));
            $allowed_columns[] = array('id' => 'executed_overall_non_billable_cost', 'name' => lang('executed overall non-billable cost'));
            $allowed_columns[] = array('id' => 'executed_overall_total_cost', 'name' => lang('executed overall total cost'));
        }

        // Invcoiced columns
        if (Plugins::instance()->isActivePlugin('income') && $can_see_billing_info) {
            $allowed_columns[] = array('id' => 'invoiced_total', 'name' => lang('invoiced total'));
            $allowed_columns[] = array('id' => 'invoiced_subtotal', 'name' => lang('invoiced subtotal'));
            $allowed_columns[] = array('id' => 'invoiced_paid_amount', 'name' => lang('invoiced paid amount'));
            $allowed_columns[] = array('id' => 'invoiced_due_amount', 'name' => lang('invoiced due amount'));
        }


        // Calculated columns
        $allowed_columns[] = array('id' => 'earned_value_vs_contract_amount', 'name' => lang('earned value vs contract amount'));
        $allowed_columns[] = array('id' => 'remaining_budgeted_vs_contract_amount', 'name' => lang('remaining budgeted vs contract amount'));
        $allowed_columns[] = array('id' => 'earned_value_vs_estimated_price', 'name' => lang('earned value vs estimated price'));
        $allowed_columns[] = array('id' => 'remaining_budgeted_vs_estimated_price', 'name' => lang('remaining budgeted vs estimated price'));
        $allowed_columns[] = array('id' => 'budgeted_cost_consumption', 'name' => lang('budgeted cost consumption'));
        $allowed_columns[] = array('id' => 'remaining_budgeted_cost', 'name' => lang('remaining budgeted cost'));
    }

    $ret[0] = $allowed_columns;
}

/**
 *
 * @param array $params
 * @param mixed $results
 */
function advanced_billing_get_columns_to_header_report($params, &$results)
{

    $field = array_var($params, 'field');
    if (str_starts_with($field, 'bill_')) {
        $results['columns']['names'][$field] = lang($field);
        $results['columns']['order'][] = $field;
        $results['columns']['types'][$field] = ObjectBillings::instance()->getColumnType(substr($field, 5));
        if ($field == 'bill_profit_margin' || $field == 'bill_profit_margin_percentage') {
            $results['columns']['types'][$field] = DATA_TYPE_INTEGER;
        }
    }
}

/**
 *
 * @param array $params
 * @param mixed $row_values
 */
function advanced_billing_get_value_columns_to_body_report($params, &$value)
{

    $field = array_var($params, 'field');
    $object = array_var($params, 'object');

    if (str_starts_with($field, 'bill_')) {
        $fieldAux = substr($field, 5);

        $idObject = $object->getColumnValue('object_id');
        if ($object instanceof Timeslot) {
            $idObject = $object->getColumnValue('rel_object_id');
        }
        $task = ProjectTasks::instance()->findById($idObject);
        if (!is_null($task)) {
            $object_billing = ObjectBillings::instance()->findByRelObjectId($task->getObjectId());
            if (!is_null($object_billing)) {
                if ($fieldAux == 'adv_billing_id' || $fieldAux == 'adv_cost_id') {
                    $billing_def = BillingDefinitions::instance()->findById($object_billing->getColumnValue($fieldAux));
                    if ($billing_def instanceof BillingDefinition) {
                        $value = $billing_def->getName();
                    }
                } else {
                    $value = $object_billing->getColumnValue($fieldAux);
                }
            }

            if ($fieldAux == 'profit_margin') {
                $value = $task->getColumnValue('estimated_price') - $task->getColumnValue('estimated_cost');
            } else if ($fieldAux == 'profit_margin_percentage') {
                $value = ($task->getColumnValue('estimated_price') - $task->getColumnValue('estimated_cost')) * 100 / $task->getColumnValue('estimated_price');
            }
        }
    }
}

function advanced_billing_get_additional_sql_join($params, &$ref)
{
    $class = array_var($params, 'object_class');

    if ($class == 'ProjectTasks') {
        $ref = array(
            'table' => TABLE_PREFIX . "object_billing",
            'jt_field' => 'rel_object_id',
            'e_field' => 'object_id',
            'join_type' => 'left'
        );
    }
    if ($class == 'Timeslots') {
        $ref = array(
            'table' => TABLE_PREFIX . "object_billing",
            'jt_field' => 'rel_object_id',
            'e_field' => 'rel_object_id',
            'join_type' => 'left'
        );
    }
}
function advanced_billing_get_additional_sql_select($params, &$ref)
{
    $class = array_var($params, 'object_class');
    if ($class == 'ProjectTasks' || $class == 'Timeslots') {
        $ref = array('o.*,e.*', 'jt.rel_object_id as bill_rel_object_id ');
    }

}

function advanced_billing_additional_totals_column($params, &$columns)
{
    $object_type = array_var($params, 'object_type');
    if ($object_type->getName() == 'task' || $object_type->getName() == 'timeslot') {
        // prevent adding the same columns more than one time if hook is called again
        if (array_var($_REQUEST, 'additional_totals_column_already_added'))
            return;

        $columns .= ", SUM( (CASE WHEN jt.total_cost is null THEN 0 ELSE  jt.total_cost END )) as bill_total_cost ";
        $columns .= ", SUM( (CASE WHEN jt.total_price is null THEN 0 ELSE  jt.total_price END )) as bill_total_price ";
        $columns .= ", jt.rate_currency_id as bill_rate_currency_id ";
        $columns .= ", jt.cost_currency_id as bill_cost_currency_id ";
        if ($object_type->getName() == 'timeslot') {
            $columns .= ", SUM(jt.total_price - jt.total_cost) as bill_profit_margin ";
        }
        if ($object_type->getName() == 'task') {
            $columns .= ", SUM(estimated_price - estimated_cost) as bill_profit_margin ";
        }
        if ($object_type->getName() == 'timeslot') {
            $columns .= ", ROUND((SUM(jt.total_price - jt.total_cost)*100)/SUM(jt.total_price),2) as bill_profit_margin_percentage";
        }
        if ($object_type->getName() == 'task') {
            $columns .= ", ROUND(SUM(estimated_price - estimated_cost) * 100 / SUM(estimated_price), 2) as bill_profit_margin_percentage";
        }

        $_REQUEST['additional_totals_column_already_added'] = true;
    }

    if ($object_type->getName() == 'timeslot') {
        $columns .= ",GROUP_CONCAT(Distinct(t.object_id)) as tasks_ids ";
    }
}
function advanced_billing_additional_totals_column_as_array($params, &$columns)
{
    $object_type = array_var($params, 'object_type');
    if ($object_type->getName() == 'task' || $object_type->getName() == 'timeslot') {
        $columns[] = "jt.total_cost as bill_total_cost ";
        $columns[] = "jt.total_price as bill_total_price ";
        $columns[] = "jt.rate_currency_id as bill_rate_currency_id ";
        $columns[] = "jt.cost_currency_id as bill_cost_currency_id ";
    }
}


function advanced_billing_list_totals_accumulate_total_rows($params, &$info)
{
    $ot = array_var($params, 'object_type');
    $is_simple_report = array_var($params, 'is_simple_report');
    if ($ot->getHandlerClass() == 'ProjectTasks') {
        $info['bill_profit_margin'] = array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'bill_rate_currency_id');
        $info['bill_profit_margin_percentage'] = array('operation' => 'average', 'format' => 'percent', 'currency_id_col' => '');
    }
}

function advanced_billing_aditional_join_listing_total_query($params, &$joins_sql)
{
    $ot = array_var($params, 'object_type');
    if ($ot instanceof ObjectType && $ot->getHandlerClass() == 'ProjectTasks') {
        $joins_sql .= " LEFT JOIN " . TABLE_PREFIX . "object_billing ob ON e.object_id = ob.rel_object_id ";
    }
}

function advanced_billing_aditional_columns_listing_total_query($params, &$ref)
{
    $ot = array_var($params, 'object_type');
    if ($ot instanceof ObjectType && $ot->getHandlerClass() == 'ProjectTasks') {
        $columns_to_aggregate = array(
            'total_price' => array('operation' => 'sum', 'format' => 'money'),
            'total_cost' => array('operation' => 'sum', 'format' => 'money'),
            'rate_currency_id' => array('operation' => '', 'format' => ''),
            'cost_currency_id' => array('operation' => '', 'format' => ''),
            'profit_margin' => array('operation' => 'rest', 'format' => 'money'),
            'profit_margin_percentage' => array('operation' => 'porcent', 'format' => 'percent'),

        );
        foreach ($columns_to_aggregate as $col => $col_data) {
            $operation = array_var($col_data, 'operation');

            if ($operation == 'sum') {
                $ref[] = "ROUND(SUM(ob.$col),2) as bill_$col";
            }
            if ($operation == 'rest') {
                if ($col == 'profit_margin') {
                    $ref[] = "ROUND(SUM(estimated_price - estimated_cost),2) as bill_$col";
                }
            }
            if ($operation == 'porcent') {
                if ($col == 'profit_margin_percentage') {
                    $ref[] = "ROUND((SUM(estimated_price  - estimated_cost)*100)/SUM(estimated_price),2) as bill_$col";
                }
            }
        }
    }
}

/**
 * Calculate the group totals
 */
function advanced_billing_more_report_group_totals($params, &$level_gr_totals)
{

    $report = array_var($params, 'report');
    $report_ot = array_var($params, 'object_type');

    if ($report_ot && $report_ot->getHandlerClass() == 'Timeslots') {
        if ( isset( $level_gr_totals)) {
            foreach ($level_gr_totals as $k => &$lgt) {
                if (isset($lgt['tasks_ids'])) {
                    $tasks_ids = $lgt['tasks_ids'];
                    $tasks_ids = explode(',', $tasks_ids);
                    $total_estimated_time = 0.0;
                    $total_final_t_price = 0.0;
                    $total_final_t_cost = 0.0;
                    if (!empty($tasks_ids)) {
                        foreach ($tasks_ids as $id) {
                            $columns = array();
                            $report_columns = ReportColumns::getAllReportColumns($report->getId());
                            foreach ($report_columns as $col) {
                                $columns[] = $col->getFieldName();
                            }

                            if (in_array('time_estimate', $columns) || in_array('bill_total_price', $columns) || in_array('bill_total_cost', $columns)) {

                                $project_task = ProjectTasks::instance()->findById($id);
                                if ($project_task instanceof ProjectTask) {
                                    $time_estimate = $project_task->getColumnValue('time_estimate');
                                    $total_estimated_time += $time_estimate;

                                    $ob_billing = ObjectBillings::findByRelObjectId($id);
                                    if ($ob_billing instanceof ObjectBilling) {
                                        $total_price = $ob_billing->getTotalPrice();
                                        $total_cost = $ob_billing->getTotalCost();
                                        if ($total_price > 0) {
                                            $total_final_t_price += $total_price;
                                        }
                                        if ($total_cost > 0) {
                                            $total_final_t_cost += $total_cost;
                                        }
                                    }
                                }
                            }

                        }
                    }
                    $lgt['time_estimate'] = $total_estimated_time;
                    $lgt['bill_total_price'] = $total_final_t_price;
                    $lgt['bill_total_cost'] = $total_final_t_cost;
                }
                ;
            }
        }
    }
}

function advanced_billing_add_dimensions_to_reload_by_object_type($params, &$result)
{
    /* 'Rate schedule'(rate_schedules) and 'Labor category'(hour_types) dimensions don't have
    association between each other. In some cases we need to simulate association between them. */
    $dimension_id = $params['dimension_id'];
    $rate_schedule_dim = Dimensions::findByCode('rate_schedules');
    if ($rate_schedule_dim instanceof Dimension && $dimension_id == $rate_schedule_dim->getId()) {
        $adv_billing_dimensions = config_option('adv_billing_dimensions');
        $hour_type_dim = Dimensions::findByCode('hour_types');
        if (($hour_type_dim instanceof Dimension && in_array($hour_type_dim->getId(), $adv_billing_dimensions)) && in_array($rate_schedule_dim->getId(), $adv_billing_dimensions)) {
            $rate_schedule_ot_id = ObjectTypes::findByName('rate_schedule')->getId();
            if (!isset($result[$rate_schedule_ot_id]))
                $result[$rate_schedule_ot_id] = array();
            $result[$rate_schedule_ot_id][] = $hour_type_dim->getId();
        }
    }
}

/**
 * Filters the labor categoires selectors using the selected rate schedule
 * It looks for the labor categories associated to the rate schedule (or it childs) defined in the billing cateegories settings
 */
function advanced_billing_modify_initial_list_dimension_members($params, &$result_members)
{
    $dimension_id = $params['dimension_id'];
    $dimension = $params['dimension'];
    $filtered_members = $params['filtered_by_members'];

    // labor cat dim
    $hour_type_dim = Dimensions::findByCode('hour_types');

    if ($hour_type_dim instanceof Dimension && $dimension_id == $hour_type_dim->getId()) {
        $rate_schedule_member = null;
        $rate_schedule_dim = Dimensions::findByCode('rate_schedules');
        $adv_billing_dimensions = config_option('adv_billing_dimensions');

        if ($rate_schedule_dim instanceof Dimension && in_array($hour_type_dim->getId(), $adv_billing_dimensions) && in_array($rate_schedule_dim->getId(), $adv_billing_dimensions)) {
            foreach ($filtered_members as $filter_m) {
                if ($filter_m instanceof Member && $filter_m->getDimensionId() == $rate_schedule_dim->getId()) {

                    $hour_type_id = $hour_type_dim->getId();
                    $member_id = $filter_m->getId();

                    // get rate schedule childs
                    $all_children_ids = $filter_m->getAllChildrenIds(true);
                    $member_ids_array = array_merge(array($member_id), $all_children_ids);

                    // build the rate schedule member ids string to use in the query
                    $member_ids_csv = implode(',', $member_ids_array);

                    $sql = "
						SELECT distinct `member_id`
						FROM `" . TABLE_PREFIX . "billing_definition_members`
						WHERE `dimension_id` = $hour_type_id AND 
                   	 		`definition_id` IN (
								SELECT distinct `definition_id` 
								FROM `" . TABLE_PREFIX . "billing_definition_members` 
								WHERE `member_id` IN ($member_ids_csv)
							)
					";

                    $sql_result = DB::execute($sql);
                    $rows = $sql_result->fetchAll();
                    $member_ids_array = array_column($rows, 'member_id');
                    if (!empty($member_ids_array)) {
                        $member_ids = implode(',', array_column($rows, 'member_id'));
                    } else {
                        $member_ids = '';
                    }

                    // also include the labor categories parents
                    $children_members = Members::instance()->findAll(array("conditions" => " id IN (" . $member_ids . ")"));
                    $all_parents = Members::getAllParentsInHierarchy($member_ids_array);
                    $result_members = array_merge($children_members, $all_parents);
                }
            }
        }
    }

}


function advanced_billing_timeslot_info_additional_data($object, &$info)
{
    if ($object instanceof Timeslot) {

        $info['billing_category_name'] = '';
        $info['billing_category_id'] = $object->getColumnValue('adv_billing_id');

        if ($info['billing_category_id'] > 0) {
            $cat = BaseBillingDefinitions::instance()->getById($info['billing_category_id']);
            if ($cat) {
                $info['billing_category_name'] = $cat->getName();
            }
        }

    }
}

function advanced_billing_task_info_additional_data($task, &$info)
{
    $can_see_billing_info = true;
    Hook::fire('get_can_see_billing_information', array('user' => logged_user()), $can_see_billing_info);
    $can_see_cost_info = true;
    Hook::fire('get_can_see_cost_information', array('user' => logged_user()), $can_see_cost_info);
    Env::useHelper('additional_tasks_columns_functions', 'advanced_billing');
    if ($can_see_cost_info)
        add_tasks_show_estimated_cost_column($task, $info);
    if ($can_see_billing_info)
        add_tasks_show_estimated_price_column($task, $info);
    if ($can_see_cost_info)
        add_tasks_show_executed_cost_column($task, $info);
    if ($can_see_billing_info)
        add_tasks_show_executed_price_column($task, $info);
    add_tasks_is_billable_column($task, $info);

    // Add financials info
    $info['executed_cost'] = array_var($task, 'executed_cost', 0);
    $info['estimated_cost'] = array_var($task, 'estimated_cost', 0);
    $info['earned_value'] = array_var($task, 'earned_value', 0);
    $info['estimated_price'] = array_var($task, 'estimated_price', 0);
    $info['is_calculated_executed_cost'] = array_var($task, 'is_calculated_executed_cost', 1);
    $info['is_calculated_estimated_cost'] = array_var($task, 'is_calculated_estimated_cost', 1);
    $info['is_calculated_earned_value'] = array_var($task, 'is_calculated_earned_value', 1);
    $info['is_calculated_estimated_price'] = array_var($task, 'is_calculated_estimated_price', 1);
    $info['is_billable'] = array_var($task, 'is_billable', 1);
    $info['is_fixed_fee'] = array_var($task, 'is_fixed_fee', 0);

    if (!isset($info['additional_data'])) $info['additional_data'] = array();
    if(user_config_option('tasksShowCalculationMethod')){
        $is_fixed_fee = array_var($task, 'is_fixed_fee', 0) ? lang('fixed fee') : lang('T&M');
        $html = '<td><div style="font-size: 12px;">' . $is_fixed_fee . '</div></td>';
        $info['additional_data']['tasksShowCalculationMethod'] = array('html' => $html, 'cls' => '');
    } else {
        $info['additional_data']['tasksShowCalculationMethod'] = array('html' => '', 'cls' => '');
    }


}

function advanced_billing_tasks_user_preferences($ignored, &$array)
{
    $array['tasksShowEstimatedCost'] = (bool) user_config_option('tasksShowEstimatedCost');
    $array['tasksShowEstimatedPrice'] = (bool) user_config_option('tasksShowEstimatedPrice');
    $array['tasksShowExecutedCost'] = (bool) user_config_option('tasksShowExecutedCost');
    $array['tasksShowExecutedPrice'] = (bool) user_config_option('tasksShowExecutedPrice');
    $array['tasksShowIsBillable'] = (bool) user_config_option('tasksShowIsBillable');
    $array['tasksShowCalculationMethod'] = (bool) user_config_option('tasksShowCalculationMethod');
}

function advanced_billing_add_task_group_totals($params, &$totals){
	
	$join_params = array_var($params, 'join_params');
	$conditions = array_var($params, 'conditions');
	$tasks_list_group_by = user_config_option('tasksGroupBy');

	if ($tasks_list_group_by == 'nothing') {
		// if the listing is not grouped then make the query with every root task to get the financials, 
		// we don't need to iterate and define which are the root tasks, they are all the parent tasks

		$conditions .= ($conditions == "" ? "" : " AND ") . "parent_id = 0";

		// only get the listing task ids query so we can use it as a subquery in the conditions when sum totals
		$task_ids_query = ProjectTasks::instance()->listing(array(
			"join_params" => $join_params,
			"extra_conditions" => $conditions,
			"count_results" => false,
			"fire_additional_data_hook" => false,
			"raw_data" => true,
			"only_return_query_string" => true,
			"select_columns" => array(
				"o.id"
			),
		));

		// get the sum of each financial value using the task ids subquery
		$total_row = DB::executeOne("
			SELECT 
				SUM(estimated_cost) as group_estimated_cost,
				SUM(estimated_price) as group_estimated_price,
				SUM(executed_cost) as group_executed_cost,
				SUM(earned_value) as group_earned_value
			FROM ".TABLE_PREFIX."project_tasks
			WHERE object_id IN ($task_ids_query)
		");

		$group_totals_financials = $total_row;
		
	} else {
		// for each group we need to determine which are the root tasks, they can be subtasks but be root in current group
		// so we need to iterate and find those tasks to accumulate the financial values
	
		$tasks = ProjectTasks::instance()->listing(array(
			"join_params" => $join_params,
			"extra_conditions" => $conditions,
			"count_results" => false,
			"fire_additional_data_hook" => false,
			"raw_data" => true,
			// only select the columns we need, for performance and memory usage
			"select_columns" => array("e.estimated_cost", "e.estimated_price", "e.executed_cost", "e.earned_value", "e.parent_id","o.id"),
		))->objects;
	
		// tasks by id array to check if it is a root node in the current group
		$group_tasks_by_id = array();
		foreach ($tasks as $t) $group_tasks_by_id[$t['id']] = $t;
	
		$highest_node_tasks = array();
		foreach ($tasks as $t) {
			// If a task's parent_id is 0, it's a root node.
			// If a task's parent_id does not exist in task IDs, it's also a root node.
			if ($t['parent_id'] == 0 || !array_var($group_tasks_by_id, $t['parent_id'])) {
				$highest_node_tasks[$t['id']] = $t;
			}
		}
	
		$group_totals_financials = array(
			'group_estimated_cost' => 0,
			'group_estimated_price' => 0,
			'group_executed_cost' => 0,
			'group_earned_value' => 0,
		);
	
		// calculate financials for each root node
		foreach ($highest_node_tasks as $task) {
			$group_totals_financials['group_estimated_cost'] += $task['estimated_cost'];
			$group_totals_financials['group_estimated_price'] += $task['estimated_price'];
			$group_totals_financials['group_executed_cost'] += $task['executed_cost'];
			$group_totals_financials['group_earned_value'] += $task['earned_value'];
		}
	
	}
	
	// format the values
	$currency_sym = config_option('currency_code');
	$group_estimated_cost = format_money_amount($group_totals_financials['group_estimated_cost'], $currency_sym);
	$group_estimated_price = format_money_amount($group_totals_financials['group_estimated_price'], $currency_sym);
	$group_executed_cost = format_money_amount($group_totals_financials['group_executed_cost'], $currency_sym);
	$group_earned_value = format_money_amount($group_totals_financials['group_earned_value'], $currency_sym);

	// set the values in the result
	$totals['estimated_cost_string'] = $group_estimated_cost;
	$totals['estimated_price_string'] = $group_estimated_price;
	$totals['executed_cost_string'] = $group_executed_cost;
	$totals['earned_value_string'] = $group_earned_value;
	
}

function advanced_billing_add_estimated_executed_info_to_view($object, &$result)
{
    if ($object instanceof ProjectTask) {
        Env::useHelper('functions', 'advanced_billing');
        $result = get_task_estimated_executed_view_info($object);
    }
}

function advanced_billing_update_calculated_and_manual_data($task_data, &$task)
{
    if ($task instanceof ProjectTask) {
        // Set estimated cost value 
        $is_calculated_estimated_cost = array_var($task_data, 'is_calculated_estimated_cost', 1);
        if ($is_calculated_estimated_cost == 0) {
            $estimated_cost_value = clean_formatted_money_amount_for_sql(array_var($task_data, 'estimated_cost_value', 0));
            $task->setColumnValue('estimated_cost', $estimated_cost_value);
        }

        // Set estimated price value
        $is_calculated_estimated_price = array_var($task_data, 'is_calculated_estimated_price', 1);
        if ($is_calculated_estimated_price == 0) {
            $estimated_price_value = clean_formatted_money_amount_for_sql(array_var($task_data, 'estimated_price_value', 0));
            $task->setColumnValue('estimated_price', $estimated_price_value);
        }

        // Set earned value
        $is_calculated_earned_value = array_var($task_data, 'is_calculated_earned_value', 1);
        if ($is_calculated_estimated_price == 0) {
            $earned_value_amount = $task->getColumnValue('estimated_price') * $task->getPercentCompleted() / 100;
            $task->setColumnValue('earned_value', $earned_value_amount);
        }
    }
}

function advanced_billing_task_clone_more_attributes($parameters, &$ignored)
{
    $original = array_var($parameters, 'original');
    $copy = array_var($parameters, 'copy');

    if (!is_null($copy) && !is_null($original)) {
        $copy->setColumnValue('executed_cost', $original->getColumnValue('executed_cost'));
        $copy->setColumnValue('earned_value', $original->getColumnValue('earned_value'));
        $copy->setColumnValue('estimated_cost', $original->getColumnValue('estimated_cost'));
        $copy->setColumnValue('estimated_price', $original->getColumnValue('estimated_price'));
        $copy->setColumnValue('is_calculated_executed_cost', 1);
        $copy->setColumnValue('is_calculated_earned_value', $original->getColumnValue('is_calculated_earned_value'));
        $copy->setColumnValue('is_calculated_estimated_cost', 1);
        $copy->setColumnValue('is_calculated_estimated_price', $original->getColumnValue('is_calculated_estimated_price'));
        $copy->setColumnValue('is_billable', $original->getColumnValue('is_billable'));
    }
}

function advanced_billing_member_listing_additional_data($params, &$listing_object)
{
    Env::useHelper('member_financials_functions', 'advanced_billing');
    $project_ot = ObjectTypes::findByName('project');
    if ($project_ot && array_var($params, 'type_id') == $project_ot->getId()) {
        $csymbol = null;
        if ( isset($currency) && $currency instanceof Currency) {
            $csymbol = $currency->getSymbol();
        } else {
            $csymbol = config_option('currency_code');
        }
        if (isset($listing_object['members'])) {
            foreach ($listing_object['members'] as &$member_data) {
                $member_id = $member_data['id'];

                // Get project member estimated totals
                $member_totals = MemberTotals::instance()->findOne(array('conditions' => array('`member_id`="' . $member_id . '"')));
                $member_has_totals = $member_totals instanceof MemberTotal;
                $member_data['estimated_labor_minutes'] = format_time_column_value(($member_has_totals ? $member_totals->getColumnValue('estimated_labor_minutes') : 0));
                $member_data['percent_completed'] = ($member_has_totals ? $member_totals->getColumnValue('percent_completed') : 0) . ' %';
                $member_data['estimated_labor_billable_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_labor_billable_price') : 0, $csymbol);
                $member_data['estimated_labor_non_billable_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_labor_non_billable_price') : 0, $csymbol);
                $member_data['estimated_labor_total_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_labor_total_price') : 0, $csymbol);
                $member_data['estimated_labor_billable_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_labor_billable_cost') : 0, $csymbol);
                $member_data['estimated_labor_non_billable_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_labor_non_billable_cost') : 0, $csymbol);
                $member_data['estimated_labor_total_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_labor_total_cost') : 0, $csymbol);

                // Get project member executed totals
                $member_data['executed_labor_minutes'] = format_time_column_value(($member_has_totals ? $member_totals->getColumnValue('executed_labor_minutes') : 0));
                $member_data['executed_labor_billable_minutes'] = format_time_column_value(($member_has_totals ? $member_totals->getColumnValue('executed_labor_billable_minutes') : 0));
                $member_data['executed_labor_non_billable_minutes'] = format_time_column_value(($member_has_totals ? $member_totals->getColumnValue('executed_labor_non_billable_minutes') : 0));

                $member_data['executed_labor_billable_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_labor_billable_price') : 0, $csymbol);
                $member_data['executed_labor_non_billable_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_labor_non_billable_price') : 0, $csymbol);
                $member_data['executed_labor_total_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_labor_total_price') : 0, $csymbol);
                $member_data['executed_labor_billable_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_labor_billable_cost') : 0, $csymbol);
                $member_data['executed_labor_non_billable_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_labor_non_billable_cost') : 0, $csymbol);
                $member_data['executed_labor_total_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_labor_total_cost') : 0, $csymbol);

                // Get project member estimated overall totals
                $member_data['estimated_overall_billable_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_overall_billable_price') : 0, $csymbol);
                $member_data['estimated_overall_non_billable_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_overall_non_billable_price') : 0, $csymbol);
                $member_data['estimated_overall_total_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_overall_total_price') : 0, $csymbol);
                $member_data['estimated_overall_billable_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_overall_billable_cost') : 0, $csymbol);
                $member_data['estimated_overall_non_billable_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_overall_non_billable_cost') : 0, $csymbol);
                $member_data['estimated_overall_total_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('estimated_overall_total_cost') : 0, $csymbol);

                // Get project member executed overall totals
                $member_data['executed_overall_billable_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_overall_billable_price') : 0, $csymbol);
                $member_data['executed_overall_non_billable_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_overall_non_billable_price') : 0, $csymbol);
                $member_data['executed_overall_total_price'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_overall_total_price') : 0, $csymbol);
                $member_data['executed_overall_billable_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_overall_billable_cost') : 0, $csymbol);
                $member_data['executed_overall_non_billable_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_overall_non_billable_cost') : 0, $csymbol);
                $member_data['executed_overall_total_cost'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('executed_overall_total_cost') : 0, $csymbol);

                /************************  Calculated columns  ************************/
                $total_budget = 0;
                $budget_custom_property = CustomProperties::instance()->findOne(array('conditions' => '`object_type_id`=' . $project_ot->getId() . ' AND `code`="budget_total"'));
                if ($budget_custom_property instanceof CustomProperty) {
                    $cp_val = CustomPropertyValues::getCustomPropertyValue($member_data['object_id'], $budget_custom_property->getId());
                    if ($cp_val instanceof CustomPropertyValue) {
                        $cp_budget = floatval(str_replace(',', '', $cp_val->getValue()));
                        $total_budget = $cp_budget > 0 ? $cp_budget : 0;
                    }
                }
                $total_earned_velue = $member_has_totals ? $member_totals->getColumnValue('executed_overall_total_price') : 0;
                $total_executed_cost = $member_has_totals ? $member_totals->getColumnValue('executed_overall_total_cost') : 0;
                $total_estimated_revenue = $member_has_totals ? $member_totals->getColumnValue('estimated_overall_total_price') : 0;
                $total_estimated_cost = $member_has_totals ? $member_totals->getColumnValue('estimated_overall_total_cost') : 0;

                // Earned value (vs. Contract amount) = Total Earned value (Project) / Contract amount
                if ($total_budget > 0) {
                    $earned_value_vs_contract_amount = number_format(100 * ($total_earned_velue / $total_budget), 2) . ' %';
                } else {
                    $earned_value_vs_contract_amount = '-';
                }

                // Remaining budgeted (vs. Contract amount) = Contract amount - Executed price (Earned value)
                $remaining_budgeted_vs_contract_amount = $total_budget - $total_earned_velue;

                // Earned value (vs. Estimated price) = Total Earned value (Project) / Total Estimated revenue (Project)
                if ($total_estimated_revenue > 0) {
                    $earned_value_vs_estimated_price = number_format(100 * ($total_earned_velue / $total_estimated_revenue), 2) . ' %';
                } else {
                    $earned_value_vs_estimated_price = '-';
                }

                // Remaining budgeted (vs. Estimated price) = Total Estimated revenue (Project) - Executed price (Earned value) (edited) 
                $remaining_budgeted_vs_estimated_price = $total_estimated_revenue - $total_earned_velue;

                // Budgeted cost consumption = Total Executed cost (Project) / Total Estimated Cost (Project)
                if ($total_estimated_cost > 0) {
                    $budgeted_cost_consumption = number_format(100 * ($total_executed_cost / $total_estimated_cost), 2) . ' %';
                } else {
                    $budgeted_cost_consumption = '-';
                }
                // Remaining budgeted cost = Total Estimated Cost (Project) - Total Executed cost (Project)
                $remaining_budgeted_cost = $total_estimated_cost - $total_executed_cost;


                $member_data['earned_value_vs_contract_amount'] = $earned_value_vs_contract_amount;
                $member_data['remaining_budgeted_vs_contract_amount'] = format_money_amount($remaining_budgeted_vs_contract_amount, $csymbol);

                $member_data['earned_value_vs_estimated_price'] = $earned_value_vs_estimated_price;
                $member_data['remaining_budgeted_vs_estimated_price'] = format_money_amount($remaining_budgeted_vs_estimated_price, $csymbol);

                $member_data['budgeted_cost_consumption'] = $budgeted_cost_consumption;
                $member_data['remaining_budgeted_cost'] = format_money_amount($remaining_budgeted_cost, $csymbol);

                /**********************  End Calculated columns  **********************/
            }
        }

        // format financial group totals
        if (isset( $listing_object['groups_info']['group_totals'] ) && is_array($listing_object['groups_info']['group_totals'])) {
            Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
            $financial_columns = get_all_member_totals_columns();
            $minutes_columns = get_all_member_totals_minutes_columns();
            $new_group_totals = array();
            $csymbol = config_option('currency_code');

            foreach ($listing_object['groups_info']['group_totals'] as $group_id => &$group_totals) {
                foreach ($group_totals as $k => &$v) {
                    if (in_array($k, $financial_columns)) {
                        if (in_array($k, $minutes_columns)) {
                            $v = format_time_column_value($v);
                        } else if ($k == 'percent_completed') {
                            $v = $v . ' %';
                        } else {
                            $v = format_money_amount($v, $csymbol);
                        }
                    }
                }
            }
        }


    }
}

function advanced_billing_template_task_to_task_columns($parameters, &$new_task)
{
    $template_task = array_var($parameters, 'template_task');
    if ($new_task instanceof ProjectTask) {
        $new_task->setColumnValue('is_calculated_executed_cost', 1);
        $new_task->setColumnValue('is_calculated_earned_value', 1);
        $new_task->setColumnValue('is_calculated_estimated_cost', 1);
        $new_task->setColumnValue('is_calculated_estimated_price', 1);
        if ($template_task instanceof TemplateTask) {
            $new_task->setColumnValue('is_billable', $template_task->getColumnValue('is_billable'));

            $new_task->setColumnValue('is_calculated_estimated_price', $template_task->getColumnValue('is_calculated_estimated_price'));
            $new_task->setColumnValue('estimated_price', $template_task->getColumnValue('estimated_price'));
        }
    }
}

function advanced_billing_save_additional_subtask_info($parameters, &$new_subtask) {
	$parent_task = array_var($parameters, 'parent_task');
	if ($new_subtask instanceof ProjectTask) {
		$new_subtask->setColumnValue('is_calculated_executed_cost', 1);
        $new_subtask->setColumnValue('is_calculated_earned_value', 1);
        $new_subtask->setColumnValue('is_calculated_estimated_cost', 1);
        $new_subtask->setColumnValue('is_calculated_estimated_price', 1);
		if ($parent_task instanceof ProjectTask) {
			$new_subtask->setColumnValue('is_billable', $parent_task->getColumnValue('is_billable'));
		}
	}
}

function advanced_billing_before_render_main_custom_properties($params, &$categories)
{

    $object = array_var($params, 'object');
    $genid = array_var($params, 'genid');
    $task_data = array_var($params, 'task_data');

    if ($object instanceof ProjectTask || $object instanceof TemplateTask) {

        tpl_assign('task', $object);
        tpl_assign('genid', $genid);
        tpl_assign('task_data', $task_data);

        echo tpl_fetch(get_template_path('tasks_billable_selector', 'task', 'advanced_billing'));
    }
}

function advanced_billing_view_task_render_more_properties($task, &$null)
{

    $is_billable = $task->getColumnValue('is_billable') ? lang('yes') : lang('no');
    $html = '</br><b>' . lang('billable') . ':</b>&nbsp;<span class="additional-list-col"><b>' . $is_billable . '</b></span>';

    echo $html;


    if($task instanceof ProjectTask) {
        $calc_method = $task->getColumnValue('is_fixed_fee') ? lang('fixed fee') : lang('T&M');
        $calc_html = '<div class="member-path-dim-block" style="font-weight: bold">' . lang('calculation method') .': '. $calc_method . '</div>';

        echo $calc_html;
    }
}

function advanced_billing_get_initial_invoicing_status_for_timeslot($params, &$status)
{
    if (Plugins::instance()->isActivePlugin('income') && Plugins::instance()->isActivePlugin('hour_types')) {
        $timeslot = array_var($params, 'timeslot');
        $task_id = array_var($params, 'task_id');
        $selected_member_ids = array_var($params, 'selected_member_ids', array());
        $task = ProjectTasks::instance()->findById($task_id);

        if ($status == '' && $timeslot->isNew() && $task instanceof ProjectTask) {
            $status = $task->getColumnValue('is_billable') ? 'pending' : 'non_billable';
        } else if ($status == '' && $timeslot->isNew()) {
            $labor_cat_ot_id = ObjectTypes::findByName('hour_type')->getId();
            foreach ($selected_member_ids as $member_id) {
                $member = Members::instance()->findById($member_id);
                if ($member instanceof Member && $member->getObjectTypeId() == $labor_cat_ot_id) {
                    $mcp = MemberCustomProperties::instance()->findOne(array('conditions' => '`object_type_id`=' . $labor_cat_ot_id . ' AND `code`="is_billable"'));
                    if ($mcp instanceof MemberCustomProperty) {
                        $mcp_id = $mcp->getId();
                        $mcp_value = MemberCustomPropertyValues::instance()->findOne(array('conditions' => '`member_id`=' . $member_id . ' AND `custom_property_id`=' . $mcp_id));
                        if ($mcp_value instanceof MemberCustomPropertyValue) {
                            $status = $mcp_value->getValue() == '-1' ? 'non_billable' : 'pending';
                        }
                    }
                }
            }
        }

    }
}

function advanced_billing_get_initial_invoicing_status_for_timeslot_using_members_and_task($params, &$status)
{
    if (Plugins::instance()->isActivePlugin('income') && Plugins::instance()->isActivePlugin('hour_types')) {
        $task_id = array_var($params, 'task_id');
        $selected_member_ids = array_var($params, 'selected_member_ids', array());
        $get_task_members = array_var($params, 'get_task_members', false);
        $ignore_task_billable_config = array_var($params, 'ignore_task_billable_config', false);
        $task = ProjectTasks::instance()->findById($task_id);
        if ($get_task_members && $task instanceof ProjectTask) {
            $selected_member_ids = array();
            $task_members = $task->getMembers();
            foreach ($task_members as $task_member) {
                $selected_member_ids[] = $task_member->getId();
            }
        }
        
        if (!$ignore_task_billable_config && $task instanceof ProjectTask) {
            $status = $task->getColumnValue('is_billable') ? 'pending' : 'non_billable';
        } else if (count($selected_member_ids) > 0) {
            $labor_cat_ot_id = ObjectTypes::findByName('hour_type')->getId();
            foreach ($selected_member_ids as $member_id) {
                $member = Members::instance()->findById($member_id);
                if ($member instanceof Member && $member->getObjectTypeId() == $labor_cat_ot_id) {
                    $mcp = MemberCustomProperties::instance()->findOne(array('conditions' => '`object_type_id`=' . $labor_cat_ot_id . ' AND `code`="is_billable"'));
                    if ($mcp instanceof MemberCustomProperty) {
                        $mcp_id = $mcp->getId();
                        $mcp_value = MemberCustomPropertyValues::instance()->findOne(array('conditions' => '`member_id`=' . $member_id . ' AND `custom_property_id`=' . $mcp_id));
                        if ($mcp_value instanceof MemberCustomPropertyValue) {
                            $status = $mcp_value->getValue() == '-1' ? 'non_billable' : 'pending';
                        }
                    }
                }
            }
        }
    }
}

function advanced_billing_remove_custom_report_columns_from_list($params, &$allowed_columns)
{
    $object_type = $params['object_type'];
    $task_ot = ObjectTypes::findByName('task')->getId();
    $columns_to_remove = array();
    if ($object_type == $task_ot) {
        $columns_to_remove = array(
            'is_calculated_executed_cost',
            'is_calculated_earned_value',
            'is_calculated_estimated_cost',
            'is_calculated_estimated_price'
        );
    }
    if (!empty($columns_to_remove)) {
        foreach ($allowed_columns as $key => $column) {
            if (in_array($column['id'], $columns_to_remove)) {
                unset($allowed_columns[$key]);
            }
        }
    }

    $allowed_columns = array_values($allowed_columns);
}

function advanced_billing_draw_additional_task_html($genid, &$task)
{
    $html = '';
    $show_financials_tab = config_option('show_financial_tab_in_task_form');
    if (!$show_financials_tab) {
        $html .= '<input type="hidden" name="task[is_calculated_estimated_cost]" value="1"/>';
        $html .= '<input type="hidden" name="task[is_calculated_estimated_price]" value="1"/>';
        $html .= '<input type="hidden" name="task[is_calculated_executed_cost]" value="1"/>';
        $html .= '<input type="hidden" name="task[is_calculated_earned_value]" value="1"/>';
    }

    echo $html;
}

function advanced_billing_get_all_member_totals_columns($params, &$financial_columns)
{
    Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
    $financial_columns = get_all_member_totals_columns();
}

function advanced_billing_get_permitted_member_totals_columns($params, &$allowed_columns)
{
    $can_see_billing_info = true;
    Hook::fire('get_can_see_billing_information', array('user' => logged_user()), $can_see_billing_info);
    $can_see_cost_info = true;
    Hook::fire('get_can_see_cost_information', array('user' => logged_user()), $can_see_cost_info);

    // Labor - estimated
    $allowed_columns[] = 'estimated_labor_minutes';
    if ($can_see_billing_info) {
        $allowed_columns[] = 'estimated_labor_billable_price';
        $allowed_columns[] = 'estimated_labor_non_billable_price';
        $allowed_columns[] = 'estimated_labor_total_price';
    }
    if ($can_see_cost_info) {
        $allowed_columns[] = 'estimated_labor_billable_cost';
        $allowed_columns[] = 'estimated_labor_non_billable_cost';
        $allowed_columns[] = 'estimated_labor_total_cost';
    }
    // Labor - executed
    $allowed_columns[] = 'executed_labor_minutes';
    if ($can_see_billing_info) {
        $allowed_columns[] = 'executed_labor_billable_price';
        $allowed_columns[] = 'executed_labor_non_billable_price';
        $allowed_columns[] = 'executed_labor_total_price';
    }
    if ($can_see_cost_info) {
        $allowed_columns[] = 'executed_labor_billable_cost';
        $allowed_columns[] = 'executed_labor_non_billable_cost';
        $allowed_columns[] = 'executed_labor_total_cost';
    }
    // Expenses
    if (Plugins::instance()->isActivePlugin('expenses2')) {
        // estimated
        if ($can_see_billing_info) {
            $allowed_columns[] = 'estimated_expense_billable_price';
            $allowed_columns[] = 'estimated_expense_non_billable_price';
            $allowed_columns[] = 'estimated_expense_total_price';
        }
        if ($can_see_cost_info) {
            $allowed_columns[] = 'estimated_expense_billable_cost';
            $allowed_columns[] = 'estimated_expense_non_billable_cost';
            $allowed_columns[] = 'estimated_expense_total_cost';
        }
        // executed
        if ($can_see_billing_info) {
            $allowed_columns[] = 'executed_expense_billable_price';
            $allowed_columns[] = 'executed_expense_non_billable_price';
            $allowed_columns[] = 'executed_expense_total_price';
        }
        if ($can_see_cost_info) {
            $allowed_columns[] = 'executed_expense_billable_cost';
            $allowed_columns[] = 'executed_expense_non_billable_cost';
            $allowed_columns[] = 'executed_expense_total_cost';
        }
    }
    // Overall - estimated
    if ($can_see_billing_info) {
        $allowed_columns[] = 'estimated_overall_billable_price';
        $allowed_columns[] = 'estimated_overall_non_billable_price';
        $allowed_columns[] = 'estimated_overall_total_price';
    }
    if ($can_see_cost_info) {
        $allowed_columns[] = 'estimated_overall_billable_cost';
        $allowed_columns[] = 'estimated_overall_non_billable_cost';
        $allowed_columns[] = 'estimated_overall_total_cost';
    }
    // Overall - executed
    if ($can_see_billing_info) {
        $allowed_columns[] = 'executed_overall_billable_price';
        $allowed_columns[] = 'executed_overall_non_billable_price';
        $allowed_columns[] = 'executed_overall_total_price';
    }
    if ($can_see_cost_info) {
        $allowed_columns[] = 'executed_overall_billable_cost';
        $allowed_columns[] = 'executed_overall_non_billable_cost';
        $allowed_columns[] = 'executed_overall_total_cost';
    }

    // Invcoiced columns
    if (Plugins::instance()->isActivePlugin('income') && $can_see_billing_info) {
        $allowed_columns[] = 'invoiced_total';
        $allowed_columns[] = 'invoiced_subtotal';
        $allowed_columns[] = 'invoiced_paid_amount';
        $allowed_columns[] = 'invoiced_due_amount';
    }
}

function advanced_billing_compare_previous_and_current_object($params, $prev_object) {
    $previous_object = array_var($params, 'previous_object');
    $current_object_data = array_var($params, 'current_object_data');

    if(Plugins::instance()->isActivePlugin('expenses2') && Plugins::instance()->isActivePlugin('income')) {
        if($previous_object instanceof Expense) {
            $prev_is_billable = $previous_object->getColumnValue('is_billable');
            $current_is_billable = array_var($current_object_data, 'is_billable', 1);
            if($prev_is_billable != $current_is_billable && $current_is_billable == 0) {
                Env::useHelper('functions', 'expenses2');
                set_related_actual_expenses_to_non_billable($previous_object);
            }
        }
    }
}
