<?php

/**
 * MemberGroupedListingController controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class MemberGroupedListingController extends ApplicationController {

	var $plugin_name = "advanced_core";
	
	/**
	 * Construct the DimensionOptionsController
	 *
	 * @access public
	 * @param void
	 * @return DimensionOptionsController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
	} // __construct
	
	
	function listing_grouped($parameters = null) {
	
		$return_the_list = true;
	
		// if parameters not specified => use the request
		if (is_null($parameters)) {
			ajx_current("empty");
			$parameters = $_REQUEST;
			$return_the_list = false;
		}
	
		$member_controller = new MemberController();
	
		// get all variables from parameters array
		$start = array_var($parameters,'start', '0');
		$limit = array_var($parameters,'limit', config_option('files_per_page'));
		$order = array_var($parameters,'sort');
		$order_dir = array_var($parameters,'dir');
		$dimension_id = array_var($parameters, 'dim_id');
		$member_type_id = array_var($parameters, 'type_id');
		$member_subtype_id = array_var($parameters, 'subtype_id');
		$parent_id = array_var($parameters, 'parent_id');
        //this is to bring just the first levels of sonf of the defined parent_id in the otherside just will bring all the element with unde the parent_id
        $just_parent_sons = array_var($parameters, 'just_parent_sons',false);
		$extra_conditions = array_var($parameters, 'extra_conditions');
		$req_group_by = array_var($parameters, 'groupBy');
	
		if (!is_numeric($start)) $start = 0;
		if (!is_numeric($limit)) $limit = config_option('files_per_page');
	
		// find current dimension
		$dimension = Dimensions::instance()->findById($dimension_id);
		if (!$dimension instanceof Dimension) return;
	
		// find member type
		$member_type = ObjectTypes::instance()->findById($member_type_id);

		// customers dimension can be separated from projects or not, we should support both, so in that case we override the dimension
		if (Plugins::instance()->isActivePlugin('crpm')) {
			if ($member_type && $member_type->getName() == 'customer') {
				Env::useHelper('functions', 'crpm');
				$dimension = get_customers_dimension();
			}
		}
	
	
		// dimension associations params
		$assoc_params = $member_controller->build_listing_associated_dimensions_parameters($dimension, $member_type_id);
		$dimension_associations = array_var($assoc_params, 'assocs');
		$associated_dimension_ids = array_var($assoc_params, 'dims');
		$associated_member_ids = array_var($assoc_params, 'assoc_members');
		$member_association_cond = array_var($assoc_params, 'member_assoc_cond');
		$only_first_row_of_joins_conds = array_var($assoc_params, 'only_first_row_of_joins');
	
		// join params to retrieve dimension association columns
		$dimension_association_joins_sql = array_var($assoc_params, 'assoc_joins_sql');
		$dimension_association_sel_cols = array_var($assoc_params, 'assoc_joins_cols');
		$group_by = array_var($assoc_params, 'group_by');


        // parent member conditions
        if ($just_parent_sons == false) {
            $parent_member_cond = $member_controller->build_listing_parent_condition($dimension, $parent_id);
        } else {
            if (!is_null($parent_id)) {
                if (is_array($parent_id)) {
                    $parent_member_cond = "AND mem.parent_member_id IN (" . implode(",", $parent_id) . ")";
                } else {
                    $parent_member_cond = "AND mem.parent_member_id IN ({$parent_id})";
                }
            }
        }


        // member type additional attributes join
		$object_type_join_sql = "";
		$object_type_cols_sql = "";
		if ($member_type instanceof ObjectType) {
			$table_name = trim($member_type->getTableName());
			if ($table_name != '' && checkTableExists(TABLE_PREFIX.$table_name)) {
				$object_type_join_sql = "INNER JOIN ".TABLE_PREFIX."$table_name obj_type_table ON obj_type_table.object_id=mem.object_id";
				$object_type_join_sql .= " INNER JOIN ".TABLE_PREFIX."objects o ON o.id=mem.object_id";
				$object_type_cols_sql = ", obj_type_table.*";
			}
		}
	
		// get order params
		$order_params = $member_controller->build_listing_order_parameters($order, $order_dir, $member_type);
		$order = array_var($order_params, 'order');
		$order_dir = array_var($order_params, 'order_dir');
		$order_join_sql = array_var($order_params, 'order_join_sql');
	
	
		// build member permissions permission conditions
		$pg_array = logged_user()->getPermissionGroupIds();
		if (logged_user()->isAdministrator()) {
			$permission_conditions = "";
		} else {
			$permission_conditions = "AND EXISTS (SELECT cmp.member_id FROM ".TABLE_PREFIX."contact_member_permissions cmp
					WHERE cmp.member_id=mem.id AND cmp.permission_group_id IN (".implode(',',$pg_array)."))";
		}
	
	
		// select columns sql part
		$columns_sql = "SELECT mem.*, mem.id as member_id $object_type_cols_sql $dimension_association_sel_cols";
	
		// from table sql part
		$from_sql = "FROM ".TABLE_PREFIX."members mem";
	
		// joins sql part
		$joins_sql = "
		--	LEFT JOIN ".TABLE_PREFIX."member_additional_data d ON c.object_id=d.customer_id
			$order_join_sql
			$object_type_join_sql
			$dimension_association_joins_sql
		";
		
		if (Plugins::instance()->isActivePlugin('object_subtypes') && $member_subtype_id > 0 && $member_type->getType() == 'dimension_object') {
			$extra_conditions .= "
				AND o.object_subtype_id='$member_subtype_id'
			";
		}

		// sql conditions part
		$all_conditions_sql = "
		mem.dimension_id=".$dimension->getId()." AND mem.archived_by_id=0
			$permission_conditions
			$member_association_cond
			$parent_member_cond
			$extra_conditions
		";

			// main sql query part
		$main_sql = "
			$from_sql
			$joins_sql
			WHERE
			$all_conditions_sql
		";

		if ($req_group_by) {
			$group_by_groups = "GROUP BY gr_id";
				
			if (str_starts_with($req_group_by, 'dimassoc_')) {

				$group_id = str_replace('dimassoc_', '', $req_group_by);
				$groups_joins_sql = "LEFT JOIN ".TABLE_PREFIX."members gr_table ON gr_table.id = mem_pm".$group_id.".property_member_id";
				$group_match_column = "mem_pm".$group_id.".property_member_id";
				$groups_order_by = "gr_table.name";
				$group_col = "gr_table.id";

			} else if (str_starts_with($req_group_by, 'cp_')) {

				if ($member_type->getType() == 'dimension_group') {
					$group_id = str_replace('cp_', '', $req_group_by);
					$groups_joins_sql = "LEFT JOIN ".TABLE_PREFIX."member_custom_property_values gr_table ON gr_table.member_id=mem.id AND gr_table.custom_property_id = $group_id";
					$cp = MemberCustomProperties::instance()->findById($group_id);
					
				} else {
					$group_id = str_replace('cp_', '', $req_group_by);
					$cp = CustomProperties::instance()->findById($group_id);
					if ($cp->getObjectTypeId() == $member_type_id) {

						$groups_joins_sql = "LEFT JOIN ".TABLE_PREFIX."custom_property_values gr_table ON gr_table.object_id=mem.object_id AND gr_table.custom_property_id = $group_id";

					} else if ($cp->getObjectTypeId() == Contacts::instance()->getObjectTypeId()) {

						$groups_joins_sql = "LEFT JOIN ".TABLE_PREFIX."custom_property_values gr_table ON gr_table.object_id=obj_type_table.contact_id AND gr_table.custom_property_id = $group_id";

					}
				}
				
				$group_match_column = "gr_table.value";
				$groups_order_by = "LPAD(COALESCE(gr_table.value,''), 25, ' ')";
				$group_col = "(COALESCE(gr_table.value, ''))";
				
				if (in_array($cp->getType(), array('boolean'))) {
					$group_col = "if (gr_table.value=0,'',(COALESCE(gr_table.value, '')))";
				}

				if (in_array($cp->getType(), array('contact','user'))) {
					$groups_joins_sql .= " LEFT JOIN ".TABLE_PREFIX."objects gr_table_aux ON gr_table_aux.id=gr_table.value ";
					$groups_order_by = "gr_table_aux.name";
				}

			} else if (str_starts_with($req_group_by, 'mem_path')) {

				$group_id = 'mem_path';
				$groups_joins_sql = "LEFT JOIN ".TABLE_PREFIX."members gr_table ON gr_table.id = mem.parent_member_id";
				$group_match_column = "mem.parent_member_id";
				$groups_order_by = "gr_table.name";
				$group_col = "gr_table.id";

			} else {
				$found_col = false;
				
				if ($member_type instanceof ObjectType) {
					$table_name = trim($member_type->getTableName());
					if ($table_name != '' && checkTableExists(TABLE_PREFIX.$table_name)) {
						eval('$managerInstance = ' . $member_type->getHandlerClass() . "::instance();");
						
						if ($managerInstance && in_array($req_group_by, $managerInstance->getColumns())) {
							$group_id = $req_group_by;
							$groups_joins_sql = "";
							$group_match_column = "obj_type_table.$req_group_by";
							$groups_order_by = "obj_type_table.$req_group_by";
							$group_col = "obj_type_table.$req_group_by";
							
							$found_col = true;
						}
					}
				}
				
				if (!$found_col) {
					return $member_controller->listing();
				}
			}

			$groups_columns_sql = str_replace("GROUP_CONCAT", "", $columns_sql);
			$add_groups_cols = "SELECT count(distinct(mem.id)) as gr_count, $groups_order_by as gr_name, $group_col as gr_id, ";
			$groups_columns_sql = str_replace("SELECT", $add_groups_cols, $groups_columns_sql);
				
			$group_list_offset = array_var($_REQUEST, 'group_list_offset', 0);
				
			// groups sql query
			$groups_sql = "
					$groups_columns_sql
					$from_sql
					$joins_sql
					$groups_joins_sql
				WHERE
					$all_conditions_sql
					$group_by_groups
				ORDER BY $groups_order_by ASC
				LIMIT $group_list_offset, $limit
			";

			$group_rows = DB::executeAll($groups_sql);
			$group_count = $group_rows ? count($group_rows) : 0;
				
			if ($group_count >= 5) $gr_max_elems = 10;
			else if ($group_count >= 3) $gr_max_elems = 15;
			else if ($group_count >= 2) $gr_max_elems = 25;
			else $gr_max_elems = 50;
				
			$groups_info = array(
					'groups' => array(), 'groups_order' => array(),
					'max_per_group' => $gr_max_elems, 'last_group_by'=> $req_group_by
			);
				
			$groups_to_query = array();

			$filter_by_group = array_var($_REQUEST, 'filter_by_group');

			if (!is_null($filter_by_group)) {
                $groups_to_query[] = $filter_by_group;
				$group_query_limit = array_var($_REQUEST, 'group_limit', $limit);
				$group_query_offset = array_var($_REQUEST, 'group_offset', 0);

			} else {
				$total_elements = 0;
				if (is_array($group_rows)) {
				  foreach ($group_rows as $gr_row) {

                      if(empty($gr_row['gr_id'])){

                          $gr_row['gr_name'] = 'none';
                          $gr_row['gr_id'] = '0';
                      }

                      $total_elements += ($gr_row['gr_count'] > $gr_max_elems ? $gr_max_elems : $gr_row['gr_count']);
					$gr_id = $gr_row['gr_id'] ? $gr_row['gr_id'] : "0";
					
					if ($total_elements <= $limit) {
						$groups_to_query[] = $gr_id;
					}
						
					$groups_info['groups'][$gr_id] = array(
							'id' => $gr_id,
							'name' => trim($gr_row['gr_name']),
							'count' => 	$gr_row['gr_count']
					);
					$groups_info['groups_order'][] = $gr_id;
				  }
				}

				$group_query_limit = $gr_max_elems;
				$group_query_offset = 0;
			}
				
			$rows = array();

			// add the cps of the related object (example: cps of the contact associated to the client)
			$add_cp_cols = "";
			$add_cp_joins_sql = "";
			if ($member_type && $member_type->getName() == 'customer') {
				$contact_cps = Customers::instance()->getAdditionalCustomProperties();
				foreach ($contact_cps as $contact_cp) {
					$cp_id = $contact_cp->getId();
					// add column to the select part
					$add_cp_cols .= ", cpval_".$cp_id.".value as cp_".$cp_id." ";
					// add joins to get the value for each cp
					$add_cp_joins_sql .= "
						LEFT JOIN ".TABLE_PREFIX."custom_property_values cpval_".$cp_id." 
						ON cpval_".$cp_id.".object_id = obj_type_table.contact_id AND cpval_".$cp_id.".custom_property_id=$cp_id and cpval_".$cp_id.".value != ''
					";
				}
			}
			// ----
				
			foreach ($groups_to_query as $gr_to_query) {
				if ($gr_to_query) {
					$group_condition_sql = "AND $group_match_column = '$gr_to_query'";
				} else {
					$group_condition_sql = "AND ($group_match_column IS NULL OR $group_match_column='' OR $group_match_column='0')";
				}

				$data_sql = "
						$columns_sql $add_cp_cols
						$from_sql
						$joins_sql
						$groups_joins_sql
						$add_cp_joins_sql
					WHERE
						$all_conditions_sql
						$group_condition_sql
						$group_by
					ORDER BY $order $order_dir
					LIMIT $group_query_offset, $group_query_limit
				";

				$this_group_rows = DB::executeAll($data_sql);
				if (!is_array($this_group_rows)) $this_group_rows = array();


				// format the data of the cps related to the associated object (example: contact associated to customer obj)
				foreach ($this_group_rows as &$this_group_row) {

					foreach ($this_group_row as $cp_key => &$cp_val) {
						if (str_starts_with($cp_key, 'cp_')) {
							$cp_id = str_replace('cp_', '', $cp_key);
							$cp = CustomProperties::getCustomProperty($cp_id);
	
							$cpv = new CustomPropertyValue();
							$cpv->setValue($cp_val);
							
							if ($member_type && $member_type->getName() == 'customer') {
								$cp_val = get_custom_property_value_for_listing($cp, $this_group_row['object_id'], array($cpv));
							}
						}
					}
				}


				$rows = array_merge($rows, $this_group_rows);
			}
			// remove duplicated rows
			$tmp_ids = array();
			foreach ($rows as $k => &$r) {
				if (in_array($r['id'], $tmp_ids)) unset($rows[$k]);
				else $tmp_ids[] = $r['id'];
			}
			
			// get group totals
			Env::useHelper('list_totals', 'advanced_core');
			$group_totals_params = build_cols_joins_sql_from_query_strings(array('type_id' => $member_type_id, 'list_type' => 'member', 'joins_sql' => $joins_sql));
			
			$add_cols = array_filter(array_var($group_totals_params, 'cols_sql'));
			$add_joins = array_filter(array_var($group_totals_params, 'cp_joins_sql'));
			
			if (count($add_cols) > 0) {
				$add_cols_sql = implode(',', $add_cols);
				$add_joins_sql = implode(' ', $add_joins);
				
				// for totals exclude duplicated rows
				$all_conditions_sql .= " " . implode("\n", $only_first_row_of_joins_conds);
					
				$group_totals_sql = "
					$add_groups_cols $add_cols_sql
						$from_sql
						$joins_sql
						$groups_joins_sql
						$add_joins_sql
					WHERE 
						$all_conditions_sql
						$group_by_groups
					ORDER BY $groups_order_by ASC
					LIMIT $group_list_offset, $limit
				";
				
				$group_totals = array();
				
				$group_totals_rows = DB::executeAll($group_totals_sql);
				if (is_array($group_totals_rows)) {
				  foreach ($group_totals_rows as $gt_row) {
                      if(empty($gt_row['gr_id'])){
                          $gt_row['gr_id'] = '0';
                      }
					$group_totals[$gt_row['gr_id']] = $gt_row;
					$group_totals[$gt_row['gr_id']]['name'] = lang('total').": ".$gt_row['total_rows'];
				  }
				}
				
				$groups_info['group_totals'] = $group_totals;
			}
			
			$this->save_member_list_group_by_as_user_config_option($dimension_id, $member_type_id, $groups_info);	
			evt_add('update last member list groups info', array("$dimension_id-$member_type_id" => $groups_info)); 
				
		} else {
			$groups_info = null;
			// no grouping sql query
			$data_sql = "
				$columns_sql
				$main_sql
				$group_by
				ORDER BY $order $order_dir
				LIMIT $start, $limit
			";
				
			// execute query
			$rows = DB::executeAll($data_sql);
			if (!is_array($rows)) $rows = array();
		}

		// count results sql
		$total_count_sql = "
			SELECT count(distinct(mem.id)) as total_count
			$main_sql
		";

		// execute count query
		$count_row = DB::executeOne($total_count_sql);
		$total_count = array_var($count_row, 'total_count');

		// build list of members information
		$object = $member_controller->prepareObject($rows, $start, $limit, $dimension, $member_type_id, $total_count, $groups_info);

		// additional data over result
		$params = array('type_id' => $member_type_id, 'from_sql' => $from_sql, 'joins_sql' => $joins_sql, 'conditions_sql' => $all_conditions_sql,
				'group_by' => $group_by, 'order' => $order, 'order_dir' => $order_dir, 'start' => $start, 'limit' => $limit, 'caller' => $this);
		Hook::fire('member_listing_additional_data', $params, $object);

		// return the data or send it to the view
		if ($return_the_list) {
			return $object;
		} else {
			ajx_extra_data($object);
			tpl_assign("listing", $object);
		}
	
	}

	function save_member_list_group_by_as_user_config_option($dimension_id, $member_type_id, $groups_info) {
		$config_option_name = 'member_list_group_by_' . $dimension_id . '_' . $member_type_id;
		$co = ContactConfigOptions::getByName($config_option_name); 
        if (!$co instanceof ContactConfigOption) {
            $co = new ContactConfigOption();
            $co->setFromAttributes(array(
                'category_name' => 'system',
                'name' => $config_option_name,
                'default_value' => '',
                'config_handler_class' => 'StringConfigHandler',
                'is_system' => '1',
                'dev_comment' => '',
                'options' => '',
            ));
            $co->save();
        }

		$group_by_value = array_var($groups_info, 'last_group_by', 'mem_path');
		set_user_config_option($config_option_name, $group_by_value, logged_user()->getId());
	}
	
} // MemberGroupedListingController

?>