<?php

/**
 * ObjectPrintOptions controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class ObjectPrintOptionsController extends ApplicationController {

	var $plugin_name = "advanced_core";
	
	/**
	 * Construct the ObjectPrintOptionsController
	 *
	 * @access public
	 * @param void
	 * @return ObjectPrintOptionsController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
		if (!Plugins::instance()->isActivePlugin('object_subtypes')) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
		}
	} // __construct
	
	
	function main_object_type_list() {
		if(!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
	
		$object_types = array();
		$ordered_object_types = array();
		$object_types_tmp = ObjectTypes::instance()->findAll(array("conditions" => "`type` IN ('content_object')  AND `name` <> 'template_task' AND name <> 'template_milestone' AND `name` <> 'file revision'", "order" => "name"));
		foreach ($object_types_tmp as $ot) {
			$ordered_object_types[$ot->getId()] = $ot->getPluralObjectTypeName();
			$object_types[$ot->getId()] = $ot->getName();
		}
		asort($ordered_object_types, SORT_STRING);
	
		tpl_assign('object_types', $object_types);
		tpl_assign('ordered_object_types', $ordered_object_types);
	
	}
	

	function list_print_options_for_type() {
		if(!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
	
		$object_type_id = get_id();
		$object_type = ObjectTypes::instance()->findById($object_type_id);
		if (!$object_type instanceof ObjectType) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
	
		$subtype_id = array_var($_GET, 'subtype_id');
		$subtype = ObjectSubtypes::instance()->findById($subtype_id);
		
		$object_print_options = ObjectPrintOptions::getAllObjectPrintPropertyOptions($object_type->getId(), $subtype_id);
		
		$properties = array();
		$conditions = "object_type_id=$object_type_id AND object_subtype_id IN (" . ($subtype instanceof ObjectSubtype ? "$subtype_id,0" : "0") .")";
		$cps = CustomProperties::instance()->findAll(array('conditions' => $conditions));
		foreach ($cps as $cp) {
			$properties[$cp->getId()] = $cp->getName();
		}
		
		$header_content = "";
		$header_option = ObjectPrintOptions::getObjectPrintHeaderOption($object_type_id, $subtype_id);
		if ($header_option instanceof ObjectPrintOption) {
			$header_content = $header_option->getHtmlContent();
		}
		$footer_content = "";
		$footer_option = ObjectPrintOptions::getObjectPrintFooterOption($object_type_id, $subtype_id);
		if ($footer_option instanceof ObjectPrintOption) {
			$footer_content = $footer_option->getHtmlContent();
		}
		
		tpl_assign('header_content', $header_content);
		tpl_assign('footer_content', $footer_content);
		tpl_assign('properties', $properties);
		tpl_assign('subtype', $subtype);
		tpl_assign('object_type', $object_type);
		tpl_assign('object_print_options', $object_print_options);
	
	}
	
	
	function save_print_options() {
		if(!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			return;
		}
		
		$object_type_id = array_var($_REQUEST, 'object_type_id');
		$object_type = ObjectTypes::instance()->findById($object_type_id);
		if (!$object_type instanceof ObjectType) {
			flash_error(lang('object dnx'));
			return;
		}
		
		$subtype_id = array_var($_REQUEST, 'subtype_id');
		$subtype = ObjectSubtypes::instance()->findById($subtype_id);
		
		$options = json_decode(array_var($_REQUEST, 'options'), true);
		$stid = ($subtype instanceof ObjectSubtype ? "$subtype_id" : "0");
		$conditions = "object_type_id=$object_type_id AND object_subtype_id=$stid";
		
		try {
			DB::beginWork();
			
			// header
			$header = array_var($_REQUEST, 'header');
			$sql = "INSERT INTO ".TABLE_PREFIX."object_print_options (object_type_id, object_subtype_id, option_type, property, html_content) VALUES
					($object_type_id, $stid, 'header', '0', '$header') ON DUPLICATE KEY UPDATE html_content='$header';";
			DB::execute($sql);
			
			// footer
			$footer = array_var($_REQUEST, 'footer');
			$sql = "INSERT INTO ".TABLE_PREFIX."object_print_options (object_type_id, object_subtype_id, option_type, property, html_content) VALUES
					($object_type_id, $stid, 'footer', '0', '$footer') ON DUPLICATE KEY UPDATE html_content='$footer';";
			DB::execute($sql);
		
			// properties
			ObjectPrintOptions::instance()->delete("option_type='property' AND $conditions");
			
			$order = 1;
			foreach ($options as $option) {
				$op_object = new ObjectPrintOption();
				$op_object->setObjectTypeId($object_type_id);
				$op_object->setObjectSubtypeId($subtype_id);
				$op_object->setOptionType('property');
				$op_object->setPosition($option['position']);
				$op_object->setProperty($option['id']);
				$op_object->setORder($order);
				$op_object->save();
				$order++;
			}
			
			DB::commit();
			
			flash_success(lang('print options saved successfully for', $subtype instanceof ObjectSubtype ? $subtype->getName() : lang($object_type->getName())));
			ajx_current("back");
			
		} catch (Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			ajx_current("empty");
		}
	}
	
} // ObjectPrintOptionsController

?>