<?php

/**
 * SendObjectEmail controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class SendObjectEmailController extends ApplicationController {

	var $plugin_name = "advanced_core";
	
	/**
	 * Construct the SendObjectEmailController
	 *
	 * @access public
	 * @param void
	 * @return SendObjectEmailController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
	} // __construct
	
	
	function send_contact_email($params_to_email) {
		
		// dummy object to use for the mail body
		$dummy_object = new Contact();
		$dummy_object->setObjectTypeId(Contacts::instance()->getObjectTypeId());
		$dummy_object->setFirstName($params_to_email['name']);
		$dummy_object->setSurname($params_to_email['email']);
		$dummy_object->setCommentsField($params_to_email['message']);
		
		if (!isset($params_to_email['subject'])) $params_to_email['subject'] = 'New message from Contact form';
		
		// template code = wordpress_contact_email
		$this->send_object_by_email($dummy_object, $params_to_email, true, true);
		
	}
	
	function send_object_by_email($object_id, $params_to_email, $send_the_email = false, $send_in_background = false) {
		
		if (!Plugins::instance()->isActivePlugin('mail')) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		
		if ($object_id instanceof ContentDataObject) $object = $object_id;
		else $object = Objects::findObject($object_id);
		if (!$object instanceof ContentDataObject) {
			return;
		}
		
		$object_type = ObjectTypes::instance()->findById($object->getObjectTypeId());
		
		$member = null;
		if ($object_type->getType() == 'dimension_object') {
			$member = Members::findOneByObjectId($object->getId());
		}
		
		$template_variables = array(
				'object' => $object,
				'member' => $member,
		);
		
		$mail_account_id = config_option('send_object_email_default_mail_account');
		if ($mail_account_id) {
			// check if the user has permissions to use this account, if not set mail_account_id as null to use another account later
			$mail_account = MailAccounts::instance()->findById($mail_account_id);
			$user_mac = MailAccountContacts::getByAccountAndContact($mail_account, logged_user());
			if (!$user_mac instanceof MailAccountContact) {
				$mail_account_id = null;
			}
		}
		
		$template_file_name = array_var($params_to_email, 'template_file_name');
		
		$mail_template_content = self::get_object_mail_template_content($object, $template_file_name, $template_variables);
		
		$mail_subject = array_var($params_to_email, 'subject');
		if (!$mail_subject) {
			$mail_subject = lang($object_type->getName())." - ".$object->getName();
		}
			
		$member_ids = $object->getMemberIds();
		if ($member instanceof Member) {
			$member_ids[] = $member->getId();
		}
		
		
		// if mail templates plugin is installed and exists a template for this case
		$mail_template_params = array(
				'triggering_object' => $object,
				'template_code' => $template_file_name,
				'template_parameters' => $template_variables,
		);
		$mail_template_data = null;
		Hook::fire("get_mail_template_data", $mail_template_params, $mail_template_data);
		
		if (is_array($mail_template_data)) {
			// override content
			if (isset($mail_template_data['body'])) {
				$mail_template_content = $mail_template_data['body'];
			}
			// override subject
			if (isset($mail_template_data['subject'])) {
				$mail_subject = $mail_template_data['subject'];
			}
			// override account
			if (isset($mail_template_data['mail_account_id'])) {
				$mail_account_id = $mail_template_data['mail_account_id'];
			}
			// override from name
			if (isset($mail_template_data['from_name'])) {
				$mail_template_from_name = $mail_template_data['from_name'];
			}
			
			// join to, cc, and bcc
			$recpt_fields = array('to', 'cc', 'bcc');
			foreach ($recpt_fields as $recpt_field) {
				if (isset($mail_template_data[$recpt_field])) {
					if (isset($params_to_email[$recpt_field]) && is_array($params_to_email[$recpt_field])) {
						$params_to_email[$recpt_field] = array_merge($params_to_email[$recpt_field], $mail_template_data[$recpt_field]);
					} else {
						$params_to_email[$recpt_field] = $mail_template_data[$recpt_field];
					}
				}
			}
		}
		// -----------
		
		
		// if no recipient is set use the mail of the config option or the admin mail
		if (count($params_to_email['to']) == 0) {
			$default_to = config_option('send_object_email_default_to');
			if (trim($default_to) != '') {
				// use configured recipient
				$params_to_email['to'][] = $default_to;
			} else {
				// if no recipient configured use the superadmin mail
				$feng_admin = Contacts::instance()->findOne(array("conditions" => "user_type=1"));
				if ($feng_admin instanceof Contact) {
					$params_to_email['to'][] = $feng_admin->getEmailAddress();
				}
			}
		}
		
		
		if ($send_the_email) {
			// save the mail_content and send it
			if (!$mail_account_id) {
				$mail_accounts = MailAccounts::getMailAccountsByUser(logged_user());
				if (count($mail_accounts) > 0) {
					$mail_account = $mail_accounts[0];
					if ($mail_account instanceof MailAccount) {
						$mail_account_id = $mail_account->getId();
					}
				}
			}
				
			$mail_data = array(
					'format' => "html",
					'subject' => $mail_subject,
					'body' => $mail_template_content,
					'account_id' => $mail_account_id,
					'member_ids' => $member_ids,
			);
			
			if (isset($mail_template_from_name)) {
				$mail_data['from_name'] = $mail_template_from_name;
			}
				
			$recpt_fields = array('to', 'cc', 'bcc');
			foreach ($recpt_fields as $recpt_field) {
				if (isset($params_to_email[$recpt_field])) {
					$mail_data[$recpt_field] = implode(',', $params_to_email[$recpt_field]);
				}
			}
				
			if ($send_in_background) {
				$mail_data['dont_send_outobx'] = true;
			}
				
			// set the mail vars to take by the controller
			$_POST['mail'] = $mail_data;
				
			// set the attachments
			$_POST['linked_objects'] = array();
			if (isset($attachs)) {
				foreach ($attachs as $idx => $attach) {
					$_POST['linked_objects'][$idx] = $attach['manager'] .":". $attach['object_id'];
					$_POST['attach_contents'][$idx] = true;
				}
			}
				
			// send the email
			$mail_controller = new MailController();
			$ok = $mail_controller->add_mail();
			if (!$ok) {
				throw new Exception("Object email could not be sent");
			}
				
		} else {
			// prepare mail variables and compose the email
			$mail_data = array(
					'is_new_mail' => true,
					'type' => "html",
					'subject' => $mail_subject,
					'body' => $mail_template_content,
					'account_id' => $mail_account_id,
			);
				
			// set the attachment info in the session
			if (isset($attachs)) {
				$_SESSION['existing_attachments'] = $attachs;
			}
			
			// set the email parameters to set in the view
			$_SESSION['add_mail_override_mail_data'] = $mail_data;
		
			// go to compose the email
			redirect_to(get_url('mail', 'add_mail', $params_to_email));
		}
		
	}
	
	
	
	private static function get_object_mail_template_content($object, $template_file_name, $template_variables) {
	
		$mail_templates_path = ROOT . "/mail_templates/";
	
		// convert the array of variables to single variables for the template
		extract($template_variables);
	
		$custom_template_path = $mail_templates_path . "$template_file_name.php";
		if (file_exists($custom_template_path)) {
			ob_start();
			include $custom_template_path;
			$mail_template_content = ob_get_clean();
		} else {
			$mail_template_content = "";
		}
	
		return $mail_template_content;
	}
	

} // SendObjectEmailController

