<?php

function upload_generated_document($name, $members, $content, $type ='text/plain', $file = null) {

	// create file
	if (!$file instanceof ProjectFile) {
		$file = new ProjectFile();
		$file->setFilename($name);
		$file->setIsVisible(true);
		$file->setCreatedById(logged_user()->getId());
		$file->save();
	
		// classify
		$file->addToMembers($members);
	
		// fill sharing table in background
		add_object_to_sharing_table($file, logged_user());
	}
	
	// save content in tmp file
	$userid = logged_user() ? logged_user()->getId() : "0";
	$tempFileName = ROOT ."/tmp/". $userid ."x". gen_id();
	$fh = fopen($tempFileName, 'w');
	fwrite($fh, $content);
	fclose($fh);

	// object to generate revision
	$fileToSave = array(
			"name" => $name,
			"type" => $type,
			"tmp_name" => $tempFileName,
			"error" => 0,
			"size" => filesize($tempFileName)
	);

	// generate file revision
	$revision = $file->handleUploadedFile($fileToSave, true, '');
	$revision->setCreatedById(logged_user()->getId());
	$revision->save();
	ApplicationLogs::createLog($file, ApplicationLogs::ACTION_ADD);

	return $file;
}


/**
 * Generate all default property groups for all object types and puts all properties in that group
 */
function fill_property_groups_default_data() {
	
	Env::useHelper("installer_functions", "advanced_core");
	$prop_groups_queries = get_default_property_groups_queries(DB::connection()->getLink(), TABLE_PREFIX);
	
	foreach ($prop_groups_queries as $q) {
		DB::executeOne($q);
	}
}

/**
 * Saves the custom property (name, value_template) for the $task
 * 
 * @param $task - is the task in which the custom properties will be instantiated
 * @param $property_name - the name of the property (note that it should start with "cp_", followed by the Custom Property ID
 * @param $property_value_template - The template data to be used for instantiating the parameter values 
 * @param $parameter_values - the array of values passed on by the user when instantiating a template
 * 
 */
function add_template_custom_property_to_template_task(TemplateTask $template_object, ProjectTask $task, $property_name, $property_value_template, $parameter_values) {
    
    $property_id = substr($property_name, 3);    
    $custom_property = CustomProperties::getCustomProperty($property_id);
    $custom_property_type = $custom_property->getColumnValue("type");
    
	$value_to_save = '';
	
	switch ($custom_property_type) {
		case 'date':
		case 'datetime':
			$result = calculate_template_task_parameter_date($parameter_values, $property_value_template, $property_name, $template_object, $task);
			$dt = array_var($result, 'value');

			if ($dt instanceof DateTimeValue) {
				if ($custom_property_type == 'date') {
					$value_to_save = format_date($dt, null, 0);
				} else {
					$value_to_save = format_datetime($dt, null, 0);
				}
			}
			break;
		case 'text':
		case 'user':
			$value_to_save = calculate_template_task_parameter_string($parameter_values, $property_value_template);
			break;
		case 'numeric':
			$value_to_save = calculate_template_task_parameter_numeric($parameter_values, $property_value_template, $property_name);
			break;
	}
    
    if ($value_to_save != '') {
        
        $object_controller = new ObjectController();
        $object_controller->add_custom_property($task, $property_id, $value_to_save);        
        
    }
}

