<?php

function build_cols_joins_sql_from_query_strings($params)
{

	$type_id = array_var($params, 'type_id');
	$list_type = array_var($params, 'list_type');
	$report = array_var($params, 'report');
	$ot = ObjectTypes::instance()->findById($type_id);
	if (!$ot instanceof ObjectType) {
		return;
	}

	$cols_sql = array();
	$cp_joins_sql = array();
	$group_by_sql = array();

	if ($list_type == 'member') {
		if (Plugins::instance()->isActivePlugin('member_custom_properties')) {
			if ($ot->getType() == 'dimension_object') {
				$custom_properties = CustomProperties::instance()->findAll(array('conditions' => "object_type_id=$type_id"));
				$cpval_table_name = "custom_property_values";
				$cpval_rel_obj_join = "object_id=o.id";
			} else {
				$custom_properties = MemberCustomProperties::instance()->findAll(array('conditions' => "object_type_id=$type_id"));
				$cpval_table_name = "member_custom_property_values";
				$cpval_rel_obj_join = "member_id=mem.id";
			}
		} else {
			$custom_properties = array();
		}
	} else {
		$custom_properties = CustomProperties::instance()->findAll(array('conditions' => "object_type_id=$type_id"));
		$cpval_table_name = "custom_property_values";
		$cpval_rel_obj_join = "object_id=o.id";
	}

	// total count row
	$idfield = ($list_type == 'member' ? "mem.id" : "o.id");
	$cols_sql[] = "COUNT(DISTINCT($idfield)) as total_rows";

	foreach ($custom_properties as $cp) {
		if ($cp->getColumnValue('total_operation') == 'sum') {
			$sql_oper = "ROUND(SUM(cpv_" . $cp->getId() . ".value),2)";
		} else if ($cp->getColumnValue('total_operation') == 'average') {
			$sql_oper = "ROUND(AVG(COALESCE(cpv_" . $cp->getId() . ".value, 0)),2)";
		} else {
			continue;
		}

		$cp_joins_sql[] = "
				LEFT JOIN " . TABLE_PREFIX . "$cpval_table_name cpv_" . $cp->getId() . "
				ON cpv_" . $cp->getId() . "." . $cpval_rel_obj_join . " AND cpv_" . $cp->getId() . ".custom_property_id=" . $cp->getId();

		$cols_sql[] = "$sql_oper as cp_" . $cp->getId();
	}


	// fixed object columns to sum
	$manager = null;
	if (trim($ot->getHandlerClass()) != '' && class_exists($ot->getHandlerClass())) {
		eval('$manager = ' . $ot->getHandlerClass() . "::instance();");
	}
	if ($manager) {
		$join_with_expense_totals_added = false;
		$join_with_member_totals_added = false;
		$all_currencies = Currencies::getCurrenciesInfo();
		$table_columns = $manager->getColumns();
		$columns_to_aggregate = $manager->getColumnsToAggregateInTotals();
		if ($report instanceof Report) {
			Hook::fire('list_totals_accumulate_total_rows', array('object_type' => $ot, 'report' => $report, 'skip' => true), $columns_to_aggregate);
		}

		$financial_columns = array();
		Hook::fire('get_all_member_totals_columns', null, $financial_columns);

		foreach ($columns_to_aggregate as $col => $col_data) {
			$operation = array_var($col_data, 'operation');

			// for expenses there are some columns that are in a separate table, we have to add the join with the totals table
			$t_prefix = '';
			if ($ot->getName() == 'expense') {
				$join_with_totals_exists = strpos($params['joins_sql'], TABLE_PREFIX . "expense_totals") > 0;
				if (!$join_with_expense_totals_added && !$join_with_totals_exists) {
					$cp_joins_sql[] = "
						INNER JOIN " . TABLE_PREFIX . "expense_totals exptotals ON exptotals.expense_id = e.object_id
					";
					$join_with_expense_totals_added = true;
				}
				if (!$join_with_totals_exists && in_array($col, array('total_cost', 'total_price'))) {
					$t_prefix = 'exptotals.';
				}
			}

			if ($ot->getName() == 'project') {
				$join_with_totals_exists = strpos(array_var($params, 'joins_sql'), TABLE_PREFIX . "member_totals") > 0;
				if (!$join_with_member_totals_added && !$join_with_totals_exists) {
					$cp_joins_sql[] = "
						LEFT JOIN " . TABLE_PREFIX . "member_totals memtotals ON memtotals.member_id = mem.id
					";
					$join_with_member_totals_added = true;
				}
				if (isset($columns_to_aggregate[$col]) && isset($financial_columns) && in_array($col, $financial_columns)) {
					$t_prefix = 'memtotals.';
				}
			}

			if ($operation == 'sum') {
				$sum_col = (in_array($col, $table_columns) ? "e." : $t_prefix) . $col;
				Hook::fire('sum_col_total_query', array('object_type' => $ot, 'list_type' => $list_type), $sum_col);
				$decimals = 2;
				if ($ot->getName() == 'payment_receipt') {
					$decimals = is_int(config_option('product_type_decimals')) ? config_option('product_type_decimals') : 2;
				}
				$sql_oper = "ROUND(SUM($sum_col),$decimals)";
				if ($col == 'bill_profit_margin') {
					$sql_oper = "ROUND(SUM(estimated_price - estimated_cost),2)";
				}
			} else if ($operation == 'average') {
				$sql_oper = "ROUND(AVG(COALESCE($col, 0)),2)";
				if ($col == 'bill_profit_margin_percentage') {
					$sql_oper = "ROUND(AVG((estimated_price - estimated_cost) * 100 / estimated_price),2)";
				}
			} else {
				continue;
			}
			$cols_sql[] = "$sql_oper as $col";
			$gb = array_var($col_data, 'group_by'); // example: for money columns
			if ($gb && !in_array($gb, $group_by_sql)) {
				$cols_sql[] = (in_array($gb, $table_columns) ? "e." : "") . $gb;

				if ($col_data['format'] == 'money' && count($all_currencies) == 1) {
					// dont group by currency if there is only one in the system
					continue;
				}
				$group_by_sql[] = (in_array($gb, $table_columns) ? "e." : "") . $gb;
			}
		}
	}

	return array('cp_joins_sql' => $cp_joins_sql, 'cols_sql' => $cols_sql, 'group_by_sql' => $group_by_sql);
}

function remove_expense_item_conditions_from_sql($conditions_sql)
{
	$exp_item_join_sql_parts = array(
		" AND (jt.object_id = (SELECT MIN(p2.object_id) FROM " . TABLE_PREFIX . "expense_items p2 WHERE p2.expense_id=jt.expense_id)) ",
	);

	foreach ($exp_item_join_sql_parts as $jsql) {
		if (strpos($conditions_sql, $jsql) !== false) {
			$conditions_sql = str_replace($jsql, "-- $jsql", $conditions_sql);
		}
	}

	return $conditions_sql;
}

function remove_expense_item_joins_from_sql($joins_sql)
{
	$exp_item_join_sql_parts = array(
		"LEFT JOIN `" . TABLE_PREFIX . "expense_items` `jt` ON `e`.`object_id` = `jt`.`expense_id`",
	);

	foreach ($exp_item_join_sql_parts as $jsql) {
		if (strpos($joins_sql, $jsql) !== false) {
			$joins_sql = str_replace($jsql, "-- $jsql", $joins_sql);
		}
	}

	return $joins_sql;
}

function remove_dimension_order_joins_from_joins_sql($joins_sql)
{
	$dim_order_join_sql_parts = array(
		"LEFT JOIN " . TABLE_PREFIX . "object_members obj_mems ",
		"LEFT JOIN " . TABLE_PREFIX . "members memb "
	);

	foreach ($dim_order_join_sql_parts as $jsql) {
		if (strpos($joins_sql, $jsql) !== false) {
			$joins_sql = str_replace($jsql, "-- $jsql", $joins_sql);
		}
	}

	return $joins_sql;
}

function generate_listing_totals_from_query_strings($params)
{

	$type_id = array_var($params, 'type_id');
	$list_type = array_var($params, 'list_type');
	$ot = ObjectTypes::instance()->findById($type_id);
	$report = array_var($params, 'report');
	$from_sql = array_var($params, 'from_sql');
	$joins_sql = array_var($params, 'joins_sql');
	Hook::fire('aditional_join_listing_total_query', array('object_type' => $ot, 'list_type' => $list_type), $joins_sql);

	$conditions_sql = array_var($params, 'conditions_sql');
	$group_by_sql = array_var($params, 'group_by');

	$result = build_cols_joins_sql_from_query_strings($params);
	$cp_joins_sql = array_var($result, 'cp_joins_sql');
	$cols_sql = array_var($result, 'cols_sql');
	Hook::fire('aditional_columns_listing_total_query', array('object_type' => $ot, 'list_type' => $list_type), $cols_sql);

	if (trim($conditions_sql) == '')
		$conditions_sql = "true";
	else if (str_starts_with(trim($conditions_sql), "AND"))
		$conditions_sql = "true $conditions_sql";

	if ($list_type == 'object') {
		$group_by_sql = " Group by o.id ";
	}
	$group_by_sqls = array_var($result, 'group_by_sql');
	Hook::fire('additional_group_by_listing_total_query', array('object_type' => $ot, 'list_type' => $list_type), $group_by_sqls);

	if ($list_type == 'object' && is_array($group_by_sqls) && count($group_by_sqls) > 0) {
		$group_by_sql = "GROUP BY " . implode(',', $group_by_sqls);
		if (count($params['member_ids']) > 0) {
			$group_by_sql .= ", om.member_id";
		}
	}

	// remove dimension order joins to prevent duplicated rows
	$joins_sql = remove_dimension_order_joins_from_joins_sql($joins_sql);

	// remove expense items conditions and joins to prevent calculation errors
	$joins_sql = remove_expense_item_joins_from_sql($joins_sql);
	$conditions_sql = remove_expense_item_conditions_from_sql($conditions_sql);

	if (isset($params['member_ids']) && is_array($params['member_ids'])) {

		// fix mysql strange error when grouping by currency_id and only one member is filtering
		if (count($params['member_ids']) == 1) {
			$conditions_sql = str_replace("AND om.member_id IN (", "AND om.member_id IN (-1,", $conditions_sql);
		}

		foreach ($params['member_ids'] as $mid) {
			if (is_numeric($mid) && $mid > 0) {
				$conditions_sql .= "
					and exists (select object_id from " . TABLE_PREFIX . "object_members where object_id=o.id and member_id=$mid) ";
			}
		}
	}

	if ( isset($cols_sql) && count($cols_sql) > 0) {
		$columns = implode(', ', $cols_sql);
		$cp_joins = implode(' ', $cp_joins_sql);

		if ($list_type == 'object') {
			$columns = "distinct(o.id), $columns";
		} else {
			$columns = "distinct(mem.id), $columns";

		}

		$totals_sql = "
			SELECT $columns
				$from_sql
				$joins_sql
				$cp_joins
			WHERE
				$conditions_sql
				$group_by_sql
		";

		$total_rows = DB::executeAll($totals_sql);
		Hook::fire('more_list_total_columns', array('object_type' => $ot, 'original_params' => $params), $total_rows);
		if (array_var($total_rows, 'totals_already_formatted')) {
			$totals = $total_rows[0];

		} else {
			$totals = accumulate_total_rows($ot, $total_rows, $report);
			// round totals
			foreach ($totals as &$t) {
				if (is_numeric($t)) {
					$t = round($t, 3);
				} else if (is_array($t)) {
					foreach ($t as &$t1)
						$t1 = round($t1, 3);
				}
			}
		}
		$totals['id'] = '__total_row__';
		$totals['object_id'] = '__total_row__';

		$ot_name_to_show = strtolower($ot->getPluralObjectTypeName());

		if (in_array($ot->getType(), array('dimension_object', 'dimension_group'))) {
			$ot_dim_ids = DimensionObjectTypes::getDimensionIdsByObjectTypeId($ot->getId());
			$ot_dim_id = array_var($ot_dim_ids, 0);
			$ot_name_to_show = Members::getTypeNameToShowByObjectType($ot_dim_id, $ot->getId(), null, true);
			$ot_name_to_show = strtolower($ot_name_to_show);
		}
		$totals['count'] = array_var($totals, 'total_rows') ?? 0;
		$totals['name'] = lang('total')." (".$totals['count']." ".$ot_name_to_show.")";
		$totals['total_rows'] = $totals['count'];
		
		return $totals;
	}
	return null;
}


function accumulate_total_rows($ot, $total_rows, $report = null)
{
	$columns_to_aggregate = array();
	if ($ot instanceof ObjectType) {
		$manager = null;
		if (trim($ot->getHandlerClass()) != '' && class_exists($ot->getHandlerClass())) {
			eval('$manager = ' . $ot->getHandlerClass() . "::instance();");
		}

		if ($manager) {
			$columns_to_aggregate_info = $manager->getColumnsToAggregateInTotals();
			Hook::fire('list_totals_accumulate_total_rows', array('object_type' => $ot, 'is_simple_report' => true, 'report' => $report), $columns_to_aggregate_info);

			$columns_to_aggregate = array_keys($columns_to_aggregate_info);
		}
	}
	$totals = array();
	if (!is_array($total_rows))
		$total_rows = array();
	foreach ($total_rows as $total_row) {
		foreach ($total_row as $col => $total) {
			if (!$total)
				continue;
			if (!isset($totals[$col]))
				$totals[$col] = 0;

			$clean_col = $col;
			if (strpos($clean_col, "|") !== false) {
				// use the clean column name for columns that are group totals
				$clean_col = substr($clean_col, strpos($clean_col, "|") + 1);
			}
			// if totals are grouped by any other column then generate an array of totals with key = group by column value
			if (in_array($clean_col, $columns_to_aggregate)) {
				$gb = array_var($columns_to_aggregate_info[$clean_col], 'group_by');
				if ($gb) {
					$this_gb = array_var($total_row, $gb);

					if (!is_array($totals[$col]))
						$totals[$col] = array();
					if (!isset($totals[$col][$this_gb]))
						$totals[$col][$this_gb] = 0;

					$totals[$col][$this_gb] += (float) $total;

				} else {
					if (is_numeric($total))
						$totals[$col] += $total;
					else
						$totals[$col] = $total;
				}
			} else if ($col == 'total_rows') {
				$totals[$col] += $total;
			} else if (str_starts_with($col, 'cp_')) {
				$totals[$col] += $total;
			} else {
				// dont't accumulate non-aggregable columns
				$totals[$col] = $total;
			}
		}
	}

	// calculate profit margin percentage
	if (key_exists('bill_profit_margin_percentage', $totals) && key_exists('estimated_price', $totals) && key_exists('estimated_cost', $totals)) {
		$totals['bill_profit_margin_percentage'] = round(($totals['estimated_price'] - $totals['estimated_cost']) * 100 / $totals['estimated_price'], 2);
	}

	return $totals;
}


function get_fixed_column_total($object_type, $totals, $column, $the_value = null, $report = null)
{
	$manager = null;
	if ($object_type) {
		if (trim($object_type->getHandlerClass()) != '' && class_exists($object_type->getHandlerClass())) {
			eval('$manager = ' . $object_type->getHandlerClass() . "::instance();");
		}
	}

	if ($manager) {
		$columns_to_aggregate_info = $manager->getColumnsToAggregateInTotals();
		Hook::fire('list_totals_accumulate_total_rows', array('object_type' => $object_type, 'report' => $report), $columns_to_aggregate_info);
		if ($object_type->getHandlerClass() == 'Timeslots') {
			$columns_to_aggregate_info['time_estimate'] = array('operation' => 'sum', 'format' => 'time');
		}


		$columns_to_aggregate = array_keys($columns_to_aggregate_info);
		if (in_array($column, $columns_to_aggregate)) {
			if (is_null($the_value)) {
				$total_value = array_var($totals, $column, '');
			} else {
				$total_value = $the_value;
			}
			$format = array_var($columns_to_aggregate_info[$column], 'format');
			switch ($format) {
				case 'money':
					if (is_array($total_value)) {
						$total_str = "";
						foreach ($total_value as $curr_id => $t_val) {
							$curr = Currencies::instance()->getCurrency($curr_id);
							if (!$curr)
								continue;
							$symbol = $curr ? $curr->getSymbol() : '';
							$decimals = null;
							Hook::fire('get_decimals_for_money_format', array('object_type' => $object_type), $decimals);
							$total_str .= ($total_str == '' ? '' : '<br/>') . format_money_amount($t_val, $symbol, $decimals);
						}
						return $total_str;
					} else {
						if (is_numeric($total_value)) {
							$def_currency = Currencies::getDefaultCurrencyInfo();
							$symbol = array_var($def_currency, 'symbol', '$');
							return format_money_amount($total_value, $symbol);
						} else {
							return $total_value;
						}
					}
				case 'number':
					$formatted_value = format_amount(clean_formatted_money_amount_for_sql($total_value));
					return $formatted_value;
				case 'minutes':
					$clean_value = clean_formatted_money_amount_for_sql($total_value);
					$formatted_value = format_time_column_value($clean_value);
					return $formatted_value;
				case 'percent':
					if (is_array($total_value)) {
						$total_str = "";
						foreach ($total_value as $key => $t_val) {
							$total_str .= ($total_str == '' ? '' : '<br/>') . '% ' . $t_val;
						}
						return $total_str;
					} else {
						return '% ' . $total_value;
					}

				case 'time':
					return format_value_to_print($column, $total_value, 'time', $object_type->getId(), '', 'Y-m-d', null, true);
				default:
					return $total_value;
			}
		}
	}
}

function get_report_fixed_column_total($report, $totals, $column, $the_value = null)
{
	if ($report instanceof Report) {
		$report_ot = ObjectTypes::instance()->findById($report->getReportObjectTypeId());
		return get_fixed_column_total($report_ot, $totals, $column, $the_value, $report);
	}
	return '';
}



function get_formatted_value_for_fixed_property_totals($report, $results, $totals, $column)
{
	$formatted_total_value = "";
	if (
		isset($results['group_columns']) && is_array($results['group_columns']['order'])
		&& in_array($column, $results['group_columns']['order'])
	) {
		$total_value = array_var($totals, $column, '');
		$orig_field_name = substr($column, strpos($column, "|") + 1);

		if (str_starts_with($orig_field_name, "cp_")) {
			$formatted_total_value = $total_value;
		} else {
			$formatted_total_value = get_report_fixed_column_total($report, $totals, $orig_field_name, $total_value);
		}

	} else {
		// is a fixed property
		$formatted_total_value = get_report_fixed_column_total($report, $totals, $column);
	}

	return $formatted_total_value;
}



function accummulate_custom_property_totals($report, $results, &$general_totals_array)
{
	// accumulate numeric custom property totals

	$group_totals = array_var($results, 'group_totals');
	$report_columns = $results['columns']['order'];
	$first_level_totals = array_var($group_totals, 0);

	$cps = array();
	// first reset current values
	foreach ($report_columns as $col) {
		if (is_numeric($col)) {
			$general_totals_array["cp_" . $col] = '';
			foreach ($report_columns as $col2) {
				if (str_ends_with($col2, "|cp_" . $col)) {
					$general_totals_array[$col2] = '';
				}
			}
			$cps[$col] = CustomProperties::instance()->findById($col);
		}
	}

	// accumulate subtotal values
	foreach ($first_level_totals as $total_row) {
		foreach ($total_row as $col => $val) {
			$is_group_total = false;
			$col_key = $col;
			if (strpos($col_key, "|cp_") !== false) {
				$is_group_total = true;
				$col_key = substr($col_key, strpos($col_key, "|") + 1);
			}

			if (str_starts_with($col_key, "cp_") || $is_group_total) {
				$cp_id = str_replace("cp_", "", $col_key);
				$cp = array_var($cps, $cp_id);

				if ($cp instanceof CustomProperty) {
					$total_operation = $cp->getColumnValue('total_operation');
					if (in_array($total_operation, array('sum', 'average'))) {
						$general_totals_array[$col] += $val;
					}
				}
			}
		}
	}
}

function accummulate_timeslots_totals($report, $results, &$general_totals_array, $final_total = false)
{
	// accumulate numeric custom property totals
	if (isset($general_totals_array['time_estimate'])) {
		$general_totals_array['time_estimate'] = 0;
	}

	$group_totals = array_var($results, 'group_totals');
	$report_columns = $results['columns']['order'];
	$first_level_totals = array_var($group_totals, 0);


	// accumulate subtotal values
	foreach ($first_level_totals as $total_row) {

		foreach ($total_row as $col => $val) {
			if ($col == 'time_estimate' && $final_total) {
				if (isset($general_totals_array[$col])) {
					$general_totals_array[$col] += $val;
				} else {
					$general_totals_array[$col] = $val;
				}
			}
		}
	}
}