<?php

function build_html_for_weekly_view($weekly_data, $genid){
    $current_filters = array_var($weekly_data, 'current_filters');
    $date_range = get_date_range_for_weekly_view($current_filters);
    $has_project_selected = array_var($weekly_data, 'has_project_selected');

    $weekly_table_class = $has_project_selected ? 'weekly-grid-table weekly-grid-table--has-project' : 'weekly-grid-table';
    $weekly_html = "<table class='".$weekly_table_class."'>";

    $weekly_html .= build_header_for_weekly_view($weekly_data, $date_range);

    $weekly_html .= "<tbody>";
    $weekly_html .= build_body_for_weekly_view($weekly_data, $date_range, $genid);
    $weekly_html .= "</tbody>";

    $weekly_html .= "<tbody class='weekly-footer'>";
    $weekly_html .= build_row_adder_for_weekly_view($weekly_data, $date_range, $genid);
    $weekly_html .= build_total_row_for_weekly_view($weekly_data, $date_range, $genid);
    $weekly_html .= "</tbody>";

    $weekly_html .= "</table>";

    return $weekly_html;
}


function build_header_for_weekly_view($weekly_data, $date_range){
    $has_project_selected = array_var($weekly_data, 'has_project_selected');
    
    $result = "<thead>";
    $result = "<tr class='grid-row grid-header'>";
    if(!$has_project_selected){
        $result .= "<th>" . lang('project') . "</th>";
    }
    $result .= "<th>" . lang('task') . "</th>";

    foreach($date_range as $date){
        $result .= "<th class='weekly-header--date-th'>";
        $result .= "<span class='weekly-header--weekday'>" . $date->format("D") . "</span>";
        $result .= "<span class='weekly-header--date'>" . $date->format("M d") . "</span>";
        $result .= "</th>";
    }

    $result .= "<th>" . lang('total') . "</th>";
    $result .= "</tr></thead>";

    return $result;
}


function build_body_for_weekly_view($weekly_data, $date_range, $genid){ 
    $task_groups = array_var($weekly_data, 'task_groups');
    $user_name = get_user_name_from_weekly_data($weekly_data);

    $result = "";
    foreach($task_groups as $group_name => $tasks){
        $has_tasks = count($tasks) ? 1 : 0;
        $hidden = !$has_tasks && $group_name == 'no_tasks';
        $style = $hidden ? 'display: none;' : '';

        $result .= build_group_name_row_for_weekly_view($group_name, $genid, $style, $user_name);
        $result .= '<tbody id="'.$genid.'-group_name-'.$group_name.'" hasTasks="'.$has_tasks.'" style="'.$style.'">';
        if($has_tasks){
            foreach($tasks as $task_info){
                $result .= build_task_row_for_weekly_view($task_info, $date_range, $genid, $weekly_data);
            }
        } else {
            $result .= build_empty_group_message_for_weekly_view($group_name);
        }
        $result .= '</tbody>';
    }
   

    return $result;
}


function get_date_range_for_weekly_view($current_filters){
    $from_date = array_var($current_filters, 'from_date');
    $until_date = array_var($current_filters, 'until_date');
    
    $begin = new DateTime( $from_date );
    $end = new DateTime( $until_date );
    $end = $end->modify( '+1 day' );

    $interval = new DateInterval('P1D');
    $daterange = new DatePeriod($begin, $interval ,$end);

    return $daterange;
}

function build_group_name_row_for_weekly_view($group_name, $genid, $style, $user_name){
    $result = "<tbody id='".$genid."-header_group_name-".$group_name."' style='".$style."'>";
    $result .= "<tr>";
    $result .= "<td class='weekly--group-name'>" . lang($group_name, $user_name) . "</td>";
    $result .= "</tr></tbody>";

    return $result;
}

function build_empty_group_message_for_weekly_view($group_name) {

    if($group_name == 'no_tasks'){
        $message = lang('no time entries');
    } else {
        $message = lang('no tasks found weekly view');
    }

    $result = "<tr>";
    $result .= "<td class='weekly--group-message'>" .  $message . "</td>";
    $result .= "</tr>";

    return $result;
}

function build_task_row_for_weekly_view($task_info, $date_range, $genid, $weekly_data){
    $has_project_selected = array_var($weekly_data, 'has_project_selected');
    $task_id = $task_info['id'];
    $project_id = $task_info['project_id'];
    $project_name = array_var($task_info, 'project_name') == '' ? lang('no specific project') : array_var($task_info, 'project_name');
    $task_name = array_var($task_info, 'name') == '' ? lang('no specific task') : array_var($task_info, 'name');
    $result = "<tr class='weekly--task-row'>";
    if(!$has_project_selected){
        if($project_id > 0) {
            $mem = Members::getMemberById($project_id); 
            if ($mem instanceof Member) {
                $mem_html_path = '<div class="task-breadcrumb-container">';
                $mem_html_path .= '<span class="member-path">';

                $mem_html_path .= '<span class="member-path cursor-pointer real-breadcrumb og-wsname-color-'.$mem->getColor().'" data-show-link="true"><span><a onclick="og.projects.onProjectClick('.$project_id.');">' .  $project_name . '</a></span></span>';
                $mem_html_path .= '</span>';
                $mem_html_path .= '</div>';
            }
            $result .= "<td class='weekly--task-name' title='". $project_name ."'>" . $mem_html_path . "</td>";
        } else {
            $result .= "<td class='weekly--task-name'  title='". $project_name ."'>" . $project_name . "</td>";
        }
    }
    if($task_id > 0){
        $task_obj = ProjectTasks::instance()->findById($task_id);
        if($task_obj instanceof ProjectTask){
            $task_url = $task_obj->getViewUrl();
            $task_title = htmlspecialchars($task_name);
            $task_name = '<a href="#" class="underline" title="'. $task_title .'" onclick="og.openLink(\''.$task_url.'\');return false;">'.$task_name.'</a>';
        }
    }
    $result .= "<td class='weekly--task-name'>" . $task_name . "</td>";
    foreach($date_range as $date){
        $time_input_and_desc_html = get_time_input_and_desc_html($task_info, $task_id, $date, $genid);
        $result .= "<td class='weekly--td-time-input'><div class='weekly--time-input-container'>" . $time_input_and_desc_html['time_input']. $time_input_and_desc_html['label_triangle'] . "</div> " .  $time_input_and_desc_html['description_button'] ."</td>";
    }
    $total_task_time = array_var($task_info, 'task_weekly_total', 0);
    $formatted_total_task_time = convert_num_to_hours_mins($total_task_time);
    $task_total_id = $genid . '-' . 'task_total' . '-' . $project_id . '-' . $task_id;
    $result .= "<td class='weekly--total-value' id='".$task_total_id."'>" . $formatted_total_task_time . "</td>";
    $result .= "</tr>";

    return $result;
}

function build_row_adder_for_weekly_view($weekly_data, $date_range, $genid) {
    $has_project_selected = array_var($weekly_data, 'has_project_selected');
    $project_member_id = array_var($weekly_data, 'selected_project_member_id');
    $result = "<tr class='weekly--task-row'>";
    $add_button_style = 'grid-column-start: 2; grid-column-end: -1;';
    $listeners = array();
	$projects_dim = Dimensions::findByCode('customer_project');
    $dimension_id = $projects_dim->getId();

    $current_filters = array_var($weekly_data, 'current_filters');
    $user_id = array_var($current_filters, 'weekly_user_id', 0);
    $selected_task_ids = array_var($weekly_data, 'selected_task_ids');
    $task_selector_params = array(
        'project_member_id' => $project_member_id,
        'user_id' => $user_id,
        'selected_task_ids' => $selected_task_ids
    );
	
    if(!$has_project_selected) {
        $result .= "<td class='weekly--project-selector'>";
        $result .= "<div id='project-selector-container'></div>";
        $result .= "</td>";
        $add_button_style = 'grid-column-start: 3; grid-column-end: -1;';
    }
    if($has_project_selected){ 
        $selector = build_task_selector_for_project_in_weekly_view($task_selector_params, $genid);
    } else {
        $selector = build_disabled_task_selector_for_project_in_weekly_view($genid);
    }
    $result .= "<td id='".$genid."task-selector-td' class='weekly--total-value'>" . $selector . "</td>";
    $result .= "<td style='".$add_button_style."'><button id='".$genid."button-add-row-weekly-view' class='btn btn-xs btn-primary' type='button' onclick='og.addTaskRowToWeeklyList()'>".lang('add row')."</button></td>";
    $result .= "</tr>";

    return $result;
}


function build_total_row_for_weekly_view($weekly_data, $date_range, $genid){
    $has_project_selected = array_var($weekly_data, 'has_project_selected');
    $current_filters = array_var($weekly_data, 'current_filters');
    $user_id = array_var($current_filters, 'weekly_user_id', 0);
    $result = "";
    $result .= "<tr class='weekly--total-row'>";
    $result .= "<td class='weekly--total-value'>" . lang('total') . "</td>";
    if(!$has_project_selected){
        $result .= "<td class='weekly--total-value'></td>";
    }
    foreach($date_range as $date){
        $weekday = strtolower($date->format("D"));
        $day_total = $weekly_data['day_totals'][$weekday];
        $formatted_day_total = convert_num_to_hours_mins($day_total);
        $day_total_id = $genid . '-' . 'day_total' . '-' . $user_id . '-' . $weekday;
        $result .= "<td id='".$day_total_id."' class='weekly--total-value'>" . $formatted_day_total . "</td>";
    }
    $grand_total = $weekly_data['weekly_total'];
    $formatted_grand_total = convert_num_to_hours_mins($grand_total);
    $id_grand_total = $genid . '-' . 'grand_total' . '-' . $user_id;
    $result .= "<td id='".$id_grand_total."' class='weekly--total-value'>" . $formatted_grand_total . "</td>";
    $result .= "</tr>";

    return $result;
}


function convert_num_to_hours_mins($time, $format = '%02d:%02d') {
    $hours = floor($time / 60);
    $minutes = ($time % 60);
    return sprintf($format, $hours, $minutes);
}

// Draw time grid on page load
function get_time_input_and_desc_html($task_info, $task_id, $date, $genid){
    // Get dimension helper functions
    Env::useHelper('dimension');
    $weekday = strtolower($date->format("D"));
    $project_id = array_var($task_info, 'project_id', 0);
    $fullday = $date->format("Y_m_d");
    $time = $task_info['weekly_timeslots'][$weekday]['task_day_total'];
    $pending_required_member = array_var($task_info, 'pending_required_member', false);
    $disabled_bottom_buttons = $pending_required_member ? 'weekly--disabled-bottom-links' : '';
    $labor_category_dim = Dimensions::findByCode('hour_types');
    $labor_category_dim_id = $labor_category_dim instanceof Dimension ? $labor_category_dim->getId() : 0;
    $formatted_time = convert_num_to_hours_mins($time);
    $time_input_disabled = '';
    $desc_data_toggle = 'data-toggle="popover"';
    $time_input_id = '';
    $desc_input_id = '';
    $desc_color = "#7C7C7C"; // 'Active' color
    $description_value = '';
    $imageClock = get_image_url("clock.png");
    $imageTrash = get_image_url("trash-bin.png");
    $popover_id = $genid . '-' . 'popover' . '-' . $project_id . '-' . $task_id . '-' . $fullday;
    $imageAdd = get_image_url("add-button.png");
    $default_labor_member_id = get_last_labor_member_id_for_weekly_view($task_info['weekly_timeslots'][$weekday]['timeslots']);
    $button_add_id = $popover_id . '-add_button';
    $button_add_time_popover = '<div class=\'weekly--popover-footer-section '.$disabled_bottom_buttons.'\'><a href=\'#\' id=\''.$button_add_id.'\' class=\'add-first-btn btn-add-work\' onclick=\'og.addWeeklyTimeToPopover(`' . $popover_id . '`, `'.$default_labor_member_id.'`);\'>
        <img class=\'iconImage marginRightTen\' src='.$imageAdd. ' alt=\'add more hours icon\' />
        Add more hours
    </a><a href=\'#\' class=\'weekly--popover-save-button '.$disabled_bottom_buttons.'\' onclick=\'og.collapsePopover(`' . $popover_id . '`);\' onkeyup=\'og.collapsePopoverUsingSpace(event, `' . $popover_id . '`);\'>'.lang('save').'</a></div>';    
    $onclick = '';
    if(count($task_info['weekly_timeslots'][$weekday]['timeslots']) == 1){ 

        $timeslot = $task_info['weekly_timeslots'][$weekday]['timeslots'][0];
        $timeslot_id = $task_info['weekly_timeslots'][$weekday]['timeslots'][0]['timeslot_id'];
        $description_value = htmlspecialchars($task_info['weekly_timeslots'][$weekday]['timeslots'][0]['description']);
        $time_input_id = $genid . '-' . 'edit'  . '-' . $project_id . '-' . $task_id . '-' . $fullday . '-' . $timeslot_id;
        $desc_input_id = $genid . '-description-edit-' . $project_id  . '-' . $task_id . '-' . $fullday . '-' .$timeslot_id;
    } else if(count($task_info['weekly_timeslots'][$weekday]['timeslots']) > 1){
        $time_input_disabled = 'readonly';
        $time_input_id = $genid . '-' . 'multiple_time' . '-' . $project_id . '-' . $task_id . '-' . $fullday;
        $onclick =" onclick='og.loadTimeInputPopover(".'"'.$time_input_id.'"'.")'";
    }else if(count($task_info['weekly_timeslots'][$weekday]['timeslots']) == 0) {
        $timeslot = array();  // *** LC se genera un array vacío cuando no hay timeslots
        $time_input_id = $genid . '-' . 'new' . '-' . $project_id . '-' . $task_id . '-' . $fullday . '-' . 0;
        $desc_input_id = $genid . '-decription-new-' . $project_id  . '-' . $task_id . '-' . $fullday . '-' . 0;
        $desc_data_toggle = '';
        $desc_color = "#CCC"; // Disabled color
    }

    // New version time input
    $onchange_func = 'og.updateTimeFromWeekly(this)';
    $class_input = "weekly--time-input";
    if($formatted_time == '00:00'){
        $class_input = "weekly--time-input-grey";
    }

    // Send input_id to generate button_id and simulate click 
    $time_input = "<input ".$onclick." data-toggle='popover' data-content='' class='".$class_input."' ".$time_input_disabled." id='".$time_input_id."' type='text' value='".$formatted_time."' prevvalue='".$formatted_time."' onchange='".$onchange_func."' onfocus='og.closePopovers();og.clearTime(this);' onblur='og.checkTimeValue(this)' pattern='^([0-1]?[0-9]|2[0-3]):[0-5][0-9]$'>";

    $labelTriangle = "<label for='input' class='".$class_input." weekly--triangule'></label>";
   
    if(count($task_info['weekly_timeslots'][$weekday]['timeslots']) > 1){
        $svg_lines = true;
        $day_timeslots = $task_info['weekly_timeslots'][$weekday]['timeslots'];
        $description_input = '<div>';
        $description_input .= '<div class=\'popover-timeslots\'>';
        $qtty_desc = 0;
        foreach($day_timeslots as $day_timeslot){
            // Get invoiced info
            $is_invoiced = $day_timeslot['invoicing_status'] == 'invoiced';
            $invoiced_readonly = $is_invoiced ? 'disabled' : '';
            $time_configs = array(
                'genid' => $genid,
                'project_id' => $project_id,
                'task_id' => $task_id,
                'fullday' => $fullday,
                'day_timeslot' => $day_timeslot,
                'dim_id' => $labor_category_dim_id,
                'is_single_timeslot' => false,
                'popover_id' => $popover_id,
            );

            $description_input .= render_time_entry_header($time_configs);

            // Add description input for each timeslot
            $day_desc_input_id = $genid . '-description-edit-' . $project_id . '-' . $task_id . '-' . $fullday . '-' .$day_timeslot['timeslot_id'];
            $day_desc_onchange_func = 'og.updateDescFromWeeklyDescDiv(this)';
            $day_description_value = htmlspecialchars($day_timeslot['description']);
            if($day_description_value == ''){
                $description_input .= '<div class=\'topMargin\'>
                    <label  class=\'descriptionLabel\'>'.lang("description").'</label>
                    <textarea '.$invoiced_readonly.' id=\''.$day_desc_input_id.'\' onchange='.$day_desc_onchange_func.' class=\'weekly--description-textarea\'></textarea>
                </div>';
            } else {
                $qtty_desc ++;
                $description_input .= '<div class=\'topMargin\'>
                    <label class=\'descriptionLabel\'>'.lang("description").'</label>
                    <textarea '.$invoiced_readonly.' id=\''.$day_desc_input_id.'\' onchange='.$day_desc_onchange_func.' class=\'weekly--description-textarea\'>'.$day_description_value.'</textarea>
                </div>';
            }

            // Labor Category selector
            $description_input .= render_labor_category_selector($time_configs);

            // Is billable checkbox
            $description_input .= render_is_billable_checkbox($time_configs);
           

            $description_input .= '</div>';
        }
        $description_input .= '</div>';
        $description_input .= $button_add_time_popover;
        $description_input .= '</div>';
        if(count($day_timeslots) == $qtty_desc){
            $desc_color = '#22CD92'; // 'Has description' color
        }else{
            $desc_color = "#7C7C7C"; // 'Active' color
        }
    } else {
        $svg_lines = false;
        $desc_onchange_func = 'og.updateDescFromWeeklyDescDiv(this)';
        $is_invoiced = isset( $timeslot) && array_var($timeslot, 'invoicing_status') == 'invoiced';
        $invoiced_readonly = $is_invoiced || $pending_required_member ? 'disabled' : '';
        $disabled_class = $pending_required_member ? 'disabled-layover' : '';
        $time_configs = array(
            'genid' => $genid,
            'project_id' => $project_id,
            'task_id' => $task_id,
            'fullday' => $fullday,
            'day_timeslot' => isset($timeslot) ? $timeslot : 0,
            'dim_id' => $labor_category_dim_id,
            'is_single_timeslot' => true,
            'popover_id' => $popover_id,
            'pending_required_member' => $pending_required_member
        );
        // Build description input
        $description_input = '<div>';
        $description_input .= '<div class=\'popover-timeslots\'>';

        if(count($task_info['weekly_timeslots'][$weekday]['timeslots']) == 1){ 
            $description_input .= render_time_entry_header($time_configs);
        }

        $description_input .= '<div class=\'topMargin '.$disabled_class.'\'>
                    <label class=\'descriptionLabel\'>'.lang("description").'</label>
                    <textarea '.$invoiced_readonly.' id=\''.$desc_input_id.'\' onchange='.$desc_onchange_func.' class=\'weekly--description-textarea\'>'.$description_value.'</textarea>
                </div>';

        if(count($task_info['weekly_timeslots'][$weekday]['timeslots']) == 1){ 
             // Labor Category selector
            $description_input .= render_labor_category_selector($time_configs);

            // Is billable checkbox
            $description_input .= render_is_billable_checkbox($time_configs);
        }

        $description_input .= '</div>';
        $description_input .= '</div>';
        
        $description_input .= $button_add_time_popover;
        if($description_value != '' && $desc_data_toggle != ''){
            $desc_color = '#22CD92';
        }
    }

     
    $svg_icon = get_popover_icon($svg_lines, $desc_color);

    $popover_time_input_id = 'timeId="'.$time_input_id.'"';
    $description_button = '
        <div id="'.$popover_id.'" class="weekly--popover-div" '.$popover_time_input_id.' onclick="og.closeOtherPopover(this);"  onkeyup="og.selectFirstDescriptionInput(event, `'.$popover_id.'`)">
        <button id="'.$popover_id.'_btn" class="weekly--desc-button" '.$desc_data_toggle.' data-content="'.$description_input.'" data-placement="bottom">
        '.$svg_icon.'
        </button>
        </div>';

    $result = array(
        'time_input' => $time_input,
        'label_triangle' => $labelTriangle,
        'description_button' => $description_button,
        'popover_id' => $popover_id,
        'description_content' => $description_input,
        'time_input_id' => $time_input_id,
        'formatted_time_value' => $formatted_time,
        'desc_color' => $desc_color
    );
    return $result;

}

function build_task_selector_for_project_in_weekly_view($task_selector_params, $genid){ 
    $project_member_id = array_var($task_selector_params, 'project_member_id');
    $user_id = array_var($task_selector_params, 'user_id', 0);
    $selected_task_ids = array_var($task_selector_params, 'selected_task_ids', array());
    $active_context_ids = active_context_members(false);
    $member_ids = array_merge(array($project_member_id), $active_context_ids);

    $tasks_res = ProjectTasks::instance()->listing(array(
        'order' => '`o`.`name`',
        'order_dir' => 'ASC',
        'member_ids' => $member_ids,
        'extra_conditions' => ' AND completed_by_id = 0 '
    ));
    $tasks = $tasks_res->objects;

    $selector = '<select id="'.$genid.'-task-selector-weekly-view" class="time-weekly--task-selector">';
    $selector .= '<option value="0">'.lang('select a task').'</option>';
    $assigned_options = '<optgroup label="'.lang('assigned to the user').'">';
    $not_assigned_options = '<optgroup label="'.lang('not assigned to the user').'">';
    
	// build tasks data to include in the selector, using the composed name if it is defined
	$tasks_available = array();
    $task_ids = array_map(function($task){ return $task->getObjectId(); }, $tasks);
	foreach($tasks as $task){
        if($task->getDepth() > 0) continue;
		$task_id = $task->getObjectId();
		$task_name = get_task_name_to_show_in_weekly_view_task_selector($task);
		$task_user_id = $task->getAssignedToContactId();
        $disabled = in_array($task_id, $selected_task_ids);
        $subtasks = get_subtasks_for_tasks_list_in_weekly_view($task, $task_ids, $selected_task_ids);
        if(count($subtasks) == 0 && $disabled) continue;

		$tasks_available[$task_name ."-". $task_id] = array(
			'task_id' => $task_id,
			'task_name' => $task_name,
			'task_user_id' => $task_user_id,
            'disabled' => $disabled,
            'subtasks' => $subtasks
		);
	}
	
	// sort the tasks using the composed name
	ksort($tasks_available);

	// append the ordered tasks in a selector group
	foreach ($tasks_available as $ta) {
		$task_id = $ta['task_id'];
		$task_name = $ta['task_name'];
		$task_user_id = $ta['task_user_id'];
        $disabled = $ta['disabled'] ? 'disabled' : '';
        $subtasks = $ta['subtasks'];


		$task_option = '<option value="' . $task_id . '" '.$disabled.'>' . $task_name . '</option>';
		if($task_user_id == $user_id){
			$assigned_options .= $task_option;
            render_subtasks_for_tasks_list_in_weekly_view($subtasks, $assigned_options);
		} else {
			$not_assigned_options .= $task_option;
            render_subtasks_for_tasks_list_in_weekly_view($subtasks, $not_assigned_options);
		}
	}

    $selector .= $assigned_options . '</optgroup>';
    $selector .= $not_assigned_options . '</optgroup>';
    $selector .= '</select>';

    return $selector;

}

function get_subtasks_for_tasks_list_in_weekly_view($task, $task_ids, $selected_task_ids) {
    $result = array();
    $subtasks = $task->getSubtasks(false,false);

    foreach($subtasks as $subtask) {
        $subtask_id = $subtask->getObjectId();
        if(!in_array($subtask_id, $task_ids)) continue;
		$subtask_name = get_task_name_to_show_in_weekly_view_task_selector($subtask);
		$subtask_user_id = $subtask->getAssignedToContactId();
        $disabled = in_array($subtask_id, $selected_task_ids);

        $result[$subtask_name ."-". $subtask_id] = array(
            'task_id' => $subtask_id,
            'task_name' => $subtask_name,
            'task_user_id' => $subtask_user_id,
            'disabled' => $disabled,
            'depth' => $subtask->getDepth(),
            'subtasks' => get_subtasks_for_tasks_list_in_weekly_view($subtask, $task_ids, $selected_task_ids)
        );
    }

    return $result;
}

function render_subtasks_for_tasks_list_in_weekly_view($tasks, &$options) {
    if(count($tasks) == 0) return;
    foreach($tasks as $task) {
        $task_id = $task['task_id'];
		$task_name = $task['task_name'];
        $depth = $task['depth'];
        $task_name = str_repeat('&nbsp;&nbsp;&nbsp;&nbsp;', $depth) . $task_name;
		$task_user_id = $task['task_user_id'];
        $disabled = $task['disabled'] ? 'disabled' : '';
        $subtasks = $task['subtasks'];

        $task_option = '<option value="' . $task_id . '" '.$disabled.'>' . $task_name . '</option>';
        $options .= $task_option;
        render_subtasks_for_tasks_list_in_weekly_view($subtasks, $options);
    }
}
/**
 * @param ProjectTask The task to calculate the name
 * @return string The composed name using the user preference "dimensions_to_append_in_weekly_task_selector"
 */
function get_task_name_to_show_in_weekly_view_task_selector($task) {

	$dimensions_to_append_in_task_name = explode(',', user_config_option("dimensions_to_append_in_weekly_task_selector"));

	$name_to_show = $task->getName();

	if (isset($dimensions_to_append_in_task_name) && count($dimensions_to_append_in_task_name) > 0) {
		$task_members = $task->getMembers();

		foreach ($dimensions_to_append_in_task_name as $dim_id) {
			foreach ($task_members as $member) {
				if ($member->getDimensionId() == $dim_id) {
					$name_to_show .= " - " . $member->getDisplayName();
				}
			}
		}
	}

	return $name_to_show;
}

function build_disabled_task_selector_for_project_in_weekly_view($genid){
    $selector = '<select id="'.$genid.'-task-selector-weekly-view" class="time-weekly--task-selector" disabled>
                        <option value="0">'.lang('select a task').'</option>
                    </select>';

    return $selector;
}

function get_user_name_from_weekly_data($weekly_data){
    $current_filters = array_var($weekly_data, 'current_filters');
    $user_id = array_var($current_filters, 'weekly_user_id', 0);
    $user = Contacts::instance()->findById($user_id);
    $user_name = $user instanceof Contact ? $user->getName() : "";
    
    return $user_name;
}

function render_is_billable_checkbox($configs) {
    // configs: genid, project_id, task_id, day_timeslot, fullday
    $genid = array_var($configs, 'genid');
    $project_id = array_var($configs, 'project_id');
    $task_id = array_var($configs, 'task_id');
    $day_timeslot = array_var($configs, 'day_timeslot');
    $fullday = array_var($configs, 'fullday');
    $pending_required_member = array_var($configs, 'pending_required_member', false);

    // Get billable property of the task
    $task = ProjectTasks::instance()->findById($task_id);
    $task_non_billable = $task instanceof ProjectTask && $task->getColumnValue('is_billable') == 0;
  
    // Prepare the checkbox elements
    $is_billable_onchange_func = 'og.updateIsBillableFromWeeklyViewDescDiv(this)';
    $is_billable_onkeyup_func = 'og.updateIsBillableInWeeklyUsingEnterKey(event, this);';
    $is_invoiced = $day_timeslot['invoicing_status'] == 'invoiced';
    $is_billable =  $day_timeslot['is_billable'] ? 'checked' : '';
    $day_is_billable_checkbox_id = $genid . '-is_billable-change-' . $project_id . '-' . $task_id . '-' . $fullday . '-' .$day_timeslot['timeslot_id'];
    $invoiced_checkbox_readonly = $is_invoiced || $pending_required_member ? 'onclick=\'return false;\'' : '';
    $disabled_class = $pending_required_member || $task_non_billable ? 'disabled-layover' : '';


    $is_billable_checkbox_html = '<div class=\'sliderContainer topMargin '.$disabled_class.'\'>
                                <label class=\'switch\'>
                                    <input '.$invoiced_checkbox_readonly.' id=\''.$day_is_billable_checkbox_id.'\' class=\'sliderWeekly\' type=\'checkbox\' '.$is_billable.' onchange=\''.$is_billable_onchange_func.'\' onkeyup=\''.$is_billable_onkeyup_func.'\'>
                                    <span class=\'slider round\'></span>
                                </label>
                                <h3 class=\'descriptionLabel\'>Billable</h3>
                            </div>';

    return $is_billable_checkbox_html;
}

function render_labor_category_selector($configs) { 
    // configs: genid, project_id, task_id, day_timeslot, fullday
    $genid = array_var($configs, 'genid');
    $project_id = array_var($configs, 'project_id');
    $task_id = array_var($configs, 'task_id');
    $day_timeslot = array_var($configs, 'day_timeslot');
    $fullday = array_var($configs, 'fullday');
    $labor_category_dim_id = array_var($configs, 'dim_id');
    $pending_required_member = array_var($configs, 'pending_required_member', false);

    // Prepare the labor category selector configs
    $lab_cat_selector_container_id = $genid . '-labor_category-selector_container-' . $project_id . '-' . $task_id . '-' . $fullday . '-' .$day_timeslot['timeslot_id'];
    $labor_category_selector_id = $genid . '-labor_category-selector-' . $project_id . '-' . $task_id . '-' . $fullday . '-' .$day_timeslot['timeslot_id'];
    if($pending_required_member){
        $original_time_id = $day_timeslot['original_time_id'];
        $original_time_value = $day_timeslot['original_time_value'];
        $original_user_id = $day_timeslot['original_user_id'];
        $labor_cat_onchange = "og.createTimeUsingLaborCategory(this, `".$original_time_id."`, `".$original_time_value."`, `".$original_user_id."`)";
    } else {
        $labor_cat_onchange = 'og.updateLaborCatFromWeeklyDescDiv(this)';
    }

    // Create labor category config for JS to render the selector
    $labor_category_configs = array(
        'dim_id' => $labor_category_dim_id,
        'genid' => $genid,
        'hf_name' => lang('labor category'),
        'selector_id' => $labor_category_selector_id,
        'container' => $lab_cat_selector_container_id,
        'selected_id' => $day_timeslot['labor_cat_member_id'],
        'selector_class' => 'weekly--selector-labor',
        'onchange' => $labor_cat_onchange
    );

    if($pending_required_member){
        $labor_category_configs['selector_class'] .= ' weekly--selector-labor--red';
    }

    $labor_cat_rendered_selector = render_plain_member_selector($labor_category_configs);

    $is_invoiced = $day_timeslot['invoicing_status'] == 'invoiced';
    $invoiced_class = $is_invoiced ? 'invoiced-labor-category' : '';
    $lab_cat_selector_html = '';
    if($pending_required_member){
        $lab_cat_selector_html .= '<span class="weekly-view--red">(*'.(lang('required')).')</span>';
    }

    $label_id = $labor_category_selector_id . '-label';
    $lab_cat_selector_html = '<div class=\'topMargin\'>
        <label id=\''.$label_id.'\'  class=\'descriptionLabel\'>'.lang("labor").' '. $lab_cat_selector_html .'</label>';
    $lab_cat_selector_html .= '<div id=\''. $lab_cat_selector_container_id.'\' class=\''.$invoiced_class.'\'>'.$labor_cat_rendered_selector.'
        </div>
    </div>';

    return $lab_cat_selector_html;
}

function render_time_entry_header($time_configs) {

    $genid = array_var($time_configs, 'genid');
    $project_id = array_var($time_configs, 'project_id');
    $task_id = array_var($time_configs, 'task_id');
    $fullday = array_var($time_configs, 'fullday');
    $day_timeslot = array_var($time_configs, 'day_timeslot');
    $is_single_timeslot = array_var($time_configs, 'is_single_timeslot');
    $imageTrash = get_image_url("trash-bin.png");
    $is_invoiced = $day_timeslot['invoicing_status'] == 'invoiced';
    $invoiced_readonly = $is_invoiced ? 'disabled' : '';
    $popover_id = array_var($time_configs, 'popover_id');
    $pending_required_member = array_var($time_configs, 'pending_required_member', false);

    $time_header = '';
    $not_invoiced_class = $is_invoiced ? '' : 'weekly--popover-time-desc-div--not-invoiced';
    $time_header .= '<div class=\'weekly--popover-time-desc-div '.$not_invoiced_class.'\'>';
    // Add time input for each timeslots
    $day_formatted_time = convert_num_to_hours_mins($day_timeslot['worked_time']);
    $day_time_input_id = $genid . '-' . 'edit' . '-' . $project_id . '-' . $task_id . '-' . $fullday . '-' . $day_timeslot['timeslot_id'];
    $day_time_onchange_func = 'og.updateTimeFromWeeklyDescDiv(this)';
    $time_header .= '<div class=\'weekly--time-input-container\'>';
    $time_header .= "<div class='containerInputDeletBtn'>";
    $time_header .= "<h2>". lang('time entry') ."</h2>";
    if($is_invoiced){
        $invoicing_status = lang($day_timeslot['invoicing_status']);
        $invoicing_status_class = 'invoicing-status-' . $day_timeslot['invoicing_status'];
        $time_header .= '<span class=\''.$invoicing_status_class.' margin-left-5\'>' . $invoicing_status . '</span>';
    }
    $time_header .= "</div>";
    $time_header .= "<div class='containerInputDeletBtn'>";
    if(!$is_single_timeslot) {
        $time_header .= "<input ".$invoiced_readonly." class='weekly--time-input' id='".$day_time_input_id."' type='text' value='".$day_formatted_time."' prevvalue='".$day_formatted_time."' onchange='".$day_time_onchange_func."' onfocus='og.clearTime(this);' pattern='^([0-1]?[0-9]|2[0-3]):[0-5][0-9]$'>";
        $time_header .= "<label for='input' class='weekly--time-input weekly--triangule'></label>";
    }
    if(!$is_invoiced && !$pending_required_member) {
        $is_multi = json_encode(!$is_single_timeslot);
        $time_header .= "<a class='list-action weekly--delete-time-btn' tabindex='-1' href='#' onclick='og.deleteTimeEntryWeeklyDescPopover(`".$day_time_input_id. "`, `".$popover_id. "`, ". $is_multi .")';><img class='iconImage' src='" . $imageTrash . "' alt='Delete icon'></a>"; 
    }
    $time_header .= "</div>";
    $time_header .= "</div>";

    // Add client name
    $time_header .= '<div class=\'clientContainer topMargin\'>
        <h2>'.lang('client').':</h2>
        <span>' . $day_timeslot['client_name'] . '</span>
    </div>';

    return $time_header;
}

function get_popover_icon($svg_lines, $desc_color) {
    if($svg_lines){
        $svg_icon = '<svg class="weekly-time-with-desc" width="22" height="22" viewBox="0 0 1000 1000" 
            fill="none" xmlns="http://www.w3.org/2000/svg">
            <g transform="translate(0,965) scale(0.100000,-0.100000)"
            fill="'.$desc_color.'">
            <path stroke-width="2" d="M2848 9790 c-385 -65 -679 -380 -714 -766 l-7 -74 215 0 215 0 6 56
            c13 130 107 258 236 322 l75 37 1973 3 1972 2 3 -1032 c3 -954 5 -1038 21
            -1092 47 -158 101 -255 201 -363 117 -125 253 -208 421 -255 80 -22 84 -22
            993 -26 l912 -3 0 -2273 c0 -2206 -1 -2275 -19 -2328 -54 -157 -188 -267 -353
            -288 -40 -5 -259 -10 -485 -10 l-413 0 0 -211 0 -210 498 3 c549 4 548 4 698
            73 232 106 392 290 471 542 l28 88 3 2414 2 2415 -36 37 c-33 34 -36 42 -31
            78 l5 39 -1315 1416 -1315 1416 -2107 -1 c-1158 -1 -2127 -5 -2153 -9z m5333
            -1770 l923 -985 -739 -3 c-639 -2 -747 0 -800 13 -160 41 -284 174 -314 339
            -13 73 -16 1629 -2 1624 5 -1 424 -446 932 -988z"/>
            <path stroke-width="2" d="M680 8504 c-351 -75 -620 -361 -670 -712 -8 -55 -10 -1082 -8 -3582
            l3 -3505 22 -72 c50 -158 118 -275 225 -381 108 -109 229 -179 386 -225 l77
            -22 3120 0 3120 0 78 23 c306 88 513 293 609 604 l23 73 3 2414 2 2415 -36 37
            c-33 34 -36 42 -31 78 l5 41 -1314 1415 -1314 1415 -2118 -1 c-1771 0 -2128
            -3 -2182 -15z m4010 -1449 l5 -1040 29 -85 c15 -47 40 -107 53 -135 87 -173
            254 -328 436 -403 150 -63 136 -62 1127 -62 l900 0 -2 -2293 -3 -2293 -37 -75
            c-46 -93 -112 -158 -206 -202 l-67 -32 -3090 0 -3090 0 -66 31 c-114 54 -190
            138 -228 252 l-21 62 2 3502 3 3503 32 66 c59 121 185 216 313 238 32 5 870 8
            1980 7 l1925 -1 5 -1040z m1389 -342 c490 -524 891 -955 891 -958 0 -3 -329
            -5 -731 -5 -821 0 -819 0 -926 69 -104 67 -170 165 -192 285 -7 40 -11 321
            -11 850 l0 791 39 -40 c21 -22 439 -468 930 -992z"/>
            <path stroke-width="2" d="M1492 6178 l3 -213 1063 -3 1062 -2 0 215 0 215 -1065 0 -1065 0 2
            -212z"/>
            <path stroke-width="2" d="M1490 4900 l0 -210 1065 0 1065 0 0 210 0 210 -1065 0 -1065 0 0
            -210z"/>
            <path stroke-width="2" d="M1490 3620 l0 -210 2345 0 2345 0 0 210 0 210 -2345 0 -2345 0 0
            -210z"/>
            <path stroke-width="2" d="M1490 2345 l0 -215 2345 0 2345 0 -2 213 -3 212 -2342 3 -2343 2 0
            -215z"/>
            </g>
            </svg>';
    } else {
        $svg_icon='<svg class="weekly-time-with-desc" width="22" height="22" viewBox="0 0 530 530" 
        fill="none" xmlns="http://www.w3.org/2000/svg">
        <g transform="translate(0,505) scale(0.100000,-0.100000)"
        fill="'.$desc_color.'">
        <path d="M1701 5098 c-36 -26 -1095 -1208 -1105 -1233 -11 -31 -7 -3782 5
        -3808 6 -13 23 -31 39 -40 26 -16 115 -17 1065 -17 1151 0 1082 -4 1111 67 20
        46 10 86 -28 118 l-30 25 -979 0 -979 0 0 1710 0 1710 500 0 500 0 32 29 33
        29 3 611 2 611 1170 0 1170 0 0 -1501 c0 -1664 -5 -1550 65 -1583 41 -20 49
        -20 90 0 70 33 65 -87 65 1638 0 1323 -2 1562 -14 1591 -29 70 67 65 -1379 65
        l-1304 0 -32 -22z m-54 -1255 c-2 -2 -181 -2 -397 -1 l-394 3 395 440 394 440
        3 -439 c1 -242 1 -441 -1 -443z"/>
        <path d="M3855 1381 c-11 -5 -29 -19 -40 -31 -18 -20 -20 -40 -25 -259 l-5
        -236 -237 -5 c-230 -5 -237 -6 -261 -28 -32 -30 -40 -74 -23 -117 25 -60 48
        -65 301 -65 l225 0 0 -236 0 -236 29 -29 c33 -33 83 -39 130 -15 47 25 51 49
        51 291 l0 225 225 0 c246 0 270 5 293 55 21 47 15 94 -17 126 l-29 29 -236 0
        -236 0 0 225 c0 253 -5 275 -65 301 -38 16 -51 16 -80 5z"/>
        </g>
        </svg>';
    }

    return $svg_icon;

}

function get_last_labor_member_id_for_weekly_view($timeslots) {
    $last_labor_member_id = 0;
    foreach($timeslots as $timeslot){
        $last_labor_member_id = array_var($timeslot,'labor_cat_member_id', 0);
    }

    return $last_labor_member_id;
   
}