<?php 
	class CObjTemplateObjects extends BaseCObjTemplateObjects {
		

		
		static function getByTemplate($template_id) {
		
			return self::instance()->findAll(array("conditions" => array("template_id = ?", $template_id)));
			
		}


		static function getRawObject($template_id, $object_id) {
			return self::instance()->findById(array("template_id" => $template_id, "object_id" => $object_id));
		}
		

		static function getNextObjectId($template_id) {
			$last_object = self::instance()->findOne(array(
				"conditions" => array("template_id = ? AND object_id = (
					SELECT MAX(x.object_id) FROM ".TABLE_PREFIX."cobj_template_objects x WHERE x.template_id = ?
				)", $template_id, $template_id)
			));

			if ($last_object instanceof CObjTemplateObject) {
				$next_id = $last_object->getObjectId() + 1;
			} else {
				$next_id = 1;
			}

			return $next_id;
		}


		/**
		 * @param ObjectType $object_type
		 * @return ContentDataObject 
		 */
		static function newContentObject(ObjectType $object_type) {
			// get data manager class
			$manager_cls = $object_type->getHandlerClass();
			// create data manager instance
			$manager = new $manager_cls();

			// get content object class
			$item_cls = $manager->getItemClass();
			// create new object
			$object = new $item_cls();

			// set type
			//$object->setObjectTypeId($object_type->getId());

			return $object;
		}


		static function buildObjectFromData(ObjectType $object_type, $data) {

			$object = self::newContentObject($object_type);

			// set basic attributes
			$object->setFromAttributes($data);

			$feng_obj = new FengObject();
			$feng_obj->setObjectTypeId($object_type->getId());
			$feng_obj->setUpdatedOn(DateTimeValueLib::now());
			$feng_obj->setCreatedOn(DateTimeValueLib::now());
			$feng_obj->setUpdatedById(logged_user()->getId());
			$feng_obj->setCreatedById(logged_user()->getId());
			$feng_obj->setName($data['name']);
			$object->setObject($feng_obj);

			$object->setNew(false);

			// if object is a budgeted expense -> set the lines
			if ($object_type->getName() == 'expense') {
				$data_lines = $data['lines'];
				$expense_items = array();
				foreach ($data_lines as $data_line) {
					foreach ($data_line as $line_col => &$line_col_value) {
						if (in_array($line_col, array('unit_cost','total_cost','unit_price','total_price','price_multiplier'))) {
							$line_col_value = clean_formatted_money_amount_for_sql($line_col_value, false); // don't try to guess thousands separator if empty
						}
					}
					$exp_item = new ExpenseItem();
					$exp_item->setFromAttributes($data_line);
					$expense_items[] = $exp_item;
				}
				$object->items = $expense_items;
			}

			// set custom properties
			$object->setUseCachedCustomProperties(true);
			$object->setCachedCustomProperties($data['object_custom_properties']);

			// members
			if (is_array($data['members'])) {
				$mem_ids = $data['members'];
			} else {
				$mem_ids = json_decode($data['members'], true);
			}
			
			if (is_array($mem_ids) && count($mem_ids) > 0) {
				$members = Members::instance()->findAll(array("conditions" => "id IN (".implode(',', $mem_ids).")"));
				$object->setMembers($members);
				$object->setMemberIds($mem_ids);
			}

			// subscribers

			// linked objects

			return $object;
		}


		static function buildObjectDataFromRequest($request) {
			$data = array();

			$object_type = ObjectTypes::instance()->findById($request['object_type_id']);

			$manager_cls = $object_type->getHandlerClass();
			$manager = new $manager_cls();
			
			$object_data = $request[$object_type->getName()];

			foreach ($manager->getColumns() as $col) {
				$col_value = array_var($object_data, $col);

				if ($col_value) {
					if ($manager->getColumnType($col) == DATA_TYPE_DATE || $manager->getColumnType($col) == DATA_TYPE_DATETIME) {
						$dt = getDateValue($col_value);
						$data[$col] = $dt->toMySQL();
					} else {
						$data[$col] = $col_value;
					}
				}
			}
			foreach (Objects::instance()->getColumns() as $col) {
				$col_value = array_var($object_data, $col);

				if ($col_value) {
					if ($manager->getColumnType($col) == DATA_TYPE_DATE || $manager->getColumnType($col) == DATA_TYPE_DATETIME) {
						$dt = getDateValue($col_value);
						$data[$col] = $dt->toMySQL();
					} else {
						$data[$col] = $col_value;
					}
				}
			}

			// for expenses, build expense items
			if ($object_type->getName() == 'expense') {
				$data['lines'] = array();
				foreach ($object_data['expense_items'] as $item) {
					$line = array();
					
					$manager = ExpenseItems::instance();
					foreach ($item as $col => $col_value) {
						if ($col_value) {
							if ($manager->getColumnType($col) == DATA_TYPE_DATE || $manager->getColumnType($col) == DATA_TYPE_DATETIME) {
								$dt = getDateValue($col_value);
								$line[$col] = $dt->toMySQL();
							} else {
								$line[$col] = $col_value;
							}
						}
					}

					$data['lines'][] = $line;
				}
			}

			// members
			$member_ids = json_decode($request['members'], true);
			$data['members'] = $member_ids;

			// custom properties
			$data['object_custom_properties'] = $request['object_custom_properties'];

			// subscribers
			$data['subscribers'] = $request['subscribers'];

			// linked objects
			$data['linked_objects'] = $request['linked_objects'];

			// index of the object in the form
			$data['idx'] = $request['idx'];

			return $data;
		}
		
	}
 