<?php
Hook::register("property_groups");


function property_groups_additional_custom_property_fields($params, &$prop) {
	$cp = array_var($params, 'cp');
	$ot = array_var($params, 'ot');
	
	$pg = PropertyGroups::getByObjectTypeAndProperty($ot->getId(), $cp['id']);
	$prop['property_group'] = $pg instanceof PropertyGroup ? $pg->getId() : '';

	$pgp = PropertyGroups::getPropertyGroupPropertyByObjectTypeAndPropertyId($ot->getId(), $cp['id']);
	if($pgp instanceof PropertyGroupProperty && $ot->getName() == 'project'){
		$prop['is_inheritable'] = $pgp->getIsInheritable() == 1 ? 'checked' : '';
	}

}

function property_groups_custom_property_form_sections($parameters, &$headers) {
	$object_type = array_var($parameters, 'ot');
	if ($object_type->getType() == 'content_object') {
		echo '<br /><div class="desc">'.lang("the group configuration only applies in object view, not yet in the add/edit form").'</div><br />';
	}
}

function property_groups_custom_properties_form_modify_headers($params, &$headers) {
	$ot = array_var($params, 'ot');
	if ($ot instanceof ObjectType && in_array($ot->getType(), array('content_object','dimension_object','dimension_group'))) {
		$last = array_pop($headers);
		$headers[] = "group";
		$headers[] = $last;
	}
	if($ot->getName() == 'project') {
		$last = array_pop($headers);
		$headers[] = "is_inheritable";
		$headers[] = $last;
	}
}

function property_groups_custom_properties_form_additional_columns($params, &$additional_columns) {
	$ot = array_var($params, 'ot');

	if ($ot instanceof ObjectType && in_array($ot->getType(), array('content_object','dimension_object','dimension_group'))) {
		$property_groups = PropertyGroups::getByObjectType($ot->getId());

		$options = array();
		foreach ($property_groups as $pg) {
			$options[] = option_tag($pg->getName(), $pg->getId());
		}
		$html = select_box("custom_properties[{number}][property_group]", $options, array("id" => "property_group"));

		$additional_columns[] = array('html' => $html);
	}

	if($ot->getName() == 'project') {
		$html = checkbox_field("custom_properties[{number}][is_inheritable]", false, array('id' => 'is_inheritable'));
		$additional_columns[] = array('html' => $html);
	}
}

function property_groups_list_custom_properties_for_type_extra_parameters($parameters, &$extra_params) {
	$object_type = array_var($parameters, 'object_type');

	if ($object_type instanceof ObjectType && !in_array($object_type->getType(), array('content_object','dimension_object','dimension_group'))) {
		if (!isset($extra_params['additional_hidden_fields'])) $extra_params['additional_hidden_fields'] = array();
		$extra_params['additional_hidden_fields']['property_group'] = '';
	}
}



function property_groups_after_custom_property_save($params, &$ret) {
	$cp = array_var($params, 'cp');
	$data = array_var($params, 'data');
	$ot_id = array_var($params, 'ot_id');
	$order = array_var($params, 'order');

	$pg = array_var($data, 'property_group');
	$is_inheritable = array_var($data, 'is_inheritable', 0);
	$old_pg = PropertyGroups::getByObjectTypeAndProperty($ot_id, $cp->getId());
	if ($old_pg instanceof PropertyGroup) {
		PropertyGroupProperties::removePropertyFromGroup($old_pg->getId(), $cp->getId());
	}
	if (is_numeric($pg) && $pg > 0) {
		$prop_id = $data['id'] != '' && !is_numeric($data['id']) ? $data['id'] : $cp->getId();
		$pgp = PropertyGroupProperties::getByGroupAndProperty($pg, $prop_id);
		if (!$pgp instanceof PropertyGroupProperty) {
			$pgp = new PropertyGroupProperty();
			$pgp->setGroupId($pg);
			$pgp->setPropertyId($prop_id);
		}
		$pgp->setIsInheritable($is_inheritable);
		$pgp->setOrder($order);
		$pgp->save();
	}
}

function property_groups_custom_properties_form_page_actions($params, &$ret) {
	$ot = array_var($params, 'ot');

	$action = "javascript:og.render_modal_form('', {c:'property_groups', a:'add', params: {ot_id:".$ot->getId()."}});";
	add_page_action(lang("add new property group"), $action, 'ico-add', null, null, true);
}

function property_groups_render_administration_icons($ignored, &$links) {
	if (can_manage_dimensions(logged_user())) {
		$links[] = array(
				'ico' => 'ico-large-custom-properties',
				'url' => get_url('property_groups', 'list_all'),
				'name' => lang('property groups'),
				'extra' => '',
		);
	}
}


function property_groups_override_render_properties($params, &$more_properties) {
	Env::useHelper('functions', 'advanced_core');
	$genid = array_var($params, 'genid', gen_id());
	
	$align_group_properties = array_var($params, 'align_group_properties', false);
    $member = array_var($params, 'member');
    if (!$member instanceof Member) {
    	$object = array_var($params, 'object');
		$ot = ObjectTypes::instance()->findById($object->getObjectTypeId());
    } else {
    	$ot = ObjectTypes::instance()->findById($member->getObjectTypeId());
    	if ($ot->getType() == 'dimension_object') {
    		$object = Objects::findObject($member->getObjectId());
    		if (!$object instanceof ContentDataObject) {
    			eval('$ot_manager = '.$ot->getHandlerClass().'::instance();');
    			if (isset($ot_manager) && $ot_manager instanceof ContentDataObjects && $ot_manager->getItemClass() != '') {
    				eval('$object = new '.$ot_manager->getItemClass().'();');
    				$object->setObjectTypeId($ot->getId());
    			}
    		}
    	} else {
    		$object = $member;
    	}
    }
    
    if ($object instanceof ContentDataObject) {
    	$additional_fixed_columns = $object->manager()->getAssociatedObjectsFixedColumns();
    }
    
	$input_base_name = $object instanceof Member ? 'member_custom_properties' : 'object_custom_properties';
	
	$visible_by_default = array_var($params, 'visible_by_default');
    if (!$visible_by_default) {
        $more_properties = array(array('id' => 0, 'html' => ''));
        return;
    }
	
	if ($object instanceof Member || $object instanceof ContentDataObject) {
		
		$grouped_properties = PropertyGroups::getAllPropertiesGroupedByPropertyGroup($object->getObjectTypeId());
		if (count($grouped_properties) > 0) {
			
			if($align_group_properties){
				$more_properties = array('left_side' => '', 'right_side' => '');
			} else {
				$more_properties = array();
			}

			foreach ($grouped_properties as $group) {
        		
				// check if user can view or edit the property group
				$pg_perm = null;
				Hook::fire('check_property_group_permissions', array('user'=>logged_user(),'property_group'=>$group), $pg_perm);
				if ($pg_perm == 'none') continue;
				
				foreach ($group['properties'] as $k => &$p) {
		        	/* @var $p PropertyGroupProperty */
		        	if (is_numeric($p['property_id'])) {
		        		if ($object instanceof Member) {
			        		$cp = MemberCustomProperties::instance()->findById($p['property_id']);
		        		} else {
			        		$cp = CustomProperties::instance()->findById($p['property_id']);
			        		
			        		$subtype_id = array_var($_REQUEST, 'object_subtype_id', $object->getColumnValue('object_subtype_id'));
			        		if ($cp instanceof CustomProperty && $cp->getColumnValue('object_subtype_id') > 0 &&
			        			$cp->getColumnValue('object_subtype_id') != $subtype_id) {
			        			// exclude other subtypes cps
			        			$cp = null;
			        		}
						}
						// If object type is customer get only contact custom properties
						$correct_contact_type = true;
						if ($cp instanceof CustomProperty && $ot instanceof ObjectType && $ot->getName() == 'customer') {
							$contact_types = array('all', 'contact', '');
							$correct_contact_type = in_array($cp->getContactType(), $contact_types);
						}
						$correct_info_type = true;
						Hook::fire('check_custom_property_info_type', array('cp'=>$cp), $correct_info_type);
		        		if (!$cp || $cp->getIsDisabled() || !$correct_info_type || !$correct_contact_type) {
		        			unset($group['properties'][$k]);
		        		}
		        	}
				}
				
				if (count($group['properties']) == 0) continue;
				
				$group_html = '<div class="property-group property-group-'.$group['id'].'">';
				//if ($group['code'] != 'default') {
					$group_html .= '<div class="property-group-header">' . $group['name'] . '</div>';
				//}
				
				$group_html .= '<div class="property-group-body">';
				
				foreach ($group['properties'] as $prop_data) {
					$property_id = $prop_data['property_id'];
					
					// check if user can view or edit the property
					$property_perm = null;
					Hook::fire('check_property_group_property_permissions', array('user'=>logged_user(),'property_group'=>$group,'property_id'=>$property_id), $property_perm);
					if ($property_perm == 'none') continue;
					
					// if permission not defined for the property -> use the group permission
					if (is_null($property_perm)) $property_perm = $pg_perm;
					
					if (is_numeric($property_id)) {
						// custom property field
						if ($object instanceof Member) {
							$customProp = MemberCustomProperties::instance()->findById($property_id);
						} else {
							$customProp = CustomProperties::instance()->findById($property_id);
						}
						if ($customProp) {
							$obj_for_cp = $object;
							if ($object instanceof ContentDataObject && $object->getObjectTypeId() != $customProp->getObjectTypeId()) {
								$tmp = $object->getAdditionalCustomPropertyAssociatedObject($customProp);
								if ($tmp instanceof ContentDataObject) $obj_for_cp = $tmp;
							}
							
							$group_html .= get_custom_property_input_html($customProp, $obj_for_cp, $genid, $input_base_name, $object->getId(), $property_perm);
						}
						
					} else if (str_starts_with($property_id, "assoc_")) {
						// associated dimension selector
						$association_id = str_replace("assoc_", "", $property_id);
						$dim_association = DimensionMemberAssociations::instance()->findById($association_id);
						
						if ($dim_association instanceof DimensionMemberAssociation) {
							if (!$member instanceof Member) {
								if ($object->getId() > 0) {
									$member = Members::findOneByObjectId($object->getId());
								}
								if (!$member instanceof Member) {
									$member = new Member();
									$member->setObjectTypeId($object->getObjectTypeId());
									
									if ($object->getObjectTypeId() == $dim_association->getObjectTypeId()) {
										$dim_id = $dim_association->getDimensionId();
									} else {
										$dim_id = $dim_association->getAssociatedDimensionMemberAssociationId();
									}
									$member->setDimensionId($dim_id);
								}
							}
							$params = array(
									'member' => $member,
									'is_new' => $member->isNew(),
									'dim_associations' => array($dim_association),
									'suffix' => $association_id,
									'readonly' => $property_perm == 'view',
							);
							
							ob_start();
							render_associated_dimensions_selectors($params);
							$sel_html = ob_get_clean();
							
							$group_html .= $sel_html;
						}
						
					} else if ($property_id == "located_under") {
						if (!$member instanceof Member && $object->getId()>0) {
							$member = Members::findOneByObjectId($object->getId());
						}
						if ($member instanceof Member && $member->canHaveParents()) {
							
						    $parent_member = 0;
						    
							if ($member->isNew()) {
								$active_context = active_context();
								foreach ($active_context as $selection) {
									if ($selection instanceof Member && $selection->getDimensionId() == $member->getDimensionId()) {
										$parent_member = $selection->getId();
										break;
									}
								}
							} else {
								$parent = $member->getParentMember();
								$parent_member = $parent instanceof Member ? $parent->getId() : 0;
							}
							
							ob_start();
							
							?><div class="field"><?php
							render_single_member_selector($member->getDimension(), $genid, array($parent_member), array(
								'is_multiple' => false, 
								'label' => lang('located under'),
								'dont_filter_this_selector' => true,
								'width' => 400,
								'select_function' => 'og.onParentMemberSelect', 
								'listeners' => array('on_remove_relation' => "og.onParentMemberRemove('".$genid."');"),
								'readonly' => $property_perm == 'view',
							), false);
							?></div><?php 
							
							$sel_html = ob_get_clean();
							
							$group_html .= $sel_html;
						
						}
						
					} else if (str_starts_with($property_id, "fixedprop_")) {
						
						$fixed_prop_str = str_replace("fixedprop_", "", $property_id);
						$fixed_prop_arr = explode("|", $fixed_prop_str);
						

						if (count($fixed_prop_arr) == 2) {
							
							$assoc_obj_col = array_var($fixed_prop_arr, 0);
							$column = array_var($fixed_prop_arr, 1);
							
							$col_info = null;
							foreach ($additional_fixed_columns[$assoc_obj_col] as $coldef) {
								if ($coldef['col'] == $column) {
									$col_info = $coldef;
									break;
								}
							}
							
							if (!is_null($col_info)) {
							
							    $associated_object = Objects::findObject($object->getColumnValue($assoc_obj_col));
								$value = "";
								if (!$object->isNew()) {
									if ($associated_object instanceof ContentDataObject) {
										$value = $associated_object->getFixedColumnValue($column, true);
										
										if (in_array($col_info['type'], array(DATA_TYPE_DATETIME)) && $value instanceof DateTimeValue) {
											$value->advance($associated_object->getTimezoneValue());
										}
									}
								} else {
									if ($col_info['type'] == DATA_TYPE_TIMEZONE) {
										$value = logged_user()->getUserTimezoneId();
									}
								}
								$input_name = "dim_obj[associated][$assoc_obj_col][$column]";
								
								$group_html .= render_object_fixed_property_input($genid, $input_name, $col_info, $value, $associated_object, $property_perm);
							}
						} else {
                    		
                    		$col = $fixed_prop_str[0];
                    		$input_name = "dim_obj[$col]";
                    		$col_info = array('label' => lang('field '.$ot->getHandlerClass(). " $col"), 'type' => $object->getColumnType($col), 'col' => $col);
                    		
                    		$group_html .= render_object_fixed_property_input($genid, $input_name, $col_info, $object->getColumnValue($col), $object, $property_perm);
                    	}
					}
				}
				
				$group_html .= '</div></div>';
				
				if ($group['code'] == 'default' && $pg_perm == 'view') {
					$group_html .= '<script>
						$("#'.$genid.'-name").attr("disabled","disabled");
						$("#'.$genid.'submit-edit-form .headerToolbar").remove();
					</script>';
				}

				if($align_group_properties){
					if($group['alignment'] == 'left'){
						$more_properties['left_side'] .= $group_html;
					} else {
						$more_properties['right_side'] .= $group_html;
					}
				}
				
				$more_properties[] = array('id' => $property_id, 'html' => $group_html);
			}
		}
	}
}


function property_groups_override_render_properties_for_view($params, &$cp_html) {
    Env::useHelper('functions', 'advanced_core');
    if (!isset($genid)) $genid = gen_id();
	
    $member = array_var($params, 'member');
    if (!$member instanceof Member) {
    	$object = array_var($params, 'object');
    } else {
    	$ot = ObjectTypes::instance()->findById($member->getObjectTypeId());
    	if ($ot->getType() == 'dimension_object') {
    		$object = Objects::findObject($member->getObjectId());
    	} else {
    		$object = $member;
    	}
    }
    
    $visible_by_default = array_var($params, 'visible_by_default');
    if (!$visible_by_default) {
        $cp_html = '';
        return;
    }
    
    $return_as_array = array_var($params, 'return_as_array');
	$properties_array = array();
    $max_breadcrumb_size = array_var($params, 'max_breadcrumb_size', '500px');
    

    if ($object instanceof Member || $object instanceof ContentDataObject) {

    	$ot = ObjectTypes::instance()->findById($object->getObjectTypeId());
    	if ($ot->getType() == 'dimension_object') {
    		$associated_object_columns = $object->manager()->getAssociatedObjectsFixedColumns();
    	}
    	
		$grouped_properties = PropertyGroups::getAllPropertiesGroupedByPropertyGroup($object->getObjectTypeId());
        if (count($grouped_properties) > 0) {
        	
        	$more_properties = array();

			// Check how many grouped properties we have
			/* PREVIOUS SPLITTER
			$left_side_number = round((count($grouped_properties) - 1) / 2);
			Logger::log($left_side_number);
			$number_of_groups = count($grouped_properties) - 1;
			Logger::log($number_of_groups);
			$group_count = 0;
			$widget_side = '';*/


			
			$html_grouped_properties_left_side = array();
			$html_grouped_properties_right_side = array();
			
            foreach ($grouped_properties as $group) {
            	
            	$pg_perm = null;
            	Hook::fire('check_property_group_permissions', array('user'=>logged_user(),'property_group'=>$group), $pg_perm);
            	if ($pg_perm == 'none') continue;
            	
				foreach ($group['properties'] as $k => &$p) {
					$group_outer_html = '';
		        	/* @var $p PropertyGroupProperty */
		        	if (is_numeric($p['property_id'])) {
		        		if ($object instanceof Member) {
		        			$cp = MemberCustomProperties::instance()->findById($p['property_id']);
		        		} else {
		        			$cp = CustomProperties::instance()->findById($p['property_id']);
						}
						$correct_info_type = true;
						Hook::fire('check_custom_property_info_type', array('cp'=>$cp), $correct_info_type);
						if ($cp instanceof CustomProperty && $ot->getName() == 'contact') {
							if($object->isUser()){
								$user_types = array('all', 'user');
								$correct_info_type = in_array($cp->getContactType(), $user_types);
							} else {
								$contact_types = array('all', 'contact');
								$correct_info_type = in_array($cp->getContactType(), $contact_types);
							}
						}
		        		if (!$cp || $cp->getIsDisabled() || !$correct_info_type) {
		        			unset($group['properties'][$k]);
		        		}
		        	}
				}

				if (count($group['properties']) == 0) continue;
				/* PREVIOUS SPLITTER
				if ($group_count < $left_side_number){
					if(!($widget_side == 'left_side')){
						$group_outer_html .= '<div class="information-widget--left-side">';
						$widget_side = 'left_side';
					}
				} else {
					if(!($widget_side == 'right_side' )){
						$group_outer_html .= '<div class="information-widget--right-side">';
						$widget_side = 'right_side';
					}
				}
				
				$group_count += 1;
				*/
					$group_outer_html .= '<div class="information-widget-group">';

              // if ($group['code'] != 'default') {
                    $group_outer_html .= '<div class="information-widget-group-header">' . $group['name'] . '</div>';
              //  }
                $group_outer_html .= '<div class="information-widget-group-table"><table style="width:100%;">';
                
                $group_html = "";

                foreach ($group['properties'] as $prop_data) {
                    $property_id = $prop_data['property_id'];
                    
                    // check if user can view or edit the property
                    $property_perm = null;
                    Hook::fire('check_property_group_property_permissions', array('user'=>logged_user(),'property_group'=>$group,'property_id'=>$property_id), $property_perm);
                    if ($property_perm == 'none') continue;
                    
                    if (is_numeric($property_id)) {
                        if ($object instanceof Member) {
		        			$customProp = MemberCustomProperties::instance()->findById($property_id);
                        } else {
							$customProp = CustomProperties::instance()->findById($property_id);
						}
                        if ($customProp) {

                            if ($customProp->getType() == 'color' || $customProp->getCode() == 'calculate the status automatically_special') {
                                continue;
                            }
                            $cp_name = $customProp->getName();
                            if ($object instanceof Member) {
                            	$cp_values = MemberCustomPropertyValues::getMemberCustomPropertyValues($object->getId(), $customProp->getId());
                            } else {
                            	if ($object->getObjectTypeId() == $customProp->getObjectTypeId()) {
	                            	// cp is associated directly to the object
	                            	$cp_values = CustomPropertyValues::getCustomPropertyValues($object->getId(), $customProp->getId());
                            	} else {
									// cp is associated to an external object associated to $object
									$cp_values = $object->getAdditionalCustomPropertyValues($customProp);
                            	}
                            }
                            
                            $cp_html_cp = "";
                            if (is_array($cp_values) && count($cp_values) > 0) {
                                $first = true;
                                $cp_html_cp .= '<tr class="information-widget-table-row"><td class="information-widget-table-definition">'.$cp_name.': </td><td class="information-widget-table-value">';
                                $cp_html_vals = "";
                                
                                if ($object instanceof Member) {
	                                foreach ($cp_values as $cp_val) {
	                                    $formatted_val = $cp_val->format_value();
	                                    if ($formatted_val == "") continue;

	                                    if (!$first) {
	                                        $cp_html_vals .= ", ";
	                                    }
	                                    $first = false;
	                                    $cp_html_vals .= $formatted_val;

	                                    if (!$customProp->getIsMultipleValues()) {
	                                        break;
	                                    }
	                                }
                                	
                                } else {
                                	$cp_html_vals = get_custom_property_value_for_listing($customProp, $object, $cp_values, false, array('table_html'=>true));
                                }
                                
                                if ($return_as_array) {
                                	$properties_array[$group['id']]['properties'][] = array('label' => $cp_name, 'value' => $cp_html_vals);
                                }
                                
                                $cp_html_cp .= $cp_html_vals;
                                $cp_html_cp .= '</td></tr>';
                                if ($cp_html_vals != "") {
                                    // content
                                    $group_html .= $cp_html_cp;
                                }
                            }

                        }
                        
                    } else if (str_starts_with($property_id, "assoc_") && $member instanceof Member) {
                    	
                    	$association_id = str_replace("assoc_", "", $property_id);
                    	$a = DimensionMemberAssociations::instance()->findById($association_id);
                    	
                    	if ($a instanceof DimensionMemberAssociation) {
	                    	$assoc_mem_ids = explode(',', MemberPropertyMembers::getAllMemberIds($a->getId(), $member->getId()));
	                    	$assoc_mem_ids = array_merge($assoc_mem_ids, explode(',', MemberPropertyMembers::getAllPropertyMemberIds($a->getId(), $member->getId())));
	                    	$assoc_mem_ids = array_unique(array_filter($assoc_mem_ids));
	                    	
	                    	if (is_array($assoc_mem_ids) && count($assoc_mem_ids) > 0) {
	                    		$did = $member->getDimensionId() == $a->getDimensionId() ? $a->getAssociatedDimensionMemberAssociationId() : $a->getDimensionId();
	                    		$assoc_otid = $member->getDimensionId() == $a->getDimensionId() ? $a->getAssociatedObjectType() : $a->getObjectTypeId();
	                    		$dim = Dimensions::getDimensionById($did);
	                    			
	                    		if ($dim->getCode() != 'feng_persons') {
	                    	
	                    			$mems = Members::instance()->findAll(array('conditions' => "id IN (".implode(',', $assoc_mem_ids).")"));
	                    	
	                    			$custom_assoc_name = DimensionAssociationsConfigs::getConfigValue($a->getId(), 'custom_association_name');
	                    			if ($custom_assoc_name) {
	                    				$label = $custom_assoc_name;
	                    			} else {
		                    			$assoc_ot = ObjectTypes::instance()->findById($assoc_otid);
		                    			$custom_ot_name = Members::getTypeNameToShowByObjectType($dim->getId(), $assoc_ot->getId());
		                    			$label = ($a->getIsMultiple() || count($assoc_mem_ids) > 1) ? $dim->getName() : $custom_ot_name;
	                    			}
	                    			
	                    			// content
	                    			$assoc_mem_html = '<tr class="information-widget-table-row"><td class="information-widget-table-definition">'.$label.': </td><td class="information-widget-table-value">';
	                    			$assoc_mem_html_vals = "";
	                    			$member_names = "";
	                    			$breadcrumb_id_prefix = array_var($params, 'breadcrumb_id_prefix', 'widget-active-context-info');
	                    	
	                    			foreach ($mems as $m) {


                                        $assoc_mem_html_vals .= '<div id="widget-active-context-info-'.$m->getId().'" class="breadcrumb-container information-widget--breadcrumb-container"></div>';

                                        $crumb_members = array(
                                            $m->getObjectTypeId() => array($m->getId()),
                                        );
                                        $crumbOptions = json_encode(array($m->getDimensionId() => $crumb_members));
                                        $assoc_mem_html_vals .= '
                                            <script>
                                                mem_path = og.getEmptyCrumbHtml('.$crumbOptions.', \'.breadcrumb-container\', null, null, null, true);
                                                $("#'.$breadcrumb_id_prefix.'-'.$m->getId().'").html(mem_path);
                                            </script>
                                        ';
                                        $member_names .= ($member_names == '' ? '' : ', ') . $m->getName();
		                    			
                                        Hook::fire('append_data_to_assoc_member_view', array('genid'=>$genid, 'member'=>$member, 'assoc_member'=>$m, 'dim_association'=>$a), $assoc_mem_html_vals);
	                    			}
	                    			
	                    			$assoc_mem_html .= $assoc_mem_html_vals . '</td></tr>';
	                    			
	                    			$group_html .= $assoc_mem_html;
	                    			
	                    			if ($return_as_array) {
	                    				$properties_array[$group['id']]['properties'][] = array('label' => $label, 'value' => $member_names);
	                    			}
	                    		}
	                    	}
                    	}
                    	
                    } else if (str_starts_with($property_id, "fixedprop_") && $ot->getType() == 'dimension_object') {
                    	
                    	$fixed_prop_str = str_replace("fixedprop_", "", $property_id);
                    	$fixed_prop_arr = explode("|", $fixed_prop_str);
						
                    	if (count($fixed_prop_arr) == 2) {
                    		
                    		$assoc_obj_col = array_var($fixed_prop_arr, 0);
                    		$column = array_var($fixed_prop_arr, 1);
							if ($column == 'picture_file') continue;
                    		
	                    	foreach ($associated_object_columns[$assoc_obj_col] as $col_info) {
	                    		if ($col_info['col'] == $column) {
	                    			
	                    			$associated_object = Objects::findObject($object->getColumnValue($assoc_obj_col));

	                    			if($associated_object instanceof ContentDataObject){
                                        $html_value = render_object_fixed_property_for_view($col_info, $associated_object->getFixedColumnValue($column), $associated_object);
                                        $group_html .= $html_value;
                                    }
	                    			break;
	                    		}
	                    	}
                    	} else {
                    		
                    		$col = $fixed_prop_str[0];
                    		$col_info = array('label' => lang('field '.$ot->getHandlerClass(). " $col"), 'type' => $object->getColumnType($col), 'col' => $col);
                    		$html_value = render_object_fixed_property_for_view($col_info, $object->getColumnValue($col), $object);
                    		$group_html .= $html_value;
                    	}
                    	
                    	if ($return_as_array) {
                    		$properties_array[$group['id']]['properties'][] = array('label' => $col_info['label'], 'value' => $assoc_mem_html_vals, 'raw_value' => $member_names);
                    	}
                    } else if($property_id == 'days_since_created') {
						$date_created = $object->getCreatedOn();
						$now = DateTimeValueLib::now();
						$timeDiff = DateTimeValueLib::get_time_difference($date_created->getTimestamp(), $now->getTimestamp());
						$days_since_created = $timeDiff['days'];
						$col_info = array('label' => lang('days since created'), 'type' => DATA_TYPE_STRING);
						$html_value = '<tr class="information-widget-table-row"><td class="information-widget-table-definition">'.lang('days since created').': </td><td class="information-widget-table-value">'.$days_since_created.'</td></tr>';
						$group_html .= $html_value;
					} /* else if($property_id == 'forecasted_profitability') {
						$can_see_billing_info = true;
						Hook::fire('get_can_see_billing_information', array('user'=>logged_user()), $can_see_billing_info);
						$can_see_cost_info = true;
						Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $can_see_cost_info);

						if($can_see_billing_info && $can_see_cost_info) {
							$col_info = array('label' => lang('forecasted profitability'), 'type' => DATA_TYPE_STRING);
							$currency_symbol = config_option('currency_code');
							$forecasted_profitability = format_money_amount(0, $currency_symbol);
							$object_id = $object->getId();
							$project_member = Members::instance()->findOne(array('conditions' => array('object_id = ?', $object_id)));
							if($project_member instanceof Member) {
								$member_id = $project_member->getId();
								$member_totals = MemberTotals::instance()->findOne(array('conditions' => array('member_id = ?', $member->getId())));
								if($member_totals instanceof MemberTotal) {
									$estimated_revenue = $member_totals->getColumnValue('estimated_overall_total_price');
									$estimated_cost = $member_totals->getColumnValue('estimated_overall_total_cost');
									$estimated_profit = $estimated_revenue - $estimated_cost;
									$forecasted_profitability = format_money_amount($estimated_profit, $currency_symbol);
								}
							}
							$html_value = '<tr class="information-widget-table-row"><td class="information-widget-table-definition">'.lang('forecasted profitability').': </td><td class="information-widget-table-value">'.$forecasted_profitability.'</td></tr>';
							$group_html .= $html_value;
						}
					} else if($property_id == 'actual_profitability') {
						$can_see_billing_info = true;
						Hook::fire('get_can_see_billing_information', array('user'=>logged_user()), $can_see_billing_info);
						$can_see_cost_info = true;
						Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $can_see_cost_info);

						if($can_see_billing_info && $can_see_cost_info) {
							$col_info = array('label' => lang('actual profitability'), 'type' => DATA_TYPE_STRING);
							$currency_symbol = config_option('currency_code');
							$actual_profitability = format_money_amount(0, $currency_symbol);
							$object_id = $object->getId();
							$project_member = Members::instance()->findOne(array('conditions' => array('object_id = ?', $object_id)));
							if($project_member instanceof Member) {
								$member_id = $project_member->getId();
								$member_totals = MemberTotals::instance()->findOne(array('conditions' => array('member_id = ?', $member->getId())));
								if($member_totals instanceof MemberTotal) {
									$actual_revenue = $member_totals->getColumnValue('executed_overall_total_price');
									$actual_cost = $member_totals->getColumnValue('executed_overall_total_cost');
									$actual_profit = $actual_revenue - $actual_cost;
									$actual_profitability= format_money_amount($actual_profit, $currency_symbol);
								}
							}
							$html_value = '<tr class="information-widget-table-row"><td class="information-widget-table-definition">'.lang('actual profitability').': </td><td class="information-widget-table-value">'.$actual_profitability.'</td></tr>';
							$group_html .= $html_value;
						}
					}*/
				}
				
				/* PREVIOUS SPLITTER
                if (trim($group_html) != "") {
					$group_outer_html .= $group_html;
					if ($group_count == $left_side_number){
						$group_outer_html .= '</table></div></div></div>';
					} else if ($group_count == $number_of_groups) {
						$group_outer_html .= '</table></div></div></div>';
					} else {
						$group_outer_html .= '</table></div></div>';
					}
	                $cp_html .= $group_outer_html;
				}
				*/
				if (trim($group_html) != "") {
					$group_outer_html .= $group_html;
					$group_outer_html .= '</table></div></div>';
					if($group['alignment'] == 'left'){
						array_push($html_grouped_properties_left_side, $group_outer_html);
					} else if($group['alignment'] == 'right'){
						array_push($html_grouped_properties_right_side, $group_outer_html);
					}
					
	                //$cp_html .= $group_outer_html;
                }
                
			}
			/* SECOND VERSION
			$group_count = count($html_grouped_properties_array);
			$group_list_divider = round($group_count / 2);
			// Populate left side
			$left_side = '<div class="information-widget--left-side">';
			for ($i = 0; $i < $group_list_divider; $i++){
				$left_side .= $html_grouped_properties_array[$i];
			}
			$left_side .= '</div>';

			// Populate right side
			$right_side = '<div class="information-widget--right-side">';
			for ($i = $group_list_divider; $i < $group_count; $i++){
				$right_side .= $html_grouped_properties_array[$i];
			}
			$right_side .= '</div>';
			*/
			// Populate left side and add it to the final $cp_html
			if(count($html_grouped_properties_left_side) > 0){
				$left_side = '<div class="information-widget--left-side">';
				foreach ($html_grouped_properties_left_side as $left_side_group){
					$left_side .= $left_side_group;
				}
				$left_side .= '</div>';
				$cp_html .= $left_side;
			}

			// Populate right side and add it to the final $cp_html
			if(count($html_grouped_properties_right_side) > 0){
				$right_side = '<div class="information-widget--right-side">';
				foreach ($html_grouped_properties_right_side as $right_side_group){
					$right_side .= $right_side_group;
				}
				$right_side .= '</div>';
				$cp_html .= $right_side;
			}

        }
		$additional_properties = "";
		Hook::fire('add_additional_custom_properties_to_view', $params, $additional_properties);
		$cp_html .= $additional_properties;

		if ($return_as_array) {
			$cp_html = $properties_array;
		}

    }
}



function property_groups_list_custom_properties_for_type_modify_cps($params, &$custom_properties) {
	
	$object_type = array_var($params, 'ot');

	if ($object_type->getType() == 'dimension_object') {
		eval('$ot_manager = '.$object_type->getHandlerClass().'::instance();');
		if (isset($ot_manager) && $ot_manager instanceof ContentDataObjects) {
			
			$additional_cps = $ot_manager->getAdditionalCustomProperties();
			foreach ($additional_cps as &$cp) {
				$cp->setIsSpecial(true);
				$info = $cp->getArrayInfo();
				
				$order = PropertyGroups::getPropertyOrderInGroup($object_type, $cp->getId());
				$info['property_order'] = $order;
				
				$custom_properties[] = $info;
			}
			
			// columns of the associated object of the member object
			$additional_fixed_columns = $ot_manager->getAssociatedObjectsFixedColumns();
			
			// columns of the member object
			$current_object_columns = $ot_manager->getPublicColumns();
			foreach ($current_object_columns as $col_data) {
				$additional_fixed_columns[$col_data['col']] = true;
			}
			if (isset($additional_fixed_columns)) {
				foreach ($additional_fixed_columns as $assoc_col_id => $object_columns) {
					foreach ($object_columns as $column) {
						if (is_array($object_columns)) {
							$prop_id = "fixedprop_" . $assoc_col_id . "|" . $column["col"];
							$label = array_var($column, 'label', lang($column["col"]));
						} else {
							$prop_id = "fixedprop_" . $assoc_col_id;
							$label = lang('field ' . get_class($ot_manager) . " $assoc_col_id");
						}
						$order = PropertyGroups::getPropertyOrderInGroup($object_type, $prop_id);
						$is_inheritable = PropertyGroups::getPropertyIsInheritable($object_type, $prop_id);

						$info = array(
							'id' => $prop_id,
							'name' => $label,
							'is_special' => true,
							'no_options' => true,
							'property_order' => $order,
							'is_inheritable' => $is_inheritable
						);
						$custom_properties[] = $info;
					}
				}
			}
		}
	}
	
	
	if (in_array($object_type->getType(), array('dimension_object', 'dimension_group'))) {
		
		$located_under_order = PropertyGroups::getPropertyOrderInGroup($object_type, "located_under");
		if ($located_under_order == 9999) $located_under_order = -1;
		$custom_properties[] = array(
			'id' => "located_under",
			'name' => lang("located_under"),
			'is_special' => true,
			'no_options' => true,
			'property_order' => $located_under_order,
		);
		
		// meter cps falsas con las asociaciones
		$dimension_ids = DimensionObjectTypes::getDimensionIdsByObjectTypeId($object_type->getId());
		if (count($dimension_ids) > 0) {
			$dimension_id = array_var($dimension_ids, 0);
			// only get associations where current object type is the main one
			$dim_associations = DimensionMemberAssociations::getAssociatations($dimension_id, $object_type->getId());
			
			$enabled_dimension_ids = config_option('enabled_dimensions');
			
			foreach ($dim_associations as $dim_association) {
				/* @var $dim_association DimensionMemberAssociation */
				
				$assoc_dimension = Dimensions::getDimensionById($dim_association->getAssociatedDimensionMemberAssociationId());
				$ot = ObjectTypes::instance()->findById($dim_association->getAssociatedObjectType());
				
				if (!in_array($assoc_dimension->getId(), $enabled_dimension_ids)) continue;
				
				$custom_assoc_name = DimensionAssociationsConfigs::getConfigValue($dim_association->getId(), 'custom_association_name');
				if ($custom_assoc_name) {
					$label = $custom_assoc_name;
				} else {
					$custom_name = $assoc_dimension->getOptionValue('custom_dimension_name');
					if ($custom_name && trim($custom_name) != "") {
						$label = $custom_name;
					} else {
						$label = Localization::instance()->lang(str_replace('_',' ', $ot->getName()) . ($dim_association->getIsMultiple() ? 's' : ''));
						if (is_null($label)) {
							$label = $assoc_dimension->getName();
						}
					}
				}
				
				$prop_id = "assoc_".$dim_association->getId();
				$is_required = $dim_association->getIsRequired();
				$is_multiple = $dim_association->getIsMultiple();


				$order = PropertyGroups::getPropertyOrderInGroup($object_type, $prop_id);
				$is_inheritable = PropertyGroups::getPropertyIsInheritable($object_type, $prop_id);
				
				$cp_data = array(
					'id' => $prop_id,
					'name' => $label,
					'is_special' => true,
					'no_options' => true,
					'property_order' => $order,
					'is_inheritable' => $is_inheritable,
					'is_required' => $is_required,
					'is_multiple_values' => $is_multiple
				);
				
				$custom_properties[] = $cp_data;
				
			}
		}
		
		// reorder all cp data items using the order
		Env::useHelper('functions', 'advanced_core');
		usort($custom_properties, "cp_info_sort_by_order");
	}
}



function property_groups_after_action($args,&$ret){

	if ($args['controller']->getControllerName() == "administration" 
		&& $args['action'] == "save_custom_properties_for_type") {
			
			if(!can_manage_configuration(logged_user())) {
				return;
			}
			
			$ot_id = array_var($_REQUEST, 'ot_id');
			$object_type = ObjectTypes::instance()->findById($ot_id);
			if (!$object_type instanceof ObjectType) {
				return;
			}
			
			$custom_properties_parameter = array_var($_POST, 'custom_properties');
			if (is_string($custom_properties_parameter)) {
				$custom_properties = json_decode($custom_properties_parameter, true);
			} else {
				$custom_properties = $custom_properties_parameter;
			}
			
			if (is_array($custom_properties)) {
				foreach ($custom_properties as $order => $data) {
					
					if (!is_numeric($data['id']) && 
						(str_starts_with($data['id'], "assoc_") || str_starts_with($data['id'], "fixedprop_") || $data['id'] == "located_under") ) {
						
						$is_inheritable = array_var($data, 'is_inheritable', 0);
						$old_pg = PropertyGroups::getByObjectTypeAndProperty($ot_id, $data['id']);
						if ($old_pg instanceof PropertyGroup) {
							PropertyGroupProperties::removePropertyFromGroup($old_pg->getId(), $data['id']);
						}

						$pg = array_var($data, 'property_group');
						
						if (is_numeric($pg) && $pg > 0) {
							$pgp = new PropertyGroupProperty();
							$pgp->setGroupId($pg);
							$pgp->setPropertyId($data['id']);
							$pgp->setOrder($order);
							$pgp->setIsInheritable($is_inheritable);
							$pgp->save();
						}
						
					}
				}
			}
	}
}



function property_groups_member_form_render_associated_dimension_selectors($params, &$render) {
	// associated dimension selectors will be rendered inside property groups
	$render = false;
}

function property_groups_custom_property_input_default_value($params, &$default_value){
	$object = array_var($params, 'object');
	$customProp = array_var($params, 'customProp');
	$member_parent_id = array_var($params, 'member_parent_id');
	$project_ot_id = ObjectTypes::findByName('project')->getId();
	
	if ($member_parent_id > 0 && $object->getObjectTypeId() == $project_ot_id) {
		$pgp = PropertyGroups::getPropertyGroupPropertyByObjectTypeAndPropertyId($project_ot_id, $customProp->getId());
		$is_inheritable = $pgp->getIsInheritable();
		if($is_inheritable){
			$project_member = Members::instance()->findById($member_parent_id);
			if($project_member instanceof Member){
				$parent_object_type_id = $project_member->getObjectTypeId();
				if($parent_object_type_id == $project_ot_id){
					$parent_object_id = $project_member->getObjectId();
					$cp_val = CustomPropertyValues::getCustomPropertyValue($parent_object_id, $customProp->getId());
					if($cp_val instanceof CustomPropertyValue){
						$default_value = $cp_val->getValue();
					}
				}
			}
		}
	}
}

function property_groups_get_parent_associated_member_ids($params, &$selected_ids){
	$member = array_var($params, 'member');
	$parent_member_id = array_var($params, 'parent_member_id');
	$project_ot_id = ObjectTypes::findByName('project')->getId();
	if(!$member instanceof Member) return;
	if (!$member->isNew()) return;
	if($member->getObjectTypeId() != $project_ot_id) return;

	$parent_member = Members::instance()->findById($parent_member_id);
	if(!$parent_member instanceof Member) return;

	// Get property group prorperty by association code
	$dim_association_code = array_var($params, 'dim_association_code');
	$dim_mem_assoc = DimensionMemberAssociations::getAssociationByCode($dim_association_code);
	if(!$dim_mem_assoc instanceof DimensionMemberAssociation) return;
	$pgp_property_id = 'assoc_'.$dim_mem_assoc->getId();
	$pgp = PropertyGroups::getPropertyGroupPropertyByObjectTypeAndPropertyId($project_ot_id, $pgp_property_id);
	if(!$pgp instanceof PropertyGroupProperty) return;

	$dimension = array_var($params, 'dimension');
	$ot = array_var($params, 'ot');
	$reverse_relation = array_var($params, 'reverse_relation');
	$is_inheritable = $pgp->getIsInheritable();

	if($is_inheritable){
		$selected_ids = get_all_associated_status_member_ids($parent_member, $dimension, $ot, $reverse_relation, $dim_association_code);
	}
}
