<?php
/**
 * Quota controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class QuotaController extends ApplicationController {

	var $plugin_name = "expenses";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
		Env::useHelper('format');
	}
	
	
	
	function get_quota_data() {
		ajx_current("empty");
		$quota = Quotas::instance()->findById(get_id());
		if(!($quota instanceof Quota)) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		
		$data = $quota->getArrayInfo();
		$data['memPath'] = $quota->getMembersIdsToDisplayPath();
		
		ajx_extra_data(array('quota' => $data));
	}
	
	
	function check_quota_members() {
		ajx_current("empty");
		$quota = Quotas::instance()->findById(get_id());
		if(!($quota instanceof Quota)) {
			ajx_current("empty");
			return;
		}
		
		$member_ids = array();
		$members_str = array_var($_REQUEST, 'member_ids');
		if ($members_str) {
			$member_ids = json_decode($members_str, true);
		}
		
		$ok = true;
		
		if (count($member_ids) > 0) {
			
			$members_to_check = array();
			$tmp_members = Members::instance()->findAll(array("conditions" => "id IN (".implode(',', $member_ids).")"));
			foreach ($tmp_members as $m) {
				if ($m instanceof Member && in_array($m->getDimension()->getCode(), array('activities', 'divisions', 'funding_sources', 'executor_units'))) {
					$members_to_check[] = $m;
				}
			}
			$expense_member_ids = $quota->getMemberIds();
			
			foreach ($members_to_check as $member) {
				if (!in_array($member->getId(), $expense_member_ids)) {
					$ok = false;
					break;
				}
			}
		}
		
		ajx_extra_data(array('ok' => $ok));
		
	}
	
	function get_quota_by_member_ids() {
		ajx_current("empty");
		$member_ids = array();
		$members_str = array_var($_REQUEST, 'member_ids');
		if ($members_str) {
			$members_str = str_replace(array("[","]"), "", $members_str);
			$member_ids = array_filter(explode(",", $members_str));
		}
		if (count($member_ids) == 0) $member_ids[] = "0";
		
		$restricted_member_ids = array();
		$quota_data = $this->find_quota_by_member_ids($member_ids, 0, $restricted_member_ids);
		
		// only update if all 4 dimensions are selected
		if ($quota_data && count($restricted_member_ids) == 4) {
			ajx_extra_data(array("quota" => $quota_data));
		}
	}
	
	
	function list_all() {
		
		ajx_current("empty");
		// Get all variables from request
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		$attributes = array("ids" => explode(',', array_var($_GET,'ids')));
		$dim_order = null;
		$cp_order = null;
		
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		$select_columns = array("*");
		$extra_conditions = "";
		$join_params = null;
		
		if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		} else if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		} else {
			switch ($order) {
				case 'date':
					$order = '`e`.`date`';
					break;
				default:
					$order = "name";
					break;
			}
		}
		
		Hook::fire("listing_extra_conditions", null, $extra_conditions);
		
		$filters = array();

		foreach ($filters as $col => $value) {
			if (!$value || $value == '0') continue;
			$extra_conditions .= " AND e.`$col` = " . DB::escape($value);
		}
		
		
		$only_count_result = array_var($_GET, 'only_result', false);
		
		$result = Quotas::instance()->listing(array(
			"order" => $order,
			"order_dir" => $order_dir,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"select_columns" => $select_columns,
			"extra_conditions" => $extra_conditions,
			"count_results" => false,
			"only_count_results" => $only_count_result,
			"join_params" => $join_params,
			"start" => $start,
			"limit" => $limit,
		));
		
		$quotas = $result->objects;
		$object = $this->prepareObject($quotas, $start, $limit, $result);
		
		ajx_current("empty");
		ajx_extra_data($object);
		tpl_assign("listing", $object);
		
	}
	
	private $quota_cps = null;
	
	private function prepareObject($quotas, $start, $limit, $res_obj) {
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"quotas" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
		
		if (!$this->quota_cps) {
			$this->quota_cps = CustomProperties::getAllCustomPropertiesByObjectType(Quotas::instance()->getObjectTypeId());
		}
		$custom_properties = $this->quota_cps;
		$count = min(array($limit, count($quotas)));
		for ($i = 0; $i < $count; $i++){
			if (isset($quotas[$i])){
				$quota = $quotas[$i];
				if ($quota instanceof Quota){
					$info = $quota->getArrayInfo(true);
					
					$info['memPath'] = json_encode($quota->getMembersIdsToDisplayPath(true));
					$object["quotas"][$i] = $info;
					
					foreach ($custom_properties as $cp) {
						$object["quotas"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $quota);
					}
    			}
			}
		}
		
		return $object;
	}
	
	
	
	function add() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$quota_data = array_var($_POST, 'quota');
		
		if(!is_array($quota_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$p = new Quota();
			$p->setObjectTypeId(Quotas::instance()->getObjectTypeId());
			
			tpl_assign('quota', $p);
			
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$quota = $this->save_quota($quota_data, $member_ids);
				
				DB::commit();
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
				return;
			}
			
			ApplicationLogs::createLog($quota, ApplicationLogs::ACTION_ADD);
			flash_success(lang('success add quota', $quota->getObjectName()));
			ajx_current("back");
			
			if (array_var($_REQUEST, 'modal')) {
				evt_add("reload current panel");
			}
		}
	}
	
	function edit() {
		$this->setTemplate('add');
		
		$quota = Quotas::instance()->findById(get_id());
		if(!($quota instanceof Quota)) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		
		if (logged_user()->isGuest() || !$quota->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		
		$quota_data = array_var($_POST, 'quota');
		
		if(!is_array($quota_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			tpl_assign('quota', $quota);
			
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$log_data = '';
				$quota = $this->save_quota($quota_data, $member_ids, $quota, $log_data);
				
				DB::commit();
	          	
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
				return;
			}
				
			ApplicationLogs::createLog($quota, ApplicationLogs::ACTION_EDIT, false, null, true, $log_data);
			flash_success(lang('success edit quota', $quota->getObjectName()));
			ajx_current("back");
			
			if (array_var($_REQUEST, 'modal')) {
				evt_add("reload current panel");
			}
		}
	}
	
	function save_quota($quota_data, $member_ids, $quota = null, &$log_data='') {
		
		$is_new = false;
		if (is_null($quota) || !$quota instanceof Quota) {
			$quota = new Quota();
			$is_new = true;
		}
		
		// Ensure that there are no other quotas in the current members of dimensions 
		// 'activities', 'divisions', 'funding_sources', 'executor_units'
		if (count($member_ids) > 0) {
			
			$restricted_member_ids = array();
			$row = $this->find_quota_by_member_ids($member_ids, $quota->getId(), $restricted_member_ids);
			/*
			if ($row) {
				$mem_names = "";
				$mems = DB::executeAll("SELECT name FROM ".TABLE_PREFIX."members WHERE id IN (".implode(',', $restricted_member_ids).")");
				foreach ($mems as $mem) {
					$mem_names .= "\n - " . $mem['name'];
				}
				throw new Exception(lang("cant add more than one quota in", $mem_names));
			}
			*/
			// set automatic name if not entered
			if (array_var($quota_data, 'name', '') == '') {
				$sel_members = Members::instance()->findAll(array("conditions" => "id IN (".implode(',', $restricted_member_ids).")"));
				$quota_data['name'] = "";
				foreach ($sel_members as $mem) {/* @var $mem Member */
					$mot = ObjectTypes::instance()->findById($mem->getObjectTypeId());
					if ($mem->getDimension()->getCode() == 'divisions' && $mot->getName()=='item') {
						$quota_data['name'] .= $mem->getName();
					}
					if ($mem->getDimension()->getCode() == 'activities' && $mot->getName()=='activity') {
						$quota_data['name'] .= ($quota_data['name']==""?"":", ") . $mem->getName();
					}
				}
			}
		}
		
		$quota->setFromAttributes($quota_data);
		$quota->save();
		
		$object_controller = new ObjectController();
		if (!is_null($member_ids)) {
			$object_controller->add_to_members($quota, $member_ids);
		}
		
		$object_controller->add_subscribers($quota);
		$object_controller->link_to_new_object($quota);
		$object_controller->add_custom_properties($quota);
		
		return $quota;
	}
	
	
	private function find_quota_by_member_ids($member_ids, $quota_id=0, &$restricted_member_ids) {
		
		$restricted_dimension_ids = Dimensions::instance()->findAll(array("id" => true, "conditions" => "code IN ('activities', 'divisions', 'funding_sources', 'executor_units')"));
		$restricted_member_ids = array();
		$restricted_members = Members::instance()->findAll(array("conditions" => "id IN (".implode(',', $member_ids).")"));
		foreach ($restricted_members as $m) {
			if (in_array($m->getDimensionId(), $restricted_dimension_ids)) {
				$restricted_member_ids[] = $m->getId();
			}
		}
		
		$row = null;
		if (count($restricted_member_ids) > 0) {
			$sql = "
				SELECT q.object_id, o.name FROM ".TABLE_PREFIX."quotas q
				INNER JOIN ".TABLE_PREFIX."objects o ON o.id=q.object_id
				INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=q.object_id
				WHERE om.member_id IN (".implode(',', $restricted_member_ids).") AND om.is_optimization=0
					AND q.object_id <> '".$quota_id."'
				GROUP BY om.object_id HAVING COUNT(DISTINCT(om.member_id)) = ".count($restricted_member_ids)."
			";
			$row = DB::executeOne($sql);
		}
		
		return $row;
	}
	
	function view() {
		
		ajx_set_no_toolbar();
		$quota = Quotas::instance()->findById(get_id());
		if(!($quota instanceof Quota)) {
			flash_error(lang('message dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$quota->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		ajx_extra_data(array("title" => $quota->getObjectName(), 'icon' => $quota->getIconClass()."s"));
		tpl_assign('quota', $quota);
	}
	
	/**
	 * Resolve action to perform
	 *
	 * @param string $action
	 * @param array $attributes
	 * @return string $message
	 */
	private function resolveAction($action, $attributes){
				 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				$messages = array();
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$quota = Quotas::instance()->findById($id);
					if (isset($quota) && $quota->canDelete(logged_user())){
						try{
							DB::beginWork();
							$quota->trash();
							DB::commit();
							ApplicationLogs::createLog($quota, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$messages[] = $e->getMessage();
							$err++;
						}
					} else {
						$err++;
						$messages[] = lang('cannot delete executed expense', $quota instanceof Quota ? clean($quota->getName()) : '');
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) .("\n - ".implode("\n - ",$messages)). ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			case "archive":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$quota = Quotas::instance()->findById($id);
					if (isset($quota) && $quota->canEdit(logged_user())){
						try{
							DB::beginWork();
							$quota->archive();
							DB::commit();
							ApplicationLogs::createLog($quota, ApplicationLogs::ACTION_ARCHIVE);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error archive objects", $err) . "<br />" . ($succ > 0 ? lang("success archive objects", $succ) : "");
				} else {
					$resultMessage = lang("success archive objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;		
		} // switch
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
	}
	
	
	
	
	function get_quota_list_config() {
		ajx_current("empty");
		
		ob_start();
		tpl_display(get_template_path('quotas_list_grid_config/columns', 'quota', 'advanced_expenses'));
		tpl_display(get_template_path('quotas_list_grid_config/toolbar', 'quota', 'advanced_expenses'));
		
		$html = ob_get_clean();
		
		ajx_extra_data(array('html' => $html));
	}
	
}
