<?php
	class Quota extends BaseQuota {
		
		protected $searchable_columns = array('object_id', 'name', 'amount', 'spent_amount');
		
		function getAvailableAmount() {
			$available = $this->getAmount() - $this->getSpentAmount();
			if ($available < 0) $available = 0;
			
			return $available;
		}
		
		
		function getArrayInfo() {
			$info = array();
			
			$columns = $this->getColumns();
			foreach ($columns as $col) {
				$info[$col] = $this->getColumnValue($col);
				if ($info[$col] instanceof DateTimeValue) $info[$col] = format_date($info[$col], null, 0);
			}
			
			$general_info = $this->getObject()->getArrayInfo();
			$info = array_merge($info, $general_info);
			
			$c = Currencies::instance()->getCurrency($this->getCurrencyId());
			$symbol = $c instanceof Currency ? $c->getSymbol() : '$';
			
			$info['amount'] = format_money_amount($this->getAmount(), $symbol);
			$info['spent_amount'] = format_money_amount($this->getSpentAmount(), $symbol);
			$info['available_amount'] = format_money_amount($this->getAvailableAmount(), $symbol);
				
			return $info;
		}
		
		
		function recalculateSpentAmount() {
			
			$payments = Payments::instance()->findAll(array("conditions" => array("quota_id=? AND trashed_by_id=0", $this->getId())));
			$total_amount = 0;
			foreach ($payments as $p) {
				$total_amount += $p->getAmount();
			}
			
			$this->setSpentAmount($total_amount);
			$this->save();
		}
		
		
		
		static function canAdd(Contact $user, $context, &$notAllowedMember = ''){
			return can_add($user, $context, Quotas::instance()->getObjectTypeId(), $notAllowedMember);
		}
		
		/**
		 * Returns true if $user can access this message
		 *
		 * @param Contact $user
		 * @return boolean
		 */
		function canView(Contact $user) {
			return can_read($user, $this->getMembers(), $this->getObjectTypeId());
		} // canView
	
		/**
		 * Check if specific user can edit this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canEdit(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canEdit
	
	
		/**
		 * Check if specific user can delete this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canDelete(Contact $user) {
			return can_delete($user, $this->getMembers(), $this->getObjectTypeId());
		} // canDelete
	
		/**
		 * Check if specific user can comment this message
		 *
		 * @access public
		 * @param void
		 * @return boolean
		 */
		function canAddComment(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canAddComment
	}