<?php

function generate_permissions_string_for_account_users(MailAccount $account) {
	
	$permissions_decoded = array();
	$macs = MailAccountContacts::instance()->getByAccount($account);
	foreach ($macs as $mac) { /* @var $mac MailAccountContact */
		$user = $mac->getContact();
		if ($user instanceof Contact) {
			$pg_obj = new stdClass();
			$pg_obj->pg = $user->getPermissionGroupId();
			$pg_obj->o = MailContents::instance()->getObjectTypeId();
			$pg_obj->d = 1;
			$pg_obj->w = 1;
			$pg_obj->r = 1;
			$permissions_decoded[] = $pg_obj;
		}
	}
	return json_encode($permissions_decoded);
}



function check_and_create_mail_account_member(MailAccount $account) {
	
	$exists = MailAccountMembers::instance()->findOne(array('conditions' => "account_id=".$account->getId()));
	if (!$exists) {
		$dim = Dimensions::findByCode('mail_imap_folders');
		$ot = ObjectTypes::instance()->findById(MailAccountMembers::instance()->getObjectTypeId());
		
		// build data to create the member
		$member_data = array(
			'parent_member_id' => 0,
			'name' => $account->getName(),
			'dimension_id' => $dim->getId(),
			'object_type_id' => $ot->getId(),
		);
		$_POST['member'] = $member_data;
		$_POST['dim_obj'] = array(
			'account_id' => $account->getId()
		);
		
		// add full permissions to users that have permissions in account
		$_REQUEST['permissions'] = generate_permissions_string_for_account_users($account);
		
		// create the member
		$member_controller = new MemberController();
		$member_controller->add();
		
		unset($_POST['member']);
		unset($_POST['dim_obj']);
		
		return true;
	}
	return false;
}



function check_and_create_mail_imap_folder_member(MailAccount $account, $folder_name, $delimiter = "/") {
	
	$exists = MailImapFolderMembers::instance()->findOne(array('conditions' => "account_id=".$account->getId()." AND folder_name='$folder_name'"));
	if (!$exists) {
		$dim = Dimensions::findByCode('mail_imap_folders');
		$ot = ObjectTypes::instance()->findById(MailImapFolderMembers::instance()->getObjectTypeId());
		
		$parent_member_id = 0;
		
		$exploded = explode($delimiter, $folder_name);
		$leaf_folder_name = array_pop($exploded);
		
		if (str_starts_with($leaf_folder_name, "INBOX".$delimiter) || str_starts_with($leaf_folder_name, "Inbox".$delimiter)) {
			$leaf_folder_name = substr_utf($leaf_folder_name, strpos($leaf_folder_name, $delimiter) + 1);
		}
		
		// get parent member
		if ($exploded && count($exploded) > 0) {
			$parent_folder_name = implode($delimiter, $exploded);
			$parent_folder_obj = MailImapFolderMembers::instance()->findOne(array('conditions' => "account_id=".$account->getId()." AND folder_name='$parent_folder_name'"));
			
			if ($parent_folder_obj instanceof MailImapFolderMember) {
				$parent_member = Members::findOneByObjectId($parent_folder_obj->getId(), $dim->getId());
				if ($parent_member instanceof Member) {
					$parent_member_id = $parent_member->getId();
				}
			}
		}
		if ($parent_member_id == 0) {
			$account_obj = MailAccountMembers::instance()->findOne(array('conditions' => "account_id=".$account->getId()));
			$parent_member = Members::findOneByObjectId($account_obj->getId(), $dim->getId());
			if ($parent_member instanceof Member) {
				$parent_member_id = $parent_member->getId();
			}
		}
		
		$new_leaf_folder_name = $leaf_folder_name;
		$member_exists = Members::instance()->findOne(array(
			"conditions" => array("parent_member_id=? AND name=? AND dimension_id=? AND object_type_id=?", $parent_member_id, $leaf_folder_name, $dim->getId(), $ot->getId())
		));
		$suffix = 0;
		while ($member_exists instanceof Member) {
			$suffix++;
			$new_leaf_folder_name = $leaf_folder_name . " $suffix";
			$member_exists = Members::instance()->findOne(array(
				"conditions" => array("parent_member_id=? AND name=? AND dimension_id=? AND object_type_id=?", $parent_member_id, $new_leaf_folder_name, $dim->getId(), $ot->getId())
			));
		}
		
		// build data to create the member
		$member_data = array(
			'parent_member_id' => $parent_member_id,
			'name' => $new_leaf_folder_name,
			'dimension_id' => $dim->getId(),
			'object_type_id' => $ot->getId(),
		);
		$_POST['member'] = $member_data;
		$_POST['dim_obj'] = array(
			'account_id' => $account->getId(),
			'folder_name' => $folder_name,
		);
		
		// create the member, permissions are inherited from the parent member
		$member_controller = new MemberController();
		$member_controller->add();
		
		unset($_POST['member']);
		unset($_POST['dim_obj']);
		
		return true;
	}
	return false;
}



function classify_mails_in_imap_folder_member($account, $folder_name, $folder_member) {
	if (!$account instanceof MailAccount) return;
	if (!$folder_member instanceof Member) return;
	
	$unclassified_cond = " AND NOT EXISTS (
			SELECT om.object_id FROM ".TABLE_PREFIX."object_members om 
			WHERE om.member_id=".$folder_member->getId()." AND f.object_id=om.object_id
	)";
	
	$mail_ids_rows = DB::executeAll("SELECT f.object_id FROM ".TABLE_PREFIX."mail_content_imap_folders f
			WHERE f.account_id=".$account->getId()." AND f.folder='$folder_name' $unclassified_cond");
	
	$mail_ids = array_unique(array_filter(array_flat($mail_ids_rows)));
	
	if (count($mail_ids) > 0) {
		// add to object_members
		foreach ($mail_ids as $mail_id) {
			ObjectMembers::addObjectToMembers($mail_id, array($folder_member));
		}
		// remake sharing table
		$ids_str = implode($mail_ids);
		add_multilple_objects_to_sharing_table($ids_str, logged_user());
	}
	
}

function classify_single_mail_in_imap_folder_member($account, $folder_name, $folder_member, $mail) {
	if (!$account instanceof MailAccount) return;
	if (!$mail instanceof MailContent) return;
	if (!$folder_member instanceof Member) return;
	
	ObjectMembers::addObjectToMembers($mail->getId(), array($folder_member));
	$mail->addToSharingTable();
}



function check_and_create_mail_account_imap_folder_members(MailAccount $account) {
	$dim = Dimensions::findByCode('mail_imap_folders');
	
	// ensure account member is created
	$mem_account_created = check_and_create_mail_account_member($account);
	$any_folder_created = false;
	$delimiter = null;
	
	// foreach imap folder create the linked member
	if ($account->getIsImap()) {
		$mailboxes = MailAccountImapFolders::getMailAccountImapFolders($account->getId());
		foreach ($mailboxes as $box) {
			if ($box->getCheckFolder() && $box->getSpecialUse() == '' && strtoupper($box->getFolderName()) != 'INBOX') {
				$folder_name = escape_character($box->getFolderName());
				
				if (is_null($delimiter)) {
					$imap = $account->imapConnect();
					$mu = new MailUtilities();
					$can_detect_special_folders = null;
					$acc_folders = $mu->get_imap_account_mailboxes($account, $can_detect_special_folders);
					if (count($acc_folders) > 0) {
						$delimiter = $acc_folders[0]['delimiter'];
					}
				}
				// create the member
				$member_created = check_and_create_mail_imap_folder_member($account, $folder_name, $delimiter);
				
				if ($member_created) {
					$any_folder_created = true;
					
					// classify existing mails in new member
					$folder_obj = MailImapFolderMembers::instance()->findOne(array('conditions' => "account_id=".$account->getId()." AND folder_name='$folder_name'"));
					if ($folder_obj instanceof MailImapFolderMember) {
						$folder_member = Members::findOneByObjectId($folder_obj->getId(), $dim->getId());
						
						classify_mails_in_imap_folder_member($account, $folder_name, $folder_member);
					}
				}
			}
		}
	}
	$account_changed = $mem_account_created || $any_folder_created;
	
	return $account_changed;
}



