<?php

/**
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */

class AdvancedReportsController extends ApplicationController {

	/**
	 * Construct the ReportingController
	 *
	 * @access public
	 * @param void
	 * @return ReportingController
	 */
	function __construct()
	{
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('grouping');
		Env::useHelper('advanced_reports', 'advanced_reports');
	} // __construct

	
	function download_file() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		ajx_current("empty");
		
		if(array_var($_REQUEST, 'file_name')){
			$file_name = array_var($_REQUEST, 'file_name');
		}
		if(array_var($_REQUEST, 'file_type')){
			$file_type = array_var($_REQUEST, 'file_type');
		}
		
		$file_path = "tmp/".$file_name;
		$size = filesize($file_path);
		
		//user current date
		$now_date = DateTimeValueLib::now();
		$now_date->advance(logged_user()->getUserTimezoneValue());
		$now = $now_date->format('Y-m-d_H:i:s');
		
		download_file($file_path, $file_type, $file_name, $size, true);
		
		unlink($file_path);
		
		die();
	}



	// ---------------------------------------------------
	//  Tasks Reports
	// ---------------------------------------------------
	
	function tasks_report_p(){
		$report_data = array();
		
		$cp_ids = CustomProperties::getCustomPropertyIdsByObjectType(ProjectTasks::instance()->getObjectTypeId());
		tpl_assign('has_custom_properties', count($cp_ids) > 0);
		
		$users = allowed_users_in_context(ProjectTasks::instance()->getObjectTypeId(), active_context());
		
		tpl_assign('users', $users);
		tpl_assign('report_data', $report_data);
	}
	
	
	function tasks_report($report_data = null, $task = null, $csv = null){
		
		$report_data = array_var($_POST, 'report', array());
		$_SESSION['tasks_report_data'] = $report_data;
		
		$extra_conditions = "";
		
		$columns = array_var($report_data, 'columns');
		if (!is_array($columns)) $columns = array_var($_POST, 'columns', array());
									
		asort($columns); //sort the array by column order
		foreach($columns as $column => $order){
			if ($order > 0) {
				$newColumn = new ReportColumn();
				//$newColumn->setReportId($newReport->getId());
				if(is_numeric($column)){
					$newColumn->setCustomPropertyId($column);
				}else{
					$newColumn->setFieldName($column);
				}				
			}
		}
	
		$now = DateTimeValueLib::now();
		$now->advance(logged_user()->getUserTimezoneValue(), true);
		switch (array_var($report_data, 'due_date_type')){
			case -1: //None
				$st = null;
				$et = null;
				break;
			case 1: //Today
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),$now->getDay(),$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),$now->getDay(),$now->getYear());break;
			case 2: //This week
				$monday = $now->getMondayOfWeek();
				$nextMonday = $now->getMondayOfWeek()->add('w',1)->add('d',-1);
				$st = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
				$et = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
			case 3: //Last week
				$monday = $now->getMondayOfWeek()->add('w',-1);
				$nextMonday = $now->getMondayOfWeek()->add('d',-1);
				$st = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
				$et = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
			case 4: //This month
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
			case 5: //Last month
				$now->add('M',-1);
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
			case 6: //Date interval
				$st = getDateValue(array_var($report_data, 'due_start_value'));
				if ($st instanceof DateTimeValue) {
					$st = $st->beginningOfDay();
				} else {
					$st = null;
				}
				
				$et = getDateValue(array_var($report_data, 'due_end_value'));
				if ($et instanceof DateTimeValue) {
					$et = $et->endOfDay();
				} else {
					$et = null;
				}
				break;
		}
		if ($et != null && $st != null) {
			$extra_conditions .= " AND due_date <= '".$et->toMySQL()."' AND due_date >= '".$st->toMySQL()."' ";
		}
		
		
		switch (array_var($report_data, 'start_date_type')){
			case -1: //None
				$st = null;
				$et = null;
				break;
			case 1: //Today
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),$now->getDay(),$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),$now->getDay(),$now->getYear());break;
			case 2: //This week
				$monday = $now->getMondayOfWeek();
				$nextMonday = $now->getMondayOfWeek()->add('w',1)->add('d',-1);
				$st = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
				$et = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
			case 3: //Last week
				$monday = $now->getMondayOfWeek()->add('w',-1);
				$nextMonday = $now->getMondayOfWeek()->add('d',-1);
				$st = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
				$et = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
			case 4: //This month
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
			case 5: //Last month
				$now->add('M',-1);
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
			case 6: //Date interval
				$st = getDateValue(array_var($report_data, 'start_start_value'));
				if ($st instanceof DateTimeValue) {
					$st = $st->beginningOfDay();
				} else {
					$st = null;
				}
				
				$et = getDateValue(array_var($report_data, 'start_end_value'));
				if ($et instanceof DateTimeValue) {
					$et = $et->endOfDay();
				} else {
					$et = null;
				}
				break;
		}
		if ($et != null && $st != null) {
			$extra_conditions .= " AND start_date <= '".$et->toMySQL()."' AND start_date >= '".$st->toMySQL()."' ";
		}
		
		
		$user = Contacts::instance()->findById(array_var($report_data, 'user'));
		if ($user instanceof Contact) {
			$extra_conditions .= " AND assigned_to_contact_id = ".$user->getId();
		}
		
		$dateFormat = user_config_option('date_format');
		$date_format_tip = date_format_tip($dateFormat);
		
		$conditions = array_var($_POST, 'conditions', array());
		foreach ($conditions as $cond) {
			if ($cond['deleted'] > 0) continue;
			if (array_var($cond, 'custom_property_id') > 0) {
				if (!in_array($cond['condition'], array('like', 'not like', '=', '<=', '>=', '<', '>', '<>', '%'))) continue;
				
				$cp = CustomProperties::getCustomProperty($cond['custom_property_id']);
				if (!$cp instanceof CustomProperty) continue;
				
				$current_condition = ' AND o.id IN ( SELECT object_id as id FROM '.TABLE_PREFIX.'custom_property_values cpv WHERE cpv.custom_property_id = '.$cond['custom_property_id'];
				
				if($cond['condition'] == 'like' || $cond['condition'] == 'not like'){
					$value = '%'.$cond['value'].'%';
				}
				if ($cp->getType() == 'date') {
					if ($value == $date_format_tip) continue;
					$dtValue = DateTimeValueLib::dateFromFormatAndString($dateFormat, $value);
					$value = $dtValue->format('Y-m-d H:i:s');
				}
				if($cond['condition'] != '%'){
					if ($cp->getType() == 'numeric') {
						$current_condition .= ' AND cpv.value '.$cond['condition'].' '.DB::escape($value);
					}else if ($cp->getType() == 'boolean') {
						$current_condition .= ' AND cpv.value '.$cond['condition'].' '.$value;
						if (!$value) {
							$current_condition .= ') OR o.id NOT IN (SELECT object_id as id FROM '.TABLE_PREFIX.'custom_property_values cpv2 WHERE cpv2.object_id=o.id AND cpv2.value=1 AND cpv2.custom_property_id = '.$cp->getId();
						}
					}else{
						$current_condition .= ' AND cpv.value '.$cond['condition'].' '.DB::escape($value);
					}
				}else{
					$current_condition .= ' AND cpv.value like '.DB::escape("%$value");
				}
				$current_condition .= ')';
				$extra_conditions .= $current_condition;
				
			}
		}
		
		switch (array_var($report_data, 'status')){
			case 1: $extra_conditions .= " AND completed_by_id=0"; break;
			case 2: $extra_conditions .= " AND completed_by_id>0"; break;
			case 3: break;
		}
		
		$tasks = ProjectTasks::instance()->listing(array(
			"extra_conditions" => $extra_conditions,
			"count_results" => false,
			"fire_additional_data_hook" => false,
		))->objects;
		
		$group_by = array();
		for ($i = 1; $i <= 3; $i++){
			$gb = array_var($report_data, 'group_by_' . $i);
			if ($gb != '0' && !in_array($gb, $group_by)) $group_by[] = $gb;
		}
		
		// group all tasks
		$gb_criterias = array();
		foreach ($group_by as $text) {
			if (in_array($text, array('assigned_to_contact_id', 'rel_object_id', 'priority'))) $gb_criterias[] = array('type' => 'column', 'value' => $text);
			else if (in_array($text, array('milestone_id'))) $gb_criterias[] = array('type' => 'assoc_obj', 'fk' => 'rel_object_id', 'value' => $text);
			else if (str_starts_with($text, 'dim_')) $gb_criterias[] = array('type' => 'dimension', 'value' => str_replace_first('dim_', '', $text));
		}
		$grouped_tasks = groupObjects($gb_criterias, $tasks);
		
		if (in_array('assigned_to_contact_id', $group_by) && !$user instanceof Contact) $user = new Contact();
		
		// group subtasks
		foreach ($grouped_tasks['grouped_objects'] as $gid => &$objects) {
			$to_remove = array();
			foreach ($objects as $t) {
				if ($t->getParentId() > 0) {
					foreach ($objects as &$parent) {
						if ($parent->getId() != $t->getId() && $parent->getId() == $t->getParentId()) {
							if (!isset($parent->tmp_subtasks)) $parent->tmp_subtasks = array();
							$parent->tmp_subtasks[] = $t;
							$to_remove[] = $t->getId();
						}
					}
				}
			}
			foreach ($to_remove as $remove_id) {
				foreach ($objects as $k => $t) {
					if ($t->getId() == $remove_id) {
						unset($objects[$k]);
						break;
					}
				}
			}
		}
		
		tpl_assign('columns', $columns);
		tpl_assign('grouped_tasks', $grouped_tasks);
		if (array_var($report_data, 'date_type') == 6) {
			$st->advance(logged_user()->getUserTimezoneValue(), true);
			$et->advance(logged_user()->getUserTimezoneValue(), true);
		}
		tpl_assign('start_time', $st);
		tpl_assign('end_time', $et);
		tpl_assign('user', $user);
		tpl_assign('title', lang('tasks report'));
		tpl_assign('post', $report_data);
		tpl_assign('allow_export', false);
		
		tpl_assign('template_name', 'tasks_report');
		$this->setTemplate('report_wrapper');
	}

	
	
	function file_access_report_p() {
		
		// users
		$users = allowed_users_in_context(ProjectFiles::instance()->getObjectTypeId(), active_context());
		$user_ids = array();
		$companies_ids = array();
		foreach ($users as $user) {
			$user_ids[] = $user->getId();
			if ($user->getCompanyId() > 0) {
				$companies_ids[$user->getCompanyId()] = $user->getCompanyId();
			}
		}

		// user groups
		$user_groups = array();
		if (count($user_ids) > 0) {
			$sql = "SELECT DISTINCT(pg.id) FROM ".TABLE_PREFIX."permission_groups pg
				INNER JOIN ".TABLE_PREFIX."contact_permission_groups cpg ON pg.id=cpg.permission_group_id
				where pg.type='user_groups' and cpg.contact_id in (".implode(',', $user_ids).");";
			$group_rows = DB::executeAll($sql);
			
			if (count($group_rows) > 0) {
				$group_ids = array_flat($group_rows);
				$user_groups = PermissionGroups::instance()->findAll(array('conditions' => 'id IN ('.implode(',', $group_ids).')'));
			}
		}
		
		// companies
		if (count($companies_ids) > 0) {
			$companies = Contacts::instance()->findAll(array('conditions' => 'is_company=1 AND object_id IN ('.implode(',', $companies_ids).')'));
		}
		
		
		tpl_assign('users', $users);
		tpl_assign('user_groups', $user_groups);
		tpl_assign('companies', $companies);
	}
	
	
	function file_access_report() {
		
		Env::useHelper('grouping');
		
		$report_data = array_var($_POST, 'report', array());
		$_SESSION['file_access_report_data'] = $report_data;
		
		$report_result = $this->execute_file_access_report($report_data);
		
		tpl_assign('post', $_POST);
		tpl_assign('users', array_var($report_result, 'users'));
		tpl_assign('grouped_logs', array_var($report_result, 'grouped_logs'));
		tpl_assign('start_time', array_var($report_result, 'start_time'));
		tpl_assign('end_time', array_var($report_result, 'end_time'));
		
		tpl_assign('title', lang('file access report'));
		tpl_assign('allow_export_pdf', false);
		
		tpl_assign('template_name', 'file_access_report');
		$this->setTemplate('report_wrapper');
		
	}
	
	
	function file_access_report_print() {
		Env::useHelper('file_access_report_csv', 'advanced_reports');
		
		$report_data = array_var($_POST, 'report');
		
		$report_result = $this->execute_file_access_report($report_data);
		
		// csv export
		if (isset($_POST['exportCSV']) && $_POST['exportCSV'] != "") {

			$filename = str_replace(" ", "-", str_replace("/", "-", lang('file access report').".csv"));
			
			generate_file_access_report_csv_file(array_var($report_result, 'grouped_logs'), array_var($report_result, 'users'), $filename);
			
			ajx_extra_data(array('filename' => $filename));
		}
		
		ajx_current("empty");
	}
	
	
	
	private function execute_file_access_report($report_data) {
		
		// date filter
		$now = DateTimeValueLib::now();
		$now->advance(logged_user()->getUserTimezoneValue(), true);
		switch (array_var($report_data, 'date_type')){
			case -1: //None
				$st = null;
				$et = null;
				break;
			case 1: //Today
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),$now->getDay(),$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),$now->getDay(),$now->getYear());break;
			case 2: //This week
				$monday = $now->getMondayOfWeek();
				$nextMonday = $now->getMondayOfWeek()->add('w',1)->add('d',-1);
				$st = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
				$et = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
			case 3: //Last week
				$monday = $now->getMondayOfWeek()->add('w',-1);
				$nextMonday = $now->getMondayOfWeek()->add('d',-1);
				$st = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
				$et = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
			case 4: //This month
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
			case 5: //Last month
				$now->add('M',-1);
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
			case 6: //Date interval
				$st = getDateValue(array_var($report_data, 'date_start_value'));
				if ($st instanceof DateTimeValue) {
					$st = $st->beginningOfDay();
				} else {
					$st = null;
				}
				
				$et = getDateValue(array_var($report_data, 'date_end_value'));
				if ($et instanceof DateTimeValue) {
					$et = $et->endOfDay();
				} else {
					$et = null;
				}
				break;
			default: //None
				$st = null;
				$et = null;
				break;
		}
		$date_conditions = "";
		if ($et != null && $st != null) {
			$date_conditions = " AND created_on <= '".$et->toMySQL()."' AND created_on >= '".$st->toMySQL()."' ";
		}
		
		// decode user filter
		$user_ids = array();
		switch(array_var($report_data, 'filter_by_users')) {
			case 'users':
				if ($report_data['user'] == 0) {
					$users = allowed_users_in_context(ProjectFiles::instance()->getObjectTypeId(), active_context());
					foreach ($users as $user) $user_ids[] = $user->getId();
				} else {
					$user_ids = array($report_data['user']);
					$users = array(Contacts::instance()->findById($report_data['user']));
				}
				break;
			case 'groups':
				$extra_cond = "";
				if ($report_data['group'] > 0) {
					$extra_cond = " AND id IN (SELECT DISTINCT contact_id FROM ".TABLE_PREFIX."contact_permission_groups WHERE permission_group_id=".$report_data['group'].")";
				}
				$users = allowed_users_in_context(ProjectFiles::instance()->getObjectTypeId(), active_context(), ACCESS_LEVEL_READ, $extra_cond);
				foreach ($users as $user) $user_ids[] = $user->getId();
				break;
			case 'companies':
				$extra_cond = "";
				if ($report_data['company'] > 0) {
					$extra_cond = " AND company_id=".$report_data['company'];
				}
				$users = allowed_users_in_context(ProjectFiles::instance()->getObjectTypeId(), active_context(), ACCESS_LEVEL_READ, $extra_cond);
				foreach ($users as $user) $user_ids[] = $user->getId();
				break;
			default: $user_ids = array(); $users = array(); break;
		}
		
		if (count($user_ids) == 0) $user_ids[] = 0;
		
		// object filter (use listing function to filter by context)
		$document_ids = ProjectFiles::instance()->listing(array(
			'raw_data' => true,
			'select_columns' => array('id'),
			'include_deleted' => true,
		))->objects;
		$document_ids = array_flat($document_ids);
		if (count($document_ids) == 0) $document_ids[] = 0;
		
		// action filter
		$action_cond = "";
		if (array_var($report_data, 'action') == ApplicationReadLogs::ACTION_DOWNLOAD) $action_cond = " AND action='download'";
		else if (array_var($report_data, 'action') == ApplicationReadLogs::ACTION_READ) $action_cond = " AND action='read'";
		
		$user_object_conditions = "taken_by_id IN (".implode(',', $user_ids).") AND rel_object_id IN (".implode(',',$document_ids).")";
		
		// get logs
		$logs = ApplicationReadLogs::instance()->findAll(array('conditions' => "$user_object_conditions $action_cond $date_conditions"));
		
		// group logs
		$group_by = array();
		for ($i = 1; $i <= 3; $i++){
			$gb = array_var($report_data, 'group_by_' . $i);
			if ($gb != '0' && !in_array($gb, $group_by)) $group_by[] = $gb;
		}
		$gb_criterias = array();
		foreach ($group_by as $text) {
			if (in_array($text, array('created_by_id','11rel_object_id'))) $gb_criterias[] = array('type' => 'column', 'value' => $text);
			else if (in_array($text, array('rel_object_id'))) $gb_criterias[] = array('type' => 'assoc_obj', 'fk' => 'rel_object_id', 'value' => $text);
			else if (str_starts_with($text, 'dim_')) $gb_criterias[] = array('type' => 'dimension', 'value' => str_replace_first('dim_', '', $text));
		}
		$grouped_logs = groupObjects($gb_criterias, $logs);
		
		$users_map = array();
		foreach ($users as $user) $users_map[$user->getId()] = $user;
		
		return array(
			'users' => $users_map,
			'grouped_logs' => $grouped_logs,
			'start_time' => $st,
			'end_time' => $et,
		);
	}
	
	
	
	function get_default_custom_reports() {
		ajx_current("empty");
		$def_report_ids = array();
		
		$reports = Reports::instance()->findAll(array("conditions" => "is_default=1"));
		foreach ($reports as $rep) {
			$def_report_ids[$rep->getReportObjectTypeId()] = $rep->getId();
		}
		
		ajx_extra_data(array("reports" => $def_report_ids));
	}
	
	
	
	
	function get_group_by_parameters_html() {
		
		ajx_current("empty");
		$genid = array_var($_REQUEST, 'object_type', gen_id());
		$report = new Report();
		$report->setReportObjectTypeId(array_var($_REQUEST, 'object_type'));
		$ret=null;
		ob_start();
		Hook::fire('add_more_custom_report_parameters', array('report' => $report, 'genid' => $genid), $ret);
		
		$html = ob_get_clean();
		
		ajx_extra_data(array('html' => $html));
	}
	
	
	
	
	
	
	function render_report_gb_member_selector() {
	
		$dim_id = array_var($_REQUEST, 'dim_id');
		$hf_name = array_var($_REQUEST, 'hf_name');
		$dim_assoc_id = array_var($_REQUEST, 'dim_assoc_id');
		$hide_label = array_var($_REQUEST, 'hide_label');
		$object_type_id = array_var($_REQUEST, 'object_type_id');
		$selected_member_ids = explode(',', array_var($_REQUEST, 'selected_member_ids', ""));
	
		if (is_numeric($dim_id)) {
			$dim = Dimensions::instance()->findById($dim_id);
			tpl_assign('dim', $dim);
		} else {
			$dim_id = 0;
		};
	
		tpl_assign('hf_name', $hf_name);
		tpl_assign('dim_assoc_id', $dim_assoc_id);
		tpl_assign('hide_label', $hide_label);
		tpl_assign('object_type_id', $object_type_id);
	
		foreach ($selected_member_ids as $k => &$v) {
			if (!is_numeric($v)) unset($selected_member_ids[$k]);
		}
		if (count($selected_member_ids) > 0) {
			$sql = "SELECT m.id FROM ".TABLE_PREFIX."members m WHERE m.id IN (".implode(',', $selected_member_ids).") AND m.dimension_id=$dim_id";
			$clean_sel_member_ids = array_flat(DB::executeAll($sql));
			tpl_assign('selected_member_ids', $clean_sel_member_ids);
		} else {
			tpl_assign('selected_member_ids', array());
		}
	}
	
	
	function build_group_conditions_html_desc() {
		ajx_current("empty");
		Env::useHelper('reporting');
		
		$html = "";
		$genid = array_var($_REQUEST, 'genid');
		$index = array_var($_REQUEST, 'index');
		$ot_name = array_var($_REQUEST, 'ot_name');
		$conditions_json = array_var($_REQUEST, 'conditions');
		if ($conditions_json != "") {
			$conditions = json_decode($conditions_json, true);
		}
		$disabled_params = array_var($_REQUEST, 'disabled_params');
		
		$ot = ObjectTypes::findByName($ot_name);
		if ($ot instanceof ObjectType) {
			
			$dummy_report = new Report();
			$dummy_report->setReportObjectTypeId($ot->getId());
			$condition_objects = array();
			
			foreach ($conditions as $condition) {
				$cond = new ReportCondition();
				$cond->setCondition($condition['condition']);
				$cond->setValue($condition['value']);
				if (is_numeric($condition['field_name'])) {
					$cond->setCustomPropertyId($condition['field_name']);
				} else {
					$cond->setFieldName($condition['field_name']);
				}
				$condition_objects[] = $cond;
			}
			
			$html = build_report_conditions_html_main($dummy_report, array(), $condition_objects, $disabled_params);
		}
		
		ajx_extra_data(array('genid' => $genid, 'index' => $index, 'html' => $html));
	}
        
        
        
        /**
     * 
     * @return type
     */
    public function add_custom_category() {
        if (logged_user()->isGuest()) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        }
        $isModal = array_var($_POST, 'modal');	
	if(!$isModal) {        
            $this->setTemplate('categories/add_custom_category');
            $action = get_url('advanced_reports', 'add_custom_category');
            $on_submit = "og.submit_modal_form('submit-edit-report-category'); return false;";

            $object_id = array_var($_GET, 'object_id');
            if($object_id){
                $category = ReportCategories::instance()->findById($object_id);
                tpl_assign('objectId',$object_id);
                tpl_assign('name', Localization::instance()->lang_exists($category->getObjectName()) ? lang($category->getObjectName()) : $category->getObjectName());
                tpl_assign('description', Localization::instance()->lang_exists($category->getDescription()) ? lang($category->getDescription()) : $category->getDescription());
            }
            
            
            tpl_assign('submit',$on_submit);
            tpl_assign('action',$action);
        }else{
            $this->save_custom_category();
            ajx_current("back");
            evt_add("reload current panel");    
        }
    }
    
    /**
     * Save or edit a category
     */
    public function save_custom_category(){
        $object_id = array_var($_POST, 'object_id');
        $category_name = array_var($_POST, 'name');
        $category_description = array_var($_POST, 'description');
        
         try {
            DB::beginWork();
            
            $this->setTemplate('categories/add_custom_category');
            if($object_id){
                $report_category = ReportCategories::getReportCategory($object_id);
            }else{
                $report_category = new ReportCategory();
            }
            $report_category->setObjectName($category_name);
            $report_category->setDescription($category_description);
            $report_category->setIsDefault(1);
            $report_category->save();
            
            DB::commit();
            
            if($object_id){
                flash_success(lang('success_edited_category'));
            }else{
                flash_success(lang('success_add_category'));
            }
            
        } catch (Exception $exc) {
            DB::rollback();
            Logger::log_r($exc);
            flash_error(lang('error'));
        }
    }
    
    /*+
     * Delete a category and remove the referencs on reports
     */
    function delete_custom_report_category() {
        if (logged_user()->isGuest()) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        }
        $report_category_id = array_var($_GET, 'id');
        $report_category = ReportCategories::getReportCategory($report_category_id);

        if (!$report_category->canDelete(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        } // if

        try {
            DB::beginWork();
            
            $list_of_reports = ReportCategories::getAllReportsForCategoryWithoutListing($report_category_id);
            if (!empty($list_of_reports)) {
                foreach ($list_of_reports as $report) {
                    $report->setColumnValue('report_category_id',0);
                    $report->save();
                }
            }
            $report_category->delete();
            
            DB::commit();
            ajx_current("reload");
        } catch (Exception $e) {
            DB::rollback();
            flash_error($e->getMessage());
            ajx_current("empty");
        } // try
    }
        
}
























