<?php
/**
 * Permission Reports controller
 *
 * @version 1.0
 * @author Alex Zonis <alex.zonis@fengoffice.com>
 */
class PermissionsReportsController extends ApplicationController {

	var $plugin_name = "advanced_reports";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('grouping');
		Env::useHelper('format');
		Env::useHelper('permissions_report_functions', 'advanced_reports');

	}
	
	/****
	 *
	 * PERMISSIONS REPORT
	 * 
	 ****/
	function permissions_report_p(){

		$report_data = array_var($_SESSION, 'permissions_report_data', array());
		$_SESSION['permissions_report_data'] = $report_data;

		$report_title = Reports::instance()->findOne(array("conditions" => "`code` = 'permissions_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);
		tpl_assign('title', $title);
	}


	function permissions_report($report_data = null, $return_data = null) {
		ini_set('memory_limit','1G');
		if (!$report_data) {
			$report_data = array_var($_POST, 'report', array());
		}
		
		$_SESSION['permissions_report_data'] = $report_data;
		
        // Get report data
		$data = get_main_permissions_report_data($report_data);

		// Get the report title
		$report_title = Reports::instance()->findOne(array("conditions" => "`code` = 'permissions_report'"))->getObjectName();
		$title = startsWith(strtolower(lang($report_title)), 'missing lang') ? $report_title : lang($report_title);

		if ($return_data) {
            return $data;  
        }
		
		tpl_assign('data', $data);
		tpl_assign('title', $title);
		tpl_assign('report_data', $report_data);
		tpl_assign('post', $report_data);
		tpl_assign('allow_export', true);
		tpl_assign('allow_export_csv', false);
		tpl_assign('genid', gen_id());

		tpl_assign('hide_print_report_header', 'true');

		//tpl_assign('print_action_url', get_url('permissions_reports', 'permissions_report_print'));

		tpl_assign('pdf_export_function', 'og.advanced_reports.export_permissions_report_pdf');
		//tpl_assign('excel_export_action', 'og.advanced_reports.export_permissions_report_excel');

		tpl_assign('report_template_path', get_template_path('permissions_report', 'permissions_reports', 'advanced_reports'));

		$this->setTemplate(get_template_path('report_wrapper', 'advanced_reports', 'advanced_reports'));
		ajx_set_no_toolbar();
	}	

	function permissions_report_print() {
		ajx_current("empty");
		
		$export_pdf = array_var($_POST, 'exportPDF');
		$export_csv = array_var($_POST, 'exportCSV');
		
		if ($export_csv) {
			$report_result = $this->permissions_report($_POST, true);
			$filename = str_replace(" ", "-", str_replace("/", "-", lang('permissions report').".csv"));
			$rows_csv = array(); //build_permissions_report_csv($report_result, $_POST);
			file_put_contents(ROOT."/tmp/$filename", $rows_csv);
		} else if($export_pdf) {
			ob_start();
			$this->setTemplate('permissions_report');
			echo stylesheet_tag('website.css');
			echo stylesheet_tag(ROOT_URL."/plugins/advanced_reports/public/assets/css/advanced_reports.css" );
			
			$this->permissions_report($_POST);
			tpl_display(get_template_path('permissions_report', 'permissions_reports', 'advanced_reports'));
			$html = ob_get_clean();

			$html_filepath = ROOT."/tmp/".gen_id().".html";
			
			file_put_contents($html_filepath, $html);
			
			// convert html to pdf
			$orientation = array_var($_REQUEST, 'pdfPageLayout') == 'L' ? 'Landscape' : 'Portrait';
			$page_size = array_var($_REQUEST, 'pdfPageSize');
			$zoom = '--zoom 0.8';
			// Set config options for pdf layout and size
			set_user_config_option('pdf_page_layout', $orientation);
			set_user_config_option('pdf_page_size', $page_size);
			$pdf_data = convert_to_pdf($html, $orientation, "permissions_report", $page_size, $zoom);
			$filename = $pdf_data['name'];
			
		}
		
		ajx_extra_data(array('filename' => $filename));
	}

	/*
	
	COMMENTED TEMPORARY BECAUSE THE THE EXCEL EXPORT IS NOT ENABLED YET AND THE FUNCTION build_permissions_report_excel DOES NOT EXISTS
	
	function permissions_report_excel() {
		ajx_current("empty");
		
		$report_result = $this->permissions_report($_POST, true);
	
		$filename = str_replace(" ", "-", str_replace("/", "-", lang('permissions report').".xlsx"));
		
		$excel_data = build_permissions_report_excel($report_result, $_POST);
		
		$sheets = array(
			array('grouped_data' => $excel_data, 'options' => array('title' => lang('permissions report'), 'fixed_cells_width' => 30), 'headers' => array(), 'data' => array()),
		);
		
		// generate the excel file
		$excel_export_controller = new ExcelExportController();
		$filepath = $excel_export_controller->generate_excel_file($sheets, array('filename' => $filename));
		
		ajx_extra_data(array('filename' => $filename));
	}
	*/

}