<?php 

	$ot = ObjectTypes::instance()->findById($report->getReportObjectTypeId());
	$options = array(option_tag(lang('none'), ''));
	
	if (in_array($ot->getType(), array('dimension_object', 'dimension_group'))) {
		$options[] = option_tag(lang('located under'), 'pm_1');
		
		if ($ot->getType() == 'dimension_object') {
			$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType($ot->getId());
		} else {
			$custom_properties = MemberCustomProperties::getAllMemberCustomPropertiesByObjectType($ot->getId());
		}
		
		$enabled_dimensions = config_option('enabled_dimensions');
		
		$dot = DimensionObjectTypes::instance()->findOne(array('conditions' => "object_type_id=".$ot->getId()));
		$associations = DimensionMemberAssociations::getAllAssociatationsForObjectType($dot->getDimensionId(), $ot->getId());
		$feng_persons_dim = Dimensions::findByCode('feng_persons');
		
		foreach ($associations as $a) {/* @var $a DimensionMemberAssociation */
			if ($a->getAssociatedDimensionMemberAssociationId() == $feng_persons_dim->getId()) continue;
			if (!in_array($a->getAssociatedDimensionMemberAssociationId(), $enabled_dimensions)) continue;
			
			$custom_assoc_name = DimensionAssociationsConfigs::getConfigValue($a->getId(), 'custom_association_name');
			if ($custom_assoc_name) {
				$assoc_name = $custom_assoc_name;
			} else {
				$assoc_ot = ObjectTypes::instance()->findById($a->getAssociatedObjectType());
				$assoc_name = lang($assoc_ot->getName());
			}
			
			$options[] = option_tag($assoc_name, 'a_'.$a->getId());
		}
		
	} else {
		
		// group by dimension intersection
		$options[] = option_tag(lang('dimension intersection'), 'intersection');
		
		// group by classification - allow to group by any member type (separated) of every enabled dimension
		$dimension_ids = config_option('enabled_dimensions');
		if (count($dimension_ids) > 0) {
			
			$dimensions = Dimensions::instance()->findAll(array("conditions" => "id IN (".implode(',', $dimension_ids).")"));
			foreach ($dimensions as $dim) {
				
				$ot_ids = DimensionObjectTypes::getObjectTypeIdsByDimension($dim->getId());
				// count the member types different to folder to use the dimension name or the ot name
				$no_folder_ots_count = 0;
				if (count($ot_ids) > 0) {
					$ots = ObjectTypes::instance()->findAll(array("conditions" => "id IN (".implode(',',$ot_ids).")"));
					foreach ($ots as $obj_type) {
						if ($obj_type->getName() != 'folder' && $obj_type->getName() != 'project_folder' && $obj_type->getName() != 'customer_folder') {
							$no_folder_ots_count++;
						}
					}
				} else {
					$ots = array();
				}
				
				foreach ($ots as $objtype) {
					
					if ($objtype instanceof ObjectType && !in_array($objtype->getName(), array('customer_folder','project_folder'))) {
						$option_label = $objtype->getPluralObjectTypeName();
						if ($no_folder_ots_count == 1 && $objtype->getName() != 'folder') {
							$option_label = $dim->getName();
						}
						$options[] = option_tag($option_label, 'dim_'.$dim->getId().'_'.$objtype->getId());
					}
				}
			}
		}
		
		// group by fixed properties
		eval('$managerInstance = ' . $ot->getHandlerClass() . "::instance();");
		if ($managerInstance) {
			$objectColumns = $managerInstance->getColumns();
			$objectColumns = array_diff($objectColumns, $managerInstance->getSystemColumns());
			$objectColumns = array_merge($objectColumns, $managerInstance->getExternalColumns());
			
			Hook::fire('validate_report_group_by_columns', array('ot' => $ot), $objectColumns);
			
			$objectColumns = array_unique($objectColumns);
			
			foreach ($objectColumns as $col) {
				if (str_starts_with($col, "repeat_")) continue;
				
				$lang = Localization::instance()->lang('field '.$ot->getHandlerClass().' '.$col);
				if (is_null($lang)) $lang = Localization::instance()->lang('field Objects '.$col);
				if (is_null($lang)) $lang = $col;
				
				$options[] = option_tag($lang, 'fp_'.$col);
			}
		}
		
		// group by custom properties
		$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType($ot->getId());
		
	}
	
	foreach ($custom_properties as $cp) {/* @var $cp CustomProperty */
		$options[] = option_tag($cp->getName(), 'cp_'.$cp->getId());
	}
	
	echo select_box('report[group_by][i]', $options, array('style' => 'display:none;', 'id' => $genid."_group_by_i"));
	
	$group_by = array_filter(explode(',', $report->getColumnValue('group_by')));
	$group_by_options = json_decode($report->getColumnValue('group_by_options'), true);
	if( isset($group_by_options)){
		foreach ($group_by_options as $k => &$opts) {
			if (is_string($opts)) {
				$opts = json_decode($opts, true);
			}
		}
	}	
	
	
	// group by intersection - dimension selector template
	if (isset($dimensions)) {
		$int_options = array();
		$int_options[] = option_tag(lang('none'), '');
		foreach ($dimensions as $d) {
			$int_options[] = option_tag($d->getName(), $d->getId());
		}
		echo select_box('report[gb_template][i][j][dim_id]', $int_options, array('style' => 'display:none;', 'id' => $genid."_group_by_i_int_j"));
	}
?>

<fieldset><legend><?php echo lang('group by') ?></legend>
	<div class="report-group-by"></div>
	<div id="<?php echo $genid?>_group_by_container">
		
	</div>
	<div id="<?php echo $genid?>_hide_details_container" style="display:none; margin-top:10px;">
		<label for="<?php echo $genid.'hide_group_details'?>"><?php echo lang('hide group details')?></label>
		<?php echo checkbox_field('report[hide_group_details]', $report->getColumnValue('hide_group_details'), array('id' => $genid.'hide_group_details', 'style'=>'margin: 3px 10px;'))?>
	</div>
	
	<div id="<?php echo $genid?>_hide_total_counts_container" style="display:none; margin-top:10px;">
		<label for="<?php echo $genid.'hide_group_total_counts'?>"><?php echo lang('hide group total counts')?></label>
		<?php echo checkbox_field('report[hide_group_total_counts]', $report->getColumnValue('hide_group_total_counts'), array('id' => $genid.'hide_group_total_counts', 'style'=>'margin: 3px 10px;'))?>
	</div>
	
	<div id="<?php echo $genid?>_show_last_group_as_column_container" style="display:none; margin-top:10px;">
		<label for="<?php echo $genid.'show_last_group_as_column'?>"><?php echo lang('show last group as column')?></label>
		<?php echo checkbox_field('report[show_last_group_as_column]', $report->getColumnValue('show_last_group_as_column'), array('id' => $genid.'hide_group_details', 'style'=>'margin: 3px 10px;'))?>
	</div>
</fieldset>

<script>
var genid = '<?php echo $genid?>';

og.add_report_group_by_selector = function(genid, index, selected, options) {
	var selector = $("#"+genid+"_group_by_i").clone();
	$(selector).attr('id', $(selector).attr('id').replace("_group_by_i", "_group_by_"+index));
	$(selector).attr('name', $(selector).attr('name').replace("[i]", "["+index+"]"));
	$(selector).attr('onchange', "og.on_report_group_by_selector_change('"+genid+"',"+(index+1)+");");
	$(selector).show();
	if (selected) {
		$(selector).val(selected);
		$("#"+genid+"_hide_details_container").show();
		$("#"+genid+"_hide_total_counts_container").show();
		if (index > 0) {
			$("#"+genid+"_show_last_group_as_column_container").show();
		}
	}
	
	var sel_container_id = genid+"_sel_container_"+index;
	
	$("#"+genid+"_group_by_container").append('<div id="'+sel_container_id+'" class="group-by-selector-container"></div>');
	$("#"+genid+"_group_by_container").append('<div id="'+sel_container_id+'_options" class="group-by-options-container"></div>');
	
	$("#"+genid+"_group_by_container #"+sel_container_id).append("<span class='gb-index'>"+(index+1)+" </span>");
	$("#"+genid+"_group_by_container #"+sel_container_id).append($(selector));

	if (selected) {
		og.report_group_by_render_options(genid, index, selected, options);
	}
}


og.render_report_gb_member_selector = function(checkbox, base_id, gb_id, sel_mem_ids, gb_index, hf_name_suffix) {

	if ($(checkbox).attr('checked') == 'checked') {
		// load selector
		var splitted = gb_id.split('_');
		if (splitted.length < 3) return;
		
		var sel_mem_ids_csv = "";
		if (sel_mem_ids) {
			var sel_mem_ids_arr = Ext.util.JSON.decode(sel_mem_ids);
			sel_mem_ids_csv = sel_mem_ids_arr.join(',');
		}
		
		var dim_id = splitted[1];
		var ot_id = splitted[2];

		if (!dim_id) return;

		var hidden_field_name = 'report[gb_options]['+gb_index+']';
		if (hf_name_suffix) {
			hidden_field_name += hf_name_suffix;
		}
		
		og.openLink(og.getUrl('advanced_reports', 'render_report_gb_member_selector', {
			dim_id: dim_id,
			hf_name: hidden_field_name,
			hide_label: true,
			object_type_id: ot_id,
			gb_index: gb_index,
			selected_member_ids: sel_mem_ids_csv
		}), {
			preventPanelLoad: true,
			hideLoading: true,
			callback: function(success, data) {
				if (success) {
					var container_id = base_id + '_details';
					$("#" + container_id).html(data.current.data);
				}
			}
		});
	} else {
		var container_id = base_id + '_details';
		$("#" + container_id).html('');
	}
}


og.report_group_by_render_options = function(genid, index, selected, options) {

	var options_id = genid+"_options_"+index;
	var fp_options_id = genid+"_fp_options_"+index;
	var intersection_sel_id = genid+"_intersection_sel_"+index;
	var sel_container_id = genid+"_sel_container_"+index;

	$("#"+genid+"_group_by_container #"+sel_container_id+"_options").html('');
	
	if (selected && selected.indexOf("dim_") == 0) {

		$("#"+intersection_sel_id).remove();
		$("#"+fp_options_id).remove();
		
		var el = document.getElementById(options_id);
		if (!el) {

			if (!options) options = '';
			var onchange_fn = "og.render_report_gb_member_selector(this, '"+options_id+"', '"+selected+"', '"+options+"', '"+index+"');";

			var checked_html = options ? 'checked="checked"' : '';
			var options_html = '<span id="'+options_id+'" class="options">' + '<table><tr><td>'
				+ '<label for="'+options_id+'_chk">' + lang('group by specific members')
				+ '</label>&nbsp;<input type="checkbox" id="'+options_id+'_chk" onchange="'+onchange_fn+'" '+checked_html+'/>&nbsp;</span>'
				+ '</td><td>'
				+ '<span id="'+options_id+'_details" class="details"></span>'
				+ '</td></tr></table></span>';
			$("#"+genid+"_group_by_container #"+sel_container_id+"_options").append(options_html);

			if (options.length > 0) {
				$('#'+options_id+'_chk').change();
			}
		}
		
	} else if (selected == "intersection") {

		$("#"+options_id).remove();
		$("#"+fp_options_id).remove();

		$("#"+genid+"_group_by_container #"+sel_container_id+"_options").append('<span id="'+genid+'_intersection_sel_'+index+'"></span>');
		if (options) {
			var sel_intersection = Ext.util.JSON.decode(options);
			var int_count = 0;
			for (var k=0; k<sel_intersection.length; k++) {
				if (sel_intersection[k]) {
					int_count++;
					og.report_group_by_render_intersection_selector(genid, index, sel_intersection[k], k);
				}
			}
			og.report_group_by_render_intersection_selector(genid, index, '', int_count);
		} else {
			og.report_group_by_render_intersection_selector(genid, index);
		}
		
	} else if (selected && selected.indexOf("fp_") == 0) {

		$("#"+options_id).remove();
		$("#"+intersection_sel_id).remove();

		var columns_with_options = ['contact_id', 'assigned_to_contact_id'];
		var column = selected.replace("fp_", "");
		
		if (columns_with_options.indexOf(column) != -1) {

			var contact_ot_id = null;
			for (var otid in og.objectTypes) {
				if (og.objectTypes[otid].name == 'contact') contact_ot_id = otid;
			}

			var decoded_options = options ? Ext.util.JSON.decode(options) : {};
			var show_empty = decoded_options && decoded_options.show_empty_groups > 0;
			var empty_first = decoded_options && decoded_options.put_empty_groups_first > 0;

			var checked_html_1 = show_empty ? 'checked="checked"' : '';
			var checked_html_2 = empty_first ? 'checked="checked"' : '';
			
			var options_html = '<div id="'+fp_options_id+'" class="fp-options" style="display:none;">'
				+ '<div class="coInputHeader"><div class="coInputTitle">'+ lang('group by') + ' \"' 
				+ $("#"+genid+"_group_by_"+index+" [value='"+selected+"']").text() +'\"</div></div>'
				+ '<div class="coInputMainBlock">'

				+ '<div id="options" class="gb-options-block"><h5>'+ lang('options') +'</h5>'
				+ '<table id="options_table"><tr><td>'

				+ '<label for="modal_'+fp_options_id+'_show_empty_groups">' + lang('show empty groups') + '</label>'
				+ '</td><td>'
				+ '<input type="checkbox" id="modal_'+fp_options_id+'_show_empty_groups" '+ checked_html_1 +' onchange="og.report_group_by_on_show_empty_change(this)"/>&nbsp;</span>'

				+ '</td></tr>'
				+ '<tr id="row_put_empty_groups_first"><td>'
				
				+ '<label for="modal_'+fp_options_id+'_put_empty_groups_first">' + lang('put empty groups first') + '</label>'
				+ '</td><td>'
				+ '<input type="checkbox" id="modal_'+fp_options_id+'_put_empty_groups_first" '+ checked_html_2 +'/>&nbsp;</span>'
				
				+ '</td></tr></table>'
				+ '</div>'

				+ '<div id="conditions" class="gb-options-block"><h5>'+ lang('conditions') +'</h5><div id="'+genid+'_cond_'+index+'"></div>'
				+ '<input id="'+genid+'_cond_'+index+'report[report_object_type_id]" type="hidden" value="'+contact_ot_id+'" />'
				+ '<div class="add-cond">'
				+ '<a href="#" class="link-ico ico-add" onclick="og.report_group_by_add_group_condition(\''+genid+'\','+index+')">'+ lang('add condition') +'</a>'
				+ '</div></div>'
				
				+ '<div id="'+fp_options_id+'_buttons">'
				+ '<button class="yes submit blue">'+ lang('save changes') +'</button><button class="no submit blue">'+ lang('cancel') +'</button>'
				+ '</div>'
				
				+ '</div></div>';

			options_html += '<input type="hidden" id="'+fp_options_id+'_show_empty_groups" name="report[gb_options]['+index+'][show_empty_groups]" value="'+(show_empty ? "1":"0")+'"/>';
			options_html += '<input type="hidden" id="'+fp_options_id+'_put_empty_groups_first" name="report[gb_options]['+index+'][put_empty_groups_first]" value="'+(empty_first ? "1":"0")+'"/>';
			options_html += '<div style="display:none;" id="'+fp_options_id+'_conditions">';
			
			if (decoded_options && decoded_options.conditions) {
				if (typeof(decoded_options.conditions.length) == "undefined") { // if conditions is an object => build an array
					var conds = [];
					for (var cc in decoded_options.conditions) conds.push(decoded_options.conditions[cc]);
					decoded_options.conditions = conds;
				}
				options_html += '<input type="hidden" id="conditions_count_'+ index +'" value="'+ decoded_options.conditions.length +'" />';
				for (var i=0; i<decoded_options.conditions.length; i++) {
					var cond = decoded_options.conditions[i];
					for (input_name in cond) {
						var hf_name = "report[gb_options]["+ index +"][conditions]["+ i +"]["+ input_name +"]";
						var input_val = cond[input_name];
						options_html += '<input type="hidden" name="'+ hf_name +'" value="'+ input_val +'" />';
					}
				}
			}
			options_html += '</div>';

			options_html += '<span class="fp-options-link">';
			options_html += '<a href="#" onclick="og.report_group_by_show_fixed_prop_group_options(\''+fp_options_id+'\', \''+genid+'\', '+index+');">' + lang('grouping options') + '</a>';
			options_html += '</span>';

			options_html += '<div class="fp-options-desc" id="'+fp_options_id+'_selected_options"></div>';
			
			$("#"+genid+"_group_by_container #"+sel_container_id+"_options").append(options_html);

			og.report_group_by_show_group_selected_options(genid, index, decoded_options.conditions);
		}
		
	
	}
}

og.report_group_by_show_group_selected_options = function(genid, index, conditions) {
	if (!conditions) conditions = [];
	
	og.openLink(og.getUrl('advanced_reports', 'build_group_conditions_html_desc'), {
		post: {
			genid: genid,
			index: index,
			ot_name: 'contact',
			conditions: Ext.util.JSON.encode(conditions)
		},
		callback: function (success, data) {
			var base_id = genid + '_fp_options_' + index;
			var container_id = base_id + '_selected_options';

			var desc_html = '<ul>';
			if ($("#" + base_id + "_show_empty_groups").val() > 0) {
				desc_html += '<li>' + lang('show empty groups') + '</li>';
				if ($("#" + base_id + "_put_empty_groups_first").val() > 0) {
					desc_html += '<li>' + lang('put empty groups first') + '</li>';
				}
			}
			desc_html += data.html
			desc_html += '</ul>';
			
			$("#" + container_id).html(desc_html);
		}
	});
}

og.report_group_by_add_group_condition = function(genid, index) {
	og.addCondition(genid+'_cond_'+index, 0, 0, '', '', '', false, false, false, true);
}

og.report_group_by_on_show_empty_change = function(checkbox, quick) {
	var t = $(checkbox).closest('table');
	var r = $("#"+t.attr('id')+" #row_put_empty_groups_first");
	var fade_duration = quick ? 0 : 400;
	
	if ($(checkbox).attr('checked') == 'checked') $(r).fadeIn(fade_duration);
	else $(r).fadeOut(fade_duration);
}

og.report_group_by_show_fixed_prop_group_options = function(div_id, genid, index) {
	var modal_params = {
			'escClose': true,
			'overlayClose': true,
			'closeHTML': '<a id="'+div_id+'_close_link" class="modal-close" title="'+lang('close')+'"></a>',
			'onShow': function (dialog) {
				$("#"+div_id+"_close_link").addClass("modal-close-img");
				$("#"+div_id+"_buttons").css('text-align', 'right').css('margin', '10px 0');
				// init checkboxes
				if ($('#'+ div_id +'_show_empty_groups').val() == '1') {
					$('#modal_'+ div_id +'_show_empty_groups').attr("checked", "checked");
				} else {
					$('#modal_'+ div_id +'_show_empty_groups').removeAttr("checked");
				}
				if ($('#'+ div_id +'_put_empty_groups_first').val() == '1') {
					$('#modal_'+ div_id +'_put_empty_groups_first').attr("checked", "checked");
				} else {
					$('#modal_'+ div_id +'_put_empty_groups_first').removeAttr("checked");
				}
				// init group conditions
				og.report_group_by_init_group_conditions(genid, index);
				
				// init checkbox visibility
				og.report_group_by_on_show_empty_change($('#modal_'+ div_id +'_show_empty_groups'), true);
				
				// save button handler
				$("#"+div_id+"_buttons button.yes").css('margin-right', '10px').click(function(){
					$('#'+ div_id +'_show_empty_groups').val($('#modal_'+ div_id +'_show_empty_groups').attr("checked") == 'checked' ? "1" : "0");
					$('#'+ div_id +'_put_empty_groups_first').val($('#modal_'+ div_id +'_put_empty_groups_first').attr("checked") == 'checked' ? "1" : "0");
					
					og.report_group_by_save_group_conditions(genid, index);
					
					$('.modal-close').click();
				});
				// cancel button handler
				$("#"+div_id+"_buttons button.no").css('margin-right', '10px').click(function(){
					$('.modal-close').click();
				});
			}
		};
	var div = document.getElementById(div_id);
	if (div) {
		$.modal(div, modal_params);
	}
}

og.report_group_by_save_group_conditions = function(genid, index) {
	var condition_divs = $("#"+genid+"_cond_"+index+" [id^=Condition]");
	var inputs_to_copy = ["field_name", "field_type", "condition", "value"];
	var base_id = genid +"_cond_"+ index;
	var html = "";
	var cond_count = 0;
	var new_conditions = [];
	
	for (var i=0; i<condition_divs.length; i++) {
		var condition_div = condition_divs[i];
		var cond_idx = $(condition_div).attr('id').replace("Condition", "");

		var is_deleted = $("#"+ base_id +" [name='conditions["+ cond_idx +"][deleted]']").val();
		if (is_deleted == 0) {
			var new_condition = {};
			for (var j=0; j<inputs_to_copy.length; j++) {
				var sel_val = $("#"+ base_id +" [name='conditions["+ cond_idx +"]["+ inputs_to_copy[j] +"]']").val();
				var hf_name = "report[gb_options]["+ index +"][conditions]["+ cond_idx +"]["+ inputs_to_copy[j] +"]";
				
				html += '<input type="hidden" name="'+ hf_name +'" value="'+ sel_val +'" />';

				new_condition[inputs_to_copy[j]] = sel_val;
			}
			cond_count++;
			
			new_conditions.push(new_condition);
		}
	}
	html += '<input type="hidden" id="conditions_count_'+ index +'" value="'+ cond_count +'" />';

	// update hidden inputs
	$("#"+ genid +"_fp_options_"+ index + "_conditions").html(html);

	// update description
	og.report_group_by_show_group_selected_options(genid, index, new_conditions);
}

og.report_group_by_init_group_conditions = function(genid, index) {
	var cond_count = $("#conditions_count_" + index).val();
	if (!cond_count) return;

	var inputs_to_copy = ["field_name", "field_type", "condition", "value"];
	
	var cond_idx = 0;
	while (cond_idx < cond_count) {
		var fname = '';
		var cpid = 0;

		var field_name = $("[name='report[gb_options]["+ index +"][conditions]["+ cond_idx +"][field_name]']").val();
		var field_type = $("[name='report[gb_options]["+ index +"][conditions]["+ cond_idx +"][field_type]']").val();
		var condition = $("[name='report[gb_options]["+ index +"][conditions]["+ cond_idx +"][condition]']").val();
		var value = $("[name='report[gb_options]["+ index +"][conditions]["+ cond_idx +"][value]']").val();

		if (isNaN(field_name)) fname = field_name;
		else cpid = field_name;

		og.addCondition(genid+'_cond_'+index, (cond_idx+1), cpid, fname, condition, value, '', false, false, true);
		
		cond_idx++;
	}
}

og.report_group_by_render_intersection_selector = function(genid, index, selected_val, opt_index) {
	if (!opt_index) opt_index = 0;

	var exists = document.getElementById(genid+"_group_by_"+index+"_int_"+opt_index);
	if (!exists) {
		var selector = $("#"+genid+"_group_by_i_int_j").clone();
		$(selector).attr('id', $(selector).attr('id').replace("_group_by_i_int_j", "_group_by_"+index+"_int_"+opt_index));
		$(selector).attr('name', $(selector).attr('name').replace("[i]", "["+index+"]"));
		$(selector).attr('name', $(selector).attr('name').replace("[j]", "["+opt_index+"]"));
		$(selector).attr('name', $(selector).attr('name').replace("[gb_template]", "[gb_options]"));

		// to draw next selector
		var onchange_functions = "og.report_group_by_render_intersection_selector('"+genid+"',"+index+", null,"+(opt_index+1)+");";
		// to reload the options
		onchange_functions += "og.reload_intersection_selector_options('"+genid+"',"+index+","+opt_index+");";
		
		$(selector).attr('onchange', onchange_functions);
		
		$(selector).show();
		
		selected_val_dim = '0';
		var selected_val_members = null;
		if (selected_val) {
			if (isNaN(selected_val)) {
				if (!isNaN(selected_val["dim_id"])) {
					selected_val_dim = selected_val["dim_id"];
				}
				if (selected_val["member_ids"]) {
					selected_val_members = selected_val["member_ids"];
				}
			} else {
				selected_val_dim = selected_val;
			}
			if (selected_val_dim) {
				$(selector).val(selected_val_dim);
			}
		}

		var sel_container_id = genid+"_sel_container_"+index;
		// append the dimension selector
		$("#"+genid+"_group_by_container #"+sel_container_id + "_options #"+genid+"_intersection_sel_"+index).append($(selector));

		
		// intersection options, such as intersect by specific members
		og.report_group_by_render_intersection_selector_options(genid, index, opt_index, selected_val_dim, selected_val_members, sel_container_id);
		
		// append new line
		$("#"+genid+"_group_by_container #"+sel_container_id + "_options #"+genid+"_intersection_sel_"+index).append("<br/>");
		
	}
}

og.reload_intersection_selector_options = function(genid, index, opt_index) {

	var dim_id = $("#"+genid+"_group_by_"+index+"_int_"+opt_index).val();
	var gb_id = "dim_"+ dim_id + "_0";
	
	var hf_suffix = "["+opt_index+"][member_ids]";

	$("#"+genid+"_options_"+index+"_"+opt_index+"_details").html('');

	var options_id = genid+"_options_"+index+"_"+opt_index;
	var onchange_fn = "og.render_report_gb_member_selector(this, '"+options_id+"', '"+gb_id+"', '', '"+index+"', '"+hf_suffix+"');";

	$("#"+genid+"_options_"+index+"_"+opt_index+"_chk").attr("onchange", onchange_fn);
	$("#"+genid+"_options_"+index+"_"+opt_index+"_chk").removeAttr("checked");
}



og.report_group_by_render_intersection_selector_options = function(genid, index, opt_index, selected_val_dim, selected_val_members, sel_container_id) {
	var options_id = genid+"_options_"+index+"_"+opt_index;
	var el = document.getElementById(options_id);
	

	if (typeof(selected_val_members) == 'string') {
		selected_val_members = Ext.util.JSON.decode(selected_val_members);
	}

	var checked_html = selected_val_members ? 'checked="checked"' : '';
	var selected_member_json = "[]";
	if (selected_val_members && selected_val_members.length > 0) {
		selected_member_json = Ext.util.JSON.encode(selected_val_members);
		selected_member_json = selected_member_json.replace(/"/ig,'');
	}

	gb_id = "dim_" + selected_val_dim + "_" + "0";

	var hf_suffix = "["+opt_index+"][member_ids]";
	
	var onchange_fn = "og.render_report_gb_member_selector(this, '"+options_id+"', '"+gb_id+"', '"+selected_member_json+"', '"+index+"', '"+hf_suffix+"');";

	var options_html = '<span id="'+options_id+'" class="options">' + '<table><tr><td>'
		+ '<label for="'+options_id+'_chk">' + lang('intersect by specific members')
		+ '</label>&nbsp;<input type="checkbox" id="'+options_id+'_chk" onchange="'+onchange_fn+'" '+checked_html+'/>&nbsp;</span>'
		+ '</td><td>'
		+ '<span id="'+options_id+'_details" class="details"></span>'
		+ '</td></tr></table></span>';

	$("#"+genid+"_group_by_container #"+sel_container_id + "_options #"+genid+"_intersection_sel_"+index).append(options_html);

	if (selected_val_members && selected_val_members.length > 0) {
		$('#'+options_id+'_chk').change();
	}
}




og.on_report_group_by_selector_change = function(genid, next_index) {
	if (next_index > 3) return;
	
	var sel = document.getElementById(genid+"_group_by_"+next_index);
	if (!sel && next_index < 3) {
		og.add_report_group_by_selector(genid, next_index);
	}
	$("#"+genid+"_hide_details_container").show();
	$("#"+genid+"_hide_total_counts_container").show();

	var index = next_index - 1;
	if (index > 0) {
		$("#"+genid+"_show_last_group_as_column_container").show();
	}
	
	og.report_group_by_render_options(genid, index, $("#"+genid+"_group_by_"+(index)).val());
}


<?php if (count($group_by) == 0) { ?>
	og.add_report_group_by_selector(genid, 0);
<?php } else {
		$i=0;
		foreach ($group_by as $gb) {
			$gb_options = array_var($group_by_options, $gb);
			$gb_options_csv = is_array($gb_options) ? json_encode($gb_options) : '';
		?>
			og.add_report_group_by_selector(genid, <?php echo $i?>, '<?php echo $gb?>', '<?php echo $gb_options_csv?>');
		<?php
			$i++;
		}
		if ($i < 3) {
		?>og.add_report_group_by_selector(genid, <?php echo $i?>);<?php
		}
?>
<?php } ?>
</script>