<?php


	function tasks_report_print_group($group_obj, $grouped_objects, $options, $skip_groups = array(), $level = 0, $prev = "", &$crumbJs = "") {
		
		$margin_left = 15 * $level;
		$cls_suffix = $level > 2 ? "all" : $level;
		$next_level = $level + 1;
		
		$group_name = $group_obj['group']['name'];
		switch ($group_name){
			case 'Missing lang: 100': $group_name = lang('low priority'); break ;
			case 'Missing lang: 200': $group_name = lang('normal priority'); break ;
			case 'Missing lang: 300': $group_name = lang('high priority'); break ;
			case 'Missing lang: 400': $group_name = lang('urgent priority'); break ;
		}
		echo '<div style="margin-left:' . $margin_left.'px;" class="report-group-heading-'.$cls_suffix.'">' . $group_name . '</div>';
		
		$mem_index = $prev . $group_obj['group']['id'];
		
		// draw the table for the values
		if (isset($grouped_objects[$mem_index]) && count($grouped_objects[$mem_index]) > 0) {
			tasks_report_print_table($grouped_objects[$mem_index], $margin_left, $options, $group_name, $crumbJs);
		}
		
		if (!is_array($group_obj['subgroups'])) return;
		
		$subgroups = order_groups_by_name($group_obj['subgroups']);
		foreach ($subgroups as $subgroup) {
			tasks_report_print_group($subgroup, $grouped_objects, $options, array(), $next_level, $prev . $group_obj['group']['id'] . "_", $crumbJs);
		}
	}
	
	
	function tasks_report_print_table($objects, $left, $options, $group_name, &$crumbJs = "") {
		echo '<div style="padding-left:'. $left .'px;">';
		echo '<table class="reporting-table"><tr class="reporting-table-heading">';
		echo '<th>' . lang('title') . '</th>';
		echo '<th>' . lang('related to') . '</th>';
		echo '<th>' . lang('description') . '</th>';
		echo '<th>' . lang('start date') . '</th>';
		echo '<th>' . lang('due date') . '</th>';
		if (!$options['user'] instanceof Contact) {
			echo '<th>' . lang('assigned to') . '</th>';
		}
		// echo status column, only if status is not completed
		$add_cols = 0;
		if (array_var($options, 'status') != 2) {
			echo '<th style="text-align:center;">' . lang('status') . '</th>';
			$add_cols++;
		}
		
		echo '</tr>';
		
		$alt_cls = "";
		$cols = 5 + $add_cols;
		if (!$options['user'] instanceof Contact) $cols++;
		foreach ($objects as $t) {
			tasks_report_print_task($t, 0, $cols, $options, $alt_cls, $crumbJs);
		}
		
		echo '</table></div>';
	}
	
	
	function tasks_report_print_task(ProjectTask $t, $margin_left, $cols, $options, &$alt_cls='', &$crumbJs='') {
		echo "<tr $alt_cls>";
		echo "<td class='name bold'><span style='padding-left:".$margin_left."px;'>" . clean($t->getObjectName()) ."</td>";
		
		echo '<td class="person" style="overflow-x:hidden;"><span id="t-'.$t->getId().'" style="font-weight:normal;"></span></td>';
		
		$crumbOptions = $t->getMembersToDisplayPath();
		$crumbJs .= escape_character("var crumbHtml = og.getEmptyCrumbHtml(".json_encode($crumbOptions).", null, null, null, 13, 50);");
		$crumbJs .= 'document.getElementById("t-'. $t->getId().'").innerHTML = crumbHtml;';
		
		echo "<td class='name'>" . clean(html_to_text($t->getText())) ."</td>";
		
		$dateFormat = user_config_option('date_format');
		Hook::fire("custom_property_date_format", null, $dateFormat);
		$startFormat = $dateFormat;
		$dueFormat = $dateFormat;
		if(!$t->getUseStartTime()){
			$startFormat = user_config_option('date_format');
		}
		if(!$t->getUseDueTime()){
			$dueFormat = user_config_option('date_format');
		}		
		
		echo "<td class='date'>" . ($t->getStartDate() instanceof DateTimeValue ? format_date($t->getStartDate(),$startFormat) : '') . "</td>";
		echo "<td class='date'>" . ($t->getDueDate() instanceof DateTimeValue ? format_date($t->getDueDate(),$dueFormat) : '') . "</td>";
		if (!$options['user'] instanceof Contact) {
			echo "<td class='person'>" . clean($t->getAssignedTo() instanceof Contact ? $t->getAssignedTo()->getObjectName() : '') ."</td>";
		}
		
		// echo status column, only if status is not completed
		if (array_var($options, 'status') != 2) {
			$today = DateTimeValueLib::now();
			$today->beginningOfDay();
			$status_style = 'padding: 1px 5px; ; margin: 3px; border-radius: 4px ; text-align: center; ';
			if ($t->isCompleted ()) {
				$status_value = lang ( "completed" );
				$status_style .= 'background-color: #73D073 ; color: #FFFFFF ;';
			} elseif ($t->getDueDate () instanceof DateTimeValue && $t->getDueDate ()->isToday ()) {
				$status_value = lang ( 'today' );
				$status_style .= 'background-color: #FFEB9C; color: #E26900; ';
			} elseif ($t->getDueDate () instanceof DateTimeValue && $t->getDueDate ()->getTimestamp () < $today->getTimestamp ()) {
				$status_value = lang ( 'days late', $t->getLateInDays () );
				$status_style .= 'background-color: #F6D4D5; color: #F6464C ';
			} else {
				$status_value = lang ( 'days left', $t->getLeftInDays () );
				$status_style .= 'background-color: #F5F5F5; color: #333333';
			}
			echo "<td class='date'><div style='$status_style'> $status_value </div></td>";
		}
		
		echo "</tr>";
		$comments = $t->getComments();
		foreach ($comments as $comment) {
			$name = '<span class="bold">'. ($comment->getCreatedBy() instanceof Contact ? clean($comment->getCreatedBy()->getObjectName()) : '') .'</span>';
			echo "<tr $alt_cls><td colspan='$cols' style='height:16px;padding-top:0;padding-bottom:0;'>";
			echo "<div class='desc' style='font-weight:normal;padding-left:".$margin_left."px;'>$name: ".$comment->getText()."</div></td></tr>";
		}
		$alt_cls = $alt_cls == "" ? 'class="alt-row"' : "";
		
		if (isset($t->tmp_subtasks) && count($t->tmp_subtasks) > 0) {
			foreach ($t->tmp_subtasks as $subt) {
				tasks_report_print_task($subt, $margin_left + 40, $cols, $options, $alt_cls, $crumbJs);
			}
		}
	}
	

	$date_format = user_config_option('date_format');
	if (array_var($post, 'due_date_type') == 6) {
		if ($start_time instanceof DateTimeValue) $start_time->advance(-1 * logged_user()->getUserTimezoneValue(), true);
		if ($end_time instanceof DateTimeValue) $end_time->advance(-1 * logged_user()->getUserTimezoneValue(), true);
	}
	
	if ($start_time instanceof DateTimeValue) { ?>
		<span class="bold"><?php echo lang('from')?></span>:&nbsp;<?php echo $start_time->format($date_format) ?>
	<?php }
	if ($end_time instanceof DateTimeValue) { ?>
		<span class="bold" style="padding-left: 10px"><?php echo lang('to date')?></span>:&nbsp;<?php echo $end_time->format($date_format) ?>
	<?php } ?>
	
	<?php if ($user instanceof Contact && $user->getObjectName() != '') { ?>
		<span class="bold"><?php echo lang('reporting user')?></span>:&nbsp;<?php echo clean($user->getObjectName()); ?>
	<?php }	?>

	<?php if (count(active_context_members(false)) > 0) : ?>
	<div class="clear"></div>
	<div style="margin-bottom: 10px; padding-bottom: 5px; float:left;">
		<h5><?php echo lang('showing information for')?>:</h5>
		<ul>
		<?php
			$context = active_context();
			foreach ($context as $selection) :
				if ($selection instanceof Member) : ?>
					<li><span class="coViewAction <?php echo $selection->getIconClass()?>"><?php echo $selection->getName()?></span></li>
		<?php 	endif;
			endforeach;
		?>
		</ul>
	</div>
	<div class="clear"></div>
	<?php endif; ?>
	
	<div class="tasks-report-container">
	<?php
	$crumbJs = "";
	
	$options = array_var($_SESSION, 'tasks_report_data', array());
	$options['user'] = $user;
	
	if (count(array_flat($grouped_tasks['grouped_objects'])) == 0) {
		?><div class="no-objects-message"><?php echo lang('there are no tasks matching the report parameters') ?></div><?php
	} else {
		$groups = order_groups_by_name($grouped_tasks['groups']);
		foreach ($groups as $gid => $group_obj) {
			tasks_report_print_group($group_obj, $grouped_tasks['grouped_objects'], $options, array(), 0, "", $crumbJs);
		}
	}
	?>
	</div>
	<script>
	$(function() {
		eval('<?php echo $crumbJs; ?>');
		og.eventManager.fireEvent('replace all empty breadcrumb');
	});
	</script>
<?php //} ?>