<?php

/**
 * WorkShift controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class WorkShiftController extends ApplicationController {

	var $plugin_name = "advanced_services";
	
	/**
	 * Construct the WorkShiftController
	 *
	 * @access public
	 * @param void
	 * @return WorkShiftController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
		Env::useHelper('functions', $this->plugin_name);
	} // __construct
	
	// ---------------------------------------------------
	//  Index
	// ---------------------------------------------------
	
	function init() {
		require_javascript("WorkShiftManager.js", $this->plugin_name);
		ajx_current("panel", "work-shift-containerpanel", null, null, true);
		ajx_replace(true);
	}
	

	
	function list_all() {
		ajx_current("empty");
		
		// Get all variables from request
		$start = array_var($_GET, 'start', 0);
		$limit = array_var($_GET, 'limit', config_option('files_per_page'));
		$order = array_var($_GET, 'sort');
		$order_dir = strtoupper(array_var($_GET, 'dir'));
		
		if (!is_numeric($start)) $start = 0;
		if (!is_numeric($limit)) $limit = config_option('files_per_page');
		
		$action = array_var($_GET, 'action');
		$attributes = array(
			"ids" => explode(',', array_var($_GET, 'ids')),
		);
		
		//Resolve actions to perform
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}

		$joins = array();
		$select_cols = array();
		
		switch ($order) {
			case 'name':
			case 'description':
				$order = "d.$order";
				break;
			default:
				$order = 'd.name';
		}
		
		if (!in_array($order_dir, array('ASC', 'DESC'))) {
			$order_dir = 'ASC';
		}
		
		
		$select_cols_sql = (count($select_cols) > 0 ? ",".implode(',',$select_cols) : "");
		$joins_sql = (count($joins) > 0 ? implode(' ', $joins) : "");
		$order_sql = "ORDER BY $order $order_dir";
		$limit_sql = "LIMIT $start, $limit";
		
		$sql = "SELECT d.* $select_cols_sql FROM ".TABLE_PREFIX."work_shifts d $joins_sql $order_sql $limit_sql";
		
		$work_shifts = DB::executeAll($sql);
		
		$count_sql = "SELECT count(d.id) as total FROM ".TABLE_PREFIX."work_shifts d $joins_sql";
		$total_rows = DB::executeAll($count_sql);
		
		// Prepare response object
		$object = $this->prepareObject($work_shifts, $start, $limit, $total_rows[0]['total']);
		ajx_extra_data($object);
		tpl_assign("listing", $object);
		
	}
	

	private function prepareObject($work_shifts, $start, $limit, $total) {
		$object = array(
			"totalCount" => $total,
			"start" => $start,
			"work_shifts" => array(),
		);
		
		for ($i = 0; $i < $limit; $i++){
			$work_shift_data = array_var($work_shifts, $i);
			
			if (!is_null($work_shift_data)) {
				$work_shift_data['object_id'] = $work_shift_data['id'];
				$object["work_shifts"][$i] = $work_shift_data;
			}
		}
		
		return $object;
	}
	
	
	/**
	 * Resolve action to perform
	 *
	 * @param string $action
	 * @param array $attributes
	 */
	private function resolveAction($action, $attributes){
		 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$work_shift = WorkShifts::instance()->findById($id);
					if (can_manage_configuration(logged_user()) && $work_shift instanceof WorkShift){
						try{
							DB::beginWork();
							
							$work_shift->delete();
							
							DB::commit();
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) . ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;
		}
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
		
	}
	
	
	

	/**
	 * View single category
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function view() {
		$this->addHelper('textile');

		$object = WorkShifts::instance()->findById(get_id());
		if(!($object instanceof WorkShift)) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		} // if

		if(!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		} // if
		
		tpl_assign('object', $object);
		
		ajx_extra_data(array("title" => $object->getName()));
		ajx_set_no_toolbar(true);
		
	} // view
	

	/**
	 * Add billing category definition
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function add() {
		
		if (!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$work_shift = new WorkShift();
		tpl_assign('object', $work_shift);

		$data = array_var($_POST, 'work_shift');
		if(!is_array($data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
		} else {
			foreach ($data as $k => &$v) {
				$v = remove_scripts($v);
			}
			try {
				DB::beginWork();
				
				$this->save_work_shift($data, $work_shift);
				
				DB::commit();
				
				flash_success(lang('success add work shift', $work_shift->getName()));
				
	          	if (array_var($_REQUEST, 'modal')) {
	          		evt_add("reload current panel");
          		} else {
          			ajx_current("back");
          		}
          		
	          	ajx_add("overview-panel", "reload");

			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
				
			} // try

		} // if
	} // add

	/**
	 * Edit billing category definition
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function edit() {
		$this->setTemplate('add');
		
		$work_shift = WorkShifts::instance()->findById(get_id());
		if (!$work_shift instanceof WorkShift) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		if (!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		tpl_assign('object', $work_shift);
		
		$data = array_var($_POST, 'work_shift');
		if(!is_array($data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			$shift_days = WorkShiftDays::instance()->findAll(array("conditions" => "shift_id=".$work_shift->getId()));
			$days_data = array();
			foreach ($shift_days as $sd) {/* @var $sd WorkShiftDay */
				$start = DateTimeValueLib::now();
				$start->setHour($sd->getStartHour());
				$start->setMinute($sd->getStartMinute());
				$start_time = $start->format(user_config_option('time_format_use_24') ? "G:i" : "g:i A");
				
				$end = DateTimeValueLib::now();
				$end->setHour($sd->getEndHour());
				$end->setMinute($sd->getEndMinute());
				$end_time = $end->format(user_config_option('time_format_use_24') ? "G:i" : "g:i A");
				
				$days_data[$sd->getWeekDay()] = array('start' => $start_time, 'end' => $end_time, 'is_day_off' => $sd->getIsDayOff());
			}
			tpl_assign('days_data', $days_data);
			
		} else {
			foreach ($data as $k => &$v) {
				$v = remove_scripts($v);
			}
			try {
				DB::beginWork();
				
				$this->save_work_shift($data, $work_shift);
				
				DB::commit();
				
				flash_success(lang('success edit work shift', $work_shift->getName()));
				
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				} else {
					ajx_current("back");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
			} // try
		} // if
	} // edit
	
	
	private function save_work_shift($data, WorkShift $work_shift) {
		
		$work_shift->setFromAttributes($data);
		
		$work_shift->setBreakMinMinutes(array_var($data['break_min_minutes'], 'hours')*60 + array_var($data['break_min_minutes'], 'minutes'));
		$work_shift->setBreakEveryXMinutes(array_var($data['break_every_x_minutes'], 'hours')*60 + array_var($data['break_every_x_minutes'], 'minutes'));
		
		$work_shift->save();
		
		WorkShiftDays::instance()->delete('shift_id='.$work_shift->getId());
		
		foreach ($data as $key => $day_data) {
			
			if (is_numeric($key) && $day_data['enabled']) {
				
				$shift_day = new WorkShiftDay();
				$shift_day->setShiftId($work_shift->getId());
				$shift_day->setWeekDay($key);
				
				$start_data = getTimeValue($day_data['start_time']);
				$shift_day->setStartHour($start_data['hours']);
				$shift_day->setStartMinute($start_data['mins']);
				
				$end_data = getTimeValue($day_data['end_time']);
				$shift_day->setEndHour($end_data['hours']);
				$shift_day->setEndMinute($end_data['mins']);
				
				$shift_day->setIsDayOff($day_data['is_day_off']);
				
				$shift_day->save();
			}
		}
	}

	/**
	 * Delete specific analysis
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function delete() {
		
		$work_shift = WorkShifts::instance()->findById(get_id());
		if (!$work_shift instanceof WorkShift) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		if (!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		try {
			DB::beginWork();
			
			$name = $work_shift->getName();
			
			$work_shift->delete();
		
			DB::commit();
		
			flash_success(lang('success delete work shift', $name));
		
			if (array_var($_REQUEST, 'modal')) {
				evt_add("reload current panel");
			} else {
				ajx_current("back");
			}
		
		} catch(Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			ajx_current("empty");
		}
	} // delete



} // WorkShiftController

?>