<?php
Hook::register("resource_users");

/**
 * Filters the array removing the users defined as resources
 * @param array $users
 * @return array
 */
function _filter_resource_users($users) {
	foreach ($users as $i => &$user) {
		if ($user instanceof Contact && $user->getColumnValue('is_resource')) {
			unset($users[$i]);
		}
	}
	return $users;
}


function resource_users_object_definition($type, &$ret){
	if ($type == 'Contact' && check_column_exists(TABLE_PREFIX.'contacts', 'is_resource')) {
		$ret['is_resource'] = DATA_TYPE_BOOLEAN;
	}
}

function resource_users_user_card_user_type_more_info($user, &$ignored) {
	if ($user instanceof Contact && $user->getColumnValue('is_resource')) {
		echo "&nbsp;(". lang('resource') .")";
	}
}


// user add/edit input to select if is resource
function resource_users_render_additional_user_data_fields($contact, &$ignored) {
	if ($contact instanceof Contact && (can_manage_security(logged_user()) || $contact->getId()==logged_user()->getId() && $contact->getColumnValue('is_confidential'))) {
	?>
		<div class="clear"></div>
		<div class="field role" id="user_role_div">
			<label><?php echo lang("is resource user")?>:</label>
			
			<div style="float:left; padding-top: 5px; margin: 0 15px 0 0;">
				<?php echo yes_no_widget('contact[is_resource]', '', $contact->getColumnValue('is_resource'), lang('yes'), lang('no')) ?>
			</div>
			<div class="clear"></div>
		</div>
	<?php
	}
}

// disable login for resource users
function resource_users_additional_login_validations($params, &$allow_login) {
	$user = array_var($params, 'user');
	if (!$user instanceof Contact || $user->getColumnValue('is_resource')) {
		$allow_login = false;
	}
}

// Users list filter
function resource_users_user_and_groups_additional_filters($params, &$filters) {
	$exists_resourse_users = Contacts::instance()->count("is_resource=1 AND disabled=0") > 0;
	if ($exists_resourse_users) {
		$f_value = array_var($_SESSION, 'users_list_current_is_resource_filter', 'internal');
		$filters[] = array(
			'filter' => 'is_resource',
			'type' => 'list',
			'width' => 100,
			'initial_val' => $f_value,
			'value' => $f_value,
			'options' => json_encode(array(array('everyone', lang('everyone')), array('internal', lang('internal')), array('resources', lang('resources')))),
		);
	}
}

function resource_users_user_and_groups_additional_filters_query($params, &$filter_conditions) {
	$exists_resourse_users = Contacts::instance()->count("is_resource=1 AND disabled=0") > 0;
	if ($exists_resourse_users) {
		$get_params = array_var($params, 'params');
		$is_resource_filter_value = array_var($get_params, 'is_resource');
		
		if ($is_resource_filter_value == 'internal') {
			$filter_conditions .= " AND c.is_resource=0 ";
		} else if ($is_resource_filter_value == 'resources') {
			$filter_conditions .= " AND c.is_resource=1 ";
		}
		$_SESSION['users_list_current_is_resource_filter'] = $is_resource_filter_value;
	}
}
// --

// permission group edition: additional class for resource users
function resource_users_perm_group_edit_user_box_add_class($user, &$add_class) {
	if ($user instanceof Contact && $user->getColumnValue('is_resource')) {
		$add_class .= " is-resource";
	}
}

// permission group edition: checkbox to show/hide resource users 
function resource_users_perm_group_edit_render_top_inputs($params, &$ignored) {
	$genid = array_var($params, 'genid');
	$exists_resourse_users = Contacts::instance()->count("is_resource=1 AND disabled=0") > 0;
	if ($exists_resourse_users) {
	?>
		<script>
		og.show_hide_resource_users = function(checkbox) {
			if ($(checkbox).attr("checked") == "checked") {
				$(".company-users .is-resource").show();
			} else {
				$(".company-users .is-resource").hide();
			}
		}
		</script>
	<?php 
		echo checkbox_field('show_resource_users', false, array('id' => $genid.'show_resource_users', 'onclick' => 'og.show_hide_resource_users(this);'));
		echo "&nbsp;". label_tag(lang("show resource users"), $genid.'show_resource_users', false, array('style' => 'display:inline-block;'), '');
		echo "</br></br>";
	}
}


// exclude resources from mail account permissions form
function resource_users_mail_account_permissions_extra_cond($params, &$conditions) {
	$conditions .= " AND is_resource=0 ";
}


// exclude from notifications
function resource_users_filter_object_notification_people($params, &$users) {
	$users = _filter_resource_users($users);
}

function resource_users_filter_object_notification_single_user($params, &$quit) {
	$user = array_var($params, 'user');
	if ($user instanceof Contact && $user->getColumnValue('is_resource')) {
		$quit = true;
	}
}
// --

// dont show resource users in created_by, updated_by, etc. in report columns conditions
function resource_users_filter_report_external_user_col_values($params, &$users) {
	$field = array_var($params, 'field');
	$ot_id = array_var($params, 'ot_id');
	$ot = ObjectTypes::instance()->findById($ot_id);
	
	if ($ot instanceof ObjectType) {
		// show resource users only if column is task assigned to or timeslot assigned to
		if ( !($ot->getName()=='task' && $field=='assigned_to_contact_id' || $ot->getName()=='timeslot' && $field=='contact_id') ) {
			$users = _filter_resource_users($users);
		}
	}
}


// dont show resource users in users' widget
function resource_users_filter_users_widget($params, &$users) {
	$users = _filter_resource_users($users);
}

// dont show resource users in subscribers
function resource_users_filter_subscribers($params, &$users) {
	$users = _filter_resource_users($users);
}

function resource_users_show_in_subscribers($params, &$show) {
	$user = array_var($params, 'user');
	if ($user instanceof Contact && $user->getColumnValue('is_resource')) {
		$show = false;
	}
}

// dont show resource users in user custom properties
function resource_users_object_contact_cp_filters($params, &$filters) {
	$cp = array_var($params, 'cp');
	$object = array_var($params, 'object');
	if ($cp instanceof CustomProperty && $cp->getType() == 'user') {
		$filters['is_resource'] = '0';
	}
}
