<?php
Hook::register('advanced_templates');


function advanced_templates_object_definition($type, &$ret) {
	if ($type == 'TemplateParameter') {
		$ret['is_required'] = DATA_TYPE_BOOLEAN;
	}
}


function advanced_templates_before_instantiate_paramters($args, &$error) {
	$template_id = array_var($args, 'id');
	$params = array_var($args, 'params');
	
	foreach ($params as $param_name => $val) {
		$tp = TemplateParameters::instance()->findOne(array('conditions' => array("template_id=? AND REPLACE(`name`,\"'\",'')=?", $template_id, $param_name)));
		if ($tp instanceof TemplateParameter && $tp->getColumnValue('is_required') && !$val) {
			$error = lang('custom property value required', $param_name);
		}
	}
}

function advanced_templates_template_param_instantiation_name($args, &$param_name) {
	$param = array_var($args, 'param');
	$template = array_var($args, 'template');
	
	if (array_var($param, 'is_required')) {
		$param_name .= " *";
	}
}

function advanced_templates_template_parameter_default_value($args, &$param_obj) {
	$parameter = array_var($args, 'parameter');
	if (!$parameter) return;
	
	$param_obj->setColumnValue('is_required', array_var($parameter, 'is_required'));
}

function advanced_templates_before_saving_instantiated_template_param($args, &$param_val) {
	$param = array_var($args, 'param');
	if (!$param) return;
	
	// if datetime parameter it is an array with 'date' and 'time'
	if ($param->getType() == 'datetime' && is_array($param_val)) {
		$param_val = implode(' ', $param_val);
	}
}

function advanced_templates_render_param_to_instantiate($args, &$param_val) {
	$parameter = array_var($args, 'param');
	if (!$parameter) return;
	
	$parameter_js_key = str_replace("'", "", $parameter['name']);
	
	if($parameter['type'] == 'datetime'){
		echo '<div style="float:left;">'. pick_date_widget2('parameterValues['.$parameter_js_key.'][date]') .'</div>';
		echo '<div style="float:left;">'. pick_time_widget2('parameterValues['.$parameter_js_key.'][time]') .'</div>';
		echo '<div class="clear"></div>';
	}

}

function advanced_templates_before_instantiate_template_date_param($args, &$result) {
	
	$date_str = array_var($args, 'date_str');
	if (!$date_str) return;
	
	// if datetime parameter it is an array with 'date' and 'time'
	if (is_array($date_str)) {
		$date_str = implode(" ", $date_str);
	}
		
	$hour_min = null;
	$date_str = trim($date_str);
	
	if (strpos($date_str, " ") > 0) {
		$exp = explode(" ", $date_str);
		$date_str = array_shift($exp);
		
		$hour_str = implode(" ", $exp);
		if ($hour_str != "") $hour_min = getTimeValue($hour_str);
		
		$result = array('date' => $date_str);
		if ($hour_min) $result['time'] = $hour_min;
	}
	
}

/**
 * Add estimated time as a possible variable to assign in templates
 */
function advanced_templates_add_custom_properties_as_template_properties($parameters, &$templateObjectProperties) {
	$templateObjectProperties[] = array(
		'id' => 'time_estimate', 
		'name' => lang('field ProjectTasks time_estimate'), 
		'type' => ProjectTasks::instance()->getColumnType('time_estimate')
	);
}


/**
 * This function is used to build the string with the numeric formula to save in the template property
 */
function advanced_templates_override_template_param_definition($params, &$propValue) {

	$controller = array_var($params, 'controller');
	$decoded_prop_inputs = array_var($params, 'decoded_prop_inputs');
	$obj_id = array_var($params, 'obj_id');
	$property = array_var($params, 'property');

	$objectPropertyValues = $controller->get_prop_input_decoded($decoded_prop_inputs, 'propValues');

	// if it is a numeric formula built using this plugin then build the string using its parameters
	if (isset($objectPropertyValues[$obj_id][$property]['num_formula'])) {

		$term_string_array = array();

		$propValueParams = $controller->get_prop_input_decoded($decoded_prop_inputs, 'propValueParam');
		$propValueOperation = $controller->get_prop_input_decoded($decoded_prop_inputs, 'propValueOperation');
		$propValueAmount = $controller->get_prop_input_decoded($decoded_prop_inputs, 'propValueAmount');
		$termOperators = $controller->get_prop_input_decoded($decoded_prop_inputs, 'propValueTermOperation');
		$propValueUnit = $controller->get_prop_input_decoded($decoded_prop_inputs, 'propValueUnit');

		$indexes = array_keys($objectPropertyValues[$obj_id][$property]['num_formula']);
		foreach ($indexes as $term_index) {
			$term_prop = $propValueParams[$obj_id][$property]['num_formula'][$term_index];
			$operator = $propValueOperation[$obj_id][$property]['num_formula'][$term_index];
			$term_amount = $propValueAmount[$obj_id][$property]['num_formula'][$term_index];
			$term_oper = $termOperators[$obj_id][$property]['num_formula'][$term_index];

			$term_string = "{".$term_prop."}" . $operator . $term_amount;

			$term_string_array[] = $term_string;
			$term_string_array[] = $term_oper;
		}

		$formula_unit = $propValueUnit[$obj_id][$property];
		if ($formula_unit) {
			$term_string_array[] = "unit=" . $formula_unit;
		}

		$propValue = implode('|', $term_string_array);
	}

}


function advanced_templates_override_template_task_numeric_param_calculation($params, &$result) {
	$parameterValues = array_var($params, 'parameterValues');
	$property = array_var($params, 'propName');
	$template_formula = array_var($params, 'value');

	Env::useHelper('functions', 'advanced_templates');
	$final_value = calculate_number_using_template_formula($template_formula, $parameterValues, $property);

	$result = array(
		'value_generated' => true,
		'value' => $final_value,
	);
}



function advanced_templates_template_param_date_calculation($parameters, &$dateNum) {
	
	$op = array_var($parameters, 'op');
	$parameterValues = array_var($parameters, 'parameterValues');
	$date = array_var($parameters, 'date');
	$unit = array_var($parameters, 'unit');
	$template_id = array_var($parameters, 'template_id');
	$original = array_var($parameters, 'original');
	$copy = array_var($parameters, 'copy');

	// is a formula
	if (!is_numeric($dateNum) && str_starts_with($dateNum, '{')) {
		$numParam = substr($dateNum, 1, strpos($dateNum, '}') - 1);
		$numParam = str_replace("'", "", $numParam);
		
		$numVal = $parameterValues[$numParam];
		if (!$numVal || !is_numeric($numVal)) $numVal = "0";
		
		$op = substr($dateNum, strpos($dateNum, '}') + 1, 1);
		if (!$op || !in_array($op, array('+','-','*','/'))) $op = '+';

		$const = substr($dateNum, strpos($dateNum, '}') + 2);
		if (!$const || !is_numeric($const)) {
			if ($op == '+' || $op == '-') $const = "0";
			if ($op == '*' || $op == '/') $const = "1";
		}

		// calculate the final value
		eval('$dateNum = '.$numVal.' '.$op.' '.$const.';');
	}
	
}