<?php
/**
 * Absence controller
 *
 * @version 1.0
 * @author Jose Bidegain <jose.bidegain@fengoffice.com>
 */
class AbsenceController extends ApplicationController {

	var $plugin_name = "attendance_absence_tracking";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
	}
	
	function index() {
		ajx_set_no_toolbar(true);
	}
	
	function list_all() {
		ajx_current("empty");
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		$dim_order = null;
		$cp_order = null;
		
		$ids = array();
		$exploded = explode(',', array_var($_GET,'ids'));
		foreach ($exploded as $exp) {
			if (is_numeric($exp)) $ids[] = $exp;
		}
		$attributes = array("ids" => $ids);
		
		if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		} else if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		} else if ($order == "dateUpdated") {
			$order = 'updated_on';
		}
		
		$extra_conditions = "";
		
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		$only_count_result = array_var($_GET, 'only_result',false);
		
		$result = Absences::instance()->listing(array(
			"order" => $order,
			"order_dir" => $order_dir,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"extra_conditions" => $extra_conditions,
			"count_results" => true,
			"only_count_results" => $only_count_result,
			"start" => $start,
			"limit" => $limit,
		));
		$absences = $result->objects;
		$object = $this->prepareObject($absences, $start, $limit, $result);
		
		ajx_extra_data($object);
	}

	
	private function prepareObject($absences, $start, $limit, $res_obj) {
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"absences" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
		
		$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType(Absences::instance()->getObjectTypeId());
		$count = min(array($limit, count($absences)));
		for ($i = 0; $i < $count; $i++){
		    if (isset($absences[$i])){
		        $obj = $absences[$i];
				if ($obj instanceof Absence){
					$info = $obj->getArrayInfo();
					$info['id'] = $obj->getId();
					$info['memPath'] = json_encode($obj->getMembersIdsToDisplayPath());
					$object["absences"][] = $info;

					foreach ($custom_properties as $cp) {
						$object["absences"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $obj);
					}
    			}
			}
		}
		return $object;
	}

	
	function add() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$absence_data = array_var($_POST, 'absence');
		
		if(!is_array($absence_data)) {
		    // set layout for modal absence
		    if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$obj = new Absence();
			$obj->setObjectTypeId(Absences::instance()->getObjectTypeId());
			
			//contact_config_option('')//work_shifts o work_shift_days o contact_shifts
			$sd = getDateValue(array_var($_POST, 'start_date', ''));
			$dd = getDateValue(array_var($_POST, 'due_date', ''));
			
			$absence_data = array(
			    'use_start_time' => 1,
			    'use_end_time' => 1,
			    'start_date' => $sd,
			    'start_time' => user_config_option('work_day_start_time'),
			    'due_date' => $dd,
			    'end_time' => user_config_option('work_day_end_time')
			);
			tpl_assign('absence', $obj);
			tpl_assign('absence_data', $absence_data);
			
			
			
		} else {
		    // absence submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$absence = $this->save_absence($absence_data, $member_ids);
				
				DB::commit();
				
			    ApplicationLogs::createLog($absence, ApplicationLogs::ACTION_ADD);
			    flash_success(lang('success add absence', $absence->getObjectName()));
			    ajx_current("back");
			    
			    if (array_var($_REQUEST, 'modal')) {
			    	evt_add("reload current panel");
			    }
				
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	function edit() {
		$this->setTemplate('add');
		
		$absence = Absences::instance()->findById(get_id());
		if(!($absence instanceof Absence)) {
			flash_error(lang('absence dnx'));
			ajx_current("empty");
			return;
		}
		
		if (logged_user()->isGuest() || !$absence->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$notAllowedMember = '';
		if(!Absence::canAdd(logged_user(), active_context(), $notAllowedMember )) {
			if (str_starts_with($notAllowedMember, '-- req dim --')) flash_error(lang('must choose at least one member of', str_replace_first('-- req dim --', '', $notAllowedMember, $in)));
			else flash_error(lang('no context permissions to add',lang("Absences"),$notAllowedMember ));
			ajx_current("empty");
			return;
		}
		
		$absence_data = array_var($_POST, 'absence');
		
		if(!is_array($absence_data)) {
			// set layout for modal absence
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			tpl_assign('absence', $absence);
			
		} else {
			// absence submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$absence = $this->save_absence($absence_data, $member_ids, $absence);
				
				DB::commit();

				ApplicationLogs::createLog($absence, ApplicationLogs::ACTION_EDIT);
				flash_success(lang('success edit absence', $absence->getObjectName()));
				//ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	private function save_absence($absence_data, $member_ids, $absence = null) {
		// Save Absence object
		$is_new = false;
		if (is_null($absence) || !$absence instanceof Absence) {
		    $absence = new Absence();
			$is_new = true;
		}
				
		//$absence_data['description'] = preg_replace("/[\n|\r|\n\r]/", '', array_var($absence_data, 'description'));
	
		$absence->setFromAttributes($absence_data);
		
		$absence->save();
		
		$object_controller = new ObjectController();
		if (!is_null($member_ids)) {
		    $object_controller->add_to_members($absence, $member_ids);
		}
		$object_controller->add_subscribers($absence);
		$object_controller->link_to_new_object($absence);
		$object_controller->add_custom_properties($absence);
		
		//@TODO: enable after timeslots generation is fixed
		//generate_absence_timeslot(array('absence'=>$absence));
		
		return $absence;
	}
	
	
	
	
	
	function view() {
		ajx_set_no_toolbar();
		$absence = Absences::instance()->findById(get_id());
		if(!($absence instanceof Absence)) {
			flash_error(lang('absence dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$absence->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		tpl_assign('absence', $absence);
	}
	
	
	
	private function resolveAction($action, $attributes){
				 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				$messages = array();
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$absence = Absences::instance()->findById($id);
					if ($absence instanceof Absence && $absence->canDelete(logged_user())){
						try{
							DB::beginWork();
							$absence->trash();
							DB::commit();
							ApplicationLogs::createLog($absence, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$messages[] = $e->getMessage();
							$err++;
						}
					} else {
						$err++;
						$messages[] = lang('cannot delete absence', $absence instanceof Absence ? clean($absence->getName()) : '');
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) .("\n - ".implode("\n - ",$messages)). ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			case "archive":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$absence = Absences::instance()->findById($id);
					if ($absence instanceof Absence && $absence->canEdit(logged_user())){
						try{
							DB::beginWork();
							$absence->archive();
							DB::commit();
							ApplicationLogs::createLog($absence, ApplicationLogs::ACTION_ARCHIVE);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error archive objects", $err) . "<br />" . ($succ > 0 ? lang("success archive objects", $succ) : "");
				} else {
					$resultMessage = lang("success archive objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;		
		} // switch
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
	}
}