<?php
/**
 * TaskAfterAction Controller controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class TaskAfterActionController extends ApplicationController {

	var $plugin_name = "conditional_tasks";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('task_workflow', 'conditional_tasks');
	}
	
	
	function check_conditions_and_execute_actions(ProjectTask $task) {
		
		$group_ids = TaskAfterActionConditions::getConditionGroups($task);
		
		foreach ($group_ids as $group_id) {
			
			$conditions_ok = TaskAfterActionConditions::checkConditionsForGroup($task, $group_id);
			if ($conditions_ok) {
				TaskAfterActionActions::executeActionsForGroup($task, $group_id);
			}
		}
	}
	
	
	
	function add_task_workflow_form() {
		
	}
	
	function add_task_workflow_group() {
		
		$available_conditions = get_task_workflow_available_conditions();
		
		$numeric_comps = array("=" => "=", "!=" => "&ne;", "<=" => "&le;", ">=" => "&ge;", "<" => "&lt;", ">" => "&gt;");
		$text_comps = array("=" => lang('equal'), "!=" => lang('not equal'), "like" => lang('like'), "not like" => lang('not like'));
		
		if (array_var($_REQUEST, 'group_info') != '') {
			$group_info_str = array_var($_REQUEST, 'group_info');
			$group_info = json_decode($group_info_str, true);
			
			if (is_array($group_info)) {
			  foreach ($group_info as $k => &$cond) {
				if ($cond['field'] == 'completed_by_id') {
					$cond['field'] = 'completed';
					$cond['type'] = 'boolean';
					$cond['value'] = $cond['comp'] == "=" ? "0" : "1";
				}
			  }
			}
			tpl_assign('group_info', $group_info);
		}
		
		tpl_assign('available_conditions', $available_conditions);
		tpl_assign('numeric_comps', $numeric_comps);
		tpl_assign('text_comps', $text_comps);
		
		tpl_assign('genid', array_var($_REQUEST, 'genid'));
		tpl_assign('index', array_var($_REQUEST, 'index'));
		ajx_extra_data(array('params' => array_var($_REQUEST, 'params'), 'index' => array_var($_REQUEST, 'index')));
	}
	

	
	function render_single_member_selector() {
	
		$dim_id = array_var($_REQUEST, 'dim_id');
		$hf_name = array_var($_REQUEST, 'hf_name');
		$dim_assoc_id = array_var($_REQUEST, 'dim_assoc_id');
		$hide_label = array_var($_REQUEST, 'hide_label');
		$object_type_id = array_var($_REQUEST, 'object_type_id');
		$selected_member_ids = explode(',', array_var($_REQUEST, 'selected_member_ids', ""));
	
		if (is_numeric($dim_id)) {
			$dim = Dimensions::instance()->findById($dim_id);
			tpl_assign('dim', $dim);
		} else {
			$dim_id = 0;
		};
	
		tpl_assign('hf_name', $hf_name);
		tpl_assign('dim_assoc_id', $dim_assoc_id);
		tpl_assign('hide_label', $hide_label);
		tpl_assign('object_type_id', $object_type_id);
	
		foreach ($selected_member_ids as $k => &$v) {
			if (!is_numeric($v)) unset($selected_member_ids[$k]);
		}
		if (count($selected_member_ids) > 0) {
			$sql = "SELECT m.id FROM ".TABLE_PREFIX."members m WHERE m.id IN (".implode(',', $selected_member_ids).") AND m.dimension_id=$dim_id";
			$clean_sel_member_ids = array_flat(DB::executeAll($sql));
			tpl_assign('selected_member_ids', $clean_sel_member_ids);
		} else {
			tpl_assign('selected_member_ids', array());
		}
		
		ajx_extra_data(array('index' => array_var($_REQUEST, 'index'), 'genid' => array_var($_REQUEST, 'genid')));
	}
	
}