<?php

/**
 * Member class
 *
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class TaskAfterActionCondition extends BaseTaskAfterActionCondition {

	function checkConditionForTask($task) {
		if (!$task instanceof ContentDataObject) return false;
		
		$value_to_check = $this->getConditionValue();
		
		if (str_starts_with($this->getConditionField(), "cp_")) {
			// custom property
			$cp_id = str_replace("cp_", "", $this->getConditionField());
			if (!is_numeric($cp_id)) return false;
			$cp = CustomProperties::instance()->findById($cp_id);
			$real_value_obj = CustomPropertyValues::getCustomPropertyValue($task->getId(), $cp_id);
			if ($cp instanceof CustomProperty && $real_value_obj instanceof CustomPropertyValue) {
				$real_value = $real_value_obj->getValue();

                if ($cp->getOgType() == DATA_TYPE_BOOLEAN && $real_value == '-1') {
                    $real_value = "0";
                }elseif ($cp->getOgType() == DATA_TYPE_BOOLEAN && $real_value == '0'){
                    return false;
                }
			} else {
				return false;
			}
		} else {
			if (!$task->manager()->columnExists($this->getConditionField())) return false;
			$real_value = $task->getColumnValue($this->getConditionField());
		}
		
		switch ($this->getConditionComparison()) {
			case "=": 
				return $value_to_check == $real_value;
			case "!=":
				return $value_to_check != $real_value;
			case ">":
				return $real_value > $value_to_check;
			case "<":
				return $real_value < $value_to_check;
			case ">=":
				return $real_value >= $value_to_check;
			case "<=":
				return $real_value <= $value_to_check;
			case "like":
				return strpos($real_value, $value_to_check) !== false;
			case "not like":
				return strpos($real_value, $value_to_check) === false;
			default:
				return false;
		}
	}
	
}
