<?php
Hook::register("conditional_tasks");

function conditional_tasks_verify_objects_before_template_instantiation($template, &$objects) {
	$sql = "select group_concat(action_data) as task_ids from ".TABLE_PREFIX."task_after_action_actions
			where `action`='instantiate' and action_data != '' and task_id IN (
				select object_id from ".TABLE_PREFIX."template_tasks where template_id=".$template->getId()."
			);";
	$rows = DB::executeAll($sql);
	
	$conditional_task_ids = "";
	if (count($rows) > 0) {
		$conditional_task_ids = array_filter(explode(',', array_var($rows[0], 'task_ids')));
	}
	
	if (is_array($conditional_task_ids) && count($conditional_task_ids)) {
		foreach ($objects as $k => &$obj) {
			$intersection = array_intersect(explode(',', $obj->getColumnValue('parents_path')), $conditional_task_ids);
			$has_conditional_parent = count($intersection) > 0;
			if (!$obj instanceof ContentDataObject || in_array($obj->getId(), $conditional_task_ids) || $has_conditional_parent) {
				unset($objects[$k]);
			}
		}
	}
}

function conditional_tasks_object_edit_categories($object, &$categories) {
	if ($object instanceof TemplateTask) {
		tpl_assign('task_for_cond', $object);
		$categories[] = array(
			'id' => 'conditional_actions',
			'name' => lang('conditional actions'),
			'required' => false,
			'visible' => true,
			'content' => tpl_fetch(get_template_path('add_conditional_actions_form', 'conditional_tasks', 'conditional_tasks')),
		);
		
		tpl_assign('task', $object);
		$categories[] = array(
			'id' => 'task_workflow',
			'name' => lang('task workflow'),
			'required' => false,
			'visible' => true,
			'content' => tpl_fetch(get_template_path('add_task_workflow_form', 'task_after_action', 'conditional_tasks')),
		);
	} else if ($object instanceof COTemplate && $object->getId() > 0) {
		// get tasks of this template that are conditional
		$sql = "select group_concat(action_data) as task_ids from ".TABLE_PREFIX."task_after_action_actions 
				where `action`='instantiate' and action_data != '' and task_id IN (select object_id from ".TABLE_PREFIX."template_tasks where template_id=".$object->getId().");";
		$rows = DB::executeAll($sql);
		
		$conditional_task_ids = "";
		if (count($rows) > 0) {
			$conditional_task_ids = array_var($rows[0], 'task_ids');
		}
		
		echo "<input type='hidden' name='conditional_task_ids' value='$conditional_task_ids' />";
	}
}

function conditional_tasks_after_object_save($object, &$ret) {
	// execute only when editing the template task, not when it is saved from other controllers
	if ($object instanceof TemplateTask && array_var($_REQUEST, 'c') == 'task') {
		$conditions = array_var($_REQUEST, 'task_action_conditions', array());
		
		if (array_var($_REQUEST, 'task_action_conditions')) {
			// delete the removed conditions
			DB::execute("DELETE FROM ".TABLE_PREFIX."task_action_conditions WHERE task_id=".$object->getId());
			
			// save conditions
			foreach ($conditions as $cond) {
				$action = array_var($cond, 'action');
				$property = array_var($cond, 'property');
				
				if (!in_array($action, array('creation', 'completion'))) continue;
				if (!str_starts_with($property, "cp_") && !in_array($property, ProjectTasks::instance()->getColumns())) continue;
				
				DB::execute("INSERT INTO ".TABLE_PREFIX."task_action_conditions (task_id, property, action, required_values, defined_only_in_templates) VALUES 
						('".$object->getId()."','$property','$action','',1) ON DUPLICATE KEY UPDATE task_id=task_id;");
				
			}
			// clean request variables to avoid saving the same conditions in subtasks
			unset($_REQUEST['task_action_conditions']);
		}
		
		
		Env::useHelper('task_workflow', 'conditional_tasks');
		
		if (array_var($_REQUEST, 'save_workflow')) {
			// workflow conditions
			$workflow_parameters = array_var($_REQUEST, 'task_workflow');
			$parsed_prameters = parse_task_workflow_conditions($workflow_parameters);
			if (is_array($parsed_prameters)) {
				save_task_workflow_parameters($object, $parsed_prameters);
			}
			
			// workflow actions
			$workflow_actions = array_var($_REQUEST, 'task_workflow_actions');
			$parsed_actions = parse_task_workflow_actions($workflow_actions);
			if (is_array($parsed_actions)) {
				save_task_workflow_actions($object, $parsed_actions);
			}
			
			// clean request variables to avoid saving the same workflow in subtasks
			unset($_REQUEST['save_workflow']);
		}
		
		
	} else if ($object instanceof ProjectTask) {
		$controller = new TaskAfterActionController();
		$controller->check_conditions_and_execute_actions($object);
	}
	
}


function conditional_tasks_after_template_object_instantiation($parameters, &$ignored) {
	$original = array_var($parameters, 'original');
	$copy = array_var($parameters, 'copy');
	
	// create conditions for instantiated tasks
	if ($original instanceof ContentDataObject && $copy instanceof ContentDataObject) {
		$otacs = TaskActionConditions::instance()->findAll(array("conditions" => "task_id=".$original->getId()));
		
		foreach ($otacs as $otac) {
			$tac = TaskActionConditions::instance()->findById(array('task_id' => $copy->getId(), 'action' => $otac->getAction(), 'property' => $otac->getProperty()));
			if (!$tac instanceof TaskActionCondition) {
				$tac = new TaskActionCondition();
				$tac->setTaskId($copy->getId());
				$tac->setAction($otac->getAction());
				$tac->setProperty($otac->getProperty());
				$tac->setDefinedOnlyInTemplates(1);
				$tac->setRequiredValues($otac->getRequiredValues());
				$tac->save();
			}
		}
	}
	
	
	// copy configuration for conditional actions (tables task_after_action_conditions and task_after_action_actions)
	if ($original instanceof ContentDataObject && $copy instanceof ContentDataObject) {
		$otaacs = TaskAfterActionConditions::instance()->findAll(array("conditions" => "task_id=".$original->getId()));
		foreach ($otaacs as $otaac) {
			$taac = TaskAfterActionConditions::instance()->findById(array('task_id' => $copy->getId(), 'group_id' => $otaac->getGroupId(), 'condition_field' => $otaac->getConditionField(), 'condition_comparison' => $otaac->getConditionComparison()));
			if (!$taac instanceof TaskAfterActionCondition) {
				$taac = new TaskAfterActionCondition();
				$taac->setTaskId($copy->getId());
				$taac->setGroupId($otaac->getGroupId());
				$taac->setConditionField($otaac->getConditionField());
				$taac->setConditionComparison($otaac->getConditionComparison());
				$taac->setConditionValue($otaac->getConditionValue());
				$taac->save();
			}
		}
		
		$otaaas = TaskAfterActionActions::instance()->findAll(array("conditions" => "task_id=".$original->getId()));
		foreach ($otaaas as $otaaa) {
			$taaa = TaskAfterActionActions::instance()->findById(array('task_id' => $copy->getId(), 'group_id' => $otaaa->getGroupId(), 'action' => $otaaa->getAction()));
			if (!$taaa instanceof TaskAfterActionAction) {
				$taaa = new TaskAfterActionAction();
				$taaa->setTaskId($copy->getId());
				$taaa->setGroupId($otaaa->getGroupId());
				$taaa->setAction($otaaa->getAction());
				$taaa->setActionData($otaaa->getActionData());
				$taaa->save();
			}
		}
	}
	
}


function conditional_tasks_before_completing_task($parameters, &$ignored) {
	$task = array_var($parameters, 'task');
	if ($task instanceof ProjectTask) {
		$tacs = TaskActionConditions::instance()->findAll(array("conditions" => "task_id=".$task->getId()." AND `action`='completion'"));
		foreach ($tacs as $tac) {
			if (str_starts_with($tac->getProperty(), "cp_")) {
				$cp_id = str_replace("cp_", "", $tac->getProperty());
				$cp_val = CustomPropertyValues::getCustomPropertyValue($task->getId(), $cp_id);
				if (!$cp_val instanceof CustomPropertyValue) {
					$cp = CustomProperties::instance()->findById($cp_id);
					throw new Exception(lang('property must be filled before completing task', $cp->getName()));
				}
			} else {
				if (!$task->getColumnValue($tac->getProperty())) {
					throw new Exception(lang('property must be filled before completing task', lang('field ProjectTasks '.$tac->getProperty())));
				}
			}
		}
	}
}

