<?php

/**
 * Customer controller
 *
 * @version 1.0
 * @author Marcos Saiz <marcos.saiz@fengoffice.com>
 */
class CustomerController extends ApplicationController {

	var $plugin_name = "crpm";
	var $dimension = null;
	var $base_conditions = array( "conditions" => "trashed_by_id = 0 AND archived_by_id = 0 " )  ;
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
		$this->addHelper('chart');
		$this->addHelper('contact_render_tab_functions');
		Env::useHelper('api', $this->plugin_name);
		Env::useHelper('functions', $this->plugin_name);
		Env::useHelper('customer_additional_data', 'crpm');
		
		$this->dimension = get_customers_dimension();
	} 
	
	
	function get_dimension_info() {
		$info = array(
			'dimension_id' => get_customers_dimension()->getId(),
			'object_type_id' => Customers::instance()->getObjectTypeId(),
		);
		ajx_current("empty");
		ajx_extra_data(array('dim_info' => $info));
	}
	

	function memberize(Customer $customer, $parent=null, $color=0, $template_id=0, $from_mail_id=0){
		return Customers::instance()->memberize($customer, $this->dimension->getId(), $parent, $color, $template_id, $from_mail_id);
	}
	
	function view() {
		
		ajx_current("empty");
		return;
		
		ajx_set_no_back(true);
		
		$id = get_id();
		if (!$id){
			if ( $memberId = array_var($_GET, "memberId")){
				$id = Members::instance()->findById($memberId)->getObjectId(); 			
			}else{
				flash_error(lang("not found"));
			}
		}
		
		$task_assignment_conditions = "";
		if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_assigned_to_other_tasks')) {
			$task_assignment_conditions = " AND assigned_to_contact_id = ".logged_user()->getId();
		}
		
		$tasks = ProjectTasks::instance()->listing(array(
			"order" => "due_date",
			"order_dir" => "ASC",
			"extra_conditions" => " AND is_template = 0 $task_assignment_conditions"
		))->objects;
		
		$completed_task_summary = array(
			"overdue" 	=> 0,
			"pending" 	=> 0,
			"today" 	=> 0,
			"completed" => 0,
		);
		
		foreach ($tasks as $task){
			$task = Objects::findObject($task->getId()); //TODO: Why load again? 
			/* @var $task ProjectTask */
			if ( $task instanceof ProjectTask ) {
				if ($task->isCompleted()) {
					$completed_task_summary['completed']++;	
				}elseif ($task->isLate()){
					$completed_task_summary['overdue']++;	
				}else{
					if ($task->getDueDate() instanceof DateTimeValue && $task->getDueDate()->isToday()) $completed_task_summary['today']++;
					else $completed_task_summary['pending']++;	
				}
			}
		}
		
		$overdue_upcoming_objects =  ProjectTasks::getOverdueAndUpcomingObjects(20) ; // FIXME: performance Killer
		
		$customer = Customers::instance()->findById($id);
		tpl_assign("projects", $customer->getProjects());
		tpl_assign("contacts", $customer->getContacts(true));
		tpl_assign("main_contact", $customer->getContact());
		/* @var $customer Customer */
		tpl_assign("installation_info",  $customer->getInstallationInfo());
		tpl_assign("contact_info", $customer->getContactInfo());
		tpl_assign("view_type", "default");
		tpl_assign("object", $customer);
		tpl_assign("customer", $customer);
		tpl_assign("status", $customer->getStatus());
		tpl_assign("content_template", array('customer_content', 'customer'));
 		if ($completed_task_summary['pending'] || $completed_task_summary['completed'] || $completed_task_summary['today'] || $completed_task_summary['overdue']) {
			tpl_assign("completed_task_summary" , $completed_task_summary);
 		}
		tpl_assign("overdue_upcoming_objects",$overdue_upcoming_objects);
		tpl_assign("unassigned_contacts",$this->getUnassignedContacts($customer));		
	}
	
	function edit() {
		$this->setTemplate('add');
		
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$customer = Customers::instance()->findById(get_id());
		if(!($customer instanceof Customer)) {
			flash_error(lang('customer dnx'));
			ajx_current("empty");
			return;
		} // if

		if(!$customer->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		} // if
		
		$customer_data = array_var($_POST, 'customer');

		if (is_array($customer_data)){
			//$customer = new Customer();
			$customer= Customers::instance()->findById(get_id());
			if($customer) {
				$action = ApplicationLogs::ACTION_EDIT;
				$this->handleSubmit($customer_data, $customer, $action);
				
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
			}else{
				flash_error(lang('object not found'));
			}
		}else{
			$customer_data = $customer->getArrayInfo();
			$member = Members::findOneByObjectId($customer->getId(), $this->dimension->getId());
		
			$ret = array();
			Hook::fire('check_additional_member_permissions', array('action' => 'edit', 'member' => $member, 'pg_ids' => logged_user()->getPermissionGroupIds()), $ret);
			if (count($ret) > 0 && !array_var($ret, 'ok')) {
				flash_error(array_var($ret, 'message'));
				ajx_current("empty");
				return;
			}
			
			tpl_assign('member', $member);
			tpl_assign('customer_data', $customer_data);
			tpl_assign('customer', $customer);
			//tpl_assign('inherit_contacts',$customer->getInheritContacts());
			tpl_assign('inherit_contacts', array());
			
			$permission_parameters = permission_member_form_parameters($member);
			tpl_assign('permission_parameters', $permission_parameters);
			
			$customer_member = Members::findOneByObjectId($customer->getId(), $this->dimension->getId());
			if ($parent = $customer_member->getParentMemberId() ){
				tpl_assign("parent_member", $parent);
			}else{
				tpl_assign("parent_member", 0);
			}
			
			$contact_data = array();
			$contact_controller = new ContactController();
			$contact = $customer->getContact();
			if ($contact instanceof Contact) {
				$contact_data = $contact_controller->get_contact_data_from_contact($contact);
			}
			tpl_assign('assoc_contact_data', $contact_data);
			
      		$this->populateComboboxes($customer);
		}
	}
	
	
	function add($customer_data = null,$is_api = false) {

	    if (!$is_api){
            if (logged_user()->isGuest()) {
                flash_error(lang('no access permissions'));
                ajx_current('empty');
                return;
            }
        }

		$customer = new Customer();
		$contact = new Contact();
		$notAllowedMember = '' ;
        if (!$is_api){
            if(!$customer->canAdd(logged_user(), active_context(), $notAllowedMember)) {
                if (str_starts_with($notAllowedMember, '-- req dim --')) flash_error(lang('must choose at least one member of', str_replace_first('-- req dim --', '', $notAllowedMember, $in)));
                else trim($notAllowedMember) == "" ? flash_error(lang('you must select where to keep', lang('the customer'))) : flash_error(lang('no context permissions to add',lang("customers"), $notAllowedMember));
                ajx_current("empty");
                return;
            } // if
        }

        if (!$is_api){
		    $data = array_var($_POST, 'customer');
        }else{
            $data = array_var($customer_data, 'customer');
        }
        $result = '';
		if(is_array($data)) {
			
			$result = $this->handleSubmit($data,$customer,null,$is_api ,$customer_data);

		}else{
			// --- Not submitting - populate data --- // 
			// See simple_select_box function for details about this array
			$customer_data = array();
			$customer_data['automatic_status'] = 1;
			if ( $name = array_var($_REQUEST, 'name') ) {
				$customer_data['name'] = $name ;
			}
			if ( $email = array_var($_REQUEST, 'email') ) {
				$customer_data['email'] = $email;
			}
			if ($parent = array_var($_REQUEST, 'parent') ){
				tpl_assign("parent_member", $parent);
			}else{
				tpl_assign("parent_member", 0);
			}
			if (isset($_REQUEST['type'])) {
				tpl_assign('obj_type_sel', $_GET['type']);
			}
			tpl_assign("current_dimension", $this->dimension);

			$ret = array();
			Hook::fire('check_additional_member_permissions', array('action' => 'add', 'parent_member_id' => $parent, 'pg_ids' => logged_user()->getPermissionGroupIds()), $ret);
			if (count($ret) > 0 && !array_var($ret, 'ok')) {
				flash_error(array_var($ret, 'message'));
				ajx_current("empty");
				return;
			}
			
			tpl_assign("customer_data", $customer_data);
			$this->populateComboboxes();
			
			$member = new Member();
			$member->setObjectTypeId(Customers::instance()->getObjectTypeId());
			$member->setDimensionId($this->dimension->getId());
			
			tpl_assign("member", $member);
			
			// Permissions
			$permission_parameters = permission_member_form_parameters(null, $this->dimension->getId());
			$logged_user_pg = array();
			foreach ($permission_parameters['allowed_object_types'] as $ot){
				$logged_user_pg[] = array('o' => $ot->getId(), 'w' => 1, 'd' => can_manage_dimension_members(logged_user()) ? 1 : 0, 'r' => 1);
			}
			$permission_parameters['member_permissions'][logged_user()->getPermissionGroupId()] = $logged_user_pg;
			
			$permission_parameters = get_default_member_permission($parent,$permission_parameters);
			
			tpl_assign('permission_parameters', $permission_parameters);
			
			// telephone types
			$all_telephone_types = TelephoneTypes::getAllTelephoneTypesInfo();
			tpl_assign('all_telephone_types', $all_telephone_types);
			
			Hook::fire('before_add_member', array('member'=>$member, 'parent'=>$parent), $ret);
		}
        if (!$is_api){
            tpl_assign("customer", $customer);
            tpl_assign("contact", $contact);
        }else{
            return $result;
        }
	}
	
	 function handleSubmit($data, $customer, $action = null, $is_api = false,$all_data_api = null) {
		foreach ($data as $k => &$v) {
			$v = remove_scripts($v);
		}
		if($action == null){
			$action = ApplicationLogs::ACTION_ADD;
		}
		$previous_data = array();
		$isNew = (boolean) $customer->isNew();
		$old_parent = -1;
		if (!$isNew) {
			$old_member = Members::findOneByObjectId($customer->getId(), $this->dimension->getId());
			$old_parent = $old_member->getParentMemberId();
			
			$previous_data = $old_member->getDataForHistory();
		}
		extract($data);
		$customer->setStatusId($status_id);
		$customer->setAutomaticStatus($automatic_status);
		$customer->setName(trim($name));
		$customer->setCustomerName($customer_name);
		$customer->setCustomerAddress($customer_address);
		$customer->setCustomerRut($customer_rut);
		$customer->setCustomerNumber($customer_number);
		
		tpl_assign("customer", $customer); 
		try {
			DB::beginWork();
			$customer->save();
			if ( isset($parent) && $parent) {
				$parentMember =  Members::instance()->findById($parent);
				if ($parentMember) {
					$member_data['parent_member_id'] = $parentMember->getId() ;
				}else{
					$member_data['parent_member_id'] = 0 ;
				}
			}
			/*
			if (!array_var($member_data, 'parent_member_id') && !SystemPermissions::userHasSystemPermission(logged_user(), 'can_manage_security')) {
				throw new Exception(lang('you cant add member without security permissions', lang('the customer')));
			}*/

			$color_cp = CustomProperties::getCustomPropertyByCode(Customers::instance()->getObjectTypeId(), 'color_special');
			if ($color_cp instanceof CustomProperty &&
					isset($_REQUEST['object_custom_properties'][$color_cp->getId()]['color'])) {
				$data['color'] = $_REQUEST['object_custom_properties'][$color_cp->getId()]['color'];
			}
			
		
			$this->memberize($customer, $parent, array_var($data, 'color'), array_var($data, 'template_id'), array_var($data, 'from_mail_id'));
			
			$member = Members::findOneByObjectId($customer->getId(), $this->dimension->getId());
			
			$ret = array();
			if ($isNew) {
				Hook::fire('check_additional_member_permissions', array('action' => 'add', 'member' => $member, 'parent_member_id' => $member->getParentMemberId(), 'pg_ids' => logged_user()->getPermissionGroupIds()), $ret);
			} else {
				Hook::fire('check_additional_member_permissions', array('action' => 'edit', 'member' => $member, 'pg_ids' => logged_user()->getPermissionGroupIds()), $ret);
			}
			if (count($ret) > 0 && !array_var($ret, 'ok')) {
				throw new Exception(array_var($ret, 'message'));
			}
			// save associated members
			Env::useHelper('dimension');
			save_associated_dimension_members(array('member' => $member, 'request' => $_REQUEST, 'is_new' => $isNew),$is_api,$all_data_api);
			
			Hook::fire("after_customer_save", array('customer_data' => $data, 'member' => $member), $customer);
			
			$ret = null;
			Hook::fire('after_member_save', array('member' => $member, 'previous_data' => $previous_data, 'is_new' => $isNew), $ret);
			
			// Finish
			//evt_add("reload dimension tree", array('dim_id' => $this->dimension->getId(), 'mid' => $member->getId(), 'pid' => $member->getParentMemberId()));
			if ($action == ApplicationLogs::ACTION_ADD) {
				$ret = null;
				Hook::fire('after_add_member', $member, $ret);
			}

				//Create a contact or company for this client
				if($contact_data_type){
					$object_controller = new ObjectController();
					$type_obj = ObjectTypes::instance()->findById($contact_data_type);
					if (isset($_REQUEST['dim_obj-associated-contact_id'])) {
						$new_contact_data = $_REQUEST['dim_obj-associated-contact_id']; //name="dim_obj-associated-contact_id[emails][0][id]"
						if (count($new_contact_data['emails']) > 0) {
							if (!$new_contact_data['emails'][0]['deleted']) {
								$new_contact_data['email'] = $new_contact_data['emails'][0]['email_address'];
								$new_contact_data['email_type'] = $new_contact_data['emails'][0]['type'];
								unset($new_contact_data['emails'][0]);
							}
						}
						$new_contact_data['comments'] = $_REQUEST['dim_obj']['associated']['contact_id']['comments'];
						$new_contact_data['timezone_id'] = $_REQUEST['dim_obj']['associated']['contact_id']['timezone_id'];
					
						$new_contact_data['company_id'] = $_REQUEST['dim_obj']['associated']['contact_id']['company_id'];
						$new_contact_data['department'] = $_REQUEST['dim_obj']['associated']['contact_id']['department'];
						$new_contact_data['birthday'] = $_REQUEST['dim_obj']['associated']['contact_id']['birthday'];
						
						if ($type_obj->getName() == 'contact') {	
								
							$_POST['contact'] = $new_contact_data;
							
						} else if ($type_obj->getName() == 'company') {
							$_POST['company'] = $new_contact_data;
						}
						
					}
					switch ($type_obj->getName()) {
						case "contact":
						 
							$contact_data = array_var($_POST, 'contact',false);
							$contact_email = $contact_data['email'];
							
							$existing_contact = null;
							
							if (!$isNew && $customer->getContactId() > 0) {
								$existing_contact = Contacts::instance()->findById($customer->getContactId());
							}
							
							if (!$existing_contact && trim($contact_email) != "") {
								if (config_option('allow_duplicated_contact_emails')) {
									$existing_contact = Contacts::getByFirstnameAndSurname(trim(array_var($contact_data, 'first_name')), trim(array_var($contact_data, 'surname')), $contact_email);
								} else {
									$contact_emails = ContactEmails::instance()->findAll(array('conditions' => array("`email_address` = ?", $contact_email)));
									foreach ($contact_emails as $ce) {
										if (can_read_sharing_table(logged_user(), $ce->getContactId())) {
											$existing_contact = Contacts::instance()->findById($ce->getContactId());
											break;
										}
									}
								}
							}
							
							$new_contact = false;							
							if ($existing_contact instanceof Contact && !$existing_contact->getIsCompany()) {
								$customer->setContactId($existing_contact->getId());
								$contact = $existing_contact;
								
							} else if ($existing_contact instanceof Contact) {
								// if existing contact is a company change its type
								$customer->setContactId($existing_contact->getId());
								$contact = $existing_contact;
								$contact->setIsCompany(false);
								
							} else {
								$contact = new Contact();
								$new_contact = true;
								
								// if new company added
								$comp_data = array_var($_POST, 'company');
								if (trim(array_var($comp_data, 'first_name', '')) != '') {
									$comp = new Contact();
									if($comp_data){
										$comp->setFromAttributes($comp_data);
									}
									$comp->setIsCompany(true);
									$comp->setObjectName();
									
									$comp->save();
									
									if(trim(array_var($comp_data, 'email', '')) != "") $comp->addEmail($comp_data['email'], $comp_data['email_type'], true);
									
									$contact_controller = new ContactController();
									$contact_controller->save_phones_addresses_webpages($comp_data, $comp);
									
									$contact_data['company_id'] = $comp->getId();
								}
								// -- end add new company
							}
							
							// birthday
							$contact_data['birthday'] = getDateValue($contact_data["birthday"]);
							
							if($contact_data){
								$contact->setFromAttributes($contact_data);
							}
							$name_exp = explode(" ", $name, 2);

							if(count($name_exp) >= 2){
								$contact->setFirstName($name_exp[0]);
								$contact->setSurname($name_exp[1]);
							}else{
								$contact->setFirstName($name);
								$contact->setSurname("");
							}

							$contact->setObjectName();
							
							$contact->add_skip_validation('email');
							$contact->save();
							
							$contact_data['email'] = $contact_email;
							$type_email = 'personal';//not getting in $_POST email type
							if(array_var($contact_data,'email') != "") {

							    //$c_email = $contact->getEmail();
							    $c_email = ContactEmails::instance()->findOne(array('conditions' => array("`contact_id` = ? AND `email_type_id` = ? AND TRIM(email_address) <> '' ",
							    		$contact->getId(), $contact_data['email_type'])));
							    
							    if ($c_email instanceof ContactEmail && $c_email->getEmailAddress() != $contact_data['email']) {
							        $c_email->setEmailAddress($contact_data['email']);
							        $c_email->setEmailTypeId($type_email);
								    $c_email->save();
							    }else if(!$c_email){
								    $email_types = EmailTypes::getAllEmailTypesInfo();
								    foreach($email_types as $email){
								        if ($email['id'] == $contact_data['email_type']){
								            $type_email = $email['code'];
								            break;
								        }
								    }
								    $contact->addEmail($contact_data['email'], $type_email, true);
								}
							} else {
							    // delete email if no input
							    if (array_var($contact_data,'email') == "") {
							        ContactEmails::instance()->delete("`is_main`=1 AND `contact_id`=".$contact->getId()." AND
										`email_type_id`='".array_var($contact_data,'email_type')."' AND TRIM(email_address) <> '' ");
							    } else {
							        // if mail didn't change check if is valid
							        if (!is_valid_email($contact_data['email'])) {
							            throw new Exception(lang('invalid email address'));
							        }
							    }
							}

							if($contact_data){
								$contact_controller = new ContactController();
								// save phones, addresses and webpages
								$contact_controller->save_phones_addresses_webpages($contact_data, $contact);
								// save additional emails
								$contact_controller->save_non_main_emails($contact_data, $contact);
							}
							
							// save instant messenger values
							$im_types = ImTypes::instance()->findAll(array('order' => '`id`'));
							$contact->clearImValues();
							foreach($im_types as $im_type) {
								$value = trim(array_var($contact_data, 'im_' . $im_type->getId()));
								if($value <> '') {
							
									$contact_im_value = new ContactImValue();
							
									$contact_im_value->setContactId($contact->getId());
									$contact_im_value->setImTypeId($im_type->getId());
									$contact_im_value->setValue($value);
									$contact_im_value->setIsMain(array_var($contact_data, 'default_im') == $im_type->getId());
							
									$contact_im_value->save();
								} // if
							}

							if ($is_api){
                                $obj_custom_properties = array_var($data, 'object_custom_properties');
                                $object_controller->add_custom_properties($contact, $obj_custom_properties);
                            }else{
                                $object_controller->add_custom_properties($contact, array_var($_POST, 'object_custom_properties'));
                            }

							// unclassify associated contact object from previous members
							$contact->removeFromAllMembers(logged_user(), array());
							// add the associated contact to this customer member
							$contact->addToMembers(array($member), true);
							// add the associated contact to the customer's related members
							$contact->addToRelatedMembers(array($member), false, true);
							
							$contact->addToSharingTable();
							
							$customer->setContactId($contact->getId());
							
							$customer->setContactObjectTypeId($contact_data_type);
							$customer->save();
							break;
						case "company":
							$company_data = array_var($_POST, 'company',false);
							
							$existing_contact = null;
							
							if (!$isNew && $customer->getContactId() > 0) {
								$existing_contact = Contacts::instance()->findById($customer->getContactId());
							}

							if (!$existing_contact && isset($contact_email) && trim($contact_email) != "") {
								$contact_emails = ContactEmails::instance()->findAll(array('conditions' => array("`email_address` = ?", $contact_email)));
								foreach ($contact_emails as $ce) {
									if (can_read_sharing_table(logged_user(), $ce->getContactId())) {
										$existing_contact = Contacts::instance()->findById($ce->getContactId());
										break;
									}
								}
							
							}
							
							$new_company = false;
							if ($existing_contact instanceof Contact && $existing_contact->getIsCompany() && can_read_sharing_table(logged_user(), $existing_contact->getId())) {
								$customer->setContactId($existing_contact->getId());
								$company = $existing_contact;
								
							} else if ($existing_contact instanceof Contact && can_read_sharing_table(logged_user(), $existing_contact->getId())) {
								// if existing contact is a contact change its type
								$customer->setContactId($existing_contact->getId());
								$company = $existing_contact;
								$company->setIsCompany(true);
								
							} else {
								$company = new Contact();
								$new_company = true;
							}
							
							if($company_data){
								$company->setFromAttributes($company_data);
							}
							$company->setFirstName($name);
							$company->setSurname("");
							$company->setObjectName();
							$company->setIsCompany(true);
							
							if (isset($_SESSION['new_contact_picture']) && $_SESSION['new_contact_picture']) {
								$company->setPictureFile($_SESSION['new_contact_picture']);
								$company->setPictureFileMedium($_SESSION['new_contact_picture_medium'], 'image/png');
								$company->setPictureFileSmall($_SESSION['new_contact_picture_small'], 'image/png');
								$_SESSION['new_contact_picture'] = null;
								$_SESSION['new_contact_picture_medium'] = null;
								$_SESSION['new_contact_picture_small'] = null;
							}
							
							$company->add_skip_validation('email');
							$company->save();
							$type_email = 'work';

							if(array_var($company_data,'email') != "") {
							    //$c_email = $company->getEmail();
							    $c_email = ContactEmails::instance()->findOne(array('conditions' => array("`contact_id` = ? AND `email_type_id` = ? AND TRIM(email_address) <> '' ",
							    		$company->getId(), $company_data['email_type'])));

							    if ($c_email instanceof ContactEmail && $c_email->getEmailAddress() != $company_data['email']) {
									$c_email->setEmailAddress($company_data['email']);
									$c_email->setEmailTypeId($company_data['email_type']);
									$c_email->save();
									
							    }elseif(!$c_email ){
							         $email_types = EmailTypes::getAllEmailTypesInfo();							         
							         foreach($email_types as $email){
							             if ($email['id'] == $company_data['email_type']){
							                 $type_email = $email['code'];
							                 break;
							             }
							         }
							         $company->addEmail($company_data['email'], $type_email, true);							        
							    }
							} else {
								// delete email if no input
								if (array_var($company_data,'email') == "") {
									ContactEmails::instance()->delete("`is_main`=1 AND `contact_id`=".$company->getId()." AND 
										`email_type_id`='".array_var($company_data,'email_type')."' AND TRIM(email_address) <> '' ");
								} else {
									// if mail didn't change check if is valid
									if (!is_valid_email($company_data['email'])) {
										throw new Exception(lang('invalid email address'));
									}
								}
							}
							
							if($company_data){
								$contact_controller = new ContactController();
								$contact_controller->save_phones_addresses_webpages($company_data, $company);
								$contact_controller->save_non_main_emails($company_data, $company);
							}


							if ($is_api){
								$obj_custom_properties = array_var($data, 'object_custom_properties');
								$object_controller->add_custom_properties($company, $obj_custom_properties);
							}else{
								$object_controller->add_custom_properties($company, array_var($_POST, 'object_custom_properties'));
							}
							//$object_controller->add_custom_properties($company);

							// unclassify associated contact object from previous members
							$company->removeFromAllMembers(logged_user(), array());
							// add the associated contact to this customer member
							$company->addToMembers(array($member), true);
							// add the associated contact to the customer's related members
							$company->addToRelatedMembers(array($member), false, true);
							
							$company->addToSharingTable();
							
							$customer->setContactId($company->getId());
							
							$customer->setContactObjectTypeId($contact_data_type);
							$customer->save();
							break;
					}
				} else {
					// "unknown" contact type and existing contact or company is selected
					if (array_var($data, 'existing_contact_id') > 0) {
						$e_contact = Contacts::instance()->findById(array_var($data, 'existing_contact_id'));
						if ($e_contact instanceof Contact) {
							$e_contact->addToMembers(array($member), true);
							$customer->setContactId($e_contact->getId());
							if ($e_contact->getIsCompany()) {
								$e_contact_ot = ObjectTypes::findByName('company')->getId();
							} else {
								$e_contact_ot = $e_contact->getObjectTypeId();
							}
							
							$customer->setContactObjectTypeId($e_contact_ot);
							$customer->save();
						}
					} else {
						$customer->setContactId(0);
						$customer->setContactObjectTypeId(0);
						$customer->save();
					}
				}


			// add custom properties
			if (Plugins::instance()->isActivePlugin('member_custom_properties')) {
				$mcp_controller = new MemberCustomPropertiesController();
				$mcp_controller->add_custom_properties($member, $customer,$all_data_api);
			}
			
			// build name to display, based on properties and relations, we do it here after saving everything needed
			$display_name = build_member_display_name($member);
			$member->setDisplayName($display_name);
			$member->save();

			if ($action == ApplicationLogs::ACTION_EDIT) {
				evt_add("update dimension tree node", array('dim_id' => $member->getDimensionId(), 'member_id' => $member->getId()));
			}elseif ($action == ApplicationLogs::ACTION_ADD) {
				$select_node = intval(DimensionObjectTypeOptions::getOptionValue($member->getDimensionId(), $member->getObjectTypeId(), 'select_after_creation'));
				if ($select_node) {
					evt_add("external dimension member click", array('dim_id' => $member->getDimensionId(),'member_id' => $member->getId()));
				} else {
					evt_add("update dimension tree node", array('dim_id' => $member->getDimensionId(), 'member_id' => $member->getId(), 'select_node' => $select_node));
				}
			}else{
				evt_add("external dimension member click", array('dim_id' => $member->getDimensionId(),'member_id' => $member->getId()));
			}
			//calculate_customers_additional_data(array('customer_id' => $customer->getId()));
			
			DB::commit();
			ApplicationLogs::createLog($member, $action);

			
			// save default permissions if request came from api
			if ($is_api && config_option('add_default_permissions_for_users')) {
				if ($member->getParentMemberId() == 0) {
					// if added from quick-add add default permissions for executives, managers and administrators
					$user_types = implode(',', config_option('give_member_permissions_to_new_users'));
					if (trim($user_types) != "") {
						$users = Contacts::instance()->findAll(array('conditions' => "user_type IN (".$user_types.")"));
						
						if (!array_var($_REQUEST, 'permissions')) $_REQUEST['permissions'] = "[]";
						$permissions_decoded = json_decode(array_var($_REQUEST, 'permissions'));
						foreach ($users as $user) {
							$role_perms = RoleObjectTypePermissions::instance()->findAll(array('conditions' => array("role_id=?", $user->getUserType())));
							foreach ($role_perms as $role_perm) {
								$pg_obj = new stdClass();
								$pg_obj->pg = $user->getPermissionGroupId();
								$pg_obj->o = $role_perm->getObjectTypeId();
								$pg_obj->d = $role_perm->getCanDelete();
								$pg_obj->w = $role_perm->getCanWrite();
								$pg_obj->r = 1;
								$permissions_decoded[] = $pg_obj;
							}
						}
						$_REQUEST['permissions'] = json_encode($permissions_decoded);
					}
				} else {
					// inherit permissions from parent member
					if ($member->getParentMemberId() > 0) {
						$perm_params = get_default_member_permission($member->getParentMemberId(), array());
						if (is_array($perm_params) && is_array(array_var($perm_params, 'member_permissions'))) {
							$mem_perms = array_var($perm_params, 'member_permissions');
							$permissions_decoded = array();
							foreach ($mem_perms as $pg_id => $perms) {
								foreach ($perms as $perm) {
									$pg_obj = new stdClass();
									$pg_obj->pg = $pg_id;
									$pg_obj->o = array_var($perm, 'o');
									$pg_obj->d = array_var($perm, 'd');
									$pg_obj->w = array_var($perm, 'w');
									$pg_obj->r = array_var($perm, 'r');
									$permissions_decoded[] = $pg_obj;
								}
							}
							$_REQUEST['permissions'] = json_encode($permissions_decoded);
						}
					}
				}
			}
			// end forced permissions

			// Save permissions
			Env::useHelper('permissions');
			save_member_permissions_background(logged_user(), $member, array_var($_REQUEST, 'permissions'), $old_parent);

            $ret = null;
            Hook::fire('after_member_save_and_commit', array('member' => $member, 'is_new' => $isNew), $ret);
			
			if (isset($old_parent) && $old_parent != $member->getParentMemberId()) {					
				Env::useHelper('dimension');
				update_all_childs_depths($member, $old_parent);
				if (!$isNew){
				    evt_add('member parent changed', array('m' => $member->getId(), 'p' => $member->getParentMemberId(), 'op' => $old_parent, 'd' => $member->getDimensionId()));
				}
				
			}

            if(!$is_api){
                flash_success(lang('success '.($isNew?'add':'edit').' customer', $customer->getName()));
                if (array_var($_POST, 'popup', false)) {
                    ajx_current("reload");
                } else {
                    ajx_current("back");
                }
            }else{
                return array('status'=>true,'message'=>'success add','member_id'=>$member->getId());
            }

			return $member;
			
		}catch (Exception $e) {
		    DB::rollback();
            if(!$is_api){
                Logger::log_r("ERROR creating client: ".$e->getMessage()."\n".$e->getTraceAsString());
                flash_error($e->getMessage());
                mark_dao_validation_error_fields($e);
                ajx_current("empty");
            }else{
                return array('status'=>false,'message'=>$e->getMessage());
            }
		}
	}
	
	private function populateComboboxes($customer = null) {
			$contacts =  array(array("","-")) ;
			$conditions = $this->base_conditions ;
			$conditions['conditions'] .= "  AND  is_company = 0 ";
			$conditions['order'] = "name"; 
			
			/*if ($customer){
				$inherit_contacts = $customer->getInheritContactIds();	
			}
			foreach (Contacts::instance()->findAll($conditions) as $contact) {
				if ($customer && in_array($contact->getId() , $inherit_contacts)){
					continue ;
				}
				
				$text  = $contact->getObjectName() ;
				$value = $contact->getId() ;
				$contacts[] =  array ($value, $text) ;
			} */
			$status =  array(array(0,"-")) ;
			foreach ( CustomerStatusManager::instance()->findAll() as $status_obj) {
				$text  = lang($status_obj->getName()) ;
				$value = $status_obj->getId() ;
				$status[] =  array ($value, $text) ;
			}
			$feedbacks =  array(array(0,"-")) ;
			foreach ( CustomerFeedbacks::instance()->findAll() as $object ){
				$text  = $object->getName() ;
				$value = $object->getId() ;
				$feedbacks[] =  array ($value, $text) ;
			}
			$payment_cycles = array(array(0,"-")) ;
			foreach ( CustomerPaymentCycles::instance()->findAll() as $object ){
				$text  = lang($object->getName()) ;
				$value = $object->getId() ;
				$payment_cycles[] =  array ($value, $text) ;
			}
			
			$create_contact_info_options = config_option('create_contact_info_for_client');
			$contact_types = array();
			if (in_array('unknown', $create_contact_info_options)) { 
				$contact_types[] = array (0, lang("no contact data"));
			}
			if (in_array('contact', $create_contact_info_options)) {
				$contact_types[] = array (ObjectTypes::findByName("contact")->getId(), lang("contact"));
			}
			if (in_array('company', $create_contact_info_options)) {
				$contact_types[] = array (ObjectTypes::findByName("company")->getId(), lang("company"));
			}
			
			$service_types 		=  array(array(0,"-")) ;
			foreach (CustomerServiceTypes::instance()->findAll() as $object){
				$text  = $object->getName() ;
				$value = $object->getId() ;
				$service_types[] =  array ($value, $text) ;
			}
			
			$combos =  array(
				"contacts" => $contacts,
				"status" => $status,
				"feedbacks" => $feedbacks,
				"payment_cycles" =>$payment_cycles,
				"contact_types" =>$contact_types,
				"service_types" => $service_types,
			);
			
			tpl_assign("contacts", $combos['contacts']);
			tpl_assign("status", $combos['status']);
			tpl_assign("feedbacks", $combos['feedbacks']);
			tpl_assign("payment_cycles", $combos['payment_cycles']);
			tpl_assign("contact_types", $combos['contact_types']);
			tpl_assign("service_types", $combos['service_types']);
			
	}
	
	private function generateUserNameFromCustomer($contact) {
		$uname = "";
		if ($contact->getSurname() == "") {
			$uname = $contact->getFirstName();
		} else if ($contact->getFirstname() == "") {
			$uname = $contact->getSurname();
		} else {
			$uname = substr_utf($contact->getFirstname(), 0, 1) . $contact->getSurname();
		}
		$uname = strtolower(trim(str_replace(" ", "", $uname)));
		if ($uname == "") {
			$uname = strtolower(str_replace(" ", "_", lang("new user")));
		}
		$base = $uname;
		for ($i=2; Customers::getByUsername($uname) instanceof Customer; $i++) {
			$uname = $base . $i;
		}
		return $uname;
	}
	
	
	
	
	/**
	 * Delete specific contact
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function delete() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		$customer = Customers::instance()->findById(get_id());
		if(!($customer instanceof Customer)) {
			flash_error(lang('contact dnx'));
			ajx_current("empty");
			return;
		} // if

		if(!$customer->canDelete(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		} // if
		
		$member = Members::findOneByObjectId($customer->getId(), $this->dimension->getId());
		$ret = array();
		Hook::fire('check_additional_member_permissions', array('action' => 'delete', 'member' => $member, 'pg_ids' => logged_user()->getPermissionGroupIds()), $ret);
		if (count($ret) > 0 && !array_var($ret, 'ok')) {
			flash_error(array_var($ret, 'message'));
			ajx_current("empty");
			return;
		}

		try {
			DB::beginWork();
			$customer->trash();
			DB::commit();
			ApplicationLogs::createLog($customer, ApplicationLogs::ACTION_TRASH );
			flash_success(lang('success delete contact', $customer->getObjectName()));
			ajx_current("back");
		} catch(Exception $e) {
			DB::rollback();
			flash_error(lang('error delete contact'));
			ajx_current("empty");
		} // try
	} // delete
	
	/**
	 * @param Customer $customer
	 */
	private function getUnassignedContacts(Customer $customer) {
		$customer_contacts = $customer->getContactIds(true);
		$conditions = $this->base_conditions ;
		$conditions['conditions'] .= "  AND  is_company = 0 ";
		$conditions['order'] = "name"; 
		$contacts = array();
		foreach (Contacts::instance()->findAll( $conditions ) as $contact) {
			/* @var $contact Contact */
			if (!in_array($contact->getId(), $customer_contacts) ) {
				$text  = $contact->getObjectName() ;
				$value = $contact->getId() ;
				$contacts[] =  array ($value, $text) ;
			}
		} 
		return $contacts;
	}
	
}