<?php 
class ObjectPrefixesController extends ApplicationController {
	
	var $plugin_name = "crpm";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
	}
		
	function index() {
		
	}
	
	function save_object_prefix_settings() {
		if (!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
	
		try {
			DB::beginWork();
			
			$data = array_var($_POST, 'op_settings');
			if (is_array($data)) {
				foreach ($data as $object_type_id => $op_data) {
					$op_setting = ObjectPrefixSettings::instance()->findOne(array("conditions" => "object_type_id = $object_type_id"));
					if (!$op_setting instanceof ObjectPrefixSetting) {
						continue;
					}
					
					$op_setting->setMaxDigits(array_var($op_data, 'max_digits'));
					if (logged_user()->isAdministrator() && isset($op_data['current_value'])) {
						// allow superadmin to change current prefix value
						if (!is_numeric(array_var($op_data, 'current_value'))) {
							$ot = ObjectTypes::instance()->findById($object_type_id);
							$ot_name = $ot instanceof ObjectType ? lang($ot->getName()) : lang('object');
							throw new Exception(lang('prefix current value must be a number', $ot_name));
						}
						$op_setting->setCurrentValue(array_var($op_data, 'current_value'));
					}
					$op_setting->save();
				}
				ajx_current("back");
			} else {
				ajx_current("emtpy");				
			}
			
			DB::commit();
			flash_success(lang('success saving object prefix configuration'));
			
		} catch (Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			ajx_current('empty');
		}
	}
	
	function list_object_prefix_settings() {
		if (!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$object_types_with_prefix = config_option("use_autonumeric_prefixes");
		$op_settings = array();
		
		foreach ($object_types_with_prefix as $object_type_id) {
			$ot = ObjectTypes::instance()->findById($object_type_id);
			if (!$ot instanceof ObjectType) continue;
			
			$op_setting = ObjectPrefixSettings::instance()->findOne(array("conditions" => "object_type_id = $object_type_id"));
			if (!$op_setting instanceof ObjectPrefixSetting) {
				if (!$op_setting instanceof ObjectPrefixSetting) {
					$op_setting = new ObjectPrefixSetting();
					$op_setting->setObjectTypeId($object_type_id);
					$op_setting->setCurrentValue(0);
					$op_setting->setEnabled(true);
					$op_setting->setMaxDigits(5);
					$op_setting->save();
				}
			}
			
			$op_settings[$object_type_id] = $op_setting->getArrayInfo();
			$op_settings[$object_type_id]['object_type_name'] = lang($ot->getName());
		}
		
		tpl_assign('op_settings', $op_settings);
	}
	
	
}