<?php 
class StatusFormulasController extends ApplicationController {
	
	var $plugin_name = "crpm";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('status_formulas', $this->plugin_name);
	}
		
	function index() {
		
	}
	
	
	function edit() {
		if (!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$status_dimension = Dimensions::findByCode("project_statuses");
		$use_project_status_dimension = $status_dimension instanceof Dimension && in_array($status_dimension->getId(), config_option('enabled_dimensions'));
		
		$project_ot = ObjectTypes::findByName('project');
		$client_ot = ObjectTypes::findByName('customer');
		
		$data = array_var($_POST, 'formula');
		
		if (!is_array($data)) {
			// Prepare form
			$is_default = array_var($_REQUEST, 'is_default');
			$object_type_id = array_var($_REQUEST, 'object_type_id');
			
			$is_new = false;
			$actual_statuses = array();
			
			$status = array();
			if ($object_type_id == $project_ot->getId()) {
				
				if ($use_project_status_dimension) {
					$status_members = $status_dimension->getAllMembers(false, '`order` desc');
					foreach ($status_members as $sm) $status[$sm->getId()] = array($sm->getId(), $sm->getName());
				} else {
					for ($i=7; $i>0; $i--) $status[$i] = array($i, lang('project_field_status'.$i));
				}
				
			} else if ($object_type_id == $client_ot->getId()) {
				foreach ( CustomerStatusManager::instance()->findAll(array('order' => 'id DESC')) as $status_obj) {
					$text = lang($status_obj->getName());
					$value = $status_obj->getId();
					$status[$value] = array ($value, $text);
				}
			}
			
			if ($is_default) {
				$formula = Formulas::instance()->findOne(array('conditions' => array('is_default = 1 AND object_type_id = ?', $object_type_id)));
				if (!$formula instanceof Formula) {
					$is_new = true;
					$formula = new Formula();
					$formula->setObjectTypeId($object_type_id);
					$formula->setIsDefault(true);
					$formula->setName('Default formula');
				} else {
					$formula->loadFactorsGroups();
					$factors_groups = $formula->getFactorsGroups();
					foreach ($factors_groups as $f) {
						$f->loadFactors();
						$actual_statuses[$f->getStatusId()][] = array('text' => $status[$f->getStatusId()][1], 'groupid' => $f->getGroupId(), 'factors' => $f->getFactors());
					}					
				}
			} else {
				// TODO: when adding specific formulas to projects or clients, check for formula_id
			}
			
			tpl_assign('status', $status);
			tpl_assign('actual_statuses', $actual_statuses);
			tpl_assign('formula', $formula);
			
		} else {
			try {
				DB::beginWork();
				// Save the formula
				$is_new = false;
				$formula = Formulas::instance()->findById(array_var($data, 'id'));
				if (!$formula instanceof Formula) {
					$formula = new Formula();
					$is_new = true;
				}
				$formula->setFromAttributes($data);
				$formula->save();
				
				// Save the formula conditions
				$all_factors = array_var($_POST, 'factor');
				
				$factors_groups_ids = array();
				foreach ($all_factors as $status_id => $status_factors_groups) {
					foreach ($status_factors_groups as $factor_group_id => $factor_group_factors) {
						$group = FormulaFactorsGroups::instance()->findById($factor_group_id);
						if (!$group instanceof FormulaFactorsGroup) {
							$group = new FormulaFactorsGroup();
							$group->setFormulaId($formula->getId());
							$group->setStatusId($status_id);
							$group->save();
							$factor_group_id = $group->getGroupId();							
						}else{
							$group->deleteAllFactors();
						}
						$factors_groups_ids[] = $factor_group_id;
						
						foreach ($factor_group_factors as $factor_name => $factor_data) {
							$factor = new FormulaFactor();
							$factor->setGroupId($factor_group_id);							
							$factor->setName($factor_name);
							$factor->setEnabled(!is_null(array_var($factor_data, 'enabled')));
							$factor->setComparissonType(array_var($factor_data, 'comparisson_type'));
							$factor->setRequiredValue(array_var($factor_data, 'required_value'));
							
							$factor->save();
						}
					}					
				}
				
				//delete				
				$formula->loadFactorsGroups();
				$factors_groups = $formula->getFactorsGroups();
				foreach ($factors_groups as $f) {
					if(!in_array($f->getGroupId(), $factors_groups_ids)){						
						$f->delete();
					}
				}
				
				if (config_option('automatic_crpm_status_calculation')) {
					calculate_all_members_status_for_dim($formula->getObjectTypeId());
				}
				
				flash_success(lang('formula saved successfully'));
				ajx_current("back");
				DB::commit();
			} catch (Exception $e) {
				flash_error($e->getMessage());
				ajx_current("empty");
				DB::rollback();
			}
		}
	}
	
}