<?php

function get_customers_dimension() {
	$client_dim = Dimensions::findByCode("customers");
	if ($client_dim instanceof Dimension && in_array($client_dim->getId(), config_option('enabled_dimensions'))) {
		return $client_dim;
	} else {
		return Dimensions::findByCode('customer_project');
	}
}


function get_client_member_of_project($member, $dim_association_code=null) {
	$client_member = null;
	
	$customers_dim = get_customers_dimension();
	$projects_dim_id = $member->getDimensionId();
	$client_ot = ObjectTypes::findByName('customer');
	
	if ($customers_dim->getId() == $projects_dim_id) {
		foreach ($member->getAllParentMembersInHierarchy(false, false) as $parent) {
			if ($parent->getObjectTypeId() == $client_ot->getId()) {
				$client_member = $parent;
				break;
			}
		}
	} else {
		
		if (is_null($dim_association_code)) $dim_association_code = 'project_customer';
		
		$client_member = Members::getMemberById(get_associated_status_member_id($member, $customers_dim, null, false, $dim_association_code));
		if (!$client_member) {
			$parent = $member->getParentMember();
			if ($parent && $parent->getObjectTypeId() == $member->getObjectTypeId()) {
				$client_member = get_client_member_of_project($parent);
			}
		}
	}
	
	return $client_member;
}


function create_client_member_by_email_and_name($client_data) {
	
	if (!array_var($client_data,'customer_name')) return;

	Env::useController('customer');

	$contact_ot_id = ObjectTypes::findByName('contact')->getId();
	$client_ot_id = ObjectTypes::findByName('customer')->getId();
	
	// check if member already exists with that name, to avoid errors when saving client member
	$existing_member = Members::instance()->findOne(array("conditions" => array("name=? AND object_type_id=?", $client_data['customer_name'], $client_ot_id)));
	if ($existing_member instanceof Member) {
		return $existing_member;
	}

	// base data
	$data['customer'] = array(
			'contact_data_type' => $contact_ot_id,
			'name' => $client_data['customer_name'],
			'status_id' => 6,
			'parent' => 0,
			'automatic_status' => '1',
			'customer_name' => $client_data['customer_name'],
			'customer_address' => '',
			'customer_rut' => '',
			'customer_number' => '',
	);
	$data['contact'] = array(
			'email' => $client_data['customer_email_address'],
	);

	// custom properties
	$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType($client_ot_id);
	foreach ($custom_properties as $cp) {
		if (isset($client_data[$cp->getCode()])) {
			$data['object_custom_properties'][$cp->getId()] = $client_data[$cp->getCode()];
		}
	}

	// save the client
	$ccontroller = new CustomerController();

	$customer = new Customer();
	$_POST['contact'] = $data['contact'];
	$member = $ccontroller->handleSubmit($data['customer'], $customer);

	// return the new client member
	return $member;
}


function find_client_member_by_email($email_address) {
	$member = null;
	
	$contact_exists_cond = " AND EXISTS (SELECT o.id FROM ".TABLE_PREFIX."objects o WHERE o.id=contact_id AND o.trashed_by_id=0)";

	$contact_email = ContactEmails::instance()->findOne(array("conditions" => array("email_address=? $contact_exists_cond", $email_address)));
	if ($contact_email instanceof ContactEmail) {

		$contact = $contact_email->getContact();
		if ($contact instanceof Contact) {
				
			$customer = Customers::instance()->findOne(array("conditions" => array("contact_id=?", $contact->getId())));
			if ($customer instanceof Customer) {
				$member = Members::findOneByObjectId($customer->getId(), get_customers_dimension()->getId());
				
			} else {
				// get the client where the contact is classified
				$contact_members = $contact->getMembers();
				
				$cli_ot = Customers::instance()->getObjectTypeId();
				$crpm_dim = Dimensions::findByCode('customer_project');
				$mem_object_type_ids = DimensionObjectTypes::getObjectTypeIdsByDimension($crpm_dim->getId());
				
				foreach ($contact_members as $contact_member) {
					if ($contact_member instanceof Member) {
						if ($contact_member->getObjectTypeId() == $cli_ot) {
								
							return $contact_member;
							
						} else if (in_array($contact_member->getObjectTypeId(), $mem_object_type_ids)) {
								
							$all_parents = $contact_member->getAllParentMembersInHierarchy();
							foreach ($all_parents as $parent) {
								if ($parent->getObjectTypeId() == $cli_ot) {
									return $parent;
								}
							}
						}
					}
				}
			}
		}
	}

	return $member;
}


function generate_first_repetitions_for_task(ProjectTask $task) {

	$count = 0;
	$task = ProjectTasks::instance()->findById($task->getObjectId());
	//$generate_repetitive_instances = $task->getcolumnValue('repeat_generate_instances');
	$generate_repetitive_instances = config_option('generate repetitive instances');
	if ($task instanceof ProjectTask && $generate_repetitive_instances) {
		$days = config_option("days range to replicate tasks");
		if ($days > 0) {
				
			$until = DateTimeValueLib::now()->add('d', $days);
				
			// don't repeat more times than the repeat_end date, if it is specified
			$repeat_end = $task->getRepeatEnd();
			if ($repeat_end instanceof DateTimeValue && $repeat_end->getTimestamp() < $until->getTimestamp()) {
				$until = null; // to avoid forcing the repeat_end date, let the controller calculate it.
			}
				
			$opt_rep_day = array('saturday' => false, 'sunday' => false);
				
			$task_controller = new TaskController();
				
			$orig_st_date = $task->getStartDate();
			$orig_due_date = $task->getDueDate();

			$count += $task_controller->repetitive_task($task, $opt_rep_day, $until);

			$task->setStartDate($orig_st_date);
			$task->setDueDate($orig_due_date);
			$task->save();
				
		}
	}
	return $count;
}
