<?php

/**
 * Calculates the additional data to show in projects listing
 */
function calculate_projects_additional_data($params) {
	$projects_dim = Dimensions::findByCode('customer_project');
	$project_ot = ObjectTypes::findByName('project');
	$projects = array();
	
	if (array_var($params, 'project_id')) {
		
		$project = Projects::instance()->findById(array_var($params, 'project_id'));
		if ($project instanceof Project) $projects[] = $project;
	
	} else if (array_var($params, 'member_id')) {
		
		$member = Members::instance()->findById(array_var($params, 'member_id'));
		if ($member instanceof Member) {
			$project = Projects::instance()->findById($member->getObjectId());
			if ($project instanceof Project) $projects[] = $project;
		}
		
	} else if (array_var($params, 'task_id')) {
		
		$task = ProjectTasks::instance()->findById(array_var($params, 'task_id'));
		if ($task instanceof ProjectTask) {
			
			$all_members = $task->getMembers();
			$task_members = array();
			foreach ($all_members as $mem) {
				if ($mem->getDimensionId() == $projects_dim->getId()) {
					$task_members = array_merge($task_members, $mem->getAllParentMembersInHierarchy(true));
				}
			}
			
			foreach ($task_members as $tm) {
				if ($tm->getObjectTypeId() == $project_ot->getId()) {
					$project = Projects::instance()->findById($tm->getObjectId());
					if ($project instanceof Project) $projects[] = $project;
				}
			}
		}
		
	} else if (array_var($params, 'multiple_task_ids')) {
		$ids = array_var($params, 'multiple_task_ids');
		if (count($ids) > 0) {
			$tasks = ProjectTasks::instance()->findAll(array("conditions" => "id IN (".implode(',', $ids).")"));
			foreach ($tasks as $task) {
			
				$all_members = $task->getMembers();
				$task_members = array();
				foreach ($all_members as $mem) {
					if ($mem->getDimensionId() == $projects_dim->getId()) {
						$task_members = array_merge($task_members, $mem->getAllParentMembersInHierarchy(true));
					}
				}
				
				foreach ($task_members as $tm) {
					if ($tm->getObjectTypeId() == $project_ot->getId() && !isset($projects[$tm->getObjectId()])) {
						$project = Projects::instance()->findById($tm->getObjectId());
						if ($project instanceof Project) $projects[$tm->getObjectId()] = $project;
					}
				}
			}
		}
		
	}
	
	foreach ($projects as $project) {
		calculate_single_project_additional_data($project);
	}
	
}

/**
 * Calculates the additional for a single project
 */
function calculate_single_project_additional_data(Project $project) {
	
	$dimension = Dimensions::findByCode('customer_project');
	$project_member = Members::findOneByObjectId($project->getId(), $dimension->getId());
	if (!$project_member instanceof Member) return;
	
	DB::execute("DELETE FROM ".TABLE_PREFIX."project_additional_data WHERE project_id=".$project->getId());
	
	$all_tasks = ProjectTasks::instance()->findAll(array(
		'join' => array('table' => TABLE_PREFIX.'object_members', 'e_field' => 'object_id', 'jt_field' => 'object_id'),
		'conditions' => "jt.member_id = ".$project_member->getId()." AND trashed_by_id=0 ",
	));
	
	$completed_num = 0;
	$total_estimated_time = 0;
	$total_worked_time = 0;
	foreach ($all_tasks as $t) {
		/* @var $t ProjectTask */
		if ($t->getCompletedById() > 0) $completed_num++;
		$total_estimated_time += $t->getTimeEstimate();
		$total_worked_time += $t->getTotalMinutes();
	}
	
	if ($total_estimated_time > 0) {
		$time_worked_p = $total_worked_time * 100 / $total_estimated_time;
	} else {
		$time_worked_p = 0;
	}
	
	$client = $project->getCustomer();
	$data = array(
		'project_id' => $project->getId(),
		'calc_customer_id' => $client instanceof Customer ? $client->getId() : 0,
		'total_tasks' => count($all_tasks),
		'completed_tasks' => $completed_num,
		'task_completion_p' => count($all_tasks) > 0 ? $completed_num * 100 / count($all_tasks) : 0,
		'total_estimated_time' => $total_estimated_time,
		'total_worked_time' => $total_worked_time,
		'time_worked_p' => $time_worked_p,
	);
	
	$columns = array_keys($data);
	$sql = "INSERT INTO ".TABLE_PREFIX."project_additional_data (". implode(',', $columns) .") VALUES ('". implode("','", $data) ."');";
	DB::execute($sql);
}
