<?php

/**
 * Customer class
 *
 * 
 */
class Customer extends BaseCustomer {
	
	protected $searchable_columns = array('name');
	
	protected $is_searchable = true;
	
	
	protected $is_read_markable = false;
	
	private $contact = null ;

	private $status = null ;
	
	private $service_type = null ;
	
	private $payment_cycle = null ;
	
	private $feedback = null ;
	
	private $contacts = null ;
	
	private $contacts_ids = null ;
	
	private $projects = null ;
	
	
	function getPicture() {
		return  $this->getContact()->getPictureUrl();
	}
	
	function __construct() {
		parent::__construct();
		Env::useHelper('functions', 'crpm');
	}
	
	// start_date
	function getStartDate() {
		if (Plugins::instance()->isActivePlugin('member_custom_properties')) {
			$cp_val = null;
			$cp = MemberCustomProperties::instance()->findOne(array('conditions' => "code='start_date' AND object_type_id=".$this->getObjectTypeId()));
			if ($cp instanceof MemberCustomProperty) {
				$member = Members::findOneByObjectId($this->getId(), get_customers_dimension()->getId());
				if ($member instanceof Member) {
					$cp_val =  MemberCustomPropertyValues::getMemberCustomPropertyValue($member->getId(), $cp->getId());
				}
			}
			if ($cp_val instanceof MemberCustomPropertyValue && $cp_val->getValue() != EMPTY_DATETIME) {
				return DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $cp_val->getValue());
			} else {
				return null;
			}
		}
		return $this->getColumnValue('start_date');
	}
	
	// end_date
	function getEndDate() {
		if (Plugins::instance()->isActivePlugin('member_custom_properties')) {
			$cp_val = null;
			$cp = MemberCustomProperties::instance()->findOne(array('conditions' => "code='end_date' AND object_type_id=".$this->getObjectTypeId()));
			if ($cp instanceof MemberCustomProperty) {
				$member = Members::findOneByObjectId($this->getId(), get_customers_dimension()->getId());
				if ($member instanceof Member) {
					$cp_val =  MemberCustomPropertyValues::getMemberCustomPropertyValue($member->getId(), $cp->getId());
				}
			}
			if ($cp_val instanceof MemberCustomPropertyValue && $cp_val->getValue() != EMPTY_DATETIME) {
				return DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $cp_val->getValue());
			} else {
				return null;
			}
		}
		return $this->getColumnValue('end_date');
	}
	
	
	
	/**
    * Return formatted value of 'CMRR' field
    *
    * @access public
    * @param void
    * @return string 
    */
    function getCmrr() {
    	if (Plugins::instance()->isActivePlugin('member_custom_properties')) {
    		$cp_val = null;
    		$cp = MemberCustomProperties::instance()->findOne(array('conditions' => "code='cmrr' AND object_type_id=".$this->getObjectTypeId()));
    		if ($cp instanceof MemberCustomProperty) {
    			$member = Members::findOneByObjectId($this->getId(), get_customers_dimension()->getId());
    			if ($member instanceof Member) {
    				$cp_val =  MemberCustomPropertyValues::getMemberCustomPropertyValue($member->getId(), $cp->getId());
    			}
    		}
    		if ($cp_val instanceof MemberCustomPropertyValue && $cp_val->getValue() != EMPTY_DATETIME) {
    			return $cp_val->getValue();
    		} else {
    			return null;
    		}
    	}
    	return $this->getColumnValue('cmrr');
    } // getCMRR()
	
	/**
	 * Construct customer object
	 *
	 * @param void
	 * @return Contact
	 */
	
	function getContact(){
    	if (!$this->contact) {
    		$this->contact = Contacts::instance()->findById($this->getContactId());
    	}
    	return $this->contact ;
	}
	
	/**
	 * @return Contact
	 */
	function getMainContact() {
		return $this->getContact();
	}
	
	/**
	 * 
	 * @param $depth Boolean: i
	 */
	function getContacts($depth = false) {
		$contactIds = $this->getContactIds($depth);
		if (!count($contactIds)) {
			$contactIds = array(0);
		} 
		$contacts =  Contacts::instance()->findAll(array("conditions" => "object_id IN (" .implode(",", $contactIds).")", "order" => "name ASC"));
		/*if ($depth) {
			$contacts = array_merge($contacts, $this->getInheritContacts());
		}*/
		return feng_sort($contacts,"getName", "getId", true ) ;	
	}
	
	
	function getInheritContacts() {
		$contacts = array();
		foreach ( $this->getProjects() as $project ){ /* @var $project Project */
			$project_contacts = $project->getContacts();
			foreach ($project_contacts as $project_contact ){
				$project_contact->project = $project ;
			}
			$contacts = array_merge($contacts, $project_contacts );
		}
		return feng_sort($contacts, 'getName','getId', true) ;
	}
	
	function getInheritContactIds() {
		$contactIds = array();
		foreach ( $this->getProjects() as $project ){ /* @var $project Project */
			$project_contacts = $project->getContactIds() ;
			$contactIds = array_merge($contactIds, $project_contacts );
		}
		return array_unique($contactIds) ;
	}
	
	function getContactIds ($depth = false) {
		if (!is_array($this->contacts_ids )){
			$a_dim = Dimensions::findByCode('feng_persons');
			Env::useHelper('functions', 'crpm');
			$proj_dim = get_customers_dimension();
			
			$a_ot = ObjectTypes::findByName('person');
			$mids = Members::findByObjectId($this->getId());		
			$mid = $mids[0]->getId();
	
			$assoc = DimensionMemberAssociations::instance()->findOne(array('conditions' => array('`dimension_id`=? AND `object_type_id`=? AND `associated_dimension_id`=? AND `associated_object_type_id`=?', 
				$proj_dim->getId(), 
				Customers::instance()->getObjectTypeId(), 
				$a_dim->getId(), $a_ot->getId()))
			);
			if (!$assoc) return array();
	
				
			$sql = "
				SELECT distinct(object_id) FROM ".TABLE_PREFIX."members m WHERE id IN 
				(
					SELECT distinct(property_member_id) FROM ".TABLE_PREFIX."member_property_members 
					WHERE association_id =".$assoc->getId(). " AND member_id = ".$mid."
				)
				AND EXISTS
				(
					SELECT id FROM ".TABLE_PREFIX."objects WHERE id = object_id AND archived_on = '0000-00-00 00:00:00' AND trashed_on = '0000-00-00 00:00:00'
				)
				AND EXISTS 
				( 
					SELECT object_id FROM  ".TABLE_PREFIX."contacts c WHERE c.object_id = m.object_id AND c.disabled = 0 
				)";

			$res = DB::executeAll($sql);
			$ids = array();
			if (is_array($res)) {
				foreach ( $res as $row ) {
					$ids[] = $row['object_id'];
				}
			}
			if ($depth){
				foreach ($this->getProjects() as $project){
					$pcids = $project->getContactIds();
					$ids = array_merge($ids, $pcids);
				}
			}
			$this->contacts_ids = array_unique($ids) ;
		}
		return $this->contacts_ids ;
	}
	
	function getStatus(){
    	if (!$this->status) {
    		if (Plugins::instance()->isActivePlugin('member_custom_properties')) {
    			$cp = MemberCustomProperties::getCustomPropertyByCode(Customers::instance()->getObjectTypeId(), 'status_special');
    			$member = Members::findOneByObjectId($this->getId());
    			
    			if ($member instanceof Member && $cp instanceof MemberCustomProperty) {
    				$cpval = MemberCustomPropertyValues::getMemberCustomPropertyValue($member->getId(), $cp->getId());
	    			if ($cpval instanceof MemberCustomPropertyValue) {
	    				$this->status = CustomerStatusManager::instance()->findById($cpval->getValue());
	    			}
    			}
    		}
    		if (!$this->status) {
    			$this->status = CustomerStatusManager::instance()->findById($this->getStatusId());
    		}
    	}
    	return $this->status ;
	}
	
	function getStatusName() {
		$status = $this->getStatus() ;
		if ($status instanceof CustomerStatus) {
			$status_name = $status->getName();
		} else {
			$status_name = $status;
		}
		return $status_name;
	}
	
	function getFeedback() {
		if ($fid = $this->getFeedbackId() ) {
			if ( !$this->feedback ) {
				$this->feedback = CustomerFeedbacks::instance()->findById($fid);
			}
		}
		return $this->feedback ;
	}
	
	function getFeedbackName() {
		$fbk = $this->getFeedback() ;
		if ($fbk) return $fbk->getName();
		return '' ;
	}
	
	/**
	 * @return Customer
	 * 
	 */
	function getPaymentCycle() {
		$pcId = $this->getPaymentCycleId() ;
		if (!$pcId) return null ; 
		if (!$this->payment_cycle) {
			$this->payment_cycle = CustomerPaymentCycles::instance()->findById($pcId);
		}
		return $this->payment_cycle;
	}
	function getPaymentCycleName() {
		if (!$this->getPaymentCycle()) return null ;
		else return $this->getPaymentCycle()->getName();
	}	

	function getContactInfo() {
		$contact_info = array(
			"contacts" => $this->getContacts(),
			"main_contact_id" =>null 
		);
		if ($main_contact = $this->getContact()){
			$contact_info["main_contact_id"] = $main_contact->getId();
			$contact_info["main_contact"] = $main_contact;
		}
		/*
		if ($contacts = $this->getContacts());
		if (count($contacts)){
			foreach ($contacts as $contact) {
				// @var $contact Contact / 
				$id = $contact->getId();
				if ($contact->getObjectName()) $contact_info["contacts"][$id]["Full name"] = $contact->getObjectName();
				if ($contact->getAddress()) $contact_info["contacts"][$id]["Address"] = $contact->getAddress();
				if ($contact->getDepartment()) $contact_info["contacts"][$id]["Departament"] = $contact->getDepartment();
				if ($contact->getJobTitle()) $contact_info["contacts"][$id]["Job Title"] = $contact->getJobTitle();
				if ($contact->getAddress()) $contact_info["contacts"][$id]["Address"] = $contact->getAddress();
				if ($contact->getIsCompany()){
					$contact_info["contacts"][$id]["Type"] = lang("company") ;
				} else {
					$contact_info["contacts"][$id]["Type"] = lang("person") ;
				}
			}
		}*/
		return $contact_info ;
	}
	/**
	 * Return view URL
	 *
	 * @access public
	 * @param void
	 * @return string
	 */
	function getViewUrl() {
		$action =   'view';
		return get_url('customer', $action, $this->getId());
	} 
	
	
	

	/**
	 * Return edit customer URL
	 *
	 * @access public
	 * @param void
	 * @return string
	 */
	function getEditUrl() {
		return get_url ( 'customer', 'edit', $this->getId () );
	}
	

	/**
	 * Return add customer URL
	 *
	 * @access public
	 * @param void
	 * @return string
	 */
	function getAddUrl() {
		return get_url ( 'customer', 'add' );
	}
	

	
	/**
	 * Delete this object
	 *
	 * @param void
	 * @return boolean
	 */
	function delete() {
		return parent::delete();
	}


	
	/**
    * Return value of 'first_name' field
    *
    * @access public
    * @param void
    * @return string 
    */
    function getName() {
      return $this->getObjectName();
    } 
    
    /**
    * Set value of 'first_name' field
    *
    * @access public   
    * @param string $value
    * @return boolean
    */
    function setName($value) {
       $this->setObjectName($value);	
    }
	

	/**
	 * Return object URl
	 *
	 * @access public
	 * @param void
	 * @return string
	 */
	function getObjectUrl() {
		return $this->getViewUrl();
	}
	
	function setInstallationId($value){
		$this->setColumnValue("installation_id", $value);
	}
	
	function getInstallationId() {
		return $this->getColumnValue("installation_id");
	}
	
	/**
	 * TODO In the Future The installations will be another content type realate to the customer  
	 * 
	 */
	function getInstallationInfo() {
		$info =  $this->getArrayInfo(true);
		// Disable fields for view
		unset($info['contact_id']) ;
		unset($info['payment_cycle_id']) ;
		unset($info['service_type_id']) ;
		unset($info['subscription_channel_id']) ;
		unset($info['payment_method_id']) ;
		unset($info['feedback_id']) ;
		unset($info['status_id']) ;
		
		//Format dates for 'view'
		if (isset($info['start_date'])) $info['start_date'] = format_date($info['start_date']);
		if (isset($info['end_date'])) $info['end_date'] = format_date($info['end_date']);
		return $info ;
	} 
	
	
	function getArrayInfo($filter_empty = false, $custom_properties = array()){
		$member = Members::findOneByObjectId($this->getId(), get_customers_dimension()->getId());
		$info = array(
			'contact' => $this->getContactId(), //Main contact
			'contact_object_type_id' => $this->getContactObjectTypeId(),
			'contacts' => $this->getContacts(),
			'contact_ids' => $this->getContactIds(false),
			'name' => $this->getObject()->getName(),
			'status_id' => $this->getStatusId(),
			'status' => $this->getStatusName(),	
			'automatic_status' => $this->getAutomaticStatus(),	
			'system_usage' => $this->getSystemUsage(),
			'cmrr' => number_format( doubleval($this->getCmrr()), 2),
			'installation_id' => $this->getInstallationId() ,	
			'industry' => $this->getIndustry(),
			'contact_id' => $this->getContactId(),
			'code' => $this->getCode(),
			'start_date' => ($this->getStartDate() instanceof DateTimeValue) ? format_date($this->getStartDate(), null, 0) : '' ,
			'end_date' => ($this->getEndDate() instanceof DateTimeValue) ? format_date($this->getEndDate(), null, 0) : '',
			'subscription_channel_id' => $this->getSubscriptionChannelId() ,	
			'subscription_channel' => $this->getSubscriptionChannelId() ,	
			'payment_method_id' => $this->getPaymentMethodId() ,	
			'feedback_id' => $this->getFeedbackId() ,	
			'feedback' => $this->getFeedbackName() ,	
			'payment_cycle' => ($this->getPaymentCycleId()) ? $this->getPaymentCycleName() : ''  ,	
			'payment_cycle_id' => ($this->getPaymentCycleId()) ? $this->getPaymentCycleId() : ''  ,	
			'payment_method_id' => $this->getPaymentMethodId() ,	
			'payment_method' => $this->getPaymentMethodId() ,	
			'service_type_id' => ($this->getServiceTypeId()) ? $this->getServiceTypeId() : '',	
			'service_type' => ($this->getServiceTypeId()) ? $this->getServiceTypeName() : '',	
			'system_usage' => $this->getSystemUsage(),	
			'users_limit' => $this->getUsersLimit() ,	
			'active_users' => $this->getActiveUsers() ,	
			'internal_users' => $this->getInternalUsers(),	
			'external_users' => $this->getExternalUsers(),
			'customer_name' => $this->getCustomerName(),
			'customer_address' => $this->getCustomerAddress(),
			'customer_rut' => $this->getCustomerRut(),
			'customer_number' => $this->getCustomerNumber(),
			'member_id' => $member->getId(),
			'object_id' => $this->getId(),
		);
		
		if (Plugins::instance()->isActivePlugin('member_custom_properties')) {
			foreach ($custom_properties as $cp) {
				$info['cp_'.$cp->getId()] = get_member_custom_property_value_for_listing($cp, $member->getId());
			}
		}
		
		if ($filter_empty) {
			$info = array_filter($info); // TODO not filter zeros
		}
		return $info ;
	}


	function getCustomerName() {
		if (Plugins::instance()->isActivePlugin('member_custom_properties')) {
			
			$val = parent::getCustomerName();
			$cp = CustomProperties::getCustomPropertyByCode($this->getObjectTypeId(), 'legal_name');
			if ($cp instanceof CustomProperty) {
				$cp_val = CustomPropertyValues::getCustomPropertyValue($this->getId(), $cp->getId());
				if ($cp_val instanceof CustomPropertyValue) {
					$val = $cp_val->getValue();
				}
			}
			return $val;
			
		} else {
			return parent::getCustomerName();
		}
	}

	function getCustomerRut() {
		if (Plugins::instance()->isActivePlugin('member_custom_properties')) {
			
			$val = parent::getCustomerRut();
			$cp = CustomProperties::getCustomPropertyByCode($this->getObjectTypeId(), 'company_id');
			if ($cp instanceof CustomProperty) {
				$cp_val = CustomPropertyValues::getCustomPropertyValue($this->getId(), $cp->getId());
				if ($cp_val instanceof CustomPropertyValue) {
					$val = $cp_val->getValue();
				}
			}
			return $val;
			
		} else {
			return parent::getCustomerRut();
		}
    }
	
	
	/**
	 * Returns true if $user can access this customer
	 *
	 * @param User $user
	 * @return boolean
	 */
	function canView(Contact $user) {
		return can_read($user, $this->getMembers(), $this->getObjectTypeId());
	} 
	
	
	static function canAdd(Contact $contact, $context, &$notAllowedMember = ''){
		return can_manage_dimension_members($contact);
	}

	/**
	 * Check if specific user can add users
	 *
	 * @access public
	 * @param Contact $user
	 * @return booelean
	 */
	function canAddUser(Contact $user) {
		if($user->isAccountOwner()) {
			return true;
		}
		return can_manage_security($user);
	}
	
	
	/**
	 * Returns true if this user can see $user
	 *
	 * @param User $user
	 * @return boolean
	 */
	function canSeeUser(Contact $user) {
		if($this->isMemberOfOwnerCompany()) {
			return true;
		}
		if($user->getCompanyId() == $this->getCompanyId()) {
			return true;
		}
		if($user->isMemberOfOwnerCompany()) {
			return true;
		} 
		return false;
	} 

	/**
	 * Check if specific user can edit this contact
	 *
	 * @access public
	 * @param User $user
	 * @return boolean
	 */
	function canEdit(Contact $user) {
		return can_manage_dimension_members($user);
	}
	

	/**
	 * Check if specific user can delete this contact
	 *
	 * @access public
	 * @param User $user
	 * @return boolean
	 */
	function canDelete(Contact $user) {
		return can_manage_dimension_members($user);
	}
	

	function canLinkObject(Contact $user) {
		return can_read($user, $this->getMembers(), $this->getObjectTypeId());
	}
	
	/**
	 * 
	 * @see DataObject::validate()
	 */
	function validate($errors) {
		/*
		if(!$this->validatePresenceOf('contact_id')) {
			$errors[] = lang('main contact required');
		} 	
		*/
	}
	
	
	/**
	 * @return array
	 * Enter description here ...
	 */
	function getProjects() {
		if(!$this->projects) {
			$this->projects = array();
			$members = Members::findByObjectId($this->getId());
			if (count($members)) {
				$member = $members[0] ;
				/* @var $member Member */
				foreach ($member->getAllChildrenIdsByType(Projects::instance()->getObjectTypeId()) as $memberId ){
					$this->projects[]=Projects::instance()->findById(Members::instance()->findById($memberId)->getObjectId()) ;					
				}
			}
		}
		return $this->projects ;
	}
	
	
	function setFromAttributes($attributes) {
		parent::setFromAttributes($attributes);
		$all_status_ids = CustomerStatusManager::instance()->findAll(array('id' => true));
		if (!in_array($this->getStatusId(), $all_status_ids)) {
			$this->setStatusId(6);
		}
	}



	function getAdditionalCustomPropertyValues($cp) {
		$cp_values = array();
		if ($cp instanceof CustomProperty) {
			$cp_values = CustomPropertyValues::getCustomPropertyValues($this->getContactId(), $cp->getId());
		}
		return $cp_values;
	}
	
	function getAdditionalCustomPropertyAssociatedObject($cp) {
		return Contacts::instance()->findById($this->getContactId());
	}
}
