<?php

/**
 * Project class
 *
 * @author Diego Castiglioni <diego.castiglioni@fengoffice.com>
 */
class Project extends BaseProject {

	protected $searchable_columns = array('name');
	
	protected $is_searchable = true;
	
	
	private $dimension_id = null;
	
	private $customer = null;
	private $customer_member = null;

	function __construct() {
		parent::__construct();
		Env::useHelper('functions', 'crpm');
	}
	
	// uses automatic status calculation
	function getAutomaticStatus() {
		$cp = CustomProperties::instance()->findOne(array('conditions' => "code='calculate the status automatically_special' AND object_type_id=".$this->getObjectTypeId()));
		if ($cp instanceof CustomProperty) {
			$cp_val = CustomPropertyValues::getCustomPropertyValue($this->getId(), $cp->getId());
			if ($cp_val instanceof CustomPropertyValue) {
				return $cp_val->getValue() > 0;
			} else {
				return false;
			}
		} else {
			return parent::getAutomaticStatus();
		}
	}


	// start_date
	function getStartDate() {
		if (Plugins::instance()->isActivePlugin('member_custom_properties')) {
			$dim = Dimensions::findByCode('customer_project');
			$cp_val = null;
			$cp = MemberCustomProperties::instance()->findOne(array('conditions' => "code='start_date' AND object_type_id=".$this->getObjectTypeId()));
			if ($cp instanceof MemberCustomProperty) {
				$member = Members::findOneByObjectId($this->getId(), $dim->getId());
				if ($member instanceof Member) {
					$cp_val =  MemberCustomPropertyValues::getMemberCustomPropertyValue($member->getId(), $cp->getId());
				}
			}
			if ($cp_val instanceof MemberCustomPropertyValue && $cp_val->getValue() != EMPTY_DATETIME) {
				return DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $cp_val->getValue());
			} else {
				return null;
			}
		}
		return $this->getColumnValue('start_date');
	}
	
	
	// end_date
	function getEndDate() {
		if (Plugins::instance()->isActivePlugin('member_custom_properties')) {
			$dim = Dimensions::findByCode('customer_project');
			$cp_val = null;
			$cp = MemberCustomProperties::instance()->findOne(array('conditions' => "code='end_date' AND object_type_id=".$this->getObjectTypeId()));
			if ($cp instanceof MemberCustomProperty) {
				$member = Members::findOneByObjectId($this->getId(), $dim->getId());
				if ($member instanceof Member) {
					$cp_val =  MemberCustomPropertyValues::getMemberCustomPropertyValue($member->getId(), $cp->getId());
				}
			}
			if ($cp_val instanceof MemberCustomPropertyValue && $cp_val->getValue() != EMPTY_DATETIME) {
				return DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $cp_val->getValue());
			} else {
				return null;
			}
		}
		return $this->getColumnValue('end_date');
	}
	
	/**
	 * Can $user view this object
	 *
	 * @param Contact $user
	 * @return boolean
	 */
	 function canView(Contact $user){
		return can_read($user, $this->getMembers(), $this->getObjectTypeId());
	 }

	
	/**
	 * Check if this user can add a new object to this project. This method is called staticly
	 *
	 * @param Contact $user
	 * @param Member $member
	 * @return boolean
	 */
	static function canAdd(Contact $contact, $context_members, &$notAllowedMember = '') {
		return can_manage_dimension_members($contact);
	} // canAdd

	
	/**
	 * Returns true if this user can edit this object
	 *
	 * @param Contact $user
	 * @return boolean
	 */
	 function canEdit(Contact $contact){
	 	return can_manage_dimension_members($contact);
	 }

	
	/**
	 * Returns true if this user can delete this object
	 *
	 * @param Contact $user
	 * @return boolean
	 */
	 function canDelete(Contact $contact) {
	 	return can_manage_dimension_members($contact);
	 }
	 
	 
	function setFromAttributes($attributes, Member $member = null) {
		if (array_var($attributes, 'status') == 0) $attributes['status'] = 5;
		parent::setFromAttributes ($attributes);
		
		if (isset($_POST['member'])) {
			$parentMemberId = array_var($_POST['member'], 'parent_member_id');
			if ( $parentMember = Members::instance()->findById($parentMemberId) ) {
				/* @var $member Member */
				if  ( $customer  = Customers::instance()->findById($parentMember->getObjectId()) ){
					$this->setCustomerId($customer->getId()); 	
				} 
			}
		}
		/*
		if ($member instanceof Member && class_exists('Customers')) {
			$context = active_context();
			foreach ($context as $selection) {
				if ($selection instanceof Member && $selection->getDimension()->getCode() == 'customers') {
					$assoc = DimensionMemberAssociations::instance()->findOne(array('conditions' => array('`dimension_id` = ? AND `object_type_id` = ? AND `associated_dimension_id` = ? AND `associated_object_type_id` = ?',
						$member->getDimensionId(), $this->manager()->getObjectTypeId(), $selection->getDimensionId(), Customers::instance()->getObjectTypeId())));
					
					if ($assoc instanceof DimensionMemberAssociation) {
						// Tell MemberController to save an association, to associate the customer
						$_POST['save_properties'] = true;
						$_POST['associated_members'] = array($selection->getId() => $assoc->getId());
					}
					break;
				}
			}
		}*/
		
		
	}
	
	function validate($errors) {
		/*if (!$this->validatePresenceOf('status')) $errors[] = lang('status required');
		else {
			if ($this->getStatus() < 1 || $this->getStatus() > 7) $errors[] = lang('status must be between 1 and 7');
		}*/
		
	}
	
	/**
	 * 
	 * CSS classname to apply 
	 */
	function getStatusClass(){
		return "project-status-".$this->getStatus();
	}
	/*
	function modifyMemberValidations($member) {
		if (!$member instanceof Member) return;
		
		$member->add_skip_validation('uniqueness of parent - name');
	}*/
	
	
	
	function getProjectsDimensionId() {
		if (is_null($this->dimension_id)) {
			$dim = Dimensions::findByCode('customer_project');
			$this->dimension_id = $dim->getId();
		}
		return $this->dimension_id;
	}
	
	
	function getProjectMemberId() {
		$res = Members::instance()->findOne(array('id' => 'true', 'conditions' => '`dimension_id` = '.$this->getProjectsDimensionId().' AND `object_id` = '.$this->getId()));
		if (is_array($res) && count($res) > 0) {
			return $res[0];
		}
		return $res;
	}
	
	function getProjectMember() {
		return Members::instance()->findOne(array('conditions' => '`dimension_id` = '.$this->getProjectsDimensionId().' AND `object_id` = '.$this->getId()));
	}
	
	
	function getContactIds () {
		$a_dim = Dimensions::findByCode('feng_persons');
		$proj_dim = Dimensions::findByCode('customer_project');
		
		$a_ot = ObjectTypes::findByName('person');
		$mid = $this->getProjectMemberId();

		$assoc = DimensionMemberAssociations::instance()->findOne(array('conditions' => array('`dimension_id`=? AND `object_type_id`=? AND `associated_dimension_id`=? AND `associated_object_type_id`=?', 
			$proj_dim->getId(), 
			Projects::instance()->getObjectTypeId(), 
			$a_dim->getId(), $a_ot->getId()))
		);
			
		$sql = "
			SELECT distinct(object_id) FROM ".TABLE_PREFIX."members m WHERE id IN 
			(
				SELECT distinct(property_member_id) FROM ".TABLE_PREFIX."member_property_members 
				WHERE association_id =".$assoc->getId(). " AND member_id = ".$mid."
			)
			AND EXISTS
			(
				SELECT id FROM ".TABLE_PREFIX."objects WHERE id = object_id AND archived_on = '0000-00-00 00:00:00' AND trashed_on = '0000-00-00 00:00:00'
			)
			AND EXISTS 
			( 
				SELECT object_id FROM  ".TABLE_PREFIX."contacts c WHERE c.object_id = m.object_id AND c.disabled = 0 
			)";
		$res = DB::executeAll($sql);
		$ids = array();
		if ($res && is_array($res)) { 
			foreach ( $res as $row ) {
				$ids[] = $row['object_id'];
			}
		}
		return $ids;
	}
	
	function getArrayInfo($include_customer_info = false, $custom_properties = array()) {
		$member = $this->getProjectMember();
		
		if ($member instanceof Member) {
			if (config_option ( "wysiwyg_projects" )) {
				if ($this->getTypeContent () == "text") {
					$desc = nl2br ( $member->getDescription () );
				} else {
					$desc = purify_html ( $member->getDescription () );
				}
			} else {
				if ($this->getTypeContent () == "text") {
					$desc = escape_html_whitespace ( $member->getDescription () );
				} else {
					$desc = html_to_text ( escape_html_whitespace ( $member->getDescription () ) );
				}
			}
		} else {
			$desc = "";
		}
                
		$info = array(
			'object_id' => $this->getObjectId(),
			'name' => $this->getObjectName(),
			'status' => $this->getStatus(),
			'automatic_status' => $this->getAutomaticStatus(),
			'description' => $desc,
			'customer_id' => $this->getCustomerId(),
			'customer_member_id' => $this->customer_member instanceof Member ? $this->customer_member->getId() : '0',
			'customer_color' => $this->customer_member instanceof Member ? $this->customer_member->getColor() : '0',
			'member_id' => $member instanceof Member ? $member->getId() : 0,
			'contact_ids' => $this->getContactIds(),
			'start_date' => $this->getStartDate() instanceof DateTimeValue ? format_date($this->getStartDate(), null, 0) : lang('n/a'),
			'end_date' => $this->getEndDate() instanceof DateTimeValue ? format_date($this->getEndDate(), null, 0) : lang('n/a'),
			'manager_id' => $this->getManagerId(),
			'manager_name' => $this->getManagerName(),
		);
		
		$info['status_html'] = $this->status_html != null ? $this->status_html : '';
		
		if ($include_customer_info && class_exists('Customers')) {
			$customer = $this->getCustomer();
			if ($customer instanceof Customer) {
				//$info['contact_id'] = $customer->getContactId();
				$info['customer_name'] = $customer->getObjectName();
			}
		}
		
		return $info;
	}
	
	private $status_html = null;
	function getStatus() {
		
		$status_dimension = Dimensions::findByCode("project_statuses");
		$status_ot = ObjectTypes::findByName("project_status");
		$use_project_status_dimension = $status_dimension instanceof Dimension && in_array($status_dimension->getId(), config_option('enabled_dimensions'));
		
		if ($use_project_status_dimension) {
			
			$project_member = Members::findOneByObjectId($this->getId(), Dimensions::findByCode('customer_project')->getId());
			$project_status = get_associated_status_member_id($project_member, $status_dimension, $status_ot, false, 'project_'.$status_ot->getName());
			$project_status_mem = Members::instance()->findById($project_status);
			
			if ($project_status_mem instanceof Member) {
				$this->status_html = '<span class="crpm-status member-path real-breadcrumb og-wsname-color-'. $project_status_mem->getColor() .'">' . clean($project_status_mem->getName()) . '</span>';
			}
			
			return $project_status;
			
		} else {
			$cp = CustomProperties::instance()->findOne(array('conditions' => "code='status_special' AND object_type_id=".$this->getObjectTypeId()));
			if ($cp instanceof CustomProperty) {
				$cp_val = CustomPropertyValues::getCustomPropertyValue($this->getId(), $cp->getId());
				if ($cp_val instanceof CustomPropertyValue) {
					return $cp_val->getValue();
				}
			} else {
				return parent::getStatus();
			}
		}
	}
	
	
	function setStatus($new_status) {
		$cp = CustomProperties::instance()->findOne(array('conditions' => "code='status_special' AND object_type_id=".$this->getObjectTypeId()));
		if ($cp instanceof CustomProperty) {
			$cp_val = CustomPropertyValues::getCustomPropertyValue($this->getId(), $cp->getId());
			if (!$cp_val instanceof CustomPropertyValue) {
				$cp_val = new CustomPropertyValue();
				$cp_val->setObjectId($this->getId());
				$cp_val->setCustomPropertyId($cp->getId());
			}
			$cp_val->setValue($new_status);
			$cp_val->save();
		}
		
		parent::setStatus($new_status);
	}
	
	
	function getCustomer($force_reload=false) {
		if ($force_reload) {
			$this->customer = null;
		}
		
		if (!$this->customer) {
			$customersType = Customers::instance()->getObjectTypeId() ;
			$members = Members::findByObjectId($this->getId());
			if (count($members)) {
				$member = $members[0];
				foreach ($member->getAllParentMembersInHierarchy() as $member){
					if ($member->getObjectTypeId() == $customersType) {
						$this->customer_member = $member;
						$this->customer = Customers::instance()->findById($member->getObjectId());
						break ;
					}
				}
			}
		}
		if (!$this->customer) {
			Env::useHelper('functions', 'crpm');
			$customers_dim = get_customers_dimension();
			$crpm_dim = Dimensions::findByCode('customer_project');
			if ($customers_dim->getId() != $crpm_dim->getId()) {
				$member = Members::findOneByObjectId($this->getId(), $crpm_dim->getid());
				$customer_member_id = get_associated_status_member_id($member, $customers_dim, ObjectTypes::findByName('customer'), false, 'project_customer');
				$customer_member = Members::instance()->findById($customer_member_id);
				if ($customer_member instanceof Member) {
					$this->customer_member = $customer_member;
					$this->customer = Customers::instance()->findById($customer_member->getObjectId());
				}
			}
		}
		return $this->customer;
	}
	
	function getMemberId(){
		
	}
	
	function getManagerName() {
		$name = '';
		if ($this->getManagerId() > 0) {
			$c = Contacts::instance()->findById($this->getManagerId());
			if ($c instanceof Contact) {
				$name = $c->getObjectName();
			}
		}
		return $name;
	}
	
	function getContacts() {
		$contactIds = $this->getContactIds();
		if (!count($contactIds))  return array(0);
		return Contacts::instance()->findAll(array("conditions" => "object_id IN (" .implode(",", $contactIds).")", "order"=>"name ASC"));	
	}
	
	
	function getCustomerId($force_reload=false) {
		if ($c = $this->getCustomer($force_reload)) {
			return $c->getId();
		}
		return 0 ;
    }
	
}
