<?php
Hook::register("confidential_users");

function confidential_users_override_contact_name($params, &$name) {
	if (logged_user() instanceof Contact && logged_user()->isAdministrator()) return;
	$contact = array_var($params, 'contact');
	if ($contact instanceof Contact && $contact->isUser() && $contact->getColumnValue('is_confidential')) {
		
		if (!logged_user() instanceof Contact || logged_user()->getId() != $contact->getId()) {
			$name = $contact->getJobTitle();
			if (trim($name) == '') {
				$name = lang($contact->getUserTypeName());
			}
		}
	
	}
}

function confidential_users_override_contact_email($params, &$email) {
	if (logged_user() instanceof Contact && logged_user()->isAdministrator()) return;
	$contact = array_var($params, 'contact');
	if ($contact instanceof Contact && $contact->isUser() && $contact->getColumnValue('is_confidential')) {
		if (!logged_user() instanceof Contact || logged_user()->getId() != $contact->getId()) {
			$email = "";
		}
	}
}

function confidential_users_object_definition($type, &$ret){
	if ($type == 'Contact' && check_column_exists(TABLE_PREFIX.'contacts', 'is_confidential')) {
		$ret['is_confidential'] = DATA_TYPE_BOOLEAN;
	}
}

function confidential_users_mail_account_permissions_extra_cond($params, &$conditions) {
	if (logged_user() instanceof Contact && logged_user()->isAdministrator()) return;
	$conditions .= " AND is_confidential=0";
}

function confidential_users_render_additional_user_data_fields($contact, &$ignored) {
	if ($contact instanceof Contact && (can_manage_security(logged_user()) || logged_user() && $contact->getId()==logged_user()->getId() && $contact->getColumnValue('is_confidential'))) {
	?>
		<div class="field role" id="user_role_div">
			<label><?php echo lang("is confidential")?>:</label>
			
			<div style="float:left; padding-top: 5px; margin: 0 15px 0 0;">
				<?php echo yes_no_widget('contact[is_confidential]', '', $contact->getColumnValue('is_confidential'), lang('yes'), lang('no')) ?>
			</div>
		</div>
	<?php
	}
}


function confidential_users_listing_extra_conditions($ignored, &$extra_conditions) {
	if (logged_user() instanceof Contact && logged_user()->isAdministrator()) return;
	$controller = array_var($_GET, 'c');

	if ($controller == 'contact' && check_column_exists(TABLE_PREFIX.'contacts', 'is_confidential') && logged_user()) {
		$extra_conditions .= " AND (is_confidential=0 OR o.id=".logged_user()->getId().")";
	}
}

function confidential_users_widget_extra_conditions($widget, &$extra_conditions) {
	if (logged_user() instanceof Contact && logged_user()->isAdministrator()) return;
	if ($widget == 'people' && check_column_exists(TABLE_PREFIX.'contacts', 'is_confidential') && logged_user()) {
		$extra_conditions .= " AND (is_confidential=0 OR o.id=".logged_user()->getId().")";
	}
}


function confidential_users_contact_can_view($contact, &$return_false) {
	if (logged_user() instanceof Contact && logged_user()->isAdministrator()) return;
	if ($contact instanceof Contact && $contact->isUser() && $contact->getColumnValue('is_confidential') && logged_user() && $contact->getId()!=logged_user()->getId()) {
		$return_false = true;
	}
}

function confidential_users_contact_check_can_view_in_array($user, &$contacts) {
	if (logged_user() instanceof Contact && logged_user()->isAdministrator()) return;

	foreach ($contacts as $key => $contact) {
		if ($contact instanceof Contact && $contact->isUser() && $contact->getColumnValue('is_confidential') && logged_user() && $contact->getId()!=logged_user()->getId()) {
			//remove
			unset($contacts[$key]);
		}

	}
}

function confidential_users_contact_can_edit($contact, &$return_false) {
	if (logged_user() instanceof Contact && logged_user()->isAdministrator()) return;
	if ($contact instanceof Contact && $contact->isUser() && $contact->getColumnValue('is_confidential') && logged_user() && $contact->getId()!=logged_user()->getId()) {
		$return_false = true;
	}
}
function confidential_users_override_contact_picture_url($contact, &$url) {
	if (logged_user() instanceof Contact && logged_user()->isAdministrator()) return;
	if ($contact instanceof Contact && $contact->isUser() && $contact->getColumnValue('is_confidential') && logged_user() && $contact->getId()!=logged_user()->getId()) {
		$url = get_image_url('default-avatar.png');
	}
}
function confidential_users_custom_report_extra_conditions($params, &$conditions) {
	if (logged_user() instanceof Contact && logged_user()->isAdministrator()) return;
	$contact_ot = ObjectTypes::findByName('contact');
	$report = array_var($params, 'report');
	if ($report->getReportObjectTypeId() == $contact_ot->getId() && logged_user()) {
		$conditions .= " AND (is_confidential=0 OR o.id=".logged_user()->getId().") ";
	}
}