<?php
Hook::register ( "crpm" );

if (!defined('SAVE_PERMISSIONS_IN_BACKGROUND')) {
	define('SAVE_PERMISSIONS_IN_BACKGROUND', true);
}

if (strtoupper(substr(PHP_OS, 0, 5)) == 'LINUX') {
	if (!function_exists('pdf_to_text_pro')) {
		function pdf_to_text_pro($file_path) {
			$tmp_name = "tmp/".gen_id().".text";
			exec("pdftotext $file_path $tmp_name");
			return file_get_contents($tmp_name);
		}
	}
}

function crpm_edit_dimension_name($params, &$dim_name) {
	$dim = array_var($params, 'dimension');
	if (is_numeric($dim)) $dim = Dimensions::getDimensionById($dim);
	
	Env::useHelper('functions', 'crpm');
	$customers_dim = get_customers_dimension();
	if ($dim instanceof Dimension && $dim->getCode() == 'customer_project' && $customers_dim->getCode() == 'customers') {
		$dim_name = lang('projects');
	}
}

function crpm_active_context_widget_edit_link($member, &$edit_onclick) {
	if ($member->getObjectTypeId() == Customers::instance()->getObjectTypeId()) {
		$url = get_url("customer", "edit", array('id' => $member->getObjectId()));
		$edit_onclick = "og.render_modal_form('', {url:'$url'});";
	} else if ($member->getObjectTypeId() == Projects::instance()->getObjectTypeId()) {
		$url = get_url("project", "edit", array('id' => $member->getObjectId()));
		$edit_onclick = "og.render_modal_form('', {url:'$url'});";
	}
}

function crpm_search_result($result, &$ret ) {
	if (array_var($result, 'otid') == Projects::instance()->getObjectTypeId()){
		if ( $member = Members::findOneByObjectId($result['id'])){
			$result['url'] = "javascript:og.projects.onProjectClick(". $member->getId() ."); og.openTab('overview-panel');";
		}
	}elseif (array_var($result, 'otid') == Customers::instance()->getObjectTypeId()) {
		if ( $member = Members::findOneByObjectId($result['id']) ) {
			$result['url'] = "javascript:og.customers.onCustomerClick(". $member->getId() .");og.openTab('overview-panel');";
		}
	}	
	$ret = $result ;
}

function crpm_total_tasks_times_csv_columns($ignored, &$cols) {
	$dimension = Dimensions::findByCode('customer_project');
	$cols[] = $dimension->getName();
}

function crpm_total_tasks_times_csv_column_values($ts, &$new_values) {
	if (!is_array($new_values)) $new_values = array();
	$dimension = Dimensions::findByCode('customer_project');
	if ($ts instanceof Timeslot) {
		$members = $ts->getMembers();
		$str = "";
		foreach ($members as $m) {
			if ($m->getDimensionId() == $dimension->getId()) {
				$str .= ($str == "" ? "" : " - "). $m->getName();
			}
		}
		$new_values[] = $str;
	}
}


function crpm_get_calculated_column_value($params, &$value) {
	$report = array_var($params, 'report');
	$object = array_var($params, 'object');
	$column = array_var($params, 'column');
	
	if ($object instanceof ProjectTask) {
		switch ($column) {
			case 'status': 
				$today = DateTimeValueLib::now();
				$today->beginningOfDay();
				$style = 'padding: 1px 5px; ; margin: 3px; border-radius: 4px ; text-align: center; ';
				if ($object->isCompleted()) {
					$value = lang ( "completed" );
					$style .= 'background-color: #73D073 ; color: #FFFFFF ;';
				} elseif ($object->getDueDate() instanceof DateTimeValue && $object->getDueDate()->isToday()) {
					$value = lang ( 'today' );
					$style .= 'background-color: #FFEB9C; color: #E26900; ';
				} elseif ($object->getDueDate() instanceof DateTimeValue && $object->getDueDate()->getTimestamp() < $today->getTimestamp()) {
					$value = lang ( 'days late', $object->getLateInDays() );
					$style .= 'background-color: #F6D4D5; color: #F6464C ';
				} else {
					$value = lang ( 'days left', $object->getLeftInDays() );
					$style .= 'background-color: #F5F5F5; color: #333333';
				}
				$value = "<div style='$style'> $value </div>";;
				break;
				
			default: 
				break;
		}
	}
}

function crpm_custom_reports_override_column_format($parameters, &$value) {
    $object = array_var($parameters, 'object');
    $column = array_var($parameters, 'field');

    if ($object instanceof Timeslot) {
    $task = ProjectTasks::instance()->findById($object->getRelObjectId());
    // if task exists
        if ($task instanceof ProjectTask) {
            crpm_get_calculated_column_value(array('object' => $task, 'column' => $column), $value);
        }
    }
}

function crpm_more_object_calculated_columns($manager, &$columns) {
	if ($manager instanceof ProjectTasks) {
		$columns[] = 'status';
	}
}

function crpm_total_task_timeslots_group_by_criterias($args, &$ret) {
	$dimension = Dimensions::findByCode ( 'customer_project' );
	$enabled_dimension_ids = config_option('enabled_dimensions');
	
	if (in_array($dimension->getId(), $enabled_dimension_ids)) {
		$ret[] = array('val' => 'dim_'.$dimension->getId(), 'name' => $dimension->getName());
	}
}

function crpm_custom_reports_additional_columns($args, &$ret) {
	// the dim columns are added by advanced_reports plugin
	if (Plugins::instance()->isActivePlugin('advanced_reports')) return;

	$ot = array_var($args, 'object_type');
	$dimension = Dimensions::findByCode ( 'customer_project' );
	$enabled_dimensions = config_option('enabled_dimensions');
	
	if ($ot instanceof ObjectType && in_array($dimension->getId(), $enabled_dimensions)) {
		$name = $dimension->getName();
		
		if (in_array($ot->getType(), array('dimension_object','dimension_group'))) {
			$associations = DimensionMemberAssociations::getAssociatations($dimension->getId(), $ot->getId());
			foreach ($associations as $assoc) {
				/* @var $assoc DimensionMemberAssociation */
				if (in_array($assoc->getAssociatedDimensionMemberAssociationId(), $enabled_dimensions)) {
					
					$d = Dimensions::instance()->findById($assoc->getAssociatedDimensionMemberAssociationId());
					$ret[] =  array('id' => 'dimassoc_'.$assoc->getId(), 'name' => $d->getName(), 'type' => DATA_TYPE_STRING);
				}
			}
			
			
		} else {
			
			$ret[] =  array('id' => 'dim_'.$dimension->getId(), 'name' => $name, 'type' => DATA_TYPE_STRING);
			
		}
	}
}

function crpm_custom_reports_get_external_column_value($parameters, &$value) {
	$external_id = array_var($parameters, 'external_id');
	$manager = array_var($parameters, 'manager');
	$field = array_var($parameters, 'field');
	$object = array_var($parameters, 'object');
	
	if ($manager instanceof Projects) {
		if ($field == 'status') {

			$status_dimension = Dimensions::findByCode("project_statuses");
			$status_ot = ObjectTypes::findByName("project_status");
			$use_project_status_dimension = $status_dimension instanceof Dimension && in_array($status_dimension->getId(), config_option('enabled_dimensions'));
			
			if ($use_project_status_dimension) {
				$project_member = $object;
				$status_mem_id = get_associated_status_member_id($project_member, $status_dimension, $status_ot, false, 'project_'.$status_ot->getName());
				$status_mem = Members::getMemberById($status_mem_id);
				
				if ($status_mem instanceof Member) {
					$value = $status_mem->getName();
				} else {
					$value = "";
				}
				
			} else {
				$value = lang('project_field_status' . $external_id);
			}
			
		} else if ($field == 'manager_id') {
			$contact = Contacts::instance()->findById($external_id);
			if ($contact instanceof Contact) {
				$value = clean($contact->getObjectName());
			} else {
				$value = "";
			}
		}
		
	} else if ($manager instanceof Customers) {
		if ($field == 'status_id') {

			$st_obj = CustomerStatusManager::instance()->findById($external_id);
			if ($st_obj instanceof CustomerStatus) {
				$value = lang($st_obj->getName());
			} else {
				$value = "";
			}
			
		} else if ($field == 'payment_cycle_id') {
			$pc_obj = CustomerPaymentCycles::instance()->findById($external_id);
			if ($pc_obj instanceof CustomerPaymentCycle) {
				$value = lang($pc_obj->getName());
			} else {
				$value = "";
			}
		}
	}
}


function crpm_custom_reports_get_possible_external_column_values($parameters, &$values) {
	$ot_id = array_var($parameters, 'ot_id');
	$field = array_var($parameters, 'field');
	$ot = ObjectTypes::instance()->findById($ot_id);
	
	if ($ot->getName() == 'project') {
		if ($field == 'status') {

			$status_dimension = Dimensions::findByCode("project_statuses");
			$status_ot = ObjectTypes::findByName("project_status");
			$use_project_status_dimension = $status_dimension instanceof Dimension && in_array($status_dimension->getId(), config_option('enabled_dimensions'));
			
			if ($use_project_status_dimension) {
				$st_members = Members::instance()->findAll(array('conditions' => 'dimension_id='.$status_dimension->getId()));
				foreach ($st_members as $st_mem) {
					$values[] = array('id' => $st_mem->getId(), 'name' => $st_mem->getName());
				}
				
			} else {
				for ($i=1; $i<=7; $i++) {
					$values[] = array('id' => $i, 'name' => lang('project_field_status' . $i));
				}
			}
			
		} else if ($field == 'manager_id') {
			
			$users = Contacts::getAllUsers();
			foreach ($users as $user) {
				if ($user instanceof Contact) {
					$values[] = array('id' => $user->getId(), 'name' => $user->getObjectName());
				}
			}
		}
		
	} else if ($ot->getName() == 'customer') {
		if ($field == 'status_id') {

			$st_objs = CustomerStatusManager::instance()->findAll();
			foreach ($st_objs as $st_obj) {
				$values[] = array('id' => $st_obj->getId(), 'name' => $st_obj->getName());
			}
			
		} else if ($field == 'payment_cycle_id') {
			$pc_objs = CustomerPaymentCycles::instance()->findAll();
			foreach ($pc_objs as $pc_obj) {
				$values[] = array('id' => $pc_obj->getId(), 'name' => $pc_obj->getName());
			}
		}
	}
}


function crpm_edit_member_properties($args, &$ret) {
	$member = $args [0];
	$member_id = $member->getId ();
	$old_properties = $args [1];
	$new_properties = $args [2];
	
	$crpm_object_type_id = Projects::instance ()->getObjectTypeId ();
    $dimension = new Dimensions();
	$persons_dim = $dimension->findOne ( array ("conditions" => "`code`='feng_persons'" ) );
	// Vincula al proyecto las personas vinculadas al customer asociado (se hace al guardar el proyecto)
	if ($crpm_object_type_id == $member->getObjectTypeId ()) {
		$has_customer = false;
		foreach ( $new_properties as $new_property ) {
			$property_member_id = $new_property->getPropertyMemberId ();
			$property_member = Members::instance()->findById( $property_member_id );
			
			if ($property_member instanceof Member)
				$dimension_code = $property_member->getDimension ()->getCode ();
			
			if (isset ( $dimension_code ) && ($dimension_code == 'customer_project' || $dimension_code == 'customers')) {
				$crpm_id = $member->getObjectId ();
				$project = Projects::instance()->findById( $crpm_id );
				if ($project instanceof Project) {
					$project->setCustomerId ( $property_member->getObjectId () );
					$project->save ();
				}
				
				// Asignar personas a proyecto
				if (class_exists ( 'Customers' )) {
					$customer = Customers::instance()->findById( $property_member->getObjectId () );
					if ($customer instanceof Customer) {
						$associations = DimensionMemberAssociations::getAssociatations ( $member->getDimensionId (), $member->getObjectTypeId () );
						foreach ( $associations as $a ) {
							foreach ( $customer->getContacts () as $contact ) {
								$members = Members::instance ()->findByObjectId ( $contact->getId () );
								foreach ( $members as $m ) {
									if ($m->getDimensionId () == $persons_dim->getId ()) {
										$mpm = MemberPropertyMembers::findOne ( array ('conditions' => array ("association_id=? AND member_id=? AND property_member_id=?", $a->getId (), $member->getId (), $m->getId () ) ) );
										if ($mpm instanceof MemberPropertyMember) {
											$mpm->setIsActive ( 1 );
											$mpm->save ();
										} else {
											$sql = "
													INSERT INTO " . TABLE_PREFIX . "member_property_members (association_id, member_id, property_member_id, is_active, created_on, created_by_id)
													VALUES (" . $a->getId () . "," . $member->getId () . "," . $m->getId () . ",1,NOW()," . logged_user ()->getId () . ") ";
											DB::execute ( $sql );
										}
									}
								}
							}
						}
					}
				}
				break;
			}
		}
	}
}




function crpm_after_user_add($object, $ignored) {
	/* @var $object Contact */
	$crpm_dim = Dimensions::instance()->findOne(array("conditions" => "`code` = 'customer_project'"));
	
	if ($crpm_dim instanceof Dimension) {
		
		$sql = "INSERT INTO `".TABLE_PREFIX."contact_dimension_permissions` (`permission_group_id`, `dimension_id`, `permission_type`)
				 SELECT `c`.`permission_group_id`, ".$crpm_dim->getId().", 'check'
				 FROM `".TABLE_PREFIX."contacts` `c` 
				 WHERE `c`.`is_company`=0 AND `c`.`user_type`!=0 AND `c`.`disabled`=0 AND `c`.`object_id`=".$object->getId()."
				 ON DUPLICATE KEY UPDATE `dimension_id`=`dimension_id`;";
		DB::execute($sql);
	}
}


function crpm_after_contact_view($contact) {
	return;
	/*
	Env::useHelper("api", "crpm");
	echo "
		<style>
			.person-view .clients-projects {
				padding-left: 10px ;
			}
			.person-view .clients-projects li {
				background-repeat: no-repeat ;
				text-indent: 20px ;
			}
			
		</style>
	";
	echo "
		<div class='clients-projects' >		
			<h2>".lang("customer_project")."</h2>
			<ul>";
	
	$persons_dim = Dimensions::findByCode('feng_persons');
	$crpm_dim = Dimensions::instance()->findOne(array("conditions" => "`code` = 'customer_project'"));
	$already_printed_customers = array();
	foreach (CrpmAPI::getProjects($contact) as $project){
		$pmems = Members::findByObjectId($project->getId(), $crpm_dim->getId());
		if (count($pmems) > 0) $pmem = $pmems[0];
		$ponclick = "";
		if ($pmem instanceof Member) {
			$ponclick = "onclick='og.projects.onProjectClick(".$pmem->getId().");'";
		}
		
		echo "<li class='ico-project'><a $ponclick href='".$project->getViewUrl()."'>".$project->getName()."</a>";
		if ($customer = $project->getCustomer()){
			$cmems = Members::findByObjectId($customer->getId(), $crpm_dim->getId());
			if (count($cmems) > 0) $cmem = $cmems[0];
			$conclick = "";
			if ($cmem instanceof Member) {
				//$conclick = "onclick='og.selectDimensionTreeMember({dim_id:".$persons_dim->getId().",node:\"root\"});og.customers.onCustomerClick(".$cmem->getId().");'";
				$conclick = "onclick='og.customers.onCustomerClick(".$cmem->getId().");'";
			}
			
			echo "<span class='customer'>
				(<a $conclick class='_coViewAction _ico-company' href='".$customer->getViewUrl()."'>".$customer->getName()."</a>)
			</span>";
			
			$already_printed_customers[] = $project->getCustomerId();
		} 
		echo "</li>";
	};
	
	foreach (CrpmAPI::getCustomers($contact) as $customer){
		if (!in_array($customer->getId(), $already_printed_customers)) {
			$cmems = Members::findByObjectId($customer->getId(), $crpm_dim->getId());
			if (count($cmems) > 0) $cmem = $cmems[0];
			$conclick = "";
			if ($cmem instanceof Member) {
				$conclick = "onclick='og.customers.onCustomerClick(".$cmem->getId().");'";
			}
			
			echo "<li class='ico-company'><span class='customer'>
					<a $conclick href='".$customer->getViewUrl()."'>".$customer->getName()."</a>
				</span></li>";
		}
	};

	echo "	</ul>
		</div>";
	*/
}



function crpm_after_object_trash($object, &$null ) {
	if ($object instanceof Contact) {
		evt_add("reset dimension tree",Dimensions::findByCode("customer_project")->getId());
	}
}

function crpm_list_objects_type_ids($arg, &$type_ids) {
	$projectType = ObjectTypes::findByName("project");
	$customerType = ObjectTypes::findByName("customer");
	$excludeTypes = array($projectType->getId(),$customerType->getId());
	
	$type_ids = array_diff($type_ids, $excludeTypes);
	//$type_ids = array();
}


/**
 * Handle when no projects / customers are created 
 */
function crpm_page_rendered() {
	$did = Dimensions::instance()->findByCode('customer_project')->getId();
	$out = '' ;
	$out .= '<script>' ;
	$out .= 'og.customers.dimension_id = '.$did.';';
	$out .= 'og.projects.dimension_id = '.$did.';';
	$out .= 'og.crpm.dimension_id = '.$did .';';
	$out .= '</script>' ;
	echo $out ;
	
	$dim_ids = array_flat(DB::executeAll("SELECT d.id FROM ".TABLE_PREFIX."dimensions d where d.is_manageable = 1 || (SELECT count(m.id) FROM ".TABLE_PREFIX."members m WHERE m.dimension_id=d.id) > 2;"));
	if (count($dim_ids) > 0) {
		$dim_ids_csv = implode(',', $dim_ids);
		if (Members::instance()->count('dimension_id IN (' . $dim_ids_csv . ')') == 0) {
			echo "<script>";
			echo "og.menuPanelCollapsed = true;"; 
			echo "</script>";
		}
	}
	
}


function crpm_after_add_member($member, &$ret) {
	$did = Dimensions::instance()->findByCode('customer_project')->getId();
	if ($member instanceof Member && $member->getDimensionId() == $did) {
		if ($x = Members::instance()->count('dimension_id = '.$did) == 1) {
			// only this => first client
			evt_add("expand menu panel", array('expand' => true));
		}
	}
}




function crpm_render_administration_icons($ignored, &$icons){
	$user = logged_user();

	if ($user instanceof Contact && can_manage_configuration($user)) {
		$icons[] = array(
			'ico' => 'ico-large-formula',
			'name' => lang('status formulas'),
			'url' => get_url('status_formulas', 'index'),
			'target' => 'status-formulas-panel'
		);
	}

}

function crpm_general_settings_additional_links($ignored, &$links) {
	$links[] = array(
		'name' => lang('non working days'),
		'desc' => lang('non working days administration desc'),
		'url' => get_url('non_working_days', 'index'),
	);
}


function crpm_after_object_save($object, &$ignored) {
	
	if ($object instanceof ContentDataObject && !$object->getDontMakeCalculations()) {
		
		// calculate client/project status using defined formulas
		Formulas::instance()->calculateStatus($object);
		
		// reload the overview panel
		ajx_add("overview-panel", "reload");

		// calculate client and project additional data
		/*$task_id = null;
		if ($object instanceof ProjectTask) {
			
			$task_id = $object->getId();
			
		} else if ($object instanceof Timeslot && $object->getRelObjectId() > 0) {
			
			$task_id = $object->getRelObjectId();
			
		}
		
		if (!is_null($task_id)) {
			// include helpers
			Env::useHelper('project_additional_data', 'crpm');
			Env::useHelper('customer_additional_data', 'crpm');
			
			// make calculations
			calculate_projects_additional_data(array('task_id' => $task_id));
			calculate_customers_additional_data(array('task_id' => $task_id));
			
			// reload client/project lists
			evt_add('reload tab panel', 'projects');
			evt_add('reload tab panel', 'customers');
		}*/
	}
}

function crpm_after_multi_object_action($params, &$ignored) {
	
	if (isset($params['objects'])) {
		$objects = array_var($params, 'objects');
		
	} else if (isset($params['objects_ids'])) {
		if (count($params['objects_ids']) > 0) {
			$objects = array();
			foreach (array_var($params, 'objects_ids') as $oid) {
				$objects[] = Objects::findObject($oid);
			}
		}
	}
	if (!isset($objects) || count($objects) == 0) {
		return;
	}
	
	$action = array_var($params, 'action');
	$options = array_var($params, 'options');
	
	$all_members = array();
	$task_ids = array();
	foreach ($objects as $obj) {
		if ($obj instanceof ProjectTask) {
			// get all task ids to recalculate client/project additional data
			$task_ids[$obj->getId()] = $obj->getId();
			$all_subtasks_ids = $obj->getAllSubtaskIdsInHierarchy();
			foreach ($all_subtasks_ids as $sub_id) {
				$task_ids[$sub_id] = $sub_id;
			}
			
			// collect members to recalculate status using formulas
			$tmp_mems = $obj->getMembers();
			foreach ($tmp_mems as $tmp_mem) {
				if ($tmp_mem instanceof Member) $all_members[$tmp_mem->getId()] = $tmp_mem;
			} 
		}
	}
	
	if (count($all_members) > 0) {
		// calculate client/project status using defined formulas
		Formulas::instance()->calculateMemberStatus($all_members);
	}
	
	//commented performance killer
	/*if (count($task_ids) > 0) {
		
		// include helpers for project/client additional data
		Env::useHelper('project_additional_data', 'crpm');
		Env::useHelper('customer_additional_data', 'crpm');
		
		// make calculations
		calculate_projects_additional_data(array('multiple_task_ids' => $task_ids));
		calculate_customers_additional_data(array('multiple_task_ids' => $task_ids));
		
		// reload client/project lists
		evt_add('reload tab panel', 'projects');
		evt_add('reload tab panel', 'customers');
	}*/
}

function crpm_after_remove_members_from_object($object, &$removedMemebersIds) {
	if (config_option('automatic_crpm_status_calculation') && ($object instanceof ProjectTask || $object instanceof Timeslot)) {
		$members_removed = array();
		$members_ids = $object->getMemberIds();

		$members_removed_ids = array_diff($removedMemebersIds, $members_ids);

		foreach ($members_removed_ids as $removed_id){
			$mem = Members::getMemberById($removed_id);
			if($mem instanceof Member){
				$members_removed[] = $mem;
			}
		}
		Formulas::instance()->calculateMemberStatus($members_removed);
	}
}




function crpm_more_panel_dimension_links($ignored, &$links) {
	$enabled_dimensions = config_option("enabled_dimensions");
	$dimension = Dimensions::findByCode('customer_project');
	
	if (!in_array($dimension->getId(), $enabled_dimensions)) return;
	
	$name = $dimension->getName();

	$step = 4;
	
	$selector = '#dimension-panel-'.$dimension->getId().' .x-tool.x-tool-options';
	$onclick = 'Ext.getCmp(\'menu-panel\').expand(true); og.highlight_link({selector:\''.$selector.'\', step:'.$step.', time_active:30000, timeout:500, animate_opacity:10, hint_text:\''.lang('click here').'\'}); return false;';
	
	$ws_link = null;
	$tmp_links = array();
	foreach ($links as $l) {
		if ($l['id'] == 'dim_workspaces') {
			$ws_link = $l;
		} else {
			$tmp_links[] = $l;
		}
	}
	$links = $tmp_links;
	
	$links[] = array(
			'id' => 'dim_customer_project',
			'ico' => 'ico-large-projects',
			'url' => get_url('dimension', 'list_members', array('dim' => $dimension->getId())),
			'name' => $name,
			'onclick' => $onclick,
			'extra' => '',
	);
	
	if ($ws_link != null) {
		$links[] = $ws_link;
	}
}

function crpm_add_member_by_type_info($data, &$info) {
	if (array_var($data, 'dim_code') == 'customer_project') {
		$p_ot = ObjectTypes::findByName('project');
		$c_ot = ObjectTypes::findByName('customer');
		$f_ot = ObjectTypes::findByName('folder');
		
		if (array_var($data, 'mem_type') == $p_ot->getId()) {
			$info = array('name' => lang('add new project'), 'url' => get_url('project', 'add'));
		} else if (array_var($data, 'mem_type') == $c_ot->getId()) {
			$info = array('name' => lang('add new client'), 'url' => get_url('customer', 'add'));
		} else if (array_var($data, 'mem_type') == $f_ot->getId()) {
			$dimension = Dimensions::findByCode('customer_project');
			$info = array('name' => lang('add new folder'), 'url' => get_url('member', 'add', array('dim_id' => $dimension->getId(), 'type' => $f_ot->getId())));
		}
	}
}


function crpm_before_edit_contact_form($params, &$ignored) {
	Env::useHelper('functions', 'crpm');
	$object = array_var($params, 'object');
	if ($object instanceof Contact) {
		$customer = Customers::instance()->findOne(array('conditions' => "contact_id=".$object->getId()));
		if ($customer instanceof Customer) {
			$cmember = Members::findOneByObjectId($customer->getId(), get_customers_dimension()->getId());
			if ($cmember instanceof Member) { 
				redirect_to(get_url('customer', 'edit', array('id' => $customer->getId())));
			}
		}
	}
}


function crpm_render_widget_member_information(Member $member, &$prop_html="") {
	if (Plugins::instance()->isActivePlugin('advanced_core')) return;
	
	$p_ot = ObjectTypes::findByName('project');
	$c_ot = ObjectTypes::findByName('customer');

	if ($member->getObjectTypeId() == $p_ot->getId()) {

		if (Plugins::instance()->isActivePlugin('member_custom_properties')) {
			$desc_custom_prop = MemberCustomProperties::getCustomPropertyByCode($member->getObjectTypeId(), 'description_special');

			if($desc_custom_prop instanceof MemberCustomProperty && !$desc_custom_prop->getIsDisabled()){
				return;
			}
		}

		$p = Projects::instance()->findById($member->getObjectId());
		if ($p instanceof Project && $member->getDescription() != "") {
			if($p->getTypeContent() == "text"){
				$content = escape_html_whitespace(convert_to_links(clean($member->getDescription())));
			}else{
				$content = convert_to_links(purify_html(nl2br($member->getDescription())));
			}
			$prop_html .= '<div style="margin-bottom:5px;">'.$content.'</div>';
		}

	} else if ($member->getObjectTypeId() == $c_ot->getId()) {

		// show address, phone and comments of the associated contact
		$client = Customers::instance()->findById($member->getObjectId());
		if ($client instanceof Customer) {

			$assoc_contact = $client->getContact();
			if ($assoc_contact instanceof Contact) {
				// addresses
				$all_addresses = ContactAddresses::instance()->findAll(array('conditions' => 'contact_id='.$assoc_contact->getId()));
				$first_row = true;
				foreach ($all_addresses as $address) {
					$prop_html .= '<tr class="cp-info"><td>';
					if ($first_row) $prop_html .= '<span class="bold">'. lang('address') .':</span>';
					$prop_html .= '</td><td>'. $address->toString() .'</td></tr>';
					$first_row = false;
				}

				// phones
				$all_phones = ContactTelephones::instance()->findAll(array('conditions' => 'contact_id='.$assoc_contact->getId()));
				$first_row = true;
				foreach ($all_phones as $phone) {
					$prop_html .= '<tr class="cp-info"><td>';
					if ($first_row) $prop_html .= '<span class="bold">'. lang('phone number') .':</span>';
					$prop_html .= '</td><td>'. $phone->getNumber() .'</td></tr>';
					$first_row = false;
				}

				// description
				if (!Plugins::instance()->isActivePlugin('member_custom_properties')) {
					$prop_html .= '<tr class="cp-info"><td><span class="bold">'. lang('description') .':</span>';
					$prop_html .= '</td><td>'. $member->getDescription() .'</td></tr>';
				}
			}
		}
	}
}

function crpm_custom_reports_object_types($parameters, &$object_types) {
	Env::useHelper('functions', 'crpm');
	$customers_dim = get_customers_dimension();
	if (in_array($customers_dim->getId(), config_option('enabled_dimensions'))) {
		$object_types[] = ObjectTypes::findByName('customer');
	}

	$projects_dim = Dimensions::findByCode('customer_project');
	if (in_array($projects_dim->getId(), config_option('enabled_dimensions'))) {
		$object_types[] = ObjectTypes::findByName('project');
	}
}


function crpm_object_view_member_path_dims($object, &$ignored) {
	if ($object instanceof Contact) {
		
		$client = Customers::instance()->findOne(array('conditions' => 'contact_id='.$object->getId()));
		if ($client instanceof Customer) {
			
			Env::useHelper('functions', 'crpm');
			$client_member = Members::findOneByObjectId($client->getId(), get_customers_dimension()->getId());
			if ($client_member instanceof Member) {
				$associations = DimensionMemberAssociations::getAllAssociatationsForObjectType($client_member->getDimensionId(), $client_member->getObjectTypeId());
				
				foreach ($associations as $a) {
					
					$assoc_mem_ids = explode(',', MemberPropertyMembers::getAllMemberIds($a->getId(), $client_member->getId()));
					$assoc_mem_ids = array_merge($assoc_mem_ids, explode(',', MemberPropertyMembers::getAllPropertyMemberIds($a->getId(), $client_member->getId())));
					$assoc_mem_ids = array_unique(array_filter($assoc_mem_ids));
					
					if (is_array($assoc_mem_ids) && count($assoc_mem_ids) > 0) {
						$did = $client_member->getDimensionId() == $a->getDimensionId() ? $a->getAssociatedDimensionMemberAssociationId() : $a->getDimensionId();
						$assoc_otid = $client_member->getDimensionId() == $a->getDimensionId() ? $a->getAssociatedObjectType() : $a->getObjectTypeId();
						$dim = Dimensions::getDimensionById($did);
						
						if ($dim->getCode() != 'feng_persons') {
					
							$mems = Members::instance()->findAll(array('conditions' => "id IN (".implode(',', $assoc_mem_ids).")"));
					
							$assoc_ot = ObjectTypes::instance()->findById($assoc_otid);
							$label = ($a->getIsMultiple() || count($assoc_mem_ids) > 1) ? $dim->getName() : lang($assoc_ot->getName());
							
							echo '<div class="member-path-dim-block"><span class="dname coViewAction">'. $label .':&nbsp;</span>';
							
							foreach ($mems as $m) {
								echo '<span class="member-path real-breadcrumb og-wsname-color-'.$m->getColor().'" style="padding: 3px 2px;">'; 
									
								echo '<a href="#" onclick="if (og.additional_on_dimension_object_click['.$m->getObjectTypeId().']) 
									eval(og.additional_on_dimension_object_click['.$m->getObjectTypeId().'].replace(\'<parameters>\', '. $m->getId() .'))">
									'.$m->getName().'</a>';
								
								echo '</span>';
							}
							
							echo '</div>';
						}
					}
					
				}
			}
		}
	}
}






function crpm_additional_member_definition_columns($ot_id, &$ext_cols) {
	$ot = ObjectTypes::instance()->findById($ot_id);
	
	if ($ot instanceof ObjectType && $ot->getName() == 'customer') {
		$ext_cols["contact_addresses"] = array(
			'id' => "contact_addresses",
			'type' => "array",
			'label' => lang("address"),
		);
		$ext_cols["contact_emails"] = array(
			'id' => "contact_emails",
			'type' => "array",
			'label' => lang("email"),
		);
		$ext_cols["contact_phones"] = array(
			'id' => "contact_phones",
			'type' => "array",
			'label' => lang("phone"),
		);
		$ext_cols["picture"] = array(
			'id' => "picture",
			'type' => "image",
			'label' => lang("picture"),
		);
	}
}



function crpm_additional_member_column_values($params, &$info) {
	$definition = array_var($params, 'definition');
	$member = array_var($params, 'member');
	
	$ot = ObjectTypes::instance()->findById($member->getObjectTypeId());
	
	if ($ot instanceof ObjectType && $ot->getName() == 'customer') {
		$customer = Objects::findObject($member->getObjectId());
		if ($customer instanceof Customer) {
			$contact = $customer->getContact();
			if (!$contact instanceof Contact) return;
			/* @var $contact Contact */
			
			$info['picture'] = "";
			if ($contact->getPictureFile() != '') {
				$info['picture'] = get_url('files', 'get_public_file', array('id' => $contact->getPictureFile()));
			}
			
			$info['contact_addresses'] = array();
			$contact_addresses = ContactAddresses::instance()->getByContact($contact);
			foreach ($contact_addresses as $a) {
				/* @var $a ContactAddress */
				$info['contact_addresses'][] = array(
					'street' => $a->getStreet(),
					'city' => $a->getCity(),
					'state' => $a->getState(),
					'country' => $a->getCountryName(),
					'zip_code' => $a->getZipCode(),
				);
			}
			
			$info['contact_emails'] = array();
			$contact_emails = ContactEmails::instance()->findAll(array('conditions' => array("`contact_id` = ?", $contact->getId())));
			foreach ($contact_emails as $e) {
				$info['contact_emails'][] = $e->getEmailAddress();
			}
			
			$info['contact_phones'] = array();
			$contact_phones = ContactTelephones::instance()->findAll(array('conditions' => array("`contact_id` = ?", $contact->getId())));
			foreach ($contact_phones as $p) {
				$info['contact_phones'][] = $p->getNumber();
			}

            $info['webpages'] = array();
            $contact_webpages = ContactWebpages::instance()->findAll(array('conditions' => array("`contact_id` = ?", $contact->getId())));
            foreach ($contact_webpages as $p) {
                $info['webpages'][] = $p->getUrl();
            }
		}
	}
}


function crpm_member_contact_cp_filters($params, &$filters) {
	$cp = array_var($params, 'cp');
	$member = array_var($params, 'member');
	$is_new = array_var($params, 'is_new');
	$ot = array_var($params, 'ot');
	
	// dont let contact cps have values of contacts of other clients
	if ($ot->getName() == 'customer' && $is_new) {
		$filters['member_ids'] = -1;
	}
}

function crpm_new_members_api($params, &$associated_members) {

    $member = array_var($params, 'member');
    $is_api = array_var($params, 'is_api');
    if ($is_api){
        if(config_option('new_members_from_api_with')){

            $customer_stage_ob = ObjectTypes::findByName('customer_stage');
            if ($customer_stage_ob instanceof ObjectType){
                $customer_stage_id = $customer_stage_ob->getId();
                $associated_id = get_associated_dimensions($member->getDimensionId(),$member->getObjectTypeId(),$customer_stage_id);

                $associated_members[$associated_id] = '['.config_option('new_members_from_api_with').']';
            }
        }
    }

}



function crpm_validate_contact_unique_mail($contact, &$do_validate_unique_mail) {
	if (!$contact instanceof Contact || $contact->getUserType() == 0) {
		$do_validate_unique_mail = !config_option('allow_duplicated_contact_emails');
	}
}

function crpm_override_check_existing_email($params, &$return_data) {
	$email = array_var($params, 'email');
	$id_contact = array_var($params, 'id_contact');
	$contact_type = array_var($params, 'contact_type');

	if (config_option('allow_duplicated_contact_emails')) {
		$return_data = array("contact" => array("status" => false));
	}
}






function crpm_after_api_add_timeslot($params, &$modified) {
	$timeslot = array_var($params, 'timeslot');

	if ($timeslot instanceof Timeslot) {
		
		$projects_ot = ObjectTypes::findByName('project');
		$project_member = null;
		
		$ts_members = $timeslot->getMembers();
		foreach ($ts_members as $m) {
			if ($m instanceof Member && $m->getObjectTypeId() == $projects_ot->getId()) {
				$project_member = $m;
			}
		}
		
		if ($project_member instanceof Member) {
			$associations = DimensionMemberAssociations::getAssociatations($project_member->getDimensionId(), $project_member->getObjectTypeId());
			$associated_dimension_ids = array();
			foreach ($associations as $association) {
				$associated_dimension_ids[$association->getId()] = $association->getAssociatedDimensionMemberAssociationId();
			}
			
			$remaining_dimension_ids = array();
			
			foreach ($associated_dimension_ids as $assoc_id => $dim_id) {
				$association_already_set = false;
				foreach ($ts_members as $m) {
					if ($m instanceof Member && $m->getDimensionId() == $dim_id) {
						$association_already_set = true;
						break;
					}
				}
				if (!$association_already_set) {
					$remaining_dimension_ids[$assoc_id] = $dim_id;
				}
			}
				
			foreach ($remaining_dimension_ids as $assoc_id => $dim_id) {
				$assoc = DimensionMemberAssociations::instance()->findById($assoc_id);
				$dim = Dimensions::getDimensionById($dim_id);
				$assoc_member_id = get_associated_status_member_id($project_member, $dim, null, false, $assoc->getCode());
				
				if ($assoc_member_id > 0) {
					$assoc_member = Members::getMemberById($assoc_member_id);
					if ($assoc_member instanceof Member) {
						ObjectMembers::addObjectToMembers($timeslot->getId(), array($assoc_member));
						$modified = true;
					}
				}
			}
		}
	}
}

function crpm_check_unique_custom_properties($params, &$result){
	$pot = ObjectTypes::findByName('project');
	$project_id_cp = CustomProperties::getCustomPropertyByCode($pot->getId(), 'project_id');
	$project = array_var($params, 'project');
	if($project instanceof Project && $project_id_cp instanceof CustomProperty){

		// Get all custom property values assigned custom property 'project_id'
		$condition = '`custom_property_id`='.$project_id_cp->getId() . " AND EXISTS (
			SELECT m.id FROM ".TABLE_PREFIX."members m WHERE m.object_id = ".TABLE_PREFIX."custom_property_values.object_id
		)";
		$all_cpvs = CustomPropertyValues::instance()->findAll(array('conditions' => $condition));
		$cpv_values = array_map(function ($cpv){
			return $cpv->getValue();
		}, $all_cpvs);

		// Get custom property 'project_id' from the form
		$custom_properties = array_var($params, 'custom_properties');
		$project_id_value = array_var($custom_properties, $project_id_cp->getId(), '');

		// Remove current project id from the array of project ids
		$is_new = array_var($params, 'is_new', true);
		if(!$is_new){
			$project_object_id = $project->getObjectId();
			$cpv_condition = '`object_id`='.$project_object_id.' AND `custom_property_id`='.$project_id_cp->getId();
			$proj_cpvs = CustomPropertyValues::instance()->findAll(array('conditions' => $cpv_condition));
			foreach($proj_cpvs as $proj_cpv){
				if (($key = array_search($proj_cpv->getValue(), $cpv_values)) !== false) {
					unset($cpv_values[$key]);
				}
			}
		}

		// Check if 'project_id' is already assigned to the other project
		if(trim($project_id_value) != '' && in_array($project_id_value, $cpv_values)){
			$result['is_unique'] = false;
			$result['error_message'] = $project_id_cp->getName() . ' ' . lang('is already assigned to another project. please use a different') . ' ' . $project_id_cp->getName();
		}
	}
}

function crpm_add_additional_custom_properties_to_view($params, &$cp_html){
	Env::useHelper('functions', 'crpm');
	$object = array_var($params, 'object');
	if ($object instanceof Contact) {
		$customer = Customers::instance()->findOne(array('conditions' => "contact_id=".$object->getId()));
		if ($customer instanceof Customer) {
			$cmember = Members::findOneByObjectId($customer->getId(), get_customers_dimension()->getId());
			if ($cmember instanceof Member) { 
				$params =  array('member' => $cmember, 'visible_by_default' => true);
    			Hook::fire('override_render_properties_for_view', $params, $cp_html);
			}
		}
	}
}

/**
 * Add 'days since created' and 'created on' columns to the to the projects list
 */
function crpm_member_listing_additional_data($params, &$listing_object) {
	$project_ot = ObjectTypes::findByName('project');
	if ($project_ot && array_var($params, 'type_id') == $project_ot->getId()) {
		$count = 0;
		if (isset($listing_object['members'])) {
			foreach ($listing_object['members'] as &$member_data) {
				$object = Objects::instance()->findById($member_data['object_id']);
				$date_created = $object->getCreatedOn();
				$now = DateTimeValueLib::now();
				$timeDiff = DateTimeValueLib::get_time_difference($date_created->getTimestamp(), $now->getTimestamp());
				$member_data['days_since_created'] = $timeDiff['days'];
				$date_format = user_config_option('date_format');
				$member_data['created_on'] = $date_created->format($date_format);
			}
		}

	}
}

function crpm_get_additional_group_property_to_the_view($params, &$group_properties){
	$project_ot = ObjectTypes::findByName('project');
	if ($project_ot && array_var($params, 'object_type_id') == $project_ot->getId()) {
		$group_properties[] = array(
			'property_id' => 'days_since_created',
			'order' => 1
		);
	}
}

function crpm_default_image_file($contact, &$default_img_file)
{
	if ($contact instanceof Contact && $contact->getId() > 0) {
		$customer = Customers::instance()->findOne(array('conditions' => "contact_id=" . $contact->getId()));
		if ($customer instanceof Customer) {
			$customer = get_image_url('default-customer.png');
		}
	}

	if ($contact->getIsCompany()) :
		$default_img_file = 'default-company.png';
	elseif ($customer != null) :
		$default_img_file = 'default-customer.png';
	else :
		$default_img_file = 'default-avatar.png';
	endif;
}

// When deleting a memeber and deleting its objects, exclude all contacts that belong to clients
function crpm_member_delete_more_objects_to_keep($params, &$object_ids_to_keep) {

	$contact_ids_that_are_clients_sql = "
		SELECT contact_id 
		FROM ".TABLE_PREFIX."customers
	";
	$contact_ids_rows = DB::executeAll($contact_ids_that_are_clients_sql);
	$contact_ids_that_are_clients = $contact_ids_rows ? array_flat($contact_ids_rows) : array();

	$object_ids_to_keep = array_merge($object_ids_to_keep, $contact_ids_that_are_clients);

}
