<?php
Hook::register ( "object_prefixes" );

// string to put between prefix and object name
define('OBJECT_PREFIX_SEPARATOR', " - ");

function object_prefixes_render_object_name_prefix($params, &$object_name) {
	
	$object = array_var($params, 'object');
	$class = array_var($params, 'add_class', '');
	$genid = array_var($params, 'genid', gen_id());
	
	$object_types_with_prefix = config_option("use_autonumeric_prefixes");
	
	if ($object instanceof ContentDataObject && !$object->isNew() && in_array($object->getObjectTypeId(), $object_types_with_prefix)) {
		// prefix settings
		$op_settings = get_object_prefix_setting($object);
		
		// prefix object
		$prefix_obj = ObjectPrefixes::instance()->findOne(array("conditions" => "object_id = '".$object->getId()."'"));
		
		if ($prefix_obj instanceof ObjectPrefix) {
			// if max digits changed, update prefix
			if (strlen($prefix_obj->getPrefix()) != $op_settings->getMaxDigits()) {
			
				$num = intval($prefix_obj->getPrefix());
				$num_str = str_pad($num, $op_settings->getMaxDigits(), '0', STR_PAD_LEFT);
			
				$name = $object->getObjectName();
				if ($pos = strpos($name, OBJECT_PREFIX_SEPARATOR) !== false) {
					$name = trim(substr($name, strpos($name, OBJECT_PREFIX_SEPARATOR) + strlen(OBJECT_PREFIX_SEPARATOR)));
					$name = str_replace($prefix_obj->getPrefix() . OBJECT_PREFIX_SEPARATOR, "", $name);
					$object->setObjectName($name);
				}
			
				ObjectPrefixes::updateObjectPrefix($object->getId(), $num_str);
				$prefix = $num_str . OBJECT_PREFIX_SEPARATOR;
			
			} else {
			
				$prefix = $prefix_obj->getPrefix() . OBJECT_PREFIX_SEPARATOR;
			}
			
			// name to set in the text input field
			$object_name = str_replace($prefix, "", $object->getObjectName());
			
			// draw prefix
			echo '<span class="object-prefix '.$class.'">' . $prefix . '</span>';
		}
	}
}


// set prefix in object name
function object_prefixes_after_object_save($object, &$ret) {
	$object_types_with_prefix = config_option("use_autonumeric_prefixes");

	if ($object instanceof ContentDataObject && in_array($object->getObjectTypeId(), $object_types_with_prefix)) {
		
		$op_settings = get_object_prefix_setting($object);
		
		$prefix_obj = get_prefix_for_object($object);
		if ($prefix_obj instanceof ObjectPrefix) {
			$prefix = $prefix_obj->getPrefix();
		}
		
		if ($prefix != '') {
			$name = str_replace("'", "\'", $object->getObjectName());
			
			// if prefix is the same, then remove it to prevent duplicated prefix
			$name = trim(str_replace($prefix . OBJECT_PREFIX_SEPARATOR, "", $name));
			
			// if name has already a different prefix (case of repetitive task) then remove it to prevent two different prefixes
			$pref_pos = strpos($name, OBJECT_PREFIX_SEPARATOR);
			if ($pref_pos > 0 && $pref_pos == strlen($prefix)) {
				$name_offset = $pref_pos + strlen(OBJECT_PREFIX_SEPARATOR);
				$name = trim(substr($name, $name_offset));
			}
			
			// put the prefix preceeding the object's name
			$name = $prefix . OBJECT_PREFIX_SEPARATOR . $name;
			
			// update database and current object
            $object->setObjectName($name);
			DB::execute("UPDATE ".TABLE_PREFIX."objects SET name='". $name ."' WHERE id=".$object->getId());
			DB::execute("UPDATE ".TABLE_PREFIX."members SET name='". $name ."' WHERE object_id=".$object->getId());
			
			// allow search by prefix
			$sql = "INSERT INTO ".TABLE_PREFIX."searchable_objects (rel_object_id, column_name, content)
					VALUES ('".$object->getId()."', 'prefix', '".$prefix."')
					ON DUPLICATE KEY UPDATE content='$prefix'";
			DB::execute($sql);
			
			// if object has an associated member then update member name
			if ($object instanceof DimensionObject) {
				$object_type = ObjectTypes::instance()->findById($object->getObjectTypeId());
				if ($object_type instanceof ObjectType) {
					$_POST[$object_type->getName()]['name'] = $name;
				}

				// build name to display, based on properties and relations, we do it here after saving everything needed
				$member = Members::findOneByObjectId($object->getId());
				if ($member) {
					$display_name = build_member_display_name($member);
					$member->setDisplayName($display_name);
					$member->save();
				}
			}
		}
	}
}

// set prefix in member name
function object_prefixes_after_member_save($params, &$ret) {
	$member = array_var($params, 'member');
	$is_new = array_var($params, 'is_new');
	
	if ($member->getObjectId() == 0) return;
	
	$object = Objects::findObject($member->getObjectId());
	
	$object_types_with_prefix = config_option("use_autonumeric_prefixes");

	if ($object instanceof ContentDataObject && in_array($object->getObjectTypeId(), $object_types_with_prefix)) {

		$op_settings = get_object_prefix_setting($object);

		$prefix_obj = get_prefix_for_object($object);
		if ($prefix_obj instanceof ObjectPrefix) {
			$prefix = $prefix_obj->getPrefix();
		}

		if ($prefix != '') {
			$name = str_replace("'", "\'", $object->getObjectName());
			$name = trim(str_replace($prefix . OBJECT_PREFIX_SEPARATOR, "", $name));
			$name = $prefix . OBJECT_PREFIX_SEPARATOR . $name;
			DB::execute("UPDATE ".TABLE_PREFIX."objects SET name='". $name ."' WHERE id=".$object->getId());
			DB::execute("UPDATE ".TABLE_PREFIX."members SET name='". $name ."' WHERE object_id=".$object->getId());
				
			// if object has an associated member then update member name
			if ($object instanceof DimensionObject) {
				$object_type = ObjectTypes::instance()->findById($object->getObjectTypeId());
				if ($object_type instanceof ObjectType) {
					$_POST[$object_type->getName()]['name'] = $name;
				}

				// build name to display, based on properties and relations, we do it here after saving everything needed
				$member = Members::findOneByObjectId($object->getId());
				if ($member) {
					$display_name = build_member_display_name($member);
					$member->setDisplayName($display_name);
					$member->save();
				}
			}
		}
	}
}


// object prefixes administration
function object_prefixes_general_settings_additional_links($ignored, &$links) {
	$user = logged_user();
	
	if ($user instanceof Contact && can_manage_configuration($user) && count(config_option("use_autonumeric_prefixes")) > 0) {
		$links[] = array(
			'name' => lang('object prefixes'),
			'desc' => lang('object prefixes administration desc'),
			'url' => get_url('object_prefixes', 'list_object_prefix_settings'),
		);
	}
}

function get_object_prefix_setting($object) {
	$op_settings = ObjectPrefixSettings::instance()->findOne(array("conditions" => "object_type_id = '".$object->getObjectTypeId()."'"));
	if (!$op_settings instanceof ObjectPrefixSetting) {
		$op_settings = new ObjectPrefixSetting();
		$op_settings->setObjectTypeId($object->getObjectTypeId());
		$op_settings->setCurrentValue(0);
		$op_settings->setEnabled(true);
		$op_settings->setMaxDigits(5);
		$op_settings->save();
	}
	
	return $op_settings;
}

function get_prefix_for_object($object) {
	$prefix_obj = null;
	$object_types_with_prefix = config_option("use_autonumeric_prefixes");

	if ($object instanceof ContentDataObject && in_array($object->getObjectTypeId(), $object_types_with_prefix)) {

		// get saved prefix, if no prefix is saved then create a new one for this object
		$prefix_obj = ObjectPrefixes::instance()->findOne(array("conditions" => "object_id = '".$object->getId()."'"));
		if (!$prefix_obj instanceof ObjectPrefix) {
			
			$sql = "UPDATE ".TABLE_PREFIX."object_prefix_settings SET current_value=current_value+1 WHERE object_type_id=".$object->getObjectTypeId();
			DB::execute($sql);
			
			$op_settings = get_object_prefix_setting($object);
			$value = $op_settings->getCurrentValue();

			$prefix = str_pad($value, $op_settings->getMaxDigits(), '0', STR_PAD_LEFT);
			
			$prefix_obj = new ObjectPrefix();
			$prefix_obj->setObjectId($object->getId());
			$prefix_obj->setObjectTypeId($object->getObjectTypeId());
			$prefix_obj->setPrefix($prefix);
			$prefix_obj->save();
		}
	}
	return $prefix_obj;
}


function object_prefixes_before_delete($object, &$ignored) {
	$object_types_with_prefix = config_option("use_autonumeric_prefixes");
	
	if ($object instanceof ContentDataObject && in_array($object->getObjectTypeId(), $object_types_with_prefix)) {
		DB::execute("DELETE FROM ".TABLE_PREFIX."object_prefixes WHERE object_id=".$object->getId());
	}
}