<?php
Hook::register ( "crpm_templates" );

function crpm_templates_object_edit_categories($object, &$categories) {
	if ($object instanceof COTemplate) {
		tpl_assign('template', $object);
		$categories[] = array(
			'id' => 'non_working_days_to_do',
			'name' => 'dates and working days',
			'required' => false,
			'visible' => true,
			'content' => tpl_fetch(get_template_path('template_nw_days_to_do', 'non_working_days', 'crpm')),
		);
		
		if (Plugins::instance()->isActivePlugin('mail')) {
			$categories[] = array(
				'id' => 'can_instance_from_mail',
				'name' => 'can instance from mail',
				'required' => false,
				'visible' => true,
				'content' => tpl_fetch(get_template_path('can_instance_from_mail', 'more_template_categories', 'crpm')),
			);
		}
	}
}

function crpm_templates_object_definition($type,&$ret){
	if ($type == 'COTemplate') {
		$ret['nw_days_todo_action'] = DATA_TYPE_STRING;
		$ret['nw_days_todo_amount'] = DATA_TYPE_INTEGER;
		$ret['use_only_working_days'] = DATA_TYPE_BOOLEAN;
		
		if (Plugins::instance()->isActivePlugin('mail')) {
			$ret['can_instance_from_mail'] = DATA_TYPE_BOOLEAN;
		}
	}
}



function crpm_templates_template_param_date_calculation($parameters, &$dateNum) {
	
	$op = array_var($parameters, 'op');
	$date = array_var($parameters, 'date');
	$unit = array_var($parameters, 'unit');
	$template_id = array_var($parameters, 'template_id');
	$original = array_var($parameters, 'original');
	$copy = array_var($parameters, 'copy');

	if (trim($dateNum) == '') {
		$dateNum = 0;
	}
	if (!is_numeric($dateNum)) {
		return;
	}
	
	if (!$date instanceof DateTimeValue || $unit != 'd') return;
	
	$template = COTemplates::instance()->findById($template_id);
	if (!$template instanceof COTemplate) return;

	$only_working_days = $template->getColumnValue('use_only_working_days');

	// Check if the calculations will go forward or backwards, define step
	$step = ($template->getColumnValue('nw_days_todo_action') == "move_back" ? -1 : 1);

	// Define temporary date using $date and $dateNum from parameters
	$tmp_date = new DateTimeValue($date->getTimestamp());
	$tmp_date->add('d', $dateNum);
	/*
	if ($only_working_days) {
		// if starting date is a non working day => advance until a working day is found
		while (NonWorkingDays::instance()->isNonWorkingDay($tmp_date)) {
			$dateNum = $dateNum + $step;
			$tmp_date->add('d', $step);
		}
	}
	*/
	if ($only_working_days) {
		
		$tmp_date = new DateTimeValue($date->getTimestamp());
		$i = 0;
		while ($i < $dateNum) {
			$tmp_date->add('d', 1);
			$i++;
			
			// if starting date is a non working day => advance until a working day is found
			while (NonWorkingDays::instance()->isNonWorkingDay($tmp_date)) {
				$dateNum = $dateNum + $step;
				$tmp_date->add('d', $step);
			}
		}
	}
	
	// Get week days that can be used as due date or start date
	$rep_fixed_days_str = trim($original->getColumnValue('repeat_fixed_days'));

	// check if the previously calculated day is one of the enabled week days in the task.
	if ($rep_fixed_days_str != '') {
		
		$rep_fixed_days = array_filter(explode(',', $rep_fixed_days_str));

		if (count($rep_fixed_days) > 0) {
			$days_to_add = 0;
			
			// check if date is valid, if its weekday is enabled in the original task
			while (!in_array($tmp_date->format('w'), $rep_fixed_days) || ($only_working_days && NonWorkingDays::instance()->isNonWorkingDay($tmp_date, false))) {
				// increase the tmp date in one day to make the next verification
				$tmp_date = $tmp_date->add('d', $step);
				
				// increment the number of days to add if current day is not available
				$days_to_add++;
				
				// hard prevent infinite loop
				if ($days_to_add > 14) break; 
			}
			// add the resulting days to the original amount of days
			$dateNum += $days_to_add * $step;
		}
	}
	
}

function crpm_templates_task_push_dates_calculation($parameters, &$hours) {

	$date = array_var($parameters, 'date');
	$only_working_days = array_var($parameters, 'only_working_days');
	
	$days = floor($hours / 24);
	
	if (!$date instanceof DateTimeValue) return;

	if ($only_working_days) {
		$tmp_date = new DateTimeValue($date->getTimestamp());
		$count = abs($days);
		$dir = $days > 0 ? "asc" : "desc";
		
		$to_add = ($dir == 'asc') ? 1 : -1;
		
		while ($count > 0) {
			$tmp_date->add('d', $to_add);
			while (NonWorkingDays::instance()->isNonWorkingDay($tmp_date)) {
				$hours = $hours + 24 * $to_add;
				$tmp_date->add('d', $to_add);
			}
			$count--;
		}
	}
}


function crpm_templates_additional_email_actions($parameters, &$ret) {
	$email = array_var($parameters, 'email');
	if (can_instantiate_templates(logged_user()) && Plugins::instance()->isActivePlugin('mail') && $email instanceof MailContent) {
		$mail_templates = COTemplates::instance()->findAll(array('conditions' => 'can_instance_from_mail>0'));
		if (is_array($mail_templates)) {
			foreach ($mail_templates as $template) {
				$template_id = $template->getId();
				$template_name = $template->getName();
				
				$template_params = TemplateParameters::getParametersByTemplate($template_id);
				if ($template_params && count($template_params) > 0) {
					$action = 'instantiate_parameters';
				} else {
					$action = 'instantiate';
				}

				$additional_member_ids = json_encode($email->getMemberIds());
				$linked_objects = json_encode(array($email->getId()));
				$action_url = get_url('template', $action, array('id' => $template_id, 'from_email' => $email->getId(), 'additional_member_ids' => $additional_member_ids, 'linked_objects' => $linked_objects));
				add_page_action(lang('new')." $template_name", $action_url, 'ico-template', null, null, true);
			}
		}
	}
}


function crpm_templates_template_parameter_default_value($parameters, &$param_obj) {/* @var $param_obj TemplateParameter */
	if (Plugins::instance()->isActivePlugin('mail')) {
		
		$default_values = array_var($_REQUEST, 'template_default_param_values');
		if (is_array($default_values)) {
			foreach ($default_values as $email_field => $pname) {
				if ($pname == $param_obj->getName()) {
					$param_obj->setDefaultValue('{{'. $email_field .'}}');
				}
			}
		}
	}
}

function crpm_templates_before_instantiating_template_param_def_value($parameters, &$dont_render_this_param) {
	if (Plugins::instance()->isActivePlugin('mail')) {
		
		$parameter = array_var($parameters, 'param');
		$email = MailContents::instance()->findById(get_id('from_email'));
		if (!$email instanceof MailContent) {
			return;
		}
		
		switch (array_var($parameter, 'default_value')) {
			case '{{email_subject}}':
				$dont_render_this_param = true;
				?>
				<tr style='height:30px;'>
					<td style="padding:6px 10px 0 10px;"><b><?php echo $parameter['name']; ?></b></td>
					<td align="left">
						<input name="parameterValues[<?php echo $parameter['name'] ?>]" type="text" class="long" value="<?php echo $email->getSubject()?>"/>
					</td>
				</tr>
				<?php
				break;
			case '{{email_body}}':
				$dont_render_this_param = true;
				?>
				<tr style='height:30px;'>
					<td style="padding:6px 10px 0 10px;"><b><?php echo $parameter['name']; ?></b></td>
					<td align="left">
						<textarea name="parameterValues[<?php echo $parameter['name'] ?>]" style="width:100%;height:200px;overflow:auto;padding:5px"><?php 
							echo html_to_text($email->getBodyHtml());
						?></textarea>
					</td>
				</tr>
				<?php
				break;
			default:
				break;
		}
		
	}
}

