<?php

Hook::register("crpm_types");


function crpm_types_after_dimensions_enabled($enabled_dimension_ids, &$ignored) {
	if (Plugins::instance()->isActivePlugin('member_custom_properties')) {
		$p_status_dimension = Dimensions::findByCode("project_statuses");
		if ($p_status_dimension instanceof Dimension) {
			$disable_cp = in_array($p_status_dimension->getId(), $enabled_dimension_ids) ? "1" : "0";
			DB::executeAll("
				UPDATE ".TABLE_PREFIX."member_custom_properties SET is_disabled=$disable_cp
				WHERE `code` IN ('status_special','calculate the status automatically_special')
					AND object_type_id=(SELECT id FROM ".TABLE_PREFIX."object_types WHERE name='project');
			");
		}
	}
}



function crpm_types_custom_reports_additional_columns($args, &$ret) {
	// the dim columns are added by this plugin too
	if (Plugins::instance()->isActivePlugin('advanced_reports')) return;
	
	$ot = array_var($args, 'object_type');
	if ($ot instanceof ObjectType && in_array($ot->getType(), array('dimension_object','dimension_group'))) return;
	
	$dimensions = Dimensions::instance()->findAll(array('conditions' => "code in ('customer_types','project_types','project_statuses','customers')"));
	foreach ($dimensions as $dimension) {
		
		$name = $dimension->getName();
		
		$ret[] =  array('id' => 'dim_'.$dimension->getId(), 'name' => $name, 'type' => DATA_TYPE_STRING);
	}
}


function crpm_types_member_quick_add_urls($parameters, &$urls) {
	Env::useHelper('functions', 'crpm');
	$dimension = array_var($parameters, 'dimension');
	$client_ot = ObjectTypes::findByName('customer');
	$clean_urls = array();
	if ($dimension->getCode() == 'customer_project' && get_customers_dimension()->getCode() != 'customer_project') {
		foreach ($urls as $k => $url) {
			if (strpos($url['url'], 'c=customer&a=add&type='.$client_ot->getId()) === false) {
				$clean_urls[] = $url;
			}
		}
		$urls = $clean_urls;
	}
}



function crpm_types_after_auto_classifying_associated_object_of_member($parameters, &$ignored) {
	$rel_obj = array_var($parameters, 'obj');
	$member = array_var($parameters, 'mem');
	if (!$member instanceof Member) return;
	
	if ($rel_obj instanceof Customer) {
		$rel_contact = $rel_obj->getContact();
		if ($rel_contact instanceof Contact) {
			ObjectMembers::addObjectToMembers($rel_contact->getId(), array($member));
		}
	}
}


function crpm_types_can_add_modify_context_array($parameters, &$context) {
	$project_phase_dimension_method = Dimensions::findByCode('project_phases')->getPermissionQueryMethod();
	if($project_phase_dimension_method == 'mandatory'){
		$project_phase_object_type_id = ObjectTypes::findByName('project_phase')->getId();
		$member_to_add = null;
		if( isset($context)){
		foreach($context as $selection){
			if($selection instanceof Member){
				if($selection->getObjectTypeId() == ObjectTypes::findByName('project')->getId()){
					
					$project_members = MemberPropertyMembers::getAllAssociatedMemberIds($selection->getId(), true);
					foreach($project_members as $assoc_id => $pm_ids){

						foreach ($pm_ids as $pm_id) {
							$property_member = Members::instance()->findById($pm_id);
							if($property_member->getObjectTypeId() == $project_phase_object_type_id){
								$member_to_add = $property_member;
								break;
							}
						}
					}
					if(!is_null($member_to_add)){
						$context[] = $member_to_add;
					}
					break;
				}
			}
		}
		}
	}
}

function crpm_types_can_access_modify_members_array($parameters, &$context) {

	$user = array_var($parameters, 'user');
	$object_type_id = array_var($parameters, 'object_type_id');
	$access_level = array_var($parameters, 'access_level');

	$phases_dimension = Dimensions::findByCode('project_phases');
	$project_phase_ot_id = ObjectTypes::findByName('project_phase')->getId();
	$project_ot_id = ObjectTypes::findByName('project')->getId();
	
	$project_present = false;
	$keys_to_remove = array();
	$members_to_add = array();

	foreach($context as $k => $selection){

		// collect the phases given in context so we can remove them before checking permissions
		if ($selection instanceof Member && $selection->getObjectTypeId() == $project_phase_ot_id) {
			$keys_to_remove[] = $k;
		}

		// find the phase associated to the project so we can add it to the context to check for permissions
		if ($selection instanceof Member && $selection->getObjectTypeId() == $project_ot_id) {
			$phase_mem_id = get_associated_status_member_id($selection, $phases_dimension);
			if ($phase_mem_id > 0) {
				$phase_mem = Members::getMemberById($phase_mem_id);
				if ($phase_mem instanceof Member) {
					$members_to_add[] = $phase_mem;
				}
			}
			$project_present = true;
		}
	}

	// only make changes to the context if we have a project and if the project has a phase
	if ($project_present && !empty($members_to_add)) {
		// remove given project phase if any
		foreach($keys_to_remove as $key){
			unset($context[$key]);
		}
	
		// add to the context the project phase associated to the project
		foreach($members_to_add as $member){
			$context[] = $member;
		}
	}

}
