<?php 
/*
 *  DO NOT DELETE. this is needed for  plugins to work
 */
class EinvoiceController extends ApplicationController {
	
	var $plugin_name = "einvoice";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('einvoice_provider', 'einvoice');
	}

	function generate_einvoice(){
		$invoice = IncomeInvoices::instance()->findById(get_id());

		if(array_var($_SESSION,'sending_invoice') == get_id()){
            ajx_current("empty");
            return;
        }else{
            $_SESSION['sending_invoice'] = get_id();
        }

		if (!($invoice instanceof IncomeInvoice)) {
			flash_error(lang('invoice dnx'));
			ajx_current("empty");
			return;
		} // if
		
		$error_generating_einvoice = false;

		try {
            if ($invoice->getStatus() == 'printed'){
                flash_error(lang('invoice dnx'));
                ajx_current("empty");
                return;
            }

			$status_changed = false;
			if ($invoice->getStatus() == 'pending' && !array_var($_REQUEST, 'generate_preview')) {
            
	            // check that all mandatory fields are present
	            $validation_errors = array();
	            $invoice->validate($validation_errors);
	            if (count($validation_errors) > 0) {
	            	throw new Exception(lang("cannot generate einvoice") ."\n". implode(',', $validation_errors));
	            }
            
	            try {
					DB::beginWork();
					$invoice->setStatus('printed');
					$invoice->save();
					DB::commit();
					$status_changed = true;
				} catch (Exception $ex) {
					DB::rollback();
					throw $ex;
				}

                $generate_next_invoice = IncomeConfigOptions::instance()->findOne(array('conditions' => "name='generate_next_invoice_repetition_on_print'"));
                $generate_next_invoice = $generate_next_invoice instanceof IncomeConfigOption && $generate_next_invoice->getValue();

                if($generate_next_invoice){
                    try {
                        $invoice->generateNextRepetition();
                    } catch (Exception $e) {
                        Logger::log('Error invoice generateNextRepetition: '.$invoice->getId());
                        Logger::log($e->getMessage());
                        Logger::log($e->getTraceAsString());
                    }
                }
			}

            EinvoiceProvider::instance()->send_invoice($invoice);

			// reload invoice view, status has changed
			if ($status_changed) {
				evt_add("reload current panel");
			}

			unset($_SESSION['sending_invoice']);
			ajx_current("empty");
			
		} catch (Exception $e) {
			if ($status_changed) {
				// return the invoice to the pending status
				try {
					DB::beginWork();
					$invoice->setStatus('pending');
					$invoice->save();
					DB::commit();
				} catch (Exception $ex) {
					DB::rollback();
					throw $ex;
				}
			}
			
			$error_generating_einvoice = true;
			
			flash_error($e->getMessage());
            unset($_SESSION['sending_invoice']);
			ajx_current("empty");
		}

		//Download invoice document
		if($status_changed && !$error_generating_einvoice){
			$this->get_einvoice_document();
		}
	}

	function get_einvoice_document(){
		$invoice = IncomeInvoices::instance()->findById(get_id());
		if (!($invoice instanceof IncomeInvoice)) {
			flash_error(lang('invoice dnx'));
			ajx_current("empty");
			return;
		} // if

		$file = EinvoiceProvider::instance()->get_invoice_document($invoice);

		if($file instanceof ProjectFile){
			$invoice->linkObject($file);
			$invoice->setColumnValue('einvoice_document_id',$file->getId());
			$invoice->save();
		}

		if (config_option('save_einvoice_qr')) {

			$qr_file = EinvoiceProvider::instance()->generateQR($invoice);
			
			if($qr_file instanceof ProjectFile){
				// link the qr file to the invoice
				$invoice->linkObject($qr_file);
				$invoice->setColumnValue('einvoice_qr_file_id',$qr_file->getId());
				$invoice->save();
				// don't show the qr file in the documents tab
				$qr_file->archive();
			}
		}

		ajx_current("empty");
		evt_add("reload current panel");
	}

	/**
	 * View a invoice in a printer-friendly format.
	 *
	 */
	function print_view() {
		$this->setLayout("html");
		$this->addHelper('textile');

		$invoice = IncomeInvoices::instance()->findById(get_id());
		if (!($invoice instanceof IncomeInvoice)) {
			flash_error(lang('invoice dnx'));
			ajx_current("empty");
			return;
		} // if

		if (!$invoice->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		} // if

		$file = ProjectFiles::instance()->findById($invoice->getColumnValue('einvoice_document_id'));
		if (!($file instanceof ProjectFile)) {
			flash_error(lang('file dnx'));
			ajx_current("empty");
			return;
		} // if

		$qr_file = ProjectFiles::instance()->findById($invoice->getColumnValue('einvoice_qr_file_id'));
		if ($qr_file instanceof ProjectFile) {
			tpl_assign('qr', $qr_file);
		} // if

		tpl_assign('invoice', $invoice);
		tpl_assign('file', $file);
	}


	function update_config() {
		if (!can_manage_invoicing_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}

		$options = ConfigOptions::instance()->findAll(array("condition" => array("category_name='einvoice' AND is_system=0")));
		tpl_assign('options', $options);

		$submited_values = array_var($_POST, 'options');
		if (is_array($submited_values)) {
			foreach ($options as $option) {
				$new_value = array_var($submited_values, $option->getName());
				if (is_null($new_value) || ($new_value == $option->getValue()))
					continue;

				$option->setValue($new_value);
				$option->save();
			}
			flash_success(lang('success update config'));
			ajx_current("back");
		} // if
	}
}