<?php

/**
 * Event Attendee controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class EventAttendeeController extends ApplicationController
{

    var $plugin_name = "event_tickets";

    function __construct()
    {
        parent::__construct();
        prepare_company_website_controller($this, 'website');
        Env::useHelper('format');
        Env::useHelper('functions', 'event_tickets');
    }


    function submit_multiple_attendees()
    {

        if (logged_user()->isGuest()) {
            flash_error(lang('no access permissions'));
            ajx_current('empty');
            return;
        }

        $data = array_var($_POST, 'attendees');
        $data_tickets = array_var($_POST, 'tickets');
        
        $event_member = null;

        $event_attendees = array();
        foreach ($data as $ticketnum => $attendees_by_ticketnum) {
        	if (is_numeric($ticketnum)) {
        		foreach ($attendees_by_ticketnum as $attendee) {
        			
	                if (!$event_member instanceof Member) {
	                    $et = EventTickets::instance()->findById($attendee['event_ticket_id']);
	                    if ($et instanceof EventTicket) {
	                        $tt = EventTicketTypes::getEventTicketById($et->getTicketTypeId());
	                        $event_member = Members::findOneByObjectId($tt->getEventId());
	                    }
	                }
	
	                $event_attendees[] = array(
	                    'contact_id' => array_var($attendee, 'event_contact_id'),
	                    'event_ticket_id' => array_var($attendee, 'event_ticket_id'),
	                    'contact_email_address' => array_var($attendee, 'email'),
	                    'contact_first_name' => array_var($attendee, 'first_name'),
	                    'contact_surname' => array_var($attendee, 'surname'),
	                );
        		}
        	}
        }

        $event_tickets = array();
        foreach ($data_tickets as $ticket_id => $values){
            $ticket_ob = EventTickets::instance()->findById($ticket_id);
            if ($ticket_ob instanceof EventTicket){
                ObjectController::add_custom_properties($ticket_ob,$values);
            }
        }

        $member_ids = active_context_members(false);
        
        $member_ids[] = $data["event_member_id"];
        if (array_var($data, "customer_member_id")) {
        	$member_ids[] = $data["customer_member_id"];
        }
        if ($event_member instanceof Member) {
            $member_ids[] = $event_member->getId();
        }
        $member_ids = array_unique(array_filter($member_ids));
        $params = array('event_attendees' => $event_attendees,'event_tickets_cp'=>$event_tickets);
        try {
            DB::beginWork();

            $new_attendees = $this->save_event_attendee_bulk($params, $member_ids);
            $new_cp =  EventTicketTypeController::save_event_ticket_cp_bulk($params, $member_ids);


            DB::commit();

            ajx_current("empty");
            ajx_extra_data(array('attendees' => $new_attendees));
            flash_success(lang("success add attendees"));

        } catch (Exception $e) {
            DB::rollback();
            ajx_current("empty");
            flash_error($e->getMessage());
        }
    }


    function add()
    {

        if (logged_user()->isGuest()) {
            flash_error(lang('no access permissions'));
            ajx_current('empty');
            return;
        }

        $event_attendee_data = array_var($_POST, 'event_attendee');

        if (!is_array($event_attendee_data)) {
            // set layout for modal form
            if (array_var($_REQUEST, 'modal')) {
                $this->setLayout("json");
                tpl_assign('modal', true);
            }
            $obj = new EventAttendee();
            $obj->setObjectTypeId(EventAttendees::instance()->getObjectTypeId());
            $obj->setEventTicketId(get_id('event_ticket_id'));

            tpl_assign('event_attendee', $obj);

        } else {
            // form submitted
            try {
                DB::beginWork();

                $member_ids = json_decode(array_var($_POST, 'members'));

                $event_attendee = $this->save_event_attendee($event_attendee_data, $member_ids);

                DB::commit();

                ApplicationLogs::createLog($event_attendee, ApplicationLogs::ACTION_ADD);
                flash_success(lang('success add event_attendee', $event_attendee->getName()));

                ajx_current("back");
                if (array_var($_REQUEST, 'modal')) {
                    evt_add("reload current panel");
                }

            } catch (Exception $e) {
                DB::rollback();
                flash_error($e->getMessage());
                Logger::log($e->getMessage() . "\n" . get_back_trace() . "\n" . $e->getTraceAsString());
                ajx_current("empty");
            }
        }

    }


    function edit()
    {
        $this->setTemplate('add');

        $event_attendee = EventAttendees::instance()->findById(get_id());
        if (!($event_attendee instanceof EventAttendee)) {
            flash_error(lang('event_ticket dnx'));
            ajx_current("empty");
            return;
        }

        if (logged_user()->isGuest() || !$event_attendee->canEdit(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current('empty');
            return;
        }

        $event_attendee_data = array_var($_POST, 'event_attendee');

        if (!is_array($event_attendee_data)) {
            // set layout for modal form
            if (array_var($_REQUEST, 'modal')) {
                $this->setLayout("json");
                tpl_assign('modal', true);
            }

            tpl_assign('event_attendee', $event_attendee);

        } else {
            // form submitted
            try {
                DB::beginWork();

                $member_ids = json_decode(array_var($_POST, 'members'));

                $event_attendee = $this->save_event_attendee($event_attendee_data, $member_ids, $event_attendee);

                ApplicationLogs::createLog($event_attendee, ApplicationLogs::ACTION_EDIT);

                DB::commit();

                flash_success(lang('success edit event_ticket', $event_attendee->getName()));

                ajx_current("back");
                if (array_var($_REQUEST, 'modal')) {
                    evt_add("reload current panel");
                }

            } catch (Exception $e) {
                DB::rollback();
                flash_error($e->getMessage());
                Logger::log("Error editing attendee" . $e->getMessage() . "\n" . $e->getTraceAsString());
                ajx_current("empty");
            }
        }
    }


    /**
     * Save EventAttendee object
     * @param array $event_attendee_data Array with the event attendee data
     * @param array $member_ids where the object will be classified
     * @param EventAttendee $event_attendee existing object, use only when editing
     * @return EventAttendee
     */
    function save_event_attendee($event_attendee_data, $member_ids, $event_attendee = null)
    {
        // Save event_attendee object

        $is_new = false;
        if (is_null($event_attendee) || !$event_attendee instanceof EventAttendee) {
            $event_attendee = new EventAttendee();
            $is_new = true;
        }

        // dont allow to add more attendees than the specified in the ticket type
        $event_ticket = EventTickets::instance()->findById(array_var($event_attendee_data, 'event_ticket_id'));
        if ($event_ticket instanceof EventTicket) {
            $att_id_cond = $is_new ? "" : " AND id<>'" . $event_attendee->getId() . "'";

            $attendee_count = EventAttendees::instance()->count(array("trashed_by_id=0 AND event_ticket_id = ? $att_id_cond", $event_ticket->getId()));
            $tt = EventTicketTypes::getEventTicketById($event_ticket->getTicketTypeId());

            if ($attendee_count > $tt->getAttendeesPerTicket()) {
                throw new Exception("cannot add more attendees to the current ticket");
            }
        }

        // if contact id not sent, try to find contact using "contact_email_address", if not found -> try to create the contact
        if (!array_var($event_attendee_data, 'contact_id') && array_var($event_attendee_data, 'contact_email_address')) {
            $contact_email = array_var($event_attendee_data, 'contact_email_address');
            // if duplicated emails are allowed then get the contact using the first name and surname too
            if (config_option('allow_duplicated_contact_emails')) {
            	$contact = Contacts::getByFirstnameAndSurname(trim($event_attendee_data['contact_first_name']), trim($event_attendee_data['contact_surname']), $contact_email);
            } else {
            	$contact = Contacts::getByEmail($contact_email);
            }
            
            if (!$contact instanceof Contact) {
                $contact = create_contact_from_ticket_data($event_attendee_data, $member_ids);
            }

            if ($contact instanceof Contact) {
                $event_attendee_data['contact_id'] = $contact->getId();
            }

            if ($event_ticket instanceof EventTicket) {
	            $tt = EventTicketTypes::getEventTicketById($event_ticket->getTicketTypeId());
	            if ($tt instanceof EventTicketType) {
	                $event_member = Members::findOneByObjectId($tt->getEventId());
	                if ($event_member instanceof Member) {
	                    $member_ids[] = $event_member->getId();
	                    $member_ids = array_unique(array_filter($member_ids));
	                }
	            }
            }
        }

        // override the object's name with its contact's name
        if (array_var($event_attendee_data, 'contact_id')) {
            $contact = Contacts::instance()->findById(array_var($event_attendee_data, 'contact_id'));
            if ($contact instanceof Contact) {
                $event_attendee->setObjectName($contact->getObjectName());
            }
        }


        // set attributes to the EventAttendee object
        $event_attendee->setFromAttributes($event_attendee_data);

        // save the EventAttendee
        $event_attendee->save();
        
        
        // put the attendee in the member that made the purchase
        $invoice = null;
        if ($event_ticket instanceof EventTicket) {
        	$invoice = Objects::instance()->findObject($event_ticket->getInvoiceId());
        }
        if ($invoice instanceof ContentDataObject) {
        	$clients_ot = ObjectTypes::findByName('customer');
        	$client_member = null;
        	$inv_members = $invoice->getMembers();
        	foreach ($inv_members as $mem) {
        		if ($mem->getObjectTypeId() == $clients_ot->getId()) {
        			$client_member = $mem;
        			break;
        		}
        	}
        	if ($client_member instanceof Member) {
        		$member_ids[] = $client_member->getId();
        	}
        }
        
        // classify web attendees in web_classes_stage_member
        if (array_var($event_attendee_data, 'from_web')) {
	        $web_classes_stage_member_id = config_option('web_classes_stage_member');
	        if ($web_classes_stage_member_id > 0) {
	        	$member_ids[] = $web_classes_stage_member_id;
	        }
        }
        

        // save the basic ContentDataObject properties
        $object_controller = new ObjectController();
        if (!is_null($member_ids)) {
        	$object_controller->add_to_members($event_attendee, $member_ids, null, false);
            if ($contact instanceof Contact) {

                /**
                 * This for each is because if we dont put all the members in the add_to_member, that function remove some of that
                 */
                foreach ($contact->getMembers() as $member){
                    /** @var $member Member */
                    $member_ids[] = $member->getColumnValue('id');
                }
                $object_controller->add_to_members($contact, $member_ids);
            }
        }
        $object_controller->add_subscribers($event_attendee);
        $object_controller->link_to_new_object($event_attendee);
        $object_controller->add_custom_properties($event_attendee);


        return $event_attendee;
    }

    function save_event_attendee_bulk($params, $member_ids)
    {

        if (isset($params) && is_array($params) && count(array_var($params, 'event_attendees', array())) > 0) {
            $response_data = array();
            try {
            	// filter non numeric member ids
            	$member_ids = array_filter($member_ids, "is_numeric");
            	
                foreach ($params['event_attendees'] as $attendee) {
                    $event_attendee_data = array(
                        "event_ticket_id" => array_var($attendee, 'event_ticket_id', 0),
                        "contact_email_address" => array_var($attendee, 'contact_email_address', ''),
                        "contact_first_name" => array_var($attendee, 'contact_first_name', ''),
                        "contact_surname" => array_var($attendee, 'contact_surname', ''),
                        "contact_id" => array_var($attendee, 'contact_id', null),
                        "from_web" => array_var($attendee, 'from_web', null)
                    );
                    $attendee_info = $this->save_event_attendee($event_attendee_data, $member_ids);
                    if ($attendee_info instanceof EventAttendee) {
                        $response_data[] = $attendee_info->getArrayInfo();
                    }

                }

                return $response_data;

            } catch (Exception $e) {
                Logger::log("Error saving event attendee: " . $e->getMessage() . "\n" . $e->getTraceAsString());
                throw $e;
            }
        }

        return array();
    }



    function delete()
    {
        $event_attendee = EventAttendee::instance()->findById(get_id());
        if (!($event_attendee instanceof EventAttendee)) {
            flash_error(lang('event_ticket dnx'));
            ajx_current("empty");
            return;
        }

        if (logged_user()->isGuest() || !$event_attendee->canDelete(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current('empty');
            return;
        }

        try {
            DB::beginWork();

            $event_attendee->trash();
            $name = $event_attendee->getName();

            ApplicationLogs::createLog($event_attendee, ApplicationLogs::ACTION_DELETE);
            DB::commit();

            flash_success(lang('success delete event_attendee', $name));

            ajx_current("back");
            if (array_var($_REQUEST, 'modal')) {
                evt_add("reload current panel");
            }

        } catch (Exception $e) {
            DB::rollback();
            ajx_current("empty");
            flash_error($e->getMessage());
        }
    }


    function list_all()
    {

        ajx_current("empty");
        // Get all variables from request
        $start = array_var($_GET, 'start', 0);
        $limit = array_var($_GET, 'limit', config_option('files_per_page'));
        $order = array_var($_GET, 'sort');
        $order_dir = array_var($_GET, 'dir');
        $action = array_var($_GET, 'action');
        $attributes = array("ids" => explode(',', array_var($_GET, 'ids')));
        $dim_order = null;
        $cp_order = null;

        $event_ticket_id = array_var($_GET, 'event_ticket_id');

        $actionMessage = array();
        if (isset($action)) {
            $actionMessage = $this->resolveAction($action, $attributes);
            if ($actionMessage["errorCode"] == 0) {
                flash_success($actionMessage["errorMessage"]);
            } else {
                flash_error($actionMessage["errorMessage"]);
            }
        }

        $select_columns = array("*");
        $extra_conditions = "";
        $join_params = null;

        if (str_starts_with($order, "dim_")) {
            $dim_order = substr($order, 4);
            $order = 'dimensionOrder';
        } else if (str_starts_with($order, "cp_")) {
            $cp_order = substr($order, 3);
            $order = 'customProp';
        } else {
            switch ($order) {
                case 'name':
                    $order = 'contact_name';
                    $join_params = array(
                        'table' => Objects::instance()->getTableName(),
                        'jt_field' => 'id',
                        'join_type' => 'LEFT',
                        'e_field' => 'contact_id',
                    );
                    $select_columns = array("e.*, o.*, jt.`name` as contact_name");
                    break;
                default:
                    $order = "name";
                    break;
            }
        }

        Hook::fire("listing_extra_conditions", array('manager' => EventAttendees::instance(), 'request' => $_REQUEST), $extra_conditions);

        if ($event_ticket_id) {
            $extra_conditions .= " AND e.event_ticket_id='$event_ticket_id' ";
        }

        $only_count_result = array_var($_GET, 'only_result', false);

        $result = EventAttendees::instance()->listing(array(
            "order" => $order,
            "order_dir" => $order_dir,
            "dim_order" => $dim_order,
            "cp_order" => $cp_order,
            "select_columns" => $select_columns,
            "extra_conditions" => $extra_conditions,
            "count_results" => false,
            "only_count_results" => $only_count_result,
            "join_params" => $join_params,
            "start" => $start,
            "limit" => $limit,
        ));

        $event_attendees = $result->objects;
        $object = $this->prepareObject($event_attendees, $start, $limit, $result);

        ajx_current("empty");
        ajx_extra_data($object);
        tpl_assign("listing", $object);

    }

    private $event_attendee_cps = null;

    private function prepareObject($event_attendees, $start, $limit, $res_obj)
    {
        $object = array(
            "totalCount" => $res_obj->total,
            "start" => $start,
            "event_attendees" => array()
        );
        foreach ($res_obj as $k => $v) {
            if ($k != 'total' && $k != 'objects') $object[$k] = $v;
        }

        if (!$this->event_attendee_cps) {
            $this->event_attendee_cps = CustomProperties::getAllCustomPropertiesByObjectType(EventAttendees::instance()->getObjectTypeId());
        }
        $custom_properties = $this->event_attendee_cps;

        $count = min(array($limit, count($event_attendees)));
        for ($i = 0; $i < $count; $i++) {
            if (isset($event_attendees[$i])) {
                $event_attendee = $event_attendees[$i];
                if ($event_attendee instanceof EventAttendee) {
                    $info = $event_attendee->getArrayInfo(true);

                    $name = "";
                    if ($event_attendee->getContactId() > 0) {
                        $contact = Contacts::instance()->findById($event_attendee->getContactId());
                        if ($contact instanceof Contact) {
                            $name = $contact->getObjectName();
                        }
                    }
                    $info['name'] = $name;

                    $info['memPath'] = json_encode($event_attendee->getMembersIdsToDisplayPath(true));
                    $object["event_attendees"][$i] = $info;

                    foreach ($custom_properties as $cp) {
                        $object["event_attendees"][$i]['cp_' . $cp->getId()] = get_custom_property_value_for_listing($cp, $event_attendee);
                    }
                }
            }
        }

        return $object;
    }


    private function resolveAction($action, $attributes)
    {

        $resultMessage = "";
        $resultCode = 0;
        switch ($action) {
            case "delete":
                $succ = 0;
                $err = 0;
                $messages = array();
                for ($i = 0; $i < count($attributes["ids"]); $i++) {
                    $id = $attributes["ids"][$i];
                    $attendee = EventAttendees::instance()->findById($id);
                    if ($attendee instanceof EventAttendee && $attendee->canDelete(logged_user())) {
                        try {
                            DB::beginWork();
                            $attendee->trash();
                            DB::commit();
                            ApplicationLogs::createLog($attendee, ApplicationLogs::ACTION_TRASH);
                            $succ++;
                        } catch (Exception $e) {
                            DB::rollback();
                            $messages[] = $e->getMessage();
                            $err++;
                        }
                    } else {
                        $err++;
                        $messages[] = lang('cannot delete attendee', $attendee instanceof EventAttendee ? clean($attendee->getName()) : '');
                    }
                }; // for
                if ($err > 0) {
                    $resultCode = 2;
                    $resultMessage = lang("error delete objects", $err) . ("\n - " . implode("\n - ", $messages)) . ($succ > 0 ? "\n" . lang("success delete objects", $succ) : "");
                } else {
                    $resultMessage = lang("success delete objects", $succ);
                }
                break;
            default:
                $resultMessage = lang("Unimplemented action: '" . $action . "'");// if
                $resultCode = 2;
                break;
        } // switch
        return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
    }


}
