<?php
/**
 * Event Ticket controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class EventTicketController extends ApplicationController {

	var $plugin_name = "event_tickets";

	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('format');
		Env::useHelper('functions', 'event_tickets');
	}

	function init() {
        $dim = Dimensions::findByCode('managed_events');

        $config = array(
            'event_manager_dim_id' => $dim->getId(),
        	'use_classes_workflow' => config_option('use_classes_workflow'),
        );

		require_javascript("EventTicketManager.js", $this->plugin_name);
		ajx_current("panel", "event-tickets-containerpanel", null, $config, true);
		ajx_replace(true);
	}


	function add_multiple() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}

		$active_context = active_context();

		$event_member = null;
		$managed_event_ot = ObjectTypes::findByName('managed_event');
		foreach ($active_context as $selection) {
			if ($selection instanceof Member && $selection->getObjectTypeId() == $managed_event_ot->getId()) {
				$event_member = $selection;
				break;
			}
		}

        $ticket_quantity = PHP_INT_MAX;
		if (!$event_member instanceof Member) {
			flash_error(lang('you have to select an event to add tickets'));
			ajx_current('empty');
			return;
		}else{
            $cp = CustomProperties::getCustomPropertyByCode($event_member->getObjectTypeId(),'event_ticket_quantity');
            if ($cp instanceof CustomProperty){
                if (!$cp->getIsDisabled()) {
                    $cp_value = CustomPropertyValues::getCustomPropertyValue($event_member->getObjectId(),$cp->getId());
                    if ($cp_value instanceof CustomPropertyValue){
                        $ticket_quantity_total_per_event = $cp_value->getValue();
                        if ($ticket_quantity_total_per_event > 0){
                            $total_tickets = 0;
                            $tickets_type = EventTickets::findAllByEventId($event_member->getObjectId());
                            $total_tickets = count($tickets_type);

                            $ticket_quantity = $ticket_quantity_total_per_event - $total_tickets;
                            if ($ticket_quantity_total_per_event <= $total_tickets){
                                // return the generated data
                                flash_error(lang('Sorry, this event is sold out'));
                                ajx_current('empty');
                                return;
                            }
                        }
                    }
                }
            }
        }


		$event_ticket_types = EventTicketTypes::findAllByEventId($event_member->getObjectId());

        tpl_assign("c_sym", config_option('currency_code'));
		tpl_assign("event_member", $event_member);
		tpl_assign("event_ticket_types", $event_ticket_types);
        tpl_assign("event_ticket_available", $ticket_quantity);

		if (config_option('use_classes_workflow')){
            $this->setTemplate('add_multiple_classes');
        };
	}



	function submit_multiple_event_tickets() {

		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}

		$data = array_var($_POST, 'tickets');
		$all_data = $_POST;

		//this members come like a array but in a string example "[1036]"
        // so we need to remove thats '[' and ']'
		$member_selection = (($_POST['members'])?str_replace("[","",str_replace("]","",$_POST['members'])):null);

		if (is_array($data)) {

			$member_ids = active_context_members(false);
			$member_ids[] = $data["event_member_id"];
            $member_ids[] = $member_selection;
			$member_ids = array_unique(array_filter($member_ids));

			$now = DateTimeValueLib::now();
			$currency_id = null;

			$event_tickets = array();
			$invoice_lines = array();
			$ticket_quantity_total = 0;
			$total_price = 0;
			foreach ($data['types'] as $ticket_type_id => $ticket_type_data) {
			    if($ticket_type_data['quantity']<=0){
			        continue;
                }
                $ticket_quantity_total += $ticket_type_data['quantity'];
				$tt = EventTicketTypes::getEventTicketById($ticket_type_id);
				if (!$tt instanceof EventTicketType) continue;
				if ($currency_id == null) $currency_id = $tt->getCurrencyId();

				$invoice_lines[] = array(
						'amount' => $ticket_type_data['quantity'],
						'unit_value' => $tt->getAmount(),
						'total' => $tt->getAmount() * $ticket_type_data['quantity'],
						'currency_id' => $currency_id,
						'detail' => $tt->getName()
						
				);
				$total_price += $tt->getAmount() * $ticket_type_data['quantity'];

				$event_tickets[] = array(
						'ticket_type_id' => $ticket_type_id,
						'quantity' => $ticket_type_data['quantity'],
						'comment' => $ticket_type_data['comment'],
						'requested_by_id' => array_var($data['requested_by'], 'contact_id'),
				);
			}


			if ($data["event_member_id"] != ""){
			    $member_event_id = $data["event_member_id"];

			    $member_event = Members::instance()->findById($member_event_id);
			    if ($member_event instanceof Member){




                    $cp = CustomProperties::getCustomPropertyByCode($member_event->getObjectTypeId(),'event_ticket_quantity');
                    if ($cp instanceof CustomProperty){
                        if (!$cp->getIsDisabled()) {
                            $cp_value = CustomPropertyValues::getCustomPropertyValue($member_event->getObjectId(),$cp->getId());
                            if ($cp_value instanceof CustomPropertyValue){
                                $ticket_quantity_total_per_event = $cp_value->getValue();
                                if ($ticket_quantity_total_per_event > 0){
                                    $total_tickets = 0;
                                    $tickets_type = EventTickets::findAllByEventId($member_event->getObjectId());
                                    $total_tickets = count($tickets_type);

                                    if ($ticket_quantity_total_per_event < $total_tickets){
                                        // return the generated data
                                        ajx_current('empty');
                                        ajx_extra_data(array(
                                            'success' => false,
                                            'error' => 'Sorry, this event is sold out'
                                        ));
                                        return;
                                    }
                                }
                            }
                        }
                    }
                }
            }

            $invoice_data = array(
                'use_mysql_dates' => true,
                'date' => $now->format(DATE_MYSQL),
                'currency_id' => $currency_id,
                'invoice_lines' => $invoice_lines,
                'ordered_by_id' => array_var($data['requested_by'], 'contact_id'),
                'payment_method' => array_var($data, 'payment_method'),
                'external_transaction_id' => array_var($data, 'external_transaction_id'),
                'external_transaction_desc' => array_var($data, 'external_transaction_desc'),
                'ordered_by_address'=> array_var($all_data,'ordered_by_address'),
                'ordered_by_city'=> array_var($all_data,'ordered_by_city'),
                'ordered_by_state'=> array_var($all_data,'ordered_by_state'),
                'ordered_by_country'=> array_var($all_data,'ordered_by_country'),
                'ordered_by_zipcode'=> array_var($all_data,'ordered_by_zipcode'),
                'ordered_by_email'=> array_var($all_data,'ordered_by_email'),
                'ordered_by_telephone'=> array_var($all_data,'ordered_by_phone'),
            );
			
			try {
				DB::beginWork();

				if ($cp_value instanceof CustomPropertyValue) {
					$cp_value->save();
				}
				
				if ($total_price > 0) {
					// create invoice
					$invoice_controller = new InvoiceController();
					$new_invoice = new IncomeInvoice();
					$invoice = $invoice_controller->save_invoice($new_invoice, $invoice_data, $member_ids);
	
				} else {
					$invoice = null; // free registration
				}
				
				// create tickets
				$all_tickets_data = array(
					'use_mysql_dates' => true,
					'date' =>  $now->format(DATE_MYSQL),
					'invoice_id' => $invoice instanceof IncomeInvoice ? $invoice->getId() : 0,
					'event_tickets' => $event_tickets,
				);

				$generated_tickets_data = $this->save_event_ticket_bulk($all_tickets_data, $member_ids);

				DB::commit();

				// return the generated data
				ajx_current('empty');
				ajx_extra_data(array(
				    'success' => true,
					'customer_member_id' => $member_selection,
					'invoice' => $invoice instanceof IncomeInvoice ? $invoice->getArrayInfo() : array(),
					'tickets' => $generated_tickets_data,
				));

			} catch (Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current('empty');
			}

		}

	}

	function view() {
		ajx_set_no_toolbar();
		$event_ticket = EventTickets::instance()->findById(get_id());
		if(!($event_ticket instanceof EventTicket)) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}

		if (!$event_ticket->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		ajx_extra_data(array("title" => $event_ticket->getObjectName(), 'icon' => $event_ticket->getIconClass()."s"));
		tpl_assign('event_ticket', $event_ticket);
	}


	/*
	function add() {

		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}

		$event_ticket_data = array_var($_POST, 'event_ticket');

		if(!is_array($event_ticket_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$obj = new EventTicket();
			$obj->setObjectTypeId(EventTickets::instance()->getObjectTypeId());
			$obj->setTicketTypeId(get_id('ticket_type_id'));
			$obj->setRequestedOn(DateTimeValueLib::now());

			$event_id = get_id("event_id");
			$event = ManagedEvents::instance()->findById($event_id);
			if (!$event instanceof ManagedEvent) {
				flash_error(lang('object dnx'));
				ajx_current('empty');
				return;
			}

			tpl_assign('event', $event);
			tpl_assign('event_ticket', $obj);

		} else {
			// form submitted
			try {
				DB::beginWork();

				$member_ids = json_decode(array_var($_POST, 'members'));

				$event_ticket = $this->save_event_ticket($event_ticket_data, $member_ids);

				DB::commit();

				ApplicationLogs::createLog($event_ticket, ApplicationLogs::ACTION_EDIT);
				flash_success(lang('success add event_ticket', $event_ticket->getName()));

				ajx_current("back");
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}

			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}

	}

	*/

	function edit() {
		$this->setTemplate('add');

		$event_ticket = EventTickets::instance()->findById(get_id());
		if(!($event_ticket instanceof EventTicket)) {
			flash_error(lang('event_ticket dnx'));
			ajx_current("empty");
			return;
		}

		if (logged_user()->isGuest() || !$event_ticket->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}

		$event_ticket_data = array_var($_POST, 'event_ticket');

		if(!is_array($event_ticket_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}

			$tt = EventTicketTypes::getEventTicketById($event_ticket->getTicketTypeId());
			$event_id = $tt instanceof EventTicketType ? $tt->getEventId() : 0;

			$event = ManagedEvents::instance()->findById($event_id);
			if (!$event instanceof ManagedEvent) {
				flash_error(lang('cant edit event ticket because ticket type does not exist'));
				ajx_current('empty');
				return;
			}

			tpl_assign('event', $event);
			tpl_assign('event_ticket', $event_ticket);

		} else {
			// form submitted
			try {
				DB::beginWork();

				$member_ids = json_decode(array_var($_POST, 'members'));

				$event_ticket = $this->save_event_ticket($event_ticket_data, $member_ids, $event_ticket);

				ApplicationLogs::createLog($event_ticket, ApplicationLogs::ACTION_EDIT);
				DB::commit();

				flash_success(lang('success edit event_ticket', $event_ticket->getName()));

				ajx_current("back");
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}

			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}


	/**
	 * Save EventTicket object
	 * @param array $event_ticket_data Array with the event ticket data
	 * @param array $member_ids where the object will be classified
	 * @param EventTicket $event_ticket existing object, use only when editing
	 * @return EventTicket
	 */
	function save_event_ticket($event_ticket_data, $member_ids, $event_ticket = null) {
		// Save event_ticket object

		try {
			DB::beginWork();

			$tt = EventTicketTypes::getEventTicketById($event_ticket_data['ticket_type_id']);
			if ($event_ticket instanceof EventTicket) {
				$old_tt = EventTicketTypes::getEventTicketById($event_ticket->getTicketTypeId());
			}

			$is_new = false;
			if (is_null($event_ticket) || !$event_ticket instanceof EventTicket) {
				$event_ticket = new EventTicket();

				$name = "";
				if ($tt instanceof EventTicketType) {
					$event = ManagedEvents::instance()->findById($tt->getEventId());
					$name = $event instanceof ManagedEvent ? $event->getObjectName() : $tt->getName();
				}
				if ($name == "") $name = "Event ticket";
				$event_ticket->setObjectName($name);

				$is_new = true;
			}

			// ensure that the ticket is classified in the event's member
			if ($tt instanceof EventTicketType) {
				$event_member = Members::findOneByObjectId($tt->getEventId());
				if ($event_member instanceof Member) {
					$member_ids[] = $event_member->getId();
					$member_ids = array_unique($member_ids);
				}
			}

			// Parse requested_on date, requests from api should come with use_mysql_dates=1 and dates in mysql format YYYY-mm-dd HH:ii:ss
			if ($event_ticket_data['use_mysql_dates']) {
				if (array_var($event_ticket_data, 'requested_on', '') != '') {
					$event_ticket_data['requested_on'] = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $event_ticket_data['requested_on']);
				}
			} else {
				$event_ticket_data['requested_on'] = getDateValue($event_ticket_data['requested_on']);
			}
			
			// if attendees data is present then use them to build the ticket name
			if (array_var($event_ticket_data, 'event_attendees')) {
				
				$attendee_names = "";
				$attendees_data = array_var($event_ticket_data, 'event_attendees');
				foreach ($attendees_data as $att_data) {
					$attendee = Contacts::instance()->findById(array_var($att_data, 'contact_id'));
					if ($attendee instanceof Contact) {
						$attendee_names .= ($attendee_names==''?'':', ') . $attendee->getObjectName();
					} else {
						if (isset($event_attendee['contact_first_name']) || isset($event_attendee['contact_surname'])) {
							$attendee_names .= ($attendee_names==''?'':', ') . trim(array_var($att_data, 'contact_first_name') .' '. array_var($att_data, 'contact_surname'));
						}
					}
					if (strlen($attendee_names) > 150) {
						break;
					}
				}
				$old_name = $event_ticket->getObjectName();
				$event_ticket->setObjectName($old_name.' - '.$attendee_names);
				
			} else {
				
				if(array_var($event_ticket_data, 'requested_by_id')){
					$requested_by_contact = Contacts::instance()->findById(array_var($event_ticket_data, 'requested_by_id'));
					$contact_name = $requested_by_contact->getName();
					if(!empty($contact_name)){
						$old_name = $event_ticket->getObjectName();
						$event_ticket->setObjectName($old_name.' - '.$contact_name);
					}
				}
			}

			// if requested_by_id not sent, try to find contact using "requested_by_email_address", if not found -> try to create the contact
			if (!array_var($event_ticket_data, 'requested_by_id') && array_var($event_ticket_data, 'contact_email_address')) {
				$requested_by_email = array_var($event_ticket_data, 'contact_email_address');
				
				// if duplicated emails are allowed then get the contact using the first name and surname too
				if (config_option('allow_duplicated_contact_emails')) {
					$requested_by = Contacts::getByFirstnameAndSurname(trim(array_var($event_ticket_data, 'contact_first_name')), trim(array_var($event_ticket_data, 'contact_surname')), $requested_by_email);
				} else {
					$requested_by = Contacts::getByEmail($requested_by_email);
				}
				
				if (!$requested_by instanceof Contact) {
					$requested_by = create_contact_from_ticket_data(array(
						'contact_email_address' => $requested_by_email,
						'contact_first_name' => array_var($event_ticket_data, 'contact_first_name'),
						'contact_surname' => array_var($event_ticket_data, 'contact_surname'),
					), $member_ids);
				}

				if ($requested_by instanceof Contact) {
					$event_ticket_data['requested_by_id'] = $requested_by->getId();
				}
			}

			// if no requested by could be get or created, check if the invoice has any contact in the ordered_by_id field
			if (!array_var($event_ticket_data, 'requested_by_id') && array_var($event_ticket_data, 'invoice_id')) {
				$inv = IncomeInvoices::instance()->findById(array_var($event_ticket_data, 'invoice_id'));
				if ($inv instanceof IncomeInvoice) {
					$event_ticket_data['requested_by_id'] = $inv->getOrderedById();
				}
			}


			if (!array_var($event_ticket_data, 'currency_id')) {
				$event_ticket_data['currency_id'] = $tt->getCurrencyId();
			}
			if (!array_var($event_ticket_data, 'unit_price')) {
				$event_ticket_data['unit_price'] = $tt->getAmount();
			}
			if (!array_var($event_ticket_data, 'status')) {
				$event_ticket_data['status'] = 'pending';
			}
			
			// set the ticket subtype id according to the event subtype
			if (!array_var($event_ticket_data, 'object_subtype_id') && $tt instanceof EventTicketType) {
				$event_obj = ManagedEvents::instance()->findById($tt->getEventId());
				if ($event_obj instanceof ManagedEvent && $event_obj->getColumnValue('object_subtype_id') > 0) {
					$row = DB::executeOne("
							SELECT dependant_object_subtype_id FROM ".TABLE_PREFIX."object_subtype_dependencies
							WHERE object_subtype_id='".$event_obj->getColumnValue('object_subtype_id')."'
					");
					if (is_array($row) && array_var($row, 'dependant_object_subtype_id')) {
						$event_ticket_data['object_subtype_id'] = array_var($row, 'dependant_object_subtype_id');
					}
				}
			}

			// set attributes to the EventTicket object
			$event_ticket->setFromAttributes($event_ticket_data);


			// also classify in all the members of the invoice
			if ($event_ticket->getInvoiceId() > 0) {
				$inv_member_ids = ObjectMembers::getMemberIdsByObject($event_ticket->getInvoiceId());
				$non_event_member_ids = array();
				if (count($inv_member_ids) > 0) {
					// exclude event members, it is already added and must be only one
					$inv_members = Members::instance()->findAll(array("conditions" => "id IN (".implode(',', $inv_member_ids).")"));
					foreach ($inv_members as $inv_member) {
						if ($inv_member->getObjectTypeId() != ManagedEvents::instance()->getObjectTypeId()) {
							$non_event_member_ids[] = $inv_member->getId();
						}
					}
				}
				$member_ids = array_merge($member_ids, $non_event_member_ids);
				$member_ids = array_unique(array_filter($member_ids));
			}

			// save the EventTicket
			$event_ticket->save();

			// save the basic ContentDataObject properties
			$object_controller = new ObjectController();
			if (!is_null($member_ids)) {
				$object_controller->add_to_members($event_ticket, $member_ids);
			}
			$object_controller->add_subscribers($event_ticket);
			$object_controller->link_to_new_object($event_ticket);
			$object_controller->add_custom_properties($event_ticket);

			// link to invoice if invoice_id is specified
			if ($event_ticket->getInvoiceId() > 0) {
				$invoice = IncomeInvoices::instance()->findById($event_ticket->getInvoiceId());
				if ($invoice instanceof IncomeInvoice) {
					$event_ticket->linkObject($invoice);
					
					// also ensure that the invoice is classified in the event member
					if ($event_member instanceof Member) {
						$invoice_member_ids = $invoice->getMemberIds();
						if (!in_array($event_member->getId(), $invoice_member_ids)) {
							$invoice_member_ids[] = $event_member->getId();
							$object_controller->add_to_members($invoice, $invoice_member_ids);
						}
					}
				}
			}
			
			// ensure that the requested by contact is classified in the event
			if ($event_member instanceof Member) {
				$requested_by = Contacts::instance()->findById($event_ticket->getRequestedById());
				if ($requested_by instanceof Contact) {
					$req_mem_ids = $requested_by->getMemberIds();
					
					if (!in_array($event_member->getId(), $req_mem_ids)) {
						$req_mem_ids[] = $event_member->getId();
						$object_controller->add_to_members($requested_by, $req_mem_ids, null, false);
					}
				}
			}
				
			// Recalculate the ticket type availability
			recalculate_ticket_types_sold_quantity(array($tt, $old_tt));

			DB::commit();

		} catch (Exception $e) {
			DB::rollback();
			Logger::log("Error saving event ticket: ".$e->getMessage()."\n".$e->getTraceAsString());
			throw $e;
		}

		// Create the empty attendees
		//create_empty_attendees_for_ticket($event_ticket);

		return $event_ticket;
	}
	
	private function build_invoice_data_from_tickets($params, $member_ids, $customer_member) {
		
		$invoice_data = array_var($params, 'invoice_data');
		$tickets_data = array_var($params,'event_tickets', array());
		$invoice_lines = array();
		$event_member_ids = array();
		
		$group_invoice_lines_by_event = config_option('group_invoice_lines_by_event');
		
		if ($group_invoice_lines_by_event) {
			// one line per event, with the attendees separated by \n
			$attendees_by_event = array();
			$events_data_by_id = array();
			
			// group attendees by event id and gather event and ticket information
			foreach ($tickets_data as $ticket) {
				/* @var $ticket_type EventTicketType */
				if (array_var($ticket, 'ticket_type_id')) {
					$ticket_type = EventTicketTypes::instance()->findById($ticket['ticket_type_id']);
				} else {
					$ticket_type = $this->find_ticket_type_from_event_and_member($customer_member, $ticket);
				}
				
				$event_member_id = 0;
				$event_member = Members::instance()->findById($ticket['event_member_id']);
				if ($event_member instanceof Member) {
					$event_member_id = $event_member->getId();
					$event_member_ids[] = $event_member->getId();
					
					if (!isset($events_data_by_id[$event_member_id])) {
						$events_data_by_id[$event_member_id] = array(
							'event_name' => $event_member->getName(),
						);
					}
				}
				
				$events_data_by_id[$event_member_id]['ticket_quantity'] = $ticket['quantity'] + array_var($events_data_by_id[$event_member_id], 'ticket_quantity', 0);
				$events_data_by_id[$event_member_id]['unit_value'] = $ticket_type->getAmount();
				$events_data_by_id[$event_member_id]['currency_id'] = $ticket_type->getCurrencyId();
				
				if (!isset($attendees_by_event[$event_member_id])) {
					$attendees_by_event[$event_member_id] = array();
				}
				
				$attendee_names = array();
				$event_attendees = array_var($ticket['event_attendees'], 0);
				if (is_array($event_attendees)) {
					foreach ($event_attendees as $event_attendee) {
						$attendee = Contacts::instance()->findById(array_var($event_attendee, 'contact_id'));
						if ($attendee instanceof Contact) {
							$attendee_names[] = $attendee->getObjectName();
						} else {
							if (isset($event_attendee['contact_first_name']) || isset($event_attendee['contact_surname'])) {
								$attendee_names[] = trim(array_var($event_attendee, 'contact_first_name') .' '. array_var($event_attendee, 'contact_surname'));
							}
						}
					}
				}
				
				$attendees_by_event[$event_member_id] = array_merge($attendees_by_event[$event_member_id], $attendee_names);
			}
			
			// build invoice lines
			foreach ($attendees_by_event as $event_id => $attendee_names) {
				$line_detail = $events_data_by_id[$event_id]['event_name'];
				foreach ($attendee_names as $attendee_name) {
					$line_detail .= "\n    " . $attendee_name;
				}
				
				$line = array();
				$line['amount'] = $events_data_by_id[$event_id]['ticket_quantity'];
				$line['unit_value'] = $events_data_by_id[$event_id]['unit_value'];
				$line['total'] = $line['unit_value'] * $line['amount'];
				$line['detail'] = $line_detail;
				$line['currency_id'] = $events_data_by_id[$event_id]['currency_id'];
					
				$invoice_lines[] = $line;
			}
			
		} else {
			// one line per ticket
			foreach ($tickets_data as $ticket) {
				/* @var $ticket_type EventTicketType */
				if (array_var($ticket, 'ticket_type_id')) {
					$ticket_type = EventTicketTypes::instance()->findById($ticket['ticket_type_id']);
				} else {
					$ticket_type = $this->find_ticket_type_from_event_and_member($customer_member, $ticket);
				}
					
				$event_member = Members::instance()->findById($ticket['event_member_id']);
				if ($event_member instanceof Member) {
					$event_member_ids[] = $event_member->getId();
				}
					
				$attendee_names = array();
				$event_attendees = array_var($ticket['event_attendees'], 0);
				if (is_array($event_attendees)) {
					foreach ($event_attendees as $event_attendee) {
						$attendee = Contacts::instance()->findById(array_var($event_attendee, 'contact_id'));
						if ($attendee instanceof Contact) {
							$attendee_names[] = $attendee->getObjectName();
						} else {
							if (isset($event_attendee['contact_first_name']) || isset($event_attendee['contact_surname'])) {
								$attendee_names[] = trim(array_var($event_attendee, 'contact_first_name') .' '. array_var($event_attendee, 'contact_surname'));
							}
						}
					}
				}
					
				$line_detail = "";
				if ($event_member instanceof Member) {
					$line_detail = $event_member->getName();
				}
				if ($ticket_type instanceof EventTicketType) {
					$line_detail .= ($line_detail == '' ? '' : ' - ') . $ticket_type->getName();
				}
				if (count($attendee_names) > 0) {
					$line_detail .= ($line_detail == '' ? '' : ' - ') . implode(', ', $attendee_names);
				}
					
				$line = array();
				$line['amount'] = $ticket['quantity'];
				$line['unit_value'] = $ticket_type->getAmount();
				$line['total'] = $line['unit_value'] * $line['amount'];
				$line['detail'] = $line_detail;
				$line['currency_id'] = $ticket_type->getCurrencyId();
					
				$invoice_lines[] = $line;
			}
			
		}

		$invoice_data['invoice_lines'] = $invoice_lines;
		$invoice_data['member_ids'] = array_unique(array_merge($member_ids, $event_member_ids));
		
		return $invoice_data;
	}
		
	private function check_if_all_attendees_are_in_one_member($params) {
		$customer_member = null;
		$ot_customer = ObjectTypes::findByName('customer');
		
		$all_attendees_data = array();
		foreach ($params['event_tickets'] as $event_ticket_data) {
			foreach (array_var($event_ticket_data, 'event_attendees', array()) as $event_attendees_array) {
				$all_attendees_data = array_merge($all_attendees_data, $event_attendees_array);
			}
		}
		
		$all_contact_ids = array();
		foreach ($all_attendees_data as $attendee_data) {
			if (array_var($attendee_data, 'contact_id') > 0) {
				$all_contact_ids[] = array_var($attendee_data, 'contact_id');
			} else {
				if ($attendee_data['contact_email_address']) {
					$attendee_contact = Contacts::getByEmail($attendee_data['contact_email_address']);
				} else {
					$attendee_contact = Contacts::getByFirstnameAndSurname($attendee_data['contact_first_name'], $attendee_data['contact_surname']);
				}
				if ($attendee_contact instanceof Contact) {
					$all_contact_ids[] = $attendee_contact->getId();
				}
			}
		}
		
		if (count($all_contact_ids) > 0) {
			$sql = "SELECT om.member_id, count(*) as cant
						FROM ".TABLE_PREFIX."object_members om
						INNER JOIN ".TABLE_PREFIX."members m ON m.id=om.member_id
						WHERE m.object_type_id=".$ot_customer->getId()."
							AND om.object_id IN (".implode(',', $all_contact_ids).")
						GROUP BY om.member_id;";
			$rows = DB::executeAll($sql);
			
			if (count($rows) == 1) {
				$customer_member = Members::instance()->findById($rows[0]['member_id']);
			}
		}
		
		return $customer_member;
	}
	
	private function identify_customer_member_from_params($params) {
		$customer_member_identified = false;
		$member = null;
		
		$ot_customer = ObjectTypes::findByName('customer');
		
		// try with member code
		if (config_option('use_ticket_type_member_pricing') && array_var($params, 'member_code')) {
			$cp_member_code = CustomProperties::getCustomPropertyByCode($ot_customer->getId(), "member_code");
			$cpvalue = CustomPropertyValues::instance()->findOne(array("conditions" => array("custom_property_id=? AND value=?", $cp_member_code->getId(), $params['member_code'])));
			if ($cpvalue instanceof CustomPropertyValue) {
				$customer_member = Members::findOneByObjectId($cpvalue->getObjectId());
				if ($customer_member instanceof Member) {
					$member = $customer_member;
					$customer_member_identified = true;
				}
			}
		}
		
		// check if attendees are all in the same customer member
		if (!$customer_member_identified) {
			$customer_member_id = array_var($params, 'customer_member_id');
			if (is_numeric($customer_member_id) && $customer_member_id > 0) {
				$customer_member = Members::instance()->findById($customer_member_id);
			}
			if (!$customer_member instanceof Member) {
				$customer_member = $this->check_if_all_attendees_are_in_one_member($params);
			}
			if ($customer_member instanceof Member) {
				$member = $customer_member;
				$customer_member_identified = true;
			}
		}
		
		// check if the billing contact is a member or if the billing contact is the billing contact of a member
		if (!$customer_member_identified) {
			if ($params['invoice_data']['ordered_by_email']) {
				$billing_contact = Contacts::getByEmail($params['invoice_data']['ordered_by_email']);
				if ($billing_contact instanceof Contact) {
					// check if the billing contact is a member
					$customer = Customers::instance()->findOne(array("conditions" => "contact_id=".$billing_contact->getId()));
					if ($customer instanceof Customer) {
						$customer_member = Members::findOneByObjectId($customer->getId());
						if ($customer_member instanceof Member) {
							$member = $customer_member;
							$customer_member_identified = true;
						}
					}
					
					// check if the billing contact is the billing contact of a member
					if (!$customer_member_identified) {
						$billing_contact_cp = CustomProperties::getCustomPropertyByCode($ot_customer->getId(), 'billing_contact');
						$cpvalue = CustomPropertyValues::instance()->findOne(array("conditions" => array("custom_property_id=? AND value=?", $billing_contact_cp->getId(), $billing_contact->getId())));
						if ($cpvalue instanceof CustomPropertyValue) {
							$customer_member = Members::findOneByObjectId($cpvalue->getObjectId());
							if ($customer_member instanceof Member) {
								$member = $customer_member;
								$customer_member_identified = true;
							}
						}
					}
				}
			}
		}
		
		
		return $member;
	}
	
	function find_ticket_type_from_event_and_member($customer_member, $ticket_data) {
		
		$event_member_id = array_var($ticket_data, 'event_member_id');
		$event_member = Members::instance()->findById($event_member_id);
		
		// check if customer member is active
		$is_active_member = $this->check_if_customer_member_is_active($customer_member);
		
		$tt = null;
		$ticket_types = EventTicketTypes::findAllByEventId($event_member->getObjectId());
		if (config_option('use_ticket_type_member_pricing')) {
			foreach ($ticket_types as $ticket_type) {
				/* @var $ticket_type EventTicketType */
				if ($is_active_member && $ticket_type->getMemberPricing()) {
					$tt = $ticket_type;
					break;
				}
				if (!$is_active_member && !$ticket_type->getMemberPricing()) {
					$tt = $ticket_type;
					break;
				}
			}
			// if is an active member and no member pricing has been found, get the first one found
			if (!$tt instanceof EventTicketType && $is_active_member) {
				$tt = array_var($ticket_types, 0);
			}
		} else {
			$tt = array_var($ticket_types, 0);
		}
		
		return $tt;
	}
	
	private function check_if_customer_member_is_active($customer_member) {
		if (!$customer_member instanceof Member) return false;
		
		$is_active_member = false;
		
		$customer_stages_dim = Dimensions::findByCode('customer_stages');
		$customer_stages_ot = ObjectTypes::findByName('customer_stage');
		if ($customer_stages_dim instanceof Dimension && $customer_stages_ot instanceof ObjectType) {
			// get the association with accounts
			$a = DimensionMemberAssociations::instance()->getAssociationObject($customer_member->getDimensionId(), $customer_member->getObjectTypeId(), $customer_stages_dim->getId(), $customer_stages_ot->getId());
			if ($a instanceof DimensionMemberAssociation) {
				// get the associated member
				$mem_ids = MemberPropertyMembers::getAllPropertyMemberIds($a->getId(), $customer_member->getId());
				$mem_ids = array_filter(explode(',', $mem_ids), 'is_numeric');
				if (count($mem_ids) > 0) {
					foreach ($mem_ids as $id){
						if ($id == config_option('compare_member_active_with')){ // the memeber is active
							$is_active_member = true;
							break;
						}
					}
				}
			}
		}
		
		return $is_active_member;
	}
	
	function request_ticket_types_info_from_attendees($params) {

        $member_name = '';
        $billing_emails = '';
		$member_code_needed = true;
		$ot_customer = ObjectTypes::findByName('customer');
		
		$member_code_received = array_var($params, 'member_code');
		if ($member_code_received) {
			$cp_member_code = CustomProperties::getCustomPropertyByCode($ot_customer->getId(), "member_code");
			$cpvalue = CustomPropertyValues::instance()->findOne(array("conditions" => array("custom_property_id=? AND value=?", $cp_member_code->getId(), $member_code_received)));
			if ($cpvalue instanceof CustomPropertyValue) {
				$customer_member = Members::findOneByObjectId($cpvalue->getObjectId());
				if ($customer_member instanceof Member) {
                    $member_name = $customer_member->getName();
					$member_code_needed = false;
				}
			}
		} else {
			$customer_member_id = array_var($params, 'customer_member_id');
			if (is_numeric($customer_member_id) && $customer_member_id > 0) {
				$customer_member = Members::instance()->findById($customer_member_id);
			}
			if (!$customer_member instanceof Member) {
				$customer_member = $this->check_if_all_attendees_are_in_one_member($params);
			}
			
			if ($customer_member instanceof Member) {
                $member_name = $customer_member->getName();
				$member_code_needed = false;
			}		
		}
		
		// gather billing emails
		if (isset($customer_member) && $customer_member instanceof Member && $ot_customer->getId() == $customer_member->getObjectTypeId()) {
			$billing_emails_array = array();
			// billing contact email
			$cp_billing_contact = CustomProperties::getCustomPropertyByCode($ot_customer->getId(), "billing_contact");
			$bc_cpvalue = CustomPropertyValues::getCustomPropertyValue($customer_member->getObjectId(), $cp_billing_contact->getId());
			if ($bc_cpvalue instanceof CustomPropertyValue) {
				$bc_contact = Contacts::instance()->findById($bc_cpvalue->getValue());
				if ($bc_contact instanceof Contact) {
					$billing_emails_array[] = $bc_contact->getEmailAddress();
				}
			}

			// representative email
			$cp_representative = CustomProperties::getCustomPropertyByCode($ot_customer->getId(), "representative");
			$rep_cpvalue = CustomPropertyValues::getCustomPropertyValue($customer_member->getObjectId(), $cp_representative->getId());
			if ($rep_cpvalue instanceof CustomPropertyValue) {
				$rep_contact = Contacts::instance()->findById($rep_cpvalue->getValue());
				if ($rep_contact instanceof Contact) {
					$billing_emails_array[] = $rep_contact->getEmailAddress();
				}
			}
			
			$billing_emails_array = array_unique($billing_emails_array);
			$billing_emails = implode(',', $billing_emails_array);
		}
		
		$event_ticket_types = array();
		
		foreach ($params['event_tickets'] as $ticket_data) {
			$ticket_type = $this->find_ticket_type_from_event_and_member($customer_member, $ticket_data);
			
			$event_member_id = array_var($ticket_data, 'event_member_id');
			
			if ($ticket_type instanceof EventTicketType) {
				$event_ticket_types[$event_member_id] = $ticket_type->getArrayInfo(true);
			}
		}
		
		$billing_contact_data = array();
		if (isset($bc_contact) && $bc_contact instanceof Contact) {
			$billing_contact_data = $bc_contact->getObjectData();
			$address = $bc_contact->getAddress('work');
			if ($address instanceof ContactAddress) {
				$billing_contact_data['address'] = $address->getArrayInfo();
			}
			$phone = $bc_contact->getPhone('work', false, false);
			if ($phone instanceof ContactTelephone) {
				$billing_contact_data['phone'] = $phone->getNumber();
			}
			$email_address_obj = ContactEmails::instance()->findOne(array("conditions" => array("contact_id=?",$bc_contact->getId())));
			if ($email_address_obj instanceof ContactEmail) {
				$billing_contact_data['email'] = $email_address_obj->getEmailAddress();
			}
		}
		
		return array(
		    'member_name' => $member_name,
			'billing_emails' => $billing_emails,
			'member_code_needed' => $member_code_needed,
			'event_ticket_types' => $event_ticket_types,
			'customer_member_id' => $customer_member instanceof Member ? $customer_member->getId() : '',
			'billing_contact' => $billing_contact_data,
		);
	}
	
	function save_event_ticket_bulk($params, $member_ids) {
		
	    if ( isset($params) && is_array($params) && count(array_var($params,'event_tickets', array())) > 0) {
	    	
	    	$customer_member = $this->identify_customer_member_from_params($params);
	    	if ($customer_member instanceof Member) {
	    		$member_ids[] = $customer_member->getId();
	    	}
	    	
	    	if (array_var($params, 'generate_invoice')) {
	    		
	    		$invoice = new IncomeInvoice();
	    		if (array_var($params,'invoice_id')) {
	    			$invoice = IncomeInvoices::instance()->findById(array_var($params,'invoice_id'));
	    		}
	    		
	    		$invoice_data = $this->build_invoice_data_from_tickets($params, $member_ids, $customer_member);
	    		$inv_member_ids = array_var($invoice_data, 'member_ids');
	    		
	    		$invoice_controller = new InvoiceController();
	    		$inv = $invoice_controller->save_invoice($invoice, $invoice_data, $inv_member_ids);
	    		if ($inv instanceof IncomeInvoice) {
	    			$params['invoice_id'] = $inv->getId();
	    		}
	    	}
	    	
	    	
	        $event_ticket_data = array(
	            "invoice_id" => array_var($params,'invoice_id',0),
	            "use_mysql_dates" => array_var($params, 'use_mysql_dates', true)
	        );

	        $date_format = $event_ticket_data["use_mysql_dates"] ? DATE_MYSQL : config_option("date_format");
	    	$now = DateTimeValueLib::now();
	        $now_formatted = $now->format($date_format);
	        $event_ticket_data['requested_on'] = array_var($params, 'requested_on', $now_formatted);

	        $response_data = array();
	        try {
	        	$invoice = IncomeInvoices::instance()->findById(array_var($event_ticket_data, "invoice_id"));
	        	if ($invoice instanceof IncomeInvoice) {
	        		$requested_by_id = $invoice->getOrderedById();
	        		$event_ticket_data["requested_by_id"] = $requested_by_id;
	        	}

	            foreach ($params['event_tickets'] as $ticket) {
	                for ($i=0; $i< $ticket['quantity']; $i++){
	                	$ticket_type_id = $ticket['ticket_type_id'];
	                	
	                	if (!$ticket_type_id) {
	                		// get the ticket type id using customer member status and member pricing
	                		$ticket_type = $this->find_ticket_type_from_event_and_member($customer_member, $ticket);
	                		$ticket_type_id = $ticket_type instanceof EventTicketType ? $ticket_type->getId() : 0;
	                	}
	                	
	                    $event_ticket_data['ticket_type_id'] = $ticket_type_id;
	                    if (!isset($event_ticket_data["requested_by_id"])) {
	                    	$event_ticket_data['requested_by_id'] = $ticket['requested_by_id'];
	                    }
	                    $event_ticket_data['contact_email_address'] = $ticket['contact_email_address'];
	                    $event_ticket_data['contact_first_name'] = $ticket['contact_first_name'];
	                    $event_ticket_data['contact_surname'] = $ticket['contact_surname'];
						$event_ticket_data['comment'] = $ticket['comment'];
	                    
	                    // attendees data
	                    $event_attendees_array = array_var($ticket, 'event_attendees');
	                    if ($event_attendees_array) {
		                    $event_attendees = array_var($event_attendees_array, $i);
		                    if (is_array($event_attendees)) {
	                    		$event_ticket_data['event_attendees'] = $event_attendees;
		                    }
	                    }
	                    
	                    $ticket_info = $this->save_event_ticket($event_ticket_data, $member_ids, $event_ticket = null);
	                    
	                    if ($ticket_info instanceof EventTicket){
	                    	// also return the ticket type data, it may be needed in the post-process
	                        $saved_ticket_data = $ticket_info->getArrayInfo(false, true);
	                        
	                        // if attendees were specified => save them
		                    if (is_array($event_attendees)) {
		                    	foreach ($event_attendees as &$attendee) {
		                    		$attendee['event_ticket_id'] = $ticket_info->getId();
		                    	}
		                    	$event_attendees_data = array('event_attendees' => $event_attendees);
		                    	
		                    	if (!isset($event_attendees_controller)) {
		                    		$event_attendees_controller = new EventAttendeeController();
		                    	}
	                    		$attendees_info = $event_attendees_controller->save_event_attendee_bulk($event_attendees_data, $member_ids);
	                    		// add atteendees data to the response
	                    		$saved_ticket_data['attendees'] = $attendees_info;
		                    }
		                    
		                    $response_data[] = $saved_ticket_data;
	                    }
	                }
	            }
	            
	            return array(
            		'invoice_data' => $invoice instanceof IncomeInvoice ? $invoice->getArrayInfo() : array(),
            		'event_tickets_data' => $response_data,
	            );
	            
	        }catch (Exception $e) {
	            Logger::log("Error saving event ticket: ".$e->getMessage()."\n".$e->getTraceAsString());
	            throw $e;
	        }
	    }
	    return array();
	}

	function list_all() {
		ajx_current("empty");
		// Get all variables from request
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		$attributes = array("ids" => explode(',', array_var($_GET,'ids')));
		$dim_order = null;
		$cp_order = null;

		$show_attendee_details = array_var($_GET, 'show_attendee_details');
		$purchase_order_id = array_var($_GET, 'purchase_order_id');

		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}

		$select_columns = array("*");
		$extra_conditions = "";
		$join_params = null;

		if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		} else if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		} else {
			switch ($order) {
				case 'quantity':
				case 'requested_on':
				case 'unit_price':
					break;
				case 'price':
					$order = "total_price";
					$select_columns = array("e.*, o.*, (e.unit_price * e.quantity) as total_price");
					break;
				case 'ticket_type_id':
					$order = 'ticket_name';
					$join_params = array(
							'table' => EventTicketTypes::instance()->getTableName(),
							'jt_field' => 'id',
							'join_type' => 'LEFT',
							'e_field' => 'ticket_type_id',
					);
					$select_columns = array("e.*, o.*, jt.`name` as ticket_name");
					break;
				case 'requested_by':
					$order = 'contact_name';
					$join_params = array(
							'table' => Objects::instance()->getTableName(),
							'jt_field' => 'id',
							'join_type' => 'LEFT',
							'e_field' => 'requested_by_id',
					);
					$select_columns = array("e.*, o.*, jt.`name` as contact_name");
					break;
				default:
					$order = "name";
					break;
			}
		}

		Hook::fire("listing_extra_conditions", array('manager' => EventTickets::instance(), 'request' => $_REQUEST), $extra_conditions);

		if ($purchase_order_id) {
			$extra_conditions .= " AND e.purchase_order_id = '$purchase_order_id'";
		}

		$only_count_result = array_var($_GET, 'only_result', false);

		$result = EventTickets::instance()->listing(array(
				"order" => $order,
				"order_dir" => $order_dir,
				"dim_order" => $dim_order,
				"cp_order" => $cp_order,
				"select_columns" => $select_columns,
				"extra_conditions" => $extra_conditions,
				"count_results" => false,
				"only_count_results" => $only_count_result,
				"join_params" => $join_params,
				"start" => $start,
				"limit" => $limit,
		));

		$event_tickets = $result->objects;

		$object = $this->prepareObject($event_tickets, $start, $limit, $result, $show_attendee_details);

		ajx_current("empty");
		ajx_extra_data($object);
		tpl_assign("listing", $object);
	}

	private $event_attendee_cps = null;

	private function prepareObject($event_tickets, $start, $limit, $res_obj, $show_attendee_details=false) {
		Env::useHelper('format');

		$object = array(
				"totalCount" => $res_obj->total,
				"start" => $start,
				"event_tickets" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}

		if (!$this->event_attendee_cps) {
			$this->event_attendee_cps = CustomProperties::getAllCustomPropertiesByObjectType(EventTickets::instance()->getObjectTypeId());
		}
		$custom_properties = $this->event_attendee_cps;
		$ticket_type_names = array();

		$count = min(array($limit, count($event_tickets)));
		for ($i = 0; $i < $count; $i++){
			if (isset($event_tickets[$i])){
				$event_ticket = $event_tickets[$i];
				if ($event_ticket instanceof EventTicket){
					$info = $event_ticket->getArrayInfo(true);

					// price
					$info['unit_price_formatted'] = format_money_amount($info['unit_price'], $info['currency_sym']);

					// ticket type
					$tt_id = $info['ticket_type_id'];
					if (!array_var($ticket_type_names, $tt_id)) {
						$tt = EventTicketTypes::getEventTicketById($tt_id);
						if ($tt instanceof EventTicketType) $ticket_type_names[$tt_id] = $tt->getName();
					}
					$info['event_ticket_type_name'] = array_var($ticket_type_names, $tt_id);

					// requested by
					$info['requested_by'] = $event_ticket->getRequestedByName();

					$info['memPath'] = json_encode($event_ticket->getMembersIdsToDisplayPath(true));
					$object["event_tickets"][$i] = $info;
					foreach ($custom_properties as $cp) {
						$object["event_tickets"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $event_ticket);
					}
				}
			}
		}

		return $object;
	}

	private function resolveAction($action, $attributes){

		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				$messages = array();
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$event_ticket = EventTickets::instance()->findById($id);
					if ($event_ticket instanceof EventTicket && $event_ticket->canDelete(logged_user())){
						try{
							DB::beginWork();
							$event_ticket->trash();
							DB::commit();
							ApplicationLogs::createLog($event_ticket, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$messages[] = $e->getMessage();
							$err++;
						}
					} else {
						$err++;
						$messages[] = lang('cannot delete event_ticket', $event_ticket instanceof EventTicket ? clean($event_ticket->getName()) : '');
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) .("\n - ".implode("\n - ",$messages)). ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if
				$resultCode = 2;
				break;
		} // switch
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
	}

    function tickets_attendee_report_params() {
        $dim = Dimensions::findByCode('managed_events');

        $condition = new ReportCondition();
        $condition->setCondition('in');
        $condition->setFieldName('dim_'.$dim->getId());
        $condition->setCustomPropertyId(0);
        $condition->setIsParametrizable(1);
        $condition->setValue('[]');


        tpl_assign('condition', $condition);
        tpl_assign('name', 'Managed events');
    }

    function tickets_attendee_report($function_params = null) {
        Env::useHelper('advanced_reports', 'advanced_reports');
        Env::useHelper('reporting');
        Env::useHelper('grouping');


        if (!$function_params) $function_params = $_POST;
        $_SESSION['tickets_attendee_report_data'] = $function_params[0];

        // get report params
        $post_params = array_var($function_params, 'params');
        $params_to_post['report_params'] = $post_params;
        $post_params = json_decode($post_params[0]);
        $report_name = lang('Event Roster');

        $columns = array();
        if (Plugins::instance()->isActivePlugin('income')) {
            $columns = array(
                'prefix' => 1,
                'surname' => 2,
                'first_name' => 3,
                'License type' => 4,
                'License number' => 5,
                'amount' => 6,
                'payment_method' =>7,
                'external_transaction_id' =>8,
                'members' => 9,
                //'member code' => 9,
                'signature' => 11,
                'attended' => 12

//                'event_name' =>12,
  //              'ticket_name' => 13,
                //'status' => 14
            );
        }else{
            $columns = array(
                'prefix' => 1,
                'surname' => 2,
                'first_name' => 3,
                'License type' => 4,
                'License number' => 5,
                'amount' => 6,
                'members' => 7,
                'signature' => 8,
                'attended' => 9

                //'event_name' =>11,
                //'ticket_name' => 12,
                //'status' => 13
            );
        }


        $report_params = array(
            'report_name' => lang('Event Roster'),
            'report_desc' => lang('Event Roster'),
            'date_type' => 6,
            'start_value' => '',
            'end_value' => '',//format_date($end_date)
            'show_details' => 0,
            'group_by_type' => array(
                'dim', 'member_ids', '0'
            ),
            'columns' => $columns,
            'columns_type' =>array(
                'prefix' => DATA_TYPE_STRING,
                'surname' => DATA_TYPE_STRING,
                'first_name' => DATA_TYPE_STRING,
                'License type' => DATA_TYPE_STRING,
                'License number' => DATA_TYPE_INTEGER,
                'amount' => DATA_TYPE_INTEGER,
                'payment_method' =>DATA_TYPE_STRING,
                'external_transaction_id' =>DATA_TYPE_STRING,
                'members' => DATA_TYPE_STRING,
                'member code' => DATA_TYPE_STRING,

                'signature' => DATA_TYPE_STRING,
                'attended' => DATA_TYPE_STRING,

                'event_name' =>DATA_TYPE_STRING,
                'ticket_name' => DATA_TYPE_STRING,
                'status' => DATA_TYPE_STRING)
        );


        // filter by current context
        $context_conditions = "";
        $context_member_ids = active_context_members(false);
        if (count($context_member_ids) > 0) {
            $context_conditions = " AND (EXISTS (SELECT om3.object_id
					FROM ".TABLE_PREFIX."object_members om3
					WHERE om3.member_id IN (" . implode ( ',', $context_member_ids ) . ") AND o.id = om3.object_id
					GROUP BY om3.object_id
					HAVING count(om3.member_id) = ".count($context_member_ids)."
				))
			";
        }


        if (Plugins::instance()->isActivePlugin('income')) {
            $sql = "
                SELECT et.object_id as id,tt.name ticket_name,mm.name event_name,et.unit_price as amount,et.currency_id , et.status,tt.attendees_per_ticket,pt.contact_id,GROUP_CONCAT(mm2.name SEPARATOR ', ') as members,
                 ii.payment_method,ii.external_transaction_id, cc.first_name,cc.surname,pt.object_id as attendee_id,ii.customer_id
                FROM ".TABLE_PREFIX."event_tickets et
                INNER JOIN ".TABLE_PREFIX."objects o ON o.id=et.object_id
                INNER JOIN ".TABLE_PREFIX."event_ticket_types tt ON tt.id = et.ticket_type_id
                LEFT JOIN ".TABLE_PREFIX."event_attendees pt ON pt.event_ticket_id = et.object_id
                LEFT JOIN ".TABLE_PREFIX."income_invoices ii ON ii.object_id = et.invoice_id
                LEFT JOIN ".TABLE_PREFIX."object_members om ON om.object_id = et.object_id
                LEFT JOIN ".TABLE_PREFIX."members mm ON mm.id = om.member_id
                LEFT JOIN ".TABLE_PREFIX."object_members om2 ON om2.object_id = et.object_id
                LEFT JOIN (SELECT mm2.name,mm2.id FROM ".TABLE_PREFIX."members mm2 
                                INNER JOIN ".TABLE_PREFIX."dimensions d ON d.id = mm2.dimension_id AND d.code = 'customer_project'
                                ) AS mm2 on mm2.id = om2.member_id
                INNER JOIN ".TABLE_PREFIX."object_types ot ON ot.id = mm.object_type_id AND ot.handler_class = 'ManagedEvents'
                 LEFT JOIN ".TABLE_PREFIX."contacts cc ON cc.object_id=pt.contact_id
            ";
        }else{
            $sql = "
                SELECT et.object_id as id,tt.name ticket_name,mm.name event_name,et.unit_price as amount,et.currency_id , et.status,tt.attendees_per_ticket,pt.contact_id
                ,GROUP_CONCAT(mm2.name SEPARATOR ', ') as members,
                  cc.first_name,cc.surname,pt.object_id as attendee_id
                FROM ".TABLE_PREFIX."event_tickets et
                INNER JOIN ".TABLE_PREFIX."objects o ON o.id=et.object_id
                INNER JOIN ".TABLE_PREFIX."event_ticket_types tt ON tt.id = et.ticket_type_id
                LEFT JOIN ".TABLE_PREFIX."event_attendees pt ON pt.event_ticket_id = et.object_id
                
                LEFT JOIN ".TABLE_PREFIX."object_members om ON om.object_id = et.object_id
                LEFT JOIN ".TABLE_PREFIX."members mm ON mm.id = om.member_id
                LEFT JOIN ".TABLE_PREFIX."object_members om2 ON om2.object_id = et.object_id
                LEFT JOIN (SELECT mm2.name,mm2.id FROM ".TABLE_PREFIX."members mm2 
                                INNER JOIN ".TABLE_PREFIX."dimensions d ON d.id = mm2.dimension_id AND d.code = 'customer_project'
                                ) AS mm2 on mm2.id = om2.member_id
                INNER JOIN ".TABLE_PREFIX."object_types ot ON ot.id = mm.object_type_id AND ot.handler_class = 'ManagedEvents'
                 LEFT JOIN ".TABLE_PREFIX."contacts cc ON cc.object_id=pt.contact_id
            ";

        }
        // select tickets data

        if (!empty($post_params)){
            $sql .= " WHERE ";
            $sql .= " mm.id in (".implode(',',$post_params).') ';
            $sql .= $context_conditions;
        }else{
            $sql .= " WHERE true ".$context_conditions;
        }
        $sql .= " AND o.trashed_by_id = 0 AND o.archived_by_id = 0 ";
        $sql .= " GROUP BY id,pt.contact_id ORDER BY IF(surname RLIKE '^[a-z]', 1, 2), surname,first_name ";

        $temp_rows = DB::executeAll($sql);

        $grouped_event_attendee = array();

        foreach ($temp_rows as $temp_row) {

            $ot_customer = ObjectTypes::findByName('customer');
            $ot_contact = ObjectTypes::findByName('contact');
            $ot_attendee = ObjectTypes::findByName('event_attendee');

            $cp_member = CustomProperties::getCustomPropertyByCode($ot_customer->getId(), "member_code");
            $cp_license_type = CustomProperties::getCustomPropertyByCode($ot_contact->getId(), "license_type_code");
            $cp_license_number = CustomProperties::getCustomPropertyByCode($ot_contact->getId(), "license_number_code");
            $cp_prefix = CustomProperties::getCustomPropertyByCode($ot_contact->getId(), "prefix_code");
            $cp_signature = CustomProperties::getCustomPropertyByCode($ot_attendee->getId(), "signature_code");
            $cp_attended = CustomProperties::getCustomPropertyByCode($ot_attendee->getId(), "attended_code");


            if (Plugins::instance()->isActivePlugin('income')) {
                $customer_id = array_var($temp_row,'customer_id');
                if (!empty($customer_id)){
                    if ($cp_member instanceof CustomProperty) {
                        $cp_val = CustomPropertyValues::getCustomPropertyValue($temp_row['customer_id'], $cp_member->getId());
                        if ($cp_val instanceof CustomPropertyValue) {
                            $temp_row['member code'] = $cp_val->getValue();
                        }
                    }
                }
            }

            $currency_id = array_var($temp_row,'currency_id');
            $amount = array_var($temp_row,'amount');
            $currency = Currencies::instance()->getCurrency($currency_id);
            $csymbol = null;
            if ($currency instanceof Currency) {
                $csymbol = $currency->getSymbol();
            }else{
                $csymbol = config_option('currency_code');
            }
            $amount = format_money_amount($amount, $csymbol);
            $temp_row['amount'] = $amount;

            $contact_id = array_var($temp_row,'contact_id');

            if (!empty($contact_id)){

                if ($cp_license_type instanceof CustomProperty) {
                    $cp_val = CustomPropertyValues::getCustomPropertyValue($temp_row['contact_id'], $cp_license_type->getId());
                    if ($cp_val instanceof CustomPropertyValue) {
                        $temp_row['License type'] = $cp_val->getValue();
                    }
                }
                if ($cp_license_number instanceof CustomProperty) {
                    $cp_val = CustomPropertyValues::getCustomPropertyValue($temp_row['contact_id'], $cp_license_number->getId());
                    if ($cp_val instanceof CustomPropertyValue) {
                        $temp_row['License number'] = $cp_val->getValue();
                    }
                }

                if ($cp_prefix instanceof CustomProperty) {
                    $cp_val = CustomPropertyValues::getCustomPropertyValue($temp_row['contact_id'], $cp_prefix->getId());
                    if ($cp_val instanceof CustomPropertyValue) {
                        $temp_row['prefix'] = $cp_val->getValue();
                    }
                }

            }

            $attendee_id = array_var($temp_row,'attendee_id');

            if (!empty($attendee_id)){

                if ($cp_signature instanceof CustomProperty) {
                    $cp_val = CustomPropertyValues::getCustomPropertyValue($temp_row['attendee_id'], $cp_signature->getId());
                    if ($cp_val instanceof CustomPropertyValue) {
                        $temp_row['signature'] = $cp_val->getValue();
                    }
                }
                if ($cp_attended instanceof CustomProperty) {
                    $cp_val = CustomPropertyValues::getCustomPropertyValue($temp_row['attendee_id'], $cp_attended->getId());
                    if ($cp_val instanceof CustomPropertyValue) {
                        $temp_row['attended'] =  $cp_val->getValue();
                    }
                }
            }

            $payment_method = array_var($temp_row,'payment_method');
            $temp_row['payment_method'] = lang($payment_method);

            $grouped_event_attendee[$temp_row['id']][] = $temp_row;

        }


        /**
         * This is for empty attendees
         */
        foreach ($grouped_event_attendee as $key => $item){
            $count = count($item);
            $attendees_per_ticket = $item[0]['attendees_per_ticket'];
            if ($count < $attendees_per_ticket){

                $item[0]['first_name'] = '';
                $item[0]['surname'] = '';
                $item[0]['License type'] = '';
                $item[0]['License type'] = '';
                $item[0]['License number'] = '';
                $item[0]['signature'] = '';
                $item[0]['attended'] = '';

                $grouped_event_attendee[$key][] = $item[0];
            }

        }

        // set variables in view
        tpl_assign('grouped_event_attendee', $grouped_event_attendee);
        tpl_assign('report_data', $report_params);

        tpl_assign('title', $report_name);
        tpl_assign('post', $params_to_post);
        tpl_assign('allow_export', true);

        tpl_assign('parameters', $post_params);
        //tpl_assign('id', $report_id);

        tpl_assign('template_name', 'tickets_attendee_report');
        $this->setTemplate('report_wrapper');

        return array(
            'grouped_event_attendee' => $grouped_event_attendee,
            'report_data' => $report_params,
        );


    }

    function tickets_attendee_report_print(){

        ajx_current("empty");

        if (array_var($_REQUEST, 'exportCSV') || array_var($_REQUEST, 'exportExcel')) {


            $report_params = array_var($_REQUEST, 'report_params');

            $report_name = lang('Event Roster');

            $result =  $this->tickets_attendee_report(array('params' => array($report_params)));

            $array_rows = tickets_attendee_report_result_to_arrays($result);

            if (array_var($_REQUEST, 'exportExcel') && Plugins::instance()->isActivePlugin('advanced_core')) {

                $excel_export_controller = new ExcelExportController();

                $headers = array_shift($array_rows);
                $conditons[0] = build_event_report_conditions_plain_main(array(0=>$report_params));
                //$headers = array_merge($conditons,$headers);
                $all_data_rows = $array_rows;
                // build a sheet for the excel file with the report data
                $sheets = array(
                    array('headers' => $headers, 'data' => $all_data_rows, 'options' => array('title' => $report_name,'conditions'=>$conditons)),
                );

                // generate the excel file
                $filename = "event_roster.xlsx";
                $filepath = $excel_export_controller->generate_excel_file($sheets, array('filename' => $filename));
                $filesize = filesize($filepath);

            } else {

                $csv_text = "";
                $conditons = build_event_report_conditions_plain_main(array(0=>$report_params));
                if ($conditons != ''){
                    $csv_text .= '"' . $conditons . '"' . "\n";
                }
                foreach ($array_rows as $row) {
                    $csv_text .= '"' . implode('","', $row) . '"' . "\n";
                }

                $filename = "event_roster.csv";
                $filepath = ROOT . "/tmp/event_roster.csv";
                file_put_contents($filepath, $csv_text);
                $filesize = filesize($filepath);
            }

            ajx_extra_data(array("filename" => $filename, 'size' => $filesize));
        }

    }

    function tickets_attendee_report_to_pdf() {
        ajx_current("empty");
        set_time_limit(0);
        ini_set("memory_limit", "512M");

        $html_filename = gen_id().'pdf.html';
        $html_filepath = ROOT . "/tmp/$html_filename";

        // include all css
        $html = stylesheet_tag('website.css');
        $css_html = "";
        Hook::fire('additional_report_print_css', null, $css_html);
        if ($css_html) $html .= $css_html;


        $report_params = array_var($_REQUEST, 'report_params');

        $report_name = lang('Event Roster');

        // title html
        $html .= '<div class="report-print-header">
                    <div class="title-container">
                    <h1>'.$report_name.'</h1>
                    <div style="padding: 8px 20px 0px;float: right;">'.build_event_report_conditions_html_main(array(0=>$report_params)).'</div>
                    </div>
                    </div>
                    <div class="clear"></div>';

        // start output buffer
        ob_start();

        // execute the report
        $this->tickets_attendee_report(array('params' => array($report_params)), false);
        // include the report html to print
        tpl_display(get_template_path('tickets_attendee_report', 'event_ticket', 'event_tickets'));

        // get the html from buffer
        $html .= ob_get_clean();

        file_put_contents($html_filepath, $html);

        // convert html to pdf
        $orientation = array_var($_REQUEST, 'pdfPageLayout') == 'L' ? 'Landscape' : 'Portrait';
        $page_size = array_var($_REQUEST, 'pdfPageSize');
        
        $pdf_data = convert_to_pdf($html, $orientation, "Event roster", $page_size);
        $real_pdf_filename = $pdf_data['name'];
        ajx_extra_data(array("filename" => $real_pdf_filename, 'size' => $pdf_data['size']));

    }

    /**
     * returns the tickets type depending on the member passed by post, this function is implemented in the workflow of classes
     */
    function get_ticket_types(){

        Env::useHelper('functions', 'crpm');
        if (logged_user()->isGuest()) {
            flash_error(lang('no access permissions'));
            ajx_current('empty');
            return;
        }

        $info = array();

        $event_id = array_var($_POST,'event_id');
        $company_id = array_var($_POST,'company_id');
        //$contact_id = array_var($_POST,'contact_id');

        $event_member = Members::getMemberById($event_id);
        if ($event_member instanceof Member){

            $is_actiuve_member = false;
            $company = Members::instance()->findById($company_id);
            if ($company instanceof Member) {
                //Client stages
                $customer_stages_dim = Dimensions::findByCode('customer_stages');
                $customer_stages_ot = ObjectTypes::findByName('customer_stage');
                if ($customer_stages_dim instanceof Dimension && $customer_stages_ot instanceof ObjectType) {
                    // get the association with accounts
                    $a = DimensionMemberAssociations::instance()->getAssociationObject($company->getDimensionId(), $company->getObjectTypeId(), $customer_stages_dim->getId(), $customer_stages_ot->getId());
                    if ($a instanceof DimensionMemberAssociation) {
                        // get the associated member
                        $mem_ids = MemberPropertyMembers::getAllPropertyMemberIds($a->getId(), $company->getId());
                        $mem_ids = array_filter(explode(',', $mem_ids), 'is_numeric');
                        if (count($mem_ids) > 0) {
                            foreach ($mem_ids as $id){
                                if ($id == config_option('compare_member_active_with')){ // the memeber is active
                                    $is_actiuve_member = true;
                                }
                            }
                        }
                    }
                }
            }

            $all_ticket_type = EventTicketTypes::findAllByEventId($event_member->getObjectId());
            if (!empty($all_ticket_type)){
                if (config_option('use_ticket_type_member_pricing')){
                    foreach ($all_ticket_type as $tt){
                        if ($tt instanceof EventTicketType) {
                        	if ($is_actiuve_member) {
                        		if ($tt->getMemberPricing()){
                        			$info['ticket_type_object'][] = $tt->getArrayInfo();
                        			break;
                        		}
                        	} else {
                        		if (!$tt->getMemberPricing()){
                        			$info['ticket_type_object'][] = $tt->getArrayInfo();
                        			break;
                        		}
                        	}
                            
                        }
                    }
                    if (count($info['ticket_type_object']) == 0) {
                    	$info['ticket_type_object'][] = $all_ticket_type[0]->getArrayInfo();
                    }
                }else{
                    $info['ticket_type_object'][] = $all_ticket_type[0]->getArrayInfo(); // only return one element.
                }
            }
        }

        // return the generated data
        ajx_current('empty');
        ajx_extra_data(array(
            'tickets_type' => $info,
        ));
    }

}




