<?php
Env::useHelper('format');
if (isset($event_ticket) && $event_ticket instanceof EventTicket) {
	$genid = gen_id();
	if (!$event_ticket->isTrashed()){
		if($event_ticket->canEdit(logged_user())) {
			add_page_action(lang('edit'), "javascript:og.render_modal_form('', {c:'event_ticket', a:'edit', params: {id:".$event_ticket->getId()."}});", 'ico-edit', null, null, true);
			if (!$event_ticket->isArchived())
				add_page_action(lang('archive'), "javascript:if(confirm(lang('confirm archive object'))) og.openLink('" . $event_ticket->getArchiveUrl() ."');", 'ico-archive-obj');
			else
				add_page_action(lang('unarchive'), "javascript:if(confirm(lang('confirm unarchive object'))) og.openLink('" . $event_ticket->getUnarchiveUrl() ."');", 'ico-unarchive-obj');
			
		} // if
	}
	if ($event_ticket->canDelete(logged_user())) {
		if ($event_ticket->isTrashed()) {
			add_page_action(lang('restore from trash'), "javascript:if(confirm(lang('confirm restore objects'))) og.openLink('" . $event_ticket->getUntrashUrl() ."');", 'ico-restore', null, null, true);
			add_page_action(lang('delete permanently'), "javascript:if(confirm(lang('confirm delete permanently'))) og.openLink('" . $event_ticket->getDeletePermanentlyUrl() ."');", 'ico-delete', null, null, true);
		} else {
			add_page_action(lang('move to trash'), "javascript:if(confirm(lang('confirm move to trash'))) og.openLink('" . $event_ticket->getTrashUrl() ."');", 'ico-trash', null, null, true);
		}
	}
?>

<div style="padding:7px">
<div class="event-ticket">
<?php
	
	$content = '';
	
	$tt = EventTicketTypes::getEventTicketById($event_ticket->getTicketTypeId());
	if ($tt instanceof EventTicketType) {
		$content .= '<div><span class="bold ">'.lang('ticket type').': </span>'.$tt->getName().'</div>';
	}
	
	$requested_by = Contacts::instance()->findById($event_ticket->getRequestedById());
	if ($requested_by instanceof Contact) {
		$content .= '<div><span class="bold ">'.lang('requested by').': </span>'.
		'<a class="internalLink underline" href="'.$requested_by->getViewUrl().'">'. $requested_by->getObjectName() .'</a>'
		.'</div>';
	}

	$content .= '<div><span class="bold ">'.lang('requested on').': </span>'.format_date($event_ticket->getRequestedOn(), null, 0).'</div>';
	
	$currency = Currencies::instance()->getCurrency($event_ticket->getCurrencyId());
	$c_symbol = $currency instanceof Currency ? $currency->getSymbol() : '$';
	$content .= '<div><span class="bold ">'.lang('price').': </span>'.format_money_amount($event_ticket->getUnitPrice(), $c_symbol).'</div>';

	$content .= '<div><span class="bold ">'.lang('status').': </span>'.lang("event ticket status ".$event_ticket->getStatus()).'</div>';
	$content .= '<div><span class="bold ">'.lang('comment').': </span>'. $event_ticket->getComment() .'</div>';
	
	$content .= '<br /><div><span class="bold">'.lang('attendees').': </span></div>';
	$content .= '<div id="'.$genid.'_attendees_grid_container" class="attendees-grid-container"></div>';

	
	
	
	$content .= '<div class="clear"></div>';
	
	tpl_assign("content", $content);
	tpl_assign("object", $event_ticket);
	tpl_assign('iconclass', $event_ticket->isTrashed()? 'ico-large-event-ticket-trashed' : ($event_ticket->isArchived() ? 'ico-large-event-ticket-archived' : 'ico-large-event-ticket'));
	
	$this->includeTemplate(get_template_path('view', 'co'));

	
	// include payment grid columns and toolbar configuration
	tpl_assign("genid", $genid);
	$this->includeTemplate(get_template_path('attendees_grid/columns', 'event_ticket', 'event_tickets'));
	$this->includeTemplate(get_template_path('attendees_grid/toolbar', 'event_ticket', 'event_tickets'));
	
?>
</div>
</div>
<script>

var grid_id = '<?php echo $genid?>_attendees_grid';

og.gridObjectNameCustomRenderer = function(value, p, r) {
    var controller = r.data.type_controller ? r.data.type_controller : r.store.baseParams.url_controller;
    return String.format('<a href="#" onclick="og.render_modal_form(\'\', {c:\''+controller+'\', a:\'edit\', params:{id:'+r.data.object_id+'}});" title="{0}" style="font-size:120%;"><span class="bold">{0}</span></a>', og.clean(value));
}


var ticket_attendees_grid = new og.ObjectGrid({
	renderTo: '<?php echo $genid?>_attendees_grid_container',
	url: og.getUrl('event_attendee', 'list_all'),
	store_params: {
		url_controller: 'event_attendee',
		url_action: 'list_all',
		event_ticket_id: <?php echo $event_ticket->getId()?>
	},
	type_name: 'event_attendee',
	response_objects_root: 'event_attendees',
	grid_id: grid_id,
	nameRenderer: og.gridObjectNameCustomRenderer,
	checkbox_sel_model: true,
	columns: attendee_columns,
	no_totals_row: true,
	skip_dimension_columns: true,
	tbar_items: tbar_items,
	stateId: 'event-ticket-attendees-list', // to remember the gui state independent of the panel
});


var active_tab = Ext.getCmp('tabs-panel').activeTab;
if (active_tab.events.resize.listeners.length < 2) {
	active_tab.on('resize', function() {
		var g = Ext.getCmp(grid_id)
		if (g) g.fireEvent('resize');
	});
}

ticket_attendees_grid.load({
	load_callback: function() {
		og.expenses.addTotalsRow(Ext.getCmp(grid_id).store, {expense_id:'<?php echo $event_ticket->getId()?>'}, grid_id);
	}
});

</script>

<?php } //if isset ?>
