<?php

$widgets_data_cache = array();

function collect_tasks_and_time_amounts_data_for_widget() {
	
	global $widgets_data_cache;
	
	Env::useHelper('functions', 'advanced_billing');
	
	// This is a patch to prevent reaching memory limit when no filters are applied and there are too much timeslots in the system
	// @TODO: Improve memory usage of the listing function for timeslots and tasks and remove this condition
	$this_year_cond = "";
	$this_year_cond_time = "";
	$active_context_members = active_context_members(false);
	$include_subtitle = false;
	if (count($active_context_members) == 0) {
		$beginning_of_year = DateTimeValueLib::now();
		if( $beginning_of_year->getMonth() < 6){
			$previous_year = $beginning_of_year->getYear() - 1;
			$beginning_of_year->setYear($previous_year);
		} 
		$beginning_of_year->setDay(1);
		$beginning_of_year->setMonth(1);
		$beginning_of_year->setHour(0);
		$beginning_of_year->setMinute(0);
		$beginning_of_year->setSecond(0);
		$beginning_of_year->add('s', -1 * logged_user()->getUserTimezoneValue());
		$this_year_cond = " AND (due_date >= '".$beginning_of_year->toMySQL()."')";
		$this_year_cond_time = " AND (start_time >= '".$beginning_of_year->toMySQL()."')";
		$include_subtitle = true;
		$subtitle_date_start = $beginning_of_year;
	}
	// --
	
	$task_assignment_conditions = "";
	if (!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_assigned_to_other_tasks')) {
		$task_assignment_conditions = " AND assigned_to_contact_id = ".logged_user()->getId();
	}
	
	if (isset($widgets_data_cache['financial_tasks'])) {
		$tasks = $widgets_data_cache['financial_tasks'];
	} else {
		$tasks_res = ProjectTasks::instance()->listing(array(
				"raw_data" => true,
				"fire_additional_data_hook" => false,
				"order" => "due_date",
				"order_dir" => "ASC",
				"extra_conditions" => " AND is_template=0 $task_assignment_conditions $this_year_cond"
		));

		$tasks = $tasks_res->objects ? $tasks_res->objects : []; // *** LC 2023-09-04 

		$taskParent = [];

		foreach ($tasks as $task) {
			$taskParent[$task['id']] = $task['parent_id'];
		}


		// create an array with tasks filteres by not has parent

		$highestNodeTasks = array_filter($tasks, function ($task) use ($taskParent) {
			// If a task's parent_id is 0, it's a root node.
			// If a task's parent_id does not exist in task IDs, it's also a root node.
			return $task['parent_id'] == 0 || !array_key_exists($task['parent_id'], $taskParent);
		});

		$tasks = $highestNodeTasks;
		$widgets_data_cache['financial_tasks'] = $tasks;
	} 

	$amounts_by_day = array();
	
	// Collect tasks information and build the estimated billing from billing category and estimated time
	$tasks_without_due_date = array();
	$total_amount_tasks_without_due_date = 0;
	$tasks_without_billing_cat = array();
	$total_amount_tasks_without_billing_cat = 0;
	$tasks_without_estimated_time = array();
	$total_amount_tasks_without_estimated_time = 0;
	$allowed_dimensions = array_filter(config_option('adv_billing_dimensions'));
	$task_ids_with_manual_earned_value = array();
	$manual_tasks_earned_value_total = 0;
	
	foreach ($tasks as $task) {
		if (count($allowed_dimensions) == 0) continue;

		handle_task_with_manual_earned_value($task, $task_ids_with_manual_earned_value, $manual_tasks_earned_value_total, $amounts_by_day);

		if ($task['due_date'] == '0000-00-00 00:00:00') {
			$tasks_without_due_date[] = $task;
			$total_amount_tasks_without_due_date += $task['estimated_price'];
			continue;
		}
		
		if ($task['total_time_estimate'] == 0 && $task['estimated_price'] == 0) { 
			$tasks_without_estimated_time[] = $task;
			$total_amount_tasks_without_estimated_time += $task['estimated_price'];
			continue;
		}
		
		$object_id = $task['id'];

		if($task['due_date'] != '0000-00-00 00:00:00'){
			$due_date = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $task['due_date']);
		} else {
			continue;
		}
		$date_key = $due_date->format('Y-m-d');
		$due_date_str = $due_date->format('m/d/Y');

		if (!isset($amounts_by_day[$date_key])) {
			$amounts_by_day[$date_key] = array(
					'date' => $due_date_str, 
					'estimated' => 0,
					'executed' => 0,
			);
		}

		$amounts_by_day[$date_key]['estimated'] += $task['estimated_price'];
	
	}

	$extra_conditions = "";
	if(!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_others_timeslots')){
		$extra_conditions .= " AND contact_id = " . logged_user()->getId();
	}
 
	// Exclude timeslots that assigned to tasks with manual earned value
	if(count($task_ids_with_manual_earned_value) > 0){
		$task_ids_string = implode(',', $task_ids_with_manual_earned_value);
		$extra_conditions .= " AND rel_object_id NOT IN (".$task_ids_string.")";
	}
	
	if (isset($widgets_data_cache['financial_timeslots'])) {
		$timeslots = $widgets_data_cache['financial_timeslots'];
	} else {
		$timeslots_res = Timeslots::instance()->listing(array(
				"raw_data" => true,
				"fire_additional_data_hook" => false,
				"order" => "o.id",
				"order_dir" => "ASC",
				"extra_conditions" => $extra_conditions . $this_year_cond_time
		));
		$timeslots = $timeslots_res->objects;
		
		$widgets_data_cache['financial_timeslots'] = $timeslots;
	}
	
	
	// Collect timeslots information and build the worked hours values
	foreach ($timeslots as $timeslot) {
		/* @var $timeslot Timeslot */
		
		$timeslot_billing = $timeslot['fixed_billing']; // ->getFixedBilling();
		
		$date_key = substr($timeslot['start_time'], 0, 10); // exclude the time part
		$start_time_str = date('m/d/Y', strtotime($date_key));
		
		if (!isset($amounts_by_day[$date_key])) {
			$amounts_by_day[$date_key] = array(
					'date' => $start_time_str, //format_date($timeslot->getStartTime()),
					'estimated' => 0,
					'executed' => 0,
			);
		}
		
		$amounts_by_day[$date_key]['executed'] += $timeslot_billing;
	}
	
	// sort the amounts by date
	ksort($amounts_by_day);
	
	$result = array(
		'amounts_by_day' => $amounts_by_day,
		'number_of_tasks' => count($tasks),
		'tasks_without_billing_cat' => $tasks_without_billing_cat,
		'tasks_without_estimated_time' => $tasks_without_estimated_time,
		'tasks_without_due_date' => $tasks_without_due_date,
		'total_amount_tasks_without_billing_cat' => $total_amount_tasks_without_billing_cat,
		'total_amount_tasks_without_estimated_time' => $total_amount_tasks_without_estimated_time,
		'total_amount_tasks_without_due_date' => $total_amount_tasks_without_due_date,
		'manual_tasks_earned_value_total' => $manual_tasks_earned_value_total,
		'include_subtitle' => $include_subtitle,
		'subtitle_date_start' => isset($subtitle_date_start) ? $subtitle_date_start : "",
	);

	return $result;
}

function handle_task_with_manual_earned_value($task, &$task_ids_with_manual_earned_value, &$manual_tasks_earned_value_total, &$amounts_by_day) {
	// Handle tasks with manual earned value
	if ($task['is_calculated_estimated_price'] == 0 && $task['is_fixed_fee'] == 1) {
		// Add task and subtasks to the list of tasks with manual earned value
		$task_ids_with_manual_earned_value[] = $task['id'];
		$task_object = ProjectTasks::instance()->findById($task['id']);
		if($task_object instanceof ProjectTask) {
			$include_trashed = false;
			$subtask_ids = $task_object->getAllSubTasksIds($include_trashed);
			if (is_array($subtask_ids)) {
				$task_ids_with_manual_earned_value = array_merge($task_ids_with_manual_earned_value, $subtask_ids);
			}
		}
		
		// Add task earned value to the amounts by day
		$has_date = false;
		if($task['due_date'] != '0000-00-00 00:00:00') {
			$task_date = $task['due_date'];
			$has_date = true;
		}
		if($has_date) {
			$date_key = substr($task_date, 0, 10); // exclude the time part
			$start_time_str = date('m/d/Y', strtotime($date_key));
			
			if (!isset($amounts_by_day[$date_key])) {
				$amounts_by_day[$date_key] = array(
						'date' => $start_time_str, //format_date($timeslot->getStartTime()),
						'estimated' => 0,
						'executed' => 0,
				);
			}
			
			$amounts_by_day[$date_key]['executed'] += $task['earned_value'];
		} else {
			$manual_tasks_earned_value_total += $task['earned_value'];
		}
	}
}
