<?php
if (SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_expenses_of_others')) {
	
	Env::useHelper('functions', 'evx_widgets');
	ini_set('memory_limit', '1G');
	$amounts_by_day = array();
	
	$show_billing_info = true;
	Hook::fire('get_can_see_billing_information', array('user'=>logged_user()), $show_billing_info);
	
	$show_cost_info = true;
	Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $show_cost_info);
	
	$result = collect_tasks_and_time_amounts_data_for_widget(); 
	$tasks_without_billing_cat = $result['tasks_without_billing_cat'];
	$tasks_without_estimated_time = $result['tasks_without_estimated_time'];
	$tasks_without_due_date = $result['tasks_without_due_date'];
	$total_amount_tasks_without_billing_cat = $result['total_amount_tasks_without_billing_cat'];
	$total_amount_tasks_without_estimated_time = $result['total_amount_tasks_without_estimated_time'];
	$total_amount_tasks_without_due_date = $result['total_amount_tasks_without_due_date'];
	$manual_tasks_earned_value_total = $result['manual_tasks_earned_value_total'];
	$include_subtitle =  array_var($result, 'include_subtitle', false);
	$subtitle_date_start =  array_var($result, 'subtitle_date_start', null);


	
	$task_and_time_amounts_by_day = $result['amounts_by_day'];
	foreach ($task_and_time_amounts_by_day as $date_key => $values) {
		
		if (!isset($amounts_by_day[$date_key])) {
			$amounts_by_day[$date_key] = array(
					'date' => $values['date'],
					'total_budget' => 0,
					'total_earned' => 0,
			);
		}
		$amounts_by_day[$date_key]['total_budget'] += $values['estimated'];
		$amounts_by_day[$date_key]['total_earned'] += $values['executed'];
	}
	
	
	if (Plugins::instance()->isActivePlugin('expenses2')) {
		Env::useHelper('functions', 'expenses2');
		$exp_params = array("exclude_expenses_assigned_to_tasks" => true);
		$result = collect_expenses_and_payment_receipts_data_for_widget($exp_params);
		$expenses_data_by_day = $result['amounts_by_day'];
		
		foreach ($expenses_data_by_day as $date_key => $exp_values) {
			
			if (!isset($amounts_by_day[$date_key])) {
				$amounts_by_day[$date_key] = array(
						'date' => $exp_values['date'],
						'total_budget' => 0,
						'total_earned' => 0,
				);
			}
			$amounts_by_day[$date_key]['total_budget'] += $exp_values['budgeted'];
			$amounts_by_day[$date_key]['total_earned'] += $exp_values['actual'];
		}
		
	}
	
	// sort the amounts by date
	ksort($amounts_by_day);
	
	// fill gaps with empty values
	$keys = array_keys($amounts_by_day);
	$first_key = $keys[0];
	$last_key = end($keys);
	$last_dt_val = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $last_key);
	$date_iter = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $first_key);
	
	// sort the amounts by date again
	ksort($amounts_by_day);
	
	// accumulate the amounts
	$last_estimated_amount = 0;
	$last_earned_amount = 0;
	foreach ($amounts_by_day as &$amounts) {
		
		$amounts['total_budget'] += $last_estimated_amount;
		$amounts['total_earned'] += $last_earned_amount;
		
		$last_estimated_amount = $amounts['total_budget'];
		$last_earned_amount = $amounts['total_earned'];	
	}

	// add the totals of tasks without billing category, estimated time and due date
	$last_estimated_amount += $total_amount_tasks_without_billing_cat;
	$last_estimated_amount += $total_amount_tasks_without_estimated_time;
	$last_estimated_amount += $total_amount_tasks_without_due_date;

	// add the totals of manual tasks earned value to the executed total
	$last_earned_amount += $manual_tasks_earned_value_total;

	
	// Set currency symbol
	$default_currency = Currencies::getDefaultCurrencyInfo();
	$currency_symbol = !empty($default_currency) ? $default_currency['symbol'] : config_option('currency_code', '$');
	
	// Set widget title
	$widget_title = lang("forecasted revenue");
	if($include_subtitle){
		$date_format = user_config_option('date_format');
		$subtitle_date_formatted = $subtitle_date_start->format($date_format);
		$widget_subtitle = lang('from') . ': ' . $subtitle_date_formatted;
	}
	if($show_billing_info && $show_cost_info) {
		include 'template.php';
	}
}