<?php
/**
 * Expense Reports controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class ExpenseReportsController extends ApplicationController {

	var $plugin_name = "expenses";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
		Env::useHelper('grouping');
		Env::useHelper('format');
	}
	
	function payments_by_state_parameters() {
		
		$report_data = array_var($_SESSION, 'payments_by_state_data', array());
		
		$report_data['column_list'] = $this->payments_by_state_report_columns($report_data);
		$report_data['ordered_columns'] = $this->payments_by_state_report_columns_ordered($report_data);
		
		$cp_ids = CustomProperties::getCustomPropertyIdsByObjectType(Expenses::instance()->getObjectTypeId());
		tpl_assign('has_custom_properties', count($cp_ids) > 0);
		
		$_SESSION['payments_by_state_data'] = $report_data;
		
	}
	
	private function payments_by_state_report_columns($report_data) {
		$columns = array_var($report_data, 'show_columns');
		$col_list = array();
		
		$col_list[] = array('id' => 'name', 'text' => lang('name'), 'selected' => array_var($columns, 'name', 1) > 0);
		$col_list[] = array('id' => 'related_to', 'text' => lang('related to'), 'selected' => array_var($columns, 'related_to', 2) > 0);
		$col_list[] = array('id' => 'date', 'text' => lang('date'), 'selected' => array_var($columns, 'date', 3) > 0);
		$col_list[] = array('id' => 'description', 'text' => lang('description'), 'selected' => array_var($columns, 'description', 4) > 0);
		$col_list[] = array('id' => 'status', 'text' => lang('status'), 'selected' => array_var($columns, 'status', 5) > 0);
		
		$i=5;
		
		$pt_count = PaymentTypes::instance()->count();
		if ($pt_count > 0) {
			$i++;
			$col_list[] = array('id' => 'payment_type', 'text' => lang('payment type'), 'selected' => array_var($columns, 'payment_type', $i) > 0);
			$i++;
			$col_list[] = array('id' => 'payment_type_details', 'text' => lang('payment type details'), 'selected' => array_var($columns, 'payment_type_details', $i) > 0);
		
		}
		
		$statuses = ExpensesStatuses::instance()->getAllStatusInfo();
		foreach ($statuses as $st) {
			if ($st['only_for_expenses']) continue;
			$i++;
			$col_list[] = array('id' => $st['code'], 'text' => lang($st['code']), 'selected' => array_var($columns, $st['code'], $i) > 0);
		}
		
		$exp_cps = CustomProperties::getAllCustomPropertiesByObjectType(Expenses::instance()->getObjectTypeId());
		foreach ($exp_cps as $cp) {
			$i++;
			$col_list[] = array('id' => 'cp_'.$cp->getId(), 'text' => $cp->getName(), 'selected' => array_var($columns, 'cp_'.$cp->getId(), $i) > 0);
		}
		
		return $col_list;
	}
	
	private function payments_by_state_report_columns_ordered($report_data) {
		$ordered = array();
		foreach ($report_data['show_columns'] as $key => $order) {
			if ($order > 0) {
				$name = lang($key);
				foreach ($report_data['column_list'] as $col) {
					if ($col['id'] == $key) {
						$name = $col['text'];
						break;
					}
				}
				$ordered[$order] = array('id' => $key, 'name' => $name);
			} 
		}
		ksort($ordered);
		
		return $ordered;
	}
	
	
	function payments_by_state() {
		
		$csv_export = array_var($_POST, 'csv_export');
		
		if ($csv_export) {
			$report_data = $_SESSION['payments_by_state_data'];
		} else {
			$report_data = array_var($_POST, 'report', array());
			$report_data['column_list'] = $this->payments_by_state_report_columns($report_data);
			$report_data['ordered_columns'] = $this->payments_by_state_report_columns_ordered($report_data);
			$report_data['cp_conditions'] = array_var($_POST, 'conditions');
			$_SESSION['payments_by_state_data'] = $report_data;
		}
		
		$extra_conditions = '';
		
		$status = array_var($report_data, 'status');
		if (is_numeric($status) && $status > 0) {
			$extra_conditions .= " AND `status`=$status";
		}
		
		$now = DateTimeValueLib::now();
		$now->advance(logged_user()->getUserTimezoneValue(), true);
		switch (array_var($report_data, 'date_type')){
			case -1: //None
				$st = null;
				$et = null;
				break;
			case 1: //Today
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),$now->getDay(),$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),$now->getDay(),$now->getYear());break;
			case 2: //This week
				$monday = $now->getMondayOfWeek();
				$nextMonday = $now->getMondayOfWeek()->add('w',1)->add('d',-1);
				$st = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
				$et = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
			case 3: //Next week
				$monday = $now->getMondayOfWeek()->add('w',1);
				$nextMonday = $now->getMondayOfWeek()->add('d',-1);
				$st = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
				$et = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
			case 4: //This month
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
			case 5: //Next month
				$now->add('M',1);
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
			case 6: //Date interval
				if (array_var($report_data, 'date_start_value')) {
					$st = getDateValue(array_var($report_data, 'date_start_value'));
					if ($st instanceof DateTimeValue) $st = $st->beginningOfDay();
				}
				if (array_var($report_data, 'date_end_value')) {
					$et = getDateValue(array_var($report_data, 'date_end_value'));
					if ($et instanceof DateTimeValue) $et = $et->endOfDay();
				}
				break;
		}
		if ($et != null && $st != null) {
			tpl_assign('start_time', $st);
			tpl_assign('end_time', $et);
			$st_sql = $st instanceof DateTimeValue ? " AND p.date >= '".$st->toMySQL()."' " : "";
			$et_sql = $st instanceof DateTimeValue ? " AND p.date <= '".$et->toMySQL()."' " : "";
			$extra_conditions .= " AND EXISTS (SELECT p.expense_id FROM ".TABLE_PREFIX."payments p 
					WHERE p.expense_id=e.object_id $et_sql $st_sql)";
		}
		
		$cp_conditions = array_var($report_data, 'cp_conditions');
		$cp_cond_str = "";
		if (is_array($cp_conditions) && count($cp_conditions)) {
			foreach ($cp_conditions as $cond) {
				if ($cond['deleted'] > 0 || trim($cond["value"]) == "") continue;
				if (!in_array($cond['condition'], array('like', 'not like', '=', '<=', '>=', '<', '>', '<>', '%'))) continue;
				
				$cp = CustomProperties::getCustomProperty($cond['custom_property_id']);
				if (!$cp instanceof CustomProperty) continue;
				
				$current_condition = ' AND o.id IN ( SELECT object_id as id FROM '.TABLE_PREFIX.'custom_property_values cpv WHERE cpv.custom_property_id = '.$cond['custom_property_id'];
				
				$value = $cond['value'];
				if($cond['condition'] == 'like' || $cond['condition'] == 'not like'){
					$value = '%'.$cond['value'].'%';
				}
				if ($cp->getType() == 'date') {
					if ($value == $date_format_tip) continue;
					$dtValue = DateTimeValueLib::dateFromFormatAndString($dateFormat, $value);
					$value = $dtValue->format('Y-m-d H:i:s');
				}
				if($cond['condition'] != '%'){
					if ($cp->getType() == 'numeric') {
						
						$current_condition .= ' AND CAST(cpv.value AS DECIMAL(10,4)) '.$cond['condition'].' '.DB::escape($value);
						
					}else if ($cp->getType() == 'boolean') {
						
						$current_condition .= ' AND cpv.value '.$cond['condition'].' '.$value;
						if (!$value) {
							$current_condition .= ') OR o.id NOT IN (SELECT object_id as id FROM '.TABLE_PREFIX.'custom_property_values cpv2 WHERE cpv2.object_id=o.id AND cpv2.value=1 AND cpv2.custom_property_id = '.$cp->getId();
						}
						
					}else{
						$current_condition .= ' AND cpv.value '.$cond['condition'].' '.DB::escape($value);
					}
				}else{
					$current_condition .= ' AND cpv.value like '.DB::escape("%$value");
				}
				$current_condition .= ')';
				$extra_conditions .= $current_condition;
				
			}
		}
		
		$expenses = Expenses::instance()->listing(array(
			'extra_conditions' => $extra_conditions,
			'count_results' => false
		))->objects;
		
		
		$group_by = array();
		for ($i = 1; $i <= 3; $i++){
			$gb = array_var($report_data, 'group_by_' . $i);
			if ($gb != '0' && !in_array($gb, $group_by)) $group_by[] = $gb;
		}
		$gb_criterias = array();
		foreach ($group_by as $text) {
			if (str_starts_with($text, 'dim_')) $gb_criterias[] = array('type' => 'dimension', 'value' => str_replace_first('dim_', '', $text));
		}
		$grouped_expenses = groupObjects($gb_criterias, $expenses);
		
		$expense_ids = array();
		foreach ($expenses as $expense) $expense_ids[] = $expense->getId();
		if (count($expense_ids) == 0) $expense_ids[] = 0;
		
		$statuses = ExpensesStatuses::instance()->getAllStatusInfo();
		
		$payments = array();
		$totals = array();
		foreach ($expenses as $exp) {
			$p_objs = $exp->getPaymentsFromHistory();
			$payments[$exp->getId()] = array();
			
			foreach ($statuses as $st) {
				if ($st['only_for_expenses']) continue;
				
				if (!isset($payments[$exp->getId()][$st['code']])) $payments[$exp->getId()][$st['code']] = array();
				if (!isset($totals[$exp->getId()])) $totals[$exp->getId()] = array();
				if (!isset($totals[$exp->getId()][$st['code']])) $totals[$exp->getId()][$st['code']] = 0;
			}
			
			foreach ($p_objs as $p) {
				$st = ExpensesStatuses::instance()->getStatus($p->getStatusId());
				$payments[$p->getExpenseId()][$st->getCode()][] = $p;
				$totals[$p->getExpenseId()][$st->getCode()] += $p->getAmount();
			}
		}

		tpl_assign('grouped_expenses', $grouped_expenses);
		tpl_assign('payments', $payments);
		tpl_assign('totals', $totals);
		tpl_assign('status_info', $statuses);
		
		tpl_assign('post', $report_data);
		tpl_assign('title', lang('payments report'));
		tpl_assign('allow_export', false);
		
		if ($csv_export) {
			$this->setLayout('empty');
			//ajx_current('empty');
			
			$_SESSION['expense_report_export_tmp_filename'] = gen_id().".csv";
			$_SESSION['expense_report_export_filename'] = lang('payments report').".csv";
			
			tpl_assign('tmp_filename', ROOT."/tmp/".$_SESSION['expense_report_export_tmp_filename']);
			$this->setTemplate('payments_by_state_csv');
			
		} else {
			tpl_assign('template_name', 'payments_by_state');
			$this->setTemplate('report_wrapper');
		}
	}
	
	
	function download_exported_file() {
		$filename = array_var($_SESSION, 'expense_report_export_tmp_filename', '');
		if ($filename != '') {
			$path = ROOT.'/tmp/'.$filename;
			$size = filesize($path);
			
			$name = array_var($_SESSION, 'expense_report_export_filename', 'report.csv');
			
			unset($_SESSION['expense_report_export_tmp_filename']);
			unset($_SESSION['expense_report_export_filename']);
			
			// download file
			header("Cache-Control: public");
			header("Content-Description: File Transfer");
			header("Content-Disposition: attachment; filename=".$name."");
			header("Content-Transfer-Encoding: binary");
			header("Content-Type: text/csv");
			readfile($path);
			
			// delete tmp file
			//unlink($path);
		}
		die();
	}
	
}