<?php
/**
 * Payment controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class PaymentController extends ApplicationController {

	var $plugin_name = "expenses";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
		Env::useHelper('format');
	}
	/*
	function init() {
		require_javascript("PaymentManager.js", $this->plugin_name);
		ajx_current("panel", "payments-containerpanel", null, null, true);
		ajx_replace(true);
	}*/
	
	function list_all() {
		
		ajx_current("empty");
		// Get all variables from request
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		$attributes = array("ids" => explode(',', array_var($_GET,'ids')));
		$dim_order = null;
		$cp_order = null;
		
		$expense_id = array_var($_GET, 'expense_id');
		
		$all_status_codes = array();
		$all_status_info = ExpensesStatuses::instance()->getAllStatusInfo();
		foreach ($all_status_info as $st) {
			$all_status_codes[] = $st['code'];
		}
		
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		$select_columns = array("*");
		$extra_conditions = "";
		$join_params = null;
		
		if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		} else if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		} else {
			switch ($order) {
				case 'description':
					$order = '`e`.`description`';
					break;
				case 'date':
					$order = '`e`.`date`';
					break;
				case 'status_id':
					$order = '`jt`.`order`';
					$join_params = array(
						'table' => ExpensesStatuses::instance()->getTableName(),
						'jt_field' => 'id',
						'e_field' => 'status_id',
					);
					$select_columns = array("e.*, o.*, jt.`order`");
					break;
				case 'expense_id':
					$order = 'expense_name';
					$join_params = array(
						'table' => Objects::instance()->getTableName(),
						'join_type' => 'left',
						'jt_field' => 'id',
						'e_field' => 'expense_id',
					);
					$select_columns = array("e.*, o.*, jt.`name` as expense_name");
					break;
				default:
					if (in_array($order, $all_status_codes)) {
						
						$select_columns = array("e.*, o.*, jt.`amount`");
						$join_params = array(
							'join_type' => 'LEFT',
							'table' => PaymentHistories::instance()->getTableName(),
							'jt_field' => 'payment_id',
							'e_field' => 'object_id',
							'on_extra' => " AND jt.id = (
									SELECT max(x.id) FROM ".TABLE_PREFIX."payment_history x
									WHERE x.payment_id=jt.payment_id AND x.status_id=(SELECT id FROM ".TABLE_PREFIX."expenses_status WHERE code='$order')
								)"
						);
						
						$order = "jt.`amount`";
							
					} else {
						$order = "name";
					}
					break;
			}
		}
		
		Hook::fire("listing_extra_conditions", array('manager' => Payments::instance(), 'request' => $_REQUEST), $extra_conditions);
		
		if ($expense_id) {
			$extra_conditions .= " AND e.expense_id='$expense_id' ";
		}
		
		
		$filters = array();
		if (array_var($_GET,'status_filter')) $filters['status_id'] = array_var($_GET,'status_filter');

		foreach ($filters as $col => $value) {
			if (!$value || $value == '0') continue;
			$extra_conditions .= " AND e.`$col` = " . DB::escape($value);
		}
		
		if (array_var($_GET, 'from_filter')) {
			$from_date = DateTimeValueLib::dateFromFormatAndString(user_config_option('date_format'), array_var($_GET, 'from_filter'));
			if ($from_date instanceof DateTimeValue) {
				$extra_conditions .= " AND e.date >= '".$from_date->toMySQL()."'";
			}
		}
		if (array_var($_GET, 'to_filter')) {
			$to_date = DateTimeValueLib::dateFromFormatAndString(user_config_option('date_format'), array_var($_GET, 'to_filter'));
			if ($to_date instanceof DateTimeValue) {
				$extra_conditions .= " AND e.date <= '".$to_date->toMySQL()."'";
			}
		}
		
		$only_count_result = array_var($_GET, 'only_result', false);
		
		$result = Payments::instance()->listing(array(
			"order" => $order,
			"order_dir" => $order_dir,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"select_columns" => $select_columns,
			"extra_conditions" => $extra_conditions,
			"count_results" => false,
			"only_count_results" => $only_count_result,
			"join_params" => $join_params,
			"use_like_in_searchable_objects" => true,
			"start" => $start,
			"limit" => $limit,
		));
		
		$payments = $result->objects;
		$object = $this->prepareObject($payments, $start, $limit, $result);
		
		ajx_current("empty");
		ajx_extra_data($object);
		tpl_assign("listing", $object);
		
	}
	
	private $payment_cps = null;
	
	private function prepareObject($payments, $start, $limit, $res_obj) {
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"payments" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
		
		if (!$this->payment_cps) {
			$this->payment_cps = CustomProperties::getAllCustomPropertiesByObjectType(Payments::instance()->getObjectTypeId());
		}
		$custom_properties = $this->payment_cps;
		$count = min(array($limit, count($payments)));
		for ($i = 0; $i < $count; $i++){
			if (isset($payments[$i])){
				$payment = $payments[$i];
				if ($payment instanceof Payment){
					$info = $payment->getArrayInfo(true);
					
					$info['memPath'] = json_encode($payment->getMembersIdsToDisplayPath(true));
					$object["payments"][$i] = $info;
					
					foreach ($custom_properties as $cp) {
						$object["payments"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $payment);
					}
    			}
			}
		}
		
		return $object;
	}
	
	
	
	function add() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$payment_data = array_var($_POST, 'payment');
		
		if(!is_array($payment_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$p = new Payment();
			$p->setObjectTypeId(Payments::instance()->getObjectTypeId());

			$expense = Expenses::instance()->findById(get_id('expense_id'));
			$p->setExpenseId(get_id('expense_id'));
			
			tpl_assign('payment', $p);
			tpl_assign('expense', $expense);
			
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$payment = $this->save_payment($payment_data, $member_ids);
				
				DB::commit();
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
				return;
			}
			
			ApplicationLogs::createLog($payment, ApplicationLogs::ACTION_ADD);
			flash_success(lang('success add payment', $payment->getObjectName()));
			ajx_current("back");
			
			if (array_var($_REQUEST, 'modal')) {
				evt_add("reload current panel");
			}
		}
	}
	
	function edit() {
		$this->setTemplate('add');
		
		$payment = Payments::instance()->findById(get_id());
		if(!($payment instanceof Payment)) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		
		if (logged_user()->isGuest() || !$payment->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		
		$payment_data = array_var($_POST, 'payment');
		
		if(!is_array($payment_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			tpl_assign('payment', $payment);
			
			$expense = Expenses::instance()->findById($payment->getExpenseId());
			tpl_assign('expense', $expense);
			
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$log_data = '';
				$payment = $this->save_payment($payment_data, $member_ids, $payment, $log_data);
				
				DB::commit();
	          	
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
				return;
			}
				
			ApplicationLogs::createLog($payment, ApplicationLogs::ACTION_EDIT, false, null, true, $log_data);
			flash_success(lang('success edit payment', $payment->getObjectName()));
			ajx_current("back");
			
			if (array_var($_REQUEST, 'modal')) {
				evt_add("reload current panel");
			}
		}
	}
	
	function save_payment($payment_data, $member_ids, $payment = null, &$log_data='') {
		
		$is_new = false;
		if (is_null($payment) || !$payment instanceof Payment) {
			$payment = new Payment();
			$is_new = true;
			$old_data = array();
		} else {
			$old_data = $payment->getArrayInfo(false);
		}
		
		// check if its expense has been changed
		$old_expense = null;
		$expense_changed = false;
		if (!$is_new) {
			$new_expense_id = array_var($payment_data, 'expense_id');
			if ($new_expense_id && $new_expense_id != $payment->getExpenseId()) {
				$expense_changed = true;
				$old_expense = Expenses::instance()->findById($payment->getExpenseId());
			}
		}
		
		$payment_data['date'] = getDateValue($payment_data['date']);
		$payment->setFromAttributes($payment_data);
		$payment->setDescription($payment->getName());
		
		$using_expenses = true;
		Hook::fire('payments_use_expenses', array('object' => $payment, 'genid' => ''), $using_expenses);
		
		if ($using_expenses) {
			$expense = Expenses::instance()->findById($payment->getExpenseId());
			if (!$expense instanceof Expense) {
				throw new Exception(lang('cant save payment without selecting an expense'));
			}
			
			if ($is_new || $expense_changed) {
				// set payment number
				$max_pm_row = DB::executeOne("SELECT max(payment_num)+1 as max_pm FROM ".TABLE_PREFIX."payments WHERE expense_id=".$expense->getId());
				$payment->setPaymentNum($max_pm_row['max_pm']);
			}
		}
		
		Hook::fire('before_save_payment', array('object' => $payment, 'members' => $member_ids), $member_ids);
		$payment->save();
		
		if ($using_expenses) {
			$exp_members = $expense->getMembers();
			foreach ($exp_members as $em) {
				$is_hidden = DimensionContentObjectOptions::getOptionValue($em->getDimensionId(), $payment->getObjectTypeId(), 'hide_member_selector_in_forms');
				if ($is_hidden) {
					$member_ids[] = $em->getId();
				}
			}
		}
		
		$cp_data = null;
		if (isset($payment_data['object_custom_properties'])) {
			$cp_data = $payment_data['object_custom_properties'];
		}
		
		
		$object_controller = new ObjectController();
		if (!is_null($member_ids)) {
			$object_controller->add_to_members($payment, $member_ids);
		}
		
		$object_controller->add_subscribers($payment);
		$object_controller->link_to_new_object($payment);
		$object_controller->add_custom_properties($payment, $cp_data);
		
		// guardar en el historico de expenses los cambios realizados
		$payment->addToHistory(logged_user(), ($is_new ? 'add' : 'edit'));
		
		if ($using_expenses) {
			if ($expense_changed) {
				// change all payment history records to the new expense
				$hist_records = PaymentHistories::instance()->findAll(array('conditions' => "payment_id=".$payment->getId()));
				foreach ($hist_records as $h) {
					$h->setExpenseId($payment->getExpenseId());
					$h->setPaymentNum($payment->getPaymentNum());
					$h->save();
				}
				$old_expense->updateCalculatedTotals();
				$old_expense->refreshStatus();
				
				$log_data = "expense changed ".$old_expense->getId()." -> ".$payment->getExpenseId();
			}
			
			// ensure that the payment's expense is classified in every member of the payment
			if ($expense instanceof Expense) {
				$more_exp_member_ids = array();
				$current_exp_member_ids = $expense->getMemberIds();
				foreach ($member_ids as $mem_id) {
					if (!in_array($mem_id, $current_exp_member_ids)) {
						$more_exp_member_ids[] = $mem_id;
					}
				}
				if (count($more_exp_member_ids) > 0) {
					$more_exp_members = Members::instance()->findAll(array('conditions' => "id IN (".implode(',', $more_exp_member_ids).")"));
					if (count($more_exp_members) > 0) {
						$expense->addToMembers($more_exp_members);
					}
				}
			}
			
			// refresh expense status
			$expense->refreshStatus();
		}
		
		$null = null;
		Hook::fire('after_save_payment', array('object' => $payment, 'data' => $payment_data, 'old_data' => $old_data), $null);
		
		return $payment;
	}
	
	function view() {
		
		ajx_set_no_toolbar();
		$payment = Payments::instance()->findById(get_id());

		if(!($payment instanceof Payment)) {
			flash_error(lang('message dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$payment->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		ajx_extra_data(array("title" => $payment->getObjectName(), 'icon' => $payment->getIconClass()."s"));
		tpl_assign('payment', $payment);
	}
	
	/**
	 * Resolve action to perform
	 *
	 * @param string $action
	 * @param array $attributes
	 * @return string $message
	 */
	private function resolveAction($action, $attributes){
				 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				$messages = array();
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$payment = Payments::instance()->findById($id);
					if (isset($payment) && $payment->canDelete(logged_user())){
						try{
							DB::beginWork();
							$payment->trash();
							DB::commit();
							ApplicationLogs::createLog($payment, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$messages[] = $e->getMessage();
							$err++;
						}
					} else {
						$err++;
						$messages[] = lang('cannot delete executed expense', $payment instanceof Payment ? clean($payment->getName()) : '');
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) .("\n - ".implode("\n - ",$messages)). ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			case "archive":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$payment = Payments::instance()->findById($id);
					if (isset($payment) && $payment->canEdit(logged_user())){
						try{
							DB::beginWork();
							$payment->archive();
							DB::commit();
							ApplicationLogs::createLog($payment, ApplicationLogs::ACTION_ARCHIVE);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error archive objects", $err) . "<br />" . ($succ > 0 ? lang("success archive objects", $succ) : "");
				} else {
					$resultMessage = lang("success archive objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;		
		} // switch
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
	}
	
	
	function show_payment_history() {
		ajx_current("empty");
	
		$offset = array_var($_REQUEST, 'offset', 0);
		$$limit = array_var($_REQUEST, 'limit', 20);
		
		$payment_id = array_var($_REQUEST, 'id');
		
		if(!$payment_id) {
		    flash_error(lang('invalid request'));
		    ajx_current("empty");
		    return;
		}
		
		$history = PaymentHistories::instance()->getFormattedHistory($payment_id, $offset, $limit);
	
		tpl_assign('title', lang('payment history'));
		tpl_assign('history', $history['data']);
		$html = tpl_fetch(get_template_path('payment_history', 'expense', 'expenses'));
	
		ajx_extra_data(array('html' => $html));
	
		return $html;
	}
	
	
	function get_payment_list_config() {
		ajx_current("empty");
		
		ob_start();
		tpl_display(get_template_path('payments_list_grid_config/columns', 'payment', 'expenses'));
		tpl_display(get_template_path('payments_list_grid_config/toolbar', 'payment', 'expenses'));
		
		$html = ob_get_clean();
		
		ajx_extra_data(array('html' => $html));
	}
	
	
	
	
}