<?php
/**
 * Payment Types controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class PaymentTypesController extends ApplicationController {

	var $plugin_name = "expenses";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
		Env::useHelper('format');
	}
	
	function list_all() {
		
		if (!can_manage_configuration(logged_user())) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$payment_types = PaymentTypes::instance()->findAll();
		$payment_types_info = array();
		
		foreach ($payment_types as $pt) {
			$payment_types_info[] = $pt->getArrayInfo();
		}
		
		tpl_assign('payment_types_info', $payment_types_info);
		
	}
	
	
	
	
	function add() {
		if (!can_manage_configuration(logged_user())) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$payment_type = new PaymentType();
		
		tpl_assign('object', $payment_type);
		tpl_assign('attributes', array());
		
		$data = array_var($_POST, 'payment_type');
		if (is_array($data)) {
			try {
				DB::beginWork();
				
				$this->save_payment_type($data, $payment_type);
				
				DB::commit();
				
				flash_success(lang('success add payment type', $payment_type->getName()));
				evt_add("reload current panel");
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				return;
			}
		}
	}
	
	function edit() {
		$this->setTemplate('add');
		
		if (!can_manage_configuration(logged_user())) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$payment_type = PaymentTypes::instance()->findById(get_id());
		$attributes = $payment_type->getAttributes();
		
		tpl_assign('object', $payment_type);
		tpl_assign('attributes', $attributes);
		
		$data = array_var($_POST, 'payment_type');
		if (is_array($data)) {
			try {
				DB::beginWork();
		
				$this->save_payment_type($data, $payment_type);
		
				DB::commit();
				
				flash_success(lang('success edit payment type', $payment_type->getName()));
				evt_add("reload current panel");
		
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				return;
			}
		}
	}
	
	private function save_payment_type($data, $payment_type) {
		
		$payment_type->setFromAttributes($data);
		$payment_type->save();
		
		$attributes = array_var($data, 'attributes');
		
		$rows = DB::executeAll("SELECT max(attribute_id) as maxid FROM ".TABLE_PREFIX."payment_type_attributes WHERE payment_type_id=".$payment_type->getId());
		$next_attr_id = $rows[0]['maxid'] + 1;
		$order = 0;
		
		if (is_array($attributes)) {
			
			foreach ($attributes as $k => $attr) {
				
				if (array_var($attr, 'id') == -1) {
					// ignore form template
					continue;
				}
				
				if (array_var($attr, 'deleted')) {
					
					if (array_var($attr, 'id') > 0) {
						PaymentTypeAttributes::instance()->delete("payment_type_id=".$payment_type->getId()." AND attribute_id=".array_var($attr, 'id'));
					}
					
				} else {
					
					if (trim(array_var($attr, 'name')) == '') {
						// ignore attributes without name
						continue;
					}
					
					$order++;
					
					$attribute = PaymentTypeAttributes::instance()->findById(array('payment_type_id' => $payment_type->getId(), 'attribute_id' => array_var($attr, 'id')));
					if (!$attribute instanceof PaymentTypeAttribute) {
						$attribute = new PaymentTypeAttribute();
						$attribute->setPaymentTypeId($payment_type->getId());
						$attribute->setAttributeId($next_attr_id);
						$next_attr_id++;
					}
					$attribute->setName(array_var($attr, 'name'));
					$attribute->setType(array_var($attr, 'type'));
					$attribute->setOrder($order);
					$attribute->setIsRequired(array_var($attr, 'is_required'));
					
					$list_values = array_var($attr, 'list_values');
					if (array_var($attr, 'type') != 'list') {
						$list_values = "";
					}
					$attribute->setListValues($list_values);
					
					$attribute->save();
					
				}
			}
		} 
		
	}
	
	
	function delete() {
		ajx_current("empty");
	
		if (!can_manage_configuration(logged_user())) {
			flash_error("no access permissions");
			return;
		}
		
		$payment_type = PaymentTypes::instance()->findById(get_id());
		if (!$payment_type instanceof PaymentType) {
			flash_error('object dnx');
			return;
		}
		
		try {
			DB::beginWork();
			
			$name = $payment_type->getName();
			$payment_type->delete();
			
			DB::commit();
			
			flash_success(lang('success delete payment type', $name));
			evt_add("reload current panel");
			
		} catch(Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			return;
		}
	}
	
	
}