<?php

function expense_status_select_box($name, $selected = null, $attributes = null) {
	$status_options = array();
	$statuses = ExpensesStatuses::instance()->findAll(array('conditions' => 'enabled=1', 'order' => '`order`'));
	foreach ($statuses as $status) {
		$sel = array();
		if ($selected == $status->getId()) $sel['selected'] = 'selected';
		$text = $status->getName() == "" ? lang($status->getCode()) : $status->getName();
		$status_options[] = option_tag($text, $status->getId(), $sel);
	}
	return select_box($name, $status_options, $attributes);
}

function currency_select_box($name, $selected = null, $include_names = false, $attributes = null) {
	$currency_options = array();
	$currencies = Currencies::instance()->findAll();
	foreach ($currencies as $currency) {
		$sel = array();
		if ($selected == null) {
			if ($currency->getIsDefault()) $sel['selected'] = 'selected'; 
		} else {
			if ($selected == $currency->getId()) $sel['selected'] = 'selected';
		}
		$text = $currency->getSymbol() . ($include_names ? ' ('.$currency->getName().')' : '');
		$currency_options[] = option_tag($text, $currency->getId(), $sel);
	}
	return select_box($name, $currency_options, $attributes);
}


function get_payments_for_expense($id, $params=array(), $filter_payments_by_context = false) {
	
	$order = strtolower(array_var($params, 'order', 'date'));
	if (!in_array($order, array('date','status'))) {
		$order = 'date';
	}
	
	$order_dir = array_var($params, 'order_dir', 'ASC');
	if (!in_array(strtoupper($order_dir), array('ASC','DESC'))) {
		$order_dir = 'ASC';
	}
	
	$context_conditions = "";
	if ($filter_payments_by_context) {
		$active_member_ids = active_context_members(false);
		if (count($active_member_ids) > 0) {
			$mem_str = implode(',', $active_member_ids);
			$context_conditions = "
				AND EXISTS (
					SELECT pp.expense_id FROM ".TABLE_PREFIX."payments pp
					INNER JOIN ".TABLE_PREFIX."object_members pm ON pm.object_id=pp.object_id
						WHERE pm.member_id IN ($mem_str)
						AND pp.expense_id=".TABLE_PREFIX."payment_history.expense_id AND pp.payment_num=".TABLE_PREFIX."payment_history.payment_num
					GROUP BY pm.object_id HAVING count(pm.member_id) = ".count($active_member_ids)."
				)";
		}
	}
	
	$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
	
	$payments_by_pnum = array();
	$payments = array();
	foreach ($all_status as $status) {
		if ($status['only_for_expenses']) continue;
		
		// check permissions over payments
		$perm_conds_sql = "";
		if (!logged_user()->isAdministrator()) {
			$logged_user_pg_ids = implode(',', logged_user()->getPermissionGroupIds());
			$perm_conds_sql = "
				AND payment_id IN (
					SELECT sh.object_id FROM ".TABLE_PREFIX."sharing_table sh
					WHERE payment_id = sh.object_id AND sh.group_id  IN ($logged_user_pg_ids)
				)
			";
		}
			
		$symbol = null;
		$bps = PaymentHistories::instance()->findAll(array('conditions' => array('expense_id = ? AND status_id = ?
						'.$context_conditions.'
						'.$perm_conds_sql.'
						AND modification_log IN (\'add\', \'edit\')
						AND modification_date = (
							SELECT max(x.modification_date) FROM '.TABLE_PREFIX.'payment_history x
							WHERE x.expense_id='.$id.' AND x.status_id='.$status['id'].'
								AND x.payment_num='.TABLE_PREFIX.'payment_history.payment_num
						)', $id, $status['id']), 'order' => 'date'));
		
		foreach ($bps as $bp) {
			
			$p = Payments::instance()->findOne(array("conditions" => "expense_id = ".$bp->getExpenseId() . " AND payment_num = ". $bp->getPaymentNum()." AND trashed_by_id=0"));
			if (!$p instanceof Payment) continue;
			
			if ($bp->getStatusId() > $p->getStatusId() || (($status['is_temporal'] || $status['is_final']) && $p->getStatusId() != $status['id']) ) {
				continue;
			}
	
			$currency = Currencies::instance()->getCurrency($bp->getCurrency());
			$symbol = $currency ? $currency->getSymbol() : '$';
			
			$info = array(
					'id' => $p->getId(),
					'amount' => format_money_amount($bp->getAmount(), $symbol),
					'amount_raw' => $bp->getAmount(),
					'currency_id' => $bp->getCurrency(),
					'desc' => $bp->getDescription(),
					'pnum' => $bp->getPaymentNum(),
					'date' => $bp->getDate() ? format_date($bp->getDate(), null, 0) : '',
					'ts' => $bp->getDate() ? $bp->getDate()->getTimestamp() : 0,
					'status_code' => $status['code'],
					'status_order' => $status['order'],
					'status' => $status['name'],
			);
			
			//$k = ($bp->getDate() instanceof DateTimeValue ? $bp->getDate()->toMySQL() : '0000').'b'.$bp->getPaymentNum()."_".$status['id'];
	/*
			if ($bp->getTypeId() > 0) {
				$type = PaymentTypes::instance()->findById($bp->getTypeId());
				if ($type instanceof PaymentType) {
	
					$desc_additional = array();
					$type_attributes = $bp->getTypeAttributes();
					$definitions = PaymentTypeAttributes::getAttributesForType($bp->getTypeId());
	
					foreach ($type_attributes as $ta) {
						$val = trim($ta->getValue());
						if ($val != '') {
							$def = array_var($definitions, $ta->getAttributeId());
							if ($def instanceof PaymentTypeAttribute) {
								if ($def->getType() == 'contact' || $def->getType() == 'user') {
									$c = Contacts::instance()->findById($val);
									if ($c instanceof Contact) $val = $c->getObjectName();
									else $val = "";
								} else if ($def->getType() == 'boolean') {
									$val = ($val > 0 ? lang('yes') : lang('no'));
								} else if ($def->getType() == 'date') {
									$dt = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $val);
									if ($dt instanceof DateTimeValue) {
										$val = format_date($dt);
									} else {
										$val = "";
									}
								}
								$desc_additional[] = $def->getName().": ".$val;
							}
						}
					}
					if (count($desc_additional) > 0) {
						$desc_additional_str = implode(', ', $desc_additional);
						$info['desc'] = Payments::getDescriptionHtml($info['desc'], $desc_additional_str, $type);
					}
				}
			}*/
	
			if (!isset($payments_by_pnum[$bp->getPaymentNum()])) $payments_by_pnum[$bp->getPaymentNum()] = array();
			$payments_by_pnum[$bp->getPaymentNum()][$status['code']] = $info;
		}
	}
	
	$pays = array();
	foreach ($payments_by_pnum as $pnum => $codes_info) {
		foreach ($codes_info as $code => $info) {
			if (!isset($pays[$pnum])) {
				$pays[$pnum] = array(
						'id' => $info['id'],
						'desc' => $info['desc'],
						'date' => $info['date'],
						'ts' => $info['ts'],
						'pnum' => $info['pnum'],
				);
				
				Hook::fire('get_expense_payments_more_data', null, $pays[$pnum]);
				
			}
			$pays[$pnum]['status_code'] = $code;
			$pays[$pnum]['status_order'] = $info['status_order'];
			$pays[$pnum]['status'] = $info['status'];
			$pays[$pnum][$code] = $info;
		}
	}

	// reorder payments
	switch($order) {
		case 'status':
			$ord_key = 'status_order';
			$ord_opt = SORT_NUMERIC;
			break;
		case 'date':
		default: 
			$ord_key = 'ts';
			$ord_opt = SORT_NUMERIC;
	}
	
	foreach ($pays as $pnum => $p) {
		$ord_pays[$p[$ord_key] . "_".$p['pnum']] = $p;
	}
	ksort($ord_pays, $ord_opt);
	if ($order_dir == 'DESC') $ord_pays = array_reverse($ord_pays);
	
	// return ordered payments
	return array_values($ord_pays);
}