<?php
	class Payment extends BasePayment {
		
		function getArrayInfo($include_external_columns = true, $calculate_totals = false) {
			$info = array();
			Env::useHelper('format');
			
			$external_cols = $this->manager()->getExternalColumns();
			
			$info['amounts'] = array();
			
			$columns = $this->getColumns();
			foreach ($columns as $col) {
				if (in_array($col, $external_cols)) {
					
					if ($include_external_columns) {
						$info[$col] = $this->getExternalColumnValue($col);
					}
					
				} else {
					if ($col == 'amount') {
						$c = Currencies::instance()->getCurrency($this->getCurrency());
						if (!$c instanceof Currency) continue;
						
						$status = ExpensesStatuses::instance()->getStatus($this->getStatusId());
						if (!$status instanceof ExpensesStatus) continue;
						
						$info[$status->getCode()] = format_money_amount($this->getAmount(), $c->getSymbol());
						
						$info['amounts'][$status->getId()] = $this->getAmount();
						
						$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
						
						if ($calculate_totals) {
							// fill previous status amounts
							foreach ($all_status as $st) {
								// only process payment status, exclude temporal and final status (if not is the current) and exclude status with order greater than the current status
								if ($st['only_for_expenses'] || (($st['is_temporal'] || $st['is_final']) && $this->getStatusId() != $st['id']) || $st['order'] >= $status->getOrder()) continue;
								
								$ph = PaymentHistories::instance()->findOne(array('conditions' => array("payment_id=? AND modification_log IN ('add', 'edit') AND status_id=?",
										$this->getId(), $st['id']), 'order' => 'modification_date DESC'));
								if ($ph) {
									$info[$st['code']] = format_money_amount($ph->getAmount(), $c->getSymbol());
									$info['amounts'][$st['id']] = $ph->getAmount();
								}
							}
							
						} else {
							
							// get amounts from cache
							$info['amounts'] = array();
							$total_rows = DB::executeAll("SELECT currency_id, status_id, amount FROM ".TABLE_PREFIX."payment_totals WHERE payment_id=".$this->getId());
							foreach ($total_rows as $trow) {
								$cid = $trow['currency_id'];
								$currency = Currencies::instance()->getCurrency($cid);
								if (!$currency instanceof Currency) continue;
									
								$st = null;
								foreach ($all_status as $stat) {
									if ($stat['id'] == $trow['status_id']) {
										$st = $stat;
										break;
									}
								}
								if (!isset($info['amounts'][$st['id']])) $info['amounts'][$st['id']] = array();
								$info['amounts'][$st['id']] = $trow['amount'];
									
								$info[$st['code']] = format_money_amount($trow['amount'], $currency->getSymbol());
							}
						}
						
						$info['amount'] = $info[$status->getCode()];
						
					} else {
						$info[$col] = format_value_to_print($col, $this->getColumnValue($col), $this->getColumnType($col), $this->getObjectTypeId());
					}
				}
			}
			
			$info['name'] = $this->getObjectName();
			$info['id'] = $this->getId();
			$info['type'] = $this->getObjectTypeName();
				
			$status = ExpensesStatuses::instance()->getStatus($this->getStatusId());
			$info['status'] = $status instanceof ExpensesStatus ? ($status->getName() == '' ? lang($status->getCode()) : $status->getName()) : '';
			$info['status_code'] = $status instanceof ExpensesStatus ? $status->getCode() : '';
			
			$info['date'] = $this->getDate() instanceof DateTimeValue ? format_date($this->getDate(), null, 0) : '';
			
			Hook::fire('modify_object_get_array_info', array('object' => $this), $info);
			
			return $info;
		}
		
		
		function updateCalculatedTotals($update_expense_totals = true) {
		
			$values = array();
			$info = array();
			if ($this->getTrashedById() == 0) {
				$info = $this->getArrayInfo(false, true);
			}
			
			foreach ($info['amounts'] as $st_id => $amount) {
				if ($amount) {
					$values[] = "(".$this->getId().", $st_id, ".$this->getCurrency().", $amount)";
				}
			}
			
			DB::execute("DELETE FROM ".TABLE_PREFIX."payment_totals WHERE payment_id=".$this->getId());
			if (count($values) > 0) {
				DB::execute("INSERT INTO ".TABLE_PREFIX."payment_totals VALUES ".implode(',',$values)." ON DUPLICATE KEY UPDATE payment_id=payment_id;");
			}
			
			if ($update_expense_totals) {
				$e = Expenses::instance()->findById($this->getExpenseId());
				if ($e instanceof Expense) {
					$e->updateCalculatedTotals();
				}
			}
		
		}
		
		
		
		
		/**
		 * Returns the expense id. Overrides parent object id implementation.
		 * {@inheritDoc}
		 * @see ContentDataObject::getParentObjectId()
		 */
		function getParentObjectId() {
			return $this->getExpenseId();
		}
		
		
		function getExternalColumnValue($column) {
			switch ($column) {
				case 'expense_id':
					$e = Expenses::instance()->findById($this->getExpenseId());
					$formatted_val = $e instanceof Expense ? $e->getName() : '';
					break;
				case 'currency':
					$c = Currencies::instance()->getCurrency($this->getCurrency());
					$formatted_val = $c instanceof Currency ? $c->getSymbol() : '';
					break;
				case 'status_id':
					$status = ExpensesStatuses::instance()->getStatus($this->getStatusId());
					$formatted_val = $status instanceof ExpensesStatus ? ($status->getName() == '' ? lang($status->getCode()) : $status->getName()) : '';
					break;
				case 'quota_id':
					if (Plugins::instance()->isActivePlugin('advanced_expenses')) {
						$quota_id = $this->getColumnValue('quota_id');
						if ($quota_id > 0) {
							$quota = Quotas::instance()->findById($quota_id);
							if ($quota instanceof Quota) {
								$formatted_val = $quota->getName();
							}
						}
					}
					break;
			}
			
			return $formatted_val;
		}
		
		
		static function canAdd(Contact $user, $context, &$notAllowedMember = ''){
			return can_add($user, $context, Payments::instance()->getObjectTypeId(), $notAllowedMember);
		}
		
		/**
		 * Returns true if $user can access this message
		 *
		 * @param Contact $user
		 * @return boolean
		 */
		function canView(Contact $user) {
			return can_read($user, $this->getMembers(), $this->getObjectTypeId());
		} // canView
		
		/**
		 * Check if specific user can edit this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canEdit(Contact $user) {
			$status = ExpensesStatuses::instance()->findById($this->getStatusId());
			$final_st = ExpensesStatuses::getFinalStatus();
			$final_status_code = $final_st->getCode();
				
			if (!$status instanceof ExpensesStatus || $status->getCode() == $final_status_code || $status->getCode() == 'part_'.$final_status_code) {
				return can_edit_executed_payments_pg_array($user->getPermissionGroupIds());
			}
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canEdit
		
		
		/**
		 * Check if specific user can delete this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canDelete(Contact $user) {
			$status = ExpensesStatuses::instance()->findById($this->getStatusId());
			$final_st = ExpensesStatuses::getFinalStatus();
			$final_status_code = $final_st->getCode();
			
			if (!can_edit_executed_payments($user->getPermissionGroupId())) {
				if (!$status instanceof ExpensesStatus || $status->getCode() == $final_status_code || $status->getCode() == 'part_'.$final_status_code) {
					return false;
				}
			}
			return can_delete($user,$this->getMembers(), $this->getObjectTypeId());
		}
		
		function addToHistory($user, $log_data, $check_for_modifications=true, $update_expense_totals=true) {
			
			// dont save duplicated logs
			$save_history = true;
			if ($check_for_modifications && !$this->hasChangesSinceLastEdition()) {
				$save_history = false;
			}
			
			if ($save_history) {
				$history = new PaymentHistory();
				
				$columns = $this->manager->getColumns();
				foreach ($columns as $col) {
					if ($history->columnExists($col)) {
						$history->setColumnValue($col, $this->getColumnValue($col));
					}
				}
				
				$status = ExpensesStatuses::instance()->findById($this->getStatusId());
				$history->setStatusName($status->getCode());
				
				$history->setUserId($user->getId());
				$history->setModificationDate(DateTimeValueLib::now());
				$history->setModificationLog($log_data);
				
				$history->setPaymentId($this->getId());
				
				$history->save();
			}
			
			$this->updateCalculatedTotals($update_expense_totals);
		}
		
		function hasChangesSinceLastEdition() {
			
			$history = PaymentHistories::instance()->findOne(array(
					'conditions' => array("expense_id=? AND payment_num=?", $this->getExpenseId(), $this->getPaymentNum()),
					'order' => 'modification_date DESC',
			));
			if (!$history instanceof PaymentHistory) {
				return true;
			}
			
			$columns = $this->manager->getColumns();
			foreach ($columns as $col) {
				if ($col == 'object_id') continue;
				
				if ($this->getColumnValue($col) != $history->getColumnValue($col)) {
					return true;
				}
			}
			return false;
		}
		
		
		function getTypeAttributes() {
			
			return Payments::getTypeAttributes($this);
			
		}
		
		function getDescriptionWithAttributes() {
			
			return Payments::getDescriptionWithAttributes($this, $separator);
			
		}
		
		function getAttributesString($separator=", ", $type='html') {
				
			return Payments::getDescriptionWithAttributes($this, $separator, false, false, $type);
				
		}
		
		
		/*
		 * override trash, delete and untrash methods to recalculate the totals
		 */

		function trash() {
			parent::trash();
			$this->addToHistory(logged_user(), 'delete');
		}
		
		function untrash() {
			parent::untrash();
			$this->addToHistory(logged_user(), 'untrash');
		}
		
		function delete() {
			parent::delete();
			$this->addToHistory(logged_user(), 'delete');
		}
	}