<?php
Env::useHelper('format');
if (isset($expense) && $expense instanceof Expense) {
	$genid = gen_id();
	if (!$expense->isTrashed()){
		if($expense->canEdit(logged_user())) {
			add_page_action(lang('edit'), "javascript:og.render_modal_form('', {c:'expense', a:'edit', params: {id:".$expense->getId()."}});", 'ico-edit', null, null, true);
			if (!$expense->isArchived())
				add_page_action(lang('archive'), "javascript:if(confirm(lang('confirm archive object'))) og.openLink('" . $expense->getArchiveUrl() ."');", 'ico-archive-obj');
			else
				add_page_action(lang('unarchive'), "javascript:if(confirm(lang('confirm unarchive object'))) og.openLink('" . $expense->getUnarchiveUrl() ."');", 'ico-unarchive-obj');
			
			add_page_action(lang('create copy of this expense'), "javascript:og.render_modal_form('', {c:'expense', a:'copy_expense', params: {id:".$expense->getId()."}});", 'ico-copy', null, null, true);
		} // if
	}
	if ($expense->canDelete(logged_user())) {
		if ($expense->isTrashed()) {
			add_page_action(lang('restore from trash'), "javascript:if(confirm(lang('confirm restore objects'))) og.openLink('" . $expense->getUntrashUrl() ."');", 'ico-restore', null, null, true);
			add_page_action(lang('delete permanently'), "javascript:if(confirm(lang('confirm delete permanently'))) og.openLink('" . $expense->getDeletePermanentlyUrl() ."');", 'ico-delete', null, null, true);
		} else {
			add_page_action(lang('move to trash'), "javascript:if(confirm(lang('confirm move to trash'))) og.openLink('" . $expense->getTrashUrl() ."');", 'ico-trash', null, null, true);
		}
	}
?>

<div style="padding:7px">
<div class="expense">
<?php
	$use_date_in_expense_level = config_option("use_date_in_expense_level");
	
	$status = ExpensesStatuses::instance()->getStatus($expense->getStatus());
	$content = '';
	$st_html = '<span class="expense-row-status-'.($status instanceof ExpensesStatus ? $status->getCode() : '').'">'.($status instanceof ExpensesStatus ? lang($status->getCode()) : '').'</span>';
	$content .= '<div class="expense-property"><span class="bold ">'.lang('status').': </span>'.$st_html.'</div>';
	if ($use_date_in_expense_level) {
		$content .= '<div class="expense-property"><span class="bold ">'.lang('date').': </span>'.format_date($expense->getDate(), null, 0).'</div>';
	}
	
	if (trim($expense->getDescription()) != "") {
		$content .= '<div class="expense-property"><span class="bold">'.lang('description').': </span>'.$expense->getDescription().'</div>';
	}
	
	$content .= '<div class="expense-property"><span class="bold">'.lang('payments').': </span></div>';
	$content .= '<div id="'.$genid.'_payments_grid_container" class="payments-grid-container"></div>';

	
	
	
	$content .= '<div class="clear"></div>';
	
	tpl_assign("content", $content);
	tpl_assign("object", $expense);
	tpl_assign('iconclass', $expense->isTrashed()? 'ico-large-expense-trashed' : ($expense->isArchived() ? 'ico-large-expense-archived' : 'ico-large-expense'));
	
	$this->includeTemplate(get_template_path('view', 'co'));

	
	// include payment grid columns and toolbar configuration
	tpl_assign("genid", $genid);
	$this->includeTemplate(get_template_path('expense_payment_grid/columns', 'expense', 'expenses'));
	$this->includeTemplate(get_template_path('expense_payment_grid/toolbar', 'expense', 'expenses'));
	
?>
</div>
</div>
<style>
.payments-grid-container .list-totals-row-container {
	position: relative;
	bottom: inherit;
}

</style>
<script>

var grid_id = '<?php echo $genid?>_payments_grid';

var status_options = [[0, '-- ' + lang('All') + ' --']];
if (og.expenses.all_status) {
	for (x=0; x<og.expenses.all_status.length; x++) {
		var st = og.expenses.all_status[x];
		if (st.only_for_expenses) continue;
		
		status_options.push([st.id, st.name]);
	}
}

var filters = {
	from_filter: {type:'date', label:lang('from date')+":"},
	to_filter: {type:'date', label:lang('to date')+":"},
	status_filter: {type:'list', label:lang('status'), options: status_options, initial_val:0, width:120}
};

var paymentsgrid = new og.ObjectGrid({
	renderTo: '<?php echo $genid?>_payments_grid_container',
	url: og.getUrl('payment', 'list_all'),
	type_name: 'payment',
	response_objects_root: 'payments',
	grid_id: grid_id,
	nameRenderer: og.gridObjectNameRenderer,
	store_params: {
		url_controller: 'payment',
		url_action: 'list_all',
		expense_id: <?php echo $expense->getId()?>
	},
	filters: filters,
	checkbox_sel_model: true,
	columns: payment_columns,
	add_default_actions_column: false,
	tbar_items: tbar_items,
	stateId: 'expense-payments-list', // to remember the gui state independent of the panel
});

var active_tab = Ext.getCmp('tabs-panel').activeTab;
if (active_tab.events.resize.listeners.length < 2) {
	active_tab.on('resize', function() {
		var g = Ext.getCmp(grid_id)
		if (g) g.fireEvent('resize');
	});
}

paymentsgrid.load({
	load_callback: function() {
		og.expenses.addTotalsRow(Ext.getCmp(grid_id).store, {expense_id:'<?php echo $expense->getId()?>'}, grid_id);
	}
});

</script>

<?php } //if isset ?>
