<?php
	Env::useHelper('format');

	function payments_report_print_group($group_obj, $grouped_objects, $options, $skip_groups = array(), $level = 0, $more_info, $prev = "", &$crumbJs = "", &$totals=array()) {
		
		$margin_left = 15 * $level;
		$cls_suffix = $level > 2 ? "all" : $level;
		$next_level = $level + 1;
		
		$group_name = $group_obj['group']['name'];
		//echo '<div style="margin-left:' . $margin_left.'px;" class="report-group-heading-'.$cls_suffix.'">' . $group_name . '</div>';
		
		echo '<tr><td colspan="'.count($options['ordered_columns']).'" style="padding-left:' . $margin_left.'px;" class="report-group-heading-'.$cls_suffix.'">' . $group_name . '</td></tr>';
		
		$mem_index = $prev . $group_obj['group']['id'];
		
		
		// draw the table for the values
		if (isset($grouped_objects[$mem_index]) && count($grouped_objects[$mem_index]) > 0) {
			$tmp_totals = null;
			payments_report_print_table($grouped_objects[$mem_index], $margin_left, $options, $group_name, $more_info, $crumbJs, $tmp_totals);
			
			foreach ($more_info['status_info'] as $st) {
				if ($st['only_for_expenses']) continue;
				$totals[$st['code']] += $tmp_totals[$st['code']];
			}
		}
		
		if (!is_array($group_obj['subgroups'])) return;
		
		$subgroups = order_groups_by_name($group_obj['subgroups']);
		foreach ($subgroups as $subgroup) {
			$tmp_totals = null;
			payments_report_print_group($subgroup, $grouped_objects, $options, array(), $next_level, $more_info, $prev . $group_obj['group']['id'] . "_", $crumbJs, $tmp_totals);
			
			foreach ($more_info['status_info'] as $st) {
				if ($st['only_for_expenses']) continue;
				$totals[$st['code']] += $tmp_totals[$st['code']];
			}
		}
		
		// TODO: Fix this using the expense currency, for now only one currency will be used
		$c = Currencies::instance()->findOne();
		$c_sym = $c instanceof Currency ? $c->getSymbol() : '$';
		
		$status_codes = array();
		foreach ($more_info['status_info'] as $st) {
			if ($st['only_for_expenses']) continue;
			$status_codes[] = $st['code'];
		}
		
		echo '<tr>';
		
		$first = true;
		foreach ($options['ordered_columns'] as $colinfo) {
			if ($first) {
				echo '<td style="padding-left:'.$margin_left.'px; font-size:13px;" class="bold">'.lang('total').' '.$group_name.'</td>';
				
			} else {
				if (in_array($colinfo['id'], $status_codes)) {
					echo '<td class="right bold" style="min-width:100px;max-width:100px;padding:2px 4px; font-size:13px;">'.format_money_amount($totals[$colinfo['id']], $c_sym).'</td>';
				} else {
					echo '<td></td>';
				}
			}
			$first = false;
		}
		
		echo '</tr>';
	}
	
	
	function payments_report_print_table($objects, $left, $options, $group_name, $more_info, &$crumbJs = "", &$table_totals) {
		
		$alt_cls = "";
		$table_totals = array();
		foreach ($objects as $e) {
			payments_report_print_expense($e, $left, $options, $more_info, $alt_cls, $crumbJs);
			foreach ($more_info['status_info'] as $st) {
				if ($st['only_for_expenses']) continue;
				if (!isset($table_totals[$st['code']])) $table_totals[$st['code']] = 0;
				$table_totals[$st['code']] += array_var($more_info['totals'][$e->getId()], $st['code']);
			}
		}
	}
	
	
	function payments_report_print_expense($exp, $margin_left, $options, $more_info, &$alt_cls='', &$crumbJs='') {
		echo "<tr $alt_cls>";
		
		$status_codes = array();
		foreach ($more_info['status_info'] as $st) {
			if ($st['only_for_expenses']) continue;
			$status_codes[] = $st['code'];
		}
		
		$this_expense_related_to = "";
		$this_expense_cps = array();
		
		foreach ($options['ordered_columns'] as $colinfo) {
			switch ($colinfo['id']) {
				
				case 'name':
					echo "<td class='name bold'><div style='padding-left:".$margin_left."px;'>" . clean($exp->getObjectName()) ."</div></td>";
					break;
					
				case 'related_to':
					echo '<td style="width:200px;"><span id="t-'.$exp->getId().'" style="font-weight:normal;"></span></td>';
					$crumbOptions = $exp->getMembersToDisplayPath();
					$crumbJs .= escape_character("var crumbHtml = og.getEmptyCrumbHtml(".json_encode($crumbOptions).");");
					$crumbJs .= 'document.getElementById("t-'. $exp->getId().'").innerHTML = crumbHtml;';
					
					break;
					
				case 'description':
					echo "<td>" . clean(html_to_text($exp->getDescription())) ."</td>";
					break;
					
				case 'payment_type':
				case 'payment_type_details':
					echo '<td></td>';
					break;
					
				case 'date':
					echo "<td>" . (config_option("use_date_in_expense_level") ? format_date($exp->getDate()) : "") ."</td>";
					break;
					
				case 'status':
					$st = null;
					$status_info = $more_info['status_info'];
					foreach ($status_info as $status) {
						if ($status['id'] == $exp->getStatus()) {
							$st = $status;
							break;
						}
					}
					echo "<td class='name'>" . $st['name'] . "</td>";
					break;
					
				default:
					if (in_array($colinfo['id'], $status_codes)) {
						
						// TODO: Fix this using the expense currency, for now only one currency will be used
						$c = Currencies::instance()->findOne();
						$c_sym = $c instanceof Currency ? $c->getSymbol() : '$';
						
						$totals = $more_info['totals'];
						echo "<td class='bold right'>".format_money_amount(array_var($totals[$exp->getId()], $colinfo['id'], 0), $c_sym)."</td>";
						
					} else if (str_starts_with($colinfo['id'], "cp_")) {
						// Custom properties
						
						$cp_id = str_replace("cp_", "", $colinfo['id']);
						$cp_val = "";
						$cp = CustomProperties::instance()->findById($cp_id);
						if ($cp instanceof CustomProperty) {
							$cp_val = get_custom_property_value_for_listing($cp, $exp);
						}
						echo "<td>".$cp_val.'</td>';
						
						$this_expense_cps[$colinfo['id']] = $cp_val;
						
					}
					break;
			}
		}
		echo "</tr>";
		
		$payments = $more_info['payments'];
		
		$st_code = $st['code'];
		if (str_starts_with($st_code, "part_")) {
			$st_code = str_replace("part_", "", $st_code);
		}
		$payments_array = array_var($payments[$exp->getId()], $st_code, array());
		
		foreach ($payments_array as $p) {
			/* @var $p Payment */
			$currency = Currencies::instance()->getCurrency($p->getCurrency());
			echo '<tr '.$alt_cls.' style="border-top:1px dotted #ddd;">';
			
			foreach ($options['ordered_columns'] as $colinfo) {
				switch ($colinfo['id']) {
		
				case 'name':
					if (array_var($options, 'print_expense_props_in_each_payment')) {
						echo "<td><div style='padding-left:".$margin_left."px;'>" . clean($exp->getObjectName()) ."</div></td>";
					} else {
						echo '<td></td>';
					}
					break;
						
				case 'related_to':
					if (array_var($options, 'print_expense_props_in_each_payment') && isset($crumbOptions) && is_array($crumbOptions) && count($crumbOptions) > 0) {
						echo '<td style="width:200px;"><span id="t-'.$exp->getId()."-".$p->getPaymentNum().'" style="font-weight:normal;"></span></td>';
						$crumbJs .= escape_character("var crumbHtml = og.getEmptyCrumbHtml(".json_encode($crumbOptions).");");
						$crumbJs .= 'document.getElementById("t-'. $exp->getId().'-'.$p->getPaymentNum().'").innerHTML = crumbHtml;';
					} else {
						echo '<td></td>';
					}
					break;
				
				case 'date':
					echo '<td>'.(config_option("use_date_in_expense_level") ? "" : format_date($p->getDate())).'</td>';
					break;
					
				case 'description':
					echo '<td>'. $p->getDescription() .'</td>';
					break;
						
				case 'payment_type':
					$pt = PaymentTypes::getById($p->getTypeId());
					echo '<td>'. ($pt instanceof PaymentType ? $pt->getName() : '') .'</td>';
					break;
						
				case 'payment_type_details':
					$pt = PaymentTypes::getById($p->getTypeId());
					echo '<td>'. ($pt instanceof PaymentType ? $p->getAttributesString(";") : '') .'</td>';
					break;
				
				case 'status':
					$st = null;
					$status_info = $more_info['status_info'];
					foreach ($status_info as $status) {
						if ($status['id'] == $exp->getStatus()) {
							$st = $status;
							break;
						}
					}
					echo "<td class='name'>" . $st['name'] . "</td>";
					break;
						
				default:
					if (in_array($colinfo['id'], $status_codes)) {

						if ($colinfo['id'] == $st_code) {
							echo '<td class="right" style="width:100px;">'.format_money_amount($p->getAmount(), $currency->getSymbol()) .'</td>';
						} else {
							echo '<td class="right" style="width:100px;">'.format_money_amount(0, $currency->getSymbol()) .'</td>';
						}
						
					} else if (str_starts_with($colinfo['id'], "cp_")) {
						
						if (array_var($options, 'print_expense_props_in_each_payment')) {
							echo "<td>". array_var($this_expense_cps, $colinfo['id'], '') .'</td>';
						} else {
							echo "<td></td>";
						}
					}
					break;
				}
			}
			
			echo "</tr>";
		}
		$alt_cls = $alt_cls == "" ? 'class="alt-row"' : "";
		
	}
	

	$date_format = user_config_option('date_format');

	if (array_var($post, 'due_date_type') == 6) {
		if ($start_time instanceof DateTimeValue) $start_time->advance(-1 * logged_user()->getUserTimezoneValue(), true);
		if ($end_time instanceof DateTimeValue) $end_time->advance(-1 * logged_user()->getUserTimezoneValue(), true);
	}
	
	if (isset($start_time) && $start_time instanceof DateTimeValue) { ?>
		<span class="bold"><?php echo lang('from')?></span>:&nbsp;<?php echo $start_time->format($date_format) ?>
	<?php }
	if (isset($end_time) && $end_time instanceof DateTimeValue) { ?>
		<span class="bold" style="padding-left:10px"><?php echo lang('to date')?></span>:&nbsp;<?php echo $end_time->format($date_format) ?>
	<?php } ?>
	
	<?php if (count(active_context_members(false)) > 0) : ?>
	<div class="clear"></div>
	<div style="margin-bottom: 10px; padding-bottom: 5px; float:left;">
		<h5><?php echo lang('showing information for')?>:</h5>
		<ul>
		<?php
			$context = active_context();
			foreach ($context as $selection) :
				if ($selection instanceof Member) : ?>
					<li><span class="coViewAction <?php echo $selection->getIconClass()?>"><?php echo $selection->getName()?></span></li>
		<?php 	endif;
			endforeach;
		?>
		</ul>
	</div>
	<div class="clear"></div>
	<?php endif; ?>
	
	<div class="tasks-report-container expenses-report-container">
	<table style="width:100%; margin-top:5px; border-top: 1px dotted #ccc;" class="reporting-table">
	<?php
	$crumbJs = "";
	
	$options = array_var($_SESSION, 'payments_by_state_data', array());
	
	$more_info = array(
		'status_info' => $status_info,
		'payments' => $payments,
		'totals' => $totals,
	);
	
	$all_totals = array('budgeted' => 0, 'executed' => 0);
	
	// headers
	echo '<tr class="reporting-table-heading">';
	$status_codes = array();
	foreach ($more_info['status_info'] as $st) {
		if ($st['only_for_expenses']) continue;
		$status_codes[] = $st['code'];
	}
	
	foreach ($options['ordered_columns'] as $colinfo) {
		if (in_array($colinfo['id'], $status_codes)) {
			echo '<th class="right">' . $colinfo['name'] . '</th>';
		} else {
			echo '<th>' . $colinfo['name'] . '</th>';
		}
	}
	echo '</tr>';
	
	$groups = order_groups_by_name($grouped_expenses['groups']);
	foreach ($groups as $gid => $group_obj) {
		$tmp_tot = array();
		foreach ($status_info as $st) {
			if ($st['only_for_expenses']) continue;
			$tmp_tot[$st['code']] = 0;
		}
		
		payments_report_print_group($group_obj, $grouped_expenses['grouped_objects'], $options, array(), 0, $more_info, "", $crumbJs, $tmp_tot);
		foreach ($status_info as $st) {
			if ($st['only_for_expenses']) continue;
			if (!isset($all_totals[$st['code']])) $all_totals[$st['code']] = 0;
			$all_totals[$st['code']] += array_var($tmp_tot, $st['code']);
		}
	}
	
	if (count($groups) > 1) {
		// TODO: Fix this using the expense currency, for now only one currency will be used
		$c = Currencies::instance()->findOne();
		$c_sym = $c instanceof Currency ? $c->getSymbol() : '$';
		
		
		
		//echo '<table style="width:100%; margin-top:5px; border-top: 1px dotted #ccc;"><tr>';
		echo '<tr>';
		
		$first = true;
		foreach ($options['ordered_columns'] as $colinfo) {
			if ($first) {
				echo '<td style="padding-left:0px; border-top:1px solid #999; font-size:15px;" class="bold">'.lang('total').'</td>';
		
			} else {
				if (in_array($colinfo['id'], $status_codes)) {
					echo '<td class="right bold" style="min-width:100px;max-width:100px;padding:2px 4px; border-top:1px solid #999; font-size:15px;">'. format_money_amount(array_var($all_totals, $colinfo['id']), $c_sym) .'</td>';
				} else {
					echo '<td id="'.$colinfo['id'].'" style="border-top:1px solid #999;"></td>';
				}
			}
			$first = false;
		}
		
		//echo '</tr></table>';
		echo '</tr>';
	}
	?>
	</table>
	</div>
	<script>
	$(function() {
		eval('<?php echo $crumbJs; ?>');
		og.eventManager.fireEvent('replace all empty breadcrumb');
	});
	</script>
<?php //} ?>