<?php
Env::useHelper('chart');
$genid = gen_id();

$all_chart_data = array();
$all_chart_labels = array();
$all_chart_options = array();
$all_shapes = array();

$expenses = Expenses::instance()->listing(array('raw_data' => true, 'select_columns' => array('id')));
$expense_ids = array_flat($expenses->objects);

$currencies = Currencies::getAllCurrencies();

$unit_selected_by_user = ListConfigHandler::getConfigOptionText('expenses_graph_unit',user_config_option("expenses_graph_unit"));
    
$saved_data = json_decode(user_config_option("expenses_graph_date_range"));

$extra_params= null;

if ( trim($saved_data->type) != '' ){
    
    $extra_params = get_dates_for_date_range_config($saved_data);
    
}


function transform_payments_to_unit_selected(&$payments,$unit){
    
    foreach ($payments as &$p) {
        
        switch ($unit){
            
            case "Millions":                
                $p['amount'] = $p['amount'] / 1000000;
                break;
                
            case "Billions":                
                $p['amount'] = $p['amount'] / 1000000000;
                break;
                
            default:
                break;
        }
    }
    
}

foreach ($currencies as $currency) {
	
	$totals = array();
	$processed_payments = array();

	// process payments
	$payments = PaymentHistories::instance()->getAllPaymentHistory($expense_ids, $currency->getId(), $extra_params);
        
        if ($unit_selected_by_user != 'Units'){
	    
	    transform_payments_to_unit_selected($payments,$unit_selected_by_user);	    
	}
	
	foreach ($payments as $p) {
		$status_id = $p['status_id'];
		if (!isset($totals[$status_id])) $totals[$status_id] = array();
		
		if ($p['date'] != '0000-00-00 00:00:00') {
			$p_date_ts = strtotime($p['date']);
			$k = date('Y-m-d', $p_date_ts);
		} else {
			// payments without date are put under index "min"
			$k = "min";
		}
		
		if (!isset($totals[$status_id][$k])) $totals[$status_id][$k] = 0;
		
		$totals[$status_id][$k] += $p['amount'];
	}
	
	// accumulate the payments under "min" index in the first date 
	foreach ($totals as $status_id => &$total_array) {
		
		if (isset($total_array['min'])) {
			$arr_keys = array_keys($total_array);
			$i = 0;
			while ($arr_keys[$i] == 'min') $i++;
			$first_key = $arr_keys[$i];
			
			$t = $total_array['min'];
			unset($total_array['min']);
			$total_array[$first_key] += $t;
		}
	}
	
	// labels
	$all_keys = array();
	$all_totals = array();
	foreach($totals as &$total) {
		ksort($total);
		$all_keys = array_merge($all_keys, array_keys($total));
		$all_totals = array_merge($all_totals, $total);
	}
	
	$chart_labels = array_unique($all_keys);
	sort($chart_labels);
	
	// values
	$statuses = array_keys($totals);
	$accumulated = array();
	$values = array();
	
	// store the last value of each line
	$last_values = array();
	foreach ($chart_labels as $cl) {
		foreach ($statuses as $st) {
			if (isset($totals[$st]) && isset($totals[$st][$cl])) {
				$last_values[$st] = $cl;
			}
		}
	}
	
	foreach ($chart_labels as $cl) {
		foreach ($statuses as $st) {
			
			// don't fill the line after the last value
			if ($last_values[$st] >= $cl) {
			
				if (!isset($accumulated[$st])) $accumulated[$st] = 0;
				if (!isset($values[$st])) $values[$st] = array();
				
				//if (!isset($values[$st][$cl])) $values[$st][$cl] = (isset($accumulated[$st]) ? $accumulated[$st] : 0);
				if (!isset($values[$st][$cl])) $values[$st][$cl] = array_var($accumulated, $st, 0);
				
				if (isset($totals[$st]) && isset($totals[$st][$cl])) {
					$values[$st][$cl] += $totals[$st][$cl];
					$accumulated[$st] += $totals[$st][$cl];
				}
				
			}
		}
	}
	// order by status id 
	ksort($values);
	
	foreach ($chart_labels as &$l) {
		$l = format_date(DateTimeValueLib::dateFromFormatAndString('Y-m-d', $l), null, 0);
	}
	
	$colors = array('#1839BF', '#34773C', '#8A8E06', '#884F7A', '#DD4444', '#EC7000');
	
	$i = 0;
	$shapes = array();
	$chart_data = array();
	foreach ($values as $st => $vals) {
		$st_obj = ExpensesStatuses::instance()->getStatus($st);
		if (!$st_obj instanceof ExpensesStatus) continue;
		$st_name = $st_obj->getName() != '' ? $st_obj->getName() : lang($st_obj->getCode());
		
		$chart_data[] = array('color' => $colors[$i % count($colors)], 'values' => array_values($vals), 'text' => $st_name, 'width' => 3);
	
		// vertical line starting at 0 in the first point. 
		$tmp_values = array_values($vals);
		$height = $tmp_values[0];
		$shapes[] = array('color' => $colors[$i % count($colors)], 'text' => '', 'points' => array(
			array('x' => 0 - 0.005, 'y' => 0),
			array('x' => 0 - 0.005, 'y' => $height),
			array('x' => 0 + 0.005, 'y' => $height),
			array('x' => 0 + 0.005, 'y' => 0),
		));
		
		$i++;
	}
	
	if (count($chart_data) > 0) {
		$all_chart_data[] = $chart_data;
		$all_chart_labels[] = $chart_labels;
		$all_shapes[] = $shapes;
		$chart_names[] = $currency->getName() . " (".$currency->getSymbol().")";
	}

}

foreach ($all_chart_data as $idx => $chart_data) {

	$max_y = 0;
	foreach ($chart_data as $k => $v) {
		$max_tmp = max($v['values']);
		if ($max_tmp > $max_y) $max_y = $max_tmp;
	}
	$top = (round($max_y / 1000, 0) + 1) * 1000;
	// ensure that the top value is not to far from the max_y
	while (floor($top / 2) > $max_y) {
		$top = floor($top / 2);
	}

	
	$bottom = 0;
	
	$x_end = count($all_chart_labels[$idx])-1;
	if ($x_end <= 0) $x_end = 1;
	
	$all_chart_options[] = array(
		'max_y' => $max_y, 'top' => $top, 'bottom' => $bottom, 'x_end' => $x_end
	);

}


include 'template.php';
