<?php
/**
 * Expense controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class ExpenseController extends ApplicationController {

	var $plugin_name = "expenses2";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
		Env::useHelper('format');
	}
	
	function init() {
		require_javascript("ExpenseManager.js", $this->plugin_name);
		ajx_current("panel", "expenses-containerpanel", null, null, true);
		ajx_replace(true);
	}
	
	function list_all() {
		
		ajx_current("empty");
		// Get all variables from request
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		$attributes = array("ids" => explode(',', array_var($_GET,'ids')));
		$dim_order = null;
		$cp_order = null;
		
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		$select_columns = array("*");
		$extra_conditions = "";
		$join_params = null;
		$added_totals_table = false;
		
		if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		} else if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		} else {
		
			switch ($order) {
				case 'date':
					$order = '`jt`.`date`';
					$join_params = array(
						'table' => ExpenseItems::instance()->getTableName(),
						'join_type' => 'LEFT',
						'jt_field' => 'expense_id',
						'e_field' => 'object_id',
					);
					$select_columns = array("e.*, o.*, jt.`date`");
					$added_totals_table = true;
					$extra_conditions .= " AND (jt.object_id = (SELECT MIN(p2.object_id) FROM ".TABLE_PREFIX."expense_items p2 WHERE p2.expense_id=jt.expense_id)) ";
					break;
				case 'total_cost':
				case 'total_price':
					$order = '`jt`.`'.$order.'`';
					$join_params = array(
						'table' => TABLE_PREFIX."expense_totals",
						'join_type' => 'LEFT',
						'jt_field' => 'expense_id',
						'e_field' => 'object_id',
					);
					$extra_conditions .= " AND (jt.status_id = e.status_id) ";
					$added_totals_table = true;
					break;
				case 'status_id':
					$order = '`jt`.`order`';
					$join_params = array(
						'table' => ExpensesStatuses::instance()->getTableName(),
						'join_type' => 'LEFT',
						'jt_field' => 'id',
						'e_field' => 'status_id',
					);
					$select_columns = array("e.*, o.*, jt.`order`");
					break;
				default:
					$order = "name";
					break;
			}
		}

		if (is_null($join_params)) {
			$join_params = array(
				'table' => TABLE_PREFIX."expense_totals",
				'join_type' => 'LEFT',
				'jt_field' => 'expense_id',
				'e_field' => 'object_id',
			);
			$select_columns[] = 'jt.total_cost';
			$select_columns[] = 'jt.total_price';
		} else {
			if(!$added_totals_table){
				if (!isset($join_params['on_extra'])) $join_params['on_extra'] = '';
				$join_params['on_extra'] .= "
					LEFT JOIN `".TABLE_PREFIX."expense_totals` as `expt` ON `expt`.`expense_id` = `e`.`object_id`
				";
				$select_columns[] = 'expt.total_cost';
				$select_columns[] = 'expt.total_price';
			}
		}
		
		Hook::fire("listing_extra_conditions", null, $extra_conditions);
		
		$filters = array();
		if (array_var($_GET,'status_filter')) $filters['status_id'] = array_var($_GET,'status_filter');

		foreach ($filters as $col => $value) {
			if (!$value || $value == '0') continue;
			$extra_conditions .= " AND e.`$col` = " . DB::escape($value);
		}
		
		if (array_var($_GET, 'from_filter')) {
			$from_date = DateTimeValueLib::dateFromFormatAndString(user_config_option('date_format'), array_var($_GET, 'from_filter'));
			if ($from_date instanceof DateTimeValue) {
				$extra_conditions .= " AND EXISTS (SELECT p.expense_id FROM ".TABLE_PREFIX."expense_items p WHERE p.expense_id=o.id AND p.date >= '".$from_date->toMySQL()."')";
			}
		}
		if (array_var($_GET, 'to_filter')) {
			$to_date = DateTimeValueLib::dateFromFormatAndString(user_config_option('date_format'), array_var($_GET, 'to_filter'));
			if ($to_date instanceof DateTimeValue) {
				$extra_conditions .= " AND EXISTS (SELECT p.expense_id FROM ".TABLE_PREFIX."expense_items p WHERE p.expense_id=o.id AND p.date <= '".$to_date->toMySQL()."')";
			}
		}
		
		$grid_message = "";
		$grid_message_cls = "";
		
		// conditions to filter the totals and expense_items by the active context
		$active_member_ids = active_context_members(false);
		if (count($active_member_ids) > 0) {
			
			$context_member_names_str = "";
			$active_context_members = Members::instance()->findAll(array("conditions" => "id IN (".implode(',', $active_member_ids).")"));
			$i=1;
			foreach ($active_context_members as $m) {
				$sep = ($i == count($active_context_members)) ? " ".lang('and')." " : ", ";
				$context_member_names_str .= ($context_member_names_str == "" ? "" : $sep) . $m->getName();
				$i++;
			}
			$grid_message = lang('expense_items and expenses totals are being filtered by their classification in', $context_member_names_str);
			$grid_message_cls = "warning";
		}

		$task_id = array_var($_REQUEST, 'task_id');
		if (is_numeric($task_id) && $task_id > 0) {
			$extra_conditions .= " AND task_id = '$task_id'";
		}
		
		$only_count_result = array_var($_GET, 'only_result',false);
		$result = Expenses::instance()->listing(array(
			"order" => $order,
			"order_dir" => $order_dir,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"select_columns" => $select_columns,
			"extra_conditions" => $extra_conditions,
			"count_results" => false,
			"only_count_results" => $only_count_result,
			"join_params" => $join_params,
			"start" => $start,
			"limit" => $limit,
		));
		$expenses = $result->objects;
		$object = $this->prepareObject($expenses, $start, $limit, $result);
		
		/*
		$object['filters'] = array(
			'status_id' => ExpensesStatuses::instance()->getAllStatusInfo(),
		);*/
		

		if ($grid_message) {
			$object['message'] = $grid_message;
			$object['message_cls'] = $grid_message_cls;
		}
		
		ajx_extra_data($object);
		tpl_assign("listing", $object);
		
	}
	
	private function prepareObject($expenses, $start, $limit, $res_obj) {
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"expenses" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
		
		$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType(Expenses::instance()->getObjectTypeId());
		$count = min(array($limit, count($expenses)));
		for ($i = 0; $i < $count; $i++){
			if (isset($expenses[$i])){
				$expense = $expenses[$i];
				if ($expense instanceof Expense){
					$info = $expense->getArrayInfo(true, true);
					$info['memPath'] = json_encode($expense->getMembersIdsToDisplayPath(true));
					$object["expenses"][$i] = $info;
					
					foreach ($custom_properties as $cp) {
						$object["expenses"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $expense);
					}
    			}
			}
		}
		return $object;
	}

	function get_tasks_in_context() {
		ajx_current("empty");
		ini_set("memory_limit","1G");

		$tasks_data = array();

		$listing_params = array(
			"order" => "name",
			"order_dir" => "ASC",
			"count_results" => false,
		);
		// if no filter applied then limit the size of the query to prevent memory/performance issues
		if (count(active_context_members(false)) == 0) {
			$listing_params["limit"] = 250;
		}

		$result = ProjectTasks::instance()->listing($listing_params);
		$tasks = $result->objects;

		foreach ($tasks as $task) {
			$task_data = $task->getArrayInfo();
			$tasks_data[] = $task_data;
		}

		ajx_extra_data(array(
			'tasks' => $tasks_data
		));
	}
	

	function get_expenses_in_context() {
		ajx_current("empty");
		ini_set("memory_limit","1G");

		$expenses_data = array();

		$listing_params = array(
			"order" => "name",
			"order_dir" => "ASC",
			"count_results" => false,
		);
		// if no filter applied then limit the size of the query to prevent memory/performance issues
		if (count(active_context_members(false)) == 0) {
			$listing_params["limit"] = 250;
		}

		$result = Expenses::instance()->listing($listing_params);
		$expenses = $result->objects;

		foreach ($expenses as $expense) {

			$exp_data = $expense->getArrayInfo();
			$exp_data['memPath'] = json_encode($expense->getMembersIdsToDisplayPath(true));
			$exp_data['items'] = get_expense_items_for_expense($expense->getId(), null, true);

			$expenses_data[] = $exp_data;
		}


		ajx_extra_data(array(
			'expenses' => $expenses_data
		));
	}
	
	
	function add() {
		if (!can_add(logged_user(), active_context(), Expenses::instance()->getObjectTypeId())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		/*
		$notAllowedMember = '';
		if(!Expense::canAdd(logged_user(), active_context(), $notAllowedMember )) {
			if (str_starts_with($notAllowedMember, '-- req dim --')) flash_error(lang('must choose at least one member of', str_replace_first('-- req dim --', '', $notAllowedMember, $in)));
			else flash_error(lang('no context permissions to add',lang("expenses"),$notAllowedMember ));
			ajx_current("empty");
			return;
		}
		*/
		$expense_data = array_var($_POST, 'expense');
		
		if(!is_array($expense_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$expense = new Expense();
			$expense->setIsBillable(true);

			$pre_selected_members = null;

			$task_id = array_var($_REQUEST, 'task_id');
			if (is_numeric($task_id) && $task_id > 0) {
				$expense->setTaskId($task_id);
				$task = ProjectTasks::instance()->findById($task_id);
				if ($task instanceof ProjectTask) {
					$pre_selected_members = $task->getMemberIds();
				}
			}

			tpl_assign('pre_selected_members', $pre_selected_members);
			tpl_assign('expense', $expense);
			tpl_assign('max_pnum', 0);
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$expense = $this->save_expense($expense_data, $member_ids);
				
				DB::commit();
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
				return;
			}
			
			ApplicationLogs::createLog($expense, ApplicationLogs::ACTION_ADD);
			flash_success(lang('success add expense', $expense->getObjectName()));
			ajx_current("back");
			
			if (array_var($_REQUEST, 'modal')) {
				evt_add("reload current panel");
			}
		}
	}
	
	function edit() {
		$this->setTemplate('add');
		
		$expense = Expenses::instance()->findById(get_id());
		if(!($expense instanceof Expense)) {
			flash_error(lang('expense dnx'));
			ajx_current("empty");
			return;
		}
		
		if (logged_user()->isGuest() || !$expense->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		
		$expense_data = array_var($_POST, 'expense');
		
		if(!is_array($expense_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			tpl_assign('expense', $expense);
			$rows = DB::executeAll("SELECT max(object_id) as num FROM ".TABLE_PREFIX."expense_items_history WHERE expense_id=".$expense->getId());
			$max_pnum = 0;
			if (count($rows) > 0) {
				$max_pnum = $rows['0']['num'];
			}
			tpl_assign('max_pnum', $max_pnum);
			
			$expense_items = ExpenseItems::instance()->findAll(array('conditions' => array('trashed_by_id=0 AND expense_id = ?', $expense->getId())));
			tpl_assign('expense_items', $expense_items);
			
		} else {
			// form submitted
			try {
				// to use when saving the application log
				$old_content_object = $expense->generateOldContentObjectData();

				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$expense = $this->save_expense($expense_data, $member_ids, $expense);
				
				DB::commit();
	          	
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
				return;
			}
				
			ApplicationLogs::createLog($expense, ApplicationLogs::ACTION_EDIT);
			flash_success(lang('success edit expense', $expense->getObjectName()));
			ajx_current("back");
			
			if (array_var($_REQUEST, 'modal')) {
				evt_add("reload current panel");
			}
		}
	}
	
	function save_expense($expense_data, $member_ids, $expense = null) {
		// Save expense object
	    
		$is_new = false;
		if (is_null($expense) || !$expense instanceof Expense) {
			$expense = new Expense();
			$is_new = true;
		}

		if(!$is_new) {
			Hook::fire("compare_previous_and_current_object", array('current_object_data' => $expense_data, 'previous_object' => $expense), $previous_expense);
		}

		$previous_task = $expense->getTask();
		
		$object_controller = new ObjectController();
		
		$expense->setFromAttributes($expense_data);
		
		$expense_items = array_var($expense_data, 'expense_items');
		
		
		if (config_option("use_date_in_expense_level")) {
			$ex_date = getDateValue(array_var($expense_data, 'date'));
			if ($ex_date instanceof DateTimeValue) {
				$expense->setDate($ex_date);
			}
		}
		/*
		if ($is_new) {
			$all_expense_item_status_ids = array_flat(DB::executeAll("SELECT id FROM ".TABLE_PREFIX."expenses_status WHERE only_for_expenses=0 AND enabled=1 ORDER BY `order`"));
			$first_status_id = array_var($all_expense_item_status_ids, 0);
			$expense->setStatusId($first_status_id);
		}*/
		
		$expense->save(false);
		
		$deleted_item_ids_json = array_var($expense_data, 'deleted_item_ids');
		$deleted_item_ids = json_decode($deleted_item_ids_json, true);
		
		if ($deleted_item_ids != null) {
		    foreach ($deleted_item_ids as $deleted_item_id) {
		        $del_exp_item = ExpenseItems::instance()->findById($deleted_item_id);
		        if ($del_exp_item instanceof ExpenseItem) {
		            ApplicationLogs::createLog($del_exp_item, ApplicationLogs::ACTION_DELETE, true, true);
		            $del_exp_item->delete();
		        }
		    }
		}
		
		$expense_expense_items = array();
		//$status_count = array();
		
		// Save expense_items
		if (is_array($expense_items)) {
		  foreach ($expense_items as $p_data) {
		  	
		  	$submitted_pdate = array_var($p_data, 'date');
		  	if (config_option("use_date_in_expense_level")) {
		  		$submitted_pdate = array_var($expense_data, 'date');
		  		if ($submitted_pdate == date_format_tip(user_config_option('date_format'))) $submitted_pdate = '';
		  	}
		  	
		  	// ignore expense_items with no data
		  	// if ($submitted_pdate == '' && array_var($p_data, 'name') == '') continue;
		  	// sanitize expense_item amount, if negative amounts are allowed then add the "-" sign to the regular expression that sanitizes the amounts
		  	$reg_exp = config_option('allow_negative_amounts') ? '/[^\d.,-]/' : '/[^\d.,]/';

		  	$p_data['unit_cost'] = preg_replace($reg_exp, '', array_var($p_data, 'unit_cost'));
		  	$p_data['total_cost'] = preg_replace($reg_exp, '', array_var($p_data, 'total_cost'));
		  	$p_data['unit_price'] = preg_replace($reg_exp, '', array_var($p_data, 'unit_price'));
		  	$p_data['total_price'] = preg_replace($reg_exp, '', array_var($p_data, 'total_price'));
		  	$p_data['price_multiplier'] = preg_replace($reg_exp, '', array_var($p_data, 'price_multiplier'));
		  	
			$id = array_var($p_data, 'id');
			$expense_item = ExpenseItems::instance()->findById($id);
			
			$is_new_item = false;
			if (!$expense_item instanceof ExpenseItem) {
				$expense_item = new ExpenseItem();
				$expense_item->setExpenseId($expense->getId());
				$is_new_item = true;
			}
			
			$pt = ProductTypes::getById(array_var($p_data, 'product_type_id'));
			if ($pt instanceof ProductType) {
				$p_data['name'] = $pt->getName();
			}
			
			// remove thousands separator
			/*
			$p_data['unit_cost'] = (double)filter_var(array_var($p_data, 'unit_cost', ''), FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION);
			$p_data['total_cost'] = (double)filter_var(array_var($p_data, 'total_cost', ''), FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION);
			$p_data['unit_price'] = (double)filter_var(array_var($p_data, 'unit_price', ''), FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION);
			$p_data['total_price'] = (double)filter_var(array_var($p_data, 'total_price', ''), FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION);
			$p_data['price_multiplier'] = (double)filter_var(array_var($p_data, 'price_multiplier', ''), FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION);
			*/
			
			// remove thousands separator and fix decimals separator before sendint the amounts to the database
			$p_data['unit_cost'] = clean_formatted_money_amount_for_sql(array_var($p_data, 'unit_cost', ''));
			$p_data['total_cost'] = clean_formatted_money_amount_for_sql(array_var($p_data, 'total_cost', ''));
			$p_data['unit_price'] = clean_formatted_money_amount_for_sql(array_var($p_data, 'unit_price', ''));
			$p_data['total_price'] = clean_formatted_money_amount_for_sql(array_var($p_data, 'total_price', ''));
			$p_data['price_multiplier'] = clean_formatted_money_amount_for_sql(array_var($p_data, 'price_multiplier', ''));
			
			// ensure that total cost and total price are correct, don't save the values sent by the interface without verification
			if ($p_data['total_cost'] != $p_data['quantity'] * $p_data['unit_cost']) {
				$p_data['total_cost'] = $p_data['quantity'] * $p_data['unit_cost'];
			}
			if ($p_data['total_price'] != $p_data['quantity'] * $p_data['unit_price']) {
				$p_data['total_price'] = $p_data['quantity'] * $p_data['unit_price'];
			}

			// Set currency id using expense's currency_id
			$p_data['currency_id'] = $expense->getCurrencyId();
			
			if (config_option("use_date_in_expense_level")) {
				$p_data['date'] = $ex_date;
			} else {
				$p_data['date'] = getDateValue(array_var($p_data, 'date'));
			}
			
			$expense_item->setFromAttributes($p_data);
			$expense_item->save();
			
			if (isset($p_data['member_ids']) && is_array($p_data['member_ids'])) {
				$mem_ids = array_filter($p_data['member_ids']);
				if (count($mem_ids) > 0) {
					$members = Members::instance()->findAll(array("conditions" => "id IN (".implode(',', $mem_ids).")"));
					if (count($members) > 0) {
						$expense_item->addToMembers($members);
					}
				}
			}
			
			$cp_data = null;
			if (isset($p_data['object_custom_properties'])) {
				$cp_data = $p_data['object_custom_properties'];
				$object_controller->add_custom_properties($expense_item, $cp_data);
			}
			
			$expense_item->addToHistory(logged_user(), ($is_new_item ? 'add' : 'edit'));
			
			// if edit, save only if there are changes, don't recalculate the expense totals in each expense_item (do it after saving the expense)
			$expense_expense_items[] = $expense_item;
			/*
			if (!isset($status_count[$expense_item->getStatusId()])) $status_count[$expense_item->getStatusId()] = 0;
			$status_count[$expense_item->getStatusId()] = $status_count[$expense_item->getStatusId()]+1;*/
		  }
		}
		
		
		
		// classify expense
		if (!is_null($member_ids)) {
			$object_controller->add_to_members($expense, $member_ids);
			
			// classify new expense_items in the same places of the expense
			if (config_option('copy_exp_classiffication_to_expense_items')) {
				$exp_members = $expense->getMembers();
				
				$exp_expense_items = ExpenseItems::instance()->findAll(array("conditions" => "expense_id=".$expense->getId()));
				if (count($exp_expense_items) > 0) {
					$dims_to_keep = array(); // this are the dimensions where exepenses are autoclassified when classfiying a expense_item, this can't be inherited by expense_items.
					
					$ot_hierarchy = ObjectTypeHierarchies::instance()->findOne(array(
						"conditions" => "parent_object_type_id=".$expense->getObjectTypeId()." AND child_object_type_id=".ExpenseItems::instance()->getObjectTypeId()
					));
					if ($ot_hierarchy instanceof ObjectTypeHierarchy) {
						$oth_options = ObjectTypeHierarchyOptions::instance()->findAll(array("conditions" => "hierarchy_id='".$ot_hierarchy->getId()."' AND `option`='autoclassify_in_parent_members'"));
						foreach ($oth_options as $opt) {
							$dims_to_keep[] = $opt->getDimensionId();
							
						}
					}
					
					// collect the members to be copied to the expense_items
					$members_to_inherit = array();
					//Debug:
					//echo "ExpenseController::save_expense->dims_to_keep: ".implode(", ",$dims_to_keep);
					//Logger::log_r("ExpenseController::save_expense->dims_to_keep: ".$dims_to_keep);
					
					foreach ($exp_members as $mem) {
						if (!in_array($mem->getDimensionId(), $dims_to_keep)) {
							$members_to_inherit[] = $mem;
						}
					}
					//Debug:
					//Logger::log_r("ExpenseController::save_expense->dims_to_keep: ".$members_to_inherit);					
					
					foreach ($exp_expense_items as $expense_item) {
						$members_to_keep = array();
						$p_members = $expense_item->getMembers();
						// collect the expense_item members to be kept in the expense_item
						foreach ($p_members as $mem) {
							if (in_array($mem->getDimensionId(), $dims_to_keep)) {
								$members_to_keep[] = $mem;
							}
						}
						
						$new_expense_item_members = array_merge($members_to_inherit, $members_to_keep);
						
						$expense_item->removeFromMembers(logged_user(), null);
						$expense_item->addToMembers($new_expense_item_members);
						$expense_item->addToSharingTable();
					}
				}
			}
		}
		
		$object_controller->add_subscribers($expense);
		
		$object_controller->link_to_new_object($expense);
		$object_controller->add_custom_properties($expense);
		
		// guardar en el historico de expenses los cambios realizados
		$expense->addToHistory(logged_user(), ($is_new ? 'add' : 'edit'));
		
		// if we removed or changed the expense's task then recalculate previous task financials
		if (Plugins::instance()->isActivePlugin('advanced_billing')) {
			if ($previous_task instanceof ProjectTask && $previous_task->getId() != $expense->getTaskId()) {
				Env::useHelper('functions', 'advanced_billing');
				calculate_task_estimated_and_executed_financials($previous_task);
			}
		}
		
		return $expense;
	}
	
	function view() {
		ajx_set_no_toolbar();
		$expense = Expenses::instance()->findById(get_id());
		if(!($expense instanceof Expense)) {
			flash_error(lang('message dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$expense->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		ajx_extra_data(array("title" => $expense->getObjectName(), 'icon' => $expense->getIconClass()."s"));
		tpl_assign('expense', $expense);
	}
	
	function get_expense_items_table_html() {
		ajx_current("empty");
		
		$expense = Expenses::instance()->findById(get_id());
		if(!($expense instanceof Expense)) {
			flash_error(lang('message dnx'));
			ajx_current("empty");
			return;
		}
		if (!$expense->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		tpl_assign('expense', $expense);
		tpl_assign('order', array_var($_REQUEST, 'order', 'date'));
		tpl_assign('order_dir', array_var($_REQUEST, 'order_dir', 'ASC'));
		
		$html = tpl_fetch(get_template_path('view_expense_items_table', 'expense', 'expenses'));
		
		ajx_extra_data(array('html' => $html));
	}
	
	
	


	function get_totals_for_listing() {
		ajx_current("empty");
	
		$extra_conditions = "";
		
		if (array_var($_REQUEST, 'expense_id')) {
			$extra_conditions .= " AND p.expense_id = '".array_var($_REQUEST, 'expense_id')."'";
		}
	
		$totals_info = array('id' => "__total_row__", 'name' => lang('total'));
		
		$expense = Expenses::instance()->findById(array_var($_REQUEST, 'expense_id'));
		if ($expense instanceof Expense) {
			$info = $expense->getArrayInfo();
			foreach ($info as $col => $val) {
				if (str_starts_with($col, "total_")) {
					$totals_info[$col] = format_money_amount($info[$col], $info["currency_sym"], config_option('product_type_decimals'));
				}
			}
		}

		ajx_extra_data(array('totals' => $totals_info));
	}
	
	
	/**
	 * Resolve action to perform
	 *
	 * @param string $action
	 * @param array $attributes
	 * @return string $message
	 */
	private function resolveAction($action, $attributes){
				 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				$messages = array();
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					if (!is_numeric($id)) continue;
					$expense = Expenses::instance()->findById($id);
					if ($expense instanceof Expense && $expense->canDelete(logged_user())){
						try{
							DB::beginWork();
							$expense->trash();
							DB::commit();
							ApplicationLogs::createLog($expense, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$messages[] = $e->getMessage();
							$err++;
						}
					} else {
						$err++;
						$messages[] = lang('cannot delete executed expense', $expense instanceof Expense ? clean($expense->getName()) : '');
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) .("\n - ".implode("\n - ",$messages)). ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			case "archive":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					//This control is for lines in the UI that do not correspond to an expense (eg: "totals" line). 
					if (!is_numeric($id)) continue;
					$expense = Expenses::instance()->findById($id);
					if ($expense instanceof Expense && $expense->canEdit(logged_user())){
						try{
							DB::beginWork();
							$expense->archive();
							DB::commit();
							ApplicationLogs::createLog($expense, ApplicationLogs::ACTION_ARCHIVE);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error archive objects", $err) . "<br />" . ($succ > 0 ? lang("success archive objects", $succ) : "");
				} else {
					$resultMessage = lang("success archive objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;		
		} // switch
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
	}
	
	
	function get_expense_items() {
		ajx_current("empty");
		$id = get_id();
		
		$expense_items = get_expense_items_for_expense($id, null, true);
		
		ajx_extra_data(array('expense_items' => $expense_items));
		
	}
	
	/**
	 * Change status from budgeted to executed and saves history
	 */
	function change_expense_item_status() {
		ajx_current("reload");
		
		if (isset($_REQUEST['expense_item_id'])) {
			
			$p = ExpenseItems::instance()->findById($_REQUEST['expense_item_id']);
			$expense = null;
			if ($p instanceof ExpenseItem) {
				$expense = Expenses::instance()->findById($p->getExpenseId());
			}
			
		} else {
			$expense_id = get_id();
			$object_id = get_id('num');
			
			$p = null;
			$expense = Expenses::instance()->findById($expense_id);
			if ($expense instanceof Expense) {
				$p = ExpenseItems::instance()->findOne(array('conditions' => "expense_id = $expense_id AND object_id = $object_id"));
			}
		}
		
		$new_status = get_id('st');
		
		// use can write instead of canEdit to allow common users to change a bufgeted expense_item when the expense status is final
		if (logged_user()->isGuest() || $expense instanceof Expense && !can_write(logged_user(), $expense->getMembers(), $expense->getObjectTypeId())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		if (!$p instanceof ExpenseItem) {
			ajx_current("empty");
			flash_error("expense_item dnx");
			return;
		}
		
		// dont allow common users to edit finalized expense_items, check for special permission
		if ($p->getStatusId() == ExpensesStatuses::getFinalStatus()->getId()) {
			/*if (!can_edit_executed_expense_items_pg_array(logged_user()->getPermissionGroupIds())) {
				flash_error(lang('no access permissions'));
				ajx_current('empty');
				return;
			}*/
		}
		
		$status_obj = ExpensesStatuses::instance()->findById($new_status);
		if (!$status_obj instanceof ExpensesStatus) {
			ajx_current("empty");
			flash_error("status dnx");
			return;
		}
		if ($status_obj->getOnlyForExpenses()) {
			ajx_current("empty");
			flash_error("invalid status for expense_items");
			return;
		}
		
		$old_data = $p->getArrayInfo();
		
		try {
			DB::beginWork();
			// change status
			$p->setStatusId($new_status);
			$p->save();
			// add to history
			$p->addToHistory(logged_user(), 'edit');
			Hook::fire('after_save_expense_item', array('object' => $p, 'data' => $p->getArrayInfo(), 'old_data' => $old_data), $null);
			
			// refresh expense status
			if ($expense instanceof Expense) $expense->refreshStatus();
			
			DB::commit();
		} catch (Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			ajx_current("empty");
		}
	}
	
	
	
	
	function get_expense_status() {
		ajx_current('empty');
		
		$all_status = ExpensesStatuses::instance()->getAllStatusInfo();
		$status_by_code = array();
		$first_final_status = null;
		foreach ($all_status as $st) {
			if ($st['is_final']) {
				$first_final_status = $st;
			}
			$status_by_code[$st['code']] = $st;
		}
		
		$next_status_array = array();
		foreach ($all_status as $status) {
		
			$next_status = null;
			$x = 0;
			while ($x < count($all_status) && $all_status[$x]['id'] != $status['id']) {
				$x++;
			}
			$x = $x + 1;
			while ($x < count($all_status) && isset($all_status[$x]) && $all_status[$x]['only_for_expenses']) $x = $x + 1;
			if ($x < count($all_status) && isset($all_status[$x])) {
				$next_status = $all_status[$x];
			}
			if ($next_status) {
				$next_status_array[$status['code']] = $next_status;
			}
		}
		
		ajx_extra_data(array('status' => $all_status, 'next_status_config' => $next_status_array));
		
	}
	
	
	
	function get_expense_data() {
		ajx_current("empty");
		$expense = Expenses::instance()->findById(get_id());
		if(!($expense instanceof Expense)) {
			flash_error(lang('message dnx'));
			ajx_current("empty");
			return;
		}
		
		$data = $expense->getArrayInfo();
		$data['memPath'] = $expense->getMembersIdsToDisplayPath();
		
		ajx_extra_data(array('expense' => $data));
	}
	
	
	function check_expense_members() {
		ajx_current("empty");
		$expense = Expenses::instance()->findById(get_id());
		if(!($expense instanceof Expense)) {
			ajx_current("empty");
			return;
		}
		
		$member_ids = array();
		$members_str = array_var($_REQUEST, 'member_ids');
		if ($members_str) {
			$member_ids = json_decode($members_str, true);
		}
		
		$expense_member_ids = $expense->getMemberIds();
		
		$ok = true;
		foreach ($member_ids as $member_id) {
			if (!in_array($member_id, $expense_member_ids)) {
				$ok = false;
				break;
			}
		}
		
		ajx_extra_data(array('ok' => $ok));
		
	}
	
	
	function administration() {
		$can_see_billing_info = true;
		Hook::fire('get_can_see_billing_information', array('user'=>logged_user()), $can_see_billing_info);
		$can_see_cost_info = true;
    	Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $can_see_cost_info);
		if (!can_manage_configuration(logged_user()) || !$can_see_billing_info || !$can_see_cost_info) {
			ajx_current("empty");
			flash_error(lang("no access permissions"));
			return;
		}
		
		$expense_administration_links = array();
		$expense_administration_links[] = array(
			'url' => get_url('expense', 'general_config_options'),
			'name' => lang("expenses general configuration"),
			'desc' => lang('expenses general configuration description'),
		);
		$expense_administration_links[] = array(
				'url' => get_url('actual_expense_tax', 'init'),
				'name' => lang("actual expenses taxes"),
				'desc' => lang('actual expenses taxes description'),
		);
		$expense_administration_links[] = array(
				'url' => get_url('product_type', 'init'),
				'name' => lang("expenses product types"),
				'desc' => lang('expenses product types description'),
		);
		
		// render checked dimensions
		$co = ConfigOptions::getByName('product_type_dimensions');
		$dimensions_form = '<div class="coInputMainBlock">';
		$dimensions_form .= $co->getConfigHandler()->do_render('product_type_dimensions', array('onchange_fn' => 'og.product_types.product_type_dimension_changed'));
		$dimensions_form .= '<div class="clear"></div></div>';
		$expense_administration_links[] = array(
			'url' => "#", // get_url('product_type', 'edit_product_type_dimensions'),
			'name' => lang("expenses product type dimensions"),
			'desc' => $dimensions_form,
		);
		
		// render dimensions that fiter product types upwards
		$co = ConfigOptions::getByName('product_type_filter_up_dimensions');
		$filter_up_dimensions_form = '<div class="coInputMainBlock" style="background-color:#eeeef3;">';
		$filter_up_dimensions_form .= $co->getConfigHandler()->do_render('product_type_filter_up_dimensions', array('onchange_fn' => 'og.product_types.product_type_filter_up_dimension_changed'));
		$filter_up_dimensions_form .= '<div class="clear"></div></div>';
		$expense_administration_links[] = array(
			'url' => "#", // get_url('product_type', 'edit_product_type_dimensions'),
			'name' => lang("expenses product type filter up dimensions"),
			'desc' => $filter_up_dimensions_form,
		);
		
		tpl_assign('expense_administration_links', $expense_administration_links);
	}
	
	function general_config_options() {
		if (!can_manage_configuration(logged_user())) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$options = ConfigOptions::instance()->findAll(array('conditions' => "category_name='expenses'"));
		tpl_assign('options', $options);
	}
	
	function save_general_config_options() {
		if (!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		try {
			DB::beginWork();
			
			$submited_values = array_var($_POST, 'options');
			if (is_array($submited_values)) {
				$options = ConfigOptions::instance()->findAll(array('conditions' => "category_name='expenses'"));
				foreach ($options as $option) {
					$new_value = array_var($submited_values, $option->getName());
					
					if (is_null($new_value) || ($new_value == $option->getValue())) {
						continue;
					}
					
					$option->setValue($new_value);
					$option->save();
				}
				flash_success(lang('success update config category', lang('expenses')));
				ajx_current("back");
			}
			
			DB::commit();
			
		} catch (Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			ajx_current("empty");
			return;
		}
	}
	
	
	function get_disabled_subtabs() {
		
		ajx_current("empty");
		
		$disabled_subtabs = array();
		
		if (config_option('disable_expenses_subtab')) {
			$disabled_subtabs[] = 'subtab-expense';
		}
		if (config_option('disable_actual_expenses_subtab')) {
			$disabled_subtabs[] = 'subtab-payment';
		}
		
		ajx_extra_data(array('disabled_subtabs' => $disabled_subtabs));
		
	}
	
	
	
	function copy_expense() {

		// set the "add expense" form
		$this->setTemplate('add');
		
		// get the expense to copy
		$expense = Expenses::instance()->findById(get_id());
		if(!($expense instanceof Expense)) {
			flash_error(lang('expense dnx'));
			ajx_current("empty");
			return;
		}
		
		// check write permissions
		if (logged_user()->isGuest() || !$expense->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		if (array_var($_REQUEST, 'modal')) {
			$this->setLayout("json");
			tpl_assign('modal', true);
		}
		
		// find max expense_item num
		$rows = DB::executeAll("SELECT max(object_id) as num FROM ".TABLE_PREFIX."expense_items_history WHERE expense_id=".$expense->getId());
		$max_pnum = 0;
		if (count($rows) > 0) {
			$max_pnum = $rows['0']['num'];
		}
		// get the expense_items to copy
		$expense_items = ExpenseItems::instance()->findAll(array('conditions' => array('expense_id = ?', $expense->getId())));
		
		// build the new expense name, if already exists keep trying until finding an unused name
		$copy_name_orig = lang('copy of', $expense->getName());
		$copy_name = $copy_name_orig;
		$copy_name_suffix = 1;
		$copy_exists = Expenses::instance()->findOne(array("conditions" => "name='$copy_name_orig'"));
		while ($copy_exists instanceof Expense) {
			$copy_name_suffix++;
			$copy_name = "$copy_name_orig $copy_name_suffix";
			$copy_exists = Expenses::instance()->findOne(array("conditions" => "name='$copy_name'"));
		}
		$expense->setObjectName($copy_name);
		
		
		// assign all the variables to the form
		tpl_assign('expense', $expense);
		tpl_assign('original_expense_id', $expense->getId());
		tpl_assign('max_pnum', $max_pnum);
		tpl_assign('expense_items', $expense_items);
		tpl_assign('override_form_action', get_url('expense', 'add'));
		
	}
	
	
	
	
	
	
}