<?php
/**
 * Expense Item Controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class ExpenseItemController extends ApplicationController {

	var $plugin_name = "expenses2";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
		Env::useHelper('format');
	}
	
	
	function list_all() {
		
		// Get all variables from request
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		$attributes = array("ids" => explode(',', array_var($_GET,'ids')));
		$dim_order = null;
		$cp_order = null;
		
		$expense_id = array_var($_GET, 'expense_id');
		
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		$select_columns = array("*");
		$extra_conditions = "";
		$join_params = null;
		
		if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		} else if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		} else {
			switch ($order) {
				case 'description':
					$order = '`e`.`description`';
					break;
				case 'date':
					$order = '`e`.`date`';
					break;
				default:
					$order = "name";
					break;
			}
		}
		
		Hook::fire("listing_extra_conditions", array('manager' => ExpenseItems::instance(), 'request' => $_REQUEST), $extra_conditions);
		
		if ($expense_id) {
			$extra_conditions .= " AND e.expense_id='$expense_id' ";
		}
		
		$only_count_result = array_var($_GET, 'only_result', false);
		
		$result = ExpenseItems::instance()->listing(array(
				"order" => $order,
				"order_dir" => $order_dir,
				"dim_order" => $dim_order,
				"cp_order" => $cp_order,
				"select_columns" => $select_columns,
				"extra_conditions" => $extra_conditions,
				"count_results" => false,
				"only_count_results" => $only_count_result,
				"join_params" => $join_params,
				"start" => $start,
				"limit" => $limit,
		));
		$items = $result->objects;
		$object = $this->prepareObject($items, $start, $limit, $result);
		
		ajx_current("empty");
		ajx_extra_data($object);
		tpl_assign("listing", $object);
		
	}
	
	

	private $expense_item_cps = null;
	
	private function prepareObject($expense_items, $start, $limit, $res_obj) {
		$object = array(
				"totalCount" => $res_obj->total,
				"start" => $start,
				"expense_items" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
	
		if (!$this->expense_item_cps) {
			$this->expense_item_cps = CustomProperties::getAllCustomPropertiesByObjectType(ExpenseItems::instance()->getObjectTypeId());
		}
		$custom_properties = $this->expense_item_cps;
		$count = min(array($limit, count($expense_items)));
		for ($i = 0; $i < $count; $i++){
			if (isset($expense_items[$i])){
				$expense_item = $expense_items[$i];
				if ($expense_item instanceof ExpenseItem){
					$info = $expense_item->getArrayInfo(true);
						
					$info['memPath'] = json_encode($expense_item->getMembersIdsToDisplayPath(true));
					$object["expense_items"][$i] = $info;
						
					foreach ($custom_properties as $cp) {
						$object["expense_items"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $expense_item);
					}
				}
			}
		}
	
		return $object;
	}
	
	

	/**
	 * Resolve action to perform
	 *
	 * @param string $action
	 * @param array $attributes
	 * @return string $message
	 */
	private function resolveAction($action, $attributes){
			
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				$messages = array();
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$expense_item = ExpenseItems::instance()->findById($id);
					if (isset($expense_item) && $expense_item->canDelete(logged_user())){
						try{
							DB::beginWork();
							$expense_item->trash();
							DB::commit();
							ApplicationLogs::createLog($expense_item, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$messages[] = $e->getMessage();
							$err++;
						}
					} else {
						$err++;
						$messages[] = lang('cannot delete executed expense', $expense_item instanceof ExpenseItem ? clean($expense_item->getName()) : '');
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) .("\n - ".implode("\n - ",$messages)). ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			case "archive":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$expense_item = ExpenseItems::instance()->findById($id);
					if (isset($expense_item) && $expense_item->canEdit(logged_user())){
						try{
							DB::beginWork();
							$expense_item->archive();
							DB::commit();
							ApplicationLogs::createLog($expense_item, ApplicationLogs::ACTION_ARCHIVE);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error archive objects", $err) . "<br />" . ($succ > 0 ? lang("success archive objects", $succ) : "");
				} else {
					$resultMessage = lang("success archive objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if
				$resultCode = 2;
				break;
		} // switch
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
	}
	
	
	function save_expense_item($data) {
		
		$expense_item = ExpenseItems::instance()->findById(array_var($data, 'id'));
		if (!$expense_item instanceof ExpenseItem) {
			$expense_item = new ExpenseItem();
		}
		
		$expense_item->setFromAttributes($data);
		$expense_item->save();
		
		$expense = Expenses::instance()->findById($expense_item->getExpenseId());
		if ($expense instanceof Expense) {
			$member_ids = $expense->getMemberIds();
			
			$object_controller = new ObjectController();
			$object_controller->add_to_members($expense_item, $member_ids);
		}
		
		$expense_item->addToSharingTable();
		
		return $expense_item;
		
	}
	
	//Added this function while resolving issue reported by Amaterra: clicking on the name throws an error.
	//@ToDo: Ideally, clicking on the name of the item in the grid while viewing an expense would edit the expense, and focus on the line item
	function view() {
		$expense_item = ExpenseItems::instance()->findById(get_id());
		if ($expense_item instanceof ExpenseItem) {
			$this->redirectTo('expense', 'view', array('id' => $expense_item->getExpenseId()));
		}
	}
}