<?php
/**
 * Expenses Approvals controller
 *
 * @version 1.0
 */
class ExpensesApprovalController extends ApplicationController {

	var $plugin_name = "expenses2";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
	}
	
	
	private function change_actual_expenses_status_member($actual_expenses, $member, $action) {
		$modified_objects = array();
	
		// If action is 'process' check if all actual expenses are non-billable, throm exception otherwise
		if($action == 'process'){
			$all_non_billable = $this->check_actual_expenses_non_billiable($actual_expenses);
			if(!$all_non_billable){
				throw new Exception(lang('you can process only non-billable actual expenses'));
				return;
			}
		}
		$processed_member_id = config_option("time_bulk_action_process_member_id");
		
		foreach ($actual_expenses as $expense) {
			
			// to use when saving the application log
			$old_content_object = $expense->generateOldContentObjectData();
			
			// check if actual_expense is classified in any timesheet status member, and mark it to remove
			$members_to_remove = array();
			$ts_members = $expense->getMembers();
			foreach ($ts_members as $m) {
				if ($m->getDimensionId() == $member->getDimensionId()) {
					// if user can't edit the actual_expense depemding on the current classification, then this function throws an exception (don't check if changing from 'processed' member id)
					if (Plugins::instance()->isActivePlugin('workflow_permissions') && $m->getId() != $processed_member_id) {
						WorkflowPermissions::checkIfUserCanEdit(logged_user(), $m, $expense);
					}
					$members_to_remove[] = $m->getId();
				}
			}
			
			// if user can't classify the actual_expense depemding on the next status member, then this function throws an exception
			if (Plugins::instance()->isActivePlugin('workflow_permissions') && $action != 'process') {
				WorkflowPermissions::checkIfUserCanChangeStatus(logged_user(), $member, $expense);
			}
			
			// remove from previous timesheet status member (if any)
			if (count($members_to_remove) > 0) {
				ObjectMembers::removeObjectFromMembers($expense, logged_user(), $members_to_remove, $members_to_remove);
			}
			
			// classify in the new timesheet status member
			ObjectMembers::addObjectToMembers($expense->getId(), array($member));
			
			$modified_objects[] = $expense;

			$log_data = (count($members_to_remove) ? 'from:'.implode(',',$members_to_remove).';' : '' ) . 'to:'.$member->getId();
			ApplicationLogs::instance()->createLog($expense, ApplicationLogs::ACTION_MOVE, false, true, true, $log_data);
		}

		$null = null;
		Hook::fire('after_expenses_bulk_action', array('modified_objects' => $modified_objects, 'action' => $action, 'member' => $member), $null);

		return $modified_objects;
	}
	
	
	private function get_actual_expenses_from_parameters() {
		return get_actual_expenses_from_grid_parameters();
	}
	
	
	private function bulk_action_actual_expenses($member_id, $action) {
		ajx_current("emtpy");
		
		$actual_expenses = $this->get_actual_expenses_from_parameters();
		if (is_array($actual_expenses)) {
			
			$member = Members::getMemberById($member_id);
			if ($member instanceof Member) {

				// If action is 'invoice' include expensess with 'approved' status only
				if($action == 'invoice'){
					$actual_expenses = $this->get_actual_expenses_with_approved_status($actual_expenses);
					
					// Create a warning if user didn't select any approved actual expense
					if(empty($actual_expenses)){
						flash_error(lang('you must select at least one approved actual expense to perform this action'));
						return;
					}
				}
				
				try {
					DB::beginWork();
					
					$modified_expenses = $this->change_actual_expenses_status_member($actual_expenses, $member, $action);
					
					// generate the invoice
					if ($action == 'invoice') {
						$project_ot = ObjectTypes::findByName('project');
						$client_ot = ObjectTypes::findByName('customer');
						$project_member = null;
						foreach (active_context() as $selection) {
							if ($selection instanceof Member && in_array($selection->getObjectTypeId(), array($project_ot->getId(), $client_ot->getId()))) {
								$project_member = $selection;
								break;
							}
						}
						$template_id = IncomeInvoiceTemplates::getMemberInvoiceTemplateId($project_member);
						$invoice_template = IncomeInvoiceTemplates::instance()->findById($template_id);
						
						$generated_invoices = null;
						$hook_params = array('member'=>$project_member, 'invoice_template'=>$invoice_template, 'timeslots'=>array(), 'payment_receipts'=>$modified_expenses, 'param_billing_period_from'=>'', 'param_billing_period_to'=>'');
						Hook::fire('override_invoice_generation', $hook_params, $generated_invoices);
						
						if (!$generated_invoices) {
							$invoice = IncomeInvoiceTemplates::generateInvoiceFromTemplateAndObjects($project_member, $invoice_template, array(), $modified_expenses, '', '');
							$generated_invoices = array($invoice);
						}
						
						foreach ($generated_invoices as $invoice) {
							if ($invoice instanceof IncomeInvoice) {
								flash_success(lang('success add invoice', $invoice->getObjectName()));
							}
						}
						evt_add('reload tab panel', 'income-panel');
					}
					
					DB::commit();
					//ajx_current("reload");
					evt_add("reload current panel");
					
				} catch (Exception $e) {
					DB::rollback();
					flash_error($e->getMessage());
					ajx_current("empty");
				}
				
			}
		} else {
			flash_error(lang('you must select at least one actual_expense or project to perform this action'));
		}
	}

	function get_actual_expenses_with_approved_status($all_actual_expenses) {
		$actual_expenses = array();
		$approved_member_id = config_option("time_bulk_action_approved_member_id");
		$aproval_status_dim_id = Dimensions::findByCode('status_timesheet')->getId();
		foreach ($all_actual_expenses as $exp){
			$exp_members = $exp->getMembers();
			foreach ($exp_members as $m) {
				if ($m->getDimensionId() == $aproval_status_dim_id && $m->getId() == $approved_member_id) {
					$actual_expenses[] = $exp;
				}
			}
		} 

		return $actual_expenses;
	}

	private function check_actual_expenses_non_billiable($actual_expenses){
		foreach ($actual_expenses as $e){
			if($e->getColumnValue('is_billable') == 1) return false;
		}
		return true;
	}
	
	
	function put_pending_actual_expenses() {
		$this->bulk_action_actual_expenses(config_option("time_bulk_action_pending_member_id"), 'pending');
	}
	
	
	function lock_actual_expenses() {
		$this->bulk_action_actual_expenses(config_option("time_bulk_action_locked_member_id"), 'lock');
	}
	
	
	function approve_actual_expenses() {
		$this->bulk_action_actual_expenses(config_option("time_bulk_action_approved_member_id"), 'approve');
	}
	
	
	function invoice_actual_expenses() {
		$this->bulk_action_actual_expenses(config_option("time_bulk_action_invoiced_member_id"), 'invoice');
	}

	function process_actual_expenses() {
		$this->bulk_action_actual_expenses(config_option("time_bulk_action_processed_member_id"), 'process');
	}
	
	function put_under_review_actual_expenses() {
		$this->bulk_action_actual_expenses(config_option("time_bulk_action_review_member_id"), 'review');
	}
		
}

