<?php

/**
 * PaymentReceipt controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class PaymentReceiptController extends ApplicationController {

	var $plugin_name = "expenses2";
	
	/**
	 * Construct the PaymentReceiptController
	 *
	 * @access public
	 * @param void
	 * @return PaymentReceiptController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
		if (Plugins::instance()->isActivePlugin('multiple_currencies')) {
			Env::useHelper('functions', 'multiple_currencies');
		}
		Env::useHelper('functions', $this->plugin_name);
	} // __construct
	
	// ---------------------------------------------------
	//  Index
	// ---------------------------------------------------
	
	function init() {
		require_javascript("PaymentReceiptsManager.js", $this->plugin_name);
		ajx_current("panel", "payment-receipts-containerpanel", null, null, true);
		ajx_replace(true);
	}
	

	function get_list_config() {
		ajx_current("empty");
		
		ob_start();
		tpl_display(get_template_path('list_grid_config/columns', 'payment_receipt', 'expenses2'));
		tpl_display(get_template_path('list_grid_config/toolbar', 'payment_receipt', 'expenses2'));
	
		$html = ob_get_clean();
	
		ajx_extra_data(array('html' => $html));
	}
	
	
	function list_all($return_objects = false, $extra_conditions = "") {
		ajx_current("empty");
		// Get all variables from request
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		$attributes = array("ids" => explode(',', array_var($_GET,'ids')));
		$dim_order = null;
		$cp_order = null;
		
		$actionMessage = array();
		if (isset($action)) {
			try {
				$actionMessage = $this->resolveAction($action, $attributes);
				if ($actionMessage["errorCode"] == 0) {
					flash_success($actionMessage["errorMessage"]);
				} else {
					$action_error_msg = $actionMessage["errorMessage"];
				}
			} catch (Exception $e) {
				$action_error_msg = $e->getMessage();
			}
		}
		if (isset($action_error_msg)) {
			flash_error($action_error_msg);
			evt_add("reload current panel");
		}
		
		$select_columns = array("*");
		//$extra_conditions = "";
		$join_params = null;
		
		if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		} else if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		} else {
		    //break;
		    
		    //(Conrado) tried replacing this with a 
		    //break;
		    //but it was failing to list payment receipts
		    //So I left it in the mean time - but we need to review as this shouldn't apply any more
		    //As we decided to remove the "status" property and replace it with a dimension
			switch ($order) {
				case 'quickbooks':
					$order = '`quickbooks_online_id`';
					break;
				case 'status_id':
					$order = '`jt`.`order`';
					$join_params = array(
						'table' => ExpensesStatuses::instance()->getTableName(),
						'join_type' => 'LEFT',
						'jt_field' => 'id',
						'e_field' => 'status_id',
					);
					$select_columns = array("e.*, o.*, jt.`order`");
					break;
				case 'product_type_name':
					$order = '`prod_type_name`';
					$join_params = array(
						'table' => ProductTypes::instance()->getTableName(),
						'join_type' => 'LEFT',
						'jt_field' => 'id',
						'e_field' => 'product_type_id',
					);
					$select_columns = array("e.*, o.*, jt.`name` as `prod_type_name`");						
					break;	  
				default:
					//$order = "name";
					break;
			}
			
		}
		
		Hook::fire("listing_extra_conditions", null, $extra_conditions);
	
		if (array_var($_GET, 'pr_from_filter') && array_var($_GET, 'pr_from_filter') != 'undefined') {
			$from_date = DateTimeValueLib::dateFromFormatAndString(user_config_option('date_format'), array_var($_GET, 'pr_from_filter'));
			if ($from_date instanceof DateTimeValue) {
				$extra_conditions .= " AND e.date >= '".$from_date->toMySQL()."'";
			}
		}
		if (array_var($_GET, 'pr_to_filter') && array_var($_GET, 'pr_to_filter') != 'undefined') {
			$to_date = DateTimeValueLib::dateFromFormatAndString(user_config_option('date_format'), array_var($_GET, 'pr_to_filter'));
			if ($to_date instanceof DateTimeValue) {
				$extra_conditions .= " AND e.date <= '".$to_date->toMySQL()."'";
			}
		}

		if (array_var($_GET, 'expense_invoicing_status_filter')) {
			$invoice_status = array_var($_GET, 'expense_invoicing_status_filter');
			if($invoice_status != ''){
				if ($invoice_status == 'non_billable') {
					$extra_conditions .= " AND is_billable=0";
				} else {
					$extra_conditions .= " AND is_billable=1 AND invoicing_status LIKE '".$invoice_status."'";
				}
			}
		}

		if(!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_expenses_of_others')){
			$extra_conditions .= " AND paid_by_id = " . logged_user()->getId();
		} else {
			if ($user_filter_val = array_var($_GET, 'pr_user_filter')) {
				if($user_filter_val != ''){
					$extra_conditions .= " AND paid_by_id = '".$user_filter_val."'";
				}
			}
		}

		$task_id = array_var($_REQUEST, 'task_id');
		if (is_numeric($task_id) && $task_id > 0) {
			$extra_conditions .= " AND task_id = '$task_id'";
		}
			
		$only_count_result = array_var($_GET, 'only_result',false);
		$result = PaymentReceipts::instance()->listing(array(
			"order" => $order,
			"order_dir" => $order_dir,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"select_columns" => $select_columns,
			"extra_conditions" => $extra_conditions,
			"count_results" => false,
			"only_count_results" => $only_count_result,
			"join_params" => $join_params,
			"start" => $start,
			"limit" => $limit,
		));			
		$payment_receipts = $result->objects;
				

		if ($return_objects) {
			return $payment_receipts;
		}
		
		$object = $this->prepareObject($payment_receipts, $start, $limit, $result);
		
		ajx_extra_data($object);		
		tpl_assign("listing", $object);
		
	}
	

	private function prepareObject($payment_receipts, $start, $limit, $res_obj) {
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"payment_receipts" => array(),
		);
		
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
		
		$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType(PaymentReceipts::instance()->getObjectTypeId());

		$count = min(array($limit, count($payment_receipts)));
		for ($i = 0; $i < $count; $i++){
			if (isset($payment_receipts[$i])){
				$payment_receipt = $payment_receipts[$i];
				
				if ($payment_receipt instanceof PaymentReceipt){
					$info = $payment_receipt->getArrayInfo(true);
					$info['memPath'] = json_encode($payment_receipt->getMembersIdsToDisplayPath(true));
					
					Hook::fire('payment_list_additional_column_values',array('payment'=>$payment_receipt),$info);

					$object["payment_receipts"][$i] = $info;
					
					foreach ($custom_properties as $cp) {
						$object["payment_receipts"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $payment_receipt);
					}
					
					$taxes = ActualExpenseTaxValues::getAllByObjectId($payment_receipt->getId());
					if ($taxes) {
						foreach ($taxes as $tax) {
							$object["payment_receipts"][$i]['tax_'.$tax->getTaxId()] = format_money_amount($tax->getValue(), '$', config_option('product_type_decimals'));
						}
					}
    			}
			}
		}
		return $object;
	}
	
	
	/**
	 * Resolve action to perform
	 *
	 * @param string $action
	 * @param array $attributes
	 */
	private function resolveAction($action, $attributes){
		 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; 
				$err = 0;
				if(Plugins::instance()->isActivePlugin('income')){
					// exclude '__total_row__' as payment receipt id
					$exp_ids_array = array_diff($attributes["ids"], array('__total_row__'));
					$exp_ids_array = array_filter($exp_ids_array, 'is_numeric');

                    $expenses_ids = '(' . implode(',', $exp_ids_array) . ')';
                    $expenses = PaymentReceipts::instance()->findAll(array('conditions' => array('`object_id` IN ' . $expenses_ids . ' AND `invoicing_status`="invoiced"')));
                    if (count($expenses)){	    
                        $resultCode = 2;
                        $resultMessage = lang("you cannot delete invoiced time entry");
                        return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
                    }	
                } 
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					if (!is_numeric($id)) continue;
					$payment_receipt = PaymentReceipts::instance()->findById($id);
					$error_details = "";
					if ($payment_receipt instanceof PaymentReceipt && $payment_receipt->canDelete(logged_user())){
						try{
							DB::beginWork();
							
							$payment_receipt->trash();
							
							DB::commit();
							$succ++;
							ApplicationLogs::createLog($payment_receipt, ApplicationLogs::ACTION_TRASH);
						} catch(Exception $e){
							DB::rollback();
							$error_details .= "\n" . $e->getMessage();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) . $error_details . ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;
		}
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
		
	}
	
	function view() {
		ajx_set_no_toolbar();
		$payment_receipt = PaymentReceipts::instance()->findById(get_id());
		if(!($payment_receipt instanceof PaymentReceipt)) {
			flash_error(lang('message dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$payment_receipt->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		tpl_assign('payment_receipt', $payment_receipt);
	}
	
	

	/**
	 * Add the Payment Receipt posted in $_POST
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function add() {
		/*$can_see_cost_info = true;
		Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $can_see_cost_info);
		if (!can_add(logged_user(), array(), PaymentReceipts::instance()->getObjectTypeId()) || !$can_see_cost_info) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}*/
		
		$payment_receipt = new PaymentReceipt();
		$payment_receipt->setObjectTypeId(PaymentReceipts::instance()->getObjectTypeId());
		tpl_assign('payment_receipt', $payment_receipt);

		$data = array_var($_POST, 'payment_receipt');
		if(!is_array($data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			$payment_receipt->setIsBillable(true);

			$pre_selected_members = null;

			$task_id = array_var($_REQUEST, 'task_id');
			if (is_numeric($task_id) && $task_id > 0) {
				$payment_receipt->setTaskId($task_id);
				$task = ProjectTasks::instance()->findById($task_id);
				if ($task instanceof ProjectTask) {
					$pre_selected_members = $task->getMemberIds();
				}
			}

			tpl_assign('pre_selected_members', $pre_selected_members);
			
			tpl_assign('expense_id', array_var($_REQUEST, 'expense_id'));
			if(array_var($_REQUEST, 'expense_id')){
				$expense = Expenses::instance()->findById(array_var($_REQUEST, 'expense_id'));
				tpl_assign('expense', $expense);
			}
			
		} else {
			foreach ($data as $k => &$v) {
				$v = remove_scripts($v);
			}
			try {
				$member_ids = json_decode(array_var($_POST, 'members'));

				$members = array();
				foreach($member_ids as $m_id){
					$member = Members::instance()->findById($m_id);
					if ($member instanceof Member){
						$members[] = $member;
					}
				}

				if (!can_add(logged_user(), $members, PaymentReceipts::instance()->getObjectTypeId())) {
					flash_error(lang('no access permissions to add actual expense'));
					ajx_current('empty');
					return;
				}

				DB::beginWork();
				$this->save_payment_receipt($data, $payment_receipt, $member_ids);
				
				DB::commit();
				
				ApplicationLogs::createLog($payment_receipt, ApplicationLogs::ACTION_ADD);
				flash_success(lang('success add payment receipt', $payment_receipt->getName()));

				if (array_var($_REQUEST, 'dont_reload')) {

					ajx_current("empty");

					// build data to send to the interface
					$exp_data = $payment_receipt->getArrayInfo(true);
					// get classification
					$exp_data['memPath'] = json_encode($payment_receipt->getMembersIdsToDisplayPath());
					// get values for custom properties
					$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType(PaymentReceipts::instance()->getObjectTypeId());
					foreach ($custom_properties as $cp) {
						$exp_data['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $payment_receipt);
					}

					// send the data
					ajx_extra_data(array("payment_receipt" => $exp_data));

				} else {

					if (array_var($_REQUEST, 'modal')) {
						evt_add("reload current panel");
					} else {
						ajx_current("back");
					}
				}
          		
	          	ajx_add("overview-panel", "reload");

			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
				
			} // try

		} // if
	} // add

	/**
	 * Edit Payment Receipt
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function edit() {
		$this->setTemplate('add');

		try {

			$payment_receipt = PaymentReceipts::instance()->findById(get_id());
			if (!$payment_receipt instanceof PaymentReceipt) {
				throw new Exception(lang('object dnx'));
			}
	
			if (!$payment_receipt->canEdit(logged_user())) {
				throw new Exception(lang('no access permissions'));
			}
	
			if (Plugins::instance()->isActivePlugin('income') && $payment_receipt->getColumnValue('invoicing_status') == 'invoiced') {
				$can_edit = (config_option('allow_to_edit_invoices') && (logged_user()->getEmailAddress() == 'support@evxsoftware.com' || logged_user()->getEmailAddress() == 'support@fengoffice.com'));
				if (!$can_edit) {
					throw new Exception(lang('you cannot edit invoiced actual expense'));
				}
			}

		} catch (Exception $e) {
			flash_error($e->getMessage());
			ajx_current('empty');
			return;
		}
		
		tpl_assign('payment_receipt', $payment_receipt);
		
		$data = array_var($_POST, 'payment_receipt');
		if(!is_array($data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			$expense = Expenses::instance()->findById($payment_receipt->getExpenseId());
			tpl_assign('expense', $expense);
			
		} else {
			foreach ($data as $k => &$v) {
				$v = remove_scripts($v);
			}
			try {
				// to use when saving the application log
				$old_content_object = $payment_receipt->generateOldContentObjectData();

				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$this->save_payment_receipt($data, $payment_receipt, $member_ids);
				
				DB::commit();
				
				ApplicationLogs::createLog($payment_receipt, ApplicationLogs::ACTION_EDIT);
				flash_success(lang('success edit payment receipt', $payment_receipt->getName()));
				
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				} else {
					ajx_current("back");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
			} // try
		} // if
	} // edit
	
	/**
	 * 
	 * @param array $data
	 * @param PaymentReceipt $payment_receipt
	 * @param array $member_ids
	 */
	function save_payment_receipt($data, $payment_receipt, $member_ids) {
		// gather uploaded image
		$tmp_file_name = array_var($data, 'document_id', '');
		if (trim($tmp_file_name) != "") {
			if (file_exists(ROOT."/tmp/$tmp_file_name")) {
				$info = array_var($_SESSION, $tmp_file_name);
				$type = array_var($info, "type");
		
				$repo_id = FileRepository::addFile(ROOT."/tmp/$tmp_file_name", array(
						'name' => array_var($info, 'name', 'Expense document'), 'size '=> array_var($info, 'size', 0), 'type' => $type, 'public' => true
				));
				
				$data['document_id'] = $repo_id;
			}
		}

		if($data['is_billable'] == 0){
			$data['unit_price'] = 0;
			$data['total_price'] = 0;
		}

		if (!array_var($data, 'currency_id')) {
			$c = Currencies::getDefaultCurrencyInfo();
			$data['currency_id'] = $c ? $c['id'] : 1; // never let currency_id=0, if we can't find the default then use currecny_id=1, it is precharged when installing
		}

		if ( config_option('actual_expense_type_mandatory') && 
			(!array_var($data, 'expense_type') || array_var($data, 'expense_type') == 'none_blank') ) {
			throw new Exception(lang('custom property value required', lang('expense_type')));
		}
				
		//remove thousands separator
		//$data['amount'] = (double)filter_var($data['amount'], FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION);
		$data['amount'] = clean_formatted_money_amount_for_sql($data['amount']);
		$data['unit_cost'] = clean_formatted_money_amount_for_sql($data['unit_cost']);
		$data['unit_price'] = clean_formatted_money_amount_for_sql($data['unit_price']);
		$data['total_price'] = clean_formatted_money_amount_for_sql($data['total_price']);
		
		$product_type_id = array_var($data, 'product_type_id', 0);
		if ($product_type_id > 0) {
			$pt = ProductTypes::getById($product_type_id);
			if ($pt && $pt->getUsesPriceMultiplier()) {
				$data['price_multiplier'] = $pt->getPriceMultiplier();
			}
		}

		if (array_var($data,'name','') == '') {
			$data['name'] = $this->autogenerate_actual_expense_name($data);
		}
		
		// get a date object from the formatted date submitted
		$data['date'] = getDateValue($data['date']);
		
		$total_tax_amount = 0;
		$taxes = array_var($data, 'taxes', array());
		if (count($taxes) > 0) {
			foreach ($taxes as $tax_id => $tax_amount) {
				$tax_amount = clean_formatted_money_amount_for_sql($tax_amount);
				$total_tax_amount += $tax_amount;
			}
		}
		$data['total_cost_without_taxes'] = $data['amount'] - $total_tax_amount;
		
		$this->validate_invoice_number($payment_receipt);

		$previous_task_id = $payment_receipt->getColumnValue('task_id');
		$payment_receipt->setFromAttributes($data);

		$payment_receipt->was_new = $payment_receipt->isNew();
		$payment_receipt->save();
		
		$data['previous_task_id'] = $previous_task_id;

		// validate if expense type is direct_to_vendor then a member of suppliers dimension must be selected
		$this->validate_supplier_or_employee_present($payment_receipt, $member_ids);
		
		//Get the Expense Id of the Payment Receipt 
		$expense_id = $payment_receipt->getExpenseId();
		//(Force) append the Expense (through the expense_id) into the array of linked objects
		if ($expense_id >0){
		  $_POST["linked_objects"][]=$expense_id;
		}
		//@ToDo: else { If the Payment Receipt is linked to an expense, and now it is not, remove the link }

		
		$object_controller = new ObjectController();
		$object_controller->add_to_members($payment_receipt, $member_ids);
		$object_controller->add_subscribers($payment_receipt);
		$object_controller->link_to_new_object($payment_receipt, false);
		$object_controller->add_custom_properties($payment_receipt);

		Hook::fire('after_payment_receipt_save', $payment_receipt, $data);
		
		//After receipt is saved, reset (remove) all linked_objects from array.
		$_POST["linked_objects"]=array();
		
		// Save actual expense taxes
		$taxes = array_var($data, 'taxes', array());
		ActualExpenseTaxValues::instance()->delete("object_id = ".$payment_receipt->getId());
		if (count($taxes) > 0) {
			foreach ($taxes as $tax_id => $tax_amount) {
				$tax_amount = clean_formatted_money_amount_for_sql($tax_amount);
				
				$tax_val = new ActualExpenseTaxValue();
				$tax_val->setTaxId($tax_id);
				$tax_val->setObjectId($payment_receipt->getId());
				$tax_val->setValue($tax_amount);
				$tax = ActualExpenseTaxes::instance()->findById($tax_id);
				$applied_value = $tax->getIsPercentage() ? $tax->getValue() : $tax_amount;
				$tax_val->setAppliedValue($applied_value);
				
				$tax_val->save();
			}
		}
	}


	private function autogenerate_actual_expense_name($data) {
		$option_value = config_option('actual_expense_name_generation');
		$include_date = $option_value == 'date_quantity_product' || $option_value == 'date_product';
		$include_quantity = $option_value == 'date_quantity_product';

		$name = "";
		if ($include_date) {
			$name .= $data['date'];
		}

		if ($include_quantity) {
			$name .= ' ' . $data['quantity'];
		}

		$pt = ProductTypes::getById($data['product_type_id']);
		if ($pt) {
			$name .= ' ' . $pt->getName();
		}

		return trim($name);
	}


	private function validate_invoice_number(PaymentReceipt $payment_receipt){
		if($payment_receipt->getExpenseType() == 'direct_to_vendor'){
			$act_exp_obj_type = ObjectTypes::findByName('payment_receipt');
			$invoice_obj_cp = CustomProperties::getCustomPropertyByCode($act_exp_obj_type->getId(), "invoice_number");
			if (!$invoice_obj_cp) return; 
			$invoice_cp_number = $invoice_obj_cp->getId();

			$data = array_var($_POST, 'object_custom_properties');
			$custom_value = $data[$invoice_cp_number];
			
			if(empty($custom_value)){
				throw new Exception(lang("invoice number is required"));
			}

			$exists_number = CustomPropertyValues::instance()->findOne(array(
				'conditions' => array("`custom_property_id` = ? AND `value` = ?", $invoice_cp_number, $custom_value)
			));

			$current_number_invoice = get_number_invoice_expense($payment_receipt);
			if(($payment_receipt->isNew() && !empty($exists_number)) || (!$payment_receipt->isNew() &&  $custom_value != $current_number_invoice && !empty($exists_number))){
				throw new Exception(lang("invoice number is exists"));
			}
		}
	}


	private function validate_supplier_or_employee_present(PaymentReceipt $payment_receipt, $member_ids) {

		if ($payment_receipt->getExpenseType() == 'direct_to_vendor' && Plugins::instance()->isActivePlugin('suppliers')) {

			$supplier_present = false;
			$suppliers_dim = Dimensions::findByCode('suppliers');
			foreach ($member_ids as $mid) {
				$member = Members::getMemberById($mid);
				if ($member->getDimensionId() == $suppliers_dim->getId()) {
					$supplier_present = true;
					break;
				}
			}

			if (!$supplier_present) {
				$supplier_ot = ObjectTypes::findByName('supplier');
				throw new Exception(lang('a supplier must be selected for direct to vendor expenses', $supplier_ot->getObjectTypeName()));
			}

		} else if ($payment_receipt->getExpenseType() == 'reimbursable' && $payment_receipt->getPaidById() == 0) {

			throw new Exception(lang('an employee must be selected for reimbursable expenses'));
		}

	}

	/**
	 * Delete specific analysis
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function delete() {
		
		$payment_receipt = PaymentReceipts::instance()->findById(get_id());
		if (!$payment_receipt instanceof PaymentReceipt) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		if (!$payment_receipt->canDelete(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}

		if(Plugins::instance()->isActivePlugin('income') && $payment_receipt->getColumnValue('invoicing_status') == 'invoiced'){ 
            flash_error(lang('you cannot delete invoiced actual expense'));
            ajx_current("empty");
            return;
        }
		
		try {
			DB::beginWork();
			
			$name = $payment_receipt->getName();
			//PaymentReceiptMembers::instance()->delete("payment_receipt_id=".$payment_receipt->getId());
			$payment_receipt->delete();
		
			DB::commit();
		
			flash_success(lang('success delete payment receipt', $name));
		
			if (array_var($_REQUEST, 'modal')) {
				evt_add("reload current panel");
			} else {
				ajx_current("back");
			}

			ApplicationLogs::createLog($payment_receipt, ApplicationLogs::ACTION_DELETE);
		
		} catch(Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			ajx_current("empty");
		}
	} // delete


	public function check_payment_receipts_invoicing_status(){
		ajx_current("empty");
		$result = false;
		if(Plugins::instance()->isActivePlugin('income')){
			$pr_ids = array_var($_GET, 'ids');
			// exclude '__total_row__' as payment receipt id
			$exp_ids_array = array_diff(explode(',', $pr_ids), array('__total_row__'));
			$exp_ids_array = array_filter($exp_ids_array, 'is_numeric');
			$expenses_ids = '(' . implode(',', $exp_ids_array) . ')';
			$expenses = PaymentReceipts::instance()->findAll(array('conditions' => '`object_id` IN ' . $expenses_ids . ' AND `invoicing_status`="invoiced"'));
			if (count($expenses)){	    
				flash_error(lang('you cannot delete invoiced actual expense'));
				$result = true;
			}	
		} 
		ajx_extra_data(array("hasInvoicedPRs" => $result));
	}



} // PaymentReceiptController

?>