<?php

function expense_status_select_box($name, $selected = null, $attributes = null) {
	$status_options = array();
	$statuses = ExpensesStatuses::instance()->findAll(array('conditions' => 'enabled=1', 'order' => '`order`'));
	foreach ($statuses as $status) {
		$sel = array();
		if ($selected == $status->getId()) $sel['selected'] = 'selected';
		$text = $status->getName() == "" ? lang($status->getCode()) : $status->getName();
		$status_options[] = option_tag($text, $status->getId(), $sel);
	}
	return select_box($name, $status_options, $attributes);
}

function currency_select_box($name, $selected = null, $include_names = false, $attributes = null) {
	$currency_options = array();
	$currencies = Currencies::getAllCurrencies();
	foreach ($currencies as $currency) {
		$sel = array();
		if ($selected == null) {
			if ($currency->getIsDefault()) $sel['selected'] = 'selected'; 
		} else {
			if ($selected == $currency->getId()) $sel['selected'] = 'selected';
		}
		$text = $currency->getSymbol() . ($include_names ? ' ('.$currency->getName().')' : '');
		$currency_options[] = option_tag($text, $currency->getId(), $sel);
	}
	return select_box($name, $currency_options, $attributes);
}

function product_types_select_box($name, $selected = null, $product_types = null, $attributes = null) {
	
	if (!$product_types) {
		$product_types = ProductTypes::getAllProductTypesData();
	}
	
	if (!$attributes) $attributes = array();
	if (!isset($attributes['class'])) $attributes['class'] = '';
	$attributes['class'] .= " product_type_selector";
	
	$options = array(option_tag('', 0));
	
	foreach ($product_types as $product_type) {
		$sel = array();
		if ($selected == $product_type['id']) $sel['selected'] = 'selected';
		$options[] = option_tag(str_replace('\\"', '"', $product_type['name']), $product_type['id'], $sel);
	}
	return select_box($name, $options, $attributes);
}

function actual_expense_type_select_box($name, $selected = null, $attributes = null) {

	$default_actual_expense_types = config_option('default_actual_expense_types');

	$actual_expense_types = config_option('set_actual_expense_type');
	if (!$attributes) $attributes = array();
	if (!isset($attributes['class'])) $attributes['class'] = '';
	$attributes['class'] .= " expense_type_selector";
	$options = array();
	
	if($default_actual_expense_types){
		array_unshift($actual_expense_types, "none_blank");
	}

	$filter_actual_expense_types = array_filter($actual_expense_types);

	foreach($filter_actual_expense_types as $type):

		if($selected != null):
			$attr = $selected == $type ? array('selected' => 'selected') : null;
		else:
			$attr = $default_actual_expense_types == $type ? array('selected' => 'selected') : null;
		endif;

		$options[] = option_tag(lang(str_replace("_", " ", $type)), $type, $attr);
		
	endforeach;
	
	// if (!$attributes) $attributes = array();
	// if (!isset($attributes['class'])) $attributes['class'] = '';
	// $attributes['class'] .= " expense_type_selector";
	
	// $options = array();

	// $attr = $selected == 'reimbursable' ? array('selected' => 'selected') : null;
	// $options[] = option_tag(lang('reimbursable expense'), 'reimbursable', $attr);

	// $attr = $selected == 'direct_to_vendor' ? array('selected' => 'selected') :  null;
	// $options[] = option_tag(lang('direct_to_vendor expense'), 'direct_to_vendor', $attr);

	// if (config_option('use_field_lab_test_expense')) {
	// 	$attr = $selected == 'field_lab_testing' ? array('selected' => 'selected') :  null;
	// 	$options[] = option_tag(lang('field_lab_testing expense'), 'field_lab_testing', $attr);
	// }

	// if (config_option('use_rentals_cons_expense')) {
	// 	$attr = $selected == 'rental_consumable' ? array('selected' => 'selected') :  null;
	// 	$options[] = option_tag(lang('rental_consumable expense'), 'rental_consumable', $attr);
	// }

	// $attr = $selected == 'credit_card_purchase' ? array('selected' => 'selected') :  null;
	// $options[] = option_tag(lang('credit_card_purchase expense'), 'credit_card_purchase', $attr);
	
	return (!empty($actual_expense_types[0])) ? select_box($name, $options, $attributes) : false;
}

/**
 * Hardcoded payment terms selector
 * Must be overriden after configutable payment terms development is ready
 */
function actual_expense_payment_term_select_box($name, $selected = null, $attributes = null) {
	
	if (!$attributes) $attributes = array();
	if (!isset($attributes['class'])) $attributes['class'] = '';
	$attributes['class'] .= " payment_term_selector";

	if (!$selected) {
		$selected = config_option('default_expense_payment_term'); // default Net 30
	}

	$available_terms = array(
		1 => lang('payment term due_on_receipt'),
		2 => lang('payment term net x', 10 ), // Net 10,
		3 => lang('payment term net x', 15 ), // Net 15,
		4 => lang('payment term net x', 30 ), // Net 30,
		5 => lang('payment term net x', 45 ), // Net 45,
		6 => lang('payment term net x', 60 ), // Net 60,
		7 => lang('payment term net x', 90 ), // Net 90,
	);

	$options = array();
	foreach ($available_terms as $id => $text) {
		$attr = $selected == $id ? array('selected' => 'selected') : null;
		$options[] = option_tag($text, $id, $attr);
	}
	
	return select_box($name, $options, $attributes);
}

/**
 * Hardcoded function to get the payment term name
 * Must be overriden after configutable payment terms development is ready
 */
function actual_expense_get_payment_term_name($id) {
	$available_terms = array(
		1 => lang('payment term due_on_receipt'),
		2 => lang('payment term net x', 10 ), // Net 10,
		3 => lang('payment term net x', 15 ), // Net 15,
		4 => lang('payment term net x', 30 ), // Net 30,
		5 => lang('payment term net x', 45 ), // Net 45,
		6 => lang('payment term net x', 60 ), // Net 60,
		7 => lang('payment term net x', 90 ), // Net 90,
	);

	return array_var($available_terms, $id, '');
}
// ---


function get_expense_items_for_expense($id, $params=array()) {
	
	$expense_items_data = array();
	$expense_items = ExpenseItems::instance()->findAll(array("conditions" => array("expense_id=?", $id)));
	
	foreach ($expense_items as $expense_item) {
		
		$info = $expense_item->getArrayInfo();
		$expense_items_data[] = $info;
		
	}
	
	return $expense_items_data;
}

/**
 * Returns all the expense items for a set of budgeted expenses, grouped by expense id
 * @param array $expense_ids The expense ids to use in items query
 * @return array all the required expense items grouped by expense id
 */
function get_expense_items_for_expenses_array($expense_ids) {
	if (!isset($expense_ids) || count($expense_ids) == 0) return array();

	$expense_ids = array_filter($expense_ids,'is_numeric');
	
	$expense_items_data = DB::executeAll("
		SELECT * FROM ".TABLE_PREFIX."expense_items
		WHERE expense_id IN (".implode(',',$expense_ids).")
	");
	
	if (!$expense_items_data) $expense_items_data = array();
	
	$expense_items_data_by_exp_id = array();

	foreach ($expense_items_data as $item_data) {
		$exp_id = $item_data['expense_id'];
		if (!isset($expense_items_data_by_exp_id[$exp_id])) {
			$expense_items_data_by_exp_id[$exp_id] = array();
		}
		$expense_items_data_by_exp_id[$exp_id][] = $item_data;
	}

	return $expense_items_data_by_exp_id;
}

/**
 * Gets the needed expense information for financials and expense progress widgets
 */
function collect_expenses_and_payment_receipts_data_for_widget($params = array()) {

	$exclude_expenses_assigned_to_tasks = array_var($params, 'exclude_expenses_assigned_to_tasks', false);
	// use this kind of cache (only valid in current request) to prevent double execution 
	// if more than one widget calls this function
	if ($exclude_expenses_assigned_to_tasks && isset($_REQUEST['expenses_data_for_financials_widget'])) {
		return $_REQUEST['expenses_data_for_financials_widget'];
	} else if (isset($_REQUEST['expenses_data_for_expenses_widget'])) {
		return $_REQUEST['expenses_data_for_expenses_widget'];
	}
	
	$join_params = array(
			'table' => ExpenseItems::instance()->getTableName(),
			'join_type' => 'LEFT',
			'jt_field' => 'expense_id',
			'e_field' => 'object_id',
	);

	Env::useHelper('persist_member_expense_totals_functions', 'expenses2');
	$use_context = true;
	// Get the ids of the tasks that have manual estimated value
    $manual_estimated_value_task_ids = get_manual_financial_value_task_ids(null, 'is_calculated_estimated_price', false, $use_context);

	// Get the ids of the tasks that have manual estimated value and are fixed fee
	$manual_earned_value_task_ids = get_manual_financial_value_task_ids(null, 'is_calculated_estimated_price', true, $use_context);

	// This is a patch to prevent reaching memory limit when no filters are applied and there are too much timeslots in the system
	// @TODO: Improve memory usage of the listing function for timeslots and tasks and remove this condition
	$this_year_cond = "";
	$bud_exp_this_year_cond = "";
	$active_context_members = active_context_members(false);
	$beginning_of_year = '';
	$include_subtitle = false;
	if (count($active_context_members) == 0) {
		$beginning_of_year = DateTimeValueLib::now();
		if( $beginning_of_year->getMonth() < 6){
			$previous_year = $beginning_of_year->getYear() - 1;
			$beginning_of_year->setYear($previous_year);
		} 
		$beginning_of_year->setDay(1);
		$beginning_of_year->setMonth(1);
		$beginning_of_year->setHour(0);
		$beginning_of_year->setMinute(0);
		$beginning_of_year->setSecond(0);

		// don't filter the period if the amount of objects is too small
		$exp_count = Expenses::instance()->count();
		$pr_count = PaymentReceipts::instance()->count();
		
		$this_year_cond = " AND (date >= '".$beginning_of_year->toMySQL()."' )";
		$bud_exp_this_year_cond = " AND (jt.date >= '".$beginning_of_year->toMySQL()."' )";

		$include_subtitle = true;
		$subtitle_date_start = $beginning_of_year;
	}
	// --

	$extra_conditions = " AND (jt.object_id = (SELECT MIN(p2.object_id) FROM ".TABLE_PREFIX."expense_items p2 WHERE p2.expense_id=jt.expense_id)) ";
	
	if ($exclude_expenses_assigned_to_tasks) {
		// Add condition to exclude expenses that are assigned to tasks
		$extra_conditions .= " AND `task_id`=0";
	} else {
		// Add condition to exclude expenses that are assigned to the manual estimated price tasks
		$extra_conditions .= !empty($manual_estimated_value_task_ids) ? ' AND `task_id` NOT IN ('.implode(',', $manual_estimated_value_task_ids).')' : '';
	}

	$extra_conditions .= $bud_exp_this_year_cond;
	$expenses_res = Expenses::instance()->listing(array(
			'select_columns' => array("o.*, e.*, jt.date as jt_date"),
			'raw_data' => true,
			'join_params' => $join_params,
			'extra_conditions' => $extra_conditions,
	)); // don't order this query it affects performance, order the expenses after we have the result
	$tmp_expenses = $expenses_res->objects;
	$expenses_totals = $expenses_res->total ;  // *** LC 2023-09-04  Undefined property: stdClass::$totals

	// sort the expenses using the expense item date
	if( isset($tmp_expenses)){
		usort($tmp_expenses, function($a, $b) {
			if ($a['jt_date'] == $b['jt_date']) {
				return 0;
			}
			return ($a['jt_date'] < $b['jt_date']) ? -1 : 1;
		});
		$expenses = array_values($tmp_expenses);
	}else{
		$expenses = array();
	}

	$pr_extra_condition = "";
	if(!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_expenses_of_others')){
		$pr_extra_condition .= " AND paid_by_id = " . logged_user()->getId();
	}
	
	// Add condition to exclude expenses that are assigned to the manual estimated price tasks
	$pr_extra_condition .= !empty($manual_estimated_value_task_ids) ? ' AND `task_id` NOT IN ('.implode(',', $manual_estimated_value_task_ids).')' : '';
	
	
	$payment_receipts_res = PaymentReceipts::instance()->listing(array(
			'raw_data' => true,
			'order' => '`date`',
			'order_dir' => 'ASC',
			'extra_conditions' => $this_year_cond . $pr_extra_condition,
	));
	$payment_receipts_data = $payment_receipts_res->objects;
	
	// collect the expense ids so we can get al the items at once
	$all_expense_ids = array_column($expenses, 'id');

	// get all the needed expense items in one query
	$all_expense_items = get_expense_items_for_expenses_array($all_expense_ids);

	$expenses_data = array();
	if ($expenses) {
		foreach ($expenses as $exp_data) {
			// assign items to each budgeted expense
			$exp_data['items'] = array_var($all_expense_items, $exp_data['id'], array());
			$expenses_data[] = $exp_data;
		}
	}

	
	$amounts_by_day = array();

	// collect each day's budgeted amount
	foreach ($expenses_data as $exp_data) {
		//$date_str = '';
		//if (isset($exp_data['items'][0])) $date_str = $exp_data['items'][0]['date'];
		foreach ($exp_data['items'] as $exp_item_data) {
		
			$day_key = '';
			$date_str = array_var($exp_item_data, 'date');
			if ($date_str && $date_str != EMPTY_DATE && $date_str != EMPTY_DATETIME) {
				$d = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $date_str);
				$day_key = $d->format('Y-m-d');
			} else {
				$d = DateTimeValueLib::now();
				$day_key = $d->format('Y-m-d');
			}
			
			if(count($active_context_members) == 0) {
				if($d->getTimestamp() >= $beginning_of_year->getTimestamp()){	
					if (!isset($amounts_by_day[$day_key])) {
						$amounts_by_day[$day_key] = array(
								'date' => $d->format('m/d/Y'),
								'budgeted' => 0,
								'actual' => 0,
						);
					}
					$amounts_by_day[$day_key]['budgeted'] += $exp_item_data['total_price'];
				}
			} else {
				if (!isset($amounts_by_day[$day_key])) {
					$amounts_by_day[$day_key] = array(
							'date' => $d->format('m/d/Y'),
							'budgeted' => 0,
							'actual' => 0,
					);
				}
				$amounts_by_day[$day_key]['budgeted'] += $exp_item_data['total_price'];
			}
	
		}
	}
	
	// collect each day's actual amount
	if ($payment_receipts_data) {
		foreach ($payment_receipts_data as $pr_data) {
			$date_str = $pr_data['date'];

			$d = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $date_str);
			$date_str = $d->format('m/d/Y');
			$day_key = $d->format('Y-m-d');

			if (!isset($amounts_by_day[$day_key])) {
				$amounts_by_day[$day_key] = array(
					'date' => $date_str,
					'budgeted' => 0,
					'actual' => 0,
				);
			}

			$amounts_by_day[$day_key]['actual'] += $pr_data['total_price'];
		}
	}

	// sort the amounts by date
	ksort($amounts_by_day);
	
	$expenses_data_for_widget = array(
			'amounts_by_day' => $amounts_by_day,
			'expenses_totals' => $expenses_totals,
			'include_subtitle' => $include_subtitle,
			'subtitle_date_start' => $subtitle_date_start,
	);
	if($exclude_expenses_assigned_to_tasks) {
		$_REQUEST['expenses_data_for_financials_widget'] = $expenses_data_for_widget;
	} else {
		$_REQUEST['expenses_data_for_expenses_widget'] = $expenses_data_for_widget;
	}
	
	return $expenses_data_for_widget;
}

function get_actual_expenses_from_grid_parameters() {
	$actual_epenses = null;
	
	if (array_var($_GET, 'expense_ids')) {
		// if user has selected some timeslots
		$expense_ids = explode(',', array_var($_GET, 'expense_ids'));
		
		$ids = array();
		foreach ($expense_ids as $id) {
			if (is_numeric($id)) $ids[] = $id;
		}
		
		$actual_epenses = array();
		if (count($ids) > 0) {
			$actual_epenses = PaymentReceipts::instance()->findAll(array('conditions' => "id IN (" . implode(',', $ids) . ")"));
		}
		
	} else if (array_var($_GET, 'all_expenses')) {
		
		$client_ot = ObjectTypes::findByName('customer');
		$project_ot = ObjectTypes::findByName('project');
		$client_ot_id = $client_ot instanceof ObjectType ? $client_ot->getId() : 0;
		$project_ot_id = $project_ot instanceof ObjectType ? $project_ot->getId() : 0;
		
		// if no timeslots selected, check if any client or project is selected
		$active_context = active_context();
		
		$client_or_project_ids = array();
		foreach ($active_context as $selection) {
			if ($selection instanceof Member) {
				if (in_array($selection->getObjectTypeId(), array($client_ot_id, $project_ot_id))) {
					$client_or_project_ids[] = $selection->getId();
				}
			}
		}
		
		// We must process the timesltos if client or project is selected or if list is being filtered by an user
		$show_error_message = count($client_or_project_ids) == 0 && !array_var($_REQUEST, 'pr_user_filter');
		
		if ($show_error_message) {
			
			flash_error(lang('you need to be in a client or a project in order to perform this action'));
			ajx_current("empty");
			return;
			
		} else {
			
			// override some of the time list request variables
			// the other parameters sent in the $_REQUEST variable are the time grid filters
			$_REQUEST['limit'] = 999999999;
			
			$extra_conditions = '';
			if (Plugins::instance()->isActivePlugin('income')) {
				$extra_conditions = " AND invoicing_status='pending'";
			}
			
			// get the same timeslots that the list is showing, with the limit increased and invoicing_status=pending
			$pr_controller = new PaymentReceiptController();
			$actual_epenses = $pr_controller->list_all(true, $extra_conditions);
			
		}
		
	} else {
		return;
	}
	
	return $actual_epenses;
}

function get_number_invoice_expense(PaymentReceipt $payment_receipt){
	$act_exp_obj_type = ObjectTypes::findByName('payment_receipt');
	$project_id_cp = CustomProperties::getCustomPropertyByCode($act_exp_obj_type->getId(), "invoice_number");
			
	$value = '';
	if ($project_id_cp instanceof CustomProperty) {
		$cpval = CustomPropertyValues::getCustomPropertyValue($payment_receipt->getId(), $project_id_cp->getId());
		if ($cpval instanceof CustomPropertyValue) {
			$value = $cpval->getValue();
		}
	}

	return $value;
}

function recalculateFinancialsForRemovedTask($task) {
	if (Plugins::instance()->isActivePlugin('advanced_billing')) {
		if ($task instanceof ProjectTask) {
			Env::useHelper('functions', 'advanced_billing');
			calculate_task_estimated_and_executed_financials($task);
		}
		Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
		update_estimated_labor_info_for_member($task);
		update_executed_labor_info_for_member($task);
	}
}

function set_related_actual_expenses_to_non_billable($expense) {
	if (Plugins::instance()->isActivePlugin('advanced_billing') && Plugins::instance()->isActivePlugin('income')) {
		if ($expense instanceof Expense) {
			set_actual_expenses_related_to_bud_expense_to_non_billable($expense);
		}
	}
}

function set_actual_expenses_related_to_bud_expense_to_non_billable($expense) {
    DB::execute("
        UPDATE ".TABLE_PREFIX."payment_receipts 
        SET `invoicing_status`='non_billable', 
            `is_billable`=0, 
            `unit_price`=0, 
            `total_price`=0
        WHERE `expense_id` = ".$expense->getObjectId().";"
    );
}
