<?php 

/**
 * Get estimated expenses info
 *
 */

function update_estimated_expenses_info_for_member($object) {
	if (Plugins::instance()->isActivePlugin('advanced_billing')) {
		// sometimes we don't want to calculate these values, example: in task templates they are calculated at the end 
		// so the first times we get into this function are not relevant because object still has not its complete data
		// in that case we set this variable to skip these calculations and improve performance
		if (isset($object->dont_calculate_project_financials) && $object->dont_calculate_project_financials) return;
	
		Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
		$project_member_id = get_object_project_member_id($object);
		if($project_member_id > 0) {
			update_estimated_expenses_info_for_member_using_member_id($project_member_id);
		}
	}
}

function update_estimated_expenses_info_for_member_using_member_id($project_member_id, $params = array()) {
    $calculate_parents = array_var($params, 'calculate_parents', true);
    $expenses_info = get_estimated_expense_info_for_member($project_member_id);
    set_new_member_totals_values($project_member_id, $expenses_info);
    update_estimated_overall_totals_for_member($project_member_id);
    $project_member = Members::instance()->findById($project_member_id);
    if($project_member instanceof Member){
        $parent_id = $project_member->getParentMemberId();
        if($parent_id > 0 && $calculate_parents){
            update_estimated_expenses_info_for_member_using_member_id($parent_id);
        }
    }
}

function get_estimated_expense_info_for_member($project_member_id) {
    $expenses_info = array(
        'estimated_expense_billable_price' => 0,
        'estimated_expense_non_billable_price' => 0,
        'estimated_expense_total_price' => 0,
        'estimated_expense_billable_cost' => 0,
        'estimated_expense_non_billable_cost' => 0,
        'estimated_expense_total_cost' => 0
    );

    if (Plugins::instance()->isActivePlugin('advanced_billing')) {
		Env::useHelper('member_financials_functions', 'advanced_billing');
		// Get price budgeted expenses info
		$manual_earned_value_task_ids = get_manual_financial_value_task_ids($project_member_id);
		$price_extra_conditions = !empty($manual_earned_value_task_ids) ? ' AND `task_id` NOT IN ('.implode(',', $manual_earned_value_task_ids).')' : '';
		$price_billable_expenses_info =  get_expenses_budgeted_financials_for_member($project_member_id, ' AND `is_billable`=1 '.$price_extra_conditions);
		$price_non_billable_expenses_info =  get_expenses_budgeted_financials_for_member($project_member_id, ' AND `is_billable`=0 '.$price_extra_conditions);

		// Get cost budgeted expenses info
		$cost_billable_expenses_info =  get_expenses_budgeted_financials_for_member($project_member_id, ' AND `is_billable`=1 ');
		$cost_non_billable_expenses_info =  get_expenses_budgeted_financials_for_member($project_member_id, ' AND `is_billable`=0 ');

		$billable_expenses_info = array('budgeted_price' => $price_billable_expenses_info['budgeted_price'], 'budgeted_cost' => $cost_billable_expenses_info['budgeted_cost']);
		$non_billable_expenses_info = array('budgeted_price' => $price_non_billable_expenses_info['budgeted_price'], 'budgeted_cost' => $cost_non_billable_expenses_info['budgeted_cost']);


		calculate_estimated_expenses_values($expenses_info, $billable_expenses_info, $non_billable_expenses_info);
	}

    return $expenses_info;
}

function calculate_estimated_expenses_values(&$expenses_info, $billable_expenses,$non_billable_expenses) {
     // Calculate estimated expense billable price
     $expenses_info['estimated_expense_billable_price'] += $billable_expenses['budgeted_price'] ? $billable_expenses['budgeted_price'] : 0;
     // Calculate estimated expense non-billable price
     $expenses_info['estimated_expense_non_billable_price'] += $non_billable_expenses['budgeted_price'] ? $non_billable_expenses['budgeted_price'] : 0;
     // Calculate estimated total price
     $expenses_info['estimated_expense_total_price'] += $billable_expenses['budgeted_price'] ? $billable_expenses['budgeted_price'] : 0;
     $expenses_info['estimated_expense_total_price'] += $non_billable_expenses['budgeted_price'] ? $non_billable_expenses['budgeted_price'] : 0;
 
      // Calculate estimated expense billable cost
      $expenses_info['estimated_expense_billable_cost'] += $billable_expenses['budgeted_cost'] ? $billable_expenses['budgeted_cost'] : 0;
      // Calculate estimated expense non-billable cost
      $expenses_info['estimated_expense_non_billable_cost'] += $non_billable_expenses['budgeted_cost'] ? $non_billable_expenses['budgeted_cost'] : 0;
      // Calculate estimated total cost
      $expenses_info['estimated_expense_total_cost'] += $billable_expenses['budgeted_cost'] ? $billable_expenses['budgeted_cost'] : 0;
      $expenses_info['estimated_expense_total_cost'] += $non_billable_expenses['budgeted_cost'] ? $non_billable_expenses['budgeted_cost'] : 0;
} 

/**
 * Get executed expenses info
 *
 */

function update_executed_expenses_info_for_member($object) {
	if (Plugins::instance()->isActivePlugin('advanced_billing')) {
		// sometimes we don't want to calculate these values, example: in task templates they are calculated at the end 
		// so the first times we get into this function are not relevant because object still has not its complete data
		// in that case we set this variable to skip these calculations and improve performance
		if (isset($object->dont_calculate_project_financials) && $object->dont_calculate_project_financials) return;
		
		Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
		$project_member_id = get_object_project_member_id($object);
		if($project_member_id > 0) {
			update_executed_expenses_info_for_member_by_using_member_id($project_member_id);
		}
	}
}

function update_executed_expenses_info_for_member_by_using_member_id($project_member_id, $params = array()) {
    $calculate_parents = array_var($params, 'calculate_parents', true);
    $expenses_info = get_executed_expense_info_for_member($project_member_id);
    set_new_member_totals_values($project_member_id, $expenses_info);
    update_executed_overall_totals_for_member($project_member_id);
    $project_member = Members::instance()->findById($project_member_id);
    if($project_member instanceof Member){
        $parent_id = $project_member->getParentMemberId();
        if($parent_id > 0 && $calculate_parents){
            update_executed_expenses_info_for_member_by_using_member_id($parent_id);
        }
    }
}

function get_executed_expense_info_for_member($project_member_id) {
    $expenses_info = array(
        'executed_expense_billable_price' => 0,
        'executed_expense_non_billable_price' => 0,
        'executed_expense_total_price' => 0,
        'executed_expense_billable_cost' => 0,
        'executed_expense_non_billable_cost' => 0,
        'executed_expense_total_cost' => 0
    );
    
	if (Plugins::instance()->isActivePlugin('advanced_billing')) {
		$executed_values = true;
		$manual_earned_value_task_ids = get_manual_financial_value_task_ids($project_member_id, 'is_calculated_estimated_price', $executed_values);

		$extra_conditions = !empty($manual_earned_value_task_ids) ? ' AND `task_id` NOT IN ('.implode(',', $manual_earned_value_task_ids).')' : '';
		$billable_cost = get_executed_expenses_totals_for_member($project_member_id, 'cost', ' AND `is_billable`=1');
		$billable_price = get_executed_expenses_totals_for_member($project_member_id, 'price', ' AND `is_billable`=1' . $extra_conditions);
		$billable_expenses_info = array('total_executed_cost' => $billable_cost[0]['total_executed_cost'], 'total_executed_price' => $billable_price[0]['total_executed_price']);
		$non_billable_cost = get_executed_expenses_totals_for_member($project_member_id, 'cost', ' AND `is_billable`=0');
		$non_billable_price = get_executed_expenses_totals_for_member($project_member_id, 'price', ' AND `is_billable`=0' . $extra_conditions);
		$non_billable_expenses_info = array('total_executed_cost' => $non_billable_cost[0]['total_executed_cost'], 'total_executed_price' => $non_billable_price[0]['total_executed_price']);

		calculate_executed_expenses_values($expenses_info, $billable_expenses_info, $non_billable_expenses_info);
	}
	
    return $expenses_info;
}

function calculate_executed_expenses_values(&$expenses_info, $billable_expenses,$non_billable_expenses) {
     // Calculate executed expense billable price
     $expenses_info['executed_expense_billable_price'] += $billable_expenses['total_executed_price'] ? $billable_expenses['total_executed_price'] : 0;
     // Calculate executed expense non-billable price
     $expenses_info['executed_expense_non_billable_price'] += $non_billable_expenses['total_executed_price'] ? $non_billable_expenses['total_executed_price'] : 0;
     // Calculate executed total price
     $expenses_info['executed_expense_total_price'] += $billable_expenses['total_executed_price'] ? $billable_expenses['total_executed_price'] : 0;
     $expenses_info['executed_expense_total_price'] += $non_billable_expenses['total_executed_price'] ? $non_billable_expenses['total_executed_price'] : 0;
 
      // Calculate executed expense billable cost
      $expenses_info['executed_expense_billable_cost'] += $billable_expenses['total_executed_cost'] ? $billable_expenses['total_executed_cost'] : 0;
      // Calculate executed expense non-billable cost
      $expenses_info['executed_expense_non_billable_cost'] += $non_billable_expenses['total_executed_cost'] ? $non_billable_expenses['total_executed_cost'] : 0;
      // Calculate executed total cost
      $expenses_info['executed_expense_total_cost'] += $billable_expenses['total_executed_cost'] ? $billable_expenses['total_executed_cost'] : 0;
      $expenses_info['executed_expense_total_cost'] += $non_billable_expenses['total_executed_cost'] ? $non_billable_expenses['total_executed_cost'] : 0;
} 

function get_executed_expenses_totals_for_member($member_id, $type, $extra_conditions = ''){
    if($type == 'cost') {
        $executed_cost = "SUM(amount) AS total_executed_cost ";
        $select_columns = array("is_billable", "amount");
        $query_wraper_start = "SELECT $executed_cost FROM (";
    } else if($type == 'price') {
        $executed_price = "SUM(total_price) AS total_executed_price ";
        $select_columns = array("is_billable", "total_price");
        $query_wraper_start = "SELECT $executed_price FROM (";
    } else if($type == 'both') {
        $executed_cost = "SUM(amount) AS total_executed_cost ";
        $executed_price = "SUM(total_price) AS total_executed_price ";
        $select_columns = array("is_billable", "amount", "total_price");
        $query_wraper_start = "SELECT $executed_cost, $executed_price FROM (";  
    }

    $expenses_info = PaymentReceipts::instance()->listing(array(
        "select_columns" => $select_columns,
        "query_wraper_start" => $query_wraper_start,
        "query_wraper_end" => ") AS pending_calc",
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        "check_sharing_table" => false,
        "count_results" => false,
        "member_ids" => array($member_id),
        "extra_conditions" => $extra_conditions
    ))->objects; 

    return $expenses_info;
}

function get_manual_financial_value_task_ids($project_member_id, $financial_value = 'is_calculated_estimated_price', $executed_values = false, $use_context = true) { 

	if (!Plugins::instance()->isActivePlugin('advanced_billing')) return array();
	
      // Get task's earned value with manual estimated price
	$task_ids = "GROUP_CONCAT(task_id) AS task_ids";
    $additional_condition = '';
    if($financial_value == 'is_calculated_estimated_price' && $executed_values) {
        $additional_condition = ' AND `is_fixed_fee`=1 ';
    }

    $task_listing_configs = array(
        "select_columns" => array("o.id as task_id", "$financial_value", "is_fixed_fee"),
        "query_wraper_start" => "SELECT $task_ids FROM (",
        "query_wraper_end" => ") AS pending_calc",
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        "check_sharing_table" => false,
        "count_results" => false,
        "member_ids" => array($project_member_id),
        "extra_conditions" => " AND `$financial_value`=0 " . $additional_condition
    );

    if($use_context) {
        // Remove keys that are not needed when we are using context
        unset($task_listing_configs['ignore_context']);
        unset($task_listing_configs['member_ids']);
    }

    $tasks_info = ProjectTasks::instance()->listing($task_listing_configs)->objects; 

	$all_task_ids = array();
    $task_ids = explode(',', $tasks_info[0]['task_ids']);
	$task_ids = array_filter($task_ids);
	if (count($task_ids) > 0) {

		$all_subtasks_ids = ProjectTasks::getAllSubtasksIdsBulk($task_ids);
		$all_task_ids = array_merge($task_ids, $all_subtasks_ids);
		$all_task_ids = array_unique($all_task_ids);
		$task_ids_string = implode(',', $all_task_ids);
	}

    return $all_task_ids;
}