<?php
	class Expense extends BaseExpense {
		
		private $totals = null;
		
		function getArrayInfo($include_dates = false, $formatted_amounts = false) {
			$info = array();
			Env::useHelper('format');
			
			$columns = $this->getColumns();
			foreach ($columns as $col) {
				$info[$col] = $this->getColumnValue($col);
			}
			$info['name'] = $this->getObjectName();
			$info['object_id'] = $this->getId();
			
			$status = ExpensesStatuses::instance()->getStatus($this->getStatusId());
			$info['status'] = $status instanceof ExpensesStatus ? ($status->getName() == '' ? lang($status->getCode()) : $status->getName()) : '';
			$info['status_code'] = $status instanceof ExpensesStatus ? $status->getCode() : '';
			
			$current_totals = $this->getExpenseTotals();
			
			$currency = Currencies::instance()->getCurrency(array_var($info, 'currency_id'));
			
			$info['currency_sym'] = $currency instanceof Currency ? $currency->getSymbol() : config_option('currency_code');

			$info['task_name'] = '';
			$task = $this->getTask();
			if ($task) {
				$info['task_name'] = clean($task->getName());
			}
			
			If ($current_totals != null) {
    			foreach ($current_totals as $k => $current_total) {
    				if (!isset($info[$k])) {
    					if (in_array($k, array('expense_id','currency_id'))) {
    						$info[$k] = $current_total;
    					} else {
    						if ($formatted_amounts) {
    							$info[$k] = format_money_amount($current_total, $info['currency_sym'], config_option('product_type_decimals'));
    						} else {
    							$info[$k] = str_replace(',', '', number_format($current_total, config_option('product_type_decimals')));
    						}
    					}
    				}
    			}
    		}
			
			if ($include_dates) {
				$info['date'] = '';
				$rows = DB::executeAll("select min(`date`) as mindate, max(`date`) as maxdate from ".TABLE_PREFIX."expense_items 
						where expense_id=".$this->getId()." AND `date`!='".EMPTY_DATETIME."'");
				
				if (count($rows) > 0) {
					$dates = $rows[0];
					$mind = DateTimeValueLib::makeFromString($dates['mindate']);
					$maxd = DateTimeValueLib::makeFromString($dates['maxdate']);
					if ($dates['mindate'] == $dates['maxdate']) {
						if ($mind instanceof DateTimeValue) {
							$info['date'] = $mind->format(user_config_option('date_format'));
						}
					} else {
						if ($mind instanceof DateTimeValue && $maxd instanceof DateTimeValue) {
							$info['date'] = $mind->format(user_config_option('date_format')) .' '.lang('date_to').' '. $maxd->format(user_config_option('date_format'));
						}
					}
				}
			}
			
			Hook::fire('modify_object_get_array_info', array('object' => $this), $info);
			
			return $info;
		}

		function getExpenseTotals() {

			if (is_null($this->totals)) {
				$this->totals = DB::executeOne("
						SELECT * FROM ".TABLE_PREFIX."expense_totals 
						WHERE expense_id=".$this->getId()." AND status_id='".$this->getStatusId()."'"
				);
			}

			return $this->totals;
		}

		function getTask() {
			$task = null;

			if ($this->getTaskId() > 0) {
				$task = ProjectTasks::instance()->findById($this->getTaskId());
			}

			return $task;
		}

		function recalculateFinancialsForAssociatedTask() {
			if (Plugins::instance()->isActivePlugin('advanced_billing')) {
				$task = $this->getTask();
				if ($task instanceof ProjectTask) {
					Env::useHelper('functions', 'advanced_billing');
					calculate_task_estimated_and_executed_financials($task);
				}
				Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
				update_estimated_labor_info_for_member($task);
				update_executed_labor_info_for_member($task);
			}
		}

		
		function recalculateInvoicingStatus() {
			if ($this->columnExists('invoicing_status')) {
				$payment_receipts = PaymentReceipts::instance()->findAll(array("conditions"=>"expense_id=".$this->getId()));
				$count_pending = 0;
				$count_invoiced = 0;
				
				foreach ($payment_receipts as $p) {
					if ($p->getColumnValue('invoicing_status') == 'pending') $count_pending++;
					else if ($p->getColumnValue('invoicing_status') == 'invoiced') $count_invoiced++;
				}
				
				if ($count_invoiced > 0 && $count_pending > 0) {
					$inv_status = 'partially_invoiced';
				} else if ($count_invoiced > 0) {
					$inv_status = 'invoiced';
				} else if ($count_pending > 0) {
					$inv_status = 'pending';
				}
				$this->setColumnValue('invoicing_status', $inv_status);
				$this->save();
			}
		}

		function updateCalculatedTotals() {
		
			$values = array();
			$total_rows = array();
			
			if ($this->getTrashedById() == 0) {
				$total_rows = DB::executeAll("
					SELECT p.currency_id,
						SUM(p.unit_cost) as unit_cost,
						SUM(p.total_cost) as total_cost,
						SUM(p.unit_price) as unit_price,
						SUM(p.total_price) as total_price
					FROM ".TABLE_PREFIX."expense_items p
					WHERE p.expense_id=".$this->getId()."
					GROUP BY p.currency_id;
				");
			}
			
			if (!is_array($total_rows)) {
			    $total_rows = array();
			}
			
			foreach ($total_rows as $tr) {
				$st_id = $this->getStatusId();
				$c_id = $tr['currency_id'];
				$unit_cost = $tr['unit_cost'];
				$total_cost = $tr['total_cost'];
				$unit_price = $tr['unit_price'];
				$total_price = $tr['total_price'];
				
				$values[] = "(".$this->getId().", $st_id, $c_id, $unit_cost, $total_cost, $unit_price, $total_price)";
			}
			
			DB::execute("DELETE FROM ".TABLE_PREFIX."expense_totals WHERE expense_id=".$this->getId());
			if (count($values) > 0) {
				DB::execute("INSERT INTO ".TABLE_PREFIX."expense_totals VALUES ".implode(',',$values)." ON DUPLICATE KEY UPDATE expense_id=expense_id;");
			}
		
		}
		
		
		
		function delete() {
			if ($this->canDelete(logged_user())) {
				$items = ExpenseItems::instance()->findAll(array("conditions" => "expense_id = ".$this->getId()));
				foreach ($items as $p) {
					$p->delete();
				}
				$this->addToHistory(logged_user(), 'delete');
				parent::delete();
			}
		}
		
		function trash($trashDate = null, $fire_hook = true) {
			if ($this->canDelete(logged_user())) {
				$items = ExpenseItems::instance()->findAll(array("conditions" => "expense_id = ".$this->getId()));
				foreach ($items as $p) {
					$p->trash();
				}
				$this->addToHistory(logged_user(), 'delete');
				parent::trash();
			}
		}
		
		function untrash($fire_hook = true) {
			if ($this->canDelete(logged_user())) {
				$items = ExpenseItems::instance()->findAll(array("conditions" => "expense_id = ".$this->getId()));
				foreach ($items as $p) {
					$p->untrash();
				}
				$this->addToHistory(logged_user(), 'untrash');
				parent::untrash();
			}
		}
		
		
		function getPaymentsFromHistory() {
			
			$current_status = null;
			$current_status_tmp = ExpensesStatuses::instance()->findById($this->getStatusId());
			if (str_starts_with($current_status_tmp->getCode(), "part_")) {
				$st_code = str_replace("part_", "", $current_status_tmp->getCode());
				$current_status = ExpensesStatuses::instance()->findOne(array('conditions' => "code='$st_code'"));
			} else {
				$current_status = $current_status_tmp;
			}
			if (!$current_status instanceof ExpensesStatus) {
				return array();
			}
			
			$statuses = ExpensesStatuses::instance()->getAllStatusInfo();
			$all_rows = array();
			foreach ($statuses as $st) {
				if ($st['only_for_expenses']) continue;
				
				$exec_cond = "";
				if ($st['code'] != 'executed') $exec_cond = "AND ph.status_id=x.status_id";
				
				// payment history status must not be greater than the expense's status
				$exp_status_condition = "AND (SELECT pst.order FROM ".TABLE_PREFIX."expenses_status pst WHERE pst.id=ph.status_id) 
						<= (SELECT est.order FROM ".TABLE_PREFIX."expenses_status est WHERE est.id=".$current_status->getId().")";
				
				$sql = "
					SELECT * FROM ".TABLE_PREFIX."payment_history ph
					WHERE
						ph.expense_id=".$this->getId()."
						$exp_status_condition
						AND ph.status_id = ".$st['id']."
						AND ph.modification_log in ('add', 'edit')
						AND EXISTS (SELECT pp.expense_id FROM ".TABLE_PREFIX."payments pp 
								INNER JOIN ".TABLE_PREFIX."objects po ON po.id=pp.object_id
								WHERE pp.expense_id=ph.expense_id AND pp.payment_num=ph.payment_num AND po.trashed_by_id=0
								LIMIT 1)
						
						AND ph.modification_date = (
							SELECT max(x.modification_date) FROM ".TABLE_PREFIX."payment_history x 
							WHERE x.expense_id=ph.expense_id AND x.payment_num=ph.payment_num 
							$exec_cond
						);
				";
				
				$rows = DB::executeAll($sql);
				
				if (is_array($rows)) {
					$all_rows = array_merge($all_rows, $rows);
				}
			}
			
			$phs = array();
			foreach ($all_rows as $row) {
				$phs[] = PaymentHistories::instance()->loadFromRow($row);
			}
			
			return $phs;
		}
		
		
		function addToHistory($user, $log_data) {
						
			$history = new ExpensesHistory();
			
			$history->setObjectId($this->getId());
			$history->setName($this->getName());
			$history->setDescription($this->getDescription());
			$history->setStatusId($this->getStatusId());
			$status = ExpensesStatuses::instance()->findById($this->getStatusId());
			//This failed 
			if ($status != null) {
			    $history->setStatusName($status->getCode());
			}
			$history->setDate($this->getDate());
			
			$history->setUserId($user->getId());
			$history->setModificationDate(DateTimeValueLib::now());
			$history->setModificationLog($log_data);
			
			$history->save();
			
			$this->updateCalculatedTotals();
		}
		
		
		function save($update_totals=true) {
			parent::save();
			if ($update_totals) {
				$this->updateCalculatedTotals();
			}
		}
		
		
		function refreshStatus() {
			/*
			$status_count = array();
			$all_payments = Payments::instance()->findAll(array('conditions' => 'expense_id = '.$this->getId()));
			
			foreach ($all_payments as $p) {
				if (!isset($status_count[$p->getStatusId()])) $status_count[$p->getStatusId()] = 0;
				$status_count[$p->getStatusId()] = $status_count[$p->getStatusId()] + 1;
			}
			
			if (count($status_count) > 1) {
				$part_app = ExpensesStatuses::instance()->getStatusByCode('part_approved');
				$st_app = ExpensesStatuses::instance()->getStatusByCode('approved');
				$part_exe = ExpensesStatuses::instance()->getStatusByCode('part_executed');
				$st_exe = ExpensesStatuses::instance()->getStatusByCode('executed');
				
				if (array_var($status_count, $st_exe->getId()) > 0) {
					$this->setStatusId($part_exe->getId());
				} else if (array_var($status_count, $st_app->getId()) > 0) {
					$this->setStatusId($part_app->getId());
				}
				
				$this->save();
				$this->addToHistory(logged_user(), 'edit');
				
			} else if (count($status_count) == 1) {
				$keys = array_keys($status_count);
				$this->setStatusId($keys[0]);
				$this->save();
				$this->addToHistory(logged_user(), 'edit');
			}
			*/
		}
		
		
		
		function getChangedRelations($old_content_object) {
			$changed_relations = array();

			// task
			if ($this->getTaskId() != $old_content_object->getTaskId()) {

				// log the relation added with the new task
				if ($this->getTaskId() > 0) {
					$changed_relations['relation_added'][] = $this->getTaskId();
				}
				// log the removed relation with the old task
				if ($old_content_object->getTaskId() > 0) {
					$changed_relations['relation_removed'][] = $old_content_object->getTaskId();
				}

			} else if ($this->getTaskId() > 0) {
				// only log that the related object to the task has been edited
				$relation_key = 'relation_edited';

				// if the object has been trashed or untrashed then the relation is added or removed
				if ($this->isTrashed() && !$old_content_object->isTrashed()) { // sent to trash
					$relation_key = 'relation_removed';
				} else if (!$this->isTrashed() && $old_content_object->isTrashed()) { // restored from trash
					$relation_key = 'relation_added';
				}

				$changed_relations[$relation_key][] = $this->getTaskId();
			}

			return $changed_relations;
		}

	
		
		
		static function canAdd(Contact $user, $context, &$notAllowedMember = ''){
			return can_add($user, $context, Expenses::instance()->getObjectTypeId(), $notAllowedMember);
		}
		
		/**
		 * Returns true if $user can access this message
		 *
		 * @param Contact $user
		 * @return boolean
		 */
		function canView(Contact $user) {
			return can_read($user, $this->getMembers(), $this->getObjectTypeId());
		} // canView
	
		/**
		 * Check if specific user can edit this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canEdit(Contact $user) {
			$status = ExpensesStatuses::instance()->findById($this->getStatusId());
			$final_st = ExpensesStatuses::getFinalStatus();
			$final_status_code = $final_st->getCode();
			/*
			$has_completed_payments = Payments::instance()->count("expense_id=".$this->getId()." AND status_id=".$final_st->getId()) > 0;
			if (!$status instanceof ExpensesStatus || $status->getCode() == $final_status_code || $status->getCode() == 'part_'.$final_status_code || $has_completed_payments) {
				return can_edit_executed_payments_pg_array($user->getPermissionGroupIds());
			}*/
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canEdit
	
	
		/**
		 * Check if specific user can delete this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canDelete(Contact $user) {
			$status = ExpensesStatuses::instance()->findById($this->getStatusId());
			if (!$status instanceof ExpensesStatus || $status->getCode() == 'executed' || $status->getCode() == 'part_executed') {
				return false;
			}
			return can_delete($user,$this->getMembers(), $this->getObjectTypeId());
		} // canDelete
	
		/**
		 * Check if specific user can comment this message
		 *
		 * @access public
		 * @param void
		 * @return boolean
		 */
		function canAddComment(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canAddComment
	}