<?php 

class Expenses extends BaseExpenses {
	function __construct() {
		parent::__construct();
		$this->object_type_name = 'expense';
	}
	
	private $cached_expenses = null;
	
	function getExpense($id) {
		if (is_null($this->cached_expenses)) $this->cached_expenses = array();
		
		$exp = array_var($this->cached_expenses, $id);
		if (!$exp instanceof Expense) {
			$exp = $this->findById($id);
			$this->cached_expenses[$id] = $exp;
		}
		
		return $exp;
	}

	function getColumnsToAggregateInTotals() {
    	return array_merge(parent::getColumnsToAggregateInTotals(), array(
    		'total_cost' => array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'currency_id', 'group_by' => 'currency_id'),
    		'total_price' => array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'currency_id', 'group_by' => 'currency_id')
    	));
	}

	function getExternalColumnValue($field, $id, $manager=null, $object=null) {
		$value = '';
		if ($field == 'task_id') {
			$task = ProjectTasks::instance()->findById($id);
			if ($task) $value = clean($task->getName());

		} else if (in_array($field, array('total_cost','total_price'))) {
			
			$amount_decimals = config_option('product_type_decimals');
			$currency = Currencies::instance()->getCurrency($object->getCurrencyId());
			$c_sym = $currency instanceof Currency ? $currency->getSymbol() : '$';
			$totals = $object->getExpenseTotals();
			$value = format_money_amount(array_var($totals, $field), $c_sym, $amount_decimals);
			
		}
		return $value;
	}

	static function getBudgetedExpensesByTask($task_id) {
		$expenses = null;

		if ($task_id > 0) {
			$expenses = self::instance()->findAll(array("conditions" => array("task_id = ? and trashed_by_id=0", $task_id)));
		}

		return $expenses;
	}

	static function getBudgetedExpensesTotalsByTasks($task_ids) {
		$totals = array('total_cost' => 0, 'total_price' => 0);

		$task_ids = array_filter($task_ids, 'is_numeric');
		if (count($task_ids) > 0) {
			$totals = DB::executeOne("
				SELECT SUM(total_cost) AS total_cost, SUM(total_price) AS total_price 
				FROM ".TABLE_PREFIX."expense_items i
				INNER JOIN ".TABLE_PREFIX."expenses e ON e.object_id=i.expense_id
				INNER JOIN ".TABLE_PREFIX."objects o ON o.id=e.object_id
				WHERE e.task_id IN (". implode(',', $task_ids) .")
					AND o.trashed_by_id=0
			");
		}

		return $totals;
	}
}
