<?php
	class PaymentReceipt extends BasePaymentReceipt {
		
		
		function getPaidByName()
		{
			if($this->getPaidById() <= 0 )
			{
				return '';
			}

			$contact = Contacts::instance()->findById($this->getPaidById());
			
			if(!($contact instanceof Contact))
			{
				return '';
			}

			return $contact->getObjectName();
		}

		function getTask() {
			$task = null;

			if ($this->getTaskId() > 0) {
				$task = ProjectTasks::instance()->findById($this->getTaskId());
			}

			return $task;
		}

		function recalculateFinancialsForAssociatedTask() {
			if (Plugins::instance()->isActivePlugin('advanced_billing')) {
				$task = $this->getTask();
				if ($task instanceof ProjectTask) {
					Env::useHelper('functions', 'advanced_billing');
					calculate_task_estimated_and_executed_financials($task);
					Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
					update_estimated_labor_info_for_member($task);
					update_executed_labor_info_for_member($task);
				}
			}
		}

		function getArrayInfo($format_the_data = false, $get_file_content = false) {
			$info = array();
			Env::useHelper('format');
				
			$columns = $this->getColumns();
			foreach ($columns as $col) {
				$info[$col] = $this->getColumnValue($col);
				if (in_array($this->manager()->getColumnType($col), array(DATA_TYPE_DATETIME, DATA_TYPE_DATE)) && $this->getColumnValue($col) instanceof DateTimeValue) {
					if ($this->getColumnValue($col)->getTimestamp() > 0) {
						$info[$col] = format_date($this->getColumnValue($col), null, 0);
						$info[$col.'Timestamp'] = $this->getColumnValue($col)->getTimestamp();
					} else {
						$info[$col] = '';
						$info[$col.'Timestamp'] = 0;
					}
				}
			}
			$info['name'] = $this->getObjectName();
			$info['object_id'] = $this->getId();
			
			$info['document_url'] = get_url('files', 'get_public_file', array('id' => $this->getDocumentId()));
			$currency = Currencies::instance()->getCurrency($this->getCurrencyId());
			$symbol = $currency ? $currency->getSymbol() : '$';
			$info['currency_sym'] = $symbol;

			if($this->getPaidById() > 0 ){
				$contact = Contacts::instance()->findById($this->getPaidById());
				if ($contact instanceof Contact) {
					$info['paid_by_name'] = $contact->getObjectName();
				} else {
					$info['paid_by_name'] = '';
				}
			} else {
				$info['paid_by_name'] = '';
			}
			
			$info['task_name'] = '';
			$task = $this->getTask();
			if ($task) {
				$info['task_name'] = clean($task->getName());
			}

			Env::useHelper("functions", "expenses2");
			$info['payment_term_name'] = actual_expense_get_payment_term_name($this->getPaymentTermId());

			if ($this->getProductTypeId() > 0) {
				$pt = ProductTypes::getById($this->getProductTypeId());
				$info["product_type_name"] = $pt ? $pt->getName() : lang("n/a");
			}

			if ($this->getExpenseId() > 0) {
				$expense = Expenses::instance()->findById($this->getExpenseId());
				if ($expense instanceof Expense) {
					$info['expense_name'] = $expense->getObjectName();
				}
				
				//Previously: WHERE expense_id=".$this->getExpenseId()." AND status_id=(SELECT id FROM ".TABLE_PREFIX."expenses_status WHERE code='budgeted')"
				$expense_cost_row = DB::executeOne("
					SELECT * FROM ".TABLE_PREFIX."expense_totals 
					WHERE expense_id=".$this->getExpenseId()
				);
				if ($expense_cost_row && count($expense_cost_row) > 0) {
					$expense_cost = $expense_cost_row['total_cost'];
					if ($this->getAmount() > $expense_cost) {
						$info['add_cls'] = 'warning';
					}
				}
			}
			if($get_file_content) {
				if ($this->getDocumentId() != '' && FileRepository::isInRepository($this->getDocumentId())) {
					$info['file_content'] = base64_encode(FileRepository::getFileContent($this->getDocumentId()));
				}
			}

			if ($this->manager()->columnExists('invoicing_status') && !$this->getIsBillable()) {
				$info['invoicing_status'] = 'non_billable';
			}

			if ($format_the_data) {
				$amount_cols = array('amount', 'unit_cost', 'unit_price', 'total_price', 'total_cost_without_taxes');
				foreach ($amount_cols as $amount_col) {
					$info[$amount_col] = format_money_amount($info[$amount_col], $symbol, config_option('product_type_decimals'));
				}
				
			}

			$info['expense_type_code'] = $info['expense_type'];
			$info['expense_type'] = lang($info['expense_type']);

			Hook::fire('additional_actual_expense_array_info_data', $this, $info);
			
			return $info;
		}
		
		
		function recalculateBudgetedExpenseInvoicingStatus() {
			$expense = Expenses::instance()->findById($this->getExpenseId());
			if ($expense instanceof Expense) {
				$expense->recalculateInvoicingStatus();
			}
		}
		
		
		function changeInvoicingStatus($status) {
			if (PaymentReceipts::instance()->columnExists('invoicing_status')) {
				// to use when saving the application log
				$old_content_object = $this->generateOldContentObjectData();

				$old_status = $this->getColumnValue('invoicing_status');
				if ($old_status == $status) {
					return;
				}
				
				// don't check workflow permissions here, because this action is not a direct action executed by the user, 
				// is triggered by other objects status change, like an invoice
				$this->override_workflow_permissions = true;

				$this->setColumnValue('invoicing_status', $status);
				if($status == 'pending') {
					$this->setColumnValue('invoice_id', 0);
				}
				$this->save();
				
				$ret = null;
				Hook::fire("after_change_object_inv_status", array('object' => $this, 'old_status' => $old_status), $ret);

				if (!isset($this->was_new) || !$this->was_new) {
					ApplicationLogs::createLog($this, ApplicationLogs::ACTION_EDIT, false, true);
				}
			}
		}
	

		function getChangedRelations($old_content_object) {
			$changed_relations = array();

			// expense
			if ($this->getExpenseId() != $old_content_object->getExpenseId()) {

				// log the relation added with the new expense
				if ($this->getExpenseId() > 0) {
					$changed_relations['relation_added'][] = $this->getExpenseId();
				}
				// log the removed relation with the old expense
				if ($old_content_object->getExpenseId() > 0) {
					$changed_relations['relation_removed'][] = $old_content_object->getExpenseId();
				}

			} else if ($this->getExpenseId() > 0) {
				// only log that the related object to the expense has been edited
				$relation_key = 'relation_edited';

				// if the object has been trashed or untrashed then the relation is added or removed
				if ($this->isTrashed() && !$old_content_object->isTrashed()) { // sent to trash
					$relation_key = 'relation_removed';
				} else if (!$this->isTrashed() && $old_content_object->isTrashed()) { // restored from trash
					$relation_key = 'relation_added';
				}

				$changed_relations[$relation_key][] = $this->getExpenseId();
			}

			// task
			if ($this->getTaskId() != $old_content_object->getTaskId()) {

				// log the relation added with the new task
				if ($this->getTaskId() > 0) {
					$changed_relations['relation_added'][] = $this->getTaskId();
				}
				// log the removed relation with the old task
				if ($old_content_object->getTaskId() > 0) {
					$changed_relations['relation_removed'][] = $old_content_object->getTaskId();
				}

			} else if ($this->getTaskId() > 0) {
				// only log that the related object to the task has been edited
				$relation_key = 'relation_edited';

				// if the object has been trashed or untrashed then the relation is added or removed
				if ($this->isTrashed() && !$old_content_object->isTrashed()) { // sent to trash
					$relation_key = 'relation_removed';
				} else if (!$this->isTrashed() && $old_content_object->isTrashed()) { // restored from trash
					$relation_key = 'relation_added';
				}

				$changed_relations[$relation_key][] = $this->getTaskId();
			}

			return $changed_relations;
		}

		
		function save() {
			$ok = parent::save();
			
			$this->recalculateBudgetedExpenseInvoicingStatus();
			
			return $ok;
		}
		
		
		function delete() {
			if ($this->canDelete(logged_user())) {
				//$this->addToHistory(logged_user(), 'delete');
				parent::delete();
			}
		}
		
		function trash($trashDate = NULL, $fire_hook = true) {
			if ($this->canDelete(logged_user())) {
				//$this->addToHistory(logged_user(), 'delete');
				parent::trash();
			}
		}
		
		function untrash($fire_hook = true) {
			if ($this->canDelete(logged_user())) {
				//$this->addToHistory(logged_user(), 'untrash');
				parent::untrash();
			}
		}
		
		
		static function canAdd(Contact $user, $context, &$notAllowedMember = ''){
			return can_add($user, $context, PaymentReceipts::instance()->getObjectTypeId(), $notAllowedMember);
		}
		
		/**
		 * Returns true if $user can access this message
		 *
		 * @param Contact $user
		 * @return boolean
		 */
		function canView(Contact $user) {
			$can_read = can_read($user, $this->getMembers(), $this->getObjectTypeId());
			if ($can_read && !SystemPermissions::userHasSystemPermission($user, 'can_see_expenses_of_others')) {
				$can_read = $this->getPaidById() == $user->getId();
			}
			return $can_read;
		} // canView
	
		/**
		 * Check if specific user can edit this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canEdit(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canEdit
	
	
		/**
		 * Check if specific user can delete this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canDelete(Contact $user) {
			
			

			return can_delete($user,$this->getMembers(), $this->getObjectTypeId());
		} // canDelete
	
		/**
		 * Check if specific user can comment this message
		 *
		 * @access public
		 * @param void
		 * @return boolean
		 */
		function canAddComment(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canAddComment
	}