<?php 
	class ProductTypes extends BaseProductTypes {
		
		static $product_types_cache = array();
		static function getById($id) {
			$m = array_var(self::$product_types_cache, $id);
			if (!$m instanceof ProductType) {
				$m = ProductTypes::instance()->findById($id);
				if ($m instanceof ProductType) {
					self::$product_types_cache[$id] = $m;
				}
			}
			return $m;
		}
		
		
		static function getAllProductTypesData($order = null) {
			$product_types_data = array();
			if (is_null($order)) {
				$order = 'name';
			}
			
			$product_types = ProductTypes::instance()->findAll(array('order' => $order));
			foreach ($product_types as $pt) {
				$product_types_data[] = $pt->getArrayInfo();
			}
			
			return $product_types_data;
		}
		
		static function getFilteredProductTypesData($members, $order = null, $return_objects = false) {
			$product_types_data = array();
			if (is_null($order)) {
				$order = 'name';
			}
			
			$product_type_dimensions = array_filter(config_option('product_type_dimensions'));
			$dimension_ids_that_filter_up = array_filter(config_option('product_type_filter_up_dimensions'));
			
			$allowed_members_to_filter = array();
			$associated_members_to_filter = array();
			
			$member_filter_conditions = array();
			
			foreach ($members as $member) {
				if ($member instanceof Member && in_array($member->getDimensionId(), $product_type_dimensions)) {
					
					$allowed_members_to_filter[$member->getDimensionId()] = $member;
					
					$mem_associations = DimensionMemberAssociations::getAssociatations($member->getDimensionId(), $member->getObjectTypeId());
					foreach ($mem_associations as $assoc) {
						/* @var $assoc DimensionMemberAssociation */
						if (in_array($assoc->getAssociatedDimensionMemberAssociationId(), $product_type_dimensions)) {
							$assoc_mem_ids_csv = MemberPropertyMembers::instance()->getAllPropertyMemberIds($assoc->getId(), $member->getId());
							
							if (trim($assoc_mem_ids_csv) != '') {
								$assoc_member = Members::instance()->findOne(array("conditions" => "id IN ($assoc_mem_ids_csv)"));
								
								if ($assoc_member instanceof Member) {
									$associated_members_to_filter[] = $assoc_member;
								}
							}
						}
					}
				}
			}
			
			foreach ($associated_members_to_filter as $assoc_member) {
				if (!isset($allowed_members_to_filter[$assoc_member->getDimensionId()])) {
					$allowed_members_to_filter[$assoc_member->getDimensionId()] = $assoc_member;
				}
			}
			
			foreach ($allowed_members_to_filter as $dim_id => $member) {
				/* @var $member Member */
				
				$main_member_ids = array($member->getId());
				if (in_array($dim_id, $dimension_ids_that_filter_up)) {
					$all_parent_members = $member->getAllParentMembersInHierarchy(false, false);
					foreach ($all_parent_members as $pm) {
						$main_member_ids[] = $pm->getId();
					}
				}
				
				// include the ones classified in child members too
				$all_child_members = $member->getAllChildrenIds(true);
				$main_member_ids = array_merge($main_member_ids, $all_child_members);
					
				$main_member_ids = array_filter($main_member_ids);
				$main_member_ids_csv = implode(',', $main_member_ids);
				
				$cond = "(
					EXISTS (
						SELECT * FROM ".TABLE_PREFIX."product_type_members ptm WHERE
							ptm.product_type_id = ".TABLE_PREFIX."product_types.id
							AND ptm.dimension_id='".$member->getDimensionId()."'
							AND ptm.member_id IN (".$main_member_ids_csv.")
							AND EXISTS(SELECT m.id FROM ".TABLE_PREFIX."members m WHERE m.id=ptm.member_id)
					)";
				
				if (in_array($dim_id, $dimension_ids_that_filter_up)) {
					
					$cond .= " OR NOT EXISTS (
						SELECT * FROM ".TABLE_PREFIX."product_type_members ptm WHERE
							ptm.product_type_id = ".TABLE_PREFIX."product_types.id
							AND ptm.dimension_id='".$member->getDimensionId()."'
							AND EXISTS(SELECT m.id FROM ".TABLE_PREFIX."members m WHERE m.id=ptm.member_id)
					)";
				}
				$cond .= ")";
				
				$member_filter_conditions[] = $cond;
			}
			
			$member_filter_conditions_sql = "";
			if (count($member_filter_conditions) > 0) {
				$member_filter_conditions_sql = implode(" AND ", $member_filter_conditions);
			}
				
			$product_types = ProductTypes::instance()->findAll(array('conditions' => $member_filter_conditions_sql, 'order' => $order));
			
			if ($return_objects) {
				return $product_types;
			}

			foreach ($product_types as $pt) {
				$product_types_data[] = $pt->getArrayInfo();
			}
				
			return $product_types_data;
		}
		
		
	}
 