<?php
if (SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_expenses_of_others')) {

	Env::useHelper('functions', 'expenses2');
	ini_set('memory_limit', '1G');
	$result = collect_expenses_and_payment_receipts_data_for_widget();
	
	$show_billing_info = true;
	Hook::fire('get_can_see_billing_information', array('user'=>logged_user()), $show_billing_info);
	
	$show_cost_info = true;
	Hook::fire('get_can_see_cost_information', array('user'=>logged_user()), $show_cost_info);
	
	$amounts_by_day = $result['amounts_by_day'];
	$expenses_totals = $result['expenses_totals'];
	
	// fill gaps with empty values
	$keys = array_keys($amounts_by_day);
	$first_key = array_var($keys, 0,"");
	$last_key = end($keys);
	$last_dt_val = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $last_key);
	$date_iter = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $first_key);
	
	// sort the amounts by date again
	ksort($amounts_by_day);
	
	// accumulate the amounts
	$last_budgeted_amount = 0;
	$last_actual_amount = 0;
	foreach ($amounts_by_day as &$amounts) {
		
		$amounts['budgeted'] += $last_budgeted_amount;
		$amounts['actual'] += $last_actual_amount;
		
		$last_budgeted_amount = $amounts['budgeted'];
		$last_actual_amount = $amounts['actual'];	
	}
	
	// Set currency symbol
	$default_currency = Currencies::getDefaultCurrencyInfo();
	$currency_symbol = !empty($default_currency) ? $default_currency['symbol'] : config_option('currency_code', '$');
	
	// Set widget title
	$widget_title = lang('expenses progress');

	// Set widget subtitle
	$include_subtitle =  array_var($result, 'include_subtitle', false);
	$subtitle_date_start =  array_var($result, 'subtitle_date_start', null);
	if($include_subtitle && $subtitle_date_start){
		$date_format = user_config_option('date_format');
		$subtitle_date_formatted = $subtitle_date_start->format($date_format);
		$widget_subtitle = lang('from') . ': ' . $subtitle_date_formatted;
	}
	
	// Show Expenses Progress, if expenses info exists
	if (count($amounts_by_day) > 0 && $show_billing_info && $show_cost_info){
		include 'template.php';
	}
}