/**
 *  ProductTypesManager
 *
 */
og.ProductTypesManager = function() {
	var actions;
	this.doNotRemove = true;
	this.needRefresh = false;
	
	og.eventManager.addListener('member changed', this.reset, this);
	
	this.fields = [
		'id', 'name', 'description', 'user_id', 'user_name', 'hourly_rate', 'hourly_cost', 'price_multiplier', 'uses_price_multiplier', 
		'updated_by_id', 'updated_by_name', 'updated_on', 'memPath', 'is_reimbursable'
   	];

	if (og.expenses2.more_product_type_columns && og.expenses2.more_product_type_columns.length > 0) {
		for (var i=0; i<og.expenses2.more_product_type_columns.length; i++) {
			this.fields.push(og.expenses2.more_product_type_columns[i]);
		}
	}
   	
	if (!og.ProductTypesManager.store) {
		og.ProductTypesManager.store = new Ext.data.Store({
			proxy: new og.GooProxy({
				url: og.getUrl('product_type', 'list_all')
			}),
			reader: new Ext.data.JsonReader({
				root: 'product_type',
				totalProperty: 'totalCount',
				id: 'id',
				fields: this.fields
			}),
			remoteSort: true,
			listeners: {
				'load': function(store, result) {
					var d = this.reader.jsonData;
					if (d.totalCount == 0) {
						var sel_context_names = og.contextManager.getActiveContextNames();
						if (sel_context_names.length > 0) {
							this.fireEvent('messageToShow', lang("no objects message", lang("product types"), sel_context_names.join(', ')));
						} else {
							this.fireEvent('messageToShow', lang("no more objects message", lang("product types")));
						}
					} else {
						this.fireEvent('messageToShow', "");
					}
					var cmp = Ext.getCmp('product-types-manager');
					if (cmp) {
						og.product_types_allowed_dimensions = d.allowed_dimensions;
						var sm = cmp.getSelectionModel();
						sm.clearSelections();
					}
					
					og.eventManager.fireEvent('replace all empty breadcrumb', null);
				}
			}
		});
		og.ProductTypesManager.store.setDefaultSort('date', 'desc');
	}
	this.store = og.ProductTypesManager.store;
	this.store.addListener({messageToShow: {fn: this.showMessage, scope: this}});

	function renderName(value, p, r) {
		
		var name = String.format('<a style="font-size:120%;" href="#" onclick="return false;">{0}</a>', og.clean(value));
		if (r.data.description != '') {
			name += '<span class="desc"> - '+ r.data.description +'</span>';
		}
		
		return name;
	}
	
	function renderIcon(value, p, r) {
		return '<div class="link-ico ico-billing"></div>';
	}
	
	function renderMoney(value, p, r) {
		return '<div class="money-cell">'+ value +'</div>';
	}

	function renderUser(value, p, r) {
		return r.data.user_name;
	}

	function renderReimbursable(value, p, r) {
		return r.data.is_reimbursable == 1 ? lang('yes') : lang('no');
	}

	function renderDate(value, p, r) {
		if (!value) return "";
		
		var userString = String.format('<a href="{1}" onclick="og.openLink(\'{1}\');return false;">{0}</a>', og.clean(r.data.updated_by_name), og.getUrl('contact', 'card', {id: r.data.updated_by_id}));
		
		return lang('last updated by at', userString, value);
	}

	function getSelectedIds() {
		var selections = sm.getSelections();
		if (selections.length <= 0) {
			return '';
		} else {
			var ret = '';
			for (var i=0; i < selections.length; i++) {
				ret += "," + selections[i].data.id;
			}
			return ret.substring(1);
		}
	}
	this.getSelectedIds = getSelectedIds;
	
	function getFirstSelectedId() {
		if (sm.hasSelection()) {
			return sm.getSelected().data.id;
		}
		return '';
	}

	var sm = new Ext.grid.CheckboxSelectionModel();
	sm.on('selectionchange',
		function() {
			var allUnread = true, allRead = true;
			var selections = sm.getSelections()
			for (var i=0; i < selections.length; i++) {
				if (selections[i].data.isRead){
					allUnread = false;
				} else {
					allRead = false;
				}
			}
			if (sm.getCount() <= 0) {
				
				actions.del.setDisabled(true);
				actions.edit.setDisabled(true);
				
			} else {
				
				actions.del.setDisabled(false);
				actions.edit.setDisabled(false);
				
			}
		});
	
	var cm_info = [
		sm,{
			id: 'icon',
			header: '&nbsp;',
			dataIndex: 'type',
			width: 28,
        	renderer: renderIcon,
        	fixed:true,
        	resizable: false,
        	hideable:false,
        	menuDisabled: true
		},{
			id: 'name',
			header: lang("name"),
			dataIndex: 'name',
			width: 250,
			renderer: renderName,
			sortable:true
		},{
			id: 'user_id',
			header: lang("user"),
			dataIndex: 'user_id',
			renderer: renderUser,
			sortable:true
		},{
			id: 'hourly_cost',
			header: lang("cost"),
			dataIndex: 'hourly_cost',
			renderer: renderMoney,
			align: 'right',
			sortable:true
		},{
			id: 'hourly_rate',
			header: lang("price"),
			dataIndex: 'hourly_rate',
			renderer: renderMoney,
			align: 'right',
			sortable:true
		},{
			id: 'price_multiplier',
			header: lang("price multiplier"),
			dataIndex: 'price_multiplier',
			renderer: 'string',
			align: 'right',
			sortable:true
		},{
			id: 'updated_on',
			header: lang("last updated by"),
			dataIndex: 'updated_on',
			width: 50,
			hidden: true,
			sortable: true,
			renderer: renderDate
		},{
			id: 'is_reimbursable',
			header: lang("reimbursable"),
			dataIndex: 'is_reimbursable',
			width: 50,
			hidden: true,
			sortable: true,
			renderer: renderReimbursable
		},
    ];

	if (og.expenses2.more_product_type_columns && og.expenses2.more_product_type_columns.length > 0) {
		for (var i=0; i<og.expenses2.more_product_type_columns.length; i++) {
			cm_info.push({
				id: og.expenses2.more_product_type_columns[i],
				header: lang(og.expenses2.more_product_type_columns[i]),
				dataIndex: og.expenses2.more_product_type_columns[i],
				width: 100,
				sortable: true,
				renderer: og.clean
			});
		}
	}
	
	// dimension columns
	for (did in og.dimensions_info) {
		if (isNaN(did)) continue;
		
		if (og.product_types_allowed_dimensions && og.product_types_allowed_dimensions.indexOf(parseInt(did)) >= 0) {
			cm_info.push({
				id: 'dim_' + did,
				header: og.dimensions_info[did].name,
				dataIndex: 'dim_' + did,
				sortable: true,
				renderer: og.renderDimCol
			});
			og.breadcrumbs_skipped_dimensions[did] = did;
		}
	}
	// create column model
	var cm = new Ext.grid.ColumnModel(cm_info);
	cm.defaultSortable = false;
	

	actions = {
		back: new Ext.Action({
			text: lang('back'),
            tooltip: lang('back'),
            iconCls: 'ico-back',
            handler: function() {
            	og.goback(this.el.dom);
			}
		}),
		newCO: new Ext.Action({
			text: lang('new product type'),
            tooltip: lang('add new product type'),
            iconCls: 'ico-new new_button',
            handler: function() {
				og.render_modal_form('', {c:'product_type', a:'add', params: {reload:1}});
			}
		}),
		del: new Ext.Action({
			text: lang('delete'),
            iconCls: 'ico-trash',
			disabled: true,
			handler: function() {
				if (confirm(lang('delete selected objects permanently'))) {
					this.load({
						action: 'delete',
						ids: getSelectedIds()
					});
					this.getSelectionModel().clearSelections();
				}
			},
			scope: this
		}),
		edit: new Ext.Action({
			text: lang('edit'),
            tooltip: lang('edit selected object'),
            iconCls: 'ico-edit',
			disabled: true,
			handler: function() {
				og.render_modal_form('', {c:'product_type', a:'edit', params: {id:getFirstSelectedId(), reload:1}});
			},
			scope: this
		})
    };
    
	var tbar = [];
	if (!og.loggedUser.isGuest) {
		tbar.push(actions.back);
		tbar.push('-');
		tbar.push(actions.newCO);
		tbar.push('-');
		tbar.push(actions.edit);
		tbar.push(actions.del);
	}
	
	
	og.ProductTypesManager.superclass.constructor.call(this, {
		store: this.store,
		layout: 'fit',
		cm: cm,
		stateful: og.preferences['rememberGUIState'],
		id: 'product-types-manager',
		stripeRows: true,
		closable: true,
		loadMask: true,
		bbar: new og.CurrentPagingToolbar({
			pageSize: og.config['files_per_page'],
			store: this.store,
			displayInfo: true,
			displayMsg: lang('displaying objects of'),
			emptyMsg: lang("no objects to display")

		}),
		viewConfig: {
			forceFit: true
		},
		sm: sm,
		tbar:tbar,
		listeners: {
			'render': {
				fn: function() {
					this.innerMessage = document.createElement('div');
					this.innerMessage.className = 'inner-message';
					var msg = this.innerMessage;
					var elem = Ext.get(this.getEl());
					var scroller = elem.select('.x-grid3-scroller');
					scroller.each(function() {
						this.dom.appendChild(msg);
					});					
				},
				scope: this
			},
			'columnmove': {
				fn: function(old_index, new_index) {
					og.eventManager.fireEvent('replace all empty breadcrumb', null);
				},
				scope: this
			}
		}
	});
	
};

Ext.extend(og.ProductTypesManager, Ext.grid.GridPanel, {
	load: function(params) {
		if (!params) params = {};
		var start;
		if (typeof params.start == 'undefined') {
			start = (this.getBottomToolbar().getPageData().activePage - 1) * og.config['files_per_page'];
		} else {
			start = 0;
		}
		
		this.store.baseParams = {
			context: og.contextManager.plainContext()
		};
		
		this.store.removeAll();
		this.store.load({
			params: Ext.apply(params, {
				start: start,
				limit: og.config['files_per_page']				
			})
		});
	},
	resetVars: function(){
	},
	
	activate: function() {
		if (this.needRefresh) {
			this.load({start:0});
		}
	},
	
	reset: function() {
		this.load({start:0});
	},
	
	showMessage: function(text) {
		this.innerMessage.innerHTML = text;
	},
	
	trashObjects: function() {
		if (confirm(lang('delete selected objects permanently'))) {
			this.load({
				action: 'delete',
				ids: this.getSelectedIds()
			});
			this.getSelectionModel().clearSelections();
		}
	}
	
	
});


Ext.reg("product-types", og.ProductTypesManager);

/************************************************
Container for ProductTypesManager,
*************************************************/
og.ProductTypesManagerPanel = function() {
	this.doNotRemove = true;
	this.needRefresh = false;
	
	this.manager = new og.ProductTypesManager();
	
	og.ProductTypesManagerPanel.superclass.constructor.call(this, {
		layout: 'fit',
		border: false,
		bodyBorder: false,
		items: [
			this.manager
		],
		closable: true
	});
}

Ext.extend(og.ProductTypesManagerPanel, Ext.Panel, {
	load: function(params) {
		this.manager.load(params);
	},
	activate: function() {
		this.manager.activate();
	},	
	reset: function() {
		this.manager.reset();
	},	
	showMessage: function(text) {
		this.manager.showMessage(text);
	}
});

Ext.reg("product-types-containerpanel", og.ProductTypesManagerPanel);