<?php

/**
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */

class FidelisReportsController extends ApplicationController {

	/**
	 * Construct the ReportingController
	 *
	 * @access public
	 * @param void
	 * @return ReportingController
	 */
	function __construct()
	{
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('grouping');
		Env::useHelper('advanced_reports', 'advanced_reports');
	} // __construct


	
	function contable_report_p(){
		$report_data = array();
		
		tpl_assign('report_data', $report_data);
	}
	
	
	private function get_client_from_worked_hours($contact_id, $st, $et) {
		$range_st = new DateTimeValue($st->getTimestamp());
		$range_st->add('d', -2);
		$range_et = new DateTimeValue($et->getTimestamp());
		$range_et->add('d', 2);
		// to use a small result set of timeslots
		$ts_range_conditions = " AND ts.start_time <= '".$range_et->toMySQL()."' AND ts.start_time >= '".$range_st->toMySQL()."' ";
		
		$ts_ot = ObjectTypes::findByName('timeslot');
		$cli_ot = ObjectTypes::findByName('customer');
		$workday_cp = CustomProperties::getCustomPropertyByCode($ts_ot->getId(), 'workday');
		
		$sql = "
			SELECT DISTINCT(m.name) as client_name
			FROM ".TABLE_PREFIX."timeslots ts
			INNER JOIN ".TABLE_PREFIX."objects o ON o.id=ts.object_id
			INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=ts.object_id
			INNER JOIN ".TABLE_PREFIX."members m ON m.id=om.member_id
			WHERE
			ts.contact_id=$contact_id $ts_range_conditions
			AND o.trashed_by_id=0
			AND m.object_type_id=".$cli_ot->getId()."
			AND o.id IN (
				SELECT object_id as id FROM ".TABLE_PREFIX."custom_property_values cpv
				WHERE cpv.object_id=o.id AND cpv.custom_property_id = ".$workday_cp->getId()."
					AND cpv.value >= '".$st->format('Y-m-d')."'
			) AND o.id IN (
				SELECT object_id as id FROM ".TABLE_PREFIX."custom_property_values cpv
				WHERE cpv.object_id=o.id AND cpv.custom_property_id = ".$workday_cp->getId()."
					AND cpv.value <= '".$et->format('Y-m-d')."'
			);
		";
		$rows = DB::executeAll($sql);
		$client_names = implode(", ", array_flat($rows));
		
		return $client_names;
	}
	
	
	private function get_worked_hours_in_single_member($member_id, $st, $et, $user_activity='active', $extra_member_id=null) {
		$range_st = new DateTimeValue($st->getTimestamp());
		$range_st->add('d', -2);
		$range_et = new DateTimeValue($et->getTimestamp());
		$range_et->add('d', 2);
		// to use a small result set of timeslots
		$ts_range_conditions = " AND t.start_time <= '".$range_et->toMySQL()."' AND t.start_time >= '".$range_st->toMySQL()."' ";
		
		$ts_ot = ObjectTypes::findByName('timeslot');
		$workday_cp = CustomProperties::getCustomPropertyByCode($ts_ot->getId(), 'workday');
		
		$user_activity_cond = '';
		if ($user_activity == 'active') {
			$user_activity_cond = "AND c.disabled=0";
		} else if ($user_activity == 'inactive') {
			$user_activity_cond = "AND c.disabled=1";
		}
		
		// ej: para obtener descanso no tomado en licencia trabajada
		$extra_member_condition = "";
		$extra_member_name = "";
		if (is_numeric($extra_member_id) && $extra_member_id > 0) {
			$extra_member_condition = "AND EXISTS (SELECT * FROM ".TABLE_PREFIX."object_members eom WHERE eom.member_id=$extra_member_id AND eom.object_id=o.id)";
			$extra_member = Members::getMemberById($extra_member_id);
			if ($extra_member instanceof Member) $extra_member_name = $extra_member->getName();
		}
		
		$main_sql = "
				SELECT
					c.object_id as contact_id,
					$member_id as member_ids,
					m.name as member_names,
					sum(t.worked_time) as worked_time
		
				FROM ".TABLE_PREFIX."timeslots t
				INNER JOIN ".TABLE_PREFIX."contacts c ON t.contact_id=c.object_id
				INNER JOIN ".TABLE_PREFIX."objects o ON o.id=t.object_id
				INNER JOIN ".TABLE_PREFIX."object_members om ON o.id=om.object_id
				INNER JOIN ".TABLE_PREFIX."members m ON m.id=om.member_id
				LEFT JOIN ".TABLE_PREFIX."custom_property_values cpv_padron ON cpv_padron.object_id=c.object_id
				INNER JOIN ".TABLE_PREFIX."custom_properties cp_padron ON cp_padron.id=cpv_padron.custom_property_id
						
				WHERE
					c.user_type > 0
					$user_activity_cond
					AND cp_padron.code='padron' AND cpv_padron.value != ''
					$ts_range_conditions

					AND o.id IN (
						SELECT object_id as id FROM ".TABLE_PREFIX."custom_property_values cpv
						WHERE cpv.object_id=o.id AND cpv.custom_property_id = ".$workday_cp->getId()."
							AND cpv.value >= '".$st->format('Y-m-d')."'
					) AND o.id IN (
						SELECT object_id as id FROM ".TABLE_PREFIX."custom_property_values cpv
						WHERE cpv.object_id=o.id AND cpv.custom_property_id = ".$workday_cp->getId()."
							AND cpv.value <= '".$et->format('Y-m-d')."'
					)
					AND o.archived_by_id=0
					AND (t.rel_object_id=0 OR (SELECT aux.trashed_by_id FROM ".TABLE_PREFIX."objects aux WHERE aux.id=t.rel_object_id)=0)
		
					AND om.member_id=$member_id
					$extra_member_condition
		";

		$sql = "$main_sql
				GROUP BY contact_id, member_ids
				ORDER BY contact_id;
		";
		
		// data
		$rows_with_worked_time = DB::executeAll($sql);
		if (!is_array($rows_with_worked_time) || count($rows_with_worked_time) == 0) {
			$member = Members::getMemberById($member_id);
			$rows_with_worked_time = array(array(
					'member_ids' => $member_id,
					'member_names' => $member instanceof Member ? $member->getName() : "",
					'worked_time' => 0,
			));
		}
		if ($extra_member_name != '') $rows_with_worked_time[0]['member_names'] .= ", ".$extra_member_name;
		
		// totals
		$totals_row = DB::executeOne($main_sql);
		$totals_row['contact_id'] = '__total_row__';
		$rows_with_worked_time[] = $totals_row;
		
		return $rows_with_worked_time;
	}
	
	
	private function get_all_rows_for_contacts($hour_types_dim_id, $main_member_id, $ord_dim_ids, $user_activity='active') {
		$user_activity_cond = '';
		if ($user_activity == 'active') {
			$user_activity_cond = "AND c.disabled=0";
		} else if ($user_activity == 'inactive') {
			$user_activity_cond = "AND c.disabled=1";
		}
		
		$skeleton_sql = "
				SELECT
					c.object_id as contact_id,
					o.name,
					cpv_ci.value as ci,
					cpv_padron.value as padron,
					CONVERT (cpv_padron.value, UNSIGNED INTEGER) as padron_order,
					concat(m.id,',',m2.id,',',m3.id,',',m4.id) as member_ids,
					concat(m.name,', ',m2.name,', ',m3.name,', ',m4.name) as member_names,
					0 as worked_time
		
				FROM ".TABLE_PREFIX."contacts c
				INNER JOIN ".TABLE_PREFIX."objects o on o.id=c.object_id
						
				LEFT JOIN ".TABLE_PREFIX."custom_property_values cpv_ci ON cpv_ci.object_id=c.object_id
				INNER JOIN ".TABLE_PREFIX."custom_properties cp_ci ON cp_ci.id=cpv_ci.custom_property_id
				
				LEFT JOIN ".TABLE_PREFIX."custom_property_values cpv_padron ON cpv_padron.object_id=c.object_id
				INNER JOIN ".TABLE_PREFIX."custom_properties cp_padron ON cp_padron.id=cpv_padron.custom_property_id
						
				JOIN ".TABLE_PREFIX."members m
				JOIN ".TABLE_PREFIX."members m2
				JOIN ".TABLE_PREFIX."members m3
				JOIN ".TABLE_PREFIX."members m4
		
				WHERE
					m.dimension_id = $hour_types_dim_id AND m.id=$main_member_id
					AND m2.dimension_id = ".$ord_dim_ids[0]."
					AND m3.dimension_id = ".$ord_dim_ids[1]."
					AND m4.dimension_id = ".$ord_dim_ids[2]."
					AND c.user_type > 0
					$user_activity_cond
					AND cp_ci.code='ci'
					AND cp_padron.code='padron' AND cpv_padron.value != ''
		
				GROUP BY contact_id, member_ids
				ORDER BY padron_order, contact_id, m.name, m2.name, m3.name, m4.name;
		";
		$all_rows = DB::executeAll($skeleton_sql);
		
		$all_rows_with_keys = array();
		foreach ($all_rows as $row) {
			$all_rows_with_keys[$row['contact_id']."-".$row['member_ids']] = $row;
		}
		
		return $all_rows_with_keys;
	}
	
	
	
	private function get_rows_with_worked_time($hour_types_dim_id, $main_member_id, $ord_dim_ids, $st, $et, $user_activity='active') {
		
		$range_st = new DateTimeValue($st->getTimestamp());
		$range_st->add('d', -2);
		$range_et = new DateTimeValue($et->getTimestamp());
		$range_et->add('d', 2);
		// to use a small result set of timeslots
		$ts_range_conditions = " AND t.start_time <= '".$range_et->toMySQL()."' AND t.start_time >= '".$range_st->toMySQL()."' ";
		
		$ts_ot = ObjectTypes::findByName('timeslot');
		$workday_cp = CustomProperties::getCustomPropertyByCode($ts_ot->getId(), 'workday');
		
		$user_activity_cond = '';
		if ($user_activity == 'active') {
			$user_activity_cond = "AND c.disabled=0";
		} else if ($user_activity == 'inactive') {
			$user_activity_cond = "AND c.disabled=1";
		}
		
		$descanso_no_tomado_member_id = config_option('break_not_taken_hours_member');
		
		$main_sql = "
				SELECT
					c.object_id as contact_id,
					concat(m.id,',',m2.id,',',m3.id,',',m4.id) as member_ids,
					sum(t.worked_time) as worked_time
		
				FROM ".TABLE_PREFIX."timeslots t
				INNER JOIN ".TABLE_PREFIX."contacts c ON t.contact_id=c.object_id
				INNER JOIN ".TABLE_PREFIX."objects o ON o.id=t.object_id
				JOIN ".TABLE_PREFIX."members m
				JOIN ".TABLE_PREFIX."members m2
				JOIN ".TABLE_PREFIX."members m3
				JOIN ".TABLE_PREFIX."members m4
				LEFT JOIN ".TABLE_PREFIX."custom_property_values cpv_padron ON cpv_padron.object_id=c.object_id
				INNER JOIN ".TABLE_PREFIX."custom_properties cp_padron ON cp_padron.id=cpv_padron.custom_property_id
				WHERE
					m.dimension_id = $hour_types_dim_id AND m.id=$main_member_id
					AND m2.dimension_id = ".$ord_dim_ids[0]."
					AND m3.dimension_id = ".$ord_dim_ids[1]."
					AND m4.dimension_id = ".$ord_dim_ids[2]."
					AND c.user_type > 0 
					$user_activity_cond
					AND cp_padron.code='padron' AND cpv_padron.value != ''
					$ts_range_conditions
						
					AND o.id IN (
						SELECT object_id as id FROM ".TABLE_PREFIX."custom_property_values cpv
						WHERE cpv.object_id=o.id AND cpv.custom_property_id = ".$workday_cp->getId()."
							AND cpv.value >= '".$st->format('Y-m-d')."'
					) AND o.id IN (
						SELECT object_id as id FROM ".TABLE_PREFIX."custom_property_values cpv
						WHERE cpv.object_id=o.id AND cpv.custom_property_id = ".$workday_cp->getId()."
							AND cpv.value <= '".$et->format('Y-m-d')."'
					)
					AND o.archived_by_id=0
					AND (t.rel_object_id=0 OR (SELECT aux.trashed_by_id FROM ".TABLE_PREFIX."objects aux WHERE aux.id=t.rel_object_id)=0)
			
					AND 4 = (
						SELECT count(mm.id)
						FROM ".TABLE_PREFIX."object_members om
						INNER JOIN ".TABLE_PREFIX."members mm ON mm.id=om.member_id
							WHERE om.object_id=o.id
							AND mm.id IN (m.id, m2.id, m4.id, m3.id)
							AND mm.dimension_id IN ($hour_types_dim_id,".$ord_dim_ids[0].",".$ord_dim_ids[1].",".$ord_dim_ids[2].")
					)
					AND NOT EXISTS (SELECT * FROM ".TABLE_PREFIX."object_members bom WHERE bom.object_id=o.id AND bom.member_id='$descanso_no_tomado_member_id')
		";
		/*
		 * AND concat(m.id,',',m2.id,',',m3.id,',',m4.id) = (
						SELECT group_concat(mm.id ORDER BY mm.dimension_id)
						FROM ".TABLE_PREFIX."object_members om
						INNER JOIN ".TABLE_PREFIX."members mm ON mm.id=om.member_id
							WHERE om.object_id=o.id
							AND mm.dimension_id IN ($hour_types_dim_id,".$ord_dim_ids[0].",".$ord_dim_ids[1].",".$ord_dim_ids[2].")
					)
		 */
		
		$sql = "$main_sql
				GROUP BY contact_id, member_ids
				ORDER BY contact_id, m.id, m2.id, m3.id, m4.id;
		";
		
		$rows_with_worked_time = DB::executeAll($sql);
		if (!is_array($rows_with_worked_time)) {
			$rows_with_worked_time = array();
		}
		
		$totals_sql = "$main_sql
				GROUP BY member_ids;
		";
		
		$total_rows = DB::executeAll($totals_sql);
		if (is_array($total_rows)) {
			foreach ($total_rows as &$total_row) {
				$total_row['contact_id'] = '__total_row__';
			}
			$rows_with_worked_time = array_merge($rows_with_worked_time, $total_rows);
		}
		
		return $rows_with_worked_time;
	}
	
	
	private function get_intersected_worked_time($main_member_id, $st, $et, $user_activity='active') {
		
		$hour_types_dim_id = Dimensions::findByCode('hour_types')->getId();
		$night_hour_types_dim_id = Dimensions::findByCode('hour_types_night')->getId();
		$extra_types_dim_id = Dimensions::findByCode('hour_types_extra')->getId();
		$weapon_types_dim_id = Dimensions::findByCode('hour_types_weapon')->getId();
		
		$ord_dim_ids = array($night_hour_types_dim_id, $weapon_types_dim_id, $extra_types_dim_id);
		//sort($ord_dim_ids, SORT_NUMERIC);
		
		$all_rows = $this->get_all_rows_for_contacts($hour_types_dim_id, $main_member_id, $ord_dim_ids, $user_activity);
		
		$rows_with_worked_time = $this->get_rows_with_worked_time($hour_types_dim_id, $main_member_id, $ord_dim_ids, $st, $et, $user_activity);
		
		if (is_array($rows_with_worked_time)) {
			foreach ($rows_with_worked_time as $row) {
				$key = $row['contact_id']."-".$row['member_ids'];
				$all_rows[$key]['worked_time'] = $row['worked_time'];
				if (!isset($all_rows[$key]['member_ids'])) {
					$all_rows[$key]['member_ids'] = $row['member_ids'];
				}
				if (!isset($all_rows[$key]['contact_id'])) {
					$all_rows[$key]['contact_id'] = $row['contact_id'];
				}
			}
		}
		
		return $all_rows;
	}
	
	
	function contable_report($report_data = null, $csv = null) {
		
		if (!$report_data) {
			$report_data = array_var($_POST, 'report', array());
		}
		$_SESSION['contable_report_data'] = $report_data;
		
		$now = DateTimeValueLib::now();
		$now->advance(logged_user()->getUserTimezoneValue(), true);
		switch (array_var($report_data, 'date_type')){
			case -1: //None
				$st = null;
				$et = null;
				break;
			case 1: //Today
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),$now->getDay(),$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),$now->getDay(),$now->getYear());break;
			case 2: //This week
				$monday = $now->getMondayOfWeek();
				$nextMonday = $now->getMondayOfWeek()->add('w',1)->add('d',-1);
				$st = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
				$et = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
			case 3: //Last week
				$monday = $now->getMondayOfWeek()->add('w',-1);
				$nextMonday = $now->getMondayOfWeek()->add('d',-1);
				$st = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
				$et = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
			case 4: //This month
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
			case 5: //Last month
				$now->add('M',-1);
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
			case 6: //Date interval
				$st = getDateValue(array_var($report_data, 'date_start_value'));
				if ($st instanceof DateTimeValue) {
					$st = $st->beginningOfDay();
				} else {
					$st = null;
				}
				
				$et = getDateValue(array_var($report_data, 'date_end_value'));
				if ($et instanceof DateTimeValue) {
					$et = $et->endOfDay();
				} else {
					$et = null;
				}
				break;
		}
		
		$contact_ot = ObjectTypes::findByName('contact');
		$ci_cp = CustomProperties::getCustomPropertyByCode($contact_ot->getId(), 'ci');
		$padron_cp = CustomProperties::getCustomPropertyByCode($contact_ot->getId(), 'padron');
		$tipo_remuneracion_cp = CustomProperties::getCustomPropertyByCode($contact_ot->getId(), 'tipo_remuneracion');
		$categoria_cp = CustomProperties::instance()->findOne(array(
			'conditions' => array("`object_type_id` = ? and `name` = ? and `type`='list'", $contact_ot->getId(), 'Categoría')
		));

		// fixed member ids
		$common_hours_member_id = config_option('common_hours_member');
		$worked_feriado_hours_member_id = 2842;
		$worked_franco_hours_member_id = config_option('day_off_hours_member');
		$worked_licencia_hours_member_id = 3572;
		
		$course_member_id = 3520;
		$lic_sindical_member_id = 3516;
		$falta_con_desc_member_id = 3341;
		$falta_disse_member_id = 3343;
		$falta_cat4_member_id = 3651;
		$descanso_no_tomado_member_id = config_option('break_not_taken_hours_member');
		
		$user_activity = array_var($report_data, 'user_activity', 'active');
		
		// worked hours by intersection
		$plain_rows = $this->get_intersected_worked_time($common_hours_member_id, $st, $et, $user_activity);
		$plain_rows_feriado = $this->get_intersected_worked_time($worked_feriado_hours_member_id, $st, $et, $user_activity);
		$plain_rows_franco = $this->get_intersected_worked_time($worked_franco_hours_member_id, $st, $et, $user_activity);
		$plain_rows_licencia = $this->get_intersected_worked_time($worked_licencia_hours_member_id, $st, $et, $user_activity);
		
		// hours in single member
		$plain_rows_course = $this->get_worked_hours_in_single_member($course_member_id, $st, $et, $user_activity);
		$plain_rows_lic_sindical = $this->get_worked_hours_in_single_member($lic_sindical_member_id, $st, $et, $user_activity);
		$plain_rows_faltas = $this->get_worked_hours_in_single_member($falta_con_desc_member_id, $st, $et, $user_activity);
		$plain_rows_faltas_disse = $this->get_worked_hours_in_single_member($falta_disse_member_id, $st, $et, $user_activity);
		$plain_rows_faltas_cat4 = $this->get_worked_hours_in_single_member($falta_cat4_member_id, $st, $et, $user_activity);
		$plain_rows_descanso_no_tomado = $this->get_worked_hours_in_single_member($descanso_no_tomado_member_id, $st, $et, $user_activity);
		$plain_rows_descanso_no_tomado_en_licencia_trabajada = $this->get_worked_hours_in_single_member($worked_licencia_hours_member_id, $st, $et, $user_activity, $descanso_no_tomado_member_id);
		
		// all worked hours array to show in report
		$plain_rows_array = array(
				$plain_rows_faltas,
				$plain_rows_faltas_disse,
				$plain_rows_faltas_cat4,
				$plain_rows,
				$plain_rows_descanso_no_tomado,
				$plain_rows_course, 
				$plain_rows_feriado,
				$plain_rows_franco, 
				$plain_rows_lic_sindical,
				$plain_rows_licencia,
				$plain_rows_descanso_no_tomado_en_licencia_trabajada,
		);
		
		
		$column_names = array(
			'padron' => $padron_cp->getName(),
			'name' => lang('name'),
			'ci' => $ci_cp->getName(),
			'categoria' => $categoria_cp->getName(),
			'tipo_remuneracion' => $tipo_remuneracion_cp->getName(),
			'cliente' => lang('customer'),
		);
		$rows_by_contact_and_member_ids = array();
		
		$all_contact_ids = array();
		// init contact name and properties
		foreach ($plain_rows as $plain_row) {
			$contact_id = array_var($plain_row, 'contact_id');
			if ($contact_id && !isset($rows_by_contact_and_member_ids[$contact_id])) {
				if ($contact_id == '__total_row__') {
					$rows_by_contact_and_member_ids[$contact_id] = array(
						'name' => lang('total'),
						'ci' => '',
						'padron' => '',
						'categoria' => '',
						'tipo_remuneracion' => '',
						'cliente' => '',
					);
				} else {
					$all_contact_ids[] = $contact_id;
					$rows_by_contact_and_member_ids[$contact_id] = array(
						'name' => $plain_row['name'],
						'ci' => $plain_row['ci'],
						'padron' => $plain_row['padron'],
						'categoria' => '',
						'tipo_remuneracion' => '',
						'cliente' => $this->get_client_from_worked_hours($contact_id, $st, $et),
					);
				}
			}
		}
		
		if (count($all_contact_ids) > 0) {
			$cat_rows = DB::executeAll("SELECT * FROM ".TABLE_PREFIX."custom_property_values WHERE custom_property_id='".$categoria_cp->getId()."'");
			foreach ($cat_rows as $cat_row) {
				$c_id = $cat_row['object_id'];
				if (isset($rows_by_contact_and_member_ids[$c_id])) {
					$rows_by_contact_and_member_ids[$c_id]['categoria'] = $cat_row['value'];
				}
			}
			
			$tr_rows = DB::executeAll("SELECT * FROM ".TABLE_PREFIX."custom_property_values WHERE custom_property_id='".$tipo_remuneracion_cp->getId()."'");
			foreach ($tr_rows as $tr_row) {
				$tr_id = $tr_row['object_id'];
				if (isset($rows_by_contact_and_member_ids[$tr_id])) {
					$rows_by_contact_and_member_ids[$tr_id]['tipo_remuneracion'] = $tr_row['value'];
				}
			}
		}
		
		// fill all data
		foreach ($plain_rows_array as $plain_rows) {
			foreach ($plain_rows as $plain_row) {
				
				$contact_id = array_var($plain_row, 'contact_id');
				if ($contact_id) {
					$worked_hours = $plain_row['worked_time'] / 60;
					$rows_by_contact_and_member_ids[$contact_id][$plain_row['member_ids']] = $worked_hours;
				}
				
				if (!isset($column_names[$plain_row['member_ids']])) {
					if (function_exists('mb_strtoupper')) $mem_names = mb_strtoupper($plain_row['member_names']);
					else $mem_names = strtoupper($plain_row['member_names']);
					
					$mem_names = str_replace('02. CON ARMA', 'C/ARMA', $mem_names);
					$mem_names = str_replace('01. SIN ARMA', 'S/ARMA', $mem_names);
					$mem_names = str_replace('02. EXTRA', 'EXTRA', $mem_names);
					$mem_names = str_replace('01. NORMAL', 'NORMAL', $mem_names);
					$mem_names = str_replace('FERIADO TRABAJADO', 'FERIADO TRAB.', $mem_names);
					$mem_names = str_replace('FRANCO TRABAJADO', 'FRANCO TRAB.', $mem_names);
					$mem_names = str_replace('LICENCIA TRABAJADA', 'LICENCIA TRAB.', $mem_names);
					
					if ($csv) $mem_names = str_replace(', ', "\n", $mem_names);
					
					$column_names[$plain_row['member_ids']] = $mem_names;
				}
			}
		}
		
		// All headers in upper case
		foreach ($column_names as &$colname) {
			if (function_exists('mb_strtoupper')) $colname = mb_strtoupper($colname);
			else $colname = strtoupper($colname);
		}
		
		if ($csv) {
			return array(
				'headers' => $column_names,
				'data' => $rows_by_contact_and_member_ids,
			);
		}
		
		tpl_assign('column_names', $column_names);
		tpl_assign('rows_by_contact_and_member_ids', $rows_by_contact_and_member_ids);
		if (array_var($report_data, 'date_type') == 6) {
			$st->advance(logged_user()->getUserTimezoneValue(), true);
			$et->advance(logged_user()->getUserTimezoneValue(), true);
		}
		tpl_assign('start_time', $st);
		tpl_assign('end_time', $et);
		tpl_assign('title', lang('contable report'));
		tpl_assign('post', $report_data);
		tpl_assign('allow_export', true);
		
		tpl_assign('template_name', 'contable_report');
		tpl_assign('print_action_url', get_url('fidelis_reports', 'contable_report_print'));
		tpl_assign('excel_export_action', 'og.fidelis_reports.submit_excel_form');
		tpl_assign('pdf_export_function', 'og.fidelis_reports.export_contable_report_pdf');
		tpl_assign('report_template_path', get_template_path('contable_report', 'fidelis_reports'));
		
		$this->setTemplate(get_template_path('report_wrapper', 'advanced_reports', 'advanced_reports'));
	}

	
	function contable_report_print() {
		ajx_current("empty");
		
		$export_csv = array_var($_POST, 'exportCSV');
		
		if ($export_csv) {
			$report_result = $this->contable_report($_POST, true);
			
			$filename = str_replace(" ", "-", str_replace("/", "-", lang('contable report').".csv"));
			
			$headers_csv = '"' . implode('","', $report_result["headers"]) . '"' . "\n";
			$rows_csv = '';
			foreach ($report_result["data"] as $row) {
				$first = true;
				foreach ($report_result["headers"] as $col_key => $col_name) {
					$def_val = in_array($col_key, array('name','padron','ci')) ? "" : "0";
					$rows_csv .= ($first ? "" : ",") . '"' . array_var($row, $col_key, $def_val) . '"';
					$first = false;
				}
				$rows_csv .= "\n";
			}
			
			file_put_contents(ROOT."/tmp/$filename", $headers_csv . $rows_csv);
			
		} else {
			ob_start();
			$this->setTemplate('contable_report');
			$this->contable_report($_POST);
			tpl_display(get_template_path('contable_report', 'fidelis_reports', 'fidelis'));
			$html = ob_get_clean();
			
			$html_filepath = ROOT."/tmp/".gen_id().".html";
			
			file_put_contents($html_filepath, $html);
			
			// convert html to pdf
			$orientation = array_var($_REQUEST, 'pdfPageLayout') == 'L' ? 'Landscape' : 'Portrait';
			$page_size = array_var($_REQUEST, 'pdfPageSize');
			
			$pdf_data = convert_to_pdf($html, $orientation, "contable_report", $page_size);
			$filename = $pdf_data['name'];
			
		}
		
		ajx_extra_data(array('filename' => $filename));
	}

	
	function contable_report_excel() {
		ajx_current("empty");
		
		$report_result = $this->contable_report($_POST, true);
		
		$filename = str_replace(" ", "-", str_replace("/", "-", lang('contable report').".xlsx"));
		
		$headers = $report_result["headers"];
		$rows = array();
		foreach ($report_result["data"] as $row) {
			$row_excel = array();
			foreach ($report_result["headers"] as $col_key => $col_name) {
				$def_val = in_array($col_key, array('name','padron','ci')) ? "" : "0";
				$row_excel[$col_key] = array_var($row, $col_key, $def_val);
			}
			$rows[] = $row_excel;
		}
		
		// build a sheet for the excel file with the report data 
		$sheets = array(
			array('headers' => $headers, 'data' => $rows, 'options' => array('title' => lang('contable report'), 'fixed_cells_width' => 15)),
		);
		
		// generate the excel file
		$excel_export_controller = new ExcelExportController();
		$filepath = $excel_export_controller->generate_excel_file($sheets, array('filename' => $filename));
			
		ajx_extra_data(array('filename' => $filename));
	}
	
}
























