<?php
/**
 * Form controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class FormController extends ApplicationController {

	var $plugin_name = "forms";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
	//	Env::useHelper('functions', $this->plugin_name);
	}
	
	function index() {
		ajx_set_no_toolbar(true);
	}
	
	function list_all() {
		ajx_current("empty");
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		$dim_order = null;
		$cp_order = null;
		
		$ids = array();
		$exploded = explode(',', array_var($_GET,'ids'));
		foreach ($exploded as $exp) {
			if (is_numeric($exp)) $ids[] = $exp;
		}
		$attributes = array("ids" => $ids);
		
		if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		} else if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		} else if ($order == "dateUpdated") {
			$order = 'updated_on';
		}
		
		$extra_conditions = "";
		
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		$only_count_result = array_var($_GET, 'only_result',false);
		
		$result = Forms::instance()->listing(array(
			"order" => $order,
			"order_dir" => $order_dir,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"extra_conditions" => $extra_conditions,
			"count_results" => true,
			"only_count_results" => $only_count_result,
			"start" => $start,
			"limit" => $limit,
		));
		$forms = $result->objects;
		$object = $this->prepareObject($forms, $start, $limit, $result);
		
		ajx_extra_data($object);
	}

	
	private function prepareObject($forms, $start, $limit, $res_obj) {
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"forms" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
		
		$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType(Forms::instance()->getObjectTypeId());
		$count = min(array($limit, count($forms)));
		for ($i = 0; $i < $count; $i++){
			if (isset($forms[$i])){
				$obj = $forms[$i];
				if ($obj instanceof Form){
					$info = $obj->getArrayInfo();
					$info['id'] = $obj->getId();
					$info['memPath'] = json_encode($obj->getMembersIdsToDisplayPath());
					$object["forms"][] = $info;

					foreach ($custom_properties as $cp) {
						$object["forms"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $obj);
					}
    			}
			}
		}
		return $object;
	}

	
	function add() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$form_data = array_var($_POST, 'form');
		
		if(!is_array($form_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$obj = new Form();
			$obj->setObjectTypeId(Forms::instance()->getObjectTypeId());

			tpl_assign('form', $obj);
			
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$form = $this->save_form($form_data, $member_ids);
				
				DB::commit();

				ApplicationLogs::createLog($form, ApplicationLogs::ACTION_ADD);
				flash_success(lang('success add form', $form->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	function edit() {
		$this->setTemplate('add');
		
		$form = Forms::instance()->findById(get_id());
		if(!($form instanceof Form)) {
			flash_error(lang('form dnx'));
			ajx_current("empty");
			return;
		}
		
		if (logged_user()->isGuest() || !$form->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$notAllowedMember = '';
		if(!Form::canAdd(logged_user(), active_context(), $notAllowedMember )) {
			if (str_starts_with($notAllowedMember, '-- req dim --')) flash_error(lang('must choose at least one member of', str_replace_first('-- req dim --', '', $notAllowedMember, $in)));
			else flash_error(lang('no context permissions to add',lang("Forms"),$notAllowedMember ));
			ajx_current("empty");
			return;
		}
		
		$form_data = array_var($_POST, 'form');
		
		if(!is_array($form_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			tpl_assign('form', $form);
			
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$form = $this->save_form($form_data, $member_ids, $form);
				
				DB::commit();

				ApplicationLogs::createLog($form, ApplicationLogs::ACTION_EDIT);
				flash_success(lang('success edit form', $form->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	private function save_form($form_data, $member_ids, $form = null) {
		// Save Form object
		$is_new = false;
		if (is_null($form) || !$form instanceof Form) {
			$form = new Form();
			$is_new = true;
		}
				
		$form_data['text'] = preg_replace("/[\n|\r|\n\r]/", '', array_var($form_data, 'text'));
		
		$form->setFromAttributes($form_data);
		
		$form->save();
		
		$object_controller = new ObjectController();
		if (!is_null($member_ids)) {
			$object_controller->add_to_members($form, $member_ids);
		}
		$object_controller->add_subscribers($form);
		$object_controller->link_to_new_object($form);
		$object_controller->add_custom_properties($form);
		
		return $form;
	}
	
	
	
	
	
	function view() {
		ajx_set_no_toolbar();
		$form = Forms::instance()->findById(get_id());
		if(!($form instanceof Form)) {
			flash_error(lang('form dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$form->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		tpl_assign('form', $form);
	}
	
	
	
	private function resolveAction($action, $attributes){
				 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				$messages = array();
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$form = Forms::instance()->findById($id);
					if ($form instanceof Form && $form->canDelete(logged_user())){
						try{
							DB::beginWork();
							$form->trash();
							DB::commit();
							ApplicationLogs::createLog($form, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$messages[] = $e->getMessage();
							$err++;
						}
					} else {
						$err++;
						$messages[] = lang('cannot delete form', $form instanceof Form ? clean($form->getName()) : '');
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) .("\n - ".implode("\n - ",$messages)). ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			case "archive":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$form = Forms::instance()->findById($id);
					if ($form instanceof Form && $form->canEdit(logged_user())){
						try{
							DB::beginWork();
							$form->archive();
							DB::commit();
							ApplicationLogs::createLog($form, ApplicationLogs::ACTION_ARCHIVE);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error archive objects", $err) . "<br />" . ($succ > 0 ? lang("success archive objects", $succ) : "");
				} else {
					$resultMessage = lang("success archive objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;		
		} // switch
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
	}
}