//use for wrap gantt library
var ganttManager = {
	isInitialized : false,
    isLoaded : new Array(),
    intervalId : null,
	//Draw Gantt
	initGantt : function() {
		//Gantt configurations
		gantt.config.duration_step = 1; 		
		gantt.config.fit_tasks = true;
		gantt.config.scale_unit = "day";
		gantt.config.step = 1;
		gantt.config.date_scale = "%d %M";
		gantt.config.round_dnd_dates = false;
		gantt.config.drag_links = true;
		gantt.config.row_height = 22;//default 35
		
		//grid columns
		gantt.config.columns = [ {name:"text", tree:true, width:'250' } ];
		
		if (ogTasks.ganttPreferences.ganttShowStartDates == 1) {
			gantt.config.columns.push({name:"start_date", align: "left", width:'110', label: lang('start date')});
		}
		if (ogTasks.ganttPreferences.ganttShowEndDates == 1) {
			gantt.config.columns.push({name:"end_date", align: "left", width:'110', label: lang('due date')});
		}
		if (ogTasks.ganttPreferences.ganttShowAssignedTo == 1) {
			gantt.config.columns.push({name:"assigned_to", align: "left", width:'110', label: lang('assigned to')});
		}
		if (ogTasks.ganttPreferences.ganttShowTimeEstimates == 1) {
			gantt.config.columns.push({name:"estimated_time", align: "left", width:'110', label: lang('estimated time')});
		}
		if (ogTasks.ganttPreferences.ganttShowTimePending == 1) {
			gantt.config.columns.push({name:"pending_time", align: "left", width:'110', label: lang('pending time')});
		}
		if (ogTasks.ganttPreferences.ganttShowTimeWorked == 1) {
			gantt.config.columns.push({name:"worked_time", align: "left", width:'110', label: lang('worked time')});
		}
		if (ogTasks.ganttPreferences.ganttShowPercentCompletedBar == 1) {
			gantt.config.columns.push({name:"percent_completed", align: "left", width:'110', label: lang('percent completed')});
		}
		
		// custom property columns
		for (x in ogTasks.custom_properties) {
			var cp = ogTasks.custom_properties[x];
			if (typeof(cp) == 'object' && ogTasks.ganttPreferences['ganttShowCP_'+cp.id] == 1) {
				gantt.config.columns.push({name:'cp_'+cp.id, align: "left", width:'110', label: cp.name});
			}
		}
		
		// dimension columns
		var dimension_columns = ganttManager.buildDimensionColumns();
		if (dimension_columns && dimension_columns.length > 0) {
			gantt.config.columns = gantt.config.columns.concat(dimension_columns);
		}
		
		// left grid width 
		if (ogTasks.userPreferences.show_left_grid_on_gantt){
			gantt.config.grid_width = 250 + (gantt.config.columns.length - 1) * 125;
		}else{
			gantt.config.grid_width = 0;
		}
		
		
		//grid start time column format
		gantt.templates.date_grid = function(date){
			var dt = ganttTasks.formatDateToSendToServer(date);
			return dt.date;
		};		
		//date format on task over info (tooltip)
		gantt.templates.tooltip_date_format=function (date){
			var dt = ganttTasks.formatDateToSendToServer(date);
			return dt.date;
		};		
		//tasks colors
		gantt.templates.task_class  = function(start, end, task){
			//Project
			if(task.type == gantt.config.types.project){
				return "gantt-project";
			}
			
			//Completed task
			if(task.type == gantt.config.types.task && task.progress == 1){
				return "gantt-completed-task";
			}
			
			var overdue = 1;
			var now = new Date();
			
			//is overdue
			if(end < now){
				overdue = 2;
			}
			
			switch (overdue){
		        case 2:
		            return "gantt-overdue-task";
		            break;
		        case 1:
		            return "gantt-ontime-task";
		            break;		        
		    }
		};
		//tooltip text
		gantt.templates.tooltip_text = function(start,end,task){
		    return "<b>"+lang('task')+":</b> "+task.text+"<br/><b>"+lang('start date')+":</b> " + 
		    gantt.templates.tooltip_date_format(start)+ 
		    "<br/><b>"+lang('due date')+":</b> "+gantt.templates.tooltip_date_format(end);
		};
		//Gantt initial scale
		if (typeof ogTasks.userPreferences.zoom_in_gantt === "undefined") {
			ogTasks.userPreferences.zoom_in_gantt = "2";
		}
		ganttManager.setScaleConfig(ogTasks.userPreferences.zoom_in_gantt);

		//Weekend color
		gantt.templates.scale_cell_class = function(date){
			if(gantt.config.scale_unit == "day" || gantt.config.scale_unit == "week"){
				if(date.getDay()==0||date.getDay()==6){
					return "weekend";
				}
			}

		};
		gantt.templates.task_cell_class = function(item,date){
			if(gantt.config.scale_unit == "day" || gantt.config.scale_unit == "week") {
				if (date.getDay() == 0 || date.getDay() == 6) {
					return "weekend";
				}
			}
		};

		gantt.templates.task_text=function(start, end, task){
			if (ogTasks.userPreferences.show_tasks_names_on_gantt){
				return task.text;
			}else{
				return "";
			}
		};
				
		//END Gantt configurations
		
		//Init Gantt
		//init events
		var menu = Ext.getCmp('menu-panel');
		menu.removeListener('collapse',ganttManager.resizeGanttContainer);
		menu.removeListener('expand',ganttManager.resizeGanttContainer);
		//detach previous atached events
		//gantt.detachAllEvents();//Do not use this because this detach all events so for example gantt tooltip crash
		$.each(ganttManager.eventsIdsForDetach, function(index, event_id) {
			gantt.detachEvent(event_id);					
		});
		//clear events id array
		ganttManager.eventsIdsForDetach.length = 0;
		
		ganttManager.initGanttEvents();
		//END init events
		
		var container_id = ganttWrapper.genid + "ganttChartContainer";
		gantt.init(container_id);
						
		//Load all groups with one task
		ganttManager.loadTaskListToGantt();

		// 'Collapse' or 'Expand' groups
		if(ogTasks.userPreferences.collapse_gantt_groups) {
			ganttManager.toggleAllGroups(true);
		} else {
			ganttManager.toggleAllGroups(false);
		}
		
		//Interval load more tasks until complete all tasks
		ganttManager.intervalId =setInterval(function(){ganttManager.loadMoreTasks()},50);
		//END Init Gantt
	},
	
	//Load all groups with one task per group 
	loadTaskListToGantt : function(){
		//remove all tasks from gantt
		gantt.clearAll();
		
		og.loading();
		
		//clear list
		ganttManager.isLoaded.length = 0;
		
		//add task list to gantt
		var list = ganttTasks.getInitalGanttTasksToAdd();
				
		//load all the group with one child
		var allGroups = {"data":[]};
		$.each(list.data, function(index, task) {
			if(task.type == gantt.config.types.project){
				ganttManager.isLoaded.push(task.id);
				ganttManager.isLoaded.push(list.data[index+1].id);
				allGroups.data.push(task);
				allGroups.data.push(list.data[index+1]);
			}						
		});
		gantt.parse(allGroups);
		allGroups.data.length = 0;
	},
	
	//Load more tasks to gantt //Use this function after call loadTaskListToGantt for the first load.
	loadMoreTasks : function(intervalId){
		var list = ganttTasks.allTasks;
		var count = 1;
		var no_adds = true;
		$.each(list.data, function(index, task) {
			if (count > 0) {
				if(ganttManager.isLoaded.indexOf(task.id) == -1){ 
					gantt_extend.addTask(task);
					ganttManager.isLoaded.push(task.id);
					count = count - 1;
					no_adds = false;
				}					
			}	
			
		});
		
		//Render gantt tasks after load 100 tasks
		if(ganttManager.isLoaded.length % 100 == 0){
			gantt.refreshData();
			gantt._adjust_scales();
		}

		//END LOAD
		if(no_adds){
			//add links before add all tasks
			$.each(ganttTasks.allTasks.links, function(index, link) {
				//check target task exist
                var s = ganttTasks.getTaskByObjectId(link.source);
                var t = ganttTasks.getTaskByObjectId(link.target);
                if (s && t) {
                    if(gantt.isTaskExists(s.id)){
                        link.source = s.id;
                        link.target = t.id;
                        gantt.addLink(link);
                    }
                }

			});
			
			//Remove interval
			window.clearInterval(ganttManager.intervalId);
			ganttManager.intervalId = null;
			
			//Render gantt tasks
			gantt.refreshData();
			gantt._adjust_scales();
			
			og.hideLoading();
			
			ganttManager.initGanttEventsAfterCompleteTaskLoad();
			
		}		
	},
	
	//Events
	initGanttEvents : function() {
		var menu = Ext.getCmp('menu-panel');
		menu.addListener('collapse',ganttManager.resizeGanttContainer, this);
		menu.addListener('expand',ganttManager.resizeGanttContainer, this);
		if(!ganttManager.isInitialized){
			//attach events only once
			//gantt ready
			gantt.attachEvent("onGanttReady", function(){
					ganttManager.isInitialized = true;
			});
							
			//task double click
			gantt.attachEvent("onTaskDblClick", function(id,e){
				var taskObj = gantt.getTask(id);
				//if is a task show the task view
				if(taskObj.$rendered_type == "task"){
					var viewUrl = og.getUrl('task', 'view', {id: taskObj.object_id});
					og.openLink(viewUrl);				
				}			
			});
				
			//after task update
			gantt.attachEvent("onAfterTaskUpdate", function(id,item){
				ganttTasks.updateTaskOnServer(item);
			});
				
			//before task drag
			gantt.attachEvent("onBeforeTaskDrag", function(id, mode, e){
				var taskObj = gantt.getTask(id);
				//if is a project
				if(mode =="progress"& taskObj.type == gantt.config.types.project){
					return false;
				}
				return true;
			});
				
			//After Task Drag
			gantt.attachEvent("onAfterTaskDrag", function(id, mode, e){
				var taskObj = gantt.getTask(id);
						
				//complete or open the task on the server
				if(mode == 'progress'){
					ganttTasks.completeOrOpenTask(taskObj);		
				}						
			});
				
			//Before Link Add
			gantt.attachEvent("onBeforeLinkAdd", function(id,link){
				var types = gantt.config.links;
				var taskObjSource = gantt.getTask(link.source);
				var taskObjTarget = gantt.getTask(link.target);
					
				//check projects
				if(taskObjSource.type == gantt.config.types.project || taskObjTarget.type == gantt.config.types.project){
					return false;
				}
				if(link.type == types.finish_to_finish) 
				return true;
				else
				return false;
			});
				
			//After Link Delete
			gantt.attachEvent("onAfterLinkDelete", function(id,item){
				ganttTasks.deleteTaskDependencyOnServer(item.source, item.target);
			});		
				
			/* enable this block for drag childs when drag parent task
			//on task drag
			gantt.attachEvent("onTaskDrag", function(id, mode, task, original){
			    var modes = gantt.config.drag_mode;
			    //drag childs too
			    if(mode == modes.move){
			        var diff = task.start_date - original.start_date;
			        gantt.eachTask(function(child){
			            child.start_date = new Date(+child.start_date + diff);
			            child.end_date = new Date(+child.end_date + diff);
			            gantt.refreshTask(child.id, true);			            
			        },id );
			    }
			    return true;
			});
				
			//after task drag
			gantt.attachEvent("onAfterTaskDrag", function(id, mode, e){
				//update subtasks in gantt and on the server
				gantt.eachTask(function(child){
				gantt.updateTask(child.id); //renders the updated task
				},id );
			});
			*/
		}
		
	},
	
	//Events to init after tasks list completely loaded
	eventsIdsForDetach : new Array(),
	initGanttEventsAfterCompleteTaskLoad: function() {
		//After Link Add
		ganttManager.eventsIdsForDetach.push( 
			gantt.attachEvent("onAfterLinkAdd", function(id,item){
				ganttTasks.addTaskDependencyOnServer(item.source, item.target);
			})
		);
		
		// render breadcrumbs
		og.eventManager.fireEvent('replace all empty breadcrumb', null);
	},
	
	//This function is used to resize gantt after menu-panel collapsed or expanded
	resizeGanttContainer: function() {
		var currentPanel = Ext.getCmp('tabs-panel').getActiveTab();
		if (currentPanel.id == 'tasks-panel') {
			if(ogTasks.userPreferences.showTasksListAsGantt){
				gantt.render();
				return;
			}			
		}		
	},
	
	//Gantt Zoom
	setScaleConfig : function(value){
		// ogTasks.userPreferences.zoom_in_gantt = value;
		switch (value) {
			//Zoom Day
			case "1":
				gantt.config.scale_unit = "day";
				gantt.config.date_scale = "%d %M";
				gantt.config.min_column_width = 70;

				gantt.config.scale_height = 54;

				gantt.config.subscales = [
					{unit:"hour", step:3, date:"%H:%i"}
				];
				break;
			//Zoom Week
			case "2":
				var weekScaleTemplate = function(date){
					var dateToStr = gantt.date.date_to_str("%d %M");
					var endDate = gantt.date.add(gantt.date.add(date, 1, "week"), -1, "day");
					return dateToStr(date) + " - " + dateToStr(endDate);
				};

				gantt.config.scale_unit = "week";
				gantt.config.step = 1;
				gantt.templates.date_scale = weekScaleTemplate;
				gantt.config.subscales = [
					{unit:"day", step:1, date:"%D" }
				];
				gantt.config.scale_height = 50;
				break;
			//Zoom Month
			case "3":
				gantt.config.scale_unit = "month";
				gantt.config.date_scale = "%F, %Y";
				gantt.config.min_column_width = 120;
				var weekScaleTemplate = function(date){
					var dateToStr = gantt.date.date_to_str("%d %M");
					var endDate = gantt.date.add(gantt.date.add(date, 1, "week"), -1, "day");
					return dateToStr(date) + " - " + dateToStr(endDate);
				};

				gantt.config.subscales = [
					{unit:"week", step:1, template:weekScaleTemplate}					
				];
				gantt.config.scale_height = 50;
				gantt.templates.date_scale = null;
				break;
			//Zoom Year
			case "4":
				gantt.config.scale_unit = "year";
				gantt.config.step = 1;
				gantt.config.date_scale = "%Y";
				gantt.config.min_column_width = 50;

				gantt.config.scale_height = 90;
				gantt.templates.date_scale = null;

				var monthScaleTemplate = function(date){
					var dateToStr = gantt.date.date_to_str("%M");
					var endDate = gantt.date.add(date, 2, "month");
					return dateToStr(date) + " - " + dateToStr(endDate);
				};

				gantt.config.subscales = [
					{unit:"month", step:3, template:monthScaleTemplate},
					{unit:"month", step:1, date:"%M" }
				];
				break;
		}
	},
	
	//Gantt show task names
	toggleTasksNames : function(show){
		gantt.templates.task_text=function(start, end, task){
			if (ogTasks.userPreferences.show_tasks_names_on_gantt){
				return task.text;
			}else{
				return "";
			}
		};
		gantt.render();
	},
	
	//Gantt show left grid
	toggleLeftGrid : function(){
		if (ogTasks.userPreferences.show_left_grid_on_gantt){
			//gantt.config.grid_width = ganttWrapper.defaultLeftGridWidth;
			gantt.config.grid_width = 250 + (gantt.config.columns.length - 1) * 125;
		}else{
			gantt.config.grid_width = 0;
		}		
		gantt.render();
	},
	
	//Expand or colapse all groups in gantt 
	toggleAllGroups : function(expand) {
		var groupsIds = gantt.getChildren("0");
		$.each(groupsIds, function(index, id) {
			if(expand){
				gantt.open(id);
			}else{
				gantt.close(id);
			}			
		});
	},
	
	//This function is used show gantt in full screen mode
	fullScreenMode: function() {
		var container_id = ganttWrapper.genid + "ganttChartContainer";
		function showElementInFullScreen(element){
			if(element.requestFullscreen) {
		    element.requestFullscreen();
		  } else if(element.mozRequestFullScreen) {
		    element.mozRequestFullScreen();
		  } else if(element.webkitRequestFullscreen) {
		    element.webkitRequestFullscreen();
		  } else if(element.msRequestFullscreen) {
		    element.msRequestFullscreen();
		  }
		}
		showElementInFullScreen($( "#"+container_id )[ 0 ]);
		gantt.render();
	},
	
	
	buildDimensionColumns: function () {
		var columns = [];
		
		for (did in og.dimensions_info) {
			if (isNaN(did)) continue;
			if (ogTasks.ganttPreferences.ganttShowDimensionCols.indexOf(did) != -1) {
				columns.push({
					id: "dim_"+did,
					name: "dim_"+did, 
					align: "left", 
					width: '125',
					label: og.dimensions_info[did].name
				});
			}
			
			if (ogTasks.list_dimension_column_hooks) {
				var dim_items = [];
				for (var j=0; j<ogTasks.list_dimension_column_hooks.length; j++) {
					var fn = ogTasks.list_dimension_column_hooks[j];
					if (typeof(fn) == 'function') {
						more_items = fn.call(null, did);
						if (more_items && more_items.length > 0) {
							dim_items = dim_items.concat(more_items);
						}
					}
				}
				
				for (var j=0; j<dim_items.length; j++) {
					var col = dim_items[j];
					if (ogTasks.ganttPreferences.ganttShowDimensionCols.indexOf(col.value) != -1) {
						columns.push({
							id: "dim_"+col.value,
							name: "dim_"+col.value, 
							align: "left", 
							width: '125',
							label: col.text
						});
					}
				}
			}
		}
		
		return columns;
	}
};
