//GanttTasks object definition
var ganttTasks = {
		//Store all gantt tasks in data and all links in links
		allTasks : {"data":[],"links":[]},
		
		//Load the gantt task list from ogTasks and return it
		loadTasksList : function(){	
			//clear previous data
			ganttTasks.allTasks.data.length = 0;
			ganttTasks.allTasks.links.length = 0;
			
			//estimate num of tasks in first level 
			var estimated_total_tasks = 0;
			$.each(ogTasks.Groups, function(index, group) {
				estimated_total_tasks += group.group_tasks.total;
			});
			//if estimated_total_tasks > 150 colapse all groups
			var colapse_groups = false;
			if(estimated_total_tasks > 150){
				colapse_groups = true;
			}
			
			//add groups
			$.each(ogTasks.Groups, function(index, group) {
				//we add the group as a gantt task project
				ganttTasks.addTaskProject(group, colapse_groups);				
			});
			return ganttTasks.allTasks;
		},
		
		getTaskByObjectId: function(object_id) {
			if (this.allTasks && this.allTasks.data) {
				for (var i=0; i<this.allTasks.data.length; i++) {
					var t = this.allTasks.data[i];
					if (t && t.type == 'task' && t.object_id == object_id) {
						return t;
					}
				}
			}
		},
		
		//Convert a task dependency to a Gantt link and add to the list allTasks
		//Gantt Link Example {id:1, source:1, target:2, type:"2"}, type 2 = "finish_to_finish" link.
		addGanttLink : function(source_id, target_id){
				var linkData = {
				    id: source_id + '_'+ target_id,
				    source: source_id,
				    target: target_id,
				    type: gantt.config.links.finish_to_finish
				};
				ganttTasks.allTasks.links.push(linkData);
		},
		
		//Convert a task to a Gantt task
		//Gantt Task Example {"id":12, "text":"Task #1", "start_date":"03-04-2013", "end_date":"05-04-2013", "parent":"11", "progress": 1, "open": true}
		toGanttTask : function(task, parentId, groupId){
			//VARIABLES		
			var realDuration = 0;
			var duration = null;
			var startDate = null;
			var dueDate = null;
			var defaultDuration = 1 * 60;//in minutes. When a task only have a start date or a due date, set duration =  defaultDuration.
			//END VARIABLES	
			
			//CHECK DATES
			//TimeEstimate parseInt
			if (parseInt(task.TimeEstimate) > 0) {
				duration = parseInt(task.TimeEstimate) ;
				realDuration = parseInt(task.TimeEstimate) ;
			}
			if (task.startDate != null) {
				startDate = ganttTasks.formatDate(task.startDate);
			}
			if (task.dueDate != null) {
				dueDate = ganttTasks.formatDate(task.dueDate);
			}
			//if start date
			if (startDate != null) {
				//if start date and due date
				if (dueDate != null) {
					duration = (dueDate.getTime() - startDate.getTime()) / (1000 * 60);
					//due date and start date can be the same or due date can be lower 
					if(duration <= 0){
						if(realDuration > 0){
							duration = realDuration;
						}else{
							duration = defaultDuration;
						}
					}										
				//if only start date
				}else if(duration == null || duration <= 0){
					duration = defaultDuration;
				}
				
				dueDate = new Date(startDate.valueOf() + duration * 60 * 1000);
								
			//if not start date	
			} else {
				//if due date 
				if (dueDate != null) {
					//if only due date
					if (duration == null || duration <= 0) {
						duration = defaultDuration;
					}
															
					startDate = new Date(dueDate.valueOf() - duration * 60 * 1000);
				//if not start date and not due date
				} else {
					$('div.gantt_alert').show();
					setTimeout(function(){$('div.gantt_alert').hide()},10000);
					return false;
				}
			}
			//END CHECK DATES
			
			var assigned_to_name = '';
			var user = og.allUsers[task.assignedToId];
			if (user) {
				assigned_to_name = user.name;
			}
			
			//SET THE GANTT TASK DATA
			var taskData = {
			    id: task.id +"-"+ groupId, // add the groupId to the id to allow duplicated tasks in different groups
			    object_id: task.id,
			    text: task.title,
			    start_date: startDate,
			    end_date: dueDate,
			    progress: (task.percentCompleted/100),
			    type: gantt.config.types.task,
			    open:true,
			    assigned_to: assigned_to_name,
			    estimated_time: task.estimatedTime,
			    pending_time: task.pending_time_string,
			    worked_time: task.worked_time_string,
			    percent_completed: task.percentCompleted + ' %'
			};
			
			if (task.custom_properties) {
				for (var cp_order=0; cp_order<task.custom_properties.length; cp_order++) {
					cpval = task.custom_properties[cp_order];
					if (cpval) {
						taskData['cp_' + cpval.id] = cpval.value;
					}
				}
			}
			
			// dimension columns
			var dim_cols = ganttTasks.addDimensionColumns(task);
			for (var k in dim_cols) {
				if (k != 0) taskData[k] = dim_cols[k];
			}
			
			//If have parent
			if(typeof parentId != "undefined"){
				taskData["parent"] = parentId;
			}
			//END SET THE GANTT TASK DATA
			
			//Add custom attributes to the gantt task
			if (task.completedById != null) {
				taskData["customAttCompleted"] = true;
				//if task is completed put the progress at 100% because task colors
		    	taskData.progress = 1;
		    }else{
		    	taskData["customAttCompleted"] = false;
		    	//if task is not completed do not put the progress at 100% because task colors
		    	if(taskData.progress == 1){
		    		taskData.progress = 0.99;
		    	}		    	
		    }			
			//END Add custom attributes to the gantt task
			
			return taskData;
		},
		
		
		addDimensionColumns: function(task) {
			if (!task) return;
			var columns = {};
			
			var mpath = Ext.util.JSON.decode(task.memPath);
			if (!mpath) return;
			
			for (var k=0; k<ogTasks.ganttPreferences.ganttShowDimensionCols.length; k++) {
				var did = ogTasks.ganttPreferences.ganttShowDimensionCols[k];
				if (!isNaN(did)) {
					columns['dim_' + did] = og.dimColEmptyBreadcrumb(did, task.memPath);
				}
				
				if (ogTasks.override_task_dim_col_value && ogTasks.override_task_dim_col_value.length > 0) {
					for (var i=0; i<ogTasks.override_task_dim_col_value.length; i++) {
						var fn = ogTasks.override_task_dim_col_value[i];
						if (typeof(fn) == 'function') {
							var result = fn.call(null, did, mpath);
							if (result) {
								dim_mpath = result.dim_mpath;
								exclude_parents_path = result.exclude_parents_path;
								
								columns['dim_' + did] = og.getEmptyCrumbHtml(dim_mpath, ".task-breadcrumb-container", null, null, exclude_parents_path);
							}
						}
					}
				}
			}
			return columns;
		},
		
		
		//Convert a group to a Gantt task Project
		//Gantt Task Project {"id":11, "text":"Project #1", type:gantt.config.types.project, "progress": 0.6, "open": true},
		toGanttTaskProject : function(group, colapse_group){
			//VARIABLES			
			var open = !colapse_group;	
			//END VARIABLES	
									
			//SET THE GANTT TASK PROJECT DATA
			var projectData = {
			    id: "group_"+group.group_id,//group_id,
			    text: group.group_name,
			    type: gantt.config.types.project,
			    progress: 0,
			    open: open,
			    assigned_to: '',
			    estimated_time: '',
			    pending_time: '',
			    worked_time: '',
			    percent_completed: ''
			};
			
			// custom property columns
			for (var cp_order=0; cp_order < ogTasks.custom_properties.length; cp_order++) {
				projectData['cp_' + ogTasks.custom_properties[cp_order].id] = '';
			}
			
			// dimension columns
			for (var k=0; k<ogTasks.ganttPreferences.ganttShowDimensionCols.length; k++) {
				var did = ogTasks.ganttPreferences.ganttShowDimensionCols[k];
				projectData['dim_' + did] = '';
			}
			
			//END SET THE GANTT TASK PROJECT DATA
			
			//For calculate total progress of the group only
			projectData.total_childs_progress = 0;
			projectData.total_childs = 0;
			
			return projectData;
		},
		
		//Add a task and all subtasks to the list allTasks for be load to gantt before
		//return true if can add the task or one of the subtasks
		addTask : function(task, parentId, projectIndex){
			var ret = false;
			var ganttTask = ganttTasks.toGanttTask(task, parentId, projectIndex);
			var id = parentId;
			if(ganttTask){
				ganttTasks.allTasks.data.push(ganttTask);
				id = task.id;
				ret = true;
				
				//Add info to calculate the progress of the group later
				ganttTasks.allTasks.data[projectIndex].total_childs_progress += ganttTask.progress;
				ganttTasks.allTasks.data[projectIndex].total_childs += 1;
			}
			//add subtasks too
			if(task.subtasks.length > 0){
				$.each(task.subtasks, function(index, subTask) {
					if(ganttTasks.addTask(subTask, id, projectIndex)){
						ret = true;
					};
				});
			}
			
			//add task links
			if(ret){				
					var dependants = task.dependants;
					$.each(dependants, function(index, id) {
						var dep_id = parseInt(id);
						if(!isNaN(dep_id) && dep_id > 0){
							ganttTasks.addGanttLink(dep_id, parseInt(task.id));
						};
					});				
			}
			return ret;
		},
		
		//Add a project and all tasks to the list allTasks for be load to gantt before
		addTaskProject : function(group, colapse_group){
			var ret = false;
			var ganttTaskProject = ganttTasks.toGanttTaskProject(group, colapse_group);
			
			var projectIndex = ganttTasks.allTasks.data.length;
			ganttTasks.allTasks.data.push(ganttTaskProject);
			
			//add tasks from the group
			for (var k=0; k<group.group_tasks_order.length; k++){
				var key = group.group_tasks_order[k];
				if (isNaN(key)) continue;
				var task = group.group_tasks[key];
				if(task && ganttTasks.addTask(task, ganttTaskProject.id, projectIndex)){
					ret = true;
				};
			}	
						
			//if no tasks of the group are added remove de group
			if(!ret){
				ganttTasks.allTasks.data.pop();
			}else{
				//Calculate the progress of the group
				ganttTaskProject.progress = ganttTaskProject.total_childs_progress / ganttTaskProject.total_childs;
			}
			
			return ret;
		},
		
		//Convert unix time in local time to javascript date in local time
		formatDate : function(taskDate) {
			// getTimezoneOffset is in minutes. convert to milliseconds //GMT+2 return -120 
			var offset = (new Date().getTimezoneOffset())*60;
			// multiplied by 1000 so that the argument is in milliseconds, not seconds
			var date = new Date((taskDate + offset) * 1000);
						
			return date;
		},
		
		//Convert javascript date in local time to obj that contains strings date and time.{date : date, time : time}
		formatDateToSendToServer : function(taskDate) {
			var hm_format = og.preferences['time_format_use_24'] == 1 ? 'G:i' : 'g:i A';
			var date = taskDate.format(og.preferences['date_format']);
			var time = taskDate.format(hm_format);
			
			out = {date : date, time : time};
			return out;
		},
		
		//Return the inital list of gantt tasks and links for add to the gantt
		getInitalGanttTasksToAdd : function() {
			ganttTasks.loadTasksList();
			
			return ganttTasks.allTasks;
		},
		
		//update task data on the server
		updateTaskOnServer : function(ganttTask){
			var startDate = ganttTask.start_date;
			var dueDate   = ganttTask.end_date;
			
			startDate = ganttTasks.formatDateToSendToServer(startDate);
			dueDate   = ganttTasks.formatDateToSendToServer(dueDate);
			
			var params 				 = {id : ganttTask.object_id};
			params.task_due_date 	 = dueDate.date;
			params.task_due_time 	 = dueDate.time;
			params.task_start_date 	 = startDate.date;
			params.task_start_time 	 = startDate.time;
			params.percent_completed = ''+(ganttTask.progress * 100)+'';
			params.keep_members = 1;
			
			//update data on the server
			ganttTasks.quickEdit(ganttTask.object_id, params);
		},
		
		//send new task data to the server
		quickEdit : function(task_id, parameters) {
			var params = [];
			for ( var i in parameters) {
				if (parameters[i] || parameters[i] === 0) {
					params["task[" + i + "]"] = parameters[i];
				}
			}
						 
			//Url		
			var url = og.getUrl('task', 'quick_edit_task', {
				id : task_id,
				dont_mark_as_read : 1
			});
			params["task[notify]"] = false;
			//Send
			og.openLink(url, {
				method : 'POST',
				post : params,
				callback : function(success, data) {
					if (success && !data.errorCode) {
						//success
					} else {
						if (!data.errorMessage || data.errorMessage == '') {
							og.err(lang("error adding task"));
						}
					}
				},
				scope : ogTasks
			});
			//END Send

		},
		
		//complete or open the task on the server
		completeOrOpenTask : function(ganttTask) {
			//complete task
			var percent_completed = ''+(ganttTask.progress * 100)+'';
			var url_action = '';
			if(percent_completed == '100'){
				if(ganttTask.customAttCompleted == true){ 
					return;
				}
				ganttTask.customAttCompleted = true;
				url_action = 'complete_task';
			}else{
				if(ganttTask.customAttCompleted == false){ 
					return;
				}
				ganttTask.customAttCompleted = false;
				url_action = 'open_task';
			}
			
			//open_task					 
			//Url		
			var url = og.getUrl('task', url_action, {
				id : ganttTask.object_id,
				quick : true
			});

			//Send
			og.openLink(url, {
				method : 'POST',
				callback : function(success, data) {
					if (success && !data.errorCode) {
						//success
					} else {
						if (!data.errorMessage || data.errorMessage == '') {
							og.err(lang("error adding task"));
						}
					}
				},
				scope : ogTasks
			});
			//END Send

		},
		
		//Delete task dependency on the server
		deleteTaskDependencyOnServer : function(previous_task_id,task_id){
			
			og.openLink(og.getUrl('taskdependency', 'remove', {pt:previous_task_id, t:task_id, reload:0}));
			
		},
		
		//Add task dependency on the server
		addTaskDependencyOnServer : function(previous_task_id,task_id){
			
			og.openLink(og.getUrl('taskdependency', 'add', {pt:previous_task_id, t:task_id, reload:0}));
			
		}
};
