<?php
Hook::register('hour_types');

function hour_types_object_definition($type, &$ret) {
	if ($type == 'Contact') {
		$ret['default_hour_type_id'] = DATA_TYPE_INTEGER;
	}
}


function hour_types_additional_system_columns($manager, &$columns) {
	if ($manager instanceof Contacts) {
		$columns[] = 'default_hour_type_id';
	}
}


function hour_types_before_render_main_custom_properties($params, &$categories) {

	$object = array_var($params, 'object');
	//$genid = array_var($params, 'genid');
	$genid = gen_id();

	if ($object instanceof Contact) {
		
		if ($object->getUserType() > 0 || ($object->isNew() && array_var($_REQUEST, 'is_user') > 0)) {
			
			$dimension = Dimensions::findByCode('hour_types');
			$default_member_id = $object->getColumnValue('default_hour_type_id');
			
			echo '<div class="dataBlock">';
			render_single_member_selector($dimension, $genid, array($default_member_id), array(
					'hidden_field_name' => 'contact[default_hour_type_id]',
					'label' => lang('default hour type'),
					'is_multiple' => false,
			));
			echo '</div>';
		}
		
	}
}

// Save the default labor category only when using the add/edit user form
function hour_types_after_add_contact($user, &$ret) {
	if ($user instanceof Contact && $user->isUser()) {
		$labor_category_controller = new HourTypeController();
		$labor_category_controller->fill_user_default_labor_category_from_request($user);
	}
}

// Save the default labor category only when using the add/edit user form
function hour_types_after_edit_contact($user, &$ret) {
	if ($user instanceof Contact && $user->isUser()) {
		$labor_category_controller = new HourTypeController();
		$labor_category_controller->fill_user_default_labor_category_from_request($user);
	}
}


function hour_types_after_contact_view($contact, &$ret) {
	
	if ($contact->getUserType() > 0 && $contact->getColumnValue('default_hour_type_id') > 0) {
		
		$mem = Members::instance()->findById($contact->getColumnValue('default_hour_type_id'));
		if ($mem instanceof Member) {
			
			$member_breadcrumb = '<span class="member-path real-breadcrumb og-wsname-color-'.$mem->getColor().'">'.$mem->getName().'</span>';
			
			$html = '<div class="basic-info">
						<div class="info-type">'. lang('default hour type') .'</div>
						<div class="info-content">
							<div class="info-content-item">'.$member_breadcrumb.'</div>
						</div>
						<div class="clear"></div>
					</div>';
			echo $html;
		}
	}
	
}

function hour_types_app_log_detail_property_format($params, &$formatted) {
	$object = array_var($params, 'object');
	$property = array_var($params, 'property');
	$value = array_var($params, 'value');

	if ($object instanceof Contact && $object->isUser()) {
		$member = Members::getMemberById($value);
		if ($member instanceof Member) {
			$formatted = '<span class="member-path real-breadcrumb og-wsname-color-'.$member->getColor().'">'.$member->getDisplayName().'</span>';
		}
	}
}


function hour_types_preselected_time_form_member_ids($params, &$pre_selected_member_ids) {
	$timeslot = array_var($params, 'object');
	
	if ($timeslot instanceof Timeslot) {
		
		$ts_contact = Contacts::instance()->findById($timeslot->getContactId());
		if ($ts_contact instanceof Contact && $ts_contact->getColumnValue('default_hour_type_id') > 0) {
			
			if (!is_array($pre_selected_member_ids)) $pre_selected_member_ids = array();
			
			$hour_type_set = false;
			if (count($pre_selected_member_ids) > 0) {
				$ht_dimension = Dimensions::findByCode('hour_types');
				foreach ($pre_selected_member_ids as $mid) {
					$m = Members::instance()->findById($mid);
					if ($m instanceof Member && $m->getDimensionId() == $ht_dimension->getId()) {
						$hour_type_set = true;
						break;
					}
				}
			}
			if (!$hour_type_set) {
				$pre_selected_member_ids[] = $ts_contact->getColumnValue('default_hour_type_id');
			}
		}
	}
}

function hour_types_after_api_add_timeslot($params, &$modified) {
	$timeslot = array_var($params, 'timeslot');
	
	if ($timeslot instanceof Timeslot) {
	
		$ts_contact = Contacts::instance()->findById($timeslot->getContactId());
		if ($ts_contact instanceof Contact && $ts_contact->getColumnValue('default_hour_type_id') > 0) {
			$hour_type_set = false;
			$ht_dimension = Dimensions::findByCode('hour_types');
			
			$ts_members = $timeslot->getMembers();
			foreach ($ts_members as $m) {
				if ($m instanceof Member && $m->getDimensionId() == $ht_dimension->getId()) {
					$hour_type_set = true;
					break;
				}
			}
			if (!$hour_type_set) {
				$hour_type_member = Members::getMemberById($ts_contact->getColumnValue('default_hour_type_id'));
				if ($hour_type_member instanceof Member) {
					ObjectMembers::addObjectToMembers($timeslot->getId(), array($hour_type_member));
					$modified = true;
				}
			}
		}
	}
}

function hour_types_after_additional_members_added_to_timeslot($params, &$ts_member_ids) {
	$user_id = array_var($params, 'user_id');
	$ts_contact = Contacts::instance()->findById($user_id);
	
	if ($ts_contact instanceof Contact && $ts_contact->getColumnValue('default_hour_type_id') > 0) {
		$hour_type_set = false;
		$ht_dimension = Dimensions::findByCode('hour_types');
		
		foreach ($ts_member_ids as $mem_id) {
			$m = Members::instance()->findById($mem_id);
			if ($m instanceof Member && $m->getDimensionId() == $ht_dimension->getId()) {
				$hour_type_set = true;
				break;
			}
		}

		$active_context = active_context();
		if (!$hour_type_set) {
			foreach ($active_context as $member) {
				if ($member instanceof Member && $member->getDimensionId() == $ht_dimension->getId()) {
					$ts_member_ids[] = $member->getId();
					$hour_type_set = true;
					break;
				}
			}
		}

		if (!$hour_type_set) {
			$hour_type_member = Members::getMemberById($ts_contact->getColumnValue('default_hour_type_id'));
			if ($hour_type_member instanceof Member) {
				$ts_member_ids[] = $hour_type_member->getId();
			}
		}
	}
	
}


/**
 * This function is used when grouping timeslots by person for invoicing
 * hour types plugin is used then the hour type (labor category) member id will be appended to the grouping key
 */
function hour_types_override_timeslot_person_key_for_grouping($t, &$key) {
	$dim = Dimensions::findByCode('hour_types');
	$members = $t->getMembers();
	foreach ($members as $m) {
		if ($m instanceof Member && $m->getDimensionId() == $dim->getId()) {
			$key .= "-".$m->getId();
			break;
		}
	}
}



function hour_types_after_render_custom_property_input($params, &$html) {
	
	$cp = array_var($params, 'cp');/* @var $cp CustomProperty */
	$config = array_var($params, 'config');
	
	if ($cp->getCode() == 'is_time_off') {
		$hour_type_ot = ObjectTypes::findByName('hour_type');
		if ($hour_type_ot->getId() == $cp->getObjectTypeId()) {

			$time_off_code_cp = MemberCustomProperties::getCustomPropertyByCode($hour_type_ot->getId(), 'tsheets_time_off_code');
			if ($time_off_code_cp instanceof MemberCustomProperty && !$time_off_code_cp->getIsDisabled()) {

				tpl_assign('cp_genid', $config['genid']);
				tpl_assign('cp', $cp);
				tpl_assign('time_off_code_cp', $time_off_code_cp);
				
				$html .= tpl_fetch(get_template_path('show_hide_time_off_code_cp', 'hour_type', 'hour_types'));
			}
			
		}
	}
}