<?php

/**
 * InvoiceController
 * Generado el 10/5/2012
 *
 */
class InvoiceController extends ApplicationController
{

    var $plugin_name = "income";

    /**
     * Construct the InvoiceController
     *
     * @access public
     * @param void
     * @return InvoiceController
     */
    function __construct()
    {
        parent::__construct();
        prepare_company_website_controller($this, 'website');
        if (Plugins::instance()->isActivePlugin('multiple_currencies')) {
            Env::useHelper('functions', 'multiple_currencies');
        }
        Env::useHelper('functions', 'income');
        Env::useHelper('invoice_mail_templates', 'income');
        Env::useHelper('dimension');
    }

    // ---------------------------------------------------
    //  Index
    // ---------------------------------------------------

    function init()
    {
        require_javascript("InvoiceManager.js", $this->plugin_name);

		$config = array(
			'allow_invoice_flexible_status_change' => income_config_option("allow_invoice_flexible_status_change"),
			'can_mark_as_paid' => logged_user()->isAdminGroup(),
		);

        ajx_current("panel", "invoices-containerpanel", null, $config, true);
        ajx_replace(true);
    }

    function list_all($only_return_objects = false)
    {
        ajx_current("empty");
        // Get all variables from request
        $start = array_var($_GET, 'start', 0);
        $limit = array_var($_GET, 'limit', config_option('files_per_page'));
        $order = array_var($_GET, 'sort');
        $order_dir = array_var($_GET, 'dir');
        $action = array_var($_GET, 'action');
        $attributes = array(
            "ids" => explode(',', array_var($_GET, 'ids')),
            "types" => explode(',', array_var($_GET, 'types')),
            "accountId" => array_var($_GET, 'account_id'),
        );

        //Resolve actions to perform
        $actionInvoice = array();
        if (isset($action)) {
            $actionInvoice = $this->resolveAction($action, $attributes);
            if ($actionInvoice["errorCode"] == 0) {
                flash_success($actionInvoice["errorInvoice"]);
            } else {
				evt_add('show error message', array('message' => $actionInvoice["errorInvoice"]));
            }
        }

		$cp_order = null;
		$dim_order = null;

        switch ($order) {
        	case 'dateUpdated':
            case 'updatedOn':
                $order = '`updated_on`';
                break;
            case 'createdOn':
                $order = '`created_on`';
                break;
            case 'name':
                $order = '`name`';
                break;
            case 'date':
				$order = array('`e`.`date`', '`e`.`object_id`'); // add object id to order clause to follow some order for invoices in the same date
                break;
			case 'expiration_date':
				$order = array('`e`.`expiration_date`', '`e`.`object_id`'); // add object id to order clause to follow some order for invoices in the same date
				break;
            case 'total':
				$order = array('`e`.`total`', '`e`.`object_id`'); // add object id to order clause to follow some order for invoices with same total
                break;
            case 'subtotal':
				$order = array('`e`.`subtotal`', '`e`.`object_id`'); // add object id to order clause to follow some order for invoices with same subtotal
                break;
            case 'status':
				$order = array('`e`.`status`', '`e`.`object_id`'); // add object id to order clause to follow some order for invoices with same status
                break;
            case 'sent':
                $order = array('`e`.`sent`', '`e`.`object_id`'); // add object id to order clause to follow some order for invoices with same sent val
                break;
            case 'client':
                $order = array('`e`.`customer_name`', '`e`.`object_id`'); // add object id to order clause to follow some order for invoices with same client
                break;
            case 'invoice_number':
                $order = "LPAD(`invoice_number`, 20, '0')"; // use lpad to order because this field is aphanumeric
                break;
            case 'qbo_sync_status':
                $order = "`quickbooks_online_id`";
                break;
            default:
				if (str_starts_with($order, "dim_")) {
					// we are ordering by a dimension column in the list
					$dim_order = substr($order, 4);
					$order = 'dimensionOrder';
				} else if (str_starts_with($order, "cp_")) {
					// we are ordering by a custom property column in the list
					$cp_order = substr($order, 3);
					$order = 'customProp';
				} else if (IncomeInvoices::instance()->columnExists($order)) {
					// check if we can use an existing column in the invoices table to use
					$order = array('`e`.`'.$order.'`', '`e`.`object_id`'); 
				} else {
					// if can't find the order column then use updated_on
					$order = '`updated_on`';
				}
                break;
        }
        if (!$order_dir) {
            switch ($order) {
                case 'name':
                    $order_dir = 'ASC';
                    break;
                default:
                    $order_dir = 'DESC';
            }
        }


        $filters = array();
        if (array_var($_GET, 'status_filter')) $filters['status'] = array_var($_GET, 'status_filter');
        if (array_var($_GET, 'period_filter')) $filters['period'] = array_var($_GET, 'period_filter');
        if (array_var($_GET, 'from_filter')) $filters['from'] = array_var($_GET, 'from_filter');
        if (array_var($_GET, 'to_filter')) $filters['to'] = array_var($_GET, 'to_filter');        
        if (array_var($_GET, 'sent_filter', '') != '') {
        	$sent_filter_val = array_var($_GET, 'sent_filter', '');
        	if ($sent_filter_val != '') {
        		$filters['sent'] = $sent_filter_val;
        	}
        }        
		
        
        $extra_conditions = "";
        $skip_cols = array('period', 'from', 'to');
        $skip_values = array('0', '');
                
        foreach ($filters as $col => $value) {
            if ($col == 'period') {
                $filter_period = getPeriodFilter($value);
                $from_date = $filter_period['from'];
                $to_date =  $filter_period['to'];                

                if ($from_date instanceof DateTimeValue) {
                    $from_date->beginningOfDay();
                    //$from_date->advance(-1 * logged_user()->getUserTimezoneValue(), true);
                    $extra_conditions .= " AND `date` >= '" . $from_date->toMySQL() . "'";
                }

                if ($to_date instanceof DateTimeValue) {
                    $to_date->endOfDay();
                    //$to_date->advance(-1 * logged_user()->getUserTimezoneValue(), true);
                    $extra_conditions .= " AND `date` <= '" . $to_date->toMySQL() . "'";
                }
            }

            if ($col == 'from') {
                 
                $from_date = DateTimeValueLib::makeFromString($value);
                
                if ($from_date instanceof DateTimeValue) {                
                    $from_date->beginningOfDay();
                    //$from_date->advance(-1 * logged_user()->getUserTimezoneValue(), true);
                    $extra_conditions .= " AND `date` >= '" . $from_date->toMySQL() . "'";
                }
            }

            if($col == 'to') {
                
                $to_date = DateTimeValueLib::makeFromString($value);

                if ($to_date instanceof DateTimeValue) {
                    $to_date->endOfDay();
                    //$to_date->advance(-1 * logged_user()->getUserTimezoneValue(), true);
                    $extra_conditions .= " AND `date` <= '" . $to_date->toMySQL() . "'";
                }
        
            }
            if (!$value || in_array($col, $skip_cols) || in_array($value, $skip_values)) continue;            
            $extra_conditions .= " AND `$col` = " . DB::escape($value);                      
        }
		
        Hook::fire("listing_extra_conditions", null, $extra_conditions);

        $only_count_result = array_var($_GET, 'only_result', false);
        $res = IncomeInvoices::instance()->listing(array(
            'count_results' => false,
            'only_count_results' => $only_count_result,
            "order" => $order,
            "dim_order" => $dim_order,
            "cp_order" => $cp_order,
            "extra_conditions" => $extra_conditions,
            "order_dir" => $order_dir,
            "start" => $start,
            "limit" => $limit,
        ));     
        $invoices = $res->objects;

        Hook::fire("listing_extra_conditions", null, $extra_conditions);

		if ($only_return_objects) {
			return $invoices;
		}

        // Prepare response object
        $object = $this->prepareObject($invoices, $start, $limit, $res);

        Hook::fire("after_prepare_list_objects", null, $object);

        $object['filters'] = array(
            'status' => array(
                array('id' => 'pending', 'name' => lang('pending')),
                array('id' => 'printed', 'name' => lang('printed')),
                array('id' => 'confirmed', 'name' => lang('confirmed')),
                array('id' => 'canceled', 'name' => lang('canceled')),
            ),
        );
        if (Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
        	$object['filters']['status'][] = array('id' => 'part_confirmed', 'name' => lang('part_confirmed'));
        }

        ajx_extra_data($object);
        tpl_assign("listing", $object);
    }

    /**
     * Resolve action to perform
     *
     * @param string $action
     * @param array $attributes
     * @return string $invoice
     */
    private function resolveAction($action, $attributes)
    {
		$attributes["ids"] = array_filter($attributes["ids"], 'is_numeric');

        $resultInvoice = "";
        $resultCode = 0;
        switch ($action) {
        	case "markasvoid":
        		$succ = 0;
        		$err = 0;
        		for ($i = 0; $i < count($attributes["ids"]); $i++) {
        			$id = $attributes["ids"][$i];
        			$invoice = IncomeInvoices::instance()->findById($id);
        			if (!$invoice instanceof IncomeInvoice) continue;
        			if (!$invoice->getStatus() == 'canceled') continue;
        			
        			$can_void = true;
        			if (income_config_option("allow_invoice_flexible_status_change") || $invoice->getStatus() == 'printed' || $invoice->getStatus() == 'pending') {
        				if (!can_write(logged_user(), $invoice->getMembers(), $invoice->getObjectTypeId())) {
        					$can_void = false;
        				}
        			} else {
        				if (!$invoice->canEdit(logged_user())) {
        					$can_void = false;
        				}
        			}
        			
        			if ($can_void) {
        				try {
        					DB::beginWork();
        					$invoice->void();
        					DB::commit();
        					$succ++;
        				} catch (Exception $e) {
        					DB::rollback();
        					$err++;
        				}
        			} else {
        				$err++;
        			}
        		};
        		if ($err > 0) {
        			$resultCode = 2;
        			$resultInvoice = lang("could not void invoices", $err) . "<br />" . ($succ > 0 ? lang("success voiding invoices", $succ) : "");
        		} else {
        			$resultInvoice = lang("success voiding invoices", $succ);
        		}
        		
        		break;
            case "delete":
                $succ = 0;
                $err = 0;
                $cant_deleted = 0;
                $resultInvoice = "";
				$error_messages = array();

                for ($i = 0; $i < count($attributes["ids"]); $i++) {
                    $id = $attributes["ids"][$i];
                    $invoice = IncomeInvoices::instance()->findById($id);
                    if ($invoice instanceof IncomeInvoice && $invoice->canDelete(logged_user())) {
                        try {
                            DB::beginWork();
                            $invoice->trash();
                            DB::commit();
                            ApplicationLogs::createLog($invoice, ApplicationLogs::ACTION_TRASH);
                            $succ++;
                        } catch (Exception $e) {
                            DB::rollback();
                            $err++;
                        }
                    } else {
						// determine the error message for current invoice depending on its status
						if ($invoice instanceof IncomeInvoice) {
							$cant_deleted++;
							if ($invoice->getStatus() == 'confirmed' || $invoice->getStatus() == 'part_confirmed') {
								$error_messages[] = " - ".lang("error deleting paid invoice", $invoice->getObjectName());

							} else if ($invoice->getStatus() == 'canceled') {
								$cant_delete_void_invoice_message = " - ". lang("error deleting void invoice");
								// don't repeat the same error message for void invoices
								if (!in_array($cant_delete_void_invoice_message, $error_messages)) {
									$error_messages[] = $cant_delete_void_invoice_message;
								}
							}
						}

                    }
                };
                if ($err > 0 || $cant_deleted > 0) {
                    $resultCode = 2;
                    if ($cant_deleted > 0 && count($error_messages) > 0) {
						$resultInvoice .= lang("error delete objects", $cant_deleted) ."\n". implode("\n", $error_messages);
                    }
                    if ($err > 0) {
                        $resultInvoice .= lang("error delete objects", $err);
                    }
                    if ($succ > 0) {
                        $resultInvoice .= "\n" . lang("success delete objects", $succ);
                    }
                } else {
                    $resultInvoice = lang("success delete objects", $succ);
                }
                break;
            case "archive":
                $succ = 0;
                $err = 0;
                for ($i = 0; $i < count($attributes["ids"]); $i++) {
                    $id = $attributes["ids"][$i];
                    $invoice = IncomeInvoices::instance()->findById($id);
                    if ($invoice instanceof IncomeInvoice && $invoice->canEdit(logged_user())) {
                        try {
                            DB::beginWork();
                            $invoice->archive();
                            DB::commit();
                            ApplicationLogs::createLog($invoice, ApplicationLogs::ACTION_ARCHIVE);
                            $succ++;
                        } catch (Exception $e) {
                            DB::rollback();
                            $err++;
                        }
                    } else {
                        $err++;
                    }
                }; // for
                if ($err > 0) {
                    $resultCode = 2;
                    $resultInvoice = lang("error archive objects", $err) . "<br />" . ($succ > 0 ? lang("success archive objects", $succ) : "");
                } else {
                    $resultInvoice = lang("success archive objects", $succ);
                }
                break;
            default:
                $resultInvoice = lang("Unimplemented action: '" . $action . "'"); // if
                $resultCode = 2;
                break;
        } // switch
        return array("errorInvoice" => $resultInvoice, "errorCode" => $resultCode);
    }

    /**
     * Prepares return object for a list of invoices
     *
     * @param array $totMsg
     * @param integer $start
     * @param integer $limit
     * @return array
     */
    private function prepareObject($totInv, $start, $limit, $res_obj)
    {
        $object = array(
            "totalCount" => $res_obj->total,
            "start" => $start,
            "invoices" => array()
        );
        foreach ($res_obj as $k => $v) {
            if ($k != 'total' && $k != 'objects') $object[$k] = $v;
        }

        $custom_properties = CustomProperties::getAllCustomPropertiesByObjectType(IncomeInvoices::instance()->getObjectTypeId());

        $ids = array();
        for ($i = 0; $i < $limit; $i++) {
            if (isset($totInv[$i])) {
                $inv = $totInv[$i];
                if ($inv instanceof IncomeInvoice) {

                    $currency = Currencies::instance()->findById($inv->getCurrencyId());
                    $symbol = $currency instanceof Currency ? $currency->getSymbol() : "";

                    $date = $inv->getDate() instanceof DateTimeValue ? format_date($inv->getDate(), null, 0) : '';
                    $created_on = $inv->getCreatedOn() instanceof DateTimeValue ? format_datetime($inv->getCreatedOn()) : '';
                    $expiration_date = $inv->getExpirationDate() instanceof DateTimeValue ? format_date($inv->getExpirationDate(), null, 0) : '';

                    $general_info = $inv->getObject()->getArrayInfo();

                    $next_inv = $inv->isRepetitive() ? $inv->getNextInvoiceInRepetition() : null;
                    /*
                    if (income_config_option("assign_number_when_printing")) {
	                    $invoice_number = ($inv->getStatus() != 'pending' && $inv->getInvoiceNumber() != '') ? $inv->getInvoiceType() . " " . $inv->getSeries() . " " . $inv->getInvoiceNumber() : lang('not assigned');
                    } else {
                    	$invoice_number = ($inv->getInvoiceNumber() != '') ? $inv->getInvoiceType() . " " . $inv->getSeries() . " " . $inv->getInvoiceNumber() : lang('not assigned');
                    }*/
                    $invoice_number = ($inv->getInvoiceNumber() != '') ? $inv->getInvoiceType() . " " . trim($inv->getSeries() . " " . $inv->getInvoiceNumber()) : lang('not assigned');

                    $info = array(
                        "invoice_number" => $invoice_number,
                        "date" => $date,
                        "created_on"=> $created_on,
                        "expiration_date" => $expiration_date,
                    	"currency_id" => $inv->getCurrencyId(),
                        "subtotal" => $symbol . ' ' . number_format($inv->getSubtotal(), 2),
                        "total" => $symbol . ' ' . number_format($inv->getTotal(), 2),
                        "client" => $inv->getClient(),
                        "client_id" => $inv->getCustomerId(),
                        "status" => $inv->getStatus(),
                        "sent" => $inv->getSent(),
                        "attn" => $inv->getAttn(),
                        "is_repetitive" => $inv->isRepetitive() && !$next_inv instanceof IncomeInvoice,
						"paid_amount" => $inv->getPaidAmount(),
						"due_amount" => $inv->getDueAmount(),
                        "memPath" => json_encode($inv->getMembersIdsToDisplayPath()),
                    );
                    $info = array_merge($info, $general_info);
                    
                    Hook::fire('invoices_list_additional_column_values', array('invoice' => $inv), $info);

                    $ids[] = $inv->getId();

                    $object["invoices"][] = $info;

                    foreach ($custom_properties as $cp) {
                        $object["invoices"][$i]['cp_' . $cp->getId()] = get_custom_property_value_for_listing($cp, $inv);
                    }
                }
            }
        }

        $read_objects = ReadObjects::getReadByObjectList($ids, logged_user()->getId());
        foreach ($object["invoices"] as &$data) {
            $data['isRead'] = isset($read_objects[$data['object_id']]);
        }

        return $object;
    }

    // ---------------------------------------------------
    //  Invoices
    // ---------------------------------------------------

    /**
     * View single invoice
     *
     * @access public
     * @param void
     * @return null
     */
    function view()
    {
        $this->addHelper('textile');

        $invoice = IncomeInvoices::instance()->findById(get_id());
        if (!($invoice instanceof IncomeInvoice)) {
            flash_error(lang('invoice dnx'));
            ajx_current("empty");
            return;
        } // if

        if (!$invoice->canView(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        } // if
        //read object for this user
        $invoice->setIsRead(logged_user()->getId(), true);

        $taxes = "";
        $invoice_taxes = IncomeInvoiceTaxes::getInvoiceTaxes($invoice->getObjectId());
        foreach ($invoice_taxes as $inv_tax) {
            $tax_data = IncomeTaxes::instance()->findById($inv_tax->getTaxId());
            if ($inv_tax->getType() == "percentage") {
                $value = $inv_tax->getValue() . "%";
            } else {
                $value = "$" . $inv_tax->getValue();
            }
            $taxes .= " " . $tax_data->getName() . " (" . $value . "),";
        }
        tpl_assign('taxes', substr($taxes, 0, -1));
        tpl_assign('invoice', $invoice);
        tpl_assign('subscribers', $invoice->getSubscribers());
        ajx_extra_data(array("title" => $invoice->getTitle(), 'icon' => $invoice->getIconClass()));
        ajx_set_no_toolbar(true);

        ApplicationReadLogs::createLog($invoice, ApplicationReadLogs::ACTION_READ);
    }

	/**
	 * @return returns the invoice print template dedicated to the current installation, based on the installation's folder name
	 */
	private function get_invoice_print_template($invoiceData, $override_layout = null) {

        $return_file_name = null;
        $template_file = "invoice_print_custom_template.php";
		$layout = IncomeInvoiceLayouts::instance()->findById($override_layout);

        $invoiceTemplate = $invoiceData->getInvoiceTemplate();
		if ($invoiceTemplate instanceof IncomeInvoiceTemplate) {
			Localization::instance()->loadSettings($invoiceTemplate->getInvoiceTemplateLanguage(), ROOT . '/language');

			if (!$layout instanceof IncomeInvoiceLayout) {
				$layout = IncomeInvoiceLayouts::instance()->findById($invoiceTemplate->getInvoiceTemplateColumnAssigned());
			}
		}
		if (!$layout instanceof IncomeInvoiceLayout) {
			$layout = IncomeInvoiceLayouts::getByCode('original');
		}

		$return_file_name = "/tmp/".$template_file;

		file_put_contents(ROOT . $return_file_name, $layout->getContent());

		return $return_file_name;
	}

	/**
	 * @return string the date format to use when printing the invoice
	 * @param IncomeInvoice $invoice the invoice to print
	 */
	private function get_invoice_date_format(IncomeInvoice $invoice) {

		$format = user_config_option('date_format');

		$invoice_template = $invoice->getInvoiceTemplate();
		if ($invoice_template instanceof IncomeInvoiceTemplate && $invoice_template->getDateFormat() != '') {
			$format = $invoice_template->getDateFormat();
		}
		
		return $format;
	}

    /**
     * View a invoice in a printer-friendly format.
     *
     */
    function print_view($params = null)
    {

        $this->setLayout("html");
        $this->addHelper('textile');

        if (is_null($params)) {
            $id = get_id();
            $generate_preview = array_var($_REQUEST, 'generate_preview');
			$override_layout = array_var($_REQUEST, 'override_layout');
        } else {
            $id = array_var($params, 'id');
            $generate_preview = array_var($params, 'generate_preview');
			$override_layout = array_var($params, 'override_layout');
        }

        $invoices=[];
        $InvoiceChilds= IncomeInvoiceChilds::getInvoiceChild($id);
        $InvoiceParents= IncomeInvoiceChilds::getInvoiceParent($id);
        $invoice_parent='';
        foreach($InvoiceParents as $InvoiceParent)
        {
            $invoiceParent_id=$InvoiceParent->getInvoiceParent();
            $invoice_parent=IncomeInvoices::instance()->findById($invoiceParent_id);
        }

        /*** por aca deberia juntar todas las lineas de las todas las invoices!!!!!!!!!! */
        foreach($InvoiceChilds as $InvoiceChild)
        {
            $invoiveChild_id=$InvoiceChild->getInvoiceChild();
            $invoices[]=IncomeInvoices::instance()->findById($invoiveChild_id);
        }


        /*it's necesary for multi invoice, only for print if I choose a child invoice, it's should return the parent invoice*/
        if($invoice_parent!='')
        {
            $invoice=$invoice_parent;
        }else{
            $invoice = IncomeInvoices::instance()->findById($id);
        }

        if (!($invoice instanceof IncomeInvoice)) {
            flash_error(lang('invoice dnx'));
            ajx_current("empty");
            return;
        } // if

        if (!$invoice->canView(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        } // if

        $status_changed = false;
        if ($invoice->getStatus() == 'pending' && !$generate_preview) {
			
			$generate_next_invoice = income_config_option("generate_next_invoice_repetition_on_print");

            if ($generate_next_invoice) {
                try {
                    $invoice->generateNextRepetition();
                } catch (Exception $e) {
                    Logger::log('Error invoice generateNextRepetition: ' . $invoice->getId());
                    Logger::log($e->getMessage());
                    Logger::log($e->getTraceAsString());
                }
            }

            $invoice->setStatus('printed');
            $invoice->save();
            $status_changed = true;
        }

        $array_tax = array();
        foreach ($invoice->getTaxes() as $tax) {
            $tax_data = IncomeTaxes::instance()->findById($tax->getTaxId());
            if ($tax->getType() == "percentage") {
                $name = $tax_data->getName() . " (" . $tax->getValue() . "%)";
            } elseif ($tax->getType() == "fixed") {
                $name = $tax_data->getName() . " ($" . $tax->getValue() . ")";
            }
            $array_tax[] = array(
                'name' => $name,
                'tax_value' => $tax->getFixedValue(),
            );
        }

        $array_copy = array();
        foreach ($invoice->getCopies() as $copy_id) {
            $id_copy[] = $copy_id->getCopyId();
        }
        foreach ($invoice->getCopies() as $copy) {
            $copy_data = IncomeCopies::instance()->findById($copy->getCopyId());
            $array_copy[] = array(
                'name' => $copy_data->getName()
            );
        }
        tpl_assign('invoice_notebook', array('press_information' => ''));
        $use_invoice_notebooks = income_config_option('use_invoice_notebooks');
        if($use_invoice_notebooks){
            $invoice_notebook = IncomeInvoiceNotebooks::instance()->findById($invoice->getInvoiceNotebookId());
            if ($invoice_notebook) {
                tpl_assign('invoice_notebook', $invoice_notebook->getArrayInfo());
            }
        }

        $invoice_options = array();
        foreach (IncomeConfigOptions::instance()->findAll() as $option) {
            if ($option->getName() == "invoice_logo") {
                $invoice_options[$option->getName()] = $option->getLogoUrl();
                $invoice_options["invoice_logo_object"] = $option;
            } else {
                $invoice_options[$option->getName()] = $option->getValue();
            }
        }

		$invoice_options['date_format'] = $this->get_invoice_date_format($invoice);

		$custom_template_file = $this->get_invoice_print_template($invoice, $override_layout);

        if (file_exists(ROOT . $custom_template_file)) {
            // use custom invoice design
            $this->setTemplate(ROOT . $custom_template_file);

        } else {
            // use default invoice design, for preview or if no custom design is present

            //$invoice_options['invoice_logo'] = false;
            //$invoice_options['view_date'] = false;
//        	$invoice_options['view_address'] = false;
            $invoice_options['view_rut'] = false;
            $invoice_options['view_invoice_number'] = $invoice->getInvoiceNumber() != '';
//        	$invoice_options['view_customer_rut'] = false;
//        	$invoice_options['view_press_information'] = false;
//        	$invoice_options['view_extra_information'] = false;
//        	$invoice_options['view_expiration_date'] = false;

            $array_copy = array(array('name' => ''));
        }
        
        $ordered_by = Contacts::instance()->findById($invoice->getOrderedById());
        $ordered_by_name = $ordered_by instanceof Contact ? $ordered_by->getObjectName() : '';

		if ($ordered_by instanceof Contact) {
			$ordered_by_address = $ordered_by->getAddress('work');
			if (!$ordered_by_address) $company_address = $ordered_by->getAddress('home');
			if (!$ordered_by_address) $company_address = $ordered_by->getAddress('postal');
			if (!$ordered_by_address) $company_address = $ordered_by->getAddress('other');	
			if ($ordered_by_address && trim($ordered_by_address->getStreet()) == '') {
				$ordered_by_address = null;
			}
		} else {
			$ordered_by_address = null;
		}
		$income_controller = new IncomeController();
		$ordered_by_address = $income_controller->find_billing_address_by_client_and_contact($invoice->getCustomerId(), $invoice->getOrderedById());

		$ordered_by_phone = $ordered_by instanceof Contact ? $ordered_by->getPhoneNumber('work') : '';
		$ordered_by_mail = $ordered_by instanceof Contact ? $ordered_by->getEmailAddress() : '';
        
        $array_bill_to_information = [
        	'ordered_by_name' => $ordered_by_name,
            'company_name' => $invoice->getCustomerName(),
            'address' => $invoice->getCustomerAddress() ? $invoice->getCustomerAddress() : ($ordered_by_address ? $ordered_by_address->getStreet() : ''),
            'city' => $ordered_by_address ? $ordered_by_address->getCity() : $invoice->getCustomerCity(),
            'state' => $ordered_by_address ? $ordered_by_address->getState() : $invoice->getCustomerState(),
            'zip_code' => $ordered_by_address ? $ordered_by_address->getZipCode() : $invoice->getCustomerZipcode(),
            'country' => $ordered_by_address ? $ordered_by_address->getCountry() : $invoice->getCustomerCountry(),
            'address_location' => $ordered_by_address ? $ordered_by_address->getCountry() : $invoice->getCustomerCountry(),
            'phone' => $ordered_by_phone ? $ordered_by_phone : $invoice->getCustomerPhone(),
            'email' => $ordered_by_mail ? $ordered_by_mail : $invoice->getCustomerEmail()
        ];
        $helper_format_address_location_bill_to = [];

        if ($invoice->getCustomerCity()) {
            $helper_format_address_location_bill_to [] = $invoice->getCustomerCity();
        }
        if ($invoice->getCustomerState()) {
            $helper_format_address_location_bill_to[] = $invoice->getCustomerState();
        }
        if ($invoice->getCustomerZipcode()) {
            $helper_format_address_location_bill_to[] = $invoice->getCustomerZipcode();
        }
        $array_bill_to_information['address_location'] = implode(", ", $helper_format_address_location_bill_to);


        $owner_company_data = owner_company();
        $array_pay_to_information = [
            'company_name' => ((!is_null($owner_company_data->getName())) ? $owner_company_data->getName() : ""),
            'address' => (($owner_company_data->getAddress('work')) ? $owner_company_data->getAddress('work')->getStreet() : ""),
            'address_location' => "",
            'phone' => (($owner_company_data->getPhone('work')) ? $owner_company_data->getPhone('work')->getNumber() : ""),
            'email' => (($owner_company_data->getMainEmails()) ? $owner_company_data->getMainEmails()[0]->getEmailAddress() : ""),
            'emailSecond' => (($owner_company_data->getNonMainEmails()) ? $owner_company_data->getNonMainEmails()[0]->getEmailAddress() : ""),
            'website' => (($owner_company_data->getWebpage('work')) ? $owner_company_data->getWebpage('work')->getUrl() : "")
        ];
        
        $company_address = $owner_company_data->getAddress('work');
        if ($company_address instanceof ContactAddress) {
	        $helper_format_address_location_pay_to = array();
            if ($company_address->getCity()) {
                $helper_format_address_location_pay_to[] = $company_address->getCity();
            }
            if ($company_address->getState()) {
                $helper_format_address_location_pay_to[] = $company_address->getState();
            }
            $array_pay_to_information['address_location'] = implode(", ", $helper_format_address_location_pay_to);
            
            if ($company_address->getZipCode()) {
                $array_pay_to_information['address_location'] .= " " . $company_address->getZipCode();
            }
            $array_pay_to_information['address_location'] = trim($array_pay_to_information['address_location']);
        }


		// override owner company information if needed
		Hook::fire('override_invoice_print_business_company', array('invoice' => $invoice), $array_pay_to_information);
		
		// set the variables containing logo information if we had to override it
		if (array_var($array_pay_to_information, 'invoice_logo', '') != '') {
			$invoice_options['invoice_logo'] = array_var($array_pay_to_information, 'invoice_logo');
			
			if ($invoice_options['invoice_logo_object'] && array_var($array_pay_to_information, 'invoice_logo_repo_id', '')) {
				$invoice_options['invoice_logo_object']->setValue(array_var($array_pay_to_information, 'invoice_logo_repo_id', ''));
			}
		}

		$invoice_options['footer_text'] = $invoice->getFooterToPrint();

        $show_invoice_status = true;
        $invoice_status = lang($invoice->getStatus());
        if ($invoice->getStatus() == 'printed') {
        	$invoice_status = lang('pending');
        }

		// used to add more info like qr code, etc
		Hook::fire('more_invoice_print_options', array('invoice' => $invoice), $invoice_options);

        tpl_assign('show_inovice_status', $show_invoice_status);
        tpl_assign('inovice_status', $invoice_status);
        tpl_assign('to_pay_information', $array_pay_to_information);
        tpl_assign('bill_to_information', $array_bill_to_information);
        tpl_assign('invoice', $invoice);
        tpl_assign('invoices', $invoices);
        tpl_assign('invoice_options', $invoice_options);
        tpl_assign('copies', $array_copy);
        tpl_assign('taxes', $array_tax);

        // reload invoice view, status has changed
        if ($status_changed) {
            evt_add("reload current panel");
        }
        return array(
            'invoice' => $invoice,
            'invoices' => $invoices,
            'invoice_options' => $invoice_options,
            'to_pay_information' => $array_pay_to_information,
            'bill_to_information' => $array_bill_to_information,
            'copies' => $array_copy,
            'taxes' => $array_tax,
            'show_invoice_status' => $show_invoice_status,
            'inovice_status' => $invoice_status
        );
    }

    /**
     * Add invoice
     *
     * @access public
     * @param void
     * @return null
     */
    function add()
    {
        $this->setTemplate('add_invoice');

        if (!can_add(logged_user(), active_context(), IncomeInvoices::instance()->getObjectTypeId())) {
            flash_error(lang('no access permissions'));
            ajx_current('empty');
            return;
        }

        $invoice = new IncomeInvoice();
		$invoice->getObject()->setColumnValue('object_subtype_id', array_var($_REQUEST, 'object_subtype_id'));
        tpl_assign('invoice', $invoice);

        $taxes = IncomeTaxes::instance()->findAll(array('condition' => array('removed_by_id = 0')));
        $array_tax = array();
        foreach ($taxes as $tax) {
        	$tax_record = $tax->getRecord()->getArrayInfo();
            $array_tax[] = array(
                'id' => $tax->getId(),
                'name' => $tax->getName(),
                'select' => $tax->getSelectDefault(),
            	'is_default' => $tax->getSelectDefault(),
            	'type' => $tax_record['type'],
            	'value' => $tax_record['value'],
            );
        }
        tpl_assign('taxes', $array_tax);

        $copies = IncomeCopies::instance()->findAll(array('condition' => array('removed_by_id = 0')));
        $array_copy = array();
        foreach ($copies as $copy) {
            $array_copy[] = array(
                'id' => $copy->getId(),
                'name' => $copy->getName(),
                'select' => $copy->getSelectDefault()
            );
        }
        tpl_assign('copies', $array_copy);

        $inv_types = IncomeInvoiceTypes::instance()->findAll(array('condition' => array('removed_by_id = 0')));
        $array_type = array();
        foreach ($inv_types as $inv_type) {
            $array_type[] = array($inv_type->getId(), $inv_type->getName());
        }
        tpl_assign('inv_types', $array_type);

        $invoice_data = array_var($_POST, 'invoice');
        if (!is_array($invoice_data)) {
            // set layout for modal form
            if (array_var($_REQUEST, 'modal')) {
                $this->setLayout("json");
                tpl_assign('modal', true);
            }

            //Invoice References
            $invoice_reference_id = array_var($_REQUEST, 'invoice_reference_id');
            $invoice_references_data = array();
            $ref_invoice = IncomeInvoices::instance()->findById($invoice_reference_id);
            if ($ref_invoice instanceof IncomeInvoice) {
                $invoice_reference_data = new IncomeInvoiceReference();
                $invoice_reference_data->setReferencedObjectId($ref_invoice->getId());
                $invoice_reference_data->setSerie($ref_invoice->getSeries());
                $invoice_reference_data->setInvoiceNumber($ref_invoice->getInvoiceNumber());
				$invoice_reference_data->setAmount($ref_invoice->getDueAmount());
                $invoice_references_data[] = $invoice_reference_data;
            }
            $data['invoice_references_data'] = $invoice_references_data;

            $data['date'] = DateTimeValueLib::now();
            $invoice_data = $data; // array

			/**
			 * HARDCODED FOR KJOLHAUG, SET DUE DATE DEPENDING ON THE VALUE OF PAYMENT TERMS
			 */
			$payment_period_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'invoice_payment_period');
			if ($payment_period_cp instanceof CustomProperty) {
				$pt_val = trim($payment_period_cp->getDefaultValue());

				$due_date = IncomeInvoices::getExpirationDateFromPaymentTerm(DateTimeValueLib::now(), $pt_val);
				
				$invoice_data['expiration_date'] = $due_date;
			}
			/**
			 * END HARDCODED FOR KJOLHAUG
			 */


        } // if
        tpl_assign('invoice_data', $invoice_data);

        //combo invoice notebooks
        $use_invoice_notebooks = income_config_option('use_invoice_notebooks');
        if($use_invoice_notebooks){
            $invoice_notebooks = IncomeInvoiceNotebooks::instance()->findAll(array('condition' => array('exhausted = 0 AND removed_by_id = 0')));
            tpl_assign('invoice_notebooks', $invoice_notebooks);
        }
        // end combo invoice notebooks

        if (is_array(array_var($_POST, 'invoice'))) {
            try {
                $member_ids = json_decode(array_var($_POST, 'members'));

                // set initial status = pending
                $invoice_data['status'] = array_var($_POST['invoice'], 'status', 'pending');

                // save invoice
                $this->save_invoice($invoice, $invoice_data, $member_ids);


                ApplicationLogs::createLog($invoice, ApplicationLogs::ACTION_ADD);

                flash_success(lang('success add invoice', $invoice->getObjectName()));
                if (!array_var($_POST, 'no_action', false)) {
                    if (array_var($_POST, 'popup', false)) {
                        ajx_current("reload", ['invoice_mail_url' => $invoice->getSendInvoiceUrl()]);
                    } else {
                        ajx_current("back", ['invoice_mail_url' => $invoice->getSendInvoiceUrl()]);
                    }
                } else {
                    ajx_current('empty');
                    ajx_extra_data(array('invoice' => $invoice->getArrayInfo()));
                }
                
                if (array_var($_REQUEST, 'modal')) {
                	evt_add("reload current panel");
                }

                ajx_add("overview-panel", "reload");
                // Error...
            } catch (Exception $e) {
                flash_error($e->getMessage());
                ajx_current("empty");
            } // try
        } // if
    }

    /**
     * Edit specific invoice
     *
     * @access public
     * @param void
     * @return null
     */
    function edit()
    {
        $this->setTemplate('add_invoice');

        if (logged_user()->isGuest()) {
            flash_error(lang('no access permissions'));
            ajx_current('empty');
            return;
        }

        $invoice = IncomeInvoices::instance()->findById(get_id());
        if (!($invoice instanceof IncomeInvoice)) {
            flash_error(lang('invoice dnx'));
            ajx_current("empty");
            return;
        } // if

        if (!income_config_option("allow_invoice_flexible_status_change")) {
            if ($invoice->getStatus() == 'confirmed' || $invoice->getStatus() == 'canceled' || $invoice->getStatus() == 'printed') {
                flash_error(lang('cannot edit confirmed or canceled invoices'));
                ajx_current("empty");
                return;
            }
        }

        if (!$invoice->canEdit(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        } // if


		// some dimension selectors must be disabled in edition (projects, clients and rate schedules)
		$disabled_dimension_ids = Dimensions::instance()->findAll(array(
			'id' => true,
			'conditions' => "code IN ('customer_project','customers','rate_schedules')"
		));
		tpl_assign('disabled_dimension_ids', $disabled_dimension_ids);
		// ---

        $inv_types = IncomeInvoiceTypes::instance()->findAll(array('condition' => array('removed_by_id = 0')));
        $array_type = array();
        foreach ($inv_types as $inv_type) {
            $array_type[] = array($inv_type->getId(), $inv_type->getName());
        }
        tpl_assign('inv_types', $array_type);

        //combo invoice notebooks
        $use_invoice_notebooks = income_config_option('use_invoice_notebooks');
        if($use_invoice_notebooks){
            $invoice_notebooks = IncomeInvoiceNotebooks::instance()->findAll(array('condition' => array('exhausted = 0 AND removed_by_id = 0')));
            tpl_assign('invoice_notebooks', $invoice_notebooks);
        }
        // end combo invoice notebooks

        $invoice_data = array_var($_POST, 'invoice');
        if (!is_array($invoice_data)) {
            // set layout for modal form
            if (array_var($_REQUEST, 'modal')) {
                $this->setLayout("json");
                tpl_assign('modal', true);
            }
            $invoice_data = $invoice->getArrayInfo();
        } // if

        tpl_assign('invoice', $invoice);
        tpl_assign('invoice_data', $invoice_data);

        $taxes = IncomeTaxes::instance()->findAll();
        $array_tax = array();
        $id_tax = array();
        foreach ($invoice->getTaxes() as $taxes_id) {
            $id_tax[] = $taxes_id->getTaxId();
        }
        foreach ($taxes as $tax) {
            $select = 0;
            if (in_array($tax->getId(), $id_tax)) {
                $select = 1;
            }
            if ($tax->getRemovedById() == 0 || $select == 1) {
            	$tax_record = $tax->getRecord()->getArrayInfo();
            	$array_tax[] = array(
                    'id' => $tax->getId(),
            		'name' => $tax->getName(),
            		'select' => $select,
            		'is_default' => $tax->getSelectDefault(),
                	'type' => $tax_record['type'],
                	'value' => $tax_record['value'],
                );
            }
        }
        tpl_assign('taxes', $array_tax);

        $copies = IncomeCopies::instance()->findAll();
        $array_copy = array();
        $id_copy = array();
        foreach ($invoice->getCopies() as $copy_id) {
            $id_copy[] = $copy_id->getCopyId();
        }
        foreach ($copies as $copy) {
            $select = 0;
            if (in_array($copy->getId(), $id_copy)) {
                $select = 1;
            }
            if ($copy->getRemovedById() == 0 || $select == 1) {
                $array_copy[] = array(
                    'id' => $copy->getId(),
                    'name' => $copy->getName(),
                    'select' => $select
                );
            }
        }
        tpl_assign('copies', $array_copy);

        if (is_array(array_var($_POST, 'invoice'))) {
            try {
                $member_ids = json_decode(array_var($_POST, 'members'));

				$this->verify_that_disabled_dimensions_are_not_changed($invoice, $member_ids);

                $this->save_invoice($invoice, $invoice_data, $member_ids);

                $use_invoice_notebooks = income_config_option('use_invoice_notebooks');
				if (array_var($invoice_data, 'invoice_notebook_id') && $use_invoice_notebooks) {
					$invoice_notebook = IncomeInvoiceNotebooks::instance()->findById($invoice_data['invoice_notebook_id']);
					if ($invoice_notebook) {
						if ($invoice_data['invoice_number'] == $invoice_notebook->getFinalNumber()) {
							$invoice_notebook->setExhausted(1);
							$invoice_notebook->save();
						}
					}
				}

                ApplicationLogs::createLog($invoice, ApplicationLogs::ACTION_EDIT);

                flash_success(lang('success edit invoice', $invoice->getObjectName()));
                if (array_var($_POST, 'popup', false)) {
                    ajx_current("reload");
                } else {
                    ajx_current("back");
                }
                ajx_extra_data(array('invoice' => $invoice->getArrayInfo()));
                if (array_var($_REQUEST, 'modal')) {
                    evt_add("reload current panel");
                }
            } catch (Exception $e) {
                flash_error($e->getMessage());
                ajx_current("empty");
            } // try
        } // if
    }

	function verify_that_disabled_dimensions_are_not_changed($invoice, $sent_member_ids) {

		$projects_dimension = Dimensions::findByCode('customer_project');
		$clients_dimension = Dimensions::findByCode('customers');
		$rate_schedules_dimension = Dimensions::findByCode('rate_schedules');

		$sent_member_ids = array_filter($sent_member_ids);
		
		if (count($sent_member_ids) > 0) {
			$sent_members = Members::instance()->findAll(array("conditions" => "id IN (".implode(',', $sent_member_ids).")"));
			$members = $invoice->getMembers();

			$sent_project_member_id = 0;
			$sent_client_member_id = 0;
			$sent_rate_schedule_member_id = 0;

			foreach ($sent_members as $sent_member) {
				
				if ($sent_member->getDimensionId() == $projects_dimension->getId()) {
					$sent_project_member_id = $sent_member->getId();

				} else if ($clients_dimension instanceof Dimension && $sent_member->getDimensionId() == $clients_dimension->getId()) {
					$sent_client_member_id = $sent_member->getId();

				} else if ($rate_schedules_dimension instanceof Dimension && $sent_member->getDimensionId() == $rate_schedules_dimension->getId()) {
					$sent_rate_schedule_member_id = $sent_member->getId();
				}
			}

			$invoice_project_member_id = 0;
			$invoice_client_member_id = 0;
			$invoice_rate_schedule_member_id = 0;

			foreach ($members as $member) {

				if ($member->getDimensionId() == $projects_dimension->getId()) {
					$invoice_project_member_id = $member->getId();

				} else if ($clients_dimension instanceof Dimension && $member->getDimensionId() == $clients_dimension->getId()) {
					$invoice_client_member_id = $member->getId();

				} else if ($rate_schedules_dimension instanceof Dimension && $member->getDimensionId() == $rate_schedules_dimension->getId()) {
					$invoice_rate_schedule_member_id = $member->getId();
				}
			}

			if ($sent_project_member_id != $invoice_project_member_id) {
				throw new Exception(lang('disabled dimension value cant be changed when editing invoice', $projects_dimension->getName()));
			}
			if ($clients_dimension instanceof Dimension) {
				if ($sent_client_member_id != $invoice_client_member_id) {
					throw new Exception(lang('disabled dimension value cant be changed when editing invoice', $clients_dimension->getName()));
				}
			}
			if ($rate_schedules_dimension instanceof Dimension) {
				if ($sent_rate_schedule_member_id != $invoice_rate_schedule_member_id) {
					throw new Exception(lang('disabled dimension value cant be changed when editing invoice', $rate_schedules_dimension->getName()));
				}
			}
		}
	}

    /**
     * Delete specific invoice
     *
     * @access public
     * @param void
     * @return null
     */
    function delete()
    {
        if (logged_user()->isGuest()) {
            flash_error(lang('no access permissions'));
            ajx_current('empty');
            return;
        }

        ajx_current("empty");
        $invoice = IncomeInvoices::instance()->findById(get_id());
        if (!($invoice instanceof IncomeInvoice)) {
            flash_error(lang('invoice dnx'));
            ajx_current("empty");
            return;
        } // if

        if (!$invoice->canDelete(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        } // if

        $result = true;
        Hook::fire('check_integration_status', array('object' => $invoice), $result);

        try {
            DB::beginWork();
            $invoice->trash();
            DB::commit();
            ApplicationLogs::createLog($invoice, ApplicationLogs::ACTION_TRASH);

            flash_success(lang('success deleted invoice', $invoice->getObjectName()));
            if (array_var($_POST, 'popup', false)) {
                ajx_current("reload");
            } else {
                ajx_current("back");
            }
            ajx_add("overview-panel", "reload");
        } catch (Exception $e) {
            DB::rollback();
            flash_error(lang('error delete invoice'));
            ajx_current("empty");
        } // try
    }

    function customer_data()
    {
        ajx_current("empty");
        $result = $this->customerDataLogic(array_var($_REQUEST, 'customer_id'));
        if ($result != false) {
            ajx_extra_data(array("customer" => $result));
        } else {
            ajx_extra_data(array("customer" => array("status" => false)));
        }
    }
    
    static function customerDataLogic($customer_id, $invoice_id = null){
        $mem_object_type_ids = (array)ObjectTypes::instance()->findAll(array('id' => true, 'conditions' => "name IN ('project','customer_folder','project_folder')"));
        $cli_ot = ObjectTypes::findByName('customer')->getId();
		$project_ot = ObjectTypes::findByName('project')->getId();
		
		Env::useHelper('functions', 'crpm');
		$customers_dim = get_customers_dimension();
		$projects_dim = Dimensions::findByCode('customer_project');

		$is_billing_client = false;

        $member_ids = explode(',', $customer_id);
        $customer_present = false;
        foreach ($member_ids as $member_id) {
            $member = Members::instance()->findById($member_id);
            if ($member instanceof Member) {

                if ($member->getObjectTypeId() == $cli_ot) {

                    $customer_present = true;
                    $customer_member = $member;

				} else if ($member->getObjectTypeId() == $project_ot && $customers_dim->getId() != $projects_dim->getId()) {

					// get the billing client member of the project
					$customer_member = get_client_member_of_project($member, 'project_billing_client');
					if (!$customer_member) {
						// if not billing client, then use the default client
						$customer_member = get_client_member_of_project($member);
					} else {
						$is_billing_client = true;
					}
					$customer_present = $customer_member instanceof Member;

                } else if (in_array($member->getObjectTypeId(), $mem_object_type_ids) && $customers_dim->getId() == $projects_dim->getId()) {

                    $all_parents = $member->getAllParentMembersInHierarchy();
                    foreach ($all_parents as $parent) {
                        if ($parent->getObjectTypeId() == $cli_ot) {
                            $customer_present = true;
                            $customer_member = $parent;
                            break;
                        }
                    }
                }
            }
            if ($customer_present) break;
        }

        if ($customer_present) {
            $customer = Customers::instance()->findById($customer_member->getObjectId());

            if ($customer) {
                $customer_name = $customer->getObjectName();
                if ($customer->getCustomerName() != "") {
                    $customer_name = $customer->getCustomerName();
                }

                $customer_rut = $customer->getCustomerRut();

                //$customer_number = $customer->getCustomerNumber();
				$customer_phone = null;
				$customer_email = null;
				$address_obj = null;
				$customer_number = null;

                $cp = CustomProperties::getCustomPropertyByCode($customer->getObjectTypeId(), "customer_number");
                if(!is_null($cp)){
                    $customer_number = CustomPropertyValues::getCustomPropertyValue($customer->getId(), $cp->getId());
                }
                $customer_address = "";
                $customer_city = "";
                $contact = $customer->getContact();
                if ($contact instanceof Contact) {
                    $address_obj = $contact->getAddress('work');
                    if (!$address_obj instanceof ContactAddress) {
                    	$address_obj = ContactAddresses::instance()->findOne(array('conditions' => array("`contact_id` = ?", $contact->getId())));
                    }
                    $customer_email = $contact->getEmailAddress('work');
                    if (!$customer_email) {
                    	$customer_email = $contact->getEmailAddress();
                    }
                    $customer_phone = $contact->getPhone('work');
                    if (!$customer_phone instanceof ContactTelephone) {
                    	$customer_phone = ContactTelephones::instance()->findOne(array('conditions' => array("`contact_id` = ?", $contact->getId())));
                    }
                }
                
                $invoice = null;
                if (!is_null($invoice_id)){
                	$invoice = IncomeInvoices::instance()->findById($invoice_id);
                }
                if (is_null($customer_email) && $invoice instanceof IncomeInvoice) {
                    $customer_email = $invoice->getCustomerEmail();
                }
                
                $address_json = (!is_null($address_obj)) ? [$address_obj->getArrayInfo()] : "";
                $number_string = (!is_null($customer_number)) ? $customer_number->getValue() : "";
                $phone_string = (!is_null($customer_phone)) ? [$customer_phone->getArrayInfo()] : "";
                
                //get billing address from custom property
                $billing_address = null;
				$billing_address_cp = CustomProperties::getCustomPropertyByCode($member->getObjectTypeId(), 'billing_address');

				if ($billing_address_cp instanceof CustomProperty) {
					
					$billing_address_value = CustomPropertyValues::getCustomPropertyValue($member->getObjectId(), $billing_address_cp->getId());
					if ($billing_address_value instanceof CustomPropertyValue) {
						$billing_address = $billing_address_value->getValue();
					}
				}

				if (isset($billing_address) && $billing_address != "") {

					$exploded = explode("|", $billing_address);
	
					foreach ($exploded as &$v) {
						$v = str_replace("%%_PIPE_%%", "|", $v);
						$v = escape_character($v);
					}
					if (count($exploded) > 0) {
						$billing_address_values = array(
							'street' => array_var($exploded, 1, ''),
							'city' => array_var($exploded, 2, ''),
							'state' => array_var($exploded, 3, ''),
							'zipcode' =>  array_var($exploded, 5, ''), 
							'country' => array_var($exploded, 4, '')
						);
						$tmp_billing_address_array = array_filter($billing_address_values);
						$billing_address_values['parsed'] = implode(' - ', $tmp_billing_address_array);
						$billing_address_json = array($billing_address_values);
					} 
				}

                // if there is any missing data from customer, try to get it from the previous invoice
                if ($invoice instanceof IncomeInvoice && $invoice->getFromInvoiceId() > 0) {
                	if (!$customer_name || !$customer_rut || !$customer_email || !$billing_address || !$number_string || !$phone_string) {
	                	
                		$prev_invoice = IncomeInvoices::instance()->findById($invoice->getFromInvoiceId());
	                	if ($prev_invoice instanceof IncomeInvoice) {
		                	if (!$customer_name) $customer_name = $prev_invoice->getCustomerName();
		                	if (!$customer_rut) $customer_rut = $prev_invoice->getCustomerRut();
		                	if (!$customer_email) $customer_email = $prev_invoice->getCustomerEmail();
		                	if (!$number_string) $number_string = $prev_invoice->getCustomerNumber();
		                	
		                	if (!$phone_string) {
		                		$phone_string = array(
			                		'number' => $prev_invoice->getCustomerPhone()
			                	);
		                	}
		                	
		                	if (!$address_json) {
		                		$address_values = array(
			                		'street' => $prev_invoice->getCustomerAddress(),
			                		'city' => $prev_invoice->getCustomerCity(),
			                		'state' => $prev_invoice->getCustomerState(),
			                		'zipcode' => $prev_invoice->getCustomerZipcode(),
			                		'country' => $prev_invoice->getCustomerCountry(),
			                	);
		                		$tmp_address_array = array_filter($address_values);
		                		$address_values['parsed'] = implode(' - ', $tmp_address_array);
		                		$address_json = array($address_values);
		                	}
	                	}
                	}
                }

                return array(
                    "id" => $member->getId(),
					"is_member" => true,
                    "contact_id" => $contact instanceof Contact ? $contact->getId() : 0,
                    "name" => $customer_name,
                    "address" => $address_json,
                    "billing_address" => ( isset($billing_address_json) ? $billing_address_json : ''),
                    "rut" => isset($customer_rut) ? $customer_rut : "",   // *** LC
                    "email" => isset( $customer_email ) ? $customer_email : "",    // *** LC
                    "number" => isset( $number_string) ? $number_string : "",   // *** LC
                    "phone"=> $phone_string,
                    "status" => true,
					"is_billing_client" => $is_billing_client
                );
            }
        }
        return false;
    }

    /**
     * Return the billing information of a contact
     */
    function get_contact_billing_info(){

        ajx_current("empty");
        $contact_id = array_var($_REQUEST, 'contact_id');
        $member_id = array_var($_REQUEST, 'member_id');

        $customer_id ='';
        $member = Members::instance()->findById($member_id);
        if ($member instanceof Member){
            $customer_id = $member->getObjectId();
        }

        $customer = Customers::instance()->findById($customer_id);
        $contact = Contacts::instance()->findById($contact_id);
        if ($customer instanceof Customer){
            if ($contact instanceof Contact){

                $customer_email_final = null;
                $customer_email = $contact->getMainEmails();
                if (!empty($customer_email)){
                    foreach ($customer_email as $email){
                        if ($email instanceof ContactEmail){
                            if (is_null($customer_email_final)){
                                $customer_email_final = $email->getEmailAddress();
                            }else{
                                continue;
                            }
                        }
                    }
                }

                $address_final = null;
                $address_obj = $contact->getAllAddresses();
                if (!empty($address_obj)){
                    foreach ($address_obj as $address){
                        if ($address instanceof ContactAddress){
                            if (is_null($address_final)){
                                $address_final = $address;
                            }else{
                                continue;
                            }
                        }
                    }
                }

                $customer_phone_final =  null;
                $customer_phone = $contact->getAllPhones();
                if (!empty($customer_phone)){
                    foreach ($customer_phone as $phone){
                        if ($phone instanceof ContactAddress){
                            if (is_null($customer_phone_final)){
                                $customer_phone_final = $phone;
                            }else{
                                continue;
                            }
                        }
                    }
                }


                $cp = CustomProperties::getCustomPropertyByCode($customer->getObjectTypeId(), "customer_number");
                if(!is_null($cp)){
                    $customer_number = CustomPropertyValues::getCustomPropertyValue($customer->getId(), $cp->getId());
                }
                $customer_rut = $customer->getCustomerRut();

                $result = array(
                    "id" => $contact_id,
                    "name" => $contact->getName(),
                    "address" => array((!is_null($address_final))?$address_final->getArrayInfo():""),
                    "rut" => $customer_rut,
                    "email" => $customer_email_final,
                    "number" => (!is_null($customer_number))?$customer_number->getValue():"",
                    "phone"=>$customer_phone_final,
                    "status" => true,
                );
            }
        }else{
            $result = false;
        }

        if ($result != false) {
            ajx_extra_data(array("contact" => $result));
        } else {
            ajx_extra_data(array("contact" => array("status" => false)));
        }
    }

    function invoice_notebook_data()
    {
        ajx_current("empty");
        $invoice_notebook_id = array_var($_REQUEST, 'invoice_notebook_id');
        $invoice_notebook = IncomeInvoiceNotebooks::instance()->findById($invoice_notebook_id);
        $use_invoice_notebooks = income_config_option('use_invoice_notebooks');

        if ($invoice_notebook && $use_invoice_notebooks) {

            $series = $invoice_notebook->getSeries();
            $type = $invoice_notebook->getInvoiceType();

            $invoice_numb = IncomeInvoice::findInvoiceNotebook($invoice_notebook->getId());
            if ($invoice_numb) {
                $num_last_inv = $invoice_numb->getInvoiceNumber();
                $new_num = $num_last_inv + 1;
                if ($new_num <= $invoice_notebook->getFinalNumber()) {
                    $number = $new_num;
                }
            } else {
                $number = $invoice_notebook->getInitialNumber();
            }
            ajx_extra_data(array(
                "invoice_notebook" => array(
                    "series" => $series,
                    "type" => $type,
                    "number" => $number,
                    "status" => true
                )));
        } else {
            ajx_extra_data(array(
                "invoice_notebook" => array(
                    "status" => false
                )));
        }
    }

    function save_invoice(IncomeInvoice $invoice, $invoice_data, $member_ids)
    {
        

        try {

            DB::beginWork();

                $res_invoice=IncomeInvoices::save_invoice($invoice, $invoice_data, $member_ids);

                if(!$res_invoice)
                {
                    throw new Exception(lang('error save invoice'));
                }

            DB::commit();

            return $res_invoice;

        } catch (Exception $e) {
            DB::rollback();
            Logger::log_r($e->getMessage());
            Logger::log_r($e->getTraceAsString());
            throw $e;
        }
    }

    /**
     * Updates some attributes for an invoice (only in income_invoices table)
     * @param integer $invoice_id
     * @param array $attributes
     * @return void|boolean
     */
    function update_invoice_attributes($invoice_id, $attributes)
    {
        try {
            $transaction_started = false;

            if (logged_user()->isGuest()) {
                throw new Exception(lang('no access permissions'));
            }

            $invoice = IncomeInvoices::instance()->findById($invoice_id);

            if (!($invoice instanceof IncomeInvoice)) {
                throw new Exception(lang('invoice dnx'));
            }
            if (!$invoice->canEdit(logged_user())) {
                throw new Exception(lang('no access permissions'));
            }

            DB::beginWork();
            $transaction_started = true;

            $cp_data = array();
            
            foreach ($attributes as $column => $value) {
            	// set a column or a custom property value
            	if (IncomeInvoices::instance()->columnExists($column)) {
            		// object column
	                $invoice->setColumnValue($column, $value);
            	} else {
            		// check if custom property exists and save it
            		$cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), $column);
            		if ($cp instanceof CustomProperty) {
            			$cp_data[$cp->getId()] = $value;
            		}
            	}
            }

            $invoice->save();
            
            if (count($cp_data) > 0) {
            	// assign required cp invoice_payment_period
            	$payment_period_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'invoice_payment_period');
            	if ($payment_period_cp instanceof CustomProperty) {
            		if (!array_var($cp_data, $payment_period_cp->getId())) {
            			$cp_val = trim($payment_period_cp->getDefaultValue());
            			if ($cp_val == '') $cp_val = 'Due on receipt';
            			$cp_data[$payment_period_cp->getId()] = $cp_val;
            		}
            	}
            	
            	$object_controller = new ObjectController();
            	$object_controller->add_custom_properties($invoice, $cp_data);
            }

            DB::commit();

        } catch (Exception $e) {
            if ($transaction_started) {
                DB::rollback();
            }
            Logger::log_r("Error updating invoice:\n" . $e->getMessage() . "\n" . $e->getTraceAsString());
            throw $e;
        }

        return true;
    }


    private function parseRepeatOptions(&$invoice_data)
    {
        IncomeInvoices::parseRepeatOptions($invoice_data);
    }


    function reopen()
    {
        if (logged_user()->isGuest()) {
            flash_error(lang('no access permissions'));
            ajx_current('empty');
            return;
        }

        $invoice = IncomeInvoices::instance()->findById(get_id());
        if (!($invoice instanceof IncomeInvoice)) {
            flash_error(lang('invoice dnx'));
            ajx_current("empty");
            return;
        }

        if (!$invoice->canEdit(logged_user())) {
            flash_error(lang('cannot edit confirmed or canceled invoices'));
            ajx_current("empty");
            return;
        }

        try {
            DB::beginWork();

            $invoice->reopen();
            
            DB::commit();
            
            flash_success(lang('success reopening invoice'));
            ajx_current("reload");

			$new_status = 'pending';
            ApplicationLogs::createLog($invoice, 'edit', false, null, true, 'status:' . $new_status);

        } catch (Exception $e) {
            DB::rollback();
            flash_error($e->getMessage());
            ajx_current("empty");
        }
    }


    function cancel()
    {
        if (logged_user()->isGuest()) {
            flash_error(lang('no access permissions'));
            ajx_current('empty');
            return;
        }

        $invoice = IncomeInvoices::instance()->findById(get_id());
        if (!($invoice instanceof IncomeInvoice)) {
            flash_error(lang('invoice dnx'));
            ajx_current("empty");
            return;
        }

        if (income_config_option("allow_invoice_flexible_status_change") || $invoice->getStatus() == 'printed' || $invoice->getStatus() == 'pending') {
            if (!can_write(logged_user(), $invoice->getMembers(), $invoice->getObjectTypeId())) {
                flash_error(lang('no access permissions'));
                ajx_current("empty");
                return;
            }
        } else {
            if (!$invoice->canEdit(logged_user())) {
                flash_error(lang('no access permissions'));
                ajx_current("empty");
                return;
            }
        }

        
        $_GET['ids'] = $invoice->getId();
        $integration_status = $this->check_invoices_integration_status();
        if(!$integration_status['result']){
            ajx_current("empty");
            return;
        }

        try {
            DB::beginWork();

            $invoice->void();

            DB::commit();
            
            flash_success(lang('success canceling invoice'));
            ajx_current("reload");
            
        } catch (Exception $e) {
            DB::rollback();
            flash_error($e->getMessage());
            ajx_current("empty");
        }
    }

    function confirm($invoice_id = false)
    {
        if (logged_user()->isGuest()) {
            flash_error(lang('no access permissions'));
            ajx_current('empty');
            return;
        }

		if (!$invoice_id) {
			$invoice_id = get_id();
		}

        $invoice = IncomeInvoices::instance()->findById($invoice_id);
        if (!($invoice instanceof IncomeInvoice)) {
            flash_error(lang('invoice dnx'));
            ajx_current("empty");
            return;
        }

        if (!can_write(logged_user(), $invoice->getMembers(), $invoice->getObjectTypeId())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        }

        try {
            DB::beginWork();

            try {
                $invoice->generateNextRepetition();
            } catch (Exception $e) {

            }

            $invoice->setStatus('confirmed');

            $invoice->disableMandatoryFieldsValidation(true);
            $invoice->save();

			// change the associated timeslots status to Invoiced (invoiced)
			$invoice->changeAssociatedTimeslotsStatus('invoiced');

            DB::commit();
            
            ApplicationLogs::createLog($invoice, 'edit', false, null, true, 'status:confirmed');

            flash_success(lang('success confirm invoice'));
            ajx_current("reload");

        } catch (Exception $e) {
            DB::rollback();
            flash_error($e->getMessage());
            ajx_current("empty");
        }
    }


    function generate_and_upload_invoice_preview($invoice)
    {
        if ($invoice instanceof IncomeInvoice && Plugins::instance()->isActivePlugin('advanced_core')) {
            // start output buffer
            ob_start();
            ajx_current("empty");

            // get print view variables
            $preview_view_options = $this->print_view(array("id" => $invoice->getId(), "generate_preview" => true));
            $owner_company_data = owner_company();
            // set print view variables to view
            $dont_print = true;
            $invoice_options = $preview_view_options['invoice_options'];
            $to_pay_information = $preview_view_options['to_pay_information'];
            $bill_to_information = $preview_view_options['bill_to_information'];
            $show_inovice_status = $preview_view_options['show_invoice_status'];
            $inovice_status = $preview_view_options['inovice_status'];
            $copies = $preview_view_options['copies'];
            $taxes = $preview_view_options['taxes'];
            // include preview template

            //because the image/logo in invoice need to be public to showed that.
            //we put the image in tmp also with the invoice
            $gen_id = gen_id();
            $invoce_logo_path = ROOT . "/tmp/{$gen_id}_invoice_logo";
            $invoce_logo_url = ROOT_URL . "/tmp/{$gen_id}_invoice_logo";

            try{
	            file_put_contents($invoce_logo_path, FileRepository::getBackend()->getFileContent($invoice_options['invoice_logo_object']->getValue()));
	            $invoice_options['invoice_logo'] = $invoce_logo_url;
            } catch (FileNotInRepositoryError $fe) {
            	
            }

			$custom_template_file = $this->get_invoice_print_template($invoice);
            if (file_exists(ROOT . "/$custom_template_file")) {
            	// use custom invoice design
            	include ROOT . "/$custom_template_file";
            } else {
            	include get_template_path('print_view', 'invoice', 'income');
            }
            // get the preview html and end output buffer
            $preview_html = ob_get_clean();

            // generate the pdf file
            $pdf_data = convert_to_pdf($preview_html, null, $gen_id);

            $file_path = ROOT . "/tmp/" . $pdf_data['name'];
            $preview_pdf_content = file_get_contents($file_path);
            $preview_pdf_name = lang('invoice preview') . " - " . $invoice->getName() . ".pdf";

            // upload generated pdf file
            Env::useHelper('functions', 'advanced_core');
            $file = null;
            if ($invoice->getColumnValue('invoice_preview_file_id') > 0) {
                $file = ProjectFiles::instance()->findById($invoice->getColumnValue('invoice_preview_file_id'));
            }
            $file = upload_generated_document($preview_pdf_name, $invoice->getMembers(), $preview_pdf_content, 'application/pdf', $file);

            //here we delete the invoice logo and the html of invoice
            @unlink($invoce_logo_path);

            // save preview file id in the invoice object
            if ($file instanceof ProjectFile) {
            	// update only the column invoice_preview_file_id
            	DB::execute("
            		UPDATE ".TABLE_PREFIX."income_invoices SET invoice_preview_file_id=".$file->getId()."
            			WHERE object_id=".$invoice->getId()."
            	");
            }

            // return the preview pdf file
            return $file;
        }
    }


    function send_invoice($invoice_id = null, $send_invoice = null, $send_in_background = false, $extra_params = array())
    {
        if (logged_user()->isGuest() || !Plugins::instance()->isActivePlugin('mail')) {
            flash_error(lang('no access permissions'));
            ajx_current('empty');
            return;
        }

        if (is_null($invoice_id)) {
        	$invoice_id = get_id();
        }
        $invoice = IncomeInvoices::instance()->findById($invoice_id);
        if (!($invoice instanceof IncomeInvoice)) {
            flash_error(lang('invoice dnx'));
            ajx_current("empty");
            return;
        }

        if (!can_write(logged_user(), $invoice->getMembers(), $invoice->getObjectTypeId())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        }

        try {
        	if (!is_null($send_invoice)) {
        		$send_the_email_automatically = $send_invoice;
        	} else {
        		$send_the_email_automatically = array_var($_REQUEST, 'send_the_email');
        	}
        	
        	$params_to_email = array();
        	if(isset($_GET['cc']) && !empty($_GET['cc'])){
        		$params_to_email['cc'] = $_GET['cc'];
        	}
        	if(isset($_GET['bcc']) && !empty($_GET['bcc'])){
        		$params_to_email['bcc'] = $_GET['bcc'];
        	}
        	if(isset($_GET['to']) && !empty($_GET['to'])){
        		if (is_array($_GET['to'])) {
        			$params_to_email['to'] = array_values($_GET['to']);
        		} else {
        			$params_to_email['to'] = array($_GET['to']);
        		}
        	}
        	
        	$recipient_fields = array('to','cc','bcc');
        	foreach ($recipient_fields as $rf) {
	        	if (array_var($extra_params, $rf)) {
	        		if (!isset($params_to_email[$rf])) $params_to_email[$rf] = array();
	        		$params_to_email[$rf][] = array_var($extra_params, $rf);
	        	}
        	}
        	
        	/* this function decides which mail template is used 
        	 * and if it is sent automatically or lets the user compose the mail
        	 */
        	send_invoice_mail($invoice, $params_to_email, $send_the_email_automatically, $send_in_background);
        	
        } catch (Exception $e) {
            flash_error($e->getMessage());
            Logger::log_r("Error sending invoice $invoice_id:\n".$e->getMessage()."\n".$e->getTraceAsString());
            ajx_current("empty");
            return array('error' => $e->getMessage());
        }
        return array('ok' => true);
    }


    function get_details()
    {
        ajx_current("empty");
        $id = get_id();

        $detail_info = array();
        $invoice = IncomeInvoices::instance()->findById($id);
        if ($invoice instanceof IncomeInvoice) {

            $currency = Currencies::instance()->getCurrency($invoice->getCurrencyId());
            $symbol = $currency instanceof Currency ? $currency->getSymbol() : '';

            $details = IncomeInvoiceLines::instance()->findAll(array('conditions' => "object_id=" . $invoice->getId()));

            foreach ($details as $detail) {
                $line_currency = Currencies::instance()->getCurrency($detail->getCurrencyId());
                $line_symbol = $line_currency instanceof Currency ? $line_currency->getSymbol() : '';

                $info = array(
                    'total' => $line_symbol . ' ' . number_format($detail->getTotal(), 2),
                    'desc' => $detail->getDetail(),
                    'amount' => $detail->getAmount(),
                    'unit_value' => $line_symbol . ' ' . number_format($detail->getUnitValue(), 2),
                    'detail_type' => 'line',
                );

                $detail_info[] = $info;
            }

            $taxes = IncomeInvoiceTaxes::instance()->findAll(array('conditions' => "invoice_id=" . $invoice->getId()));
            foreach ($taxes as $tax) {
                $tax_data = IncomeTaxes::instance()->findById($tax->getTaxId());
                $info = array(
                    'total' => $symbol . ' ' . number_format($tax->getFixedValue(), 2),
                    'desc' => $tax_data->getName() . ($tax->getType() == 'percentage' ? ' (' . $tax->getValue() . '%)' : ''),
                    'detail_type' => 'tax',
                );

                $detail_info[] = $info;
            }
        }

        ajx_extra_data(array('details' => $detail_info));
    }


    function generate_next_repetition()
    {

        if (logged_user()->isGuest()) {
            flash_error(lang('no access permissions'));
            ajx_current('empty');
            return;
        }

        $invoice = IncomeInvoices::instance()->findById(get_id());
        if (!($invoice instanceof IncomeInvoice)) {
            flash_error(lang('invoice dnx'));
            ajx_current("empty");
            return;
        }

        if (!$invoice->canGenerateNextInvoice(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        }

        try {
            DB::beginWork();

            $new_invoice = $invoice->generateNextRepetition();
            if ($new_invoice instanceof IncomeInvoice) {
                flash_success(lang('new invoice repetition generated successfully', $new_invoice->getName()));
                ajx_current("reload");
            } else {
                flash_error(lang('new invoice could not be generated'));
                ajx_current("empty");
            }

            DB::commit();
        } catch (Exception $e) {
            DB::rollback();
            flash_error($e->getMessage());
            ajx_current("empty");
        }
    }


    function print_parameters()
    {
        $invoice = IncomeInvoices::instance()->findById(get_id());
        if (!($invoice instanceof IncomeInvoice)) {
            flash_error(lang('invoice dnx'));
            ajx_current("empty");
            return;
        }

		$use_serie_for_invoice_number = income_config_option("use_serie_for_invoice_number");

        $invoice_type = ObjectSubtypes::instance()->findById($invoice->getColumnValue('object_subtype_id'));

        $invoice_data = array_var($_POST, 'invoice_data');
        if (!is_array($invoice_data)) {

            $invoice_data = array(
				'id' => $invoice->getId(),
				'invoice_notebook_id' => $invoice->getInvoiceNotebookId(),
				'invoice_number' => $invoice->getInvoiceNumber(),
				'series' => $invoice->getSeries(),
			);

			// if invoice number not set then try to get it from a notebook
            $use_invoice_notebooks = income_config_option('use_invoice_notebooks');
			if (!$invoice_data["invoice_number"] && $use_invoice_notebooks) {

				// get last invoice number, serie and notebook_id and fill the form
				$last_invoice_notebook = user_config_option('last_notebook_used');
	
				$invoice_notebooks = IncomeInvoiceNotebooks::instance()->findAll(array('condition' => array('exhausted = 0 AND removed_by_id = 0 AND invoice_type_id=?', $invoice_type->getId())));
				tpl_assign('invoice_notebooks', $invoice_notebooks);
	
				$notebook_ids = array();
				$notebooks_info = array();
				foreach ($invoice_notebooks as $n) {
					$notebook_ids[] = $n->getId();
	
					$info = array('serie' => $n->getSeries());
					$last_num_obj = IncomeInvoice::findInvoiceNotebook($n->getId());
					if ($last_num_obj instanceof IncomeInvoice) {
						$info['next_num'] = $last_num_obj->getInvoiceNumber() + 1;
					} else {
						$info['next_num'] = $n->getInitialNumber();
					}
					$notebooks_info[$n->getId()] = $info;
				}
				tpl_assign('notebooks_info', $notebooks_info);
	
				if (!in_array($last_invoice_notebook, $notebook_ids)) $last_invoice_notebook = null;
	
				if (!$last_invoice_notebook) {
					foreach ($invoice_notebooks as $n) {
						if ($n->getInvoiceTypeId() == $invoice_type->getId() && !$n->getExhausted()) {
							$last_invoice_notebook = $n->getId();
							break;
						}
					}
				}
	
				if ($last_invoice_notebook > 0) {
					$data = array();
					$invoice_notebook = IncomeInvoiceNotebooks::instance()->findById($last_invoice_notebook);
	
					if ($invoice_notebook && !$invoice_notebook->getExhausted()) {
						$invoice_numb = IncomeInvoice::findInvoiceNotebook($invoice_notebook->getId());
						if ($invoice_numb) {
							$num_last_inv = $invoice_numb->getInvoiceNumber();
							$new_num = $num_last_inv + 1;
							if ($new_num <= $invoice_notebook->getFinalNumber()) {
								$number = $new_num;
							}
						} else {
							$number = $invoice_notebook->getInitialNumber();
						}
	
						$invoice_data["invoice_notebook_id"] = $last_invoice_notebook;
						$invoice_data["invoice_number"] = $number;
						$invoice_data["series"] = $invoice_notebook->getSeries();
					}
				}
			}


            tpl_assign('invoice_data', $invoice_data);

        } else {

            if (($use_serie_for_invoice_number && !array_var($invoice_data, 'series')) || !array_var($invoice_data, 'invoice_number')) {
                ajx_current("empty");
                ajx_extra_data(array('errorCode' => 1, 'errorMessage' => lang('invoice serie and number are mandatory')));
                return;
            }

            // update last notebook used config option and check if notebook is exhausted
            $use_invoice_notebooks = income_config_option('use_invoice_notebooks');
            if($use_invoice_notebooks){
                $invoice_notebook = IncomeInvoiceNotebooks::instance()->findById($invoice_data['invoice_notebook_id']);
                if ($invoice_notebook) {
                    if ($invoice_data['invoice_number'] == $invoice_notebook->getFinalNumber()) {
                        $invoice_notebook->setExhausted(1);
                        $invoice_notebook->save();
                    }
                }
            }

            $option = ContactConfigOptions::getByName('last_notebook_used');
            $option->setContactValue(array_var($invoice_data, 'invoice_notebook_id'), logged_user()->getId());
            $option->save();
            evt_add("user preference changed", array('name' => 'last_notebook_used', 'value' => $invoice_data['invoice_notebook_id']));

            // update the invoice, set serie, number and notebook_id
            $invoice->setInvoiceNotebookId(array_var($invoice_data, 'invoice_notebook_id'));
            $invoice->setSeries(array_var($invoice_data, 'series'));
            $invoice->setInvoiceNumber(array_var($invoice_data, 'invoice_number'));
            $invoice->save();

            evt_add("reload current panel");
            ajx_current("empty");
            ajx_extra_data(array('id' => $invoice->getId()));

        }

    }


    /**deprecated!!!*/
    function generate_invoice_from_tasks()
    {
        if (!Plugins::instance()->isActivePlugin('advanced_billing')) {
            ajx_current("empty");
            return;
        }

        $task_ids = explode(',', array_var($_GET, 'task_ids'));
        $ids = array();
        foreach ($task_ids as $tid) {
            if (is_numeric($tid)) $ids[] = $tid;
        }

        $tasks = array();
        if (count($ids) > 0) {
            $tasks = ProjectTasks::instance()->findAll(array('conditions' => "id IN (" . implode(',', $ids) . ")"));
        }

        if (count($tasks) == 0) {
            ajx_current("empty");
            flash_error(lang('object dnx'));
            return;
        }
        
        // Use member templates if any is available
        $project_ot = ObjectTypes::findByName('project');
        $client_ot = ObjectTypes::findByName('customer');
        $project_member = null;
        foreach (active_context() as $selection) {
        	if ($selection instanceof Member && in_array($selection->getObjectTypeId(), array($project_ot->getId(), $client_ot->getId()))) {
        		$project_member = $selection;
        		break;
        	}
        }
        
        $generate_using_old_way = true;
        
        if ($project_member instanceof Member) {
        	
        	$template_id = IncomeInvoiceTemplates::getMemberInvoiceTemplateId($project_member);
        	$invoice_template = IncomeInvoiceTemplates::instance()->findById($template_id);
        	if ($invoice_template instanceof IncomeInvoiceTemplate) {
        		
        		$generate_using_old_way = false;
        		
        		$timeslots = Timeslots::instance()->findAll(array(
        				"conditions" => "invoicing_status='pending' AND trashed_by_id=0 AND rel_object_id IN (".implode(',',$ids).")"
        		));
        		
        		$generated_invoices = null;
        		$hook_params = array('member'=>$project_member, 'invoice_template'=>$invoice_template, 'timeslots'=>$timeslots, 'payment_receipts'=>array(), 'param_billing_period_from'=>'', 'param_billing_period_to'=>'');
        		Hook::fire('override_invoice_generation', $hook_params, $generated_invoices);
        		
        		if (!$generated_invoices) {
        			$invoice = IncomeInvoiceTemplates::generateInvoiceFromTemplateAndObjects($project_member, 
                                                                                             $invoice_template, 
                                                                                             $timeslots, 
                                                                                             array(), 
                                                                                             '', 
                                                                                             '');
        			$generated_invoices = array($invoice);
        		}
        		
        		foreach ($generated_invoices as $invoice) {
	        		if ($invoice instanceof IncomeInvoice) {
	        			flash_success(lang('success add invoice', $invoice->getObjectName()));
	        		}
        		}
        		evt_add('reload tab panel', 'income-panel');
        		ajx_current("reload");
        	}
        }
        
        // If no invoice template could be used then generate the invoice using the old way
        if ($generate_using_old_way) {

	        Env::useHelper('functions', 'advanced_billing');
	        if (function_exists("generate_invoice_data_from_objects") && function_exists("group_objects_for_invoices")) {
	
	            $tasks_without_client = array();
	            $tasks_without_unbilled_time = array();
	            $invoiced_tasks = array();
	            $tasks_to_reload = array();
	
	            $datas = array();
	            try {
	                // generate invoices data
	                $grouped_tasks = group_objects_for_invoices($tasks);
	                foreach ($grouped_tasks as $member_id => $tasks_group) {
	
	                    if ($member_id > 0) {
	
	                        $tasks_without_time = array();
	                        $data = generate_invoice_data_from_objects($tasks_group, logged_user(), $member_id, $tasks_without_time);
	
	                        if ($data) {
	                            $datas[] = $data;
	                        }
	
	                        foreach ($tasks_without_time as $task) {
	                            $tasks_without_unbilled_time[] = $task->getObjectName();
	                        }
	
	                    } else {
	                        foreach ($tasks_group as $task) {
	                            $tasks_without_client[] = $task->getObjectName();
	                        }
	                    }
	                    // collect tasks to reload in list
	                    foreach ($tasks_group as $tt) {
	                    	$tasks_to_reload[] = $tt;
	                    }
	                }
	
	            } catch (Exception $e) {
	                ajx_current("empty");
	                flash_error($e->getMessage());
	                return;
	            }
	
	            foreach ($datas as $data) {
	
	                if ($data && count($data) > 0) {
	                    $invoice = new IncomeInvoice();
	                    $invoice->disableMandatoryFieldsValidation(true);
	
	                    $member_ids = array_var($data, 'members');
	                    $_POST = $data;
	                    $invoice_data = array_var($_POST, 'invoice');
	                    try {

	                        $inv = $this->save_invoice($invoice, $invoice_data, $member_ids);
	                        if ($inv instanceof IncomeInvoice) {
	                            $invoiced_tasks = array_var($data, 'invoiced_tasks', array());
	                            foreach ($invoiced_tasks as $inv_task) {
	                                if ($inv_task instanceof ProjectTask){
	                                    $task_timestlots = Timeslots::instance()->findAll(array("conditions" => array("`rel_object_id` = ? AND `invoicing_status` = 'pending'", $inv_task->getId())));
	                                    
										// change the associated timeslots status to Invoiced (invoiced), this also fires the hooks that reclassifies them in the "invoiced" member at the approval status dim
										$inv->changeAssociatedTimeslotsStatus('invoiced', $task_timestlots);

	                                } elseif($inv_task instanceof Timeslot){
	                                    $ts = $inv_task;

										// to use when saving the application log
										$old_content_object = $ts->generateOldContentObjectData();

										$old_status = $ts->getColumnValue('invoicing_status');

	                                    $ts->setColumnValue('invoicing_status', 'invoiced');
	                                    $ts->setColumnValue('invoice_id', $inv->getId());
	                                    $ts->save();

										$ret = null;
										Hook::fire("after_change_object_inv_status", array('object' => $ts, 'old_status' => $old_status), $ret);

										ApplicationLogs::createLog($ts, ApplicationLogs::ACTION_EDIT, false, true);
	                                }
	                            	
	                            }
	                        }
	
	                    } catch (Exception $e) {
	                        ajx_current("empty");
	                        flash_error($e->getMessage());
	                        return;
	                    }
	
	                    try {
	                        ApplicationLogs::createLog($invoice, ApplicationLogs::ACTION_ADD);
	
	                        evt_add("popup", array('title' => lang('invoice saved'), 'timeout' => 3, 'message' => lang('success add invoice', $invoice->getObjectName())));
	
	                    } catch (Exception $e) {
	                        ajx_current("empty");
	                        flash_error($e->getMessage());
	                        return;
	                    }
	                }
	
	            }
	
	            $error_message = "";
	            if (count($tasks_without_unbilled_time) > 0) {
	                $error_message .= "\n \n" . lang('the following tasks does not have unbilled time records', "\n" . '"' . implode('"' . "\n" . '"', $tasks_without_unbilled_time) . '"');
	            }
	            if (count($tasks_without_client) > 0) {
	                $error_message .= ($error_message == "" ? "" : "\n \n");
	                $error_message .= lang('the following tasks are not associated to a client', "\n" . '"' . implode('"' . "\n" . '"', $tasks_without_client) . '"');
	            }
	            if ($error_message != "") {
	                $error_message = lang('there are tasks that cannot be invoiced') . $error_message;
	                flash_error($error_message);
	            }
	
	            $reload_view = array_var($_REQUEST, 'reload');
	            if ($reload_view) {
	            	ajx_current("reload");
	            } else {
		            ajx_current("empty");
	            }
	            evt_add('reload tab panel', 'overview-panel');
	            evt_add('reload tab panel', 'income-panel');
	            
	            if (count($tasks_to_reload) > 0) {
	            	$tasks_data_to_reload = array();
	            	foreach ($tasks_to_reload as $tt) {
	            		$tasks_data_to_reload[] = $tt->getArrayInfo();
	            	}
	            	if (count($tasks_data_to_reload) > 0) {
	            		evt_add('update tasks in list', array('tasks' => $tasks_data_to_reload));
	            	}
	            }
	        }
        }

    }
    
    
    function list_project_invoices_to_generate_tab() {
    	require_javascript("BatchInvoicingManager.js", $this->plugin_name);
    	$config = array();
    	ajx_current("panel", "batch-invoicing-containerpanel", null, $config, true);
    	ajx_replace(true);
    }
    
    /* deprecated ????*/
    function list_project_invoices_to_generate() {
    	
    	ajx_current("empty");
    	
    	$_SESSION["invoices_to_generate"] = array();
    	$_SESSION["invoices_objects_by_project"] = array();
    	
    	$param_billing_period = array_var($_REQUEST, 'billing_period');
    	
    	$now = DateTimeValueLib::now();
    	$now->advance(logged_user()->getUserTimezoneValue(), true);
    	switch ($param_billing_period) {
    		case 'today':
    			$billing_period_from = DateTimeValueLib::make(0,0,0,$now->getMonth(),$now->getDay(),$now->getYear());
    			$billing_period_to = DateTimeValueLib::make(23,59,59,$now->getMonth(),$now->getDay(),$now->getYear());break;
    			break;
    			
    		case 'last_week':
    			$monday = $now->getMondayOfWeek()->add('w',-1);
    			$nextMonday = $now->getMondayOfWeek()->add('d',-1);
    			$billing_period_from = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
    			$billing_period_to = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
    			break;
    			
    		case 'this_week':
    			$monday = $now->getMondayOfWeek();
    			$nextMonday = $now->getMondayOfWeek()->add('w',1)->add('d',-1);
    			$billing_period_from = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
    			$billing_period_to = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
    			break;
    			
    		case 'last_month':
    			$now->add('M',-1);
    			$billing_period_from = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
    			$billing_period_to = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
    			break;
    			
    		case 'this_month':
    			$billing_period_from = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
    			$billing_period_to = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
    			break;
    			
    		case 'select_dates':
    			$param_billing_period_from = array_var($_REQUEST, 'billing_period_from');
    			$param_billing_period_to = array_var($_REQUEST, 'billing_period_to');
    			$billing_period_from = getDateValue($param_billing_period_from);
    			$billing_period_to = getDateValue($param_billing_period_to);

                // Check if dates are selected, notify the user if not
				if (!$billing_period_from instanceof DateTimeValue && !$billing_period_to instanceof DateTimeValue) {
                    flash_error('please select the date range');
                    return;
				} else if (!$billing_period_from instanceof DateTimeValue) {
                    flash_error('"from" date is not selected');
                    return;
				} else if (!$billing_period_to instanceof DateTimeValue) {
                    flash_error('"until" date is not selected');
                    return;
				}

                $billing_period_to->endOfDay();
    			break;
    	}
    	// put dates in gmt 0 to make the queries
    	if ($billing_period_from instanceof DateTimeValue) {
    		$billing_period_from->advance(-1 * logged_user()->getUserTimezoneValue(), true);
    	}
    	if ($billing_period_to instanceof DateTimeValue) {
    		$billing_period_to->advance(-1 * logged_user()->getUserTimezoneValue(), true);
    	}
    	// set these variables to use when generating invoice
    	if (!isset($param_billing_period_from)) $param_billing_period_from = format_date($billing_period_from);
    	if (!isset($param_billing_period_to)) $param_billing_period_to = format_date($billing_period_to);
    	
    	if ($billing_period_from && $billing_period_to) {
    		
    	  try {
    		
    		$invoices_to_generate = array();
    		$invoices_for_listing = array();
	    	
	    	$find_objects = find_objects_to_generate_invoice($billing_period_from, $billing_period_to);
	    	$timeslots = $find_objects['timeslots'];
	    	$payment_receipts = $find_objects['payment_receipts'];
	    	
	    	$grouped_by_project = group_objects_to_generate_invoice($timeslots, $payment_receipts);
	    	$grouped_ids_by_project = array();
	    	
	    	foreach ($grouped_by_project as $member_id => $objects) {
	    		
	    		$member = Members::getMemberById($member_id);
	    		if (!$member) continue;
	    		
	    		$tmp_member = array($member);
	    		build_member_list_text_to_show_in_trees($tmp_member);
	    		
	    		$template_id = IncomeInvoiceTemplates::getMemberInvoiceTemplateId($member);
	    		$invoice_template = IncomeInvoiceTemplates::instance()->findById($template_id);
	    		
	    		if ($invoice_template) {
	    			
	    			$generated_invoices = null;
	    			$hook_params = array('member'=>$member, 'invoice_template'=>$invoice_template, 'timeslots'=>$objects['timeslots'], 'payment_receipts'=>$objects['payment_receipts'], 'param_billing_period_from'=>$param_billing_period_from, 'param_billing_period_to'=>$param_billing_period_to, 'only_return_raw_data'=>true);
	    			Hook::fire('override_invoice_generation', $hook_params, $generated_invoices);
	    			
	    			if (!$generated_invoices) {
		    			$generated_data = IncomeInvoiceTemplates::generateInvoiceFromTemplateAndObjects($member, 
                                                                                                        $invoice_template, 
                                                                                                        $objects['timeslots'], 
                                                                                                        $objects['payment_receipts'], 
                                                                                                        $param_billing_period_from, 
                                                                                                        $param_billing_period_to, 
                                                                                                        true);
		    			$invoices_to_generate[$member_id] = $generated_data;
		    			$generated_invoices = array($generated_data);
	    			} else {
	    				$i=0;
	    				foreach ($generated_invoices as $generated_data) {
	    					$i++;
	    					$mem_key = count($generated_invoices) > 1 ? $member_id."-$i" : $member_id;
	    					$invoices_to_generate[$mem_key] = $generated_data;
	    				}
	    			}
	    			
	    			$i=0;
	    			foreach ($generated_invoices as $generated_data) {
						
		    			$invoice_data = $generated_data['invoice'];
		    			
		    			$i++;
		    			$mem_key = count($generated_invoices) > 1 ? $member_id."-$i" : $member_id;
		    			
		    			$invoice_data['id'] = $mem_key;
		    			$invoice_data['name'] = $member->getName();
		    			
		    			$invoice_total = 0;
		    			$c_sym = null;
		    			foreach ($generated_data['invoice_lines'] as $line) {
		    				if (!$c_sym) {
		    					$c = Currencies::instance()->getCurrency($line['currency_id']);
		    					if ($c instanceof Currency) $c_sym = $c->getSymbol();
		    				}
		    				$invoice_total += $line['total'];
		    			}
		    			if (!$c_sym) $c_sym = '$';
		    			$invoice_data['currency_sym'] = $c_sym;
						$invoice_data['total_raw'] = $invoice_total;
						$invoice_data['subtotal'] = number_format($invoice_total, 2);
		    			$invoice_data['total'] = number_format($invoice_total, 2);
		    			$invoice_data['additional_information'] = $invoice_data['additional_information'];
                        $invoice_data['edited_po_number'] = $invoice_data['edited_po_number'];
		    			
		    			$invoices_for_listing[] = $invoice_data;

		    			$grouped_ids_by_project[$mem_key] = array('timeslots'=>array(),'payment_receipts'=>array());
		    			foreach ($objects['timeslots'] as $obj) $grouped_ids_by_project[$mem_key]['timeslots'][] = $obj->getId();
		    			foreach ($objects['payment_receipts'] as $obj) $grouped_ids_by_project[$mem_key]['payment_receipts'][] = $obj->getId();
	    			}
	    			
	    		}
	    	}

	    	$result = array(
					"start" => 0,
	    			"totalCount" => count($invoices_for_listing),
	    			"invoices" => $invoices_for_listing
	    	);
	    	
	    	if (count($invoices_for_listing) == 0) {
	    		flash_error(lang('there is no unbilled time entries or expenses to use for invoice generation within the specified date range'));
	    	}
	    	
	    	$_SESSION["invoices_to_generate"] = $invoices_to_generate;
	    	$_SESSION["invoices_objects_by_project"] = $grouped_ids_by_project;
	    	
    	  } catch (Exception $e) {
    	  	flash_error($e->getMessage());
    	  	return;
    	  }
	    	
    	} else {
    		$result = array("start" => 0, "totalCount" => 0, "invoices" => array());
    	}
    	
    	ajx_extra_data($result);
    }
    
    
    
    function generate_batch_invoices() {
    	$member_ids = explode(',', array_var($_REQUEST, 'member_ids', ''));
    	$properties_json = array_var($_REQUEST, 'properties');
    	$properties = json_decode($properties_json, true);
    	$generated_invoice_names = array();
    	$generated_invoices = array();

		// set a flag indicating that batch invoicing is running so other processes (like third party synchronizations) know and don't execute, they should execute using the hook at the end of this function
		$_SESSION['batch_invoice_running'] = true;
    	
    	foreach ($member_ids as $member_id) {
    		
    		$invoice_generated_data = array_var($_SESSION["invoices_to_generate"], $member_id);
    		if ($invoice_generated_data) {
    			
    			$timeslot_ids = array_var($_SESSION["invoices_objects_by_project"][$member_id], 'timeslots', array());
    			$timeslots = array();
    			if (count($timeslot_ids) > 0) {
    				$timeslots = Timeslots::instance()->findAll(array("conditions" => "id IN (".implode(',',$timeslot_ids).")"));
    			}
    			$payment_receipt_ids = array_var($_SESSION["invoices_objects_by_project"][$member_id], 'payment_receipts', array());
    			$payment_receipts = array();
    			if (count($payment_receipt_ids) > 0) {
    				$payment_receipts = PaymentReceipts::instance()->findAll(array("conditions" => "id IN (".implode(',',$payment_receipt_ids).")"));
    			}
    			
    			apply_batch_properties_to_invoice_data($invoice_generated_data, $member_id, $properties);
    			
    			try {
    				DB::beginWork();
    			
    				$invoice = IncomeInvoiceTemplates::createInvoiceFromArrayData($invoice_generated_data, $timeslots, $payment_receipts);
    				
    				DB::commit();
    				
    				if ($invoice instanceof IncomeInvoice) {
    					$generated_invoices[] = $invoice;
    				}
    				
    			} catch (Exception $e) {
    				DB::rollback();
    				flash_error($e->getMessage());
    			}
    			
    			unset($_SESSION["invoices_to_generate"][$member_id]);
    			unset($_SESSION["invoices_objects_by_project"][$member_id]);
    		}
    		
    	}

		// remove the flag
		unset($_SESSION['batch_invoice_running']);

		foreach ($generated_invoices as $invoice) {
			$generated_invoice_names[] = $invoice->getObjectName();
		}
    	
    	if (count($generated_invoice_names) > 0) {
    		$prefix = "\n - ";
    		$inv_names_to_show = $prefix . implode($prefix, $generated_invoice_names);
    		flash_success(lang('success generating invoices', $inv_names_to_show));
    	}
    	
    	ajx_current("reload");
    	evt_add('reload tab panel', 'overview-panel');
    	evt_add('reload tab panel', 'income-panel');
    	

		Hook::fire('after_generate_batch_invoices', array('invoices' => $generated_invoices), $generated_invoices);
    }
    
    /**
     * given a project, this function collects all the timeslots and generates the invoice 
     * depending on the invoices template used by the proejct
     */
    function generate_member_invoice() {
    	
        //0 - get Clients, Members, and objects types
    	$member = Members::getMemberById(array_var($_REQUEST, 'member_id'));
    	$project_ot = ObjectTypes::findByName('project');
    	$client_ot = ObjectTypes::findByName('customer');
        $mainProject=$member->getId();
    	
        if($member instanceof Member)
        {
            $projectParent=$member->getParentMemberId();
            if($projectParent>0)
            {
                $newParentMember = Members::getMemberById($member->getParentMemberId());
                if ($newParentMember instanceof Member ) {
                    $mainProject=$newParentMember->getId();
                }
            }
        }

    	if (!$member instanceof Member || !in_array($member->getObjectTypeId(), array($project_ot->getId()))) {
    		ajx_current("empty");
    		flash_error(lang('you must select a project or client to generate the invoice'));
    		return;
    	}
    	
        //1 - get the Invoice template
        $template_id = array_var($_REQUEST, 'invoice_template_id');
    	if ($template_id == 0) {
    		$template_id = IncomeInvoiceTemplates::getMemberInvoiceTemplateId($member);
    	}
        $invoice_template = IncomeInvoiceTemplates::instance()->findById($template_id);
       
        //2 - get and set the billing period
    	$param_billing_period_from = array_var($_REQUEST, 'billing_period_from');
    	$param_billing_period_to = array_var($_REQUEST, 'billing_period_to');
        
       
    	$billing_period_from = getDateValue($param_billing_period_from);
    	$billing_period_to = getDateValue($param_billing_period_to);

    	if ($billing_period_from instanceof DateTimeValue) {
    		$billing_period_from->advance(-1 * logged_user()->getUserTimezoneValue(), true);
    	}
    	if ($billing_period_to instanceof DateTimeValue) {
			$billing_period_to->endOfDay(); // set hour to 23:59:59 before applying timezone, in order to include last day's objects
    		$billing_period_to->advance(-1 * logged_user()->getUserTimezoneValue(), true);
    	}
    	
        //3 - get the objects to generate invoice
    	if ($invoice_template) {
            $groupsAux=[];
            $timeslots='';
            $tasks='';
           // $payment_receipts='';
           
            $find_objects = get_objects_to_generate_invoice($billing_period_from, 
                                                            $billing_period_to, 
                                                            $member,
                                                            $invoice_template);
            // *** LC 2023-10-03
            //$tasks = $find_objects['tasks'];
            $tasks = array_var($find_objects, 'tasks', array());
    		$payment_receipts = array_var($find_objects, 'payment_receipts', array());
            $timeslots = array_var($find_objects, 'timeslots', array());

            $project_ot_id = ObjectTypes::findByName('project')->getId();
            $invoiceTemplateBreakDown=$invoice_template->getBreakDownBySubprojects();
             
            if($invoiceTemplateBreakDown=='MULTI')
            {
                //group timeslot by project
                foreach ($timeslots as $t)
                {
                    $project_member = $t->getMemberOfType($project_ot_id);
                    $project_member_id = $project_member ? $project_member->getId() : 0;
                    $group_id = $project_member_id;
                    $groupsAux[$group_id]['timeslots'][] = $t;
                    $groupsAux[$group_id]['member_id'] = $group_id;
                }

                //group by payments by project
                foreach ($payment_receipts as $p) 
                {
                    $project_member = $p->getMemberOfType($project_ot_id);
                    $project_member_id = $project_member ? $project_member->getId() : 0;
                    $group_id = $project_member_id;

                    $groupsAux[$group_id]['payment_receipts'][] = $p;
                    $groupsAux[$group_id]['member_id'] = $group_id;
                }
                
                //group by tasks
                foreach ($tasks as $k) 
                {
                    $project_member = $k->getMemberOfType($project_ot_id);
                    $project_member_id = $project_member ? $project_member->getId() : 0;
                    $group_id = $project_member_id;

                    $groupsAux[$group_id]['tasks'][] = $k;
                    $groupsAux[$group_id]['member_id'] = $group_id;
                }
            }else{
                $groupsAux[0]['payment_receipts']=$payment_receipts;
                $groupsAux[0]['timeslots']= $timeslots;
                $groupsAux[0]['tasks']= $tasks;
                $groupsAux[0]['member_id']=array_var($_REQUEST, 'member_id');
            }

            //4 - save de invoice
            try {
    			DB::beginWork();
                $invoiceParentId='';
                $generated_id_invoices=[];
                $invoiceTasks=false;

                /*get the status of the task*/
                if( isset($groupsAux[0]['tasks'])){
                    foreach($groupsAux[0]['tasks'] as $groupAux)
                    {
                        $tasksStatus=$groupAux->getInvoicingStatus();
                        if($tasksStatus!='invoiced')
                        {
                            /* if template is 'fixed fee tasks' and the task is partially invoiced and task_allow_billing_per_percent is true, the invoice it*/
                            if($invoice_template->getTemplateType()=='fixed fee tasks' 
                                && $tasksStatus=='partially_invoiced'
                                && $invoice_template->getTaskAllowBillingPerPercent())
                            {
                                $invoiceTasks=true;
                            }
                            else if($tasksStatus!='partially_invoiced')
                            {
                                $invoiceTasks=true;
                            }
                        }
                    }
                }

                foreach($groupsAux as $groupAux)
                {
                    if ($invoice_template->getTemplateType()!='fixed fee tasks' && count($groupAux['timeslots']) == 0 && count($groupAux['payment_receipts']) == 0 ) {
                        throw new Exception(lang('there are not timeslots or expenses to invoice'));
                    }else if($invoice_template->getTemplateType()=='lumpsum' && count($groupAux['timeslots']) == 0) {
                        throw new Exception(lang('there are not timeslots to invoice'));
                    }else if($invoice_template->getTemplateType()=='fixed fee tasks' && count($groupAux['tasks']) == 0 ) //fixed fee tast type
                    {
                        throw new Exception(lang('there are not tasks to invoice'));
                    }else if($invoice_template->getTemplateType()=='fixed fee tasks' && count($groupAux['tasks']) > 0 && $invoiceTasks==false)
                    {
                        throw new Exception(lang('there are not tasks to invoice'));
                    }else {
                        $generated_invoices = null;
                        $member = Members::getMemberById($groupAux['member_id']);
                        $hook_params = array('member'=>$member, 'invoice_template'=>$invoice_template, 'timeslots'=>$groupAux['timeslots'], 'payment_receipts'=>$groupAux['payment_receipts'], 'param_billing_period_from'=>$param_billing_period_from, 'param_billing_period_to'=>$param_billing_period_to,'tasks'=>$groupAux['tasks']);
                        Hook::fire('override_invoice_generation', $hook_params, $generated_invoices);
                        
                        if (!$generated_invoices) {
                            $invoice = IncomeInvoiceTemplates::generateInvoiceFromTemplateAndObjects($member, 
                                                                                                     $invoice_template, 
                                                                                                     $groupAux['timeslots'], 
                                                                                                     $groupAux['payment_receipts'], 
                                                                                                     $param_billing_period_from, 
                                                                                                     $param_billing_period_to,
                                                                                                     false,
                                                                                                     null,
                                                                                                     100,
                                                                                                     null, 
                                                                                                     $groupAux['tasks']);
                            if (!$invoice) {
                                throw new Exception(lang('Error when creating the invoice'));
                            }

                            $generated_invoices = array($invoice);

                            if($invoiceTemplateBreakDown=='MULTI')
                            {
                                $project_member_invoice = $invoice->getMemberOfType($project_ot_id);
                                $project_member_invoice_id= $project_member_invoice ? $project_member_invoice->getId() : 0;

                                if($project_member_invoice_id==$mainProject)
                                {
                                    $invoiceParentId=$invoice->getObjectId();
                                }else{
                                    $invoiceParentId=$invoice->getObjectId();
                                }

                                $generated_id_invoices[]=$invoice->getObjectId();
                            }
                        }
                    }
                    
                /**************link invoices*******************/
                }
                foreach($generated_id_invoices as $generated_id_invoice)
                {
                    $resInvoiceChilds=IncomeInvoiceChilds::createInvoiceChild($invoiceParentId,$generated_id_invoice);
                    if (!$resInvoiceChilds) {
                        throw new Exception(lang('Error linking invoices'));
                    }
                }

                $generated_commit_control=false;
                foreach ($generated_invoices as $invoice) {
                    if($invoice instanceof IncomeInvoice && $invoice->getTotal()>0)
                    {
                        $generated_commit_control=true;
                    }
                }

                if($generated_commit_control)
                {
                    DB::commit();
                }else{
                    $generated_invoices='';
                    DB::rollback();
                    throw new Exception(lang('there are not items to invoice'));
                }
                   
                foreach ($generated_invoices as $invoice) 
                {
                    if ($invoice instanceof IncomeInvoice) {
                            flash_success(lang('success add invoice', $invoice->getObjectName()));
                    }
                }
                
                Hook::fire('after_invoice_save', array('invoice' => $invoice), $ignored);
                
                ajx_current("reload");
    			
    		} catch (Exception $e) {
    			DB::rollback();
    			Logger::log_r($e->getMessage());
    			flash_error($e->getMessage());
    			ajx_current("empty");
    		}
     		
    	} else {
    		
    		flash_error(lang("invoice could not be generated becasue no invoice template could be found for project or client"));
    		ajx_current("empty");
    		return;
    	}
    }

    /**deprecated */
    function generate_invoice_from_timeslots()
    {

        if (!Plugins::instance()->isActivePlugin('advanced_billing')) {
            ajx_current("empty");
            return;
        }

        $timeslots = get_timeslots_from_grid_parameters();
        

        if (count($timeslots) == 0) {
            ajx_current("empty");
            flash_error(lang('you must select at least one timeslot to generate the invoice'));
            return;
        }
        
        
        // Use member templates if any is available
        $project_ot = ObjectTypes::findByName('project');
        $client_ot = ObjectTypes::findByName('customer');
        $project_member = null;
        foreach (active_context() as $selection) {
        	if ($selection instanceof Member && in_array($selection->getObjectTypeId(), array($project_ot->getId(), $client_ot->getId()))) {
        		$project_member = $selection;
        		break;
        	}
        }
        
        $generate_using_old_way = true;
        
        if ($project_member instanceof Member) {
        	
	        $template_id = IncomeInvoiceTemplates::getMemberInvoiceTemplateId($project_member);
	        $invoice_template = IncomeInvoiceTemplates::instance()->findById($template_id);
	        if ($invoice_template instanceof IncomeInvoiceTemplate) {
	        
	        	$generate_using_old_way = false;
	        	
	        	$generated_invoices = null;
	        	$hook_params = array('member'=>$project_member, 'invoice_template'=>$invoice_template, 'timeslots'=>$timeslots, 'payment_receipts'=>array(), 'param_billing_period_from'=>'', 'param_billing_period_to'=>'');
	        	Hook::fire('override_invoice_generation', $hook_params, $generated_invoices);
	        	
	        	if (!$generated_invoices) {
	        		$invoice = IncomeInvoiceTemplates::generateInvoiceFromTemplateAndObjects($project_member, $invoice_template, $timeslots, array(), '', '');
	        		$generated_invoices = array($invoice);
	        	}
	        	
	        	foreach ($generated_invoices as $invoice) {
			        if ($invoice instanceof IncomeInvoice) {
			        	flash_success(lang('success add invoice', $invoice->getObjectName()));
			        }
	        	}
	        	evt_add('reload tab panel', 'income-panel');
	        	ajx_current("reload");
	        }
        }
        
        // If no invoice template could be used then generate the invoice using the old way
        if ($generate_using_old_way) {
        
	        Env::useHelper('functions', 'advanced_billing');
	        if (function_exists("generate_invoice_data_from_objects") && function_exists("group_objects_for_invoices")) {
	
	            $timeslots_without_client = array();
	            $timeslots_without_unbilled_time = array();
	            $invoiced_timeslots = array();
	
	            $datas = array();
	            try {
	                // generate invoices data
	                $grouped_timeslots = group_objects_for_invoices($timeslots);
	                foreach ($grouped_timeslots as $member_id => $timeslots_group) {
	
	                    if ($member_id > 0) {
	
	                    	$tasks_without_time = array();
	                    	$data = generate_invoice_data_from_objects($timeslots_group, logged_user(), $member_id, $tasks_without_time);
	
	                        if ($data) {
	                            $datas[] = $data;
	                        }
	
	                    } else {
	                        foreach ($timeslots_group as $timeslot) {
	                            $timeslots_without_client[] = $timeslot->getObjectName();
	                        }
	                    }
	                }
	
	            } catch (Exception $e) {
	                ajx_current("empty");
	                flash_error($e->getMessage());
	                return;
	            }
	
	            foreach ($datas as $data) {
	
	                if ($data && count($data) > 0) {
	                    $invoice = new IncomeInvoice();
	                    $invoice->disableMandatoryFieldsValidation(true);
	
	                    $member_ids = array_var($data, 'members');
	                    $_POST = $data;
	                    $invoice_data = array_var($_POST, 'invoice');
	
	                    try {
	                        $inv = $this->save_invoice($invoice, $invoice_data, $member_ids);
	
	                        if ($inv instanceof IncomeInvoice) {
	                            $invoiced_timeslots = array_var($data, 'invoiced_tasks', array());
								
								// change the associated timeslots status to Invoiced (invoiced), this also fires the hooks that reclassifies them in the "invoiced" member at the approval status dim
								$inv->changeAssociatedTimeslotsStatus('invoiced', $invoiced_timeslots);
	                            
	                        }
	
	                    } catch (Exception $e) {
	                        DB::rollback(); // begin work and commit are made in save_invoice function
	                        ajx_current("empty");
	                        flash_error($e->getMessage());
	                        return;
	                    }
	
	                    try {
	                        ApplicationLogs::createLog($invoice, ApplicationLogs::ACTION_ADD);
	
	                        evt_add("popup", array('title' => lang('invoice saved'), 'timeout' => 3, 'message' => lang('success add invoice', $invoice->getObjectName())));
	
	                    } catch (Exception $e) {
	                        ajx_current("empty");
	                        flash_error($e->getMessage());
	                        return;
	                    }
	                }
	
	            }
	
	            $error_message = "";
	            if (count($timeslots_without_client) > 0) {
	                $error_message .= ($error_message == "" ? "" : "\n \n");
	                $error_message .= lang('the following timeslots are not associated to a client', "\n" . '"' . implode('"' . "\n" . '"', $timeslots_without_client) . '"');
	            }
	            if ($error_message != "") {
	                $error_message = lang('there are timeslots that cannot be invoiced') . $error_message;
	                flash_error($error_message);
	            }
	
	            ajx_current("reload");
	            evt_add('reload tab panel', 'overview-panel');
	            evt_add('reload tab panel', 'income-panel');
	        }

        }
    }

    /**
     * Function that return the available bill to for a invoice
     * This function works with the POST parameter sended
     * Need to send company_id and also contact_id to return the available conctacs to the bill to of invoice
     * this method gonna return 3 values:
     *  - contact_information
     *  - billing_information
     *  - representative_information
     *
     * The values returned via ajax
     */
    
    //todo: @fedegara change when use this for get_contacts_of_member of member_controller
    public function get_available_bill_to_for_invoice()
    {

        $data = $_POST;
        $contacts_ids = [$data['contact_id'] => true];// an array to avoid return duplicated contacts. This can happend because we select the same contact as primary o billing contact
        if ($data['company_id']) {
            $company_information = Members::instance()->findById($data['company_id']);
            $billing_contact_cp = CustomProperties::getCustomPropertyByCode(Customers::instance()->getObjectTypeId(), "billing_contact");

            // get the billing contact
            if ($billing_contact_cp instanceof CustomProperty && $company_information instanceof Member) {
                $billing_contact_cp_val = CustomPropertyValues::getCustomPropertyValue($company_information->getObjectId(), $billing_contact_cp->getId());
                if ($billing_contact_cp_val instanceof CustomPropertyValue) {
                    $billing_contact = Contacts::instance()->findById($billing_contact_cp_val->getValue());
                    if ($billing_contact instanceof Contact && !isset($contacts_ids[$billing_contact->getId()])) {
                        $contacts_ids[$billing_contact->getId()] = true;
                    } else {
                        $billing_contact = null;
                    }
                }
            }

            $representative_contact_cp = CustomProperties::getCustomPropertyByCode(Customers::instance()->getObjectTypeId(), "representative");

            // get the billing contact
            if ($representative_contact_cp instanceof CustomProperty && $company_information instanceof Member) {
                $billing_contact_cp_val = CustomPropertyValues::getCustomPropertyValue($company_information->getObjectId(), $representative_contact_cp->getId());
                if ($billing_contact_cp_val instanceof CustomPropertyValue) {
                    $representative_contact = Contacts::instance()->findById($billing_contact_cp_val->getValue());
                    if ($representative_contact instanceof Contact && !isset($contacts_ids[$representative_contact->getId()])) {
                        $contacts_ids[$representative_contact->getId()] = true;
                    } else {
                        $representative_contact = null;
                    }
                }
            }
        }

        $return = [
            'contact_information' => ((!empty($_POST['contact_id'])) ? $this->buildInvoiceInformationFromContact(Contacts::instance()->findById($data['contact_id'])) : null),
            'billing_information' => ((!empty($billing_contact)) ? $this->buildInvoiceInformationFromContact($billing_contact) : null),
            'representative_information' => ((!empty($representative_contact)) ? $this->buildInvoiceInformationFromContact($representative_contact) : null),
        ];
        ajx_current('empty');
        ajx_extra_data(['result' => $return]);
    }

    /**
     * With the contact sended via parameter we recopilate all the information for
     * the bill to in the invoice
     *
     * @param Contact $contact
     * @return array
     */
    private function buildInvoiceInformationFromContact(Contact $contact)
    {
        $contact_address = null;
        $contact_email = null;
        $contact_phone = null;
        if (!is_null($contact->getAddress('work'))) {
            $contact_address = $contact->getAddress('work');
        } else {
            $contact_address = $contact->getAllAddresses()[0];
        }

        if (!is_null($contact->getEmailAddress('work'))) {
            /** @var ContactEmail $contact_email */
            $contact_email = $contact->getEmailAddress('work');
        } else {
            /** @var ContactEmail $contact_email */
            $contact_email = $contact->getAllEmails()[0];
            if (!is_null($contact_email)) {
                $contact_email = $contact_email->getEmailAddress();
            }
        }
        if (!is_null($contact->getPhone('work'))) {
            /** @var ContactTelephone $contact_phone */
            $contact_phone = $contact->getPhone('work');
        } elsE {
            /** @var ContactTelephone $contact_phone */
            $contact_phone = $contact->getAllPhones()[0];
        }

        $contact_information = [
            'customer_state' => ((!is_null($contact_address)) ? $contact_address->getColumnValue('state') : ""),
            'customer_country' => ((!is_null($contact_address)) ? $contact_address->getColumnValue('country') : ""),
            'customer_zipcode' => ((!is_null($contact_address)) ? $contact_address->getColumnValue('zip_code') : ""),
            'customer_email' => ((!is_null($contact_email)) ? $contact_email : ""),
            'customer_phone' => ((!is_null($contact_phone)) ? $contact_phone->getNumber() : ""),
            'customer_city' => ((!is_null($contact_address)) ? $contact_address->getColumnValue('city') : ""),
            'customer_address' => ((!is_null($contact_address)) ? $contact_address->getColumnValue('street') : ""),
            'customer_name' => ((!is_null($contact)) ? $contact->getName() : ""),
            'customer_id' => ((!is_null($contact)) ? $contact->getId() : null)

        ];
        return $contact_information;
    }
    
    public function get_contacts_of_member_from_invoice(){
        $invoice = IncomeInvoices::instance()->findById(array_var($_REQUEST,'invoice_id'));
        $invoice_id = array_var($_REQUEST,'invoice_id');
        /** @var $invoice IncomeInvoice */
        ajx_current("empty");
        if(!is_null($invoice) and ($invoice instanceof IncomeInvoice)){
            $client = $invoice->getMemberInvoice();

            ajx_extra_data(['response'=>MemberController::getContactsOfMember($client,array_var($_REQUEST, 'company_data',false),$invoice_id)]);
        }else{
            ajx_extra_data(false);
        }
    }

	/**
	 * Verifies if there is any integration restriction before executing some action like trash, void, etc.
	 */
    public function check_invoices_integration_status(){
        $ids = array_filter(explode(',', array_var($_GET,'ids')), 'is_numeric');
		$additional_data = array();
        $result = true;
        for ($i = 0; $i < count($ids); $i++) {
			$id = $ids[$i];
            $invoice = IncomeInvoices::instance()->findById($id);
            Hook::fire('check_integration_status', array('object' => $invoice), $result);
            if(!$result) break;
        }

		Hook::fire('check_invoices_integration_status_additional_data', array('invoice_ids' => $ids), $additional_data);

		$return_values = array('result'=>$result, 'additional_data' => $additional_data);

		ajx_current("empty");
		
		// send the result to the caller in the interface
		ajx_extra_data($return_values);

		// return the result so we can use this function from inside other requests
		return $return_values;

    }

    public function mark_as_paid(){

		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}

		$ids = array_filter(explode(',', array_var($_REQUEST, 'ids')), 'is_numeric');

		foreach($ids as $invoice_id){
			$this->confirm($invoice_id);
		}

        ajx_current("reload");
    }


    public function get_html_for_invoice_widget()
    {
        ajx_current("empty");
        $type = $_POST['type'];

        $invoices = IncomeInvoices::get_invoices_by_date_range();

        Env::useHelper('invoice_widget_helper_functions', 'income');
        $html = build_html_table_for_invoice_widget($invoices);

        ajx_extra_data([
            'html' => $html,
        ]);
    }

}

// InvoiceController
?>
