<?php


class InvoiceLayoutController extends ApplicationController {

	var $plugin_name = "income";

	/**
	 * Construct the InvoiceLayoutController
	 *
	 * @access public
	 * @param void
	 * @return InvoiceLayoutController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', 'income');
	}

	/**
	 * Show invoice_layouts
	 *
	 * @param void
	 * @return null
	 */
	function index() {
		if (!can_manage_invoicing_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		} // if
		$this->addHelper('textile');

		$invoice_layouts = IncomeInvoiceLayouts::instance()->findAll();
		tpl_assign('invoice_layouts', $invoice_layouts);
	}

	function add() {
		if (!can_manage_invoicing_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
	
		$invoice_layout_post = array_var($_POST, 'invoice_layout');
		if (!is_array($invoice_layout_post)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			tpl_assign('invoice_layout', new IncomeInvoiceLayout());
		} else {
			try {
				DB::beginWork();
				$invoice_layout = new IncomeInvoiceLayout();                
				$this->save_invoice_layout($invoice_layout_post, $invoice_layout, true);
				
				flash_success(lang('success add invoice template'));
	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
					ajx_current("empty");
				} else {
					ajx_current("reload");
				}
	
				DB::commit();
			} catch (Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
			}
		}
	}

	function edit() {
		$this->setTemplate("add");
		if (!can_manage_invoicing_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
		
		$id = get_id();
		$invoice_layout = IncomeInvoiceLayouts::instance()->findById($id);
	
		$invoice_layout_post = array_var($_POST, 'invoice_layout');             
		if (!is_array($invoice_layout_post)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			tpl_assign('invoice_layout', $invoice_layout);
			tpl_assign('invoice_layout_edit', $invoice_layout->getArrayInfo());
		} else {
			try {
				DB::beginWork();
				$this->save_invoice_layout($invoice_layout_post, $invoice_layout, false);
				
				flash_success(lang('success edit invoice template'));
	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
					ajx_current("empty");
				} else {
					ajx_current("reload");
				}
	
				DB::commit();
			} catch (Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
			}
		}
	}

	function delete() {
		ajx_current("empty");
		$invoice_layout = IncomeInvoiceLayouts::instance()->findById(get_id());
		if ($invoice_layout instanceof IncomeInvoiceLayout) {
			$invoice_layout->delete();
			flash_success(lang('success delete invoice template'));
		}

	}

	function download_layout_content() {
		ajx_current("empty");
		$invoice_layout = IncomeInvoiceLayouts::instance()->findById(get_id());
		if ($invoice_layout instanceof IncomeInvoiceLayout) {
			$content = $invoice_layout->getContent();

			$file_name = "invoice_layout_".$invoice_layout->getId().".php";

			download_contents($content, 'text/plain', $file_name, strlen($content), true);
			die();
		}

	}


	function preview_layout() {
		ajx_current("empty");
		$invoice_layout = IncomeInvoiceLayouts::instance()->findById(get_id());
		if ($invoice_layout instanceof IncomeInvoiceLayout) {
			
			$test_invoice = IncomeInvoices::instance()->findOne(array(
				"conditions" => "trashed_by_id = 0",
				"order" => "id DESC",
			));
			if (!$test_invoice) {
				die(lang("please create an invoice to preview the layout"));
			} else {

				$this->redirectTo('invoice', 'print_view', array(
					'id' => $test_invoice->getId(),
					'generate_preview' => true,
					'override_layout' => $invoice_layout->getId(),
				));
			}
		}

	}


	function save_invoice_layout($invoice_layout_post, IncomeInvoiceLayout $invoice_layout, $is_new=true) {

		// keep old content
		$invoice_layout_post['content'] = $invoice_layout->getContent();
		
		// set basic properties
		$invoice_layout->setFromAttributes($invoice_layout_post);

		// set file content if new file was uploaded
		if ($invoice_layout_post['tmp_file_id']) {
			$new_content = file_get_contents(ROOT."/tmp/".$invoice_layout_post['tmp_file_id']);
			if ($new_content) {

				// make some security validations, throw exception if any risk is found in the content

				// set the content
				$invoice_layout->setContent($new_content);
			}
		}

		// save
		$invoice_layout->save();
	
		return $invoice_layout;
	}
	

	
	
	function get_invoice_layouts_data() {
		ajx_current("empty");

		$data = IncomeInvoiceLayouts::getInvoiceLayoutsInfo();
		
		ajx_extra_data(array("invoice_layouts" => $data));
	}
}
