<?php

/**
 * InvoiceTemplateController
 * Generado el 20/8/2012
 * 
 */
class InvoiceTemplateController extends ApplicationController {

    var $plugin_name = "income";

    /**
     * Construct the InvoiceTemplateController
     *
     * @access public
     * @param void
     * @return InvoiceTemplateController
     */
    function __construct() {
        parent::__construct();
        prepare_company_website_controller($this, 'website');
        Env::useHelper('functions', 'income');
    }

    /**
     * Show invoice_templates
     *
     * @param void
     * @return null
     */
    function index() {
        if (!can_manage_invoicing_configuration(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        } // if
        $this->addHelper('textile');

        $invoice_templates = IncomeInvoiceTemplates::instance()->findAll();
        tpl_assign('invoice_templates', $invoice_templates);
    }

    function add() {
    	if (!can_manage_invoicing_configuration(logged_user())) {
    		flash_error(lang('no access permissions'));
    		ajx_current("empty");
    		return;
    	}
    
    	$invoice_template_post = array_var($_POST, 'invoice_template');
    	if (!is_array($invoice_template_post)) {
    		// set layout for modal form
    		if (array_var($_REQUEST, 'modal')) {
    			$this->setLayout("json");
    			tpl_assign('modal', true);
    		}
    		
    		tpl_assign('invoice_template', new IncomeInvoiceTemplate());
    	} else {
    		try {
    			DB::beginWork();
    			$invoice_template = new IncomeInvoiceTemplate();                
                $this->save_invoice_template($invoice_template_post, $invoice_template, true);
    			
                flash_success(lang('success add invoice template'));
    
    			if (array_var($_REQUEST, 'modal')) {
    				evt_add("reload current panel");
    				ajx_current("empty");
    			} else {
    				ajx_current("reload");
    			}
    
    			DB::commit();
    		} catch (Exception $e) {
    			DB::rollback();
    			flash_error($e->getMessage());
    			ajx_current("empty");
    		}
    	}
    }

    function edit() {
        $this->setTemplate("add");
    	if (!can_manage_invoicing_configuration(logged_user())) {
    		flash_error(lang('no access permissions'));
    		ajx_current("empty");
    		return;
        }
        
        $id = get_id();
    	$invoice_template = IncomeInvoiceTemplates::instance()->findById($id);
      
    	$invoice_template_post = array_var($_POST, 'invoice_template');             
    	if (!is_array($invoice_template_post)) {
    		// set layout for modal form
    		if (array_var($_REQUEST, 'modal')) {
    			$this->setLayout("json");
    			tpl_assign('modal', true);
    		}
    		
            tpl_assign('invoice_template', $invoice_template);
            tpl_assign('invoice_template_edit', $invoice_template->getArrayInfo());
    	} else {
    		try {
                DB::beginWork();
                $this->save_invoice_template($invoice_template_post, $invoice_template, false);
    			
                flash_success(lang('success edit invoice template'));
    
    			if (array_var($_REQUEST, 'modal')) {
    				evt_add("reload current panel");
    				ajx_current("empty");
    			} else {
    				ajx_current("reload");
    			}
    
    			DB::commit();
    		} catch (Exception $e) {
    			DB::rollback();
    			flash_error($e->getMessage());
    			ajx_current("empty");
    		}
    	}
    }

    function delete() {
        ajx_current("empty");
        $invoice_template = IncomeInvoiceTemplates::instance()->findById(get_id("template_id"));
        if ($invoice_template instanceof IncomeInvoiceTemplate) {
            $invoice_template->delete();
            flash_success(lang('success delete invoice template'));
        }

    }

    /**
     * Save invoice template option
     */
    function save_invoice_template_option($template_id, $groupOption, $name, $visible, $label, $order_number, $selected_option) 
    {
		$invoice_template_option_object = null;

        //search for existing option
        if($template_id>0)
        {
            $invoice_template_option_object = IncomeInvoiceTemplateOptions::instance()->findOne(array('conditions' => 'template_id='.$template_id." AND name='".$name."'" ));
        }

        //if not exists create new
        if(!($invoice_template_option_object instanceof IncomeInvoiceTemplateOption)){
            $invoice_template_option_object = new IncomeInvoiceTemplateOption();
            $invoice_template_option_object->setTemplateId($template_id);
            $invoice_template_option_object->setGroupOption($groupOption);
            $invoice_template_option_object->setName($name);
        }

        
		if ($name == 'project_complete_name') {
			$invoice_template_option_object->setVisible($selected_option != 'no');
			$invoice_template_option_object->setSelectedOption($selected_option);
		} else {
			$invoice_template_option_object->setVisible($visible);
			$invoice_template_option_object->setSelectedOption('');
		}

        $invoice_template_option_object->setLabel($label);
        $invoice_template_option_object->setOrderNumber($order_number);

        //save
        $invoice_template_option_object->save();
    }

    function save_invoice_template($invoice_template_post, IncomeInvoiceTemplate $invoice_template_data, $is_new=true) {                                                                                                                     
        $labor_category_dimension = Dimensions::findByCode('hour_types');
        if($invoice_template_post['is_default']){
            $default_template = IncomeInvoiceTemplates::instance()->findOne(array('conditions' => ' is_default=1 '));
            if($default_template instanceof IncomeInvoiceTemplate && $default_template->getId() != $invoice_template_data->getId()){
                $default_template->setIsDefault(0);
                $default_template->save();
            }  
        }
        
        $invoice_template_data->setTemplateType($invoice_template_post['template_type']);
        
        $invoice_template_data->setName($invoice_template_post['name']);
        $invoice_template_data->setIsDefault($invoice_template_post['is_default']);
        $invoice_template_data->setGroupBy($invoice_template_post['group_by']);

        $invoice_template_data->setBreakDownBySubprojects($invoice_template_post['break_down_by_subprojects']);
		$invoice_template_data->setBreakDownExpensesByCategory($invoice_template_post['break_down_expenses_by_category']);
		$invoice_template_data->setShowExpensesMarkupSeparated($invoice_template_post['show_expenses_markup_separated']);
		$invoice_template_data->setAccumulateDiscounts($invoice_template_post['accumulate_discounts']);
		
        //FOR TIMESLOTS
        //columns
        if(!$invoice_template_post['include_date'] 
        //   && !$invoice_template_post['include_user_name_col']
           && !$invoice_template_post['category_for_timeslot_col']
            && !$invoice_template_post['include_description']
            && !$invoice_template_post['include_hours']
            && !$invoice_template_post['include_rate']
            && !$invoice_template_post['include_total']
        )
        {
            $invoice_template_post['include_date']=1;
            $invoice_template_post['include_description']=1;
            $invoice_template_post['include_hours']=1;
            $invoice_template_post['include_rate']=1;
            $invoice_template_post['include_total']=1;
        }
        $invoice_template_data->setIncludeDate($invoice_template_post['include_date']);
        $invoice_template_data->setincludeTimeLineDate($invoice_template_post['include_time_line_date']);
        $invoice_template_data->setCategoryForTimeslotCol($invoice_template_post['category_for_timeslot_col']);
        $invoice_template_data->setProductTypeForTimeslotCol($invoice_template_post['product_type_for_timeslot_col']);
        $invoice_template_data->setCategoryForTimeslot($invoice_template_post['category_for_timeslot']);
        $invoice_template_data->setProductTypeForTimeslot($invoice_template_post['product_type_for_timeslot']);
        $invoice_template_data->setIncludeDescription($invoice_template_post['include_description']);
        $invoice_template_data->setIncludeUserNameCol($invoice_template_post['include_user_name_col']);
        
        
        //for descripction
        $invoice_template_data->setIncludeUserName($invoice_template_post['include_user_name']);
        $invoice_template_data->setIncludeLaborCategory($invoice_template_post['include_labor_category']);
        $invoice_template_data->setIncludeTaskName($invoice_template_post['include_task_name']);
        $invoice_template_data->setincludeTimeEntryDescription($invoice_template_post['include_time_entry_description']);
        $invoice_template_data->setIncludeProjectName($invoice_template_post['include_project_name']);
        //totals
        $invoice_template_data->setIncludeHours($invoice_template_post['include_hours']);
        $invoice_template_data->setIncludeRate($invoice_template_post['include_rate']);
        $invoice_template_data->setIncludeTotal($invoice_template_post['include_total']);

        //lump sum
        $invoice_template_data->setIncludeLumpSumProjectNumber($invoice_template_post['include_lump_sum_project_number']);
        $invoice_template_data->setIncludeLumpSumProjectName($invoice_template_post['include_lump_sum_project_name']);
        $invoice_template_data->setIncludeLumpSumFromTo($invoice_template_post['include_lump_sum_from_to']);
        $invoice_template_data->setincludeLumpSumBudgetCompletion($invoice_template_post['include_lump_sum_budget_completion']);
        $invoice_template_data->setIncludeLumpQuantity($invoice_template_post['include_lump_sum_quantity']);
        $invoice_template_data->setIncludeLumpSumDateCol($invoice_template_post['include_lump_sum_date_col']);
        $invoice_template_data->setIncludeLumpSumUnitPrice($invoice_template_post['include_lump_sum_unit_price']);
        $invoice_template_data->SetIncludeLumpDate($invoice_template_post['include_lump_sum_date']);
        $invoice_template_data->setIncludeLumpSumProjectType($invoice_template_post['include_lump_sum_project_type']);
        $invoice_template_data->setCategoryForLumpSum($invoice_template_post['category_for_lump_sum']);
        $invoice_template_data->setCategoryForLumpSumCol($invoice_template_post['category_for_lump_sum_col']);
        $invoice_template_data->setProductTypeForLumpSumcol($invoice_template_post['product_type_for_lump_sum_col']);
        $invoice_template_data->setProductTypeForLumpSum($invoice_template_post['product_type_for_lump_sum']);
        $invoice_template_data->setIncludeLumpSumDeetailCol($invoice_template_post['include_lump_sum_deetail_col']);
        
        //expenses
        //columns
        if(!array_var($invoice_template_post, 'include_exp_date')
           && !array_var($invoice_template_post, 'category_for_expenses_col')
              && !array_var($invoice_template_post, 'include_paid_by_col')
            && !array_var($invoice_template_post, 'product_type_for_expenses_col')
            && !array_var($invoice_template_post, 'include_exp_description')
            && !array_var($invoice_template_post, 'include_exp_quantity')
            && !array_var($invoice_template_post, 'include_exp_amount')
            && !array_var($invoice_template_post, 'include_exp_fee')
            && !array_var($invoice_template_post, 'include_exp_total'))
        {
             $invoice_template_post['include_exp_date']=1;
                $invoice_template_post['include_exp_description']=1;
                $invoice_template_post['include_exp_quantity']=1;
                $invoice_template_post['include_exp_amount']=1;
                $invoice_template_post['include_exp_total']=1;
        }
        $invoice_template_data->setExpGroupBy(array_var($invoice_template_post, 'exp_group_by'));
        $invoice_template_data->setIncludeExpQuantity(array_var($invoice_template_post, 'include_exp_quantity'));
        $invoice_template_data->setIncludeExpAmount(array_var($invoice_template_post, 'include_exp_amount'));
       
        $invoice_template_data->setIncludeExpFee(array_var($invoice_template_post, 'include_exp_fee'));
        $invoice_template_data->setCustomLabelIncludeExpFee(array_var($invoice_template_post, 'custom_label_include_exp_fee'));
       
        $invoice_template_data->setIncludeExpMarkupAmount(array_var($invoice_template_post, 'include_exp_markup_amount'));
        $invoice_template_data->setCustomLabelIncludeExpMarkupAmount(array_var($invoice_template_post, 'custom_label_include_exp_markup_amount'));

        $invoice_template_data->setIncludeExpUnitCost(array_var($invoice_template_post, 'include_exp_unit_cost'));
        $invoice_template_data->setCustomLabelIncludeExpUnitCost(array_var($invoice_template_post, 'custom_label_include_exp_include_exp_unit_cost'));

        $invoice_template_data->setIncludeExpTotal(array_var($invoice_template_post, 'include_exp_total'));
        $invoice_template_data->setDisplayCostOrPrice(array_var($invoice_template_post, 'display_cost_or_price'));
        $invoice_template_data->setIncludeExpDate(array_var($invoice_template_post, 'include_exp_date'));
        $invoice_template_data->setIncludeExpTypeDate(array_var($invoice_template_post, 'include_exp_type_date'));
        $invoice_template_data->setIncludePaidBy(array_var($invoice_template_post, 'include_paid_by'));
        $invoice_template_data->setIncludeProductType(array_var($invoice_template_post, 'include_product_type'));
        $invoice_template_data->setIncludeExpenseCategory(array_var($invoice_template_post, 'include_expense_category'));
        $invoice_template_data->setIncludeExpenseName(array_var($invoice_template_post, 'include_expense_name'));
		$invoice_template_data->setIncludeExpenseDescription(array_var($invoice_template_post, 'include_expense_description'));
        $invoice_template_data->setIncludeExpenseProjectNumber(array_var($invoice_template_post, 'include_expense_project_number'));
		$invoice_template_data->setIncludeExpenseProjectName(array_var($invoice_template_post, 'include_expense_project_name'));
        $invoice_template_data->SetIncludeExpenseTaskName(array_var($invoice_template_post, 'include_expense_task_name'));
        

        $invoice_template_data->setExpenseDetailVendorName(array_var($invoice_template_post, 'expense_detail_vendor_name'));
        $invoice_template_data->setExpenseDetailBillNo(array_var($invoice_template_post, 'expense_detail_bill_no'));


        $invoice_template_data->setIncludeExpUserNameCol($invoice_template_post['include_exp_user_name_col']);
        $invoice_template_data->setCategoryForExpenseCol($invoice_template_post['category_for_expenses_col']);
        $invoice_template_data->setCategoryForExpense($invoice_template_post['category_for_expenses']);
        $invoice_template_data->setProductTypeForExpenseCol($invoice_template_post['product_type_for_expenses_col']);
        $invoice_template_data->setProductTypeForExpense($invoice_template_post['product_type_for_expenses']);
        $invoice_template_data->setIncludeExpDescription($invoice_template_post['include_exp_description']);

        $invoice_template_data->setShowTaxesForEachLine(array_var($invoice_template_post, 'show_taxes_for_each_line'));
        $invoice_template_data->setShowHeadersBeforeDetails(array_var($invoice_template_post, 'show_headers_before_details'));
        $invoice_template_data->setIncludeProjectLocation(array_var($invoice_template_post, 'include_project_location'));
        $invoice_template_data->setIncludeAccountSummary(array_var($invoice_template_post, 'include_account_summary'));
        $invoice_template_data->setIncludeFooterText(array_var($invoice_template_post, 'invoice_footer_disclaimer'));
		$invoice_template_data->setIncludeAdditionalInformationInFooter(array_var($invoice_template_post, 'include_additional_info_in_footer'));
        $invoice_template_data->setCustomLabelIncludeUserNameCol(array_var($invoice_template_post, 'custom_label_include_user_name_col'));

        $invoice_template_data->setCustomLabelIncludeDate(array_var($invoice_template_post, 'custom_label_include_date'));
        $invoice_template_data->setCustomLabelCategoryForTimeslotCol(array_var($invoice_template_post, 'custom_label_category_for_timeslot_col'));
        $invoice_template_data->setCustomLabelIncludeDescription(array_var($invoice_template_post, 'custom_label_include_description'));
        $invoice_template_data->setCustomLabelIncludeHours(array_var($invoice_template_post, 'custom_label_include_hours'));
        $invoice_template_data->setCustomLabelIncludeRate(array_var($invoice_template_post, 'custom_label_include_rate'));
        $invoice_template_data->setCustomLabelIncludeTotal(array_var($invoice_template_post, 'custom_label_include_total'));
        $invoice_template_data->setCustomLabelIncludeExpDate(array_var($invoice_template_post, 'custom_label_include_exp_date'));
        $invoice_template_data->setCustomLabelIncludePaidByCol(array_var($invoice_template_post, 'custom_label_include_paid_by_col'));
        $invoice_template_data->setCustomLabelIncludeExpUserNameCol(array_var($invoice_template_post, 'custom_label_include_exp_user_name_col'));

        $invoice_template_data->setCustomLabelCategoryForExpensesCol(array_var($invoice_template_post, 'custom_label_category_for_expenses_col'));
        $invoice_template_data->setCustomLabelProductTypeForExpensesCol(array_var($invoice_template_post, 'custom_label_product_type_for_expenses_col'));
        $invoice_template_data->setCustomLabelIncludeExpDescription(array_var($invoice_template_post, 'custom_label_include_exp_description'));
        $invoice_template_data->setCustomLabelIncludeExpQuantity(array_var($invoice_template_post, 'custom_label_include_exp_quantity'));
        $invoice_template_data->setCustomLabelIncludeExpAmount(array_var($invoice_template_post, 'custom_label_include_exp_amount'));

        
        $invoice_template_data->setCustomLabelIncludeExpTotal(array_var($invoice_template_post, 'custom_label_include_exp_total'));

        $invoice_template_data->setCustomLabelIncludeLumpSumDateCol(array_var($invoice_template_post, 'custom_label_include_lump_sum_date_col'));
        $invoice_template_data->setCustomLabelCategoryForLumpSumCol(array_var($invoice_template_post, 'custom_label_category_for_lump_sum_col'));
        $invoice_template_data->setCustomLabelProductTypeForLumpSumCol(array_var($invoice_template_post, 'custom_label_product_type_for_lump_sum_col'));
        $invoice_template_data->setCustomLabelIncludeLumpSumDeetailCol(array_var($invoice_template_post, 'custom_label_include_lump_sum_deetail_col'));
        $invoice_template_data->setCustomLabelIncludeLumpSumQuantity(array_var($invoice_template_post, 'custom_label_include_lump_sum_quantity'));
        $invoice_template_data->setCustomLabelIncludeLumpSumUnitPrice(array_var($invoice_template_post, 'custom_label_include_lump_sum_unit_price'));

        /*tasks lines*/
        $invoice_template_data->setTaskAllExpensesIncludes(array_var($invoice_template_post, 'task_all_expenses_includes'));
        
        $invoice_template_data->setTaskGroup(array_var($invoice_template_post, 'task_group'));

        if(array_var($invoice_template_post, 'task_group') == 'aggregated into a single line' && $invoice_template_post['template_type']=='fixed fee tasks'){

            $invoice_template_data->setIncludeUserName('No name');
            $invoice_template_data->setIncludeLaborCategory(false);
            $invoice_template_data->setIncludeTaskName(false);
            $invoice_template_data->setincludeTimeEntryDescription(false);
            $invoice_template_data->setIncludeProjectName(false);

            $invoice_template_data->setIncludeLumpSumProjectNumber(false);
            $invoice_template_data->setIncludeLumpSumProjectName(false);
            $invoice_template_data->setIncludeLumpSumFromTo(false);
            $invoice_template_data->setIncludeLumpSumProjectType(false);
            $invoice_template_data->setCategoryForLumpSum(false);
            $invoice_template_data->setIncludeLumpSumFromTo(false);
        }

        $invoice_template_data->setTaskCustomDetail(array_var($invoice_template_post, 'task_custom_detail'));

        $invoice_template_data->setLaborDetailSeparatorValue(array_var($invoice_template_post, 'labor_detail_separator_value'));
        $invoice_template_data->setExpenseDetailSeparatorValue(array_var($invoice_template_post, 'expense_detail_separator_value'));
        $invoice_template_data->setTasksDetailSeparatorValue(array_var($invoice_template_post, 'tasks_detail_separator_value'));
        $invoice_template_data->setLumpSumDetailSeparatorValue(array_var($invoice_template_post, 'lump_sum_detail_separator_value'));
        $invoice_template_data->setTaskAllowBillingPerPercent(array_var($invoice_template_post, 'task_allow_billing_per_percent'));
        $invoice_template_data->setTaskInvoiceBillingType(array_var($invoice_template_post, 'task_invoice_billing_type'));
        
		// when not grouping, for the include exp description and include exp name => use the value of the combobox "expense_use_name_or_desc"
		if (array_var($invoice_template_post, 'exp_group_by') == 'payment_receipts') {
			
			$field_to_show = array_var($invoice_template_post, 'expense_use_name_or_desc');

			if ($field_to_show == 'use_exp_name') {

				$invoice_template_data->setIncludeExpenseName(true);
				$invoice_template_data->setIncludeExpenseDescription(false);

			} else if ($field_to_show == 'use_exp_desc') {

				$invoice_template_data->setIncludeExpenseName(false);
				$invoice_template_data->setIncludeExpenseDescription(true);

			} else if ($field_to_show == 'none') {

				$invoice_template_data->setIncludeExpenseName(false);
				$invoice_template_data->setIncludeExpenseDescription(false);
			}
		}

        // INVOICE TEMPLATES ::
        $invoice_template_data->setInvoiceTemplateColumnAssigned(array_var($invoice_template_post, 'invoice_template_assigned'));

        // INVOICE LANGUAGE ::
        $invoice_template_data->setInvoiceTemplateLanguage(array_var($invoice_template_post, 'template_language_assigned'));

		$invoice_template_data->setDateFormat(array_var($invoice_template_post, 'date_format'));

        $invoice_template_data->save();

         //save new options schema
         foreach($invoice_template_post as $invoice_template_post_key => $invoice_template_post_value)
         {
             //save all items for project options groups
             if($invoice_template_post_key=='project_options')
             {
                 foreach($invoice_template_post_value as $invoice_template_post_value_key => $invoice_template_post_value_value)
                 {
                     $this->save_invoice_template_option($invoice_template_data->getId(), 
                                                         $invoice_template_post_key,
                                                         $invoice_template_post_value_key, 
                                                         $invoice_template_post_value_value['visible'], 
														 array_var($invoice_template_post_value_value, 'label', lang($invoice_template_post_value_key)),
                                                         $invoice_template_post_value_value['order'],
														 array_var($invoice_template_post_value_value, 'selected_option', ''));
                 }
             }
         }
    
    	return $invoice_template_data;
    }
    

    
    
    function get_invoice_templates_data() {
    	ajx_current("empty");
    	
    	$data = array();
    	$templates = IncomeInvoiceTemplates::instance()->findAll();
    	foreach ($templates as $t) { /* @var $t IncomeInvoiceTemplate */
    		$data[] = $t->getArrayInfo();
    	}
    	
    	ajx_extra_data(array("invoice_templates" => $data));
    }
}
