<?php

/**
 * InvoiceTypeController
 * Generado el 20/8/2012
 * 
 */
class InvoiceTypeController extends ApplicationController {

    var $plugin_name = "income";

    /**
     * Construct the InvoiceTypeController
     *
     * @access public
     * @param void
     * @return InvoiceTypeController
     */
    function __construct() {
        parent::__construct();
        prepare_company_website_controller($this, 'website');
    }

    /**
     * Show invoice_types
     *
     * @param void
     * @return null
     */
    function index() {
        if (!can_manage_invoicing_configuration(logged_user())) {
            flash_error(lang('no access permissions'));
            ajx_current("empty");
            return;
        } // if
        $this->addHelper('textile');

        $invoice_types = IncomeInvoiceTypes::instance()->findAll(array('conditions' => 'removed_by_id=0'));
        tpl_assign('invoice_types', $invoice_types);
    }
    
    

    function add() {
    	if (!can_manage_invoicing_configuration(logged_user())) {
    		flash_error(lang('no access permissions'));
    		ajx_current("empty");
    		return;
    	}
    
    	$invoice_type_post = array_var($_POST, 'invoice_type');
    	if (!is_array($invoice_type_post)) {
    		// set layout for modal form
    		if (array_var($_REQUEST, 'modal')) {
    			$this->setLayout("json");
    			tpl_assign('modal', true);
    		}
    		tpl_assign('invoice_type_edit', array());
    		tpl_assign('invoice_type', new IncomeInvoiceType());
    	} else {
    		try {
    			DB::beginWork();
    			$invoice_type = new IncomeInvoiceType();
    
    			$this->save_invoice_type($invoice_type_post, $invoice_type, true);
    			 
    			if (array_var($_REQUEST, 'modal')) {
    				evt_add("reload current panel");
    				ajx_current("empty");
    			} else {
    				ajx_current("reload");
    			}
    
    			DB::commit();
    		} catch (Exception $e) {
    			DB::rollback();
    			flash_error($e->getMessage());
    			ajx_current("empty");
    		}
    	}
    }
    
    
    function edit() {
    	$this->setTemplate("add");
    	if (!can_manage_invoicing_configuration(logged_user())) {
    		flash_error(lang('no access permissions'));
    		ajx_current("empty");
    		return;
    	}
    
    	$id = get_id();
    	$invoice_type = IncomeInvoiceTypes::instance()->findById($id);
    
    	$invoice_type_post = array_var($_POST, 'invoice_type');
    	if (!is_array($invoice_type_post)) {
    		// set layout for modal form
    		if (array_var($_REQUEST, 'modal')) {
    			$this->setLayout("json");
    			tpl_assign('modal', true);
    		}
    
    		tpl_assign('invoice_type', $invoice_type);
    		$invoice_type_edit = array(
    				'id' => $invoice_type->getId(),
    				'name' => $invoice_type->getName(),
    				'select_default' => $invoice_type->getSelectDefault()
    		);
    		tpl_assign('invoice_type_edit', $invoice_type_edit);
    
    	} else {
    		try {
    			DB::beginWork();
    
    			$this->save_invoice_type($invoice_type_post, $invoice_type, false);
    
    			if (array_var($_REQUEST, 'modal')) {
    				evt_add("reload current panel");
    				ajx_current("empty");
    			} else {
    				ajx_current("reload");
    			}
    
    			DB::commit();
    		} catch (Exception $e) {
    			DB::rollback();
    			flash_error($e->getMessage());
    			ajx_current("empty");
    		}
    	}
    }
    
    
    function save_invoice_type($invoice_type_post, $invoice_type_data, $is_new=true) {
    	 
    	$invoice_type_data->setName($invoice_type_post['name']);
    	$invoice_type_data->setSelectDefault($invoice_type_post['select_default']);
    	$invoice_type_data->save();
    	 
    	if ($is_new) {
    		flash_success(lang('success add invoice type'));
    	} else {
    		flash_success(lang('success edit invoice type'));
    	}
    	 
    }

    function delete() {
        ajx_current("empty");
        $invoice_type = IncomeInvoiceTypes::instance()->findById(get_id("invoice_type_id"));
        if ($invoice_type instanceof IncomeInvoiceType) {
        	try {
        		DB::beginWork();
        		$invoice_type->delete();
        		DB::commit();
        	} catch (Exception $e) {
    			DB::rollback();
    			flash_error($e->getMessage());
    			ajx_current("empty");
        	}
        }

        flash_success(lang('success delete invoice type'));
    }

}

// InvoiceTypeController
?>