<?php

function build_file_info_to_attach_in_mail(ProjectFile $file) {
	
	$fName = utf8_encode_mime_header_value($file->getName());
	$fName = str_replace(':', ' ', $fName);
	$fName = str_replace("'", '', $fName);
	
	$type_string = $file->getTypeString();
	
	if (strpos($fName, '.') === FALSE) {
		$split_type = explode('/', $type_string);
		$ext = $split_type[count($split_type) - 1];
		if ($ext) $fName .= '.' . $ext;
	}
	$attachs = array(
			array(
					'manager' => 'ProjectFiles',
					'object_id' => $file->getId(),
					'name' => $fName,
					'type' => $type_string,
					'ico' => "ico-file ico-" . str_replace("/", "-", $type_string),
			),
	);
	
	return $attachs;
}

function send_invoice_mail(IncomeInvoice $invoice, $params_to_email, $send_the_email = false, $send_in_background = false) {
	
	$file = null;
	// if invoice has attached an e-invoice pdf => use it!
	if ($invoice->columnExists('einvoice_document_id')) {
		$file = ProjectFiles::instance()->findById($invoice->getColumnValue('einvoice_document_id'));
	}
	
	if (!$file instanceof ProjectFile) {
		// generate preview document
		$invoice_controller = new InvoiceController();
		$file = $invoice_controller->generate_and_upload_invoice_preview($invoice);
	}
	
	if ($file instanceof ProjectFile) {
		// Remove the invoice preview from the file name
		$file_name = $file->getTitle();
		if(strpos($file_name, lang('invoice preview')) != -1) {
			$file->setFileName($invoice->getObjectName() . ".pdf");
			$file->save();
		}

		// get the attachment info
		$attachs = build_file_info_to_attach_in_mail($file);
	
		$billing_contact = null;
		$billing_contact_cp = CustomProperties::getCustomPropertyByCode(Customers::instance()->getObjectTypeId(), "billing_contact");
	
		$client_member = Members::instance()->findById($invoice->getCustomerId());
	
		// get the billing contact of the client
		if ($billing_contact_cp instanceof CustomProperty && $client_member instanceof Member) {
	
			$billing_contact_cp_val = CustomPropertyValues::getCustomPropertyValue($client_member->getObjectId(), $billing_contact_cp->getId());
			if ($billing_contact_cp_val instanceof CustomPropertyValue) {
				$billing_contact = Contacts::instance()->findById($billing_contact_cp_val->getValue());
			}
		}
	
		if ($billing_contact instanceof Contact) {
			$billing_contact_mail = $billing_contact->getEmailAddress();
			$billing_contact_name = $billing_contact->getObjectName();
		} else {
			$billing_contact = Contacts::instance()->findById($invoice->getOrderedById());
			if ($billing_contact instanceof Contact) {
				$billing_contact_mail = $billing_contact->getEmailAddress();
				$billing_contact_name = $billing_contact->getObjectName();
			} else {
				$billing_contact_mail = "";
				$billing_contact_name = "";
			}
			// if we couldn't find an email in the contact use the one in the invoice
			if (trim($billing_contact_mail) == "") {
				$billing_contact_mail = $invoice->getCustomerEmail();
			}
			// the same with the contact name
			if (trim($billing_contact_name) == "") {
				$billing_contact_name = $invoice->getAttn();
			}
		}

		//// get the billing contact of the project
		//get project type
		$project_ot = ObjectTypes::findByName('project');
		//get the project of the invoice
		$invoice_members = $invoice->getMembers();
        foreach ($invoice_members as $member) {
            if ($member instanceof Member && $member->getObjectTypeId() == $project_ot->getId()) {
                $invoice_project_member = $member;
                break;
            }
		}
		//get contact from custom property
		$project_billing_contact_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "billing_contact");
		if ($project_billing_contact_cp instanceof CustomProperty && $invoice_project_member instanceof Member) {
			$invoice_project_member_id = $invoice_project_member->getObjectId();
			$project_billing_contact_cp_val = CustomPropertyValues::getCustomPropertyValue($invoice_project_member_id, $project_billing_contact_cp->getId());
			if ($project_billing_contact_cp_val instanceof CustomPropertyValue) {
				$project_billing_contact = Contacts::instance()->findById($project_billing_contact_cp_val->getValue());
			}
		}
		//get mail from contact
		$project_billing_contact_mail='';
		if ($project_billing_contact instanceof Contact) {
			$project_billing_contact_mail = $project_billing_contact->getEmailAddress();
		}

		//// get the client contact of the client
		//get client type
	    //$client_ot = ObjectTypes::findByName('customer');
		$get_customer_id = ObjectTypes::findByName('customer')->getId();
		$invoice_client_member = $invoice->getMemberOfType($get_customer_id);
		//get contact from custom property
		$client_billing_contact_cp = CustomProperties::getCustomPropertyByCode($get_customer_id, "billing_contact");
		if ($client_billing_contact_cp instanceof CustomProperty && $invoice_client_member instanceof Member) {
			$invoice_client_member_id = $invoice_client_member->getObjectId();
			$client_billing_contact_cp_val = CustomPropertyValues::getCustomPropertyValue($invoice_client_member_id, $client_billing_contact_cp->getId());
			if ($client_billing_contact_cp_val instanceof CustomPropertyValue) {
				$client_billing_contact = Contacts::instance()->findById($client_billing_contact_cp_val->getValue());
			}
		}
		//get mail from contact
		$client_billing_contact_mail='';
		if ($client_billing_contact instanceof Contact) {
			$client_billing_contact_mail = $client_billing_contact->getEmailAddress();
		}

		// get the invoice mail content template
		$invoice_payment_url = (IncomeConfigOptions::instance()->findOne(array('conditions' => "name='invoice_payment_url'")));
		$external_path_invoice_payment = $invoice_payment_url->getValue();
		$invoice_id = $invoice->getId();
		$customer_id = null;
		$customer_name = null;
		$customer_mail= null;
		$contact_id = null;
		$billing_invoice = null;
		if (!is_null($invoice->getMemberInvoice())) {
			$customer_id = $invoice->getMemberInvoice()->getId();
			$customer_name=$invoice->getMemberInvoice()->getName();
			$customer_mail=$invoice->getCustomerEmail();
		} else {
			if ($invoice->getOrderedById()) {
				$contact_id = $invoice->getOrderedById();
			}
		}
		
		// ensure that the person that ordered the invoice is included in the recipients
		$ordered_by = null;
		if ($invoice->getOrderedById() > 0) {
			$ordered_by = Contacts::instance()->findById($invoice->getOrderedById());
		}
		if ($ordered_by instanceof Contact) {
			$billing_contact_name = $ordered_by->getObjectName();
			$ordered_by_email = $ordered_by->getEmailAddress();
			if ($ordered_by_email) {
				$params_to_email['to'][] = $ordered_by_email;
			}
		}
		
		$additional_template_variables = array();
		$invoice_type='';

		//1 buscar primero por id, sino por invoice type
		$template_number= get_template_id_for_sending_mail($invoice, $additional_template_variables);
		
		if(!$template_number){
			$invoice_type = get_invoice_type_for_sending_mail($invoice, $additional_template_variables);
		}

		$get_customer_id = ObjectTypes::findByName('customer')->getId();
		$member = $invoice->getMemberOfType($get_customer_id);
		
		$template_variables = array(
				'invoice' => $invoice,
				'billing_contact_name' => $billing_contact_name,
				'billing_contact_mail' => $billing_contact_mail,
				'project_billing_contact_mail' => $project_billing_contact_mail,
				'client_billing_contact_mail' => $client_billing_contact_mail,
				'customer_id' => $customer_id,
				'customer_name' => $customer_name,
				'customer_mail' => $customer_mail,
				'contact_id' => $contact_id,
				'member'=>$member,
				'external_path_invoice_payment' => $external_path_invoice_payment
		);
		foreach ($additional_template_variables as $var_name => $var_value) {
			$template_variables[$var_name] = $var_value;
		}
		
		$mail_template_content = get_invoice_mail_template_content($invoice, $invoice_type, $template_variables);
		if(!empty($billing_contact_mail) && !in_array($billing_contact_mail, $params_to_email['to'])){
			$params_to_email['to'][] = $billing_contact_mail;
		}
		
		// if no recipient is set use the mail saved in the invoice
		if (count($params_to_email['to']) == 0 && trim($invoice->getCustomerEmail()) != '') {
			$params_to_email['to'][] = trim($invoice->getCustomerEmail());
		}
		
		$default_bcc = income_config_option('send_invoice_bcc');
		if ($default_bcc){
			$params_to_email['bcc'][] = $default_bcc;
		}
		
		$mail_account_id = income_config_option('send_invoice_default_mail_account');
		if ($mail_account_id) {
			// check if the user has permissions to use this account, if not set mail_account_id as null to use another account later
			$mail_account = MailAccounts::instance()->findById($mail_account_id);
			$user_mac = MailAccountContacts::getByAccountAndContact($mail_account, logged_user());
			if (!$user_mac instanceof MailAccountContact) {
				$mail_account_id = null;
			}
		}
		
		$mail_template_subject = lang('invoice attached') . ": " . $invoice->getObjectName();
		
		
		
		// if mail templates plugin is installed and exists a template for this case
		$mail_template_params = array(
				'triggering_object' => $invoice,
				'template_code' => $invoice_type,
				'template_id' => $template_number,
				'template_parameters' => $template_variables,
		);
		$mail_template_data = null;
		Hook::fire("get_mail_template_data", $mail_template_params, $mail_template_data);
		if (is_array($mail_template_data)) {
			// override content
			if (isset($mail_template_data['body'])) {
				$mail_template_content = $mail_template_data['body'];
			}
			// override subject
			if (isset($mail_template_data['subject'])) {
				$mail_template_subject = $mail_template_data['subject'];
			}
			// override account
			if (isset($mail_template_data['mail_account_id'])) {
				$mail_account_id = $mail_template_data['mail_account_id'];
			}
			// override from name
			if (isset($mail_template_data['from_name'])) {
				$mail_template_from_name = $mail_template_data['from_name'];
			}
			// join to, cc, and bcc
			$recpt_fields = array('to', 'cc', 'bcc');
			foreach ($recpt_fields as $recpt_field) {
				if (isset($mail_template_data[$recpt_field])) {
					if (isset($params_to_email[$recpt_field]) && is_array($params_to_email[$recpt_field])) {
						//si el template tiene un mail definido usa ese y no el default de ls invoice
						if($mail_template_data[$recpt_field])
						{
							$params_to_email[$recpt_field] = $mail_template_data[$recpt_field];
						}
					} else {
						$params_to_email[$recpt_field] = $mail_template_data[$recpt_field];
					}
				}
			}
		}
		// -----------
		
		if ($send_the_email) {
			// save the mail_content and send it

			if (!$mail_account_id) {
				$mail_accounts = MailAccounts::getMailAccountsByUser(logged_user());
				if (count($mail_accounts) > 0) {
					$mail_account = $mail_accounts[0];
					if ($mail_account instanceof MailAccount) {
						$mail_account_id = $mail_account->getId();
					}
				}
			}
			
			$mail_data = array(
					'format' => "html",
					'subject' => $mail_template_subject,
					'body' => $mail_template_content,
					'account_id' => $mail_account_id,
					'additional_info' => array(
							'invoice_id' => $invoice->getId(),
					),
					'member_ids' => $invoice->getMemberIds(),
			);
			
			if (isset($mail_template_from_name)) {
				$mail_data['from_name'] = $mail_template_from_name;
			}
			
			$recpt_fields = array('to', 'cc', 'bcc');
			foreach ($recpt_fields as $recpt_field) {
				if (isset($params_to_email[$recpt_field])) {
					$mail_data[$recpt_field] = implode(',', $params_to_email[$recpt_field]);
				}
			}
			
			if ($send_in_background) {
				$mail_data['dont_send_outobx'] = true;
			}
			
			// set the mail vars to take by the controller
			$_POST['mail'] = $mail_data;
			
			// set the attachments
			$_POST['linked_objects'] = array();
			foreach ($attachs as $idx => $attach) {
				$_POST['linked_objects'][$idx] = $attach['manager'] .":". $attach['object_id'];
				$_POST['attach_contents'][$idx] = true;
			}
			
			// send the email
			$mail_controller = new MailController();
			$ok = $mail_controller->add_mail();
			if (!$ok) {
				throw new Exception("Invoice email could not be sent");
			}
			
		} else {
			// prepare mail variables and compose the email
			$mail_data = array(
					'is_new_mail' => true,
					'type' => "html",
					'subject' => $mail_template_subject,
					'body' => $mail_template_content,
					'account_id' => $mail_account_id,
					'cc' => is_array($params_to_email['cc']) ? implode(',', $params_to_email['cc']) : '',
					'bcc' => is_array($params_to_email['bcc']) ? implode(',', $params_to_email['bcc']) : '',
					'additional_info' => array(
							'invoice_id' => $invoice->getId(),
					)
			);
			
			// set the attachment info in the session
			$_SESSION['existing_attachments'] = $attachs;
			
			// set the email parameters to set in the view
			$_SESSION['add_mail_override_mail_data'] = $mail_data;
		
			// go to compose the email
			redirect_to(get_url('mail', 'add_mail', $params_to_email));
		}
		
	}
}


function get_mail_template_from_customs_properties(IncomeInvoice $invoice) {

	$project_member = null;
	$project_ot = ObjectTypes::findByName('project');
	$client_ot = ObjectTypes::findByName('customer');
	$invoice_members = $invoice->getMembers();
	foreach ($invoice_members as $member) {
		if ($member->getObjectTypeId() == $project_ot->getId()) {
			$project_member = $member;
		}
		if ($member->getObjectTypeId() == $client_ot->getId()) {
			$client_member = $member;
		}
	}

	$invoice_type='';
	$code_template='';

	/* Dont delete it's for object properties feature
	//Find Custom properties for project
	if ($invoice->getStatus() == 'confirmed') {
		$code_template = 'project_template_invoice_paid';//
	} else {
		$code_template = 'project_email_template_invoice_pending'; //
	}
	$email_template_code = CustomProperties::getCustomPropertyByCode($project_ot->getId(), $code_template);

	//////////////Search for Project's email code template 
	if ($email_template_code instanceof CustomProperty) {
		$email_template_codeVal = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $email_template_code->getId());
		if ($email_template_codeVal instanceof CustomPropertyValue) {
			$email_template_id = trim($email_template_codeVal->getValue());
			if($email_template_id!=0)
			{
				$mail_template_objects = MailTemplates::getById($email_template_id);
				$invoice_type = $mail_template_objects->getColumnValue('code');
				$email_template_id = $mail_template_objects->getColumnValue('id');

				if($invoice_type!=''){
					return $invoice_type;//email_template_id
				}
			}
		}
	}

	/////////////////////Search for Client's email code template
	if ($invoice->getStatus() == 'confirmed') {
		$code_template = 'client_email_template_invoice_paid';//
	} else {
		$code_template = 'client_email_template_invoice_pending'; //
	}

	$email_template_code = CustomProperties::getCustomPropertyByCode($client_ot->getId(), $code_template);

	if ($email_template_code instanceof CustomProperty) {
		$email_template_codeVal = CustomPropertyValues::getCustomPropertyValue($client_member->getObjectId(), $email_template_code->getId());
		if ($email_template_codeVal instanceof CustomPropertyValue) {
			$email_template_id = trim($email_template_codeVal->getValue());
			if($email_template_id!=0)
			{
				$mail_template_objects = MailTemplates::getById($email_template_id);
				$invoice_type = $mail_template_objects->getColumnValue('code');
				$email_template_id = $mail_template_objects->getColumnValue('id');
				
				if($invoice_type!=''){
					return $invoice_type;
				}
			}
		}
	}*/

	// if not found serach email template by object_status_types_id
	if (Plugins::instance()->isActivePlugin('object_templates')) { 
	    $res = MailTemplates::getByTemplateByStatus($invoice->getObjectTypeId(),$invoice->getStatus());
	    if($res['mail_template_id']!=0){
		    return $res['mail_template_id'];
	    }
	}

	return '';
}

function get_template_id_for_sending_mail(IncomeInvoice $invoice, &$additional_template_variables) {
	
	//cal the manager's invoice template
	$template_id=get_mail_template_from_customs_properties($invoice);
	
	// allow other plugins to override the type and add more variables to the template
	$hook_result = array();
	
	Hook::fire('modify_invoice_type_for_sending_mail', array('invoice' => $invoice), $hook_result);
	
	if (array_var($hook_result, 'invoice_type')) {
		$template_id_hook = array_var($hook_result, 'invoice_type');
		if($template_id_hook>0)
		{
			$template_id=$template_id_hook;
		}
	}
	
	if (array_var($hook_result, 'more_variables')) {
		$more_variables = array_var($hook_result, 'more_variables');
		foreach ($more_variables as $k => $v) {
			$additional_template_variables[$k] = $v;
		}
	}
	// --
	return $template_id;
}

function get_invoice_type_for_sending_mail(IncomeInvoice $invoice, &$additional_template_variables) {
	
	//cal the manager's invoice template
	$invoice_type = "";

	$invoice_type=get_mail_template_from_customs_properties($invoice);

	if($invoice_type==''){
		if ($invoice->getStatus() == 'confirmed') {
			$invoice_type= 'general_invoice_paid';
		}else{
			$invoice_type='general_invoice_pending';
		}
	}

	// allow other plugins to override the type and add more variables to the template
	$hook_result = array();
	
	Hook::fire('modify_invoice_type_for_sending_mail', array('invoice' => $invoice), $hook_result);
	
	if (array_var($hook_result, 'invoice_type')) {
		$invoice_type = array_var($hook_result, 'invoice_type');
	}
	if (array_var($hook_result, 'more_variables')) {
		$more_variables = array_var($hook_result, 'more_variables');
		foreach ($more_variables as $k => $v) {
			$additional_template_variables[$k] = $v;
		}
	}
	// --
	return $invoice_type;
}


function get_invoice_mail_template_filename(IncomeInvoice $invoice, $invoice_type) {
	
	return $invoice_type;
	
}


function get_invoice_mail_template_content($invoice, $invoice_type, $template_variables) {
	
	$mail_template_content = null;
	$mail_template_params = array(
		'triggering_object' => $invoice,
		'template_code' => $invoice_type,
		'template_parameters' => $template_variables,
	);
	// if mail templates plugin is installed and exists a template for this case
	Hook::fire("get_mail_template_content", $mail_template_params, $mail_template_content);

	if (!is_null($mail_template_content)) {
		return $mail_template_content;
	}
	
	$mail_templates_path = ROOT . "/invoice_mail_templates/";
	
	//$template_file_name = 'invoice_mail_template';
	$template_file_name = get_invoice_mail_template_filename($invoice, $invoice_type);
	
	// convert the array of variables to single variables for the template
	extract($template_variables);
	
	$custom_template_path = $mail_templates_path . "$template_file_name.php";
	if (file_exists($custom_template_path)) {
		ob_start();
		include $custom_template_path;
		$mail_template_content = ob_get_clean();
	} else {
		$mail_template_content = "";
	}
	return $mail_template_content;
}



