<?php 

function build_html_table_for_invoice_widget($invoices) {
    $invoice_count = count($invoices);
    $show_more_style = $invoice_count > 10 ? '' : 'display: none;';

	if ($invoice_count > 10) {
		// if we have 11 invoices, remove the last one, it is only here to decide if we show the "view all" button
		array_pop($invoices);
	}

    if($invoice_count == 0) {
        $html = '<div id="table-container">
                    <div class="flex justify-center">
                        <h5 class="text-center">'.lang('No data to display for the selected time period').'</h5>
                    </div>
                </div>';

        return $html;
    }

    $html = '<div id="table-container">
                <table id="table-invoices-data">
                    <tbody id="invoices-body">';

    $project_ot = ObjectTypes::instance()->findOne(array('conditions' => '`name`="project"'));
    foreach($invoices as $invoice) {
        $invoice_members = $invoice->getMembers();
        $project_member = null;
        $project_member_name = '';
        foreach($invoice_members as $member){
            if($member->getObjectTypeId() == $project_ot->getId()){
                $project_member = $member;
                $project_member_name = $member->getName();
                break;
            }
        }

        $currency_id = $invoice->getCurrencyId();
        $currency = Currencies::instance()->findOne(array('conditions' => 'id='.$currency_id));
        if($currency instanceof Currency) {
            $currency_symbol = $currency->getSymbol();
        } else { 
            $currency_symbol = Currencies::getDefaultCurrencySymbol();
        }

        $invoice_paid_class = 'btn-trasnparent';
        $invoice_paid_label_class = '';
        $invoice_paid_label = lang('paid');
        if($invoice->getDueAmount() == 0){
            $invoice_paid_class = 'btn-success';
            $invoice_paid_label_class = 'btn-success-label';
        } else if($invoice->getDueAmount() < $invoice->getTotal()){
            $invoice_paid_class = 'btn-danger';
            $invoice_paid_label_class = 'btn-danger-label';
            $invoice_paid_label = lang('partially paid');
        }

        $invoice_due_class = 'btn-trasnparent';
        $invoice_due_label_class = '';
        if($invoice->getDueAmount() > 0){
            $invoice_due_class = 'btn-alert';
            $invoice_due_label_class = 'btn-alert-label';
        }

        $html .= '<tr class="invoice-row">
                    <td class="flex justify-left invoice-name-td" colspan="20">
                        <img loading="lazy" src="' . get_public_url('assets/images/invoices_widget/invoice.png', 'income') . '"
                            alt="'.lang('invoice logo').'" class="invoice-logo">
                        <div>
                            <p class="invoice-number bold">
                                <a href="#" onclick="og.openLink(\'' . $invoice->getViewUrl() . '\');return false;">
                                    ' . lang('invoice') . ' ' . $invoice->getInvoiceNumber() . ' - ' . format_date($invoice->getDate(), null, 0) . '
                                </a>
                            </p>
                            <p class="invoice-client">
                                <strong>' . $invoice->getCustomerName() . ': </strong>' . $project_member_name . '
                            </p>
                        </div>
                    </td>
                    <td class="invoice-amount" style="float: right" colspan="1">
                        <button class="btn btn-trasnparent flex align-middle justify-arround">
                            <div>
                                <strong>' . lang('total') . '</strong>
                                <p>' . format_money_amount($invoice->getTotal(), $currency_symbol) . '</p>
                            </div>
                        </button>
                    </td>
                    <td class="invoice-amount" colspan="1">
                        <button class="btn ' . $invoice_paid_class . ' flex align-middle justify-arround">
                            <img loading="lazy" src="' . get_public_url('assets/images/invoices_widget/dollar-sign.png', 'income') . '"
                                alt="'.lang('dollar sign').'" class="dollar-sign" />
                            <div>
                                <strong class="' . $invoice_paid_label_class . '">' . $invoice_paid_label . '</strong>
                                <p>' . format_money_amount($invoice->getPaidAmount(), $currency_symbol) . '</p>
                            </div>
                        </button>
                    </td>
                    <td class="invoice-amount" colspan="1">
                        <button class="btn ' . $invoice_due_class . ' flex align-middle justify-arround">
                            <img loading="lazy" src="' . get_public_url('assets/images/invoices_widget/dollar-sign.png', 'income') . '"
                                alt="'.lang('dollar sign').'" class="dollar-sign">
                            <div>
                                <strong class="' . $invoice_due_label_class . '">' . lang('due') . '</strong>
                                <p>' . format_money_amount($invoice->getDueAmount(), $currency_symbol) . '</p>
                            </div>
                        </button>
                    </td>
                </tr>';
    }

    $html .= '</tbody>
            </table>
        </div>
        <div style="' . $show_more_style . '" class="row flex justify-evenly mt-4">
            <button class="btn btn-lg btn-outline-light" onclick="og.showMoreBtnClick();" id="showMoreItemsBtn" value="show_more">
                ' . lang('view all') . '
            </button>
        </div>';

    return $html;

}
