<?php

function update_invoice_total_info_for_member($object){
    Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
    $project_member_id = get_object_project_member_id($object);
    if($project_member_id > 0) {
        update_invoice_total_info_for_member_by_using_member_id($project_member_id);
    }
}

function update_invoice_total_info_for_member_by_using_member_id($project_member_id) {
    $invoices_info = get_invoice_totals_for_member($project_member_id);
    set_new_member_totals_values($project_member_id, $invoices_info);
    $project_member = Members::instance()->findById($project_member_id);
    if($project_member instanceof Member){
        $parent_id = $project_member->getParentMemberId();
        if($parent_id > 0){
            update_invoice_total_info_for_member_by_using_member_id($parent_id);
        }
    }
}

function get_invoice_totals_for_member($project_member_id) { 
    $invoices_info = array(
        'invoiced_total' => 0,
        'invoiced_subtotal' => 0,
        'invoiced_paid_amount' => 0,
        'invoiced_due_amount' => 0
    );

    $select_columns = array("total, subtotal, o.object_subtype_id");
    $inv_totals = "SUM(total) AS invoiced_total ";
    $inv_totals .= ", SUM(subtotal) AS invoiced_subtotal ";
    if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
        $inv_totals .= ", SUM(paid_amount) AS invoiced_paid_amount ";
        $inv_totals .= ", SUM(due_amount) AS invoiced_due_amount ";
        $select_columns[] = "paid_amount";
        $select_columns[] = "due_amount";
    }
    
    $credit_note_type = ObjectSubtypes::instance()->findOne(array('conditions' => "code='credit_note'"));
    $credit_note_id = $credit_note_type->getId();

    // Get invoices totals
    $additional_conditions_invoices = " AND o.object_subtype_id != $credit_note_id ";
    $invoice_totals_info = IncomeInvoices::instance()->listing(array(
        "select_columns" => $select_columns,
        "query_wraper_start" => "SELECT $inv_totals FROM (",
        "query_wraper_end" => ") AS pending_calc",
        "raw_data" => true,
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        "check_sharing_table" => false,
        "count_results" => false,
        "extra_conditions" => $additional_conditions_invoices,
        "member_ids" => array($project_member_id)
    ))->objects; 

    // Add invoices totals
    $invoices_info['invoiced_total'] = $invoice_totals_info[0]['invoiced_total'] ? $invoice_totals_info[0]['invoiced_total'] : 0;
    $invoices_info['invoiced_subtotal'] = $invoice_totals_info[0]['invoiced_subtotal'] ? $invoice_totals_info[0]['invoiced_subtotal'] : 0;
    if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
        $invoices_info['invoiced_paid_amount'] = $invoice_totals_info[0]['invoiced_paid_amount'] ? $invoice_totals_info[0]['invoiced_paid_amount'] : 0;
        $invoices_info['invoiced_due_amount'] = $invoice_totals_info[0]['invoiced_due_amount'] ? $invoice_totals_info[0]['invoiced_due_amount'] : 0;
    }

    // Get credit notes totals
    $additional_conditions_credit_notes = " AND o.object_subtype_id = $credit_note_id ";
    $credit_note_totals_info = IncomeInvoices::instance()->listing(array(
        "select_columns" => $select_columns,
        "query_wraper_start" => "SELECT $inv_totals FROM (",
        "query_wraper_end" => ") AS pending_calc",
        "raw_data" => true,
        "check_sharing_table" => false,
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        "count_results" => false,
        "extra_conditions" => $additional_conditions_credit_notes,
        "member_ids" => array($project_member_id)
    ))->objects; 

    // Subtract credit notes totals
    $invoices_info['invoiced_total'] -= $credit_note_totals_info[0]['invoiced_total'] ? $credit_note_totals_info[0]['invoiced_total'] : 0;
    $invoices_info['invoiced_subtotal'] -= $credit_note_totals_info[0]['invoiced_subtotal'] ? $credit_note_totals_info[0]['invoiced_subtotal'] : 0;
    if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
        $invoices_info['invoiced_paid_amount'] -= $credit_note_totals_info[0]['invoiced_paid_amount'] ? $credit_note_totals_info[0]['invoiced_paid_amount'] : 0;
        $invoices_info['invoiced_due_amount'] -= $credit_note_totals_info[0]['invoiced_due_amount'] ? $credit_note_totals_info[0]['invoiced_due_amount'] : 0;
    }
    
    return $invoices_info;
}
