<?php

/**
 * IncomeInvoiceTemplates
 * Created: 06/15/20
 * Author: Alex Zonis
 * 
 */
class IncomeInvoiceTemplates extends BaseIncomeInvoiceTemplates {

    function __construct() {
        parent::__construct();
    }

    
    static function getInvoiceTemplatesInfo() {
    	$info = array();
    	$templates = self::instance()->findAll();
    	
    	foreach ($templates as $template) {
    		$info[$template->getId()] = $template->getArrayInfo(false);
    	}
    	
    	return $info;
    }
    
	static function getDefaultInvoiceTemplate() {
		
		$default_template = IncomeInvoiceTemplates::instance()->findOne(array('conditions' => ' is_default=1 '));

		return $default_template;
	}
    
    static function getMemberInvoiceTemplateId($member) {
    	
    	$template_id = null;
    	
    	if ($member instanceof Member) {
    		
    		$project_ot = ObjectTypes::findByName('project');
    		$client_ot = ObjectTypes::findByName('customer');
    		
    		$project_template_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), 'invoice_template');
    		$client_template_cp = CustomProperties::getCustomPropertyByCode($client_ot->getId(), 'invoice_template');
    		
	    	if ($member->getObjectTypeId() == $project_ot->getId()) {
	    		$project_cp_value = CustomPropertyValues::getCustomPropertyValue($member->getObjectId(), $project_template_cp->getId());
	    		if ($project_cp_value instanceof CustomPropertyValue && is_numeric($project_cp_value->getValue()) && $project_cp_value->getValue() > 0) {
	    			$template_id = $project_cp_value->getValue();
	    		}
	    	}
	    	
	    	if (!$template_id) {
	    		if ($member->getObjectTypeId() == $project_ot->getId()) {
	    			Env::useHelper('functions', 'crpm');
	    			$client_member = get_client_member_of_project($member);
	    			
	    		} else if ($member->getObjectTypeId() == $client_ot->getId()) {
	    			$client_member = $member;
	    		}
	    		
	    		if ($client_member instanceof Member) {
	    			$client_cp_value = CustomPropertyValues::getCustomPropertyValue($member->getObjectId(), $client_template_cp->getId());
	    			if ($client_cp_value instanceof CustomPropertyValue && is_numeric($client_cp_value->getValue()) && $client_cp_value->getValue() > 0) {
	    				$template_id = $client_cp_value->getValue();
	    			}
	    		}
	    	}
    	}
    	
    	if (!$template_id) {
    		// get the default invoice template
    		$template_id = IncomeInvoiceTemplates::instance()->findOne(array("id" => true, "conditions" => "is_default=1"));
    	}
    	
    	return $template_id;
    }
    
    /**
	 * get template by id
	 */
	static function getTemplateById($template_id, $member)
	{
		$invoice_template='';
    	
		if ($template_id == 0) {
    		$template_id = IncomeInvoiceTemplates::getMemberInvoiceTemplateId($member);
    	}
		
		if($template_id)
		{
		    $invoice_template = IncomeInvoiceTemplates::instance()->findById($template_id);
		}

		return $invoice_template;
	}

    /**
	 * add discount to lines if allocation percentage is less than 100%
	 */
	static function addDiscountToLines($invoice_lines, $allocation_percentage)
	{
		$discount_percentage = 100 - $allocation_percentage;
    	if ($discount_percentage > 0) {
    		
    		// calculate the discount amount
    		$total_lines_amount = 0;
    		foreach ($invoice_lines as $l) {
    			$total_lines_amount += $l['total'];
				$prev_line = $l;
    		}
    		$discount_amount = $total_lines_amount * ($discount_percentage / 100);
    		
    		$invoice_lines[] = array(
    				'currency_id' => $prev_line['currency_id'],
    				'amount' => 1,
    				'unit_value' => -1 * $discount_amount,
    				'total' => -1 * $discount_amount,
    				'detail' => lang('discount')." (".$discount_percentage."%)",
					'item_type_id' => 9 //3 --> type discount allocation percentage
    		);
    	}

		return $invoice_lines;
	}

	/**
	 * get the the lines to add discout "project Tasks Previously Billed"
	 */
	static function addLinesToDiscountPreviousBill($invoice_template, $project_member, $forced_invoice_number, $invoice_lines)
	{
		
	    if($invoice_template->getTemplateType()=='fixed fee tasks' && $invoice_template->getTaskInvoiceBillingType()=='Invoice completed task and discount previous bill')														   
	    {
			$account_summary=IncomeInvoice::getSummaryForMember($project_member);
			$last_invoice_number_without_subfix=substr($account_summary['last_invoice_member'], 0, -2);
			$forced_invoice_number_without_subfix=substr($forced_invoice_number, 0, -2); 
			
			///check if the last invoice number without subfix  is different to the current invoice number without prefix (different group) && total>0
			if($account_summary['totals_btd'][1]['total_labor']>0)
			{
				if($forced_invoice_number_without_subfix<>$last_invoice_number_without_subfix)
				{
				    //|| ($prev_group_invoices<>$forced_invoice_number_without_subfix
				    $total_account_summary_btd=$account_summary['totals_btd'][1]['total_labor'];
		 	        $invoice_lines[] = array(
		 			    //'currency_id' => $prev_line['currency_id'],
		 			    'amount' => 1,
		 			    'unit_value' => -1 * $account_summary['totals_btd'][1]['total_labor'],
		 			    'total' => -1 * $account_summary['totals_btd'][1]['total_labor'],
		 			    'detail' => lang('project Tasks Previously Billed'),
					    'item_type_id' => 8 //3 --> type discount project Tasks Previously Billed
		 	        );
				}else{ //if is the same group, check previus invoice has disscount previusly billed
					$prev_invoice=IncomeInvoices::instance()->findById($account_summary['last_invoice_member_id']);
					$prev_invoice_lines=$prev_invoice->getInvoiceLines();
					foreach ($prev_invoice_lines as $prev_line) {
						if($prev_line->getItemTypeId()==8)
						{
							$invoice_lines[] = array(
								//'currency_id' => $prev_line['currency_id'],
								'amount' => 1,
								'unit_value' => -1 * abs($prev_line->getTotal()),
								'total' => -1 * abs($prev_line->getTotal()),
								'detail' => lang('project Tasks Previously Billed'),
								'item_type_id' => 8 //3 --> type discount project Tasks Previously Billed
							);
						}
					}
				}
		    }
	    }

		return $invoice_lines;
	}

	/**
	 * Receive and prepare all the data, then generate the invoice
	 */
    static function generateInvoiceFromTemplateAndObjects(Member $member, 
	                                                      IncomeInvoiceTemplate $invoice_template, 
														  $timeslots = array(), 
														  $payment_receipts = array(), 
														  $billing_period_from = '', 
														  $billing_period_to = '', 
														  $only_return_raw_data = false, 
														  $forced_client_member = null, 
														  $allocation_percentage=100, 
														  $forced_invoice_number=null,
														  $tasks = array()) {
		Env::useHelper('functions', 'crpm');
    	Env::useHelper('functions', 'income');
    	
    	$client_ot = ObjectTypes::findByName('customer');
		$project_ot = ObjectTypes::findByName('project');
    	
    	$project_member = null;
    	
    	if ($client_ot->getId() == $member->getObjectTypeId()) {
    		$client_member = $member;
    	} else {
    		$client_member = null;
    		$billing_client_association_id = income_config_option('billing_client_dim_assoc');
    		$billing_client_association = DimensionMemberAssociations::instance()->findById($billing_client_association_id);
    		$billing_client_association_code = $billing_client_association ? $billing_client_association->getCode() : null;
    		
    		$client_member = get_client_member_of_project($member, $billing_client_association_code);
    		if (!$client_member) {
    			// if not billing client, then use the default client
    			$client_member = get_client_member_of_project($member);
    		}
    		
    		// the main member is the project
    		$project_member = $member;
    		
    		$invoice_name_suffix = '';
    		if ($forced_client_member instanceof Member) {
    			$invoice_name_suffix = " - ".$forced_client_member->getName();
    			$client_member = $forced_client_member;
    		}
    	}
    	
    	if (!$client_member instanceof Member) {
			throw new Exception(lang('cant generate invoice there is no client related to the project', strtolower($project_ot->getObjectTypeName()), $member->getName()));
    	}
    	$customer = Customers::instance()->findById($client_member->getObjectId());
    	if (!$customer instanceof Customer) {
    		throw new Exception(lang('cant generate invoice there is no client related to the project', strtolower($project_ot->getObjectTypeName()), $member->getName()));
    	}
    	
		// // ensure that invoiced objects are not included in this batch
    	// $tmp_ta = array();
    	// foreach ($tasks as $t) {
    	// 	if ($t->getColumnValue('invoicing_status') == 'pending') $tmp_ta[] = $t;
    	// }
    	// $tasks = $tmp_ta;

    	// ensure that invoiced objects are not included in this batch
    	$tmp_t = array();
    	foreach ($timeslots as $t) {
    		if ($t->getColumnValue('invoicing_status') == 'pending') $tmp_t[] = $t;
    	}
    	$timeslots = $tmp_t;
    	$tmp_e = array();
    	foreach ($payment_receipts as $e) {
    		if ($e->getColumnValue('invoicing_status') == 'pending') $tmp_e[] = $e;
    	}
    	$payment_receipts = $tmp_e;
    	
    	// group timeslots, expenses, and tasks, then generate invoice lines
		$grouped_tasks='';
	    $grouped_tasks= self::groupTasksUsingTemplate($invoice_template, $tasks);

		$grouped_timeslots='';
   	    $grouped_timeslots = self::groupTimeslotsUsingTemplate($invoice_template, $timeslots);
		
		$grouped_payment_receipts='';
   	    $grouped_payment_receipts = self::groupActualExpensesUsingTemplate($invoice_template, $payment_receipts);
    	
		//build invoice lines
		$invoice_lines = self::buildInvoiceLinesFromGroups($invoice_template, 
		                                                   $grouped_timeslots, 
														   $grouped_payment_receipts, 
														   $billing_period_from, 
														   $billing_period_to, 
														   $grouped_tasks);
    	
	
		//add discount line "project Tasks Previously Billed" if it apply
		$invoice_lines=self::addLinesToDiscountPreviousBill($invoice_template, $project_member, $forced_invoice_number, $invoice_lines);

		//add discount if allocation percentage is less than 100%, only if split mode = 'multiple-with-discount'
		if (income_config_option('split_invoice_for_billing_clients') == 'multiple-with-discount') {
			$invoice_lines=self::addDiscountToLines($invoice_lines, $allocation_percentage);
		}
    	
    	// default invoice type
    	$invoice_type = ObjectSubtypes::instance()->findOne(array('conditions' => "code='invoice'"));
    	$invoice_type_id = $invoice_type->getId();
    	
    	// taxes
    	$tax_ids = IncomeTaxes::instance()->findAll(array('id' => true, 'conditions' => 'select_default=1 AND removed_by_id = 0'));
    	$taxes = array();
    	foreach ($tax_ids as $tid) $taxes[$tid] = 1;
    	
    	// copies
    	$copy_ids = IncomeCopies::instance()->findAll(array('id' => true, 'conditions' => 'select_default=1 AND removed_by_id = 0'));
    	$copies = array();
    	foreach ($copy_ids as $cid) $copies[$cid] = 1;
    	
    	// only logged user is subscribed
    	$subscribers = array('user_' . logged_user()->getId() => 1);
    	
    	$inv_template_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'invoice_template');
    	$billing_from_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'billing_period_from');
    	$billing_to_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'billing_period_to');
    	$po_number_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'po_number');
    	
    	$date_str = format_date(DateTimeValueLib::now());
    	
    	// get billing information from client
    	$client_company = Contacts::instance()->findById($customer->getContactId());
		$all_client_phones = array();
		if ($client_company instanceof Contact) {
			$all_client_phones = $client_company->getAllPhones();
		}
    	
    	// get the billing contact of the project or client, if it doesn't have a billing contact then use the client information
    	$billing_contact = null;
    	$billing_contact_cp = null;
		$is_client_billing_contact_cp = false;
		
		$attn_value='';

		// try to get the billing contact from project
    	if ($project_member instanceof Member) {
    		$billing_contact_cp = CustomProperties::getCustomPropertyByCode($project_member->getObjectTypeId(), 'billing_contact');
			if ($billing_contact_cp) {
				$cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $billing_contact_cp->getId());
				if ($cp_val){
					$billing_contact = Contacts::instance()->findById($cp_val->getValue());
					if($billing_contact)
					{
						$attn_value=$billing_contact->getName();
					}
				} 
			}
    	}

		// try to get the billing contact from client
    	if (!$billing_contact) {
    		$billing_contact_cp = CustomProperties::getCustomPropertyByCode($customer->getObjectTypeId(), 'billing_contact');
			if ($billing_contact_cp) {
				$cp_val = CustomPropertyValues::getCustomPropertyValue($client_member->getObjectId(), $billing_contact_cp->getId());
				if ($cp_val){
					$billing_contact = Contacts::instance()->findById($cp_val->getValue());
					if($billing_contact && $attn_value=='')
					{
						$attn_value=$billing_contact->getName();
					}
				} 

				if ($billing_contact instanceof Contact) $is_client_billing_contact_cp = true;
			}
    	}

		$company_address = null;
        
    	if (!$billing_contact instanceof Contact) {
    		$billing_contact = $client_company;
    	} else {
    		$bc_phones = $billing_contact->getAllPhones();
    		if (!empty($bc_phones)) $all_client_phones = $bc_phones;

			// if billing contact comes from client, try to get billing address from client's custom property
			if ($is_client_billing_contact_cp) {
				$company_address = get_client_billing_address_from_cp($client_member);
			}
    	}
		// if we don't have billing address yet, get it from the billing contact
		if (!$company_address) {
			$company_address = isset($billing_contact)?$billing_contact->getAddress('work'):false;
			if (!$company_address) isset($billing_contact)?$billing_contact->getAddress('home'):false;
			if (!$company_address) isset($billing_contact)?$billing_contact->getAddress('postal'):false;
			if (!$company_address) isset($billing_contact)?$billing_contact->getAddress('other'):false;
			
			if ($company_address && trim($company_address->getStreet()) == '') {
				$company_address = null;
			}
		}


		//if config option is set to no use normal mode
		if(income_config_option('if_no_billing_contact_then_use_main_address'))
		{
			// if we don't have billing address yet, get it from the client billing address
	    	if(!$company_address && $client_member instanceof Member)
    		{
	    		$company_address = get_client_billing_address_from_cp($client_member);
    		}
		
	    	// if we don't have billing address yet, get it from the client contact address
    	    if (!$company_address && $client_company) {
    		    if (!$company_address) $company_address = $client_company->getAddress('work');
    		    if (!$company_address) $company_address = $client_company->getAddress('home');
    		    if (!$company_address) $company_address = $client_company->getAddress('postal');
    		    if (!$company_address) $company_address = $client_company->getAddress('other');
    	    }
		}
    	
    	$customer_address = $company_address ? $company_address->getStreet() : '';
    	$customer_city = $company_address ? $company_address->getCity() : '';
    	$customer_state = $company_address ? $company_address->getState() : '';
    	$customer_country = $company_address ? $company_address->getCountry() : '';
    	$customer_zipcode = $company_address ? $company_address->getZipCode() : '';

    	$customer_email = isset($billing_contact)?$billing_contact->getEmailAddress():false;
		// if (!$customer_email && $client_company && $client_company->getId() != $billing_contact->getId()) {
		// 	$customer_email = $client_company->getEmailAddress();
		// }

    	$customer_phone = '';
    	if (count($all_client_phones) > 0) {
    		$customer_phone = $all_client_phones[0]->getNumber();
    	}
    	
		$project_invoice_memo = '';
    	$project_invoice_memo_cp = $project_member instanceof Member ? CustomProperties::getCustomPropertyByCode($project_member->getObjectTypeId(), 'invoice_memo'): null;
    	if ($project_invoice_memo_cp) {
    		$project_invoice_memo_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $project_invoice_memo_cp->getId());
    		if ($project_invoice_memo_val) $project_invoice_memo = $project_invoice_memo_val->getValue();
    	}
    	
    	$project_and_po_number_text = "";
    	if ($po_number_cp instanceof CustomProperty && $member instanceof Member) {
    		$project_po_num_cp = CustomProperties::getCustomPropertyByCode($member->getObjectTypeId(), "po_number");
    		$po_number_cp_val = "";
    		if ($project_po_num_cp instanceof CustomProperty) {
    			$cpval = CustomPropertyValues::getCustomPropertyValue($member->getObjectId(), $project_po_num_cp->getId());
    			if ($cpval instanceof CustomPropertyValue) {
    				$po_number_cp_val = $cpval->getValue();
    			}
    		}
    		
    		//$project_and_po_number_text = $member->getName();
    		if ($po_number_cp_val != "") {
				//$project_and_po_number_text .= " / $po_number_cp_val";
				$project_and_po_number_text = $po_number_cp_val;
    		}
    	}
    	
    	// ---

		$inv_customer_data = array(
			'name' => $customer->getName(),
			'rut' => $customer->getCustomerRut(),
			'number' => $customer->getCustomerNumber(),
			'address' => $customer_address,
			'city' => $customer_city,
			'state' => $customer_state,
			'country' => $customer_country,
			'zipcode' => $customer_zipcode,
			'email' => $customer_email,
			'phone' => $customer_phone,
		);
		
		
		if (income_config_option('split_invoice_for_billing_clients') == 'single') {
			
			Hook::fire("automatic_invoice_override_customer_data", array('member'=>$member), $inv_customer_data);

		} else if (income_config_option('split_invoice_for_billing_clients') == 'multiple') {

			// if the invoice is splitted in multiple clients modify each line amounts depending on the invoice allocation percentage
			$invoice_lines = self::recalculateInvoiceLinesUsingAllocationPercentage($invoice_lines, $allocation_percentage);
		}
    	
    	// invoice general data
    	$invoice_data = array(
    			'name' => 'Invoice for '.$member->getName() . $invoice_name_suffix . ' - '.$date_str,
    			'date' => $date_str,
    			'customer_name' => array_var($inv_customer_data, 'name', ''),
    			'customer_rut' => array_var($inv_customer_data, 'rut', ''),
    			'customer_number' => array_var($inv_customer_data, 'number', ''),
    			'customer_address' => array_var($inv_customer_data, 'address', ''),
    			'customer_city' => array_var($inv_customer_data, 'city', ''),
    			'customer_state' => array_var($inv_customer_data, 'state', ''),
    			'customer_country' => array_var($inv_customer_data, 'country', ''),
    			'customer_zipcode' => array_var($inv_customer_data, 'zipcode', ''),
    			'customer_email' => array_var($inv_customer_data, 'email', ''),
    			'customer_phone' => array_var($inv_customer_data, 'phone', ''),
    			'invoice_type_id' => $invoice_type_id,
    			'additional_information' => $project_invoice_memo, 
    			'ordered_by_id' => isset($billing_contact)?$billing_contact->getId():false,
    			'status' => 'pending',
    			'invoice_notebook_id' => 0,
    			'series' => '',
    			'invoice_number' => $forced_invoice_number ? $forced_invoice_number : 0,
				'allocation_percentage' => $allocation_percentage,
				'attn'=>$attn_value,
    	);
    	
    	$object_custom_properties = array(
    			$inv_template_cp->getId() => $invoice_template->getId(),
    			$billing_from_cp->getId() => $billing_period_from,
    			$billing_to_cp->getId() => $billing_period_to,
    	);
    	if ($po_number_cp instanceof CustomProperty) {
    		$object_custom_properties[$po_number_cp->getId()] = $project_and_po_number_text;
    	}

		$invoice_members = array($member->getId());
		if ($client_member && $client_member->getId() != $member->getId()) {
			$invoice_members[] = $client_member->getId();
		}
    	
    	$data = array(
    			'invoice' => $invoice_data,
    			'taxes' => $taxes,
    			'copies' => $copies,
    			'members' => $invoice_members,
    			'invoice_lines' => $invoice_lines,
    			'subscribers' => $subscribers,
    			'object_custom_properties' => $object_custom_properties,
    			'client_member_id' => $client_member->getId(),
    	);
    	
    	// for batch invoicing pre-invoice generation
    	if ($only_return_raw_data) {
    		return $data;
    	}
    	
    	// Save the invoice
    	$inv = self::createInvoiceFromArrayData($data, $timeslots, $payment_receipts, $tasks, $invoice_template);
    	
    	return $inv;
    }
    
    
    static function createInvoiceFromArrayData($data, $timeslots, $payment_receipts, $tasks = null, $invoice_template = null) {
    	
    	$invoice = new IncomeInvoice();
    	$invoice->disableMandatoryFieldsValidation(true);
    	
    	$member_ids = array_var($data, 'members');
    	$invoice_data = array_var($data, 'invoice');
    	$_POST = $data;
    	
		$inv= IncomeInvoices::save_invoice($invoice, $invoice_data, $member_ids, $invoice_template);
    	
    	if ($inv instanceof IncomeInvoice) {
    		
			// change the associated timeslots status to Invoiced (invoiced), this also fires the hooks that reclassifies them in the "invoiced" member at the approval status dim
			$inv->changeAssociatedTimeslotsStatus('invoiced', $timeslots);
    		
    		foreach ($payment_receipts as $payment_receipt) {
    			// update payment_receipt invoice id and status
    			$payment_receipt->setColumnValue('invoice_id', $inv->getId());
    			$payment_receipt->changeInvoicingStatus('invoiced');
    		}

			foreach ($tasks as $task) {
    			// update tasks invoice id and status
    			$task->setColumnValue('invoice_id', $inv->getId());
				$taskCompleted='invoiced';

				if($task->getPercentCompleted()<100)
				{
					$taskCompleted='partially_invoiced';
				}

    			$task->changeInvoicingStatus($taskCompleted);
    		}
    	}

		ApplicationLogs::createLog($inv, ApplicationLogs::ACTION_ADD);
    	
    	return $inv;
    }

	/**
	 * Recalculates each invoice line's amounts applying the allocation percentage, this is used only if invoice split mode is 'multiple'
	 * @return array the modified invoice lines after applying allocation percentage to each one of them
	 * @param array $invoice_lines The lines to be recalculated
	 * @param int $allocation_percentage the percentage used to calculate new values
	 */
	static function recalculateInvoiceLinesUsingAllocationPercentage($invoice_lines, $allocation_percentage) {

		if ($allocation_percentage < 100 && $allocation_percentage > 0) {

			foreach ($invoice_lines as &$line) {
				$new_total = $line['total'] * $allocation_percentage / 100;
				$new_unit_value = $new_total / $line['amount'];

				$line['total'] = $new_total;
				$line['unit_value'] = $new_unit_value;
			}
		}

		return $invoice_lines;
	}
    
    
	/**
	 * Function to order multidemensional array by a given col
	 */
    static function array_msort($array, $cols)
	{
		$colarr = array();
		foreach ($cols as $col => $order) {
			$colarr[$col] = array();
			foreach ($array as $k => $row) { $colarr[$col]['_'.$k] = strtolower($row[$col]); }
		}
		$eval = 'array_multisort(';
		foreach ($cols as $col => $order) {
			$eval .= '$colarr[\''.$col.'\'],'.$order.',';
		}
		$eval = substr($eval,0,-1).');';
		eval($eval);
		$ret = array();
		foreach ($colarr as $col => $arr) {
			foreach ($arr as $k => $v) {
				$k = substr($k,1);
				if (!isset($ret[$k])) $ret[$k] = $array[$k];
				$ret[$k][$col] = $array[$k][$col];
			}
		}
		return $ret;
	}



	static function buildInvoiceLinesTimeslotsLines($invoice_lines,$template_info, $cat_dim_id, $grouped,$billing_period_from, $billing_period_to)
	{
		$projects_dim = Dimensions::findByCode("customer_project");	// project name
		$project_ot = ObjectTypes::findByName('project');

		foreach ($grouped as $group) {
			$line_text='';
			$amount = 0;
			$total = 0;
			$timeslots = $group['items'];
			$category_name='';
			$line_text_date='';
			$first_item_date = null;
			$last_item_date = null;

			foreach ($timeslots as $ts) {// @var $ts Timeslot 
				
				$currency_id = $ts->getRateCurrencyId();
				$ts_minutes = $ts->getMinutes();
				$amount += $ts->getMinutes() / 60;
				$total += $ts->getFixedBilling();

				// get timeslot date
				$line_date = $ts->getStartTime();

				if ($line_date instanceof DateTimeValue) {
					// store first item date of the group
					if (is_null($first_item_date) || $line_date->getTimestamp() <= $first_item_date->getTimestamp()) {
						$first_item_date = $line_date;
					}
	
					// store last item date of the group
					if (is_null($last_item_date) || $line_date->getTimestamp() >= $last_item_date->getTimestamp()) {
						$last_item_date = $line_date;
					}
				}
			
				foreach ($ts->getMembers() as $ts_member) {
					if ($ts_member->getDimensionId() == $cat_dim_id) {
						$category_id = $ts_member->getId();
						$category_name=$ts_member->getName();

						if ($template_info['group_by'] == 'timeslots' && !isset($group['category_id'])) {
							$group['category_id'] = $category_id;
						}
						break;
					}

				}

				//try to get the project name
				$project_number='';
				$project_name='';
				
				$projectLine= self::getInvoiceLinesProject($ts);
				$project_number= array_var($projectLine, 'project_number', "");
				$project_name= array_var($projectLine, 'project_name', "");

				
			}
			

			//// 0 columna DATE
			if ($template_info['include_time_line_date']=='Billing from date' ) {
				
				$line_text_date = format_date(strtotime($billing_period_from),null,0);

			} else if($template_info['include_time_line_date']=='Billing to date' ) {
				
				$line_text_date = format_date(strtotime($billing_period_to),null,0);
			
			} else if($template_info['include_time_line_date']=='Invoice date' ) {
			
				$line_text_date = format_date(null,null,0);
			
			} else if ($template_info['include_time_line_date']=='Date of the earliest item') {
			
				$line_text_date = $first_item_date instanceof DateTimeValue ? format_date($first_item_date) : '';
			
			} else if ($template_info['include_time_line_date']=='Date of the most recent item') {
			
				$line_text_date = $last_item_date instanceof DateTimeValue ? format_date($last_item_date) : '';
			
			} else if($template_info['include_time_line_date']=='Item date') {
			
				$line_text_date = format_date($line_date);
			}

			//1 person name
			if($template_info['include_user_name'] == 'Full name')
			{
				$contact = Contacts::instance()->findById($ts->getContactId());
				if ($contact) $line_text .= $contact->getObjectName();
				else $line_text = $ts->getObjectName();
			}
			
			$labor_separator='';
			$separator = array (
				"Blank space"=>' ',
				"Line break" =>"\n",
				"Middle dash" =>' - ',
				"Forward slash" => ' / ',
			  );

			$labor_separator=$separator[$template_info['labor_detail_separator_value']];


			///line detail for  expenses
			$line_text='';
			// if($template_info['include_expense_project_number'])
			// {
			// 	$line_text.=$project_number;
			// }

			if($template_info['include_project_name'])
			{
				$line_text.=$project_name;
			}

			$category_id=$group['category_id'];
			$product_type_id=0;

			if($template_info['group_by']=='users' || $template_info['group_by']=='date-users' || $template_info['group_by']=='project')
			{
			 $category_id=$template_info['category_for_timeslot'];
			 $product_type_id=$template_info['product_type_for_timeslot'];

				$cat_m = Members::getMemberById($category_id);
				if ($cat_m instanceof Member) 
				{
					$category_name = $cat_m->getName();
				}
			}

			//2 labor category
			if($template_info['include_labor_category'] )
			{
				$line_text.=$labor_separator.$category_name;//$labor_separator
			}
			//3 task name
			if($template_info['include_task_name'] )
			{
				$task = $ts->getRelObject();
				if ($task instanceof ProjectTask)
				{
					 $line_text .= $labor_separator.$task->getName();
				}
			}

			//4 time entry
			if($template_info['include_time_entry_description'] )
			{
				$line_text .=$labor_separator.$ts->getDescription();
			}

			if($template_info['include_project_name'])
			{
				$line_text.=$labor_separator.$group['name'];
			}
			
			if ($amount > 0) {

				//clean first and last separator
				$line_text=trim($line_text, $labor_separator);

				$data = array(
					'currency_id' => $currency_id,
					'amount' => $amount,
					'unit_value' => $total / $amount,
					'total' => $total,
					'detail' => $line_text,//$group['name'],
					'date' => $line_text_date,
					'category_id' => $category_id,
					'product_type_id' => $product_type_id,

					'user_id' => array_var($group, 'user_id', 0),
					'from_object_type_id' => Timeslots::instance()->getObjectTypeId(),
					'member_id' => array_var($group, 'project_id'),
					
				);
				$invoice_lines[] = $data;
			}
		}

		return $invoice_lines;
	}


	/**
	 * build task lines
	 */
	static function buildInvoiceLinesTasksLines($invoice_lines,
	                                            $template_info, 
												$cat_dim_id, 
												$grouped = array(),
												$billing_period_from, 
												$billing_period_to,
												$task_separator=' ')
	{
		$first_date_prev= date("Y/m/d");
		$last_date='';
		$first_date='';
		$last_date_prev='';
		$line_date='';
		$projects_dim = Dimensions::findByCode("customer_project");	// project name
		$project_ot = ObjectTypes::findByName('project');
		foreach ($grouped as $group) {
			$line_text='';
			$amount = 0;
			$total = 0;
			$tasks = $group['items'];
			$category_name='';
			$line_text_date='';

			foreach ($tasks as $ts) {// @var $ts Task
				$currency_id='';
				
			     $category_id_first=true;
				 $category_id=0;
				 $ts_object=ProjectTasks::instance()->findById($ts['id']);
				foreach ($ts_object->getMembers() as $ts_member) {
				 	if ($ts_member->getDimensionId() == $cat_dim_id) {
				 		$category_id = $ts_member->getId();
				 		break;
				 	}
				}
				
				//try to get the project name
				$project_number='';
				$project_name='';
			}
			

			//// 0 columna DATE
			if($template_info['include_time_line_date']=='Billing from date' )
			{
				$line_text_date=format_date(strtotime($billing_period_from),null,0);
			}else if($template_info['include_time_line_date']=='Billing to date' )
			{
				$line_text_date=format_date(strtotime($billing_period_to),null,0);
			}else if($template_info['include_time_line_date']=='Invoice date' )
			{
				$line_text_date=format_date(null,null,0);
			}else if ($template_info['include_time_line_date']=='Date of the earliest item')
			{
				$line_text_date=$first_date_prev;
			}else if ($template_info['include_time_line_date']=='Date of the most recent item')
			{
				$line_text_date=$last_date_prev;
			}else if($template_info['include_time_line_date']=='Item date')
			{
				$line_text_date=format_date($line_date);
			}

			//1 person name
			$line_text='';
			if($template_info['include_user_name'] == 'Full name')
			{
				$contact = Contacts::instance()->findById($ts['assignedToContactId']);
				if ($contact instanceof Contact)
				{
					$line_text .= $contact->getObjectName();
				}
			}

			if($template_info['include_project_name'])
			{
				$line_text.=$task_separator.$project_name;
			}

			//$category_id=$group['category_id'];
			$product_type_id=0;

			$category_name='';
			if($template_info['category_for_timeslot']!='')
			{
			 $category_id=$template_info['category_for_timeslot'];
				$cat_m = Members::getMemberById($category_id);
				if ($cat_m instanceof Member) 
				{
					$category_name = $cat_m->getName();
				}
			}

			//2 labor category
			if($template_info['include_labor_category'] )
			{
				$cat_m = Members::getMemberById($category_id);
				if ($cat_m instanceof Member) 
				{
					$category_name = $cat_m->getName();
				}
				$line_text.=$task_separator.$category_name;
			}

			if($template_info['include_task_name'] )
			{
				$line_text .= $task_separator.$ts['name'];
				/*if ($ts instanceof ProjectTask)
				{
					 $line_text .= '  '.$ts->getName();
				}*/
			}

			

			// if($template_info['include_project_name'])
			// {
			// 	$line_text.=$task_separator.$group['name'];
			// }
			//clean first and last separator
			$line_text=trim($line_text, $task_separator);

			//if ($amount > 0) {
				$data = array(
					'currency_id' => $currency_id,
					'amount' => 1,//$ts['worked_time'],//$amount, //////////////quantity
					'unit_value' => $ts['earned_value'],//$ts['earned_value']/$ts['worked_time'],//$total / $amount,  ////unit value
					'total' => $ts['earned_value'],
					'detail' => $line_text,
					'date' => $line_text_date,
					'category_id' => $category_id,
					'product_type_id' => $product_type_id,

					'user_id' => $ts['assignedToContactId'],//$group['user_id'],
					'from_object_type_id' => Timeslots::instance()->getObjectTypeId(),
					'member_id' => array_var($group, 'project_id'),
					
				);
				$invoice_lines[] = $data;
			//}
		}

		return $invoice_lines;
	}

	/**
	 * build task lines
	 */
	static function buildInvoiceLinesExpensesLines($invoice_lines,
	                                               $template_info, 
												   $cat_dim_id, 
												   $grouped, 
												   $billing_period_from, 
												   $billing_period_to,
												   $expense_separator=' ')
	{
		$line_date='';
		$total_expense=0;
		$projects_dim = Dimensions::findByCode("customer_project");	// project name
		$project_ot = ObjectTypes::findByName('project');
		$exp_cat_dim = Dimensions::findByCode('expense_categories');
		$suplliers_dim = Dimensions::findByCode('suppliers');
		foreach ($grouped as $group) {
			$line_detail='';
			$payment_receipts = $group['items'];

			$first_item_date = null;
			$last_item_date = null;
			
			foreach ($payment_receipts as $p) {/* @var $p PaymentReceipt */
				$category_id = 0;
				$category_name = null;
				if ($exp_cat_dim instanceof Dimension) {
					$ts_members = $p->getMembers();
					foreach ($ts_members as $ts_member) {
						if ($ts_member->getDimensionId() == $exp_cat_dim->getId()) {
							$category_id = $ts_member->getId();
							$category_name = $ts_member->getName();
							break;
						}
					}
				}

				//get vendor
				$vendor_name='';
				$vendor_id='';
				if ($suplliers_dim instanceof Dimension) {
					$ts_members = $p->getMembers();
					foreach ($ts_members as $ts_member) {
						if ($ts_member->getDimensionId() == $suplliers_dim->getId()) {
							$vendor_id = $ts_member->getId();
							$vendor_name=$ts_member->getName();
							break;
						}
					}
				}

				//get bill number custom property
				$expense_bill_no='';
				$payment_receipt_ot = ObjectTypes::findByName('payment_receipt');
				if($payment_receipt_ot instanceof ObjectType)
				{
					$expense_bill_no_cp = CustomProperties::getCustomPropertyByCode($payment_receipt_ot->getId(), 'payment_receipt_bill_no');		
					if($expense_bill_no_cp instanceof CustomProperty)
					{
	
						$cp_val=CustomPropertyValues::getCustomPropertyValue($p->getId(), $expense_bill_no_cp->getId());
						
						if($cp_val instanceof CustomPropertyValue && $cp_val)
						{
							$expense_bill_no = $cp_val->getValue();
						}
					}
				}


				//try to get the project name
				$project_number='';
				$project_name='';
				
				$projectLine= self::getInvoiceLinesProject($p);
				$project_number= array_var($projectLine, 'project_number', "");
				$project_name= array_var($projectLine, 'project_name', "");

				// get timeslot date
				$line_date = $p->getDate();

				if ($line_date instanceof DateTimeValue) {
					// store first item date of the group
					if (is_null($first_item_date) || $line_date->getTimestamp() <= $first_item_date->getTimestamp()) {
						$first_item_date = $line_date;
					}
	
					// store last item date of the group
					if (is_null($last_item_date) || $line_date->getTimestamp() >= $last_item_date->getTimestamp()) {
						$last_item_date = $line_date;
					}
				}

				// 0 date
				//// 0 columna DATE
				if ($template_info['include_exp_type_date']=='Billing from date' ) {

					$line_text_date = format_date(strtotime($billing_period_from),null,0);

				} else if($template_info['include_exp_type_date']=='Billing to date' ) {
					
					$line_text_date = format_date(strtotime($billing_period_to),null,0);

				} else if($template_info['include_exp_type_date']=='Invoice date' ) {
					
					$line_text_date = format_date(null,null,0);

				} else if ($template_info['include_exp_type_date']=='Date of the earliest item') {
					
					$line_text_date = $first_item_date instanceof DateTimeValue ? format_date($first_item_date) : '';

				} else if ($template_info['include_exp_type_date']=='Date of the most recent item') {
					
					$line_text_date = $last_item_date instanceof DateTimeValue ? format_date($last_item_date) : '';

				} else if($template_info['include_exp_type_date']=='Item date') {
					
					$line_text_date = format_date($line_date,null,0);
				}

				///line detail for  expenses
				$line_detail='';
				if($template_info['include_expense_project_number'])
				{
					$line_detail.=$project_number;
				}

				if($template_info['include_expense_project_name'])
				{
					$line_detail.=$expense_separator.$project_name;
				}

				//add task name
				if($template_info['include_expense_task_name'])
				{
					//find task name by task id
					$task = ProjectTasks::instance()->findById($p->getTaskId());
					if($task instanceof ProjectTask)
					{
    				 $line_detail .= $expense_separator.$task->getName();
					}
				}

				//1 paid by
				
				if ($template_info['include_paid_by']) {
					$contact = Contacts::instance()->findById($p->getPaidById());
					$line_detail .= $expense_separator.$contact instanceof Contact ? $contact->getObjectName() : 'Without user';
				}

				//2 expense category
				if ($template_info['include_expense_category'] && isset($category_name)) {

					$line_detail .= $expense_separator.$category_name;
				}
				
				//3 product type
				if ($template_info['include_product_type']) {
					$product_type_line = ProductTypes::getById($p->getProductTypeId());
					if($product_type_line instanceof ProductType)
					{
						$line_detail .= $expense_separator.$product_type_line->getName()?$product_type_line->getName():'';
					}
				}

				//4 expense name
				if ($template_info['include_expense_name']) {
						$line_detail .= $expense_separator.$p->getName();
				}
				
				//4 expense description
				if($template_info['include_expense_description'])
				{
						$line_detail .= $expense_separator.$p->getDescription();
				}

				// expense vendor name 
				if($template_info['expense_detail_vendor_name'])
				{
						$line_detail .= $expense_separator.$vendor_name;
				}

				// expense bill number
				if($template_info['expense_detail_bill_no'])
				{
						$line_detail .= $expense_separator.$expense_bill_no;
				}

				$group_by_cat_id = self::getExpenseGroupByCategoryId($p, $template_info);
				
				//clean first and last separator
				$line_detail=trim($line_detail, $expense_separator);

				// current expense total
				$current_expense_total = $p->getTotalPrice() > 0 ? $p->getTotalPrice() : $p->getAmount();

				//sum total expense
				$total_expense += $current_expense_total;

				// ensure we have quantiy > 0 to before creating the invoice
				$quantity = $p->getQuantity();
				if ($quantity <= 0) {
					$quantity = 1;
				}
				// ensure we have unit value > 0 to before creating the invoice
				$unit_price = $p->getUnitPrice() > 0 ? $p->getUnitPrice() : $p->getUnitCost();
				if ($unit_price == 0) {
					$unit_price = $current_expense_total / $quantity;
				}

				$data = array(
						'currency_id' => $p->getCurrencyId(),
						'amount' => $quantity,
						'unit_value' => $unit_price,
						'total' => $current_expense_total,
						'detail' => $line_detail,
						'unit_cost' => $p->getUnitCost(),
						'price_multiplier' => $p->getPriceMultiplier(),
						'product_type_id' => $p->getProductTypeId(),
						'category_id' => $category_id,
						'user_id' => $p->getPaidById(),
						'date' => $line_text_date,//format_date($p->getDate(), null, 0),
						'rel_object_id' => $p->getId(),
						'from_object_type_id' => $p->getObjectTypeId(),
						'member_id' => array_var($group, 'project_id'),
						'group_by_category_id' => $group_by_cat_id,
						
				);
				$invoice_lines[] = $data;
			}
			
		}

		//if lump sum-> not create single line, only add the total
		if($template_info['template_type']=='lumpsum')
		{
			$data2=array(
				'totalExpense'=>$total_expense,
			);
			//overrride all lines
			unset($invoice_lines);

			$invoice_lines[] = $data2;
		}
		return $invoice_lines;
	}
	
	static function getInvoiceLinesProject($p)
	{
		//try to get the project name
		$projects_dim = Dimensions::findByCode("customer_project");	// project name
		$project_ot = ObjectTypes::findByName('project');
		$project=[];
		if($projects_dim instanceof Dimension)
		{
			$ts_members = $p->getMembers();
			foreach ($ts_members as $ts_member) {
				if ($ts_member->getDimensionId() == $projects_dim->getId()) {
					$group_id = $ts_member->getId();
					//$project_name = $ts_member->getName();
					$project['project_name'] = $ts_member->getName();
					$project_member = $ts_member;
					
					//try to get the project number
					if ($project_member instanceof Member) {		
						$project_number_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), 'project_id');		
						if($project_number_cp)
						{
							$cp_val=CustomPropertyValues::getCustomPropertyValue($ts_member->getObjectId(), $project_number_cp->getId());
							
							if($cp_val instanceof CustomPropertyValue && $cp_val)
							{
							    $project['project_number'] = $cp_val->getValue();
							}
						}
					}
					break;
				}
			}
		}

		return $project;
	}

    /**
	 * Returns the Lines ordered by date
	 * grouped_timeslots, grouped_payment_receipts and grouped task always generate
	 * if it not wanted to generate, send '' instead grouped wanted
	 */
	static function buildInvoiceLinesFromGroups(IncomeInvoiceTemplate $invoice_template, 
	                                            $grouped_timeslots = array(), 
												$grouped_payment_receipts = array(), 
												$billing_period_from='', 
												$billing_period_to='',
												$grouped_tasks = array()) {
    	$invoice_lines = array();
    	$template_info = $invoice_template->getArrayInfo();

		$project_ot = ObjectTypes::findByName('project');

		$separator = array (
			"Blank space"=>' ',
			"Line break" =>"\n",
			"Middle dash" =>' - ',
			"Forward slash" => ' / ',
		  );

		$expense_separator=$separator[$template_info['expense_detail_separator_value']];
		$lump_sum_detail_separator_value=$separator[$template_info['lump_sum_detail_separator_value']];
		$task_separator=$separator[$template_info['labor_detail_separator_value']];
		// Single line with grand total of time and expenses
    	if ($template_info['template_type'] == 'lumpsum' 
		   || ($template_info['template_type']=='fixed fee tasks' && $template_info['task_group']=='aggregated into a single line'))
		{
			$timeslot_first_date_prev= date("Y/m/d");
			$timeslot_last_date='';
			$timeslot_first_date='';
			$timeslot_last_date_prev='';
    		$expense_line_date='';
    		$expenses_total = 0;

			//if expenses are not included in lump sum task
			if(($template_info['template_type']=='fixed fee tasks' && $template_info['task_all_expenses_includes']==0) || $template_info['template_type']=='lumpsum')
			{
                // expenses detailed lines
                $invoice_lines=self::buildInvoiceLinesExpensesLines($invoice_lines,
                                                                    $template_info, 
                                                                    '',//$labor_cat_dim->getId(), 
                                                                    $grouped_payment_receipts, 
                                                                    $billing_period_from,
                                                                    $billing_period_to,
																	$expense_separator);
			}

			//only for lump sum, get the total and clean all lines
			if($template_info['template_type']=='lumpsum')
			{
				$expenses_total=isset($invoice_lines[0]['totalExpense'])?$invoice_lines[0]['totalExpense']:0;
				unset($invoice_lines);
			}
			
    		$line_text='';
			$group_count = 0;
			foreach ($grouped_timeslots as $group) {
    			$amount = 0;
				
				// initialize the total with expenses amount ONLY ONCE, if there are more than one timeslot group we don't want to duplicate the expenses amount
				if ($group_count == 0) {
					$total = $expenses_total;
				} else {
					// for the next timeslot groups initialize $total with 0
					$total = 0;
				}
				$group_count++;

    			$task_names_list = array();
    			$timeslots = $group['items'];
			
				$timeslot_last_date='';
    			foreach ($timeslots as $ts) {/* @var $ts Timeslot */
    				$currency_id = $ts->getRateCurrencyId();
    				$ts_minutes = $ts->getMinutes();
    				//$amount += $ts->getMinutes() / 60;
					$amount =1;
					$total += $ts->getFixedBilling();
    				
    				if ($ts->getRelObject() instanceof ContentDataObject) {
    					$task_names_list[$ts->getRelObjectId()] = $ts->getRelObject()->getObjectName();
    				}
					
					try {
						
						$timeslot_first_date = format_date($timeslot_first_date, null, 0);
						if($timeslot_first_date<=$timeslot_first_date_prev)
						{
							$timeslot_first_date_prev=$timeslot_first_date;
							
						}
						$timeslot_last_date = format_date($ts->getEndTime(), null, 0);
						if($timeslot_last_date>=$timeslot_last_date_prev || $timeslot_last_date_prev=='')
						{
							$timeslot_last_date_prev=$timeslot_last_date;
						}

					} catch (Exception $e) {
					}
    			}

				//sort($task_names_list);

    			$project_member = Members::getMemberById($group['id']);
				if ($project_member instanceof Member) {
					// start detail using the project name
					$lump_sum_detail ='';// $project_member->getName();deprecated
					$project_name_line = $project_member->getName();
					
					$project_types_dim = Dimensions::findByCode('project_types');
					$assoc_project_type_id = get_associated_status_member_id($project_member, $project_types_dim, null, false, 'project_project_type');
					$project_type_member = Members::getMemberById($assoc_project_type_id);
	
					/* for number project*/
					$customProject_cp = CustomProperties::getCustomPropertyByCode($project_member->getObjectTypeId(), "project_id");
					if ($customProject_cp instanceof CustomProperty) {
	
						$customProject_cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $customProject_cp->getId());
						if ($customProject_cp_val instanceof CustomPropertyValue) {
							$project_number_line = $customProject_cp_val->getValue();
						}
					}
				}
    			

    			
				// 1 columna DATE
				if($template_info['include_lump_sum_date']=='Billing from date' )
				{
					$line_text_date=format_date(strtotime($billing_period_from),null,0);
				}else if($template_info['include_lump_sum_date']=='Billing to date' )
				{
					$line_text_date=format_date(strtotime($billing_period_to),null,0);
				}else if($template_info['include_lump_sum_date']=='Invoice date' )
				{
					$line_text_date=format_date(null,null,0);
				}else if ($template_info['include_lump_sum_date']=='Date of the earliest item')
				{
					$line_text_date=$timeslot_first_date_prev;
				}else if ($template_info['include_lump_sum_date']=='Date of the most recent item')
				{
					$line_text_date=$timeslot_last_date_prev;
				}else if ($template_info['include_lump_sum_date']=='Job scheduled' && $project_member instanceof Member)
				{
					$project_scheduled_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "job_scheduled");
					if ($project_scheduled_cp instanceof CustomProperty) {
						$project_scheduled_cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $project_scheduled_cp->getId());
						if ($project_scheduled_cp_val instanceof CustomPropertyValue) {
							$line_text_date = format_date(strtotime($project_scheduled_cp_val->getValue()), null, 0);
						}
					}
				}
			
				//3 columna detail
				
				//1 project number

				if($template_info['include_lump_sum_project_number'])
				{
					$line_text=$project_number_line;
				}
				//1 project name
				if($template_info['include_lump_sum_project_name'])
				{
					$line_text.=$lump_sum_detail_separator_value.$project_name_line;
				}

				if($template_info['include_lump_sum_project_type'] && $project_type_member instanceof Member)
				{
					$line_text.=$lump_sum_detail_separator_value.$project_type_member->getName();
				}

    			// if we have a budget defined in the project then include it in the description
				if ($template_info['include_lump_sum_budget_completion'])
				{
					$total_budget = 0;
					if ($project_member instanceof Member) {
						$budget_total_cp = CustomProperties::getCustomPropertyByCode($project_member->getObjectTypeId(), "budget_total");
						if ($budget_total_cp instanceof CustomProperty) {
							$budget_total_cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $budget_total_cp->getId());
							if ($budget_total_cp_val) {
								$total_budget = $budget_total_cp_val->getValue();
								$currency = Currencies::instance()->getCurrency($currency_id);
								$sym = $currency instanceof Currency ? $currency->getSymbol() : '$';
								
								$lump_sum_detail .= "\n" . $budget_total_cp->getName() . ": " . format_money_amount($total_budget, $sym);
							}
						}
						
						// calculate total project billing until now
						$total_project_billed = $total + IncomeInvoices::instance()->getInvoicedTotalInMembers(array($project_member->getId()));
					}

					// if we have a budget -> calculate percentage of completion until now
					$percent_complete_string = "";
					if ($total_budget > 0) {
						
						$completion_percentage = $total_project_billed * 100 / $total_budget;
						if ($completion_percentage > 100) $completion_percentage = 100;
						if ($completion_percentage < 0) $completion_percentage = 0;
						
						$percent_complete_string .= "\n% " . lang('complete') . ": " . round($completion_percentage).'%';
					}
					
					$lump_sum_detail .= "\n" . lang('amount billed to date') . ": " . format_money_amount($total_project_billed, $sym);
					$lump_sum_detail .= $percent_complete_string;
					$line_text.=$lump_sum_detail_separator_value.$lump_sum_detail;
				}

				//1 project from-to
				if($template_info['include_lump_sum_from_to'])
				{
					$line_text.=$lump_sum_detail_separator_value.'From'. ' '.$billing_period_from.' to '.$billing_period_to;
				}
    			
				//clean first and last separator
				$line_text=trim($line_text, $lump_sum_detail_separator_value);
				
    			if ($amount > 0) {
    				$data = array(
    						'currency_id' => $currency_id,
    						'date' => $line_text_date,
							'category_id' => $template_info['category_for_lump_sum'],
							'product_type_id' => $template_info['product_type_for_lump_sum'],
							'amount' => $amount,
    						'unit_value' => $amount > 0 ? $total / $amount : 0,
    						'total' => $total,
    						'detail' =>$line_text,// $lump_sum_detail,
    						'from_object_type_id' => Timeslots::instance()->getObjectTypeId(),
    				);
    				$invoice_lines[] = $data;
    			}
    		}
			
			foreach ($grouped_tasks as $group) {
    			$amount = 0;
    			$total = $expenses_total;
    			$task_names_list = array();
    			$tasks = $group['items'];
			
				$timeslot_last_date='';
				$category_id_first=true;
				$cat_dim=Dimensions::findByCode('hour_types');
    			foreach ($tasks as $ts) {/* @var $ts Timeslot */
    				
    				$amount =1;//$ts['worked_time'];//$amount, //////////////quantity
					$unit_value += $ts['earned_value'];//$ts['worked_time'];//$total / $amount,  ////unit value
					$total += $ts['earned_value'];
			
					 if($template_info['category_for_timeslot']=='')
					 {
					    $ts_object= ProjectTasks::instance()->findById($ts['id']);
					    foreach ($ts_object->getMembers() as $ts_member) {
     						if ($ts_member->getDimensionId() == $cat_dim->getId()) {
	     						$category_id = $ts_member->getId();
		     					$category_name=$ts_member->getName();
					    		break;
						    }
					    }

						if($category_id_first)
					    {
						    $category_id_first=false;
					    	$category_id_old=$category_id;
					    }
					    if ($category_id!=$category_id_old) {
						    $category_id_old='';
     				    }
					}
    			}

    			$project_member = Members::getMemberById($group['id']);
				if ($project_member instanceof Member) {

					// start detail using the project name
					$lump_sum_detail ='';// $project_member->getName();deprecated
					$project_name_line = $project_member->getName();
					
					$project_types_dim = Dimensions::findByCode('project_types');
					$assoc_project_type_id = get_associated_status_member_id($project_member, $project_types_dim, null, false, 'project_project_type');
					$project_type_member = Members::getMemberById($assoc_project_type_id);

					/* for number project*/
					$customProject_cp = CustomProperties::getCustomPropertyByCode($project_member->getObjectTypeId(), "project_id");
					if ($customProject_cp instanceof CustomProperty) {

						$customProject_cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $customProject_cp->getId());
						if ($customProject_cp_val instanceof CustomPropertyValue) {
							$project_number_line = $customProject_cp_val->getValue();
						}
					}
				}
    			
				// 1 columna DATE
				if($template_info['include_lump_sum_date']=='Billing from date' )
				{
					$line_text_date=format_date(strtotime($billing_period_from),null,0);
				}else if($template_info['include_lump_sum_date']=='Billing to date' )
				{
					$line_text_date=format_date(strtotime($billing_period_to),null,0);
				}else if($template_info['include_lump_sum_date']=='Invoice date' )
				{
					$line_text_date=format_date(null,null,0);
				}else if ($template_info['include_lump_sum_date']=='Date of the earliest item')
				{
					$line_text_date=$timeslot_first_date_prev;
				}else if ($template_info['include_lump_sum_date']=='Date of the most recent item')
				{
					$line_text_date=$timeslot_last_date_prev;
				}else if ($template_info['include_lump_sum_date']=='Job scheduled' && $project_member instanceof Member)
				{
					$project_scheduled_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "job_scheduled");
					if ($project_scheduled_cp instanceof CustomProperty) {
						$project_scheduled_cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $project_scheduled_cp->getId());
						if ($project_scheduled_cp_val instanceof CustomPropertyValue) {
							$line_text_date = format_date(strtotime($project_scheduled_cp_val->getValue()), null, 0);
						}
					}
				}

				//3 columna detail
				if($template_info['task_custom_detail'])
				{
					$line_text=$template_info['task_custom_detail'];
				}

				//1 project number
				if($template_info['include_lump_sum_project_number'])
				{
					$line_text=$task_separator.$project_number_line;
				}
				//1 project name
				if($template_info['include_lump_sum_project_name'])
				{
					$line_text.=$task_separator.$project_name_line;
				}

				if($template_info['include_lump_sum_project_type'])
				{
					$line_text.=$task_separator.$project_type_member->getName();
				}

				if($template_info['category_for_lump_sum']!='')
				{
					$category_id_old= $template_info['category_for_lump_sum'];
				}else if ($template_info['category_for_timeslot']!='')
				{
                    $category_id_old= $template_info['category_for_timeslot'];
				}

    			// if we have a budget defined in the project then include it in the description
				if ($template_info['include_lump_sum_budget_completion'])
				{
					$total_budget = 0;
					if ($project_member instanceof Member) {
						$budget_total_cp = CustomProperties::getCustomPropertyByCode($project_member->getObjectTypeId(), "budget_total");
						if ($budget_total_cp instanceof CustomProperty) {
							$budget_total_cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $budget_total_cp->getId());
							if ($budget_total_cp_val) {
								$total_budget = $budget_total_cp_val->getValue();
								$currency = Currencies::instance()->getCurrency($currency_id);
								$sym = $currency instanceof Currency ? $currency->getSymbol() : '$';
								
								$lump_sum_detail .= "\n" . $budget_total_cp->getName() . ": " . format_money_amount($total_budget, $sym);
							}
						}
						
						// calculate total project billing until now
						$total_project_billed = $total + IncomeInvoices::instance()->getInvoicedTotalInMembers(array($project_member->getId()));
					}
					// if we have a budget -> calculate percentage of completion until now
					$percent_complete_string = "";
					if ($total_budget > 0) {
						
						$completion_percentage = $total_project_billed * 100 / $total_budget;
						if ($completion_percentage > 100) $completion_percentage = 100;
						if ($completion_percentage < 0) $completion_percentage = 0;
						
						$percent_complete_string .= "\n% " . lang('complete') . ": " . round($completion_percentage).'%';
					}
					
					$lump_sum_detail .= "\n" . lang('amount billed to date') . ": " . format_money_amount($total_project_billed, $sym);
					$lump_sum_detail .= $percent_complete_string;
					$line_text.=$task_separator.$lump_sum_detail;
				}

				//1 project from-to
				if($template_info['include_lump_sum_from_to'])
				{
					$line_text.=$task_separator.'From'. ' '.$billing_period_from.' to '.$billing_period_to;
				}
    			
				$currency_id=1;
				
				//clean first and last separator
				$line_text=trim($line_text, $task_separator);

    			//if ($amount > 0) {
    				$data = array(
    						'currency_id' => $currency_id,
    						'date' => $line_text_date,
							'category_id' =>$category_id_old,
							'product_type_id' => $template_info['product_type_for_lump_sum'],
							'amount' => 1,
    						'unit_value' => $total,
    						'total' => $total,
    						'detail' =>$line_text,// $lump_sum_detail,
    						'from_object_type_id' => Timeslots::instance()->getObjectTypeId(),
    				);
    				$invoice_lines[] = $data;
					
    			//}
    		}
		} else {

			$projects_dim = Dimensions::findByCode('customer_project');
	    	$labor_cat_dim = Dimensions::findByCode('hour_types');
			$projects_dim = Dimensions::findByCode("customer_project");	// project name

			// timeslots lines
			$invoice_lines=self::buildInvoiceLinesTimeslotsLines($invoice_lines,
										$template_info, 
										$labor_cat_dim->getId(), 
										$grouped_timeslots, 
										$billing_period_from,
										$billing_period_to);

			// Sort invoice lines by grouped name
			if(count($invoice_lines) > 0){
				$groups_array = array('project_labor_category', 'project', 'labor_category', 'users', 'date-users', 'tasks');
				if(in_array($template_info['group_by'], $groups_array)){
					$group_names = array_column($invoice_lines, 'detail');
					array_multisort($group_names, SORT_ASC, $invoice_lines);
				}
			}

			//tasks lines
			$labor_cat_dim=Dimensions::findByCode('hour_types');
			$invoice_lines=self::buildInvoiceLinesTasksLines($invoice_lines,
										$template_info, 
										$labor_cat_dim->getId(), 
										$grouped_tasks, 
										$billing_period_from,
										$billing_period_to,
									    $task_separator);
		

			// Sort invoice lines by grouped name
			if(count($invoice_lines) > 0){
				$groups_array = array('project_labor_category', 'project', 'labor_category', 'users', 'date-users', 'tasks');
				if(in_array($template_info['group_by'], $groups_array)){
					$group_names = array_column($invoice_lines, 'detail');
					array_multisort($group_names, SORT_ASC, $invoice_lines);
				}
			}
	    	
	    	// actual expenses lines
	    	$exp_cat_dim = Dimensions::findByCode('expense_categories');

			$first_date_prev= date("Y/m/d");
			$last_date='';
			$first_date='';
			$last_date_prev='';
			$line_date='';
	    	if ($template_info['exp_group_by'] == 'payment_receipts') {//actual expenses
				$line_detail='';
				$category_name='';
				$line_text_date='';

				$invoice_lines=self::buildInvoiceLinesExpensesLines($invoice_lines,
				                                                    $template_info, 
				                                                    '',//$labor_cat_dim->getId(), 
				                                                    $grouped_payment_receipts, 
				                                                    $billing_period_from,
				                                                    $billing_period_to,
																    $expense_separator);

				// foreach ($grouped_payment_receipts as $group) {
				// 	$line_detail='';
				// 	$payment_receipts = $group['items'];

					
				// 	foreach ($payment_receipts as $p) {/* @var $p PaymentReceipt */
				// 		$category_id = 0;
				// 		if ($exp_cat_dim instanceof Dimension) {
				// 			$ts_members = $p->getMembers();
				// 			foreach ($ts_members as $ts_member) {
				// 				if ($ts_member->getDimensionId() == $exp_cat_dim->getId()) {
				// 					$category_id = $ts_member->getId();
				// 					break;
				// 				}
				// 			}
				// 		}

				// 		//try to get the project name
				// 		$project_number='';
				// 		$project_name='';
						
				// 		$projectLine= self::getInvoiceLinesProject($p);
				// 		$project_number=$projectLine['project_number'];
				// 		$project_name=$projectLine['project_name'];

				// 		////get current date
				// 		try{
				// 			//$line_date=$p->getStartTime();//o getDate
				// 			$line_date=$p->getDate();//o getDate
				// 		}catch(Exception $e){
				// 		}

				// 		///set format date
				// 		$first_date = format_date($line_date, null, 0);
				// 		$last_date = format_date($line_date, null, 0);

				// 		//get min date
				// 		if($first_date<=$first_date_prev)
				// 		{
				// 			$first_date_prev=$first_date;
				// 		}

				// 		//get max date
				// 		if($last_date>=$last_date_prev || $last_date_prev=='')
				// 		{
				// 			$last_date_prev=$last_date;
				// 		}

				// 		// 0 date
				// 		//// 0 columna DATE
				// 		if($template_info['include_exp_type_date']=='Billing from date' )
				// 		{
				// 			$line_text_date=format_date(strtotime($billing_period_from),null,0);
				// 		}else if($template_info['include_exp_type_date']=='Billing to date' )
				// 		{
				// 			$line_text_date=format_date(strtotime($billing_period_to),null,0);
				// 		}else if($template_info['include_exp_type_date']=='Invoice date' )
				// 		{
				// 			$line_text_date=format_date(null,null,0);
				// 		}else if ($template_info['include_exp_type_date']=='Date of the earliest item')
				// 		{
				// 			$line_text_date=$first_date_prev;
				// 		}else if ($template_info['include_exp_type_date']=='Date of the most recent item')
				// 		{
				// 			$line_text_date=$last_date_prev;
				// 		}else if($template_info['include_exp_type_date']=='Item date')
				// 		{
				// 			$line_text_date=format_date($line_date,null,0);
				// 		}
				// 		///line detail for  expenses
				// 		$line_detail='';
				// 		if($template_info['include_expense_project_number'])
				// 		{
				// 			$line_detail.=$project_number;
				// 		}

				// 		if($template_info['include_expense_project_name'])
				// 		{
				// 			$line_detail.=' '.$project_name;
				// 		}

				// 		//1 paid by
						
				// 		if ($template_info['include_paid_by']) {
				// 			$contact = Contacts::instance()->findById($p->getPaidById());
				// 			$line_detail = $contact instanceof Contact ? $contact->getObjectName() : 'Without user';
				// 		}

				// 		//2 expense category
				// 		if ($template_info['include_expense_category']) {

				// 			$line_detail .= '  '.$category_name;
				// 		}
						
				// 		//3 product type
				// 		if ($template_info['include_product_type']) {
				// 			$product_type_line = ProductTypes::getById($p->getProductTypeId());
				// 			if($product_type_line instanceof ProductType)
				// 			{
				// 				$line_detail .= '  '.$product_type_line->getName()?$product_type_line->getName():'';
				// 			}
				// 		}

				// 		//4 expense name
				// 		if ($template_info['include_expense_name']) {
				// 				$line_detail .= '  '.$p->getName();
				// 		}
						
				// 		//4 expense description
				// 		if($template_info['include_expense_description'])
				// 		{
				// 				$line_detail .= '  '.$p->getDescription();
				// 		}
						
						

				// 		$group_by_cat_id = self::getExpenseGroupByCategoryId($p, $template_info);
						
				// 		$data = array(
				// 				'currency_id' => $p->getCurrencyId(),
				// 				'amount' => $p->getQuantity(),
				// 				'unit_value' => $p->getUnitPrice() > 0 ? $p->getUnitPrice() : $p->getUnitCost(),
				// 				'total' => $p->getTotalPrice() > 0 ? $p->getTotalPrice() : $p->getAmount(),
				// 				'detail' => $line_detail,
				// 				'unit_cost' => $p->getUnitCost(),
				// 				'price_multiplier' => $p->getPriceMultiplier(),
				// 				'product_type_id' => $p->getProductTypeId(),
				// 				'category_id' => $category_id,
				// 				'user_id' => $p->getPaidById(),
				// 				'date' => $line_text_date,//format_date($p->getDate(), null, 0),
				// 				'rel_object_id' => $p->getId(),
				// 				'from_object_type_id' => $p->getObjectTypeId(),
				// 				'member_id' => array_var($group, 'project_id'),
				// 				'group_by_category_id' => $group_by_cat_id,
								
				// 		);
				// 		$invoice_lines[] = $data;
				// 	}
					
				// }
	    		
	    	} else if ($template_info['exp_group_by'] == 'expense_description' || $template_info['exp_group_by'] == 'expense_name') {
				// grouped by description and unit price or name and unit price
	    		$first_date_prev= date("Y/m/d");
				$last_date='';
				$first_date='';
				$last_date_prev='';
				$line_date='';
	    		foreach ($grouped_payment_receipts as $group) {
	    			
	    			$category_id = 0;
					$category_id_old=-1;
					$product_type_id=0;
					$product_type_id_old=-1;
	    			$unit_cost = 0;
	    			$price_mult = 0;
	    			$amount = 0;
	    			$total = 0;
					$group_by_cat_id=0;
	    			
					$line_detail='';
					$category_name='';
					$line_text_date='';

	    			$payment_receipts = $group['items'];
	    			foreach ($payment_receipts as $p) {/* @var $p PaymentReceipt */
						
						//manage product type
						$product_type_id = $p->getProductTypeId();
						
						//set first product type
						if($product_type_id_old==-1)
						{
							$product_type_id_old=$product_type_id;
						}
						
						//if it is a diferent product type, set product type to 0 for non show it.
						if($product_type_id!=$product_type_id_old)
						{
							$product_type_id=0;
						}

						$currency_id = $p->getCurrencyId();
	    				$amount += $p->getQuantity();
	    				$total += ($p->getTotalPrice() > 0 ? $p->getTotalPrice() : $p->getAmount());
	    				$unit_cost = $p->getUnitCost();
						$price_mult = $p->getPriceMultiplier();
	    				
	    				if ( $exp_cat_dim instanceof Dimension) {
		    				$ts_members = $p->getMembers();
		    				foreach ($ts_members as $ts_member) {
		    					if ($ts_member->getDimensionId() == $exp_cat_dim->getId()) {
		    						$category_id = $ts_member->getId();
		    						break;
		    					}
		    				}
	    				}

						//set first category
						if($category_id_old==-1)
						{
							$category_id_old=$category_id;
						}

						//if it is a diferent category, set category to 0 for non show it.
						if($category_id!=$category_id_old)
						{
							$category_id=0;
						}
						
						//try to get the project name
						$project_number='';
						$project_name='';
						
						$projectLine= self::getInvoiceLinesProject($p);
						$project_number=$projectLine['project_number'];
						$project_name=$projectLine['project_name'];

						////get current date
						try{
							//$line_date=$p->getStartTime();//o getDate
							$line_date=$p->getDate();//o getDate
						}catch(Exception $e){
						}

						///set format date
						$first_date = format_date($line_date, null, 0);
						$last_date = format_date($line_date, null, 0);

						//get min date
						if($first_date<=$first_date_prev)
						{
							$first_date_prev=$first_date;
						}

						//get max date
						if($last_date>=$last_date_prev || $last_date_prev=='')
						{
							$last_date_prev=$last_date;
						}
						
						// get the exp category to group by when printing if breaking down by exp cat
						if (!$group_by_cat_id) {
							$group_by_cat_id = self::getExpenseGroupByCategoryId($p, $template_info);
						}
	    			}

					// 0 date
					//// 0 columna DATE
					if($template_info['include_exp_type_date']=='Billing from date' )
					{
						$line_text_date=format_date(strtotime($billing_period_from),null,0);
					}else if($template_info['include_exp_type_date']=='Billing to date' )
					{
						$line_text_date=format_date(strtotime($billing_period_to),null,0);
					}else if($template_info['include_exp_type_date']=='Invoice date' )
					{
						$line_text_date=format_date(null,null,0);
					}else if ($template_info['include_exp_type_date']=='Date of the earliest item')
					{
						$line_text_date=$first_date_prev;
					}else if ($template_info['include_exp_type_date']=='Date of the most recent item')
					{
						$line_text_date=$last_date_prev;
					}else if($template_info['include_exp_type_date']=='Item date')
					{
						$line_text_date=$last_date_prev;
					}
					///line detail for  expenses
					$line_detail='';
					if($template_info['include_expense_project_number'])
					{
						$line_detail.=$project_number;
					}

					if($template_info['include_expense_project_name'])
					{
						$line_detail.=$expense_separator.$project_name;
					}	

					//add task name
					if($template_info['include_expense_task_name'])
					{
						//find task name by task id
						$task = ProjectTasks::instance()->findById($p->getTaskId());
						if($task instanceof ProjectTask)
						{
						$line_detail .= $expense_separator.$task->getName();
						}
					}

					if ($template_info['include_paid_by']) {
						$contact = Contacts::instance()->findById($p->getPaidById());
						$line_detail = $contact instanceof Contact ? $contact->getObjectName() : 'Without user';
					}

					//2 expense category
					if($template_info['category_for_expenses']>0)
					{
						$category_id=$template_info['category_for_expenses'];
					}
					
					if($template_info['product_type_for_expenses']>0)
					{
						$product_type_id=$template_info['product_type_for_expenses'];
					}

					if ($template_info['include_expense_category']) {

						$line_detail .= $expense_separator.$category_name;
					}
					
					//3 product type
					if ($template_info['include_product_type']) {
						$product_type_line = ProductTypes::getById($product_type_id);
						if($product_type_line instanceof ProductType)
						{
							$line_detail .= $expense_separator.$product_type_line->getName()?$product_type_line->getName():'';
						}
					}

					//4 expense name
					if ($template_info['include_expense_name']) {
						
							$line_detail .= $expense_separator.$p->getName();
					}
					
					//4 expense description
					if($template_info['include_expense_description'])
					{
							$line_detail .= $expense_separator.$p->getDescription();
					}
	    			if ($amount == 0) $amount = 1;
	    			
	    			if ($amount > 0) {

						//clean first and last separator
						$line_detail=trim($line_detail, $expense_separator);

	    				$data = array(
	    						'currency_id' => $currency_id,
	    						'amount' => $amount,
	    						'unit_value' => $total / $amount,
	    						'total' => $total,
	    						'detail' => $line_detail,//$group['name'],
								'date' => $line_text_date,//format_date($p->getDate(), null, 0),
	    						'product_type_id' => $product_type_id,//$group['id']
	    						'unit_cost' => $unit_cost,
	    						'price_multiplier' => $price_mult,
	    						'category_id' => $category_id,
	    						'from_object_type_id' => PaymentReceipts::instance()->getObjectTypeId(),
								'member_id' => array_var($group, 'project_id'),
								'group_by_category_id' => $group_by_cat_id,
								
	    				);
	    				$invoice_lines[] = $data;
	    			}
	    		}
	    	} else if ($template_info['exp_group_by'] == 'product_type') {
	    		
				$first_date_prev= date("Y/m/d");
				$last_date='';
				$first_date='';
				$last_date_prev='';
				$line_date='';
	    		foreach ($grouped_payment_receipts as $group) {
	    			
	    			$product_type = ProductTypes::getById($group['id']);
	    			
	    			$category_id = 0;
	    			$sum_unit_cost = 0;
	    			$avg_price_mult = 0;
	    			$amount = 0;
	    			$total = 0;
					$group_by_cat_id = 0;
	    			
					$line_detail='';
					$category_name='';
					$line_text_date='';

	    			$payment_receipts = $group['items'];
	    			foreach ($payment_receipts as $p) {/* @var $p PaymentReceipt */
	    				
	    				$currency_id = $p->getCurrencyId();
	    				$amount += $p->getQuantity();
	    				$total += ($p->getTotalPrice() > 0 ? $p->getTotalPrice() : $p->getAmount());
	    				$sum_unit_cost += $p->getUnitCost();
	    				$avg_price_mult += $product_type && $product_type->getUsesPriceMultiplier() ? $product_type->getPriceMultiplier() : 1;
	    				
	    				if ($category_id == 0 && $exp_cat_dim instanceof Dimension) {
		    				$ts_members = $p->getMembers();
		    				foreach ($ts_members as $ts_member) {
		    					if ($ts_member->getDimensionId() == $exp_cat_dim->getId()) {
		    						$category_id = $ts_member->getId();
									$category_name=$ts_member->getName();
		    						break;
		    					}
		    				}
	    				}

						//try to get the project name
						$project_number='';
						$project_name='';
						
						$projectLine= self::getInvoiceLinesProject($p);
						$project_number=$projectLine['project_number'];
						$project_name=$projectLine['project_name'];
						

						////get current date
						try{
							//$line_date=$p->getStartTime();//o getDate
							$line_date=$p->getDate();//o getDate
						}catch(Exception $e){
						}

						///set format date
						$first_date = format_date($line_date, null, 0);
						$last_date = format_date($line_date, null, 0);

						//get min date
						if($first_date<=$first_date_prev)
						{
							$first_date_prev=$first_date;
						}

						//get max date
						if($last_date>=$last_date_prev || $last_date_prev=='')
						{
							$last_date_prev=$last_date;
						}

						// get the exp category to group by when printing if breaking down by exp cat
						if (!$group_by_cat_id) {
							$group_by_cat_id = self::getExpenseGroupByCategoryId($p, $template_info);
						}
	    			}
	    			if (count($payment_receipts) > 0) {
	    				$avg_price_mult = $avg_price_mult / count($payment_receipts);
	    			}

					//// 0 columna DATE
					if($template_info['include_exp_type_date']=='Billing from date' )
					{
						$line_text_date=format_date(strtotime($billing_period_from),null,0);
					}else if($template_info['include_exp_type_date']=='Billing to date' )
					{
						$line_text_date=format_date(strtotime($billing_period_to),null,0);
					}else if($template_info['include_exp_type_date']=='Invoice date' )
					{
						$line_text_date=format_date(null,null,0);
					}else if ($template_info['include_exp_type_date']=='Date of the earliest item')
					{
						$line_text_date=$first_date_prev;
					}else if ($template_info['include_exp_type_date']=='Date of the most recent item')
					{
						$line_text_date=$last_date_prev;
					}else if($template_info['include_exp_type_date']=='Item date')
					{
						$line_text_date=$last_date_prev;
					}
					///line detail for  expenses
					$line_detail='';
					if($template_info['include_expense_project_number'])
					{
						$line_detail.=$project_number;
					}

					if($template_info['include_expense_project_name'])
					{
						$line_detail.=$expense_separator.$project_name;
					}

					//add task name
					if($template_info['include_expense_task_name'])
					{
						//find task name by task id
						$task = ProjectTasks::instance()->findById($p->getTaskId());
						if($task instanceof ProjectTask)
						{
						$line_detail .= $expense_separator.$task->getName();
						}
					}

					if ($template_info['include_paid_by']) {
						$contact = Contacts::instance()->findById($p->getPaidById());
						$line_detail = $contact instanceof Contact ? $contact->getObjectName() : 'Without user';
					}

					
					$category_id=$template_info['category_for_expenses'];
					
					

					//2 expense category
					if ($template_info['include_expense_category']) {

						$line_detail .= $expense_separator.$category_name;
					}
					
					//3 product type
					if ($template_info['include_product_type']) {
						$product_type_line = ProductTypes::getById($p->getProductTypeId());
						if($product_type_line instanceof ProductType)
						{
							$line_detail .= $expense_separator.$product_type_line->getName()?$product_type_line->getName():'';
						}
					}

					//4 expense name
					if ($template_info['include_expense_name']) {
						
							$line_detail .= $expense_separator.$p->getName();
					}
					
					//4 expense description
					if($template_info['include_expense_description'])
					{
							$line_detail .= $expense_separator.$p->getDescription();
					}

	    			if ($amount == 0) $amount = 1;
	    			
	    			if ($amount > 0) {

						//clean first and last separator
						$line_detail=trim($line_detail, $expense_separator);

	    				$data = array(
	    						'currency_id' => $currency_id,
	    						'amount' => $amount,
	    						'unit_value' => $total / $amount,
	    						'total' => $total,
	    						'detail' => $line_detail,//$group['name'],
								'date' => $line_text_date,//format_date($p->getDate(), null, 0),
	    						'product_type_id' => $group['product_type_id'],
	    						'unit_cost' => $sum_unit_cost / count($payment_receipts),
	    						'price_multiplier' => $avg_price_mult,
	    						'category_id' => $category_id,
	    						'from_object_type_id' => PaymentReceipts::instance()->getObjectTypeId(),
								'member_id' => array_var($group, 'project_id'),
								'group_by_category_id' => $group_by_cat_id,
								
	    				);
	    				$invoice_lines[] = $data;
	    			}
	    		}
	    		
	    	} else if ($template_info['exp_group_by'] == 'paid_by') {
	    		$first_date_prev= date("Y/m/d");
				$last_date='';
				$first_date='';
				$last_date_prev='';
				$line_date='';
	    		foreach ($grouped_payment_receipts as $group) {
	    			$line_detail='';
					$category_name='';
					
	    			$sum_unit_cost = 0;
	    			$avg_price_mult = 0;
	    			$amount = 0;
	    			$total = 0;
					$group_by_cat_id = 0;
	    			
	    			$payment_receipts = $group['items'];
	    			foreach ($payment_receipts as $p) {/* @var $p PaymentReceipt */
	    				$product_type = ProductTypes::getById($p->getProductTypeId());
	    				
	    				$currency_id = $p->getCurrencyId();
	    				$amount += $p->getQuantity();
	    				$total += $p->getTotalPrice() > 0 ? $p->getTotalPrice() : $p->getAmount();
	    				$sum_unit_cost += $p->getUnitCost();
	    				$avg_price_mult += $product_type && $product_type->getUsesPriceMultiplier() ? $product_type->getPriceMultiplier() : 1;
	    				
						// get the exp category to group by when printing if breaking down by exp cat
						if (!$group_by_cat_id) {
							$group_by_cat_id = self::getExpenseGroupByCategoryId($p, $template_info);
						}

						//try to get the project name
						$project_number='';
						$project_name='';
						
						$projectLine= self::getInvoiceLinesProject($p);
						$project_number=$projectLine['project_number'];
						$project_name=$projectLine['project_name'];

						////get current date
						try{
							//$line_date=$p->getStartTime();//o getDate
							$line_date=$p->getDate();//o getDate
						}catch(Exception $e){
						}

						///set format date
						$first_date = format_date($line_date, null, 0);
						$last_date = format_date($line_date, null, 0);

						//get min date
						if($first_date<=$first_date_prev)
						{
							$first_date_prev=$first_date;
						}

						//get max date
						if($last_date>=$last_date_prev || $last_date_prev=='')
						{
							$last_date_prev=$last_date;
						}
	    			}

					// 0 date
					//// 0 columna DATE
					if($template_info['include_exp_type_date']=='Billing from date' )
					{
						$line_text_date=format_date(strtotime($billing_period_from),null,0);
					}else if($template_info['include_exp_type_date']=='Billing to date' )
					{
						$line_text_date=format_date(strtotime($billing_period_to),null,0);
					}else if($template_info['include_exp_type_date']=='Invoice date' )
					{
						$line_text_date=format_date(null,null,0);
					}else if ($template_info['include_exp_type_date']=='Date of the earliest item')
					{
						$line_text_date=$first_date_prev;
					}else if ($template_info['include_exp_type_date']=='Date of the most recent item')
					{
						$line_text_date=$last_date_prev;
					}else if($template_info['include_exp_type_date']=='Item date')
					{
						$line_text_date=$last_date_prev;
					}

					///line detail for  expenses
					
					///line detail for  expenses
					$line_detail='';
					if($template_info['include_expense_project_number'])
					{
						$line_detail.=$project_number;
					}

					if($template_info['include_expense_project_name'])
					{
						$line_detail.=$expense_separator.$project_name;
					}	

					//add task name
					if($template_info['include_expense_task_name'])
					{
						//find task name by task id
						$task = ProjectTasks::instance()->findById($p->getTaskId());
						if($task instanceof ProjectTask)
						{
						$line_detail .= $expense_separator.$task->getName();
						}
					}

					//1 paid by
					if ($template_info['include_paid_by']) {
						$contact = Contacts::instance()->findById($p->getPaidById());
						$line_detail = $contact instanceof Contact ? $contact->getObjectName() : 'Without user';
					}

					//2 expense category
					$category_id=$template_info['category_for_expenses'];
					$product_type_id=$template_info['product_type_for_expenses'];
					if ($template_info['include_expense_category']) {

						$line_detail .= $expense_separator.$category_name;
					}
					
					//3 product type
					if ($template_info['include_product_type']) {
						$product_type_line = ProductTypes::getById($product_type_id);
						if($product_type_line instanceof ProductType)
						{
							$line_detail .= $expense_separator.$product_type_line->getName()?$product_type_line->getName():'';
						}
					}

					//4 expense name
					if ($template_info['include_expense_name']) {
						
							$line_detail .= $expense_separator.$p->getName();
					}
					
					//4 expense description
					if($template_info['include_expense_description'])
					{
							$line_detail .= $expense_separator.$p->getDescription();
					}

	    			if (count($payment_receipts) > 0) {
	    				$avg_price_mult = $avg_price_mult / count($payment_receipts);
	    			}
	    			if ($amount == 0) $amount = 1;
	    			
	    			if ($amount > 0) {

						//clean first and last separator
						$line_detail=trim($line_detail, $expense_separator);

	    				$data = array(
	    						'currency_id' => $currency_id,
	    						'amount' => $amount,
	    						'unit_value' => $total / $amount,
	    						'total' => $total,
	    						'detail' => $line_detail,//$group['name'],
								'date' => $line_text_date,//format_date($p->getDate(), null, 0),
	    						'unit_cost' => $sum_unit_cost / count($payment_receipts),
	    						'price_multiplier' => $avg_price_mult,
	    						'user_id' => $group['user_id'],
								'product_type_id' => $product_type_id,
								'category_id' => $category_id,
	    						'from_object_type_id' => PaymentReceipts::instance()->getObjectTypeId(),
								'member_id' => array_var($group, 'project_id'),
								'group_by_category_id' => $category_id//$group_by_cat_id,
								
	    				);
	    				$invoice_lines[] = $data;
	    			}
	    		}
	    		
	    	} else if ($template_info['exp_group_by'] == 'expense_category') {
	    		
				$first_date_prev= date("Y/m/d");
				$last_date='';
				$first_date='';
				$last_date_prev='';
				$line_date='';
				$line_detail='';
				$category_name='';
				$line_text_date='';

	    		foreach ($grouped_payment_receipts as $group) {
					$line_detail='';

	    			$sum_unit_cost = 0;
	    			$avg_price_mult = 0;
	    			$amount = 0;
	    			$total = 0;
					$group_by_cat_id = 0;
	    			
	    			$payment_receipts = $group['items'];
					$product_type='';
	    			foreach ($payment_receipts as $p) {/* @var $p PaymentReceipt */
	    				$product_type = ProductTypes::getById($p->getProductTypeId());
						$product_type_id = $p->getProductTypeId();
	    				 
	    				$currency_id = $p->getCurrencyId();
	    				$amount += $p->getQuantity();
	    				$total += $p->getTotalPrice() > 0 ? $p->getTotalPrice() : $p->getAmount();
	    				$sum_unit_cost += $p->getUnitCost();
	    				$avg_price_mult += $product_type && $product_type->getUsesPriceMultiplier() ? $product_type->getPriceMultiplier() : 1;
	    				
						// get the exp category to group by when printing if breaking down by exp cat
						if (!$group_by_cat_id) {
							$group_by_cat_id = self::getExpenseGroupByCategoryId($p, $template_info);
						}

						//try to get the project name
						$project_number='';
						$project_name='';
						
						$projectLine= self::getInvoiceLinesProject($p);
						$project_number=$projectLine['project_number'];
						$project_name=$projectLine['project_name'];

						////get current date
						try{
							//$line_date=$p->getStartTime();//o getDate
							$line_date=$p->getDate();//o getDate
						}catch(Exception $e){
						}

						///set format date
						$first_date = format_date($line_date, null, 0);
						$last_date = format_date($line_date, null, 0);

						//get min date
						if($first_date<=$first_date_prev)
						{
							$first_date_prev=$first_date;
						}

						//get max date
						if($last_date>=$last_date_prev || $last_date_prev=='')
						{
							$last_date_prev=$last_date;
						}

	    			}

					// 0 date
						//// 0 columna DATE
						if($template_info['include_exp_type_date']=='Billing from date' )
						{
							$line_text_date=format_date(strtotime($billing_period_from),null,0);
						}else if($template_info['include_exp_type_date']=='Billing to date' )
						{
							$line_text_date=format_date(strtotime($billing_period_to),null,0);
						}else if($template_info['include_exp_type_date']=='Invoice date' )
						{
							$line_text_date=format_date(null,null,0);
						}else if ($template_info['include_exp_type_date']=='Date of the earliest item')
						{
							$line_text_date=$first_date_prev;
						}else if ($template_info['include_exp_type_date']=='Date of the most recent item')
						{
							$line_text_date=$last_date_prev;
						}else if($template_info['include_exp_type_date']=='Item date')
						{
							$line_text_date=$last_date_prev;
						}
						///line detail for  expenses
						$line_detail='';
						if($template_info['include_expense_project_number'])
						{
							$line_detail.=$project_number;
						}

						if($template_info['include_expense_project_name'])
						{
							$line_detail.=$expense_separator.$project_name;
						}

						//add task name
						if($template_info['include_expense_task_name'])
						{
							$task = $p->getRelObject();
							if ($task instanceof ProjectTask)
							{
								$line_detail .= $expense_separator.$task->getName();
							}
						}

						//1 paid by
						if ($template_info['include_paid_by']) {
							$contact = Contacts::instance()->findById($p->getPaidById());
							$line_detail = $contact instanceof Contact ? $contact->getObjectName() : 'Without user';
						}

						//2 expense category
						if ($template_info['include_expense_category']) {
							$line_detail .= $expense_separator.$category_name;
						}
						
						//2 product type selected
						if($template_info['product_type_for_expenses'])
						{
							$product_type_id=$template_info['product_type_for_expenses'];
						}
						
						if ($template_info['include_product_type']) {
							$product_type_line = ProductTypes::getById($product_type_id); //$p->getProductTypeId()
							if($product_type_line instanceof ProductType)
							{
								$line_detail .= $expense_separator.$product_type_line->getName()?$product_type_line->getName():'';
							}
						}

						//4 expense name
						if ($template_info['include_expense_name']) {
							
								$line_detail .= $expense_separator.$p->getName();
						}
						
						//4 expense description
						if($template_info['include_expense_description'])
						{
								$line_detail .= $expense_separator.$p->getDescription();
						}

	    			if (count($payment_receipts) > 0) {
	    				$avg_price_mult = $avg_price_mult / count($payment_receipts);
	    			}
	    			if ($amount == 0) $amount = 1;
	    			
	    			if ($amount > 0) {

						//clean first and last separator
						$line_detail=trim($line_detail, $expense_separator);

	    				$data = array(
	    						'currency_id' => $currency_id,
	    						'amount' => $amount,
	    						'unit_value' => $total / $amount,
	    						'total' => $total,
	    						'detail' => $line_detail,//$group['name'],
								'date' => $line_text_date,//format_date($p->getDate(), null, 0),
	    						'unit_cost' => $sum_unit_cost / count($payment_receipts),
	    						'price_multiplier' => $avg_price_mult,
								'product_type_id' => $p->getProductTypeId(),
	    						'category_id' => $group['category_id'],
	    						'from_object_type_id' => PaymentReceipts::instance()->getObjectTypeId(),
								'member_id' => array_var($group, 'project_id'),
								'group_by_category_id' => $group_by_cat_id,
								
	    				);
	    				$invoice_lines[] = $data;
	    			}
	    		}
	    		
	    	} else if ($template_info['exp_group_by'] == 'project') {
	    		
				$first_date_prev= date("Y/m/d");
				$last_date='';
				$first_date='';
				$last_date_prev='';
				$line_date='';
				$line_text_date='';
	    		foreach ($grouped_payment_receipts as $group) {
	    			
	    			$sum_unit_cost = 0;
	    			$avg_price_mult = 0;
	    			$amount = 0;
	    			$total = 0;
					$group_by_cat_id = 0;
					$line_text='';	    			
	    			$payment_receipts = $group['items'];
					$line_detail='';
					$category_name='';
	    			foreach ($payment_receipts as $p) {/* @var $p PaymentReceipt */
	    				$product_type = ProductTypes::getById($p->getProductTypeId());
	    				
	    				$currency_id = $p->getCurrencyId();
	    				$amount += $p->getQuantity();
	    				$total += $p->getTotalPrice() > 0 ? $p->getTotalPrice() : $p->getAmount();
	    				$sum_unit_cost += $p->getUnitCost();
	    				$avg_price_mult += $product_type && $product_type->getUsesPriceMultiplier() ? $product_type->getPriceMultiplier() : 1;
	    				
						// get the exp category to group by when printing if breaking down by exp cat
						if (!$group_by_cat_id) {
							$group_by_cat_id = self::getExpenseGroupByCategoryId($p, $template_info);
						}
						/*if ($template_info['group_by'] == 'timeslots' && !isset($group['category_id'])) {
							$group['category_id'] = $category_id;
						}*/
						////get current date
						try{
							//$line_date=$p->getStartTime();//o getDate
							$line_date=$p->getDate();//o getDate
						}catch(Exception $e){
						}

						///set format date
						$first_date = format_date($line_date, null, 0);
						$last_date = format_date($line_date, null, 0);

						//get min date
						if($first_date<=$first_date_prev)
						{
							$first_date_prev=$first_date;
						}

						//get max date
						if($last_date>=$last_date_prev || $last_date_prev=='')
						{
							$last_date_prev=$last_date;
						}
	    			}

					// 0 date
						//// 0 columna DATE
						if($template_info['include_exp_type_date']=='Billing from date' )
						{
							$line_text_date=format_date(strtotime($billing_period_from),null,0);
						}else if($template_info['include_exp_type_date']=='Billing to date' )
						{
							$line_text_date=format_date(strtotime($billing_period_to),null,0);
						}else if($template_info['include_exp_type_date']=='Invoice date' )
						{
							$line_text_date=format_date(null,null,0);
						}else if ($template_info['include_exp_type_date']=='Date of the earliest item')
						{
							$line_text_date=$first_date_prev;
						}else if ($template_info['include_exp_type_date']=='Date of the most recent item')
						{
							$line_text_date=$last_date_prev;
						}else if($template_info['include_exp_type_date']=='Item date')
						{
							$line_text_date=$last_date_prev;
						}
						///line detail for  expenses
						//1 paid by
						$line_text='';
						if ($template_info['include_paid_by']) {
							$contact = Contacts::instance()->findById($p->getPaidById());
							$line_text = $contact instanceof Contact ? $contact->getObjectName() : 'Without user';
						}

						//2 expense category
						$category_id=$template_info['category_for_expenses'];
						$product_type_id=$template_info['product_type_for_expenses'];

						if ($template_info['include_expense_category']) {

							$cat_m = Members::getMemberById($category_id);
							if ($cat_m instanceof Member) 
							{
								$category_name = $cat_m->getName();
							}

							$line_text .= $expense_separator.$category_name;
						}
						
						//3 product type
						if ($template_info['include_product_type']) {
							$product_type_line = ProductTypes::getById($p->getProductTypeId());
							if($product_type_line instanceof ProductType)
							{
								$line_text .= $expense_separator.$product_type_line->getName()?$product_type_line->getName():'';
							}
						}

						//4 expense name
						if ($template_info['include_expense_name']) {
							
								$line_text .= $expense_separator.$p->getName();
						}
						
						//4 expense description
						if($template_info['include_expense_description'])
						{
								$line_text .= $expense_separator.$p->getDescription();
						}

					if($template_info['include_expense_project_number'])
					{
						$line_text.=$group['project_id'];
					}

					if($template_info['include_expense_project_name'])
					{
						$line_text.=$expense_separator.$group['name'];
					}

					//add task name
					if($template_info['include_expense_task_name'])
					{
						//find task name by task id
						$task = ProjectTasks::instance()->findById($p->getTaskId());
						if($task instanceof ProjectTask)
						{
						$line_detail .= $expense_separator.$task->getName();
						}
					}
					
	    			if (count($payment_receipts) > 0) {
	    				$avg_price_mult = $avg_price_mult / count($payment_receipts);
	    			}
	    			if ($amount == 0) $amount = 1;
	    			
	    			if ($amount > 0) {

						//clean first and last separator
						$line_text=trim($line_text, $expense_separator);

	    				$data = array(
	    						'currency_id' => $currency_id,
	    						'amount' => $amount,
	    						'unit_value' => $total / $amount,
	    						'total' => $total,
	    						'detail' =>$line_text,// $group['name'],
								'date' => $line_text_date,//format_date($p->getDate(), null, 0),
	    						'unit_cost' => $sum_unit_cost / count($payment_receipts),
	    						'price_multiplier' => $avg_price_mult,
	    						'member_id' => $group['id'],
								'product_type_id' => $product_type_id,
								'category_id' => $category_id,
	    						'from_object_type_id' => PaymentReceipts::instance()->getObjectTypeId(),
								'group_by_category_id' => $group_by_cat_id,
								
	    				);
	    				$invoice_lines[] = $data;
	    			}
	    		}
	    	} else if ($template_info['exp_group_by'] == 'project_product') {
	    		
				$first_date_prev= date("Y/m/d");
				$last_date='';
				$first_date='';
				$last_date_prev='';
				$line_date='';
				$line_text_date='';
	    		foreach ($grouped_payment_receipts as $group) {
	    			
	    			$sum_unit_cost = 0;
	    			$avg_price_mult = 0;
	    			$amount = 0;
	    			$total = 0;
					$group_by_cat_id = 0;
					$line_text='';	    			
	    			$payment_receipts = $group['items'];
					$line_detail='';
					$category_name='';
	    			foreach ($payment_receipts as $p) {/* @var $p PaymentReceipt */
	    				$product_type = ProductTypes::getById($p->getProductTypeId());
	    				
	    				$currency_id = $p->getCurrencyId();
	    				$amount += $p->getQuantity();
	    				$total += $p->getTotalPrice() > 0 ? $p->getTotalPrice() : $p->getAmount();
	    				$sum_unit_cost += $p->getUnitCost();
	    				$avg_price_mult += $product_type && $product_type->getUsesPriceMultiplier() ? $product_type->getPriceMultiplier() : 1;
	    				
						// get the exp category to group by when printing if breaking down by exp cat
						if (!$group_by_cat_id) {
							$group_by_cat_id = self::getExpenseGroupByCategoryId($p, $template_info);
						}
						/*if ($template_info['group_by'] == 'timeslots' && !isset($group['category_id'])) {
							$group['category_id'] = $category_id;
						}*/
						////get current date
						try{
							//$line_date=$p->getStartTime();//o getDate
							$line_date=$p->getDate();//o getDate
						}catch(Exception $e){
						}

						///set format date
						$first_date = format_date($line_date, null, 0);
						$last_date = format_date($line_date, null, 0);

						//get min date
						if($first_date<=$first_date_prev)
						{
							$first_date_prev=$first_date;
						}

						//get max date
						if($last_date>=$last_date_prev || $last_date_prev=='')
						{
							$last_date_prev=$last_date;
						}
	    			}

					// 0 date
						//// 0 columna DATE
						if($template_info['include_exp_type_date']=='Billing from date' )
						{
							$line_text_date=format_date(strtotime($billing_period_from),null,0);
						}else if($template_info['include_exp_type_date']=='Billing to date' )
						{
							$line_text_date=format_date(strtotime($billing_period_to),null,0);
						}else if($template_info['include_exp_type_date']=='Invoice date' )
						{
							$line_text_date=format_date(null,null,0);
						}else if ($template_info['include_exp_type_date']=='Date of the earliest item')
						{
							$line_text_date=$first_date_prev;
						}else if ($template_info['include_exp_type_date']=='Date of the most recent item')
						{
							$line_text_date=$last_date_prev;
						}else if($template_info['include_exp_type_date']=='Item date')
						{
							$line_text_date=$last_date_prev;
						}
						///line detail for  expenses
						//1 paid by
						$line_text='';
						if ($template_info['include_paid_by']) {
							$contact = Contacts::instance()->findById($p->getPaidById());
							$line_text = $contact instanceof Contact ? $contact->getObjectName() : 'Without user';
						}

						//2 expense category
						$category_id=$template_info['category_for_expenses'];
						$product_type_id=$template_info['product_type_for_expenses'];

						if ($template_info['include_expense_category']) {

							$cat_m = Members::getMemberById($category_id);
							if ($cat_m instanceof Member) 
							{
								$category_name = $cat_m->getName();
							}

							$line_text .= $expense_separator.$category_name;
						}
						
						//3 product type
						if ($template_info['include_product_type']) {
							$product_type_line = ProductTypes::getById($p->getProductTypeId());
							if($product_type_line instanceof ProductType)
							{
								$line_text .= $expense_separator.$product_type_line->getName()?$product_type_line->getName():'';
							}
						}

						//4 expense name
						if ($template_info['include_expense_name']) {
							
								$line_text .= $expense_separator.$p->getName();
						}
						
						//4 expense description
						if($template_info['include_expense_description'])
						{
								$line_text .= $expense_separator.$p->getDescription();
						}

					if($template_info['include_expense_project_number'])
					{
						$line_text.=$group['project_id'];
					}

					if($template_info['include_expense_project_name'])
					{
						$line_text.=$expense_separator.$group['name'];
					}

					//add task name
					if($template_info['include_expense_task_name'])
					{
						//find task name by task id
						$task = ProjectTasks::instance()->findById($p->getTaskId());
						if($task instanceof ProjectTask)
						{
						$line_detail .= $expense_separator.$task->getName();
						}
					}
					
	    			if (count($payment_receipts) > 0) {
	    				$avg_price_mult = $avg_price_mult / count($payment_receipts);
	    			}
	    			if ($amount == 0) $amount = 1;
	    			
	    			if ($amount > 0) {
						
						//clean first and last separator
						$line_text=trim($line_text, $expense_separator);

	    				$data = array(
	    						'currency_id' => $currency_id,
	    						'amount' => $amount,
	    						'unit_value' => $total / $amount,
	    						'total' => $total,
	    						'detail' =>$line_text,// $group['name'],
								'date' => $line_text_date,//format_date($p->getDate(), null, 0),
	    						'unit_cost' => $sum_unit_cost / count($payment_receipts),
	    						'price_multiplier' => $avg_price_mult,
	    						'member_id' => $group['id'],
								'product_type_id' => $product_type_id,
								'category_id' => $category_id,
	    						'from_object_type_id' => PaymentReceipts::instance()->getObjectTypeId(),
								'group_by_category_id' => $group_by_cat_id,
								
	    				);
	    				$invoice_lines[] = $data;
	    			}
	    		}
	    	} else if ($template_info['exp_group_by'] == 'project_expense_category') {
	    		
	    		foreach ($grouped_payment_receipts as $group) {
	    			
	    			$sum_unit_cost = 0;
	    			$avg_price_mult = 0;
	    			$amount = 0;
	    			$total = 0;
					$group_by_cat_id = 0;
	    			
	    			$payment_receipts = $group['items'];
	    			foreach ($payment_receipts as $p) {/* @var $p PaymentReceipt */
	    				$product_type = ProductTypes::getById($p->getProductTypeId());
	    				
	    				$currency_id = $p->getCurrencyId();
	    				$amount += $p->getQuantity();
	    				$total += $p->getTotalPrice() > 0 ? $p->getTotalPrice() : $p->getAmount();
	    				$sum_unit_cost += $p->getUnitCost();
	    				$avg_price_mult += $product_type && $product_type->getUsesPriceMultiplier() ? $product_type->getPriceMultiplier() : 1;
	    				
						// get the exp category to group by when printing if breaking down by exp cat
						if (!$group_by_cat_id) {
							$group_by_cat_id = self::getExpenseGroupByCategoryId($p, $template_info);
						}

						////get current date
						try{
							//$line_date=$p->getStartTime();//o getDate
							$line_date=$p->getDate();//o getDate
						}catch(Exception $e){
						}

						///set format date
						$first_date = format_date($line_date, null, 0);
						$last_date = format_date($line_date, null, 0);

						//get min date
						if($first_date<=$first_date_prev)
						{
							$first_date_prev=$first_date;
						}

						//get max date
						if($last_date>=$last_date_prev || $last_date_prev=='')
						{
							$last_date_prev=$last_date;
						}
	    			}

					// 0 date
						//// 0 columna DATE
						if($template_info['include_exp_type_date']=='Billing from date' )
						{
							$line_text_date=format_date(strtotime($billing_period_from),null,0);
						}else if($template_info['include_exp_type_date']=='Billing to date' )
						{
							$line_text_date=format_date(strtotime($billing_period_to),null,0);
						}else if($template_info['include_exp_type_date']=='Invoice date' )
						{
							$line_text_date=format_date(null,null,0);
						}else if ($template_info['include_exp_type_date']=='Date of the earliest item')
						{
							$line_text_date=$first_date_prev;
						}else if ($template_info['include_exp_type_date']=='Date of the most recent item')
						{
							$line_text_date=$last_date_prev;
						}else if($template_info['include_exp_type_date']=='Item date')
						{
							$line_text_date=$last_date_prev;
						}
						///line detail for  expenses
						//1 paid by
						$line_detail='';
						if ($template_info['include_paid_by']) {
							$contact = Contacts::instance()->findById($p->getPaidById());
							$line_detail = $contact instanceof Contact ? $contact->getObjectName() : 'Without user';
						}

						//2 expense category
						if ($template_info['include_expense_category']) {

							$line_detail .= $expense_separator.$category_name;
						}
						
						//3 product type
						if ($template_info['include_product_type']) {
							$product_type_line = ProductTypes::getById($p->getProductTypeId());
							if($product_type_line instanceof ProductType)
							{
								$line_detail .= $expense_separator.$product_type_line->getName()?$product_type_line->getName():'';
							}
						}

						//4 expense name
						if ($template_info['include_expense_name']) {
							
								$line_detail .= $expense_separator.$p->getName();
						}
						
						//4 expense description
						if($template_info['include_expense_description'])
						{
								$line_detail .= $expense_separator.$p->getDescription();
						}

						
	    			if (count($payment_receipts) > 0) {
	    				$avg_price_mult = $avg_price_mult / count($payment_receipts);
	    			}
	    			if ($amount == 0) $amount = 1;
	    			
	    			if ($amount > 0) {
						$exploded_id = explode('-', $group['id']);
						$project_id = $exploded_id[0];
						$labor_cat_id = $exploded_id[1];

						//clean first and last separator
						$line_detail=trim($line_detail, $expense_separator);

	    				$data = array(
	    						'currency_id' => $currency_id,
	    						'amount' => $amount,
	    						'unit_value' => $total / $amount,
	    						'total' => $total,
	    						'detail' => $line_detail,//$group['name'],
								'date' => $line_text_date,//format_date($p->getDate(), null, 0),
	    						'unit_cost' => $sum_unit_cost / count($payment_receipts),
	    						'price_multiplier' => $avg_price_mult,
	    						'member_id' => $project_id,
	    						'category_id' => $group['category_id'],
	    						'from_object_type_id' => PaymentReceipts::instance()->getObjectTypeId(),
								'group_by_category_id' => $group_by_cat_id,
								
	    				);
	    				$invoice_lines[] = $data;
	    			}
	    		}
	    		
	    	}
    	}

		// use mysql format (yyyy-mm-dd) in dates to order
		foreach ($invoice_lines as &$invoice_line) {
			if ($invoice_line['date']) {
				$date_obj = getDateValue($invoice_line['date']);
				if ($date_obj) {
					$invoice_line['raw_date'] = $date_obj->toMySQL();
				}
			}
		}
    	
		/*Use this for order the lines by date, if you don't want to use comment this part and return invoice_lines*/
		 $arr_ordered=[];
		 $arr_ordered = IncomeInvoiceTemplates::array_msort($invoice_lines, array('raw_date'=>SORT_ASC,'detail'=>SORT_ASC));
		 $order_invoice_line_new=0;
		 $invoice_lines_ordered=[];
		 foreach($arr_ordered as $arr2_o){
			
			$data_ordered = $arr2_o;
			$data_ordered['order'] = $order_invoice_line_new;

			$order_invoice_line_new++;
			$invoice_lines_ordered[] = $data_ordered;
		 }
		/**************************** */

    	return $invoice_lines_ordered;//invoice_lines;
    }
	
    static function groupTasksUsingTemplate(IncomeInvoiceTemplate $invoice_template, 
	                                        $tasks) 
	{
		$template_info = $invoice_template->getArrayInfo();
    	$groups = array();

		$project_ot_id = ObjectTypes::findByName('project')->getId();
    	
    	if ($template_info['task_group'] == 'aggregated into a single line'){
			//$template_info['template_type'] == 'lumpsum') {
    		
    		foreach ($tasks as $t) {
				$group_id = 0;
    			$group_name = "";
    			$projects_dim = Dimensions::findByCode("customer_project");
    			$ts_members = $t->getMembers();

    			foreach ($ts_members as $ts_member) {
    				if ($ts_member->getDimensionId() == $projects_dim->getId()) {
    					$group_id = $ts_member->getId();
    					$group_name = $ts_member->getName();
    					break;
    				}
    			}

    			if (!isset($groups[$group_id])) {
    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'items' => array());
    			}
    			$groups[$group_id]['items'][] = $t->getArrayInfo();
    		}
    		return $groups;
    	}

		if ($template_info['task_group'] == 'separate by task') {
			if ($template_info['break_down_by_subprojects']=='SINGLE') {
				foreach ($tasks as $t) {
					
					$project_member = $t->getMemberOfType($project_ot_id);
					$project_member_id = $project_member ? $project_member->getId() : 0;

					$group_id = $project_member_id;

					if (!isset($groups[$group_id])) {
						$group_name = $project_member ? $project_member->getName() : '';
						$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $project_member_id, 'items' => array());
					}
					$groups[$group_id]['items'][] = $t->getArrayInfo();
				}
			} else {
				foreach ($tasks as $t) {
					$group_name='';
					$project_member = $t->getMemberOfType($project_ot_id);
					$project_member_id = $project_member ? $project_member->getId() : 0;

					$group_id = $project_member_id;

					if (!isset($groups[$group_id])) {
						$group_name = $project_member ? $project_member->getName() : '';
					}
					$groups[] = array( 'name' => $group_name, 'items' => array($t->getArrayInfo()));
				}
			}

			return $groups;
		}
	}
	
    static function groupTimeslotsUsingTemplate(IncomeInvoiceTemplate $invoice_template, $timeslots) {
    	$template_info = $invoice_template->getArrayInfo();
    	$groups = array();

		$project_ot_id = ObjectTypes::findByName('project')->getId();
    	
    	if ($template_info['template_type'] == 'lumpsum') {
    		
    		foreach ($timeslots as $t) {
	    		$group_id = 0;
    			$group_name = "";
    			$projects_dim = Dimensions::findByCode("customer_project");
    			$ts_members = $t->getMembers();
    			foreach ($ts_members as $ts_member) {
    				if ($ts_member->getDimensionId() == $projects_dim->getId()) {
    					$group_id = $ts_member->getId();
    					$group_name = $ts_member->getName();
    					break;
    				}
    			}
    			if (!isset($groups[$group_id])) {
    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'items' => array());
    			}
    			$groups[$group_id]['items'][] = $t;
    		}
    		return $groups;
    	}
    	
    	if ($template_info['group_by'] == 'timeslots') {

			if ($template_info['break_down_by_subprojects']=='SINGLE') {
				foreach ($timeslots as $t) {
					
					$project_member = $t->getMemberOfType($project_ot_id);
					$project_member_id = $project_member ? $project_member->getId() : 0;

					$group_id = $project_member_id;

					if (!isset($groups[$group_id])) {
						$group_name = $project_member ? $project_member->getName() : '';
						$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $project_member_id, 'items' => array());
					}
					$groups[$group_id]['items'][] = $t;
				}
				
			} else {
				foreach ($timeslots as $t) {
					$groups[] = array('id' => $t->getId(), 'name' => '', 'items' => array($t), 'user_id' => $t->getContactId());
				}
				//$groups[0] = array('id' => 0, 'name' => '', 'items' => $timeslots);
			}
    		
    	} else if ($template_info['group_by'] == 'tasks') {
    		
    		foreach ($timeslots as $t) {
    			$group_id = $t->getRelObjectId();
    			if (!isset($groups[$group_id])) {
    				$task = ProjectTasks::instance()->findById($group_id);
					
					$project_member = $t->getMemberOfType($project_ot_id);
					$project_member_id = $project_member ? $project_member->getId() : 0;

    				$group_name = $task instanceof ProjectTask ? $task->getObjectName() : 'Without task';
    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $project_member_id, 'items' => array());
    			}
    			$groups[$group_id]['items'][] = $t;
    		}
    		
    	} else if ($template_info['group_by'] == 'users') {
    		
    		foreach ($timeslots as $t) {
				$hourly_billing = $t->getColumnValue('hourly_billing');
				
    			$group_id = $t->getContactId() . "-" . $hourly_billing;

				if ($template_info['break_down_by_subprojects']=='SINGLE') {
					$project_member = $t->getMemberOfType($project_ot_id);
					$project_member_id = $project_member ? $project_member->getId() : 0;
					
					$group_id = $project_member_id . "-" . $group_id;
				} else {
					$project_member_id = 0;
				}

    			if (!isset($groups[$group_id])) {
					$contact = Contacts::instance()->findById($t->getContactId());
    				$group_name = $contact instanceof Contact ? $contact->getObjectName() : 'Without user';
    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $project_member_id, 'items' => array(), 'user_id' => $t->getContactId());
    			}
    			$groups[$group_id]['items'][] = $t;
    		}
    		
    	} else if ($template_info['group_by'] == 'date-users') {
    		
    		foreach ($timeslots as $t) {
				$hourly_billing = $t->getColumnValue('hourly_billing');

    			$group_id = format_date($t->getStartTime(),'Y-m-d') .'|'. $t->getContactId() . "-" . $hourly_billing;
					
				if ($template_info['break_down_by_subprojects']=='SINGLE') {
					$project_member = $t->getMemberOfType($project_ot_id);
					$project_member_id = $project_member ? $project_member->getId() : 0;
					
					$group_id = $project_member_id . "-" . $group_id;
				} else {
					$project_member_id = 0;
				}
				
    			if (!isset($groups[$group_id])) {
    				$contact = Contacts::instance()->findById($t->getContactId());
    				$group_name = format_date($t->getStartTime()) ." - ". ($contact instanceof Contact ? $contact->getObjectName() : 'Without user');
    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $project_member_id, 'items' => array(), 'user_id' => $t->getContactId());
    			}
    			$groups[$group_id]['items'][] = $t;
    		}

		} else if ($template_info['group_by'] == 'date_users_labor_category') {

			$labor_cat_dim = Dimensions::findByCode('hour_types');

			foreach ($timeslots as $t) {
				$cat_id = 0;
				$hourly_billing = $t->getColumnValue('hourly_billing');
    			$cat_member = null;
    			$ts_members = $t->getMembers();
    			foreach ($ts_members as $ts_member) {
    				if ($ts_member->getDimensionId() == $labor_cat_dim->getId()) {
    					$cat_id = $ts_member->getId();
    					$cat_member = $ts_member;
    					break;
    				}
    			}

    			$group_id = format_date($t->getStartTime(),'Y-m-d') .'|'. $t->getContactId() .'|'. $cat_id . "-" . $hourly_billing;
					
				if ($template_info['break_down_by_subprojects']=='SINGLE') {
					$project_member = $t->getMemberOfType($project_ot_id);
					$project_member_id = $project_member ? $project_member->getId() : 0;
					
					$group_id = $project_member_id . "-" . $group_id;
				} else {
					$project_member_id = 0;
				}
				
    			if (!isset($groups[$group_id])) {
    				$contact = Contacts::instance()->findById($t->getContactId());
    				$group_name = format_date($t->getStartTime()) ." - ". ($contact instanceof Contact ? $contact->getObjectName() : 'Without user');
    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $project_member_id, 'items' => array(), 'user_id' => $t->getContactId(), 'category_id' => $cat_id, 'date' => format_date($t->getStartTime()));
    			}
    			$groups[$group_id]['items'][] = $t;
    		}

		} else if ($template_info['group_by'] == 'labor_category_tasks') {

			$labor_cat_dim = Dimensions::findByCode('hour_types');

			foreach ($timeslots as $t) {
				$cat_id = 0;
				$hourly_billing = $t->getColumnValue('hourly_billing');
    			$cat_member = null;
    			$ts_members = $t->getMembers();
    			foreach ($ts_members as $ts_member) {
    				if ($ts_member->getDimensionId() == $labor_cat_dim->getId()) {
    					$cat_id = $ts_member->getId();
    					$cat_member = $ts_member;
    					break;
    				}
    			}

				$task_id = $t->getRelObjectId();

    			$group_id = $task_id .'|'. $cat_id . "-" . $hourly_billing;

				if ($template_info['break_down_by_subprojects']=='SINGLE') {
					$project_member = $t->getMemberOfType($project_ot_id);
					$project_member_id = $project_member ? $project_member->getId() : 0;

					$group_id = $project_member_id . "-" . $group_id;
				} else {
					$project_member_id = 0;
				}

    			if (!isset($groups[$group_id])) {
    				$task = $t->getRelObject();
					$group_name = $cat_member instanceof Member ? $cat_member->getDisplayName() : "Without category";
					$group_name .= " - " . ($task instanceof ProjectTask ? $task->getObjectName() : "Without task");

    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $project_member_id, 'items' => array(), 'task_id' => $task_id, 'category_id' => $cat_id);
    			}
    			$groups[$group_id]['items'][] = $t;
    		}
			
    	} else if ($template_info['group_by'] == 'labor_category') {
    		
    		$labor_cat_dim = Dimensions::findByCode('hour_types');
    		
    		foreach ($timeslots as $t) {
    			$cat_id = 0;
				$hourly_billing = $t->getColumnValue('hourly_billing');
    			$cat_member = null;
    			$ts_members = $t->getMembers();
    			foreach ($ts_members as $ts_member) {
    				if ($ts_member->getDimensionId() == $labor_cat_dim->getId()) {
    					$cat_id = $ts_member->getId();
    					$cat_member = $ts_member;
    					break;
    				}
    			}
				$group_id = $cat_id . '-' . $hourly_billing;
					
				if ($template_info['break_down_by_subprojects']=='SINGLE') {
					$project_member = $t->getMemberOfType($project_ot_id);
					$project_member_id = $project_member ? $project_member->getId() : 0;
					
					$group_id = $project_member_id . "-" . $group_id;
				} else {
					$project_member_id = 0;
				}
    			
    			if (!isset($groups[$group_id])) {
    				$group_name = $cat_member instanceof Member ? $cat_member->getName() : 'Without category';
    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $project_member_id, 'items' => array(), 'category_id' => $cat_id);
    			}
    			$groups[$group_id]['items'][] = $t;
    		}
    		
    	} else if ($template_info['group_by'] == 'project') {
    		
    		$projects_dim = Dimensions::findByCode('customer_project');
    		
    		foreach ($timeslots as $t) {
    			$group_id = 0;
    			$p_member = null;
    			$ts_members = $t->getMembers();
    			foreach ($ts_members as $ts_member) {
    				if ($ts_member->getDimensionId() == $projects_dim->getId()) {
    					$group_id = $ts_member->getId();
    					$p_member = $ts_member;
    					break;
    				}
    			}
    			
    			if (!isset($groups[$group_id])) {
    				$group_name = $p_member instanceof Member ? $p_member->getName() : 'Without project';
    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $group_id, 'items' => array());
    			}
    			$groups[$group_id]['items'][] = $t;
    		}
    		
    	} else if ($template_info['group_by'] == 'project_labor_category') {
    		
    		$projects_dim = Dimensions::findByCode('customer_project');
			$labor_cat_dim = Dimensions::findByCode('hour_types');
    		
    		foreach ($timeslots as $t) {
    			$group_id = 0;
				$p_group_id = 0;
				$cat_group_id = 0;
				$hourly_billing = $t->getColumnValue('hourly_billing');
    			$p_member = null;
				$cat_member = null;
    			$ts_members = $t->getMembers();
    			foreach ($ts_members as $ts_member) {
    				if ($ts_member->getDimensionId() == $projects_dim->getId()) {
    					$p_group_id = $ts_member->getId();
    					$p_member = $ts_member;
    				}
					if ($ts_member->getDimensionId() == $labor_cat_dim->getId()) {
						$cat_group_id = $ts_member->getId();
    					$cat_member = $ts_member;
					}
    			}
				$group_id = $p_group_id . "-" . $cat_group_id . '-' . $hourly_billing;
    			
    			if (!isset($groups[$group_id])) {
    				$group_name = $p_member instanceof Member ? $cat_member->getName() : 'Without category';
    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $p_group_id, 'items' => array(), 'category_id' => $cat_group_id);
    			}
    			$groups[$group_id]['items'][] = $t;
    		}
    		
    	}
    	
    	
    	return $groups;
    	
    }
    
    static function groupActualExpensesUsingTemplate($invoice_template, $payment_receipts) {
    	
    	$template_info = $invoice_template->getArrayInfo();
    	$groups = array();

		$project_ot_id = ObjectTypes::findByName('project')->getId();
    	
    	// special code when making sigle giant sum lime
    	if ($template_info['template_type'] == 'lumpsum') {
    		
    		foreach ($payment_receipts as $t) {
    			$group_id = 0;
    			if (!isset($groups[$group_id])) {
    				$group_name = "";
    				$projects_dim = Dimensions::findByCode("customer_project");
    				$ts_members = $t->getMembers();
    				foreach ($ts_members as $ts_member) {
    					if ($ts_member->getDimensionId() == $projects_dim->getId()) {
    						$group_id = $ts_member->getId();
    						$group_name = $ts_member->getName();
    						break;
    					}
    				}
					if(!isset($groups[$group_id]))
					{
    					$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'items' => array());
					}
					
    			}
    			$groups[$group_id]['items'][] = $t;
    		}
    		return $groups;
    	}
    	// --
    	
    	
    	if ($template_info['exp_group_by'] == 'payment_receipts') {
    		
			if ($template_info['break_down_by_subprojects']=='SINGLE') {
				foreach ($payment_receipts as $p) {
					
					$project_member = $p->getMemberOfType($project_ot_id);
					$project_member_id = $project_member ? $project_member->getId() : 0;

					$group_id = $project_member_id;

					if (!isset($groups[$group_id])) {
						$group_name = $project_member ? $project_member->getName() : '';
						$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $project_member_id, 'items' => array());
					}
					$groups[$group_id]['items'][] = $p;
				}

			} else {
    			$groups[0] = array('id' => 0, 'name' => '', 'items' => $payment_receipts);
			}
    		
    	} else if ($template_info['exp_group_by'] == 'expense_name') {

			// group by name and unit price (same name and different rate must appear in different lines)

			foreach ($payment_receipts as $p) { /* @var $p PaymentReceipt */
				// use unit price/cost for the group key because we can't group expenses with different rate
				$group_rate = $p->getUnitPrice() > 0 ? $p->getUnitPrice() : $p->getUnitCost();
				
				// group by name - product - rate
    			$group_id = $p->getObjectName();// . "-" . $p->getProductTypeId() . "-" . $group_rate;

				if ($template_info['break_down_by_subprojects']=='SINGLE') {
					$project_member = $p->getMemberOfType($project_ot_id);
					$project_member_id = $project_member ? $project_member->getId() : 0;
					
					$group_id = $project_member_id . "-" . $group_id;
				} else {
					$project_member_id = 0;
				}

    			if (!isset($groups[$group_id])) {
    				$group_name = $p->getObjectName();
    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $project_member_id, 'items' => array());
    			}
    			$groups[$group_id]['items'][] = $p;
    		}
    		
    	} else if ($template_info['exp_group_by'] == 'expense_description') {

			// group by description and unit price (same description and different rate must appear in different lines)

			foreach ($payment_receipts as $p) { /* @var $p PaymentReceipt */
				
				// use unit price/cost for the group key because we can't group expenses with different rate
				$group_rate = $p->getUnitPrice() > 0 ? $p->getUnitPrice() : $p->getUnitCost();

				// group by description - product - rate
    			$group_id = $p->getDescription();// . "-" . $p->getProductTypeId() . "-" . $group_rate;

				if ($template_info['break_down_by_subprojects']=='SINGLE') {
					$project_member = $p->getMemberOfType($project_ot_id);
					$project_member_id = $project_member ? $project_member->getId() : 0;
					
					$group_id = $project_member_id . "-" . $group_id;
				} else {
					$project_member_id = 0;
				}

    			if (!isset($groups[$group_id])) {
    				$group_name = $p->getDescription();
    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $project_member_id, 'items' => array());
    			}
    			$groups[$group_id]['items'][] = $p;
    		}
    		
    	} else if ($template_info['exp_group_by'] == 'product_type') {
    		
    		foreach ($payment_receipts as $p) { /* @var $p PaymentReceipt */

				// use unit price/cost for the group key because we can't group expenses with different rate
				$group_rate = $p->getUnitPrice() > 0 ? $p->getUnitPrice() : $p->getUnitCost();
    			$group_id = $p->getProductTypeId() . "-" . $group_rate;

				if ($template_info['break_down_by_subprojects']=='SINGLE') {
					$project_member = $p->getMemberOfType($project_ot_id);
					$project_member_id = $project_member ? $project_member->getId() : 0;
					
					$group_id = $project_member_id . "-" . $group_id;
				} else {
					$project_member_id = 0;
				}

    			if (!isset($groups[$group_id])) {
    				$pt = ProductTypes::instance()->getById($p->getProductTypeId());
    				$group_name = $pt instanceof ProductType ? $pt->getName() : 'Without product';
    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $project_member_id, 'items' => array(), 'product_type_id' => $p->getProductTypeId());
    			}
    			$groups[$group_id]['items'][] = $p;
    		}
    		
    	} else if ($template_info['exp_group_by'] == 'paid_by') {
    		
    		foreach ($payment_receipts as $p) { /* @var $p PaymentReceipt */

				// use unit price/cost for the group key because we can't group expenses with different rate
				$group_rate = $p->getUnitPrice() > 0 ? $p->getUnitPrice() : $p->getUnitCost();

				// group by user - product - rate
    			$group_id = $p->getPaidById() . "-" . $p->getProductTypeId() . "-" . $group_rate;

				if ($template_info['break_down_by_subprojects']=='SINGLE') {
					$project_member = $p->getMemberOfType($project_ot_id);
					$project_member_id = $project_member ? $project_member->getId() : 0;
					
					$group_id = $project_member_id . "-" . $group_id;
				} else {
					$project_member_id = 0;
				}

    			if (!isset($groups[$group_id])) {
    				$contact = Contacts::instance()->findById($p->getPaidById());
    				$group_name = $contact instanceof Contact ? $contact->getObjectName() : 'Without user';
    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $project_member_id, 'items' => array(), 'user_id' => $p->getPaidById());
    			}
    			$groups[$group_id]['items'][] = $p;
    		}
    		
    	} else if ($template_info['exp_group_by'] == 'expense_category') {
    		
    		$exp_cat_dim = Dimensions::findByCode('expense_categories');
    		
    		foreach ($payment_receipts as $p) { /* @var $p PaymentReceipt */
    			$cat_id = 0;
    			$cat_member = null;
    			$exp_members = $p->getMembers();
    			foreach ($exp_members as $exp_member) {
    				if ($exp_member->getDimensionId() == $exp_cat_dim->getId()) {
    					$cat_id = $exp_member->getId();
    					$cat_member = $exp_member;
    					break;
    				}
    			}
				$group_id = $cat_id;

				// use unit price/cost for the group key because we can't group expenses with different rate
				$group_rate = $p->getUnitPrice() > 0 ? $p->getUnitPrice() : $p->getUnitCost();

				// group by category - product - rate
				$group_id = $group_id . "-" . $p->getProductTypeId() . "-" . $group_rate;

				if ($template_info['break_down_by_subprojects']=='SINGLE') {
					$project_member = $p->getMemberOfType($project_ot_id);
					$project_member_id = $project_member ? $project_member->getId() : 0;
					
					$group_id = $project_member_id . "-" . $group_id;
				} else {
					$project_member_id = 0;
				}
    			
    			if (!isset($groups[$group_id])) {
    				$group_name = $cat_member instanceof Member ? $cat_member->getName() : 'Without category';
    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $project_member_id, 'items' => array(), 'category_id' => $cat_id);
    			}
    			$groups[$group_id]['items'][] = $p;
    		}
    		
    	} else if ($template_info['exp_group_by'] == 'project') {
    		
    		$projects_dim = Dimensions::findByCode('customer_project');
    		
    		foreach ($payment_receipts as $p) { /* @var $p PaymentReceipt */
    			$group_id = 0;
    			$p_member = null;
    			$exp_members = $p->getMembers();
    			foreach ($exp_members as $exp_member) {
    				if ($exp_member->getDimensionId() == $projects_dim->getId()) {
    					$group_id = $exp_member->getId();
    					$p_member = $exp_member;
    					break;
    				}
    			}

				// use unit price/cost for the group key because we can't group expenses with different rate
				//$group_rate = $p->getUnitPrice() > 0 ? $p->getUnitPrice() : $p->getUnitCost();
				
				// group by project - product - rate
				//$group_id = $group_id . "-" . $p->getProductTypeId() . "-" . $group_rate;
    			
    			if (!isset($groups[$group_id])) {
    				$group_name = $p_member instanceof Member ? $p_member->getName() : 'Without project';
    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $group_id, 'items' => array());
    			}

    			$groups[$group_id]['items'][] = $p;
    		}
		} else if ($template_info['exp_group_by'] == 'project_product') {
    		
    		$projects_dim = Dimensions::findByCode('customer_project');
    		
    		foreach ($payment_receipts as $p) { /* @var $p PaymentReceipt */
    			$group_id = 0;
    			$p_member = null;
    			$exp_members = $p->getMembers();
    			foreach ($exp_members as $exp_member) {
    				if ($exp_member->getDimensionId() == $projects_dim->getId()) {
    					$group_id = $exp_member->getId();
    					$p_member = $exp_member;
    					break;
    				}
    			}

				// use unit price/cost for the group key because we can't group expenses with different rate
				$group_rate = $p->getUnitPrice() > 0 ? $p->getUnitPrice() : $p->getUnitCost();
				
				// group by project - product - rate
				$group_id = $group_id . "-" . $p->getProductTypeId() . "-" . $group_rate;
    			
    			if (!isset($groups[$group_id])) {
    				$group_name = $p_member instanceof Member ? $p_member->getName() : 'Without project';
    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $group_id, 'items' => array());
    			}

    			$groups[$group_id]['items'][] = $p;
    		}
    		
    	} else if ($template_info['exp_group_by'] == 'project_expense_category') {
    		
    		$projects_dim = Dimensions::findByCode('customer_project');
			$exp_cat_dim = Dimensions::findByCode('expense_categories');
    		
    		foreach ($payment_receipts as $p) { /* @var $p PaymentReceipt */
    			$group_id = 0;
				$p_group_id = 0;
				$cat_group_id = 0;
    			$p_member = null;
    			$cat_member = null;
    			$exp_members = $p->getMembers();
    			foreach ($exp_members as $exp_member) {
    				if ($exp_member->getDimensionId() == $projects_dim->getId()) {
    					$p_group_id = $exp_member->getId();
    					$p_member = $exp_member;
    				}
    				if ($exp_member->getDimensionId() == $exp_cat_dim->getId()) {
    					$cat_group_id = $exp_member->getId();
    					$cat_member = $exp_member;
    				}
    			}
				$group_id = $p_group_id . "-" . $cat_group_id;

				// use unit price/cost for the group key because we can't group expenses with different rate
				$group_rate = $p->getUnitPrice() > 0 ? $p->getUnitPrice() : $p->getUnitCost();

				// group by project - category - product - rate
				$group_id = $group_id . "-" . $p->getProductTypeId() . "-" . $group_rate;
    			
    			if (!isset($groups[$group_id])) {
    				$group_name = $cat_member instanceof Member ? $cat_member->getName() : 'Without category';
    				$groups[$group_id] = array('id' => $group_id, 'name' => $group_name, 'project_id' => $p_group_id, 'items' => array(), 'category_id' => $cat_group_id);
    			}
    			$groups[$group_id]['items'][] = $p;
    		}
    		
    	}

		return $groups;
    }
    
	static function getExpenseCategoryAtDepth(ContentDataObject $object, $depth) {
		$exp_cate_ot = ObjectTypes::findByName('expense_category');
		if (!$exp_cate_ot) return null;

		$real_category = $object->getMemberOfType($exp_cate_ot->getId());
		if ($real_category instanceof Member) {

			if ($real_category->getDepth() <= $depth) {
				return $real_category;
			} else {
				$parent_cat = $real_category->getParentMember();
				while ($parent_cat instanceof Member && $parent_cat->getDepth() > $depth) {
					$parent_cat = $parent_cat->getParentMember();
				}
				return $parent_cat;
			}
		}

		return null;
	}

	static function getExpenseGroupByCategoryId(ContentDataObject $object, $template_info) {
		$group_cat = null;

		if ($template_info['break_down_expenses_by_category'] == "only top level") {
			
			$group_cat = self::getExpenseCategoryAtDepth($object, 1);

		} else if ($template_info['break_down_expenses_by_category'] == "each category") {
			
			$exp_cate_ot = ObjectTypes::findByName('expense_category');
			$group_cat = $object->getMemberOfType($exp_cate_ot->getId());
			
		}

		return $group_cat instanceof Member ? $group_cat->getId() : 0;
	}
    
}
