<?php

/**
 * IncomeInvoice
 * Generado el 10/5/2012
 *
 */
class IncomeInvoice extends BaseIncomeInvoice {

    protected $searchable_columns = array('name','invoice_number','series','additional_information','customer_name','customer_rut','customer_number');
    private $member_invoice=null;

    /**
     * Cached array of related forms
     *
     * @var array
     */
    private $related_forms;

    private $previous_invoice = null;
    private $next_invoice = null;

    /**
     * flag to disable the mandatory fields validation, e.g: when generating next repetition
     * @var boolean
     */
    protected $disable_mandatory_fields_validation = false;

    function disableMandatoryFieldsValidation($disable) {
    	$this->disable_mandatory_fields_validation = $disable;
    }

	// flag used to avoid triggering unnecesary calculations
	public $dont_update_totals = false;

    // ---------------------------------------------------
    //  Comments
    // ---------------------------------------------------

    /**
     * Create new comment. This function is used by ProjectForms to post comments
     * to the messages
     *
     * @param string $content
     * @param boolean $is_private
     * @return Comment or NULL if we fail to save comment
     * @throws DAOValidationError
     */
    function addComment($content, $is_private = false) {
        $comment = new Comment();
        $comment->setText($content);
        return $this->attachComment($comment);
    }

    // ---------------------------------------------------
    //  Related forms
    // ---------------------------------------------------

    /**
     * Get project forms that are in relation with this message
     *
     * @param void
     * @return array
     */
    function getRelatedForms() {
        if (is_null($this->related_forms)) {
            $this->related_forms = ProjectForms::instance()->findAll(array(
                        'conditions' => '`action` = ' . DB::escape(ProjectForm::ADD_COMMENT_ACTION) . ' AND `in_object_id` = ' . DB::escape($this->getId()),
                        'order' => '`order`'
                    )); // findAll
        } // if
        return $this->related_forms;
    }

    // ---------------------------------------------------
    //  Permissions
    // ---------------------------------------------------

    static function canAdd(Contact $user, $context, &$notAllowedMember = '') {
        return can_add($user, $context, IncomeInvoices::instance()->getObjectTypeId(), $notAllowedMember);
    }

    /**
     * Returns true if $user can access this message
     *
     * @param Contact $user
     * @return boolean
     */
    function canView(Contact $user) {
        return can_read($user, $this->getMembers(), $this->getObjectTypeId());
    }

    /**
     * Check if specific user can edit this messages
     *
     * @access public
     * @param Contact $user
     * @return boolean
     */
    function canEdit(Contact $user) {
    	Env::useHelper('functions', 'income');
    	if (income_config_option("allow_invoice_flexible_status_change") || $this->getStatus() == 'pending') {
        	return can_write($user, $this->getMembers(), $this->getObjectTypeId());
    	} else if ($this->getStatus() == 'confirmed' || $this->getStatus() == 'part_confirmed' || $this->getStatus() == 'canceled' || $this->getStatus() == 'printed') {
    		return can_edit_confirmed_invoices(logged_user()) && can_write($user, $this->getMembers(), $this->getObjectTypeId());
    	}
    }

    function canGenerateNextInvoice(Contact $user) {
    	return can_write($user, $this->getMembers(), $this->getObjectTypeId());
    }

    /**
     * Check if specific user can delete this messages
     *
     * @access public
     * @param Contact $user
     * @return boolean
     */
    function canDelete(Contact $user) {
        return $this->getStatus() == 'pending' && can_delete($user, $this->getMembers(), $this->getObjectTypeId());
    }

    /**
     * Check if specific user can comment this message
     *
     * @access public
     * @param void
     * @return boolean
     */
    function canAddComment(Contact $user) {
        return can_write($user, $this->getMembers(), $this->getObjectTypeId());
    }


	/**
	 * Returns true if user can link an object to this object
	 *
	 * @param Contact $user
	 * @return boolean
	 */
	function canLinkObject(Contact $user) {
		if(can_link_objects($user)){
			return $this->canView($user);
		}else{
			return false;
		}
	} // canLinkObject

    // ---------------------------------------------------
    //  URLS
    // ---------------------------------------------------

    /**
     * Return view message URL
     *
     * @access public
     * @param void
     * @return string
     */
    function getViewUrl() {
        return get_url('invoice', 'view', array('id' => $this->getId()));
    }

    /**
     * Return edit message URL
     *
     * @access public
     * @param void
     * @return string
     */
    function getEditUrl() {
        return get_url('invoice', 'edit', array('id' => $this->getId()));
    }

    /**
     * Return delete message URL
     *
     * @access public
     * @param void
     * @return string
     */
    function getDeleteUrl() {
        return get_url('invoice', 'delete', array('id' => $this->getId()));
    }

    /**
     * Return print view URL
     *
     * @return string
     */
    function getPrintViewUrl() {
        return get_url('invoice', 'print_view', array('id' => $this->getId()));
    }

    function getGeneratePreviewUrl() {
        return get_url('invoice', 'print_view', array('id' => $this->getId(), 'generate_preview' => 1));
    }

    function getSendInvoiceUrl() {
        return get_url('invoice', 'send_invoice', array('id' => $this->getId()));
    }


    function getReopenUrl($req_channel = '') {
    	return get_url('invoice', 'reopen', array('id' => $this->getId(), 'req_channel' => $req_channel));
    }
    function getConfirmUrl($req_channel = '') {
    	return get_url('invoice', 'confirm', array('id' => $this->getId(), 'req_channel' => $req_channel));
    }
    function getCancelUrl($req_channel = '') {
    	return get_url('invoice', 'cancel', array('id' => $this->getId(), 'req_channel' => $req_channel));
    }

    // ---------------------------------------------------
    //  System
    // ---------------------------------------------------

    /**
     * Delete this object
     *
     * @access public
     * @param void
     * @return boolean
     */
    function delete() {
        //delete invoice line
        DB::execute("DELETE FROM `" . TABLE_PREFIX . "income_invoice_lines` WHERE `object_id` = " . DB::escape($this->getObjectId()));
        return parent::delete();
    }
    
    /**
     * Sends the invoice to trash and sets the related timeslots invoicing status to pending
     * {@inheritDoc}
     * @see ContentDataObject::trash()
     */
    function trash($trashDate = null, $fire_hook = true) {
    	if ($this->canDelete(logged_user())) {

			$this->changeAssociatedTimeslotsStatus('pending');
			$this->changeAssociatedTasksStatus('pending');

			// for credit notes recalculate affected invoices paid and due amounts
			$this->recalculateReferencedInvoicesAmounts();

    		parent::trash($trashDate, $fire_hook);
    	}
    }

	/**
     * Recovers the invoice from trash and sets the related timeslots invoicing status to invoiced
     * {@inheritDoc}
     * @see ContentDataObject::untrash()
     */
    function untrash($trashDate = null, $fire_hook = true) {
    	if ($this->canDelete(logged_user())) {

			$this->changeAssociatedTimeslotsStatus('invoiced');
			$this->changeAssociatedTasksStatus('invoiced');

			// for credit notes recalculate affected invoices paid and due amounts
			$this->recalculateReferencedInvoicesAmounts();

    		parent::untrash($fire_hook);
    	}
    }
    
    function reopen() {
    	
		// Always set to "pending", because the user can put a number to pending invoices and currently we are not tracking old status changes
		$new_status = 'pending';
    	
    	$this->setStatus($new_status);
    	
    	$this->disableMandatoryFieldsValidation(true);
    	$this->save();
    	
		// change the associated timeslots status to Invoiced (invoiced)
		$this->changeAssociatedTimeslotsStatus('invoiced');
		$this->changeAssociatedTasksStatus('invoiced');
		
    	
    	$ignored = null;
    	Hook::fire('after_invoice_reopen', array('invoice' => $this), $ignored);
    }
    
    function void() {
					
    	$this->setStatus('canceled');
    	
    	$this->disableMandatoryFieldsValidation(true);
    	$this->save();
    	
		// change the associated timeslots status to Unbilled (pending)
		$this->changeAssociatedTimeslotsStatus('pending');
    	$this->changeAssociatedTasksStatus('pending');
    	ApplicationLogs::createLog($this, 'edit', false, null, true, 'status:canceled');
    	
    	$ignored = null;
    	Hook::fire('after_invoice_cancel', array('invoice' => $this), $ignored);
    	
    }
	function changeAssociatedTasksStatus($status, $tasks = null) {

		// if advanced_billing plugin is not active then the column invoice_id does not exists in tasks and an error will be get if the following code is executed
		if (Plugins::instance()->isActivePlugin('advanced_billing')) {

			if (!in_array($status, array('invoiced','pending')) || !$this instanceof IncomeInvoice) return;
		
			if (is_array($tasks) && count($tasks) > 0) {
				$invoiced_tasks = $tasks;
			} else {
				$invoiced_tasks = ProjectTasks::instance()->findAll(array("conditions" => array("invoice_id=?", $this->getId())));
			}
			
			foreach ($invoiced_tasks as $inv_task) {
				// to use when saving the application log
				$old_content_object = $inv_task->generateOldContentObjectData();
	
				$old_status = $inv_task->getColumnValue('invoicing_status');
				$invoice_id = $status == 'pending' ? 0 : $this->getId();
				// update timeslot status
				$inv_task->setColumnValue('invoice_id', $invoice_id);
				$inv_task->setColumnValue('invoicing_status', $status);
				//$inv_task->setForceRecalculateBilling(true);
				$inv_task->save();
				
				$ret = null;
				Hook::fire("after_change_object_inv_status", array('object' => $inv_task, 'old_status' => $old_status), $ret);
	
				ApplicationLogs::createLog($inv_task, ApplicationLogs::ACTION_EDIT, false, true);
			}
		}

	}

	function changeAssociatedTimeslotsStatus($status, $timeslots = null) {

		if (!in_array($status, array('invoiced','pending')) || !$this instanceof IncomeInvoice) return;
	
		if (is_array($timeslots) && count($timeslots) > 0) {
			$invoiced_timeslots = $timeslots;
		} else {
			$invoiced_timeslots = Timeslots::instance()->findAll(array("conditions" => array("invoice_id=?", $this->getId())));
		}
		
		foreach ($invoiced_timeslots as $inv_timeslot) {
			$inv_timeslot->changeInvoicingStatus($status, $this->getId());
		}

	}

    /**
     * Validate before save
     *
     * @access public
     * @param array $errors
     * @return null
     */
    function validate($errors) {
    	Env::useHelper('functions', 'income');
        if (!$this->validatePresenceOf('customer_id') && income_config_option("mandatory_customer")) {
            $errors[] = lang('invoice customer required');
        }

        if (!$this->validatePresenceOf('date')) {
            $errors[] = lang('invoice date required');
        }

        if (!$this->validatePresenceOf('customer_name') && income_config_option("mandatory_customer")) {
            $errors[] = lang('invoice customer name required');
        }

        if (!$this->disable_mandatory_fields_validation) {
	        $option = IncomeConfigOptions::instance()->findOne(array('conditions' => "name='mandatory_customer_address'"));
			$mandatory_customer_address = $option instanceof IncomeConfigOption && $option->getValue();
			if ($mandatory_customer_address) {
				if (!$this->validatePresenceOf('customer_address')) {
		            $errors[] = lang('invoice customer address required');
		        }
			}

	        $option = IncomeConfigOptions::instance()->findOne(array('conditions' => "name='mandatory_customer_city'"));
			$mandatory_customer_city = $option instanceof IncomeConfigOption && $option->getValue();
			if ($mandatory_customer_city) {
				if (!$this->validatePresenceOf('customer_city')) {
		            $errors[] = lang('invoice customer city required');
		        }
			}

	        $option = IncomeConfigOptions::instance()->findOne(array('conditions' => "name='mandatory_customer_rut'"));
			$mandatory_customer_rut = $option instanceof IncomeConfigOption && $option->getValue();
			if ($mandatory_customer_rut) {
				if (!$this->validatePresenceOf('customer_rut')) {
					if ($this->getFinalConsumer()) {
						$errors[] = lang('invoice customer document required');
					} else {
						$errors[] = lang('invoice customer rut required');
					}
		        }
			}
        }
    }

    function getClient() {
        if ($this->getMemberInvoice() instanceof Member) {
        	return $this->getMemberInvoice()->getName();
        } else {
        	return lang('n/a');
        }
    }

    /**
	 * Function that set in a private variable the member of the invoice and also return it
	 *
     * @return Member|null
     */
    public function getMemberInvoice(){
    	if(is_null($this->member_invoice)){
    		$this->member_invoice = Members::instance()->findById($this->getCustomerId());
		}
        return $this->member_invoice;
	}

    function getInvoiceLines() {
        $invoice_lines = IncomeInvoiceLines::getInvoiceLines($this->getObjectId());
        return $invoice_lines;
    }

	// function getInvoiceLinesChilds() {
    //     $invoice_lines = IncomeInvoiceLines::getInvoiceLinesChilds($this->getObjectId());
    //     return $invoice_lines;
    // }

	function getInvoiceReferences() {
		$invoice_references = IncomeInvoiceReferences::getInvoiceReferences($this->getObjectId());
		return $invoice_references;
	}

	function getCreditNoteReferences() {
		return IncomeInvoiceReferences::getCreditNoteReferences($this->getId());
	}

    function getTaxes() {
        return IncomeInvoiceTaxes::instance()->findAll(array(
                    'conditions' => array('`invoice_id` = ?', $this->getId())
                ));
    }

    function getCopies() {
        return IncomeInvoiceCopies::instance()->findAll(array(
                    'conditions' => array('`invoice_id` = ?', $this->getId())
                ));
    }

    static function findInvoiceNotebook($notebook_id) {
        return IncomeInvoices::instance()->findOne(array(
                    'conditions' => array('`invoice_notebook_id` = ? AND trashed_on = ? AND trashed_by_id = 0', $notebook_id,EMPTY_DATETIME),
                    'order' => 'CAST(invoice_number as UNSIGNED) DESC'
                ));
    }

    /**
     * Return object URL
     *
     * @access public
     * @param void
     * @return string
     */
    function getObjectUrl() {
        return $this->getViewUrl();
    }

    function getId() {
        return parent::getObjectId();
    }

    function getArrayInfo() {
    	$invoice_lines=[];
		foreach($this->getInvoiceLines() as $invoiceLine){
			$invoice_lines[]=$invoiceLine->getArrayInfo();
		}

    	// set invoice data
        $info = array(
            "object_id" => $this->getObjectId(),
            "invoice_notebook_id" => $this->getInvoiceNotebookId(),
            "name" => $this->getName(),
            "invoice_number" => $this->getInvoiceNumber(),
            "invoice_type" => $this->getInvoiceType(),
            "series" => $this->getSeries(),
            "date" => $this->getDate() instanceof DateTimeValue ? $this->getDate() : null,
			"expiration_date" => $this->getExpirationDate() instanceof DateTimeValue ? $this->getExpirationDate() : null,
            "final_consumer" => $this->getFinalConsumer(),
            "subtotal" => $this->getSubtotal(),
            "total" => $this->getTotal(),
			"paid_amount" => $this->getPaidAmount(),
			"due_amount" => $this->getDueAmount(),
            "customer_id" => $this->getCustomerId(),
            "customer_name" => $this->getCustomerName(),
            "customer_address" => $this->getCustomerAddress(),
        	"customer_city" => $this->getCustomerCity(),
            "customer_rut" => $this->getCustomerRut(),
            "customer_number" => $this->getCustomerNumber(),
        	"is_repetitive" => $this->isRepetitive() ? "1" : "0",
        	"additional_information" => $this->getAdditionalInformation(),
			"status"=>$this->getStatus(),
            "invoice_line" => $invoice_lines,
            'customer_state' => $this->getCustomerState(),
            'customer_country' => $this->getCustomerCountry(),
			'customer_zipcode' => $this->getCustomerZipcode(),
			'customer_email' => $this->getCustomerEmail(),
			'customer_phone' => $this->getCustomerPhone(),
        );

		if (Plugins::instance()->isActivePlugin('multiple_currencies')) {
        	$info['currency_id'] = $this->getCurrencyId();
        	$c = Currencies::instance()->findById($info['currency_id']);
        	if ($c instanceof Currency) {
        		$info['currency_symbol'] = $c->getSymbol();
        	}
        }
        if (Plugins::instance()->isActivePlugin('einvoice')) {
            $info['einvoice_destiny_rate'] = $this->getColumnValue('einvoice_destiny_rate');
            $info['einvoice_destiny_currency_id'] = $this->getColumnValue('einvoice_destiny_currency_id');
        }

        // set repeat options
        $columns = $this->getColumns();
        foreach ($columns as $col) {
        	if (str_starts_with($col, 'repeat_')) {
        		$info[$col] = $this->getColumnValue($col);
        	}
        }

        return $info;
    }


    function isRepetitive() {
    	return $this->getRepeatD() > 0 || $this->getRepeatM() > 0 || $this->getRepeatY() > 0 || $this->getRepeatH() > 0;
    }


    function generateNextRepetition() {

    	if ($this->isRepetitive()) {

    		$new_date = $this->getNextRepetitionDate($this->getDate());
    		if ($new_date instanceof DateTimeValue) {

    			$tmp_inv = $this->manager()->findOne(array('conditions' => "from_invoice_id = ".$this->getId()));
    			if ($tmp_inv instanceof IncomeInvoice) {
    				throw new Exception(lang("next invoice repetition already generated"));
    			}

    			if ($this->getRepeatEnd() instanceof DateTimeValue) {
    				if ($this->getRepeatEnd()->getTimestamp() < $new_date->getTimestamp()) {
    					throw new Exception(lang("cannot generate more repetitions after date", format_date($this->getRepeatEnd(), null, 0)));
    				}
    			}

    			$new_name = $this->getName();
    			$date_str = format_date($this->getDate(), null, 0);
    			if (strpos($new_name, $date_str)) {
    				$new_name = str_replace($date_str, format_date($new_date, null, 0), $new_name);
    			}


				// copy invoice record
    			$inv = new IncomeInvoice();
    			$columns = $this->manager()->getColumns();
    			foreach ($columns as $col) {
					if($col == 'paid_amount'){
						$inv->setColumnValue('paid_amount', 0);
					}else if($col == 'due_amount'){
						$inv->setColumnValue('due_amount', $inv->getTotal());
					}else if ($col != 'object_id' && $col != 'date') {
    					$inv->setColumnValue($col, $this->getColumnValue($col));
    				}
    			}

    			$inv->object->setColumnValue('object_subtype_id', $this->getColumnValue('object_subtype_id'));

				$new_expiration_date = $this->getNextRepetitionDate($this->getExpirationDate());
				
				$temp_data = array(
						'object_subtype_id' => $this->getColumnValue('object_subtype_id'),
						'invoice_number' => '', 'series' => '', 'invoice_notebook_id' => 0
				);
				Env::useHelper('functions', 'income');
				fill_invoice_data_serie_and_number($inv, $temp_data);

    			$inv->setInvoiceNotebookId($temp_data['invoice_notebook_id']);
    			$inv->setSeries($temp_data['series']);
    			$inv->setInvoiceNumber($temp_data['invoice_number']);
    			
    			$inv->setObjectName($new_name);
    			$inv->setDate($new_date);
				$inv->setExpirationDate($new_expiration_date);
    			$inv->setStatus('pending');
    			$inv->setFromInvoiceId($this->getId());
    			if ($inv->getRepeatNum() > 0) {
	    			// decrease repetition number
    				$inv->setRepeatNum($inv->getRepeatNum() - 1);
    			}

    			$inv->disableMandatoryFieldsValidation(true);
    			$inv->save();

    			// copy invoice lines
    			$lines = $this->getInvoiceLines();
    			foreach ($lines as $line) {
    				$new_line = new IncomeInvoiceLine();
    				$columns = $line->manager()->getColumns();
    				foreach ($columns as $col) {
    					if ($col != 'object_id' && $col != 'id') {
    						$new_line->setColumnValue($col, $line->getColumnValue($col));
    					}
    				}
    				$new_line->setObjectId($inv->getId());
    				$new_line->save();
    			}

    			// copy invoice taxes
    			$taxes = $this->getTaxes();
    			foreach ($taxes as $tax) {
    				$new_tax = new IncomeInvoiceTax();
    				$columns = $tax->manager()->getColumns();
    				foreach ($columns as $col) {
    					if ($col != 'object_id' && $col != 'id') {
    						$new_tax->setColumnValue($col, $tax->getColumnValue($col));
    					}
    				}
    				$new_tax->setInvoiceId($inv->getId());
    				$new_tax->save();
    			}

    			// copy invoice copies
    			$copies = $this->getCopies();
    			foreach ($copies as $copy) {
    				$new_copy = new IncomeInvoiceCopy();
    				$columns = $copy->manager()->getColumns();
    				foreach ($columns as $col) {
    					if ($col != 'object_id' && $col != 'id') {
    						$new_copy->setColumnValue($col, $copy->getColumnValue($col));
    					}
    				}
    				$new_copy->setInvoiceId($inv->getId());
    				$new_copy->save();
    			}

    			// add to members
    			$object_controller = new ObjectController();
    			$object_controller->add_to_members($inv, $this->getMemberIds());

    			// copy subscribers
    			$subscribers = $this->getSubscribers();
    			foreach ($subscribers as $user) {
    				$inv->subscribeUser($user);
    			}

    			// copy custom properties
    			$cp_vals = CustomPropertyValues::instance()->findAll(array('conditions' => array("`object_id` = ?", $this->getId())));
    			foreach ($cp_vals as $cp_val) {
    				$new_cp_val = new CustomPropertyValue();
    				$new_cp_val->setObjectId($inv->getId());
    				$new_cp_val->setCustomPropertyId($cp_val->getCustomPropertyId());
    				$new_cp_val->setValue($cp_val->getValue());
    				$new_cp_val->save();
    			}

    			return $inv;
    		}
    	} else {
    		throw new Exception(lang("invoice is not repetitive"));
    	}
    }


	function getNextRepetitionDate($current_date) {
    	$new_date = null;

    	if ($current_date instanceof DateTimeValue) {
    		$new_date = new DateTimeValue($current_date->getTimestamp());
    	}

    	if ($new_date instanceof DateTimeValue) {
	    	if ($this->getRepeatD() > 0) {

		    	$new_date = $new_date->add('d', $this->getRepeatD());

	    	} else if ($this->getRepeatM() > 0) {

	    		$new_date = $new_date->add('M', $this->getRepeatM());

	    	} else if ($this->getRepeatY() > 0) {

	    		$new_date = $new_date->add('y', $this->getRepeatY());

	    	} else if ($this->getRepeatH() > 0) {

	    		$new_date = $new_date->add('M', $this->getRepeatMjump());
	    		switch ($this->getRepeatWnum()) {
	    			case 1: $week = 'first'; break;
	    			case 2: $week = 'second'; break;
	    			case 3: $week = 'third'; break;
	    			case 4: $week = 'fourth'; break;
	    		}
	    		switch ($this->getRepeatDow()) {
	    			case 1: $dow = 'sun'; break;
	    			case 2: $dow = 'mon'; break;
	    			case 3: $dow = 'tue'; break;
	    			case 4: $dow = 'wed'; break;
	    			case 5: $dow = 'thu'; break;
	    			case 6: $dow = 'fri'; break;
	    			case 7: $dow = 'sat'; break;
	    		}
	    		$month_name = $new_date->format("F");
	    		$year = $new_date->getYear();

	    		$formatted = "$week $dow of $month_name $year";

	    		$timestamp = strtotime($formatted);
	    		$new_date = new DateTimeValue($timestamp);
	    	}
    	}

    	return $new_date;
    }


    function getCurrencyId() {
    	if ($this->columnExists('currency_id')) {
    		return $this->getColumnValue('currency_id');
    	}
    	return 0;
    }



    function getNextInvoiceInRepetition() {
    	if (is_null($this->next_invoice)) {
    		return $this->manager()->findOne(array("conditions" => array("from_invoice_id=?", $this->getId())));
    	}
    	return $this->next_invoice;
    }

    function getPreviousInvoiceInRepetition() {
    	if (is_null($this->previous_invoice)) {
    		$this->previous_invoice = $this->manager()->findById($this->getFromInvoiceId());
    	}
    	return $this->previous_invoice;
    }
    
    function getCustomerLegalName() {
    	$legal_name = null;
    	$client_member = $this->getMemberInvoice();
    	if ($client_member instanceof Member) {
    		$cp = CustomProperties::getCustomPropertyByCode($client_member->getObjectTypeId(), 'legal_name');
    		if ($cp) {
    			$cp_val = CustomPropertyValues::getCustomPropertyValue($client_member->getObjectId(), $cp->getId());
    			if ($cp_val) $legal_name = $cp_val->getValue();
    		}
    	}
    	if (!$legal_name){
    		$legal_name = $this->getCustomerName();
    	}
    	return $legal_name;
    }
    
    function getInvoiceTemplate() {
    	$template = null;
    	
    	$cp = CustomProperties::getCustomPropertyByCode($this->getObjectTypeId(), 'invoice_template');
    	if ($cp instanceof CustomProperty) {
    		$cp_val = CustomPropertyValues::getCustomPropertyValue($this->getId(), $cp->getId());
    		if ($cp_val) {
    			$template = IncomeInvoiceTemplates::instance()->findById($cp_val->getValue());
    		}
    	}

		// if no template defined for this invoice, try to find the default one
		if (!$template) {
			$template = IncomeInvoiceTemplates::getDefaultInvoiceTemplate();
		}
    	
    	return $template;
    }


	/**
	 * Gets the summary information of the previous invoices classified in the same project of this invoice
	 * @return array
	 */
	function getProjectSummary($summary_options = array()) {

		$project_ot = ObjectTypes::findByName('project');
		$members = $this->getMembers();
		$project_member = null;
		foreach ($members as $member) {
			if ($member->getObjectTypeId() == $project_ot->getId()) {
				$project_member = $member;
				break;
			}
		}

		if ($project_member instanceof Member) {
			return $this->getSummaryForMember($project_member, $summary_options);
		}

		return null;
	}


	/**
	 * Gets the summary information of the previous invoices classified in the same client of this invoice
	 * @return array
	 */
	function getClientSummary($summary_options = array()) {

		$client_ot = ObjectTypes::findByName('customer');
		$members = $this->getMembers();
		$client_member = null;
		foreach ($members as $member) {
			if ($member->getObjectTypeId() == $client_ot->getId()) {
				$client_member = $member;
				break;
			}
		}

		if ($client_member instanceof Member) {
			return $this->getSummaryForMember($client_member, $summary_options);
		}

		return null;
	}

	/**
	 * Gets the summary information of the previous invoices classified in $member
	 * @return array
	 */
	function getSummaryForMember(Member $member, $summary_options = array()) {

		if(isset($this))
		{
			$dateInvoice=" AND `date` <= '".$this->getDate()->toMySQL()."'";
		}else{
			$dateInvoice='';
		}
		$extra_conditions = " AND `status` != 'canceled' $dateInvoice";
		

		// get all the past invoices to sum the services and expenses amounts in this member (project)
		$member_invoices = IncomeInvoices::instance()->listing(array(
            "count_results" => false,
            "extra_conditions" => $extra_conditions,
			"ignore_context" => true,
			"member_ids" => array($member->getId()),
            "order" => "`date`",
            "order_dir" => "ASC",
		))->objects;

		//search for the last invoice
		$last_date='';
		if(count($member_invoices)==0)
		{
			return 0;
		}
		foreach ($member_invoices as $member_last_invoice) { 
		    $invoice_date=$member_last_invoice->getDate();
		    if($invoice_date>$last_date)
		    {
               $last_invoce_replace = $member_last_invoice;
		    }
		}

		if(isset($this))
		{
			$actual_invoice=$this;
		}else{
			$actual_invoice=$last_invoce_replace;
		}

		// last invoice info is needed at footer
		$last_invoice = null;
		$last_paid_amount = null;
		$totals_by_currency = array();
		$TotalDueChilds=0;
		$last_invoice_total=0;
		$last_invoice_totalJoint=0;
		
		$InvoiceParents= IncomeInvoiceChilds::getInvoiceParent($actual_invoice->getId());

		$invoiceChild_ids = [];
		foreach($InvoiceParents as $InvoiceParent)
		{
			$invoiceChilds=IncomeInvoiceChilds::getInvoiceChild($InvoiceParent->getInvoiceParent());
			//$invoiceChilds=$InvoiceParent->getInvoiceChild();
			foreach($invoiceChilds as $invoiceChild)
			{
				$invoiceChild_ids[]=$invoiceChild->getObjectIdChild();
			}
		}
			
		///for each invoice in the member sum the amounts  by currency
		foreach ($member_invoices as $member_invoice) { /* @var $member_invoice IncomeInvoice */
			$lines = $member_invoice->getInvoiceLines();
	
			$currency = Currencies::instance()->getCurrency($member_invoice->getCurrencyId());
			if (!$currency instanceof Currency)
			{
				$currency = Currencies::getDefaultCurrencyInfo();
				$c_sym = $currency['symbol'];
			        $c_id = $currency['id'];
			}else
			{
			    $c_sym = $currency->getSymbol();
			    $c_id = $currency->getId();
			}

			if (!isset($totals_by_currency[$c_id])) {
				$totals_by_currency[$c_id] = array(
					'currency_symbol' => $c_sym,
					'total_labor' => 0,
					'total_expenses' => 0,
					'prev_due_amount' => 0,
					'total_due_amount' => 0,
					'total_paid_amount' => 0,
				);
			}
	
			foreach ($lines as $line) {/* @var $line IncomeInvoiceLine */
				if ($line->getFromObjectTypeId() == 0 || $line->getFromObjectTypeId() == Timeslots::instance()->getObjectTypeId()) {
					$totals_by_currency[$c_id]['total_labor'] += $line->getTotal();
				} else if (Plugins::instance()->isActivePlugin('expenses2') && $line->getFromObjectTypeId() == PaymentReceipts::instance()->getObjectTypeId()) {
					$totals_by_currency[$c_id]['total_expenses'] += $line->getTotal();
				}
			}

			if ($member_invoice->columnExists('due_amount') && $member_invoice->getId() != $actual_invoice->getId() && !in_array($member_invoice->getId(), $invoiceChild_ids)) {
				$totals_by_currency[$c_id]['prev_due_amount'] += $member_invoice->getColumnValue('due_amount');
			}
			
			if ($member_invoice->columnExists('due_amount') && $member_invoice->getId() != $actual_invoice->getId() && in_array($member_invoice->getId(), $invoiceChild_ids)) 
			{
				$TotalDueChilds+=$member_invoice->getColumnValue('due_amount');
			}
			
			//gets always member last invoice
			$last_invoice_member = $member_invoice;

			if ( !in_array($member_invoice->getId(), $invoiceChild_ids) && $member_invoice->getId() != $actual_invoice->getId()) {
				$last_invoice_total = $member_invoice->getTotal();
				$last_invoice = $member_invoice;
			}

			//check parent for previous invoice
			$InvoiceMemberParents='';
			$InvoiceMemberParents= IncomeInvoiceChilds::getInvoiceParent($member_invoice->getId());

			foreach($InvoiceMemberParents as $InvoiceMemberParent)
			{
				$invoiceMemberChilds=IncomeInvoiceChilds::getInvoiceChild($InvoiceMemberParent->getInvoiceParent());
				foreach($invoiceMemberChilds as $invoiceMemberChild)
				{
					$invoiceMemberChild_ids[]=$invoiceMemberChild->getObjectIdChild();
				}
			}

			if(in_array($member_invoice->getId(),$invoiceMemberChild_ids) && !in_array($member_invoice->getId(),$invoiceChild_ids))
			{
				$last_invoice_totalJoint+=$member_invoice->getTotal();

			}

			// get the last paid amount for an invoice in the current member
			if ($member_invoice->columnExists('paid_amount') && $member_invoice->getColumnValue('paid_amount') > 0) {
				$last_paid_amount = array(
					'amount' => $member_invoice->getColumnValue('paid_amount'),
					'currency_symbol' => $c_sym,
				);

				// accumulate the total paid amount of the member
				$totals_by_currency[$c_id]['total_paid_amount'] += $member_invoice->getColumnValue('paid_amount');
			}
		}

		if($last_invoice_totalJoint>0)
		{
			$last_invoice_total=$last_invoice_totalJoint;
		}
		
		// sum current invoice's total to the prev due amount to get the total due amount
		$this_invoice_due_amount = $actual_invoice->columnExists('due_amount') ? $actual_invoice->getColumnValue('due_amount') : $actual_invoice->getTotal();
		$totals_by_currency[$actual_invoice->getCurrencyId()]['total_due_amount'] = $totals_by_currency[$actual_invoice->getCurrencyId()]['prev_due_amount'] + $this_invoice_due_amount+$TotalDueChilds;//+$totalantDue;
		
		$summary = array(
			'totals_btd' => $totals_by_currency,
			'last_paid_amount' => $last_paid_amount,
			
		);
		$summary['last_invoice_member']=$last_invoice_member->getInvoiceNumber();
		$summary['last_invoice_member_id']=$last_invoice_member->getId();

		if ($last_invoice instanceof IncomeInvoice) {
			$c = Currencies::instance()->getCurrency($last_invoice->getCurrencyId());

			$summary['last_invoice'] = array(
				'number' => $last_invoice->getInvoiceNumber(),
				'date' => format_date($last_invoice->getDate(), null, 0),
				'currency_symbol' => $c ? $c->getSymbol() : '',
				'amount' =>$last_invoice_total,//last_invoice->getTotal(),
			);
		}

		$contract_amount_cp = CustomProperties::getCustomPropertyByCode($member->getObjectTypeId(), 'budget_total');
		if ($contract_amount_cp) {
			$cp_val = CustomPropertyValues::getCustomPropertyValue($member->getObjectId(), $contract_amount_cp->getId());
			if ($cp_val) {
				$summary['contract_amount'] = $cp_val->getValue();
				$c_sym = '$';
				$c = Currencies::instance()->getCurrency($cp_val->getCurrencyId());
				if ($c) $c_sym = $c->getSymbol();
				$summary['contract_amount_currency_symbol'] = $c_sym;
			}
		}

		Hook::fire("invoice_member_summary_more_properties", array('invoice' => $actual_invoice, 'summary_options' => $summary_options), $summary);
         

		return $summary;

	}


	/**
	 * @return string The text to print in the invoice's footer, depending on the invoice template configuration
	 */
	function getFooterToPrint() {
		$footer_text = "";
		$invoice_template = $this->getInvoiceTemplate();

		if ($invoice_template instanceof IncomeInvoiceTemplate) {

			if ($invoice_template->getIncludeAdditionalInformationInFooter()) {
				$add_info = trim($this->getAdditionalInformation());
				if ($add_info != '') {
					$footer_text = $add_info;
				}
			}

			// add empty line between invoice additional information and template's footer
			if ($footer_text != '' && $invoice_template->getIncludeFooterText() != '') {
				$footer_text .= "<br/><br/>";
			}
			
			$footer_text .= $invoice_template->getIncludeFooterText(); 

		}
		return $footer_text;
	}



	function recalcualtePaidAndDueAmounts() {

		$total_paid_amount = 0;

		// get all credit notes affecting this invoice and sum the amounts
		$references = $this->getCreditNoteReferences();
		foreach ($references as $invoice_ref) {
			$total_paid_amount += $invoice_ref->getAmount();
		}

		// let other plugins affect this calculation (like invoice payment receipts)
		Hook::fire('on_invoice_recalculate_paid_amount', $this, $total_paid_amount);

		if ($total_paid_amount > $this->getTotal()) {
			$total_paid_amount = $this->getTotal();
		}
		
		// set due and paid amounts
		$this->setPaidAmount($total_paid_amount);
		$this->setDueAmount($this->getTotal() - $total_paid_amount);

		// calculate status
		$this->recalculateStatusUsingAmounts();

		$this->save();
		
	}

	function recalculateStatusUsingAmounts() {

		if ($this->getPaidAmount() == 0) {
			$invoice_status = 'pending';
		} else if (round($this->getPaidAmount(), 2) >= round($this->getTotal(), 2)) {
			$invoice_status = 'confirmed';
		} else {
			$invoice_status = 'part_confirmed';
		}
		
		$this->setStatus($invoice_status);
	}


	/**
	 * for credit notes, recalculate paid and due amounts
	 */ 
	function recalculateReferencedInvoicesAmounts() {
		$references = $this->getInvoiceReferences();
		foreach ($references as $ref) {
			$inv = $ref->getReferencedObject();
			if ($inv instanceof IncomeInvoice) {
				$inv->recalcualtePaidAndDueAmounts();
			}
		}
	}


}


