<?php

/**
 * IncomeInvoices
 * Generado el 10/5/2012
 * 
 */
class IncomeInvoices extends BaseIncomeInvoices {

    function __construct() {
        parent::__construct();
        $this->object_type_name = 'invoice';
    }


    function getColumnsToAggregateInTotals() {
    	return array_merge(parent::getColumnsToAggregateInTotals(), array(
    		'subtotal' => array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'currency_id', 'group_by' => 'currency_id'),
    		'total' => array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'currency_id', 'group_by' => 'currency_id'),
    		'paid_amount' => array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'currency_id', 'group_by' => 'currency_id'),
    		'due_amount' => array('operation' => 'sum', 'format' => 'money', 'currency_id_col' => 'currency_id', 'group_by' => 'currency_id')
    	));
    }
    
    
    function getInvoicesInMembers($member_ids, $extra_conditions = "") {
    	
    	$member_invoices = $this->listing(array(
    			"extra_conditions" => "AND status != 'canceled' AND trashed_by_id=0 $extra_conditions",
    			"ignore_context" => true,
    			"member_ids" => $member_ids,
    	))->objects;
    	
    	return $member_invoices;
    }
    
    function getInvoicedTotalInMembers($member_ids, $extra_conditions = "") {
    	
    	$total_billed = 0;
    	$member_invoices = $this->getInvoicesInMembers($member_ids, $extra_conditions);
    	foreach ($member_invoices as $invoice) {
    		$total_billed += $invoice->getTotal();
    	}
    	
    	return $total_billed;
    }

	/**
	 * This function allows you get the invoices
	 * loaded in the system, orderded by date, 
	 * with a limit. Default, 6.
	 * 
	 * @param int $limit
	 */
	static function getAllInvoices($limit = null) {
		return IncomeInvoices::instance()->findAll(array(
			'conditions' => array('trashed_by_id=0'),
			'order' => 'date DESC',
			'limit' => $limit,
		));
	}

	static function getInvoicesInDateRange($date_from, $date_to, $limit = 12) {
		return IncomeInvoices::instance()->findAll(array(
			'conditions' => array('date > "'.$date_from.'" AND date < "'.$date_to.'" AND trashed_by_id=0'),
			'order' => 'date DESC',
			'limit' => $limit,
		));
	}

	static function getAmountInvoicesPaid() {
		return count(IncomeInvoices::instance()->findAll(array(
			'conditions' => array('status="confirmed" AND trashed_by_id=0'),
		)));
	}

	static function getTotalInvoices() {
		return count(IncomeInvoices::instance()->findAll(array(
			'conditions' => array('trashed_by_id=0'),
		)));
	}

	static function getInvoiceWithGreatestNumber() {
		return IncomeInvoices::instance()->findOne(array(
			'conditions' => array('trashed_on = ? AND trashed_by_id = 0', EMPTY_DATETIME),
			'order' => 'CAST(invoice_number as UNSIGNED) DESC',
		));
	}

	static function findInvoiceByNumber($invoice_number) {
		return IncomeInvoices::instance()->findOne(array(
			'conditions' => array('invoice_number = ? AND trashed_by_id=0', $invoice_number)
		));
	}

	static function findInvoiceContainNumber($invoice_number) {
		return IncomeInvoices::instance()->findOne(array(
			'conditions' => array('invoice_number LIKE "'.$invoice_number.'%"  AND trashed_by_id=0'),
			'order' => 'invoice_number DESC'
		));
	}

	static function findInvoicesContainNumberForClient($invoice_number, $customer_id) {
		return IncomeInvoices::instance()->findOne(array(
			'conditions' => array('invoice_number LIKE "'.$invoice_number.'%" AND customer_id = ?  AND trashed_by_id=0', $customer_id),
			'order' => 'invoice_number DESC'
		));
	}

    
    static function getInvoiceTemplateFromMembers($members) {
    	
    	$project_ot = ObjectTypes::findByName('project');
    	$client_ot = ObjectTypes::findByName('customer');
    	
    	$project_member = null;
    	$client_member = null;
    	
    	foreach ($members as $m) {
    		if ($m instanceof Member) {
	    		if ($m->getObjectTypeId() == $project_ot->getId()) $project_member = $m;
	    		if ($m->getObjectTypeId() == $client_ot->getId()) $client_member = $m;
    		}
    	}
    	
    	$invoice_template = null;
    	
    	if ($project_member instanceof Member) {
    		
    		$cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), 'invoice_template');
    		$cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $cp->getId());
    		if ($cp_val) $invoice_template = IncomeInvoiceTemplates::instance()->findById($cp_val->getValue());
    		
    		if (!$invoice_template) {
    			if (!$client_member instanceof Member) {
    				Env::useHelper('functions', 'crpm');
    				$clients_dimension = get_customers_dimension();
    				if ($clients_dimension->getId() != $project_member->getDimensionId()) {
    					$client_mem_id = get_associated_status_member_id($project_member, $clients_dimension, $client_ot, false, 'project_customer');
    					$client_member = Members::getMemberById($client_mem_id);
    				} else {
    					$project_parents = $project_member->getAllParentMembersInHierarchy(false, false);
    					foreach ($project_parents as $p) {
    						if ($p->getObjectTypeId() == $client_ot->getId()) {
    							$client_member = $p;
    							break;
    						}
    					}
    				}
    			}
    			
    			if ($client_member instanceof Member) {
    				$cp = CustomProperties::getCustomPropertyByCode($client_ot->getId(), 'invoice_template');
    				$cp_val = CustomPropertyValues::getCustomPropertyValue($client_member->getObjectId(), $cp->getId());
    				if ($cp_val) $invoice_template = IncomeInvoiceTemplates::instance()->findById($cp_val->getValue());
    			}
    		}
    		
    	} else if ($client_member instanceof Member) {
    		
    		$cp = CustomProperties::getCustomPropertyByCode($client_ot->getId(), 'invoice_template');
    		$cp_val = CustomPropertyValues::getCustomPropertyValue($client_member->getObjectId(), $cp->getId());
    		if ($cp_val) $invoice_template = IncomeInvoiceTemplates::instance()->findById($cp_val->getValue());
    	}
    	
    	return $invoice_template;
    }

	static function save_invoice(IncomeInvoice $invoice, $invoice_data, $member_ids, $invoice_template='')
	{
		// to use when saving the application log
		$old_content_object = $invoice->generateOldContentObjectData();

		$invoice_line_data = array_var($invoice_data, 'invoice_lines');
			
			/*if (is_null($invoice_line_data)) {
				$invoice_line_data = array_var($_POST, 'invoice_lines');
			}*/

		$customer_present = false;
		$customer_member = null;
		$crpm_dim = Dimensions::findByCode('customer_project');
		$cli_ot = ObjectTypes::findByName('customer')->getId();

		$allow_duplicated_invoice_numbers = income_config_option('allow_duplicated_invoice_numbers');


		if($invoice_data['invoice_number']){
			if(
				!$allow_duplicated_invoice_numbers &&
				($invoice->isNew() || (!empty($invoice->getInvoiceNumber()) && ($invoice->getInvoiceNumber() != $invoice_data['invoice_number'])))
			){
				$exist_invoice = IncomeInvoices::findInvoiceByNumber($invoice_data['invoice_number']);
				if($exist_invoice){
					throw new Exception(lang('invoice exist'));
				}
			}
		}

		// modify the invoice number if exists. append sufix.
		if(!$allow_duplicated_invoice_numbers && $invoice_data['invoice_number'] && $invoice->isNew()){
			$exist_invoice = IncomeInvoices::findInvoiceByNumber($invoice_data['invoice_number']);
			if($exist_invoice){
				$latest = IncomeInvoices::findInvoiceContainNumber($invoice_data['invoice_number']."-");
				if($latest){
					$latest_invoice_number = $latest->getInvoiceNumber();
					$number_expl = explode("-", $latest_invoice_number);
					$number = (int) $number_expl[1];
					$final_number = (($number + 1) < 10) ? "0". ($number + 1) : ($number + 1);
				}else{
					$final_number = "01";
				}

				$invoice_data['invoice_number'] = $invoice_data['invoice_number']."-".$final_number;
			}
		}

		$mem_object_type_ids = DimensionObjectTypes::getObjectTypeIdsByDimension($crpm_dim->getId());

		foreach ($member_ids as $member_id) {
			$member = Members::instance()->findById($member_id);
			if ($member instanceof Member) {
				if ($member->getObjectTypeId() == $cli_ot) {

					$customer_present = true;
					$customer_member = $member;

				} else if (in_array($member->getObjectTypeId(), $mem_object_type_ids)) {

					$all_parents = $member->getAllParentMembersInHierarchy();
					foreach ($all_parents as $parent) {
						if ($parent->getObjectTypeId() == $cli_ot) {
							$customer_present = true;
							$customer_member = $parent;
							break;
						}
					}
				}
			}
		}

		// if an email address has been sent => search client by its associated contact's email, if not exists => create it
		if (!$customer_present) {
			Env::useHelper('functions', 'crpm');

			$client_email_address = array_var($invoice_data, 'customer_email_address');
			$customer_member = find_client_member_by_email($client_email_address);

			if (!$customer_member instanceof Member) {
				$customer_member = create_client_member_by_email_and_name_and_set_billing_contact($invoice_data);
			}

			if ($customer_member instanceof Member) {
				$member_ids[] = $customer_member->getId();
				$customer_present = true;
			}
		}

		if (!$customer_present) {
			if (income_config_option("mandatory_customer")) {
				throw new Exception(lang('you have to select a client or a project or folder below a client'));
			}
		} else {
			if (!array_var($invoice_data, 'customer_name')) $invoice_data['customer_name'] = $customer_member->getName();
			$invoice_data['customer_id'] = $customer_member->getId();
		}

		IncomeInvoices::parseRepeatOptions($invoice_data);

		if ( isset($invoice_data['use_mysql_dates'])) {
			if (array_var($invoice_data, 'date', '') != '') {
				$invoice_data['date'] = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $invoice_data['date']);
			}
			if (array_var($invoice_data, 'expiration_date', '') != '') {
				$invoice_data['expiration_date'] = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $invoice_data['expiration_date']);
			}
		} else {
			$invoice_data['date'] = getDateValue($invoice_data['date']);
			$invoice_data['expiration_date'] = isset($invoice_data['expiration_date']) ? getDateValue($invoice_data['expiration_date']) : null;
		}
		$invoice_data['customer_rut'] = trim(array_var($invoice_data, 'customer_rut', ''));


		// if currency not set and only one, put that currency
		$all_currencies = Currencies::getCurrenciesInfo();
		if (count($all_currencies) == 1) {
			$invoice_data['currency_id'] = $all_currencies[0]['id'];
		}

		if (
			(!$invoice->isNew()
			&& income_config_option("allow_invoice_flexible_status_change") 
			&& !isset($invoice_data['status']))
			|| $invoice->isNew()
		){
			$invoice_data['status'] = 'pending';
		}

		// create ordered_by contact, if needed
		if (!array_var($invoice_data, 'ordered_by_id') &&
			(array_var($invoice_data, 'ordered_by_email') || array_var($invoice_data, 'ordered_by_name') || array_var($invoice_data, 'ordered_by_lastname'))) {

			$ordered_by = null;
			if (array_var($invoice_data, 'ordered_by_email')) {
				$contact_email = array_var($invoice_data, 'ordered_by_email');
				
				// if duplicated emails are allowed then get the contact using the first name and surname too
				if (config_option('allow_duplicated_contact_emails')) {
					$ordered_by = Contacts::getByFirstnameAndSurname(trim(array_var($invoice_data, 'ordered_by_name')), trim(array_var($invoice_data, 'ordered_by_lastname')), $contact_email);
				} else {
					$ordered_by = Contacts::getByEmail($contact_email);
				}
			}
			if (!$ordered_by instanceof Contact) {
				$ordered_by = create_ordered_by_contact_from_invoice_data($invoice_data, $member_ids);
			}

			if ($ordered_by instanceof Contact) {
				$invoice_data['ordered_by_id'] = $ordered_by->getId();
			}
		}

		//set info of who is buying if this come from the web
		if (
			isset($invoice_data['ordered_by_address']) or
			isset($invoice_data['ordered_by_telephone']) or
			isset($invoice_data['ordered_by_city']) or
			isset($invoice_data['ordered_by_state']) or
			isset($invoice_data['ordered_by_country']) or
			isset($invoice_data['ordered_by_zipcode'])
		) {
			$invoice->setCustomerName("{$invoice_data['ordered_by_name']} {$invoice_data['ordered_by_lastname']}");
			$invoice->setCustomerEmail($invoice_data['ordered_by_email']);
			$invoice->setCustomerAddress($invoice_data['ordered_by_address']);
			$invoice->setCustomerCity($invoice_data['ordered_by_city']);
			$invoice->setCustomerPhone($invoice_data['ordered_by_telephone']);
			$invoice->setCustomerState($invoice_data['ordered_by_state']);
			$invoice->setCustomerCountry($invoice_data['ordered_by_country']);
			$invoice->setCustomerZipcode($invoice_data['ordered_by_zipcode']);
		}


		// set deafult name if not specified
		if ($invoice_data['name'] == "") {
			$customer_name = array_var($invoice_data, 'customer_name');
			if (!$customer_name) {
				$ordered_by = Contacts::instance()->findById(array_var($invoice_data, 'ordered_by_id'));
				if ($ordered_by instanceof Contact) {
					$customer_name = $ordered_by->getObjectName();
				}
			}
			
			$invoice_subtype_id = $invoice_data['object_subtype_id']; 
			$invoice_subtype_code = (ObjectSubtypes::instance()->findById($invoice_subtype_id))->getCode();
			//$invoice_name = lang($invoice_subtype_code.'_for', $customer_name, format_date($invoice_data['date'], null, 0));
			$invoice_name = lang($invoice_subtype_code.'_for');
			$invoice_name = $invoice_name." ".$customer_name;
			$invoice_name = $invoice_name." ".format_date($invoice_data['date']);

			Hook::fire('override_autogenerated_invoice_name', array(
				'invoice_data' => $invoice_data,
				'member_ids' => $member_ids,
				'customer_name' => $customer_name,
			), $invoice_name);
			
			$invoice_data['name'] = $invoice_name;
		}


		// if ordered by belongs to a client => then classify the invoice in that client
		if (!$customer_member instanceof Member && array_var($invoice_data, 'ordered_by_id') > 0) {
			if (!isset($ordered_by) || !$ordered_by instanceof Contact) {
				$ordered_by = Contacts::instance()->findById(array_var($invoice_data, 'ordered_by_id'));
			}
			// get the contact's client member
			if ($ordered_by instanceof Contact) {
				// check if the contact is a customer
				$temp_customer = Customers::instance()->findOne(array("conditions" => array("contact_id=?", $ordered_by->getId())));
				// check if the contact's comapany is a customer
				if (!$temp_customer instanceof Customer) {
					$temp_customer = Customers::instance()->findOne(array("conditions" => array("contact_id=?", $ordered_by->getCompanyId())));
				}
				// check if the contact is classified in any client
				if (!$temp_customer instanceof Customer) {
					$ordered_by_members = $ordered_by->getMembers();
					foreach ($ordered_by_members as $om) {
						/* @var $om Member */
						if ($om->getObjectTypeId() == Customers::instance()->getObjectTypeId()) {
							$customer_member = $om;
							break;
						}
					}
				}
				if (!$customer_member instanceof Member && $temp_customer instanceof Customer) {
					$customer_member = Members::findOneByObjectId($temp_customer->getId());
				}
				
				Hook::fire('additional_search_for_invoice_customer_member', array('invoice_data' => $invoice_data, 'ordered_by' => $ordered_by), $customer_member);
				
				// classify ordered_by in the customer member
				if ($customer_member instanceof Member) {
					$ob_member_ids = array($customer_member->getId());
					foreach ($ordered_by->getMembers() as $member){
						$ob_member_ids[] = $member->getId();
					}
					$object_controller = new ObjectController();
					$object_controller->add_to_members($ordered_by, $ob_member_ids);
				}
			}
		}

		if ($customer_member instanceof Member) {
			$member_ids[] = $customer_member->getId();
			$member_ids = array_unique(array_filter($member_ids));
		}
		
		if (!array_var($invoice_data, 'object_subtype_id')) {
			$invoice_subtype = ObjectSubtypes::instance()->findOne(array(
					'conditions' => array("object_type_id=? AND `name`='invoice'", IncomeInvoices::instance()->getObjectTypeId()),
			));
			if ($invoice_subtype instanceof ObjectSubtype) {
				$invoice_data['object_subtype_id'] = $invoice_subtype->getId();
			}
		}

		if (!isset($invoice_data['override_invoice_number'])) {
			$sent_invoice_number = array_var($invoice_data, 'invoice_number', '');
			$invoice_data['override_invoice_number'] = $sent_invoice_number != '' && $sent_invoice_number != '0';
		}
		
		// sets the fields invoice_number, serie and notebook_id in the $invoice_data variable
		// if quickbooks plugin is active then don't generate the number, it will be set after synchronizing to quickbooks
		if (!array_var($invoice_data, 'override_invoice_number') && !Plugins::instance()->isActivePlugin('quickbooks')) {
			fill_invoice_data_serie_and_number($invoice, $invoice_data);
		}
		
		if (!array_var($invoice_data, 'ordered_by_id') && $customer_member instanceof Member) {
			$customer_object = Customers::instance()->findById($customer_member->getObjectId());
			if ($customer_object instanceof Customer) {
				$invoice_data['ordered_by_id'] = $customer_object->getContactId();
			}
		}
		
		$invoice->setFromAttributes($invoice_data);

		// if type is credit_note or debit_note then it must have at least one reference
		$references_to_add = 0;
		$invoice_references_data = array_var($_POST, 'invoice_references');
		if (is_array($invoice_references_data) && count($invoice_references_data) > 0) {
			foreach ($invoice_references_data as $ir) {
				if (!$ir['deleted']) {
					$references_to_add++;
				}
			}
		}

		$inv_subtype_id = $invoice->getColumnValue('object_subtype_id');

		if ($references_to_add == 0) {
			$credit_note_type = ObjectSubtypes::instance()->findOne(array('conditions' => "code='credit_note'"));
			$debit_note_type = ObjectSubtypes::instance()->findOne(array('conditions' => "code='debit_note'"));
			
			
			if ($credit_note_type && $inv_subtype_id == $credit_note_type->getId()) {
				throw new Exception(lang('credit notes must have at least one reference'));
			} else if ($debit_note_type && $inv_subtype_id == $debit_note_type->getId()) {
				throw new Exception(lang('debit notes must have at least one reference'));
			}
		}
		// --

		try {

			//DB::beginWork();
			$invoice->save();
			$invoice_lines = IncomeInvoiceLines::getInvoiceLines($invoice->getObjectId());
			if (is_array($invoice_lines)) {
				foreach ($invoice_lines as $invoice_line) {
					$invoice_line->delete();
				}
			}

			$allow_multiline_line_desc = income_config_option("allow_multiline_line_desc");

			$use_taxes_in_lines = income_config_option("use_line_taxes");

			$invoice_line_data = array_var($invoice_data, 'invoice_lines');
			if (is_null($invoice_line_data)) {
				$invoice_line_data = array_var($_POST, 'invoice_lines');
			}

			$amounts_per_tax_id = array();

			$subtotal = 0;
			$lines_total = 0;
			if (is_array($invoice_line_data)) {
				foreach ($invoice_line_data as $invoice_l) {
					if ($invoice_l['unit_value'] == 0 && $invoice_l['detail'] == '') {
						continue;
					}
					
					
					if (!isset($invoice_l['subtotal'])) {
						$invoice_l['subtotal'] = $invoice_l['total'];
					}

					if (!$allow_multiline_line_desc) {
						$detail = str_replace(array("\r\n", "\r", "\n"), " ", $invoice_l['detail']);
					} else {
						$detail = $invoice_l['detail'];
					}

					if ($invoice_l['subtotal'] > 0 && $invoice_l['unit_value'] == 0) {
						if ($invoice_l['amount'] == 0) $invoice_l['amount'] = 1;
						$invoice_l['unit_value'] = $invoice_l['subtotal'] / $invoice_l['amount'];
					}

					$invoice_line = new IncomeInvoiceLine();
					$invoice_line->setObjectId($invoice->getId());
					$invoice_line->setOrder($invoice_l['order']);
					$invoice_line->setAmount($invoice_l['amount']);
					$invoice_line->setUnitValue($invoice_l['unit_value']);
					$invoice_line->setDetail($detail);
					$invoice_line->setTotal($invoice_l['total']);
					$invoice_line->setSubTotal($invoice_l['subtotal']);
					
					
					$invoice_line->setUserId($invoice_l['user_id']);
					if ( isset($invoice_l['rel_object_id']) && is_numeric($invoice_l['rel_object_id'])) $invoice_line->setRelObjectId($invoice_l['rel_object_id']);
					$invoice_line->setDate(getDateValue($invoice_l['date']));

					$category_member_id = 0; // labor or expense category member id
					if ($invoice_l['category_id']) {
						if (is_numeric($invoice_l['category_id'])) {
							$category_member_id = $invoice_l['category_id'];
						} else {
							$cat_decoded = json_decode($invoice_l['category_id'], true);
							if (isset($cat_decoded) && count($cat_decoded) > 0) {
								$category_member_id = $cat_decoded[0];
							}
						}
					}
					$invoice_line->setCategoryId($category_member_id);
					
					$line_main_member_id = 0; // project_id
					if ($invoice_l['member_id']) {
						if (is_numeric($invoice_l['member_id'])) {
							$line_main_member_id = $invoice_l['member_id'];
						} else {
							$mem_decoded = json_decode($invoice_l['member_id'], true);
							if (isset($mem_decoded) && count($mem_decoded) > 0) {
								$line_main_member_id = $mem_decoded[0];
							}
						}
					}
					$invoice_line->setMemberId($line_main_member_id);
					$invoice_line->setFromObjectTypeId(array_var($invoice_l, 'from_object_type_id'));
					$invoice_line->setProductTypeId(array_var($invoice_l, 'product_type_id'));
					$invoice_line->setUnitCost(array_var($invoice_l, 'unit_cost'));
					$invoice_line->setPriceMultiplier(array_var($invoice_l, 'price_multiplier'));
					$invoice_line->setItemTypeId(array_var($invoice_l, 'item_type_id'));
					$invoice_line->setDiscountPercentValue(array_var($invoice_l, 'discount_percent_value'));
					$invoice_line->setDiscountPercentType(array_var($invoice_l, 'discount_percent_type'));
					
					// only set this attribute if it is present, to prevent erasing previous value when not present in the request
					if (isset($invoice_l['group_by_category_id'])) {
						$invoice_line->setGroupByCategoryId(array_var($invoice_l, 'group_by_category_id'));
					}
					
					$invoice_line->setTaxes($invoice_l['total'] - $invoice_l['subtotal']);
					
					
					$exchange_rate = 1;
					if (Plugins::instance()->isActivePlugin('multiple_currencies')) {

						$invoice_line->setColumnValue('currency_id', $invoice_l['currency_id']);

						if (isset($invoice_l['exchange_rate'])) {
							$exchange_rate = $invoice_l['exchange_rate'];
						} else if ($invoice_l['currency_id'] != $invoice_data['currency_id']) {
							$exchange_rate = ExchangeRates::getLastExchangeRate($invoice_l['currency_id'], $invoice_data['currency_id']);
						}

						$invoice_line->setColumnValue('exchange_rate', $exchange_rate);
					}

					$invoice_line->save();

					// if invoice doesn't have a currency assigned yet, then use current line currency
					if (!$invoice->getCurrencyId()) {
						$invoice->setColumnValue('currency_id', $invoice_line->getCurrencyId());
					}
					
					// *** LC 2023-10-25
					if(array_var($invoice_l, 'item_type_id')<>7)//7 is retainer ando i must not be calculated in total
					{
       					$subtotal += $invoice_l['subtotal']; //* $exchange_rate;
		    			$lines_total += $invoice_l['total'];
					}
					
					if ($use_taxes_in_lines) {
						foreach ($invoice_l['taxes'] as $tax) {
							if ($tax['value'] > 0) {
								$line_tax = new IncomeInvoiceLineTax();
								$line_tax->setAppliedValue($tax['applied_value']);
								$line_tax->setValue($tax['value']);
								$line_tax->setInvoiceLineId($invoice_line->getId());
								$line_tax->setTaxId($tax['id']);
								$line_tax->save();
								
								$amounts_per_tax_id[$tax['id']] += $tax['value'];
							}
						}
					}
				}
			}
			$invoice->setSubtotal($subtotal);

			$invoice_taxes = IncomeInvoiceTaxes::getInvoiceTaxes($invoice->getObjectId());
			if (is_array($invoice_taxes)) {
				foreach ($invoice_taxes as $inv_tax) {
					$inv_tax->delete();
				}
			}

			if ($use_taxes_in_lines) {
				$invoice_taxes_data = $amounts_per_tax_id;
			} else {
				$invoice_taxes_data = array_var($_POST, 'taxes');
			}
			if (is_array($invoice_taxes_data)) {
				foreach ($invoice_taxes_data as $key => $invoice_t) {
					if (!$invoice_t) continue;
					$tax_data = IncomeTaxRecords::LastByTaxIdAndDate($key, $invoice_data['date']);
					if (!$tax_data) {
						$tax_data = IncomeTaxRecords::LastByTaxId($key);
					}
					$invoice_tax = new IncomeInvoiceTax();
					$invoice_tax->setInvoiceId($invoice->getId());
					$invoice_tax->setTaxId($tax_data->getTaxId());
					$invoice_tax->setType($tax_data->getType());
					$invoice_tax->setValue($tax_data->getValue());
					$invoice_tax->save();
				}
			}

			$invoice_copies = IncomeInvoiceCopies::getInvoiceCopies($invoice->getObjectId());
			if (is_array($invoice_copies)) {
				foreach ($invoice_copies as $inv_copy) {
					$inv_copy->delete();
				}
			}

			$invoice_copies_data = array_var($_POST, 'copies');
			if (is_array($invoice_copies_data)) {
				foreach ($invoice_copies_data as $copy_id => $invoice_c) {
					if (!$invoice_c) continue;
					$invoice_copy = new IncomeInvoiceCopy();
					$invoice_copy->setInvoiceId($invoice->getId());
					$invoice_copy->setCopyId($copy_id);
					$invoice_copy->save();
				}
			}
			
			if ($use_taxes_in_lines) {
				$total = $lines_total;
				foreach ($invoice->getTaxes() as $tax) {
					$total_tax = $amounts_per_tax_id[$tax->getTaxId()];
					$tax->setFixedValue($total_tax);
					$tax->save();
				}
			} else {
				$total = $subtotal;
				foreach ($invoice->getTaxes() as $tax) {
					if ($tax->getType() == "percentage") {
						$total_tax = ($subtotal * $tax->getValue()) / 100;
					} elseif ($tax->getType() == "fixed") {
						$total_tax = $tax->getValue();
					}
					$tax->setFixedValue($total_tax);
					$tax->save();
					$total = $total + $total_tax;
				}
			}
			$invoice->setTotal($total);

			$inv_subtype = ObjectSubtypes::instance()->findById($inv_subtype_id);

			if (strtolower($inv_subtype->getCode()) == 'invoice') {
				// set due amount for invoices
				$invoice->setDueAmount($total - $invoice->getPaidAmount());
			} else {
				// credit and debit notes don't have due amounts
				$invoice->setDueAmount(0);
			}

			//Invoice References
			$invoice_references_data = array_var($_POST, 'invoice_references');
			if (isset($invoice_references_data) && is_array($invoice_references_data)) {

				foreach ($invoice_references_data as $invoice_reference_data) {
					$ref_obj = null;

					if (!$invoice_reference_data['deleted']) {

						$invoice_reference = null;
						if (array_var($invoice_reference_data, 'id') > 0) {
							$invoice_reference = IncomeInvoiceReferences::instance()->findById(array_var($invoice_reference_data, 'id'));
						}
						if (!$invoice_reference instanceof IncomeInvoiceReference) {
							$invoice_reference = new IncomeInvoiceReference();
						}

						
						$invoice_reference->setObjectId($invoice->getId());
						$invoice_reference->setReferencedObjectId($invoice_reference_data['referenced_object_id']);
						$invoice_reference->setDetail($invoice_reference_data['detail']);

						$invoice_reference_data['amount'] = clean_formatted_money_amount_for_sql($invoice_reference_data['amount']);
						$invoice_reference->setAmount($invoice_reference_data['amount']);

						$invoice_reference->save();

						$ref_obj = $invoice_reference->getReferencedObject();

						if ($invoice_reference->getReferencedObjectId() > 0) {
							$_POST['linked_objects'][] = $invoice_reference->getReferencedObjectId();
						}
					} else {
						if (array_var($invoice_reference_data, 'id') > 0) {
							$invoice_reference = IncomeInvoiceReferences::instance()->findById(array_var($invoice_reference_data, 'id'));
						}
						if ($invoice_reference instanceof IncomeInvoiceReference) {
							$ref_obj = $invoice_reference->getReferencedObject();
							$invoice_reference->delete();
						}
					}

					// for each relation that was added/mofified/deleted recalculate the related invoice paid and due amounts
					if ($ref_obj instanceof IncomeInvoice) {
						// calculate paid and due amounts of affected invoice
						$ref_obj->recalcualtePaidAndDueAmounts();
					}
				}
			}

            if($invoice_template instanceof InvoiceTemplate)
			{
			    if($invoice_template->getTemplateType()=='fixed fee tasks' && $invoice->getTotal()<=0)
	    	    {
				    throw new Exception(lang('there are not tasks to invoice'));
		        }
		    }

			$invoice->save();
			
			// assign required cps for invoices
			$invoice_subtype = ObjectSubtypes::instance()->findOne(array(
					'conditions' => array("object_type_id=? AND `name`='invoice'", IncomeInvoices::instance()->getObjectTypeId()),
			));
			if ($invoice_subtype instanceof ObjectSubtype && $invoice_subtype->getId() == $invoice_data['object_subtype_id']) {
				
				$payment_period_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'invoice_payment_period');
				if ($payment_period_cp instanceof CustomProperty) {
					$cp_values = array_var($_POST, 'object_custom_properties');
					if (!array_var($cp_values, $payment_period_cp->getId())) {
						$cp_val = trim($payment_period_cp->getDefaultValue());
						if ($cp_val == '') $cp_val = 'Due on receipt';
						$_POST['object_custom_properties'][$payment_period_cp->getId()] = $cp_val;
					}
				}

				/**
				 * HARDCODED FOR KJOLHAUG, SET DUE DATE DEPENDING ON THE VALUE OF PAYMENT TERMS
				 */
				if (!$invoice->getExpirationDate() || $invoice->getExpirationDate()->getTimestamp() < 0) {
					$pt_val = trim($_POST['object_custom_properties'][$payment_period_cp->getId()]);
					
					$due_date = self::getExpirationDateFromPaymentTerm($invoice->getDate(), $pt_val);
					
					$invoice->setExpirationDate($due_date);
					$invoice->save();
				}
				/**
				 * END HARDCODED FOR KJOLHAUG
				 */
			}
			
			// assign other custom properties if they are in the $invoice_data by code
			foreach ($invoice_data as $key => $value) {
				if (!IncomeInvoices::instance()->columnExists($key)) {
					$cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), $key);
					if ($cp instanceof CustomProperty) {
						$_POST['object_custom_properties'][$cp->getId()] = $value;
					}
				}
			}
			

			$object_controller = new ObjectController();
			if (!is_null($member_ids)) {
				$object_controller->add_to_members($invoice, $member_ids);
			}
			 $object_controller->link_to_new_object($invoice);
			 $object_controller->add_subscribers($invoice);
			 $object_controller->add_custom_properties($invoice);


			$invoice->resetIsRead();

			// regenerate invoice preview if it was already generated
			if ($invoice->getColumnValue('invoice_preview_file_id') > 0) {
				$invoice_controller = new InvoiceController();
		        $file = $invoice_controller->generate_and_upload_invoice_preview($invoice);
			}
			
			$ignored = null;
			Hook::fire('after_invoice_save', array('invoice' => $invoice), $ignored);


		} catch (Exception $e) {
			//DB::rollback();
			Logger::log_r($e->getMessage());
			Logger::log_r($e->getTraceAsString());
			throw $e;
		}

		return $invoice;
		
	}

	static function getExpirationDateFromPaymentTerm($invoice_date, $pt_val) {

		$due_date = new DateTimeValue($invoice_date->getTimestamp());

		switch ($pt_val) {
			case '1% 30 Days':
				$due_date->add('d',30);
				break;
			case 'net 45 term':
				$due_date->add('d',45);
				break;
			default: 
				// this will cover each payment term called "Net X" where X is the amount of days like Net 30, Net 60, etc.
				if (str_starts_with(strtolower($pt_val), 'net ')) {
					$exploded = explode(' ', $pt_val);
					$days = (int)end($exploded);
					if (is_numeric($days) && $days > 0) {
						$due_date->add('d', $days);
					}
				}
			break;
		}
		return $due_date;
	}

	static function parseRepeatOptions(&$invoice_data)
    {
        // 1=repeat once, 2=repeat daily, 3=weekly, 4=monthy, 5=yearly, 6=holiday repeating
        $rend = null;
        $rnum = 0;
        $forever = array_var($invoice_data, 'repeat_option') == 1;

        $jump = array_var($invoice_data, 'occurance_jump');
        if (array_var($invoice_data, 'repeat_option') == 2) $rnum = array_var($invoice_data, 'repeat_num');
        if (array_var($invoice_data, 'repeat_option') == 3) $rend = getDateValue(array_var($invoice_data, 'repeat_end'));

        $repeat_d = 0;
        $repeat_m = 0;
        $repeat_y = 0;
        $repeat_h = 0;
        $oend = ($forever == 1) ? null : $rend;

        switch (array_var($invoice_data, 'occurance')) {
            case "1":
                $oend = null;
                break;
            case "2":
                $repeat_d = $jump;
                break;
            case "3":
                $repeat_d = 7 * $jump;
                break;
            case "4":
                $repeat_m = $jump;
                break;
            case "5":
                $repeat_y = $jump;
                break;
            case "6":
                $repeat_h = 1;
                $forever = 1;
                $invoice_data['repeat_num'] = 0;
                break;
        }

        $invoice_data['repeat_end'] = $rend;
        $invoice_data['repeat_forever'] = $forever;
        $invoice_data['repeat_d'] = $repeat_d;
        $invoice_data['repeat_m'] = $repeat_m;
        $invoice_data['repeat_y'] = $repeat_y;
        $invoice_data['repeat_h'] = $repeat_h;
        if ($repeat_h == 0) {
            $invoice_data['repeat_dow'] = 0;
            $invoice_data['repeat_wnum'] = 0;
            $invoice_data['repeat_mjump'] = 0;
        }
    }


	static function get_invoices_by_date_range($type = null)
	{
		$range_dates = IncomeInvoices::get_start_and_end_date_range($type);
		$start_date = $range_dates['start_date'];
		$end_date = $range_dates['end_date'];

        $invoices = IncomeInvoices::instance()->listing(array(
            "order" => "date",
            "order_dir" => "DESC",
			"limit" => 11, // get 11 to see if there are more thant 10 so we decied if we put the "view all" button or not
        //    "extra_conditions" => " AND `date` >= '$start_date' AND `date` <= '$end_date'"
        ))->objects;

		return $invoices;
	}

	static function get_start_and_end_date_range($rangeType, $apply_timezone = false, $format = 'Y-m-d') {
		$now = DateTimeValueLib::now(); // Current date and time
		
		switch ($rangeType) {
			case 'ytd':
				$start = DateTimeValueLib::makeFromString($now->getYear() . '-01-01');
				$end = $now;
				break;
			
			case 'this-month':
				$start = DateTimeValueLib::makeFromString($now->getYear() . '-' . $now->getMonth() . '-01');
				$end = DateTimeValueLib::makeFromString($now->format('Y-m-t')); // Last day of this month
				break;
			
			case 'last-month':
				$start = DateTimeValueLib::makeFromString('first day of last month');
				$end = DateTimeValueLib::makeFromString('last day of last month');
				break;
			
			case 'this-quarter':
				$month = $now->getMonth();
				$quarterStartMonth = ((int)(($month - 1) / 3) * 3) + 1; // Calculate start month of the current quarter
				$start = DateTimeValueLib::makeFromString($now->getYear() . '-' . $quarterStartMonth . '-01');
				$end_dtv = DateTimeValueLib::makeFromString($start->format('Y-m-d'));
				$end = DateTimeValueLib::makeFromString($end_dtv->add('M', 2)->format('Y-m-t')); // End of the quarter
				break;
			
			case 'last-quarter':
				$month = $now->getMonth();
				$quarterStartMonth = ((int)(($month - 1) / 3) * 3) + 1;
				$startMonthOfPrevQuarter = $quarterStartMonth - 3;
				if ($startMonthOfPrevQuarter < 1) {
					$startMonthOfPrevQuarter += 12;
					$year = $now->getYear() - 1;
				} else {
					$year = $now->getYear();
				}
				$start = DateTimeValueLib::makeFromString($year . '-' . $startMonthOfPrevQuarter . '-01');
				$end_dtv = DateTimeValueLib::makeFromString($start->format('Y-m-d'));
				$end = DateTimeValueLib::makeFromString($end_dtv->add('M', 2)->format('Y-m-t')); // End of the previous quarter
				break;
			
			default:
				$start = DateTimeValueLib::makeFromString($now->getYear() . '-01-01');
				$end = $now;
				break;
		}
	
		if ($apply_timezone) {
			if (isset($start) && $start instanceof DateTimeValue) {
				$start->advance(logged_user()->getUserTimezoneValue() * -1, true);
			}
			if (isset($end) && $end instanceof DateTimeValue) {
				$end->advance(logged_user()->getUserTimezoneValue() * -1, true);
			}		
		}
	
		$start_formatted = $start->format($format);
		$end_formatted = $end->format($format);
		
		return array('start_date' => $start_formatted, 'end_date' => $end_formatted);
	}
}

?>