<?php
if (isset($invoice) && $invoice instanceof IncomeInvoice) {
	
	function canPreviewIvoice($invoice){
		if ($invoice->getStatus() != 'canceled') {
			if (is_invoice_status_available("printed")) {
				if ($invoice->getStatus() == 'pending') {
					add_page_action(lang('print invoice'), "javascript:og.render_modal_form('', {c:'invoice', a:'print_parameters', params: {id:".$invoice->getId()."}});", " ico-print");
				} else {
					add_page_action(lang('print invoice'), $invoice->getPrintViewUrl(), " ico-print", "_blank");
				}
			}
			
			add_page_action(lang('generate invoice preview'), $invoice->getGeneratePreviewUrl(), " ico-print", "_blank");
		}
	}
	
	$user_can_edit_invoice = $invoice->canEdit(logged_user());
		
	if(!$invoice->isTrashed() && $user_can_edit_invoice) {
		
		if (income_config_option("allow_invoice_flexible_status_change") && $invoice->getStatus() != 'pending') {
			add_page_action(lang('edit'), "javascript:og.render_modal_form('', {c:'invoice', a:'edit', params: {id:".$invoice->getId().", req_channel:'invoice view - edit'}});", 'ico-edit', null, null, true);
		}
		
		if ($invoice->getStatus() == 'pending') {

			add_page_action(lang('edit'), "javascript:og.render_modal_form('', {c:'invoice', a:'edit', params: {id:".$invoice->getId().", req_channel:'invoice view - edit'}});", 'ico-edit', null, null, true);

			if (logged_user()->isAdminGroup()) {
				add_page_action(lang('confirm invoice'), $invoice->getConfirmUrl('invoice view - mark as paid'), 'ico-complete', null, null, true);
			}
			
			add_page_action(lang('cancel invoice'), $invoice->getCancelUrl('invoice view - mark as void'), 'ico-delete', null, null, true);

		} else if ($invoice->getStatus() == 'confirmed' && can_edit_confirmed_invoices(logged_user())) {

			add_page_action(lang('reopen invoice'), $invoice->getReopenUrl('invoice view - reopen'), 'ico-reopen', null, null, true);
			add_page_action(lang('cancel invoice'), $invoice->getCancelUrl('invoice view - void'), 'ico-delete', null, null, true);

		} else if ($invoice->getStatus() == 'canceled' && can_edit_confirmed_invoices(logged_user())) {

			//add_page_action(lang('reopen invoice'), $invoice->getReopenUrl(), 'ico-reopen', null, null, true);

		} else if ($invoice->getStatus() == 'printed') {

			if (logged_user()->isAdminGroup()) {
				add_page_action(lang('confirm invoice'), $invoice->getConfirmUrl('invoice view - mark as paid'), 'ico-complete', null, null, true);
			}
			add_page_action(lang('cancel invoice'), $invoice->getCancelUrl('invoice view - void'), 'ico-delete', null, null, true);
		}

		$next_invoice = null;
		if($invoice->canGenerateNextInvoice(logged_user()) && $invoice->isRepetitive()) {
			$next_invoice = $invoice->getNextInvoiceInRepetition();
			if (!$next_invoice instanceof IncomeInvoice) {
				add_page_action(lang('generate next invoice'), get_url('invoice', 'generate_next_repetition', array('id' => $invoice->getId(), 'req_channel' => 'invoice view - generate next repetition')), 'ico-recurrent', null, null, true);
			}
		}
		
		canPreviewIvoice($invoice);
		
		$is_mail_active = Plugins::instance()->isActivePlugin('mail');
		$send_invoice_url = $is_mail_active ? $invoice->getSendInvoiceUrl() : "#";
		add_page_action(lang('send invoice'), $send_invoice_url, "ico-sent" . ($is_mail_active ? "" : " disabled"));
		
		
		if (!$invoice->isArchived()) {
			add_page_action(lang('archive'), "javascript:if(confirm(lang('confirm archive object'))) og.openLink('" . $invoice->getArchiveUrl() ."');", 'ico-archive-obj');
		} else {
			add_page_action(lang('unarchive'), "javascript:if(confirm(lang('confirm unarchive object'))) og.openLink('" . $invoice->getUnarchiveUrl() ."');", 'ico-unarchive-obj');
		}
		
	}else{
		canPreviewIvoice($invoice);
	}
	
	if ($invoice->canDelete(logged_user())) {
		if ($invoice->isTrashed()) {
			//add_page_action(lang('restore from trash'), "javascript:if(confirm(lang('confirm restore objects'))) og.openLink('" . $invoice->getUntrashUrl() ."');", 'ico-restore', null, null, true);
			add_page_action(lang('delete permanently'), "javascript:if(confirm(lang('confirm delete permanently'))) og.openLink('" . $invoice->getDeletePermanentlyUrl() ."');", 'ico-delete', null, null, true);
		} else {
			add_page_action(lang('move to trash'), "javascript:og.income.trash_invoice('" . $invoice->getObjectId() ."');", 'ico-trash', null, null, true);
		}
	}


	$related_invoices = array();
	if($invoice->isRepetitive()) {
		$next_invoice = $invoice->getNextInvoiceInRepetition();
		$prev_invoice = $invoice->getPreviousInvoiceInRepetition();

		if ($prev_invoice instanceof IncomeInvoice) {
			$related_invoices[] = array(
					'label' => lang("previous invoice"),
					'object' => $prev_invoice,
			);
		}
		if ($next_invoice instanceof IncomeInvoice) {
			$related_invoices[] = array(
				'label' => lang("next invoice"),
				'object' => $next_invoice,
			);
		}
	}
	tpl_assign("related_invoices", $related_invoices);

?>

<div style="padding:7px">
	<a id="printinvoicelink_<?php echo $invoice->getId()?>" target="_blank" href="#" onclick="javascript:og.openLink('<?php echo get_url('invoice', 'print_view', array('id' => $invoice->getId()))?>');" style="display:none"></a>
    <div class="invoice"><?php
		tpl_assign("content_template", array('invoice_list', 'invoice'));
		tpl_assign("object", $invoice);
		tpl_assign('iconclass', $invoice->isTrashed()? 'ico-large-invoice-trashed' : ($invoice->isArchived() ? 'ico-large-invoice-archived' : 'ico-large-invoice'));
		$this->includeTemplate(get_template_path('view', 'co'));
	?></div>
</div>
<?php } ?>
