<?php
Hook::register('income');
Env::useHelper('functions','crpm');

function income_object_definition($type, &$ret) {
	if ($type == 'SystemPermission' || $type == 'MaxSystemPermission') {
		$ret['can_edit_confirmed_invoices'] = DATA_TYPE_BOOLEAN;
		$ret['can_manage_invoicing_configuration'] = DATA_TYPE_BOOLEAN;
	}
	
	if ($type == 'Timeslot') {
		$ret['invoicing_status'] = DATA_TYPE_STRING;
		$ret['invoice_id'] = DATA_TYPE_INTEGER;
	}

	
	// DEFAULT BILLING EMAIL AND ADDRESS :::::::::::::::::::::::::::
	if ($type == 'ContactEmail') $ret['default_billing_email'] = DATA_TYPE_BOOLEAN;
	if ($type == 'ContactAddress') $ret['default_billing_address'] = DATA_TYPE_BOOLEAN;
	// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

	
	if (Plugins::instance()->isActivePlugin('expenses2')) {
		if ($type == 'PaymentReceipt') {
			$ret['invoicing_status'] = DATA_TYPE_STRING;
			$ret['invoice_id'] = DATA_TYPE_INTEGER;
		}
		if ($type == 'Expense') {
			$ret['invoicing_status'] = DATA_TYPE_STRING;
		}
	}
}

function income_before_object_save($object, &$ret) {
	if ($object instanceof Timeslot) {

		$timeslot_data = array_var($_POST, 'timeslot');
		$current_status = $object->getColumnValue('invoicing_status');

		if ($current_status == '' && array_var($timeslot_data, 'invoicing_status', '') == '') {
			$object->setColumnValue('invoicing_status', 'pending');
		}

	}
	
	if (Plugins::instance()->isActivePlugin('expenses2') && $object instanceof PaymentReceipt) {
		$current_status = $object->getColumnValue('invoicing_status');
		if ($current_status == '') {
			$object->changeInvoicingStatus('pending');
		}
	}
}

function income_after_object_save($object, &$ret) {
	if ($object instanceof IncomeInvoice && !$object->dont_update_totals) {
		Env::useHelper('persist_member_invoice_totals', 'income');
		update_invoice_total_info_for_member($object);
	}
}

// when trashed from invoice view
function income_after_object_trash($invoice, &$ignored) {
	if ($invoice instanceof IncomeInvoice && Plugins::instance()->isActivePlugin('expenses2')) {
		$payment_receipts = PaymentReceipts::instance()->findAll(array("conditions"=>"invoice_id=".$invoice->getId()));
		foreach ($payment_receipts as $p) {
			$p->changeInvoicingStatus('pending');
		}
	}
	if ($invoice instanceof IncomeInvoice) {
		Env::useHelper('persist_member_invoice_totals', 'income');
		update_invoice_total_info_for_member($invoice);
	}
}

// when trashing from invoices list
function income_after_content_object_trash($params, &$ignored) {
	$invoice = array_var($params, 'object');
	if ($invoice instanceof IncomeInvoice && Plugins::instance()->isActivePlugin('expenses2')) {
		$payment_receipts = PaymentReceipts::instance()->findAll(array("conditions"=>"invoice_id=".$invoice->getId()));
		foreach ($payment_receipts as $p) {
			$p->changeInvoicingStatus('pending');
		}
	}
	if ($invoice instanceof IncomeInvoice) {
		Env::useHelper('persist_member_invoice_totals', 'income');
		update_invoice_total_info_for_member($invoice);
	}
}

function income_after_content_object_untrash($params, &$ignored) {
	$invoice = array_var($params, 'object');
	if ($invoice instanceof IncomeInvoice && Plugins::instance()->isActivePlugin('expenses2')) {
		$payment_receipts = PaymentReceipts::instance()->findAll(array("conditions"=>"invoice_id=".$invoice->getId()));
		foreach ($payment_receipts as $p) {
			$p->changeInvoicingStatus('invoiced');
		}
	}
	if ($invoice instanceof IncomeInvoice) {
		Env::useHelper('persist_member_invoice_totals', 'income');
		update_invoice_total_info_for_member($invoice);
	}
}


function income_modify_object_get_array_info($params, &$info) {
	$object = array_var($params, 'object');
	if (Plugins::instance()->isActivePlugin('expenses2') && $object instanceof Expense) {
		$info['invoicing_status'] = $object->getColumnValue('invoicing_status');
	}
}

function income_custom_reports_get_possible_external_column_values($parameters, &$values) {
	$ot_id = array_var($parameters, 'ot_id');
	$field = array_var($parameters, 'field');
	$ot = ObjectTypes::instance()->findById($ot_id);
	if ($ot->getName() == 'timeslot' || $ot->getName() == 'payment_receipt') {
		if ($field == 'invoicing_status') {
			$values[] = array('id' => 'all', 'name' => lang('All'));
			$values[] = array('id' => 'pending', 'name' => lang('invoicing_status pending'));
			$values[] = array('id' => 'non_billable', 'name' => lang('invoicing_status non_billable'));
			$values[] = array('id' => 'invoiced', 'name' => lang('invoicing_status invoiced'));
		}
	}
}

function income_after_remove_members_from_object($object, $removedMemebersIds){
    if (Plugins::instance()->isActivePlugin('advanced_billing') && $object instanceof IncomeInvoice) {
		Env::useHelper('persist_member_invoice_totals', 'income');
        $project_ot_id = ObjectTypes::findByName('project')->getId();

        foreach($removedMemebersIds as $memberId){
            $member = Members::instance()->findById($memberId);
            if($member->getObjectTypeId() == $project_ot_id){
				update_invoice_total_info_for_member_by_using_member_id($memberId);
            }
        }
    }
}



function income_after_add_mail_content($params, &$ret) {
	$mail_data = array_var($params, 'mail_data');
	$mail = array_var($params, 'mail');
	
	if ($mail instanceof MailContent) {
		
		$mail_data_additional_info = array_var($mail_data, 'additional_info');
		if (is_array($mail_data_additional_info)) {
			
			$invoice_id = array_var($mail_data_additional_info, 'invoice_id');
			if (is_numeric($invoice_id) && $invoice_id > 0) {
				
				$invoice = IncomeInvoices::instance()->findById($invoice_id);
				if ($invoice instanceof IncomeInvoice) {
					$invoice->setColumnValue('sent_invoice_mail_id', $mail->getId());
					$sent_status_value = '';
					switch ($invoice->getStatus()) {
						case 'pending': $sent_status_value = 'invoice_sent'; break;
						case 'confirmed': $sent_status_value = 'receipt_sent'; break;
						case 'canceled': $sent_status_value = 'void_sent'; break;
						default: break;
					}
					$invoice->setSent($sent_status_value);
					$invoice->save();
					// link the invoice to the mail
					$invoice->linkObject($mail);
				}
			}
		}
	}
}


function income_additional_task_list_filter_conditions($params, &$result) {
	$filter = array_var($params, 'filter');
	$filter_value = array_var($params, 'filter_value');
	
	if (Plugins::instance()->isActivePlugin('advanced_billing') && $filter == 'invoicing_status') {
		$cond_str = "";
		if (in_array($filter_value, array('pending', 'invoiced', 'partially_invoiced', 'non_billable'))) {
			if ($filter_value == 'partially_invoiced') {
				$cond_str = " AND EXISTS (SELECT t.object_id FROM ".TABLE_PREFIX."timeslots t WHERE t.rel_object_id=e.object_id AND invoicing_status='pending')";
				$cond_str .= " AND EXISTS (SELECT t.object_id FROM ".TABLE_PREFIX."timeslots t WHERE t.rel_object_id=e.object_id AND invoicing_status='invoiced')";
			} else {
				$cond_str = " AND EXISTS (SELECT t.object_id FROM ".TABLE_PREFIX."timeslots t WHERE t.rel_object_id=e.object_id AND invoicing_status='$filter_value')";
			}
		}
		
		$result = array('conditions' => array($cond_str));
	}
}


function income_timeslot_info_additional_data($timeslot, &$info) {
	if (Plugins::instance()->isActivePlugin('advanced_billing') && $timeslot instanceof Timeslot) {
		// invoicing status
		$status_html = '';
		$status = $timeslot->getColumnValue('invoicing_status');
		if ($status != "") {
			$status_html .= $status == "" ? "" : ('<span class="invoicing-status-'.$status.'">' . lang("invoicing_status $status") . '</span>');		
		}

		$info['additional_data']['invoicing_status_html'] = array("html" => $status_html, "width" => "130px");
		$info['invoicing_status'] = $status;
		
		// Billable additional data
		$status_html = '';
		$status = $timeslot->getColumnValue('invoicing_status');
		if ($status != "") {
			$status_html .= $status == "non_billable" ? lang('no') : lang('yes');
		}
		$info['additional_data']['is_billable'] = array("html" => $status_html, "width" => "130px");
	}
}


function income_task_info_additional_data($task, &$info) {
	if (Plugins::instance()->isActivePlugin('advanced_billing')) {

		Env::useHelper('functions', 'income');
		$status = get_task_billable_status_to_show($task);

		if (!isset($info['additional_data'])) $info['additional_data'] = array();
		
		if (user_config_option('tasksShowInvoicingStatus')) {
			$status_html = '<td class="additional-list-col">';
			$status_html .= $status == "" ? "" : ('<div class="invoicing-status-'.$status.'">' . lang("invoicing_status $status") . '</div>');
			$status_html .= '</td>';

			$info['additional_data']['tasksShowInvoicingStatus'] = array(
				"html" => $status_html,
				"cls" => "invoicing-status-$status",
			);
		} else {
			$info['additional_data']['tasksShowInvoicingStatus'] = array("html" => "");
		}
	}
}

function income_view_task_render_more_properties($task, &$null) {
	if (Plugins::instance()->isActivePlugin('advanced_billing')) {

		Env::useHelper('functions', 'income');
		$status = get_task_billable_status_to_show($task);
		
		$status_html = '<b>' . lang('invoicing status') . ':</b>&nbsp;<span class="additional-list-col">';
		$status_html .= $status == "" ? "" : ('<span class="invoicing-status-'.$status.'">' . lang("invoicing_status $status") . '</span>');
		$status_html .= '</span>';
		
		echo $status_html;
	}
}

function income_view_timeslot_render_more_columns($timeslot, &$additional_columns) {
	if (Plugins::instance()->isActivePlugin('advanced_billing') && $timeslot instanceof Timeslot) {
		// Invoicing status
		$status = $timeslot->getColumnValue('invoicing_status');
		$status_html = '<span class="additional-list-col" style="width:150px;vertical-align: middle;">';
		$status_html .= $status == "" ? "" : ('<span class="invoicing-status-'.$status.'">' . lang("invoicing_status $status") . '</span>');
		if ($status == 'invoiced') {
			$inv = Incomeinvoices::instance()->findById($timeslot->getColumnValue('invoice_id'));
			if ($inv instanceof IncomeInvoice) {
				$status_html .= '<a href="'.$inv->getViewUrl().'" class="underline" style="margin-left:5px;" target="_blank">'.lang('view invoice').'</a>';
			}
		}
		$status_html .= '</span>';
		
		if (!is_array($additional_columns)) $additional_columns = array();
		$additional_columns['invoicing_status_html'] = $status_html;

		// Billable column
		$status = $timeslot->getColumnValue('invoicing_status');
		$status_html = '<span class="additional-list-col">';
		$status_html .= $status == "non_billable" ? lang('no') : lang('yes');
		$status_html .= '</span>';
		
		if (!is_array($additional_columns)) $additional_columns = array();
		$additional_columns['is_billable'] = $status_html;
	}
}

function income_timeslot_list_additional_actions($params, &$actions) {
	$pg_ids = implode(',', logged_user()->getPermissionGroupIds());
	$is_income_tab_enabled = TabPanelPermissions::isModuleEnabled('income-panel', $pg_ids);

	//suspended on 30-08-2023, its needs revision
	// if (Plugins::instance()->isActivePlugin('advanced_billing') && !Plugins::instance()->isActivePlugin('timesheet_approval') && $is_income_tab_enabled) {
	// 	$actions[] = array(
	// 		'url' => '#',
	//  		'onclick' => "og.income.generate_invoice_from_timeslots('".array_var($params, 'panel_id')."');",
	//  		'cls' => 'ico-invoice',
	//  		'text' => lang('generate invoice'),
	// 	);
	// }

	// tool to change invoicing status of selected timeslots
	if (logged_user()->isAdministrator()) {
		$mark_as_actions = array();
		
		if (logged_user()->getEmailAddress() == 'support@evxsoftware.com') {
			$mark_as_actions[] = array(
				'url' => '#',
				'onclick' => "og.income.mark_time_as_invoiced('".array_var($params, 'panel_id')."');",
				'cls' => 'ico-invoice',
				'text' => lang('mark as invoiced'),
			);
		}
		$mark_as_actions[] = array(
			'url' => '#',
			'onclick' => "og.income.mark_time_as_unbilled('".array_var($params, 'panel_id')."');",
			'cls' => 'ico-undo',
			'text' => lang('mark as unbilled'),
		);
		$mark_as_actions[] = array(
			'url' => '#',
			'onclick' => "og.income.mark_time_as_non_billable('".array_var($params, 'panel_id')."');",
			'cls' => 'ico-delete',
			'text' => lang('mark as non billable'),
		);
		
		
		// make a dropdown menu with the actions
		$action = array(
			'type' => 'menu',
			'text' => 'Mark as',
			'items' => $mark_as_actions,
		);
		$actions[] = $action;
		
	}
}


function income_timeslot_list_additional_columns($params, &$columns) {
	if (Plugins::instance()->isActivePlugin('advanced_billing')) {
		$columns[] = array('id' => 'invoicing_status_html', 'name' => lang('invoicing status'));
	}
}


function income_tasks_user_preferences($ignored, &$array){
	if (Plugins::instance()->isActivePlugin('advanced_billing')) {
		$array['tasksShowInvoicingStatus'] = (bool) user_config_option('tasksShowInvoicingStatus');
	}
}

function income_view_task_actions($task, &$ret) {
	$pg_ids = implode(',', logged_user()->getPermissionGroupIds());
	$is_income_tab_enabled = TabPanelPermissions::isModuleEnabled('income-panel', $pg_ids);
	
	if (Plugins::instance()->isActivePlugin('advanced_billing') && $is_income_tab_enabled) {
		if ($task instanceof ProjectTask) {
			
			$timeslots = $task->getTimeslots();
			$pending = false;
			foreach ($timeslots as $time) {
				$status = $time->getColumnValue('invoicing_status');
				if ($status == 'pending') {
					$pending = true;
					break;
				}
			}
			
			if ($pending) {
				$url = get_url("invoice", "generate_invoice_from_tasks", array("task_ids" => $task->getId(), "reload" => "1"));
				add_page_action(lang('generate invoice'), $url, 'ico-invoice', null, null, true);
			}
		}
	}
}

function income_override_view_history_log($params, &$output) {
	$object = array_var($params, 'object');
	$log = array_var($params, 'log');
	if ($object instanceof IncomeInvoice && $log instanceof ApplicationLog && $log->getAction() == 'edit' && $log->getActionData() != '') {
		$data = explode(':', $log->getActionData());
		if (array_var($data, 0) == 'status') {
			$action = array_var($data, 1);
			if (in_array($action, array('canceled','pending','confirmed'))) {
				
				$object_link = '<a style="font-weight:bold" href="' . $object->getObjectUrl() . '">&nbsp;<span style="padding: 2px 0 3px 24px;" class="link-ico ico-invoice"/>'.clean($object->getObjectName()).'</a>';
				$output = lang('activity ' . $action, lang('the invoice'), clean($log->getTakenByDisplayName()), $object_link);
			}
		}
	}
}


function income_additional_external_columns($manager, &$columns) {
	if ($manager instanceof Timeslots) {
		$columns[] = 'invoice_id';
		$columns[] = 'invoicing_status';
	}
	if (Plugins::instance()->isActivePlugin('expenses2') && $manager instanceof PaymentReceipts) {
		$columns[] = 'invoice_id';
		$columns[] = 'invoicing_status';
	}
}

function income_custom_reports_get_external_column_value($parameters, &$value) {
	$external_id = array_var($parameters, 'external_id');
	$manager = array_var($parameters, 'manager');
	$field = array_var($parameters, 'field');
	//$object = array_var($parameters, 'object');

	if ($manager instanceof Timeslots || (Plugins::instance()->isActivePlugin('expenses2') && $manager instanceof PaymentReceipts)) {
		if ($field == 'invoice_id') {
			$inv = IncomeInvoices::instance()->findById($external_id);
			if ($inv instanceof IncomeInvoice) {
				$value = $inv->getObjectName();
			} else {
				$value = "";
			}
		} else if ($field == 'invoicing_status') {
			// ** LC 2023-09-04 ** Al menos en el caso billing reports de bca esto producía una dobre traducción, se vuelve a versión 3.10.1.0-rc3
/* 			if ($external_id != '') {
				$value = lang($external_id);
			}
 */	
			$value = $external_id;	
		}
	}
}


function income_additional_timeslots_tab_filters($request, &$extra_conditions) {

	$status_filter_value = array_var($request, 'invoicing_status_filter', '');
	if ($status_filter_value != '' && in_array($status_filter_value, array('pending','non_billable','invoiced'))) {
		$extra_conditions .= " AND `invoicing_status` = '$status_filter_value'";
	}

}

function income_additional_timeslots_tab_filters_config($request, &$current_time_module_filters) {
	
	$status_filter_value = array_var($request, 'invoicing_status_filter', '');
	$current_time_module_filters['invoicing_status_filter'] = $status_filter_value;
	
}


function income_custom_reports_override_column_format($parameters, &$value) {
    $report = array_var($parameters, 'report');
    $manager = array_var($parameters, 'manager');
    $field = array_var($parameters, 'field');
    $object = array_var($parameters, 'object');
    if ($manager instanceof IncomeInvoices) {

        $addParenthesesFlag = false;
        $creditNote = ObjectSubtypes::instance()->findOne(array('conditions' => "code = 'credit_note'"));
        if ($creditNote->getColumnValue('id') == $object->getColumnValue('object_subtype_id')) {
            if ($field == 'subtotal' || $field == 'total' || substr($field, 0, 4) == 'tax_') {
                if ($report->getColumnValue('show_negatives')) {
                    if ($value > 0) {
                        $value = "-" . $value;
                    }
                } else {
                    $addParenthesesFlag =true;
                }
            }
        }
        if ($report->getColumnValue('show_currency')) {
        	if (in_array($field, array('subtotal','total','paid_amount','due_amount')) || substr($field, 0, 4) == 'tax_') {
                $currency = Currencies::instance()->getCurrency($object->getCurrencyId());
                $csymbol = null;
                if ($currency instanceof Currency) {
                    $csymbol = $currency->getSymbol();
                }
                $value = format_money_amount($value, $csymbol);
            }
        }
        if ($addParenthesesFlag) {
            $value = "(" . $value . ")";
        }
    } else if ($manager instanceof Timeslots) {
        if ($field == 'invoicing_status') {
            $value = lang('invoicing_status ' . $value);
        }
    }
}

function income_render_additional_billing_config_options($null, &$draw_disabled_options) {
	$draw_disabled_options = false;
	$use_invoice_notebooks = income_config_option('use_invoice_notebooks');
	tpl_assign('use_invoice_notebooks', $use_invoice_notebooks);
	echo tpl_fetch(get_template_path('administration', 'income', 'income'));
}


function income_timeslot_main_tab_additional_fields($params, &$categories) {
	$timeslot = array_var($params, 'timeslot');
	$task_id = array_var($params, 'task_id');
	$selected_member_ids = array_var($params, 'selected_member_ids');
	$genid = array_var($params, 'genid');
	if ($timeslot instanceof Timeslot) {
		tpl_assign('timeslot', $timeslot);
		tpl_assign('task_id', $task_id);
		tpl_assign('selected_member_ids', $selected_member_ids);
		tpl_assign('genid', $genid);
		echo tpl_fetch(get_template_path('edit_timestlot_status_tab', 'income', 'income'));
	}
}
function income_object_billing_adv_billing_additional_fields($object, &$categories) {
    tpl_assign('objectBilling', $object);
    echo tpl_fetch(get_template_path('edit_object_billing_status_tab', 'income', 'income'));
}

function income_include_tasks_template($ignored, &$more_content_templates) {
	$more_content_templates[] = array(
		'template' => 'tasks_toolbar_actions',
		'controller' => 'income',
		'plugin' => 'income'
	);
}

function income_allow_generic_type_object_subtypes($object, &$allow_generic_type) {
	if ($object instanceof IncomeInvoice) {
		$allow_generic_type = false;
	}
}

function income_allow_generic_type_object_subtypes_ot($ot, &$allow_generic_type) {
	if ($ot instanceof ObjectType &&  $ot->getName() == 'invoice') {
		$allow_generic_type = false;
	}
}

function income_render_page_actions($invoice, &$ret) {
	if ($invoice instanceof IncomeInvoice && !$invoice->isTrashed() && $invoice->canEdit(logged_user())){
		if($invoice->getStatus() == 'printed' || $invoice->getStatus() == 'confirmed' || $invoice->getStatus() == 'part_confirmed'){
			$subtype = ObjectSubtypes::instance()->findById($invoice->getColumnValue('object_subtype_id'));
			if($subtype instanceof ObjectSubtype && $subtype->getCode() == 'invoice'){
				$credit_note_type = ObjectSubtypes::instance()->findOne(array('conditions' => "code='credit_note'"));
				if ($credit_note_type instanceof ObjectSubtype) {
					add_page_action(lang('generate invoice type '.$credit_note_type->getCode()), "javascript:og.render_modal_form('', {c:'invoice', a:'add', params: {object_subtype_id:".$credit_note_type->getId().",invoice_reference_id : ".$invoice->getId()."}});", 'ico-income', null, null, true);
				}
				
				$debit_note_type = ObjectSubtypes::instance()->findOne(array('conditions' => "code='debit_note'"));
				if ($debit_note_type instanceof ObjectSubtype) {
					add_page_action(lang('generate invoice type '.$debit_note_type->getCode()), "javascript:og.render_modal_form('', {c:'invoice', a:'add', params: {object_subtype_id:".$debit_note_type->getId().",invoice_reference_id : ".$invoice->getId()."}});", 'ico-income', null, null, true);
				}
			}

		};
	}
}

// PERMISSION FUNCTIONS
function can_edit_confirmed_invoices($user) {
	return SystemPermissions::userHasSystemPermission($user, 'can_edit_confirmed_invoices');
}

function can_manage_invoicing_configuration($user) {
	return SystemPermissions::userHasSystemPermission($user, 'can_manage_invoicing_configuration');
}

/**
 * Add tax row to list in reports
 * @param array $params
 * @param array $ret
 */
function income_get_allow_taxes_row_for_reports($params, &$ret) {

    if (Plugins::instance()->isActivePlugin('income')) {

        $allowed_columns = $ret[0];
        $option_groups = $ret[1];

        $object_type = array_var($params, 'object_type');
        $columns = array_var($params, 'columns');

        if ($object_type == IncomeInvoices::instance()->getObjectTypeId()) {

            $ts_group_count = count($allowed_columns);
            $managerTaxesInstance = IncomeTaxes::instance();
            $columnCurr = $managerTaxesInstance->findAll();
            $columnsAux = explode(',', $columns);
            $conutDuplicated = 0;
            foreach ($columnCurr as $object) {
                $value = $object->getId();
                if (in_array('tax_' . $value, $columnsAux)) {
                    $conutDuplicated +=2;
                }
                $allowed_columns[] = array('id' => 'tax_' . $object->getName(), 'name' => $object->getName(), 'type' => $object->getName());
            }
            $addMoreColumns = count($columnsAux) - $conutDuplicated;
            $option_groups[] = array('count' => $ts_group_count - count($columnCurr) - $addMoreColumns, 'name' => lang('taxes'));
        }

        $ret[0] = $allowed_columns;
        $ret[1] = $option_groups;
    }
}

/**
 * 
 * @param array $params
 * @param array $results
 */
function income_get_taxes_columns_to_header_report($params, &$results) {

    $field = array_var($params, 'field');
    $ot = array_var($params, 'object_type');
    if ($ot && $ot->getName() == 'invoice' && str_starts_with($field, 'tax_')) {
        $results['columns']['names'][$field] = substr($field, 4);
        $results['columns']['order'][] = $field;
        $results['columns']['types'][$field] = DATA_TYPE_INTEGER;
    }
}

/**
 * 
 * @param array $params
 * @param mixed $row_values
 */
function currencies_get_taxes_value_columns_to_body_report($params, &$value) {

    $field = array_var($params, 'field');
    $object = array_var($params, 'object');
    if ($object instanceof IncomeInvoice && str_starts_with($field, 'tax_')) {
        $tax = IncomeTaxes::instance()->findOne(array("conditions" => "name = '" . substr($field, 4) . "'"));
        $idObject = $object->getColumnValue('id');
        $invoice = IncomeInvoices::instance()->findById($idObject);
        $invoice_taxes = IncomeInvoiceTaxes::instance()->findOne(array("conditions" => ' invoice_id = ' . $invoice->getId() . ' AND tax_id =' . $tax->getId()));
        if (!is_null($invoice_taxes)) {
            $value = $invoice_taxes->getFixedValue();
        }else{
            $value = 0;
        }
    }
}


function income_add_report_more_options($params, &$more_options_html_array) {
	$report = array_var($params, 'report');
	$genid = array_var($params, 'genid');
	
	if ($report instanceof Report) {
		$ot = ObjectTypes::instance()->findById($report->getReportObjectTypeId());
		
		if ($ot->getName() == 'invoice') {
			
			$more_options_html_array['show_currency'] = '
				<div class="report-option">
	                <label for="'.$genid.'show_currency">'. lang('show currency') .':</label>
	                <input id="'.$genid.'show_currency" type="checkbox" class="checkbox" name="report[show_currency]" 
	                       '. ($report->getColumnValue('show_currency') ? 'checked="checked"' : '0') .'/>
	            </div>
			';
			
			$more_options_html_array['show_negatives'] = '
				<div class="report-option">
	                <label for="'.$genid.'show_negatives">'. lang('show negatives') .':</label>
	                <input id="'.$genid.'show_negatives" type="checkbox" class="checkbox" name="report[show_negatives]" 
	                       '. ($report->getColumnValue('show_negatives') ? 'checked="checked"' : '0') .'/>
	            </div>
			';
			
		}
	}
	
}

function income_override_list_custom_property_values($params, &$custom_property_values){
	$custom_property = array_var($params, 'cp');
	if($custom_property instanceof CustomProperty && $custom_property->getCode() == 'invoice_template'){
		$invoice_templates = IncomeInvoiceTemplates::instance()->findAll();
		$invoice_template_values = array('0@'.lang('none'));
		foreach($invoice_templates as $template){
			$invoice_template_values[] = $template->getId().'@'.str_replace(',', '', $template->getName());
		}
		$custom_property_values = implode(',', $invoice_template_values);
	}
	
	
	/* Dont delete it will be used in the future por custom properties emails template
	//////custom properties for invoice Project's paid email template
	if($custom_property instanceof CustomProperty && $custom_property->getCode() == 'project_template_invoice_paid'){
		$email_templates = MailTemplates::instance()->findAll();
		$email_template_values = array('0@'.lang('none'));
		foreach($email_templates as $template){
			$email_template_values[] = $template->getId().'@'.str_replace(',', '', $template->getName());
		}
		$custom_property_values = implode(',', $email_template_values);
	}

	////custom properties for invoice Project's pending email template
	if($custom_property instanceof CustomProperty && $custom_property->getCode() == 'project_email_template_invoice_pending'){
		$email_templates = MailTemplates::instance()->findAll();
		$email_template_values = array('0@'.lang('none'));
		foreach($email_templates as $template){
			$email_template_values[] = $template->getId().'@'.str_replace(',', '', $template->getName());
		}
		$custom_property_values = implode(',', $email_template_values);
	}
	////custom properties for invoice Client paid email template
	if($custom_property instanceof CustomProperty && $custom_property->getCode() == 'client_email_template_invoice_paid'){
		$email_templates = MailTemplates::instance()->findAll();
		$email_template_values = array('0@'.lang('none'));
		foreach($email_templates as $template){
			$email_template_values[] = $template->getId().'@'.str_replace(',', '', $template->getName());
		}
		$custom_property_values = implode(',', $email_template_values);
	}
	/////custom properties for invoice Client pending email template
	if($custom_property instanceof CustomProperty && $custom_property->getCode() == 'client_email_template_invoice_pending'){
		$email_templates = MailTemplates::instance()->findAll();
		$email_template_values = array('0@'.lang('none'));
		foreach($email_templates as $template){
			$email_template_values[] = $template->getId().'@'.str_replace(',', '', $template->getName());
		}
		$custom_property_values = implode(',', $email_template_values);
	}*/
}



function income_render_more_total_task_times_report_params($report_data, &$ret) {
	if (!isset($report_data['invoicing_status'])) {
		$report_data['invoicing_status'] = user_config_option('timeReportInvoicingStatus');
	}
	
	$options = array(option_tag(lang('All'), ''));
	$attr = array_var($report_data, 'invoicing_status') == 'pending' ? array("selected"=>"selected") : null;
	$options[] = option_tag(lang('invoicing_status pending'), 'pending', $attr);
	$attr = array_var($report_data, 'invoicing_status') == 'invoiced' ? array("selected"=>"selected") : null;
	$options[] = option_tag(lang('invoicing_status invoiced'), 'invoiced', $attr);
	$attr = array_var($report_data, 'invoicing_status') == 'non_billable' ? array("selected"=>"selected") : null;
	$options[] = option_tag(lang('invoicing_status non_billable'), 'non_billable', $attr);
	echo "
		<tr style='height:30px;'>
			<td><span class='bold'>". lang('invoicing status') ."</span></td>
			<td align='left'>
				". select_box('report[invoicing_status]', $options, array("id" => "report[invoicing_status]")) ."
			</td>
		</tr>
	";
}

function income_more_total_task_times_extra_conditions($report_data, &$extra_conditions) {
	$invoicing_status_filter = array_var($report_data, 'invoicing_status', '');
	if ($invoicing_status_filter != '') {
		$extra_conditions .= " AND e.invoicing_status='$invoicing_status_filter'";
	}
	
	set_user_config_option('timeReportInvoicingStatus', $invoicing_status_filter , logged_user()->getId());
}



function income_after_render_custom_property_input($params, &$html) {
	
	$cp = array_var($params, 'cp');/* @var $cp CustomProperty */
	$config = array_var($params, 'config');
	
	if ($cp->getCode() == 'billing_contact') {
		$project_ot = ObjectTypes::findByName('project');
		if ($project_ot->getId() == $cp->getObjectTypeId()) {
			
			$client_assoc = DimensionMemberAssociations::getAssociationByCode('project_customer');
			$billing_client_assoc = DimensionMemberAssociations::getAssociationByCode('project_billing_client');
			$client_assoc_id = $client_assoc ? $client_assoc->getId() : "0";
			$billing_client_assoc_id = $billing_client_assoc ? $billing_client_assoc->getId() : "0";
			
			$genid = $config['genid'];
			
			$html .= '<div class="input-container">';
			$html .= label_tag(lang('billing address'), null, false, array('style' => 'display:inline-block;'));
			
			$html .= '<div class="field" style="float:left;">
						<div id="'.$genid.'billing_address_container" class="address-input-container en_us">
							<span class="desc">'.lang('select billing client or contact to get the billing address').'</span>
						</div>
					  </div>';
			
			$html .= '<script>
						og.income.refresh_project_billing_address = function() {
							og.income.get_billing_address_from_project_inputs(\''.$genid.'\', '.$cp->getId().', '.$client_assoc_id.', '.$billing_client_assoc_id.');
						}
						og.income.refresh_project_billing_address();
					  </script>
					';
			
			$html .= '<div class="clear"></div></div>';
			
		}
	}
}

function income_contact_cp_selector_onchange($params, &$onchange_fn) {
	$cp = array_var($params, 'cp');
	if ($cp && $cp->getCode() == 'billing_contact') {
		$onchange_fn = "do_refresh_project_billing_address";
	}
}

function income_override_contact_cp_filters($params, &$filters) {
	$cp = array_var($params, 'cp');
	if ($cp && $cp->getCode() == 'billing_contact') {
		if (!is_array($filters)) $filters = array();
		$filters['include_companies'] = 1;
	}
}


function income_before_render_associated_dimension_selector($parameters, &$listeners) {
	
	$genid = array_var($parameters, 'genid');
	$selected_ids = array_var($parameters, 'selected_ids');
	$dim_association = array_var($parameters, 'dim_association');/* @var $dim_association DimensionMemberAssociation */
	$member = array_var($parameters, 'member');
	
	$project_ot = ObjectTypes::findByName('project');
	$client_ot = ObjectTypes::findByName('customer');
	if ($dim_association->getObjectTypeId() == $project_ot->getId() && $dim_association->getAssociatedObjectType() == $client_ot->getId()) {
		
		$reload_project_billing_address_fn = "do_refresh_project_billing_address();";
		
		if (!isset($listeners['on_selection_change'])) $listeners['on_selection_change'] = '';
		if (!isset($listeners['on_remove_relation'])) $listeners['on_remove_relation'] = '';
		
		$listeners['on_selection_change'] .= $reload_project_billing_address_fn;
		$listeners['on_remove_relation'] .= $reload_project_billing_address_fn;
		
	}

}

function income_add_allowed_object_types_to_import($parameters, &$allowed_object_types_to_import){
	$allowed_object_types_to_import[] = 'invoice';
}


/**
 * Add billing address and billing email to the to the projects list
 */
function income_member_listing_additional_data($params, &$listing_object) {

	if (Plugins::instance()->isActivePlugin('advanced_core') && array_var($params, 'caller') instanceof MemberGroupedListingController) {

		$project_ot = ObjectTypes::findByName('project');
		if ($project_ot && array_var($params, 'type_id') == $project_ot->getId()) {

			$billing_contact_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), 'billing_client');

			$income_controller = new IncomeController();

			foreach ($listing_object['members'] as &$member_data) {
				$member = Members::getMemberById($member_data['id']);

				$billing_contact_id = null;
				if ($billing_contact_cp instanceof CustomProperty) {
					$cpv = CustomPropertyValues::getCustomPropertyValue($member_data['object_id'], $billing_contact_cp->getId());
					if ($cpv instanceof CustomPropertyValue) {
						$billing_contact_id = $cpv->getValue();
					}
				}

				// get the billing client member of the project
				$client_member = get_client_member_of_project($member, 'project_billing_client');
				if (!$client_member) {
					// if not billing client, then use the default client
					$client_member = get_client_member_of_project($member);
				}
				if ($client_member instanceof Member) {
					$billing_client_id = $client_member->getId();
				}

				// find billing address
				$billing_address = $income_controller->find_billing_address_by_client_and_contact($billing_client_id, $billing_contact_id);

				$billing_address_str = "";
				if ($billing_address instanceof ContactAddress) {
					$billing_address_str = $billing_address->toString();
				}

				// find billing email
				$billing_email = $income_controller->find_billing_email_by_client_and_contact($billing_client_id, $billing_contact_id);
				

				// add new columns to the member data object
				$member_data['billing_address'] = $billing_address_str;
				$member_data['billing_email'] = $billing_email;
			}

		}

	}

	$project_ot = ObjectTypes::findByName('project');
	if ($project_ot && array_var($params, 'type_id') == $project_ot->getId() && Plugins::instance()->isActivePlugin('advanced_billing')) {
        $csymbol = null;
        if (isset($currency) && $currency instanceof Currency) {
            $csymbol = $currency->getSymbol();
        }else{
            $csymbol = config_option('currency_code');
        }
		if (isset($listing_object['members'])) {
			foreach ($listing_object['members'] as &$member_data) {
				// Get project member estimated totals
				$member_id = $member_data['id'];
				$member_totals = MemberTotals::instance()->findOne(array('conditions' => array('`member_id`="' . $member_id . '"')));
				$member_has_totals = $member_totals instanceof MemberTotal;

				// Add invoice totals to member data
				$member_data['invoiced_total'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('invoiced_total') : 0, $csymbol);
				$member_data['invoiced_subtotal'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('invoiced_subtotal') : 0, $csymbol);
				$member_data['invoiced_paid_amount'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('invoiced_paid_amount') : 0, $csymbol);
				$member_data['invoiced_due_amount'] = format_money_amount($member_has_totals ? $member_totals->getColumnValue('invoiced_due_amount') : 0, $csymbol);
			}
		}
	}

}

function income_check_objects_to_get_restored($params, &$result){
	$ids = array_var($params, 'ids');
	$imploded_ids = '(' . implode(',', $ids) . ')';
	$invoices = IncomeInvoices::instance()->findAll(array('conditions' => '`object_id` IN '.$imploded_ids));
	if ($invoices) {
		if (count($invoices)) {
			$result = array(
				'allowed_to_restore' => false,
				'error' => count($ids),
				'errorMessage' => lang('you cannot untrash invoice')
			);
		}
	}
}

function income_custom_property_input_default_value($params, &$default_value){
	$object = array_var($params, 'object');
	$customProp = array_var($params, 'customProp');
	$project_ot_id = ObjectTypes::findByName('project')->getId();
	$project_ot = ObjectTypes::findByName('project');

	if ($customProp instanceof CustomProperty) {
		if($customProp->getCode() == 'po_number'){

			//get the po_number from the invoice
			$current_value = '';
			$cpvalue = CustomPropertyValues::getCustomPropertyValue($object->getId(), $customProp->getId());
			if ($cpvalue instanceof CustomPropertyValue) {
				$current_value = $cpvalue->getValue();
			}

			//if po_number isnull and the invoice is new get the po_number from the context project
			if(empty($current_value) && $object->isNew()==1){
				$project_member='';
				foreach (active_context() as $selection) {
					if ($selection instanceof Member && in_array($selection->getObjectTypeId(), array($project_ot->getId()))) {
						$project_member = $selection;
						break;
					}
				}

				if ($project_member instanceof Member) {
					$project_id_cp = CustomProperties::getCustomPropertyByCode($project_ot_id, "po_number");
					$tmp_project_obj_id = $project_member->getObjectId();

					if ($project_id_cp instanceof CustomProperty) {
						$cpval = CustomPropertyValues::getCustomPropertyValue($tmp_project_obj_id, $project_id_cp->getId());
						if ($cpval instanceof CustomPropertyValue) {
							$default_value = $cpval->getValue();
						}
					}
				}
			}

			//if no new object and the po_number is empty, keep it empty
		}
	}
}

function income_get_all_associated_members_finish($params, &$value){
	$object = array_var($params, 'object');
	$project_number = null;
	$project_ot_id = ObjectTypes::findByName('project')->getId();
	$invoice_ot_id = ObjectTypes::findByName('invoice')->getId();

	$invoice_id_cp = CustomProperties::getCustomPropertyByCode($invoice_ot_id, "po_number");
	$invoice_po_number_id = $invoice_id_cp->getId();

	if ($object instanceof Member) {
		$project_member = $object;
	}

	if ($project_member instanceof Member) {
		$project_id_cp = CustomProperties::getCustomPropertyByCode($project_ot_id, "po_number");
		$tmp_project_obj_id = $project_member->getObjectId();
		
		if ($project_id_cp instanceof CustomProperty) {
			$cpval = CustomPropertyValues::getCustomPropertyValue($tmp_project_obj_id, $project_id_cp->getId());
			if ($cpval instanceof CustomPropertyValue) {
				$project_number = $cpval->getValue();
			}
		}
	}

	$value['project_id'] = $object->getId();
	$value['project_number'] = $project_number;
	$value['invoice_po_number_id'] = $invoice_po_number_id;
}

function income_render_invoice_view_before_tab($params, &$invoiceObject){
	$beforeTemplate = '<div class="wrapper">
		<div class="tabs">
			<nav class="tabs__nav" role="tablist">
			<button class="tabs__btn is-active" data-tab-target="tab-2" type="button" role="tab" aria-selected="true">Printed view</button>	
			<button class="tabs__btn " data-tab-target="tab-1" type="button" role="tab" aria-selected="false">Invoice details</button>
				
			</nav>
			<div class="tabs__content">
				<div class="tabs__pane opacityTemp" id="tab-1" role="tabpanel">';
	echo $beforeTemplate;
}

function income_render_invoice_view_after_tab($params, &$invoiceObject){
	$url = $invoiceObject->getGeneratePreviewUrl();
	$sectionIframe = '</div>
				<div class="tabs__pane is-visible" id="tab-2" role="tabpanel">
					<div id="parentIframe" style="position: relative;">
						<iframe id="idIframe" src="' . $url . '" frameborder="0" scrolling="no" onload="App.modules.addInvoiceForm.resizeIframe(this)" allowfullscreen style="height: 100%; width:100%;"> </iframe>
						<!--div id="refVelo" style="position: absolute; height: 100%; width:100%; z-index: 999;"> </div-->
					</div>
				</div>
			</div>
		</div>
	</div>';
	echo $sectionIframe;
}

function income_get_allow_columns_for_reports($params, &$ret) {
    $allowed_columns = $ret[0];
    $object_type = array_var($params, 'object_type');

	if(Plugins::instance()->isActivePlugin('expenses2') && $object_type == PaymentReceipts::instance()->getObjectTypeId()){
		$allowed_columns[] = array('id' => 'invoice_date', 'name' => lang('invoice date'), 'type' => 'date');
	}

    $ret[0] = $allowed_columns;
}

function income_get_value_columns_to_body_report($params, &$value) {

    $field = array_var($params, 'field');
    $object = array_var($params, 'object');
	
    if (Plugins::instance()->isActivePlugin('expenses2') && $object instanceof PaymentReceipt) {
		if($field == 'invoice_date') {
			$invoice_id = $object->getColumnValue('invoice_id');
			$invoice = IncomeInvoices::instance()->findById($invoice_id);
			if ($invoice instanceof IncomeInvoice) {
				$invoice_dtv = $invoice->getDate();
				$dateFormat = user_config_option('date_format');
				$dt = new DateTimeValue($invoice_dtv->getTimestamp());
				$value = $dt->format($dateFormat);
			}
		}
	}
}

function income_get_columns_to_header_report($params, &$results) {

    $field = array_var($params, 'field');
	if (Plugins::instance()->isActivePlugin('expenses2') && $field == 'invoice_date') {	
		$results['columns']['names'][$field] = lang('invoice date');
		$results['columns']['order'][] = $field;
		$results['columns']['types'][$field] = DATA_TYPE_DATE;

	}
}

function income_custom_reports_fixed_additional_columns($args, &$ret) {
	
	// Can be used to add additional columns to the conditions of the report
	/*$ot = array_var($args, 'object_type');
	if (Plugins::instance()->isActivePlugin('expenses2') && $ot instanceof ObjectType && $ot->getId() == PaymentReceipts::instance()->getObjectTypeId()) {
		$ret[] = array('id' => 'invoice_date', 'name' => lang('invoice date'), 'type' => 'date');
	}*/

}

function income_get_additional_sql_join($params, &$ref) {
    $class = array_var($params, 'object_class');

    if ($class == 'PaymentReceipts') {
        $ref = array(
            'table' => TABLE_PREFIX . "income_invoices",
            'jt_field' => 'object_id',
            'e_field' => 'invoice_id',
            'join_type' => 'left'
        );
    }
}
function income_get_additional_sql_select($params, &$ref) {
    $class = array_var($params, 'object_class');
    if ($class == 'PaymentReceipts') {
        $ref = array('o.*,e.*', 'jt.date as invoice_date ');
    }

}

function income_custom_report_override_order_column($params, &$new_order_params) {
	$report = array_var($params, 'report');
	$ot = array_var($params, 'ot');
	$order_by_col = array_var($params, 'order_by_col');
	if (Plugins::instance()->isActivePlugin('expenses2') && $ot instanceof ObjectType && $ot->getId() == PaymentReceipts::instance()->getObjectTypeId()) {
		if ($order_by_col == 'invoice_date') {
			$new_order_params = array(
				'order_by_col' => 'invoice_date'
			);
		}
	}
}


// don't autoclassify in billing clients when adding object to a member's related members
function income_skipped_associations_in_add_to_realted_members($params, &$skipped_association_codes) {
	
	$skipped_association_codes[] = 'project_billing_client';

}

function income_get_objects_list_more_element_data($params, &$info_elem) {
	$object = array_var($params, 'object');
	if ($object instanceof IncomeInvoice) {

		$info_elem['invoice_number'] = $object->getInvoiceNumber();
		$info_elem['total'] = $object->getTotal();
		$info_elem['due_amount'] = $object->getDueAmount();
		$info_elem['paid_amount'] = $object->getPaidAmount();

	}
	
}
