<?php
/* @var $invoice IncomeInvoice */
set_page_title($invoice->getObjectName());

$currency = Currencies::instance()->getCurrency($invoice->getCurrencyId());
$currency_symbol = $currency instanceof Currency ? $currency->getSymbol() : "";

$project_member = null;
$project_ot = ObjectTypes::findByName('project');
$invoice_members = $invoice->getMembers();
foreach ($invoice_members as $member) {
    if ($member instanceof Member && $member->getObjectTypeId() == $project_ot->getId()) {
        $project_member = $member;
        break;
    }
}

$invoice_template = $invoice->getInvoiceTemplate();
if ($invoice_template instanceof IncomeInvoiceTemplate && $invoice_template->getBreakDownBySubprojects() == 'MULTI') {
    if ($project_member instanceof Member) {
        $projectParent = $project_member->getParentMemberId();
        if ($projectParent > 0) {
            $newParentMember = Members::getMemberById($project_member->getParentMemberId());
            if ($newParentMember instanceof Member) {
                $project_member = $newParentMember;
            }
        }
    }
}

$terms_text = "";
if (class_exists('IncomePaymentTerms')) {

    $payment_term = IncomePaymentTerms::instance()->findById($invoice->getPaymentTermId());
    if (!$payment_term) {
        $payment_term = IncomePaymentTerms::getDefaultPaymentTerm();
    }
    if ($payment_term) {
        $terms_text = $payment_term->getName();
    }
} else {
    // use old custom property
    $payment_period_cp = CustomProperties::getCustomPropertyByCode($invoice->getObjectTypeId(), 'invoice_payment_period');
    if ($payment_period_cp instanceof CustomProperty) {
        $payment_period_cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $payment_period_cp->getId());
        if ($payment_period_cp_val instanceof CustomPropertyValue) {
            $terms_text = $payment_period_cp_val->getValue();
        }
    }
}

$project_and_po_number_text = "";
$inv_po_number_cp = CustomProperties::getCustomPropertyByCode($invoice->getObjectTypeId(), "po_number");
if ($inv_po_number_cp instanceof CustomProperty) {
    $cpval = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $inv_po_number_cp->getId());
    if ($cpval instanceof CustomPropertyValue) {
        $project_and_po_number_text = trim($cpval->getValue());
    }
}
if ($project_and_po_number_text == '' && $project_member instanceof Member) {
    $project_id_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "po_number");
    $po_number_cp_val = "";
    if ($project_id_cp instanceof CustomProperty) {
        $cpval = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $project_id_cp->getId());
        if ($cpval instanceof CustomPropertyValue) {
            $po_number_cp_val = $cpval->getValue();
        }
    }

    $project_and_po_number_text = $project_member->getName();
    if ($po_number_cp_val != "") {
        $project_and_po_number_text .= " / $po_number_cp_val";
    }
}


$billing_from_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'billing_period_from');
$billing_to_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'billing_period_to');
$billing_from_date = '';
$billing_to_date = '';
$cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $billing_from_cp->getId());
if ($cp_val && $cp_val->getValue() != '') {
    try {
        $dt = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $cp_val->getValue());
        $billing_from_date = format_date($dt, null, 0);
    } catch (Exception $e) {
    }
}
$cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $billing_to_cp->getId());
if ($cp_val && $cp_val->getValue() != '') {
    try {
        $dt = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $cp_val->getValue());
        $billing_to_date = format_date($dt, null, 0);
    } catch (Exception $e) {
    }
}

?>
<style>
    * {
        margin: 0;
        padding: 0;
    }

    body {
        font: 12px/1.2 Sans-Serif;
    }

    #page-wrap {
        width: 700px;
        margin: 10px auto;
        padding: 10px 0;
    }

    table {
        border-collapse: collapse;
    }

    table td,
    table th {
        border: 1px solid #555;
        padding: 3px;
        font-size: 14px;
    }

    table th {
        font-size: 12px;
    }

    #customer {
        overflow: hidden;
    }

    #logo {
        text-align: right;
        float: right;
        position: relative;
        margin-top: 25px;
        border: 1px solid #fff;
        max-width: 540px;
        max-height: 100px;
        overflow: hidden;
    }

    #customer-title {
        font-size: 20px;
        font-weight: bold;
        float: left;
    }

    #meta {
        width: 300px;
        float: right;
    }

    #meta td {
        text-align: right;
    }

    #meta td.meta-head {
        text-align: left;
        background: #eee;
    }

    #meta td {
        text-align: right;
    }

    #items {
        clear: both;
        width: 100%;
        margin: 0 0 0 0;
        border: 1px solid #555;
        min-height: 400px;
    }

    #pre-items {
        clear: both;
        width: 100%;
        margin: 10px 0 0 0;
        border: 1px solid #555;
    }

    #pre-items,
    #pre-items td {
        border-bottom: 0px none;
    }

    #items th,
    #pre-items th {
        background: #eee;
        text-align: center !important;
        text-transform: uppercase;
        -webkit-print-color-adjust: exact !important;
    }

    #items tr.item-row td {
        border: 0;
        vertical-align: top;
        border-right: 1px solid #555;
    }


    #items td.description {
        /* width: 300px; */
    }

    #items .right {
        text-align: right;
    }

    #items .left {
        text-align: left;
    }

    #items .center {
        text-align: center;
    }

    #items .short {
        width: 90px;
        max-width: 120px;
    }

    #items td.total-line {
        border-right: 0;
        text-align: right;
        font-weight: bold;
    }

    #items td.total-line-tax {
        border-right: 0;
        text-align: right;
    }

    #items td.total-value {
        border-left: 0;
        padding: 5px;
        height: 15px;
        background: none;
        text-align: right;
        font-weight: bold;
    }

    #items td.balance {
        background: #eee;
    }

    #items td.blank {
        border: 0;
    }

    #terms {
        text-align: center;
        margin: 20px 0 0 0;
    }

    #terms h5 {
        text-transform: uppercase;
        font: 13px Sans-Serif;
        letter-spacing: 10px;
        border-bottom: 1px solid #555;
        padding: 0 0 8px 0;
        margin: 0 0 8px 0;
    }

    #terms {
        width: 100%;
        text-align: center;
    }

    td.print-invoice-status {
        font-weight: bold;
    }

    td.print-invoice-status.pending {}

    td.print-invoice-status.confirmed {
        background-color: #a8ce80;
    }

    td.print-invoice-status.canceled {
        background-color: #222;
        color: #eee;
    }

    div.bill-to {
        margin: 20px 5px;
    }

    div.bill-to div {
        margin: 4px 0px;
    }

    div.owner-company-info {
        margin: 0 5px 5px 5px;

    }

    div.project-table {
        float: right;
        margin-top: -95px;
        width: 300px;
    }

    div.project-table table {
        width: 100%;
        text-align: center;
    }

    .right.TotalWidht {
        white-space: nowrap;
    }
</style>



<div id="page-wrap">
    <div id="customer">
        <div style="float: left; width: 299px;">
            <?php if ($invoice_options['invoice_logo']) : ?>
                <div>
                    <img src="<?php echo $invoice_options['invoice_logo'] ?>" alt="logo" />
                </div>
            <?php endif; ?>

            <div class="owner-company-info">
                <div><?php echo $to_pay_information['address']; ?></div>
                <div><?php echo $to_pay_information['address_location']; ?></div>
                <div><?php echo lang('phone') . ": " . $to_pay_information['phone']; ?></div>
            </div>

        </div>
        <div style="float: right; margin-top:25px;">
            <div style="text-align:center; font-weight:bold; font-size:150%; margin-bottom:20px;">
                <?php echo strtoupper($invoice->getInvoiceType()) ?>
            </div>
            <table>
                <tr>
                    <td style="text-align:center; font-weight:bold; font-size:12px; min-width:100px;"><?php echo strtoupper(lang('date')) ?></td>
                    <td style="text-align:center; font-size:12px; "><?php echo format_date($invoice->getDate()->advance(logged_user()->getUserTimezoneValue() * -1, false)); ?></td>
                </tr>
                <tr>
                    <td style="text-align:center; font-weight:bold; font-size:12px; min-width:100px;"><?php echo strtoupper(lang('invoice')) . " #" ?></td>
                    <td style="text-align:center; font-size:12px; "><?php echo trim($invoice->getInvoiceNumber()) ?></td>
                </tr>
                <tr>
                    <td style="text-align:center; font-weight:bold; font-size:12px; min-width:100px;"><?php echo "TERMS" ?></td>
                    <td style="text-align:center; font-size:12px; "><?php echo $terms_text ?></td>
                </tr>
            </table>
        </div>
    </div>

    <div class="bill-to">
        <div><b><?php echo strtoupper(lang('bill to')) ?></b></div>
        <div><?php echo $bill_to_information['company_name']; ?></div>
        <div><?php echo $bill_to_information['ordered_by_name']; ?></div>
        <div><?php echo $bill_to_information['address']; ?></div>
        <div><?php
                $city_state_str = $bill_to_information['city'];
                $city_state_str .= ($city_state_str == "" ? "" : ", ") . trim($bill_to_information['state'] . " " . $bill_to_information['zip_code']);
                echo $city_state_str;
                ?></div>
    </div>

    <div class="project-table">
        <table>
            <tr>
                <td><b>P.O. # / Contact</b></td>
                <td><?php echo "-" //$project_and_po_number_text 
                    ?></td>
            </tr>
            <tr>
                <td><b>SERVICES RENDERED</b></td>
                <td><b>REPORT DATE</b></td>
            </tr>
            <tr>
                <td><?php echo "$billing_from_date - $billing_to_date" ?></td>
                <td>N/A</td>
            </tr>
        </table>
    </div>

    <?php
    if ($invoice_template instanceof IncomeInvoiceTemplate) {
    ?>
       
    <?php
        echo $invoice_template->getHtmlDetailsTableForInvoice($invoice);
    } else {
    ?>

        <table id="items" style="margin:10px 0 0 0;">
            <tr>
                <th><?php echo strtoupper(lang('description')) ?></th>
                <th style="width:90px;"><?php echo strtoupper('quantity') ?></th>
                <th style="width:120px;"><?php echo strtoupper('rate') ?></th>
                <th style="width:120px;"><?php echo strtoupper(lang('amount')) ?></th>
            </tr>
            <?php
            $lines_left = 20;
            foreach ($invoice->getInvoiceLines() as $invoice_line) :
                $line_currency = Currencies::instance()->getCurrency($invoice_line->getCurrencyId());
                $line_currency_symbol = $line_currency instanceof Currency ? $line_currency->getSymbol() : "";
                $lines_left--;
            ?>
                <tr class="item-row">
                    <td class="description"><?php echo nl2br($invoice_line->getDetail()) ?></td>
                    <td align="center"><?php echo $invoice_line->getAmount() ?></td>
                    <td align="right"><?php echo $line_currency_symbol . ' ' . number_format($invoice_line->getUnitValue(), 2) ?></td>
                    <td align="right"><?php echo $line_currency_symbol . ' ' . number_format($invoice_line->getTotal(), 2) ?></td>
                </tr>
            <?php endforeach; ?>
            <?php while ($lines_left > 0) :
                $lines_left--;
            ?>
                <tr class="item-row">
                    <td>&nbsp;</td>
                    <td class="description">&nbsp;</td>
                    <td></td>
                    <td></td>
                </tr>
            <?php endwhile; ?>
            <tr>
                <td colspan="2" class="center"><b>Thank you for your business!</b></td>
                <td class="total-line">Balance Due</td>
                <td class="total-value">
                    <div id="total"><?php echo $currency_symbol . ' ' . number_format($invoice->getTotal(), 2) ?></div>
                </td>
            </tr>
        </table>

    <?php } ?>
    <div style="margin: 10px 0 0 0;">
        <div style="float: left;">
            <table style="text-align: left; margin-right:20px;">
                <?php
                if ($invoice_options['view_press_information']) :
                ?>
                    <tr>
                        <td colspan="2" style="border: 0px;">
                            <?php echo nl2br($invoice_notebook['press_information']) ?>
                        </td>
                    </tr>
                <?php
                endif;
                ?>
            </table>
        </div>

        <div style="clear:both"></div>
    </div>
</div>


<?php if (!isset($dont_print) || !$dont_print) { ?>
    <script>
        if (window.location == window.parent.location) {
            window.print();
        }
    </script>
<?php } ?>