<?php
/* @var $invoice IncomeInvoice */
set_page_title($invoice->getObjectName());

$currency = Currencies::instance()->getCurrency($invoice->getCurrencyId());
$currency_symbol = $currency instanceof Currency ? $currency->getSymbol() : "";

$project_member = null;
$project_ot = ObjectTypes::findByName('project');
$invoice_members = $invoice->getMembers();
foreach ($invoice_members as $member) {
    if ($member instanceof Member && $member->getObjectTypeId() == $project_ot->getId()) {
        $project_member = $member;
        break;
    }
}

$invoice_template = $invoice->getInvoiceTemplate();
if ($invoice_template instanceof IncomeInvoiceTemplate && $invoice_template->getBreakDownBySubprojects() == 'MULTI') {
    if ($project_member instanceof Member) {
        $projectParent = $project_member->getParentMemberId();
        if ($projectParent > 0) {
            $newParentMember = Members::getMemberById($project_member->getParentMemberId());
            if ($newParentMember instanceof Member) {
                $project_member = $newParentMember;
            }
        }
    }
}

$terms_text = "";
if (class_exists('IncomePaymentTerms')) {

    $payment_term = IncomePaymentTerms::instance()->findById($invoice->getPaymentTermId());
    if (!$payment_term) {
        $payment_term = IncomePaymentTerms::getDefaultPaymentTerm();
    }
    if ($payment_term) {
        $terms_text = $payment_term->getName();
    }
} else {
    // use old custom property
    $payment_period_cp = CustomProperties::getCustomPropertyByCode($invoice->getObjectTypeId(), 'invoice_payment_period');
    if ($payment_period_cp instanceof CustomProperty) {
        $payment_period_cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $payment_period_cp->getId());
        if ($payment_period_cp_val instanceof CustomPropertyValue) {
            $terms_text = $payment_period_cp_val->getValue();
        }
    }
}

$project_id_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "project_id");
$client_project_number = '';
if ($project_member instanceof Member) {

    $client_project = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "client_project_number");
    $client_number_assigned = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $client_project->getId());
    if ($client_number_assigned instanceof CustomPropertyValue) {
        $client_project_number = $client_number_assigned->getValue();
    }

    $project_number_and_name = $project_member->getName();

    if ($project_id_cp instanceof CustomProperty) {
        $project_id_cpval = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $project_id_cp->getId());
        if ($project_id_cpval instanceof CustomPropertyValue) {

            $project_number_and_name =  $project_id_cpval->getValue() . " - " . $project_member->getName();
            // print_r($project_number_and_name);
        }
    }
}

$project_and_po_number_text = "";
$inv_po_number_cp = CustomProperties::getCustomPropertyByCode($invoice->getObjectTypeId(), "po_number");
if ($inv_po_number_cp instanceof CustomProperty) {
    $cpval = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $inv_po_number_cp->getId());
    if ($cpval instanceof CustomPropertyValue) {
        $project_and_po_number_text = trim($cpval->getValue());
    }
}
if ($project_and_po_number_text == '' && $project_member instanceof Member) {

    $po_number_cp_val = "";
    if ($project_id_cp instanceof CustomProperty) {
        $project_id_cpval = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $project_id_cp->getId());
        if ($project_id_cpval instanceof CustomPropertyValue) {
            $po_number_cp_val = $project_id_cpval->getValue();
        }
    }

    $project_and_po_number_text = $project_member->getName();
    if ($po_number_cp_val != "") {
        $project_and_po_number_text .= " / $po_number_cp_val";
    }
}

$due_date_text = '';
if ($invoice->getExpirationDate() instanceof DateTimeValue) {
    $due_date_text = format_date($invoice->getExpirationDate(), null, 0);
}


$billing_from_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'billing_period_from');
$billing_to_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'billing_period_to');
$billing_from_date = '';
$billing_to_date = '';
$cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $billing_from_cp->getId());
if ($cp_val && $cp_val->getValue() != '') {
    try {
        $dt = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $cp_val->getValue());
        $billing_from_date = format_date($dt, null, 0);
    } catch (Exception $e) {
    }
}
$cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $billing_to_cp->getId());
if ($cp_val && $cp_val->getValue() != '') {
    try {
        $dt = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $cp_val->getValue());
        $billing_to_date = format_date($dt, null, 0);
    } catch (Exception $e) {
    }
}

function get_custom_property_value($ot_id, $member, $code)
{
    $cp = CustomProperties::getCustomPropertyByCode($ot_id, $code);
    if ($cp instanceof CustomProperty) {
        $value = CustomPropertyValues::getCustomPropertyValue($member->getObjectId(), $cp->getId());
        return $value instanceof CustomPropertyValue ? $value->getValue() : '';
    } else {
        return '';
    }
}

// Get client info
$client_id = get_associated_status_member_id($project_member, Dimensions::findByCode('customers'));
$client_member = Members::getMemberById($client_id);
$client_name = $client_member->getName();
$client_ot = ObjectTypes::findByName('customer');
$client_ot_id = $client_ot->getId();

$billing_contact_name = "";
$billing_contact_phone = "";
$billing_contact_email = "";
$billing_contact_id = get_custom_property_value($project_ot->getId(), $project_member, 'client_project_manager');

if ($billing_contact_id) {
    $billing_contact = Contacts::instance()->findById($billing_contact_id);
    if ($billing_contact instanceof Contact) {
        // Get name
        $billing_contact_name = $billing_contact->getName();
        // Get telephone
        // $contact_telephone = ContactTelephones::instance()->findOne(array('conditions' => "contact_id=" . $billing_contact_id));
        // if ($contact_telephone instanceof ContactTelephone) {
        //     $billing_contact_phone = $contact_telephone->getNumber();
        // }
        //Get email
        // $contact_email = $billing_contact->getEmail();
        // $billing_contact_email = $contact_email instanceof ContactEmail ? $contact_email->getEmailAddress() : '';
    }
}

?>
<style>
    * {
        margin: 0;
        padding: 0;
    }

    body {
        font: 12px/1.2 Sans-Serif;
    }

    #page-wrap {
        width: 700px;
        margin: 10px auto;
        padding: 10px 0;
    }

    table {
        border-collapse: collapse;
    }

    table td,
    table th {
        border: 1px solid #555;
        padding: 3px;
        font-size: 14px;
    }

    table th {
        font-size: 12px;
    }

    #customer {
        overflow: hidden;
    }

    #logo {
        text-align: right;
        float: right;
        position: relative;
        margin-top: 25px;
        border: 1px solid #fff;
        max-width: 540px;
        max-height: 100px;
        overflow: hidden;
    }

    #customer-title {
        font-size: 20px;
        font-weight: bold;
        float: left;
    }

    #meta {
        width: 300px;
        float: right;
    }

    #meta td {
        text-align: right;
    }

    #meta td.meta-head {
        text-align: left;
        background: #eee;
    }

    #meta td {
        text-align: right;
    }

    #items {
        clear: both;
        width: 100%;
        margin: 0 0 0 0;
        border: 1px solid #555;
        /*min-height: 300px;*/
    }

    #pre-items {
        clear: both;
        width: 100%;
        margin: 10px 0 0 0;
        border: 1px solid #555;
    }

    #pre-items,
    #pre-items td {
        border-bottom: 0px none;
    }

    #items thead {
        height: 20px;
    }

    #items th,
    #pre-items th {
        background: #eee;
        text-align: center !important;
        text-transform: uppercase;
    }

    #items tr.item-row td {
        border: 0;
        vertical-align: top;
        border-right: 1px solid #555;
    }


    #items td.description {
        width: 300px;
    }

    #items .right {
        text-align: right;
    }

    #items .left {
        text-align: left;
    }

    #items .center {
        text-align: center;
    }

    #items .short {
        width: 90px;
        max-width: 120px;
    }

    #items tbody.total-tbody {
        height: 20px;
    }

    #items td.total-line {
        border-right: 0;
        text-align: right;
        font-weight: bold;
    }

    #items td.total-line-tax {
        border-right: 0;
        text-align: right;
    }

    #items td.total-value {
        border-left: 0;
        padding: 5px;
        height: 15px;
        background: none;
        text-align: right;
        font-weight: bold;
    }

    #items td.balance {
        background: #eee;
    }

    #items td.blank {
        border: 0;
    }

    #terms {
        text-align: center;
        margin: 20px 0 0 0;
    }

    #terms h5 {
        text-transform: uppercase;
        font: 13px Sans-Serif;
        letter-spacing: 10px;
        border-bottom: 1px solid #555;
        padding: 0 0 8px 0;
        margin: 0 0 8px 0;
    }

    #terms {
        width: 100%;
        text-align: center;
    }

    td.print-invoice-status {
        font-weight: bold;
    }

    td.print-invoice-status.pending {}

    td.print-invoice-status.confirmed {
        background-color: #a8ce80;
    }

    td.print-invoice-status.canceled {
        background-color: #222;
        color: #eee;
    }

    div.bill-to {
        margin: 30px 5px 0;
        /* margin: 0px 5px; */
        float: left;
    }

    div.bill-to div {
        /* margin: 4px 0px; */
    }

    div.owner-company-info {
        margin: 0 5px 5px 5px;
    }
    div.project-table {
        margin-top: 10px;
        margin-bottom: 10px;
    }

    div.project-table td {
        text-align: left;
     /*   width: 28%;*/
    }

    div.project-table table {
        width: 100%;
        text-align: center;
    }

    div.account-summary {
        margin-top: 20px;
        padding: 5px 2px;
    }

    div.account-summary .total-due {
        text-align: right;
        padding: 5px;
        font-size: 14px;
        font-weight: bold;
    }

    div.account-summary .title {
        font-size: 16px;
    }

    div.account-summary table {
        width: 100%;
    }

    div.account-summary th {
        border: 1px solid #555;
        font-weight: normal;
        font-size: 12px;
    }

    div.account-summary td {
        border: 1px solid #555;
        font-size: 12px;
        text-align: right;
    }

	#items td {
		min-width: 75px;
	}
	#items td.TotalWidht {
		min-width: 85px;
		white-space: nowrap;
	}
</style>



<div id="page-wrap">
    <div id="customer">
        <div style="width: 299px;">
            <?php if ($invoice_options['invoice_logo']) : ?>
                <div>
                    <img src="<?php echo $invoice_options['invoice_logo'] ?>" alt="logo" />
                </div>
            <?php endif; ?>

            <div class="owner-company-info">
                <div><?php echo $to_pay_information['address']; ?></div>
                <div><?php echo $to_pay_information['address_location']; ?></div>
                <div><?php if ($to_pay_information['phone']) echo lang('phone') . ": " . $to_pay_information['phone']; ?></div>
            </div>

        </div>

    </div>

    <div class="bill-to">
        <div style="font-size:125%;"><b><?php echo strtoupper(lang('bill to')) ?></b></div>
        <div><?php echo nl2br($bill_to_information['company_name']); ?></div>
        <?php if ($invoice->getCustomerNumber() && $invoice->getCustomerNumber() > 0) {
        ?><div><?php echo $invoice->getCustomerNumber() ?></div><?php
                                                            }
                                                                ?>
        <div><?php echo nl2br($bill_to_information['address']); ?></div>
        <div><?php
                $city_state_str = $bill_to_information['city'];
                $city_state_str .= ($city_state_str == "" ? "" : ", ") . trim($bill_to_information['state'] . " " . $bill_to_information['zip_code']);
                echo $city_state_str;
                ?></div>
    </div>

    <div style="float: right;text-align:right;margin-bottom:10px;">
        <div style="text-align:right; font-weight:bold; font-size:125%;">
            <?php echo strtoupper($invoice->getInvoiceType()) ?>
        </div>
        <table style="float: right;">
            <tr>
                <td style="text-align:center; font-weight:bold; font-size:12px; min-width:100px;"><?php echo strtoupper(lang('date')) ?></td>
                <td style="text-align:center; font-weight:bold; font-size:12px; min-width:100px;"><?php echo strtoupper(lang('invoice')) . " #" ?></td>
            </tr>
            <tr>
                <td style="text-align:center; font-size:12px; "><?php echo format_date($invoice->getDate()->advance(logged_user()->getUserTimezoneValue() * -1, false)); ?></td>
                <td style="text-align:center; font-size:12px; "><?php echo trim($invoice->getSeries() . " " . $invoice->getInvoiceNumber()) ?></td>
            </tr>
        </table>
    </div>


    <?php
    $include_account_summay = array();

    if ($invoice_template instanceof IncomeInvoiceTemplate) {
    ?>
        <div style="clear:both"></div>
        <!-- <table id="pre-items">
            <tr>
                <th style="width:110px;"><?php echo strtoupper('date of service') ?></th>
                <th><?php echo strtoupper(lang('project')) ?></th>
                <th colspan="2"><?php echo strtoupper('terms') ?></th>
            </tr>
            <tr>
                <td style="text-align: center; width: 160px;"><?php echo "$billing_from_date - $billing_to_date" ?></td>
                <td style="text-align: center;"><?php echo $project_number_and_name; //$project_member instanceof Member ? $project_member->getName() : ""; 
                                                ?></td>
                <td style="text-align: center; min-width: 165px;" colspan="2"><?php echo $terms_text ?></td>
            </tr>
        </table> -->
        <?php

        $include_account_summay = $invoice_template->getIncludeAccountSummary();
        if ($include_account_summay) {
            $account_summary = $invoice->getProjectSummary();
        }

        $footer_disclaimer = $invoice_template->getIncludeFooterText();

        echo $invoice_template->getHtmlDetailsTableForInvoice($invoice);
    } else {
        ?>

        <table id="items" style="margin:10px 0 0 0;">

            <tr>
                <th style="width:90px;"><?php echo strtoupper('quantity') ?></th>
                <th><?php echo strtoupper(lang('description')) ?></th>
                <th style="width:110px;"><?php echo strtoupper('rate') ?></th>
                <th style="width:110px;"><?php echo strtoupper(lang('amount')) ?></th>
            </tr>
            <?php
            $lines_left = 20;
            foreach ($invoice->getInvoiceLines() as $invoice_line) :
                $line_currency = Currencies::instance()->getCurrency($invoice_line->getCurrencyId());
                $line_currency_symbol = $line_currency instanceof Currency ? $line_currency->getSymbol() : "";
                $lines_left--;
            ?>
                <tr class="item-row">
                    <td align="center"><?php echo $invoice_line->getAmount() ?></td>
                    <td class="description"><?php echo nl2br($invoice_line->getDetail()) ?></td>
                    <td align="right"><?php echo $line_currency_symbol . ' ' . number_format($invoice_line->getUnitValue(), 2) ?></td>
                    <td align="right"><?php echo $line_currency_symbol . ' ' . number_format($invoice_line->getTotal(), 2) ?></td>
                </tr>
            <?php endforeach; ?>
            <?php while ($lines_left > 0) :
                $lines_left--;
            ?>
                <tr class="item-row">
                    <td>&nbsp;</td>
                    <td class="description">&nbsp;</td>
                    <td></td>
                    <td></td>
                </tr>
            <?php endwhile; ?>
            <tr>
                <td colspan="2"><b>Thank you for your business!</b></td>
                <td class="total-line">Total</td>
                <td class="total-value">
                    <div id="total"><?php echo $currency_symbol . ' ' . number_format($invoice->getTotal(), 2) ?></div>
                </td>
            </tr>
        </table>

        <table style="margin-top:10px;">
            <tr>
                <td style="text-align:center; font-weight:bold; font-size:12px; min-width:100px;"><?php echo strtoupper('Terms') ?></td>
                <td style="text-align:center; font-weight:bold; font-size:12px; min-width:100px;"><?php echo strtoupper(lang('phone')) ?></td>
            </tr>
            <tr>
                <td style="text-align:center; font-size:12px; "><?php echo $terms_text ?></td>
                <td style="text-align:center; font-size:12px; "><?php echo $to_pay_information['phone'] ?></td>
            </tr>
        </table>
        <div class="clear"></div>

    <?php } ?>


    <?php if ($include_account_summay) { ?>
        <div class="account-summary">
            <div class="title">Account Summary</div>
            <table>

                <tr>
                    <th>Contract Amt</th>
                    <th>Services BTD</th>
                    <th>Expenses BTD</th>
                    <th>Last Inv Num</th>
                    <th>Last Inv Date</th>
                    <th>Last Inv Amt</th>
                    <th>Last Pay Amt</th>
                    <th>Prev Unpaid Amt</th>
                </tr>

                <tr>
                    <td><?php
                        if ($account_summary && $account_summary['contract_amount']) {
                            echo format_money_amount($account_summary['contract_amount'], $account_summary['contract_amount_currency_symbol']);
                        }
                        ?></td>
                    <td><?php
                        if ($account_summary && $account_summary['totals_btd']) {
                            foreach ($account_summary['totals_btd'] as $currency_id => $data) {
                                echo format_money_amount($data['total_labor'], $data['currency_symbol']) . "<br />";
                            }
                        }
                        ?></td>
                    <td><?php
                        if ($account_summary && $account_summary['totals_btd']) {
                            foreach ($account_summary['totals_btd'] as $currency_id => $data) {
                                echo format_money_amount($data['total_expenses'], $data['currency_symbol']) . "<br />";
                            }
                        }
                        ?></td>
                    <td><?php
                        if ($account_summary && $account_summary['last_invoice']) {
                            echo $account_summary['last_invoice']['number'];
                        } else {
                            echo '--';
                        }
                        ?></td>
                    <td><?php
                        if ($account_summary && $account_summary['last_invoice']) {
                            echo $account_summary['last_invoice']['date'];
                        } else {
                            echo '--';
                        }
                        ?></td>
                    <td><?php
                        if ($account_summary && $account_summary['last_invoice']) {
                            echo format_money_amount($account_summary['last_invoice']['amount'], $account_summary['last_invoice']['currency_symbol']);
                        }
                        ?></td>
                    <td><?php
                        if ($account_summary && $account_summary['last_paid_amount']) {
                            echo format_money_amount($account_summary['last_paid_amount']['amount'], $account_summary['last_paid_amount']['currency_symbol']);
                        }
                        ?></td>
                    <td><?php
                        if ($account_summary && $account_summary['totals_btd']) {
                            foreach ($account_summary['totals_btd'] as $currency_id => $data) {
                                echo format_money_amount($data['prev_due_amount'], $data['currency_symbol']) . "<br />";
                            }
                        }
                        ?></td>
                </tr>

            </table>
            <div class="total-due">
                Total amount due including this invoice: <?php
                                                            if ($account_summary && $account_summary['totals_btd']) {
                                                                foreach ($account_summary['totals_btd'] as $currency_id => $data) {
                                                                    echo format_money_amount($data['total_due_amount'], $data['currency_symbol']) . "<br />";
                                                                }
                                                            } else {
                                                                echo format_money_amount($invoice->getTotal(), $currency_symbol);
                                                            }
                                                            ?>
            </div>
        </div>
    <?php } ?>

    <div class="project-table" style="margin-top: 20px;">
        <table>
            <?php if ($client_project_number != '' || $billing_contact_name != '') : ?>
                <tr>
                    <?php if ($client_project_number != '') : ?>
                        <td style="width:200px; max-width:200px;"><b>Client Project #</b></td>
                    <?php endif; ?>
                    <?php if ($billing_contact_name != '') : ?>
                        <td style="width:220px; max-width:220px;"><b>Client Project Manager</b></td>
                    <?php endif; ?>
                </tr>
                <tr>
                    <?php if ($client_project_number != '') : ?>
                        <td><?= $client_project_number; ?></td>
                    <?php endif; ?>
                    <?php if ($billing_contact_name != '') : ?>
                        <td><?= $billing_contact_name; ?></td>
                    <?php endif; ?>
                </tr>
            <?php endif; ?>
        </table>
    </div>

    <div class="footer-info"><?php echo $footer_disclaimer ?></div>

</div>


<?php if (!isset($dont_print) || !$dont_print) { ?>
    <script>
        if (window.location == window.parent.location) {
            window.print();
        }
    </script>
<?php } ?>