<?php
/* @var $invoice IncomeInvoice */
set_page_title($invoice->getObjectName());

$currency = Currencies::instance()->getCurrency($invoice->getCurrencyId());
$currency_symbol = $currency instanceof Currency ? $currency->getSymbol() : "";

?>
<style>
    * {
        margin: 0;
        padding: 0;
    }

    body {
        font: 12px/1.2 Sans-Serif;
    }

    #page-wrap {
        width: 600px;
        margin: 10px auto;
        padding: 10px 0;
    }

    table {
        border-collapse: collapse;
    }

    table td,
    table th {
        border: 1px solid #555;
        padding: 3px;
    }

    #customer {
        overflow: hidden;
    }

    #logo {
        text-align: right;
        float: right;
        position: relative;
        margin-top: 25px;
        border: 1px solid #fff;
        max-width: 540px;
        max-height: 100px;
        overflow: hidden;
    }

    #customer-title {
        font-size: 20px;
        font-weight: bold;
        float: left;
    }

    #meta {
        width: 300px;
        float: right;
    }

    #meta td {
        text-align: right;
        padding: 5px 5px 4px;
    }

    #meta td.meta-head {
        text-align: left;
        background: #eee;
        -webkit-print-color-adjust: exact !important;
    }

    #meta td {
        text-align: right;
    }

    #items {
        clear: both;
        width: 100%;
        margin: 10px 0 0 0;
        border: 1px solid #555;
    }

    #items th {
        background: #eee;
        -webkit-print-color-adjust: exact !important;
    }

    #items td.head {
        border-bottom: 1px solid #555;
        font-weight: bold;
    }

    #items tr.item-row td {
        border: 0px none;
        border-right: 1px solid #555;
        vertical-align: top;
    }

    #items tr.totals-first-row {
        border-top: 1px solid #555;
    }

    #items td {
        border: 0px none;
    }

    #items td.description {
        width: 300px;
    }

    #items td.total-line {
        border-right: 0;
        border-left: 1px solid #555;
        border-bottom: 1px solid #555;
        font-weight: bold;
        white-space: nowrap;
    }

    #items td.total-line-tax {
        border-right: 0;
        text-align: right;
    }

    #items td.total-value {
        border-left: 1px solid #555;
        border-bottom: 1px solid #555;
        background: none;
        text-align: right;
        min-width: 80px;
    }

    #items td.balance {
        background: #eee;
    }

    #items td.blank {
        border: 0;
    }

    #terms {
        text-align: center;
        margin: 20px 0 0 0;
    }

    #terms h5 {
        text-transform: uppercase;
        font: 13px Sans-Serif;
        letter-spacing: 10px;
        border-bottom: 1px solid #555;
        padding: 0 0 8px 0;
        margin: 0 0 8px 0;
    }

    #terms {
        width: 100%;
        text-align: center;
    }

    td.print-invoice-status {
        font-weight: bold;
    }

    td.print-invoice-status.pending {}

    td.print-invoice-status.confirmed {
        background-color: #a8ce80;
    }

    td.print-invoice-status.canceled {
        background-color: #222;
        color: #eee;
    }

    .right.TotalWidht {
        white-space: nowrap;
    }
</style>

<div id="page-wrap">
    <div id="customer">
        <div style="float: left; width: 299px;">
            <?php if ($invoice_options['invoice_logo']) : ?>
                <div>
                    <img src="<?php echo $invoice_options['invoice_logo'] ?>" alt="logo" />
                </div>
            <?php endif; ?>
            <div>
                <table style="text-align: left;">
                    <?php
                    if ($invoice_options['view_address']) :
                    ?>
                        <tr>
                            <td colspan="2" style="border: 0px; text-align: left;">
                                <?php echo $invoice_notebook['address'] ?>
                            </td>
                        </tr>
                    <?php
                    endif;
                    ?>
                </table>
            </div>
        </div>
        <div style="float: left;">
            <table id="meta">
                <?php if ($invoice_options['view_rut']) : ?>
                    <tr>
                        <td style="border: 0px; text-align: center; font-size:125%;" colspan="2">
                            <b>Invoice</b>
                        </td>
                    </tr>
                    <tr>
                        <td style="border: 0px; text-align: center;" colspan="2">&nbsp;</td>
                    </tr>
                <?php endif; ?>
                <?php if ($invoice_options['view_date']) : ?>
                    <tr>
                        <td class="meta-head"><?php echo lang('date') ?></td>
                        <td><?php echo format_date($invoice->getDate()->advance(logged_user()->getUserTimezoneValue() * -1, false)); ?></td>
                    </tr>
                <?php endif; ?>

                <tr>
                    <td class="meta-head">Invoice #</td>
                    <td><?php echo $invoice->getSeries() . " " . $invoice->getInvoiceNumber() ?></td>
                </tr>

                <?php if ($invoice_options['view_customer_number'] && $invoice->getCustomerNumber()) : ?>
                    <tr>
                        <td class="meta-head"><?php echo lang('customer number') ?></td>
                        <td><?php echo $invoice->getCustomerNumber() ?></td>
                    </tr>
                <?php endif; ?>
                <?php if ($show_inovice_status === true) : ?>
                    <tr>
                        <td class="meta-head"><?php echo lang('status') ?></td>
                        <td class="print-invoice-status <?php echo $invoice->getStatus() ?>"><?php echo $inovice_status ?></td>
                    </tr>
                <?php endif; ?>

            </table>
        </div>
    </div>
    <table id="items">
        <tr>
            <th style="text-align:left;"><?php echo lang('bill to') ?>: </th>
        </tr>
        <tr>
            <td colspan="1"><?php echo $bill_to_information['ordered_by_name']; ?></td>
        </tr>
        <tr>
            <td colspan="1"><?php echo $bill_to_information['company_name']; ?></td>
        </tr>
        <tr>
            <td colspan="1"><?php echo $bill_to_information['address']; ?></td>
        </tr>
        <tr>
            <td colspan="1"><?php echo $bill_to_information['city'] . ", " . $bill_to_information['state'] . " " . $bill_to_information['zip_code']; ?></td>
        </tr>
        <tr>
            <td colspan="1"><?php echo $bill_to_information['country']; ?></td>
        </tr>
    </table>

    <table id="items">
        <tr>
            <th><?php echo lang('detail') ?></th>
            <th><?php echo lang('quantity') ?></th>
            <th><?php echo lang('price') ?></th>
            <th><?php echo lang('total') ?></th>
        </tr>
        <?php
        $count = 0;
        foreach ($invoice->getInvoiceLines() as $invoice_line) :
            $line_currency = Currencies::instance()->getCurrency($invoice_line->getCurrencyId());
            $line_currency_symbol = $line_currency instanceof Currency ? $line_currency->getSymbol() : "";
        ?>
            <tr class="item-row">
                <td class="description"><?php echo str_replace("    ", "&nbsp;&nbsp;&nbsp;&nbsp;", nl2br($invoice_line->getDetail())) ?></td>
                <td align="center"><?php echo $invoice_line->getAmount() ?></td>
                <td align="right"><?php echo $line_currency_symbol . ' ' . number_format($invoice_line->getUnitValue(), 2) ?></td>
                <td align="right"><?php echo $line_currency_symbol . ' ' . number_format($invoice_line->getTotal(), 2) ?></td>
            </tr>
        <?php
            $count++;
        endforeach;
        while ($count < 10) {
            $count++;
        ?>
            <tr class="item-row">
                <td class="description">&nbsp;</td>
                <td align="center"></td>
                <td align="right"></td>
                <td align="right"></td>
            </tr>
        <?php
        }
        ?>
        <?php
        if ($invoice_options['view_taxes']) :
            foreach ($taxes as $tax) :
        ?>
                <tr>
                    <td colspan="2" class="blank"></td>
                    <td class="total-line-tax"><?php echo $tax['name'] ?></td>
                    <td class="total-value">
                        <div id="subtotal"><?php echo $currency_symbol . ' ' . number_format($tax['tax_value'], 2) ?></div>
                    </td>
                </tr>
        <?php
            endforeach;
        endif;
        ?>
        <tr class="totals-first-row">
            <td colspan="2" rowspan="3" class="blank description"><?php
                                                                    if (trim($invoice->getAdditionalInformation()) != '') {
                                                                        echo "<b>" . lang('additional information') . ":</b><br />" . $invoice->getAdditionalInformation();
                                                                    }
                                                                    ?></td>
            <td class="total-line"><?php echo lang('subtotal') ?></td>
            <td class="total-value">
                <div id="subtotal"><?php echo $currency_symbol . ' ' . number_format($invoice->getSubtotal(), 2) ?></div>
            </td>
        </tr>
        <tr>
            <td class="total-line"><?php echo lang('total') ?></td>
            <td class="total-value">
                <div id="total"><?php
                                echo $currency_symbol . ' ' . number_format($invoice->getTotal(), 2)
                                /*
			$pm_cp = CustomProperties::getCustomPropertyByCode($invoice->getObjectTypeId(), 'payment_method');
			$pm_cpval = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $pm_cp->getId());
        		if ($pm_cpval instanceof CustomPropertyValue) {
            			echo lang($pm_cpval->getValue());
        		}			*/
                                ?></div>
            </td>
        </tr>
        <tr>
            <td class="total-line">Balance due</td>
            <td class="total-value">
                <?php $balance_owed = $invoice->getStatus() == 'confirmed' ? 0 : $invoice->getTotal(); ?>
                <div id="total"><?php echo $currency_symbol . ' ' . number_format($balance_owed, 2) ?></div>
            </td>
        </tr>
    </table>

    <div>

        <table style="text-align: left; margin-top:10px; width: 100%;">
            <tr>
                <td colspan="2" style="border: 0px;">
                    Please remit to: PHCC-Texas
                    &nbsp;-&nbsp;
                    <?php echo $to_pay_information['address']; ?>,
                    <?php echo $to_pay_information['address_location']; ?>
                    &nbsp;-&nbsp;
                    Phone: <?php echo $to_pay_information['phone']; ?>

                </td>
            </tr>
            <?php
            if ($invoice_options['view_press_information']) :
            ?>
                <tr>
                    <td colspan="2" style="border: 0px;">
                        <?php echo nl2br($invoice_notebook['press_information']) ?>
                    </td>
                </tr>
            <?php
            endif;
            ?>
        </table>

        <!--            <div style="float: left; margin-right:20px;">-->
        <!--                --><?php
                                //                if ($invoice_options['view_expiration_date']):
                                //                    
                                ?>
        <!--                    <div style="text-align: center; padding: 3px;">-->
        <!--                        --><?php //echo lang('expiration date') 
                                        ?>
        <!--                    </div>-->
        <!--                    <div style="text-align: center; padding: 3px;">-->
        <!--                        --><?php //echo format_date($invoice_notebook['expiration_date']) 
                                        ?>
        <!--                    </div>-->
        <!--                --><?php
                                //                endif;
                                //                
                                ?>
        <!--            </div>-->
        <div style="float: right;">
            <table style="text-align: right;">
                <tr>
                    <td colspan="2" style="border: 0px;">
                        <?php echo $copy['name'] ?>
                    </td>
                </tr>
            </table>
        </div>
        <div style="clear:both"></div>
    </div>
</div>


<?php if (!isset($dont_print) && !$dont_print) { ?>
    <script>
        if (window.location == window.parent.location) {
            window.print();
        }
    </script>
<?php } ?>