<?php
/* @var $invoice IncomeInvoice */
set_page_title($invoice->getObjectName());

$print_options = array(
    'ignore_discount' => true,
    'labels' => array(
        'discounts' => 'Descuentos',
        'labor' => 'Mano de obra',
        'subtotal_labor' => 'Subtotal',
        'expenses' => 'Gastos',
        'subtotal_expenses' => 'Subtotal',
        'total' => 'Total',
        'amount' => 'Importe',
        'amount_due' => 'Importe adeudado',
    )
);

$currency = Currencies::instance()->getCurrency($invoice->getCurrencyId());
$currency_symbol = $currency instanceof Currency ? $currency->getSymbol() : "";

$project_member = null;
$project_ot = ObjectTypes::findByName('project');
$project_dimension = Dimensions::findByCode('customer_project');
$invoice_members = $invoice->getMembers();
foreach ($invoice_members as $member) {
    if ($member instanceof Member && $member->getDimensionId() == $project_dimension->getId()) {
        $project_member = $member;
        break;
    }
}

$invoice_template = $invoice->getInvoiceTemplate();
if ($invoice_template instanceof IncomeInvoiceTemplate && $invoice_template->getBreakDownBySubprojects() == 'MULTI') {
    if ($project_member instanceof Member) {
        $projectParent = $project_member->getParentMemberId();
        if ($projectParent > 0) {
            $newParentMember = Members::getMemberById($project_member->getParentMemberId());
            if ($newParentMember instanceof Member) {
                $project_member = $newParentMember;
            }
        }
    }
}

$terms_text = "";
if (class_exists('IncomePaymentTerms')) {

    $payment_term = IncomePaymentTerms::instance()->findById($invoice->getPaymentTermId());
    if (!$payment_term) {
        $payment_term = IncomePaymentTerms::getDefaultPaymentTerm();
    }
    if ($payment_term) {
        $terms_text = $payment_term->getName();
    }
} else {
    // use old custom property
    $payment_period_cp = CustomProperties::getCustomPropertyByCode($invoice->getObjectTypeId(), 'invoice_payment_period');
    if ($payment_period_cp instanceof CustomProperty) {
        $payment_period_cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $payment_period_cp->getId());
        if ($payment_period_cp_val instanceof CustomPropertyValue) {
            $terms_text = $payment_period_cp_val->getValue();
        }
    }
}

$project_and_po_number_text = "";
$inv_po_number_cp = CustomProperties::getCustomPropertyByCode($invoice->getObjectTypeId(), "po_number");
if ($inv_po_number_cp instanceof CustomProperty) {
    $cpval = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $inv_po_number_cp->getId());
    if ($cpval instanceof CustomPropertyValue) {
        $project_and_po_number_text = trim($cpval->getValue());
    }
}
if ($project_and_po_number_text == '' && $project_member instanceof Member) {
    $project_id_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "po_number");
    $po_number_cp_val = "";
    if ($project_id_cp instanceof CustomProperty) {
        $cpval = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $project_id_cp->getId());
        if ($cpval instanceof CustomPropertyValue) {
            $po_number_cp_val = $cpval->getValue();
        }
    }

    $project_and_po_number_text = $project_member->getName();
    if ($po_number_cp_val != "") {
        $project_and_po_number_text .= " / $po_number_cp_val";
    }
}

$due_date_text = '';
if ($invoice->getExpirationDate() instanceof DateTimeValue) {
    $due_date_text = format_date($invoice->getExpirationDate(), null, 0);
}


$billing_from_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'billing_period_from');
$billing_to_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'billing_period_to');
$billing_from_date = '';
$billing_to_date = '';
$cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $billing_from_cp->getId());
if ($cp_val && $cp_val->getValue() != '') {
    try {
        $dt = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $cp_val->getValue());
        $billing_from_date = format_date($dt, null, 0);
    } catch (Exception $e) {
    }
}
$cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $billing_to_cp->getId());
if ($cp_val && $cp_val->getValue() != '') {
    try {
        $dt = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $cp_val->getValue());
        $billing_to_date = format_date($dt, null, 0);
    } catch (Exception $e) {
    }
}

$project_member_complete_name = "";
if ($project_member) {
    $tmp_array = array($project_member);
    build_member_list_text_to_show_in_trees($tmp_array);
    $project_member_complete_name = $project_member->getName();
}

$project_manager_name = "";
if ($project_member) {
    $project_manager_id = get_associated_status_member_id($project_member, Dimensions::findByCode('project_managers'));
    $project_manager = Members::getMemberById($project_manager_id);
    if ($project_manager) $project_manager_name = $project_manager->getName();
}

?>
<style>
    @import url('https://fonts.googleapis.com/css2?family=Montserrat:wght@200;300;400;500;600;700&display=swap');

    * {
        margin: 0;
        padding: 0;
        box-sizing: border-box;
    }

    body {
        font-family: 'Montserrat', sans-serif;
        font-size: 13px;
    }

    table {
        margin: 0 auto;
        border-spacing: 3px;
        width: 100%;
    }

    .tabledata,
    th {
        padding: 5px;
    }

    #page-wrap {
        width: 900px;
        margin: 10px auto;
        padding: 10px 0;
    }

    div.owner-company-logo {
        float: none;
        text-align: center;
    }

    .invoiceLogo {
        /*max-width: 260px;*/
        padding-left: 3px;
        /*max-height: 260px;*/
    }

    .invoiceLogo img {
        /*width: 100%;*/
        max-height: 80px;
    }

    .invoiceContainer {
        width: 100%;
        max-width: 800px;
        margin: auto;
        /* padding: 30px; */
    }

    .invoiceHeader {
        display: flex;
        justify-content: space-between;
        position: relative;
    }

    .headerLine {
        position: absolute;
        width: 100%;
        height: 3px;
        background-color: #2A9D8F;
        -webkit-print-color-adjust: exact !important;
        top: 50px;
    }

    .columnsHeader span {
        display: block;
    }

    .columHeaderItem {
        padding: 25px 3px 0;
    }

    .columHeaderItem h3 {
        margin-bottom: 15px;
    }

    .invoiceTitle {
        background-color: #2A9D8F;
        -webkit-print-color-adjust: exact !important;
        padding: 4px 33px;
        text-align: right;
        color: #ffffff;
        margin-right: -30px;
        min-height: 53px;
        display: flex;
        flex-direction: column;
        justify-content: center;
    }

    .invoiceTitle h2 {
        font-size: 1.4rem;
    }

    .invoiceTitle span {
        font-size: .9rem;
    }

    .invoiceData {
        display: grid;
        /* grid-template-columns: repeat(2, calc(50% - 10px)); */
        grid-gap: 20px;
        margin: 10px 0;
        padding: 3px;
    }

    .invoiceDataItem {
        text-align: right;
    }

    #items {
        /* table-layout: fixed; */
    }

    #items tr th {
        background: #2E4551;
        -webkit-print-color-adjust: exact !important;
        color: white;
    }

    .columnsHeader h3 {
        font-size: 1rem;
        color: #2A9D8F;
    }

    #items tbody tr:nth-child(even) td {
        background: #E6E6E6;
        -webkit-print-color-adjust: exact !important;
    }

    #items tbody tr:nth-child(odd) td {
        background: #FFFFFF;
        -webkit-print-color-adjust: exact !important;
    }

    #items {
        clear: both;
        width: 100%;
        margin: 10px 0 0 0;
    }

    #items th {
        background: #eee;
        -webkit-print-color-adjust: exact !important;
    }

    #items tr.item-row td,
    #items tr td {
        border: 0;
        padding: 5px;
        font-size: 13px;
        /* min-width: 70px; */
    }

    #items td.description {
        word-break: break-word;
        /* width: 100%; */
        /* min-width: 250px; */
    }

    #items .left {
        text-align: left;
    }

    #items .right {
        text-align: right;
    }

    #items .center {
        text-align: center;
    }

    #items .short {
        width: 125px;
    }

    #items td.total-line {
        border-right: 0;
        text-align: right;
        font-weight: bold
    }

    #items td.total-line-tax {
        border-right: 0;
        text-align: right;
    }

    #items td.total-value {
        border-left: 0;
        padding: 5px;
        height: 20px;
        background: none;
        text-align: right;
    }

    #items td.balance {
        background: #eee;
        -webkit-print-color-adjust: exact !important;
    }

    #items td.blank {
        border: 0;
    }

    .item-row td :first-child {
        padding-left: 0;
    }

    .hiddenColumn {
        background: transparent !important;
    }

    .invoiceDataTitle {
        color: #2A9D8F;
        white-space: nowrap;
        font-size: .9rem;
    }

    .invoiceDataValue {
        color: #2E4551;
        font-size: .8rem;
        text-transform: capitalize;
    }

    .corpData {
        display: flex;
        align-items: flex-end;
        flex-direction: column;
        margin-top: 30px;
    }

    .corpDataColumn {
        text-align: right;
        margin: 5px 0;
    }

    .corpDataColumn h3 {
        color: #2A9D8F;
    }

    td.print-invoice-status {
        font-weight: bold;
    }

    td.print-invoice-status.pending {}

    td.print-invoice-status.confirmed {
        background-color: #a8ce80;
        -webkit-print-color-adjust: exact !important;
    }

    td.print-invoice-status.canceled {
        background-color: #222;
        -webkit-print-color-adjust: exact !important;
        color: #eee;
    }

    .billtoTitle {
        padding: 20px 0px 0;
    }

    .billtoTitle h3 {
        margin-bottom: 5px;
    }

    .billtoTitle div {
        margin-bottom: 10px;
    }

    .total-line,
    .total-value,
    .total-line.subtotal,
    .total-value.subtotal,
    .totalLine {
        background: #2A9D8F !important;
        -webkit-print-color-adjust: exact !important;
        color: #ffffff !important;
        font-weight: bold;
        text-transform: uppercase;
    }

    .footerWrapper {
        margin: auto;
        max-width: 800px;
        padding: 0 4px;
    }

    .global-total-value {
        /* font-size: 17px; */
    }

    .right.TotalWidht {
        min-width: 90px;
        white-space: nowrap;
    }


    #items th.center.dateWidht {
        width: 90px;
        min-width: 90px;
    }

    #items .center,
    #items .right {
        width: 90px;
    }

    #items td.countColumn_6 {
        width: 16.6666%;
    }

    #items td.countColumn_5 {
        width: 20%;
    }

    #items td.countColumn_4 {
        width: 25%;
    }

    #items td.countColumn_3 {
        width: 33.3333%;
    }

    #items td.countColumn_2 {
        width: 50%;
    }

    #items td.countColumn_1 {
        width: 100%;
    }

    .maxColumn td {
        width: 10% !important;
        min-width: 10% !important;
    }
</style>

<div id="page-wrap">
    <div class="invoiceContainer">
        <div id="customer">
            <div>
                <?php if ($invoice_options['invoice_logo']) : ?>
                    <figure class="invoiceLogo owner-company-logo">
                        <img src="<?php echo $invoice_options['invoice_logo'] ?>" alt="logo" />
                    </figure>
                <?php endif; ?>
                <header class="invoiceHeader">
                    <div class="headerLine"></div>
                    <div class="columnsHeader columHeaderItem">
                        <h3>
                            <?= $to_pay_information['company_name']; ?>
                        </h3>
                        <div class="owner-company-info">
                            <span>
                                <?= $to_pay_information['address']; ?>
                            </span>
                            <span>
                                <?= $to_pay_information['address_location']; ?>
                            </span>
                            <?php if ($to_pay_information['phone']) : ?>
                                <span>
                                    <?= lang('phone') . ": " . $to_pay_information['phone']; ?>
                                </span>
                            <?php endif; ?>
                            <span><?= $to_pay_information['email']; ?></span>
                            <span><?= $to_pay_information['website']; ?></span>
                        </div>
                        <div class="columnsHeader columHeaderItem billtoTitle">
                            <h3> <?= lang('bill to'); ?> </h3>
                            <div>
                                <span><?php echo $invoice->getCustomerLegalName(); ?></span>
                                <span><?php echo $invoice->getAttn(); ?></span>
                                <span><?php echo $invoice->getCustomerAddress(); ?></span>
                                <span>
                                    <?php
                                    $city_state_str =  $invoice->getCustomerCity();
                                    $city_state_str .= ($city_state_str == "" ? "" : ", ") . trim($invoice->getCustomerState() . " " .  $invoice->getCustomerZipcode());
                                    echo $city_state_str;
                                    ?>
                                </span>
                                <?php if ($invoice->getCustomerPhone()) : ?>
                                    <span><?= lang('phone') . ": " . $invoice->getCustomerPhone(); ?></span>
                                <?php endif; ?>
                            </div>
                        </div>
                    </div>
                    <div class="columnsHeader">
                        <div class="invoiceTitle">
                            <h2><?php echo strtoupper($invoice->getInvoiceType()) ?></h2>
                            <?php if ($invoice->getInvoiceNumber()) : ?>
                                <span>#<?php echo trim($invoice->getSeries() . " " . $invoice->getInvoiceNumber()) ?></span>
                            <?php endif; ?>
                        </div>
                        <section class="invoiceData">
                            <div class="invoiceDataItem">
                                <h4 class="invoiceDataTitle"><?= lang('date'); ?></h4>
                                <span class="invoiceDataValue"><?= format_date($invoice->getDate()->advance(logged_user()->getUserTimezoneValue() * -1, false)); ?></span>
                            </div>
                            <div class="invoiceDataItem">
                                <h4 class="invoiceDataTitle"><?= lang('expiration date'); ?></h4>
                                <span class="invoiceDataValue"><?= $due_date_text ?></span>
                            </div>
                            <!-- <div class="invoiceDataItem">
                                <h4 class="invoiceDataTitle">Customer ID</h4>
                                <span class="invoiceDataValue"><?php // $invoice->getCustomerId(); 
                                                                ?></span>
                            </div> -->
                            <div class="invoiceDataItem">
                                <h4 class="invoiceDataTitle">Estado</h4>
                                <span class="invoiceDataValue"><?= lang($invoice->getStatus()); ?></span>
                            </div>
                        </section>
                    </div>
                </header>

            </div>

        </div>

        <div style="clear:both"></div>


        <div class="project-table">
            <table id="items">
                <tr>
                    <th><b><?php echo strtoupper(lang('property')) ?></b></th>
                    <!-- <th><b><?php //echo strtoupper(lang('project manager')) 
                                ?></b></th> -->
                    <!-- <th><b><?php echo strtoupper(lang('terms')) ?></b></th>
                    <th><b><?php echo strtoupper(lang('due date')) ?></b></th> -->
                </tr>
                <tr>
                    <td><?php echo $project_member_complete_name ?></td>
                    <!-- <td><?php //echo $project_manager_name 
                                ?></td> -->
                    <!-- <td style="text-align: center;"><?php echo $terms_text ?></td>
                    <td style="text-align: center;"><?php echo $due_date_text ?></td>
                </tr> -->
            </table>
        </div>

        <div style="clear:both"></div>

        <?php
        if ($invoice_template instanceof IncomeInvoiceTemplate) : ?>
            <div style="clear:both"></div>
            <?= $invoice_template->getHtmlDetailsTableForInvoice($invoice, $print_options); ?>
        <?php else : ?>
            <table id="items" style="margin:10px 0 0 0;">

                <tr>
                    <th style="width:90px;"><?php echo strtoupper(lang('quantity')) ?></th>
                    <th><?php echo strtoupper(lang('description')) ?></th>
                    <th style="width:110px;"><?php echo strtoupper(lang('rate')) ?></th>
                    <th style="width:110px;"><?php echo strtoupper(lang('amount')) ?></th>
                </tr>
                <?php
                $lines_left = 20;
                foreach ($invoice->getInvoiceLines() as $invoice_line) :
                    $line_currency = Currencies::instance()->getCurrency($invoice_line->getCurrencyId());
                    $line_currency_symbol = $line_currency instanceof Currency ? $line_currency->getSymbol() : "";
                    $lines_left--;
                ?>
                    <tr class="item-row">
                        <td align="center"><?php echo $invoice_line->getAmount() ?></td>
                        <td class="description"><?php echo nl2br($invoice_line->getDetail()) ?></td>
                        <td align="right"><?php echo $line_currency_symbol . ' ' . number_format($invoice_line->getUnitValue(), 2) ?></td>
                        <td align="right"><?php echo $line_currency_symbol . ' ' . number_format($invoice_line->getTotal(), 2) ?></td>
                    </tr>
                <?php endforeach; ?>
                <?php while ($lines_left > 0) :
                    $lines_left--;
                ?>
                    <!-- <tr class="item-row">
                        <td>&nbsp;</td>
                        <td class="description">&nbsp;</td>
                        <td></td>
                        <td></td>
                    </tr> -->
                <?php endwhile; ?>
                <tr>
                    <td colspan="2"><b>¡Gracias por hacer negocios con nosotros!</b></td>
                    <td class="total-line">Total</td>
                    <td class="total-value">
                        <div id="total"><?php echo $currency_symbol . ' ' . number_format($invoice->getTotal(), 2) ?></div>
                    </td>
                </tr>
            </table>
            <div class="clear"></div>
        <?php endif; ?>

        <div style="margin: 10px 0 0 0;">
            <div style="float: left;">
                <table style="text-align: left; margin-right:20px;">
                    <?php
                    if ($invoice_options['view_press_information']) :
                    ?>
                        <tr>
                            <td colspan="2" style="border: 0px;">
                                <?php echo nl2br($invoice_notebook['press_information']) ?>
                            </td>
                        </tr>
                    <?php
                    endif;
                    ?>
                </table>
            </div>

            <div style="clear:both"></div>
        </div>
    </div>
    <div class="footerWrapper">
        <div class="footerContent">
            <?php
            $footer_disclaimer = income_config_option('invoice_footer_disclaimer');
            $footer_disclaimer = $invoice_template->getIncludeFooterText();
            echo $footer_disclaimer;
            ?>
        </div>
    </div>
</div>

<?php if (!isset($dont_print) || !$dont_print) { ?>
    <script>
        if (window.location == window.parent.location) {
            window.print();
        }
    </script>
<?php } ?>