<?php
/* @var $invoice IncomeInvoice */
set_page_title($invoice->getObjectName());

$footer_disclaimer = income_config_option('invoice_footer_disclaimer');

$currency = Currencies::instance()->getCurrency($invoice->getCurrencyId());
$currency_symbol = $currency instanceof Currency ? $currency->getSymbol() : "";

$project_member = null;
$project_ot = ObjectTypes::findByName('project');
$invoice_members = $invoice->getMembers();
foreach ($invoice_members as $member) {
    if ($member instanceof Member && $member->getObjectTypeId() == $project_ot->getId()) {
        $project_member = $member;
        break;
    }
}

$invoice_template = $invoice->getInvoiceTemplate();
if ($invoice_template instanceof IncomeInvoiceTemplate && $invoice_template->getBreakDownBySubprojects() == 'MULTI') {
    if ($project_member instanceof Member) {
        $projectParent = $project_member->getParentMemberId();
        if ($projectParent > 0) {
            $newParentMember = Members::getMemberById($project_member->getParentMemberId());
            if ($newParentMember instanceof Member) {
                $project_member = $newParentMember;
            }
        }
    }
}

$billing_from_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'billing_period_from');
$billing_to_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'billing_period_to');
$billing_from_date = '';
$billing_to_date = '';
$cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $billing_from_cp->getId());
if ($cp_val && $cp_val->getValue() != '') {
    try {
        $dt = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $cp_val->getValue());
        $billing_from_date = format_date($dt, null, 0);
    } catch (Exception $e) {
    }
}
$cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $billing_to_cp->getId());
if ($cp_val && $cp_val->getValue() != '') {
    try {
        $dt = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $cp_val->getValue());
        $billing_to_date = format_date($dt, null, 0);
    } catch (Exception $e) {
    }
}


$terms_text = "";
$payment_period_cp = CustomProperties::getCustomPropertyByCode($invoice->getObjectTypeId(), 'invoice_payment_period');
if ($payment_period_cp instanceof CustomProperty) {
    $payment_period_cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $payment_period_cp->getId());
    if ($payment_period_cp_val instanceof CustomPropertyValue) {
        $terms_text = $payment_period_cp_val->getValue();
    }
}

if ($project_member instanceof Member) {
    $project_id_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "project_id");
    $project_id_cp_val = "";
    if ($project_id_cp instanceof CustomProperty) {
        $tmp_project_obj_id = $project_member->getObjectId();

        $tmp_mem = $project_member;
        while ($project_id_cp_val == "" && $tmp_project_obj_id > 0) {
            $cpval = CustomPropertyValues::getCustomPropertyValue($tmp_project_obj_id, $project_id_cp->getId());
            if ($cpval instanceof CustomPropertyValue) {
                $project_id_cp_val = $cpval->getValue();
            }
            if ($project_id_cp_val == "") {
                $parent_member = $tmp_mem->getParentMember();
                if ($parent_member instanceof Member) $tmp_project_obj_id = $parent_member->getObjectId();
                else $tmp_project_obj_id = 0;

                $tmp_mem = $parent_member;
            }
        }
    }

    // PO Number
    $po_number_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "po_number");
    $project_po_number = "";
    if ($po_number_cp instanceof CustomProperty && $project_member instanceof Member) {
        $cpval = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $po_number_cp->getId());
        if ($cpval instanceof CustomPropertyValue) {
            $project_po_number = $cpval->getValue();
        }
    }

    // description
    $project_description = "";
    $po_desc_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "description_special");
    if ($po_desc_cp instanceof CustomProperty && $project_member instanceof Member) {
        $cpval = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $po_desc_cp->getId());
        if ($cpval instanceof CustomPropertyValue) {
            $project_description = $cpval->getValue();
        }
    }

    // manager
    $project_member_manager = '';
    $manager_member = null;
    $managers_dim = Dimensions::findByCode('project_managers');
    // if invoice is classified in a manager then use that member
    foreach ($invoice->getMembers() as $inv_mem) {
        if ($inv_mem->getDimensionId() == $managers_dim->getId()) {
            $manager_member = $inv_mem;
            break;
        }
    }
    // if not classified in any manager then use the manager associated to the project
    if (!$manager_member) {
        $manager_member_id = get_associated_status_member_id($project_member, $managers_dim);
        $manager_member = Members::getMemberById($manager_member_id);
    }
    if ($manager_member instanceof Member) {
        $project_member_manager = $manager_member->getName();
    }
}


?>
<style>
    * {
        margin: 0;
        padding: 0;
    }

    @media print {
        .account-summary {
            page-break-inside: avoid;
        }
    }

    body {
        font: 12px/1.2 Sans-Serif;
    }

    #page-wrap {
        width: 100%;
        max-width: 793.7px;
        /* max-width: 900px; */
        margin: auto;
        padding: 20px 15px;
        position: relative;
    }

    table {
        border-collapse: collapse;
    }

    table td,
    table th {
        /*border: 1px solid #555;*/
        padding: 3px;
        font-size: 14px;
    }

    #customer {
        overflow: hidden;
    }

    #logo {
        text-align: right;
        float: right;
        position: relative;
        margin-top: 25px;
        border: 1px solid #fff;
        max-width: 540px;
        max-height: 100px;
        overflow: hidden;
    }

    #customer-title {
        font-size: 20px;
        font-weight: bold;
        float: left;
    }

    #meta {
        width: 300px;
        float: right;
    }

    #meta td {
        text-align: right;
    }

    #meta td.meta-head {
        text-align: left;
        background: #eee;
    }

    #meta td {
        text-align: right;
    }

    #items {
        clear: both;
        width: 100%;
        margin: 10px 0 0 0;
    }

    #items th {
        color: navy;
        padding: 5px 3px;
        border-top: 3px solid navy;
        border-bottom: 2px solid navy;
        white-space: nowrap;
    }

    #items tr.item-row td {
        border: 0;
        vertical-align: top;
    }

    #items td.description {
        /*width: 300px;*/
    }

    #items td.user {
        white-space: nowrap;
    }

    #items .right {
        text-align: right;
    }

    #items .left {
        text-align: left;
    }

    #items .center {
        text-align: center;
    }

    #items .short {
        width: 90px;
    }

    #items .total-value.short {
        min-width: 110px;
    }

    #items td.total-line,
    td.totalLine .global-total-line {
        text-align: right;
        font-weight: bold;
        font-size: 130%;
        padding-right: 5px;
        display: flex;
        align-items: center;
        margin-right: 0 !important;
    }

    #items td.total-line.subtotal,
    td.totalLine .global-subtotal-line {
        text-align: right;
        font-weight: normal;
        font-size: 14px;
        display: flex;
        align-items: center;
        margin-right: 5px !important;
    }

    #items td.total-line-tax {
        border-right: 0;
        text-align: right;
    }

    #items td.total-value,
    td.totalLine .global-total-value {
        padding: 5px 3px 5px 5px;
        height: 24px;
        background: none;
        text-align: right;
        font-weight: bold !important;

        font-size: 130%;
        color: darkred;
        border-top: 5px double #555;
        border-bottom: 5px double #555;
        min-width: 120px;
    }

    #items td.total-value.subtotal,
    td.totalLine .global-subtotal-value {
        font-size: 14px;
        color: #333;
        border-top: 1px solid #555;
        border-bottom: 1px solid #555;
        font-weight: bold;
        min-width: 120px;
        text-align: right;
        padding: 5px 3px 5px 5px;
    }

    td.totalLine .global-subtotal-value.lineNotImportant {
        font-weight: normal !important;
    }

    #items td.balance {
        background: #eee;
    }

    #items td.blank {
        border: 0;
    }

    #items td.due {
        font-size: 17px;
    }

    #terms {
        text-align: center;
        margin: 20px 0 0 0;
    }

    #terms h5 {
        text-transform: uppercase;
        font: 13px Sans-Serif;
        letter-spacing: 10px;
        border-bottom: 1px solid #555;
        padding: 0 0 8px 0;
        margin: 0 0 8px 0;
    }

    #terms {
        width: 100%;
        text-align: center;
    }

    td.print-invoice-status {
        font-weight: bold;
    }

    td.print-invoice-status.pending {}

    td.print-invoice-status.confirmed {
        background-color: #a8ce80;
    }

    td.print-invoice-status.canceled {
        background-color: #222;
        color: #eee;
    }


    div.owner-company-info {
        margin: 0 5px 5px 5px;
    }

    /* div.project-table {
        float: left;
        width: 49%;
        margin: 30px 0 20px 0px;
        max-width: 400px;
        position: relative;
        left: 350px;
        top: 30px;
    } */

    div.project-table {
        position: absolute;
        width: 389.2913px;
        /* background: blue; */
        height: 140px;
        right: 0;
        display: flex;
        align-items: center;
    }

    div.project-table table {
        width: 100%;
    }

    div.project-table .label {
        color: navy;
        min-width: 110px;
    }

    div.project-table td {
        padding: 1px 3px;
        vertical-align: top;
    }

    /* div.bill-to {
        float: left;
        width: 49%;
        margin: 30px 0 20px 0px;
        padding-left: 24px;
        padding-top: 12px;
    } */

    div.bill-to {
        position: absolute;
        top: 230.6693px;
        margin: 0;
        left: 79.37px;
        padding: 0;
        /* background: yellow; */
        width: 325.0386px;
    }

    div.bill-to table {
        width: 100%;
    }

    div.bill-to td {
        padding: 1px 3px;
    }

    div.account-summary {
        background-color: #ddd;
        text-align: center;
        margin-top: 20px;
        padding: 5px 2px;
        border: 2px solid #aaa;
    }

    div.account-summary .total-due {
        text-align: right;
        padding: 5px;
        font-size: 14px;
    }

    div.account-summary .title {
        font-size: 16px;
    }

    div.account-summary table {
        width: 100%;
    }

    div.account-summary th {
        border: 1px solid #555;
        color: navy;
        font-weight: normal;
        font-size: 12px;
    }

    div.account-summary td {
        border: 1px solid #555;
        font-size: 12px;
    }



    table.company-contact {
        width: 100%;
        text-align: center;
    }

    div.footer-info {
        margin-top: 10px;
        /*font-family: "Times New Roman";*/
        text-align: center;
        font-size: 15px;
        font-weight: bold;
    }


    table.header {
        width: 100%;
    }

    table.header td {
        width: 33%;
        vertical-align: top;
    }

    table.header td.logo {
        text-align: center;
    }

    table.header td table.header-invoice-info td {
        border: 1px solid #555;
        padding: 3px;
    }

    .items-group-name {
        font-size: 125%;
        margin-top: 12px;
        font-weight: bold;
        border-top: 1px dotted #000;
        padding-top: 12px;
    }

    .project-desc {
        font-size: 12px;
        border: 2px solid #aaa;
        margin: 10px 0px 5px;
        padding: 2px;
    }

    .invoiceAddress {
        white-space: pre-line;
    }

    td.totalLine {
        padding: 3px 0 3px 3px !important;
    }
    .TotalWidht { white-space: nowrap; }
    #items .right { white-space: nowrap; }
    
    .TotalWidht {
        width: 112px;
    }
    .dateWidht {
        max-width: 70px;
        width: 70px;
    }
   
</style>



<div id="page-wrap">
    <div id="customer">
        <table class="header">
            <tr>
                <td>

                    <div class="owner-company-info">
                        <div>2105 South C Street</div>
                        <div>Tacoma, WA 98402</div>
                        <div>Phone: 253-475-7711</div>
                        <div>Fax: 253-472-5846</div>
                        <div>www.robinson-noble.com</div>
                    </div>

                </td>
                <td class="logo">

                    <div>
                        <img src="<?php echo $invoice_options['invoice_logo'] ?>" alt="logo" style="height:150px;" />
                    </div>

                </td>
                <td>

                    <div style="float: right;">
                        <table class="header-invoice-info">
                            <tr>
                                <td style="text-align:center; font-weight:bold; font-size:12px; min-width:100px;"><?php echo 'Invoice date' ?></td>
                                <td style="text-align:center; font-weight:bold; font-size:12px; min-width:100px;"><?php echo 'Invoice number' ?></td>
                            </tr>
                            <tr>
                                <td style="text-align:center; font-size:12px; "><?php echo format_date($invoice->getDate()->advance(logged_user()->getUserTimezoneValue() * -1, false)); ?></td>
                                <td style="text-align:center; font-size:12px; "><?php echo trim($invoice->getSeries() . " " . $invoice->getInvoiceNumber()) ?></td>
                            </tr>
                            <tr>
                                <td style="text-align:center; font-weight:bold; font-size:12px; min-width:100px;"><?php echo 'Billing from' ?></td>
                                <td style="text-align:center; font-weight:bold; font-size:12px; min-width:100px;"><?php echo 'Billing to' ?></td>
                            </tr>
                            <tr>
                                <td style="text-align:center; font-size:12px; "><?php echo $billing_from_date ?></td>
                                <td style="text-align:center; font-size:12px; "><?php echo $billing_to_date ?></td>
                            </tr>
                        </table>
                    </div>
                </td>
            </tr>
        </table>
    </div>

    <div class="bill-to">
        <?php
        $bill_to_company = $invoice->getCustomerName(); // $bill_to_information['company_name'];
        $bill_to_contact = $invoice->getAttn();
        ?>
        <table>
            <?php if (trim($bill_to_contact) != '' && trim(strtolower($bill_to_company)) != trim(strtolower($bill_to_contact))) { ?>
                <tr>
                    <td>
                        <div><?php echo $bill_to_contact; ?></div>
                    </td>
                </tr>
            <?php } ?>
            <tr>
                <td>
                    <div><?php echo $bill_to_company; ?></div>
                </td>
            </tr>
            <tr>
                <td>
                    <div style="white-space: pre-line;"><?php echo $invoice->getCustomerAddress(); ?></div>
                    <!--$bill_to_information['address']-->
                </td>
            </tr>
            <tr>
                <td>
                    <div><?php
                            $city_state_str = $invoice->getCustomerCity(); //$bill_to_information['city']
                            $city_state_str .= ($city_state_str == "" ? "" : ", ") . trim($invoice->getCustomerState() . " " . $invoice->getCustomerZipcode());
                            echo $city_state_str;
                            ?></div>
                </td>
            </tr>
        </table>
    </div>

    <div style="font-size: 300%; text-align:center; margin-top: 160px;">INVOICE</div>

    <?php if (trim($invoice->getAdditionalInformation())) { ?>
        <div class="project-desc"><?php echo nl2br($invoice->getAdditionalInformation()) ?></div>
    <?php } ?>

    <div style="clear:both"></div>

    <?php /* @var $invoice IncomeInvoice */

    //if joint invoice use the summary for the main project
    if ($invoice_template instanceof IncomeInvoiceTemplate && $invoice_template->getBreakDownBySubprojects() == 'MULTI') {
        $account_summary = $invoice->getSummaryForMember($project_member);
    } else {
        $account_summary = $invoice->getProjectSummary();
    }

    if ($invoice_template instanceof IncomeInvoiceTemplate) {

        $print_options = array(
            'labels' => array(
                'labor' => 'Services',
                'subtotal_labor' => 'Subtotal Services',
                'subtotal_expenses' => 'Subtotal Expenses',
                'amount_due' => 'Amount Due',
            )
        );

        $include_account_summay = $invoice_template->getIncludeAccountSummary();

        $footer_disclaimer = $invoice_template->getIncludeFooterText();

        echo $invoice_template->getHtmlDetailsTableForInvoice($invoice);
    } else {
    ?>

        <table id="items">

            <tr>
                <th style="width:335px;"><?php echo strtoupper(lang('description')) ?></th>
                <th style="width:70px;"><?php echo strtoupper('quantity') ?></th>
                <th style="width:90px;"><?php echo strtoupper('rate') ?></th>
                <th style="width:90px;"><?php echo strtoupper(lang('amount')) ?></th>
            </tr>
            <?php
            $lines_left = 20;
            foreach ($invoice->getInvoiceLines() as $invoice_line) :
                $line_currency = Currencies::instance()->getCurrency($invoice_line->getCurrencyId());
                $line_currency_symbol = $line_currency instanceof Currency ? $line_currency->getSymbol() : "";
                $lines_left--;
            ?>
                <tr class="item-row">
                    <td class="description"><?php echo nl2br($invoice_line->getDetail()) ?></td>
                    <td align="center"><?php echo $invoice_line->getAmount() ?></td>
                    <td align="right"><?php echo $line_currency_symbol . ' ' . number_format($invoice_line->getUnitValue(), 2) ?></td>
                    <td align="right"><?php echo $line_currency_symbol . ' ' . number_format($invoice_line->getTotal(), 2) ?></td>
                </tr>
            <?php endforeach; ?>
            <?php while ($lines_left > 0) :
                $lines_left--;
            ?>
                <tr class="item-row">
                    <td>&nbsp;</td>
                    <td class="description">&nbsp;</td>
                    <td></td>
                    <td></td>
                </tr>
            <?php endwhile; ?>
            <tr>
                <td colspan="2"></td>
                <td class="total-line">Total</td>
                <td class="total-value">
                    <div id="total"><?php echo $currency_symbol . ' ' . number_format($invoice->getTotal(), 2) ?></div>
                </td>
            </tr>
            <tr>
                <td colspan="2"></td>
                <td class="total-line">Payments/Credits</td>
                <td class="total-value">
                    <div id="total"><?php echo $currency_symbol . ' ' . "0" ?></div>
                </td>
            </tr>
            <tr>
                <td colspan="2" style="padding:0;"></td>
                <td class="total-line due">Balance Due</td>
                <td class="total-value due">
                    <div id="total"><?php echo $currency_symbol . ' ' . number_format($invoice->getTotal(), 2) ?></div>
                </td>
            </tr>
        </table>
    <?php } ?>


    <?php if ($include_account_summay) { ?>
        <div class="account-summary">
            <div class="title">Account Summary</div>
            <table>

                <tr>
                    <th>Contract Amt</th>
                    <th>Services BTD</th>
                    <th>Expenses BTD</th>
                    <th>Last Inv Num</th>
                    <th>Last Inv Date</th>
                    <th>Last Inv Amt</th>
                    <th>Last Pay Amt</th>
                    <th>Prev Unpaid Amt</th>
                </tr>

                <tr>
                    <td><?php
                        if ($account_summary && $account_summary['contract_amount']) {
                            echo format_money_amount($account_summary['contract_amount'], $account_summary['contract_amount_currency_symbol']);
                        }
                        ?></td>
                    <td><?php
                        if ($account_summary && $account_summary['totals_btd']) {
                            foreach ($account_summary['totals_btd'] as $currency_id => $data) {
                                echo format_money_amount($data['total_labor'], $data['currency_symbol']) . "<br />";
                            }
                        }
                        ?></td>
                    <td><?php
                        if ($account_summary && $account_summary['totals_btd']) {
                            foreach ($account_summary['totals_btd'] as $currency_id => $data) {
                                echo format_money_amount($data['total_expenses'], $data['currency_symbol']) . "<br />";
                            }
                        }
                        ?></td>
                    <td><?php
                        if ($account_summary && $account_summary['last_invoice']) {
                            echo $account_summary['last_invoice']['number'];
                        } else {
                            echo '--';
                        }
                        ?></td>
                    <td><?php
                        if ($account_summary && $account_summary['last_invoice']) {
                            echo $account_summary['last_invoice']['date'];
                        } else {
                            echo '--';
                        }
                        ?></td>
                    <td><?php
                        if ($account_summary && $account_summary['last_invoice']) {
                            echo format_money_amount($account_summary['last_invoice']['amount'], $account_summary['last_invoice']['currency_symbol']);
                        }
                        ?></td>
                    <td><?php
                        if ($account_summary && $account_summary['last_paid_amount']) {
                            echo format_money_amount($account_summary['last_paid_amount']['amount'], $account_summary['last_paid_amount']['currency_symbol']);
                        }
                        ?></td>
                    <td><?php
                        if ($account_summary && $account_summary['totals_btd']) {
                            foreach ($account_summary['totals_btd'] as $currency_id => $data) {
                                echo format_money_amount($data['prev_due_amount'], $data['currency_symbol']) . "<br />";
                            }
                        }
                        ?></td>
                </tr>

            </table>
            <div class="total-due">
                Total amount due including this invoice: <?php
                                                            if ($account_summary && $account_summary['totals_btd']) {
                                                                foreach ($account_summary['totals_btd'] as $currency_id => $data) {
                                                                    echo format_money_amount($data['total_due_amount'], $data['currency_symbol']) . "<br />";
                                                                }
                                                            } else {
                                                                echo format_money_amount($invoice->getTotal(), $currency_symbol);
                                                            }
                                                            ?>
            </div>
        </div>
    <?php } ?>

    <div class="footer-info"><?php echo $footer_disclaimer ?></div>

</div>


<?php if (!isset($dont_print) || !$dont_print) { ?>
    <script>
        if (window.location == window.parent.location) {
            //window.print();
        }
    </script>
<?php } ?>