<?php
/* @var $invoice IncomeInvoice */
set_page_title($invoice->getObjectName());

$currency = Currencies::instance()->getCurrency($invoice->getCurrencyId());
$currency_symbol = $currency instanceof Currency ? $currency->getSymbol() : "";

$project_member = null;
$project_ot = ObjectTypes::findByName('project');
$invoice_members = $invoice->getMembers();
foreach ($invoice_members as $member) {
    if ($member instanceof Member && $member->getObjectTypeId() == $project_ot->getId()) {
        $project_member = $member;
        break;
    }
}

$invoice_template = $invoice->getInvoiceTemplate();
if ($invoice_template instanceof IncomeInvoiceTemplate && $invoice_template->getBreakDownBySubprojects() == 'MULTI') {
    if ($project_member instanceof Member) {
        $projectParent = $project_member->getParentMemberId();
        if ($projectParent > 0) {
            $newParentMember = Members::getMemberById($project_member->getParentMemberId());
            if ($newParentMember instanceof Member) {
                $project_member = $newParentMember;
            }
        }
    }
}

$terms_text = "";
$payment_period_cp = CustomProperties::getCustomPropertyByCode($invoice->getObjectTypeId(), 'invoice_payment_period');
if ($payment_period_cp instanceof CustomProperty) {
    $payment_period_cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $payment_period_cp->getId());
    if ($payment_period_cp_val instanceof CustomPropertyValue) {
        $terms_text = $payment_period_cp_val->getValue();
    }
}

$work_performed_cp = CustomProperties::instance()->getCustomPropertyByCode($invoice->getObjectTypeId(), 'work_performed');
$work_performed = "";
if ($work_performed_cp instanceof CustomProperty) {
    $cp_val = CustomPropertyValues::getCustomPropertyValue($invoice->getId(), $work_performed_cp->getId());
    if ($cp_val instanceof CustomPropertyValue) {
        $work_performed = $cp_val->getValue();
    }
}

$project_manager_name = "";
if ($project_member instanceof Member) {
    $project_managers_dim = Dimensions::findByCode('project_managers');
    $pm_id = get_associated_status_member_id($project_member, $project_managers_dim);
    if ($pm_id > 0) {
        $pm_member = Members::instance()->findById($pm_id);
        if ($pm_member instanceof Member) {
            $project_manager_name = $pm_member->getName();
        }
    }
}
?>
<style>
    * {
        margin: 0;
        padding: 0;
    }

    body {
        font: 14px/1.2 Times New Roman;
    }

    #page-wrap {
        width: 736px;
        margin: 10px auto;
        padding: 10px 0;
    }

    table {
        border-collapse: collapse;
    }

    table td,
    table th {
        border: 1px solid #555;
        padding: 3px;
        font-size: 14px;
    }

    table th {
        font-size: 12px;
    }

    #customer {
        overflow: hidden;
    }

    #logo {
        text-align: right;
        float: right;
        position: relative;
        margin-top: 25px;
        border: 1px solid #fff;
        max-width: 540px;
        max-height: 100px;
        overflow: hidden;
    }

    #customer-title {
        font-size: 20px;
        font-weight: bold;
        float: left;
    }

    #meta {
        width: 300px;
        float: right;
    }

    #meta td {
        text-align: right;
    }

    #meta td.meta-head {
        text-align: left;
        background: #eee;
    }

    #meta td {
        text-align: right;
    }

    #items {
        clear: both;
        width: 100%;
        margin: 10px 0 0 0;
        border: 1px solid #555;
    }

    #items th {
        background: #eee;
    }

    #items tr.item-row td {
        border: 0;
        vertical-align: top;
        border-right: 1px solid #555;
    }

    #items td.description {
        width: 300px;
    }

    #items td.total-line {
        border-right: 0;
        text-align: center;
        font-weight: bold
    }

    #items td.total-line-tax {
        border-right: 0;
        text-align: right;
    }

    #items td.total-value {
        border-left: 0;
        padding: 5px;
        height: 15px;
        background: none;
        text-align: right;
        font-weight: bold;
    }

    #items td.balance {
        background: #eee;
    }

    #items td.blank {
        border: 0;
    }

    #terms {
        text-align: center;
        margin: 20px 0 0 0;
    }

    #terms h5 {
        text-transform: uppercase;
        font: 13px Sans-Serif;
        letter-spacing: 10px;
        border-bottom: 1px solid #555;
        padding: 0 0 8px 0;
        margin: 0 0 8px 0;
    }

    #terms {
        width: 100%;
        text-align: center;
    }

    td.print-invoice-status {
        font-weight: bold;
    }

    td.print-invoice-status.pending {}

    td.print-invoice-status.confirmed {
        background-color: #a8ce80;
    }

    td.print-invoice-status.canceled {
        background-color: #222;
        color: #eee;
    }

    div.bill-to {
        margin: 10px 5px;
        float: left;
    }

    div.bill-to div {
        margin: 4px 0px;
    }

    div.owner-company-info {
        margin: 0 5px 5px 5px;
    }

    div.payment-warning-table {
        float: right;
        margin: 10px 5px;
        width: 220px;

    }

    div.payment-warning-table table {
        width: 100%;
    }

    table.items-table {
        width: 550px;
        margin-left: 100px;
    }

    table.items-table td {
        border: 0px none;
    }

    table.items-table td span {
        background-color: #FFF;
    }

    table.items-table td.name:before {
        clip: rect(0px, 500px, 20px, 0px);
        content: " ........................................................................................................................................................................ ";
        position: absolute;
        z-index: -1;
    }

    table.items-table td.amt {
        text-align: right;
    }

    table.items-table tr.total-line td {
        font-weight: bold;
        border: 1px solid #333;
    }

    table.items-table tr.total-line td.name {
        border-right: 0px none;
    }

    table.items-table tr.total-line td.amt {
        border-left: 0px none;
    }

    .clear {
        clear: both;
    }

    .right.TotalWidht {
        white-space: nowrap;
    }
</style>



<div id="page-wrap">
    <div id="customer">
        <div style="width: 100%;">
            <?php if ($invoice_options['invoice_logo']) : ?>
                <div>
                    <img src="<?php echo $invoice_options['invoice_logo'] ?>" alt="logo" />
                </div>
            <?php endif; ?>
        </div>
        <div style="margin-top:40px; border-bottom:3px double #333; height: 20px;">
            <div style="float:left; margin-left:5px;">
                <?php echo format_date($invoice->getDate()->advance(logged_user()->getUserTimezoneValue() * -1, false), "F j, Y"); ?>
            </div>
            <div style="float:right; margin-right:5px;">
                <?php echo $invoice->getInvoiceType() . " # " . trim($invoice->getSeries() . " " . $invoice->getInvoiceNumber()) ?>
            </div>
            <div class="clear"></div>
        </div>
        <div style="text-align:center; font-weight:bold; font-size:125%; margin-top:10px;">
            <?php echo strtoupper($invoice->getInvoiceType()) ?>
        </div>
    </div>

    <div class="bill-to">
        <div><?php echo $bill_to_information['company_name']; ?></div>
        <div><?php echo $bill_to_information['address']; ?></div>
        <div><?php
                $city_state_str = $bill_to_information['city'];
                $city_state_str .= ($city_state_str == "" ? "" : ", ") . trim($bill_to_information['state'] . " " . $bill_to_information['zip_code']);
                echo $city_state_str;
                ?></div>

        <?php if ($project_manager_name != "") { ?>
            <br />
            <div>Project Coordinator: <?php echo $project_manager_name ?></div>
        <?php } ?>
    </div>

    <div class="payment-warning-table">
        <table>
            <tr>
                <td>
                    Payment is due within 30 days of this
                    invoice. A late payment fee of 2%
                    will be added to the amount due for
                    every 30 days that payment is late.
                </td>
            </tr>
        </table>
    </div>

    <div class="clear"></div>
    <div style="border-bottom:3px double #333; height: 3px;">
    </div>

    <div style="margin-top:15px; height: 20px; font-weight:bold; text-align:center;">
        <?php echo $invoice->getName() ?>
    </div>

    <?php if ($project_member instanceof Member) { ?>
        <div style="margin-top:15px; height: 20px; font-weight:bold;">
            <?php echo "Project: " . $project_member->getName() ?>
        </div>
    <?php } ?>

    <?php if ($work_performed != "") { ?>
        <div style="margin-top:15px;">
            <?php echo "<b>Work performed: </b>" . $work_performed ?>
        </div>
    <?php } ?>

    <div class="clear"></div>
    <div style="border-bottom:3px double #333; height: 3px; margin-top:5px;">
    </div>

    <div style="margin-top:15px; height: 20px;"><b>Fees: </b></div>

    <table class="items-table">
        <?php foreach ($invoice->getInvoiceLines() as $invoice_line) :
            $line_currency = Currencies::instance()->getCurrency($invoice_line->getCurrencyId());
            $line_currency_symbol = $line_currency instanceof Currency ? $line_currency->getSymbol() : "";
        ?>
            <tr>
                <td class="name"><span><?php echo nl2br($invoice_line->getDetail()) ?></span></td>
                <td class="amt"><span><?php echo $line_currency_symbol . ' ' . number_format($invoice_line->getTotal(), 2) ?></span></td>
            </tr>
        <?php endforeach; ?>
        <tr>
            <td>&nbsp;</td>
            <td>&nbsp;</td>
        </tr>
        <tr class="total-line">
            <td class="name"><span>TOTAL AMOUNT DUE</span></td>
            <td class="amt"><span><?php echo $currency_symbol . ' ' . number_format($invoice->getTotal(), 2) ?></span></td>
        </tr>
    </table>


    <div style="font-size:200%; font-weight:bold; margin-top:100px; text-align:center;">
        <img src="<?php echo ROOT_URL ?>/est.png" style="float:left;" />
        <img src="<?php echo ROOT_URL ?>/bbb.png" style="float:right;" />
        <div style="height:25px;"></div>
        <a style="color:#000">www.source-environmental.com</a>
    </div>

    <?php if (!isset($dont_print) || !$dont_print) { ?>
        <script>
            if (window.location == window.parent.location) {
                window.print();
            }
        </script>
    <?php } ?>