/**
 *  BatchInvoicingManager
 *
 */
og.BatchInvoicingManager = function() {
	var actions;
	this.doNotRemove = true;
	this.needRefresh = false;

	this.billing_period = 'last_month';
	this.billing_period_from = null;
	this.billing_period_to = null;
	
	//og.eventManager.addListener('member changed', this.reset, this);
	
	this.fields = [
		'id', 'name', 'date', 'additional_fee', 'subtotal', 'discount', 'total', 'total_raw', 'currency_sym', 'additional_information',
		'customer_name', 'customer_address', 'customer_city', 'customer_state', 'customer_country', 'customer_zipcode', 'customer_email', 
   	];
   	
	if (!og.BatchInvoicingManager.store) {
		og.BatchInvoicingManager.store = new Ext.data.Store({
			proxy: new og.GooProxy({
				url: og.getUrl('invoice', 'list_project_invoices_to_generate')
			}),
			reader: new Ext.data.JsonReader({
				root: 'invoices',
				totalProperty: 'totalCount',
				id: 'id',
				fields: this.fields
			}),
			remoteSort: false,
			listeners: {
				'load': function(store, result) {
					var d = this.reader.jsonData;
					if (d.totalCount == 0) {
						var sel_context_names = og.contextManager.getActiveContextNames();
						if (sel_context_names.length > 0) {
							this.fireEvent('messageToShow', lang("no objects message", lang("invoices"), sel_context_names.join(', ')));
						} else {
							this.fireEvent('messageToShow', lang("no more objects message", lang("invoices")));
						}
					} else {
						this.fireEvent('messageToShow', "");
					}
					
					og.eventManager.fireEvent('replace all empty breadcrumb', null);
				}
			}
		});
		og.BatchInvoicingManager.store.setDefaultSort('name', 'asc');
	}
	this.store = og.BatchInvoicingManager.store;
	this.store.addListener({messageToShow: {fn: this.showMessage, scope: this}});

	function renderName(value, p, r) {
		return String.format('<a style="font-size:120%;" href="#" onclick="return false;">{0}</a>', og.clean(value));
	}
	
	function renderIcon(value, p, r) {
		return '<div class="link-ico ico-invoice"></div>';
	}
	
	function renderInlineDiscountInput(value, p, r) {
		return '<input type="text" name="discount['+r.data.id+']" id="discount_'+r.data.id+'" onchange="og.format_money_amount(this.value, 2);"/>';
	}
	
	function renderInlineFeeInput(value, p, r) {
		return '<input type="text" name="fee['+r.data.id+']" id="fee_'+r.data.id+'" onchange="og.format_money_amount(this.value, 2);"/>';
	}
	
	function renderInlineInput(value, p, r) {
		
	}
	
	function renderAmount(value, p, r) {
		if (!value || typeof(value) == 'undefined') {
			return '';
		}
		
		var amount_prefix = "";
		if (value.toString().indexOf(r.data.currency_sym) < 0) {
			amount_prefix = r.data.currency_sym + " ";
		}
		
		str_val = "<div class='amounts-container'>";
		str_val += "<div class='amount-container'>" + amount_prefix + value + "</div>";
		str_val += "</div>";
		
		return str_val;
	}

	function renderDate(value, p, r) {
		if (!value) return "";
		return value;
	}

	function renderAddress(value, p, r) {
		
		var address_string = r.data.customer_address;
		if (r.data.customer_city) address_string += ', ' + r.data.customer_city;
		if (r.data.customer_state) address_string += ', ' + r.data.customer_state;
		if (r.data.customer_zipcode) address_string += ', ' + r.data.customer_zipcode;
		if (r.data.customer_country) address_string += ', ' + r.data.customer_country;

		return address_string;
	}
	
	this.renderActions = function(value, p, r) {
		return String.format(
				'<a class="link-ico list-action ico-edit" href="#" onclick="og.income.edit_pre_invoice_properties(\'{0}\')" title="{1}">&nbsp;</a>',
				r.data.id, lang('edit'));
	}

	function getSelectedIds() {
		var selections = sm.getSelections();
		if (selections.length <= 0) {
			return '';
		} else {
			var ret = '';
			for (var i=0; i < selections.length; i++) {
				ret += "," + selections[i].data.id;
			}
			return ret.substring(1);
		}
	}
	this.getSelectedIds = getSelectedIds;
	
	function getFirstSelectedId() {
		if (sm.hasSelection()) {
			return sm.getSelected().data.id;
		}
		return '';
	}

	var sm = new Ext.grid.CheckboxSelectionModel();
	sm.on('selectionchange',
		function() {
			if (sm.getCount() <= 0) {
				actions.generate.setDisabled(true);
			} else {
				actions.generate.setDisabled(false);
			}
		});
	
	var cm_info = [
		sm,{
			id: 'icon',
			header: '&nbsp;',
			dataIndex: 'type',
			width: 28,
        	renderer: renderIcon,
        	fixed:true,
        	resizable: false,
        	hideable:false,
        	menuDisabled: true
		},{
			id: 'name',
			header: lang("project"),
			dataIndex: 'name',
			width: 250,
			renderer: renderName,
			sortable: true
		},{
			id: 'customer_name',
			header: lang("customer"),
			dataIndex: 'customer_name',
			renderer: og.clean,
			sortable: true
		},{
			id: 'date',
			header: lang("date"),
			dataIndex: 'date',
			renderer: renderDate,
		},{
			id: 'additional_fee',
			header: lang("adm fee"),
			dataIndex: 'additional_fee',
			renderer: renderAmount,
			align: 'right',
			sortable: true
		},{
			id: 'subtotal',
			header: lang("subtotal"),
			dataIndex: 'subtotal',
			renderer: renderAmount,
			align: 'right',
			sortable: true
		},{
			id: 'discount',
			header: lang("discount"),
			dataIndex: 'discount',
			renderer: renderAmount,
			align: 'right',
			sortable: true
		},{
			id: 'total',
			header: lang("total"),
			dataIndex: 'total',
			renderer: renderAmount,
			align: 'right',
			sortable: true
		},{
			id: 'additional_information',
			header: lang("additional information"),
			dataIndex: 'additional_information',
			renderer: og.clean,
			sortable: true
        },{
			id: 'billing_email',
			header: lang("billing email"),
			dataIndex: 'customer_email',
			renderer: og.clean,
			sortable: true
        },{
			id: 'billing_address',
			header: lang("billing address"),
			dataIndex: 'customer_address',
			renderer: renderAddress,
			sortable: true
        },{
			id: 'actions',
			header: '',
			dataIndex: '',
			width: 50,
			renderer: this.renderActions,
			sortable: false
		}
    ];
	
	
	// create column model
	var cm = new Ext.grid.ColumnModel(cm_info);
	cm.defaultSortable = false;
	
	var uid = Ext.id();

	actions = {

			list: new Ext.Action({
				id: uid + 'list_invoices_btn',
				text: lang('list invoices'),
	            tooltip: lang('list invoices to generate'),
	            iconCls: 'ico-properties',
	            handler: function() {
	            	var man = Ext.getCmp('batch-invoicing-manager');
	            	var params = {};
	            	if (man.billing_period_from) params.billing_period_from = man.billing_period_from;
	            	if (man.billing_period_to) params.billing_period_to = man.billing_period_to;
	            	if (man.billing_period) params.billing_period = man.billing_period;
	            	man.load(params);
				}
			}),
			
			generate: new Ext.Action({
				id: uid + 'generate_invoices_btn',
				text: lang('generate invoices'),
	            tooltip: lang('generate invoices'),
	            iconCls: 'ico-add',
	            disabled: true,
	            handler: function() {
	            	var man = Ext.getCmp('batch-invoicing-manager');
	            	var ids_str = man.getSelectedIds();
	            	
	            	if (ids_str.length > 0) {
	            		
	            		var properties = {};
	            		var ids = ids_str.split(',');
		            	for (var x=0; x<ids.length; x++) {
		            		var member_id = ids[x];
		            		var row = man.store.data.map[member_id];
		            		if (row) {
		            			var data = {};
		            			data.additional_information = row.data.additional_information;
		            			if (!isNaN(row.data.discount)) data.discount = row.data.discount;
		            			if (!isNaN(row.data.additional_fee)) data.additional_fee = row.data.additional_fee;
		            			properties[member_id] = data;
		            		}
		            	}
		            	
		            	var properties_json = Ext.util.JSON.encode(properties);
	            		
	            		og.openLink(og.getUrl('invoice', 'generate_batch_invoices'), {
							post: {
								member_ids: ids_str, 
								properties: properties_json,
								req_channel: 'batch invoicing - generate invoices'
							}
						});

	            	} else {
	            		og.msg(lang('you have to select the invoices to generate'));
	            	}
				}
			})
    };
	
	var period_combo_store_data = [
		['today', lang('today')]
		,['this_week',lang('this week')]
		,['last_week', lang('last week+')]
		,['this_month', lang('this month')]
		,['last_month', lang('last month+')]
		,['select_dates', lang('select dates...')]
	];
	
	this.filters = {
			period: new Ext.form.ComboBox({
		    	id: uid + 'filter_period',
		        store: new Ext.data.SimpleStore({
		        	fields: ['value', 'text'],
		        	data : period_combo_store_data
		    	}),
		        displayField:'text',
		        mode: 'local',
		        triggerAction: 'all',
		        selectOnFocus:true,
		        width:120,
		        valueField: 'value',
		        value: 'last_month',
		        listeners: {
		        	'select' : function(combo, record) {
		        		var man = Ext.getCmp('batch-invoicing-manager');
		        		if (record.data.value == 'select_dates') {
		        			man.filters.filter_from.show();
		        			man.filters.filter_to.show();
		        		} else {
		        			man.filters.filter_from.hide();
		        			man.filters.filter_to.hide();
		        		}
		        		
		        		man.billing_period = record.data.value;
		        	}
		        }
		    }),
		    
			filter_from: new Ext.Action({
				id: uid + 'filter_date_from',
				text: lang('from date'),
		        tooltip: lang('select a date'),
		        hidden: true,
		        menu: new og.drawDateMenuPicker({
		        	items:[new Ext.menu.Item({
		        		id: uid + 'filter_from_remove',
		        		text: lang('remove filter'),
		        		iconCls: 'ico-delete',
		        		hidden: true,
		        		handler: function() {
			        		var man = Ext.getCmp('batch-invoicing-manager');
		        			man.billing_period_from = null;
		        			this.hide();
		        			Ext.getCmp(uid + 'filter_date_from').setText(lang('from date'));
		        		}
		        	})],
		        	listeners: {
		        		'select': function(dp, date) {
				        	var man = Ext.getCmp('batch-invoicing-manager');
			    			man.billing_period_from = date.format(og.preferences['date_format']);
				        	Ext.getCmp(uid + 'filter_date_from').setText(date.format(og.preferences['date_format']));
				        	Ext.getCmp(uid + 'filter_from_remove').show();
		        		}
			        }
		        })
			}),
			
			filter_to: new Ext.Action({
				id: uid + 'filter_date_to',
				text: lang('to date'),
		        tooltip: lang('select a date'),
		        hidden: true,
		        menu: new og.drawDateMenuPicker({
		        	items:[new Ext.menu.Item({
		        		id: uid + 'filter_to_remove',
		        		text: lang('remove filter'),
		        		iconCls: 'ico-delete',
		        		hidden: true,
		        		handler: function() {
			        		var man = Ext.getCmp('batch-invoicing-manager');
		        			man.billing_period_to = null;
		        			this.hide();
		        			Ext.getCmp(uid + 'filter_date_to').setText(lang('to date'));
		        		}
		        	})],
		        	listeners: {
		        		'select': function(dp, date) {
				        	var man = Ext.getCmp('batch-invoicing-manager');
			    			man.billing_period_to = date.format(og.preferences['date_format']);
				        	Ext.getCmp(uid + 'filter_date_to').setText(date.format(og.preferences['date_format']));
				        	Ext.getCmp(uid + 'filter_to_remove').show();
		        		}
			        }
		        })
			})
	}
    
	var tbar = [];
	if (!og.loggedUser.isGuest) {
		tbar.push('&nbsp;'+lang('date range') + ": ");
		tbar.push(this.filters.period);
		tbar.push(' ');
		tbar.push(this.filters.filter_from);
		tbar.push(' ');
		tbar.push(this.filters.filter_to);
		tbar.push('-');
		tbar.push(actions.list);
		tbar.push('-');
		tbar.push(actions.generate);
	}
	
	
	og.BatchInvoicingManager.superclass.constructor.call(this, {
		store: this.store,
		layout: 'fit',
		cm: cm,
		stateful: og.preferences['rememberGUIState'],
		id: 'batch-invoicing-manager',
		stripeRows: true,
		closable: true,
		loadMask: true,
		bbar: new og.CurrentPagingToolbar({
			pageSize: 10000,//og.config['files_per_page'],
			store: this.store,
			displayInfo: true,
			displayMsg: lang('displaying objects of'),
			emptyMsg: lang("no objects to display")

		}),
		viewConfig: {
			forceFit: false
		},
		sm: sm,
		tbar:tbar,
		listeners: {
			'render': {
				fn: function() {
					this.innerMessage = document.createElement('div');
					this.innerMessage.className = 'inner-message';
					var msg = this.innerMessage;
					var elem = Ext.get(this.getEl());
					var scroller = elem.select('.x-grid3-scroller');
					scroller.each(function() {
						this.dom.appendChild(msg);
					});					
				},
				scope: this
			},
			'columnmove': {
				fn: function(old_index, new_index) {
					og.eventManager.fireEvent('replace all empty breadcrumb', null);
				},
				scope: this
			}
		}
	});
	
};

Ext.extend(og.BatchInvoicingManager, Ext.grid.GridPanel, {
	load: function(params) {
		if (!params) params = {};
		var start = 0;
		
		this.store.baseParams = {
			context: og.contextManager.plainContext()
		};

		if (params.billing_period) this.store.baseParams.billing_period = params.billing_period;
		if (params.billing_period_from) this.store.baseParams.billing_period_from = params.billing_period_from;
		if (params.billing_period_to) this.store.baseParams.billing_period_to = params.billing_period_to;
		
		
		this.store.removeAll();
		this.store.load({
			params: Ext.apply(params, {
				start: start,
				limit: og.config['files_per_page']				
			})
		});
	},
	resetVars: function(){
	},
	
	activate: function() {
		if (this.needRefresh) {
			this.load({start:0});
		}
	},
	
	reset: function() {
		this.load({start:0});
	},
	
	showMessage: function(text) {
		this.innerMessage.innerHTML = text;
	},
	
	trashObjects: function() {
		if (confirm(lang('delete selected objects permanently'))) {
			this.load({
				action: 'delete',
				ids: this.getSelectedIds()
			});
			this.getSelectionModel().clearSelections();
		}
	}
	
	
});


Ext.reg("batch-invoicing", og.BatchInvoicingManager);

/************************************************
Container for BatchInvoicingManager,
*************************************************/
og.BatchInvoicingManagerPanel = function() {
	this.doNotRemove = true;
	this.needRefresh = false;
	
	this.manager = new og.BatchInvoicingManager();
	
	og.BatchInvoicingManagerPanel.superclass.constructor.call(this, {
		layout: 'fit',
		border: false,
		bodyBorder: false,
		items: [
			this.manager
		],
		closable: true
	});
}

Ext.extend(og.BatchInvoicingManagerPanel, Ext.Panel, {
	load: function(params) {
		this.manager.load(params);
	},
	activate: function() {
		this.manager.activate();
	},	
	reset: function() {
		this.manager.reset();
	},	
	showMessage: function(text) {
		this.manager.showMessage(text);
	}
});

Ext.reg("batch-invoicing-containerpanel", og.BatchInvoicingManagerPanel);