<?php
/**
 * IndicatorValue controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class IndicatorValueController extends ApplicationController {

	var $plugin_name = "indicators";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
	//	Env::useHelper('functions', $this->plugin_name);
	}
	
	function init() {
		require_javascript("IndicatorValueManager.js", $this->plugin_name);
		ajx_current("panel", "indicator_values-containerpanel", null, null, true);
		ajx_replace(true);
	}
	
	
	
	function add() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$indicator_value_data = array_var($_POST, 'indicator_value');
		
		if(!is_array($indicator_value_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$obj = new IndicatorValue();
			$obj->setObjectTypeId(IndicatorValues::instance()->getObjectTypeId());
			$obj->setIndicatorId(array_var($_REQUEST, 'indicator_id'));
			tpl_assign('indicator_value', $obj);
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$indicator_value = $this->save_indicator_value($indicator_value_data, $member_ids);
				
				DB::commit();

				ApplicationLogs::createLog($indicator_value, ApplicationLogs::ACTION_ADD);
				flash_success(lang('success add indicator_value', $indicator_value->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	function edit() {
		$this->setTemplate('add');
		
		$indicator_value = IndicatorValues::instance()->findById(get_id());
		if(!($indicator_value instanceof IndicatorValue)) {
			flash_error(lang('indicator_value dnx'));
			ajx_current("empty");
			return;
		}
		
		if (logged_user()->isGuest() || !$indicator_value->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$indicator_value_data = array_var($_POST, 'indicator_value');
		
		if(!is_array($indicator_value_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			tpl_assign('indicator_value', $indicator_value);
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$indicator_value = $this->save_indicator_value($indicator_value_data, $member_ids, $indicator_value);
				
				DB::commit();

				ApplicationLogs::createLog($indicator_value, ApplicationLogs::ACTION_EDIT);
				flash_success(lang('success edit indicator_value', $indicator_value->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	private function save_indicator_value($indicator_value_data, $member_ids, $indicator_value = null) {
		// Save IndicatorValue object
		$is_new = false;
		if (is_null($indicator_value) || !$indicator_value instanceof IndicatorValue) {
			$indicator_value = new IndicatorValue();
			$is_new = true;
		}
		
		$indicator_value->setFromAttributes($indicator_value_data);
		
		$indicator_value->save();
		
		$object_controller = new ObjectController();
		if (!is_null($member_ids)) {
			$object_controller->add_to_members($indicator_value, $member_ids);
		}
		$object_controller->add_subscribers($indicator_value);
		$object_controller->link_to_new_object($indicator_value);
		$object_controller->add_custom_properties($indicator_value);
		
		return $indicator_value;
	}
	
	
	

	function delete() {
	
		$indicator_value = IndicatorValues::instance()->findById(get_id());
		if(!($indicator_value instanceof IndicatorValue)) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
	
		if (logged_user()->isGuest() || !$indicator_value->canDelete(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
	
		// delete the indicator value
		try {
			DB::beginWork();

			$indicator_value->trash();

			DB::commit();

			ApplicationLogs::createLog($indicator_value, ApplicationLogs::ACTION_TRASH);
			flash_success(lang('success delete indicator_value', $indicator_value->getObjectName()));
			ajx_current("reload");
			
		} catch(Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
			ajx_current("empty");
		}
	
	}
	
	
	
	function view() {
		ajx_set_no_toolbar();
		$indicator_value = IndicatorValues::instance()->findById(get_id());
		if(!($indicator_value instanceof IndicatorValue)) {
			flash_error(lang('indicator_value dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$indicator_value->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		tpl_assign('indicator_value', $indicator_value);
	}
	
	

}