<?php

/**
 * InvoicePaymentReceiptController
 *
 */
class InvoicePaymentReceiptController extends ApplicationController
{

	var $plugin_name = "invoice_payment_receipts";

	/**
	 * Construct the InvoicePaymentReceiptController
	 *
	 * @access public
	 * @param void
	 * @return InvoicePaymentReceiptController
	 */
	function __construct()
	{
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		if (Plugins::instance()->isActivePlugin('multiple_currencies')) {
			Env::useHelper('functions', 'multiple_currencies');
		}
		Env::useHelper('functions', 'income');
		Env::useHelper('functions', $this->plugin_name);
	}



	function list_all()
	{
		ajx_current("empty");
		// Get all variables from request
		$start = array_var($_GET, 'start', 0);
		$limit = array_var($_GET, 'limit', config_option('files_per_page'));
		$order = array_var($_GET, 'sort');
		$order_dir = array_var($_GET, 'dir');
		$action = array_var($_GET, 'action');
		$attributes = array(
			"ids" => explode(',', array_var($_GET, 'ids')),
			"types" => explode(',', array_var($_GET, 'types')),
			"accountId" => array_var($_GET, 'account_id'),
		);

		//Resolve actions to perform
		$actionInvoice = array();
		if (isset($action)) {
			$actionInvoice = $this->resolveAction($action, $attributes);
			if ($actionInvoice["errorCode"] == 0) {
				flash_success($actionInvoice["errorInvoice"]);
			} else {
				flash_error($actionInvoice["errorInvoice"]);
			}
		}

		switch ($order) {
			case 'dateUpdated':
			case 'updatedOn':
				$order = '`updated_on`';
				break;
			case 'createdOn':
				$order = '`created_on`';
				break;
			case 'name':
				$order = '`name`';
				break;
			case 'amount':
				$order = array('`currency_id`','`amount`');
				break;
			case 'balance':
				$order = array('`currency_id`','`balance`');
				break;
			case 'qbo_sync_status':
				$order = "`quickbooks_online_id`";
				$join_params = array(
					'table' => TABLE_PREFIX.'invoice_payment_receipts',
					'jt_field' => 'id',
					'e_field' => 'object_id',
					'join_type' => 'inner'
				);				
				break;				
			default:
				break;
		}
		if (!$order_dir) {
			switch ($order) {
				case 'name':
					$order_dir = 'ASC';
					break;
				default:
					$order_dir = 'DESC';
			}
		}

		$cp_order = null;
		$dim_order = null;

		if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		} else if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		}
		
		if (!$order) $order = '`updated_on`';
		
		$extra_conditions = "";
		
		Hook::fire("listing_extra_conditions", null, $extra_conditions);

		$only_count_result = array_var($_GET, 'only_result', false);
		$res = InvoicePaymentReceipts::instance()->listing(array(
			'count_results' => false,
			'only_count_results' => $only_count_result,
			"order" => $order,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"extra_conditions" => $extra_conditions,
			"order_dir" => $order_dir,
			"start" => $start,
			"limit" => $limit,
		));
		$invoices = $res->objects;

		// Prepare response object
		$object = $this->prepareObject($invoices, $start, $limit, $res);
		
		ajx_extra_data($object);
		tpl_assign("listing", $object);
	}

	/**
	 * Resolve action to perform 
	 *
	 * @param string $action
	 * @param array $attributes
	 * @return string $invoice
	 */
	private function resolveAction($action, $attributes) {
		$resultInvoice = "";
		$resultCode = 0;
		switch ($action) {
			case "delete":
				$succ = 0;
				$err = 0;
				$resultInvoice = "";
				for ($i = 0; $i < count($attributes["ids"]); $i++) {
					$id = $attributes["ids"][$i];
					$invoice_pr = InvoicePaymentReceipts::instance()->findById($id);
					if ($invoice_pr instanceof InvoicePaymentReceipt && $invoice_pr->canDelete(logged_user())) {
						try {
							DB::beginWork();
							$invoice_pr->trash();
							DB::commit();
							ApplicationLogs::createLog($invoice_pr, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch (Exception $e) {
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				};
				if ($err > 0) {
					$resultCode = 2;
					if ($err > 0) {
						$resultInvoice .= lang("error delete objects", $err);
					}
					if ($succ > 0) {
						$resultInvoice .= "\n" . lang("success delete objects", $succ);
					}
				} else {
					$resultInvoice = lang("success delete objects", $succ);
				}
				break;
			case "archive":
				$succ = 0;
				$err = 0;
				for ($i = 0; $i < count($attributes["ids"]); $i++) {
					$id = $attributes["ids"][$i];
					$invoice_pr = InvoicePaymentReceipts::instance()->findById($id);
					if ($invoice_pr instanceof InvoicePaymentReceipt && $invoice_pr->canEdit(logged_user())) {
						try {
							DB::beginWork();
							$invoice_pr->archive();
							DB::commit();
							ApplicationLogs::createLog($invoice_pr, ApplicationLogs::ACTION_ARCHIVE);
							$succ++;
						} catch (Exception $e) {
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultInvoice = lang("error archive objects", $err) . "<br />" . ($succ > 0 ? lang("success archive objects", $succ) : "");
				} else {
					$resultInvoice = lang("success archive objects", $succ);
				}
				break;
			default:
				$resultInvoice = lang("Unimplemented action: '" . $action . "'"); // if
				$resultCode = 2;
				break;
		} // switch
		return array("errorInvoice" => $resultInvoice, "errorCode" => $resultCode);
	}

	/**
	 * Prepares return object for a list of invoices
	 *
	 * @param array $totMsg
	 * @param integer $start
	 * @param integer $limit
	 * @return array
	 */
	private function prepareObject($totInv, $start, $limit, $res_obj) {
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"invoices" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}

		$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType(InvoicePaymentReceipts::instance()->getObjectTypeId());

		$ids = array();
		for ($i = 0; $i < $limit; $i++) {
			if (isset($totInv[$i])) {
				$inv = $totInv[$i];
				if ($inv instanceof InvoicePaymentReceipt) {

					$currency = Currencies::instance()->findById($inv->getCurrencyId());
					$symbol = $currency instanceof Currency ? $currency->getSymbol() : "$";
					
					$general_info = $inv->getArrayInfo();
					$general_info = array_merge($general_info, $inv->getObject()->getArrayInfo());
					
					$info = array(
						"amount" => format_money_amount($inv->getAmount(), $symbol, 2),
						"balance" => format_money_amount($inv->getBalance(), $symbol, 2),
						"memPath" => json_encode($inv->getMembersIdsToDisplayPath()),
					);
					$info = array_merge($general_info, $info);
					
					$ids[] = $inv->getId();

					$object["invoice_payment_receipts"][] = $info;

					foreach ($custom_properties as $cp) {
						$object["invoice_payment_receipts"][$i]['cp_' . $cp->getId()] = get_custom_property_value_for_listing($cp, $inv);
					}
				}
			}
		}

		$read_objects = ReadObjects::getReadByObjectList($ids, logged_user()->getId());
		if( isset($object["invoice_payment_receipts"])){
			foreach ($object["invoice_payment_receipts"] as &$data) {
				$data['isRead'] = isset($read_objects[$data['object_id']]);
			}
		}
		return $object;
	}

	// ---------------------------------------------------
	//  Invoices
	// ---------------------------------------------------

	/**
	 * View single invoice
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function view() {
		$this->addHelper('textile');

		$invoice_pr = InvoicePaymentReceipts::instance()->findById(get_id());
		if (!($invoice_pr instanceof InvoicePaymentReceipt)) {
			flash_error(lang('invoice dnx'));
			ajx_current("empty");
			return;
		} // if

		if (!$invoice_pr->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		} // if
		//read object for this user
		$invoice_pr->setIsRead(logged_user()->getId(), true);

		tpl_assign('invoice_pr', $invoice_pr);
		tpl_assign('subscribers', $invoice_pr->getSubscribers());
		ajx_extra_data(array("title" => $invoice_pr->getTitle(), 'icon' => $invoice_pr->getIconClass()));
		ajx_set_no_toolbar(true);

		ApplicationReadLogs::createLog($invoice_pr, ApplicationReadLogs::ACTION_READ);
	}


	/**
	 * Add invoice
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function add() {

		if (!can_add(logged_user(), active_context(), InvoicePaymentReceipts::instance()->getObjectTypeId())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}

		$invoice_payment_receipt = new InvoicePaymentReceipt();
		tpl_assign('invoice_payment_receipt', $invoice_payment_receipt);

		

		$invoice_pr_data = array_var($_POST, 'invoice_payment_receipt');
		if (!is_array($invoice_pr_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			// payment for specific invoice
			if (array_var($_REQUEST, 'invoice_id')) {
				$for_invoice = IncomeInvoices::instance()->findById(array_var($_REQUEST, 'invoice_id'));
				tpl_assign('for_invoice', $for_invoice);
			}
			
			$invoice_pr_data['date'] = DateTimeValueLib::now();
			tpl_assign('invoice_data', $invoice_pr_data);
			
		} else  {
			try {
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$this->save_invoice_payment_receipt($invoice_payment_receipt, $invoice_pr_data, $member_ids);

				ApplicationLogs::createLog($invoice_payment_receipt, ApplicationLogs::ACTION_ADD);

				flash_success(lang('success add invoice payment receipt', $invoice_payment_receipt->getObjectName()));
				
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
					ajx_current("empty");
				} else {
					ajx_current("back");
				}
				//ajx_add("overview-panel", "reload");
				
			} catch (Exception $e) {
				flash_error($e->getMessage());
				Logger::log_r($e->getMessage()."\n".$e->getTraceAsString());
				ajx_current("empty");
			} // try
		} // if
	}

	/**
	 * Edit specific invoice
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function edit() {
		$this->setTemplate('add');

		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}

		$invoice_payment_receipt = InvoicePaymentReceipts::instance()->findById(get_id());
		if (!($invoice_payment_receipt instanceof InvoicePaymentReceipt)) {
			flash_error(lang('invoice dnx'));
			ajx_current("empty");
			return;
		} // if

		if (!$invoice_payment_receipt->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		} // if

		$invoice_pr_data = array_var($_POST, 'invoice_payment_receipt');
		
		if (!is_array($invoice_pr_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			
			$affected_invoices = $this->get_possible_invoices_to_affect_data($invoice_payment_receipt->getMemberIds(), $invoice_payment_receipt->getId(), $invoice_payment_receipt->getAmount(), false);
			
			tpl_assign('affected_invoices', $affected_invoices);
			tpl_assign('invoice_payment_receipt', $invoice_payment_receipt);
			
		} else  {
			
			try {
				$member_ids = json_decode(array_var($_POST, 'members'));
				$this->save_invoice_payment_receipt($invoice_payment_receipt, $invoice_pr_data, $member_ids);

				ApplicationLogs::createLog($invoice_payment_receipt, ApplicationLogs::ACTION_EDIT);

				flash_success(lang('success edit invoice payment receipt', $invoice_payment_receipt->getObjectName()));
				
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
					ajx_current("empty");
				} else {
					ajx_current("back");
				}
			} catch (Exception $e) {
				flash_error($e->getMessage());
				Logger::log_r($e->getMessage()."\n".$e->getTraceAsString());
				ajx_current("empty");
			} // try
		} // if
	}

	/**
	 * Delete specific invoice
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function delete() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}

		ajx_current("empty");
		$invoice_pr = InvoicePaymentReceipts::instance()->findById(get_id());
		if (!($invoice_pr instanceof InvoicePaymentReceipt)) {
			flash_error(lang('invoice dnx'));
			ajx_current("empty");
			return;
		} // if

		if (!$invoice_pr->canDelete(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		} // if

		try {
			DB::beginWork();
			$invoice_pr->trash();
			DB::commit();
			ApplicationLogs::createLog($invoice_pr, ApplicationLogs::ACTION_TRASH);

			flash_success(lang('success delete invoice payment receipt', $invoice_pr->getObjectName()));
			if (array_var($_POST, 'popup', false)) {
				ajx_current("reload");
			} else {
				ajx_current("back");
			}
			ajx_add("overview-panel", "reload");
		} catch (Exception $e) {
			DB::rollback();
			flash_error(lang('error delete invoice'));
			ajx_current("empty");
		} // try
	}
	

	function save_invoice_payment_receipt(InvoicePaymentReceipt $invoice_pr, $data, $member_ids) {

		// if currency not set and only one, put that currency
		if (!$data['currency_id']) {
			$all_currencies = Currencies::getCurrenciesInfo();
			if (count($all_currencies) == 1) {
				$data['currency_id'] = $all_currencies[0]['id'];
			}
		}
		
		$data['amount'] = clean_formatted_money_amount_for_sql($data['amount']);
		$data['balance'] = 0; // this will be calculated later if necesary

		if (!array_var($data, 'allow_zero_amount') && (!is_numeric($data['amount']) || $data['amount'] <= 0)) {
			throw new Exception(lang('amount must be greater than 0'));
		}
		
		if (trim($data['name']) == '') {
			$date_cp = CustomProperties::getCustomPropertyByCode($invoice_pr->getObjectTypeId(), 'date');
			$date_str = array_var($_REQUEST['object_custom_properties'], $date_cp->getId());
			
			$project_or_client_name = '';
			$project_ot = ObjectTypes::instance()->findByName('project');
			$client_ot = ObjectTypes::instance()->findByName('customer');
			
			$members = Members::instance()->findAll(array("conditions" => "id IN (".implode(',', $member_ids).")"));
			foreach ($members as $member) {
				if ($member->getObjectTypeId() == $project_ot->getId()) $project_or_client_name = $member->getName();
				if ($project_or_client_name != '') break;
			}
			if ($project_or_client_name == '') {
				foreach ($members as $member) {
					if ($member->getObjectTypeId() == $client_ot->getId()) $project_or_client_name = $member->getName();
					if ($project_or_client_name != '') break;
				}
			}
			
			$data['name'] = lang('invoice payment receipt') . " - $project_or_client_name $date_str";
		}
		$invoice_pr->setFromAttributes($data);
		
		try {
			DB::beginWork();

			$invoice_pr->save();
			
			$object_controller = new ObjectController();
			if (!is_null($member_ids)) {
				$object_controller->add_to_members($invoice_pr, $member_ids);
			}
			$object_controller->add_subscribers($invoice_pr);
			$object_controller->link_to_new_object($invoice_pr);
			$object_controller->add_custom_properties($invoice_pr);
			
			// reset paid amount of prev affected invoices
			$affected_invoice_ids = $invoice_pr->getAffectedInvoicesIds();
			InvoicePaymentReceiptInvoices::instance()->delete("payment_receipt_id=".$invoice_pr->getId());
			foreach ($affected_invoice_ids as $invoice_id) {
				InvoicePaymentReceiptInvoices::instance()->recalculateInvoicePaidAmount($invoice_id);
			}
			
			$total_to_pay = 0;
			// save the details in table invoice_payment_receipt_invoices
			$invoices_to_affect = array_var($_REQUEST, 'invoices', array());
			foreach ($invoices_to_affect as $inv_data) {
				
				$inv_data['to_pay'] = clean_formatted_money_amount_for_sql($inv_data['to_pay']);
				
				if (is_numeric($inv_data['to_pay']) && $inv_data['to_pay'] > 0) {
					
					$inv_affected = new InvoicePaymentReceiptInvoice();
					$inv_affected->setInvoiceId($inv_data['id']);
					$inv_affected->setPaymentReceiptId($invoice_pr->getId());
					$inv_affected->setCurrencyId($inv_data['currency_id']);
					$inv_affected->setAmount($inv_data['to_pay']);
					$inv_affected->save();
					
					$total_to_pay += $inv_data['to_pay'];
				}
			}
			
			// update balance column
			if ($invoice_pr->getAmount() > $total_to_pay) {
				
				//////////temporary FIx
				if(($invoice_pr->getAmount() - $total_to_pay)<0.005)
				{
					$set_balance=0;
				}else{
					$set_balance=$invoice_pr->getAmount() - $total_to_pay;
				}
				$invoice_pr->setBalance($set_balance);
				//////////////////////////////

				//$invoice_pr->setBalance($invoice_pr->getAmount() - $total_to_pay);
				$invoice_pr->save();
			}

			// Generate repetition
			$for_invoice_id = array_var($_POST, 'for_invoice_id');
			$invoice = IncomeInvoices::instance()->findById($for_invoice_id);

			if($invoice instanceof IncomeInvoice)
			{
				if($invoice->isRepetitive() && $invoice->getColumnValue('status') == 'confirmed')
				{
					$invoice->generateNextRepetition();
				}
			}

			Hook::fire('after_invoice_payment_receipt_save', array('object' => $invoice_pr), $invoice_pr);
			
			DB::commit();

		} catch (Exception $e) {
			DB::rollback();
			Logger::log_r($e->getMessage()."\n".$e->getTraceAsString());
			throw $e;
		}

		return $invoice_pr;
	}
	
	private function get_possible_invoices_to_affect_data($member_ids, $payment_receipt_id, $payment_total, $override_to_pay_amount=true, $for_invoice_id=0) {
		$possible_invoices_to_affect = array();
		
		if (is_array($member_ids) && count($member_ids) > 0) {
			
			$project_ot = ObjectTypes::instance()->findByName('project');
			$client_ot = ObjectTypes::instance()->findByName('customer');
			
			$members = Members::instance()->findAll(array("conditions" => "id IN (".implode(',', $member_ids).")"));
			$members_to_filter = array();
			foreach ($members as $member) {
				if (in_array($member->getObjectTypeId(), array($project_ot->getId(), $client_ot->getId()))) {
					$members_to_filter[] = $member->getId();
				}
			}
			
			if (count($members_to_filter) > 0) {
				
				$payment_receipt_cond = "";
				$affected_invoices_cond = "";
				$affected_invoices = array();
				$affected_invoices_ids = array();
				
				$for_invoice = null;
				if ($for_invoice_id) {
					$for_invoice = IncomeInvoices::instance()->findById($for_invoice_id);
				}
				
				if ($for_invoice instanceof IncomeInvoice) {
					$invoices = array($for_invoice);
					
				} else {
					
					if (is_numeric($payment_receipt_id) && $payment_receipt_id > 0) {
						$payment_receipt_cond = " AND payment_receipt_id != '$payment_receipt_id'";
						$affected_invoices = InvoicePaymentReceiptInvoices::instance()->getAffectedInvoices($payment_receipt_id);
						foreach ($affected_invoices as $ai) $affected_invoices_ids[] = $ai->getId();
						if (count($affected_invoices_ids) > 0) {
							$affected_invoices_cond = " AND o.id NOT IN (".implode(',', $affected_invoices_ids).")";
						}
					}
					
					$res = IncomeInvoices::instance()->listing(array(
							'count_results' => false,
							'only_count_results' => false,
							'ignore_context' => true,
							'extra_conditions' => " AND status != 'canceled' $affected_invoices_cond",
							'member_ids' => $members_to_filter,
							'order' => '`date`',
							'order_dir' => 'ASC',
					));
					$invoices = $res->objects;
				}
				
				$available_to_pay = $payment_total;
				
				$invoices = array_merge($affected_invoices, $invoices);
				
				
				foreach ($invoices as $invoice) {
					/* @var $invoice IncomeInvoice */
					$date = $invoice->getDate();
					
					$pending_amount = $invoice->getDueAmount();
					
					if ($pending_amount > 0 || in_array($invoice->getId(), $affected_invoices_ids)) {
						$to_pay = 0;
						
						if (in_array($invoice->getId(), $affected_invoices_ids)) {
							
							$affected_obj = InvoicePaymentReceiptInvoices::instance()->getByInvoiceAndPaymentReceiptId($invoice->getId(), $payment_receipt_id);
							if ($affected_obj instanceof InvoicePaymentReceiptInvoice) {
								$to_pay = $affected_obj->getAmount();
							}
						} else {
							if ($override_to_pay_amount) {
								$to_pay = $pending_amount > $available_to_pay ? $available_to_pay : $pending_amount;
							}
						}
						$available_to_pay -= $to_pay;
						$inv_data = array(
								'id' => $invoice->getId(),
								'name' => $invoice->getObjectName(),
								'date' => $date instanceof DateTimeValue ? format_date($date, null, 0) : '',
								'total' => $invoice->getTotal(),
								'currency_id' => $invoice->getCurrencyId(),
								'pending_amount' => $pending_amount,
								'to_pay' => round($to_pay,2),
						);
						
						$possible_invoices_to_affect[] = $inv_data;
					}
					
				}
			}
		}
		
		return $possible_invoices_to_affect;
	}
	
	function get_possible_invoices_to_affect() {
		
		ajx_current("empty");
		
		$member_ids = array_filter(json_decode(array_var($_REQUEST, 'member_ids', '[]'), true), 'is_numeric');
		$payment_receipt_id = array_var($_REQUEST, 'payment_receipt_id', '');
		$payment_total = array_var($_REQUEST, 'payment_total', 0);
		
		//if thousand separtor==',' crash
		if(user_config_option('thousand_separator')==',')
		{
			$payment_total=str_replace ( ",", "", $payment_total);
		}
		//don't use ',' as decimal separator for calculations
		if(user_config_option('decimals_separator')==',')
		{
			$payment_total=str_replace ( ",", ".", $payment_total);
		}

		$genid = array_var($_REQUEST, 'genid', '');
		$for_invoice_id = array_var($_REQUEST, 'for_invoice_id');
		
		
		$possible_invoices_to_affect = $this->get_possible_invoices_to_affect_data($member_ids, $payment_receipt_id, $payment_total, true, $for_invoice_id);
		
		ajx_extra_data(array('invoices' => $possible_invoices_to_affect, 'genid' => $genid));
	}

	public function check_payment_receipts_integration_status(){
        $ids = explode(',', array_var($_GET,'ids'));
        $result = true;
        for ($i = 0; $i < count($ids); $i++) {
            $id = $ids[$i];
            $payment_receipt = InvoicePaymentReceipts::instance()->findById($id);
            Hook::fire('check_integration_status', array('object' => $payment_receipt), $result);
            if(!$result) break;
        }
        ajx_current("empty");
        ajx_extra_data(array('result'=>$result));

    }


}

// InvoicePaymentReceiptController

