<?php
class InvoicePaymentReceipt extends BaseInvoicePaymentReceipt {
		
	
	function getArrayInfo() {
		$info = array();
		
		$columns = $this->getColumns();
		foreach ($columns as $col) {
			$info[$col] = $this->getColumnValue($col);
			if ($info[$col] instanceof DateTimeValue) $info[$col] = format_date($info[$col], null, 0);
		}
		
		return $info;
	}
	
	
	function getAffectedInvoicesIds() {
		$result = InvoicePaymentReceiptInvoices::instance()->getByPaymentReceiptId($this->getId());
		$ids = array();
		foreach ($result as $obj) $ids[] = $obj->getInvoiceId();
		return $ids;
	}
	
	
	
	function trash($trashDate = null, $fire_hook = true) {
		$ok = parent::trash($trashDate, $fire_hook);
		
		$affected_invoice_ids = $this->getAffectedInvoicesIds();
		foreach ($affected_invoice_ids as $invoice_id) {
			InvoicePaymentReceiptInvoices::instance()->recalculateInvoicePaidAmount($invoice_id);
		}
		
		return $ok;
	}
	
	function untrash($trashDate = null, $fire_hook = true) {
		$ok = parent::untrash($fire_hook);
		
		$affected_invoice_ids = $this->getAffectedInvoicesIds();
		foreach ($affected_invoice_ids as $invoice_id) {
			InvoicePaymentReceiptInvoices::instance()->recalculateInvoicePaidAmount($invoice_id);
		}
		
		return $ok;
	}
	
	
	function delete() {
		$affected_invoice_ids = $this->getAffectedInvoicesIds();
		InvoicePaymentReceiptInvoices::instance()->delete("payment_receipt_id=".$this->getId());
		
		foreach ($affected_invoice_ids as $invoice_id) {
			InvoicePaymentReceiptInvoices::instance()->recalculateInvoicePaidAmount($invoice_id);
		}
		
		return parent::delete();
	}
	
	function getEditUrl() {
		return get_url('invoice_payment_receipt', 'edit', array('id' => $this->getId()));
	}
	
	static function canAdd(Contact $user, $context, &$notAllowedMember = ''){
	    return can_add($user, $context, ObjectsName::instance()->getObjectTypeId(), $notAllowedMember);
	}
	
	/**
	 * Returns true if $user can access this message
	 *
	 * @param Contact $user
	 * @return boolean
	 */
	function canView(Contact $user) {
		return can_read($user, $this->getMembers(), $this->getObjectTypeId());
	} // canView

	/**
	 * Check if specific user can edit this messages
	 *
	 * @access public
	 * @param Contact $user
	 * @return boolean
	 */
	function canEdit(Contact $user) {
		return can_write($user, $this->getMembers(), $this->getObjectTypeId());
	} // canEdit


	/**
	 * Check if specific user can delete this messages
	 *
	 * @access public
	 * @param Contact $user
	 * @return boolean
	 */
	function canDelete(Contact $user) {
		return can_delete($user, $this->getMembers(), $this->getObjectTypeId());
	} // canDelete

	/**
	 * Check if specific user can comment this message
	 *
	 * @access public
	 * @param void
	 * @return boolean
	 */
	function canAddComment(Contact $user) {
		return can_write($user, $this->getMembers(), $this->getObjectTypeId());
	} // canAddComment

	/**
	 * 
	 * {@inheritDoc}
	 * @see DataObject::manager()
	 */
	function manager() {
		if (!($this->manager instanceof InvoicePaymentReceipts))
			$this->manager = InvoicePaymentReceipts::instance();
			return $this->manager;
	}
}

