<?php 

class SplittedInvoiceThreads extends BaseSplittedInvoiceThreads {
	
	function getNextThreadId() {

		$row = DB::executeOne("SELECT MAX(thread_id) as max_id FROM ".$this->getTableName());

		$max_id = 0;
		if ($row) {
			$max_id = array_var($row, 'max_id');
		}

		return $max_id + 1;

	}


	function saveSplittedInvoicesThread($invoices) {

		$thread_id = $this->getNextThreadId();
		foreach ($invoices as $invoice) {
			if (!$invoice instanceof IncomeInvoice) continue;
			$thread_reg = new SplittedInvoiceThread();
			$thread_reg->setThreadId($thread_id);
			$thread_reg->setInvoiceId($invoice->getId());
			$thread_reg->save();
		}
	}


	function getInvoicesFromThread($thread_id) {

		$invoices = array();
		$thread_regs = $this->findAll(array("conditions" => array("thread_id = ?", $thread_id)));

		foreach ($thread_regs as $thread_reg) {
			$inv = $thread_reg->getInvoice();
			if ($inv) {
				$invoices[] = $inv;
			}
		}

		return $invoices;
	}

	
	function getRelatedInvoices(IncomeInvoice $invoice) {
		$related_invoices = array();
		
		$inv_thread_reg = $this->findOne(array("conditions" => array("invoice_id = ?", $invoice->getId())));
		if ($inv_thread_reg) {
			$all_invoices_from_thread = $this->getInvoicesFromThread($inv_thread_reg->getThreadId());
			foreach ($all_invoices_from_thread as $inv) {
				if ($inv->getId() != $invoice->getId()) {
					$related_invoices[] = $inv;
				}
			}
		}

		return $related_invoices;
	}

	
	function trashRelatedInvoices(IncomeInvoice $invoice) {
		$related_invoices = $this->getRelatedInvoices($invoice);
		
		foreach ($related_invoices as $inv) {
			if (!$inv->isTrashed()) {
				$inv->trash();
			}
		}

		return $related_invoices;
	}

	
	function voidRelatedInvoices(IncomeInvoice $invoice) {
		$related_invoices = $this->getRelatedInvoices($invoice);
		
		foreach ($related_invoices as $inv) {
			if ($inv->getStatus() != 'canceled') {
				$inv->void();
			}
		}

		return $related_invoices;
	}

	
	function untrashRelatedInvoices(IncomeInvoice $invoice) {
		$related_invoices = $this->getRelatedInvoices($invoice);
		
		foreach ($related_invoices as $inv) {
			if ($inv->isTrashed()) {
				$inv->untrash();
			}
		}

		return $related_invoices;
	}
	
}
