<?php
Hook::register('attribute_features');


function attribute_features_automatic_invoice_override_customer_data($params, &$inv_customer_data) {

	$member = array_var($params, 'member');

	if (!$member instanceof Member) return;

	$project_ot = ObjectTypes::findByName('project');
	$client_ot = ObjectTypes::findByName('customer');
	$projects_dim = Dimensions::findByCode('customer_project');
	Env::useHelper('functions','crpm');
	Env::useHelper('functions','income');
	$clients_dim = get_customers_dimension();

	if ($member->getObjectTypeId() == $project_ot->getId()) {

		// billing clients association between clients and projects
		$billing_client_assoc = DimensionMemberAssociations::getAssociationByCode("project_billing_client");
		if (!$billing_client_assoc) return;
		
		$assoc_client_member_csv = MemberPropertyMembers::getAllPropertyMemberIds($billing_client_assoc->getId(), $member->getId());
		
		$billing_client_names_and_address = array();

		if ($assoc_client_member_csv != '') {
			$billing_clients = Members::instance()->findAll(array("conditions" => "id IN ($assoc_client_member_csv)"));
			// don't override client data if there is only one.
			if (count($billing_clients) <= 1 || income_config_option('split_invoice_for_billing_clients') == 'single') return;

			foreach ($billing_clients as $bc) {
				if (!$bc) continue;

				$name_and_address = '';
				$customer = Customers::instance()->findById($bc->getObjectId());
				if ($customer) {
					$contact = $customer->getContact();
					
					$company_address = $contact->getAddress('work');
					if (!$company_address) $company_address = $contact->getAddress('home');
					if (!$company_address) $company_address = $contact->getAddress('postal');
					if (!$company_address) $company_address = $contact->getAddress('other');	
					if ($company_address && trim($company_address->getStreet()) == '') {
						$company_address = null;
					}

					$address_str = '';
					if ($company_address) {
						$address_str = $company_address->getStreet()."\n".$company_address->getCity()." ".$company_address->getState()." ".$company_address->getCountry()." ".$company_address->getZipCode();
					}

					$name_and_address = $bc->getName() . ($address_str == '' ? '' : "\n") . $address_str;
				}

				if ($name_and_address) {
					$billing_client_names_and_address[] = $name_and_address;
				}
			}
		}

		if (count($billing_client_names_and_address) > 0) {
			
			$inv_customer_data['name'] = implode("\n\n", $billing_client_names_and_address);

			$inv_customer_data['address'] = '';
			$inv_customer_data['city'] = '';
			$inv_customer_data['state'] = '';
			$inv_customer_data['country'] = '';
			$inv_customer_data['zipcode'] = '';
		}
	}
}


function attribute_features_invoice_member_summary_more_properties($params, &$summary) {
	
	$invoice = array_var($params, 'invoice');
	$summary_options = array_var($params, 'summary_options', array());

	if ($invoice instanceof IncomeInvoice) {

		$project_ot = ObjectTypes::findByName('project');
		$client_ot = ObjectTypes::findByName('customer');
		$invoice_ot = ObjectTypes::findByName('invoice');
		$projects_dim = Dimensions::findByCode('customer_project');
		Env::useHelper('functions','crpm');
		$clients_dim = get_customers_dimension();

		$project_member = null;

		//check invoices with same number
		$invoice_number = $invoice->getInvoiceNumber();

		foreach ($invoice->getMembers() as $m) {
			if ($m->getObjectTypeId() == $project_ot->getId()) {
				$project_member = $m;
				break;
			}
		}

		$invoice_total = $invoice->getTotal();

		if (array_var($summary_options, 'ignore_discount')) {
			$inv_discount = 0;
			$lines = $invoice->getInvoiceLines();
			foreach ($lines as $l) {
				if ($l->getTotal() < 0) {
					$inv_discount += abs($l->getTotal());
				}
			}
			if ($inv_discount > 0) {
				$invoice_total += $inv_discount;
			}
		}

		// billing clients association between clients and projects
		$billing_client_assoc = DimensionMemberAssociations::getAssociationByCode("project_billing_client");
		if (!$billing_client_assoc) return;
		
		$invoice_allocation_attr = MemberAssociationAttributes::getAttributeByCode($billing_client_assoc->getId(), 'invoice_allocation');
		// if no invoice allocation attribute is defined then continue with normal flow
		if (!$invoice_allocation_attr) return;

		$client_mpms = MemberPropertyMembers::getAssociatedMemberRecords($billing_client_assoc->getId(), $project_member->getId());

		// build allocation percentage per client
		$current_total_allocation = 0;
		$allocation_per_client = array();
		$client_members = array();
		foreach ($client_mpms as $mpm) {
			
			$client_member = Members::getMemberById($mpm->getPropertyMemberId());
			if (!$client_member) continue;
			
			// get the allocation percentage for each client
			$allocation_percentage = MemberPropertyMemberAttributes::getAttributeValue($mpm->getId(), $invoice_allocation_attr->getId());
			if ($allocation_percentage > 100) $allocation_percentage = 100;
			
			// dont invoice more than 100%
			if ($allocation_percentage > 100 - $current_total_allocation) {
				$allocation_percentage = 100 - $current_total_allocation;
			}

			
			if ($allocation_percentage > 0) {
				
				$client_members[] = $client_member;
				
					$invoice_without_subfix=substr($invoice->getInvoiceNumber(), 0, -2);
        			$resSameBulkInvoices= IncomeInvoices::findInvoicesContainNumberForClient($invoice_without_subfix, $client_member->getId());
		
		        	if(isset($resSameBulkInvoices))
			        {
    			        if($resSameBulkInvoices->getObjectTypeId() == $invoice_ot->getId())
	    		        {
		    		        $resInvoice= IncomeInvoices::instance()->findById($resSameBulkInvoices->getId());
			    	        $invoice_total=$resInvoice->getTotal();
			            }
		            }

					$allocation_amount = $invoice_total;
				$allocation_per_client[$client_member->getId()] = array(
					'name' => $client_member->getName(),
					'allocation' => $allocation_percentage,
					'amount' => $allocation_amount,
				);
				
				$current_total_allocation += $allocation_percentage;
			}
		}
		
		// dont invoice less than 100%
		if ($current_total_allocation < 100) {
			$last_client = end($client_members);
			$difference = 100 - $current_total_allocation;
			$amount_diff = $invoice_total * $difference / 100;
			if ($last_client instanceof Member) {
				$allocation_per_client[$last_client->getId()]['allocation'] += $difference;
				$allocation_per_client[$last_client->getId()]['amount'] += $amount_diff;
			}
		}

		$summary['allocation_per_client'] = $allocation_per_client;
	}

}



function attribute_features_object_definition($type, &$ret) {
	if ($type == 'IncomeInvoice') {
		$ret['allocation_percentage'] = DATA_TYPE_FLOAT;
	}
}

function attribute_features_override_invoice_generation($params, &$generated_invoices) {
	
	$member = array_var($params, 'member');
	$invoice_template = array_var($params, 'invoice_template');
	$timeslots = array_var($params, 'timeslots');
	$tasks = array_var($params, 'tasks');
	$payment_receipts = array_var($params, 'payment_receipts');
	$param_billing_period_from = array_var($params, 'param_billing_period_from');
	$param_billing_period_to = array_var($params, 'param_billing_period_to');
	$only_return_raw_data = array_var($params, 'only_return_raw_data');
	
	$new_invoices = array();
	
	if (!$member instanceof Member) return;
	
	$project_ot = ObjectTypes::findByName('project');
	$client_ot = ObjectTypes::findByName('customer');
	$projects_dim = Dimensions::findByCode('customer_project');
	Env::useHelper('functions','crpm');
	$clients_dim = get_customers_dimension();
	
	// if clients and projects are in the same dimension, the project has only one client so continue with normal flow
	if ($projects_dim->getId() == $clients_dim->getId()) return;
	
	// if main member is a client then continue with the normal flow
	if ($member->getObjectTypeId() == $client_ot->getId()) return;

	// if main member is a client then check if we have to divide the invoice between clients
	if ($member->getObjectTypeId() == $project_ot->getId()) {
		
		// default association between clients and projects
		//$client_assoc = DimensionMemberAssociations::getAssociationByCode($project_ot->getName()."_".$client_ot->getName());

		// association between project and billing clients
		$client_assoc = DimensionMemberAssociations::getAssociationByCode("project_billing_client");
		if (!$client_assoc) return;
		
		$invoice_allocation_attr = MemberAssociationAttributes::getAttributeByCode($client_assoc->getId(), 'invoice_allocation');
		// if no invoice allocation attribute is defined then continue with normal flow
		if (!$invoice_allocation_attr) return;

		// if the config option to split invoice into separate ones for each billing client is not enabled then continue with normal flow
		Env::useHelper('functions', 'income');
		if (income_config_option('split_invoice_for_billing_clients') == 'single') return;
		
		$client_mpms = MemberPropertyMembers::getAssociatedMemberRecords($client_assoc->getId(), $member->getId());
		
		// if project has only one client then continue with normal flow
		if (count($client_mpms) <= 1) return;
		
		// build allocation percentage per client
		$current_total_allocation = 0;
		$allocation_per_client = array();
		$client_members = array();
		foreach ($client_mpms as $mpm) {
			
			$client_member = Members::getMemberById($mpm->getPropertyMemberId());
			if (!$client_member) continue;
			
			// get the allocation percentage for each client
			$allocation_percentage = MemberPropertyMemberAttributes::getAttributeValue($mpm->getId(), $invoice_allocation_attr->getId());
			if ($allocation_percentage > 100) $allocation_percentage = 100;
			
			// dont invoice more than 100%
			if ($allocation_percentage > 100 - $current_total_allocation) {
				$allocation_percentage = 100 - $current_total_allocation;
			}
			
			if ($allocation_percentage > 0) {
				
				$client_members[] = $client_member;
				$allocation_per_client[$client_member->getId()] = $allocation_percentage;
				
				$current_total_allocation += $allocation_percentage;
			}
		}
		
		// dont invoice less than 100%
		if ($current_total_allocation < 100) {
			$last_client = end($client_members);
			$difference = 100 - $current_total_allocation;
			$allocation_per_client[$last_client->getId()] += $difference;
		}

		$invoice_number_suffixes = "ABCDEFGHJIJKLMNOPQRSTUVWXYZ";
		$pos = 0;
		
		// generate the invoice for each client
		$first_invoice=true;
		foreach ($client_members as $client_member) {
			
			// set timeslots and expenses invoicing status to 'pending' before each iteration because we are going to invoice them multiple times, once per client
			foreach ($timeslots as $t) $t->setColumnValue('invoicing_status', 'pending');
			foreach ($payment_receipts as $p) $p->setColumnValue('invoicing_status', 'pending');
			
			// calculate the discount for this client
			$allocation_percentage = $allocation_per_client[$client_member->getId()];

			// get the next invoice number to force it into the new invoices
			// build dummy data
			$dummy_invoice = new IncomeInvoice();
			$tmp_invoice_data = array();
			$invoice_subtype = ObjectSubtypes::instance()->findOne(array('conditions' => array("object_type_id=? AND `name`='invoice'", IncomeInvoices::instance()->getObjectTypeId())));
			if ($invoice_subtype instanceof ObjectSubtype) {
				$tmp_invoice_data['object_subtype_id'] = $invoice_subtype->getId();
			}
			// get the next number
			fill_invoice_data_serie_and_number($dummy_invoice, $tmp_invoice_data, $first_invoice);

			if(income_config_option('split_invoice_for_billing_clients') != 'single' && $first_invoice)
			{
				$first_invoice=false;
			}
			// append the suffix
			$number_suffix = substr($invoice_number_suffixes, $pos, 1);
			$forced_invoice_number = $tmp_invoice_data['invoice_number'] . "-" . $number_suffix;
			
			// build the invoice as in the normal flow using the current client
			$invoice = IncomeInvoiceTemplates::generateInvoiceFromTemplateAndObjects($member, 
																					$invoice_template,
																					$timeslots, 
																					$payment_receipts,
																					$param_billing_period_from,
																					$param_billing_period_to, 
																					$only_return_raw_data, 
																					$client_member, 
																					$allocation_percentage, 
																					$forced_invoice_number,
																					$tasks);
			
			if ($invoice) {
				$new_invoices[] = $invoice;
			}
			$pos++;
		}
		
		if (count($new_invoices) > 0) {
			$generated_invoices = $new_invoices;

			SplittedInvoiceThreads::instance()->saveSplittedInvoicesThread($new_invoices);
		}
		
	}
	
}

// replace the invoice view's trash action so we can check if invoice has twins
function attribute_features_render_page_actions($invoice, &$ret) {
	if ($invoice instanceof IncomeInvoice && !$invoice->isTrashed() && $invoice->canDelete(logged_user())){
		
		PageActions::instance()->removeAction('ico-trash');
		
		$js_function = "og.splitted_invoices.trash(".$invoice->getId().")";
		
		add_page_action(lang('move to trash'), "javascript:$js_function", "ico-trash", null, null, true);
		
	}
}

// add more information to the message when using the move to trash button at the list
function attribute_features_check_invoices_integration_status_additional_data($params, &$additional_data) {

	$invoice_ids = array_var($params, 'invoice_ids', array());
	if (count($invoice_ids) > 0) {
		$related_invoices = array();
		
		$invoices = IncomeInvoices::instance()->findAll(array("conditions" => "id IN (". implode(',', $invoice_ids) .")"));

		foreach ($invoices as $invoice) {
			$rel_invoices = SplittedInvoiceThreads::instance()->getRelatedInvoices($invoice);
			foreach ($rel_invoices as $rel_inv) $related_invoices[] = $rel_inv;
		}

		if (count($related_invoices) > 0) {
			$rel_inv_nums = array();
			foreach ($related_invoices as $rel_inv) {
				$rel_inv_nums[] = $rel_inv->getInvoiceNumber() ? $rel_inv->getInvoiceNumber() : $rel_inv->getObjectName();
			}

			$additional_data['additional_trash_msg'] = lang("some selected invoices are split invoice therefore the following invoices will be also deleted", implode(', ', $rel_inv_nums));

		}
	}

}

// when trashed from invoice view
function attribute_features_after_object_trash($invoice, &$ignored) {
	if ($invoice instanceof IncomeInvoice) {
		SplittedInvoiceThreads::instance()->trashRelatedInvoices($invoice);
	}
}

// when trashing from invoices list
function attribute_features_after_content_object_trash($params, &$ignored) {
	$invoice = array_var($params, 'object');
	if ($invoice instanceof IncomeInvoice) {
		SplittedInvoiceThreads::instance()->trashRelatedInvoices($invoice);
	}
}

// when removing from trash
function attribute_features_after_content_object_untrash($params, &$ignored) {
	$invoice = array_var($params, 'object');
	if ($invoice instanceof IncomeInvoice) {
		SplittedInvoiceThreads::instance()->untrashRelatedInvoices($invoice);
	}
}

// when voiding an invoice also void the related ones
function attribute_features_after_invoice_cancel($params, &$ignored) {
	$invoice = array_var($params, 'invoice');
	if ($invoice instanceof IncomeInvoice) {
		SplittedInvoiceThreads::instance()->voidRelatedInvoices($invoice);
	}
}

// let qbo plugin know that the invoice to sync also has other "twin" invoices that need synchronization
function attribute_features_qbo_related_invoices_to_sync($params, &$related_invoices) {
	$main_invoice = array_var($params, 'main_invoice');
	if ($main_invoice instanceof IncomeInvoice) {
		$related_invoices = SplittedInvoiceThreads::instance()->getRelatedInvoices($main_invoice);
	}
}

