<?php

/**
 * Analysis controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class AnalysisController extends ApplicationController {

	var $plugin_name = "lab_samples";
	
	/**
	 * Construct the AnalysisController
	 *
	 * @access public
	 * @param void
	 * @return AnalysisController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
	//	Env::useHelper('functions', $this->plugin_name);
	} // __construct
	
	// ---------------------------------------------------
	//  Index
	// ---------------------------------------------------
	
	function init() {
		require_javascript("AnalysisManager.js", $this->plugin_name);
		ajx_current("panel", "analysis-containerpanel", null, null, true);
		ajx_replace(true);
	}
	
	function list_all() {
		ajx_current("empty");
		
		// Get all variables from request
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$dim_order = null;
		$cp_order = null;
		
		$action = array_var($_GET,'action');
		$attributes = array(
			"ids" => explode(',', array_var($_GET,'ids')),
		);
		
		//Resolve actions to perform
		$actionAnalysis = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		} 
		
		
		$join_params = array();
		$select_columns = array('*');
		$extra_conditions = "";
		$group_by = null;

		switch ($order){
			case 'updatedOn':
				$order = '`updated_on`';
				break;
			case 'createdOn':
				$order = '`created_on`';
				break;
			case 'name':
				$order = '`name`';
				break;
			default:
				$order = '`updated_on`';  
				break;
		}
		if (!$order_dir){
			switch ($order){
				case 'name': $order_dir = 'ASC'; break;
				default: $order_dir = 'DESC';
			}
		}
		
		if (str_starts_with($order, "cp_")){
			$cp_order = substr($order, 3);
			$order = 'customProp';
		} else if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		}
		
		Hook::fire("listing_extra_conditions", null, $extra_conditions);
		
		$only_count_result = array_var($_GET, 'only_result',false);
		$context = active_context();
		$res = Analysiss::instance()->listing(array(
			"order" => $order,
			"order_dir" => $order_dir,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"start" => $start,
			"limit" => $limit,
			"extra_conditions" => $extra_conditions,
			'count_results' => false,
			'only_count_results' => $only_count_result,
			"join_params"=> $join_params,
			"select_columns"=> $select_columns
		));
		$all_analysis = $res->objects; 
		
		// Prepare response object
		$object = $this->prepareObject($all_analysis, $start, $limit, $res);
		ajx_extra_data($object);
		tpl_assign("listing", $object);
		
	}
	
	/**
	 * Resolve action to perform
	 *
	 * @param string $action
	 * @param array $attributes
	 * @return string $analysis
	 */
	private function resolveAction($action, $attributes){
				 
		$resultAnalysis = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$analysis = Analysiss::instance()->findById($id);
					if ($analysis instanceof Analysis && $analysis->canDelete(logged_user())){
						try{
							DB::beginWork();
							$analysis->trash();
							DB::commit();
							ApplicationLogs::createLog($analysis, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultAnalysis = lang("error delete objects", $err) . ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultAnalysis = lang("success delete objects", $succ);
				}
				break;
			case "markasread":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$analysis = Analysiss::instance()->findById($id);
					if (!$analysis instanceof Analysis) continue;
					try {
						$analysis->setIsRead(logged_user()->getId(),true);						
						$succ++;
					} catch(Exception $e) {
						$err ++;
					} // try
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultAnalysis = lang("error markasread objects", $err) . "<br />" . ($succ > 0 ? lang("success markasread objects", $succ) : "");
				}
				break;
			case "markasunread":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$analysis = Analysiss::instance()->findById($id);
					if (!$analysis instanceof Analysis) continue;
					try {
						$analysis->setIsRead(logged_user()->getId(),false);						
						$succ++;
					} catch(Exception $e) {
						$err ++;
					} // try
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultAnalysis = lang("error markasunread objects", $err) . "<br />" . ($succ > 0 ? lang("success markasunread objects", $succ) : "");
				}
				break;
			case "archive":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$analysis = Analysiss::instance()->findById($id);
					if ($analysis instanceof Analysis && $analysis->canEdit(logged_user())){
						try{
							DB::beginWork();
							$analysis->archive();
							DB::commit();
							ApplicationLogs::createLog($analysis, ApplicationLogs::ACTION_ARCHIVE);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultAnalysis = lang("error archive objects", $err) . "<br />" . ($succ > 0 ? lang("success archive objects", $succ) : "");
				} else {
					$resultAnalysis = lang("success archive objects", $succ);
				}
				break;
			default:
				$resultAnalysis = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;		
		} // switch
		return array("errorAnalysis" => $resultAnalysis, "errorCode" => $resultCode);
	}
	
	/**
	 * Prepares return object for a list of emails and analysiss
	 *
	 * @param array $totMsg
	 * @param integer $start
	 * @param integer $limit
	 * @return array
	 */
	private function prepareObject($totMsg, $start, $limit, $res_obj) {
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"analysiss" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
		
		$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType(Analysiss::instance()->getObjectTypeId());
		$ids = array();
		for ($i = 0; $i < $limit; $i++){
			if (isset($totMsg[$i])){
				$analysis = $totMsg[$i];
				if ($analysis instanceof Analysis){
					$object["analysis"][$i] = array(
					    "id" => $analysis->getId(),
						"object_id" => $analysis->getId(),
						"ot_id" => $analysis->getObjectTypeId(),
						"type" => $analysis->getObjectTypeName(),
						"name" => $analysis->getObjectName(),
						"date" => $analysis->getUpdatedOn() instanceof DateTimeValue ? ($analysis->getUpdatedOn()->isToday() ? format_time($analysis->getUpdatedOn()) : format_datetime($analysis->getUpdatedOn())) : '',
						"is_today" => $analysis->getUpdatedOn() instanceof DateTimeValue ? $analysis->getUpdatedOn()->isToday() : 0,
						"updaterId" => $analysis->getUpdatedById() ? $analysis->getUpdatedById() : $analysis->getCreatedById(),
						"updaterName" => $analysis->getUpdatedById() ? $analysis->getUpdatedByDisplayName() : $analysis->getCreatedByDisplayName(),
						"memPath" => json_encode($analysis->getMembersIdsToDisplayPath()),
					);
					$ids[] = $analysis->getId();
					
					foreach ($custom_properties as $cp) {
						$object["analysis"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $analysis);
					}
    			}
			}
		}
		
		$read_objects = ReadObjects::getReadByObjectList($ids, logged_user()->getId());
		if (is_array(array_var($object, "analysis"))) {
			foreach($object["analysis"] as &$data) {
				$data['isRead'] = isset($read_objects[$data['object_id']]);
			}
		}
		
		return $object;
	}
	

	// ---------------------------------------------------
	//  Analysiss
	// ---------------------------------------------------
	
	/**
	 * View single analysis
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function view() {
		$this->addHelper('textile');

		$analysis = Analysiss::instance()->findById(get_id());
		if(!($analysis instanceof Analysis)) {
			flash_error(lang('analysis dnx'));
			ajx_current("empty");
			return;
		} // if

		if(!$analysis->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		} // if
		
		//read object for this user
		$analysis->setIsRead(logged_user()->getId(),true);
		
		tpl_assign('analysis', $analysis);
		tpl_assign('subscribers', $analysis->getSubscribers());
		ajx_extra_data(array("title" => $analysis->getTitle(), 'icon'=>$analysis->getIconClass()));
		ajx_set_no_toolbar(true);
		
		ApplicationReadLogs::createLog($analysis, ApplicationReadLogs::ACTION_READ);
	} // view
	

	/**
	 * Add analysis
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function add() {
		
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$notAllowedMember = '';
		if(!Analysis::canAdd(logged_user(), active_context(), $notAllowedMember )) {
			if (str_starts_with($notAllowedMember, '-- req dim --')) flash_error(lang('must choose at least one member of', str_replace_first('-- req dim --', '', $notAllowedMember, $in)));
			else trim($notAllowedMember) == "" ? flash_error(lang('you must select where to keep', lang('the analysis'))) : flash_error(lang('no context permissions to add',lang("analysiss"),$notAllowedMember ));
			ajx_current("empty");
			return;
		} // if
		
		$analysis = new Analysis();
		tpl_assign('analysis', $analysis);

		$analysis_data = array_var($_POST, 'analysis');
		if(!is_array($analysis_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$analysis_data = array();
		}

		if(is_array(array_var($_POST, 'analysis'))) {
			foreach ($analysis_data as $k => &$v) {
				$v = remove_scripts($v);
			}
			try {
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				DB::beginWork();
				
				$this->save_analysis($analysis_data, $analysis, $member_ids);
				
				DB::commit();
				ApplicationLogs::createLog($analysis, ApplicationLogs::ACTION_ADD);
				
				flash_success(lang('success add analysis', $analysis->getObjectName()));
				
	          	if (array_var($_REQUEST, 'modal')) {
	          		evt_add("reload current panel");
          		} else {
          			ajx_current("back");
          		}
          		
	          	ajx_add("overview-panel", "reload");

			} catch(Exception $e) {
				DB::rollback();
				$analysis->setNew(true);
				flash_error($e->getMessage());
				ajx_current("empty");
				
			} // try

		} // if
	} // add

	/**
	 * Edit specific analysis
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function edit() {
		$this->setTemplate('add');
		
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}

		$analysis = Analysiss::instance()->findById(get_id());
		if(!($analysis instanceof Analysis)) {
			flash_error(lang('analysis dnx'));
			ajx_current("empty");
			return;
		} // if

		if(!$analysis->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		} // if
		
		$analysis_data = array_var($_POST, 'analysis');
		if(!is_array($analysis_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
		} // if
		
		tpl_assign('analysis', $analysis);

		if(is_array(array_var($_POST, 'analysis'))) {
			foreach ($analysis_data as $k => &$v) {
				$v = remove_scripts($v);
			}
			try {
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				DB::beginWork();
				
				$this->save_analysis($analysis_data, $analysis, $member_ids);

				DB::commit();
				ApplicationLogs::createLog($analysis, ApplicationLogs::ACTION_EDIT);
				
				flash_success(lang('success edit analysis', $analysis->getObjectName()));
				ajx_current("back");
				
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
			} // try
		} // if
	} // edit
	
	
	function save_analysis($analysis_data, $analysis, $member_ids) {
		
		$analysis->setFromAttributes($analysis_data);
		$analysis->save();
		
		$object_controller = new ObjectController();
		
		if (!is_null($member_ids)) {
			$object_controller->add_to_members($analysis, $member_ids);
		}
		$object_controller->link_to_new_object($analysis);
		$object_controller->add_subscribers($analysis);
		$object_controller->add_custom_properties($analysis);
		
		$analysis->resetIsRead();
	}

	/**
	 * Delete specific analysis
	 *
	 * @access public
	 * @param void
	 * @return null
	 */
	function delete() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		ajx_current("empty");
		$analysis = Analysiss::instance()->findById(get_id());
		if(!($analysis instanceof Analysis)) {
			flash_error(lang('analysis dnx'));
			ajx_current("empty");
			return;
		} // if

		if(!$analysis->canDelete(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		} // if

		try {

			DB::beginWork();
			$analysis->trash();
			DB::commit();
			
			ApplicationLogs::createLog($analysis, ApplicationLogs::ACTION_TRASH);
			
			flash_success(lang('success delete analysis', $analysis->getObjectName()));
			if (array_var($_POST, 'popup', false)) {
				ajx_current("reload");
          	} else {
          		ajx_current("back");
          	}
          	ajx_add("overview-panel", "reload");          	
		} catch(Exception $e) {
			DB::rollback();
			flash_error(lang('error delete analysis'));
			ajx_current("empty");
		} // try
	} // delete



} // AnalysisController

?>