<?php
Hook::register('lab_samples');

/**
 * When saving analysis type: create associated analysis subtype
 */
function lab_samples_after_member_save($params, &$ret) {
	$member = array_var($params, 'member');
	$is_new = array_var($params, 'is_new');

	$at_ot = ObjectTypes::findByName("analysis_type");
	
	if ($at_ot->getId() == $member->getObjectTypeId() && $is_new) {
	
		if (Plugins::instance()->isActivePlugin('object_subtypes')) {
			Env::useHelper('functions', 'object_subtypes');
	
			$analisis_ot = ObjectTypes::findByName('analysis');
	
			create_object_subtype_from_member($member, $analisis_ot->getId());
	
			$template_ids = array();
			$template_ids_data = array_var($_REQUEST, 'task_templates');
				
			if (is_array($template_ids_data) && count($template_ids_data)) {
				foreach ($template_ids_data as $id => $value) {
					if (is_numeric($id)) $template_ids[] = $id;
				}
			}
			associate_task_templates_to_object_subtype_from_member($member, $analisis_ot->getId(), $template_ids);
		}
	}
}

/**
 * When creating a sample: instantiate the analysis for each analysis subtype associated to the selected analysis types
 */
function lab_samples_after_associating_members($params, &$ignored) {
	
	$member = array_var($params, 'member');
	$association = array_var($params, 'association');
	$is_new = array_var($params, 'is_new');
	$assoc_member_ids = array_var($params, 'assoc_member_ids');
	$rel_dim = array_var($params, 'rel_dim');
	$rel_ot = array_var($params, 'rel_ot');
	
	$sample_ot = ObjectTypes::findByName("sample");
	
	// samples: create analysis from associated analysis types
	if ($is_new && $sample_ot->getId() == $member->getObjectTypeId()) {
	
		$at_dimension = Dimensions::findByCode("analysis_types");
		$at_ot = ObjectTypes::findByName("analysis_type");
		
		if ($at_dimension->getId() == $rel_dim->getId() && $at_ot->getId() == $rel_ot->getId()) {
			
			// create associated analysis, each one can trigger a task template instantiation
			foreach ($assoc_member_ids as $at_id) {
				
				$analysis_type = Members::instance()->findById($at_id);
				if ($analysis_type instanceof Member) {
					$analysis = new Analysis();
					$subtype_code = $at_ot->getName()."_".$at_id;
					$subtype = ObjectSubtypes::instance()->findOne(array('conditions' => "code='$subtype_code'"));
						
					if ($subtype instanceof ObjectSubtype) {
						$data = array('object_subtype_id' => $subtype->getId(), 'name' => $analysis_type->getName());
		
						// classify analisys in its analisis type member and in the sample
						$classify_in_member_ids = array($member->getId(), $analysis_type->getId());
		
						// create analysis
						$analysis_controller = new AnalysisController();
						$analysis = $analysis_controller->save_analysis($data, $analysis, $classify_in_member_ids);
		
						// trigger template instantiation
						$_REQUEST['associated_member_id'] = $member->getId();
						$ret=null; Hook::fire('after_add_content_object', $analysis, $ret);
					}
				}
			}
		}
		
	}
}


function lab_samples_after_add_member($member, &$ret) {
	$sample_ot = ObjectTypes::findByName("sample");
	
	if ($sample_ot->getId() == $member->getObjectTypeId() && config_option('print_label_after_add_sample')) {
		evt_add("lab sample member added", array('id' => $member->getId(), 'name' => escape_character($member->getName())));
	}
}




function lab_samples_additional_dashboard_actions($ignored, &$actions) {
	
	$sample_ot = ObjectTypes::findByName("sample");
	$context = active_context();
	if (is_array($context)) {
		foreach ($context as $selection) {
			if ($selection instanceof Member && $selection->getObjectTypeId() == $sample_ot->getId()) {
				array_unshift($actions, array(
					'id' => 'reprint-sample-label',
					'name' => lang('reprint sample label'),
					'class' => 'link-ico ico-print',
					'href' => get_url('sample', 'print_sample_label', array('id' => $selection->getId())),
					'target' => '_blank',
				));
				break;
			}
		}
	}
	
	$enabled_dimensions = config_option("enabled_dimensions");
	$crpm_dim = Dimensions::findByCode('customer_project');
	
	if (in_array($crpm_dim->getId(), $enabled_dimensions)) {
		$sample_dot = DimensionObjectTypes::instance()->findOne(array('conditions' => 'dimension_id='.$crpm_dim->getId().' AND object_type_id='.$sample_ot->getId()));
		if ($sample_dot instanceof DimensionObjectType && $sample_dot->getEnabled()) {
			if (active_context_can_contain_member_type($crpm_dim->getId(), $sample_ot->getId())) {
				$actions[] = array(
					'id' => 'samples-list',
					'assoc_ot' => $sample_ot->getId(),
					'assoc_dim' => $crpm_dim->getId(),
					'name' => lang('samples list'),
					'class' => 'link-ico ico-sample',
					'onclick' => "og.openLink(og.getUrl('member', 'init', {dim_id:".$crpm_dim->getId().", type_id:".$sample_ot->getId()."}), {caller:'samples'}); return false;",
				);
			}
		}
	}
}



function lab_samples_custom_reports_additional_columns($args, &$ret) {
	// the dim columns are added by this plugin too
	if (Plugins::instance()->isActivePlugin('advanced_reports')) return;

	$ot = array_var($args, 'object_type');
	if ($ot instanceof ObjectType && in_array($ot->getType(), array('dimension_object', 'dimension_group'))) return;

	$dimensions = Dimensions::findAll ( array("conditions" => "code IN ('sample_types','sample_statuses','analysis_types')") );
	foreach ($dimensions as $dimension) {
		if (in_array($dimension->getId(), config_option('enabled_dimensions'))) {
				
			$name = $dimension->getName();
				
			$ret[] =  array('id' => 'dim_'.$dimension->getId(), 'name' => $name, 'type' => DATA_TYPE_STRING);
		}
	}
}



function lab_samples_custom_reports_object_types($parameters, &$object_types) {
	Env::useHelper('functions', 'crpm');
	$customers_dim = get_customers_dimension();
	if (in_array($customers_dim->getId(), config_option('enabled_dimensions'))) {
		$object_types[] = ObjectTypes::findByName('sample');
	}
}


/**
 * When saving an object, if it is classified under a client or a project then it must be classified in its type too.
 */
function lab_samples_after_object_save($object, &$ret) {

	if ($object instanceof ContentDataObject) {
		Env::useHelper('functions', 'crpm');
		$crpm_dimension = Dimensions::findByCode("customer_project");

		$sample_ot = ObjectTypes::findByName("sample");
		
		$sql = "SELECT om.member_id FROM ".TABLE_PREFIX."object_members om INNER JOIN ".TABLE_PREFIX."members m ON m.id=om.member_id
				WHERE om.object_id=".$object->getId()." AND om.is_optimization=0 AND (m.dimension_id=".$crpm_dimension->getId().")";
		$rows = DB::executeAll($sql);

		if (!is_array($rows)) return;

		foreach ($rows as $row) {
			$status_member = null;
			$type_member = null;
			$analysis_members = array();
			$crpm_member = Members::instance()->findById($row['member_id']);
			if ($crpm_member instanceof Member) {
				if ($crpm_member->getObjectTypeId() == $sample_ot->getId()) {
						
					$st_dimension = Dimensions::findByCode("sample_statuses");
					$st_ot = ObjectTypes::findByName("sample_status");
					$type_dimension = Dimensions::findByCode("sample_types");
					$type_ot = ObjectTypes::findByName("sample_type");
					$at_dimension = Dimensions::findByCode("analysis_types");
					$at_ot = ObjectTypes::findByName("analysis_type");

					$status_member = Members::instance()->findById(get_associated_status_member_id($crpm_member, $st_dimension, $st_ot));
					$type_member = Members::instance()->findById(get_associated_status_member_id($crpm_member, $type_dimension, $type_ot));
					$analysis_member_ids = get_all_associated_status_member_ids($crpm_member, $at_dimension, $at_ot);
					if (count($analysis_member_ids) > 0) {
						$analysis_members = Members::instance()->findAll(array('conditions' => 'id IN ('.implode(',', $analysis_member_ids).')'));
					}
						
				}
			}
			if ($type_member instanceof Member) {
				ObjectMembers::addObjectToMembers($object->getId(), array($type_member));
			}
			if ($status_member instanceof Member) {
				ObjectMembers::addObjectToMembers($object->getId(), array($status_member));
			}
			/* dont autoclassify in analysis types
			if (count($analysis_members) > 0) {
				ObjectMembers::addObjectToMembers($object->getId(), $analysis_members);
			}*/
		}

	}
}





function lab_samples_member_edit_categories($params, &$categories) {
	require_javascript("og/CustomPropertyFunctions.js");
	
	$member = array_var($params, 'member');
	$genid = array_var($params, 'genid');
	$aot = ObjectTypes::findByName('analysis');
	$atot = ObjectTypes::findByName('analysis_type');

	if ($member instanceof Member && $atot->getId() == $member->getObjectTypeId() && Plugins::instance()->isActivePlugin('advanced_core')) {
		$custom_properties_html = "";
		
		if (!$member->isNew()) {
			$code = $atot->getName() ."_". $member->getId();
			$subtype = ObjectSubtypes::instance()->findOne(array('conditions' => "code='$code'"));
			$subt_id = $subtype instanceof ObjectSubtype ? $subtype->getId() : -1;
			
			$request = array('id' => $aot->getId(), 'subtype_id' => $subt_id);
		} else {
			
			$request = array('id' => $aot->getId(), 'subtype_id' => -1);
		}
			
		$object_type = $aot;
		$dont_fire_hook = true;
		
		$extra_params = array('extra_conditions' => '', 'form_title' => '', 'add_link_text' => '');
		Hook::fire('list_custom_properties_for_type_extra_parameters', array('request' => $request, 'object_type' => $aot), $extra_params);
		
		$custom_properties = CustomProperties::instance()->findAll(array('conditions' => "object_type_id=".$aot->getId()." AND object_subtype_id=".$request['subtype_id']));
		
		ob_start();
		include get_template_path('list_custom_properties_for_type', 'administration');
		$custom_properties_html = ob_get_clean();
		
		
		// new tab with custom property definition
		$categories[] = array(
			'id' => $genid . 'subtype_custom_properties_div',
			'link_id' => $genid . 'subtype_custom_properties_link',
			'name' => lang('custom properties'),
			'content' => '<div class="form-tab" id="'.$genid.'subtype_custom_properties_div">'. $custom_properties_html .'</div>',
			'class' => 'subtype-custom-properties-container',
		);

		// hide original custom properties tab
		echo "
		<script>$(function(){
			$('#".$genid."add_custom_properties_li').hide();
			$('#".$genid."subtype_custom_properties_div .coInputHeader').remove();
			$('#".$genid."subtype_custom_properties_div button.submit ').remove();
		});</script>";
		
		
		
		
		// tab for task template instantiation
		$available_task_templates = COTemplates::instance()->findAll();
		$subtype_templates = ObjectSubtypeTaskTemplates::instance()->findAll(array('conditions' => "object_type_id=".$aot->getId()." AND object_subtype_id=".$request['subtype_id']));
		$selected_ids = array();
		foreach ($subtype_templates as $t) {
			$selected_ids[] = $t->getTaskTemplateId();
		}
		
		ob_start();
		include get_template_path('subtype_task_templates_tab', 'object_subtypes', 'advanced_core');
		$task_templates_html = ob_get_clean();
		
		$categories[] = array(
			'id' => $genid . 'subtype_task_templates_div',
			'link_id' => $genid . 'subtype_task_templates_link',
			'name' => lang('templates'),
			'content' => '<div class="form-tab" id="'.$genid.'subtype_task_templates_div">'. $task_templates_html .'</div>',
			'class' => 'subtype-task-templates-container',
		);
	}
}