<?php
/**
 * Managed Events controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class ManagedEventsController extends ApplicationController {

	var $plugin_name = "managed_events";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('format');
	}
	
	function init() {
		require_javascript("ManagedEventsManager.js", $this->plugin_name);
		ajx_current("panel", "events-containerpanel", null, null, true);
		ajx_replace(true);
	}
	
	

	
	
	function get_events_list_config() {
		ajx_current("empty");
		
		ob_start();
		tpl_display(get_template_path('events_list_grid_config/columns', 'managed_events', $this->plugin_name));
		tpl_display(get_template_path('events_list_grid_config/toolbar', 'managed_events', $this->plugin_name));
		
		$html = ob_get_clean();
		
		ajx_extra_data(array('html' => $html));
	}
	
	
	
	


	function list_all() {
	
		ajx_current("empty");
		// Get all variables from request
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		$attributes = array("ids" => explode(',', array_var($_GET,'ids')));
		$dim_order = null;
		$cp_order = null;
		
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
	
		$select_columns = array("*");
		$extra_conditions = "";
		$join_params = null;
	
		if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		} else if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		} else {
			switch ($order) {
				case 'description':
					$order = '`e`.`description`';
					break;
				case 'start':
					$order = '`e`.`start`';
					break;
				case 'duration':
					$order = '`e`.`duration`';
					break;
				default:
					$order = "name";
					break;
			}
		}
	
		Hook::fire("listing_extra_conditions", null, $extra_conditions);
		
		$extra_conditions .= " AND e.is_managed_event=1 ";
		
		$only_count_result = array_var($_GET, 'only_result', false);
	
		$result = ProjectEvents::instance()->listing(array(
				"order" => $order,
				"order_dir" => $order_dir,
				"dim_order" => $dim_order,
				"cp_order" => $cp_order,
				"select_columns" => $select_columns,
				"extra_conditions" => $extra_conditions,
				"count_results" => false,
				"only_count_results" => $only_count_result,
				"join_params" => $join_params,
				"start" => $start,
				"limit" => $limit,
		));
			
		$events = $result->objects;
		$object = $this->prepareObject($events, $start, $limit, $result);
	
		ajx_current("empty");
		ajx_extra_data($object);
		tpl_assign("listing", $object);
	
	}
	
	private $event_cps = null;
	
	private function prepareObject($events, $start, $limit, $res_obj) {
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"managed_events" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
	
		if (!$this->event_cps) {
			$this->event_cps = CustomProperties::getAllCustomPropertiesByObjectType(ProjectEvents::instance()->getObjectTypeId());
			
			$ot_id = ObjectTypes::findByName('managed_event')->getId();
			$more_cps = CustomProperties::getAllCustomPropertiesByObjectType(ManagedEvents::instance()->getObjectTypeId());
			$this->event_cps = array_merge($this->event_cps, $more_cps);
		}
		
		$dimension = Dimensions::findByCode("managed_events");
		$member_type = ObjectTypes::findByName("managed_event");
		$custom_properties = $this->event_cps;
		$count = min(array($limit, count($events)));
		
		for ($i = 0; $i < $count; $i++){
			$event = $events[$i];
			if ($event instanceof ProjectEvent){
				$info = $event->getArrayInfo();
				
				$info['memPath'] = json_encode($event->getMembersIdsToDisplayPath(true));
				
				$managed_event = ManagedEvents::instance()->findOne(array("conditions" => array("event_id=?",$event->getId())));

				if ($managed_event instanceof ManagedEvent) {
					
					$member = Members::findOneByObjectId($managed_event->getId(), $dimension->getId());
	
					if ($member instanceof Member) {
						$info['member_id'] = $member->getId();
						$info['dimension_code'] = 'managed_events';
					}
					
					$info['managed_event_id'] = $managed_event->getId();
				
					foreach ($custom_properties as $cp) {
						if ($cp->getCode() == 'description_special') {
							$object["managed_events"][$i]['cp_'.$cp->getId()] = $info['description'];
							$info['description'] = '';
						} else {
							if ($managed_event instanceof ManagedEvent && $cp->getObjectTypeId() == $member_type->getId()) {
								$cp_val = get_custom_property_value_for_listing($cp, $managed_event);
							} else {
								$cp_val = get_custom_property_value_for_listing($cp, $event);
							}
							$info['cp_'.$cp->getId()] = $cp_val;
						}
					}
				}
				
				$object["managed_events"][$i] = $info;
			}
		}
	
		return $object;
	}
	
	
	
	function view() {
		$this->redirectTo('event', 'view', array('id' => get_id()));
	}
	
	
	function edit() {
		
		$object = ProjectEvents::instance()->findById(get_id());
		if (!$object instanceof ProjectEvent) {
			ajx_current("empty");
			flash_error(lang('event dnx'));
			return;
		}
		
		$dimension = Dimensions::findByCode('managed_events');
		
		$managed_event = ManagedEvents::instance()->findByEventId($object->getId());
		if (!$managed_event instanceof ManagedEvent) {
			ajx_current("empty");
			flash_error(lang('event dnx'));
			return;
		}
			
		$managed_event_member = Members::findOneByObjectId($managed_event->getId(), $dimension->getId());
		if ($managed_event_member instanceof Member) {
			$this->redirectTo('member', 'edit', array('id' => $managed_event_member->getId()));
		} else {
			ajx_current("empty");
			flash_error(lang('event dnx'));
			return;
		}
		
	}
}