<?php 
class MemberCustomPropertiesController extends ApplicationController {
	
	var $plugin_name = "member_custom_properties";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
	}
	
	
	function list_custom_properties_for_type() {
		if(!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
	
		$object_type_id = get_id();
		$object_type = ObjectTypes::instance()->findById($object_type_id);
		if (!$object_type instanceof ObjectType) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
	
		$custom_properties = MemberCustomProperties::getAllMemberCustomPropertiesByObjectType($object_type_id, 'all', true);
	
		tpl_assign('object_type', $object_type);
		tpl_assign('custom_properties', $custom_properties);
		
		$genid = gen_id();
		tpl_assign('genid', $genid);
		
		$save_js_function = "og.saveMemberTypeCustomProperties('$genid');";
		tpl_assign('save_js_function', $save_js_function);
	
		
		$this->setTemplate(get_template_path('list_custom_properties_for_type', 'administration'));
	}
	
	
	function save_custom_properties_for_type() {
		ajx_current("empty");
	
		if(!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			return;
		}
	
		$obj_type_id = array_var($_REQUEST, 'ot_id');
		$object_type = ObjectTypes::instance()->findById($obj_type_id);
		if (!$object_type instanceof ObjectType) {
			flash_error(lang('object dnx'));
			return;
		}
	
		$custom_properties = json_decode(array_var($_POST, 'custom_properties'), true);
	
		if (is_array($custom_properties)) {
			try {
				DB::beginWork();
				$processed_ids = array();
				foreach ($custom_properties as $order => $data) {
					
					if (in_array($data['id'], $processed_ids)) continue;
					$processed_ids[] = $data['id'];
					
					$new_cp = null;
					if($data['id'] != ''){
						$new_cp = MemberCustomProperties::getCustomProperty($data['id']);
					}
					if ($new_cp == null) {
						$new_cp = new MemberCustomProperty();
					}
						
					if($data['deleted'] == "1"){
						if (!$new_cp->isNew()) {
							$new_cp->delete();
						}
						continue;
					}
	
					if (array_var($data, 'name') == '') {
						throw new Exception(lang('custom property name empty'));
					}
	
					$new_cp->setFromAttributes($data);
					$new_cp->setObjectTypeId($obj_type_id);
					$new_cp->setOrder($order);
					
					if ($new_cp->getCode() == 'color_special') {
						$new_cp->setType('color');
					} 
	
					if ($data['type'] == 'list') {
						$values = array();
						$list = explode(",", $data['values']);
						foreach ($list as $l) {
							$values[] = trim($l);
						}
						$value = implode(",", $values);
						$new_cp->setValues($value);
					} else if($data['type'] == 'boolean'){
					    $new_cp->setDefaultValue($data['default_value_bool']);
					}else {
						$new_cp->setValues($data['values']);
					}
	
					if (!$data['id'] || is_numeric($data['id'])) {
						$new_cp->save();
					}
				
					$ret = null;
					Hook::fire('after_custom_property_save', array('cp' => $new_cp, 'data' => $data, 'order' => $order), $ret);
				}
					
				DB::commit();
				flash_success(lang('custom properties updated'));
				ajx_current("back");
					
			} catch (Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
			}
				
		}
	}
	
	
	
	
	function get_custom_properties(){
		$object_type = array_var($_GET, 'object_type');
		if($object_type){
			$cp = MemberCustomProperties::getAllMemberCustomPropertiesByObjectType($object_type, 'all', true);
			$customProperties = array();
			foreach($cp as $custom){
				$prop = array();
				$prop['id'] = $custom->getId();
				$prop['name'] = $custom->getName();
				$prop['object_type'] = $custom->getObjectTypeId();
				$prop['description'] = $custom->getDescription();
				$prop['type'] = $custom->getType();
				$prop['values'] = escape_character($custom->getValues());
				$prop['default_value'] = $custom->getDefaultValue();
				$prop['required'] = $custom->getIsRequired();
				$prop['multiple_values'] = $custom->getIsMultipleValues();
				$prop['visible_by_default'] = $custom->getVisibleByDefault();
				$prop['show_in_lists'] = $custom->getShowInLists();
				$prop['order'] = $custom->getOrder();
				$prop['co_types'] = '';
				$prop['is_special'] = $custom->getIsSpecial();
				$prop['is_disabled'] = $custom->getIsDisabled();
				$customProperties[] = $prop;
			}
			ajx_current("empty");
			ajx_extra_data(array("custom_properties" => $customProperties));
		}
	}
	
	
	
	
	/**
	 * Adds the custom properties of a member into the database.
	 * 
	 * @param $object
	 * @return unknown_type
	 */
	function add_custom_properties(Member $member, $associated_object=null,$data_api = null) {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
        $member_custom_properties = array_var($_POST, 'member_custom_properties');
		if(!is_null($data_api)){
            $member_custom_properties = array_var($data_api, 'object_custom_properties');
        }
        $time_cp_values = array_var($member_custom_properties, "time");
		
		$ot = ObjectTypes::instance()->findById($member->getObjectTypeId());
		if ($ot->getName()=='project_folder' || $ot->getName()=='customer_folder') {
			$ot = ObjectTypes::findByName('folder');
		}

		$object_type = ObjectTypes::instance()->findById($ot->getId());
		$object = null;
		if ($object_type->getType() == 'dimension_object') {
			if ($associated_object instanceof ContentDataObject) {
				$object = Objects::findObject($associated_object->getId());;
			} else {
				$object = Objects::findObject($member->getObjectId());
			}
			if (!$member_custom_properties) {
				$member_custom_properties = array_var($_POST, 'object_custom_properties');
			}
			// save object custom properties
			$obj_controller = new ObjectController();
			$obj_controller->add_custom_properties($object, $member_custom_properties);
			return;
		}
		
		$customProps = MemberCustomProperties::getAllMemberCustomPropertiesByObjectType($ot->getId());
		//Sets all boolean custom properties to 0. If any boolean properties are returned, they are subsequently set to 1.
		foreach($customProps as $cp){
			if($cp->getType() == 'boolean'){
				$custom_property_value = MemberCustomPropertyValues::getMemberCustomPropertyValue($member->getId(), $cp->getId());
				if(!$custom_property_value instanceof MemberCustomPropertyValue){
					$custom_property_value = new MemberCustomPropertyValue();
				}
				$custom_property_value->setMemberId($member->getId());
				$custom_property_value->setCustomPropertyId($cp->getId());
				$custom_property_value->setValue(0);
				$custom_property_value->save();
			}
		}
		if (is_array($member_custom_properties)){
			foreach($member_custom_properties as $id => $value){
				if (!is_numeric($id)) continue;
				//Get the custom property
				$custom_property = null;
				foreach ($customProps as $cp){
					if ($cp->getId() == $id){
						$custom_property = $cp;
						break;
					}
				}
				
				if ($custom_property instanceof MemberCustomProperty){
					// save dates in standard format "Y-m-d H:i:s", because the column type is string
					if ($custom_property->getType() == 'date' || $custom_property->getType() == 'datetime') {
						$d_format = $custom_property->getType() == 'datetime' ? "Y-m-d H:i:s" : "Y-m-d";
						if(is_array($value)){
							$newValues = array();
							foreach ($value as $idx => $val) {
								if ($val == '' || $val == EMPTY_DATE || $val == EMPTY_DATETIME || $val == date_format_tip(user_config_option('date_format'))) {
									continue; // dont save
								}
								$dtv = DateTimeValueLib::dateFromFormatAndString(user_config_option('date_format'), $val);
								if ($custom_property->getType() == 'datetime') {
									$time_val = getTimeValue($time_cp_values[$id][$idx]);
									if ($time_val) {
										$dtv->setHour(array_var($time_val, 'hours', 0));
										$dtv->setMinute(array_var($time_val, 'mins', 0));
									}
								}
								$newValues[] = $dtv->format($d_format);
							}
							$value = $newValues;
						} else {
							if ($value == '' || $value == EMPTY_DATE || $value == EMPTY_DATETIME || $value == date_format_tip(user_config_option('date_format'))) {
								continue; // dont save
							}
							$dtv = DateTimeValueLib::dateFromFormatAndString(user_config_option('date_format'), $value);
							if ($custom_property->getType() == 'datetime') {
								$time_val = getTimeValue($time_cp_values[$id]);
								if ($time_val) {
									$dtv->setHour(array_var($time_val, 'hours', 0));
									$dtv->setMinute(array_var($time_val, 'mins', 0));
								}
							}
							$value = $dtv->format($d_format);
						}
					}

                    //Check inline images in tmp
                    if ($custom_property->getType() == 'memo') {
                        $file_content = $value;
                        $image_file_ids = array();
                        preg_match_all("/<img[^>]*src=[\"']([^\"']*)[\"']/", $file_content, $matches);
                        $urls = array_var($matches, 1);
                        if (is_array($urls)) {
                            $object_controller = new ObjectController();
                            $file_mem_id = array($member->getId());
                            $img_num = 1;
                            foreach ($urls as $url) {
                                if (strpos(html_entity_decode($url), get_url('files', 'download_image')) === false ) {
                                    // prepare the file object
                                    $file = new ProjectFile();
                                    $name = $object->getName();
                                    $file->setObjectTypeId($file->getObjectTypeId());
                                    $file->setFilename($name);
                                    $file->setIsVisible(true);

                                    $img_file_id = FilesController::upload_document_image($url, $file->getFilename(), $img_num);
                                    $file_content = str_replace($url, get_url('files', 'download_image', array('id' => $img_file_id, 'inline' => 1)) , $file_content);
                                    $image_file_ids[] = $img_file_id;

                                    $uploaded_file = ProjectFiles::instance()->findById($img_file_id);
                                    if($uploaded_file instanceof ProjectFile) {
                                        $object_controller->add_to_members($uploaded_file, $file_mem_id);
                                    }
                                }
                                $img_num++;
                            }
                            $value = $file_content;
                        }
                    }
					
					if(is_array($value)){
						if ($custom_property->getType() == 'address') {
							// Address custom property
							if ($custom_property->getIsRequired()) {
								if (array_var($value, 'street') == '' && array_var($value, 'city') == '' && array_var($value, 'state') == '' && array_var($value, 'country') == '' && array_var($value, 'zip_code') == '') {
									throw new Exception(lang('custom property value required', $custom_property->getName()));
								}
							}
							$val = array_var($value, 'type') .'|'. array_var($value, 'street') .'|'. array_var($value, 'city') .'|'. array_var($value, 'state') .'|'. array_var($value, 'country') .'|'. array_var($value, 'zip_code'); 
							MemberCustomPropertyValues::deleteMemberCustomPropertyValues($member->getId(), $id);
							$custom_property_value = new MemberCustomPropertyValue();
							$custom_property_value->setMemberId($member->getId());
							$custom_property_value->setCustomPropertyId($id);
							$custom_property_value->setValue($val);
							$custom_property_value->save();
							
						} else if ($custom_property->getType() == 'list') {
							MemberCustomPropertyValues::deleteMemberCustomPropertyValues($member->getId(), $id);
							foreach($value as $key => $val){
								if($val){
									$custom_property_value = new MemberCustomPropertyValue();
									$custom_property_value->setMemberId($member->getId());
									$custom_property_value->setCustomPropertyId($id);
									$custom_property_value->setValue($key);
									$custom_property_value->save();
								}
							}
							
						} else {
							//Save multiple values
							MemberCustomPropertyValues::deleteMemberCustomPropertyValues($member->getId(), $id);
							foreach($value as &$val){
								if (is_array($val)) {
									// CP type == table
									$str_val = '';
									foreach ($val as $col_val) {
										$col_val = str_replace("|", "\|", $col_val);
										$str_val .= ($str_val == '' ? '' : '|') . $col_val;
									}
									$val = $str_val;
								}
								if($val != ''){
									$custom_property_value = new MemberCustomPropertyValue();
									$custom_property_value->setMemberId($member->getId());
									$custom_property_value->setCustomPropertyId($id);
									$custom_property_value->setValue($val);
									$custom_property_value->save();
								}
							}
						}

					} else if($custom_property->getType() == 'image') {
					
						if (trim($value) != "") {
							if (file_exists(ROOT."/tmp/$value")) {
								$info = array_var($_SESSION, $value);
								$type = array_var($info, "type");
					
								$repo_id = FileRepository::addFile(ROOT."/tmp/$value", array('type' => $type, 'public' => true));
					
								$object_to_save = array(
									'repository_id' => $repo_id,
									'size' => array_var($info, "size"),
									'type' => $type,
								);
					
								$cpv = MemberCustomPropertyValues::getMemberCustomPropertyValue($object->getId(), $id);
								if (!$cpv instanceof MemberCustomPropertyValue) {
									$cpv = new MemberCustomPropertyValue();
									$cpv->setMemberId($member->getId());
									$cpv->setCustomPropertyId($id);
								}
								$cpv->setValue(json_encode($object_to_save));
								$cpv->save();
							}
								
						} else {
							MemberCustomPropertyValues::deleteMemberCustomPropertyValues($member->getId(), $id);
						}
						
					}else{
						if ($custom_property->getIsRequired() && $custom_property->getType() != 'boolean') {
							if (!$value) {
								throw new Exception(lang('custom property value required', $custom_property->getName()));
							}
						}
						$cpv = MemberCustomPropertyValues::getMemberCustomPropertyValue($member->getId(), $id);
						if($cpv instanceof MemberCustomPropertyValue){
							$custom_property_value = $cpv;
						} else {
							$custom_property_value = new MemberCustomPropertyValue();
						}
						$custom_property_value->setMemberId($member->getId());
						$custom_property_value->setCustomPropertyId($id);
						$custom_property_value->setValue($value);
						$custom_property_value->save();
					}
					
					if ($custom_property->getIsSpecial() && !$custom_property->getIsDisabled()) {
						$modified = false;
						switch ($custom_property->getCode()) {
							case 'description_special':
								$modified = true;
								$member->setDescription($value);
								break;
							case 'color_special':
								$modified = true;
								$member->setColor(array_var($value,'color'));
								break;
						}
						if ($modified) {
							$member->save();
						}
					}
					
					//Add to searchable objects
					if ($object && $object->isSearchable() && 
						($custom_property->getType() == 'text' || $custom_property->getType() == 'list' || $custom_property->getType() == 'numeric')){
						
						$name = escape_character($custom_property->getName());
						$searchable_object = SearchableObjects::instance()->findOne(array("conditions" => "`rel_object_id` = ".$object->getId()." AND `column_name` = '$name'"));
						if (!$searchable_object)
							$searchable_object = new SearchableObject();
						
						if (is_array($value))
							$value = implode(', ', $value);
							
						$searchable_object->setRelObjectId($object->getId());
						$searchable_object->setColumnName($name);
						$searchable_object->setContent($value);
						
						$searchable_object->save();
					}
				}
			}
		}

	}
	
}


