<?php

/**
 * MemberCustomPropertyValue class
 */
class MemberCustomPropertyValue extends BaseMemberCustomPropertyValue {

	/**
	 * Construct the object
	 *
	 * @param void
	 * @return null
	 */
	function __construct() {
		parent::__construct();
	} // __construct

	/**
	 * Validate before save
	 *
	 * @access public
	 * @param array $errors
	 * @return null
	 */
	function validate($errors) {
		$cp = MemberCustomProperties::getCustomProperty($this->getCustomPropertyId());
		if($cp instanceof MemberCustomProperty){
			if($cp->getIsRequired() && ($this->getValue() == '')){
				$errors[] = lang('custom property value required', $cp->getName());
			}
			if($cp->getType() == 'numeric'){
				if($cp->getIsMultipleValues()){
					foreach(explode(',', $this->getValue()) as $value){
						if($value != '' && !is_numeric($value)){
							$errors[] = lang('value must be numeric', $cp->getName());
						}
					}
				}else{
					if($this->getValue() != '' && !is_numeric($this->getValue())){
						$errors[] = lang('value must be numeric', $cp->getName());
					}
				}
			}
		}//if
	} // validate

	
	function format_value() {
		$formatted = "";
		
		$cp = MemberCustomProperties::getCustomProperty($this->getCustomPropertyId());
		if($cp instanceof MemberCustomProperty) {
			switch ($cp->getType()) {
				case 'text':
				case 'numeric':
					$formatted = $this->getValue();
					break;
				case 'memo':
					$formatted = nl2br($this->getValue());
					break;
				case 'user':
				case 'contact':
					$c = Contacts::instance()->findById($this->getValue());
					$formatted = $c instanceof Contact ? clean($c->getObjectName()) : '';
					break;
				case 'boolean':
					//$formatted = '<div class="db-ico ico-'. ($this->getValue() ? 'complete' : 'delete') .'">&nbsp;</div>';
				    $formatted = format_boolean_to_string($this->getValue());
					break;
				case 'date':
					if ($this->getValue() != '' && $this->getValue() != EMPTY_DATE && $this->getValue() != EMPTY_DATETIME) {
						$dtv = DateTimeValueLib::dateFromFormatAndString(DATE_MYSQL, $this->getValue());
						if ($dtv instanceof DateTimeValue) {
							$formatted = format_date($dtv, null, 0);
						}
					}
					break;
				case 'list':
					$possible_values = explode(',', $cp->getValues());
					foreach ($possible_values as $pval) {
						if (strpos($pval, '@') !== false) {
							$exp = explode('@', $pval);
							if ($this->getValue() == $exp[0]) $formatted = $exp[1];
						} else {
							if ($this->getValue() == $pval) $formatted = $this->getValue();
						}
					}
					break;
				case 'table':
							
						$headers = explode(',', $cp->getValues());
						$rows = array();
					
						$cpvs = MemberCustomPropertyValues::getMemberCustomPropertyValues($this->getMemberId(), $cp->getId());
						foreach ($cpvs as $cpval) {
							$row = array();
							$values = str_replace("\|", "%%_PIPE_%%", $cpval->getValue());
							$exploded = explode("|", $values);
							foreach ($exploded as &$v) {
								$v = str_replace("%%_PIPE_%%", "|", $v);
								$v = escape_character($v);
								$row[] = $v;
							}
							$rows[] = $row;
						}
					
						$table_html = '<div class="og-custom-properties"><table class="og-add-custom-properties"><tr>';
						foreach ($headers as $h) $table_html .= '<th>'.$h.'</th>';
						$table_html .= '</tr>';
					
						foreach ($rows as $row) {
							$table_html .= '<tr>';
							foreach ($row as $rowval) $table_html .= '<td>'.$rowval.'</td>';
							$table_html .= '</tr>';
						}
						$table_html .= '</table></div>';
					
						$formatted = $table_html;
					
					break;
				case 'address':
					$values = str_replace("\|", "%%_PIPE_%%", $this->getValue());
					$exploded = explode("|", $values);
					foreach ($exploded as &$v) {
						$v = str_replace("%%_PIPE_%%", "|", $v);
						$v = escape_character($v);
					}
					if (count($exploded) > 0) {
						$address_type = array_var($exploded, 0, '');
						$street = array_var($exploded, 1, '');
						$city = array_var($exploded, 2, '');
						$state = array_var($exploded, 3, '');
						$country = array_var($exploded, 4, '');
						$zip_code = array_var($exploded, 5, '');
							
						$out = $street;
						if($city != '') $out .= ' - ' . $city;
						if($state != '') $out .= ' - ' . $state;
						if($country != '') $out .= ' - ' . lang("country $country");
					
						$formatted = '<div class="info-content-item">'. $out .'&nbsp;<a class="map-link coViewAction ico-map" href="http://maps.google.com/?q='. $out .'" target="_blank">'. lang('map') .'</a></div>';
					}
					break;

				case 'image':
					$formatted = render_image_custom_property_value("", $cp, $this->getValue());
					break;
					
				default:
					$formatted = $this->getValue();
			}
		}
		
		return $formatted;
	}

} // ObjectPropertyValue

?>