<?php

/**
 * Members
 *
 * @author Feng Office
 */
class MemberTemplatesTaskTemplates extends BaseMemberTemplatesTaskTemplates {
	
	/**
	 * This function returns all the task templates 
	 * 
	 * @param integer $template_id
	 * @param array $associated_member_ids
	 * @return array
	 */
    static function getTemplateIdsToInstantiate($template_id, $associated_member_ids = array(), $is_new = false) {
		
		sort($associated_member_ids, SORT_NUMERIC);
		
		If ($is_new == true) {
		    $conditions = "template_id = $template_id AND is_initial = 1 ";
		} else {
		    $conditions = "template_id = $template_id AND associated_member_ids = '".implode(',', $associated_member_ids)."'";
		}
		
		$reg = self::instance()->findOne(array('conditions' => $conditions));
		if ($reg instanceof MemberTemplatesTaskTemplate && trim($reg->getTaskTemplateIds()) != '') {
			$tids = explode(',', trim($reg->getTaskTemplateIds()));
			foreach ($tids as $k => &$tid) {
				if (!is_numeric($tid)) unset($tids[$k]);
			}
			return $tids;
		} else {
			
			// check for 'any' (if template member conditions are included in the associated_member_ids)
			$tids = array();
			$all_possible_regs = self::instance()->findAll(array("conditions" => "template_id=$template_id"));
			foreach ($all_possible_regs as $reg) {
				if ($reg instanceof MemberTemplatesTaskTemplate && trim($reg->getTaskTemplateIds()) != '') {
					$mem_cond_ids = explode(',', $reg->getAssociatedMemberIds());
					if (!is_array($mem_cond_ids) || count($mem_cond_ids) == 0) continue;
					// if all elements of $mem_cond_ids are contained in $associated_member_ids then array_diff will return zero elements and this is the template step that we want
					$diff = array_diff($mem_cond_ids, $associated_member_ids);
					if (count($diff) == 0) {
						$tids = explode(',', trim($reg->getTaskTemplateIds()));
						foreach ($tids as $k => &$tid) {
							if (!is_numeric($tid)) unset($tids[$k]);
						}
						break;
					}
				}
			}
			
			return $tids;
		}
		
	}
	
	
	static function getWorkflowStepsInfo($template_id) {
		$steps_info = array();
		$steps = self::instance()->findAll(array('conditions' => "template_id = $template_id AND is_initial=0"));
		
		foreach ($steps as $step) {
			$member_associations = explode(',', $step->getAssociatedMemberIds());
			$tt_ids = explode(',', $step->getTaskTemplateIds());
			$steps_info[] = array('members' => $member_associations, 'task_templates' => $tt_ids);
		}
		
		return $steps_info;
	}
}
