<?php 
Hook::register("microsoft_integrations_mail");

function microsoft_integrations_mail_download_emails_oauth2($params, &$mailsReceived) {

	$account = array_var($params, 'account');
	$maxPerAccount = array_var($params, 'maxPerAccount');

	if ($account instanceof MailAccount && $account->getUsesOauth2() && $account->getOauth2Provider() == 'microsoft') {

		$ms_controller = new MicrosoftController();

		$mailsReceived += $ms_controller->download_emails($account, $maxPerAccount);

	}

}

function microsoft_integrations_mail_get_microsoft_mail_account_info($params, &$account_info) {
	$account = array_var($params, 'account');
	if ($account instanceof MailAccount && $account->getUsesOauth2() && $account->getOauth2Provider() == 'microsoft') {
		$ms_controller = new MicrosoftController();
		$account_info = $ms_controller->get_mail_account_info($account);
	}
}



function microsoft_integrations_mail_oauth2_providers($account, &$providers) {

	if (!is_array($providers)) $providers = array();

	$providers[] = array(
		'code' => 'microsoft',
		'name' => 'Microsoft',
		'logo' => get_image_url('16x16/microsoft.png'),
		'connect_url' => get_url(
			'microsoft', 
			'connect_to_microsoft_mail_account',
			 array(
				'controller'=> 'microsoft', 
				'action' => 'reload_current_panel', 
				'account_id'=> $account->getId(), 
				'id'=>'dummy'
			)
		)
	);

}

function microsoft_integrations_mail_mail_account_get_oauth_access_token($params, &$token) {
	$account = array_var($params, 'mail_account');
	if ($account instanceof MailAccount && $account->getUsesOauth2() && $account->getOauth2Provider() == 'microsoft') {
		$ms_controller = new MicrosoftController();
		$token = $ms_controller->get_mail_account_oauth_access_token($account);
	}
}

function microsoft_integrations_mail_get_imap_connect_config_using_mail_account($params, &$imap_configs) {
	$account = array_var($params, 'account');
	if ($account instanceof MailAccount && $account->getUsesOauth2() && $account->getOauth2Provider() == 'microsoft') {
		$imap_configs = array(
			'host' => 'outlook.office365.com',
			'port' => 993,
			'encryption' => 'ssl'
		);
	}
}

function microsoft_integrations_mail_get_imap_login_config_using_mail_account($params, &$imap_login_configs) {
	$account = array_var($params, 'account');
	if ($account instanceof MailAccount && $account->getUsesOauth2() && $account->getOauth2Provider() == 'microsoft') {
		$username = $account->getEmailAddress();
		
		// Get MS access token token
		$ms_controller = new MicrosoftController();
		$token = $ms_controller->get_mail_account_oauth_access_token($account);
		//$xoauth2_token = base64_encode("user=$username\1auth=Bearer $token\1\1");

		$imap_login_configs = array(
			'email' => $username,
			'password' => $token,
			'use_authenticate' => 'XOAUTH2',
			'select_mail_box' => false
		);
	}
}
