<?php
Hook::register('multiple_bussiness_inv');


// autoclassify in business company associated to the client if not done in the form or in the automatic generation
function multiple_bussiness_inv_after_invoice_save($params, &$ignored) {
	$invoice = array_var($params, 'invoice');
	/* @var $invoice IncomeInvoice */
	
	if ($invoice instanceof IncomeInvoice) {
		$business_company_dim = Dimensions::findByCode('business_companies');
		
		// if already classified in business company don't do anything
		foreach ($invoice->getMembers() as $member) {
			if ($member->getDimensionId() == $business_company_dim->getId()) return;
		}
		
		// if not classified in business_companies dimension -> classify in the business_company associated to the client
		$client_ot = ObjectTypes::findByName('customer');
		$client_member = $invoice->getMemberOfType($client_ot->getId());

		if ($client_member instanceof Member) {
			// get the client's business company
			$business_company_id = get_associated_status_member_id($client_member, $business_company_dim);
			if ($business_company_id > 0) {
				$business_company = Members::getMemberById($business_company_id);
				// classify in business company
				if ($business_company instanceof Member) {
					$invoice->addToMembers(array($business_company));
				}
			}
		}
	}
}

function multiple_bussiness_inv_override_invoice_print_business_company($params, &$array_pay_to_information) {

	$invoice = array_var($params, 'invoice');

	$ot_id = ObjectTypes::findByName('business_company')->getId();
	$business_member = $invoice->getMemberOfType($ot_id);

	if ($business_member instanceof Member) {

		// here get data from business company and override the print data

		$address = $business_member->getCustomPropertyValueByCode('invoicing_address');
		$phone = $business_member->getCustomPropertyValueByCode('invoicing_phone');
		$email = $business_member->getCustomPropertyValueByCode('invoicing_email');
		$website = $business_member->getCustomPropertyValueByCode('invoicing_website');
		$logo_repo_info = $business_member->getCustomPropertyValueByCode('invoicing_logo');

		// process the address to show
		$address1 = '';
		$address2 = '';
		$exploded = explode('|', $address);
		if (count($exploded) > 0) {
			$address_type = array_var($exploded, 0, '');
			$street = array_var($exploded, 1, '');
			$city = array_var($exploded, 2, '');
			$state = array_var($exploded, 3, '');
			$country = array_var($exploded, 4, '');
			$zip_code = array_var($exploded, 5, '');

			$address1 = $street;
			$address2 = $city;
			if ($state != '') $address2 .= ", " . $state;
			if ($zip_code != '') $address2 .= " " . $zip_code;
		}

		// set the address, email and all the data we want to override in the owner company section
		$array_pay_to_information['company_name'] = $business_member->getName();
		$array_pay_to_information['address'] = $address1;
		$array_pay_to_information['address_location'] = $address2;
		$array_pay_to_information['phone'] = $phone;
		$array_pay_to_information['email'] = $email;
		$array_pay_to_information['website'] = $website;

		// parse logo custom property and override the invoice logo if we have a value in the cp
		if ($logo_repo_info != '') {
			$repo_data = json_decode($logo_repo_info, true);
			
			$array_pay_to_information['invoice_logo'] = get_url('files', 'get_public_file', array('id' => $repo_data['repository_id']));
			$array_pay_to_information['invoice_logo_repo_id'] = $repo_data['repository_id'];
		}

	}
}


function multiple_bussiness_inv_check_if_invoice_uses_einvoice($params, &$use_e_invoice) {

	$invoice = array_var($params, 'invoice');

	$ot_id = ObjectTypes::findByName('business_company')->getId();
	$business_member = $invoice->getMemberOfType($ot_id);

	if ($business_member instanceof Member) {

		$business_use_einvoice = $business_member->getCustomPropertyValueByCode('uses_e_invoice');
		$business_einvoice_provider = $business_member->getCustomPropertyValueByCode('e_invoice_provider');
		if ($business_use_einvoice && $business_einvoice_provider != '') {
			$use_e_invoice = true;
		} else {
			$use_e_invoice = false;
		}
	}
}