<?php
/**
 * Currencies controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class CurrenciesController extends ApplicationController {

	var $plugin_name = "multiple_currencies";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
		Env::useHelper('format');
	}
	
	function list_all() {
		
		if (!can_manage_configuration(logged_user())) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$currencies = Currencies::instance()->findAll(array('order' => 'name ASC'));
		$currencies_info = array();
		
		foreach ($currencies as $c) {
			$currencies_info[] = $c->getArrayInfo();
		}
		
		tpl_assign('currencies_info', $currencies_info);
		
	}
	
	
	
	
	function add() {
		if (!can_manage_configuration(logged_user())) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$currency = new Currency();
		
		tpl_assign('object', $currency);
		
		$data = array_var($_POST, 'currency');
		if (is_array($data)) {
			try {
				DB::beginWork();
				
				$this->save_currency($data, $currency);
				
				DB::commit();
				
				flash_success(lang('success add currency', $currency->getName()));
				evt_add("reload current panel");
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				return;
			}
		}
	}
	
	function edit() {
		$this->setTemplate('add');
		
		if (!can_manage_configuration(logged_user())) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$currency = Currencies::instance()->findById(get_id());
		
		tpl_assign('object', $currency);
		
		$data = array_var($_POST, 'currency');
		if (is_array($data)) {
			try {
				DB::beginWork();
		
				$this->save_currency($data, $currency);
		
				DB::commit();
				
				flash_success(lang('success edit currency', $currency->getName()));
				evt_add("reload current panel");
		
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				return;
			}
		}
	}
	
	private function save_currency($data, $currency) {
		
		$currency->setFromAttributes($data);
		$currency->save();
		
		if ($currency->getIsDefault()) {
			// only one currency can be default
			DB::execute("UPDATE ".TABLE_PREFIX."currencies SET is_default=0 WHERE id<>".$currency->getId());
		}
		
	}
	
	
	function delete() {
		ajx_current("empty");
	
		if (!can_manage_configuration(logged_user())) {
			flash_error("no access permissions");
			return;
		}
		
		$currency = Currencies::instance()->findById(get_id());
		if (!$currency instanceof Currency) {
			flash_error('object dnx');
			return;
		}
		
		// @TODO: verificar que no hayan objetos relacionados a esta moneda, si hay entonces no se puede borrar
		
		try {
			DB::beginWork();
			
			// check if currency can be deleted
			$id = $currency->getId();
			$ts = Timeslots::instance()->findOne(array("conditions" => "rate_currency_id=$id"));
			if ($ts instanceof Timeslot) {
				throw new Exception(lang('cannot delete currency timeslot exists', $currency->getName()));
			}
			if (Plugins::instance()->isActivePlugin('income')) {
				$inv = IncomeInvoices::instance()->findOne(array("conditions" => "currency_id=$id"));
				if ($inv instanceof IncomeInvoice) {
					throw new Exception(lang('cannot delete currency invoice exists', $currency->getName()));
				}
			}
			if (Plugins::instance()->isActivePlugin('expenses')) {
				$p = Payments::instance()->findOne(array("conditions" => "currency=$id"));
				if ($p instanceof Payment) {
					throw new Exception(lang('cannot delete currency payment exists', $currency->getName()));
				} else {
					$ph = PaymentHistories::instance()->findOne(array("conditions" => "currency=$id"));
					if ($ph instanceof PaymentHistory) {
						throw new Exception(lang('cannot delete currency payment exists', $currency->getName()));
					}
				}
			}
			if (Plugins::instance()->isActivePlugin('advanced billing')) {
				$bd = BillingDefinitions::instance()->findOne(array("conditions" => "rate_currency_id=$id OR cost_currency_id=$id"));
				if ($bd instanceof BillingDefinition) {
					throw new Exception(lang('cannot delete currency billing definition exists', $currency->getName()));
				}
			}
			
			$name = $currency->getName();
			$currency->delete();
			
			DB::commit();
			
			flash_success(lang('success delete currency', $name));
			evt_add("reload current panel");
			
		} catch(Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			return;
		}
	}
	
	
}