<?php
/**
 * ExchangeRates controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class ExchangeRatesController extends ApplicationController {

	var $plugin_name = "multiple_currencies";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
		Env::useHelper('format');
	}
	
	function list_all() {
		
		if (!can_manage_configuration(logged_user())) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$exchange_rates = ExchangeRates::instance()->findAll(array('order' => 'date desc'));
		$exchange_rates_info = array();
		
		foreach ($exchange_rates as $c) {
			$exchange_rates_info[] = $c->getArrayInfo();
		}
		
		tpl_assign('exchange_rates', $exchange_rates_info);
		
	}
	
	
	
	
	function add() {
		if (!can_manage_configuration(logged_user())) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$exchange_rate = new ExchangeRate();
		if (isset($_REQUEST['from']) && is_numeric($_REQUEST['from'])) {
			$exchange_rate->setSourceCurrencyId($_REQUEST['from']);
		} else {
			$exchange_rate->setSourceCurrencyId(0);
		}
		if (isset($_REQUEST['to']) && is_numeric($_REQUEST['to'])) {
			$exchange_rate->setDestinyCurrencyId($_REQUEST['to']);
		} else {
			$exchange_rate->setDestinyCurrencyId(0);
		}
		if (isset($_REQUEST['date']) && is_numeric($_REQUEST['date'])) {
			$d = new DateTimeValue($_REQUEST['date']);
			$exchange_rate->setDate($d);
		}
		
		tpl_assign('object', $exchange_rate);
		
		$data = array_var($_POST, 'exchange_rate');
		if (is_array($data)) {
			try {
				DB::beginWork();
				
				$this->save_exchange_rate($data, $exchange_rate);
				
				DB::commit();
				
				flash_success(lang('success add exchange rate'));
				evt_add("reload current panel");
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				return;
			}
		}
	}
	
	function edit() {
		$this->setTemplate('add');
		
		if (!can_manage_configuration(logged_user())) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$exchange_rate = ExchangeRates::instance()->findById(get_id());
		if (!$exchange_rate instanceof ExchangeRate) {
			flash_error('object dnx');
			return;
		}
		
		tpl_assign('object', $exchange_rate);
		
		$data = array_var($_POST, 'exchange_rate');
		if (is_array($data)) {
			try {
				DB::beginWork();
		
				$this->save_exchange_rate($data, $exchange_rate);
		
				DB::commit();
				
				flash_success(lang('success edit exchange rate'));
				evt_add("reload current panel");
		
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				return;
			}
		}
	}
	
	private function save_exchange_rate($data, $exchange_rate) {
		
		$date = getDateValue(array_var($data, 'date'));
		$data['date'] = $date;
		
		$exchange_rate->setFromAttributes($data);
		$exchange_rate->save();
		
	}
	
	
	function delete() {
		ajx_current("empty");
	
		if (!can_manage_configuration(logged_user())) {
			flash_error("no access permissions");
			return;
		}
		
		$exchange_rate = ExchangeRates::instance()->findById(get_id());
		if (!$exchange_rate instanceof ExchangeRate) {
			flash_error('object dnx');
			return;
		}
		
		try {
			DB::beginWork();
			
			$exchange_rate->delete();
			
			DB::commit();
			
			flash_success(lang('success delete exchange rate'));
			evt_add("reload current panel");
			
		} catch(Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			return;
		}
	}
	
	
}