<?php
/**
 * NewsletterSending controller
 *
 * @version 1.0
 */
class NewsletterSendingController extends ApplicationController {

	var $plugin_name = "newsletters";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
	//	Env::useHelper('functions', $this->plugin_name);
	}
	
	
	function init_newsletter_sending() {
		ajx_current("empty");
		// @TODO: verificar permisos de envío
		
		$newsletter = Newsletters::instance()->findById(get_id());
		if (!$newsletter instanceof Newsletter) {
			flash_error('object dnx');
			return;
		}
		if (!$newsletter->canEdit(logged_user())) {
			flash_error('no access permissions');
			return;
		}

		$continue_sendig = true;
		Hook::fire('before_init_newsletter_sending',$newsletter,$continue_sendig);
		if (!$continue_sendig) {
			return;
		}

		try {
			DB::beginWork();
			// change status to 'sending' so the cron event can send the newsletter.
			$newsletter->setStatus('sending');
			$newsletter->save();
			
			DB::commit();

			flash_success(lang('newsletter successfully queued for sending'));
			ajx_current("reload");
			
		} catch (Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
		}
	}
	
	
	function pause_newsletter_sending() {
		ajx_current("empty");
		// @TODO: verificar permisos de envío
	
		$newsletter = Newsletters::instance()->findById(get_id());
		if (!$newsletter instanceof Newsletter) {
			flash_error('object dnx');
			return;
		}
		if (!$newsletter->canEdit(logged_user())) {
			flash_error('no access permissions');
			return;
		}
	
		try {
			DB::beginWork();
			// change status to 'sending' so the cron event can send the newsletter.
			$newsletter->setStatus('pending');
			$newsletter->save();
	
			DB::commit();
				
			flash_success(lang('newsletter sending successfully paused'));
			ajx_current("reload");
			
		} catch (Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
		}
	}
	
	
	function send_queued_newsletters() {
		
		try {
			$newsletters = Newsletters::instance()->findAll(array("conditions" => "status = 'sending'"));
			$emails_sent = 0;
			
			foreach ($newsletters as $newsletter) {
				
				$pending_records = NewsletterContactsPendings::getNextPendingRecipients($newsletter->getId(), $persons_num);
				if (is_array($pending_records) && count($pending_records) > 0) {
					$this->send_single_newsletter_emails($newsletter, $emails_sent);
				}
				
				// if newsletter has been paused => save log and stop sending mails
				$n = Newsletters::instance()->findById($newsletter->getId());
				if ($n instanceof Newsletter && $newsletter->getStatus() != 'sending') {
					$message = "Newsletter sending has been paused: ".$newsletter->getId();
					Logger::log($message);
					$this->log_newsletter_error($message);
				}
				
				// if there are no more recipients pending then set newsletter status to "finished"
				if ($n instanceof Newsletter) {
					$persons_num = config_option('newsletters_persons_x_email');
					if ($persons_num > 20) $persons_num = 20;
					
					$pending_contacts = NewsletterContactsPendings::getNextPendingRecipients($n->getId(), $persons_num);
					if (!is_array($pending_contacts) || count($pending_contacts) == 0) {
						$n->setStatus('finished');
						$n->save();
					}
				}
			}
		} catch (Exception $e) {
			$message = "Error sending newsletter ".$newsletter->getId()."\n".$e->getMessage()."\n".$e->getTraceAsString();
			Logger::log($message);
			$this->log_newsletter_error($message);
		}
		
		return $emails_sent;
	}
	
	
	
	private function send_single_newsletter_emails(Newsletter $newsletter, &$emails_sent) {
		
		$email_limit = config_option('newsletters_emails_x_step');
		
		while ($emails_sent < $email_limit) {
			
			$ok = $this->send_single_newsletter_single_email($newsletter);
			if ($ok) {
				$emails_sent++;
			} else {
				break;
			}
		}
		
	}
	
	
	private function send_single_newsletter_single_email(Newsletter $newsletter) {
		
		$persons_num = config_option('newsletters_persons_x_email');
		if ($persons_num > 20) $persons_num = 20;
		
		try {
			DB::beginWork();
			// set status to 'sending' to the recipients
			
			$contact_rows = DB::executeAll("SELECT x.contact_id FROM ".TABLE_PREFIX."newsletter_contacts_pending x 
						WHERE newsletter_id = ".$newsletter->getId()." AND status='pending'
						AND (SELECT o.trashed_by_id FROM ".TABLE_PREFIX."objects o WHERE o.id=x.contact_id) = 0
						LIMIT $persons_num");
			$contact_ids = array_filter(array_flat($contact_rows));
			
			if (count($contact_ids) > 0) {
				$sql = "UPDATE ".TABLE_PREFIX."newsletter_contacts_pending 
						SET status='sending' 
						WHERE newsletter_id = ".$newsletter->getId()."
						AND contact_id IN (".implode(',', $contact_ids).")
						AND (SELECT o.trashed_by_id FROM ".TABLE_PREFIX."objects o WHERE o.id=contact_id) = 0;";
				DB::execute($sql);
			}
			
			DB::commit();
		} catch (Exception $e) {
			DB::rollback();
			$message = "Error changing to sending status. Newsletter: ".$newsletter->getId()."\n". $e->getMessage() ."\n" . $e->getTraceAsString();
			Logger::log($message);
			$this->log_newsletter_error($message);
			return false;
		}
		
		// get contact emails
		$contact_emails = array();
        $contacts_ids_in_trash = array();
		if (count($contact_ids) > 0) {
			$contacts = Contacts::instance()->findAll(array("conditions" => "id IN (".implode(',', $contact_ids).")"));
			foreach ($contacts as $contact) {/* @var $contact Contact */
			    if($contact->isTrashed()){
                    $contacts_ids_in_trash[] = $contact->getId();
			        continue;
                }
				$contact_emails[] = array(
					'name' => $contact->getObjectName(),
					'email' => trim($contact->getEmailAddress()),
				);
			}
			
			if(count($contacts_ids_in_trash) > 0){
            $this->set_contact_status_pending_with_error($newsletter, $contacts_ids_in_trash);
            $contact_ids = array_diff($contact_ids,$contacts_ids_in_trash);
            }


			// ---------------------- SEND EMAIL -------------------------
			try {
				
				$this->do_send_newsletter_email($newsletter, $contact_emails);
				
			} catch (Exception $e) {
				
				Logger::log("Error sending newsletter email. Newsletter: ".$newsletter->getId()."\n". $e->getMessage() ."\n" . $e->getTraceAsString());
			
                $this->set_contact_status_pending_with_error($newsletter);
				
				return false;
			}
			// -----------------------------------------------------------
			
			try {
				DB::beginWork();
				
				// put recipients in the sent table after email is sent
				$sql_vals = array();
				foreach ($contact_ids as $contact_id) {
					$sql_vals[] = "('".$newsletter->getId()."', '$contact_id', NOW())";
				}
				if (count($sql_vals) > 0) {
					$sql = "INSERT INTO ".TABLE_PREFIX."newsletter_contacts_sent (newsletter_id, contact_id, sent_date)
							VALUES " . implode(',', $sql_vals) . " ON DUPLICATE KEY UPDATE newsletter_id=newsletter_id;";
					//Logger::log_r($sql);
					DB::execute($sql);
				}
				
				if (count($contact_ids) > 0) {
					// delete from pending table
					NewsletterContactsPendings::instance()->delete("newsletter_id=".$newsletter->getId()." 
						AND contact_id IN (".implode(',', $contact_ids).")");
				}
				
				DB::commit();
			} catch (Exception $e) {
				DB::rollback();
				Logger::log("Error sending newsletter email. Newsletter: ".$newsletter->getId()."\n". $e->getMessage() ."\n" . $e->getTraceAsString());
				return false;
			}
		} else {
			return false;
		}
		
		return true;
	}
	
	
	
	private function do_send_newsletter_email(Newsletter $newsletter, $contact_emails) {

		// email content
		$subject = $newsletter->getSubject();
		$body = $newsletter->getText();
		$content_type = $newsletter->getTypeContent() == 'html' ? 'text/html' : 'text/plain';
		
		// get account to send the email
		$ma = MailAccounts::instance()->findById($newsletter->getMailAccountId());
		if (!$ma instanceof MailAccount) {
			throw new Exception("Email account does not exist.");
		}

		// include swift mailer library
		Env::useLibrary('swift');
		
		// prepare swift mailer object
		$transport = $ma->getOutgoingTrasnportType();
		if (!in_array($transport, array('tls', 'ssl'))) {
			$transport = null;
		}
		
		$mail_transport = Swift_SmtpTransport::newInstance($ma->getSmtpServer(), $ma->getSmtpPort(), $transport);
		$smtp_authenticate = $ma->smtpUsername() != null;
		$mu = new MailUtilities();
		if ($smtp_authenticate) {
			$mail_transport->setUsername($ma->smtpUsername());
			$mail_transport->setPassword($mu->ENCRYPT_DECRYPT($ma->smtpPassword()));
		}

		$local_domain = parse_url(ROOT_URL);
		if(is_array($local_domain) && array_key_exists('host', $local_domain) && $local_domain['host'] !== ''){
			$mail_transport->setLocalDomain($local_domain['host']);
		}

		$mailer = Swift_Mailer::newInstance($mail_transport);
		
		// register swift logger plugin
		$swift_logger = new Swift_Plugins_Loggers_ArrayLogger();
		$mailer->registerPlugin(new Swift_Plugins_LoggerPlugin($swift_logger));

		// build "from" information
		$from_name = "";
		$account_owner = Contacts::instance()->findById($ma->getContactId());
		$user_settings = MailAccountContacts::getByAccountAndContact($ma, $account_owner);
		if ($user_settings instanceof MailAccountContact && $user_settings->getSenderName()) {
			$from_name = $user_settings->getSenderName();
		}
		if ($from_name == "") {
			$owner_company = owner_company();
			if ($owner_company instanceof Contact) {
				$from_name = owner_company()->getObjectName();
			} else {
				$from_name = $ma->getEmailAddress();
			}
		}
		$from = array($ma->getEmailAddress() => $from_name);
		
		// create swift message
		$message = Swift_Message::newInstance($subject)
			->setFrom($from)
			->setBody($body)
			->setContentType($content_type)
		;

		// set message id
		$message_id = MailUtilities::generateMessageId($ma->getEmailAddress());
		if ($message_id) {
			if (str_starts_with($message_id, "<")) $message_id = substr($message_id, 1, -1);
			$message->setId($message_id);
		}else{
			$message_id = $message->getId();
		}

		// add text/plain part to make the email multipart/alternative and prevent raise the spam score by "MIME_HTML_ONLY BODY: Message only has text/html MIME parts"
		$plain_body = html_to_text($body);
		$message->addPart($plain_body, 'text/plain');
		
		$persons_num = config_option('newsletters_persons_x_email');
		if ( $persons_num == 1 ) {

		    $recipient_count = 0;
		    $ce = $contact_emails[0];
		    if (array_var($ce, 'email') != '') {
		        // add email recipient in To
		        $message->addTo(array_var($ce, 'email'), array_var($ce, 'name'));
		        $recipient_count++;
		    }
		    
		}else{
		    
		    $message->addTo($ma->getEmailAddress(), lang('undisclosed recipients'));
		    
		    // add email recipients in BCC
		    $recipient_count = 0;
		    foreach ($contact_emails as $ce) {
		        if (array_var($ce, 'email') != '') {
		            $message->addBcc(array_var($ce, 'email'), array_var($ce, 'name'));
		            $recipient_count++;
		        }
		    }
		}
		
		// send the email
		if ($recipient_count > 0) {

			$time_start = microtime(true);
			
			try {
				
				// connect to mail server and send the message
				$result = $mailer->send($message);
				
			} catch (Exception $e) {
				
				// If error => log the error details
				$error_log = $e->getMessage()." (newsletter_id=".$newsletter->getId()." - msg_id=$message_id)\n".$e->getTraceAsString()."\nSwift log: ".$swift_logger->dump();
				$swift_logger->clear();
				
				$this->log_newsletter_error($error_log);
				
				// save mail with error in "newsletter_sent_emails" table
				self::save_sent_newsletter_email_log($newsletter, $subject, $body, $from, $contact_emails, $message_id, 'error', $error_log);
				
				// save log in server
				if (defined('EMAIL_ERRORS_LOGDIR') && file_exists(EMAIL_ERRORS_LOGDIR) && is_dir(EMAIL_ERRORS_LOGDIR)) {
					$err_msg = ROOT_URL."\nError sending newsletter (subject=$subject) using account ".$ma->getEmailAddress()."\n\nError detail:\n$error_log";
					file_put_contents(EMAIL_ERRORS_LOGDIR . "newsletter_err_".basename(ROOT), $err_msg, FILE_APPEND);
				}
				
				// try to stop the mailer
				try {
					$mailer->getTransport()->stop();
				} catch (Exception $e2) {
					$this->log_newsletter_error("ERROR when stopping the mailer in catch (newsletter_id=".$newsletter->getId()." - msg_id=$message_id)\n".$e2->getMessage()."\n".$e2->getTraceAsString());
				}
				
				// throw the exception to manage the contact status
				throw $e;
			}
			
			try {
				// close the connection with mail server
				$mailer->getTransport()->stop();
				
			} catch (Exception $e) {
				// log the error and continue, dont throw the exception
				$this->log_newsletter_error("ERROR when stopping the mailer (newsletter_id=".$newsletter->getId()." - msg_id=$message_id)\n".$e->getMessage()."\n".$e->getTraceAsString());
			}
			
			
			if ($result) {
				// save email history after sending
				self::save_sent_newsletter_email_log($newsletter, $subject, $body, $from, $contact_emails, $message_id);
				
			} else {
				
				$error_log = $swift_logger->dump();
				$this->log_newsletter_error($error_log);
				$swift_logger->clear();
				
				// save log in server
				if (defined('EMAIL_ERRORS_LOGDIR') && file_exists(EMAIL_ERRORS_LOGDIR) && is_dir(EMAIL_ERRORS_LOGDIR)) {
					$err_msg = ROOT_URL."\nError sending newsletter 2 (subject=$subject) using account ".$ma->getEmailAddress()."\n\nError detail:\n$error_log";
					file_put_contents(EMAIL_ERRORS_LOGDIR . "newsletter_err_".basename(ROOT), $err_msg, FILE_APPEND);
				}
				
				throw new Exception("Error sending newsletter email.\n$error_log");
			}
		}
		
	}
	
	private function log_newsletter_error($error_message) {
		file_put_contents(CACHE_DIR."/newsletter_errors.txt", "\n" . gmdate("Y-m-d H:i:s") . "\n" 
			. $error_message . "\n----------------------------------------------------------------------------", FILE_APPEND);
	}
	
	private function save_sent_newsletter_email_log(Newsletter $newsletter, $subject, $body, $from, $contact_emails, $message_id, $status='sent', $additional_data='') {
		
		$from_str = "";
		$bcc_str = "";
		
		foreach ($from as $address => $name) {
			$from_str = $name=='' ? $address : "$name <$address>";
			break;
		}
		
		$persons_num = config_option('newsletters_persons_x_email');
		if ( $persons_num == 1 ) {
		    $ce = $contact_emails[0];
		    $to_str = array_var($ce, 'name'). " <".array_var($ce, 'email').">";
		}else{
		    // fill the To header with "Undisclosed recipìents" and the same email address as the From header.
		    $to_str = lang('undisclosed recipients')." <$address>";
		    
		    // put all recipients in bcc
		    foreach ($contact_emails as $ce) {
		        if (array_var($ce, 'email') != '') {
		            $address_str = array_var($ce, 'name') == '' ? array_var($ce, 'email') : array_var($ce, 'name')." <".array_var($ce, 'email').">";
		            $bcc_str .= ($bcc_str=='' ? '' : ';' ) . $address_str;
		        }
		    }
		}
		
		$email_info = array(
			'to' => $to_str, 'cc' => '', 'bcc' => $bcc_str, 'from' => $from_str, 'subject' => $subject, 'body' => $body, 'attachments' => '',  'message_id' => $message_id
		);
		
		$values = array(
			$newsletter->getId(),
			DB::escape(DateTimeValueLib::now()->toMySQL()),
			DB::escape($email_info['to']),
			DB::escape($email_info['cc']),
			DB::escape($email_info['bcc']),
			DB::escape($email_info['from']),
			DB::escape($email_info['subject']),
			DB::escape($email_info['body']),
			DB::escape($email_info['attachments']),
			DB::escape($email_info['message_id']),
			DB::escape($status),
			DB::escape($additional_data),
		);

		// columns to set
		$columns_sql = "`newsletter_id`, `sent_date`, `to`, `cc`, `bcc`, `from`, `subject`, `body`, `attachments`, `message_id`, `status`, `additional_data`";
		
		// sql query
		$sql = "INSERT INTO ".TABLE_PREFIX."newsletter_sent_emails ($columns_sql)
				VALUES (". implode(",", $values) .")
				ON DUPLICATE KEY UPDATE id=id";
		
		try {
			DB::execute($sql);
		} catch (Exception $e) {
			$message = "Error when saving sent newsletter email log: $sql";
			Logger::log($message);
			$this->log_newsletter_error($message);
		}
	}
	
    //set contact status as 'pending_with_error' to verify later
    private function set_contact_status_pending_with_error($newsletter, $contact_ids = null){
        try {
            DB::beginWork();

            if(!is_array($contact_ids) || count($contact_ids) == 0){
                $sending_records = NewsletterContactsPendings::getCurrentSendingRecipients($newsletter->getId());
            }else{
                $sending_records = NewsletterContactsPendings::instance()->findAll("newsletter_id=".$newsletter->getId()." 
						AND contact_id IN (".implode(',', $contact_ids).")");
            }

            foreach ($sending_records as $r) {
                $r->setStatus('pending_with_error');
                $r->save();
            }

            DB::commit();
        } catch (Exception $e) {
            DB::rollback();
            $message = "Error making contact status rollback. Newsletter: ".$newsletter->getId()."\n". $e->getMessage() ."\n" . $e->getTraceAsString();
            Logger::log($message);
            $this->log_newsletter_error($message);
        }
    }

	
}