<?php
require_javascript("og/ObjectList.js");
require_javascript('og/ObjectSelector.js');

$genid = gen_id();
if (isset($newsletter) && $newsletter instanceof Newsletter) {
	if (!$newsletter->isTrashed()){
		if($newsletter->canEdit(logged_user())) {
			add_page_action(lang('edit'), "javascript:og.render_modal_form('', {c:'newsletter', a:'edit', params: {id:".$newsletter->getId()."}});", 'ico-edit', null, null, true);
			
			if (!$newsletter->isArchived()) {
				add_page_action(lang('archive'), "javascript:if(confirm(lang('confirm archive object'))) og.openLink('" . $newsletter->getArchiveUrl() ."');", 'ico-archive-obj');
			} else {
				add_page_action(lang('unarchive'), "javascript:if(confirm(lang('confirm unarchive object'))) og.openLink('" . $newsletter->getUnarchiveUrl() ."');", 'ico-unarchive-obj');
			}
		}
	}
	if ($newsletter->canDelete(logged_user())) {
		if ($newsletter->isTrashed()) {
			add_page_action(lang('restore from trash'), "javascript:if(confirm(lang('confirm restore objects'))) og.openLink('" . $newsletter->getUntrashUrl() ."');", 'ico-restore', null, null, true);
			add_page_action(lang('delete permanently'), "javascript:if(confirm(lang('confirm delete permanently'))) og.openLink('" . $newsletter->getDeletePermanentlyUrl() ."');", 'ico-delete', null, null, true);
		} else {
			add_page_action(lang('move to trash'), "javascript:if(confirm(lang('confirm move to trash'))) og.openLink('" . $newsletter->getTrashUrl() ."');", 'ico-trash', null, null, true);
		}
	}


	if ($newsletter->getStatus() == 'pending') {
		add_page_action(lang('send'), "javascript:if(confirm(lang('confirm send newsletter'))) og.openLink('" .get_url('newsletter_sending', 'init_newsletter_sending', array('id' => $newsletter->getId())) ."',{callback:function(){og.customDashboard('contact','init',{},true)}});", 'ico-email',null, null, true);
	}
	if ($newsletter->getStatus() == 'sending') {
		add_page_action(lang('pause sending'), "javascript:if(confirm(lang('confirm pause sending'))) og.openLink('" .get_url('newsletter_sending', 'pause_newsletter_sending', array('id' => $newsletter->getId())) ."',{callback:function(){og.customDashboard('contact','init',{},true)}});", 'ico-time-pause',null, null, true);
	}
	
	add_page_action(lang('recipients'), "javascript:og.selectContacts(this);", 'ico-users', null, null, true);
?>

<div style="padding:7px">
<div class="newsletter">
<?php
	$content = '';
	
	if($newsletter->getTypeContent() == "text"){
		$content = escape_html_whitespace(convert_to_links(clean($newsletter->getText())));
	}else{
		$file_url = ROOT_URL."/tmp/".gen_id().".html";
		$path = str_replace(ROOT_URL, ROOT, $file_url);
		file_put_contents($path, $newsletter->getText());
		$content .= '<iframe id="'.$genid.'ifr" name="'.$genid.'ifr" style="width:100%;height:100%" frameborder="0" src="'.$file_url.'"
							onload="javascipt:iframe=document.getElementById(\''.$genid.'ifr\'); iframe.parentNode.style.height = Math.min(600, iframe.contentWindow.document.body.scrollHeight + 60) + \'px\' ;">
						</iframe>';
	}

	//Start Date
    if($newsletter->getStartDate() instanceof DateTimeValue) { 
    	if ($newsletter->getStartDate()->isToday()){
			$datetime = format_time($newsletter->getStartDate());					
		} else {
			$tz_hours_offset = logged_user()->getUserTimezoneHoursOffset();
			$datetime =  format_date($newsletter->getStartDate(), null, $tz_hours_offset).' <b>'.lang('at').'</b> '.format_time($newsletter->getStartDate(), null, $tz_hours_offset);
		}	
		$content .= '<div class="member-path-dim-block startDate"><span class="bold">'.lang("start date").': </span>'.$datetime.'</div>';
	}		
    
	$rows = DB::executeAll("SELECT contact_id FROM ".TABLE_PREFIX."newsletter_contacts_pending where newsletter_id=".$newsletter->getId());
	$all_newsletter_contact_ids = array_flat($rows);
	
	$rows = DB::executeAll("SELECT contact_id FROM ".TABLE_PREFIX."newsletter_contacts_sent where newsletter_id=".$newsletter->getId());
	$sent_ids = array_flat($rows);
	$all_newsletter_contact_ids = array_merge($all_newsletter_contact_ids, $sent_ids);
	
	// initialize object selector with the selected people of this newsletter.
	Env::useHelper('object_selector');
	save_tmp_objects_ids($all_newsletter_contact_ids, true, $genid);
	
	$status_cls = "newsletter-status ".$newsletter->getStatus();
	$status_str = lang("status").": <span class='$status_cls'>".lang($newsletter->getStatus())."</span>";
	
	tpl_assign("content", $content);
	tpl_assign("object", $newsletter);
	tpl_assign('iconclass', 'ico-large-newsletter');
	tpl_assign('description', $status_str);

	$extra_templates_to_include = array();
	$extra_templates_to_include[] = array('path' => get_template_path('contact_list', 'newsletter', 'newsletters'));
	tpl_assign('extra_templates_to_include', $extra_templates_to_include);
	
	$this->includeTemplate(get_template_path('view', 'co'));
?>

</div>
</div>
<script>
	var genid = '<?php echo $genid?>';
	var newsletter_id = '<?php echo $newsletter->getId()?>';
    var newsletter_object_type_id = '<?php echo Contacts::instance()->getObjectTypeId()?>';
	
	og.selectContacts = function(before) {

		og.newsletters.addContactsToNewsletter(before, newsletter_id,newsletter_object_type_id, genid);

	};



</script>
<?php } ?>
