<?php

/**
 * NotificationsManager controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class NotificationsManagerController extends ApplicationController {

	var $plugin_name = "notifications_manager";
	
	/**
	 * Construct the NotificationsManagerController
	 *
	 * @access public
	 * @param void
	 * @return NotificationsManagerController
	 */
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
		// Access permissions
		if(!(logged_user()->isExecutiveGroup())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
		}
		
		Env::useHelper('functions', $this->plugin_name);
		
	} // __construct
	
	// ---------------------------------------------------
	//  Index
	// ---------------------------------------------------
	
	
	function test_preview_task_notification() {
		
		$task = ProjectTasks::instance()->findById(get_id());
		if (!$task instanceof ProjectTask) {
			flash_error(lang('object dnx'));
			ajx_current("empty");
			return;
		}
		
		$subscriber_emails = array();
		foreach ($task->getSubscribers() as $subscriber) {
			/* @var $subscriber Contact */
			$subscriber_emails[] = $subscriber->getEmailAddress();
		}
		$subscriber_emails = array_filter($subscriber_emails);
		
		$params_to_email = array(
			'template_file_name' => 'task_notification',
			'to' => $subscriber_emails
		);
		
		$send_obj_email_controller = new SendObjectEmailController();
		$send_obj_email_controller->send_object_by_email($task->getId(), $params_to_email, false, false);
	}

	
	function administration() {
		if(!can_manage_configuration(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		}
	
		$administration_links = array();
		
		$administration_links[] = array(
				'url' => get_url('notifications_manager', 'notification_trigger_options'),
				'name' => lang("notification triggers"),
				'desc' => lang('notification triggers description'),
		);
		
		$administration_links[] = array(
				'url' => get_url('mail_template', 'index', array('only_notification_templates' => 1)),
				'name' => lang("email templates"),
				'desc' => lang('email templates description'),
		);
		
		$administration_links[] = array(
				'url' => get_url('notifications_manager', 'email_system_setup'),
				'name' => lang("email system setup"),
				'desc' => lang('email system setup description'),
		);
		
		
		tpl_assign('administration_links', $administration_links);
	}
	
	
	function notification_trigger_options() {
		if (!can_manage_configuration(logged_user())) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$options = ContactConfigOptions::instance()->findAll(array('conditions' => "category_name='notification_trigger_options'"));
		tpl_assign('options', $options);
	}
	
	
	function save_notification_trigger_options() {
		ajx_current("empty");
		
		$config_controller = new ConfigController();
		
		$config_category = ContactConfigCategories::instance()->findOne(array('conditions' => "name='notification_trigger_options'"));
		if ($config_category instanceof ContactConfigCategory) {
			$_GET['id'] = $config_category->getId();
			$config_controller->update_default_user_preferences();
		} else {
			flash_error('object dnx');
			ajx_current("empty");
		}
	}
	
	
	function email_system_setup() {
		if (!can_manage_configuration(logged_user())) {
			ajx_current("empty");
			flash_error("no access permissions");
			return;
		}
		
		$options = ContactConfigOptions::instance()->findAll(array('conditions' => "category_name='email_system_setup'"));
		tpl_assign('options', $options);
	}
	
	
	function save_email_system_setup() {
		ajx_current("empty");
		
		$config_controller = new ConfigController();
		
		$config_category = ContactConfigCategories::instance()->findOne(array('conditions' => "name='email_system_setup'"));
		if ($config_category instanceof ContactConfigCategory) {
			$_GET['id'] = $config_category->getId();
			$config_controller->update_default_user_preferences();
		} else {
			flash_error('object dnx');
			ajx_current("empty");
		}
	}
	
	
	
} // NotificationsManagerController

?>