<?php


function send_notification_summaries() {

	_log("Sending notification summaries...");
	try {
	
	$all_summaries = NotificationSummaries::instance()->findAll();
	
	$now = DateTimeValueLib::now();
	
	foreach ($all_summaries as $summary) {
		/* @var $summary NotificationSummary */
		
		if ($summary->getNextExecutionDate()->getTimestamp() <= $now->getTimestamp()) {
			
			$date_from = DateTimeValueLib::now();
			$date_from->add('m', -1 * $summary->getDelay());
			
			$logs = ApplicationLogs::instance()->findAll(array(
					"conditions" => array("created_on >= ? AND `action` NOT IN ('login','logout','link','unlink')", $date_from),
					"order" => "rel_object_id, created_on"
			));

			// sort the logs by date
			$sorted_logs = array();
			foreach ($logs as $l) $sorted_logs[$l->getCreatedOn()->getTimestamp()] = $l;
			ksort($sorted_logs);
			$logs = $sorted_logs;

			$application_logs_in_notifications = array();
			
			$people = array();
			
			$people_logs = array();
			
			foreach ($logs as $log) {
				/* @var $log ApplicationLog */
				
				$object = Objects::findObject($log->getRelObjectId());
				$subscribers = array();
				
				if ($object instanceof ContentDataObject) {
					
					$ot = ObjectTypes::instance()->findById($object->getObjectTypeId());
					if (!$ot || ($ot->getName() == 'mail' && $log->getAction()==ApplicationLogs::ACTION_ADD)) continue;
					
					$subscribers = $object->getSubscribers();
					foreach($subscribers as $subscriber) {
						if (!isset($people[$subscriber->getId()])) {
							$people[$subscriber->getId()] = $subscriber;
						}
						
						if (!isset($people_logs[$subscriber->getId()])) {
							$people_logs[$subscriber->getId()] = array(
									'logs' => array($log),
									'user' => $subscriber,
							);
						} else {
							$people_logs[$subscriber->getId()]['logs'][] = $log;
						}
					}
				}
			}
			
			foreach ($people_logs as $user_log) {
				
				$user_logs = $user_log['logs'];
				$user = $user_log['user'];
			
				foreach ($user_logs as $log) {
					/* @var $log ApplicationLog */
					$object = Objects::findObject($log->getRelObjectId());
					if ($object instanceof ContentDataObject) {
						$subscribers = $object->getSubscribers();
					} else {
						$subscribers = array();
					}
					
					switch ($log->getAction()) {
						case 'add':
							if ($object instanceof Timeslot) {
								$option_val = user_config_option('timeslot_logged', null, $user->getId());
							}
							if ($object instanceof Comment) {
								$option_val = user_config_option('comment_added', null, $user->getId());
							}
							// Get subscribers from the object where timeslot was added
							if ($object instanceof Timeslot) {
								$rel_obj = $object->getRelObject();
								if ($rel_obj instanceof ContentDataObject) {
									$subscribers = $rel_obj->getSubscribers();
								} else {
									$subscribers = array();
								}
							} else {
								if ($object instanceof ContentDataObject) {
									$subscribers = $object->getSubscribers();
								}
							}
							foreach($subscribers as $subscriber) {
								if (!isset($people[$subscriber->getId()])) {
									$people[$subscriber->getId()] = $subscriber;
								}
							}		
							break;
							
						case 'comment':
							if ($object instanceof Comment) {
								$option_val = user_config_option('comment_added', null, $user->getId());
							}
							// Get subscribers from the object where comment was added
							
							if ($object instanceof Comment) {
								$subscribers = $object->getRelObject()->getSubscribers();
							} else {
								if ($object instanceof ContentDataObject) {
									$subscribers = $object->getSubscribers();
								}
							}
							foreach($subscribers as $subscriber) {
								if (!isset($people[$subscriber->getId()])) {
									$people[$subscriber->getId()] = $subscriber;
								}
							}
							break;
							
						case 'edit':
							// Get values from 'property' column from application_log_details table using log id
							$summaryId = $summary->getId();
							$logId = $log->getId();
							$logDetails = ApplicationLogDetails::instance()->findAll(array("conditions" => "application_log_id=" . $logId));
							$detailProperties = array();
							foreach($logDetails as $logDetail){
								$detailProperties[] = $logDetail->getProperty();
							}
							
							$notifiableProperties = array();
							// Get config options
							$descriptionConfig = explode(',',user_config_option('description_changed', null, $user->getId()));
							$assignedConfig = explode(',',user_config_option('user_assigned_to_task', null, $user->getId()));
							$startDueDateConfig = explode(',',user_config_option('start_or_due_date_modified', null, $user->getId()));
							$classificationConfig = explode(',',user_config_option('classification_changed', null, $user->getId()));
							$nameChangedConfig = explode(',',user_config_option('name_changed', null, $user->getId()));
							$timeEstimateConfig = explode(',',user_config_option('time_estimate_changed', null, $user->getId()));
							$percentCompletedConfig = explode(',',user_config_option('percent_completed_changed', null, $user->getId()));
							$priorityChangedConfig = explode(',',user_config_option('priority_changed', null, $user->getId()));
							
							// Check if config options has $summaryId => insert log detail in $notifiableProperties
							if(in_array($summaryId, $descriptionConfig)){
								$notifiableProperties[] = 'text';
							}
							if(in_array($summaryId, $assignedConfig)){
								$notifiableProperties[] = 'assigned_to_contact_id';
							}
							if(in_array($summaryId, $startDueDateConfig)){
								$notifiableProperties[] = 'start_date';
								$notifiableProperties[] = 'due_date';
							}
							if(in_array($summaryId, $classificationConfig)){
								$notifiableProperties[] = 'classification';
							}
							if(in_array($summaryId, $nameChangedConfig)){
								$notifiableProperties[] = 'name';
							}
							if(in_array($summaryId, $timeEstimateConfig)){
								$notifiableProperties[] = 'time_estimate';
							}
							if(in_array($summaryId, $percentCompletedConfig)){
								$notifiableProperties[] = 'percent_completed';
							}
							if(in_array($summaryId, $priorityChangedConfig)){
								$notifiableProperties[] = 'priority';
							}
							
							// If log property is found in $notifiableProperty => notify about this log
							foreach($detailProperties as $property){
								if(in_array($property, $notifiableProperties)){
									$option_val = $summaryId;
									break;
								}
							}
							break;
							
						case 'delete':
						case 'trash':
							$option_val = '';
							break;
							
						case 'subscribe':
							$option_val = user_config_option('user_subscribed', null, $user->getId());
							// Get subscribers who had been subscribed or unsubscribed from the task
							$subscribersId = $log->getLogData();
							if(!is_array($subscribersId)){
								$subscribersId = explode(',', $subscribersId);
							}
							$logCreatedById = $log->getCreatedById();
							// Check if affected contact is not in the $people list and if log wasn't created by the person who suppose to receive notification
							$subscribers = array();
							foreach($subscribersId as $subId) {
								if (!is_numeric($subId) || $subId <= 0) continue;
								if ($logCreatedById != $subId) {
									$subscribers[] = Contacts::instance()->findOne(array("conditions" =>  array("object_id=?" , $subId)));
								}
							}
							foreach($subscribers as $subscriber) {
								if (!isset($people[$subscriber->getId()])) {
									$people[$subscriber->getId()] = $subscriber;
								}
							}
							break;
							
						case 'unsubscribe':
							$option_val = user_config_option('user_subscribed', null, $user->getId());
							// Get subscribers who had been subscribed or unsubscribed from the task
							$subscribersId = $log->getLogData();
							if(!is_array($subscribersId)){
								$subscribersId = explode(',', $subscribersId);
							}
							$logCreatedById = $log->getCreatedById();
							// Check if affected contact is not in the $people list and if log wasn't created by the person who suppose to receive notification
							$subscribers = array();
							foreach($subscribersId as $subId) {
								if (!is_numeric($subId) || $subId <= 0) continue;
								if ($logCreatedById != $subId) {
									$subscribers[] = Contacts::instance()->findOne(array("conditions" =>  array("object_id=?" , $subId)));
								}
							}
							foreach($subscribers as $subscriber) {
								if (!isset($people[$subscriber->getId()])) {
									$people[$subscriber->getId()] = $subscriber;
								}
							}
							break;
							
						case 'upload':
							break;
							
						case 'open':
						case 'close':
							if ($object instanceof ProjectTask) {
								$option_val = user_config_option('task_started_or_completed', null, $user->getId());
							}
							break;
							
						default:
								
					}
	
					if (!isset($option_val)) {
						$option_val = array();
					}
					if (!is_array($option_val)) {
						$option_val = explode(',', $option_val);
					}
					$include_this_log = true;
					if (!in_array($summary->getId(), $option_val)) {
						$include_this_log = false;
					} // possible bug
					
					if ($include_this_log && $user->getId() != $log->getCreatedById()) {
						$application_logs_in_notifications[$user->getId()][] = $log;
					}
				}
			}
			
			if (count($application_logs_in_notifications) > 0) {
				foreach ($people as $user) {
					$user_application_logs = array_var($application_logs_in_notifications, $user->getId());
					if (!is_array($user_application_logs)) continue;
					$template_variables = array(
							'email_type' => $summary->getId(),
							'logs' => $user_application_logs,
							'summary' => $summary,
							'include_log_object_name' => true,
					);
				
					$mail_template_params = array(
							'triggering_object' => null,
							'template_code' => 'notification_summary',
							'template_parameters' => $template_variables,
					);
					$mail_template_data = null;
					Hook::fire("get_mail_template_data", $mail_template_params, $mail_template_data);
				
					if ($mail_template_data) {
						$sendername = "Feng Office";
						if (isset($mail_template_data['from_name'])) {
							$sendername = $mail_template_data['from_name'];
						}
						
						// check if have to use forced smtp address and from name
						if (config_option("mail_transport") == Notifier::MAIL_TRANSPORT_SMTP) {
							$smtp_address = config_option("smtp_address");
							if (trim($smtp_address) != "") {
								$senderemail = $smtp_address;
							}
								
							$from_name = config_option("notification_from_name");
							if (trim($from_name) != "") {
								$sendername = $from_name;
							}
						}
						
						if (!$senderemail) {
							$senderemail = trim(owner_company()->getEmailAddress());
							if (!$senderemail) {
								$first_admin = Contacts::instance()->findOne(array("conditions" => "user_type > 0", "order" => "user_type, object_id"));
								$senderemail = trim($first_admin->getEmailAddress());
							}
						}
						$from = "$sendername <$senderemail>";
					
						$to_addresses = array_var($mail_template_data, 'to', array());
						
						$email_address = trim($user->getEmailAddress());
						if ($email_address != '') {
							$to_addresses[$user->getId()] = Notifier::prepareEmailAddress($email_address, $user->getObjectName());
						}
						
						
						$subject = isset($mail_template_data['subject']) ? $mail_template_data['subject'] : "";
						if (!$subject) {
							$subject = $summary->getName();
						}
						
						if (count($to_addresses) > 0) {
							Notifier::queueEmail(0,
								$to_addresses, array_var($mail_template_data, 'cc'), array_var($mail_template_data, 'bcc'),
								$from, $subject, $mail_template_data['body']);
						}
					}
				}
			}
			
			
			try {
				DB::beginWork();
				$nextdate = DateTimeValueLib::now()->add("m", $summary->getDelay());
				$summary->setNextExecutionDate($nextdate);
				$summary->save();
				DB::commit();
			} catch (Exception $e) {
				_log("Error saving next execution date for summary ".$summary->getId().": ". $e->getMessage());
				DB::rollback();
			}
		}
	}
	
	} catch (Exception $ex) {
		_log("Error generating notification summaries ".$$ex->getMessage()."\n".$ex->getTraceAsString());
	}
	_log("Finished sending notification summaries");
}