<?php
Hook::register('notifications_manager');


function notifications_manager_put_email_in_queue($email_data, &$queue_this_email) {
	// dont queue or send email if this plugin is activated.
	$queue_this_email = array('dummy' => true);
	$object = Objects::findObject(array_var($email_data, 'object_id'));
	if ($object instanceof ContentDataObject) {
		$last_log = ApplicationLogs::instance()->findOne(array(
				"conditions" => "rel_object_id=" . $object->getId() . " AND is_silent=0", 
				"order" => "created_on DESC",
		));
		if ($last_log instanceof ApplicationLog) {
			$queue_this_email['log_action'] = $last_log->getAction();
		}
	}
}

function notifications_manager_additional_cron_function_files($ignored, &$files) {
	$files[] = ROOT . "/plugins/notifications_manager/application/cron_functions.php";
}


function notifications_manager_view_task_add_actions($params, &$null) {
	$task = array_var($params, 'task');

	if ($task instanceof ProjectTask){
		// Send file by email option
		add_page_action(lang('preview notification email'), "javascript:og.openLink(og.getUrl('notifications_manager', 'test_preview_task_notification', {id:".$task->getId()."}));", 'ico-email', null, null, true);
	}
}


function notifications_manager_object_definition($type, &$ret) {
	if ($type == 'MailTemplate') {
		$ret['is_notification_template'] = DATA_TYPE_BOOLEAN;
	}
	if ($type == 'QueuedEmail') {
		$ret['log_action'] = DATA_TYPE_STRING;
	}
}

function notifications_manager_object_edit_categories($object, &$categories) {
	if ($object instanceof MailTemplate) {
		if ($object->isNew()) {
			$val = array_var($_REQUEST, 'is_notification_template', 0);
		} else {
			$val = $object->getColumnValue('is_notification_template');
		}
		$categories[] = array(
			'id' => 'is_notification_template_tab',
			'name' => 'is_notification_template_tab',
			'required' => false,
			'visible' => false,
			'hidden' => true,
			'content' => '<input type="hidden" name="mail_template[is_notification_template]" value="'.$val.'" />',
		);
	}
}

function notifications_manager_new_mail_template_btn_params($ignored, &$params) {
	if (array_var($_REQUEST, 'only_notification_templates')) {
		$params['is_notification_template'] = 1;
	}
}

function notifications_manager_render_administration_icons($ignored, &$links) {
	if (can_manage_dimensions(logged_user())) {
		$links[] = array(
				'ico' => 'ico-large-notifications',
				'url' => get_url('notifications_manager', 'administration'),
				'name' => lang('notifications and alerts'),
				'extra' => '',
		);
	}
}

function notifications_manager_email_templates_list_extra_conditions($params, &$extra_conditions) {
	
	$request = array_var($params, 'request');
	if (array_var($request, 'only_notification_templates')) {
		$extra_conditions .= " AND is_notification_template=1";
	} else {
		$extra_conditions .= " AND is_notification_template=0";
	}
	
}

function notifications_manager_additional_mail_templates_grid_params($params, &$grid_params) {

	$request = array_var($params, 'request');
	if (array_var($request, 'only_notification_templates')) {
		$grid_params['only_notification_templates'] = 1;
	}

}


function notifications_manager_override_object_notification($params, &$continue_with_original_process) {
	
	$continue_with_original_process = false;

	$notification_type = array_var($params, 'notification');
	$object = array_var($params, 'object');
	$people = array_var($params, 'people');
	$sendername = array_var($params, 'sendername');
	$senderemail = array_var($params, 'senderemail');
	$senderid = array_var($params, 'senderid');
	
	if (!logged_user() instanceof Contact) {
		$logged_user = Contacts::instance()->findById($senderid);
		if (!$logged_user instanceof Contact) {
			$logged_user = array_var($people, 0);
		}
		if (!$logged_user instanceof Contact) {
			$logged_user = Contacts::instance()->findOne(array("conditions" => "user_type>0", "order" => "user_type, object_id"));
		}
		if ($logged_user instanceof Contact) {
			CompanyWebsite::instance()->logUserIn($logged_user);
		}
	}

	$content_object_id = $object->getId();
	if ($object instanceof Comment) {
		$content_object_id = $object->getRelObjectId();
	}
	
	// get all logs in the last 6 minutes
	$since_date = DateTimeValueLib::now();
	$consolidation_minutes = user_config_option('minutes_for_consolidation');
	$since_date->add('m', -1 * $consolidation_minutes);
	$last_application_logs = ApplicationLogs::instance()->findAll(array(
		'conditions' => array('rel_object_id = ? AND created_on > ? AND is_silent=0', $content_object_id, $since_date),
		'order' => 'created_on',
	));
	if (!is_array($last_application_logs)) $last_application_logs = array();

	// enqueue this email to be sent in $consolidation_minutes minutes
	$mail_ts = DateTimeValueLib::now();
	$mail_ts->add('m', $consolidation_minutes);
	
	$all_comments = Comments::getCommentsByObjectIds($content_object_id);
	if (count($all_comments) > 0) {
		$comment_ids = array();
		foreach ($all_comments as $c) $comment_ids[] = $c->getId();
		$comment_ids_str = implode(',', $comment_ids);
		
		$last_application_comment_logs = ApplicationLogs::instance()->findAll(array(
			'conditions' => array('rel_object_id IN ('.$comment_ids_str.') AND created_on > ? AND is_silent=0', $since_date),
			'order' => 'created_on',
		));
		
		foreach ($last_application_comment_logs as $l) $last_application_logs[] = $l;
	}
	
	// sort the logs by date
	$sorted_logs = array();
	foreach ($last_application_logs as $l) $sorted_logs[$l->getCreatedOn()->getTimestamp()] = $l;
	ksort($sorted_logs);
	$last_application_logs = $sorted_logs;

	$object_type = ObjectTypes::instance()->findById($object->getObjectTypeId());
	
	$member = null;
	if ($object_type->getType() == 'dimension_object') {
		$member = Members::findOneByObjectId($object->getId());
	}
	
	$template_code = $object_type->getName() . "_notification";
	
	$use_template_subject = true;
	
	
	$application_logs_in_notifications = array();

	
	if($notification_type == 'due_date reminder'){
		if (MailTemplates::getByCode('due_date reminder') instanceof MailTemplate) {
			$template_code = 'due_date reminder';
		}
		$subscribers = $object->getSubscribers();
		$people = array();
		$last_log = isset($object->last_application_log) ? $object->last_application_log : null;
		foreach ($subscribers as $subscriber) {
			if (!isset($people[$subscriber->getId()])) {
				$people[$subscriber->getId()] = $subscriber;
			}
			if (!isset($application_logs_in_notifications[$subscriber->getId()])) {
				$application_logs_in_notifications[$subscriber->getId()] = array();
			}
			$application_logs_in_notifications[$subscriber->getId()][] = $last_log;
		}
		

	} else{
		$people = array();
		foreach ($last_application_logs as $log) {
			/* @var $log ApplicationLog */
			
			$object = Objects::findObject($log->getRelObjectId());
			$subscribers = array();
			$subscribers = $object->getSubscribers();
			$template_code = $object->getObjectTypeName() . '_notification';
			if ($object instanceof ContentDataObject) {
				
				$ot = ObjectTypes::instance()->findById($object->getObjectTypeId());
				if (!$ot || ($ot->getName() == 'mail' && $log->getAction()==ApplicationLogs::ACTION_ADD)) continue;
				
				$subscribers = $object->getSubscribers();
				foreach($subscribers as $subscriber) {
					if (!isset($people[$subscriber->getId()])) {
						$people[$subscriber->getId()] = $subscriber;
					}
				}
			}
			
			switch ($log->getAction()) {
				case 'add':
					if ($object instanceof Timeslot) {
						$option_val = user_config_option('timeslot_logged');
						$subscribers = $object->getRelObject()->getSubscribers();
					}
					if ($object instanceof Comment) {
						$option_val = user_config_option('comment_added');
						$subscribers = $object->getRelObject()->getSubscribers();
					}

					if ($object instanceof ProjectTask && $template_code == 'task_notification'){
						$option_val = 'Immediate';
					}

					// Get subscribers from the object where timeslot was added
					foreach($subscribers as $subscriber) {
						if (!isset($people[$subscriber->getId()])) {
							$people[$subscriber->getId()] = $subscriber;
						}
					}
					if (MailTemplates::getByCode($template_code.'_new') instanceof MailTemplate) {
						$template_code = $template_code.'_new';
					}	
					break;
				case 'comment':
					if ($object instanceof Comment) {
						$option_val = user_config_option('comment_added');
					}
					// Get subscribers from the object where comment was added
					$subscribers = $object->getRelObject()->getSubscribers();
					$object = $object->getRelObject();
					foreach($subscribers as $subscriber) {
						if (!isset($people[$subscriber->getId()])) {
							$people[$subscriber->getId()] = $subscriber;
						}
					}
					if (MailTemplates::getByCode($template_code.'_new') instanceof MailTemplate) {
						$template_code = $template_code.'_new';
					}
					break;
				case 'edit':
					// Get values from 'property' column from application_log_details table using log id
					$logId = $log->getId();
					$logDetails = ApplicationLogDetails::instance()->findAll(array("conditions" => "application_log_id=" . $logId));
					$detailProperties = array();
					foreach($logDetails as $logDetail){
						$detailProperties[] = $logDetail->getProperty();
					}
					$notifiableProperties = array();
					// Get config options
					$descriptionConfig = explode(',',user_config_option('description_changed'));
					$assignedConfig = explode(',',user_config_option('user_assigned_to_task'));
					$startDueDateConfig = explode(',',user_config_option('start_or_due_date_modified'));
					$classificationConfig = explode(',',user_config_option('classification_changed'));
					$nameChangedConfig = explode(',',user_config_option('name_changed'));
					$timeEstimateConfig = explode(',',user_config_option('time_estimate_changed'));
					$percentCompletedConfig = explode(',',user_config_option('percent_completed_changed'));
					$priorityChangedConfig = explode(',',user_config_option('priority_changed'));
					// Check if config options has 'Immediate' => insert log detail in $notifiableProperties
					if(in_array('Immediate', $descriptionConfig)){
						$notifiableProperties[] = 'text';
					}
					if(in_array('Immediate', $assignedConfig)){
						$notifiableProperties[] = 'assigned_to_contact_id';
						$template_code = 'task_assigned_notification';
					}
					if(in_array('Immediate', $startDueDateConfig)){
						$notifiableProperties[] = 'start_date';
						$notifiableProperties[] = 'due_date';
					}
					if(in_array('Immediate', $classificationConfig)){
						$notifiableProperties[] = 'classification';
					}
					if(in_array('Immediate', $nameChangedConfig)){
						$notifiableProperties[] = 'name';
					}
					if(in_array('Immediate', $timeEstimateConfig)){
						$notifiableProperties[] = 'time_estimate';
					}
					if(in_array('Immediate', $percentCompletedConfig)){
						$notifiableProperties[] = 'percent_completed';
					}
					if(in_array('Immediate', $priorityChangedConfig)){
						$notifiableProperties[] = 'priority';
					}
					// If log property is found in $notifiableProperty => notify about this log
					foreach($detailProperties as $property){
						if(in_array($property, $notifiableProperties)){
							$option_val = 'Immediate';
							break;
						}
					}
					break;
				case 'delete':
				case 'trash':
					$option_val = '';
					break;
				case 'subscribe':
					$option_val = user_config_option('user_subscribed');
					// Get subscribers who had been subscribed or unsubscribed from the task
					$subscribersId = $log->getLogData();
					if(!is_array($subscribersId)){
						$subscribersId = explode(',', $subscribersId);
					}
					$logCreatedById = $log->getCreatedById();
					// Check if affected contact is not in the $people list and if log wasn't created by the person who suppose to receive notification
					$subscribers = array();
					foreach($subscribersId as $subId) {
						if ($logCreatedById != $subId) {
							$subscribers[] = Contacts::instance()->findOne(array("conditions" =>  "object_id=" . $subId));
						}
					}
					foreach($subscribers as $subscriber) {
						if (!isset($people[$subscriber->getId()])) {
							$people[$subscriber->getId()] = $subscriber;
						}
					}
					$template_code = 'user_subscribed';
					break;
				case 'unsubscribe':
					$option_val = user_config_option('user_subscribed');
					// Get subscribers who had been subscribed or unsubscribed from the task
					$subscribersId = $log->getLogData();
					if(!is_array($subscribersId)){
						$subscribersId = explode(',', $subscribersId);
					}
					$logCreatedById = $log->getCreatedById();
					// Check if affected contact is not in the $people list and if log wasn't created by the person who suppose to receive notification
					$subscribers = array();
					foreach($subscribersId as $subId) {
						if ($logCreatedById != $subId) {
							$subscribers[] = Contacts::instance()->findOne(array("conditions" =>  "object_id=" . $subId));
						}
					}
					foreach($subscribers as $subscriber) {
						if (!isset($people[$subscriber->getId()])) {
							$people[$subscriber->getId()] = $subscriber;
						}
					}
					$template_code = 'user_unsubscribed';
					break;
				case 'upload':
					break;
				case 'open':
				case 'close':
					if ($object instanceof ProjectTask) {
						$option_val = user_config_option('task_started_or_completed');
					}
					$use_template_subject = false;
					if (MailTemplates::getByCode('task_' . $log->getAction()) instanceof MailTemplate) {
						$template_code = 'task_' . $log->getAction();
						$use_template_subject = true;
					}
					break;
				default:
						
			}
	
			if (!isset($option_val)) {
				$option_val = array();
			}
			if (!is_array($option_val)) {
				$option_val = explode(',', $option_val);
			}
			$include_this_log = true;
			if (!in_array('Immediate', $option_val)) {
				$include_this_log = false;
			} 
			
			if ($include_this_log) {
				foreach ($subscribers as $subscriber) {
					if($subscriber->getId() != $log->getCreatedById()){
						if (!isset($application_logs_in_notifications[$subscriber->getId()])) {
							$application_logs_in_notifications[$subscriber->getId()] = array();
						}
						$application_logs_in_notifications[$subscriber->getId()][] = $log;
					}
				}
			}
		}

	}
		
	if (count($application_logs_in_notifications) > 0) {
		foreach ($people as $user) {
			$user_application_logs = array_var($application_logs_in_notifications, $user->getId());
			if (!is_array($user_application_logs)) continue;
			if((count($user_application_logs) > 1)) {
				$template_code = $object->getObjectTypeName() . '_notification';
			}

			$is_new_task = false;
			foreach($user_application_logs as $log){
				if($log instanceof ApplicationLog){
					if($log->getAction() == 'add' && $log->getRelObjectId() == $content_object_id){
						$is_new_task = true;
					}
				}
			}

			if($object instanceOf ProjectTask){
				if($notification_type == 'task_assigned' || $is_new_task){
					$assigned_to = $object->getAssignedTo();
					if($assigned_to instanceOf Contact && $assigned_to->getId() == $user->getId()){
						$template_code = 'task_notification_new';
					} else {
						$template_code = 'user_subscribed';
					}
				}
			}
			
			if (!MailTemplates::getByCode($template_code) instanceof MailTemplate) {
				$template_code = 'general_notification';
			}
			
			$template_variables = array(
					'email_type' => 'Immediate',
					'logs' => $user_application_logs,
					'object' => $object,
					'include_log_object_name' => true,
					'template_code' => $template_code,
			);

			$mail_template_params = array(
					'triggering_object' => $object,
					'template_code' => $template_code,
					'template_parameters' => $template_variables,
			);
			$mail_template_data = null;
			Hook::fire("get_mail_template_data", $mail_template_params, $mail_template_data);
			
			if ($mail_template_data) {
				$sendername = "Feng Office";
				if (isset($mail_template_data['from_name'])) {
					$sendername = $mail_template_data['from_name'];
				}
				
				// check if have to use forced smtp address and from name
				if (config_option("mail_transport") == Notifier::MAIL_TRANSPORT_SMTP) {
					$smtp_address = config_option("smtp_address");
					if (trim($smtp_address) != "") {
						$senderemail = $smtp_address;
					}
						
					$from_name = config_option("notification_from_name");
					if (trim($from_name) != "") {
						$sendername = $from_name;
					}
				}
				
				if (!$senderemail) {
					$senderemail = trim(owner_company()->getEmailAddress());
					if (!$senderemail) {
						$first_admin = Contacts::instance()->findOne(array("conditions" => "user_type > 0", "order" => "user_type, object_id"));
						$senderemail = trim($first_admin->getEmailAddress());
					}
				}
				$from = "$sendername <$senderemail>";
			
				$to_addresses = array_var($mail_template_data, 'to', array());
				
				$email_address = trim($user->getEmailAddress());
				if ($email_address != '') {
					$to_addresses[$user->getId()] = Notifier::prepareEmailAddress($email_address, $user->getObjectName());
				}
				
				$subject = $mail_template_data['subject'];
				if (!$subject && $summary) {
					$subject = $summary->getName();
				}
				
				$delete_previous_queued_emails = true;
				if (!str_ends_with($notification_type, ' reminder')) {
					// if not consilidating add, open or close actions then dont queue this email and let the other ones, if this email is to open or close a task then queue it
					$exclude_consolidation_add_close = user_config_option('exclude_consolidation_add_close');
					if ($exclude_consolidation_add_close) {
						$qe = QueuedEmails::instance()->findOne(array("conditions" => "`object_id` = ".$content_object_id." AND `log_action` IN ('add','open','close') AND `timestamp` > '".$since_date->toMySQL()."'"));
						if ($qe instanceof QueuedEmail) {
							$delete_previous_queued_emails = false;
							if (!in_array($notification_type, array('new','open','closed'))) {
								return;
							} else {
								$mail_ts = DateTimeValueLib::now();
							}
						}
					}
				}
				
				// dont consolidate if this email is a reminder
				if (str_ends_with($notification_type, ' reminder')) {
					$delete_previous_queued_emails = false;
				}
				
				// delete queued emails with logs included in the new notification
				if ($delete_previous_queued_emails) {
					QueuedEmails::instance()->delete("`object_id` = ".$content_object_id." AND `to` = '".$to_addresses[$user->getId()]. "' AND `timestamp` > '".$since_date->toMySQL()."'");
				}

				// don't use $content_object_id data if notification type is '% reminder'
				if (str_ends_with($notification_type, ' reminder')){
					$content_object_id = '';
				}

				if (count($to_addresses) > 0) {
					Notifier::queueEmail($content_object_id, 
						$to_addresses, array_var($mail_template_data, 'cc'), array_var($mail_template_data, 'bcc'), 
						$from, $subject, $mail_template_data['body'], 'text/html', '8bit', array(), $mail_ts);
				}
			}
		}
	}

}


