<?php

function can_manage_mail_templates($user) {
	return SystemPermissions::userHasSystemPermission($user, 'can_manage_mail_templates');;
}


function get_mail_template_available_variables() {
	$variables = array(
		// Product variables
		'product.logo',
		// invoice variables
		'invoice.id', 'invoice.name', 'invoice.date', 'invoice.created_on', 'invoice.external_payment_path',
		'invoice.number', 'invoice.total', 'invoice.mail',
		
		// affinity specific client properties variables
		'billing_contact.name', 'billing_contact.mail',
		'representative.name', 'representative.mail', 
		'client_billing_contact.mail','project_billing_contact.mail',
		
		// specific variables for client members
		'customer.id', 'contact.id',
		
		// managed event variables
		'event_member.id', 'event_member.name', 'event_member.description',
		'event_member.venue', 'event_member.venue.street', 'event_member.venue.city', 
		'event_member.venue.state','event_member.venue.zip_code', 'event_member.venue.country', 
		
		'event_member.start_date', 'event_member.end_date', 'event_member.start_datetime', 'event_member.end_datetime', 
		'summary_of_all_events',
		
		// client variables
		'client_member.id', 'client_member.name', 'client_member.mail', 
		'client_member.phone', 'client_member.address', 'client_member.webpage', 
			
		// general object variables
		'object.id', 'object.name', 'object.created_on', 'object.created_by', 'object.updated_on', 'object.updated_by', 'object.updated_by_email', 'object.object_type_name',
		'object.archived_on', 'object.archived_by', 'object.trashed_on', 'object.trashed_by',
		'object.comments', 'object.classification', 'object.subscribers', 'object.link',
		
		// logs
		'log.action', 'log.action_past', 'log.details',
		
		// comments variables
		'comment.text', 'comment.object_name',
		
		// summaries data
		'summary.name',
		
		// specific variables for tasks
		'task.information', 'task.assigned_to', 'task.assigned_by', 'task.assigned_on',
		'task.due_date', 'task.start_date', 'task.estimated_time', 'task.total_worked_time', 
		'task.status', 'task.priority', 'task.description',
		'task.completed_by', 'task.completed_on',

		//project
		'project.number',
		'project.name'
	);
	
	Hook::fire('add_more_mail_template_variables', null, $variables);
	
	return $variables;
}


function replace_mail_template_variables($original_string, $triggering_object, $template_parameters) {
	$result = $original_string;
	$available_variables = get_mail_template_available_variables();
	
	foreach ($available_variables as $var_name) {
		if (strpos($result, $var_name) !== false) {
			$var_value = null;
			
			$var_value = find_mail_template_variable_value($var_name, $triggering_object, $template_parameters);
			
			$result = replace_mail_template_variable_in_string($var_name, $var_value, $result);
		}
	}
	
	return $result;
}

function find_mail_template_variable_value($varname, $triggering_object, $template_parameters) {
	Env::useHelper('format');
	$email_type = '';

	if(isset($template_parameters['email_type']))
	{
		$email_type = $template_parameters['email_type'];
	}

	$email_styles ="<style>
			
	body{
		background-color:#fff;
	}

	.all-logs{
		max-width: 540px;
		min-width: 400px;
		margin: 40px auto 0;
	}

	.logs-group {
		background-color: #f1f1f4; /*#f1f1f4*/
		padding: 10px;
		width: 100%;
		box-sizing: border-box;
		border-radius: 5px;
		border-left: 5px solid #27a9ff;
		font-family: 'Open-sans', sans-serif;
		font-size: 16px;
		line-height: 1.4;
		color: #4a4a4a;
		margin: 10px 0;
	}

	.log-header {
		color: #8c8c8c;
	}

	.log-details {
		margin: 10px 15px;
		padding: 0;
		list-style-type: none;
	}

	.log-detail {
		color: #4a4a4a;
		font-weight: 300;
	}

	.log-detail--comment {
		margin: 10px 15px;
		font-weight: 400;
		font-style: italic;
		color: #1c1c1c;
		word-wrap: break-word;
	}

	.log-detail--description {
		font-weight: 400;
		color: #1c1c1c;
		word-wrap: break-word;
	}

	.log-detail--new-value {
		font-weight: 400;
		color: #1c1c1c;
		word-wrap: break-word;
	}

	.log-detail--old-value {
		text-decoration: line-through;
		color: #8c8c8c;
		word-wrap: break-word;
	}

	.object-type-name {
		font-family: 'Open-sans', sans-serif;
		font-weight: 400;
		margin-top: 30px;
		line-height: 1.6;
		text-align: center;
		font-size: 24px;
		color: #1c1c1c;
	}

	.object-name--link {
		color: #003562;
	}

	.object-type {
		color: #1c1c1c;
	}

	.task-info--table{
		margin: 15px auto;
		table-layout: fixed;
		font-family: 'Open-sans', sans-serif;
		font-weight: 300;
		padding: 15px;
		text-align: left;
		line-height: 1.6;
		border-radius: 5px;
		width: 100%;
		max-width: 540px;
		min-width: 400px;
		background-color: #f1f1f4;
	}

	.task-info--caption {
		margin: 30px 0 10px 0;
		font-size: 24px;
		font-weight: 400;
		font-family: 'Open-sans', sans-serif;
	}

	.task-info--table th{
		font-weight: 400;
		text-align: left;
		color: #8c8c8c;
		vertical-align: top;
		padding: 5px 20px 5px 0;
		word-wrap: break-word;
	}

	.task-info--table td{
		padding: 5px 0 5px 10px;
		width: 70%;
		word-wrap: break-word;
	}

	.subscribe-info {
		font-family: 'Open-sans', sans-serif;
		color: #1c1c1c;
		font-size: 16px;
		line-height: 1.4;
	}

	.summary-heading {
		font-family: 'Open-sans', sans-serif;
		color: #1c1c1c;
		font-size: 40px;
		text-align: center;
	}
	</style>";
	$var_value = '';
	
	switch ($varname) {
		// Notification container
		//  <div style="width: 100%; max-width: 600px; min-width: 400px; padding: 0 30px 30px 30px; box-sizing:border-box; box-shadow: 0 0px 10px rgba(0,0,0,0.1); margin: 20px auto; border-radius: 10px;">
		case 'product.logo':
			$is_evx_active = Plugins::instance()->isActivePlugin('evx_edition');
			$img_src = '';
			if($is_evx_active){
				$img_src = ROOT_URL.'/plugins/evx_edition/files/custom_logo.png';
				$img_style = "height:80px;
							  width:auto;
							  margin: 20px auto 10px;";
				$var_value = '<img style="'.$img_style.'" src="'.$img_src.'" alt="Company logo">';
			} else {
				$img_src = ROOT_URL.'/public/assets/themes/default/images/feng_logo.png';
				$img_container_style = 'max-width: 600px;
										min-width: 400px;
										margin: 0 -30px 10px;
										background-color: #7bb64f;
										border-radius: 10px 10px 0 0;';
				$img_style = "height:100px;
							  width:auto;
							  margin: 0 auto;
							  padding:5px 0;";
				$var_value = '<div style="'.$img_container_style.'"><img style="'.$img_style.'" src="'.$img_src.'" alt="Company logo"></div>';
			}	
			break;
		case 'object.id':
			$var_value = $triggering_object->getId();
			break;
		case 'object.name':
			$var_value = $triggering_object->getObjectName();
			break;
		case 'object.object_type_name':
			$var_value = $triggering_object->getObjectTypeName();
			break;
		case 'object.created_on':
			$var_value = format_datetime($triggering_object->getCreatedOn());
			break; 
		case 'object.created_by':
			$var_value = $triggering_object->getCreatedByDisplayName();
			break; 
		case 'object.updated_on':
			$var_value = format_datetime($triggering_object->getUpdatedOn());
			break;
		case 'object.updated_by':
			if(array_var($template_parameters, 'template_code') == 'comment_notification_new'){
				$log = $template_parameters['logs'][0];
				$var_value = Contacts::instance()->findById($log->getCreatedById())->getObjectName();
			} else {
				$var_value = $triggering_object->getUpdatedByDisplayName();
			}
			break;
		case 'object.updated_by_email':
			$var_value = $triggering_object->getUpdatedBy() ? $triggering_object->getUpdatedBy()->getEmailAddress() : '';
			break;
		case 'object.archived_on':
			$var_value = $triggering_object->getArchivedOn() instanceof DateTimeValue ? format_datetime($triggering_object->getArchivedOn()) : '';
			break;
		case 'object.archived_by':
			$archived_by = Contacts::instance()->findById($triggering_object->getArchivedById());
			$var_value = $archived_by instanceof Contact ? $archived_by->getObjectName() : '';
			break;
		case 'object.trashed_on':
			$var_value = $triggering_object->getTrashedOn() instanceof DateTimeValue ? format_datetime($triggering_object->getTrashedOn()) : '';
			break;
		case 'object.trashed_by':
			$var_value = $triggering_object->getTrashedByDisplayName();
			break;
		case 'object.comments':
			$var_value = "";
			$comments = $triggering_object->getComments();
			foreach ($comments as $comment) {/* @var $comment Comment */
				$var_value .= $comment->getAuthorName() . " - " . format_datetime($comment->getUpdatedOn()).":<br />";
				$var_value .= nl2br($comment->getText()) ."<br /><br />";
			}
			break; 
		case 'object.classification':
			$context_result = Notifier::getContext($triggering_object);
			$contexts = array_var($context_result, 'contexts', array());
			
			$first = true;
			foreach ($contexts as $dimension => $context) {
				if ($first) {
					$first = false;
				} else {
					$var_value .= '<br />';
				}
				
				if($dimension == "customer_project" || $dimension == "customers") {
					foreach ($context as $obj => $cont){
						$var_value .= lang($obj). ": ";
						foreach ($cont as $c){
							$var_value .= $c;
						}
						$var_value .= "&nbsp;";
					}
				} else {
					$var_value .= lang($dimension). ": ";
					$f = true;
					foreach ($context as $c){
						$var_value .= ($f ? "" : " - ") . $c;
						$f = false;
					}
					$var_value .= "&nbsp;";
				}
			}
			
			//$var_value = render_co_view_member_path($triggering_object);
			break; 
		case 'object.subscribers':
			$subscriber_names = array();
			$subscribers = $triggering_object->getSubscribers();
			foreach ($subscribers as $subscriber) {
				if ($subscriber instanceof Contact) {
					$subscriber_names[] = $subscriber->getObjectName();
				}
			}
			$var_value = implode(", ", $subscriber_names);
			break; 
		case 'object.link':
			$var_value = $triggering_object->getViewUrl();
			break;
			
			
		case 'comment.text':
			if ($triggering_object instanceof Comment) {
				$var_value = nl2br($triggering_object->getText());
			}
			break;
		case 'comment.object_name':
			if ($triggering_object instanceof Comment) {
				$rel_object = $triggering_object->getRelObject();
				if ($rel_object instanceof ContentDataObject) {
					$var_value = $rel_object->getObjectName();
				}
			}
			break;
			
		case 'task.assigned_to':
			$var_value = $triggering_object->getAssignedToName();
			break;
		case 'task.assigned_by':
			$var_value = $triggering_object->getAssignedBy()->getObjectName();
			break;
		case 'task.assigned_on':
			$var_value = format_datetime($triggering_object->getAssignedOn());
			break;
		case 'task.completed_by':
			$var_value = $triggering_object->getCompletedByName();
			break;
		case 'task.completed_on':
			$var_value = format_datetime($triggering_object->getCompletedOn());
			break;
		case 'task.due_date':
			if ($triggering_object->getDueDate() instanceof DateTimeValue) {
				$var_value = $triggering_object->getUseDueTime() ? format_datetime($triggering_object->getDueDate()) : format_date($triggering_object->getDueDate());
			}
			break;
		case 'task.information':
			if($triggering_object instanceof ProjectTask){
				// Task information header
				$var_value .= $email_styles;
				$var_value .= '<table class="task-info--table"><caption class="task-info--caption">Task Information</caption>';
				// Name
				$object_link = $triggering_object->getViewUrl();
				$task_name = '<tr><th>Name</th><td><a style="color: #003562;" href="'.$object_link.'">'.$triggering_object->getObjectName().'</a></td></tr>';
				// Assigned to
				$assigned_to = $triggering_object->getAssignedToName();
				$assigned_by = $triggering_object->getAssignedBy()->getObjectName();
				$task_assigned_to = '<tr><th>Assigned to</th><td>'.$assigned_to.' by '.$assigned_by.'</td></tr>';
				// Due date
				if ($triggering_object->getDueDate() instanceof DateTimeValue) {
					$due_date = $triggering_object->getUseDueTime() ? format_datetime($triggering_object->getDueDate()) : format_date($triggering_object->getDueDate());
					$task_due_date = '<tr><th>Due Date</th><td>'.$due_date.'</td></tr>';
				} else {
					$task_due_date = '';
				}
				// Status
				$status = $triggering_object->getCompletedById() > 0 ? lang('completed') : lang('pending');
				$task_status = '<tr><th>Status</th><td>'.$status.'</td></tr>';
				// Priority
				$priority = lang('normal priority');
				if ($triggering_object->getPriority() >= ProjectTasks::PRIORITY_URGENT) {
					$priority = lang('urgent priority');
				}else if ($triggering_object->getPriority() >= ProjectTasks::PRIORITY_HIGH) {
					$priority = lang('high priority');
				} else if ($triggering_object->getPriority() <= ProjectTasks::PRIORITY_LOW) {
					$priority = lang('low priority');
				} 
				$task_priority = '<tr><th>Priority</th><td>'.$priority.'</td></tr>';
				// Classifications
				$context_result = Notifier::getContext($triggering_object);
				$contexts = array_var($context_result, 'contexts', array());
				$classifications = '';
				$first = true;
				foreach ($contexts as $dimension => $context) {
					if ($first) {
						$first = false;
					} else {
						$classifications .= '<br />';
					}
					
					if($dimension == "customer_project" || $dimension == "customers") {
						foreach ($context as $obj => $cont){
							$classifications .= lang($obj). ": ";
							foreach ($cont as $c){
								$classifications .= $c;
							}
							$classifications .= "&nbsp;";
						}
					} else {
						$classifications .= lang($dimension). ": ";
						$f = true;
						foreach ($context as $c){
							$classifications .= ($f ? "" : " - ") . $c;
							$f = false;
						}
						$classifications .= "&nbsp;";
					}
				}
				$task_classifications = '<tr><th>Classifications</th><td>'.$classifications.'</td></tr>';
				// Description
				$task_description = '<tr><th>Description</th><td style="word-wrap: break-word;">'.trim($triggering_object->getText(), '<br />&nbsp;').'</td></tr>';
				// Final $var_value
				$var_value .= $task_name.$task_assigned_to.$task_due_date.$task_status.$task_priority.$task_classifications.$task_description.'</table>';
			}
			break;
		case 'task.start_date':
			if ($triggering_object->getStartDate() instanceof DateTimeValue) {
				$var_value = $triggering_object->getUseStartTime() ? format_datetime($triggering_object->getStartDate()) : format_date($triggering_object->getStartDate());
			}
			break;
		case 'task.estimated_time':
			if ($triggering_object->getTimeEstimate() > 0) {
				$var_value = DateTimeValue::FormatTimeDiff(new DateTimeValue(0), new DateTimeValue($triggering_object->getTimeEstimate() * 60), 'hm', 60);
			}
			break;
		case 'task.total_worked_time':
			if ($triggering_object->getTotalWorkedTime() > 0) {
				$var_value = DateTimeValue::FormatTimeDiff(new DateTimeValue(0), new DateTimeValue($triggering_object->getTotalWorkedTime() * 60), 'hm', 60);
			}
			break;
		case 'task.status':
			$var_value = $triggering_object->getCompletedById() > 0 ? lang('completed') : lang('pending');
			break;
		case 'task.priority':
			$priority = lang('normal priority');
			if ($triggering_object->getPriority() >= ProjectTasks::PRIORITY_URGENT) {
				$priority = lang('urgent priority');
			}else if ($triggering_object->getPriority() >= ProjectTasks::PRIORITY_HIGH) {
				$priority = lang('high priority');
			} else if ($triggering_object->getPriority() <= ProjectTasks::PRIORITY_LOW) {
				$priority = lang('low priority');
			}
			$var_value = $priority;
			break;
		case 'task.description':
			$var_value = $triggering_object->getText();
			break;
			

		case 'log.details':
			$var_value .= $email_styles;
			$var_value .= '<div class="all-logs">';
			if (isset($template_parameters['logs'])) {
				$last_rel_object_id = null;
				foreach ($template_parameters['logs'] as $log) {
					if (!$log instanceof ApplicationLog) continue;
					/* @var $log ApplicationLog */
					$log_detail_html = '';
					if (!in_array($log->getAction(), array('open','close'))) {
						$log_detail_html = ApplicationLogDetails::buildLogDetailsHtml($log, $email_type);
					}
					
					if ($log_detail_html == '') {
						$log_object = Objects::findObject($log->getRelObjectId());
						if ($log_object instanceof ContentDataObject) {
							
							if ($log_object instanceof Comment) {
								$log_detail_html = '<div class="logs-group"><div class="log-header">'.format_datetime($log_object->getUpdatedOn()) . ', ' .$log_object->getUpdatedByDisplayName() .' commented:</div><div class="log-detail log-detail--comment">"'. nl2br(trim($log_object->getText())).'"</div></div>';
							
							} else {
								
								if (in_array($log->getAction(), array('open','close'))) {
									$action_text = $log->getAction() == 'open' ? 'Reopened' : 'Completed';
									$log_detail_html = '<div class="logs-group"><div class="log-header">'.format_datetime($log->getCreatedOn()) . ' by ' .$log->getTakenByDisplayName() .':</div><ul class="log-details"><li class="log-detail">'.$action_text.' the task '. clean($log_object->getObjectName()).'</li></ul></div>';
									
								} else {
									$log_data_text = '';
									if (in_array($log->getAction(), array('subscribe','unsubscribe')) && $log->getLogData() != '') {
										$row = DB::executeOne("select group_concat(name) as names from ".TABLE_PREFIX."objects where id in (".$log->getLogData().")");
										$log_data_text = str_replace(',', ', ', $row['names']);

									}
									if($log_data_text != ''){
										$log_detail_html = '<div class="logs-group"><div class="log-header">'.format_datetime($log->getCreatedOn()) .':</div><ul class="log-details"><li class="log-detail">'. lang('activity ' . $log->getAction(),
										lang('the '.$log_object->getObjectTypeName()," "), $log->getTakenByDisplayName(),
										clean($log_object->getObjectName()), 
										$log_data_text) .'</li></ul></div>';
									} 

									/*lang('activity ' . $log->getAction(),
									lang('the '.$log_object->getObjectTypeName()," "), $log->getTakenByDisplayName(),
									clean($log_object->getObjectName()), 
									$log_data_text)*/ 
								}
							}
						}
					}
					

					$log_object = $log->getObject();	
					$isComment = $log_object instanceof Comment;
					
					if ($log_detail_html != '') {
						if($isComment){
							if (array_var($template_parameters, 'include_log_object_name') && $last_rel_object_id != $log_object->getRelObjectId()) {
								$var_value .= '<div class="object-type-name"><span class="object-type">'.lang($log_object->getRelObject()->getObjectTypeName()) ."</span> <a class='object-name--link' href='".$log_object->getRelObject()->getViewUrl()."'>".($log_object instanceof ContentDataObject ? $log_object->getObjectName() : $log->getObjectName()).'</a></div>';
							}
						} else {
							if (array_var($template_parameters, 'include_log_object_name') && $last_rel_object_id != $log->getRelObjectId()) {
								$var_value .= '<div class="object-type-name"><span class="object-type">'.lang($log->getObjectTypeName()) ."</span> <a class='object-name--link' href='".$log_object->getViewUrl()."'>". ($log_object instanceof ContentDataObject ? $log_object->getObjectName() : $log->getObjectName()).'</a></div>';
							}
						}
						$var_value .= $log_detail_html;
					}

					$last_rel_object_id = $isComment ? $log_object->getRelObjectId() : $log->getRelObjectId();

				}
			}
			$var_value .= '</div>';
			break;
		case 'log.action':
			if (isset($template_parameters['logs']) && count($template_parameters['logs'] > 0)) {
				if (count($template_parameters['logs']) == 1) {
					$var_value = $template_parameters['logs'][0] instanceof ApplicationLog ? $template_parameters['logs'][0]->getAction() : '';
				} else {
					$var_value = 'edited';
				}
			}
			break;
		case 'log.action_past':
			if (isset($template_parameters['logs']) && count($template_parameters['logs'] > 0)) {
				if (count($template_parameters['logs']) == 1) {
					$var_value = $template_parameters['logs'][0] instanceof ApplicationLog ? $template_parameters['logs'][0]->getAction().'ed' : '';
				} else {
					$var_value = 'edited';
				}
			}
			break;
			
		case 'summary.name':
			if (array_var($template_parameters, 'summary') instanceof NotificationSummary) {
				$var_value.= '<div class="summary-heading">' . $template_parameters['summary']->getName() . '</div>';
			}
			break;
		
		
		case 'invoice.id':
			$var_value = $template_parameters['invoice'] instanceof IncomeInvoice ? $template_parameters['invoice']->getId() : '';
			break;
		case 'invoice.name':
			$var_value = $template_parameters['invoice'] instanceof IncomeInvoice ? $template_parameters['invoice']->getName() : '';
			break;
		case 'invoice.date':
			if ($template_parameters['invoice'] instanceof IncomeInvoice && $template_parameters['invoice']->getDate() instanceof DateTimeValue) {
				$var_value = format_date($template_parameters['invoice']->getDate(), null, 0);
			}
			break;
		case 'invoice.created_on':
			if ($template_parameters['invoice'] instanceof IncomeInvoice) {
				$var_value = format_datetime($template_parameters['invoice']->getCreatedOn());
			}
			break;
		case 'invoice.external_payment_path':
			$var_value = $template_parameters['external_path_invoice_payment'];
			break;
		case 'invoice.number':
			//$var_value = $template_parameters['invoice'] instanceof IncomeInvoice ? $template_parameters['invoice']->getNumber() : '';
			$var_value = $template_parameters['invoice'] instanceof IncomeInvoice ? $template_parameters['invoice']->getInvoiceNumber() : '';
			break;
		case 'invoice.total':
			if ($template_parameters['invoice'] instanceof IncomeInvoice) {
				$c = Currencies::instance()->findById($template_parameters['invoice']->getCurrencyId());
				$c_sym = $c instanceof Currency ? $c->getSymbol() : '$';
				$var_value = format_money_amount($template_parameters['invoice']->getTotal(), $c_sym);
			}
			break;
		case 'invoice.mail':
			if ($template_parameters['invoice'] instanceof IncomeInvoice) {
				$var_value = $template_parameters['invoice']->getCustomerEmail();
			}
			break;
		case 'billing_contact.name':
			if (isset($template_parameters['billing_contact_name'])) {
				$var_value = $template_parameters['billing_contact_name'];
			} else {
				if ($template_parameters['member'] instanceof Member) {
					$billing_contact_cp = CustomProperties::instance()->getCustomPropertyByCode($template_parameters['member']->getObjectTypeId(), 'billing_contact');
					if ($billing_contact_cp instanceof CustomProperty) {
						$billing_contact_cp_val = CustomPropertyValues::getCustomPropertyValue($template_parameters['member']->getObjectId(), $billing_contact_cp->getId());
						if ($billing_contact_cp_val instanceof CustomPropertyValue) {
							$bc_contact = Contacts::instance()->findById($billing_contact_cp_val->getValue());
							$var_value = $bc_contact instanceof Contact ? $bc_contact->getObjectName() : '';
						}
					}
				}
			}
			break;
		case 'project_billing_contact.mail':
				if (isset($template_parameters['project_billing_contact_mail'])) {
					$var_value = $template_parameters['project_billing_contact_mail'];
				} 
				break;
		case 'client_billing_contact.mail':
			if (isset($template_parameters['client_billing_contact_mail'])) {
				$var_value = $template_parameters['client_billing_contact_mail'];
			} 
			break;
		case 'billing_contact.mail':
			if (isset($template_parameters['billing_contact_mail'])) {
				$var_value = $template_parameters['billing_contact_mail'];
			} else {
				if ($template_parameters['member'] instanceof Member) {
					$billing_contact_cp = CustomProperties::instance()->getCustomPropertyByCode($template_parameters['member']->getObjectTypeId(), 'billing_contact');
					if ($billing_contact_cp instanceof CustomProperty) {
						$billing_contact_cp_val = CustomPropertyValues::getCustomPropertyValue($template_parameters['member']->getObjectId(), $billing_contact_cp->getId());
						if ($billing_contact_cp_val instanceof CustomPropertyValue) {
							$bc_contact = Contacts::instance()->findById($billing_contact_cp_val->getValue());
							$var_value = $bc_contact instanceof Contact ? $bc_contact->getEmailAddress() : '';
						}
					}
				}
			}
			break;
		case 'representative.name':
			if ($template_parameters['member'] instanceof Member) {
				$representative_cp = CustomProperties::instance()->getCustomPropertyByCode($template_parameters['member']->getObjectTypeId(), 'representative');
				if ($representative_cp instanceof CustomProperty) {
					$representative_cp_val = CustomPropertyValues::getCustomPropertyValue($template_parameters['member']->getObjectId(), $representative_cp->getId());
					if ($representative_cp_val instanceof CustomPropertyValue) {
						$rep_contact = Contacts::instance()->findById($representative_cp_val->getValue());
						$var_value = $rep_contact instanceof Contact ? $rep_contact->getObjectName() : '';
					}
				}
			}
			break;
		case 'representative.mail':
			if ($template_parameters['member'] instanceof Member) {
				$representative_cp = CustomProperties::instance()->getCustomPropertyByCode($template_parameters['member']->getObjectTypeId(), 'representative');
				if ($representative_cp instanceof CustomProperty) {
					$representative_cp_val = CustomPropertyValues::getCustomPropertyValue($template_parameters['member']->getObjectId(), $representative_cp->getId());
					if ($representative_cp_val instanceof CustomPropertyValue) {
						$rep_contact = Contacts::instance()->findById($representative_cp_val->getValue());
						$var_value = $rep_contact instanceof Contact ? $rep_contact->getEmailAddress() : '';
					}
				}
			}
			break;
		case 'contact.id':
			$var_value = $template_parameters['contact_id'];
			break;
		case 'customer.id':
			$var_value = $template_parameters['customer_id'];
			if (!$var_value) {
				$var_value = $template_parameters['member'] instanceof Member ? $template_parameters['member']->getObjectId() : '';
			}
			break;
		case 'event_member.id':
			$var_value = $template_parameters['event_member'] instanceof Member ? $template_parameters['event_member']->getId() : '';
			break;
		case 'event_member.name':
			$var_value = $template_parameters['event_member'] instanceof Member ? $template_parameters['event_member']->getName() : '';
			break;
		case 'event_member.venue':
		case 'event_member.venue.street':
		case 'event_member.venue.city':
		case 'event_member.venue.state':
		case 'event_member.venue.zip_code':
		case 'event_member.venue.country':
			if ($template_parameters['event_member'] instanceof Member) {
				$venue_cp = CustomProperties::instance()->getCustomPropertyByCode($template_parameters['event_member']->getObjectTypeId(), 'venue');
				if ($venue_cp instanceof CustomProperty) {
					$venue_cp_val = CustomPropertyValues::getCustomPropertyValue($template_parameters['event_member']->getObjectId(), $venue_cp->getId());
					if ($venue_cp_val instanceof CustomPropertyValue) {
						$tmp = explode('|', $venue_cp_val->getValue());
						if ($varname == 'event_member.venue') {
							unset($tmp[0]); //remove address type
							$var_value = implode(' - ', array_filter($tmp));
						} else if ($varname == 'event_member.venue.street') {
							$var_value = $tmp[1];
						} else if ($varname == 'event_member.venue.city') {
							$var_value = $tmp[2];
						} else if ($varname == 'event_member.venue.state') {
							$var_value = $tmp[3];
						} else if ($varname == 'event_member.venue.zip_code') {
							$var_value = $tmp[4];
						} else if ($varname == 'event_member.venue.country') {
							$var_value = Countries::getCountryNameByCode($tmp[5]);
						}
					}
				}
			}
			break;
		case 'event_member.description':
			if ($template_parameters['event_member'] instanceof Member) {
				$desc_cp = CustomProperties::instance()->getCustomPropertyByCode($template_parameters['event_member']->getObjectTypeId(), 'description_special');
				if ($desc_cp instanceof CustomProperty) {
					$desc_cp_val = CustomPropertyValues::getCustomPropertyValue($template_parameters['event_member']->getObjectId(), $desc_cp->getId());
					if ($desc_cp_val instanceof CustomPropertyValue) {
						$var_value = $desc_cp_val->getValue();
					}
				}
			}
			break;
		case 'event_member.start_datetime':
		case 'event_member.start_date':
			if ($template_parameters['event_member'] instanceof Member) {
				$managed_event = ManagedEvents::instance()->findById($template_parameters['event_member']->getObjectId());
				if ($managed_event instanceof ManagedEvent) {
					$event = ProjectEvents::instance()->findById($managed_event->getEventId());
					if ($event instanceof ProjectEvent && $event->getStart() instanceof DateTimeValue) {
						if ($varname == 'event_member.start_datetime') {
							$var_value = format_datetime($event->getStart());
						} else {
							$var_value = format_date($event->getStart(), null, 0);
						}
					}
				}
			}
			break;
		case 'event_member.end_datetime':
		case 'event_member.end_date':
			if ($template_parameters['event_member'] instanceof Member) {
				$managed_event = ManagedEvents::instance()->findById($template_parameters['event_member']->getObjectId());
				if ($managed_event instanceof ManagedEvent) {
					$event = ProjectEvents::instance()->findById($managed_event->getEventId());
					if ($event instanceof ProjectEvent && $event->getDuration() instanceof DateTimeValue) {
						if ($varname == 'event_member.end_datetime') {
							$var_value = format_datetime($event->getDuration());
						} else {
							$var_value = format_date($event->getDuration(), null, 0);
						}
					}
				}
			}
			break;
		case 'summary_of_all_events':
			$members = $triggering_object instanceof ContentDataObject ? $triggering_object->getMembers() : array();
			tpl_assign('members', $members);
			$var_value = tpl_fetch(get_template_path('summary_of_all_events', 'managed_events', 'managed_events'));
			break;
		case 'client_member.id':
			$var_value = $template_parameters['customer_id'];
			if (!$var_value) {
			    $var_value = $template_parameters['member'] instanceof Member ? $template_parameters['member']->getId() : '';
			}
			break;
		case 'client_member.name':
			$var_value = $template_parameters['customer_name'];
			if (!$var_value) {
				$var_value = $template_parameters['member'] instanceof Member ? $template_parameters['member']->getName() : '';
			}
			break;
		case 'client_member.mail':
			$var_value = $template_parameters['customer_mail'];
			if (!$var_value) {
		       if ($template_parameters['member'] instanceof Member) {
			      $client = Customers::instance()->findById($template_parameters['member']->getObjectId());
			      if ($client instanceof Customer) {
				     $contact = $client->getContact();
				     if ($contact instanceof Contact) {
					    $var_value = $contact->getEmailAddress();
				     }
			      }
		        }
	        }
			break;
		case 'client_member.phone':
			if ($template_parameters['member'] instanceof Member) {
				$client = Customers::instance()->findById($template_parameters['member']->getObjectId());
				if ($client instanceof Customer) {
					$contact = $client->getContact();
					if ($contact instanceof Contact) {
						$contact_phone = ContactTelephones::instance()->findOne(array("conditions"=>"contact_id=".$contact->getId()));
						$var_value = $contact_phone instanceof ContactTelephone ? $contact_phone->getNumber() : '';
					}
				}
			}
			break;
		case 'client_member.address':
			if ($template_parameters['member'] instanceof Member) {
				$client = Customers::instance()->findById($template_parameters['member']->getObjectId());
				if ($client instanceof Customer) {
					$contact = $client->getContact();
					if ($contact instanceof Contact) {
						$contact_addresses = ContactAddresses::instance()->getByContact($contact);
						if (count($contact_addresses) > 0) {
							$contact_address = $contact_addresses[0];
							$var_value = $contact_address instanceof ContactAddress ? $contact_address->toString() : '';
						}
					}
				}
			}
			break;
		case 'client_member.webpage':
			if ($template_parameters['member'] instanceof Member) {
				$client = Customers::instance()->findById($template_parameters['member']->getObjectId());
				if ($client instanceof Customer) {
					$contact = $client->getContact();
					if ($contact instanceof Contact) {
						$var_value = $contact->getWebpageUrl('work');
						$contact_webpage = ContactWebpages::instance()->findOne(array("conditions"=>"contact_id=".$contact->getId()));
						$var_value = $contact_webpage instanceof ContactWebpage ? $contact_webpage->getUrl() : '';
					}
				}
			}
			break;
		case 'project.number':
				/* for project number*/
				$project_ot_id = ObjectTypes::findByName('project')->getId();
				$customProject_cp = CustomProperties::getCustomPropertyByCode($project_ot_id, "project_id");

				if ($customProject_cp instanceof CustomProperty) {
					$project_member = $template_parameters['invoice']->getMemberOfType($project_ot_id);
    				$customProject_cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $customProject_cp->getId());

					if ($customProject_cp_val instanceof CustomPropertyValue) {
						$var_value = $customProject_cp_val->getValue();
					}
				}
				break;
		case 'project.name':
			/* for project name*/
			$project_ot_id = ObjectTypes::findByName('project')->getId();
			$project_member = $template_parameters['invoice']->getMemberOfType($project_ot_id);
			$var_value= $project_member ? $project_member->getName() : '';
			break;
		default:
			$hook_params = array('triggering_object' => $triggering_object, 'template_parameters' => $template_parameters);
			Hook::fire('find_mail_template_variable_value', $hook_params, $var_value);
			break;
	}
	
	return $var_value;
}

function replace_mail_template_variable_in_string($varname, $value, $string) {
	$varname = '{{'.$varname.'}}';
	$string = str_replace('%7B%7B', '{{', $string);
	$string = str_replace('%7D%7D', '}}', $string);
	return str_replace($varname, $value, $string);
}


/**
 * Generates a temporary mail template to use only when there are no templates defined
 */
function build_default_temporary_mail_template() {

	$mail_template = new MailTemplate();

	$body = tpl_fetch(get_template_path('default_notification_body', 'mail_template', 'object_templates'));

	$mail_template->setCode('general_notification');
	$mail_template->setEnabled(true);
	$mail_template->setIsNotificationTemplate(true);
	$mail_template->setBody($body);
	$mail_template->setSubject("Activity on {{object.object_type_name}}: {{object.name}}");

	return $mail_template;
}
