<?php
/**
 * Objective controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class ObjectiveController extends ApplicationController {

	var $plugin_name = "objectives";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
	}
	
	function init() {
		require_javascript("ObjectiveManager.js", $this->plugin_name);
		ajx_current("panel", "objectives-containerpanel", null, null, true);
		ajx_replace(true);
	}
	
	function list_all() {
		ajx_current("empty");
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		$dim_order = null;
		$cp_order = null;
		$attributes = array("ids" => explode(',', array_var($_GET,'ids')));
		
		if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		} else if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		}
		
		$extra_conditions = "";
		$status_filter = array_var($_GET,'status_filter');
		if ($status_filter != -1) {
			$extra_conditions .= " AND `status`=$status_filter";
		}
		
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		$result = Objectives::instance()->listing(array(
			"order" => $order,
			"order_dir" => $order_dir,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"extra_conditions" => $extra_conditions,
			"count_results" => true,
			"only_count_results" => $only_count_result,
			"start" => $start,
			"limit" => $limit,
		));
		$objectives = $result->objects;
		$object = $this->prepareObject($objectives, $start, $limit, $result);
		
		$object['filters'] = array('status' => array(
			array('id' => 0, 'name' => lang('incomplete')),
			array('id' => 1, 'name' => lang('in progress')),
			array('id' => 2, 'name' => lang('complete')),
		));
		
		ajx_extra_data($object);
	}

	
	private function prepareObject($objectives, $start, $limit, $res_obj) {
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"objectives" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
		
		$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType(Objectives::instance()->getObjectTypeId());
		$count = min(array($limit, count($objectives)));
		for ($i = 0; $i < $count; $i++){
			if (isset($objectives[$i])){
				$obj = $objectives[$i];
				if ($obj instanceof Objective){
					$info = $obj->getArrayInfo();
					$info['memPath'] = json_encode($obj->getMembersIdsToDisplayPath());
					$object["objectives"][] = $info;

					foreach ($custom_properties as $cp) {
						$object["objectives"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $obj);
					}
    			}
			}
		}
		return $object;
	}

	
	function add() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		/*
		$notAllowedMember = '';
		if(!Objective::canAdd(logged_user(), active_context(), $notAllowedMember )) {
			if (str_starts_with($notAllowedMember, '-- req dim --')) flash_error(lang('must choose at least one member of', str_replace_first('-- req dim --', '', $notAllowedMember, $in)));
			else flash_error(lang('no context permissions to add',lang("objectives"),$notAllowedMember ));
			ajx_current("empty");
			return;
		}
		*/
		$objective_data = array_var($_POST, 'objective');
		
		if(!is_array($objective_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$obj = new Objective();
			$obj->setObjectTypeId(Objectives::instance()->getObjectTypeId());
			tpl_assign('objective', $obj);
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$objective = $this->save_objective($objective_data, $member_ids);
				
				DB::commit();

				ApplicationLogs::createLog($objective, ApplicationLogs::ACTION_ADD);
				flash_success(lang('success add objective', $objective->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	function edit() {
		$this->setTemplate('add');
		
		$objective = Objectives::instance()->findById(get_id());
		if(!($objective instanceof Objective)) {
			flash_error(lang('objective dnx'));
			ajx_current("empty");
			return;
		}
		
		if (logged_user()->isGuest() || !$objective->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$notAllowedMember = '';
		if(!Objective::canAdd(logged_user(), active_context(), $notAllowedMember )) {
			if (str_starts_with($notAllowedMember, '-- req dim --')) flash_error(lang('must choose at least one member of', str_replace_first('-- req dim --', '', $notAllowedMember, $in)));
			else flash_error(lang('no context permissions to add',lang("objectives"),$notAllowedMember ));
			ajx_current("empty");
			return;
		}
		
		$objective_data = array_var($_POST, 'objective');
		
		if(!is_array($objective_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			tpl_assign('objective', $objective);
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$objective = $this->save_objective($objective_data, $member_ids, $objective);
				
				DB::commit();

				ApplicationLogs::createLog($objective, ApplicationLogs::ACTION_EDIT);
				flash_success(lang('success edit objective', $objective->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	private function save_objective($objective_data, $member_ids, $objective = null) {
		// Save objective object
		$is_new = false;
		if (is_null($objective) || !$objective instanceof Objective) {
			$objective = new Objective();
			$is_new = true;
		}
		
		$objective_data['start_date'] = getDateValue($objective_data['start_date']);
		$objective_data['end_date'] = getDateValue($objective_data['end_date']);
		
		$objective->setFromAttributes($objective_data);
		$this->check_objective_definition($objective);
		$objective->refreshAccomplishmentPercent();
		$objective->save();
		
		$object_controller = new ObjectController();
		if (!is_null($member_ids)) {
			$object_controller->add_to_members($objective, $member_ids);
		}
		$object_controller->add_subscribers($objective);
		$object_controller->link_to_new_object($objective);
		$object_controller->add_custom_properties($objective);
		
		// guardar en el historico los cambios realizados
		$objective->addToHistory(logged_user(), ($is_new ? 'add' : 'edit'));
		
		return $objective;
	}
	
	private function check_objective_definition(Objective $objective) {
		if (trim($objective->getObjectName() == '')) {
			throw new Exception(lang('objective name required'));
		}
		if ($objective->getDirection() == 'asc') {
			if ($objective->getInitialValue() > $objective->getFinalObjective()) {
				throw new Exception(lang('initial value cant be greater than final objective'));
			} else {
				if ($objective->getInitialValue()) {
					if ($objective->getIntermediateObjective() > $objective->getFinalObjective()) {
						throw new Exception(lang('intermediate objective cant be greater than final objective'));
					} else if ($objective->getIntermediateObjective() < $objective->getInitialValue()) {
						throw new Exception(lang('intermediate objective cant be smaller than final objective'));
					}
				}
			}
		} else {
			if ($objective->getInitialValue() < $objective->getFinalObjective()) {
				throw new Exception(lang('initial value cant be smaller than final objective'));
			} else {
				if ($objective->getInitialValue()) {
					if ($objective->getIntermediateObjective() < $objective->getFinalObjective()) {
						throw new Exception(lang('intermediate objective cant be smaller than final objective'));
					} else if ($objective->getIntermediateObjective() > $objective->getInitialValue()) {
						throw new Exception(lang('intermediate objective cant be greater than initial value'));
					}
				}
			}
		}
	}
	
	
	
	function view() {
		ajx_set_no_toolbar();
		$objective = Objectives::instance()->findById(get_id());
		if(!($objective instanceof Objective)) {
			flash_error(lang('objective dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$objective->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		tpl_assign('objective', $objective);
	}
	
	
	
	private function resolveAction($action, $attributes){
				 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				$messages = array();
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$objective = Objectives::instance()->findById($id);
					if ($objective instanceof Objective && $objective->canDelete(logged_user())){
						try{
							DB::beginWork();
							$objective->trash();
							DB::commit();
							ApplicationLogs::createLog($objective, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$messages[] = $e->getMessage();
							$err++;
						}
					} else {
						$err++;
						$messages[] = lang('cannot delete executed objective', $objective instanceof Objective ? clean($objective->getName()) : '');
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) .("\n - ".implode("\n - ",$messages)). ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			case "archive":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$objective = Objectives::instance()->findById($id);
					if ($objective instanceof Objective && $objective->canEdit(logged_user())){
						try{
							DB::beginWork();
							$objective->archive();
							DB::commit();
							ApplicationLogs::createLog($objective, ApplicationLogs::ACTION_ARCHIVE);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error archive objects", $err) . "<br />" . ($succ > 0 ? lang("success archive objects", $succ) : "");
				} else {
					$resultMessage = lang("success archive objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;		
		} // switch
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
	}
}