<?php
	class Objective extends BaseObjective {
		
		
		function getArrayInfo() {
			$info = array();
			
			$columns = $this->getColumns();
			foreach ($columns as $col) {
				$info[$col] = $this->getColumnValue($col);
				if ($info[$col] instanceof DateTimeValue) $info[$col] = format_date($info[$col], null, 0);
			}
			
			$general_info = $this->getObject()->getArrayInfo();
			$info = array_merge($info, $general_info);
			
			$info['row_color'] = $this->getRowColor();
			$info['accomplishment_percent'] = number_format($info['accomplishment_percent'], 2)." %";
			
			return $info;
		}
		
		function getRowColor() {
			if ($this->getDirection() == 'asc') {
				if ($this->getCurrentValue() >= $this->getFinalObjective()) return 'green';
				else if ($this->getCurrentValue() >= $this->getIntermediateObjective()) return 'yellow';
				else return 'red';
			} else {
				if ($this->getCurrentValue() <= $this->getFinalObjective()) return 'green';
				else if ($this->getCurrentValue() <= $this->getIntermediateObjective()) return 'yellow';
				else return 'red';
			}
		}
		
		function refreshAccomplishmentPercent() {
			$total = abs($this->getFinalObjective() - $this->getInitialValue());
			$current = $this->getCurrentValue() - $this->getInitialValue();
			if ($current < 0) $current = 0;
			$percent = abs($current * 100 / $total);
			$percent = number_format($percent, 2);
			// percent
			$this->setAccomplishmentPercent($percent);
			
			// status
			if ($this->getDirection() == 'asc') {
				if ($this->getCurrentValue() >= $this->getFinalObjective()) $status = 2;
				else if ($this->getCurrentValue() >= $this->getIntermediateObjective()) $status = 1;
				else $status = 0;
			} else {
				if ($this->getCurrentValue() <= $this->getFinalObjective()) $status = 2;
				else if ($this->getCurrentValue() <= $this->getIntermediateObjective()) $status = 1;
				else $status = 0;
			}
			$this->setStatus($status);
		}
		
		function addToHistory($user, $log_data) {
			
			$history = new ObjectiveHistory();
			
			$info = array();
			$columns = $this->getColumns();
			foreach ($columns as $col) {
				$info[$col] = $this->getColumnValue($col);
			}
			
			$history->setFromAttributes($info);
			
			$history->setUserId($user->getId());
			$history->setModificationDate(DateTimeValueLib::now());
			$history->setModificationLog($log_data);
			
			$history->save();
		}
		
		static function canAdd(Contact $user, $context, &$notAllowedMember = ''){
			return can_add($user, $context, Objectives::instance()->getObjectTypeId(), $notAllowedMember);
		}
		
		/**
		 * Returns true if $user can access this message
		 *
		 * @param Contact $user
		 * @return boolean
		 */
		function canView(Contact $user) {
			return can_read($user, $this->getMembers(), $this->getObjectTypeId());
		} // canView
	
		/**
		 * Check if specific user can edit this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canEdit(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canEdit
	
	
		/**
		 * Check if specific user can delete this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canDelete(Contact $user) {
			return can_delete($user, $this->getMembers(), $this->getObjectTypeId());
		} // canDelete
	
		/**
		 * Check if specific user can comment this message
		 *
		 * @access public
		 * @param void
		 * @return boolean
		 */
		function canAddComment(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canAddComment
	}