
og.ObjectiveManager = function() {
	var actions;
	this.doNotRemove = true;
	this.needRefresh = false;
	
	this.actual_from_filter = null;
	this.actual_to_filter = null;
	this.actual_status_filter = -1;
	
	this.is_loading = false;
	
	this.fields = ['object_id', 'name', 'description', 'initial_value', 'intermediate_objective', 'final_objective', 'current_value',
	               'start_date', 'end_date', 'direction', 'row_color', 'accomplishment_percent', 'memPath'];
	
  	var cps = og.custom_properties_by_type['objective'] ? og.custom_properties_by_type['objective'] : [];
  	var cp_names = [];
  	for (i=0; i<cps.length; i++) {
  		cp_names.push('cp_' + cps[i].id);
  	}
  	this.fields = this.fields.concat(cp_names);
   	
   	var dim_names = [];
   	for (did in og.dimensions_info) {
		if (isNaN(did)) continue;
		dim_names.push('dim_' + did);
	}
   	this.fields = this.fields.concat(dim_names);
  	
	
	if (!og.ObjectiveManager.store) {
		og.ObjectiveManager.store = new Ext.data.Store({
			proxy: new og.GooProxy({
				url: og.getUrl('objective', 'list_all')
			}),
			reader: new Ext.data.JsonReader({
				root: 'objectives',
				totalProperty: 'totalCount',
				id: 'id',
				fields: this.fields
			}),
			remoteSort: true,
			listeners: {
				'load': function(store, result) {
					
					var d = this.reader.jsonData;
					this.fireEvent('after list load', "");
					if (d.totalCount == 0) {
						var sel_context_names = og.contextManager.getActiveContextNames();
						if (sel_context_names.length > 0) {
							this.fireEvent('messageToShow', lang("no objects message", lang("objectives"), sel_context_names.join(', ')));
						} else {
							this.fireEvent('messageToShow', lang("no more objects message", lang("objectives")));
						}
					} else {
						this.fireEvent('messageToShow', "");
					}
					
					var man = Ext.getCmp("objective-manager");
					
					if (d.filters.status) {
						var items = [['-1', '-- ' + lang('All') + ' --']];
						for (i=0; i<d.filters.status.length; i++) {
							items[items.length] = [d.filters.status[i].id, d.filters.status[i].name];
						}
						var status_filter = Ext.getCmp('ogObjectivesStatusFilterCombo');
						status_filter.reset();
						status_filter.store.removeAll();
						status_filter.store.loadData(items);
						
						status_filter.setValue(man.actual_status_filter);
						status_filter.collapse();
					}
					
					var sm = man.getSelectionModel();
					sm.clearSelections();
					
					og.eventManager.fireEvent('after grid panel load', {man:man, data:d});
					
					if (d && d.totals) {
						man.updateGridPagingToolbar({totalCount: d.totals.total_rows});
					} else {
						man.reloadGridPagingToolbar('objective','list_all','objective-manager');
					}
					man.is_loading = false;
					
					og.eventManager.fireEvent('replace all empty breadcrumb', null);
				}
			}
		});
		og.ObjectiveManager.store.setDefaultSort('start_date', 'asc');
	}
	this.store = og.ObjectiveManager.store;
	this.store.addListener({messageToShow: {fn: this.showMessage, scope: this}});

	function renderDragHandle(value, p, r, ix) {
		if (r.data.object_id == -1) {
			return '';
		}
		return '<div class="img-grid-drag" title="' + lang('click to drag') + '" onmousedown="var sm = Ext.getCmp(\'objective-manager\').getSelectionModel();if (!sm.isSelected('+ix+')) sm.clearSelections();sm.selectRow('+ix+', true);"></div>';
	}
	
	function renderExpandHandle(value, p, r, ix) {
		if (r.data.object_id == -1) {
			return '';
		}
		return '<div id="expand-handle-'+ix+'" class="db-ico ico-expand cursor-pointer" title="' + lang('click to expand') + '" onclick="og.objectives.requestPayments(this, '+r.data.object_id+');"></div>';
	}
	
	var readClass = 'read-unread-' + Ext.id();
	function renderName(value, p, r) {
		if (isNaN(r.data.object_id)) {
			return '<span class="bold" id="'+r.data.object_id+'">'+ (value ? og.clean(value) : '') +'</span>';
		}
		
		var text = '<span class="bold">'+ og.clean(value) +'</span>';
		if (r.data.description && r.data.description.length > 0) {
			text += '<span class="desc"> - ' + r.data.description + '</span>';
		}
		
		var onclick = 'og.openLink(og.getUrl(\'objective\', \'view\', {id: '+r.data.object_id+'})); return false;';

		var dir = '&nbsp;&nbsp;<span class="coViewAction '+(r.data.direction=='desc'?'ico-arrowdown':'ico-arrowup')+'">&nbsp;</span>';
		
		return String.format('<a style="font-size:120%;" href="{1}" onclick="{3}" title="{2}">{0}</a>',
				text, og.getUrl('objective', 'view', {id: r.data.object_id}), og.clean(value), onclick) + dir;
		
	}
	
	function renderDate(value, p, r) {
		return og.clean(value);
	}

	function renderIcon(value, p, r) {
		if (r.data.object_id == -1) {
			return '';
		}
		return '<div class="link-ico ico-objective"></div>';
	}
	
	function renderAmount(value, p, r) {
		return '<span class="bold">'+ value +'</span>';
	}
	
	function getSelectedIds() {
		var selections = sm.getSelections();
		if (selections.length <= 0) {
			return '';
		} else {
			var ret = '';
			for (var i=0; i < selections.length; i++) {
				ret += "," + selections[i].data.object_id;
			}
			return ret.substring(1);
		}
	}
	this.getSelectedIds = getSelectedIds;
	
	function getFirstSelectedId() {
		var selections = sm.getSelections();
		if (selections.length <= 0) {
			return '';
		} else {
			return selections[0].data.object_id;
		}
		return '';
	}

	var sm = new Ext.grid.CheckboxSelectionModel();
	sm.on('selectionchange', function() {
		if (sm.getCount() <= 0) {
			actions.edit.setDisabled(true);
			actions.del.setDisabled(true);
		} else {
			actions.edit.setDisabled(false);
			actions.del.setDisabled(false);
		}
	});
	
	var cm_info = [
		sm,{
			/*id: 'draghandle',
			header: '&nbsp;',
			dataIndex: 'draghandle',
			width: 18,
        	renderer: renderDragHandle,
        	fixed:true,
        	resizable: false,
        	hideable:false,
        	menuDisabled: true
		},{*/
			id: 'icon',
			header: '&nbsp;',
			dataIndex: 'type',
			width: 28,
        	renderer: renderIcon,
        	fixed:true,
        	resizable: false,
        	hideable:false,
        	menuDisabled: true
		},{
			id: 'name',
			header: lang("objective"),
			dataIndex: 'name',
			width: 120,
			renderer: renderName,
			sortable: true
        },{
			id: 'description',
			header: lang("description"),
			dataIndex: 'description',
			width: 120,
			renderer: og.clean,
			hidden:true
        },{
			id: 'initial_value',
			header: lang("initial value"),
			dataIndex: 'initial_value',
			width: 60,
			align: 'center',
			renderer: og.clean,
			sortable: true
        },{
			id: 'intermediate_objective',
			header: lang("intermediate objective"),
			dataIndex: 'intermediate_objective',
			width: 60,
			align: 'center',
			renderer: og.clean,
			sortable: true
        },{
			id: 'final_objective',
			header: lang("final objective"),
			dataIndex: 'final_objective',
			width: 60,
			align: 'center',
			renderer: og.clean,
			sortable: true
        },{
			id: 'current_value',
			header: lang("current value"),
			dataIndex: 'current_value',
			width: 60,
			align: 'center',
			renderer: og.clean,
			sortable: true
        },{
			id: 'start_date',
			header: lang("start date"),
			dataIndex: 'start_date',
			width: 60,
			align: 'center',
			renderer: renderDate,
			sortable: true
        },{
			id: 'end_date',
			header: lang("end date"),
			dataIndex: 'end_date',
			width: 60,
			align: 'center',
			renderer: renderDate,
			sortable: true
        },{
			id: 'accomplishment_percent',
			header: "%",
			dataIndex: 'accomplishment_percent',
			width: 60,
			align: 'right',
			renderer: og.clean,
			sortable: true
        }];
	
	    
    // custom property columns
	var cps = og.custom_properties_by_type['objective'] ? og.custom_properties_by_type['objective'] : [];
	this.addCustomPropertyColumns(cps, cm_info, 'objective-manager');

	// dimension columns
	// Commented because if these columns are before the objective columns, when rendering the payments table cells are in the wrong order
	for (did in og.dimensions_info) {
		if (isNaN(did)) continue;
		var key = 'lp_dim_' + did + '_show_as_column';
		if (og.preferences['listing_preferences'][key]) {
			cm_info.push({
				id: 'dim_' + did,
				header: og.dimensions_info[did].name,
				dataIndex: 'dim_' + did,
				sortable: true,
				renderer: og.renderDimCol
			});
			og.breadcrumbs_skipped_dimensions[did] = did;
		}
	}
	
	// create column model
    var cm = new Ext.grid.ColumnModel(cm_info);
    cm.on('hiddenchange', this.afterColumnShowHide, this);
    cm.defaultSortable = false;
		
	actions = {
		newCO: new Ext.Action({
			id: 'new_button_objective',
			text: lang('new'),
            tooltip: lang('add new objective'),
            iconCls: 'ico-new new_button',
            hidden: og.replace_list_new_action && og.replace_list_new_action.objective,
            handler: function() {
				og.render_modal_form('', {c:'objective', a:'add'});
			}
		}),
		edit: new Ext.Action({
			text: lang('edit'),
            tooltip: lang('edit selected object'),
            iconCls: 'ico-edit',
			disabled: true,
			handler: function() {
				og.render_modal_form('', {c:'objective', a:'edit', params: {id:getFirstSelectedId()}});
			},
			scope: this
		}),
		del: new Ext.Action({
			text: lang('move to trash'),
            tooltip: lang('move selected objects to trash'),
            iconCls: 'ico-trash',
			disabled: true,
			handler: function() {
				var confirm_trash_config = parseInt(og.preferences['enableTrashConfirmation']);
				
				if (og.confirmNorification(lang('confirm move to trash'), confirm_trash_config)) {
					this.load({
						action: 'delete',
						ids: getSelectedIds()
					});
					this.getSelectionModel().clearSelections();
				}
			},
			scope: this
		})
    };
	
	var uid = Ext.id();
	filters = {
		status_filter: new Ext.form.ComboBox({
	    	id: 'ogObjectivesStatusFilterCombo',
	    	store: new Ext.data.SimpleStore({
		        fields: ['value', 'text'],
		        data : []
		    }),
		    displayField:'text',
	        //typeAhead: true,
	        mode: 'local',
	        triggerAction: 'all',
	        selectOnFocus:true,
	        width:160,
	        valueField: 'value',
	        valueNotFoundText: '',
	        listeners: {
	        	'select' : function(combo, record) {
					var man = Ext.getCmp("objective-manager");
					man.actual_status_filter = combo.getValue();
					man.reloadFiltering();
	        	}
	        }
		})
		/*
		filter_from: new Ext.Action({
			id: uid + 'filter_date_from',
			text: lang('select a date'),
	        tooltip: lang('select a date'),
	        menu: new og.drawDateMenuPicker({
	        	items:[new Ext.menu.Item({
	        		id: uid + 'filter_from_remove',
	        		text: lang('remove filter'),
	        		iconCls: 'ico-delete',
	        		hidden: true,
	        		handler: function() {
		        		var man = Ext.getCmp('objective-manager');
	        			man.actual_from_filter = null;
	        			man.reloadFiltering();
	        			this.hide();
	        			Ext.getCmp(uid + 'filter_date_from').setText(lang('select a date'));
	        		}
	        	})],
	        	listeners: {
	        		'select': function(dp, date) {
			        	var man = Ext.getCmp('objective');
		    			man.actual_from_filter = date.format(og.preferences['date_format']);
			        	Ext.getCmp(uid + 'filter_date_from').setText(date.format(og.preferences['date_format']));
			        	man.reloadFiltering();
			        	Ext.getCmp(uid + 'filter_from_remove').show();
	        		}
		        }
	        })
		}),
		filter_to: new Ext.Action({
			id: uid + 'filter_date_to',
			text: lang('select a date'),
	        tooltip: lang('select a date'),
	        menu: new og.drawDateMenuPicker({
	        	items:[new Ext.menu.Item({
	        		id: uid + 'filter_to_remove',
	        		text: lang('remove filter'),
	        		iconCls: 'ico-delete',
	        		hidden: true,
	        		handler: function() {
		        		var man = Ext.getCmp('objective');
	        			man.actual_to_filter = null;
	        			man.reloadFiltering();
	        			this.hide();
	        			Ext.getCmp(uid + 'filter_date_to').setText(lang('select a date'));
	        		}
	        	})],
	        	listeners: {
	        		'select': function(dp, date) {
	        			var man = Ext.getCmp('objective');
	        			man.actual_to_filter = date.format(og.preferences['date_format']);
			        	Ext.getCmp(uid + 'filter_date_to').setText(date.format(og.preferences['date_format']));
			        	man.reloadFiltering();
			        	Ext.getCmp(uid + 'filter_to_remove').show();
	        		}
		        }
	        })
		})*/
	}
    
	var tbar = [];
	if (!og.loggedUser.isGuest) {
		if (og.replace_list_new_action && og.replace_list_new_action.objective) {
			tbar.push(og.replace_list_new_action.objective);
		}
		
		tbar.push(actions.newCO);
		tbar.push('-');
		tbar.push(actions.edit);
		tbar.push(actions.del);
		tbar.push('-');
		tbar.push(lang('status'));
		tbar.push(filters.status_filter);
		/*
		tbar.push(lang('from date') + ": ");
		tbar.push(filters.filter_from);
		tbar.push(' ');
		tbar.push(lang('to date') + ": ");
		tbar.push(filters.filter_to);*/
	}
	
	og.ObjectiveManager.superclass.constructor.call(this, {
		store: this.store,
		layout: 'fit',
		cm: cm,
		enableDrag: true,
		ddGroup: 'MemberDD',
		stateful: og.preferences['rememberGUIState'],
		id: 'objective-manager',
		stripeRows: true,
		closable: true,
		loadMask: true,
		bbar: new og.CurrentPagingToolbar({
			pageSize: og.config['files_per_page'],
			store: this.store,
			displayInfo: true,
			displayMsg: lang('displaying objects of'),
			emptyMsg: lang("no objects to display")

		}),
		viewConfig: {
			forceFit: true
		},
		sm: sm,
		tbar:tbar,
		listeners: {
			'render': {
				fn: function() {
					this.innerMessage = document.createElement('div');
					this.innerMessage.className = 'inner-message';
					var msg = this.innerMessage;
					var elem = Ext.get(this.getEl());
					var scroller = elem.select('.x-grid3-scroller');
					scroller.each(function() {
						this.dom.appendChild(msg);
					});
				},
				scope: this
			},
			'columnmove': {
				fn: function(old_index, new_index) {
					og.eventManager.fireEvent('replace all empty breadcrumb', null);
				},
				scope: this
			}
		}
	});
	
};

Ext.extend(og.ObjectiveManager, Ext.grid.GridPanel, {
	load: function(params) {
		if (!params) params = {};
		var start;
		if (typeof params.start == 'undefined') {
			start = (this.getBottomToolbar().getPageData().activePage - 1) * og.config['files_per_page'];
		} else {
			start = 0;
		}
		
		this.store.baseParams = {
			context: og.contextManager.plainContext(),
			status_filter: params.status_filter ? params.status_filter : this.actual_status_filter
		};
		
		this.updateColumnModelHiddenColumns();
		
		this.store.removeAll();
		this.store.load({
			params: Ext.apply(params, {
				start: start,
				limit: og.config['files_per_page']				
			})
		});
	},
	
	activate: function() {
		if (this.needRefresh) {
			this.reset();
		}
	},
	
	reset: function() {
		var params = {start:0};
		
		this.load(params);
	},
	
	showMessage: function(text) {
		this.innerMessage.innerHTML = text;
	},
	
	reloadFiltering: function() {
		if (this.isLoading()) return;
		this.is_loading = true;
		
		var config = {
			start:0,
			status_filter: this.actual_status_filter
		};
		
		this.load(config);
	},
	
	isLoading: function() {
		return this.is_loading;
	},
	
	trashObjects: function() {
		if (confirm(lang('confirm move to trash'))) {
			this.load({
				action: 'delete',
				ids: this.getSelectedIds()
			});
			this.getSelectionModel().clearSelections();
		}
	},
	
	archiveObjects: function() {
		if (confirm(lang('confirm archive selected objects'))) {
			this.load({
				action: 'archive',
				ids: this.getSelectedIds()
			});
			this.getSelectionModel().clearSelections();
		}
	}
});


Ext.reg("objectives", og.ObjectiveManager);

/************************************************
Container for ObjectiveManager,
*************************************************/
og.ObjectiveManagerPanel = function() {
	this.doNotRemove = true;
	this.needRefresh = false;
	
	this.manager = new og.ObjectiveManager();
	
	og.ObjectiveManagerPanel.superclass.constructor.call(this, {
		layout: 'fit',
		border: false,
		bodyBorder: false,
		items: [
			this.manager
		],
		closable: true
	});
}

Ext.extend(og.ObjectiveManagerPanel, Ext.Panel, {
	load: function(params) {
		this.manager.load(params);
	},
	activate: function() {
		this.manager.activate();
	},	
	reset: function() {
		this.manager.reset();
	},	
	showMessage: function(text) {
		this.manager.showMessage(text);
	}
});

Ext.reg("objectives-containerpanel", og.ObjectiveManagerPanel);


Ext.grid.GridView.override({
	getRowClass: function (record, index, rowParams, store) {
		if (record.data.row_color) {
			return "objective-row-"+record.data.row_color;
		}else{
			return "";
		}
	}/*,
	focusCell : function(row, col, hscroll){
		this.syncFocusEl(this.ensureVisible(row, col, hscroll));
		this.focusEl.focus.defer(1, this.focusEl);
	},

    syncFocusEl : function(row, col, hscroll){
        var xy = row;
        if(!Ext.isArray(xy)){
            row = Math.min(row, Math.max(0, this.getRows().length-1));
            //xy = this.getResolvedXY(this.resolveCell(row, col, hscroll));
        }
        this.focusEl.setXY(xy||this.scroller.getXY());
    }*/
});
