<?php
function get_fixed_allowed_columns_by_objet_type_name($object_type_name) {
    $allowed_fields = array();
    switch ($object_type_name) {
        case 'task':
            $allowed_fields = array(
                'name',
                'text',
                'start_date',
                'due_date',
                'priority',
                'time_estimate',
                'completed_on',
                'assigned_to_contact_id',
                'completed_by_id'
            );
            break;
        case 'payment_receipt':
            $allowed_fields = array(
                'name',
                'date',
                'quantity',
                'description',
                'unit_cost',
                'unit_price',
                'amount',
                'total_price',
                'paid_by_id',
                'is_billable',
                'expense_type',
                'price_multiplier',
                'currency_id',
                'invoicing_status',
                'total_cost_without_taxes'
            );
            break;
        case 'timeslot':
            $allowed_fields = array(
                'name',
                'date',
                'description',
                'total_hours',
                'fixed_billing',
                'hourly_billing',
                'fixed_cost',
                'hourly_cost',
                'contact_id',
                'invoicing_status',
                'rate_currency_id',
                'cost_currency_id',
                'billing_category_status'
            );
            break;
        case 'invoice':
            $allowed_fields = array(
                'name',
                'date',
                'expiration_date',
                'object_subtype_id',
                'additional_information',
                'series',
                //'invoice_number',
                'due_date',
                'status',
                'sent',
                'customer_email',
                'customer_address',
                'customer_city',
                'customer_state',
                'customer_country',
                'customer_zipcode',
                'invoice_type'
            );
            break;
        case 'invoice_payment_receipt':
            $allowed_fields = array(
                'name'
            );
            break;
        default:
            $allowed_fields = array(
                'name',
                'text',
                'description'
            );
    }
    return $allowed_fields;
}

function get_allowed_columns_by_objet_type($object_type) {
    $fields = array();
    if(isset($object_type)){
        $ot = ObjectTypes::instance()->findById($object_type);
        $customProperties = array();
        $memberCustomProperties = array();
        if ($ot->getType() == 'dimension_group') {
            $memberCustomProperties = MemberCustomProperties::getAllMemberCustomPropertiesByObjectType($object_type);
        } else {
            $customProperties = CustomProperties::getAllCustomPropertiesByObjectType($object_type);
        }
        $objectFields = array();

        foreach($customProperties as $cp){
            if ($cp->getType() == 'table') continue;

            if ($cp->getType() == 'address'){
                $fields[] = array('cp_field_address' => 1,'id' => $cp->getId(), 'sub_id' => 'street', 'name' => $cp->getName().' '.lang('street'));
                $fields[] = array('cp_field_address' => 1,'id' => $cp->getId(), 'sub_id' => 'city', 'name' => $cp->getName().' '.lang('city'));
                $fields[] = array('cp_field_address' => 1,'id' => $cp->getId(), 'sub_id' => 'state', 'name' => $cp->getName().' '.lang('state'));
                $fields[] = array('cp_field_address' => 1,'id' => $cp->getId(), 'sub_id' => 'country', 'name' => $cp->getName().' '.lang('country'));
                $fields[] = array('cp_field_address' => 1,'id' => $cp->getId(), 'sub_id' => 'zip_code', 'name' => $cp->getName().' '.lang('zip code'));
            } else {
                $fields[] = array('cp_field' => 1,'id' => $cp->getId(), 'name' => $cp->getName(), 'type' => $cp->getType(), 'values' => $cp->getValues(), 'multiple' => $cp->getIsMultipleValues(), 'is_required' => $cp->getIsRequired()); 
            }
        }

        foreach($memberCustomProperties as $cp){
            if ($cp->getType() == 'table') continue;

            if ($cp->getType() == 'address'){
                $fields[] = array('member_cp_field_address' => 1,'id' => $cp->getId(), 'sub_id' => 'street', 'name' => $cp->getName().' '.lang('street'));
                $fields[] = array('member_cp_field_address' => 1,'id' => $cp->getId(), 'sub_id' => 'city', 'name' => $cp->getName().' '.lang('city'));
                $fields[] = array('member_cp_field_address' => 1,'id' => $cp->getId(), 'sub_id' => 'state', 'name' => $cp->getName().' '.lang('state'));
                $fields[] = array('member_cp_field_address' => 1,'id' => $cp->getId(), 'sub_id' => 'country', 'name' => $cp->getName().' '.lang('country'));
                $fields[] = array('member_cp_field_address' => 1,'id' => $cp->getId(), 'sub_id' => 'zip_code', 'name' => $cp->getName().' '.lang('zip code'));
            } else {
                $fields[] = array('member_cp_field' => 1,'id' => $cp->getId(), 'name' => $cp->getName(), 'type' => $cp->getType(), 'values' => $cp->getValues(), 'multiple' => $cp->getIsMultipleValues(), 'is_required' => $cp->getIsRequired()); 
            }
        }

        $ot = ObjectTypes::instance()->findById($object_type);
        if (class_exists($ot->getHandlerClass())) {
            eval('$managerInstance = ' . $ot->getHandlerClass() . "::instance();");
            $objectColumns = $managerInstance->getColumns();
            $externalFields = $managerInstance->getExternalColumns();
            $calculatedColumns = $managerInstance->getCalculatedColumns();
        } else {
            $objectColumns = array();
            $externalFields = array();
            $calculatedColumns = array();
        }

        $objectFields = array();

        foreach ($calculatedColumns as $calc_col_name) {
            $objectFields[$calc_col_name] = 'calculated';
        }

        if (class_exists($ot->getHandlerClass())) {
            $objectColumns = array_diff($objectColumns, $managerInstance->getSystemColumns());
            foreach($objectColumns as $column){
                if (!in_array($column, $externalFields)) {
                    $objectFields[$column] = $managerInstance->getColumnType($column);
                }
            }


            foreach($externalFields as $extField){
                /*$field_name = Localization::instance()->lang('field '.$ot->getHandlerClass().' '.$extField);
                if (is_null($field_name)) $field_name = lang('field Objects '.$extField);

                $f = array('id' => $extField, 'name' => $field_name, 'type' => 'external', 'multiple' => 0);
*/
                $objectFields[$extField] = $managerInstance->getColumnType($extField);
            }


        }

        $common_columns = Objects::instance()->getColumns(false);

        if (class_exists($ot->getHandlerClass())) {
            $system_columns = $managerInstance->getSystemColumns();
            $common_columns = array_diff_key($common_columns, array_flip($system_columns));
        }
        $objectFields = array_merge($objectFields, $common_columns);
        //allowed properties by type
        $allowed_fields = get_fixed_allowed_columns_by_objet_type_name($ot->getName());

        if ($ot->getName() == 'timeslot'){
            $fields_array_1 = array('id' => 'date', 'name' => lang('date'), 'type' => 'date', 'is_required' => false);
            $fields_array_2 = array('id' => 'total_hours', 'name' => lang('total_hours'), 'type' => 'numeric', 'is_required' => false);
            $fields_array_3 = array('id' => 'task_name', 'name' => lang('task name'), 'type' => 'text', 'is_required' => false);
            $fields[] = $fields_array_1;
            $fields[] = $fields_array_2;
            $fields[] = $fields_array_3;
        }

        if ($ot->getName() == 'payment_receipt'){
            $fields[] = array('id' => 'task_name', 'name' => lang('task name'), 'type' => 'text', 'is_required' => false);
        }

        if ($ot->getName() == 'invoice_payment_receipt'){
            $fields_array_1 = array('id' => 'invoice_number', 'name' => lang('invoice'), 'type' => 'text', 'is_required' => true);
            $fields_array_2 = array('id' => 'amount', 'name' => lang('amount'), 'type' => 'numeric', 'is_required' => true);
            $fields[] = $fields_array_1;
            $fields[] = $fields_array_2;
        }

        if ($ot->getName() == 'invoice'){
            $fields[] = array('id' => 'description', 'name' => lang('invoice line detail'), 'type' => 'text', 'is_required' => false);
            $fields[] = array('id' => 'quantity', 'name' => lang('quantity'), 'type' => 'numeric', 'is_required' => true);
            $fields[] = array('id' => 'unit_value', 'name' => lang('unit_value'), 'type' => 'numeric', 'is_required' => true);
            $fields[] = array('id' => 'invoice_line_total', 'name' => lang('invoice line total'), 'type' => 'numeric', 'is_required' => true);
            $fields[] = array('id' => 'invoice_number', 'name' => lang('invoice number'), 'type' => 'text', 'is_required' => true);
            $fields[] = array('id' => 'invoice_line_date', 'name' => lang('invoice line date'), 'type' => 'date', 'is_required' => false);
            
        }
    
        foreach($objectFields as $name => $type){
            //allowed properties by type
            if(!in_array($name, $allowed_fields)){
                continue;
            }

            if($type == DATA_TYPE_FLOAT || $type == DATA_TYPE_INTEGER){
                $type = 'numeric';
            }else if($type == DATA_TYPE_STRING){
                $type = 'text';
            }else if($type == DATA_TYPE_BOOLEAN){
                $type = 'boolean';
            }else if($type == DATA_TYPE_DATE || $type == DATA_TYPE_DATETIME){
                $type = 'date';
            }

            $is_required = false;
            if($name == 'name'){
                $is_required = true;
            }

            $field_name = Localization::instance()->lang('field '.$ot->getHandlerClass().' '.$name);
            if (is_null($field_name)) $field_name = lang('field Objects '.$name);

            $fields_array = array('id' => $name, 'name' => $field_name, 'type' => $type, 'is_required' => $is_required);

            $fields[] = $fields_array;
        }


        $content_object_dimensions = array();
        if($ot->getType() == 'content_object' || $ot->getType() == 'located'){
            $enabled_dimension_ids = config_option('enabled_dimensions');

            $hide_sel_options = DimensionContentObjectOptions::instance()->findAll(array("conditions" => "`option`='hide_member_selector_in_forms' AND `value`=0 AND content_object_type_id=".$ot->getId()));

            foreach($hide_sel_options as $dim_opt){
                if(in_array($dim_opt->getDimensionId(), $enabled_dimension_ids)){
                    $content_dimension = Dimensions::instance()->findById($dim_opt->getDimensionId());
                    if($content_dimension instanceof Dimension){
                        $object_types = DimensionObjectTypes::getObjectTypeIdsByDimension($content_dimension->getId());
                        foreach($object_types as $ot_id){
                            $object_type = ObjectTypes::instance()->findById($ot_id);
                            $name = count($object_types) == 1 ? $content_dimension->getName() : $content_dimension->getName() . ": " . lang($object_type->getName());
                            $content_object_dim_array = array('dim_and_ot_id' => $dim_opt->getDimensionId() . "_" . $ot_id,
                                                          'name' => $name,
                                                          'content_object_dim' => 1);
                            $content_object_dimensions[] = $content_object_dim_array;
                            
                        }
                    }
                }
            }
        }
                

        // Get all the associated members if it is a dimension object
        $assoc_fields = array();
        if($ot->getType() == 'dimension_object'){
            $all_associations = DimensionMemberAssociations::instance()->findAll(array('conditions' => 'object_type_id='.$ot->getId()));
            $enabled_dimensions = config_option('enabled_dimensions');
            foreach($all_associations as $assoc){
                $assoc_dimension = Dimensions::instance()->findById($assoc->getAssociatedDimensionMemberAssociationId());
                if(in_array($assoc_dimension->getId(), $enabled_dimensions)) {
                    $assoc_field_array = array('assoc_id' => $assoc->getId(),
                                               'name' => $assoc_dimension->getName(),
                                               'assoc_dim' => 1);
                    $assoc_fields[] = $assoc_field_array;
                }
            }
        }

        // Add client fields
        $client_fields = array();
        if($ot->getName() == 'customer' || $ot->getName() == 'supplier'){
            $client_fields = array(
                array('name' => 'work_phone', 'client_field' => true),
                array('name' => 'mobile_phone', 'client_field' => true),
                array('name' => 'fax_phone', 'client_field' => true),
                array('name' => 'email', 'client_field' => true),
                array('name' => 'url', 'client_field' => true),
                array('name' => 'street', 'client_field' => true),
                array('name' => 'city', 'client_field' => true),
                array('name' => 'state', 'client_field' => true),
                array('name' => 'zip_code', 'client_field' => true),
                array('name' => 'country', 'client_field' => true)
            );
             
        }

        $project_template_fields = array();
        if($ot->getName() == 'project') {
            $project_template_fields = array(
                array('name' => 'is_template', 'project_template_field' => true),
                array('name' => 'template_name', 'project_template_field' => true),
                array('name' => 'template_due_date', 'project_template_field' => true),
                array('name' => 'template_start_date', 'project_template_field' => true)
            );
        }

        //Sort fields alphabetically , first required fields
        $required_fields = array();
        $not_required_fields = array();
        foreach($fields as $field){
            if(array_var($field, 'is_required')){
                $required_fields[] = $field;
            }else{
                $not_required_fields[] = $field;
            }
        }
        $nr_and_assoc_fields = array_merge($not_required_fields, $assoc_fields, $client_fields, $content_object_dimensions, $project_template_fields);
        usort($required_fields, function($a, $b) {
            return strnatcmp($a['name'], $b['name']);;
        });
        usort($nr_and_assoc_fields, function($a, $b) {
            return strnatcmp($a['name'], $b['name']);;
        });
        $fields = array_merge($required_fields, $nr_and_assoc_fields);



    }


    return $fields;
}