<?php

Hook::register('overtime_reports');



function overtime_reports_object_definition($type, &$ret) {
	
	if ($type == 'Timeslot') {
		$ret['regular_minutes'] = DATA_TYPE_INTEGER;
		$ret['overtime_minutes'] = DATA_TYPE_INTEGER;
		$ret['double_overtime_minutes'] = DATA_TYPE_INTEGER;
	}
}


// add the new overtime columns as "time format" columns to the reports
function overtime_reports_additional_column_timeslots($params, &$results){

    $ot = array_var($params, 'ot');
    if ($ot && $ot->getName() == 'timeslot') {
        $results[] = 'regular_minutes';
        $results[] = 'overtime_minutes';
        $results[] = 'double_overtime_minutes';
    }

}

// specify how to add the new overtime columns to the totals row in list and reports
function overtime_reports_more_columns_to_aggregate_in_totals($manager, &$columns) {
	if ($manager instanceof Timeslots) {
		$columns['regular_minutes'] = array('operation' => 'sum', 'format' => 'time');
		$columns['overtime_minutes'] = array('operation' => 'sum', 'format' => 'time');
		$columns['double_overtime_minutes'] = array('operation' => 'sum', 'format' => 'time');
	}
}

// get the formatted data of the new columns to show in the time list
function overtime_reports_timeslot_info_additional_data($object, &$info) {
	if ($object instanceof Timeslot) {

		if ($object->getColumnValue('regular_minutes') > 0) {
			$info['regular_minutes'] = DateTimeValue::FormatTimeAmount($object->getColumnValue('regular_minutes') * 60);
		} else {
			$info['regular_minutes'] = '';
		}
		if ($object->getColumnValue('overtime_minutes') > 0) {
			$info['overtime_minutes'] = DateTimeValue::FormatTimeAmount($object->getColumnValue('overtime_minutes') * 60);
		} else {
			$info['overtime_minutes'] = '';
		}
		if ($object->getColumnValue('double_overtime_minutes') > 0) {
			$info['double_overtime_minutes'] = DateTimeValue::FormatTimeAmount($object->getColumnValue('double_overtime_minutes') * 60);
		} else {
			$info['double_overtime_minutes'] = '';
		}
	}
}

// add overtime columns to the time list
function overtime_reports_view_timeslot_render_more_columns($object, &$additional_columns) {
	Env::useHelper('format');
	if (!is_array($additional_columns)) $additional_columns = array();
	$additional_columns['regular_minutes'] = $object->getColumnValue('regular_minutes');
	$additional_columns['overtime_minutes'] = $object->getColumnValue('overtime_minutes');
	$additional_columns['double_overtime_minutes'] = $object->getColumnValue('double_overtime_minutes');

}


function overtime_reports_define_colum_order_list($params, &$cols_data) {
	$order = array_var($params, 'order');
	if (in_array($order, array('regular_minutes', 'overtime_minutes', 'double_overtime_minutes'))) {
		$cols_data = $order;
	}
}


// perform the overtime calculations for a timeslot
function do_calculate_week_next_timeslots_worked_time($object) {
	if ($object instanceof Timeslot && !$object->isOpen()) {
		Env::useHelper('functions', 'overtime_reports');
		calculate_week_next_timeslots_worked_time($object);
	}
}

// listen to several events so we can perform the overtime calculations after save, classify, etc.
function overtime_reports_after_add_to_members($object, &$ret) {
	do_calculate_week_next_timeslots_worked_time($object);
}

function overtime_reports_after_object_save($object, &$ret) {
	do_calculate_week_next_timeslots_worked_time($object);
}


function overtime_reports_after_dragdrop_classify($parameters, &$member) {
	$objects = array_var($parameters, 'objects');
	$sorted_objects = array();
	foreach ($objects as $obj) {
		if ($obj instanceof Timeslot) {
			$sorted_objects[$obj->getStartTime()->toMySQL()] = $obj;
		}
	}
	ksort($sorted_objects);

	foreach ($sorted_objects as $obj) {
		do_calculate_week_next_timeslots_worked_time($obj);
	}
}
// --------------------------