<?php 

function overtime_reports_update_1_2() {

	DB::execute("
	INSERT INTO `".TABLE_PREFIX."objects` (`object_type_id`, `name`, `created_on`, `created_by_id`, `updated_on`, `updated_by_id`, `trashed_by_id`, `archived_by_id`) VALUES
	((SELECT id FROM ".TABLE_PREFIX."object_types WHERE name='report_category'), 'overtime reports', NOW(), 0, NOW(), 0, 0, 0);
	");

	DB::execute("
	INSERT INTO `".TABLE_PREFIX."report_category` (`object_id`,  `description`, `is_default`) VALUES
	((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=(SELECT id FROM ".TABLE_PREFIX."object_types WHERE name='report_category')), '', 1);
    ");

    DB::execute("
	INSERT INTO `".TABLE_PREFIX."objects` (`object_type_id`, `name`, `created_on`, `created_by_id`, `updated_on`, `updated_by_id`, `trashed_by_id`, `archived_by_id`) VALUES
	((SELECT id FROM ".TABLE_PREFIX."object_types WHERE name='report'), 'total overtime hours report', NOW(), 0, NOW(), 0, 0, 0);
	");

	DB::execute("
	INSERT INTO `".TABLE_PREFIX."reports` (`object_id`, `description`, `report_object_type_id`, `is_order_by_asc`, `ignore_context`, `is_default`, `code`, `function_url`, `report_category_id`) VALUES
	((SELECT max(id) FROM ".TABLE_PREFIX."objects WHERE object_type_id=(SELECT id FROM ".TABLE_PREFIX."object_types WHERE name='report')), 'total overtime hours report description', (SELECT id FROM ".TABLE_PREFIX."object_types WHERE name='billing'), 1, 1, 1, 'overtime_hours_report', '?c=overtime_reports&a=total_overtime_hours_parameters',(SELECT max(object_id) FROM ".TABLE_PREFIX."report_category));
    ");
    
}


function overtime_reports_update_2_3() {

	// delete deprecated config option for regular worked time member
	ConfigOptions::instance()->delete("name = 'normal_work_member_id'");

	// hide overtime hours dimension
	$enabled_dimension_ids = config_option('enabled_dimensions');
	$oh_dim = Dimensions::findByCode('overtime_hours');
	if ($oh_dim instanceof Dimension && in_array($oh_dim->getId(), $enabled_dimension_ids)) {
		$idx = array_search($oh_dim->getId(), $enabled_dimension_ids);
		unset($enabled_dimension_ids[$idx]);
		set_config_option('enabled_dimensions', implode(',', array_values($enabled_dimension_ids)));
	}

	// add new cp for users to say if they use overtime or not
	$applies_for_overtime_cp = CustomProperties::getCustomPropertyByCode(Contacts::instance()->getObjectTypeId(), 'applies_for_overtime');
	if (!$applies_for_overtime_cp) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."custom_properties` (`object_type_id`, `name`, `code`, `type`, `description`, `values`, `default_value`, `is_required`, `is_multiple_values`, `property_order`, `visible_by_default`, `is_special`, `is_disabled`, `show_in_lists`, `object_subtype_id`, `total_operation`, `default_currency_id`, `contact_type`) VALUES
			((SELECT id FROM ".TABLE_PREFIX."object_types WHERE name='contact'), 'Applies for overtime', 'applies_for_overtime', 'boolean', 'If set to No, this will override all the rules and none time entry for that user will consider overtime for cost calculations', '', '1', '0', '0', '25', '1', '1', '0', '1', '0', '', '0', 'user');
		");
	}

	// add new columns to timeslots for regular, overtime and double overtime time
	if (!check_column_exists(TABLE_PREFIX . 'timeslots', 'regular_minutes')) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."timeslots`
			ADD `regular_minutes` int(10) unsigned NOT NULL DEFAULT 0;
		");
	}
	if (!check_column_exists(TABLE_PREFIX . 'timeslots', 'overtime_minutes')) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."timeslots`
			ADD `overtime_minutes` int(10) unsigned NOT NULL DEFAULT 0;
		");
	}
	if (!check_column_exists(TABLE_PREFIX . 'timeslots', 'double_overtime_minutes')) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."timeslots`
			ADD `double_overtime_minutes` int(10) unsigned NOT NULL DEFAULT 0;
		");
	}

	// new config options for overtime multipliers
	$option = ConfigOptions::getByName('overtime_multiplier');
	if (!$option instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`,`options`) VALUES
			('overtime_calculations', 'overtime_multiplier', '1.5', 'FloatConfigHandler', 0, 30, '','')
			ON DUPLICATE KEY UPDATE `category_name`=`category_name`;
		");
	}
	$option = ConfigOptions::getByName('double_overtime_multiplier');
	if (!$option instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`,`options`) VALUES
			('overtime_calculations', 'double_overtime_multiplier', '2', 'FloatConfigHandler', 0, 40, '','')
			ON DUPLICATE KEY UPDATE `category_name`=`category_name`;
		");
	}

	// fill all property groups
	if (Plugins::instance()->isActivePlugin('advanced_core')) {
		Env::useHelper('functions', 'advanced_core');
		fill_property_groups_default_data();
	}
}


function overtime_reports_update_3_4() {

	Env::useHelper('functions', 'overtime_reports');
		
	$timeslots_to_recalculate = Timeslots::instance()->findAll(array(
		"conditions" => "regular_minutes > 0 OR overtime_minutes > 0 OR double_overtime_minutes > 0"
	));

	foreach ($timeslots_to_recalculate as $object) {
		calculate_week_next_timeslots_worked_time($object);
	}
}

/** 
 * Adds new config option to set a start date for overtime calculations
 * If there are already some time entries with the calcualtions done then use the first one's start date
 * If not then use today
 */
function overtime_reports_update_4_5() {

	$ts = Timeslots::instance()->findOne(array(
		"conditions" => "regular_minutes > 0",
		"order" => "start_time ASC",
	));

	if ($ts instanceof Timeslot) {
		$date_str = $ts->getStartTime()->toMySQL();
	} else {
		$now = DateTimeValueLib::now();
		$date_str = $now->toMySQL();
	}

	$option = ConfigOptions::getByName('calculate_overtime_from_date');
	if (!$option instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`,`options`) VALUES
			('overtime_calculations', 'calculate_overtime_from_date','".$date_str."', 'DateTimeConfigHandler', 0, 50, '','')
			ON DUPLICATE KEY UPDATE `category_name`=`category_name`;
		");
	}
}
