<?php

class PrintReceiptController extends ApplicationController {

	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'dialog');
		Env::useHelper('constants', 'paemfe');
	}
	
	private function get_receipt_image_url($num) {
		try {
			$logo_url = null;
			$content = FileRepository::getFileContent(config_option("receipt_image_$num"));
			if ($content) {
				$cache_filename = CACHE_DIR . "/receipt_image$num.png";
				file_put_contents($cache_filename, $content);
				$dummy = gen_id();
				$logo_url = ROOT_URL . "/cache/receipt_image$num.png?x=$dummy";
			}
			return $logo_url;
		} catch (FileNotInRepositoryError $e) {
			Logger::log("Receipt image $num not exists in repository");
			return null;
		} catch (Exception $e) {
			return null;
		}
	}
	
	function upload_receipt_image() {
		$num = array_var($_POST, 'image_num');
		$user = logged_user();
		
		if(!owner_company()->canEdit($user)) {
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return;
		} // if

		$image = array_shift($_FILES);
		
		if(!is_array($image)) {

			tpl_assign('image_url1', self::get_receipt_image_url(1));
			tpl_assign('image_url2', self::get_receipt_image_url(2));
			
			$this->setTemplate('upload_receipt_images');			
		} else {
			try {
				if(!isset($image['name']) || !isset($image['type']) || !isset($image['size']) || !isset($image['tmp_name']) || !is_readable($image['tmp_name'])) {
					throw new InvalidUploadError($image, lang('error upload file'));
				} // if

				$valid_types = array('image/jpg', 'image/jpeg', 'image/pjpeg', 'image/gif', 'image/png','image/x-png');

				if(!in_array($image['type'], $valid_types)) {
					throw new InvalidUploadError($image, lang('invalid upload type', 'JPG, GIF, PNG'));
				} // if

				DB::beginWork();

				$old_rep_id = config_option("receipt_image_$num");
				
				$rep_id = FileRepository::addFile($image['tmp_name'], array('public' => true));				
				set_config_option("receipt_image_$num", $rep_id);
				
				if (FileRepository::isInRepository($old_rep_id)) {
					FileRepository::deleteFile($old_rep_id);
				}
				DB::commit();

				flash_success(lang('success edit avatar'));
				ajx_current("back");

			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
			} // try
		} // if
	}
	
	function print_receipt() {
		$this->setLayout("html");
		$id = array_var($_GET, 'id');
		
		$task = ProjectTasks::instance()->findById($id);
		if (!$task instanceof ProjectTask) {
			flash_error(lang('task dnx'));
			return;
		}
		if (!$task->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			return;
		}
		
		$logo_url = self::get_receipt_image_url(1);
		$logo_url2 = self::get_receipt_image_url(2);
				
		tpl_assign('company_logo_url1', $logo_url);
		tpl_assign('company_logo_url2', $logo_url2);
		tpl_assign('task', $task);
		
		$signing_users = array();
		$subscribers = $task->getSubscribers();
		foreach ($subscribers as $user) {
			$signing_users[] = $user->getObjectName();
		}
		tpl_assign('signing_users', $signing_users);
		
		$owner_company = owner_company();
		
		$city_str = "";
		$addresses = ContactAddresses::instance()->getByContact($owner_company);
		if (count($addresses) > 0) $city_str = $addresses[0]->getCity();
		
		tpl_assign('company_city', $city_str);
		
		$company = null;
		$linked_objs = $task->getLinkedObjects();
		foreach ($linked_objs as $lo) {
			if ($lo instanceof Contact && $lo->getIsCompany()) {
				$company = $lo;
				tpl_assign('client', $lo);
				break;
			}
		}
		if ($company instanceof Contact && $company->getIsCompany()) {
			$cpv = CustomPropertyValues::getCustomPropertyValue($company->getId(), DEPENDENCY_CP_ID);
			tpl_assign('dependency', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
		} else {
			tpl_assign('dependency', '');
			tpl_assign('client', '');
		}
		
		foreach ($linked_objs as $lo) {
			if ($lo instanceof Contact && !$lo->getIsCompany()) {
				$contact = $lo;
				tpl_assign('contact', $lo);
				break;
			}
		}
		if (isset($contact) && $contact instanceof Contact && !$contact->getIsCompany()) {
			$cpv = CustomPropertyValues::getCustomPropertyValue($contact->getId(), CONTACT_DOC_ID);
			tpl_assign('contact_doc_id', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
		}
		
		tpl_assign('page_height', config_option('print page height', 1080));
		
		$type_id = $task->getColumnValue('object_subtype_id');
		$object_subtype = ObjectSubtypes::instance()->findById($type_id);
		switch ($object_subtype->getCode()) {
			case 'inst_au': //Instalación de aula:
				self::printReceiptInstAu($task, $company);
				break;
			case 'servicio': //Servicio
				self::printReceiptServ($task, $company);
				break;
			case 'recepcion': //Recepción
				self::printReceiptRec($task);
				break;
			case 'devolucion': //Devolución
				self::printReceiptDev($task);
				break;
			case 'taller': //Taller
				self::printReceiptTaller($task);
				break;
			default:
				ajx_current("empty");
				return;
		}
		
		ajx_current("back");
	}
	
	private function getTable($props, $columns, $min_rows = 0) {
		$props_array = array();
		foreach ($props as $p) {
			$values = str_replace("\|", "%%_PIPE_%%", $p->getValue());
			$exploded = explode("|", $values);
			$values = array();
			foreach ($exploded as $v) {
				$values[] = str_replace("%%_PIPE_%%", "|", $v);
			}
			$element = array();
			$i = 0;
			foreach ($columns as $col => $hard_val) {
				if ($hard_val) {
					$element[$col] = $hard_val;
				} else {
					$element[$col] = clean(array_var($values, $i));
					$i++;
				}				
			}
			$props_array[] = $element;
		}
		while (count($props_array) < $min_rows) $props_array[] = array("&nbsp;");
		return $props_array;
	}
	
	private function printReceiptInstAu($task, $company) {
		if ($company instanceof Contact && $company->getIsCompany()) {
			$cpv = CustomPropertyValues::getCustomPropertyValue($company->getId(), ADSL_IP_CP_ID);
			tpl_assign('adsl_ip', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
			$cpv = CustomPropertyValues::getCustomPropertyValue($company->getId(), ADSL_SERVICE_NUM_CP_ID);
			tpl_assign('adsl_service_num', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
		}
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), SERVICE_NUMBER_CP_ID);
		tpl_assign('formal_apply', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : '');
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), INSTALLATION_DATE_CP_ID);
		tpl_assign('inst_date', $cpv instanceof CustomPropertyValue ? self::buildDateValue($cpv->getValue()) : '');
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), SERVICE_DATE_CP_ID);
		tpl_assign('apply_date', $cpv instanceof CustomPropertyValue ? self::buildDateValue($cpv->getValue()) : '');
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), INSTALLATION_TYPE_CP_ID);
		tpl_assign('installation_type', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : '');
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), FIREWALL_CP_ID);
		tpl_assign('firewall', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : '');
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), LINKS_CP_ID);
		tpl_assign('links', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : '');
		
		$cpv = CustomPropertyValues::getCustomPropertyValues($task->getId(), EXISTING_CLASSROOMS_CP_ID);
		$data = self::getTable($cpv, array('classroom'=>null, 'machines'=>null, 'model'=>null, 'quantity'=>null, 'configuration'=>null, 'obs'=>null), 2);
		tpl_assign('existing_classrooms', $data);
		
		$cpv = CustomPropertyValues::getCustomPropertyValues($task->getId(), SURVEYED_DATA_CP_ID);
		$data = self::getTable($cpv, array('classroom'=>null, 'bars'=>null, 'alarm'=>null, 'desktops'=>null, 'elec_inst'=>null, 'net_inst'=>null, 'enabled'=>null), 2);
		tpl_assign('surveyed_data', $data);
		
		$cpv = CustomPropertyValues::getCustomPropertyValues($task->getId(), DEVICES_TO_INSTALL_CP_ID);
		$data = self::getTable($cpv, array('device'=>null, 'quantity'=>null, 'brand'=>null, 'model'=>null, 'os'=>null), 4);
		tpl_assign('devices_to_install', $data);
		
		$used_materials_1 = array();
		$used_materials_2 = array();
		$cps = CustomProperties::getAllCustomPropertiesByObjectType('ProjectTasks', $task->getObjectSubtype());
		foreach($cps as $k => $cp) {
			if ($cp->getId() < 300) unset($cps[$k]);
		}
		foreach($cps as $cp) {
			$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), $cp->getId());
			$val = $cpv instanceof CustomPropertyValue ? $cpv->getValue() : '';
			if (count($used_materials_1) < count($cps) / 2) {
				$used_materials_1[$cp->getName()] = $val;
			} else $used_materials_2[$cp->getName()] = $val;
		}
		tpl_assign('used_materials_1', $used_materials_1);
		tpl_assign('used_materials_2', $used_materials_2);
		
		$classroom_inventary = self::buildExistingInventaryFromTask($task);
		tpl_assign('classroom_inventary', $classroom_inventary);
		
		$this->setTemplate('receipt_inst_au');
	}
	
	private function printReceiptServ($task, $company) {
		if ($company instanceof Contact && $company->getIsCompany()) {
			$cpv = CustomPropertyValues::getCustomPropertyValue($company->getId(), ADSL_IP_CP_ID);
			tpl_assign('adsl_ip', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
			$cpv = CustomPropertyValues::getCustomPropertyValue($company->getId(), ADSL_SERVICE_NUM_CP_ID);
			tpl_assign('adsl_service_num', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
		}
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), SERVICE_NUMBER_CP_ID);
		tpl_assign('service_num', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), SERVICE_DATE_CP_ID);
		tpl_assign('service_date', $cpv instanceof CustomPropertyValue ? self::buildDateValue($cpv->getValue()) : "");
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), IMPORTANT_DATA_CP_ID);
		tpl_assign('important_data', array($cpv instanceof CustomPropertyValue ? $cpv->getValue() : "&nbsp;", "&nbsp;"));

		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), SERVICE_STARTDATE_CP_ID);
		tpl_assign('service_start_date', $cpv instanceof CustomPropertyValue ? self::buildDateValue($cpv->getValue()) : "");
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), SERVICE_ENDDATE_CP_ID);
		tpl_assign('service_end_date', $cpv instanceof CustomPropertyValue ? self::buildDateValue($cpv->getValue()) : "");

		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), SERVICE_DETAILS_CP_ID);
		tpl_assign('service_details', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
		
		$cpv = CustomPropertyValues::getCustomPropertyValues($task->getId(), IN_MOVEMENTS_CP_ID);
		$in_movements = self::getTable($cpv, array('inout'=>'Entrada', 'description'=>null, 'serial_num'=>null, 'code'=>null), 0);
		$cpv = CustomPropertyValues::getCustomPropertyValues($task->getId(), OUT_MOVEMENTS_CP_ID);
		$out_movements = self::getTable($cpv, array('inout'=>'Salida', 'description'=>null, 'serial_num'=>null, 'code'=>null), 0);
		$movements = array_merge($in_movements, $out_movements);
		if (count($movements) == 0) $movements[] = "&nbsp;";
		tpl_assign('part_movements', $movements);
		
		$cpv = CustomPropertyValues::getCustomPropertyValues($task->getId(), EQ_RETIREMENT_CP_ID);
		$eq_retirement = self::getTable($cpv, array('type'=>null, 'brand'=>null, 'model'=>null, 'serial_num'=>null, 'code'=>null, 'fail'=>null), 4);
		tpl_assign('eq_retirement', $eq_retirement);
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), PENDING_TASKS_CP_ID);
		tpl_assign('pending_tasks', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
		
		$this->setTemplate('receipt_serv');
	}
	
	private function printReceiptTaller($task) {
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), RECEIPT_NOTE_CP_ID);
		tpl_assign('note', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), RECEIPT_COPIES_CP_ID);
		$copies = $cpv instanceof CustomPropertyValue ? explode(",", $cpv->getValue()) : array("ORIGINAL", "COPIA");
		tpl_assign('copies', $copies);
		
		$cpv = CustomPropertyValues::getCustomPropertyValues($task->getId(), WORKSHOP_EQS_CP_ID);
		$data = self::getTable($cpv, array('quantity'=>null, 'description'=>null, 'serial_num'=>null, 'code'=>null, 'fail'=>null));
		tpl_assign('repairs', $data);
		
		$cpv = CustomPropertyValues::getCustomPropertyValues($task->getId(), IN_MOVEMENTS_CP_ID);
		$in_movements = self::getTable($cpv, array('inout'=>'Entrada', 'description'=>null, 'serial_num'=>null, 'code'=>null), 3);
		$cpv = CustomPropertyValues::getCustomPropertyValues($task->getId(), OUT_MOVEMENTS_CP_ID);
		$out_movements = self::getTable($cpv, array('inout'=>'Salida', 'description'=>null, 'serial_num'=>null, 'code'=>null), 3);
		$movements = array_merge($in_movements, $out_movements);
		tpl_assign('part_movements', $movements);
		
		if ($task->getAssignedTo() instanceof User) {
			$cpv = CustomPropertyValues::getCustomPropertyValue($task->getAssignedTo()->getId(), USER_DOC_ID_CP_ID);
			tpl_assign('user_doc_id', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
		}
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), WORKSHOP_REV_PROCESS_CP_ID);
		tpl_assign('rev_date', $cpv instanceof CustomPropertyValue ? self::buildDateValue($cpv->getValue()) : "");
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), WORKSHOP_REP_PROCESS_CP_ID);
		tpl_assign('rep_date', $cpv instanceof CustomPropertyValue ? self::buildDateValue($cpv->getValue()) : "");
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), WORKSHOP_IN_WARRANTY_CP_ID);
		tpl_assign('in_warr_date', $cpv instanceof CustomPropertyValue ? self::buildDateValue($cpv->getValue()) : "");
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), WORKSHOP_OUT_WARRANTY_CP_ID);
		tpl_assign('out_warr_date', $cpv instanceof CustomPropertyValue ? self::buildDateValue($cpv->getValue()) : "");
		
		$cpv = CustomPropertyValues::getCustomPropertyValues($task->getId(), WORKSHOP_COMP_WARRANTY_CP_ID);
		$data = self::getTable($cpv, array('name'=>null, 'loc'=>null));
		if (count($data) > 0) $data = array_shift($data);
		tpl_assign('warr_comp', $data);
		
		$this->setTemplate('receipt_ta');
	}
	
	private function printReceiptRec(ProjectTask $task) {
		
		$cpv = CustomPropertyValues::getCustomPropertyValues($task->getId(), RECEPTION_ING_BY_CP_ID);
		$data = self::getTable($cpv, array('name'=>null, 'ci'=>null));
		tpl_assign('ing_by_data', $data);
		
		if ($task->getAssignedTo() instanceof User) {
			$cpv = CustomPropertyValues::getCustomPropertyValue($task->getAssignedToContactId(), USER_DOC_ID_CP_ID);
			tpl_assign('user_doc_id', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
		}
		
		$cpv = CustomPropertyValues::getCustomPropertyValues($task->getId(), DEVOLUTION_EQS_CP_ID);
		$data = self::getTable($cpv, array('description'=>null, 'serial_num'=>null, 'code'=>null, 'fail'=>null));
		tpl_assign('receptions', $data);
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), DEVOLUTION_BY_CP_ID);
		tpl_assign('rec_by', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), DEVOLUTION_ARCH_NUM_CP_ID);
		tpl_assign('arch_num', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), RECEIPT_NOTE_CP_ID);
		tpl_assign('note', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), RECEIPT_COPIES_CP_ID);
		$copies = $cpv instanceof CustomPropertyValue ? explode(",", $cpv->getValue()) : array("ORIGINAL", "COPIA");
		tpl_assign('copies', $copies);
		
		$this->setTemplate('receipt_rec');
	}
	
	private function printReceiptDev($task) {
		if ($task->getAssignedTo() instanceof User) {
			$cpv = CustomPropertyValues::getCustomPropertyValue($task->getAssignedTo()->getId(), USER_DOC_ID_CP_ID);
			tpl_assign('user_doc_id', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
		}
		
		$cpv = CustomPropertyValues::getCustomPropertyValues($task->getId(), DEVOLUTION_EQS_CP_ID);
		$data = self::getTable($cpv, array('description'=>null, 'serial_num'=>null, 'code'=>null, 'fail'=>null));
		tpl_assign('devolutions', $data);
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), DEVOLUTION_BY_CP_ID);
		tpl_assign('dev_by', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), DEVOLUTION_ARCH_NUM_CP_ID);
		tpl_assign('arch_num', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), DEVOLUTION_NOTE_CP_ID);
		tpl_assign('note', $cpv instanceof CustomPropertyValue ? $cpv->getValue() : "");
		
		$cpv = CustomPropertyValues::getCustomPropertyValue($task->getId(), RECEIPT_COPIES_CP_ID);
		$copies = $cpv instanceof CustomPropertyValue ? explode(",", $cpv->getValue()) : array("ORIGINAL", "COPIA");
		tpl_assign('copies', $copies);
		
		$this->setTemplate('receipt_dev');
	}
	
	
	private function buildExistingInventary($company) {
		$classroom_inventary = array();
		self::fillExistingInventaryData($company, INSTALLED_PCS_CP_ID, $classroom_inventary);
		self::fillExistingInventaryData($company, INSTALLED_PRINTERS_CP_ID, $classroom_inventary);
		self::fillExistingInventaryData($company, INSTALLED_SWITCHES_CP_ID, $classroom_inventary);
		self::fillExistingInventaryData($company, INSTALLED_MOUSES_CP_ID, $classroom_inventary);
		self::fillExistingInventaryData($company, INSTALLED_WIRES_CP_ID, $classroom_inventary);
		return $classroom_inventary;
	}
	
	private function fillExistingInventaryData($company, $cp_id, &$classroom_inventary) {
		if ($company instanceof Contact && $company->getIsCompany()) {
			$cpv = CustomPropertyValues::getCustomPropertyValues($company->getId(), $cp_id);
			
			switch ($cp_id) {
				case INSTALLED_PCS_CP_ID: $key = 'pcs';
					$installed_items = self::getTable($cpv, array('classroom'=>null, 'terminal'=>null, 'brand'=>null, 'model'=>null, 'serial_cpu'=>null, 'serial_mon'=>null, 'serial_key'=>null));
					break;
				case INSTALLED_PRINTERS_CP_ID: $key = 'printers';
					$installed_items = self::getTable($cpv, array('classroom'=>null, 'brand'=>null, 'model'=>null, 'serial_num'=>null));
					break;
				case INSTALLED_MOUSES_CP_ID: $key = 'mouses';
					$installed_items = self::getTable($cpv, array('classroom'=>null, 'brand'=>null, 'model'=>null, 'quantity'=>null));
					break;
				case INSTALLED_SWITCHES_CP_ID: $key = 'switches';
					$installed_items = self::getTable($cpv, array('classroom'=>null, 'brand'=>null, 'model'=>null, 'serial_num'=>null));
					break;
				case INSTALLED_WIRES_CP_ID: $key = 'wires';
					$installed_items = self::getTable($cpv, array('classroom'=>null, 'type'=>null, 'model'=>null, 'quantity'=>null));
					break;
				default: return;
			}
			
			foreach ($installed_items as $item) {
				$id = $item['classroom'];
				if (!isset($classroom_inventary[$id])) {
					$classroom_inventary[$id] = array('pcs' => array(), 'printers' => array(), 'switches' => array(), 'mouses' => array(), 'wires' => array());
				}
				$classroom_inventary[$id][$key][] = $item; 
			}
		}
	}
	
	
	private function buildExistingInventaryFromTask($task) {
		$classroom_inventary = array();
		self::fillExistingInventaryDataFromTask($task, TASKS_INSTALLED_PCS_CP_ID, $classroom_inventary);
		self::fillExistingInventaryDataFromTask($task, TASKS_INSTALLED_PRINTERS_CP_ID, $classroom_inventary);
		self::fillExistingInventaryDataFromTask($task, TASKS_INSTALLED_SWITCHES_CP_ID, $classroom_inventary);
		self::fillExistingInventaryDataFromTask($task, TASKS_INSTALLED_MOUSES_CP_ID, $classroom_inventary);
		self::fillExistingInventaryDataFromTask($task, TASKS_INSTALLED_WIRES_CP_ID, $classroom_inventary);
		return $classroom_inventary;
	}
	
	private function fillExistingInventaryDataFromTask($task, $cp_id, &$classroom_inventary) {
		if ($task instanceof ProjectTask) {
			$cpv = CustomPropertyValues::getCustomPropertyValues($task->getId(), $cp_id);
			
			switch ($cp_id) {
				case TASKS_INSTALLED_PCS_CP_ID: $key = 'pcs';
					$installed_items = self::getTable($cpv, array('classroom'=>null, 'terminal'=>null, 'brand'=>null, 'model'=>null, 'serial_cpu'=>null, 'serial_mon'=>null, 'serial_key'=>null));
					break;
				case TASKS_INSTALLED_PRINTERS_CP_ID: $key = 'printers';
					$installed_items = self::getTable($cpv, array('classroom'=>null, 'brand'=>null, 'model'=>null, 'serial_num'=>null));
					break;
				case TASKS_INSTALLED_MOUSES_CP_ID: $key = 'mouses';
					$installed_items = self::getTable($cpv, array('classroom'=>null, 'brand'=>null, 'model'=>null, 'quantity'=>null));
					break;
				case TASKS_INSTALLED_SWITCHES_CP_ID: $key = 'switches';
					$installed_items = self::getTable($cpv, array('classroom'=>null, 'brand'=>null, 'model'=>null, 'serial_num'=>null));
					break;
				case TASKS_INSTALLED_WIRES_CP_ID: $key = 'wires';
					$installed_items = self::getTable($cpv, array('classroom'=>null, 'type'=>null, 'model'=>null, 'quantity'=>null));
					break;
				default: return;
			}
			
			foreach ($installed_items as $item) {
				$id = $item['classroom'];
				if (!isset($classroom_inventary[$id])) {
					$classroom_inventary[$id] = array('pcs' => array(), 'printers' => array(), 'switches' => array(), 'mouses' => array(), 'wires' => array());
				}
				$classroom_inventary[$id][$key][] = $item; 
			}
		}
	}
	
	
	private function buildDateValue($str) {
		$date = DateTimeValueLib::dateFromFormatAndString("Y-m-d H:i:s", $str);
		$date = $date->add('s', -1 * logged_user()->getUserTimezoneValue());
		return format_date($date);
	}
	
	
}

/*
 * SQL PARA ACTUALIZACION
 * 
INSERT INTO og_custom_properties (id, object_type, name, `type`, description, `values`, default_value, is_required, is_multiple_values, property_order, visible_by_default)
VALUES (84, 'ProjectTasks', 'Descripción', 'list', '', 'Valor 1, Valor 2, Valor 3', '', 1, 0, 0, 0);
INSERT INTO og_custom_properties_by_co_type (co_type_id, cp_id) VALUES (1,84),(2,84);
ALTER TABLE `og_custom_property_values`
 ADD INDEX `object_cp`(`object_id`, `custom_property_id`),
 ADD INDEX `cp_object`(`custom_property_id`, `object_id`);
ALTER TABLE `og_linked_objects`
 ADD INDEX `created_on`(`created_on`),
 ADD INDEX `related_object`(`object_manager`, `object_id`);

UPDATE og_custom_properties SET `values` = concat(`values`,',Fecha') WHERE id IN (61,62,63,64,65);
UPDATE og_custom_property_values SET `value` = concat(`value`,'|') WHERE custom_property_id IN (61,62,63,64,65);

INSERT INTO `og_custom_properties` (id, object_type, name, `type`, description, `values`, default_value, property_order, visible_by_default, is_required, is_multiple_values)
 SELECT 85, 'ProjectTasks', cp.name, cp.`type`, cp.description, cp.`values`, cp.default_value, (SELECT max(cp2.property_order)+1 FROM og_custom_properties cp2 WHERE cp2.object_type='ProjectTasks'), cp.visible_by_default, cp.is_required, cp.is_multiple_values
 FROM og_custom_properties cp WHERE id = 61;
INSERT INTO `og_custom_properties` (id, object_type, name, `type`, description, `values`, default_value, property_order, visible_by_default, is_required, is_multiple_values)
 SELECT 86, 'ProjectTasks', cp.name, cp.`type`, cp.description, cp.`values`, cp.default_value, (SELECT max(cp2.property_order)+1 FROM og_custom_properties cp2 WHERE cp2.object_type='ProjectTasks'), cp.visible_by_default, cp.is_required, cp.is_multiple_values
 FROM og_custom_properties cp WHERE id = 62;
INSERT INTO `og_custom_properties` (id, object_type, name, `type`, description, `values`, default_value, property_order, visible_by_default, is_required, is_multiple_values)
 SELECT 87, 'ProjectTasks', cp.name, cp.`type`, cp.description, cp.`values`, cp.default_value, (SELECT max(cp2.property_order)+1 FROM og_custom_properties cp2 WHERE cp2.object_type='ProjectTasks'), cp.visible_by_default, cp.is_required, cp.is_multiple_values
 FROM og_custom_properties cp WHERE id = 63;
INSERT INTO `og_custom_properties` (id, object_type, name, `type`, description, `values`, default_value, property_order, visible_by_default, is_required, is_multiple_values)
 SELECT 88, 'ProjectTasks', cp.name, cp.`type`, cp.description, cp.`values`, cp.default_value, (SELECT max(cp2.property_order)+1 FROM og_custom_properties cp2 WHERE cp2.object_type='ProjectTasks'), cp.visible_by_default, cp.is_required, cp.is_multiple_values
 FROM og_custom_properties cp WHERE id = 64;
INSERT INTO `og_custom_properties` (id, object_type, name, `type`, description, `values`, default_value, property_order, visible_by_default, is_required, is_multiple_values)
 SELECT 89, 'ProjectTasks', cp.name, cp.`type`, cp.description, cp.`values`, cp.default_value, (SELECT max(cp2.property_order)+1 FROM og_custom_properties cp2 WHERE cp2.object_type='ProjectTasks'), cp.visible_by_default, cp.is_required, cp.is_multiple_values
 FROM og_custom_properties cp WHERE id = 65;

INSERT INTO og_custom_properties_by_co_type (cp_id, co_type_id)
 SELECT cp.id, t.id FROM og_custom_properties cp JOIN og_project_co_types t WHERE t.object_manager='ProjectTasks' AND cp.id IN (85,86,87,88,89);
*/
?>