<?php

function build_excel_actual_vs_forecast_report($data, $report_data) {
    $revenue_types = array('forecasts', 'actuals');
    $use_proj_type = array_var($report_data, 'group_by') == 'project_type';

    $excel = array();

    foreach($revenue_types as $revenue_type) {
        $excel[] = build_excel_revenue_type_name_actual_vs_forecast_report($data, $revenue_type, $report_data);
        $excel[] = build_excel_main_header_actual_vs_forecast_report($data, $report_data);
        $excel[] = build_excel_overall_totals_actual_vs_forecast_report($data, $report_data, $revenue_type);
        if($use_proj_type) {
            foreach($data['project_types'] as $proj_type_data) {
                $excel[] = build_excel_project_type_header_actual_vs_forecast_report($proj_type_data, $data, $revenue_type);
                $excel[] = build_excel_projects_for_actual_vs_forecast_report($proj_type_data['projects'], $data, $revenue_type);
            }
        } else {
            $excel[] = build_excel_projects_for_actual_vs_forecast_report($data['projects'], $data, $revenue_type);
        }
        if($revenue_type == 'forecasts') {
            $excel[] = array('main_header' => array(''));
            $excel[] = array('main_header' => array(''));
        }
    }

    return $excel;
}

function build_excel_revenue_type_name_actual_vs_forecast_report($data, $revenue_type, $report_data) {
    $financial_type = array_var($report_data, 'financial_type', 'revenue');
    $revenue_type_name = strtoupper(lang($revenue_type . ' ' . $financial_type));

    $excel_table = array(
		'main_header' => array($revenue_type_name)
    );
   
    return $excel_table;
}

function build_excel_project_type_header_actual_vs_forecast_report($proj_type_data, $data, $revenue_type) {
    $columns = array_var($data, 'months_between_dates');
    $columns[] = 'total';
    $totals = $proj_type_data[$revenue_type];
    $currency_sym = config_option('currency_code');
    $pm_name = $proj_type_data['type_name'];

    $excel_table = array(
        'totals' => array($pm_name)
    );

    foreach($columns as $column) {
        $money_val = format_money_amount($totals[$column], $currency_sym);
        $excel_table['totals'][] =  $money_val;
    }

    return $excel_table;
}

function build_excel_projects_for_actual_vs_forecast_report($projects, $data, $revenue_type) {
    $columns = array_var($data, 'months_between_dates');
    $columns[] = 'total';
    $currency_sym = config_option('currency_code');

    $excel_table = array(
		'data' => array()
    );

    foreach($projects as $project) {
        $project_row = array();
        $totals = $project[$revenue_type];
        $project_row[] = $project['name'];
        foreach($columns as $column) {
            $money_val = format_money_amount($totals[$column], $currency_sym);
            $project_row[] = $money_val;
        }
        $excel_table['data'][] = $project_row;
    }

    return $excel_table;
}

function build_excel_main_header_actual_vs_forecast_report($data, $report_data) {
    $columns = array_var($data, 'months_between_dates');
    
    $excel_table = array(
		'main_header' => array('')
    );

    foreach($columns as $column) {
        $column_arr = explode('-', $column);
        $year = $column_arr[0];
        $month = $column_arr[1];
        $month_str = ucfirst(lang('month '. intval($month)));
        $month_abbr = substr($month_str, 0, 3);

        $excel_table['main_header'][] = $month_abbr . ' ' . $year;
    }
    $excel_table['main_header'][] = lang('totals');

    return $excel_table;
}

function build_excel_overall_totals_actual_vs_forecast_report($data, $report_data, $revenue_type) {
    $columns = array_var($data, 'months_between_dates');
    $columns[] = 'total';
    $totals = array_var($data, $revenue_type, array());

    $currency_sym = config_option('currency_code');

    $excel_table = array(
		'main_header' => array('')
    );

    foreach($columns as $column) {
        $money_val = format_money_amount($totals[$column], $currency_sym);
        $excel_table['main_header'][] = $money_val;
    }

    return $excel_table;
}
