<?php

function get_main_actual_vs_forecast_report_data($report_data) {

    $filtered_project_member_ids = get_actual_vs_forecast_project_member_ids_filtered_by_date($report_data);

    $result_data = array(
        'forecasts' => array(),
        'actuals' => array(),
    );

    get_project_type_dimension_for_actual_vs_forecast_report($report_data, $result_data);

    if(array_var($report_data, 'group_by') == 'project_type') {
        $result_data['project_types'] = array();
    } else {
        $result_data['projects'] = array();
    }

    getMonthsBetweenDates($report_data, $result_data);
    
    foreach($filtered_project_member_ids as $project_id) {
        $project_data = collect_project_info_for_actual_vs_forecast_report($report_data, $project_id);
        if(empty($project_data) || (array_var($project_data, 'total_forecast') == 0 && array_var($project_data, 'total_actual') == 0)) {
            continue;
        }
        if(array_var($report_data, 'group_by') == 'project_type') {
            $type_id = $project_data['type_id'];
            $type_name = $project_data['type_name'];
            if(!isset($result_data['project_types'][$type_id])) {
                $result_data['project_types'][$type_id] = array(
                    'type_name' => $type_name,
                    'forecasts' => 0,
                    'actuals' => 0,
                    'projects' => array()
                );
            }
            $result_data['project_types'][$type_id]['projects'][$project_id] = $project_data;
        } else {
            $result_data['projects'][$project_id] = $project_data;
        }
    }

    $totals_by_type = 'projects';
    if(array_var($report_data, 'group_by') == 'project_type') {
        $totals_by_type = 'project_types';
        foreach($result_data['project_types'] as $type_id => $type_data) {
            // Calculate totals
            $result_data['project_types'][$type_id]['forecasts'] = calculate_totals_for_actual_vs_forecast_report($report_data, $type_data['projects'], 'forecasts');
            $result_data['project_types'][$type_id]['actuals'] = calculate_totals_for_actual_vs_forecast_report($report_data,$type_data['projects'], 'actuals');
            // Sort projects by name
            uasort($result_data['project_types'][$type_id]['projects'], function($a, $b) {
                return strcmp($a['name'], $b['name']);
            });
        }
        // Sort project types by name
        uasort($result_data['project_types'], function($a, $b) {
            return strcmp($a['type_name'], $b['type_name']);
        });
    } else {
        // Sort projects by name
        uasort($result_data['projects'], function($a, $b) {
            return strcmp($a['name'], $b['name']);
        });
    }
    $result_data['forecasts'] = calculate_totals_for_actual_vs_forecast_report($report_data, $result_data[$totals_by_type], 'forecasts');
    $result_data['actuals'] = calculate_totals_for_actual_vs_forecast_report($report_data, $result_data[$totals_by_type], 'actuals');
    

    return $result_data;
}

function get_project_type_dimension_for_actual_vs_forecast_report(&$report_data, &$result_data) {
    $enabled_dimensions = config_option('enabled_dimensions');
    $project_type_dimension = Dimensions::instance()->findByCode('project_types');
    $project_type_dim_id = $project_type_dimension instanceof Dimension ? $project_type_dimension->getId() : 0;
    $proj_proj_type_association = DimensionMemberAssociations::instance()->getAssociationByCode('project_project_type');
    if ($project_type_dimension instanceof Dimension && $proj_proj_type_association instanceof DimensionMemberAssociation && in_array($project_type_dim_id, $enabled_dimensions)) {
        $report_data['project_type_dim_id'] = $project_type_dim_id;
        $report_data['active_proj_type'] = true;
        $result_data['project_type_dim_id'] = $project_type_dim_id;
        $result_data['active_proj_type'] = true;
    }
}


function calculate_totals_for_actual_vs_forecast_report($report_data, $projects, $revenue_type = 'actuals') {
    $columns = array_var($report_data, 'months_between_dates');
    $columns[] = 'total';
    $totals = array_fill_keys($columns, 0);
    foreach($projects as $project) {
        $fin_values = array_var($project, $revenue_type, array());
        foreach($columns as $column) {
            $totals[$column] += array_var($fin_values, $column, 0);
        }
    }

    return $totals;
}

function get_actual_vs_forecast_project_member_ids_filtered_by_date($report_data){
    Env::useHelper('report_helper_functions', 'project_reports');
	$project_ot = ObjectTypes::findByName('project');

	// Get start and end date SQLs
	$start_date = array_var($report_data, 'start_date');
	$end_date = array_var($report_data, 'end_date');
	$sql_st_date = $start_date->toMySQL();
	$sql_end_date = $end_date->toMySQL();

	// Get project member ids assigned to timeslots in the given date range
	$time_sql = "SELECT DISTINCT(m.id)
			FROM ".TABLE_PREFIX."timeslots t INNER JOIN ".TABLE_PREFIX."object_members om ON t.object_id=om.object_id
            INNER JOIN ".TABLE_PREFIX."objects o ON t.object_id=o.id
			INNER JOIN ".TABLE_PREFIX."members m ON om.member_id=m.id
			WHERE o.trashed_by_id=0
            AND t.start_time BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
			AND m.object_type_id=".$project_ot->getId().";";
	$time_rows = DB::executeAll($time_sql);
	$time_project_member_ids = !empty($time_rows) ? array_column($time_rows, 'id') : array();

    // Get project member ids assigned to tasks in the given date range 
    $task_sql = "SELECT DISTINCT(m.id)
    FROM ".TABLE_PREFIX."project_tasks t INNER JOIN ".TABLE_PREFIX."object_members om ON t.object_id=om.object_id
    INNER JOIN ".TABLE_PREFIX."objects o ON t.object_id=o.id
    INNER JOIN ".TABLE_PREFIX."members m ON om.member_id=m.id
    WHERE o.trashed_by_id=0
    AND (t.due_date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."' OR t.completed_on BETWEEN '".$sql_st_date."' AND '".$sql_end_date."')
    AND m.object_type_id=".$project_ot->getId().";";
    $task_rows = DB::executeAll($task_sql);
    $task_project_member_ids = !empty($task_rows) ? array_column($task_rows, 'id') : array();

    prepare_dates_for_expenses_sql($start_date, $end_date, $sql_st_date, $sql_end_date);

	// Get project member ids assigned to actual expenses in the given date range									
	$expense_sql = "SELECT DISTINCT(m.id)
				FROM ".TABLE_PREFIX."payment_receipts pr INNER JOIN ".TABLE_PREFIX."object_members om ON pr.object_id=om.object_id
                INNER JOIN ".TABLE_PREFIX."objects o ON pr.object_id=o.id
				INNER JOIN ".TABLE_PREFIX."members m ON om.member_id=m.id
				WHERE o.trashed_by_id=0
                AND pr.date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
				AND m.object_type_id=".$project_ot->getId().";";
	$expense_rows = DB::executeAll($expense_sql);
    $expense_project_member_ids = !empty($expense_rows) ? array_column($expense_rows, 'id') : array();

    // Get project member ids assigned to expenses in the given date range
    $budget_expense_sql = "SELECT DISTINCT(m.id)
                FROM ".TABLE_PREFIX."expenses pr INNER JOIN ".TABLE_PREFIX."object_members om ON pr.object_id=om.object_id
                INNER JOIN ".TABLE_PREFIX."expense_items ei on ei.expense_id=pr.object_id
                INNER JOIN ".TABLE_PREFIX."objects o ON pr.object_id=o.id
                INNER JOIN ".TABLE_PREFIX."members m ON om.member_id=m.id
                WHERE o.trashed_by_id=0
                AND ei.date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
                AND m.object_type_id=".$project_ot->getId().";"; 
    $budget_rows = DB::executeAll($budget_expense_sql);
    $budget_expense_project_member_ids = !empty($budget_rows) ? array_column($budget_rows, 'id') : array(); 
    
    revert_dates_after_expenses_sql($start_date, $end_date);

	// Merge time and expense project member ids										
	$combined_project_member_ids = array_unique(array_merge($time_project_member_ids, $expense_project_member_ids, $task_project_member_ids, $budget_expense_project_member_ids));

	return $combined_project_member_ids;
}


function collect_project_info_for_actual_vs_forecast_report($report_data, $project_id) {
    $project_member = Members::instance()->findOne(array('conditions' => '`id`='.$project_id));
    if(!$project_member instanceof Member || $project_member->getDepth() > 1) {
        return array(); 
    }
   
    $project_data = array(
        'name' => $project_member->getDisplayName(),
        'forecasts' => array(),
        'actuals' => array(),
        'total_forecast' => 0,
        'total_actual' => 0
    );

    get_project_type_assigned_to_project($report_data, $project_member, $project_data);

    // Get forecasted and actual values for this project
    get_forecasted_values_for_project_for_actual_vs_forecast_report($report_data, $project_member, $project_data);
    get_actual_values_for_project_for_actual_vs_forecast_report($report_data, $project_member, $project_data);
    

    return $project_data;
}

function get_project_type_assigned_to_project($report_data, $project_member, &$project_data) {

    if(!array_var($report_data, 'group_by') == 'project_type') return;

    $project_type_dim = Dimensions::instance()->findByCode('project_types');
    $project_type_id = get_associated_status_member_id($project_member, $project_type_dim, null, false, 'project_project_type');
    $projec_type_member = Members::instance()->findOne(array('conditions' => '`id`='.$project_type_id));
    if ($projec_type_member instanceof Member) {
        $project_data['type_id'] = $project_type_id;
        $project_data['type_name'] = $projec_type_member->getDisplayName();
    } else {
        $project_data['type_id'] = 0;
        $project_data['type_name'] = lang('without for actual_vs_forecast') . ' ' . $project_type_dim->getName();
    }
}

function get_forecasted_values_for_project_for_actual_vs_forecast_report($report_data, $project_member, &$project_data) {
    Env::useHelper('report_helper_functions', 'project_reports');
    $financial_type = array_var($report_data, 'financial_type', 'revenue');

    $start_date = array_var($report_data, 'start_date');
    $end_date = array_var($report_data, 'end_date');
    $sql_st_date = $start_date->toMySQL();
    $sql_end_date = $end_date->toMySQL();

    $forecasted_revenue = 0;

    // Get estimated revenue from tasks grouped by month and year for this project
    $value_total_column = $financial_type == 'revenue' ? 'estimated_price' : 'estimated_cost';
    $task_row = DB::execute("
        SELECT SUM(t.".$value_total_column.") as total_revenue, LPAD(MONTH(t.due_date),2,'0') as month, YEAR(t.due_date) as year
        FROM ".TABLE_PREFIX."project_tasks t
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=t.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        AND (t.due_date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."')
        AND om.member_id=".$project_member->getId()."
        AND t.depth=0
        GROUP BY MONTH(t.due_date), YEAR(t.due_date);");
    $task_rows = $task_row->fetchAll();

    if($task_rows) {
        foreach($task_rows as $task_row) {
            $forecasted_revenue += $task_row['total_revenue'];
            add_row_to_project_revenue($project_data, $task_row, 'forecasts');
        }
    }

    prepare_dates_for_expenses_sql($start_date, $end_date, $sql_st_date, $sql_end_date);

    // Get estimated revenue from expenses grouped by month and year for this project
    $exp_fin_value_column = $financial_type == 'revenue' ? 'total_price' : 'total_cost';
    $expense_row = DB::execute("
        SELECT SUM(ei.".$exp_fin_value_column.") as total_revenue, LPAD(MONTH(ei.date),2,'0') as month, YEAR(ei.date) as year
        FROM ".TABLE_PREFIX."expenses e
        INNER JOIN ".TABLE_PREFIX."expense_items ei on ei.expense_id=e.object_id
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=e.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        AND (ei.date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."')
        AND om.member_id=".$project_member->getId()."
        AND e.task_id=0
        GROUP BY MONTH(ei.date), YEAR(ei.date);");  
    $expense_rows = $expense_row->fetchAll();

    if($expense_rows) {
        foreach($expense_rows as $expense_row) {
            $forecasted_revenue += $expense_row['total_revenue'];
            add_row_to_project_revenue($project_data, $expense_row, 'forecasts');
        }
    }

    revert_dates_after_expenses_sql($start_date, $end_date);

    // Set totals
    $project_data['forecasts']['total'] = $forecasted_revenue;
    $project_data['total_forecast'] = $forecasted_revenue;
}

function get_actual_values_for_project_for_actual_vs_forecast_report($report_data, $project_member, &$project_data) {
    Env::useHelper('report_helper_functions', 'project_reports');
    $financial_type = array_var($report_data, 'financial_type', 'revenue');

    $start_date = array_var($report_data, 'start_date');
    $end_date = array_var($report_data, 'end_date');
    $sql_st_date = $start_date->toMySQL();
    $sql_end_date = $end_date->toMySQL();

    $actual_revenue = 0;

    // Get all task ids and it's subtasks with manual estimated price and fixed fee
    $fixed_fee_task_ids = array();
    if(Plugins::instance()->isActivePlugin('advanced_billing') && $financial_type == 'revenue') {
            Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
            $fixed_fee_task_ids = get_tasks_and_subtask_ids_with_manual_financial_value($project_member->getId(), 'is_calculated_estimated_price', true);
    } 

    // Get fixed fee task ids for this project grouped by month and year
    if(!empty($fixed_fee_task_ids)) {
        $fixed_fee_cond = 'AND t.object_id IN ('.implode(',', $fixed_fee_task_ids).')';
        $date_column = "t.due_date";
        $fixed_fee_task_row = DB::execute("
            SELECT SUM(t.earned_value) as total_revenue, LPAD(MONTH(".$date_column."),2,'0') as month, YEAR(t.due_date) as year
            FROM ".TABLE_PREFIX."project_tasks t
            INNER JOIN ".TABLE_PREFIX."objects o on o.id=t.object_id
            INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
            WHERE o.trashed_by_id=0
            ".$fixed_fee_cond."
            AND t.depth=0 
            AND ".$date_column." BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
            AND om.member_id=".$project_member->getId()."
            GROUP BY MONTH(".$date_column."), YEAR(".$date_column.");");
        $fixed_fee_task_rows = $fixed_fee_task_row->fetchAll();

        if($project_member->getId() == 560) {
            Logger::log_r($fixed_fee_task_rows);
        }

        if($fixed_fee_task_rows) {
            foreach($fixed_fee_task_rows as $fixed_fee_task_row) {
                $actual_revenue += $fixed_fee_task_row['total_revenue'];
                add_row_to_project_revenue($project_data, $fixed_fee_task_row, 'actuals');
            }
        }
    }

    // Get timeslots for this project grouped by month and year
    $time_fixed_fee_cond = !empty($fixed_fee_task_ids) ? 'AND ts.rel_object_id NOT IN ('.implode(',', $fixed_fee_task_ids).')' : '';
    $time_fin_value_column = $financial_type == 'revenue' ? 'fixed_billing' : 'fixed_cost';
    $time_row = DB::execute("
        SELECT SUM(ts.".$time_fin_value_column.") as total_revenue, LPAD(MONTH(ts.start_time),2,'0') as month, YEAR(ts.start_time) as year
        FROM ".TABLE_PREFIX."timeslots ts
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=ts.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        AND ts.start_time BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
        AND om.member_id=".$project_member->getId()."
        ".$time_fixed_fee_cond."
        GROUP BY MONTH(ts.start_time), YEAR(ts.start_time);");
    $time_rows = $time_row->fetchAll();

    if($time_rows) {
        foreach($time_rows as $time_row) {
            $actual_revenue += $time_row['total_revenue'];
            add_row_to_project_revenue($project_data, $time_row, 'actuals');
        }
    }

    prepare_dates_for_expenses_sql($start_date, $end_date, $sql_st_date, $sql_end_date);

    // Get actual expenses(payment_receipts) for this project grouped by month and year
    $exp_fixed_fee_cond = !empty($fixed_fee_task_ids) ? 'AND pr.task_id NOT IN ('.implode(',', $fixed_fee_task_ids).')' : '';
    $exp_fin_value_column = $financial_type == 'revenue' ? 'total_price' : 'amount';
    $expense_row = DB::execute("
        SELECT SUM(pr.".$exp_fin_value_column.") as total_revenue, LPAD(MONTH(pr.date),2,'0') as month, YEAR(pr.date) as year
        FROM ".TABLE_PREFIX."payment_receipts pr
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=pr.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        AND pr.date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
        AND om.member_id=".$project_member->getId()."
        ".$exp_fixed_fee_cond."
        GROUP BY MONTH(pr.date), YEAR(pr.date);");
    $expense_rows = $expense_row->fetchAll();

    if($expense_rows) {
        foreach($expense_rows as $expense_row) {
            $actual_revenue += $expense_row['total_revenue'];
            add_row_to_project_revenue($project_data, $expense_row, 'actuals');
        }
    }

    revert_dates_after_expenses_sql($start_date, $end_date);

    // Set totals
    $project_data['actuals']['total'] = $actual_revenue;
    $project_data['total_actual'] = $actual_revenue;

}

function add_row_to_project_revenue(&$project_data, $data_row, $revenue_type = 'actual') {
    $year = $data_row['year'];
    $month = $data_row['month'];
    $revenue = $data_row['total_revenue'];

    if(!isset($project_data[$revenue_type][$year.'-'.$month])) {
        $project_data[$revenue_type][$year.'-'.$month] = 0;
    }
    $project_data[$revenue_type][$year.'-'.$month] += $revenue;
}

function getMonthsBetweenDates(&$report_data, &$result_data) {

    $start = array_var($report_data, 'start_value');
    $end = array_var($report_data, 'end_value');
    $result = array($start);

    $start_date = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $start . '-01');
    $end_date = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $end . '-01');

    $current = $start_date->add('M', 1);

    while($current->getTimestamp() <= $end_date->getTimestamp()){
        $result[] = $current->format('Y-m');
        $current->add('M', 1);
    }

    $result_data['months_between_dates'] = $result;
    $report_data['months_between_dates'] = $result;
}
