<?php

function build_aging_invoices_report_excel($data, $report_data){
    $excel = build_all_aging_invoices_report_excel($data, $report_data);
    return $excel;
}

function build_all_aging_invoices_report_excel($data, $report_data){
    $excel = array();
    $contacts = $data['contacts'];
    foreach($contacts as $contact_id => $contact_info){
        $excel[] = build_contact_info_excel($contact_info, $report_data);
        $project_and_invoices_info = $contact_info['project_and_invoices_info'];
        
        $countCurrency = count($project_and_invoices_info['contact_totals']['currencies']);
        if($countCurrency > 1):
            for ($x = 1; $x <= $countCurrency; $x++) {
                $tmp = array();
                $tmp['currencies'][$project_and_invoices_info['contact_totals']['currencies'][$x]['currency_id']] = $project_and_invoices_info['contact_totals']['currencies'][$x];
                if($x > 1) $tmp['same_total_line'] = true;
                $excel[] = build_contact_totals_excel($contact_info['contact_name'], $tmp, $report_data);
            }
        else:
            $excel[] = build_contact_totals_excel($contact_info['contact_name'], $project_and_invoices_info['contact_totals'], $report_data);
        endif;
    }

    $excel[] = build_report_totals_excel($data['totals'], $data['distribution'], $report_data);

    return $excel;
}


function build_contact_info_excel($contact_info, $report_data){
    // Get custom property 'project_manager'
    $group_by = array_var($report_data, 'group_by', 'project');
    $has_selected_pm_id = array_var($report_data, 'has_selected_pm_id') == 'false' ? false : true;
    $project_ot = ObjectTypes::findByName('project');
    $cp_project_manager = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="project_manager"'));
    $pm_label = $cp_project_manager instanceof CustomProperty ? $cp_project_manager->getname() : lang('project manager');

    $pm_name = '';
    if($has_selected_pm_id || $group_by == 'pm') {
        $pm_label = $cp_project_manager instanceof CustomProperty ? $cp_project_manager->getname() : lang('project manager');
        $pm_name = $pm_label.': '.$contact_info['contact_name'];

    }
    if(!$has_selected_pm_id && $group_by == 'report_sent') {
        $project_ot = ObjectTypes::findByName('project');
        $cp_report_sent = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="report_sent"'));
        $report_sent_date = $contact_info['contact_name'];
        if(strpos($report_sent_date, lang('without')) === false) {
            $dt = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $report_sent_date);
            $report_sent_date = format_date($dt);
        }
        $label = $cp_report_sent->getName();
        $pm_name = $label.': '.$report_sent_date;
    }
    
    
    // Project Manager name
    $header = array($pm_name, '', '', '', '', '', '', '', '');
    if($report_data['show_invoice_issued_date']) {
        $header[] = '';
    }
    if($report_data['include_current_invoices']) {
        $header[] = '';
    }

    // Header columns
    $data = array();
    $header_row_1 = array(' ', lang('invoice'));
    if($report_data['show_invoice_issued_date']) $header_row_1[] = 'date';
    $header_row_2 = array(lang('due_date'),  lang('balance'));
    if($report_data['include_current_invoices']) $header_row_2[] = lang('current not due');
    $header_row_3 = array(lang('1-30'),  lang('31-60'),  lang('61-90'),  lang('91-120'),  lang('over 120'));
    $header_row = array_merge($header_row_1, $header_row_2, $header_row_3);
    $data[] = $header_row;

    // Build projects
    $project_and_invoices_info = $contact_info['project_and_invoices_info'];
    foreach($project_and_invoices_info['projects'] as $project_id => $project_info){

        $billing_contact_data = array_var($project_info['project_info'], 'billing_contact_info', null);
        $has_selected_pm_id = array_var($report_data, 'has_selected_pm_id');
        $group_by = array_var($report_data, 'group_by', 'project');

		// Show client name if exist
        $project_rows = array();

		$project_info_string = lang('customer') . ": " . $project_info['client_name'];
        $project_rows[] = array($project_info_string, '', '', '', '', '', '', '', '');
        $project_info_string = lang('project').': '.$project_info['project_info']['full_project_name'];
        $project_rows[] = array($project_info_string, '', '', '', '', '', '', '', '');

        if(isset($report_data['show_billing_contact'])) {
            $cp_billing_contact_name = array_var($report_data, 'cp_billing_contact_name', 'billing contact');
            if(is_null($billing_contact_data)) {
                $project_info_string = lang($cp_billing_contact_name) . ": " . "-" . "\n";
            } else {
                $project_info_string = lang($cp_billing_contact_name) . ": " . $billing_contact_data['name'];
                $project_info_string .= " (" . lang('email') . ": " . $billing_contact_data['email'] . "; ";
                $project_info_string .= lang('phone') . ": " . $billing_contact_data['phone'] . ")";
            }
            $project_rows[] = array($project_info_string, '', '', '', '', '', '', '', '');
        }
        $groups_by_cp_report_sent = !$has_selected_pm_id && $group_by == 'report_sent';
        if(isset($report_data['show_report_sent']) && !$groups_by_cp_report_sent) {
            $cp_report_sent_name = array_var($report_data, 'cp_report_sent_name', 'Report sent');
            $report_sent_date = $project_info['project_info']['report_sent'];
            $project_info_string = $cp_report_sent_name . ": " . $report_sent_date;
            $project_rows[] = array($project_info_string, '', '', '', '', '', '', '', '');
        }
        if(isset($report_data['show_invoice_sent'])) {
            $cp_invoice_sent_name = array_var($report_data, 'cp_invoice_sent_name', 'Invoice sent');
            $invoice_sent_date = $project_info['project_info']['invoice_sent'];
            $project_info_string = $cp_invoice_sent_name . ": " . $invoice_sent_date;
            $project_rows[] = array($project_info_string, '', '', '', '', '', '', '', '');
        }
        if(isset($report_data['show_collection_notes'])) {
            $cp_collection_notes_name = array_var($report_data, 'cp_collection_notes_name', 'Collection notes');
            $collection_notes = $project_info['project_info']['collection_notes'];
            $project_info_string = $cp_collection_notes_name . ": " . $collection_notes;
            $project_rows[] = array($project_info_string, '', '', '', '', '', '', '', '');
        }

        foreach($project_rows as $project_row){
            if($report_data['show_invoice_issued_date']) $project_row[] = '';
            if($report_data['include_current_invoices']) $project_row[] = '';
            $data[] = $project_row;
        }
       
        // Render invoices
        foreach($project_info['invoices'] as $invoice){
            $data[] = build_invoice_info_excel($invoice, $report_data);
        }
        
        // Project total row
        $countCurrency = count($project_info['project_totals']['currencies']);
        if ($countCurrency > 1) :
            for ($x = 1; $x <= $countCurrency; $x++) {
                $tmp = array();
                $tmp['currencies'][$project_info['project_totals']['currencies'][$x]['currency_id']] = $project_info['project_totals']['currencies'][$x];
                if ($x > 1) $tmp['same_total_line'] = true;
                $data[] = build_project_total_excel($tmp, $report_data);
            }
        else :
            $data[] = build_project_total_excel($project_info, $report_data);
        endif;
        
    }

    return array('headers' => $header, 'data' => $data);
}


function build_invoice_info_excel($invoice, $report_data){
    $currency = Currencies::instance()->getCurrency($invoice['currency_id']);
    $currency_sym = $currency->getSymbol();
    // Build invoice row
    $invoice_row = array('', $invoice['invoice_number']);
    if($report_data['show_invoice_issued_date']) $invoice_row[] = $invoice['issued_date'];
    $invoice_row[] = $invoice['date'];
    $invoice_row[] = format_money_amount($invoice['total'], $currency_sym);
    if($report_data['include_current_invoices']) {
        $invoice_row[] = format_money_amount($invoice['current_time'], $currency_sym);
    }
    $invoice_row[] = format_money_amount($invoice['first_month'], $currency_sym);
    $invoice_row[] = format_money_amount($invoice['second_month'], $currency_sym);
    $invoice_row[] = format_money_amount($invoice['third_month'], $currency_sym);
    $invoice_row[] = format_money_amount($invoice['fourth_month'], $currency_sym);
    $invoice_row[] = format_money_amount($invoice['greater_than_four_month'], $currency_sym);
    return $invoice_row;
}

function build_project_total_excel($project_info, $report_data){
    $project_totals = $project_info['project_totals'];
    // Build project totals row
    $project_total_title = lang('total for').' '.$project_info['project_info']['project_id'];
    

    if (isset($project_totals['same_total_line'])) :
        $project_totals_row = array('', '', '');
    else :
        $project_totals_row = array($project_total_title, '', '');
    endif;
    if($report_data['show_invoice_issued_date']) $project_totals_row[] = '';

    foreach($project_totals['currencies'] as $key => $value):
        $currency = Currencies::instance()->getCurrency($value['currency_id']);
        $project_totals_row[] = format_money_amount($value['total'], $currency->getSymbol());
        if($report_data['include_current_invoices']) {
            $project_totals_row[] = format_money_amount($value['current_time'], $currency->getSymbol());
        }
        $project_totals_row[] = format_money_amount($value['first_month'], $currency->getSymbol());
        $project_totals_row[] = format_money_amount($value['second_month'], $currency->getSymbol());
        $project_totals_row[] = format_money_amount($value['third_month'], $currency->getSymbol());
        $project_totals_row[] = format_money_amount($value['fourth_month'], $currency->getSymbol());
        $project_totals_row[] = format_money_amount($value['greater_than_four_month'], $currency->getSymbol());
    endforeach;
    
    return $project_totals_row;
}

function build_contact_totals_excel($contact_name, $contact_totals, $report_data){
    $group_by = array_var($report_data, 'group_by', 'project');
    $has_selected_pm_id = array_var($report_data, 'has_selected_pm_id') == 'false' ? false : true;
    $excel = "";
    // Build project totals row
    if(!$has_selected_pm_id && $group_by == 'report_sent') {
        if(strpos($contact_name, lang('without')) === false) {
            $dt = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $contact_name);
            $contact_name = format_date($dt);
        }
    } 
    $contact_total_title = lang('total for').' '.$contact_name;

    if(isset($contact_totals['same_total_line'])):
        $contact_totals_row = array('', '', '');
    else:
        $contact_totals_row = array($contact_total_title, '', '');
    endif;
    if($report_data['show_invoice_issued_date']) $contact_totals_row[] = '';

    foreach ($contact_totals['currencies'] as $key => $value) :
        $currency = Currencies::instance()->getCurrency($value['currency_id']);
        $contact_totals_row[] = format_money_amount($value['total'], $currency->getSymbol());
        if($report_data['include_current_invoices']) {
            $contact_totals_row[] = format_money_amount($value['current_time'], $currency->getSymbol());
        }
        $contact_totals_row[] = format_money_amount($value['first_month'], $currency->getSymbol());
        $contact_totals_row[] = format_money_amount($value['second_month'], $currency->getSymbol());
        $contact_totals_row[] = format_money_amount($value['third_month'], $currency->getSymbol());
        $contact_totals_row[] = format_money_amount($value['fourth_month'], $currency->getSymbol());
        $contact_totals_row[] = format_money_amount($value['greater_than_four_month'], $currency->getSymbol());
    endforeach;
    
    return array('totals' => $contact_totals_row);
}


function build_report_totals_excel($totals, $distribution_totals, $report_data){
   
    $totals_excel = array();
    // Main header
    $header = array(lang('totals'), '', '', '', '', '', '', '', '');
    if($report_data['show_invoice_issued_date']) $header[] = '';
    if($report_data['include_current_invoices']) $header[] = '';

    // Build totals header
    $total_header_row_1 = array('', '', '');
    if($report_data['show_invoice_issued_date']) $total_header_row_1[] = '';
    $total_header_row_2 = array(lang('balance'));
    if($report_data['include_current_invoices']) $total_header_row_2[] = lang('current not due');
    $total_header_row_3 = array(lang('1-30'), lang('31-60'), lang('61-90'), lang('91-120'), lang('over 120'));
    $total_header_row = array_merge($total_header_row_1, $total_header_row_2, $total_header_row_3);
    $totals_excel[] = $total_header_row;

     // Build totals row
    $total_title = lang('final totals');
    $tt = count($totals['currencies']);

    for ($x = 1; $x <= $tt; $x++) {

        $currency = Currencies::instance()->getCurrency($totals['currencies'][$x]['currency_id']);

        if ($x > 1) :
            $totals_row = array('', '', '');
        else :
            $totals_row = array($total_title, '', '');
        endif;
        if($report_data['show_invoice_issued_date']) $totals_row[] = '';

        $totals_row[] = format_money_amount($totals['currencies'][$x]['total'], $currency->getSymbol());
        if($report_data['include_current_invoices']) {
            $totals_row[] = format_money_amount($totals['currencies'][$x]['current_time'], $currency->getSymbol());
        }
        $totals_row[] = format_money_amount($totals['currencies'][$x]['first_month'], $currency->getSymbol());
        $totals_row[] = format_money_amount($totals['currencies'][$x]['second_month'], $currency->getSymbol());
        $totals_row[] = format_money_amount($totals['currencies'][$x]['third_month'], $currency->getSymbol());
        $totals_row[] = format_money_amount($totals['currencies'][$x]['fourth_month'], $currency->getSymbol());
        $totals_row[] = format_money_amount($totals['currencies'][$x]['greater_than_four_month'], $currency->getSymbol());
        $totals_excel[] =  $totals_row;

    }

    $distribution_title = lang('distribution');

    $tt = count($distribution_totals['currencies']);
    for ($x = 1; $x <= $tt; $x++) {
        $currency = Currencies::instance()->getCurrency($distribution_totals['currencies'][$x]['currency_id']);

        if ($x > 1) :
            $distribution_row = array('', '', '');
        else :
            $distribution_row = array($distribution_title, '', '');
        endif;
        if($report_data['show_invoice_issued_date']) $distribution_row[] = '';

        $distribution_row[] = format_money_amount($distribution_totals['currencies'][$x]['total'], '', 0) . ' %';
        if($report_data['include_current_invoices']) {
            $distribution_row[] = format_money_amount($distribution_totals['currencies'][$x]['current_time'], '', 0) . ' %';
        }
        $distribution_row[] = format_money_amount($distribution_totals['currencies'][$x]['first_month'], '', 0) . ' %';
        $distribution_row[] = format_money_amount($distribution_totals['currencies'][$x]['second_month'], '', 0) . ' %';
        $distribution_row[] = format_money_amount($distribution_totals['currencies'][$x]['third_month'], '', 0) . ' %';
        $distribution_row[] = format_money_amount($distribution_totals['currencies'][$x]['fourth_month'], '', 0) . ' %';
        $distribution_row[] = format_money_amount($distribution_totals['currencies'][$x]['greater_than_four_month'], '', 0) . ' %';
        $totals_excel[] = $distribution_row;
    }

    return array('headers' => $header, 'data' => $totals_excel);
}
