<?php

function get_all_aging_invoices_report_data(&$report_data) {
    $project_ot = ObjectTypes::findByName('project');
    $project_manager_dimension = Dimensions::instance()->findByCode('project_managers');
    $pm_dim_id = $project_manager_dimension->getId();
    $enabled_dimensions = config_option('enabled_dimensions');
    // Get custom property 'project_manager'
    $cp_project_manager = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="project_manager"'));
    // Get association between project and project manager
    $proj_proj_manager_association = DimensionMemberAssociations::instance()->getAssociationByCode('project_project_manager');
    $group_by = array_var($report_data, 'group_by', 'project');

    $has_selected_pm_id = false;

    if ($project_manager_dimension instanceof Dimension && $proj_proj_manager_association instanceof DimensionMemberAssociation && in_array($pm_dim_id, $enabled_dimensions)) {
        // Get project manager member ids
        $selected_pm_id = str_replace(array('[', ']'), '', array_var($report_data,'project_manager_member_id'));
        if($selected_pm_id > 0) {
            $pm_contact_ids = array($selected_pm_id);
            $has_selected_pm_id = true;
        } else {
            $pm_cp_values = Members::instance()->findAll((array('conditions' => '`dimension_id`='.$project_manager_dimension->getId())));
            $pm_contact_ids = array_map('get_member_id', $pm_cp_values);
        }
    } else if ($cp_project_manager instanceof CustomProperty) {
        // Get project manager contact ids
        if(array_var($report_data,'project_manager_id')) {
            $pm_contact_ids = array($report_data['project_manager_id']);
            $has_selected_pm_id = true;
        } else {
            $pm_cp_values = CustomPropertyValues::instance()->findAll((array('conditions' => '`custom_property_id`='.$cp_project_manager->getId())));
            $pm_all_ids = array_map('get_cp_value', $pm_cp_values);
            $pm_contact_ids = array_unique($pm_all_ids);
        }
    } else {
        return array();
    }

    $report_data['has_selected_pm_id'] = $has_selected_pm_id;

    $result = array(
        'contacts' => array(),
        'totals' => array('currencies' => array()),
        'distribution' => array('currencies' => array())
    );
    $processed_projects = array();
    
    if($has_selected_pm_id || $group_by == 'pm') {
        foreach($pm_contact_ids as $pm_contact_id) {
            if ($project_manager_dimension instanceof Dimension && $proj_proj_manager_association instanceof DimensionMemberAssociation && in_array($pm_dim_id, $enabled_dimensions)) {
                $pm_member = Members::instance()->findById($pm_contact_id);
                if(!($pm_member instanceof Member)) continue;
                $pm_name = $pm_member->getName();
            } else if ($cp_project_manager instanceof CustomProperty) {
                $pm_contact = Contacts::instance()->findById($pm_contact_id);
                if(!($pm_contact instanceof Contact)) continue;
                $pm_name = $pm_contact->getName();
            }

            // Get project manager's projects
            $project_invoices = get_project_manager_aging_invoices($pm_contact_id, $cp_project_manager, $report_data);
            if(!empty($project_invoices['projects'])){
                foreach($project_invoices['projects'] as $project_id => $project_info){
                    $processed_projects[] = $project_id;
                }
                add_project_manager_aging_invoices_to_totals($result, $project_invoices, $pm_contact_id, $pm_name, $report_data);
            }
        }
    }

    // Get unassigned projects  
    if(!$has_selected_pm_id && $group_by == 'pm') {
        $unassigned_project_invoices = get_unassigned_aging_invoices($processed_projects, $report_data);
        if(!empty($unassigned_project_invoices['projects'])){
            $unassigned_name = lang('unassigned');
            add_project_manager_aging_invoices_to_totals($result, $unassigned_project_invoices, 0, $unassigned_name, $report_data);
        }
    }

    if(!$has_selected_pm_id && $group_by == 'project') {
        $unassigned_project_invoices = get_unassigned_aging_invoices($processed_projects, $report_data);
        if(!empty($unassigned_project_invoices['projects'])){
            $unassigned_name = lang('unassigned');
            add_project_manager_aging_invoices_to_totals($result, $unassigned_project_invoices, 0, $unassigned_name, $report_data);
        }
    }

    if(!$has_selected_pm_id && $group_by == 'report_sent') {
        $project_invoices_grouped_by_report_sent = get_aging_invoices_grouped_by_project_report_sent($report_data);
        foreach($project_invoices_grouped_by_report_sent as $group_name => $report_sent_group) {
            if(empty($report_sent_group['projects'])) continue;
            add_project_manager_aging_invoices_to_totals($result, $report_sent_group, $group_name, $group_name, $report_data);
        }
    }

    // Sort currencies
    ksort($result['totals']['currencies']);

    if($result['totals']['currencies']){
        foreach($result['totals']['currencies'] as $currency_id => $curr_totals){
            $result['distribution']['currencies'][$currency_id] = array(
                'currency_id' => $currency_id,
                'total' => 100,
                'current_time' => 0,
                'first_month' => 0,
                'second_month' => 0,
                'third_month' => 0,
                'fourth_month' => 0,
                'greater_than_four_month' => 0
            );

            // Calculate distribution for each currency
            $total = $curr_totals['total'];
            if ($total > 0) {
                $result['distribution']['currencies'][$currency_id]['current_time'] = round(($curr_totals['current_time'] / $total) * 100);
                $result['distribution']['currencies'][$currency_id]['first_month'] = round(($curr_totals['first_month'] / $total) * 100);
                $result['distribution']['currencies'][$currency_id]['second_month'] = round(($curr_totals['second_month'] / $total) * 100);
                $result['distribution']['currencies'][$currency_id]['third_month'] = round(($curr_totals['third_month'] / $total) * 100);
                $result['distribution']['currencies'][$currency_id]['fourth_month'] = round(($curr_totals['fourth_month'] / $total) * 100);
                $result['distribution']['currencies'][$currency_id]['greater_than_four_month'] = round(($curr_totals['greater_than_four_month'] / $total) * 100);
            }
        }
    }

    return $result;

}

function add_project_manager_aging_invoices_to_totals(&$result, $project_invoices, $pm_contact_id, $pm_name, $report_data) {
      $result['contacts'][$pm_contact_id] = array(
        'project_and_invoices_info' => $project_invoices,
        'contact_name' => $pm_name
    );
    $contact_totals = $project_invoices['contact_totals'];
    foreach($contact_totals['currencies'] as $currency_id => $curr_totals){
        if(!array_key_exists($currency_id, $result['totals']['currencies'])){
            $result['totals']['currencies'][$currency_id] = array(
                'currency_id' => $currency_id,
                'total' => 0,
                'current_time' => 0,
                'first_month' => 0,
                'second_month' => 0,
                'third_month' => 0,
                'fourth_month' => 0,
                'greater_than_four_month' => 0
            );
        } 

        // Calculate totals for each currency
        $result['totals']['currencies'][$currency_id]['total'] += $curr_totals['total'];
        $result['totals']['currencies'][$currency_id]['current_time'] += $curr_totals['current_time'];
        $result['totals']['currencies'][$currency_id]['first_month'] += $curr_totals['first_month'];
        $result['totals']['currencies'][$currency_id]['second_month'] += $curr_totals['second_month'];
        $result['totals']['currencies'][$currency_id]['third_month'] += $curr_totals['third_month'];
        $result['totals']['currencies'][$currency_id]['fourth_month'] += $curr_totals['fourth_month'];
        $result['totals']['currencies'][$currency_id]['greater_than_four_month'] += $curr_totals['greater_than_four_month'];
    }
}

function get_project_manager_aging_invoices($pm_contact_id, $cp_project_manager, $report_data){
    $project_ot = ObjectTypes::findByName('project');
    $project_manager_dimension = Dimensions::instance()->findByCode('project_managers');
    $pm_dim_id = $project_manager_dimension->getId();
    $enabled_dimensions = config_option('enabled_dimensions');
    // Get custom property 'project_manager'
    $cp_project_manager = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="project_manager"'));
    // Get association between project and project manager
    $proj_proj_manager_association = DimensionMemberAssociations::instance()->getAssociationByCode('project_project_manager');


    if ($project_manager_dimension instanceof Dimension && $proj_proj_manager_association instanceof DimensionMemberAssociation && in_array($pm_dim_id, $enabled_dimensions)) {
        $mpms = MemberPropertyMembers::instance()->findAll(array('conditions' => '`association_id`='.$proj_proj_manager_association->getId().' AND `property_member_id`='.$pm_contact_id));
        $member_project_ids = array_map('get_project_member_ids', $mpms);
    } else if ($cp_project_manager instanceof CustomProperty) {
         // Get project ids under project manager
        $project_cp_values = CustomPropertyValues::instance()->findAll((array('conditions' => '`custom_property_id`='.$cp_project_manager->getId().' AND `value`='.$pm_contact_id)));

        if(empty($project_cp_values)) return array();

        $object_project_ids = array_map('get_cp_value_object_id', $project_cp_values);
        $member_project_ids = array_map('get_object_member_id',  $object_project_ids);
    }

    $grouped_by_project_invoices = collect_and_organize_invoices_for_projects($member_project_ids, $report_data);
    
    return $grouped_by_project_invoices;
}

function get_unassigned_aging_invoices($processed_projects, $report_data) {
    $active_context_project_member_ids = get_project_member_ids_for_project_manager_report(false);
    $unassigned_projects_member_ids = array_diff($active_context_project_member_ids, $processed_projects);
    $grouped_by_project_invoices = collect_and_organize_invoices_for_projects($unassigned_projects_member_ids, $report_data);

    return $grouped_by_project_invoices;
}

function collect_and_organize_invoices_for_projects($member_project_ids, $report_data){
    $group_by = array_var($report_data, 'group_by', 'project');
    $has_selected_pm_id = array_var($report_data, 'has_selected_pm_id', false);
    $grouped_by_project_invoices = array(
        'projects' => array(),
        'contact_totals' => array(
           'currencies' => array(),
        )
    );
    $group_by_report_sent = !$has_selected_pm_id && $group_by == 'report_sent';

    if ($group_by_report_sent) {
        $grouped_by_project_invoices = array('report_sent_groups' => array());
    }
       
    $object_members_table_name = ObjectMembers::instance()->getTableName();
    $now = DateTimeValueLib::now()->add('d', '-1')->endOfDay();
    $seen_invoices_ids = array();

    foreach($member_project_ids as $project_id){
        $member = Members::instance()->findById($project_id);
        $has_project_as_parent = member_has_project_as_parent($member);
        if(!($member instanceof Member) || $has_project_as_parent) continue;

        $report_sent_cp_val = get_report_sent_cp_val($member);
        if($group_by_report_sent) {
            if(!array_key_exists($report_sent_cp_val, $grouped_by_project_invoices['report_sent_groups'])) {
                $grouped_by_project_invoices['report_sent_groups'][$report_sent_cp_val] = array(
                    'projects' => array(),
                    'contact_totals' => array(
                        'currencies' => array(),
                    )
                );
            }
        }

        $project_invoices = get_invoices_by_project_id($project_id, $object_members_table_name);
        if(empty($project_invoices)) continue;
        $processed_invoices = array();
        $project_totals = array(
            'total' => 0,
            'first_month' => 0,
            'second_month' => 0,
            'third_month' => 0,
            'fourth_month' => 0,
            'greater_than_four_month' => 0
        );

        $project_totals = array('currencies' => array());
        foreach($project_invoices as $invoice){
            // Skip invoices that have already been seen
            if(in_array($invoice['id'], $seen_invoices_ids)) continue;
            $seen_invoices_ids[] = $invoice['id'];

            // Get invoice number
            $invoice_number = ($invoice['invoice_number'] > 0) ? $invoice['series'] . " " . $invoice['invoice_number'] : lang('not assigned');
            $currency_id = $invoice['currency_id'];
            if($currency_id == 0) {
                $currency_info = Currencies::getDefaultCurrencyInfo();
                $currency_id = $currency_info['id'];
            }
            if($invoice['date'] == '0000-00-00' || $invoice['date'] == '') {
                $issued_date = '';
            } else {
                $issued_date_sql = $invoice['date']; 
                $dtv_issued_date = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $issued_date_sql);
                $dtv_issued_date->advance(-1 * logged_user()->getUserTimezoneValue(), true);
                $issued_date = format_date($dtv_issued_date);
            }
            $due_date_sql = $invoice['expiration_date'];

            // Use 'due_amount' if invoice_payment_receipts plugin is active, otherwise use 'total'
            $due_amount = Plugins::instance()->isActivePlugin('invoice_payment_receipts') ? $invoice['due_amount'] : $invoice['total'];
            

            // Skip invoices with zero due amount
            if($due_amount == 0) continue;

            // Get invoice date
            if($due_date_sql == '0000-00-00' || $due_date_sql == '') {
                if(!array_var($report_data, 'include_current_invoices')) continue;
                $formatted_invoice_date = '';
                $current_time = $due_amount;
                $first_month = 0;
                $second_month = 0;
                $third_month = 0;
                $fourth_month = 0;
                $greater_than_four_month = 0;
            } else {
                // Get invoice date datetime value
                $invoice_date = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $due_date_sql);
                // Advance invoice date to user timezone
                $invoice_date->advance(-1 * logged_user()->getUserTimezoneValue(), true);
                // Skip invoices that are not due yet
                if(!array_var($report_data, 'include_current_invoices') && $now < $invoice_date) continue;

                $formatted_invoice_date = format_date($invoice_date);
                // Find invoice age
                $current_time = in_current_time($invoice_date) ? $due_amount : 0;
                $first_month = in_first_month($invoice_date) ? $due_amount : 0;
                $second_month = in_second_month($invoice_date) ? $due_amount : 0;
                $third_month = in_third_month($invoice_date) ? $due_amount : 0;
                $fourth_month = in_fourth_month($invoice_date) ? $due_amount : 0;
                $greater_than_four_month = greater_than_four_month($invoice_date) ? $due_amount : 0;
            }

            // Build invoice info
            $processed_invoices[] = array(
                'invoice_number' => $invoice_number,
                'currency_id' => $currency_id,
                'date' => $formatted_invoice_date,
                'issued_date' => $issued_date,
                'total' => $due_amount,
                'current_time' => $current_time,
                'first_month' => $first_month,
                'second_month' => $second_month,
                'third_month' => $third_month,
                'fourth_month' => $fourth_month,
                'greater_than_four_month' => $greater_than_four_month
            );

            // Create project totals for each currency
            if(!array_key_exists($currency_id, $project_totals['currencies'])){
                $project_totals['currencies'][$currency_id] = array(
                    'currency_id' => $currency_id,
                    'total' => 0,
                    'current_time' => 0,
                    'first_month' => 0,
                    'second_month' => 0,
                    'third_month' => 0,
                    'fourth_month' => 0,
                    'greater_than_four_month' => 0
                );
            }

            // Add to project totals
            $project_totals['currencies'][$currency_id]['total'] += $due_amount;
            $project_totals['currencies'][$currency_id]['current_time'] += $current_time;
            $project_totals['currencies'][$currency_id]['first_month'] += $first_month;
            $project_totals['currencies'][$currency_id]['second_month'] += $second_month;
            $project_totals['currencies'][$currency_id]['third_month'] += $third_month;
            $project_totals['currencies'][$currency_id]['fourth_month'] += $fourth_month;
            $project_totals['currencies'][$currency_id]['greater_than_four_month'] += $greater_than_four_month;

            // Create contact totals for each currency
            if($group_by_report_sent) {
                if(!array_key_exists($currency_id, $grouped_by_project_invoices[$report_sent_cp_val]['contact_totals']['currencies'])){
                    $grouped_by_project_invoices[$report_sent_cp_val]['contact_totals']['currencies'][$currency_id] = array(
                        'currency_id' => $currency_id,
                        'total' => 0,
                        'current_time' => 0,
                        'first_month' => 0,
                        'second_month' => 0,
                        'third_month' => 0,
                        'fourth_month' => 0,
                        'greater_than_four_month' => 0
                    );
                }

                // Add to project manager totals
                $grouped_by_project_invoices[$report_sent_cp_val]['contact_totals']['currencies'][$currency_id]['total'] += $due_amount;
                $grouped_by_project_invoices[$report_sent_cp_val]['contact_totals']['currencies'][$currency_id]['current_time'] += $current_time;
                $grouped_by_project_invoices[$report_sent_cp_val]['contact_totals']['currencies'][$currency_id]['first_month'] += $first_month;
                $grouped_by_project_invoices[$report_sent_cp_val]['contact_totals']['currencies'][$currency_id]['second_month'] += $second_month;
                $grouped_by_project_invoices[$report_sent_cp_val]['contact_totals']['currencies'][$currency_id]['third_month'] += $third_month;
                $grouped_by_project_invoices[$report_sent_cp_val]['contact_totals']['currencies'][$currency_id]['fourth_month'] += $fourth_month;
                $grouped_by_project_invoices[$report_sent_cp_val]['contact_totals']['currencies'][$currency_id]['greater_than_four_month'] += $greater_than_four_month;

                // Sort currencies
                ksort($project_totals['currencies']);
                ksort($grouped_by_project_invoices[$report_sent_cp_val]['contact_totals']['currencies']);
            } else {
                if(!array_key_exists($currency_id, $grouped_by_project_invoices['contact_totals']['currencies'])){
                    $grouped_by_project_invoices['contact_totals']['currencies'][$currency_id] = array(
                        'currency_id' => $currency_id,
                        'total' => 0,
                        'current_time' => 0,
                        'first_month' => 0,
                        'second_month' => 0,
                        'third_month' => 0,
                        'fourth_month' => 0,
                        'greater_than_four_month' => 0
                    );
                }

                // Add to project manager totals
                $grouped_by_project_invoices['contact_totals']['currencies'][$currency_id]['total'] += $due_amount;
                $grouped_by_project_invoices['contact_totals']['currencies'][$currency_id]['current_time'] += $current_time;
                $grouped_by_project_invoices['contact_totals']['currencies'][$currency_id]['first_month'] += $first_month;
                $grouped_by_project_invoices['contact_totals']['currencies'][$currency_id]['second_month'] += $second_month;
                $grouped_by_project_invoices['contact_totals']['currencies'][$currency_id]['third_month'] += $third_month;
                $grouped_by_project_invoices['contact_totals']['currencies'][$currency_id]['fourth_month'] += $fourth_month;
                $grouped_by_project_invoices['contact_totals']['currencies'][$currency_id]['greater_than_four_month'] += $greater_than_four_month;

                // Sort currencies
                ksort($project_totals['currencies']);
                ksort($grouped_by_project_invoices['contact_totals']['currencies']);
            }
        }

        if(empty($processed_invoices)) continue;
        // Get project info
        $project_member = Members::instance()->findById($project_id);

        if($project_member instanceof Member){
			Env::useHelper('functions', 'crpm');
			
			// check if null
			$client_member = get_client_member_of_project($project_member); 
			$client_name = (!empty($client_member)) ? $client_member->getName() : '';

            $proj_info = array(
                'invoices' => $processed_invoices,
                'project_totals' => $project_totals,
                'project_info' => get_project_info($project_member, $report_data),
				'client_name' => $client_name,
                'report_sent_cp_val' => $report_sent_cp_val
            );

            
            if($group_by_report_sent) {
                $grouped_by_project_invoices[$report_sent_cp_val]['projects'][$project_id] = $proj_info;
            } else {
                $grouped_by_project_invoices['projects'][$project_id] = $proj_info;
            }
        }
    }

    return $grouped_by_project_invoices;
}

function get_aging_invoices_grouped_by_project_report_sent($report_data) {
    $active_context_project_member_ids = get_project_member_ids_for_project_manager_report(false);
    $grouped_by_report_sent = collect_and_organize_invoices_for_projects($active_context_project_member_ids, $report_data);
    sort_grouped_by_report_sent($grouped_by_report_sent);

    return $grouped_by_report_sent;
}

function get_invoices_by_project_id($project_id, $object_members_table_name){
    $join_params = array(
        'table' => $object_members_table_name,
        'join_type' => 'INNER',
        'jt_field' => 'object_id',
        'e_field' => 'object_id',
    );

    $extra_conditions = " AND `jt`.`member_id`=".$project_id.' AND `e`.`status`!="confirmed" AND `e`.`status`!="canceled" ';
	$invoices_res = IncomeInvoices::instance()->listing(array( 
			'raw_data' => true,
			'join_params' => $join_params,
			'extra_conditions' => $extra_conditions,
			'order' => '`e`.`expiration_date`',
			'order_dir' => 'ASC',
	));

    $invoices = $invoices_res->objects;

    return $invoices;
}

/**
 * Helper function
 */

function get_cp_value($cpv){
    return $cpv->getValue();
}

function get_member_id($member){
    return $member->getId();
}

function get_project_member_ids($mpm) {
    return $mpm->getMemberId();
}

function get_cp_value_object_id($cpv){
    return $cpv->getObjectId();
}

function get_object_member_id($object_id){
    $member = Members::instance()->findOne(array('conditions' => '`object_id`='.$object_id));
    if($member instanceof Member){
        return $member->getId();
    } else {
        return 0;
    }
}

function check_date_in_range($start_date, $end_date, $inv_date) {
    $start_date_ts = $start_date->beginningOfDay()->getTimestamp();
    $end_date_ts = $end_date->endOfDay()->getTimestamp();
    $inv_date_ts = $inv_date->getTimestamp();
    // Check that date is between start & end
    return (($start_date_ts <= $inv_date_ts) && ($inv_date_ts <= $end_date_ts));
}

function in_current_time($inv_date){
    $inv_date_ts = $inv_date->getTimestamp();
    $end_date = DateTimeValueLib::now()->beginningOfDay();
    $end_date_ts = $end_date->getTimestamp();

    return $inv_date_ts >= $end_date_ts;
}

function in_first_month($inv_date){
    $start_date = DateTimeValueLib::now()->add('d', '-30');
    $end_date = DateTimeValueLib::now()->add('d', '-1');

    return check_date_in_range($start_date, $end_date, $inv_date);
}

function in_second_month($inv_date){
    $start_date = DateTimeValueLib::now()->add('d', '-60');
    $end_date = DateTimeValueLib::now()->add('d', '-31');

    return check_date_in_range($start_date, $end_date, $inv_date);
}

function in_third_month($inv_date){
    $start_date = DateTimeValueLib::now()->add('d', '-90');
    $end_date = DateTimeValueLib::now()->add('d', '-61');

    return check_date_in_range($start_date, $end_date, $inv_date);
}

function in_fourth_month($inv_date){
    $start_date = DateTimeValueLib::now()->add('d', '-120');
    $end_date = DateTimeValueLib::now()->add('d', '-91');

    return check_date_in_range($start_date, $end_date, $inv_date);
}

function greater_than_four_month($inv_date){
    $start_date = DateTimeValueLib::dateFromFormatAndString('Y-m-d', '2000-01-01');
    $end_date = DateTimeValueLib::now()->add('d', '-121');

    return check_date_in_range($start_date, $end_date, $inv_date);
}

function get_project_info(Member $project_member, $report_data) {

    $project_prefix_object = ObjectPrefixes::instance()->findById($project_member->getObjectId());
	if($project_prefix_object){
		$project_id = $project_prefix_object->getPrefix();
	} else {
		$pot = ObjectTypes::findByName('project');
		$project_id_cp = CustomProperties::getCustomPropertyByCode($pot->getId(), 'project_id');
		$project_id = '';
		if ($project_id_cp instanceof CustomProperty) {
			$cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $project_id_cp->getId());
			if ($cp_val instanceof CustomPropertyValue) {
                $project_id = $cp_val->getValue();
			}
		}
	}

    $project_name = $project_member->getName();
    $full_project_name = $project_id.' - '.$project_member->getName();

	$result = array(
        'project_id' => $project_id,
        'project_name' => $project_name,
        'full_project_name' => $full_project_name,
        'billing_contact_info' => get_billing_contact_info($project_member, $report_data),
        'report_sent' => get_project_member_cp_val($project_member, $report_data, 'report_sent'),
        'invoice_sent' => get_project_member_cp_val($project_member, $report_data, 'invoice_sent'),
        'collection_notes' => get_project_member_cp_val($project_member, $report_data, 'collection_notes'),
    );

    return $result;
}

function member_has_project_as_parent($member) {
    if(!($member instanceof Member)) return false;

    $parent_members = $member->getAllParentMembersInHierarchy();
    $project_ot_id = ObjectTypes::findByName('project')->getId();
    foreach($parent_members as $parent_member){
        if($parent_member->getObjectTypeId() == $project_ot_id) return true;
    }

    return false;

}

function get_report_sent_cp_val($project_member) {
    $report_sent_cp = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_member->getObjectTypeId().' AND `code`="report_sent"'));
    if($report_sent_cp instanceof CustomProperty) {
        $report_sent_cp_val = CustomPropertyValues::instance()->findOne(array('conditions' => '`custom_property_id`='.$report_sent_cp->getId().' AND `object_id`='.$project_member->getObjectId()));
        if($report_sent_cp_val instanceof CustomPropertyValue) {
            $val = $report_sent_cp_val->getValue();
            if($val != '') {
                $report_sent_cp_val = $val;
            } else {
                $report_sent_cp_val = lang('without') . ' ' . $report_sent_cp->getName();
            }
        } else {
            $report_sent_cp_val = lang('without') . ' ' . $report_sent_cp->getName();
        }
    } else {
        $report_sent_cp_val = '';
    }

    return $report_sent_cp_val;
}

function sort_grouped_by_report_sent(&$grouped_by_report_sent) {
    $project_ot = ObjectTypes::findByName('project');
    $report_sent_cp = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='. $project_ot->getId().' AND `code`="report_sent"'));
    $without_tag = lang('without') . ' ' . $report_sent_cp->getName();

    $without_info = $grouped_by_report_sent[$without_tag];
    unset($grouped_by_report_sent[$without_tag]);
    ksort($grouped_by_report_sent);
    $grouped_by_report_sent[$without_tag] = $without_info;
}

function get_billing_contact_info($project_member, $report_data, $skip_check = true) {
    $show_billing_contact = array_var($report_data, 'show_billing_contact', '') == 'checked';
    if(!$show_billing_contact && $skip_check) return null;

    $project_ot = ObjectTypes::findByName('project');
    $billing_contact_cp = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='. $project_ot->getId().' AND `code`="billing_contact"'));

    if($billing_contact_cp instanceof CustomProperty) {
        $billing_contact_cp_val = CustomPropertyValues::instance()->findOne(array('conditions' => '`custom_property_id`='.$billing_contact_cp->getId().' AND `object_id`='.$project_member->getObjectId()));
        if($billing_contact_cp_val instanceof CustomPropertyValue) {
            $billing_contact_id = $billing_contact_cp_val->getValue();
            if($billing_contact_id > 0) {
                $bc_contact = Contacts::instance()->findById($billing_contact_id);
                if($bc_contact instanceof Contact) {
                    $billing_contact_data = array(
                        'name' => $bc_contact->getObjectName(),
                        'email' => '',
                        'phone' => ''
                    );
                    $email_address_obj = ContactEmails::instance()->findOne(array("conditions" => array("contact_id=?",$bc_contact->getId())));
                    $billing_contact_data['email'] = $email_address_obj instanceof ContactEmail ? $email_address_obj->getEmailAddress() : strtoupper(lang('n/a'));
                   
                    $phone = $bc_contact->getPhone('work', false, false);
                    $billing_contact_data['phone'] = $phone instanceof ContactTelephone ? $phone->getNumber() : strtoupper(lang('n/a'));

                    return $billing_contact_data;
                }
            }
        }
    }

    return null;
}

function get_project_member_cp_val($project_member, $report_data, $cp_code, $skip_check = true)  {
    $show_cp = array_var($report_data, 'show_'.$cp_code, '') == 'checked';
    if(!$show_cp && $skip_check) return '';

    $project_ot = ObjectTypes::findByName('project');
    $cp = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='. $project_ot->getId().' AND `code`="'.$cp_code.'"'));

    if($cp instanceof CustomProperty) {
        $cp_val = CustomPropertyValues::instance()->findOne(array('conditions' => '`custom_property_id`='.$cp->getId().' AND `object_id`='.$project_member->getObjectId()));
        if($cp_val instanceof CustomPropertyValue) {
            $val = $cp_val->getValue();
            if($cp->getType() == 'date') {
                if ($val != '') {
                    $dtv = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $val);
                    $dtv->advance(-1 * logged_user()->getUserTimezoneValue(), true);
                    $val = format_date($dtv);
                } else {
                    $val = '';
                }
                
            }
        } else {
            $val = strtoupper(lang('n/a'));
        }
    }

    return $val;
}
