<?php

function build_ar_ledger_report_excel($data) {

	// columns and report data
	$columns = array_var($data, 'columns');
	$ar_ledger_data = array_var($data, 'ar_ledger_data');
	$ar_ledger_totals = array_var($data, 'ar_ledger_totals');

	// render data for each project
	foreach ($ar_ledger_data as $project_data) {
		$excel[] = build_ar_ledger_project_excel_data($project_data, $columns);
	}
	
	// render totals table
    $excel[] = build_ar_ledger_report_excel_totals($ar_ledger_totals, $columns);

	return $excel;
}

function build_ar_ledger_project_excel_data($project_data, $columns) {
	
	$excel_table = array(
		'main_header' => array(),
		'headers' => array(), 
		'data' => array(),
		'totals' => array(),
		'totals_after_data' => true,
	);

	// main header = project name
	$excel_table['main_header'][] = $project_data['project_name'];
	$i=1;
	while ($i < count($columns)) {
		$excel_table['main_header'][] = '';
		$i++;
	}

	// table headers
    foreach($columns as $c_name){
        $excel_table['headers'][] = lang($c_name);
    }

    // invoices data
    $invoices_info = $project_data['grouped_invoices'];
    foreach($invoices_info as $invoice_info){
		$excel_table['data'] = array_merge($excel_table['data'], build_ar_ledger_invoice_data_excel($invoice_info, $columns));
    }

    // Render Contact Totals
	$excel_table['totals'] = build_ar_ledger_project_totals_excel($project_data, $columns);


    return $excel_table;
}

function build_ar_ledger_invoice_data_excel($invoice_info, $columns) {

	$excel_rows = array();

	$invoice = $invoice_info['invoice'];
	$payments = $invoice_info['payments'];
	$currency = Currencies::instance()->getCurrency($invoice['currency_id']);
	$currency_sym = $currency instanceof Currency ? $currency->getSymbol() : '$';

	$inv_num = lang($invoice['invoice_type_name']) . " " .$invoice['invoice_number'];

	// invoice header
	$invoice_title_row = array($inv_num . " - " . clean($invoice['name']));
	$i=1;
	while ($i < count($columns)) {
		$invoice_title_row[] = '';
		$i++;
	}
	$excel_rows[] = $invoice_title_row;

	$amount_prefix = '';
	$amount_suffix = '';
	if (!$invoice['sum_in_total']) {
		// for credit and debit notes
		$amount_prefix = '(';
		$amount_suffix = ')';		
	}

	$invoice_data_row = array();
    
	foreach ($columns as $col) {
		switch ($col) {
			case 'invoice':
				$cell_value = $inv_num;
				break;
			case 'receipt_detail':
				$cell_value = '';
				break;
			case 'date':
				$cell_value = $invoice['date'];
				break;
			case 'total':
				$cell_value = $amount_prefix . format_money_amount($invoice['total'], $currency_sym) . $amount_suffix;
				break;
			case 'fees':
				$cell_value = $amount_prefix . format_money_amount($invoice['subtotal'], $currency_sym) . $amount_suffix;
				break;
			case 'taxes':
				$cell_value = $amount_prefix . format_money_amount($invoice['taxes'], $currency_sym) . $amount_suffix;
				break;
		}

		$invoice_data_row[] = $cell_value;
	}
	$excel_rows[] = $invoice_data_row;

	if (is_array($payments) && count($payments) > 0) {
		foreach ($payments as $payment) {
			$payment_data_row = array();

			$p_currency = Currencies::instance()->getCurrency($payment['currency_id']);
			$p_currency_sym = $p_currency instanceof Currency ? $p_currency->getSymbol() : '$';

			foreach ($columns as $col) {
				$cell_class = '';
				switch ($col) {
					case 'invoice':
						$cell_value = lang('payment for')." ".$inv_num;
						break;
					case 'receipt_detail':
						$cell_class = 'name';
						$cell_value = $payment['ext_transaction_id'] ? $payment['ext_transaction_id'] : $payment['name'];
						break;
					case 'date':
						$cell_value = $payment['date'];
						break;
					case 'total':
						$cell_value = '('.format_money_amount($payment['amount'], $p_currency_sym).')';
						break;
					case 'fees':
						$cell_value = '';
						break;
					case 'taxes':
						$cell_value = '';
						break;
				}

				$payment_data_row[] = $cell_value;
			}
			$excel_rows[] = $payment_data_row;
		}
	}

	$invoice_total_row = array();

	foreach ($columns as $col) {
		$cell_value = '';
		switch ($col) {
			case 'invoice':
				$cell_value = lang('total')." ".$inv_num;
				break;
			case 'receipt_detail':
			case 'date':
				break;
			case 'total':
				$cell_value = $amount_prefix . format_money_amount($invoice['due_amount'], $currency_sym) . $amount_suffix;
				break;
			case 'fees':
				$cell_value = $amount_prefix . format_money_amount($invoice['subtotal'], $currency_sym) . $amount_suffix;
				break;
			case 'taxes':
				$cell_value = $amount_prefix . format_money_amount($invoice['taxes'], $currency_sym) . $amount_suffix;
				break;
		}

		$invoice_total_row[] = $cell_value;
	}
	$excel_rows[] = $invoice_total_row;

	return $excel_rows;
}

function build_ar_ledger_project_totals_excel($project_data, $columns) {

	$totals = $project_data['totals'];
	$currency = Currencies::instance()->getCurrency($totals['currency_id']);
	$currency_sym = $currency instanceof Currency ? $currency->getSymbol() : '$';

	$project_total_row = array();
	foreach ($columns as $col) {
		$cell_value = '';
		switch ($col) {
			case 'invoice':
				$cell_value = lang('total')." ".$project_data['project_name'];
				break;
			case 'receipt_detail':
			case 'date':
				break;
			case 'total':
				$cell_value = format_money_amount($totals['due_amount'], $currency_sym);
				break;
			case 'fees':
				$cell_value = format_money_amount($totals['subtotal'], $currency_sym);
				break;
			case 'taxes':
				$cell_value = format_money_amount($totals['taxes'], $currency_sym);
				break;
		}

		$project_total_row[] = $cell_value;
	}

	return $project_total_row;
}

function build_ar_ledger_report_excel_totals($ar_ledger_totals, $columns) {

	$currency = Currencies::instance()->getCurrency($ar_ledger_totals['currency_id']);
	$currency_sym = $currency instanceof Currency ? $currency->getSymbol() : '$';

	$total_table = array(
		'main_header' => array(),
		'headers' => array(), 
		'data' => array(),
		'totals' => array(),
		'totals_after_data' => true,
	);

	$total_table['main_header'][] = lang('total');
	$i=1;
	while ($i < count($columns)) {
		$total_table['main_header'][] = '';
		$i++;
	}

	// table headers
    foreach($columns as $c_name){
		if (in_array($c_name, array('invoice','receipt_detail','date'))) {
        	$total_table['headers'][] = '';
		} else {
			$total_table['headers'][] = lang($c_name);
		}
    }

	foreach ($columns as $col) {
		$cell_value = '';
		switch ($col) {
			case 'invoice':
			case 'receipt_detail':
			case 'date':
				break;
			case 'total':
				$cell_value = format_money_amount($ar_ledger_totals['due_amount'], $currency_sym);
				break;
			case 'fees':
				$cell_value = format_money_amount($ar_ledger_totals['subtotal'], $currency_sym);
				break;
			case 'taxes':
				$cell_value = format_money_amount($ar_ledger_totals['taxes'], $currency_sym);
				break;
		}

		$total_table['totals'][] = $cell_value;
	}

    return $total_table;
}


