<?php

function build_budget_by_tasks_report_excel($data, $report_data) {

    $excel_data = array();
    $header_data = build_excel_header_for_budget_by_tasks($report_data);
    $excel_data = array_merge($excel_data,  $header_data);

    $project_data = build_excel_for_project_for_budget_by_tasks($data['project_data'], $report_data);

    $excel_data = array_merge($excel_data, $project_data);

    return $excel_data;
}


function build_excel_header_for_budget_by_tasks($report_data) {

    $main_header = array('', '', '', '', '', lang('time'), '', '', lang('cost'), '', '', lang('price'), '');

    $secondary_header = array('', '', lang('percent completed'), lang('status'), lang('estimated (hrs)'), lang('worked'), lang('remaining budgeted'), lang('estimated cost'), lang('executed cost'), lang('remaining budgeted'), lang('estimated price'), lang('earned value'), lang('remaining budgeted'));

    $excel_data = array();
    $excel_data[] = array('headers' => $main_header);
    $excel_data[] = array('headers' => $secondary_header);

    return $excel_data;
}

function build_excel_for_project_for_budget_by_tasks($project, $report_data) {
    $group_by_job_phase = array_var($report_data, 'group_by', false) == 1;
    $tasks = $project['tasks_data'];
    $subprojects = $project['subprojects_data'];

    $excel_data = array();

    if((($group_by_job_phase && count($tasks['phases']) == 0) || count($tasks) == 0) && count($subprojects) == 0) {
        return $excel_data;
    }
 
    $project_header = build_excel_project_header_with_totals_for_budget_by_tasks($project, $report_data);
    $excel_data = array_merge($excel_data, array($project_header));

    if($group_by_job_phase) {
        $phases = array();
        foreach($tasks['phases'] as $phase) {
            $phases[] = build_excel_phase_for_budget_by_tasks($phase, $report_data);
        }
        $excel_data = array_merge($excel_data, $phases);
    } else {
		$all_tasks_data = array();
        foreach($tasks as $task) {
			$this_task_and_subtasks_data = build_excel_task_for_budget_by_tasks($task, $report_data);
			$all_tasks_data = array_merge($all_tasks_data, $this_task_and_subtasks_data);
        }
        $last_excel_data_group_index = count($excel_data) - 1;
		if ( $last_excel_data_group_index < 0)  $last_excel_data_group_index = 0;
		
		$excel_data[$last_excel_data_group_index]['data'] = $all_tasks_data;

        $time_data = build_excel_unassigned_time_for_budget_by_tasks($project, $report_data);
        $excel_data = array_merge($excel_data, $time_data);

        $actual_exp_data = build_excel_unassigned_actual_expenses_for_budget_by_tasks($project, $report_data);
        $excel_data = array_merge($excel_data, $actual_exp_data);

        $budgeted_exp_data = build_excel_unassigned_budgeted_expenses_for_budget_by_tasks($project, $report_data);
        $excel_data = array_merge($excel_data, $budgeted_exp_data);
    }

    foreach($subprojects as $subproject) {
        $subproject_data = build_excel_for_project_for_budget_by_tasks($subproject, $report_data);
        $excel_data = array_merge($excel_data, $subproject_data);
    } 

    return $excel_data;
}

function build_excel_project_header_with_totals_for_budget_by_tasks($project, $report_data, $is_phase = false) {

    $currency_sym = config_option('currency_code');

    $title = $is_phase ? $project['phase_name'] : $project['project_name'];
    $project = $is_phase ? $project['totals'] : $project;
    $remaining_minutes = get_formatted_remaining_minutes_excel($project['remaining_minutes']);
    $remaining_cost = get_formatted_remaining_value_excel($project['remaining_cost']);
    $remaining_price = get_formatted_remaining_value_excel($project['remaining_price']);
    
    $project_header = array(
        $title,
        '',
        round($project['percent_completed']),
        '',
        format_amount(round($project['estimated_minutes'] / 60, 2), 2),
        format_amount(round($project['worked_minutes'] / 60, 2), 2),
        $remaining_minutes,
        format_money_amount($project['estimated_cost'], $currency_sym),
        format_money_amount($project['executed_cost'], $currency_sym),
        $remaining_cost,
        format_money_amount($project['estimated_price'], $currency_sym),
        format_money_amount($project['executed_price'], $currency_sym),
        $remaining_price
    );

    $excel_data = array('totals' => $project_header, 'data' => array());

    return $excel_data;
}

function build_excel_phase_for_budget_by_tasks($phase, $report_data) {
    $tasks = $phase['tasks'];
    $has_tasks = count($tasks) > 0;
    $time_entries = $phase['time_data']['time_entries'];
    $has_time = count($time_entries) > 0;
    $budgeted_expenses = $phase['budgeted_expenses_data']['budgeted_expenses'];
    $has_budgeted_expenses = count($budgeted_expenses) > 0;
    $actual_expenses = $phase['actual_expenses_data']['actual_expenses'];
    $has_actual_expenses = count($actual_expenses) > 0;

    if(!$has_tasks && !$has_time && !$has_budgeted_expenses && !$has_actual_expenses) {
        return array();
    }

    $excel_data = array();
    // build the phase name row
    $phase_header = build_excel_project_header_with_totals_for_budget_by_tasks($phase, $report_data, true);
    $excel_data = array_merge($excel_data, $phase_header);

    if($has_tasks) {
        $tasks_data = array('data' => array());
        foreach($tasks as $task) {
            $task_and_subtasks_data = build_excel_task_for_budget_by_tasks($task, $report_data);
            $tasks_data['data'] = array_merge($tasks_data['data'], $task_and_subtasks_data);
        }
        $excel_data = array_merge($excel_data, $tasks_data);
    }

    $time_data = build_excel_unassigned_time_for_budget_by_tasks($phase, $report_data, false, true);
    if(!empty($time_data)) {
        foreach($time_data[0]['data'] as $time_entry) {
            $excel_data['data'][] = $time_entry;
        }
    }

    $actual_exp_data = build_excel_unassigned_actual_expenses_for_budget_by_tasks($phase, $report_data, false, true);
    if(!empty($actual_exp_data)) {
        foreach($actual_exp_data[0]['data'] as $actual_expense) {
            $excel_data['data'][] = $actual_expense;
        }
    }

    $budgeted_exp_data = build_excel_unassigned_budgeted_expenses_for_budget_by_tasks($phase, $report_data, false, true);
    if(!empty($budgeted_exp_data)) {
        foreach($budgeted_exp_data[0]['data'] as $budgeted_expense) {
            $excel_data['data'][] = $budgeted_expense;
        }
    }

    return $excel_data;
}


function build_excel_task_for_budget_by_tasks($task, $report_data, $data_level = 0) {

    $currency_sym = config_option('currency_code');
    $subtasks = $task['subtasks'];
   
    $completed = $task['is_completed'] ? lang('completed') : lang('open');
    $remaining_minutes = get_formatted_remaining_minutes_excel($task['remaining_minutes']);
    $remaining_cost = get_formatted_remaining_value_excel($task['remaining_cost']);
    $remaining_price = get_formatted_remaining_value_excel($task['remaining_price']);

	// indent task name depending on the hierarchy level
	$name_prefix = "";
	$i = 0;
	while ($i < $data_level) {
		$name_prefix .= "    ";
		$i++;
	}
    
    $task_data = array(
        $name_prefix . $task['task_name'],
        '',
        round($task['percent_completed']),
        $completed,
        format_amount(round($task['estimated_minutes'] / 60, 2), 2),
        format_amount(round($task['worked_minutes'] / 60, 2), 2),
        $remaining_minutes,
        format_money_amount($task['estimated_cost'], $currency_sym),
        format_money_amount($task['executed_cost'], $currency_sym),
        $remaining_cost,
        format_money_amount($task['estimated_price'], $currency_sym),
        format_money_amount($task['executed_price'], $currency_sym),
        $remaining_price
    );

	$all_tasks_data = array($task_data);

	$subtasks_level = $data_level + 1;
	foreach ($subtasks as $subtask) {
		$this_subtask_data = build_excel_task_for_budget_by_tasks($subtask, $report_data, $subtasks_level);
		$all_tasks_data = array_merge($all_tasks_data, $this_subtask_data);
	}
    
    return $all_tasks_data;
}

function build_excel_unassigned_time_for_budget_by_tasks($project, $report_data, $is_task = false, $is_phase = false) {
    if(!$is_phase) {
        $unassigned_time_data = !$is_task ? $project['unassigned_time_entries'] : $project['time_data'];
    } else {
        $unassigned_time_data = $project['time_data'];
    }

    $time_entries = array_var($unassigned_time_data, 'time_entries', array());
    if(count($time_entries) == 0) {
        return array();
    }
   
    $row_title = !$is_task ? lang('time entries not linked to tasks') : lang('time entries linked to') . ' '. $project['task_name'];
    $currency_sym = config_option('currency_code');
   
    $remaining_minutes = $unassigned_time_data['worked_minutes'] > 0 ? $unassigned_time_data['worked_minutes'] * -1 : 0;
    $formatted_remaining_minutes = get_formatted_remaining_minutes_excel($remaining_minutes);
    $remaining_cost = $unassigned_time_data['executed_cost'] > 0 ? $unassigned_time_data['executed_cost'] * -1 : 0;
    $formatted_remaining_cost = get_formatted_remaining_value_excel($remaining_cost);
    $remaining_price = $unassigned_time_data['executed_price'] > 0 ? $unassigned_time_data['executed_price'] * -1 : 0;
    $formatted_remaining_price = get_formatted_remaining_value_excel($remaining_price);

    $time_data = array(array('data' => array(array(
        $row_title,
        '',
        '',
        '',
        '',
        format_amount(round($unassigned_time_data['worked_minutes'] / 60, 2), 2),
        $formatted_remaining_minutes,
        '',
        format_money_amount($unassigned_time_data['executed_cost'], $currency_sym),
        $formatted_remaining_cost,
        '',
        format_money_amount($unassigned_time_data['executed_price'], $currency_sym),
        $formatted_remaining_price
    ))));

    return $time_data;
}

function build_excel_unassigned_actual_expenses_for_budget_by_tasks($project, $report_data, $is_task = false, $is_phase = false) {
    if(!$is_phase) {
        $unassigned_actual_expenses = !$is_task ? $project['unassigned_actual_expenses'] : $project['actual_expenses_data'];
    } else {
        $unassigned_actual_expenses = $project['actual_expenses_data'];
    }
    $actual_expenses = array_var($unassigned_actual_expenses, 'actual_expenses', array());
    if(count($actual_expenses) == 0) {
        return array();
    }
   
    $currency_sym = config_option('currency_code');
    $row_title = !$is_task ? lang('actual expenses not budgeted and not linked to tasks') : lang('actual expenses not budgeted, linked to') . ' '. $project['task_name'];
    
    $remaining_cost = $unassigned_actual_expenses['executed_cost'] > 0 ? $unassigned_actual_expenses['executed_cost'] * -1 : 0;
    $formatted_remaining_cost = get_formatted_remaining_value_excel($remaining_cost);
    $remaining_price = $unassigned_actual_expenses['executed_price'] > 0 ? $unassigned_actual_expenses['executed_price'] * -1 : 0;
    $formatted_remaining_price = get_formatted_remaining_value_excel($remaining_price);

    $excel_data = array(array('data' => array(array(
        $row_title,
        '',
        '',
        '',
        '',
        '',
        '',
        '',
        format_money_amount($unassigned_actual_expenses['executed_cost'], $currency_sym),
        $formatted_remaining_cost,
        '',
        format_money_amount($unassigned_actual_expenses['executed_price'], $currency_sym),
        $formatted_remaining_price
    ))));

    return $excel_data;
}

function build_excel_unassigned_budgeted_expenses_for_budget_by_tasks($project, $report_data, $is_task = false, $is_phase = false) {
    if(!$is_phase) {
        $unassigned_budgeted_expenses = !$is_task ? $project['unassigned_budgeted_expenses'] : $project['budgeted_expenses_data'];
    } else {
        $unassigned_budgeted_expenses = $project['budgeted_expenses_data'];
    }
    $budgeted_expenses = array_var($unassigned_budgeted_expenses, 'budgeted_expenses', array());
    if(count($budgeted_expenses) == 0) {
        return array();
    }
   
    $currency_sym = config_option('currency_code');
    $row_title = !$is_task ? lang('budgeted expenses not linked to tasks') : lang('budgeted expenses linked to') . ' '. $project['task_name'];

    $remaining_cost = get_formatted_remaining_value_excel($unassigned_budgeted_expenses['remaining_cost']);
    $negative_price = $unassigned_budgeted_expenses['remaining_price'] < 0 ? 'negative' : '';
    $remaining_price = get_formatted_remaining_value_excel($unassigned_budgeted_expenses['remaining_price']);
   
    $excel_data = array(array('data' => array(array(
        $row_title,
        '',
        '',
        '',
        '',
        '',
        '',
        format_money_amount($unassigned_budgeted_expenses['estimated_cost'], $currency_sym),
        format_money_amount($unassigned_budgeted_expenses['executed_cost'], $currency_sym),
        $remaining_cost,
        format_money_amount($unassigned_budgeted_expenses['estimated_price'], $currency_sym),
        format_money_amount($unassigned_budgeted_expenses['executed_price'], $currency_sym),
        $remaining_price
    ))));

    return $excel_data;
}



function get_formatted_remaining_value_excel($remaining_value) {
    $currency_sym = config_option('currency_code');
    $corrected_remaining_value = $remaining_value < 0 ? $remaining_value * -1 : $remaining_value;
    $formatted_value = $remaining_value < 0 ? '('.format_money_amount($corrected_remaining_value, $currency_sym).')' : format_money_amount($corrected_remaining_value, $currency_sym);

    return $formatted_value;
}

function get_formatted_remaining_minutes_excel($remaining_minutes) {
    $corrected_remaining_minutes = $remaining_minutes < 0 ? $remaining_minutes * -1 : $remaining_minutes;
    $formatted_value = $remaining_minutes < 0 ? '('.format_amount(round($corrected_remaining_minutes / 60, 2), 2).')' : format_amount(round($corrected_remaining_minutes / 60, 2), 2);

    return $formatted_value;
}