<?php

function get_budget_by_tasks_report_data($report_data) {
    $project_selector_empty = $report_data['filter_by'] == 1 && empty(json_decode($report_data['project_member_id']));
    $project_ot = ObjectTypes::findByName('project');   
    $project_manager_dimension = Dimensions::instance()->findByCode('project_managers');
    $pm_dim_id = $project_manager_dimension->getId();
    // Get association between project and project manager
    $proj_proj_manager_association = DimensionMemberAssociations::instance()->getAssociationByCode('project_project_manager');
    $enabled_dimensions = config_option('enabled_dimensions');
    // Get custom property 'project_manager'
    $cp_project_manager = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="project_manager"'));
    if($project_manager_dimension instanceof Dimension && $proj_proj_manager_association instanceof DimensionMemberAssociation && in_array($pm_dim_id, $enabled_dimensions)){
        $project_manager_empty = $report_data['filter_by'] == 2 && empty(json_decode($report_data['project_member_id']));
    } else if ($cp_project_manager instanceof CustomProperty) {
        $project_manager_empty = $report_data['filter_by'] == 2 && empty($report_data['project_manager_id']);
    }
    if($project_selector_empty /*|| $project_manager_empty*/){
        flash_error(lang('please select a project'));
        ajx_current("empty");
        return;
    }

    if(!$project_selector_empty && $report_data['filter_by'] == 1) {
        $project_member_id = json_decode($report_data['project_member_id']);
        $project_member_id = $project_member_id[0];
        $project_data = collect_data_for_project_budget_by_tasks_report($project_member_id, $report_data);

        $report_data = array(
            'has_project_manager' => false,
            'project_data' => $project_data
        );
    } else if(!$project_manager_empty && $report_data['filter_by'] == 2) {
        if($project_manager_dimension instanceof Dimension && $proj_proj_manager_association instanceof DimensionMemberAssociation && in_array($pm_dim_id, $enabled_dimensions)) { 
            $project_manager_id = json_decode($report_data['project_member_id'])[0];
            $project_manager = Members::instance()->findById($project_manager_id);
            $project_manager_name = $project_manager->getDisplayName();
            $report_data['use_other_associated_member_ids'] = true;
            $report_data['other_associated_member_ids'] = array($project_manager_id);
            $project_manager_project_member_ids = get_project_member_ids_for_project_manager_report(false, $report_data);
        } else if ($cp_project_manager instanceof CustomProperty) {
             // Get project ids under project manager
            $pm_contact_id = $report_data['project_manager_id'];
            $manager_contact = Contacts::instance()->findById($pm_contact_id);
            $project_manager_name = $manager_contact->getName();
            $project_cp_values = CustomPropertyValues::instance()->findAll((array('conditions' => '`custom_property_id`='.$cp_project_manager->getId().' AND `value`='.$pm_contact_id)));
            if(empty($project_cp_values)) return array();
            $object_project_ids = array_map('get_cp_value_object_id', $project_cp_values);
            $project_manager_project_member_ids = array_map('get_object_member_id',  $object_project_ids);
        }
        $project_data = array();
        foreach($project_manager_project_member_ids as $project_id) {
            $project_member = Members::instance()->findById($project_id);
            if(!$project_member instanceof Member || $project_member->getParentMemberId() > 0) continue;
            $new_project = collect_data_for_project_budget_by_tasks_report($project_id, $report_data);
            if(!empty($new_project['tasks_data']) || !empty($new_project['subprojects_data'])) {
                $project_data[] = $new_project;
            }
        }

        $manager_totals = calculate_manager_totals_budget_by_tasks_report($project_data);

        $report_data = array(
            'has_project_manager' => true,
            'project_manager_name' => $project_manager_name,
            'project_data' => $project_data,
            'manager_totals' => $manager_totals
        );
    }

    return $report_data;
}

function collect_data_for_project_budget_by_tasks_report($project_member_id, $report_data) {

    $project_member = Members::instance()->findById($project_member_id);
    $member_totals = MemberTotals::instance()->findOne(array('conditions' => array('`member_id`="'.$project_member_id.'"')));
    $estimated_minutes = $member_totals instanceof MemberTotal ? $member_totals->getColumnValue('estimated_labor_minutes') : 0;
    $worked_minutes = $member_totals instanceof MemberTotal ? $member_totals->getColumnValue('executed_labor_minutes') : 0;
    $remaining_minutes = $estimated_minutes - $worked_minutes;
    $estimated_cost = $member_totals instanceof MemberTotal ? $member_totals->getColumnValue('estimated_overall_total_cost') : 0;
    $executed_cost = $member_totals instanceof MemberTotal ? $member_totals->getColumnValue('executed_overall_total_cost') : 0;
    $remaining_cost = $estimated_cost - $executed_cost;
    $estimated_price = $member_totals instanceof MemberTotal ? $member_totals->getColumnValue('estimated_overall_total_price') : 0;
    $executed_price = $member_totals instanceof MemberTotal ? $member_totals->getColumnValue('executed_overall_total_price') : 0;
    $remaining_price = $estimated_price - $executed_price;
    $percent_completed = $member_totals instanceof MemberTotal ? $member_totals->getColumnValue('percent_completed') : 0;
    $project_tasks = collect_tasks_data_for_project_budget_by_tasks_report($project_member_id, $report_data);
    $subprojects = collect_subprojects_data_for_project_budget_by_tasks_report($project_member_id, $report_data);
    $unassigned_time_entries = collect_unassigned_time_entries_for_project_budget_by_tasks_report($project_member_id, $report_data);
    $unassigned_budgeted_expenses = collect_unassigned_budgeted_expenses_for_project_budget_by_tasks_report($project_member_id, $report_data);
    $unassigned_actual_expenses = collect_unassigned_actual_expenses_for_project_budget_by_tasks_report($project_member_id, $report_data, null, $unassigned_budgeted_expenses);

    $project_data = array(
        'project_id' => $project_member_id,
        'project_name' => $project_member->getDisplayName(),
        'subprojects_data' => $subprojects,
        'tasks_data' => $project_tasks,
        'unassigned_time_entries' => $unassigned_time_entries,
        'unassigned_actual_expenses' => $unassigned_actual_expenses,
        'unassigned_budgeted_expenses' => $unassigned_budgeted_expenses,
        'estimated_minutes' => $estimated_minutes,
        'worked_minutes' => $worked_minutes,
        'remaining_minutes' => $remaining_minutes,
        'estimated_cost' => $estimated_cost,
        'executed_cost' => $executed_cost,
        'remaining_cost' => $remaining_cost,
        'estimated_price' => $estimated_price,
        'executed_price' => $executed_price,
        'remaining_price' => $remaining_price,
        'percent_completed' => $percent_completed
    );

    return $project_data;
}

function collect_tasks_data_for_project_budget_by_tasks_report($project_member_id, $report_data) {

    $group_by_job_phase = array_var($report_data, 'group_by', false) == 1;
    $job_phase_object_type_id = ObjectTypes::findByName('job_phase')->getId();
    $job_phase_dimension = Dimensions::instance()->findByCode('job_phases');
    $jp_dim_name = $job_phase_dimension->getName();

    $tasks = ProjectTasks::instance()->listing(array(
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        'order' => '`o`.`name`',
        'order_dir' => 'ASC',
        'include_deleted' => false,
        "count_results" => false,
        "show_only_member_objects" => true,
        "member_ids" => array($project_member_id),
        "extra_conditions" => " AND `parent_id`=0 AND `o`.`trashed_by_id`=0 AND `o`.`archived_by_id`=0 "
    ))->objects;


    if($group_by_job_phase) {
        $tasks_data = array(
            'phases' => array()
        );
    } else {
        $tasks_data = array();
    }

    foreach($tasks as $parent_task) {
        $subtasks_data = collect_subtask_data($parent_task);

        $parent_task_data = array(
            'task_id' => $parent_task->getId(),
            'task_name' => $parent_task->getObjectName(),
            'percent_completed' => $parent_task->getPercentCompleted(),
            'is_completed' => $parent_task->getColumnValue('completed_by_id') > 0 ? true : false,
            'estimated_minutes' => $parent_task->getTotalTimeEstimate(),
            'worked_minutes' => $parent_task->getOverallWorkedTime(),
            'remaining_minutes' => $parent_task->getTotalTimeEstimate() - $parent_task->getOverallWorkedTime(),
            'estimated_cost' => $parent_task->getColumnValue('estimated_cost'),
            'executed_cost' => $parent_task->getColumnValue('executed_cost'),
            'remaining_cost' => $parent_task->getColumnValue('estimated_cost') - $parent_task->getColumnValue('executed_cost'),
            'estimated_price' => $parent_task->getColumnValue('estimated_price'),
            'executed_price' => $parent_task->getColumnValue('earned_value'),
            'remaining_price' => $parent_task->getColumnValue('estimated_price') - $parent_task->getColumnValue('earned_value'),
            'subtasks' => $subtasks_data
        );

        collect_time_and_expenses_for_task($parent_task, $parent_task_data);

        if($group_by_job_phase) {
            $task_job_phase = $parent_task->getMemberOfType($job_phase_object_type_id);
            $job_phase_name = $task_job_phase instanceof Member ? $task_job_phase->getDisplayName() : lang('no job phase dimension name assigned', $jp_dim_name);
            $job_phase_id = $task_job_phase instanceof Member ? $task_job_phase->getId() : 0;
            if(!isset($tasks_data['phases'][$job_phase_id])) {
                build_empty_phase_data($tasks_data, $job_phase_id, $job_phase_name);
            }
            $tasks_data['phases'][$job_phase_id]['tasks'][] = $parent_task_data;

        } else {
            $tasks_data[] = $parent_task_data;
        } 
    }   

    // Get unassigned time entries and put it in the job phase group
    if($group_by_job_phase) {
        collect_unassigned_time_entries_for_job_phase_project_budget_by_tasks_report($project_member_id, $report_data, $tasks_data);
        $unassigned_budgeted_expenses = collect_unassigned_budgeted_expenses_for_job_phase_project_budget_by_tasks_report($project_member_id, $report_data, $tasks_data);
        collect_unassigned_actual_expenses_for_job_phase_project_budget_by_tasks_report($project_member_id, $report_data, $tasks_data, $unassigned_budgeted_expenses); 
    }

    if($group_by_job_phase) {
        // Sort by phase name
        uasort($tasks_data['phases'], function($a, $b) {
            return strcmp($a['phase_name'], $b['phase_name']);
        });
        // Move the tasks without job phase to the end
        if (isset($tasks_data['phases'][0])) {
            $value_to_move = $tasks_data['phases'][0];
            unset($tasks_data['phases'][0]);
            $tasks_data['phases'][0] = $value_to_move;
        }

        calculate_job_phase_totals($tasks_data);
    }


    return $tasks_data;

}

function build_empty_phase_data(&$tasks_data, $job_phase_id, $job_phase_name) {
    $tasks_data['phases'][$job_phase_id] = array(
        'phase_id' => $job_phase_id,
        'phase_name' => $job_phase_name,
        'tasks' => array(),
        'totals' => array(
            'estimated_minutes' => 0,
            'worked_minutes' => 0,
            'remaining_minutes' => 0,
            'estimated_cost' => 0,
            'executed_cost' => 0,
            'remaining_cost' => 0,
            'estimated_price' => 0,
            'executed_price' => 0,
            'remaining_price' => 0,
            'percent_completed' => 0
        )
    );

    $tasks_data['phases'][$job_phase_id]['time_data'] = array(
        'worked_minutes' => 0,
        'executed_cost' => 0,
        'executed_price' => 0,
        'time_entries' => array()
    );

    $tasks_data['phases'][$job_phase_id]['budgeted_expenses_data'] = array(
        'estimated_cost' => 0,
        'estimated_price' => 0,
        'executed_cost' => 0,
        'executed_price' => 0,
        'remaining_cost' => 0,
        'remaining_price' => 0,
        'budgeted_expenses' => array()
    );

    $tasks_data['phases'][$job_phase_id]['actual_expenses_data'] = array(
        'executed_cost' => 0,
        'executed_price' => 0,
        'actual_expenses' => array()
    );
}

function collect_subtask_data($parent_task) {
        $subtasks = $parent_task->getSubtasks(false, false);
        $subtasks_info = array();

        foreach($subtasks as $subtask) {
            $sub_subtasks = collect_subtask_data($subtask);

            $subtask_data = array(
                'task_id' => $subtask->getId(),
                'task_name' => $subtask->getObjectName(),
                'percent_completed' => $subtask->getPercentCompleted(),
                'is_completed' => $subtask->getColumnValue('completed_by_id') > 0 ? true : false,
                'estimated_minutes' => $subtask->getTotalTimeEstimate(),
                'worked_minutes' => $subtask->getOverallWorkedTime(),
                'remaining_minutes' => $subtask->getTotalTimeEstimate() - $subtask->getOverallWorkedTime(),
                'estimated_cost' => $subtask->getColumnValue('estimated_cost'),
                'executed_cost' => $subtask->getColumnValue('executed_cost'),
                'remaining_cost' => $subtask->getColumnValue('estimated_cost') - $subtask->getColumnValue('executed_cost'),
                'estimated_price' => $subtask->getColumnValue('estimated_price'),
                'executed_price' => $subtask->getColumnValue('earned_value'),
                'remaining_price' => $subtask->getColumnValue('estimated_price') - $subtask->getColumnValue('earned_value'),
                'subtasks' => $sub_subtasks
            );

            collect_time_and_expenses_for_task($subtask, $subtask_data);
            $subtasks_info[] = $subtask_data;
        }
    
        return $subtasks_info;
} 

function collect_subprojects_data_for_project_budget_by_tasks_report($project_member_id, $report_data) {
    $subprojects_data = array();
    $parent_member = Members::instance()->findById($project_member_id);
    $subprojects = $parent_member->getAllChildrenInHierarchy();
    foreach($subprojects as $subproject) {
        $subproject_data = collect_data_for_project_budget_by_tasks_report($subproject->getId(), $report_data);
        $subprojects_data[] = $subproject_data;
    }

    return $subprojects_data;
}

function calculate_manager_totals_budget_by_tasks_report($project_data) {
    $manager_totals = array(
        'estimated_minutes' => 0,
        'worked_minutes' => 0,
        'remaining_minutes' => 0,
        'estimated_cost' => 0,
        'executed_cost' => 0,
        'remaining_cost' => 0,
        'estimated_price' => 0,
        'executed_price' => 0,
        'remaining_price' => 0,
        'percent_completed' => 0
    );
    $percentage_numerator = 0;
    $percentage_denominator = 0;

    foreach($project_data as $project) {
        $manager_totals['estimated_minutes'] += $project['estimated_minutes'];
        $manager_totals['worked_minutes'] += $project['worked_minutes'];
        $manager_totals['remaining_minutes'] += $project['remaining_minutes'];
        $manager_totals['estimated_cost'] += $project['estimated_cost'];
        $manager_totals['executed_cost'] += $project['executed_cost'];
        $manager_totals['remaining_cost'] += $project['remaining_cost'];
        $manager_totals['estimated_price'] += $project['estimated_price'];
        $manager_totals['executed_price'] += $project['executed_price'];
        $manager_totals['remaining_price'] += $project['remaining_price'];

        $percentage_numerator += $project['percent_completed'] * $project['estimated_minutes'];
        $percentage_denominator += $project['estimated_minutes'];
    }

    $manager_totals['percent_completed'] = $percentage_denominator > 0 ? $percentage_numerator / $percentage_denominator : 0;

    return $manager_totals;
}

function collect_unassigned_time_entries_for_project_budget_by_tasks_report($project_member_id, $report_data, $task = null) {

    $time_data = array(
        'worked_minutes' => 0,
        'executed_cost' => 0,
        'executed_price' => 0,
        'time_entries' => array()
    );

    $group_by_job_phase = array_var($report_data, 'group_by', false) == 1;
    if($group_by_job_phase) {
        return $time_data;
    }

    if($task instanceof ProjectTask) {
        $task_id = $task->getId();
        $time_entries = Timeslots::instance()->listing(array(
            "fire_additional_data_hook" => false,
            "ignore_context" => true,
            'order' => '`o`.`name`',
            'order_dir' => 'ASC',
            'include_deleted' => false,
            "count_results" => false,
            "extra_conditions" => " AND `rel_object_id`=" . $task_id . " AND `o`.`trashed_by_id`=0 AND `o`.`archived_by_id`=0 "
        ))->objects;
    } else {
        $time_entries = Timeslots::instance()->listing(array(
            "fire_additional_data_hook" => false,
            "ignore_context" => true,
            'order' => '`o`.`name`',
            'order_dir' => 'ASC',
            'include_deleted' => false,
            "count_results" => false,
            "show_only_member_objects" => true,
            "member_ids" => array($project_member_id),
            "extra_conditions" => " AND `rel_object_id`=0 AND `o`.`trashed_by_id`=0 AND `o`.`archived_by_id`=0 "
        ))->objects;
    }

    foreach($time_entries as $time) {

        $contact_id = $time->getContactId();
        $contact_name = '';
        $contact = Contacts::instance()->findById($contact_id);
        if($contact) {
            $contact_name = $contact->getDisplayName();
        }

        $new_time_data = array(
            'contact_name' => $contact_name,
            'date' => $time->getStartTime(),
            'description' => $time->getDescription(),
            'worked_minutes' => $time->getWorkedTime(),
            'executed_cost' => $time->getColumnValue('fixed_cost'),
            'executed_price' => $time->getFixedBilling(),
        );

        $time_data['time_entries'][] = $new_time_data;
        $time_data['worked_minutes'] += $new_time_data['worked_minutes'];
        $time_data['executed_cost'] += $new_time_data['executed_cost'];
        $time_data['executed_price'] += $new_time_data['executed_price'];
    }

    return $time_data;
}

function collect_unassigned_time_entries_for_job_phase_project_budget_by_tasks_report($project_member_id, $report_data, &$tasks_data) {
    $job_phase_object_type_id = ObjectTypes::findByName('job_phase')->getId();
    $job_phase_dimension = Dimensions::instance()->findByCode('job_phases');
    $jp_dim_name = $job_phase_dimension->getName();

    $time_entries = Timeslots::instance()->listing(array(
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        'order' => '`o`.`name`',
        'order_dir' => 'ASC',
        'include_deleted' => false,
        "count_results" => false,
        "show_only_member_objects" => true,
        "member_ids" => array($project_member_id),
        "extra_conditions" => " AND `rel_object_id`=0 AND `o`.`trashed_by_id`=0 AND `o`.`archived_by_id`=0 "
    ))->objects;
    if(empty($time_entries)) return;

    foreach($time_entries as $time) {
            $time_job_phase = $time->getMemberOfType($job_phase_object_type_id);
            $job_phase_name = $time_job_phase instanceof Member ? $time_job_phase->getDisplayName() : lang('no job phase dimension name assigned', $jp_dim_name);
            $job_phase_id = $time_job_phase instanceof Member ? $time_job_phase->getId() : 0;
            if(!isset($tasks_data['phases'][$job_phase_id])) {
                build_empty_phase_data($tasks_data, $job_phase_id, $job_phase_name);
            }

            $contact_id = $time->getContactId();
            $contact_name = '';
            $contact = Contacts::instance()->findById($contact_id);
            if($contact) {
                $contact_name = $contact->getDisplayName();
            }

            $new_time_data = array(
                'contact_name' => $contact_name,
                'date' => $time->getStartTime(),
                'description' => $time->getDescription(),
                'worked_minutes' => $time->getWorkedTime(),
                'executed_cost' => $time->getColumnValue('fixed_cost'),
                'executed_price' => $time->getFixedBilling(),
            );

            $tasks_data['phases'][$job_phase_id]['time_data']['time_entries'][] = $new_time_data;
            $tasks_data['phases'][$job_phase_id]['time_data']['worked_minutes'] += $new_time_data['worked_minutes'];
            $tasks_data['phases'][$job_phase_id]['time_data']['executed_cost'] += $new_time_data['executed_cost'];
            $tasks_data['phases'][$job_phase_id]['time_data']['executed_price'] += $new_time_data['executed_price'];
    }
}

function collect_unassigned_actual_expenses_for_project_budget_by_tasks_report($project_member_id, $report_data, $task = null, $budgeted_expenses_data = array()) {

    $actual_expenses_data = array(
        'executed_cost' => 0,
        'executed_price' => 0,
        'actual_expenses' => array()
    );

    $group_by_job_phase = array_var($report_data, 'group_by', false) == 1;
    if($group_by_job_phase) {
        return $actual_expenses_data;
    }


    $budgeted_expenses = array_var($budgeted_expenses_data, 'budgeted_expenses', array());
    $budgeted_expenses_ids = array_keys($budgeted_expenses);
    $be_condition = count($budgeted_expenses_ids) > 0 ? 'AND `expense_id` NOT IN (' .implode(',', $budgeted_expenses_ids) .')' : '';

    if($task instanceof ProjectTask) {
        $task_id = $task->getId();
        $payment_receipts = PaymentReceipts::instance()->listing(array(
            "fire_additional_data_hook" => false,
            "ignore_context" => true,
            'order' => '`o`.`name`',
            'order_dir' => 'ASC',
            'include_deleted' => false,
            "count_results" => false,
            "extra_conditions" => " AND `task_id`=" . $task_id . " ".$be_condition." AND `o`.`trashed_by_id`=0 AND `o`.`archived_by_id`=0 "
        ))->objects;

    } else {
        $payment_receipts = PaymentReceipts::instance()->listing(array(
            "fire_additional_data_hook" => false,
            "ignore_context" => true,
            'order' => '`o`.`name`',
            'order_dir' => 'ASC',
            'include_deleted' => false,
            "count_results" => false,
            "show_only_member_objects" => true,
            "member_ids" => array($project_member_id),
            "extra_conditions" => " AND `task_id`=0 AND `o`.`trashed_by_id`=0  ".$be_condition." AND `o`.`archived_by_id`=0 "
        ))->objects;
    
    }

    foreach($payment_receipts as $actual_expense) {
        $new_actual_expense_data = array(
            'expense_name' => $actual_expense->getObjectName(),
            'date' => $actual_expense->getDate(),
            'description' => $actual_expense->getDescription(),
            'executed_cost' => $actual_expense->getAmount(),
            'executed_price' => $actual_expense->getTotalPrice(),
        );

        $actual_expenses_data['actual_expenses'][] = $new_actual_expense_data;
        $actual_expenses_data['executed_cost'] += $new_actual_expense_data['executed_cost'];
        $actual_expenses_data['executed_price'] += $new_actual_expense_data['executed_price'];
    }

    return $actual_expenses_data;
}

function collect_unassigned_actual_expenses_for_job_phase_project_budget_by_tasks_report($project_member_id, $report_data, &$tasks_data, $budgeted_expenses_data) {
    $job_phase_object_type_id = ObjectTypes::findByName('job_phase')->getId();
    $job_phase_dimension = Dimensions::instance()->findByCode('job_phases');
    $jp_dim_name = $job_phase_dimension->getName();

    $budgeted_expenses = array_var($budgeted_expenses_data, 'budgeted_expenses', array());
    $budgeted_expenses_ids = array_keys($budgeted_expenses);
    $be_condition = count($budgeted_expenses_ids) > 0 ? 'AND `expense_id` NOT IN (' .implode(',', $budgeted_expenses_ids) .')' : '';

    $payment_receipts = PaymentReceipts::instance()->listing(array(
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        'order' => '`o`.`name`',
        'order_dir' => 'ASC',
        'include_deleted' => false,
        "count_results" => false,
        "show_only_member_objects" => true,
        "member_ids" => array($project_member_id),
        "extra_conditions" => " AND `task_id`=0 AND `o`.`trashed_by_id`=0  ".$be_condition." AND `o`.`archived_by_id`=0 "
    ))->objects;


    foreach($payment_receipts as $actual_expense) {
        $exp_job_phase = $actual_expense->getMemberOfType($job_phase_object_type_id);
        $job_phase_name = $exp_job_phase instanceof Member ? $exp_job_phase->getDisplayName() : lang('no job phase dimension name assigned', $jp_dim_name);
        $job_phase_id = $exp_job_phase instanceof Member ? $exp_job_phase->getId() : 0;
        if(!isset($tasks_data['phases'][$job_phase_id])) {
            build_empty_phase_data($tasks_data, $job_phase_id, $job_phase_name);
        }
        
        $new_actual_expense_data = array(
            'expense_name' => $actual_expense->getObjectName(),
            'date' => $actual_expense->getDate(),
            'description' => $actual_expense->getDescription(),
            'executed_cost' => $actual_expense->getAmount(),
            'executed_price' => $actual_expense->getTotalPrice(),
        );

        $tasks_data['phases'][$job_phase_id]['actual_expenses_data']['actual_expenses'][] = $new_actual_expense_data;
        $tasks_data['phases'][$job_phase_id]['actual_expenses_data']['executed_cost'] += $new_actual_expense_data['executed_cost'];
        $tasks_data['phases'][$job_phase_id]['actual_expenses_data']['executed_price'] += $new_actual_expense_data['executed_price'];
    }
}

function collect_unassigned_budgeted_expenses_for_project_budget_by_tasks_report($project_member_id, $report_data, $task = null) {
    $budgeted_expenses_data = array(
        'estimated_cost' => 0,
        'estimated_price' => 0,
        'executed_cost' => 0,
        'executed_price' => 0,
        'remaining_cost' => 0,
        'remaining_price' => 0,
        'budgeted_expenses' => array()
    );

    $group_by_job_phase = array_var($report_data, 'group_by', false) == 1;
    if($group_by_job_phase) {
        return $budgeted_expenses_data;
    }

    if($task instanceof ProjectTask) {
        $task_id = $task->getId();
        $expenses = Expenses::instance()->listing(array(
            "fire_additional_data_hook" => false,
            "ignore_context" => true,
            'order' => '`o`.`name`',
            'order_dir' => 'ASC',
            'include_deleted' => false,
            "count_results" => false,
            "extra_conditions" => " AND `task_id`=" . $task_id . " AND `o`.`trashed_by_id`=0 AND `o`.`archived_by_id`=0 "
        ))->objects;
    } else {
        $expenses = Expenses::instance()->listing(array(
            "fire_additional_data_hook" => false,
            "ignore_context" => true,
            'order' => '`o`.`name`',
            'order_dir' => 'ASC',
            'include_deleted' => false,
            "count_results" => false,
            "show_only_member_objects" => true,
            "member_ids" => array($project_member_id),
            "extra_conditions" => " AND `task_id`=0 AND `o`.`trashed_by_id`=0 AND `o`.`archived_by_id`=0 "
        ))->objects;
    }

    foreach($expenses as $exp) {
       $expense_totals = $exp->getExpenseTotals();
       $exp_id = $exp->getId();
       $new_budgeted_expense = array(
        'expense_id' => $exp_id,
        'expense_name' => $exp->getObjectName(),
        'date' => $exp->getDate(),
        'description' => $exp->getDescription(),
        'estimated_cost' => $expense_totals['total_cost'],
        'estimated_price' => $expense_totals['total_price'],
        'executed_cost' => 0,
        'executed_price' => 0,
        'remaining_cost' => 0,
        'remaining_price' => 0,
        'actual_expenses' => array(),
       );

        // Calculate estimated expenses
        $budgeted_expenses_data['estimated_cost'] += $new_budgeted_expense['estimated_cost'];
        $budgeted_expenses_data['estimated_price'] += $new_budgeted_expense['estimated_price'];

        // Calculate actual expenses
        collect_actual_expenses_linked_to_budgeted_expense($new_budgeted_expense, $project_member_id, $task);
        $budgeted_expenses_data['executed_cost'] += $new_budgeted_expense['executed_cost'];
        $budgeted_expenses_data['executed_price'] += $new_budgeted_expense['executed_price'];

        // Calculate remaining expenses
        $budgeted_expenses_data['remaining_cost'] += $new_budgeted_expense['remaining_cost'];
        $budgeted_expenses_data['remaining_price'] += $new_budgeted_expense['remaining_price'];
        

        // Add budgeted expense to the list
        $budgeted_expenses_data['budgeted_expenses'][$exp_id] = $new_budgeted_expense;
    }

    return $budgeted_expenses_data;
}

function collect_unassigned_budgeted_expenses_for_job_phase_project_budget_by_tasks_report($project_member_id, $report_data, &$tasks_data) {
    $job_phase_object_type_id = ObjectTypes::findByName('job_phase')->getId();
    $job_phase_dimension = Dimensions::instance()->findByCode('job_phases');
    $jp_dim_name = $job_phase_dimension->getName();

    $expenses = Expenses::instance()->listing(array(
        "fire_additional_data_hook" => false,
        "ignore_context" => true,
        'order' => '`o`.`name`',
        'order_dir' => 'ASC',
        'include_deleted' => false,
        "count_results" => false,
        "show_only_member_objects" => true,
        "member_ids" => array($project_member_id),
        "extra_conditions" => " AND `task_id`=0 AND `o`.`trashed_by_id`=0 AND `o`.`archived_by_id`=0 "
    ))->objects;

    if(empty($expenses)) return;
    $unassigned_budgeted_expenses = array('budgeted_expenses' => array());

    foreach($expenses as $exp) {
        $exp_job_phase = $exp->getMemberOfType($job_phase_object_type_id);
        $job_phase_name = $exp_job_phase instanceof Member ? $exp_job_phase->getDisplayName() : lang('no job phase dimension name assigned', $jp_dim_name);
        $job_phase_id = $exp_job_phase instanceof Member ? $exp_job_phase->getId() : 0;
        if(!isset($tasks_data['phases'][$job_phase_id])) {
            build_empty_phase_data($tasks_data, $job_phase_id, $job_phase_name);
        }

        $expense_totals = $exp->getExpenseTotals();
        $exp_id = $exp->getId();
        $new_budgeted_expense = array(
         'expense_id' => $exp_id,
         'expense_name' => $exp->getObjectName(),
         'date' => $exp->getDate(),
         'description' => $exp->getDescription(),
         'estimated_cost' => $expense_totals['total_cost'],
         'estimated_price' => $expense_totals['total_price'],
         'executed_cost' => 0,
         'executed_price' => 0,
         'remaining_cost' => 0,
         'remaining_price' => 0,
         'actual_expenses' => array(),
        );

        // Calculate estimated expenses
        $tasks_data['phases'][$job_phase_id]['budgeted_expenses_data']['estimated_cost'] += $new_budgeted_expense['estimated_cost'];
        $tasks_data['phases'][$job_phase_id]['budgeted_expenses_data']['estimated_price'] += $new_budgeted_expense['estimated_price'];

        // Calculate actual expenses
        collect_actual_expenses_linked_to_budgeted_expense($new_budgeted_expense, $project_member_id);
        $tasks_data['phases'][$job_phase_id]['budgeted_expenses_data']['executed_cost'] += $new_budgeted_expense['executed_cost'];
        $tasks_data['phases'][$job_phase_id]['budgeted_expenses_data']['executed_price'] += $new_budgeted_expense['executed_price'];

        // Calculate remaining expenses
        $tasks_data['phases'][$job_phase_id]['budgeted_expenses_data']['remaining_cost'] += $new_budgeted_expense['remaining_cost'];
        $tasks_data['phases'][$job_phase_id]['budgeted_expenses_data']['remaining_price'] += $new_budgeted_expense['remaining_price'];

        // Add budgeted expense to the list
        $tasks_data['phases'][$job_phase_id]['budgeted_expenses_data']['budgeted_expenses'][$exp_id] = $new_budgeted_expense;
        $unassigned_budgeted_expenses['budgeted_expenses'][$exp_id] = $new_budgeted_expense;
    }

    return $unassigned_budgeted_expenses;
}

function calculate_job_phase_totals(&$tasks_data) {

    foreach($tasks_data['phases'] as &$phase){
        $phase_totals = $phase['totals'];
        $phase_tasks = $phase['tasks'];
        $phase_time_data = $phase['time_data'];
        $phase_budgeted_expenses_data = $phase['budgeted_expenses_data'];

        $percentage_numerator = 0;
        $percentage_denominator = 0;

        foreach($phase_tasks as $task) {
            $phase_totals['estimated_minutes'] += $task['estimated_minutes'];
            $phase_totals['worked_minutes'] += $task['worked_minutes'];
            $phase_totals['remaining_minutes'] += $task['remaining_minutes'];
            $phase_totals['estimated_cost'] += $task['estimated_cost'];
            $phase_totals['executed_cost'] += $task['executed_cost'];
            $phase_totals['remaining_cost'] += $task['remaining_cost'];
            $phase_totals['estimated_price'] += $task['estimated_price'];
            $phase_totals['executed_price'] += $task['executed_price'];
            $phase_totals['remaining_price'] += $task['remaining_price'];

            $percentage_numerator += $task['percent_completed'] * $task['estimated_minutes'];
            $percentage_denominator += $task['estimated_minutes'];
        }

        // calculate time data
        $phase_totals['worked_minutes'] += $phase_time_data['worked_minutes'];
        $phase_totals['executed_cost'] += $phase_time_data['executed_cost'];
        $phase_totals['executed_price'] += $phase_time_data['executed_price'];

        // Calculate budgeted expenses data
        $phase_totals['estimated_cost'] += $phase_budgeted_expenses_data['estimated_cost'];
        $phase_totals['estimated_price'] += $phase_budgeted_expenses_data['estimated_price'];
        $phase_totals['executed_cost'] += $phase_budgeted_expenses_data['executed_cost'];
        $phase_totals['executed_price'] += $phase_budgeted_expenses_data['executed_price'];
        $phase_totals['remaining_cost'] += $phase_budgeted_expenses_data['remaining_cost'];
        $phase_totals['remaining_price'] += $phase_budgeted_expenses_data['remaining_price'];

        // Calculate actual expenses data
        $phase_totals['executed_cost'] += $phase['actual_expenses_data']['executed_cost'];
        $phase_totals['executed_price'] += $phase['actual_expenses_data']['executed_price'];

        $phase_totals['percent_completed'] = $percentage_denominator > 0 ? $percentage_numerator / $percentage_denominator : 0;

        $phase['totals'] = $phase_totals;

    }

}

function collect_time_and_expenses_for_task($task, &$task_data) {
    $task_data['time_data'] = collect_unassigned_time_entries_for_project_budget_by_tasks_report(null, null, $task);
    $task_data['budgeted_expenses_data'] = collect_unassigned_budgeted_expenses_for_project_budget_by_tasks_report(null, null, $task);
    $task_data['actual_expenses_data'] = collect_unassigned_actual_expenses_for_project_budget_by_tasks_report(null, null, $task, $task_data['budgeted_expenses_data']);
}

function collect_actual_expenses_linked_to_budgeted_expense(&$new_budgeted_expense, $project_member_id, $task = null) {
    $exp_id = $new_budgeted_expense['expense_id'];
    if($task instanceof ProjectTask) {
        $task_id = $task->getId();
        $payment_receipts = PaymentReceipts::instance()->listing(array(
            "fire_additional_data_hook" => false,
            "ignore_context" => true,
            'order' => '`o`.`name`',
            'order_dir' => 'ASC',
            'include_deleted' => false,
            "count_results" => false,
            "extra_conditions" => " AND `task_id`=" . $task_id . "  AND `expense_id`=" . $exp_id . " AND  `o`.`trashed_by_id`=0 AND `o`.`archived_by_id`=0 "
        ))->objects;

    } else {
        $payment_receipts = PaymentReceipts::instance()->listing(array(
            "fire_additional_data_hook" => false,
            "ignore_context" => true,
            'order' => '`o`.`name`',
            'order_dir' => 'ASC',
            'include_deleted' => false,
            "count_results" => false,
            "show_only_member_objects" => true,
            "member_ids" => array($project_member_id),
            "extra_conditions" => " AND `task_id`=0 AND `expense_id`=" . $exp_id . " AND `o`.`trashed_by_id`=0 AND `o`.`archived_by_id`=0 "
        ))->objects;
    
    }

    foreach($payment_receipts as $actual_expense) {
        $new_actual_expense_data = array(
            'expense_name' => $actual_expense->getObjectName(),
            'date' => $actual_expense->getDate(),
            'description' => $actual_expense->getDescription(),
            'executed_cost' => $actual_expense->getAmount(),
            'executed_price' => $actual_expense->getTotalPrice(),
        );

        $new_budgeted_expense['actual_expenses'][] = $new_actual_expense_data;
        $new_budgeted_expense['executed_cost'] += $new_actual_expense_data['executed_cost'];
        $new_budgeted_expense['executed_price'] += $new_actual_expense_data['executed_price'];
    }

    // Calculate remaining expenses
    $new_budgeted_expense['remaining_cost'] = $new_budgeted_expense['estimated_cost'] - $new_budgeted_expense['executed_cost'];
    $new_budgeted_expense['remaining_price'] = $new_budgeted_expense['estimated_price'] - $new_budgeted_expense['executed_price'];
}

