<?php

function build_collections_information_report_excel($data, $report_data){
    $excel = build_all_collections_information_report_excel($data, $report_data);
    return $excel;
}

function build_all_collections_information_report_excel($data, $report_data){
    $excel = array();
    $contacts = $data['contacts'];
    foreach($contacts as $contact_id => $contact_info){
        $excel[] = build_contact_info_excel_collec_info($contact_info, $report_data);
        $project_and_invoices_info = $contact_info['project_and_invoices_info'];
        
        $countCurrency = count($project_and_invoices_info['contact_totals']['currencies']);
        if($countCurrency > 1):
            for ($x = 1; $x <= $countCurrency; $x++) {
                $tmp = array();
                $tmp['currencies'][$project_and_invoices_info['contact_totals']['currencies'][$x]['currency_id']] = $project_and_invoices_info['contact_totals']['currencies'][$x];
                if($x > 1) $tmp['same_total_line'] = true;
                $excel[] = build_contact_totals_excel_coll_info($contact_info['contact_name'], $tmp, $report_data);
            }
        else:
            $excel[] = build_contact_totals_excel_coll_info($contact_info['contact_name'], $project_and_invoices_info['contact_totals'], $report_data);
        endif;
    }

    $excel[] = build_report_totals_excel_coll_info($data['totals'], $data['distribution'], $report_data);

    return $excel;
}


function build_contact_info_excel_collec_info($contact_info, $report_data){
    // Get custom property 'project_manager'
    $group_by = 'project';
    $has_selected_pm_id = array_var($report_data, 'has_selected_pm_id') == 'false' ? false : true;
    $project_ot = ObjectTypes::findByName('project');
    $cp_project_manager = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="project_manager"'));
    $pm_label = $cp_project_manager instanceof CustomProperty ? $cp_project_manager->getname() : lang('project manager');
    $project_columns = array_var($report_data, 'project_columns', array('full_project_name'));
    $project_ot = ObjectTypes::findByName('project');
    
    
    // Project Manager name
   /*  $header = array($pm_name, '', '', '', '', '', '', '', '');
    foreach($project_columns as $column){
        $header[] = '';
    }
    $header[] = '';
    if($report_data['include_current_invoices']) {
        $header[] = '';
    } */

    // Header columns
    $data = array();
    $header_row_1 = array(lang('invoice'), 'client_name');
    foreach($project_columns as $c_name){
        if($c_name == 'client_name') {
            $client_dim = Dimensions::instance()->findByCode('customers');
            $col_name = $client_dim->getName();
        } else if($c_name == 'full_project_name') {
            $project_dim = Dimensions::instance()->findByCode('customer_project');
            $col_name = $project_dim->getName();
        } else if($c_name == 'project_phase') {
            $project_phase_dim = Dimensions::instance()->findByCode('project_phases');
            $col_name = $project_phase_dim->getName();
        } else if($c_name == 'project_manager') {
            $col_name = $cp_project_manager->getName();
        } else if($c_name == 'date_received') {
            $cp_report_sent = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="date_received"'));
            $col_name = $cp_report_sent->getName();
        } else if($c_name == 'billing_contact') {
            $cp_billing_contact = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="billing_contact"'));
            $col_name = $cp_billing_contact->getName();
        } else if($c_name == 'referred_by') {
            $cp_invoice_sent = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="referred_by"'));
            $col_name = $cp_invoice_sent->getName();
        } else if($c_name == 'project_location') {
            $cp_invoice_sent = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="project_location"'));
            $col_name = $cp_invoice_sent->getName();
        } else if($c_name == 'last_follow_up_date') {
            $cp_invoice_sent = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="last_follow_up_date"'));
            $col_name = $cp_invoice_sent->getName();
        } else {
            $col_name = lang($c_name);
        }
        $header_row_1[] = $col_name;
    }
    $header_row_2 = array(lang('payment_received'), lang('due_date'), lang('paid_amount'), lang('balance'));
    if(isset($report_data['include_current_invoices'])) $header_row_2[] = lang('current not due');
    $header_row_3 = array(lang('1-30'),  lang('31-60'),  lang('61-90'),  lang('91-120'),  lang('over 120'));
    $header = array_merge($header_row_1, $header_row_2, $header_row_3);
    //$data[] = $header_row;

    // Build projects
    $project_and_invoices_info = $contact_info['project_and_invoices_info'];
    foreach($project_and_invoices_info['projects'] as $project_id => $project_info){

        $billing_contact_data = array_var($project_info['project_info'], 'billing_contact_info', null);
        $has_selected_pm_id = array_var($report_data, 'has_selected_pm_id');
        $group_by = array_var($report_data, 'group_by', 'project');

		// Show client name if exist
        $project_rows = array();
       
        // Render invoices
        foreach($project_info['invoices'] as $invoice){
            $data[] = build_invoice_info_excel_coll_info($invoice, $project_info, $report_data);
        }
    
        
    }

    return array('headers' => $header, 'data' => $data);
}


function build_invoice_info_excel_coll_info($invoice, $project_info, $report_data){
    $project_columns = array_var($report_data, 'project_columns', array('full_project_name'));
    $currency = Currencies::instance()->getCurrency($invoice['currency_id']);
    $currency_sym = $currency->getSymbol();

    $current_time_val = $invoice['current_time'] > 0 ? format_money_amount($invoice['current_time'], $currency_sym) : '';
    $first_month_val = $invoice['first_month'] > 0 ? format_money_amount($invoice['first_month'], $currency_sym) : '';
    $second_month_val = $invoice['second_month'] > 0 ? format_money_amount($invoice['second_month'], $currency_sym) : '';
    $third_month_val = $invoice['third_month'] > 0 ? format_money_amount($invoice['third_month'], $currency_sym) : '';
    $fourth_month_val = $invoice['fourth_month'] > 0 ? format_money_amount($invoice['fourth_month'], $currency_sym) : '';
    $greater_than_four_month_val = $invoice['greater_than_four_month'] > 0 ? format_money_amount($invoice['greater_than_four_month'], $currency_sym) : '';

    // Build invoice row
    $invoice_row = array($invoice['invoice_number']);
    $invoice_row[] = $invoice['client_name'];
    foreach($project_columns as $column) {
        $invoice_row[] = $project_info['project_info'][$column];
    }
    if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
        $invoice_row[] = $invoice['payment_received'];
    }
    $invoice_row[] = $invoice['date'];
    $invoice_row[] = format_money_amount($invoice['paid_amount'], $currency_sym);
    $invoice_row[] = format_money_amount($invoice['total'], $currency_sym);
    if(isset($report_data['include_current_invoices'])) {
        $invoice_row[] = $current_time_val;
    }
    $invoice_row[] = $first_month_val;
    $invoice_row[] = $second_month_val;
    $invoice_row[] = $third_month_val;
    $invoice_row[] = $fourth_month_val;
    $invoice_row[] = $greater_than_four_month_val;
    return $invoice_row;
}

function build_project_total_excel_coll_info($project_info, $report_data){
    $project_totals = $project_info['project_totals'];
    $project_columns = array_var($report_data, 'project_columns', array('full_project_name'));
    // Build project totals row
    $project_total_title = lang('total for').' '.$project_info['project_info']['project_id'];
    

    if (isset($project_totals['same_total_line'])) :
        $project_totals_row = array('', '');
    else :
        $project_totals_row = array($project_total_title, '');
    endif;
    $project_totals_row[] = '';
    foreach($project_columns as $column) {
        $project_totals_row[] = '';
    }
    if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
        $project_totals_row[] = '';
    }
    foreach($project_totals['currencies'] as $key => $value):
        $currency = Currencies::instance()->getCurrency($value['currency_id']);
        $project_totals_row[] = format_money_amount($value['paid_amount'], $currency->getSymbol());
        $project_totals_row[] = format_money_amount($value['total'], $currency->getSymbol());
        if(isset($report_data['include_current_invoices'])) {
            $project_totals_row[] = format_money_amount($value['current_time'], $currency->getSymbol());
        }
        $project_totals_row[] = format_money_amount($value['first_month'], $currency->getSymbol());
        $project_totals_row[] = format_money_amount($value['second_month'], $currency->getSymbol());
        $project_totals_row[] = format_money_amount($value['third_month'], $currency->getSymbol());
        $project_totals_row[] = format_money_amount($value['fourth_month'], $currency->getSymbol());
        $project_totals_row[] = format_money_amount($value['greater_than_four_month'], $currency->getSymbol());
    endforeach;
    
    return $project_totals_row;
}

function build_contact_totals_excel_coll_info($contact_name, $contact_totals, $report_data){
    $project_columns = array_var($report_data, 'project_columns', array('full_project_name'));
    $group_by = array_var($report_data, 'group_by', 'project');
    $has_selected_pm_id = array_var($report_data, 'has_selected_pm_id') == 'false' ? false : true;
    $excel = "";
    // Build project totals row
    if(!$has_selected_pm_id && $group_by == 'report_sent') {
        if(strpos($contact_name, lang('without')) === false) {
            $dt = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $contact_name);
            $contact_name = format_date($dt);
        }
    } 
    $contact_total_title = lang('totals');

    if(isset($contact_totals['same_total_line'])):
        $contact_totals_row = array('', '');
    else:
        $contact_totals_row = array($contact_total_title, '');
    endif;
    $contact_totals_row[] = '';
    foreach($project_columns as $column) {
        $contact_totals_row[] = '';
    }
    if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
        $contact_totals_row[] = '';
    }

    foreach ($contact_totals['currencies'] as $key => $value) :
        $currency = Currencies::instance()->getCurrency($value['currency_id']);
        $contact_totals_row[] = format_money_amount($value['paid_amount'], $currency->getSymbol());
        $contact_totals_row[] = format_money_amount($value['total'], $currency->getSymbol());
        if(isset($report_data['include_current_invoices'])) {
            $contact_totals_row[] = format_money_amount($value['current_time'], $currency->getSymbol());
        }
        $contact_totals_row[] = format_money_amount($value['first_month'], $currency->getSymbol());
        $contact_totals_row[] = format_money_amount($value['second_month'], $currency->getSymbol());
        $contact_totals_row[] = format_money_amount($value['third_month'], $currency->getSymbol());
        $contact_totals_row[] = format_money_amount($value['fourth_month'], $currency->getSymbol());
        $contact_totals_row[] = format_money_amount($value['greater_than_four_month'], $currency->getSymbol());
    endforeach;
    
    return array('totals' => $contact_totals_row);
}


function build_report_totals_excel_coll_info($totals, $distribution_totals, $report_data){
   
    $project_columns = array_var($report_data, 'project_columns', array('full_project_name'));
    $totals_excel = array();
    // Main header
    $header = array(lang('totals'), '', '', '', '', '', '', '', '');
    if(isset($report_data['include_current_invoices'])) $header[] = '';
    foreach($project_columns as $column) {
        $header[] = '';
    }
    $header[] = '';
    if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
        $header[] = '';
    }

    // Build totals header
    $total_header_row_1 = array('', '', '');
    foreach($project_columns as $column) {
        $total_header_row_1[] = '';
    }
    if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
        $total_header_row_1[] = '';
    }
    $total_header_row_2 = array(lang('paid_amount'), lang('balance'));
    if(isset($report_data['include_current_invoices'])) $total_header_row_2[] = lang('current not due');
    $total_header_row_3 = array(lang('1-30'), lang('31-60'), lang('61-90'), lang('91-120'), lang('over 120'));
    $total_header_row = array_merge($total_header_row_1, $total_header_row_2, $total_header_row_3);
    $totals_excel[] = $total_header_row;

     // Build totals row
    $total_title = lang('final totals');
    $tt = count($totals['currencies']);

    for ($x = 1; $x <= $tt; $x++) {

        $currency = Currencies::instance()->getCurrency($totals['currencies'][$x]['currency_id']);

        if ($x > 1) :
            $totals_row = array('', '', '');
        else :
            $totals_row = array($total_title, '', '');
        endif;
        foreach($project_columns as $column) {
            $totals_row[] = '';
        }
        if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
            $totals_row[] = '';
        }

        $totals_row[] = format_money_amount($totals['currencies'][$x]['paid_amount'], $currency->getSymbol());
        $totals_row[] = format_money_amount($totals['currencies'][$x]['total'], $currency->getSymbol());
        if(isset($report_data['include_current_invoices'])) {
            $totals_row[] = format_money_amount($totals['currencies'][$x]['current_time'], $currency->getSymbol());
        }
        $totals_row[] = format_money_amount($totals['currencies'][$x]['first_month'], $currency->getSymbol());
        $totals_row[] = format_money_amount($totals['currencies'][$x]['second_month'], $currency->getSymbol());
        $totals_row[] = format_money_amount($totals['currencies'][$x]['third_month'], $currency->getSymbol());
        $totals_row[] = format_money_amount($totals['currencies'][$x]['fourth_month'], $currency->getSymbol());
        $totals_row[] = format_money_amount($totals['currencies'][$x]['greater_than_four_month'], $currency->getSymbol());
        $totals_excel[] =  $totals_row;

    }

    $distribution_title = lang('distribution');

    $tt = count($distribution_totals['currencies']);
    for ($x = 1; $x <= $tt; $x++) {
        $currency = Currencies::instance()->getCurrency($distribution_totals['currencies'][$x]['currency_id']);


        if ($x > 1) :
            $distribution_row = array('', '', '');
        else :
            $distribution_row = array($distribution_title, '', '');
        endif;
        foreach($project_columns as $column) {
            $distribution_row[] = '';
        }
        if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
            $distribution_row[] = '';
        }

        $distribution_row[] = '';
        $distribution_row[] = format_money_amount($distribution_totals['currencies'][$x]['total'], '', 0) . ' %';
        if(isset($report_data['include_current_invoices'])) {
            $distribution_row[] = format_money_amount($distribution_totals['currencies'][$x]['current_time'], '', 0) . ' %';
        }
        $distribution_row[] = format_money_amount($distribution_totals['currencies'][$x]['first_month'], '', 0) . ' %';
        $distribution_row[] = format_money_amount($distribution_totals['currencies'][$x]['second_month'], '', 0) . ' %';
        $distribution_row[] = format_money_amount($distribution_totals['currencies'][$x]['third_month'], '', 0) . ' %';
        $distribution_row[] = format_money_amount($distribution_totals['currencies'][$x]['fourth_month'], '', 0) . ' %';
        $distribution_row[] = format_money_amount($distribution_totals['currencies'][$x]['greater_than_four_month'], '', 0) . ' %';
        $totals_excel[] = $distribution_row;
    }

    return array('headers' => $header, 'data' => $totals_excel);
}
