<?php

Env::useHelper('aging_invoices_report_functions', 'project_reports');

function get_all_collections_information_report_data(&$report_data) {
    $project_ot = ObjectTypes::findByName('project');
    $enabled_dimensions = config_option('enabled_dimensions');
    $group_by = 'project';

    $result = array(
        'contacts' => array(),
        'totals' => array('currencies' => array()),
        'distribution' => array('currencies' => array())
    );
    $processed_projects = array();
    
    get_columns_for_collections_information_report($report_data, $result_data);

    // Get all projects and organized invoices
    $all_project_invoices = get_all_projects_and_organized_invoices_for_collections_information($processed_projects, $report_data);
    if(!empty($all_project_invoices['projects'])){
        $holder_name = lang('all');
        add_project_manager_collections_information_to_totals($result, $all_project_invoices, 0, $holder_name, $report_data);
    }

    // Sort currencies
    ksort($result['totals']['currencies']);

    if($result['totals']['currencies']){
        foreach($result['totals']['currencies'] as $currency_id => $curr_totals){
            $result['distribution']['currencies'][$currency_id] = array(
                'currency_id' => $currency_id,
                'total' => 100,
                'paid_amount' => 0,
                'current_time' => 0,
                'first_month' => 0,
                'second_month' => 0,
                'third_month' => 0,
                'fourth_month' => 0,
                'greater_than_four_month' => 0
            );

            // Calculate distribution for each currency
            $total = $curr_totals['total'];
            if ($total > 0) {
                $result['distribution']['currencies'][$currency_id]['current_time'] = round(($curr_totals['current_time'] / $total) * 100);
                $result['distribution']['currencies'][$currency_id]['first_month'] = round(($curr_totals['first_month'] / $total) * 100);
                $result['distribution']['currencies'][$currency_id]['second_month'] = round(($curr_totals['second_month'] / $total) * 100);
                $result['distribution']['currencies'][$currency_id]['third_month'] = round(($curr_totals['third_month'] / $total) * 100);
                $result['distribution']['currencies'][$currency_id]['fourth_month'] = round(($curr_totals['fourth_month'] / $total) * 100);
                $result['distribution']['currencies'][$currency_id]['greater_than_four_month'] = round(($curr_totals['greater_than_four_month'] / $total) * 100);
            }
        }
    }

    return $result;
}

function add_project_manager_collections_information_to_totals(&$result, $project_invoices, $pm_contact_id, $pm_name, $report_data) {
      $result['contacts'][$pm_contact_id] = array(
        'project_and_invoices_info' => $project_invoices,
        'contact_name' => $pm_name
    );
    $contact_totals = $project_invoices['contact_totals'];
    foreach($contact_totals['currencies'] as $currency_id => $curr_totals){
        if(!array_key_exists($currency_id, $result['totals']['currencies'])){
            $result['totals']['currencies'][$currency_id] = array(
                'currency_id' => $currency_id,
                'total' => 0,
                'paid_amount' => 0,
                'current_time' => 0,
                'first_month' => 0,
                'second_month' => 0,
                'third_month' => 0,
                'fourth_month' => 0,
                'greater_than_four_month' => 0
            );
        } 

        // Calculate totals for each currency
        $result['totals']['currencies'][$currency_id]['paid_amount'] += $curr_totals['paid_amount'];
        $result['totals']['currencies'][$currency_id]['total'] += $curr_totals['total'];
        $result['totals']['currencies'][$currency_id]['current_time'] += $curr_totals['current_time'];
        $result['totals']['currencies'][$currency_id]['first_month'] += $curr_totals['first_month'];
        $result['totals']['currencies'][$currency_id]['second_month'] += $curr_totals['second_month'];
        $result['totals']['currencies'][$currency_id]['third_month'] += $curr_totals['third_month'];
        $result['totals']['currencies'][$currency_id]['fourth_month'] += $curr_totals['fourth_month'];
        $result['totals']['currencies'][$currency_id]['greater_than_four_month'] += $curr_totals['greater_than_four_month'];
    }
}

function get_project_manager_collections_information($pm_contact_id, $cp_project_manager, $report_data){
    $project_ot = ObjectTypes::findByName('project');
    $project_manager_dimension = Dimensions::instance()->findByCode('project_managers');
    $pm_dim_id = $project_manager_dimension->getId();
    $enabled_dimensions = config_option('enabled_dimensions');
    // Get custom property 'project_manager'
    $cp_project_manager = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="project_manager"'));
    // Get association between project and project manager
    $proj_proj_manager_association = DimensionMemberAssociations::instance()->getAssociationByCode('project_project_manager');


    if ($project_manager_dimension instanceof Dimension && $proj_proj_manager_association instanceof DimensionMemberAssociation && in_array($pm_dim_id, $enabled_dimensions)) {
        $mpms = MemberPropertyMembers::instance()->findAll(array('conditions' => '`association_id`='.$proj_proj_manager_association->getId().' AND `property_member_id`='.$pm_contact_id));
        $member_project_ids = array_map('get_project_member_ids', $mpms);
    } else if ($cp_project_manager instanceof CustomProperty) {
         // Get project ids under project manager
        $project_cp_values = CustomPropertyValues::instance()->findAll((array('conditions' => '`custom_property_id`='.$cp_project_manager->getId().' AND `value`='.$pm_contact_id)));

        if(empty($project_cp_values)) return array();

        $object_project_ids = array_map('get_cp_value_object_id', $project_cp_values);
        $member_project_ids = array_map('get_object_member_id',  $object_project_ids);
    }

    $grouped_by_project_invoices = collect_and_organize_invoices_for_projects_collections_information($member_project_ids, $report_data);
    
    return $grouped_by_project_invoices;
}

function get_all_projects_and_organized_invoices_for_collections_information($processed_projects, $report_data) {
    $params = array(
        'include_subproject_info' => 'checked',
    );
    $active_context_project_member_ids = get_project_member_ids_for_project_manager_report(false, $params);
    $grouped_by_project_invoices = collect_and_organize_invoices_for_projects_collections_information($active_context_project_member_ids, $report_data);

    return $grouped_by_project_invoices;
}

function collect_and_organize_invoices_for_projects_collections_information($member_project_ids, $report_data){
    // Add empty project to get unassigned invoices
    $member_project_ids[] = 0;
    $group_by = 'project';
    $grouped_by_project_invoices = array(
        'projects' => array(),
        'contact_totals' => array(
           'currencies' => array(),
        )
    );
       
    $object_members_table_name = ObjectMembers::instance()->getTableName();
    $now = DateTimeValueLib::now()->add('d', '-1')->endOfDay();
    $seen_invoices_ids = array();

    foreach($member_project_ids as $project_id){
        $member = Members::instance()->findById($project_id);
        $has_project_as_parent = member_has_project_as_parent($member);
        if((!($member instanceof Member) || $has_project_as_parent) && $project_id != 0) continue;

        $project_invoices = get_invoices_by_project_id_coll_info($project_id, $object_members_table_name, $seen_invoices_ids);
        if(empty($project_invoices)) continue;
        $processed_invoices = array();
        $project_totals = array(
            'total' => 0,
            'paid_amount' => 0,
            'first_month' => 0,
            'second_month' => 0,
            'third_month' => 0,
            'fourth_month' => 0,
            'greater_than_four_month' => 0
        );

        $project_totals = array('currencies' => array());
        foreach($project_invoices as $invoice){
            // Skip invoices that have already been seen
            if(in_array($invoice['id'], $seen_invoices_ids)) continue;
            $seen_invoices_ids[] = $invoice['id'];

            // Get invoice number
            $invoice_number = ($invoice['invoice_number'] > 0) ? $invoice['series'] . " " . $invoice['invoice_number'] : lang('not assigned');
            $currency_id = $invoice['currency_id'];
            if($currency_id == 0) {
                $currency_info = Currencies::getDefaultCurrencyInfo();
                $currency_id = $currency_info['id'];
            }
            $due_date_sql = $invoice['expiration_date'];

            // Use 'due_amount' if invoice_payment_receipts plugin is active, otherwise use 'total'
            $due_amount = Plugins::instance()->isActivePlugin('invoice_payment_receipts') ? $invoice['due_amount'] : $invoice['total'];

            $payment_received = '';
            if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
                Env::useHelper('functions', 'invoice_payment_receipts');
                $payment_received_date = get_latest_payment_received($invoice['id']);
                $payment_received = $payment_received_date ? format_date($payment_received_date) : '';
            }
            

            // Skip invoices with zero due amount
            if($due_amount == 0) continue;

            // Get invoice date
            if($due_date_sql == '0000-00-00' || $due_date_sql == '') {
                if(!array_var($report_data, 'include_current_invoices')) continue;
                $formatted_invoice_date = '';
                $current_time = $due_amount;
                $first_month = 0;
                $second_month = 0;
                $third_month = 0;
                $fourth_month = 0;
                $greater_than_four_month = 0;
            } else {
                // Get invoice date datetime value
                $invoice_date = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $due_date_sql);
                // Advance invoice date to user timezone
                $invoice_date->advance(-1 * logged_user()->getUserTimezoneValue(), true);
                // Skip invoices that are not due yet
                if(!array_var($report_data, 'include_current_invoices') && $now < $invoice_date) continue;

                $formatted_invoice_date = format_date($invoice_date);
                // Find invoice age
                $current_time = in_current_time($invoice_date) ? $due_amount : 0;
                $first_month = in_first_month($invoice_date) ? $due_amount : 0;
                $second_month = in_second_month($invoice_date) ? $due_amount : 0;
                $third_month = in_third_month($invoice_date) ? $due_amount : 0;
                $fourth_month = in_fourth_month($invoice_date) ? $due_amount : 0;
                $greater_than_four_month = greater_than_four_month($invoice_date) ? $due_amount : 0;
            }

            $processed_invoices[] = array(
                'invoice_number' => $invoice_number,
                'currency_id' => $currency_id,
                'date' => $formatted_invoice_date,
                'payment_received' => $payment_received,
                'client_name' => array_var($invoice, 'customer_name'),
                'total' => $due_amount,
                'paid_amount' => $invoice['paid_amount'],
                'current_time' => $current_time,
                'first_month' => $first_month,
                'second_month' => $second_month,
                'third_month' => $third_month,
                'fourth_month' => $fourth_month,
                'greater_than_four_month' => $greater_than_four_month
            );

            // Create project totals for each currency
            if(!array_key_exists($currency_id, $project_totals['currencies'])){
                $project_totals['currencies'][$currency_id] = array(
                    'currency_id' => $currency_id,
                    'total' => 0,
                    'paid_amount' => 0,
                    'current_time' => 0,
                    'first_month' => 0,
                    'second_month' => 0,
                    'third_month' => 0,
                    'fourth_month' => 0,
                    'greater_than_four_month' => 0
                );
            }

            // Add to project totals
            $project_totals['currencies'][$currency_id]['total'] += $due_amount;
            $project_totals['currencies'][$currency_id]['paid_amount'] += $invoice['paid_amount'];
            $project_totals['currencies'][$currency_id]['current_time'] += $current_time;
            $project_totals['currencies'][$currency_id]['first_month'] += $first_month;
            $project_totals['currencies'][$currency_id]['second_month'] += $second_month;
            $project_totals['currencies'][$currency_id]['third_month'] += $third_month;
            $project_totals['currencies'][$currency_id]['fourth_month'] += $fourth_month;
            $project_totals['currencies'][$currency_id]['greater_than_four_month'] += $greater_than_four_month;

            // Create contact totals for each currency
            if(!array_key_exists($currency_id, $grouped_by_project_invoices['contact_totals']['currencies'])){
                $grouped_by_project_invoices['contact_totals']['currencies'][$currency_id] = array(
                    'currency_id' => $currency_id,
                    'total' => 0,
                    'paid_amount' => 0,
                    'current_time' => 0,
                    'first_month' => 0,
                    'second_month' => 0,
                    'third_month' => 0,
                    'fourth_month' => 0,
                    'greater_than_four_month' => 0
                );
            }

            // Add to project manager totals
            $grouped_by_project_invoices['contact_totals']['currencies'][$currency_id]['total'] += $due_amount;
            $grouped_by_project_invoices['contact_totals']['currencies'][$currency_id]['paid_amount'] += $invoice['paid_amount'];
            $grouped_by_project_invoices['contact_totals']['currencies'][$currency_id]['current_time'] += $current_time;
            $grouped_by_project_invoices['contact_totals']['currencies'][$currency_id]['first_month'] += $first_month;
            $grouped_by_project_invoices['contact_totals']['currencies'][$currency_id]['second_month'] += $second_month;
            $grouped_by_project_invoices['contact_totals']['currencies'][$currency_id]['third_month'] += $third_month;
            $grouped_by_project_invoices['contact_totals']['currencies'][$currency_id]['fourth_month'] += $fourth_month;
            $grouped_by_project_invoices['contact_totals']['currencies'][$currency_id]['greater_than_four_month'] += $greater_than_four_month;

            // Sort currencies
            ksort($project_totals['currencies']);
            ksort($grouped_by_project_invoices['contact_totals']['currencies']);
        }
        

        if(empty($processed_invoices)) continue;
        // Get project info
        $project_member = Members::instance()->findById($project_id);

        if($project_member instanceof Member){
			Env::useHelper('functions', 'crpm');

            $proj_info = array(
                'invoices' => $processed_invoices,
                'project_totals' => $project_totals,
                'project_info' => get_project_info_collections_info($project_member, $report_data),
            );
            $grouped_by_project_invoices['projects'][$project_id] = $proj_info;
        }
        if ($project_id == 0) {
            $proj_info = array(
                'invoices' => $processed_invoices,
                'project_totals' => $project_totals,
                'project_info' => array(),
            );
            $grouped_by_project_invoices['projects'][$project_id] = $proj_info;
        }

    }

    return $grouped_by_project_invoices;
}

function get_collections_information_grouped_by_project_report_sent($report_data) {
    $active_context_project_member_ids = get_project_member_ids_for_project_manager_report(false);
    $grouped_by_report_sent = collect_and_organize_invoices_for_projects_collections_information($active_context_project_member_ids, $report_data);
    sort_grouped_by_report_sent($grouped_by_report_sent);

    return $grouped_by_report_sent;
}

function get_invoices_by_project_id_coll_info($project_id, $object_members_table_name, $seen_invoices_ids){
    if ($project_id == 0) {
        return get_unassigned_invoices_for_collections_information($seen_invoices_ids);
    }
    $join_params = array(
        'table' => $object_members_table_name,
        'join_type' => 'INNER',
        'jt_field' => 'object_id',
        'e_field' => 'object_id',
    );

    $extra_conditions = " AND `jt`.`member_id`=".$project_id.' AND `e`.`status`!="confirmed" AND `e`.`status`!="canceled" ';
	$invoices_res = IncomeInvoices::instance()->listing(array( 
			'raw_data' => true,
			'join_params' => $join_params,
			'extra_conditions' => $extra_conditions,
			'order' => '`e`.`expiration_date`',
			'order_dir' => 'ASC',
	));

    $invoices = $invoices_res->objects;

    return $invoices;
}

function get_unassigned_invoices_for_collections_information($seen_invoices_ids) {
    $invoice_ids_cond = count($seen_invoices_ids) > 0 ? ' AND `e`.`object_id` NOT IN (' . implode(',', $seen_invoices_ids) . ')' : '';
    $extra_conditions = $invoice_ids_cond . ' AND `e`.`status`!="confirmed" AND `e`.`status`!="canceled" ';
	$invoices_res = IncomeInvoices::instance()->listing(array( 
			'raw_data' => true,
			'join_params' => $join_params,
			'extra_conditions' => $extra_conditions,
			'order' => '`e`.`expiration_date`',
			'order_dir' => 'ASC',
	));

    $invoices = $invoices_res->objects;

    return $invoices;
}

function get_project_info_collections_info(Member $project_member, $report_data) {

    $project_prefix_object = ObjectPrefixes::instance()->findById($project_member->getObjectId());
	if($project_prefix_object){
		$project_id = $project_prefix_object->getPrefix();
	} else {
		$pot = ObjectTypes::findByName('project');
		$project_id_cp = CustomProperties::getCustomPropertyByCode($pot->getId(), 'project_id');
		$project_id = '';
		if ($project_id_cp instanceof CustomProperty) {
			$cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $project_id_cp->getId());
			if ($cp_val instanceof CustomPropertyValue) {
                $project_id = $cp_val->getValue();
			}
		}
	}

    $project_name = $project_member->getName();
    $full_project_name = $project_id.' - '.$project_member->getName();


    // Get project 'project phase' assoc member id
    $project_phase_name = '';
    $project_phase_dim = Dimensions::instance()->findByCode('project_phases');
    $project_phase_id = get_associated_status_member_id($project_member, $project_phase_dim, null, false, 'project_project_phase');
    $projec_phase_member = Members::instance()->findOne(array('conditions' => '`id`='.$project_phase_id));
    if ($projec_phase_member instanceof Member) {
        $project_phase_name = $projec_phase_member->getName();
    }
    
    // Referred by custom property
    $referred_by = '';
    $referred_by_val = get_project_member_cp_val($project_member, $report_data, 'referred_by', false);
    $referred_contact = Contacts::instance()->findById($referred_by_val);
    if($referred_contact instanceof Contact) {
        $referred_by = $referred_contact->getName();
    }

    $billing_contact_name = '';
    $billing_contact_info = get_billing_contact_info($project_member, $report_data, false);
    if($billing_contact_info) {
        $billing_contact_name = $billing_contact_info['name'];
    }

	$result = array(
        'project_id' => $project_id,
        'project_name' => $project_name,
        'full_project_name' => $full_project_name,
        'billing_contact' => $billing_contact_name,
        'project_location' => get_project_member_cp_val($project_member, $report_data, 'project_location', false),
        'project_phase' => $project_phase_name,
        'referred_by' => $referred_by,
        'last_follow_up_date' => get_project_member_cp_val($project_member, $report_data, 'last_follow_up_date', false),
        'date_received' => get_project_member_cp_val($project_member, $report_data, 'date_received', false),
        'payment_received' => get_project_member_cp_val($project_member, $report_data, 'payment_received', false)
    );

    return $result;
}

function get_columns_for_collections_information_report(&$report_data, &$result_data) {
    $project_ot = ObjectTypes::findByName('project');
    $columns = array('project_id');

    // project location cp
    $project_location_cp = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='. $project_ot->getId().' AND `code`="project_location"'));
    if($project_location_cp instanceof CustomProperty) {
        $columns[] = 'project_location';
    }

    // Billing contact cp
    $billing_contact_cp = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='. $project_ot->getId().' AND `code`="billing_contact"'));
    if($billing_contact_cp instanceof CustomProperty) {
        $columns[] = 'billing_contact';
    }

    // Project phase member assoc
    $project_phase_assoc = DimensionMemberAssociations::instance()->getAssociationByCode('project_project_phase');
    if($project_phase_assoc instanceof DimensionMemberAssociation) {
        $columns[] = 'project_phase';
    }

    // Referred by cp
    $referred_by_cp = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='. $project_ot->getId().' AND `code`="referred_by"'));
    if($referred_by_cp instanceof CustomProperty) {
        $columns[] = 'referred_by';
    }


    // Last follow up date cp
    $last_follow_up_date_cp = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='. $project_ot->getId().' AND `code`="last_follow_up_date"'));
    if($last_follow_up_date_cp instanceof CustomProperty) {
        $columns[] = 'last_follow_up_date';
    }

    // Invoice sent cp
    $date_received_cp = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='. $project_ot->getId().' AND `code`="date_received"'));
    if($date_received_cp instanceof CustomProperty) {
        $columns[] = 'date_received';
    }

    /* // Payment received cp
    if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
        $columns[] = 'payment_received';
    }
 */
    $report_data['project_columns'] = $columns;
    $result_data['project_columns'] = $columns;
} 
