<?php

function build_contract_status_report_excel($data, $report_data) {

    $excel = array();

    $excel[] = build_main_header_contract_status_report_excel($data, $report_data);
    $excel[] = build_overall_totals_contract_status_report_excel($data, $report_data);
    $projects = array_var($data, 'projects', array());
    foreach($projects as $proj_data) {
        $excel[] = build_projects_contract_status_report_excel($proj_data, $data);
    }   

    return $excel;
}

function build_projects_contract_status_report_excel($proj_data, $data) {
    $columns = array_var($data, 'columns', array());
    $currency_sym = config_option('currency_code');
    $name_columns = array('name', 'contract_type', 'project_manager', 'project_phase', 'project_type');
    $time_columns = array('executed_labor_minutes');

    $excel_table = array(
		'data' => array()
    );

    $header_row = array();

    foreach($columns as $column) {
        if(in_array($column, $time_columns)) {
            $val = format_amount($proj_data[$column]/60);
        } else if(in_array($column, $name_columns)) {
            $val = $proj_data[$column];
        } else {
            $val = format_money_amount($proj_data[$column], $currency_sym);
        }
        $header_row[] = $val;
    }

    $excel_table['data'][] = $header_row;

    return $excel_table;
}

function build_main_header_contract_status_report_excel($data, $report_data) {
    $columns = array_var($data, 'columns', array());

    $excel_table = array(
		'main_header' => array('')
    );

    foreach($columns as $column) {
        if($column == 'name') continue;
        if($column == 'contract_type') {
            $column_name = $data['contract_type_cp_name'];
        } else if($column == 'project_manager') {
            $column_name = $data['project_manager_dim_name'];
        } else if($column == 'project_phase') {
            $column_name = $data['project_phase_dim_name'];
        } else if($column == 'project_type') {
            $column_name = $data['project_type_dim_name'];
        } else if($column == 'budget_total') {
            $column_name = $data['budget_total_cp_name'];
        } else if ($column == 'revised_budget_total') {
            $column_name = $data['revised_budget_total_cp_name'];
        } else if ($column == 'executed_cost') {
            $column_name = lang('executed cost');
        } else if ($column == 'invoiced_subtotal') {
            $column_name = lang('invoiced');
        } else if ($column == 'invoiced_paid') {
            $column_name = lang('invoiced paid');
        } else if ($column == 'remaining_budgeted_vs_contract') {
            $column_name = lang('remaining budgeted vs contract contract_status_report');
        } else {
            $column_name = lang($column);
        }

       $excel_table['main_header'][] = $column_name;
    }

    return $excel_table;
}

function build_overall_totals_contract_status_report_excel($data, $report_data) {
    $columns = array_var($data, 'columns', array());
    $totals = array_var($data, 'totals', array());
    $currency_sym = config_option('currency_code');
    $name_columns = array('name', 'contract_type', 'project_manager', 'project_phase', 'project_type');
    $time_columns = array('executed_labor_minutes');

    $excel_table = array(
        'totals' => array(lang('totals'))
    );

    foreach($columns as $column) {
        if($column == 'name') continue;
        if(in_array($column, $time_columns)) {
            $val = format_amount($totals[$column]/60);
        } else if(in_array($column, $name_columns)) {
            $val = '';
        } else {
            $val = format_money_amount($totals[$column], $currency_sym);
        }
        $excel_table['totals'][] = $val;
    }

    return $excel_table;
}
