<?php

function get_main_contract_status_report_data($report_data) {

    $project_ot = ObjectTypes::instance()->findOne(array('conditions' => "`name` = 'project'"));
    $project_ot_id = $project_ot->getId();

    $filtered_project_member_ids = get_contract_status_project_member_ids_filtered_by_date($report_data);

    $result_data = array(
        'projects' => array(),
        'totals' => array()
    );

    if(empty($filtered_project_member_ids)) {
        return $result_data;
    }

    // Get columns for report
    get_columns_for_contract_status_report($report_data, $result_data);

    // Get executed cost for each project member
    $all_member_totals = get_member_total_executed_cost_for_contract_status_report($report_data, $filtered_project_member_ids);

    // Get project data for each project member
    foreach($filtered_project_member_ids as $project_id) {
        $project_data = collect_project_info_for_contract_status_report($report_data, $project_id, $all_member_totals);
        if(empty($project_data)) {
            continue;
        }
        $result_data['projects'][$project_data['name']] = $project_data;
    }

    // Calculate overall totals
    $result_data['totals'] = calculate_totals_for_contract_status_report($result_data['projects']);

    return $result_data;
}


function get_columns_for_contract_status_report(&$report_data, &$result_data) {
    $cp_object_type_id = ObjectTypes::instance()->findOne(array('conditions' => "`name` = 'project'"))->getId();
    $enabled_dimensions = config_option('enabled_dimensions');
    // Name column
    $columns = array('name');

    // 'contract type' cp column
    $contract_type_cp = CustomProperties::instance()->findOne(array('conditions' => '`object_type_id`='.$cp_object_type_id. ' AND `code`="contract_type"'));
    if ($contract_type_cp instanceof CustomProperty) {
        $columns[] = 'contract_type';
        $report_data['contract_type_cp_id'] = $contract_type_cp->getId();
        $result_data['contract_type_cp_id'] = $contract_type_cp->getId();
        $result_data['contract_type_cp_name'] = $contract_type_cp->getName();
    }

    // 'Project type' member
    $project_type_dimension = Dimensions::instance()->findByCode('project_types'); 
    $ptype_dim_id = $project_type_dimension instanceof Dimension ? $project_type_dimension->getId() : 0;
    $proj_proj_type_association = DimensionMemberAssociations::instance()->getAssociationByCode('project_project_type');
    if ($project_type_dimension instanceof Dimension && $proj_proj_type_association instanceof DimensionMemberAssociation && in_array($ptype_dim_id, $enabled_dimensions)) {
        $columns[] = 'project_type';
        $report_data['project_type_dim_id'] = $project_type_dimension->getId();
        $result_data['project_type_dim_id'] = $project_type_dimension->getId();
        $result_data['project_type_dim_name'] = $project_type_dimension->getName();
    }

    // 'Project manager' member
    $project_manager_dimension = Dimensions::instance()->findByCode('project_managers');
    $pm_dim_id = $project_manager_dimension instanceof Dimension ? $project_manager_dimension->getId() : 0;
    $proj_proj_manager_association = DimensionMemberAssociations::instance()->getAssociationByCode('project_project_manager');
    if ($project_manager_dimension instanceof Dimension && $proj_proj_manager_association instanceof DimensionMemberAssociation && in_array($pm_dim_id, $enabled_dimensions)) {
        $columns[] = 'project_manager';
        $report_data['project_manager_dim_id'] = $project_manager_dimension->getId();
        $result_data['project_manager_dim_id'] = $project_manager_dimension->getId();
        $result_data['project_manager_dim_name'] = $project_manager_dimension->getName();
    }

    // 'Project phase' member
    $project_phase_dimension = Dimensions::instance()->findByCode('project_phases');
    $pp_dim_id = $project_phase_dimension instanceof Dimension ? $project_phase_dimension->getId() : 0;
    $proj_proj_phase_association = DimensionMemberAssociations::instance()->getAssociationByCode('project_project_phase');
    if ($project_phase_dimension instanceof Dimension && $proj_proj_phase_association instanceof DimensionMemberAssociation && in_array($pp_dim_id, $enabled_dimensions)) {
        $columns[] = 'project_phase';
        $report_data['project_phase_dim_id'] = $project_phase_dimension->getId();
        $result_data['project_phase_dim_id'] = $project_phase_dimension->getId();
        $result_data['project_phase_dim_name'] = $project_phase_dimension->getName();
    }


    // 'Revised budget total'
    $revised_budget_total_cp = CustomProperties::instance()->findOne(array('conditions' => '`object_type_id`='.$cp_object_type_id. ' AND `code`="revised_budget_total"'));
    if ($revised_budget_total_cp instanceof CustomProperty) {
        $columns[] = 'revised_budget_total';
        $report_data['revised_budget_total_cp_id'] = $revised_budget_total_cp->getId();
        $result_data['revised_budget_total_cp_id'] = $revised_budget_total_cp->getId();
        $result_data['revised_budget_total_cp_name'] = $revised_budget_total_cp->getName();
    }
    // Executed columns
    $columns[] = 'executed_labor_minutes';
    $columns[] = 'executed_labor_cost';
    $columns[] = 'executed_expense_cost';
    $columns[] = 'executed_cost';
    $columns[] = 'executed_price';
    // 'Invoiced subtotal'
    if(Plugins::instance()->isActivePlugin('income')) {
        $columns[] = 'invoiced_subtotal';
    }
    // 'Invoiced paid'
    if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
        $columns[] = 'invoiced_paid';
    }

    // 'Budget total'
    $budget_total_cp = CustomProperties::instance()->findOne(array('conditions' => '`object_type_id`='.$cp_object_type_id. ' AND `code`="budget_total"'));
    if ($budget_total_cp instanceof CustomProperty) {
        $columns[] = 'budget_total';
        $report_data['budget_total_cp_id'] = $budget_total_cp->getId();
        $result_data['budget_total_cp_id'] = $budget_total_cp->getId();
        $result_data['budget_total_cp_name'] = $budget_total_cp->getName();
    }

    // 'Remaining budgeted'
    if(in_array('budget_total', $columns) && Plugins::instance()->isActivePlugin('advanced_billing')) {
        $columns[] = 'remaining_budgeted_vs_contract';
    }

    $report_data['columns'] = $columns;
    $result_data['columns'] = $columns;
}

function calculate_totals_for_contract_status_report($pm_projects) {
    $totals = array();
    $exclude_columns = array('name', 'project_manager', 'project_type', 'project_phase', 'contract_type');
    foreach($pm_projects as $project) {
        foreach($project as $column => $value) {
            if(in_array($column, $exclude_columns)) {
                continue;
            }
            if(!isset($totals[$column])) {
                $totals[$column] = 0;
            }
            $totals[$column] += $value;
        }
    }
    return $totals;
}

function get_contract_status_project_member_ids_filtered_by_date($report_data){
	$project_ot = ObjectTypes::findByName('project');

	// Get start and end date SQLs
	$start_date = array_var($report_data, 'start_date');
	$end_date = array_var($report_data, 'end_date');
	$sql_st_date = $start_date->toMySQL();
	$sql_end_date = $end_date->toMySQL();

	// Get project member ids assigned to timeslots in the given date range
	$time_sql = "SELECT DISTINCT(m.id)
			FROM ".TABLE_PREFIX."timeslots t INNER JOIN ".TABLE_PREFIX."object_members om ON t.object_id=om.object_id
            INNER JOIN ".TABLE_PREFIX."objects o ON t.object_id=o.id
			INNER JOIN ".TABLE_PREFIX."members m ON om.member_id=m.id
			WHERE o.trashed_by_id=0
            AND t.start_time BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
			AND m.object_type_id=".$project_ot->getId().";";
	$time_rows = DB::executeAll($time_sql);
	$time_project_member_ids = !empty($time_rows) ? array_column($time_rows, 'id') : array();

    // Rewrite start and end date SQLs to exclude timezone offset (the bug in our current system)
    $start_date = $start_date->add('s', logged_user()->getUserTimezoneValue());
    $sql_st_date = $start_date->toMySQL();
    $end_date = $end_date->add('s', logged_user()->getUserTimezoneValue());
    $sql_end_date = $end_date->toMySQL();

	// Get project member ids assigned to actual expenses in the given date range									
	$expense_sql = "SELECT DISTINCT(m.id)
				FROM ".TABLE_PREFIX."payment_receipts pr INNER JOIN ".TABLE_PREFIX."object_members om ON pr.object_id=om.object_id
                INNER JOIN ".TABLE_PREFIX."objects o ON pr.object_id=o.id
				INNER JOIN ".TABLE_PREFIX."members m ON om.member_id=m.id
				WHERE o.trashed_by_id=0
                AND pr.date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
				AND m.object_type_id=".$project_ot->getId().";";
	$expense_rows = DB::executeAll($expense_sql);
    $expense_project_member_ids = !empty($expense_rows) ? array_column($expense_rows, 'id') : array();

	// Merge time and expense project member ids										
	$combined_project_member_ids = array_unique(array_merge($time_project_member_ids, $expense_project_member_ids));

    // Return start and end date SQLs to include timezone offset (the bug in our current system)
    $start_date = $start_date->add('s', -1 *logged_user()->getUserTimezoneValue());
    $end_date = $end_date->add('s', -1 * logged_user()->getUserTimezoneValue());

	return $combined_project_member_ids;
}

function collect_project_info_for_contract_status_report($report_data, $project_id, $all_member_totals) {
    $project_member = Members::instance()->findOne(array('conditions' => '`id`='.$project_id));
    if(!$project_member instanceof Member || $project_member->getDepth() > 1) {
        return array(); 
    }
    $columns = $report_data['columns'];
    $project_data = array(
        'name' => $project_member->getDisplayName()
    );

    // Contract type
    if(in_array('contract_type', $columns)) {
        $contract_type_cp_id = $report_data['contract_type_cp_id'];
        $cpv = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $contract_type_cp_id);
        if ($cpv instanceof CustomPropertyValue) {
            $project_data['contract_type'] = $cpv->getValue();
        }
    }

    // Project type
    if(in_array('project_type', $columns)) {
        $project_type_dim_id = $report_data['project_type_dim_id'];
        $project_types_dim = Dimensions::instance()->findOne(array('conditions' => '`id`='.$project_type_dim_id));
        $project_type_id = get_associated_status_member_id($project_member, $project_types_dim, null, false, 'project_project_type');
        $projec_type_member = Members::instance()->findOne(array('conditions' => '`id`='.$project_type_id));
        if ($projec_type_member instanceof Member) {
            $project_data['project_type'] = $projec_type_member->getDisplayName();
        } else {
            $project_data['project_type'] = '';
        }
    }

    // Project manager
    if(in_array('project_manager', $columns)) {
        $project_manager_dim_id = $report_data['project_manager_dim_id'];
        $project_manager_dim = Dimensions::instance()->findOne(array('conditions' => '`id`='.$project_manager_dim_id));
        $project_manager_id = get_associated_status_member_id($project_member, $project_manager_dim, null, false, 'project_project_manager');
        $project_manager_member = Members::instance()->findOne(array('conditions' => '`id`='.$project_manager_id));
        if ($project_manager_member instanceof Member) {
            $project_data['project_manager'] = $project_manager_member->getDisplayName();
        } else {
            $project_data['project_manager'] = '';
        }
    }

    // Project phase
    if(in_array('project_phase', $columns)) {
        $project_phase_dim_id = $report_data['project_phase_dim_id'];
        $project_phase_dim = Dimensions::instance()->findOne(array('conditions' => '`id`='.$project_phase_dim_id));
        $project_phase_id = get_associated_status_member_id($project_member, $project_phase_dim, null, false, 'project_project_phase');
        $project_phase_member = Members::instance()->findOne(array('conditions' => '`id`='.$project_phase_id));
        if ($project_phase_member instanceof Member) {
            $project_data['project_phase'] = $project_phase_member->getDisplayName();
        } else {
            $project_data['project_phase'] = '';
        }
    }

   
    // Revised budget total
    if(in_array('revised_budget_total', $columns)) {
        $revised_budget_total_cp_id = $report_data['revised_budget_total_cp_id'];
        $cpv = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $revised_budget_total_cp_id);
        if ($cpv instanceof CustomPropertyValue) {
            $project_data['revised_budget_total'] = $cpv->getValue();
        }
    }

    // Invoiced values
    $invoice_ids = array();
    if(in_array('invoiced_subtotal', $columns)) {
        $invoices_data = get_invoiced_subtotal_for_contract_status_report($report_data, $project_id);
        $project_data['invoiced_subtotal'] = $invoices_data['invoiced_subtotal'];
        $invoice_ids = $invoices_data['invoice_ids'];
    }
    if(in_array('invoiced_paid', $columns)) {
        $project_data['invoiced_paid'] = get_invoiced_paid_for_contract_status_report($report_data, $invoice_ids);
    }
    
    // Get executed values
    $executed_values = get_executed_values_for_contract_status_report($report_data, $project_id);
    $project_data['executed_labor_minutes'] = $executed_values['executed_labor_minutes'];
    $project_data['executed_labor_cost'] = $executed_values['executed_labor_cost'];
    $project_data['executed_expense_cost'] = $executed_values['executed_expense_cost'];
    $project_data['executed_cost'] = $executed_values['executed_cost'];
    $project_data['executed_price'] = $executed_values['executed_price'];
    
    // Get budgeted total
    if(in_array('budget_total', $columns)) {
        $budget_total_cp_id = $report_data['budget_total_cp_id'];
        $cpv = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $budget_total_cp_id);
        if ($cpv instanceof CustomPropertyValue) {
            $project_data['budget_total'] = $cpv->getValue();
        }
    }

    // Remaining budgeted
    if(in_array('remaining_budgeted_vs_contract', $columns)) {
        $overall_executed_cost = array_var($all_member_totals, $project_id, 0);
        $project_data['remaining_budgeted_vs_contract'] = $project_data['budget_total'] - $overall_executed_cost;
    }

    return $project_data;
}

function get_executed_values_for_contract_status_report($report_data, $project_id) {
    $start_date = array_var($report_data, 'start_date');
    $end_date = array_var($report_data, 'end_date');
    $sql_st_date = $start_date->toMySQL();
    $sql_end_date = $end_date->toMySQL();

    $result = array(
        'executed_labor_minutes' => 0,
        'executed_labor_cost' => 0,
        'executed_expense_cost' => 0,
        'executed_cost' => 0,
        'executed_price' => 0
    );

    // Get tasks with manual 'estimated price' and 'fixed fee' for this project
    $task_row = DB::executeOne("
        SELECT SUM(t.earned_value) as total_price
        FROM ".TABLE_PREFIX."project_tasks t
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=t.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        AND t.due_date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
        AND t.is_calculated_estimated_price=0
        AND t.is_fixed_fee=1
        AND om.member_id=".$project_id.";
    ");

    if ($task_row) {
        $result['executed_price'] += $task_row['total_price'];
    }

    // Get time cost from timeslots for this project
    $time_row = DB::executeOne("
        SELECT SUM(ts.fixed_cost) as total_cost, SUM(ts.worked_time) as total_minutes
        FROM ".TABLE_PREFIX."timeslots ts
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=ts.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        AND ts.start_time BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
        AND om.member_id=".$project_id.";
    ");
    if ($time_row) {
        $result['executed_labor_minutes'] += $time_row['total_minutes'];
        $result['executed_labor_cost'] += $time_row['total_cost'];
        $result['executed_cost'] += $time_row['total_cost'];
    }


    // Rewrite start and end date SQLs to exclude timezone offset (the bug in our current system)
    $start_date = $start_date->add('s', logged_user()->getUserTimezoneValue());
    $sql_st_date = $start_date->toMySQL();
    $end_date = $end_date->add('s', logged_user()->getUserTimezoneValue());
    $sql_end_date = $end_date->toMySQL();

    // Get fixed fee task ids
    $fixed_fee_task_ids = array();
    if(Plugins::instance()->isActivePlugin('advanced_billing')) {
        Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
        $fixed_fee_task_ids = get_tasks_and_subtask_ids_with_manual_financial_value($project_id, 'is_calculated_estimated_price', true);
    }

    // Get time price from tasks with calculated 'estimated price' for this project
    $extra_condition = count($fixed_fee_task_ids) > 0 ? "AND ts.rel_object_id NOT IN (".implode(',', $fixed_fee_task_ids).")" : "";
    $calc_time_row = DB::executeOne("
        SELECT SUM(ts.fixed_billing) as total_price 
        FROM ".TABLE_PREFIX."timeslots ts
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=ts.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        ".$extra_condition."
        AND ts.start_time BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
        AND om.member_id=".$project_id.";
    ");
    if ($calc_time_row) {
        $result['executed_price'] += $calc_time_row['total_price'];
    }

    // Get cost for expenses for this project
    $expense_row = DB::executeOne("
        SELECT SUM(pr.amount) as total_cost 
        FROM ".TABLE_PREFIX."payment_receipts pr
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=pr.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        AND pr.date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
        AND om.member_id=".$project_id.";
    ");
    if ($expense_row) {
        $result['executed_expense_cost'] += $expense_row['total_cost'];
        $result['executed_cost'] += $expense_row['total_cost'];
    }

    // Get price for expenses for this project
    $extra_condition = count($fixed_fee_task_ids) > 0 ? "AND pr.task_id NOT IN (".implode(',', $fixed_fee_task_ids).")" : "";
    $expense_price_row = DB::executeOne("
        SELECT SUM(pr.total_price) as total_price 
        FROM ".TABLE_PREFIX."payment_receipts pr
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=pr.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        ".$extra_condition."
        AND pr.date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
        AND om.member_id=".$project_id.";
    ");
    if ($expense_price_row) {
        $result['executed_price'] += $expense_price_row['total_price'];
    }

    // Return start and end date SQLs to include timezone offset (the bug in our current system)
    $start_date = $start_date->add('s', -1 *logged_user()->getUserTimezoneValue());
    $end_date = $end_date->add('s', -1 * logged_user()->getUserTimezoneValue());

    return $result;
}

function get_invoiced_subtotal_for_contract_status_report($report_data, $project_id) {
    $start_date = array_var($report_data, 'start_date');
    $end_date = array_var($report_data, 'end_date');
    $sql_st_date = $start_date->toMySQL();
    $sql_end_date = $end_date->toMySQL();

    $result = array(
        'invoiced_subtotal' => 0,
        'invoice_ids' => array()
    );

    // Get invoices for this project
    $invoice_row = DB::executeOne("
        SELECT GROUP_CONCAT(i.object_id) as invoice_ids, SUM(i.subtotal) as subtotal
        FROM ".TABLE_PREFIX."income_invoices i
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=i.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        AND i.date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
        AND om.member_id=".$project_id.";
    ");
    if ($invoice_row) {
        $result['invoiced_subtotal'] += $invoice_row['subtotal'];
        $result['invoice_ids'] = $invoice_row['invoice_ids'] == '' ? array() : explode(',', $invoice_row['invoice_ids']);
    } 

    return $result;
}

function get_invoiced_paid_for_contract_status_report($report_data, $invoice_ids) {

    if(count($invoice_ids) == 0) {
        return 0;
    }

    $invoice_ids_string = implode(',', $invoice_ids);
    $invoiced_paid = 0;

    $payment_row = DB::executeOne("
        SELECT SUM(ipri.amount) as total_paid
        FROM ".TABLE_PREFIX."invoice_payment_receipt_invoices ipri 
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=ipri.payment_receipt_id
        WHERE o.trashed_by_id=0 
        AND ipri.invoice_id IN (".$invoice_ids_string.")
    ");
    if ($payment_row) $invoiced_paid += $payment_row['total_paid'];

    return $invoiced_paid;
}

function get_member_total_executed_cost_for_contract_status_report($report_data, $filtered_project_member_ids) {
    $columns = $report_data['columns'];
    if(!in_array('remaining_budgeted_vs_contract', $columns)) {
        return array();
    }

    // Get executed cost for each project member
    $sql = "SELECT member_id as 'member_id', executed_overall_total_cost as 'executed_cost'
            FROM ".TABLE_PREFIX."member_totals
            WHERE member_id IN (".implode(',', $filtered_project_member_ids).")";
    $rows = DB::executeAll($sql);

    // Build array of member_id => executed_cost
    $result = array();
    foreach($rows as $row) {
        $result[$row['member_id']] = $row['executed_cost'];
    }
    
    return $result;
}
