<?php


function build_fixed_report_csv_from_excel_data($data, $excel_data) {

	$csv_data = '';

	foreach ($excel_data as $data_group) {

		if (isset($data_group['main_header']) && count($data_group['main_header']) > 0) {
			$csv_data .= '"' . implode('","', $data_group['main_header']) . '"' . "\n";
		}

		if (isset($data_group['headers']) && count($data_group['headers']) > 0) {
			$csv_data .= '"' . implode('","', $data_group['headers']) . '"' . "\n";
		}

		if (isset($data_group['data'])) {
			foreach ($data_group['data'] as $row) {
				if (count($row) > 0) {
					$csv_data .= '"' . implode('","', $row) . '"' . "\n";
				}
			}
		}

		if (isset($data_group['totals']) && count($data_group['totals']) > 0) {
			$csv_data .= '"' . implode('","', $data_group['totals']) . '"' . "\n";
		}

	}

	return $csv_data;
}


function get_main_budget_report_data(Member $project_member, $report_data) {
	
	
	$all_data = array(
			'project_member' => $project_member,
			'projects_data' => array(),
	);

	get_additional_info_for_project_for_budget_report($project_member, $all_data, $report_data);

	$list_subprojects = array_var($report_data, 'list_supbrojects');
	$group_columns_by_subprojects = array_var($report_data, 'group_columns_by');

	if($group_columns_by_subprojects){
		$all_data['projects_data'][$project_member->getName()] = get_budget_report_data($project_member, $report_data);
	} elseif($list_subprojects) {
		$ot_conditions = " AND object_type_id = ".$project_member->getObjectTypeId();
		$subprojects = Members::getSubmembers($project_member, false, $ot_conditions);
		if (!is_array($subprojects) || count($subprojects) == 0) {
			$all_data['projects_data'][$project_member->getName()] = get_budget_report_data($project_member, $report_data);
		} else {
			// do not include internal projects in the report
			$filtered_subprojects = array();
			$internal_project_type_member_id = config_option('internal_project_member');
			if ($internal_project_type_member_id > 0) {
				
				$project_types_dim = Dimensions::findByCode('project_types');
				foreach ($subprojects as $subproject) {
					$assoc_project_type_id = get_associated_status_member_id($subproject, $project_types_dim, null, false, 'project_project_type');
					if ($assoc_project_type_id != $internal_project_type_member_id) {
						$filtered_subprojects[] = $subproject;
					}
				}
				
				$subprojects = $filtered_subprojects;
			}
			
			// build the name shown in trees
			build_member_list_text_to_show_in_trees($subprojects);
			// order projects by the name
			$tmp_projects = array();
			foreach ($subprojects as $s){
				$tmp_projects[$s->getName()] = Members::getMemberById($s->getId());
			}
			ksort($tmp_projects);
			$subprojects = $tmp_projects;
			foreach ($subprojects as $subproject) {
				$all_data['projects_data'][$subproject->getName()] = get_budget_report_data($subproject, $report_data);
			}
		}
	} else {
		$all_data['projects_data'][$project_member->getName()] = get_budget_report_data($project_member, $report_data);
	}
	
	$project_totals = array('labor_total' => 0, 'expenses_total' => 0);
	$project_phases = array();
	foreach ($all_data['projects_data'] as $project_data) {
		
		if (isset($project_data['labor_summary']['subtotal_amount'])) {
			$summary = $project_data['labor_summary']['subtotal_amount'];
			$project_totals['labor_total'] += $summary;
		}
		
		if (isset($project_data['expenses_summary']['subtotal_amount'])) {
			$summary = $project_data['expenses_summary']['subtotal_amount'];
			$project_totals['expenses_total'] += $summary;
		}

		$tmp_phases = array_unique(array_merge($project_phases, $project_data['project_phases']));
		$project_phases = $tmp_phases;
	}
	sort($project_phases);
	$all_data['project_phases'] = $project_phases;
	$all_data['totals'] = $project_totals;

	calculate_totals_and_taxes_for_budget_report($all_data, $report_data);

	return $all_data;
}

function get_budget_report_data(Member $project_member, $report_data = array()) {

	$project_name_and_number_info = get_project_name_and_number_info($project_member);

	// Get project object type name
    $project_object_type_name = Objects::instance()->findById($project_member->getObjectId())->getObjectTypeName();

	$labor_data = get_budget_report_labor_data($project_member, $report_data);
	$labor_summary = get_budget_report_labor_summary_data($project_member, $labor_data, $report_data);
	$expenses_data = get_budget_report_expenses_data($project_member, $report_data);
	$expenses_summary = get_budget_report_expenses_summary($project_member, $expenses_data, $report_data);

	// Merger project phases from labor and expenses
	$labor_phases = isset($labor_data['project_phases']) ? array_keys($labor_data['project_phases']) : array();
	$expenses_phases = isset($expenses_data['project_phases']) ? array_keys($expenses_data['project_phases']) : array();
	$project_phases = array_unique(array_merge($labor_phases, $expenses_phases));

	$all_data = array(
			'project_name' => array_var($project_name_and_number_info, 'project_name'),
			'project_id' => array_var($project_name_and_number_info, 'project_id'),
			'project_id_raw' => array_var($project_name_and_number_info, 'project_id_raw'),
			'project_object_id' => $project_member->getObjectId(),
			'project_object_type_name' => $project_object_type_name,
			'labor_data' => $labor_data,
			'labor_summary' => $labor_summary,
			'expenses_data' => $expenses_data,
			'expenses_summary' => $expenses_summary,
			'project_phases' => $project_phases,
	);
	
	return $all_data;
}

function get_project_name_and_number_info($project_member){
	// Get project name and project number
	$project_prefix_object = ObjectPrefixes::instance()->findById($project_member->getObjectId());
	if($project_prefix_object){
		$project_prefix = $project_prefix_object->getPrefix();
		$project_id = '<span style="color: #999; text-transform: capitalize;">'.lang('number').' </span>'.$project_prefix;
		$project_id_raw = lang('number') . ' ' . $project_prefix;
		$project_name = str_replace($project_prefix.' - ', '', $project_member->getName());
	} else {
		$pot = ObjectTypes::findByName('project');
		$project_id_cp = CustomProperties::getCustomPropertyByCode($pot->getId(), 'project_id');
		$project_id = '';
		$project_id_raw = '';
		if ($project_id_cp instanceof CustomProperty) {
			$cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $project_id_cp->getId());
			if ($cp_val instanceof CustomPropertyValue) {
				$project_number_custom_property_name = CustomProperties::instance()->findById($cp_val->getCustomPropertyId())->getName();
				$project_id = '<span style="color: #999; text-transform: capitalize;">'.$project_number_custom_property_name.' </span>'.$cp_val->getValue();
				$project_id_raw = $project_number_custom_property_name . ' ' . $cp_val->getValue();
			}
		}
		$project_name =  $project_member->getName();
	}

	$result = array(
		'project_id' => $project_id,
		'project_id_raw' => $project_id_raw,
		'project_name' => $project_name,
	);

	return $result;
}

function get_budget_report_labor_data(Member $project_member, $report_data = array()){
	$group_labor_by = array_var($report_data, 'group_labor_by', 'task');
	$extra_conditions = ' AND parent_id=0';
	$task_params = array('more_conditions' => $extra_conditions);
	$project_tasks = get_member_project_tasks_budget_report($project_member, $report_data, $task_params);
	// Get all Labor category names and project phases
	$labor_data['labor_names'] = array();
	$project_phases = array();
	$group_columns_by_subprojects = array_var($report_data, 'group_columns_by');
	$labor_object_type_name = ObjectTypes::findByName('hour_type')->getObjectTypeName();
	
	foreach($project_tasks as $p_data){
		if ($group_labor_by == 'task') {
			$task_id = $p_data['id'];
			$task_name = $p_data['name'];
			if(!array_key_exists($task_id, $labor_data['labor_names'])){
				$labor_data['labor_names'][$task_id] = $task_name;
			}
		} else {
			$task_labor_id = '';
			$task_labor_name = '';
			foreach($p_data['members_data'] as $member_data){
				if($member_data['object_type_name']=='hour_type'){
					$task_labor_id = $member_data['id'];
					$task_labor_name = $member_data['name'];
				}
			}
			if($task_labor_id == ''){
				$task_labor_id = 'without_labor_category';
				$task_labor_name = ucwords(lang('without ').$labor_object_type_name);
			};
			if(!array_key_exists($task_labor_id, $labor_data['labor_names'])){
				$labor_data['labor_names'][$task_labor_id] = $task_labor_name;
			}
		}
	}
	// Sort Labor names and Project phases
	asort($labor_data['labor_names']);
	// Create for each project phase array with the length of 'labor_names' arrays with 0 as a value
	$length_labor_names = count($labor_data['labor_names']);
	$labor_data['project_phases'] = array();

	$labor_data['total_labor_per_phase_amount'] = array();
	$labor_data['total_per_labor_hours'] = array();
	$labor_data['total_per_labor_amount'] = array();
	$labor_data['hourly_rates'] = array();

	$job_phase_ot = ObjectTypes::findByName('job_phase');
	if ($job_phase_ot instanceof ObjectType){
		$phase_object_type_name = $job_phase_ot->getObjectTypeName();
	} else {
		$phase_object_type_name = lang('job phase');
	}
	
	// Assign values to variables for later use in find_billing_def_by_user_and_members()
	$allowed_dimensions = array_filter(config_option('adv_billing_dimensions'));
	$cant_decide_between_billing_definitions = null;

	foreach($project_tasks as $p_data){
		$task_labor_id = '';
		$task_hour_type_dimension_id = '';
		$task_rate_schedule_id = '';
		$task_rate_schedule_dimension_id = '';

		foreach($p_data['members_data'] as $member_data){
			if($member_data['object_type_name']=='hour_type'){
				$task_labor_id = $member_data['id'];
				$task_hour_type_dimension_id = $member_data['dimension_id'];
			}
			if($member_data['object_type_name']=='rate_schedule'){
				$task_rate_schedule_id = $member_data['id'];
				$task_rate_schedule_dimension_id = $member_data['dimension_id'];
			}
			if($group_columns_by_subprojects){
				// Use project name as project phase if columns are subprojects
				if($member_data['object_type_name']=='project'){
					$task_project_id = $member_data['id'];
					$project_phase = $member_data['name'];
				}	
			}


		}

		// Set task_id as grouping key if group by task
		if ($group_labor_by == 'task') {
			$task_labor_id = $p_data['id'];
		}

		if($task_labor_id == ''){
			$task_labor_id = 'without_labor_category';
		};
		if(!$group_columns_by_subprojects){
			$project_phase = isset($p_data['job_phases'][0]) ? $p_data['job_phases'][0] : ucfirst(strtolower(lang('without ').$phase_object_type_name));
		} else {
			$project_phase = $p_data['customer_project'][0];
		}
		
		// Use total time estimate as estimated hours, because getting only parent tasks
		$estimated_hours = $p_data['totalTimeEstimate']/60;

		// Add estimated hours to the matched project phase and labor category
		if(!isset($labor_data['project_phases'][$project_phase][$task_labor_id])){
			$labor_data['project_phases'][$project_phase][$task_labor_id] = 0;
		}
		$labor_data['project_phases'][$project_phase][$task_labor_id] += $estimated_hours;

		// Calculate total estimates hours per each labor category
		if(!isset($labor_data['total_per_labor_hours'][$task_labor_id])){
			$labor_data['total_per_labor_hours'][$task_labor_id] = 0;
		}
		$labor_data['total_per_labor_hours'][$task_labor_id] += $estimated_hours;

		// Get hourly rate for each labor category
		if ($group_labor_by == 'hour_types_dim') {
			$dim_members = array(
					$task_rate_schedule_dimension_id => array($task_rate_schedule_id),
					$task_hour_type_dimension_id => array($task_labor_id),
			);
			$billing_definition = find_billing_def_by_user_and_members(0, $dim_members, $allowed_dimensions, 'billing', $cant_decide_between_billing_definitions);
			if($billing_definition instanceof BillingDefinition){
				$labor_data['hourly_rates'][$task_labor_id] = $billing_definition->getHourlyRate();
			} else {
				$labor_data['hourly_rates'][$task_labor_id] = 0;
			}
		}

		if(!isset($labor_data['total_labor_per_phase_amount'][$project_phase])){
			$labor_data['total_labor_per_phase_amount'][$project_phase] = 0;
		}

		// Calculate total amount per each project phase
		$labor_data['total_labor_per_phase_amount'][$project_phase] += $p_data['estimated_price'];
		
		if(!isset($labor_data['total_per_labor_amount'][$task_labor_id])){
			$labor_data['total_per_labor_amount'][$task_labor_id] = 0;
		}

		// Calculate total amount per each labor category / task
		$labor_data['total_per_labor_amount'][$task_labor_id] += $p_data['estimated_price'];
	}

	ksort($labor_data['project_phases']);

	return $labor_data;
}

function get_budget_report_labor_summary_data(Member $project_member, $labor_data = null, $report_data = array()){
	if (is_null($labor_data)) {
		$labor_data = get_budget_report_labor_data($project_member, $report_data);
	}

	// Get total hours per each project phase
	$labor_summary['project_phases_subtotals'] = array();
	foreach($labor_data['project_phases'] as $phase_name => $hours_per_labor){
		$labor_summary['project_phases_subtotals'][$phase_name] = array_sum($hours_per_labor);
	}

	$labor_summary['subtotal_hours'] = array_sum($labor_data['total_per_labor_hours']);
	$labor_summary['subtotal_amount'] = array_sum($labor_data['total_per_labor_amount']);

	return $labor_summary;
}

function get_budget_report_expenses_data(Member $project_member, $report_data){
	$expenses_objects = get_member_expenses_budget_report($project_member, $report_data);

	$expenses_data = array();
	$group_expenses_by = isset($report_data['group_expenses_by']) ? $report_data['group_expenses_by'] : 0;
	$group_columns_by_subprojects = array_var($report_data, 'group_columns_by');

	if($group_expenses_by == 0) {
		// Group by product type
		$expenses_data['total_expenses_amount_per_phase'] = array();
		$expenses_data['expenses_names'] = array();
		$expenses_data['expenses_total_quantity'] = array();
		$expenses_data['unit_prices'] = array();
		$expenses_data['project_phases'] = array();
		$expenses_data['amount_per_expense'] = array();

		$job_phase_ot = ObjectTypes::findByName('job_phase');
		if ($job_phase_ot instanceof ObjectType){
			$phase_object_type_name = $job_phase_ot->getObjectTypeName();
		} else {
			$phase_object_type_name = lang('job phase');
		}

		foreach($expenses_objects as $expense){
			if($group_columns_by_subprojects){
				$project_phase = $expense['customer_project'][0];
			} else {
				$project_phase = isset($expense['job_phases'][0]) ? $expense['job_phases'][0] : ucwords(lang('without ').$phase_object_type_name);
			}
			
			foreach($expense['expense_items'] as $item){
				$item_id = $item['product_type_id'].$item['unit_price'];

				// Get all unique product types
				if (!isset($expenses_data['expenses_names'][$item_id])){
					$expenses_data['expenses_names'][$item_id] = $item['name'];
				}

				// Get unit price for new expense item
				if(!isset($expenses_data['unit_prices'][$item_id])){
					$expenses_data['unit_prices'][$item_id] = $item['unit_price'];
				}

				// Get quantity from each item and add it to matching project phase
				if (!isset($expenses_data['project_phases'][$project_phase][$item_id])){
					$expenses_data['project_phases'][$project_phase][$item_id] = 0;
				}
				$expenses_data['project_phases'][$project_phase][$item_id] += $item['quantity'];

				// Get total hours for each product type
				if (!isset($expenses_data['expenses_total_quantity'][$item_id])){
					$expenses_data['expenses_total_quantity'][$item_id] = 0;
				}
				$expenses_data['expenses_total_quantity'][$item_id] += $item['quantity'];

				// Calculate total for each expense
				if (!isset($expenses_data['amount_per_expense'][$item_id])){
					$expenses_data['amount_per_expense'][$item_id] = 0;
				}
				$expenses_data['amount_per_expense'][$item_id] += $item['total_price'];

				//Calculate total for each phase
				if (!isset($expenses_data['total_expenses_amount_per_phase'][$project_phase])){
					$expenses_data['total_expenses_amount_per_phase'][$project_phase] = 0;
				}
				$expenses_data['total_expenses_amount_per_phase'][$project_phase] += $item['total_price']; 


			}
		} 

		asort($expenses_data['expenses_names']);

	} else if($group_expenses_by == 1){
		// Group by expense categories
		$expenses_data['expenses_names'] = array();
		$expenses_data['project_phases'] = array();
		$expenses_data['amount_per_expense'] = array();
		
		$job_phase_ot = ObjectTypes::findByName('job_phase');
		if ($job_phase_ot instanceof ObjectType){
			$phase_object_type_name = $job_phase_ot->getObjectTypeName();
		} else {
			$phase_object_type_name = lang('job phase');
		}
		$expense_category_type_name = ObjectTypes::findByName('expense_category')->getObjectTypeName();

		foreach($expenses_objects as $expense){	
			if($group_columns_by_subprojects){
				$project_phase = $expense['customer_project'][0];
			} else {
				$project_phase = isset($expense['job_phases'][0]) ? $expense['job_phases'][0] : ucwords(lang('without ').$phase_object_type_name);
			}

			$expense_category_name = isset($expense['expense_categories'][0]) ? $expense['expense_categories'][0] : ucwords(lang('without ').$expense_category_type_name);

			if(!in_array($expense_category_name, $expenses_data['expenses_names'])){
				$expenses_data['expenses_names'][] = $expense_category_name;
			}


			foreach($expense['expense_items'] as $item){
				$item_id = $item['product_type_id'];


				if(!isset($expenses_data['project_phases'][$project_phase][$expense_category_name])){
					$expenses_data['project_phases'][$project_phase][$expense_category_name] = 0;
				}
				$expenses_data['project_phases'][$project_phase][$expense_category_name] +=  $item['total_price'];

				if(!isset($expenses_data['amount_per_expense'][$expense_category_name])){
					$expenses_data['amount_per_expense'][$expense_category_name] = 0;
				}
				$expenses_data['amount_per_expense'][$expense_category_name] += $item['total_price'];


			}
		} 

		sort($expenses_data['expenses_names']);

	}
	return $expenses_data;
}

function get_budget_report_expenses_summary(Member $project_member, $expenses_data = null, $report_data){
	$expenses_summary['subtotal_amount'] = array_sum($expenses_data['amount_per_expense']);
	return $expenses_summary;
}

function calculate_totals_and_taxes_for_budget_report(&$all_data, $report_data) {
	$tax_ids = array();
	foreach($report_data as $key => $value) {
		if(strpos($key, 'tax_') !== false) {
			$tax_ids[] = intval(str_replace('tax_', '', $key));
		}
	}
	
	$totals = array_var($all_data, 'totals', array());
	$all_data['totals']['subtotal'] = array_var($totals, 'labor_total', 0) + array_var($totals, 'expenses_total', 0);
	if (count($tax_ids) > 0) {
		$all_data['totals']['include_taxes'] = true;
		$all_data['totals']['taxes'] = array();
		$all_data['totals']['taxes']['tax_rate_total'] = 0;
		foreach($tax_ids as $tax_id) {
			$tax = IncomeTaxes::instance()->findById($tax_id);
			if ($tax instanceof IncomeTax) {
				$tax_record = $tax->getRecord()->getArrayInfo();
				$tax_data = array(
					'id' => $tax_id,
					'is_tax' => true,
					'name' => $tax->getName(),
					'type' => $tax_record['type'],
					'rate' => $tax_record['value']
				);
				if ($tax_data['type'] == 'percentage') {
					$tax_data['amount'] =  $all_data['totals']['subtotal'] * $tax_data['rate'] / 100;
				} else {
					$tax_data['amount'] = $tax_data['rate'];
				}
				$all_data['totals']['taxes']['tax_rate_total'] += $tax_data['amount'];
				$all_data['totals']['taxes'][] = $tax_data;
			}
		}
		$all_data['totals']['grand_total'] = $all_data['totals']['subtotal'] + $all_data['totals']['taxes']['tax_rate_total'];
	} else {
		$all_data['totals']['grand_total'] = $all_data['totals']['subtotal'];
	}
}

function get_additional_info_for_project_for_budget_report(Member $project_member, &$all_data, $report_data) {
	// Get quoting notes cp value
	$project_ot = ObjectTypes::findByName('project');
	$quoting_notes_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), 'project_quoting_notes');
	$all_data['quoting_notes'] = '';
	if ($quoting_notes_cp instanceof CustomProperty) {
		$cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $quoting_notes_cp->getId());
		if ($cp_val instanceof CustomPropertyValue) {
			$all_data['quoting_notes'] = $cp_val->getValue();
		}
	}

	$all_data['project_name'] = $project_member->getName();
	// Get client name
	$all_data['client_name'] = '';
	$client_member_id = 0;
	$project_client_assoc = DimensionMemberAssociations::getAssociationByCode('project_customer');
	if ($project_client_assoc instanceof DimensionMemberAssociation) {
		$project_client_assoc_id = $project_client_assoc->getId();
		$project_client_ids = explode(',', MemberPropertyMembers::instance()->getAllPropertyMemberIds($project_client_assoc_id, $project_member->getId()));
		$project_client = Members::instance()->findById($project_client_ids[0]);
		if ($project_client instanceof Member) {
			$client_member_id = $project_client->getId();
			$all_data['client_name'] = $project_client->getName();
		}
	}

	// Get project address using billing contact cp
	$billing_contact_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), 'billing_contact');
	$all_data['project_address'] = array();
	if ($billing_contact_cp instanceof CustomProperty && Plugins::instance()->isActivePlugin('income')) {
		$income_controller = new IncomeController();

		$cp_val = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $billing_contact_cp->getId());
		$billing_contact_id = 0;
		if ($cp_val instanceof CustomPropertyValue) {
			$billing_contact_id = $cp_val->getValue();
		}
		$address = $income_controller->find_billing_address_by_client_and_contact($client_member_id, $billing_contact_id);
		if ($address instanceof ContactAddress) {
			$all_data['project_address'] = $address->getArrayInfo();
		}
	}
}


function get_main_project_manager_report_data($project_member, $report_data) {
	
	$all_data = array(
		'projects_data' => array(),
	);
	
	if ($project_member instanceof Member) {
		$check_active_context = false;
		$filtered_project_member_ids = get_project_member_ids_for_project_manager_report($check_active_context, $report_data);
		$project_member_ids = array($project_member->getId());
	} else {
		ini_set('memory_limit', '2G');
		set_time_limit(0);
		$filtered_project_member_ids = array();  // *** LC 2023-09-29
		$project_billing_report_id = Reports::instance()->findOne(array("conditions" => "`code` = 'project_billing_report'"))->getId();
		$project_billing_alt_report_id = Reports::instance()->findOne(array("conditions" => "`code` = 'project_billing_alt_report'"))->getId();
		if($report_data['report_id']==$project_billing_report_id || $report_data['report_id']==$project_billing_alt_report_id){
			$check_active_context = false;
			$all_project_member_ids = get_project_member_ids_for_project_manager_report($check_active_context, $report_data);
			$filter_by_date_pm_ids = get_project_member_ids_filtered_by_date($report_data);
			$project_member_ids = array();
			foreach($filter_by_date_pm_ids as $p_id){
				if(in_array($p_id, $all_project_member_ids)) $project_member_ids[] = $p_id;
			}
		} else {
			$check_active_context = true;
			$project_member_ids = get_project_member_ids_for_project_manager_report($check_active_context, $report_data);
		}
		
	}
	
	// build the name shown in trees
	build_member_list_text_to_show_in_trees($projects);

	$calculated_member_ids = array();
	if ($project_member_ids) {
		foreach ($project_member_ids as $p_id) {
			$project = Members::instance()->findById($p_id);
			if (!$project instanceof Member)
				continue;
			$project_data = get_project_manager_report_data($project, $report_data, $calculated_member_ids, true, $filtered_project_member_ids);
			if ($project_data) {
				$all_data['projects_data'][] = $project_data;
			}
		}
	}

	// Add unassigned time and expenses data to project billing report
	if ((isset($project_billing_report_id) && $report_data['report_id'] == $project_billing_report_id) || (isset($project_billing_alt_report_id) && $report_data['report_id'] == $project_billing_alt_report_id)) {
		$unassigned_data = get_unassigned_project_manager_report_data($report_data, $calculated_member_ids, $filtered_project_member_ids);
		if ($unassigned_data) {
			$all_data['projects_data'][] = $unassigned_data;
		}
	}



	// Sort projects by name
	$sort_columns = array_column($all_data['projects_data'], 'project_name');
    array_multisort($sort_columns, SORT_ASC, $all_data['projects_data']);

	if(array_var($report_data, 'sort_projects_by') == 'client_name'){
		// Sort projects by client name
		$sort_columns = array_column($all_data['projects_data'], 'client_name');
		array_multisort($sort_columns, SORT_ASC, $all_data['projects_data']);
	}
	
	$project_totals = array(
		'timeslots_total_cost' => 0,
		'timeslots_total' => 0,
		'expenses_total_cost' => 0,
		'expenses_total' => 0,
		'worked_hours' => 0,
		'miles' => 0,
		'estimated_hours' => 0,
		'labor_estimated_billing' => 0,
		'estimated_expense_billing' => 0,
		'total_estimated_billing' => 0,
		'labor_pending_billing' => 0,
		'pending_expense_billing' => 0,
		'total_pending_billing' => 0,
		'labor_invoiced' => 0,
		'expense_invoiced' => 0,
		'total_invoiced' => 0,
		'labor_not_invoiced' => 0,
		'expense_not_invoiced' => 0,
		'total_not_invoiced' => 0,
		'project_total_budget' => 0
	);

	foreach ($all_data['projects_data'] as $project_data) {
		if (isset($project_data['timeslots_summary']['subtotal'])) {
			$summary = $project_data['timeslots_summary']['subtotal'];
			
			if (!isset($project_totals['estimated_hours'])) $project_totals['estimated_hours'] = 0;
			if (!isset($project_totals['estimated_billing'])) $project_totals['estimated_billing'] = 0;
			if (!isset($project_totals['pending_hours'])) $project_totals['pending_hours'] = 0;
			if (!isset($project_totals['pending_billing'])) $project_totals['pending_billing'] = 0;
			if (!isset($project_totals['miles'])) $project_totals['miles'] = 0;
			
			$project_totals['worked_hours'] += array_var($summary, 'worked_hours');
			$project_totals['estimated_hours'] += array_var($summary, 'estimated_hours');
			$project_totals['labor_estimated_billing'] += array_var($summary, 'estimated_billing');
			$project_totals['total_estimated_billing'] += array_var($summary, 'estimated_billing');
			$project_totals['pending_hours'] += array_var($summary, 'pending_hours');
			$project_totals['labor_pending_billing'] += array_var($summary, 'pending_billing');
			$project_totals['total_pending_billing'] += array_var($summary, 'pending_billing');
			$project_totals['timeslots_total'] += array_var($summary, 'totalbilling');
			$project_totals['timeslots_total_cost'] += array_var($summary, 'total_cost');
			$project_totals['currency_sym'] = array_var($summary, 'currency_sym');
			$project_totals['miles'] += array_var($summary, 'miles', 0);
			$project_totals['labor_invoiced'] += array_var($summary, 'invoiced', 0);
			$project_totals['total_invoiced'] += array_var($summary, 'invoiced', 0);
			$project_totals['labor_not_invoiced'] += array_var($summary, 'not_invoiced', 0);
			$project_totals['total_not_invoiced'] += array_var($summary, 'not_invoiced', 0);
		}
		
		if (isset($project_data['expenses_summary']['subtotal'])) {
			$summary = $project_data['expenses_summary']['subtotal'];
			
			$project_totals['expenses_total'] += array_var($summary, 'amount');
			$project_totals['expenses_total_cost'] += array_var($summary, 'expenses_total_cost');
			$project_totals['estimated_expense_billing'] += array_var($summary, 'estimated_expense_billing');
			$project_totals['total_estimated_billing'] += array_var($summary, 'estimated_expense_billing');
			$project_totals['pending_expense_billing'] += array_var($summary, 'pending_expense_billing');
			$project_totals['total_pending_billing'] += array_var($summary, 'pending_expense_billing');
			$project_totals['currency_sym'] = array_var($summary, 'currency_sym');
			$project_totals['expense_invoiced'] += array_var($summary, 'invoiced', 0);
			$project_totals['total_invoiced'] += array_var($summary, 'invoiced', 0);
			$project_totals['expense_not_invoiced'] += array_var($summary, 'not_invoiced', 0);
			$project_totals['total_not_invoiced'] += array_var($summary, 'not_invoiced', 0);
		}

		if(isset($project_data['totals'])){
			$project_totals['project_total_budget'] += array_var($project_data['totals'], 'project_total_budget', 0);
		}
	}

	// Add 'Contract Type'
	$project_totals['contract_type'] = $all_data['projects_data'][0]['totals']['contract_type'];
	
	$all_data['totals'] = $project_totals;
	
	return $all_data;
}


function filter_internal_projects($subprojects) {
	// do not include internal projects in the report
	$filtered_subprojects = array();
	$internal_project_type_member_id = config_option('internal_project_member');
	if ($internal_project_type_member_id > 0) {
		
		$project_types_dim = Dimensions::findByCode('project_types');
		foreach ($subprojects as $subproject) {
			$assoc_project_type_id = get_associated_status_member_id($subproject, $project_types_dim, null, false, 'project_project_type');
			if ($assoc_project_type_id != $internal_project_type_member_id) {
				$filtered_subprojects[] = $subproject;
			}
		}
		
		return $filtered_subprojects;
		
	} else {
		return $subprojects;
	}
}


function get_project_manager_report_data(Member $project_member, $report_data = array(), &$calculated_member_ids, $check_filtered_members = false, $filtered_project_member_ids = array()) {
	Env::useHelper('functions', 'crpm');

	if(in_array($project_member->getId(), $calculated_member_ids)){
		return;
	} else {
		$calculated_member_ids[] = $project_member->getId();
	}

	$client_id = 0;
	$client_name = lang('not assigned');
	$client_member = get_client_member_of_project($project_member);
	if($client_member instanceof Member){
		$client_id = $client_member->getId();
		$client_name = $client_member->getName();
	}

	$tmp_array = array($project_member);
	build_member_list_text_to_show_in_trees($tmp_array);

	$timeslots_data = get_project_manager_report_timeslots_data($project_member, $report_data);
	
	$expenses_data = get_project_manager_report_expenses_data($project_member, $report_data);
	
	$subprojects_data = array();

	$include_subprojects = array_var($report_data, 'include_subproject_info') == 'checked' ? true : false;
	if($include_subprojects){
		$ot_conditions = " AND object_type_id = ".$project_member->getObjectTypeId();
		$subprojects = Members::getSubmembers($project_member, false, $ot_conditions);
		
		if (is_array($subprojects) && count($subprojects) > 0) {
			
			// dont show internal projects
			$subprojects = filter_internal_projects($subprojects);
			
			// build the name shown in trees
			build_member_list_text_to_show_in_trees($subprojects);
			
			// order projects by the name
			$tmp_projects = array();
			foreach ($subprojects as $s) $tmp_projects[$s->getName()] = $s;
			ksort($tmp_projects);
			$subprojects = $tmp_projects;
			
			// get the data foreach subproject, recursive
			foreach ($subprojects as $subproject) {
				if($check_filtered_members && in_array($subproject->getId(), $filtered_project_member_ids)){
					$subp_data = get_project_manager_report_data($subproject, $report_data, $calculated_member_ids, $check_filtered_members, $filtered_project_member_ids);
					if ($subp_data) {
						$subprojects_data[] = $subp_data;
					}
				}
			}
		}
	}

	$timeslots_summary = get_project_manager_report_timeslots_summary($project_member, $timeslots_data, $subprojects_data, $report_data);
	
	$expenses_summary = get_project_manager_report_expenses_summary($project_member, $expenses_data, $subprojects_data, $report_data);

	$totals_summary = get_project_manager_report_totals_summary($timeslots_summary, $expenses_summary);
	get_project_total_budget($project_member, $totals_summary, $report_data);

	$project_start_date = lang('none');
	$project_ot = ObjectTypes::findByName('project');
	$project_id_cp = CustomProperties::getCustomPropertyByCode($project_ot->getId(), "start_date");	
	$project_obj_id = $project_member->getObjectId();

	if ($project_id_cp instanceof CustomProperty) {
		$cpval = CustomPropertyValues::getCustomPropertyValue($project_obj_id, $project_id_cp->getId());
		if ($cpval instanceof CustomPropertyValue) {
			$dt = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $cpval->getValue());
			$project_start_date = format_date($dt, null, 0);
		}
	}

  // Sort subprojects by name
	$sort_columns = array_column($subprojects_data, 'project_name');
  array_multisort($sort_columns, SORT_ASC, $subprojects_data);


	$all_data = array(
			'project_id' => $project_member->getId(),
			'project_name' => $project_member->getName(),
			'client_id' => $client_id,
			'client_name' => $client_name,
			'project_start_date' => $project_start_date,
			'timeslots_data' => $timeslots_data,
			'timeslots_summary' => $timeslots_summary,
			'expenses_data' => $expenses_data,
			'expenses_summary' => $expenses_summary,
			'subprojects' => $subprojects_data,
			'totals' => $totals_summary
	);
	
	return $all_data;
}

function get_unassigned_project_manager_report_data($report_data, $calculated_member_ids, $filtered_project_member_ids) {
	Env::useHelper('functions', 'crpm');
	$dummy_member = new Member();

	$subprojects_data = array();  // *** LC 2023-09-29
	
	$timeslots_data = get_project_manager_report_timeslots_data($dummy_member, $report_data);
	
	$expenses_data = get_project_manager_report_expenses_data($dummy_member, $report_data);

	$timeslots_summary = get_project_manager_report_timeslots_summary($dummy_member, $timeslots_data, $subprojects_data, $report_data);
	
	$expenses_summary = get_project_manager_report_expenses_summary($dummy_member, $expenses_data, $subprojects_data, $report_data);

	$totals_summary = get_project_manager_report_totals_summary($timeslots_summary, $expenses_summary);
	$totals_summary['project_total_budget'] = 0;
	$totals_summary['contract_type'] = '';

	$project_start_date = lang('none');

	$all_data = array(
			'project_id' => 0,
			'project_name' => lang('unclassified'),
			'client_id' => 0,
			'client_name' => lang('unclassified'),
			'project_start_date' => $project_start_date,
			'timeslots_data' => $timeslots_data,
			'timeslots_summary' => $timeslots_summary,
			'expenses_data' => $expenses_data,
			'expenses_summary' => $expenses_summary,
			'subprojects' => array(),
			'totals' => $totals_summary
	);
	
	return $all_data;
}



function get_member_timeslots(Member $project_member, $report_data = array(), $only_current_level = false) { 

	$member_ids = $project_member->isNew() ? array() : array($project_member->getId());
	if(!$project_member->isNew()) {
		$extra_conditions = '';
	} else {
		$project_object_type_id = ObjectTypes::findByName('project')->getId();
		$extra_conditions = ' AND NOT EXISTS (SELECT * FROM '.TABLE_PREFIX.'object_members omem WHERE omem.object_id = o.id AND omem.member_id IN (SELECT m.id FROM fo_members m WHERE m.object_type_id='.$project_object_type_id.')) ';
	}

	// Exclude running and paused timers
	$extra_conditions .= ' AND `end_time` != "'.EMPTY_DATETIME.'"';

	if (isset($report_data['start_date'])) {
		$d = getDateValue($report_data['start_date']);
		if ($d instanceof DateTimeValue) {
			$extra_conditions .= " AND start_time >= ".DB::escape($d);
		}
	}
	if (isset($report_data['end_date'])) {
		$d = getDateValue($report_data['end_date']);
		if ($d instanceof DateTimeValue) {
			$extra_conditions .= " AND start_time <= ".DB::escape($d);
		}
	}

	$report_invoicing_status = array_var($report_data, 'invoice_status');
	if($report_invoicing_status == 'pending_only'){
		$extra_conditions .= " AND invoicing_status = 'pending'";
	} else if($report_invoicing_status == 'pending_all'){
		$extra_conditions .= " AND (invoicing_status = 'pending' OR invoicing_status = 'non_billable')";
	} else if($report_invoicing_status == 'invoiced'){
		$extra_conditions .= " AND invoicing_status = 'invoiced'";
	}

	$exclude_non_billable = array_var($report_data, 'exclude_non_billable', false);
	if ($exclude_non_billable && !in_array($report_invoicing_status, array('pending_all', 'all_regular'))) {
		$extra_conditions .= " AND invoicing_status!='non_billable'";
	}

	$show_only_member_objects = false;
	if ($only_current_level && !$project_member->isNew()) {
		$extra_conditions .= " AND om.is_optimization=0 ";
		$show_only_member_objects = true;
	}

	if (Plugins::instance()->isActivePlugin('timesheet_approval') && $report_invoicing_status == 'pending_all') {
		$processed_member_id = config_option("time_bulk_action_processed_member_id");
		if($processed_member_id > 0){
			$extra_conditions .= " AND NOT EXISTS (
				SELECT * FROM ".TABLE_PREFIX."object_members x WHERE x.member_id=".$processed_member_id." AND x.object_id=e.object_id
				) ";
		}
	}
	
	if(!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_others_timeslots')){
		$extra_conditions .= " AND contact_id = " . logged_user()->getId();
	}

	// Add associated to timeslots active context members
	$timeslot_ot_id = ObjectTypes::findByName('timeslot')->getId();
	$assoc_active_member_ids = get_object_assoc_active_context_members($timeslot_ot_id);
	$member_ids = array_merge($member_ids, $assoc_active_member_ids);

	$res = Timeslots::instance()->listing(array(
			"join_ts_with_task" => false,
			"order" => 'start_time',
			"order_dir" => 'ASC',
			"start" => 0,
			"limit" => 0,
			"ignore_context" => false,
			"extra_conditions" => $extra_conditions,
			"count_results" => false,
			"show_only_member_objects" => $show_only_member_objects,
			"only_count_results" => false,
			"member_ids" => $member_ids,
	));
	$timeslots = $res->objects;
	
	return $timeslots;
}

function get_member_project_tasks_budget_report(Member $project_member, $report_data = array(), $more_params = array()) {

	$more_conditions = array_var($more_params, 'more_conditions', '');
	$extra_conditions = '';
	if ($more_conditions != '') {
		$extra_conditions .= $more_conditions;
	}

	if (isset($report_data['start_date'])) {
		$d = getDateValue($report_data['start_date']);
		if ($d instanceof DateTimeValue) {
			$extra_conditions .= " AND `start_date` >= ".DB::escape($d);
		}
	}
	if (isset($report_data['end_date'])) {
		$d = getDateValue($report_data['end_date']);
		if ($d instanceof DateTimeValue) {
			$d->endOfDay();
			$extra_conditions .= " AND due_date <= ".DB::escape($d);
		}
	}
	
	$res = ProjectTasks::instance()->listing(array(
			"order" => 'start_date',
			"order_dir" => 'ASC',
			"start" => 0,
			"limit" => 0,
			"ignore_context" => true,
			"extra_conditions" => $extra_conditions,
			"count_results" => false,
			"only_count_results" => false,
			"member_ids" => array($project_member->getId()),
	));
	$tasks = $res->objects;
	
	$tasks_data = array();
	foreach ($tasks as $task) {
		/* @var $task ProjectTask */
		
		$t_data = $task->getArrayInfo(true, true, false);
		
		$members = $task->getMembers();
		foreach ($members as $member) {
			/* @var $member Member */
			$mem_dim = $member->getDimension();
			if($mem_dim->getCode() == 'customer_project'){
				$project_info = get_project_name_and_number_info($member);
				$memberList = array($member);
				// build the name shown in trees
				build_member_list_text_to_show_in_trees($memberList);
			}
			if (!isset($t_data[$mem_dim->getCode()])) $t_data[$mem_dim->getCode()] = array();
			if($mem_dim->getCode() == 'customer_project'){  
				$t_data[$mem_dim->getCode()][] = $memberList[0]->getName();
			} else {
				$t_data[$mem_dim->getCode()][] = $member->getName();
			}
		}
		
		$tasks_data[] = $t_data;
	}
	//Logger::log_r($tasks_data);
	return $tasks_data;
}


function get_member_project_tasks(Member $project_member, $report_data = array(), $only_current_level = false, $include_mem_path = true) {

	if($project_member->isNew()) return array();
	$member_ids = array($project_member->getId());
	
	$extra_conditions = '';
	if (isset($report_data['start_date'])) {
		$d = getDateValue($report_data['start_date']);
		if ($d instanceof DateTimeValue) {
			$extra_conditions .= " AND `start_date` >= ".DB::escape($d);
		}
	}
	if (isset($report_data['end_date'])) {
		$d = getDateValue($report_data['end_date']);
		if ($d instanceof DateTimeValue) {
			$d->endOfDay();
			$extra_conditions .= " AND due_date <= ".DB::escape($d);
		}
	}
	
	if ($only_current_level) {
		$extra_conditions .= " AND om.is_optimization=0 ";
	}
	
	// Add active context members
	$active_member_ids = active_context_members(false);
	$member_ids = array_merge($member_ids, $active_member_ids);
	
	$res = ProjectTasks::instance()->listing(array(
			"order" => 'start_date',
			"order_dir" => 'ASC',
			"start" => 0,
			"limit" => 0,
			"ignore_context" => false,
			"extra_conditions" => $extra_conditions,
			"count_results" => false,
			"only_count_results" => false,
			"member_ids" => $member_ids,
	));
	$tasks = $res->objects;
	
	$allowed_dimensions = array_filter(config_option('adv_billing_dimensions'));

	$tasks_data = array();
	foreach ($tasks as $task) {
		/* @var $task ProjectTask */
		
		$t_data = $task->getArrayInfo(true, false, $include_mem_path);
		
		$members_sql = "SELECT m.name as member_name, d.code as dimension_code
						FROM ".TABLE_PREFIX."object_members om
						INNER JOIN ".TABLE_PREFIX."members m ON om.member_id=m.id
						INNER JOIN ".TABLE_PREFIX."dimensions d ON d.id = m.dimension_id
						WHERE om.object_id=".$task->getId()." AND om.is_optimization=0";
		
		$member_rows = DB::executeAll($members_sql);
		
		foreach ($member_rows as $row) {
			if (!isset($t_data[$row['dimension_code']])) $t_data[$row['dimension_code']] = array();
			$t_data[$row['dimension_code']][] = $row['member_name'];
		}
		
	
		// GET BILLING DEFINITION FOR BILLING RATE
		$object_id = $task->getObjectId();
		
		$sql = "SELECT m.dimension_id, om.member_id FROM ".TABLE_PREFIX."object_members om
					INNER JOIN ".TABLE_PREFIX."members m ON om.member_id = m.id
					WHERE om.is_optimization=0 AND dimension_id IN (".implode(',', $allowed_dimensions).") AND om.object_id = '$object_id'";
		
		$dim_member_rows = DB::executeAll($sql);
		$dim_members = array();
		if (is_array($dim_member_rows)) {
			foreach ($dim_member_rows as $dmr) {
				if (!isset($dim_members[$dmr['dimension_id']])) $dim_members[$dmr['dimension_id']] = array();
				$dim_members[$dmr['dimension_id']][] = $dmr['member_id'];
			}
		}
		
		$cant_decide_between_billing_definitions = null;
		
		$billing_definition = find_billing_def_by_user_and_members(0, $dim_members, $allowed_dimensions, 'billing', $cant_decide_between_billing_definitions);
		$t_data['hourlybilling'] = $billing_definition instanceof BillingDefinition ? $billing_definition->getHourlyRate() : 0;


		$tasks_data[] = $t_data;
	}

	return $tasks_data;
}

function get_member_expenses(Member $project_member, $report_data = array(), $only_current_level = false) {

	$member_ids = $project_member->isNew() ? array() : array($project_member->getId());
	if($project_member->isNew()) {
		$project_object_type_id = ObjectTypes::findByName('project')->getId();
		$extra_conditions = ' AND NOT EXISTS (SELECT * FROM '.TABLE_PREFIX.'object_members omem WHERE omem.object_id = o.id AND omem.member_id IN (SELECT m.id FROM fo_members m WHERE m.object_type_id='.$project_object_type_id.')) ';
	} else {
		$extra_conditions = '';
	}

	if (isset($report_data['expense_start_date'])) {
		$d = getDateValue($report_data['expense_start_date']);
		if ($d instanceof DateTimeValue) {
			if ($extra_conditions)
				$extra_conditions .= " AND `date` >= " . DB::escape($d);
			else
				$extra_conditions = " AND `date` >= " . DB::escape($d);
		}
	}
	if (isset($report_data['expense_end_date'])) {
		$d = getDateValue($report_data['expense_end_date']);
		if ($d instanceof DateTimeValue) {
			$extra_conditions .= " AND `date` <= ".DB::escape($d);
		}
	}
	
	$show_only_member_objects = false;
	if ($only_current_level && !$project_member->isNew()) {
		$extra_conditions .= " AND om.is_optimization=0 ";
		$show_only_member_objects = true;
	}

	$report_invoicing_status = array_var($report_data, 'invoice_status');
	if($report_invoicing_status == 'pending_only'){
		$extra_conditions .= " AND invoicing_status = 'pending'";
	} else if($report_invoicing_status == 'pending_all'){
		$extra_conditions .= " AND (invoicing_status = 'pending' OR invoicing_status = 'non_billable')";
	} else if($report_invoicing_status == 'invoiced'){
		$extra_conditions .= " AND invoicing_status = 'invoiced'";
	}

	$exclude_non_billable = array_var($report_data, 'exclude_non_billable', false);
	if($exclude_non_billable && !in_array($report_invoicing_status, array('pending_all', 'all_regular'))){
		$extra_conditions .= " AND is_billable=1";
	}

	if (Plugins::instance()->isActivePlugin('timesheet_approval') && $report_invoicing_status == 'pending_all') {
		$processed_member_id = config_option("time_bulk_action_processed_member_id");
		if($processed_member_id > 0){
			$extra_conditions .= " AND NOT EXISTS (
				SELECT * FROM ".TABLE_PREFIX."object_members x WHERE x.member_id=".$processed_member_id." AND x.object_id=e.object_id
				) ";
		}
	}

	if(!SystemPermissions::userHasSystemPermission(logged_user(), 'can_see_expenses_of_others')){
		$extra_conditions .= " AND paid_by_id = " . logged_user()->getId();
	}

	// Add associated to expenses active context members
	$payment_receipt_ot_id = ObjectTypes::findByName('payment_receipt')->getId();
	$assoc_active_member_ids = get_object_assoc_active_context_members($payment_receipt_ot_id);
	$member_ids = array_merge($member_ids, $assoc_active_member_ids);
	
	$res = PaymentReceipts::instance()->listing(array(
			"order" => 'date',
			"order_dir" => 'ASC',
			"start" => 0,
			"limit" => 0,
			"ignore_context" => false,
			"extra_conditions" => isset($extra_conditions) ? $extra_conditions : "",
			"count_results" => false,
			"only_count_results" => false,
			"show_only_member_objects" => $show_only_member_objects,
			"member_ids" => $member_ids,
	));
	$expenses = $res->objects;
	
	return $expenses;
}

function get_member_expenses_budget_report(Member $project_member, $report_data = array(), $ignore_context = true) {
	$extra_conditions = '';

	$group_labor_by = array_var($report_data, 'group_labor_by', 'task');
	if ($group_labor_by == 'task') {
		$extra_conditions .= ' AND task_id=0';
	}

	$member_ids = array($project_member->getId());

	if($project_member->isNew()) return array();
	if(!$ignore_context) {
		$active_member_ids = active_context_members(false);
		$member_ids = array_merge($member_ids, $active_member_ids);
	}

	$res = Expenses::instance()->listing(array(
		"order" => 'date',
		"order_dir" => 'ASC',
		"start" => 0,
		"limit" => 0,
		"ignore_context" => $ignore_context,
		"count_results" => false,
		"only_count_results" => false,
		"extra_conditions" => $extra_conditions,
		"member_ids" => $member_ids,
	));
	$expenses = $res->objects;

	$expenses_data = array();
	foreach ($expenses as $expense) {
		/* @var $task ProjectTask */
		
		$e_data = $expense->getArrayInfo(true, true, false);
		$members = $expense->getMembers();
		foreach ($members as $member) {
			/* @var $member Member */
			$mem_dim = $member->getDimension();
			if($mem_dim->getCode() == 'customer_project'){
				$project_info = get_project_name_and_number_info($member);
				$memberList = array($member);
				// build the name shown in trees
				build_member_list_text_to_show_in_trees($memberList);
			}
			if (!isset($e_data[$mem_dim->getCode()])) $e_data[$mem_dim->getCode()] = array();
			if($mem_dim->getCode() == 'customer_project'){  
				$e_data[$mem_dim->getCode()][] = $memberList[0]->getName();
			} else {
				$e_data[$mem_dim->getCode()][] = $member->getName();
			}
		}
		$e_data['expense_items'] = get_expense_items_for_expense($expense->getObjectId());
		$expenses_data[] = $e_data;
	}

	return $expenses_data;
}

function get_project_manager_report_timeslots_data(Member $project_member, $report_data = array(), $timeslots = null) {
	
	$timeslots_data = array();
	
	if (is_null($timeslots)) {
		$include_subprojects = array_var($report_data, 'include_subproject_info') == 'checked' ? true : false;
		$only_current_level = $include_subprojects ? true : false;
		$timeslots = get_member_timeslots($project_member, $report_data, $only_current_level);
	}
	
	$cp_miles = null;
	if (array_var($report_data, 'include_miles')) {
		$cp_miles = CustomProperties::getCustomPropertyByCode(Timeslots::instance()->getObjectTypeId(), 'miles');
	}
	
	foreach ($timeslots as $timeslot) {
		/* @var $timeslot Timeslot */
		
		$t_data = $timeslot->getArrayInfo(true, true, false, false);
		
		$members_sql = "SELECT m.name as member_name, d.code as dimension_code 
						FROM ".TABLE_PREFIX."object_members om
						INNER JOIN ".TABLE_PREFIX."members m ON om.member_id=m.id
						INNER JOIN ".TABLE_PREFIX."dimensions d ON d.id = m.dimension_id
						WHERE om.object_id=".$timeslot->getId()." AND om.is_optimization=0";
		
		$member_rows = DB::executeAll($members_sql);
		
		foreach ($member_rows as $row) {
			if (!isset($t_data[$row['dimension_code']])) $t_data[$row['dimension_code']] = array();
			$t_data[$row['dimension_code']][] = $row['member_name'];
		}
		
		if ($cp_miles) {
			$cp_val = CustomPropertyValues::getCustomPropertyValue($timeslot->getId(), $cp_miles->getId());
			$t_data['miles'] = $cp_val instanceof CustomPropertyValue ? $cp_val->getValue() : 0;
			if (trim($t_data['miles']) == '') $t_data['miles'] = 0;
		}
		
		$timeslots_data[] = $t_data;
	}
	
	return $timeslots_data;
}



function get_project_manager_report_timeslots_summary(Member $project_member, $timeslots_data = null, $subprojects_data = array(), $report_data = array()) {

	if (is_null($timeslots_data)) {
		$timeslots_data = get_project_manager_report_timeslots_data($project_member);
	}

	$summary = array();
	$show_invoice_detail = get_show_invoicing_detail($report_data);
	// collect totals of current member timeslots
	foreach ($timeslots_data as $t_data) {

		$t_data_hour_types = array_var($t_data, 'hour_types');
		if (!$t_data_hour_types) {
			$group_name = 'Without labor category';
		} else {
			$group_name = $t_data_hour_types[0];
		}

		if(in_array(array_var($report_data, 'financial_type'), array('cost', 'price_and_cost'))){
			$group_key = $group_name . "-" . array_var($t_data, 'hourlybilling', 0) . "-" . array_var($t_data, 'hourly_cost', 0);
		} else {
			$group_key = $group_name . "-" . array_var($t_data, 'hourlybilling', 0);
		}

		if (!isset($summary[$group_key])) {
			$summary[$group_key] = array(
				'worked_hours' => 0,
				'hour_type' => $group_name,
				'currency_sym' => isset($t_data['rate_currency_sym']) ? $t_data['rate_currency_sym']: config_option('currency_code'),
				'hourlybilling' => $t_data['hourlybilling'],
				'hourly_cost' => $t_data['hourly_cost'],
				'total_cost' => 0,
				'billing_rate' => '',
				'totalbilling' => 0,
				'miles' => 0,
				'estimated_hours' => 0,
				'invoiced' => 0,
				'not_invoiced' => 0
			);
		}
		$summary[$group_key]['worked_hours'] += $t_data['worked_hours'];
		$summary[$group_key]['totalbilling'] += $t_data['totalbilling'];
		$summary[$group_key]['total_cost'] += $t_data['total_cost'];
		$summary[$group_key]['miles'] += array_var($t_data, 'miles', 0);
		$summary[$group_key]['pending_hours'] = $summary[$group_key]['estimated_hours'] - $summary[$group_key]['worked_hours'];
		$currency_sym = array_var($t_data, 'rate_currency_sym');
		
		if($show_invoice_detail){
			if($t_data['invoicing_status'] == 'invoiced'){
				$summary[$group_key]['invoiced'] += $t_data['totalbilling'];
			} else if($t_data['invoicing_status'] == 'pending'){
				$summary[$group_key]['not_invoiced'] += $t_data['totalbilling'];
			}
		}
	}
	
	// collect totals of submembers member timeslots
	foreach ($subprojects_data as $subproject_data) {
		if (isset($subproject_data['timeslots_summary'])) {
			
			foreach ($subproject_data['timeslots_summary'] as $group_name => $group_summary) {
				if ($group_name == 'subtotal') continue;

				if(in_array(array_var($report_data, 'financial_type'), array('cost', 'price_and_cost'))){
					$group_key = $group_name . "-" . array_var($group_summary, 'hourlybilling', 0) . "-" . array_var($group_summary, 'hourly_cost', 0);
				} else {
					$group_key = $group_name . "-" . array_var($group_summary, 'hourlybilling', 0);
				}
				
				if (!isset($summary[$group_key])) {
					$summary[$group_key] = array(
						'worked_hours' => 0,
						'hour_type' => $group_name,
						'currency_sym' => isset($group_summary['currency_sym']) ? $group_summary['currency_sym'] : config_option('currency_code'),
						'hourlybilling' => array_var($group_summary, 'hourlybilling'),
						'hourly_cost' => array_var($group_summary, 'hourly_cost'),
						'billing_rate' => '',
						'total_cost' => 0,
						'totalbilling' => 0,
						'miles' => 0,
						'estimated_hours' => 0,
						'estimated_billing' => 0,
						'pending_billing' => 0,
						'invoiced' => 0,
						'not_invoiced' => 0
					);
				}
				$summary[$group_key]['worked_hours'] += $group_summary['worked_hours'];
				$summary[$group_key]['totalbilling'] += $group_summary['totalbilling'];
				$summary[$group_key]['total_cost'] += $group_summary['total_cost'];
				$summary[$group_key]['estimated_hours'] += array_var($group_summary, 'estimated_hours', 0);
				$summary[$group_key]['estimated_billing'] += array_var($group_summary, 'estimated_billing', 0);
				$summary[$group_key]['miles'] += array_var($group_summary, 'miles', 0);
				$summary[$group_key]['pending_hours'] = $summary[$group_name]['estimated_hours'] - $summary[$group_name]['worked_hours'];
				$summary[$group_key]['pending_billing'] += array_var($group_summary, 'pending_billing', 0);
				$currency_sym = $group_summary['currency_sym'];
				if($show_invoice_detail){
					$summary[$group_key]['invoiced'] += $group_summary['invoiced'];
					$summary[$group_key]['not_invoiced'] += $group_summary['not_invoiced'];
				}
			}
			
		}
	}

	if( array_var($report_data, 'include_estimated_time') || array_var($report_data, 'include_pending_time') || array_var($report_data, 'include_estimated_billing') || array_var($report_data, 'include_pending_billing')){
		$only_current_level = array_var($report_data, 'include_subproject_info', '') == 'checked';
		$projects_data = get_member_project_tasks($project_member, $report_data, $only_current_level, false);

		foreach ($projects_data as $p_data) {
			// Get Labor category name from 'hour_types' and set group_name
			$p_data_hour_types = array_var($p_data, 'hour_types');
			$group_name = !$p_data_hour_types ? 'Without labor category' : $p_data_hour_types[0];

			if(in_array(array_var($report_data, 'financial_type'), array('cost', 'price_and_cost'))){
				$group_key = $group_name . "-" . array_var($p_data, 'hourlybilling', 0) . "-" . array_var($p_data, 'hourly_cost', 0);
			} else {
				$group_key = $group_name . "-" . array_var($p_data, 'hourlybilling');
			}

			if (!isset($summary[$group_key])){ 
				$summary[$group_key] = array(
					'worked_hours' => 0,
					'hour_type' => $group_name,
					'currency_sym' => isset($currency_sym) ? $currency_sym : config_option('currency_code'),
					'hourlybilling' => $p_data['hourlybilling'],
					'totalbilling' => 0,
					'total_cost' => 0,
					'miles' => 0,
					'estimated_hours' => $p_data['timeEstimate']/60,
					'estimated_billing' => 0,
					'pending_billing' => 0
 				);
			} else {
				$summary[$group_key]['estimated_hours'] += $p_data['timeEstimate']/60;
				$summary[$group_key]['hourlybilling'] = $p_data['hourlybilling'];
			}
			if($group_name != 'Without labor category'){
				$summary[$group_key]['estimated_billing'] += $p_data['timeEstimate']/60 * $summary[$group_key]['hourlybilling'];
			} 
			$summary[$group_key]['pending_hours'] = $summary[$group_key]['estimated_hours'] - $summary[$group_key]['worked_hours'];
			if($group_name != 'Without labor category'){
				$summary[$group_key]['pending_billing'] = $summary[$group_key]['pending_hours'] * $summary[$group_key]['hourlybilling'];
			} 
		}

	}
	
	// order by labor category name
	ksort($summary);

	// Summary subtotals
	if (count($summary) > 0) {
		$subtotals = array('hour_type' => lang('labor subtotal'),
						  'total_cost' => 0,
		                  'totalbilling' => 0,
						  'worked_hours' => 0,
						  'estimated_hours' => 0,
						  'estimated_billing' => 0,
						  'pending_hours' => 0,
						  'pending_billing' => 0,
						  'hourlybilling' => '',
						  'miles' => 0,
	    				  'invoiced' => 0,
						  'not_invoiced' => 0);
		foreach ($summary as $key => $s) {
			if ($key == 'subtotal') continue;
			
			if (!isset($subtotals['currency_sym'])) $subtotals['currency_sym'] = $s['currency_sym'];
			$subtotals['worked_hours'] += $s['worked_hours'];
			$subtotals['total_cost'] += $s['total_cost'];
			$subtotals['totalbilling'] += $s['totalbilling'];
			$subtotals['estimated_hours'] += $s['estimated_hours'];
			if( isset( $s['estimated_billing'] )) $subtotals['estimated_billing'] +=  $s['estimated_billing'];
			$subtotals['pending_hours'] += $s['pending_hours'];
			if( isset( $s['pending_billing'] )) $subtotals['pending_billing'] += $s['pending_billing'];
			$subtotals['miles'] += array_var($s, 'miles', 0);
			$subtotals['invoiced'] += array_var($s, 'invoiced', 0);
			$subtotals['not_invoiced'] += array_var($s, 'not_invoiced', 0);
		}
		$summary['subtotal'] = $subtotals;
	}

	return $summary;
}



function get_project_manager_report_expenses_data(Member $project_member, $report_data = array(), $expenses = null) {
	
	$expenses_data = array();
	
	if (is_null($expenses)) {
		$include_subprojects = array_var($report_data, 'include_subproject_info') == 'checked' ? true : false;
		$only_current_level = $include_subprojects ? true : false;
		$expenses = get_member_expenses($project_member, $report_data, $only_current_level);
	}

	foreach ($expenses as $expense) {
		/* @var $expense PaymentReceipt */
		$e_data = $expense->getArrayInfo();
		$members_sql = "SELECT m.name as member_name, d.code as dimension_code
						FROM ".TABLE_PREFIX."object_members om
						INNER JOIN ".TABLE_PREFIX."members m ON om.member_id=m.id
						INNER JOIN ".TABLE_PREFIX."dimensions d ON d.id = m.dimension_id
						WHERE om.object_id=".$expense->getId()." AND om.is_optimization=0";
		
		$member_rows = DB::executeAll($members_sql);
		
		foreach ($member_rows as $row) {
			if (!isset($e_data[$row['dimension_code']])) $e_data[$row['dimension_code']] = array();
			$e_data[$row['dimension_code']][] = $row['member_name'];
		}
		
		$expenses_data[] = $e_data;
	}
	return $expenses_data;
	
}

function get_project_manager_report_expenses_summary(Member $project_member, $expenses_data, $subprojects_data = array(), $report_data = array()) {

	if (is_null($expenses_data)) {
		$expenses_data = get_project_manager_report_expenses_data($project_member);
	}

	$summary = array();
	$show_invoice_detail = get_show_invoicing_detail($report_data);
	// totalize expenses data grouped by expense categories dimension
	foreach ($expenses_data as $e_data) {
		
		$e_data_exp_cats = array_var($e_data, 'expense_categories');
		if (!$e_data_exp_cats) {
			$group_name = 'Without expense category';
		} else {
			$group_name = $e_data_exp_cats[0];
		}
		
		if (!isset($summary[$group_name])) $summary[$group_name] = array(
				'amount' => 0,
				'expenses_total_cost' => 0,
				'currency_sym' => $e_data['currency_sym'],
				'expense_category' => $group_name,
				'invoiced' => 0,
				'not_invoiced' => 0
		);

		$summary[$group_name]['amount'] += $e_data['total_price'];
		$summary[$group_name]['expenses_total_cost'] += $e_data['amount'];
		if($show_invoice_detail){
			if($e_data['invoicing_status'] == 'invoiced'){
				$summary[$group_name]['invoiced'] += $e_data['total_price'];
			} else if($e_data['invoicing_status'] == 'pending'){
				$summary[$group_name]['not_invoiced'] += $e_data['total_price'];
			}
		}

	}
	// collect expense totals from subprojects
	foreach ($subprojects_data as $subproject_data) {
		if (isset($subproject_data['expenses_summary'])) {
			
			foreach ($subproject_data['expenses_summary'] as $group_name => $group_summary) {
				if ($group_name == 'subtotal') continue;
		
				if (!isset($summary[$group_name])) $summary[$group_name] = array(
						'amount' => 0,
						'expenses_total_cost' => 0,
						'currency_sym' => $group_summary['currency_sym'],
						'expense_category' => $group_name,
						'invoiced' => 0,
						'not_invoiced' => 0
				);
		
				$summary[$group_name]['amount'] += $group_summary['amount'];
				$summary[$group_name]['expenses_total_cost'] += $group_summary['expenses_total_cost'];
				if($show_invoice_detail){
					$summary[$group_name]['invoiced'] += $group_summary['invoiced'];
					$summary[$group_name]['not_invoiced'] += $group_summary['not_invoiced'];
				}
			}
		}
	}

	if( array_var($report_data, 'include_estimated_billing') || array_var($report_data, 'include_pending_billing')){

		$budgeted_expenses = get_member_expenses_budget_report($project_member, $report_data, false);
		foreach($budgeted_expenses as $expense){
			
			// Get expense category name from 'expense_category' and set group_name
			$expense_category_data = array_var($expense, 'expense_categories');
			$group_name = !$expense_category_data ? 'Without expense category' : $expense_category_data[0];
			if(array_var($expense,'expense_items')){
				$expense_total_price = 0;
				foreach($expense['expense_items'] as $e_item){
					$expense_total_price += $e_item['total_price'];
				}

			} else {
				$expense_total_price = 0;
			}
			if (!isset($summary[$group_name])){ 
				$summary[$group_name] = array(
					'amount' => 0,
					'expenses_total_cost' => 0,
					'currency_sym' => isset($group_summary['currency_sym']) ? $group_summary['currency_sym']: config_option('currency_code'),
					'expense_category' => $group_name,
					'estimated_expense_billing' => 0,
					'penging_expense_billing' => 0
 				);
			}
			$summary[$group_name]['estimated_expense_billing'] += $expense_total_price;
			$summary[$group_name]['pending_expense_billing'] = $summary[$group_name]['estimated_expense_billing'] - $summary[$group_name]['amount'];
		}
	}
	
	// order by labor category name
	ksort($summary);
	
	// Summary subtotals
	if (count($summary) > 0) {
		$subtotals = array('expense_category' => lang('expenses subtotal'),
						   'expenses_total_cost' => 0,
		                   'amount' => 0,
						   'estimated_expense_billing' => 0,
						   'pending_expense_billing' => 0,
						   'invoiced' => 0,
						   'not_invoiced' => 0);
		foreach ($summary as $key => $s) {
			if ($key == 'subtotal') continue;
			$subtotals['amount'] += $s['amount'];
			$subtotals['expenses_total_cost'] += $s['expenses_total_cost'];
			$subtotals['invoiced'] += array_var($s, 'invoiced', 0);
			$subtotals['not_invoiced'] += array_var($s, 'not_invoiced', 0);
			
			if (!isset($subtotals['currency_sym'])) $subtotals['currency_sym'] = $s['currency_sym'];
			if( array_var($report_data, 'include_estimated_billing') || array_var($report_data, 'include_pending_billing')){
				$subtotals['estimated_expense_billing'] += $s['estimated_expense_billing'];
				$subtotals['pending_expense_billing'] += $s['pending_expense_billing'];
			}
		}
		$summary['subtotal'] = $subtotals;
	}
	return $summary;
}

function get_project_manager_report_totals_summary($timeslots_summary, $expenses_summary){
	$project_totals = array(
		'timeslots_total_cost' => 0,
		'timeslots_total' => 0,
		'expenses_total' => 0,
		'expenses_total_cost' => 0,
		'worked_hours' => 0,
		'miles' => 0,
		'estimated_hours' => 0,
		'labor_estimated_billing' => 0,
		'estimated_expense_billing' => 0,
		'total_estimated_billing' => 0,
		'labor_pending_billing' => 0,
		'pending_expense_billing' => 0,
		'total_pending_billing' => 0,
		'labor_invoiced' => 0,
		'expense_invoiced' => 0,
		'total_invoiced' => 0,
		'labor_not_invoiced' => 0,
		'expense_not_invoiced' => 0,
		'total_not_invoiced' => 0,
		'project_total_budget' => 0
	);
	
	if (isset($timeslots_summary['subtotal'])) {
		$summary = $timeslots_summary['subtotal'];
		
		if (!isset($project_totals['estimated_hours'])) $project_totals['estimated_hours'] = 0;
		if (!isset($project_totals['estimated_billing'])) $project_totals['estimated_billing'] = 0;
		if (!isset($project_totals['pending_hours'])) $project_totals['pending_hours'] = 0;
		if (!isset($project_totals['pending_billing'])) $project_totals['pending_billing'] = 0;
		if (!isset($project_totals['miles'])) $project_totals['miles'] = 0;
		
		$project_totals['worked_hours'] += array_var($summary, 'worked_hours');
		$project_totals['estimated_hours'] += array_var($summary, 'estimated_hours');
		$project_totals['labor_estimated_billing'] += array_var($summary, 'estimated_billing');
		$project_totals['total_estimated_billing'] += array_var($summary, 'estimated_billing');
		$project_totals['pending_hours'] += array_var($summary, 'pending_hours');
		$project_totals['labor_pending_billing'] += array_var($summary, 'pending_billing');
		$project_totals['total_pending_billing'] += array_var($summary, 'pending_billing');
		$project_totals['timeslots_total'] += array_var($summary, 'totalbilling');
		$project_totals['timeslots_total_cost'] += array_var($summary, 'total_cost');
		$project_totals['currency_sym'] = array_var($summary, 'currency_sym');
		$project_totals['miles'] += array_var($summary, 'miles', 0);
		$project_totals['labor_invoiced'] += array_var($summary, 'invoiced', 0);
		$project_totals['total_invoiced'] += array_var($summary, 'invoiced', 0);
		$project_totals['labor_not_invoiced'] += array_var($summary, 'not_invoiced', 0);
		$project_totals['total_not_invoiced'] += array_var($summary, 'not_invoiced', 0);
	}
	
	if (isset($expenses_summary['subtotal'])) {
		$summary = $expenses_summary['subtotal'];
		$project_totals['expenses_total_cost'] += array_var($summary, 'expenses_total_cost');
		$project_totals['expenses_total'] += array_var($summary, 'amount');
		$project_totals['estimated_expense_billing'] += array_var($summary, 'estimated_expense_billing');
		$project_totals['total_estimated_billing'] += array_var($summary, 'estimated_expense_billing');
		$project_totals['pending_expense_billing'] += array_var($summary, 'pending_expense_billing');
		$project_totals['total_pending_billing'] += array_var($summary, 'pending_expense_billing');
		$project_totals['currency_sym'] = array_var($summary, 'currency_sym');
		$project_totals['expense_invoiced'] += array_var($summary, 'invoiced', 0);
		$project_totals['total_invoiced'] += array_var($summary, 'invoiced', 0);
		$project_totals['expense_not_invoiced'] += array_var($summary, 'not_invoiced', 0);
		$project_totals['total_not_invoiced'] += array_var($summary, 'not_invoiced', 0);
	}

	return $project_totals;
}

function get_project_total_budget($project_member, &$totals_summary, $report_data){ 
	if(array_var($report_data, 'for_budget_use') == 'tasks_expenses'){
		$project_member_id = $project_member->getId();
		$project_totals = MemberTotals::instance()->findOne(array('conditions' => array('`member_id`="'.$project_member_id.'"')));
		$totals_summary['project_total_budget'] = $project_totals instanceof MemberTotal ? $project_totals->getColumnValue('estimated_overall_total_price') : 0;
	} else if(array_var($report_data, 'for_budget_use') == 'budget_total_cp'){
		$cp_object_type_id = $project_member->getObjectTypeId();
		$budget_custom_property = CustomProperties::instance()->findOne(array('conditions' => '`object_type_id`='.$cp_object_type_id. ' AND `code`="budget_total"'));
		$values = CustomPropertyValues::getCustomPropertyValues($project_member->getObjectId(), $budget_custom_property->getId());
		$custom_value = !empty($values) && $values[0] instanceof CustomPropertyValue ? $values[0]->getValue() : 0;
		$total_budget = floatval(str_replace(',','', $custom_value)) > 0 ? floatval(str_replace(',','', $custom_value)) : 0;  
		$totals_summary['project_total_budget'] = $total_budget;
	}
	// Add contract type
	$cp_object_type_id = $project_member->getObjectTypeId();
	$contract_type_custom_property = CustomProperties::instance()->findOne(array('conditions' => '`object_type_id`='.$cp_object_type_id. ' AND `code`="contract_type"'));
	$contract_type = '';
	if($contract_type_custom_property instanceof CustomProperty){
		$cp_values = CustomPropertyValues::getCustomPropertyValues($project_member->getObjectId(), $contract_type_custom_property->getId());
		$contract_type = !empty($cp_values) && $cp_values[0] instanceof CustomPropertyValue ? $cp_values[0]->getValue() : '';
	}
	$totals_summary['contract_type'] = $contract_type;
}

function get_project_member_ids_for_project_manager_report($check_active_context, $report_data = array()){
	$include_subprojects = array_var($report_data, 'include_subproject_info') == 'checked' ? true : false;
	$use_other_associated_member_ids = array_var($report_data, 'use_other_associated_member_ids', false);
	$other_associated_member_ids = array_var($report_data, 'other_associated_member_ids', array());
	
	$project_ot = ObjectTypes::findByName('project');
	$project_dimension = Dimensions::findByCode('customer_project');
	$associated_member_ids = get_associated_member_ids_from_active_context($project_dimension->getId(), $project_ot->getId());
	if($check_active_context && !count($associated_member_ids)){
		flash_error(lang('please select a project or a dimension member associated with projects'));
		ajx_current('empty');
		return ;
	}

	$JOIN_SQL = '';
	$CONDITION_SQL = '';
	$associated_member_ids = $use_other_associated_member_ids ? $other_associated_member_ids : $associated_member_ids;
	if(count($associated_member_ids) > 0){
		$count = 0;
		foreach($associated_member_ids as $m_id){
			$member = Members::instance()->findById($m_id);
			// Exclude associated subprojects from the conditions
			if($member->getObjectTypeId() == $project_ot->getId()) continue;
			$table_alias = $count == 0 ? 'mpm' : 'mpm'.$count;
			$JOIN_SQL .= " INNER JOIN ".TABLE_PREFIX."member_property_members ".$table_alias." ON m.id=".$table_alias.".member_id ";
			$CONDITION_SQL .= " AND ".$table_alias.".property_member_id=".$m_id." ";
			$count++;
		}
	}

	// Get project member ids
	if($include_subprojects) {
		$SQL = "SELECT DISTINCT(m.id)
			FROM ".TABLE_PREFIX."members m ".$JOIN_SQL." 
			WHERE m.object_type_id=" . $project_ot->getId() . $CONDITION_SQL;

		$p_rows = DB::executeAll($SQL);
		$project_member_ids = array_map(function ($row){
								return $row['id'];
							}, $p_rows);

	} else { 
		// Get parent projects only
		$PARENT_SQL = "SELECT DISTINCT(m.id)
				FROM ".TABLE_PREFIX."members m ".$JOIN_SQL." 
				WHERE m.object_type_id=" . $project_ot->getId() . $CONDITION_SQL . " AND m.parent_member_id=0";

		$parent_rows = DB::executeAll($PARENT_SQL);
		$parent_project_member_ids = array_map(function ($row){
								return $row['id'];
							}, $parent_rows);

		// Get projects located under other members than projects
		$non_project_parent_sql = "SELECT DISTINCT(m.id)
				FROM ".TABLE_PREFIX."members m ".$JOIN_SQL." 
				WHERE m.object_type_id=" . $project_ot->getId() . $CONDITION_SQL . " AND (SELECT object_type_id FROM fo_members p WHERE p.id=m.parent_member_id)!=" .  $project_ot->getId();

		$non_project_parent_rows = DB::executeAll($non_project_parent_sql);
		$non_project_parent_project_member_ids = array_map(function ($row){
								return $row['id'];
							}, $non_project_parent_rows ? $non_project_parent_rows : array());

		// Filter out projects that have parent projects somewhere in the hierarchy
		$other_parent_project_member_ids = array();
		foreach($non_project_parent_project_member_ids as $project_member_id) {
			$project_member = Members::instance()->findById($project_member_id);
			$parents = $project_member->getAllParentMembersInHierarchy();
			$is_parent = true;
			foreach($parents as $parent) {
				if($parent->getObjectTypeId() == $project_ot->getId()) {
					$is_parent = false;
					break;
				}
			}
			if($is_parent) $other_parent_project_member_ids[] = $project_member_id;
		}

		// Merge all project ids
		$project_member_ids = array_merge($parent_project_member_ids, $other_parent_project_member_ids);
	}

	return $project_member_ids;
}

function get_associated_member_ids_from_active_context($dimension_id, $object_type_id){
	$persons_dim = Dimensions::findByCode('feng_persons');
	if ($persons_dim instanceof Dimension) $persons_dim_id = $persons_dim->getId();
	else $persons_dim_id = 0;

	// get associations (exclude persons dimension)
	$associated_dimension_ids = array();
	$dimension_associations = DimensionMemberAssociations::instance()->findAll(array(
			"conditions" => array("(`dimension_id` = ? AND `associated_dimension_id` != ? AND `object_type_id` = ?)
					OR (`associated_dimension_id` = ? AND `dimension_id` != ? AND `associated_object_type_id` = ?)",
					$dimension_id, $persons_dim_id, $object_type_id, $dimension_id, $persons_dim_id, $object_type_id)
	));
	
	// dimension ids
	$enabled_dimensions = config_option("enabled_dimensions");
	foreach ($dimension_associations as $da) {
		/* @var $da DimensionMemberAssociation */
		if ($da->getDimensionId() == $dimension_id && in_array($da->getAssociatedDimensionMemberAssociationId(), $enabled_dimensions)) {
			$associated_dimension_ids[] = $da->getAssociatedDimensionMemberAssociationId();
		} else if (in_array($da->getDimensionId(), $enabled_dimensions)) {
			$associated_dimension_ids[] = $da->getDimensionId();
		}
	}

	// get current selected members and its children that area associated to this dimension
	$associated_member_ids = array();
	$context = active_context();
	if (is_array($context)) {
		foreach ($context as $sel) {
			if ($sel instanceof Member && in_array($sel->getDimensionId(), $associated_dimension_ids)) {
				$associated_member_ids[] = $sel->getId();
				$mem_ob_type_id = $sel->getObjectTypeId();
				$mem_children = $sel->getAllChildrenIdsByType($mem_ob_type_id);
				if(!empty($mem_children)){
					foreach($mem_children as $m_child){
						$associated_member_ids[] = $m_child;
					}
				}
			}
		}
	}

	return array_unique($associated_member_ids);
}

function get_project_member_ids_filtered_by_date($report_data){
	$project_ot = ObjectTypes::findByName('project');

	// Get start and end date SQLs
	$start_date = array_var($report_data, 'start_date');
	$end_date = array_var($report_data, 'end_date');
	$sql_st_date = $start_date->toMySQL();
	$sql_end_date = $end_date->toMySQL();
	
	// Get start and end date SQLs for expenses
	$expense_start_date = array_var($report_data, 'expense_start_date');
	$expense_end_date = array_var($report_data, 'expense_end_date');
	$sql_exp_st_date = $expense_start_date->toMySQL();
	$sql_exp_end_date = $expense_end_date->toMySQL();

	// Get project member ids assigned to timeslots in the given date range
	$time_sql = "SELECT DISTINCT(m.id)
			FROM ".TABLE_PREFIX."timeslots t INNER JOIN ".TABLE_PREFIX."object_members om ON t.object_id=om.object_id
			INNER JOIN ".TABLE_PREFIX."members m ON om.member_id=m.id
			WHERE t.start_time>='".$sql_st_date."' AND t.start_time<='".$sql_end_date."'
			AND m.object_type_id=".$project_ot->getId().";";
	$time_rows = DB::executeAll($time_sql);
	$timeslot_project_member_ids = array();
	if(!empty($time_rows)){
		$timeslot_project_member_ids = array_map(function ($row){
											return $row['id'];
										}, $time_rows);
	}

	// Get project member ids assigned to actual expenses in the given date range										
	$expense_sql = "SELECT DISTINCT(m.id)
				FROM ".TABLE_PREFIX."payment_receipts pr INNER JOIN ".TABLE_PREFIX."object_members om ON pr.object_id=om.object_id
				INNER JOIN ".TABLE_PREFIX."members m ON om.member_id=m.id
				WHERE pr.date>='".$sql_exp_st_date."' AND pr.date<='".$sql_exp_end_date."'
				AND m.object_type_id=".$project_ot->getId().";";
	$expense_rows = DB::executeAll($expense_sql);
	$expense_project_member_ids = array();
	if(!empty($expense_rows)){
		$expense_project_member_ids = array_map(function ($row){
										return $row['id'];
									}, $expense_rows);
	}


	// Merge time and expense project member ids										
	$combined_project_member_ids = array_unique(array_merge($timeslot_project_member_ids, $expense_project_member_ids));

	return $combined_project_member_ids;
}

function get_object_assoc_active_context_members($object_type_id){
	$member_ids = array();
	$active_member_ids = active_context_members(false);
	if(is_array($active_member_ids)){
		foreach($active_member_ids as $mem_id){
			$member = Members::instance()->findById($mem_id);
			if(!$member instanceof Member) continue;
			$dimension_id = $member->getDimensionId();
			$ot_dim_option = DimensionContentObjectOptions::getOptionValue($dimension_id, $object_type_id, 'hide_member_selector_in_forms');
			if(!$ot_dim_option instanceof DimensionContentObjectOption) continue;
			if ($ot_dim_option->getValue() == 1) continue;
			$member_ids[] = $mem_id;
		}
	}
	return $member_ids;
}

function get_show_invoicing_detail($report_data) {

	$invoicing_status = array_var($report_data, 'invoice_status', 'empty');
	$result = in_array($invoicing_status, array('all_regular', 'all_billable_only', 'empty'))  && array_var($report_data, 'include_invoicing_details', '') == 'checked';

	return $result;
}
