<?php 

function build_excel_gross_margin_report($data, $report_data) {
    $excel = array();
    
    $excel[] = prepare_header_for_gross_margin_report_excel($data, $report_data);
    $excel[] = prepare_fin_data_for_gross_margin_report_excel($data, $report_data);
   
    return $excel;
}

function prepare_header_for_gross_margin_report_excel($data, $report_data) {
    $months_range = array_var($data, 'months_between_dates', array());
    $current_colspan = array_var($data, 'count_current_months', 0);
    $forecasted_colspan = array_var($data, 'count_forecasted_months', 0);
    $mapped_months = array_var($data, 'mapped_months', array());
    $columns = array_var($data, 'columns', array());

    $excel_table = array(
        'totals' => array('')
    );

    // Header
    foreach($months_range as $column) {
        $column_arr = explode('-', $column);
        $year = $column_arr[0];
        $month = $column_arr[1];
        $month_str = ucfirst(lang('month '. intval($month)));
        $month_abbr = substr($month_str, 0, 3);
        $class = $mapped_months[$column] == 'forecasted' ? 'forecasted_level_text' : '';

        $excel_table['totals'][] = $month_abbr . ' ' . $year;
    }

    $excel_table['totals'][] = lang('totals');

    return $excel_table;
}

function prepare_fin_data_for_gross_margin_report_excel($data, $report_data) {
    $actual_revenue_type = array_var($report_data, 'revenue', 'earned_value');
    $mapped_months = array_var($data, 'mapped_months', array());
    $fin_types = array('estimated_revenue');
    $fin_types[] = $actual_revenue_type == 'earned_value' ? 'earned_value' : 'invoiced';
    $rest_fin_types = array('estimated_costs', 'actual_costs', 'gross_profit_estimated', 'gross_profit_actual', 'gross_margin_estimated', 'gross_margin_actual');
    $fin_types = array_merge($fin_types, $rest_fin_types);

    $excel_table = array(
        'data' => array()
    );

    $currency_sym = config_option('currency_code');
    foreach($fin_types as $fin_type) {
        $data_row = array();
        $data_row[] = lang($fin_type.' gross_margin');
        $columns = array_var($data, 'months_between_dates', array());
        foreach($columns as $column) {
            $amount = array_var($data['fin_data'][$fin_type]['months'], $column, 0);
            if($fin_type == 'gross_margin_estimated' || $fin_type == 'gross_margin_actual') {
                $val = format_amount($amount) . ' %';
            } else {
                $val = $amount != 0 ? format_money_amount($amount, $currency_sym) : '';
            }
            $data_row[] = $val;
        }
        $total_amount = array_var($data['fin_data'][$fin_type], 'total', 0);
        if($fin_type == 'gross_margin_estimated' || $fin_type == 'gross_margin_actual') {
            $total_val = format_amount($total_amount) . ' %';
        } else {
            $total_val = $total_amount != 0 ? format_money_amount($total_amount, $currency_sym) : '';
        }
        $data_row[] = $total_val;
        
        $excel_table['data'][] = $data_row;
    }
    
    return $excel_table;
}
