<?php

function get_main_gross_margin_report_data($report_data) {
    $report_for = array_var($report_data, 'report_for');
    $selected_member_id = 0;
    if(strpos($report_for, '_dim')) {
	    $member_ids = json_decode($report_data[$report_for.'_id']);
        $selected_member_id = !empty($member_ids) ? $member_ids[0] : 0;
    }

    Env::useHelper('actual_vs_forecast_report_functions', 'project_reports');
    $result = array();
    getMonthsBetweenDates($report_data, $result);
    map_months_to_previous_and_forecasted_months($report_data, $result);

    $fin_data = array(
        'estimated_revenue' => array('months' => array(), 'total' => 0),
        'earned_value' => array('months' => array(), 'total' => 0),
        'invoiced' => array('months' => array(), 'total' => 0),
        'estimated_costs' => array('months' => array(), 'total' => 0),
        'actual_costs' => array('months' => array(), 'total' => 0),
        'gross_profit_estimated' => array('months' => array(), 'total' => 0),
        'gross_profit_actual' => array('months' => array(), 'total' => 0),
        'gross_margin_estimated' => array('months' => array(), 'total' => 0),
        'gross_margin_actual' => array('months' => array(), 'total' => 0),
    );

    hydrate_fin_data_with_months_keys($fin_data, $result);

    collect_revenue_and_costs_for_gross_margin_report($fin_data, $selected_member_id, $report_data);
    calculate_gross_profit_and_margin_for_gross_margin_report($fin_data, $report_data);

    $result['fin_data'] = $fin_data;

    return $result;
}

function collect_revenue_and_costs_for_gross_margin_report(&$fin_data, $selected_member_id, $report_data) {
    get_estimated_values_for_gross_margin_report($fin_data, $report_data, $selected_member_id);
    get_actual_and_invoiced_values_for_gross_margin_report($fin_data, $report_data, $selected_member_id);
}

function calculate_gross_profit_and_margin_for_gross_margin_report(&$fin_data, $report_data) {
    $revenue_type = array_var($report_data, 'revenue', 'earned_value');

    foreach($fin_data['estimated_revenue']['months'] as $month => $estimated_revenue) {
        // Estimated gross profit
        $estimated_costs = array_var($fin_data['estimated_costs']['months'], $month, 0);
        $gross_profit_estimated = $estimated_revenue - $estimated_costs;
        $fin_data['gross_profit_estimated']['months'][$month] = $gross_profit_estimated;
        $fin_data['gross_profit_estimated']['total'] += $gross_profit_estimated;

        // Actual gross profit
        if($revenue_type == 'earned_value') {
            $actual_revenue = array_var($fin_data['earned_value']['months'], $month, 0);
        } else {
            $actual_revenue = array_var($fin_data['invoiced']['months'], $month, 0);
        }
        $actual_costs = array_var($fin_data['actual_costs']['months'], $month, 0);
        $gross_profit_actual = $actual_revenue - $actual_costs;
        $fin_data['gross_profit_actual']['months'][$month] = $gross_profit_actual;
        $fin_data['gross_profit_actual']['total'] += $gross_profit_actual;
    
        // Estimated gross margin
        $gross_margin_estimated = $estimated_revenue != 0 ? ($gross_profit_estimated / $estimated_revenue) * 100 : 0;
        $fin_data['gross_margin_estimated']['months'][$month] = $gross_margin_estimated;

        // Actual gross margin
        $gross_margin_actual = $actual_revenue != 0 ? ($gross_profit_actual / $actual_revenue) * 100 : 0;
        $fin_data['gross_margin_actual']['months'][$month] = $gross_margin_actual;        
    }

    // Total gross margin
    $fin_data['gross_margin_estimated']['total'] = $fin_data['estimated_revenue']['total'] != 0 ? ($fin_data['gross_profit_estimated']['total'] / $fin_data['estimated_revenue']['total']) * 100 : 0;
    $fin_data['gross_margin_actual']['total'] = $fin_data['earned_value']['total'] != 0 ? ($fin_data['gross_profit_actual']['total'] / $fin_data['earned_value']['total']) * 100 : 0;
}

function get_estimated_values_for_gross_margin_report(&$fin_data, $report_data, $selected_member_id) {
    Env::useHelper('report_helper_functions', 'project_reports');

    $start_date = array_var($report_data, 'start_date');
    $end_date = array_var($report_data, 'end_date');
    $sql_st_date = $start_date->toMySQL();
    $sql_end_date = $end_date->toMySQL();

    $member_cond = '';
    if($selected_member_id > 0) {
        $member_cond = " AND om.member_id=".$selected_member_id." ";
    }

    // Get estimated revenue and cost from tasks grouped by month and year
    $task_row = DB::execute("
        SELECT SUM(t.estimated_price) as estimated_revenue, SUM(t.estimated_cost) as estimated_costs, LPAD(MONTH(t.due_date),2,'0') as month, YEAR(t.due_date) as year
        FROM ".TABLE_PREFIX."project_tasks t
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=t.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        AND (t.due_date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."')
        ".$member_cond."
        AND t.depth=0
        GROUP BY MONTH(t.due_date), YEAR(t.due_date);");
    $task_rows = $task_row->fetchAll();

    if($task_rows) {
        foreach($task_rows as $task_row) {
            add_row_to_fin_data($fin_data, $task_row, 'estimated_revenue');
            add_row_to_fin_data($fin_data, $task_row, 'estimated_costs');
        }
    }

    prepare_dates_for_expenses_sql($start_date, $end_date, $sql_st_date, $sql_end_date);

    // Get estimated revenue and cost from expenses grouped by month and year for this project
    $expense_row = DB::execute("
        SELECT SUM(ei.total_price) as estimated_revenue, SUM(ei.total_cost) as estimated_costs, LPAD(MONTH(ei.date),2,'0') as month, YEAR(ei.date) as year
        FROM ".TABLE_PREFIX."expenses e
        INNER JOIN ".TABLE_PREFIX."expense_items ei on ei.expense_id=e.object_id
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=e.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        AND (ei.date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."')
        ".$member_cond."
        AND e.task_id=0
        GROUP BY MONTH(ei.date), YEAR(ei.date);");  
    $expense_rows = $expense_row->fetchAll();

    if($expense_rows) {
        foreach($expense_rows as $expense_row) {
            add_row_to_fin_data($fin_data, $expense_row, 'estimated_revenue');
            add_row_to_fin_data($fin_data, $expense_row, 'estimated_costs');
        }
    }

    revert_dates_after_expenses_sql($start_date, $end_date);
}

function get_actual_and_invoiced_values_for_gross_margin_report(&$fin_data, $report_data, $selected_member_id) {
    Env::useHelper('report_helper_functions', 'project_reports');

    $start_date = array_var($report_data, 'start_date');
    $end_date = array_var($report_data, 'end_date');
    $sql_st_date = $start_date->toMySQL();
    $sql_end_date = $end_date->toMySQL();

    // Get all task ids and it's subtasks with manual estimated price and fixed fee
    $fixed_fee_task_ids = array();
    if(Plugins::instance()->isActivePlugin('advanced_billing')) {
            Env::useHelper('persisting_member_totals_functions', 'advanced_billing');
            $mem_id = $selected_member_id > 0 ? $selected_member_id : 0;
            $fixed_fee_task_ids = get_tasks_and_subtask_ids_with_manual_financial_value($mem_id, 'is_calculated_estimated_price', true);
    } 

    $member_cond = '';
    if($selected_member_id > 0) {
        $member_cond = ' AND om.member_id='.$selected_member_id;
    }

    // Get fixed fee task ids for this project grouped by month and year
    if(!empty($fixed_fee_task_ids)) {
        $fixed_fee_cond = 'AND t.object_id IN ('.implode(',', $fixed_fee_task_ids).')';
        $date_column = "t.due_date";
        $fixed_fee_task_row = DB::execute("
            SELECT SUM(t.earned_value) as earned_value, LPAD(MONTH(".$date_column."),2,'0') as month, YEAR(".$date_column.") as year
            FROM ".TABLE_PREFIX."project_tasks t
            INNER JOIN ".TABLE_PREFIX."objects o on o.id=t.object_id
            INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
            WHERE o.trashed_by_id=0
            ".$fixed_fee_cond."
            AND t.depth=0 
            AND ".$date_column." BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
            ".$member_cond." 
            GROUP BY MONTH(".$date_column."), YEAR(".$date_column.");");
        $fixed_fee_task_rows = $fixed_fee_task_row->fetchAll();

        if($fixed_fee_task_rows) {
            foreach($fixed_fee_task_rows as $fixed_fee_task_row) {
                add_row_to_fin_data($fin_data, $fixed_fee_task_row, 'earned_value');
            }
        }
    }

    // Get invoiced amount for grouped by month and year
    $invoiced_row = DB::execute("
        SELECT SUM(i.total) as invoiced, LPAD(MONTH(i.date),2,'0') as month, YEAR(i.date) as year
        FROM ".TABLE_PREFIX."income_invoices i
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=i.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        AND i.date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
        ".$member_cond."
        GROUP BY MONTH(i.date), YEAR(i.date);");
    $invoiced_rows = $invoiced_row->fetchAll();

    if($invoiced_rows) {
        foreach($invoiced_rows as $invoiced_row) {
            add_row_to_fin_data($fin_data, $invoiced_row, 'invoiced');
        }
    }

    // Get timeslots grouped by month and year
    $time_fixed_fee_cond = !empty($fixed_fee_task_ids) ? 'AND ts.rel_object_id NOT IN ('.implode(',', $fixed_fee_task_ids).')' : '';
    $time_row = DB::execute("
        SELECT SUM(ts.fixed_billing) as earned_value, SUM(ts.fixed_cost) as actual_costs, LPAD(MONTH(ts.start_time),2,'0') as month, YEAR(ts.start_time) as year
        FROM ".TABLE_PREFIX."timeslots ts
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=ts.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        AND ts.start_time BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
        ".$member_cond."
        ".$time_fixed_fee_cond."
        GROUP BY MONTH(ts.start_time), YEAR(ts.start_time);");
    $time_rows = $time_row->fetchAll();

    if($time_rows) {
        foreach($time_rows as $time_row) {
            add_row_to_fin_data($fin_data, $time_row, 'earned_value');
            add_row_to_fin_data($fin_data, $time_row, 'actual_costs');
        }
    }

    prepare_dates_for_expenses_sql($start_date, $end_date, $sql_st_date, $sql_end_date);

    // Get actual expenses(payment_receipts) for this project grouped by month and year
    $exp_fixed_fee_cond = !empty($fixed_fee_task_ids) ? 'AND pr.task_id NOT IN ('.implode(',', $fixed_fee_task_ids).')' : '';
    $expense_row = DB::execute("
        SELECT SUM(pr.total_price) as earned_value, SUM(pr.amount) as actual_costs, LPAD(MONTH(pr.date),2,'0') as month, YEAR(pr.date) as year
        FROM ".TABLE_PREFIX."payment_receipts pr
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=pr.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        AND pr.date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
        ".$member_cond."
        ".$exp_fixed_fee_cond."
        GROUP BY MONTH(pr.date), YEAR(pr.date);");
    $expense_rows = $expense_row->fetchAll();

    if($expense_rows) {
        foreach($expense_rows as $expense_row) {
            add_row_to_fin_data($fin_data, $expense_row, 'earned_value');
            add_row_to_fin_data($fin_data, $expense_row, 'actual_costs');
        }
    }

    revert_dates_after_expenses_sql($start_date, $end_date);
}

function add_row_to_fin_data(&$fin_data, $data_row, $data_type) {
    $year = $data_row['year'];
    $month = $data_row['month'];
    $month_val = $year.'-'.$month;
    $value = $data_row[$data_type];

    if(!isset($fin_data[$data_type]['months'][$month_val])) {
        $fin_data[$data_type]['months'][$month_val] = 0;
    }
    $fin_data[$data_type]['months'][$month_val] += $value;
    $fin_data[$data_type]['total'] += $value;
}

function hydrate_fin_data_with_months_keys(&$fin_data, $result) {
    $months_between_dates = $result['months_between_dates'];
    foreach($fin_data as $data_type => $data) {
        foreach($months_between_dates as $month) {
            if(!isset($fin_data[$data_type]['months'][$month])) {
                $fin_data[$data_type]['months'][$month] = 0;
            }
        }
    }
}

function map_months_to_previous_and_forecasted_months(&$report_data, &$result) {
    $months = $result['months_between_dates'];

    $current = 0;
    $forecasted = 0;
    $mapped_months = array();
    $today_dtv = DateTimeValueLib::now();
    foreach($months as $month) {
        $month_str = $month;
        $month_dtv = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $month.'-01');
        $month = $month_dtv->add('M', 1)->add('d', -1);
        if($month < $today_dtv) {
            $mapped_months[$month_str] = 'current';
            $current += 1;
        } else {
            $mapped_months[$month_str] = 'forecasted';
            $forecasted += 1;
        }
    }

    $report_data['mapped_months'] = $mapped_months;
    $report_data['count_current_months'] = $current;
    $report_data['count_forecasted_months'] = $forecasted;

    $result['mapped_months'] = $mapped_months;
    $result['count_current_months'] = $current;
    $result['count_forecasted_months'] = $forecasted;
}
