<?php


function get_project_detail_jdt_report_data($report_data) {

	/**
	 * get all the timeslots and actual expenses of a project grouped by job phase
	 * for each phase make one section for labor and other for actual expenses
	 * group timeslots by task and user
	 * group actual expenses by expense category
	 *
	 * 
	 * Project AAAAA
	 * 
	 *    phase p1
	 * 		Labor
	 * 		  Task A
	 * 			User 1
	 * 				Timeslot X: date - hours - cost - billing
	 * 				Timeslot Y: date - hours - cost - billing
 	 *	 			...
	 * 			Total User 1
	 * 			User 2
	 * 				Timeslot X: date - hours - cost - billing
	 * 				Timeslot Y: date - hours - cost - billing
 	 *	 			...
	 * 			Total User 2
	 * 		  Total Task A
	 * 		Totals Labor
	 * 		Expenses
	 * 		  Expense category 1
	 * 			Expense X: expense_id - date - user - cost - billing
	 * 			Expense Y data
	 * 			...
	 * 		  Total Expense category 1
	 * 		Total Expenses
	 *   Total Job phase p1 (labor + expenses)
	 * 
	 *   Job phase p2
	 * 		...
	 *   Total Job phase p2
	 * 
	 * Totals AAAAA (sum of cost and billing of all phases)
	 */

	// selected project

	$project_member_id_param = json_decode(array_var($report_data, 'project_member_id'), true);
	$project_member_id = array_var($project_member_id_param, 0);
	$project_member = Members::getMemberById($project_member_id);

	if (!$project_member instanceof Member) {
		flash_error('A project has to be selected to run the report');
		ajx_current('empty');
		return;
	}

	// get the grouped labor data
	$grouped_labor_and_expenses = get_project_detail_jdt_report_labor_data($project_member_id);
	
	// get the grouped expenses data
	if (array_var($report_data, 'include_expenses')) {
		$grouped_labor_and_expenses = get_project_detail_jdt_report_expenses_data($project_member_id, $grouped_labor_and_expenses);
	}

	// build totals for project
	// *** LC 2023-09-04  $report_result no definido ¿ tal vez se quizo usar $report_data ?????
	$grouped_labor_and_expenses['totals'] = array_var($report_data, 'totals', array('hours' => 0, 'cost' => 0, 'billing' => 0, 'currency_id' => null)); // *** LC

	foreach ($grouped_labor_and_expenses['phases'] as &$phase_group) {
		$grouped_labor_and_expenses['totals']['hours'] += $phase_group['totals']['hours'];
		$grouped_labor_and_expenses['totals']['cost'] += $phase_group['totals']['cost'];
		$grouped_labor_and_expenses['totals']['billing'] += $phase_group['totals']['billing'];
		$grouped_labor_and_expenses['totals']['currency_id'] = $phase_group['totals']['currency_id'];
	}

	// calculate project full name
	$tmp_mem = array($project_member);
	build_member_list_text_to_show_in_trees($tmp_mem);

	// add the project member object to the data
	$grouped_labor_and_expenses['project_member_id'] = $project_member->getId();
	$grouped_labor_and_expenses['project_name'] = $project_member->getName();

	$columns = array('item', 'date', 'hours', 'cost', 'billing');

	$data = array(
		'columns' => $columns,
		'report_data' => $grouped_labor_and_expenses,
	);

	return $data;
}

/**
 * @return array The time rows grouped by job phase, task and user
 */
function get_project_detail_jdt_report_labor_data($project_member_id) {

	$job_phases_dim = Dimensions::findByCode('job_phases');

	$phase_col_sql = "coalesce((select p.id from ".TABLE_PREFIX."members p inner join ".TABLE_PREFIX."object_members om on om.member_id=p.id 
		where om.object_id=t.object_id and p.dimension_id = ".$job_phases_dim->getId()." limit 1), 0) as phase_id";

	$labor_sql = "
		select 
			t.object_id,
			t.start_time,
			t.worked_time,
			t.description,
			t.rel_object_id as task_id, ta.name as task_name,
			t.contact_id, c.name as contact_name,
			t.rate_currency_id, t.fixed_billing,
			t.cost_currency_id, t.fixed_cost,
			$phase_col_sql
			
		from ".TABLE_PREFIX."timeslots t
		inner join ".TABLE_PREFIX."objects o on o.id = t.object_id
		inner join ".TABLE_PREFIX."object_members om on om.object_id = t.object_id
		inner join ".TABLE_PREFIX."members m on m.id = om.member_id
		inner join ".TABLE_PREFIX."objects c on c.id = t.contact_id
		left join ".TABLE_PREFIX."objects ta on ta.id = t.rel_object_id
		
		where m.id = $project_member_id
		and o.trashed_on = 0
	";

	$time_rows = DB::executeAll($labor_sql);

	// group the plain timeslot rows by phase, task and user
	$phase_name_cache = array();
	$phase_groups = array();
	if ($time_rows) {
		foreach ($time_rows as &$time_row) {

			// get the full name of the phase
			$time_row['phase_name'] = '';
			if ($time_row['phase_id'] > 0) {
				$time_row['phase_name'] = array_var($phase_name_cache, $time_row['phase_id']);
				if (!$time_row['phase_name']) {
					$phase_member = Members::getMemberById($time_row['phase_id']);
					if ($phase_member) {
						$time_row['phase_name'] = $phase_member->getName();
						$phase_name_cache[$time_row['phase_id']] = $time_row['phase_name'];
					}
				}
			}

			// phase group
			$phase_key = $time_row['phase_id'] > 0 ? $time_row['phase_name'] : 'ZZZZZ';
			if (!isset($phase_groups[$phase_key])) {
				$phase_groups[$phase_key] = array(
					'group_name' => $time_row['phase_id'] > 0 ? $time_row['phase_name'] : 'Without phase',
					'labor' => array(),
					'expenses' => array(),
					'totals' => array('hours' => 0, 'cost' => 0, 'billing' => 0, 'currency_id' => $time_row['rate_currency_id']),
					'labor_totals' => array('hours' => 0, 'cost' => 0, 'billing' => 0, 'currency_id' => $time_row['rate_currency_id']),
					'expense_totals' => array('hours' => 0, 'cost' => 0, 'billing' => 0, 'currency_id' => $time_row['rate_currency_id']),
				);
			}

			// task group inside the phase group
			$task_key = $time_row['task_id'] > 0 ? $time_row['task_name'] : 'ZZZZZ';
			if (!isset($phase_groups[$phase_key]['labor'][$task_key])) {
				$phase_groups[$phase_key]['labor'][$task_key] = array(
					'users' => array(),
					'group_name' => $time_row['task_id'] > 0 ? $time_row['task_name'] : 'Without task',
					'totals' => array('hours' => 0, 'cost' => 0, 'billing' => 0, 'currency_id' => $time_row['rate_currency_id']),
				);
			}

			// user group inside the task group
			$user_key = $time_row['contact_name'];
			if (!isset($phase_groups[$phase_key]['labor'][$task_key]['users'][$user_key])) {
				$phase_groups[$phase_key]['labor'][$task_key]['users'][$user_key] = array(
					'timeslots' => array(),
					'group_name' => $user_key,
					'totals' => array('hours' => 0, 'cost' => 0, 'billing' => 0, 'currency_id' => $time_row['rate_currency_id']),
				);
			}

			// calculate the amount of hours from the worked time in minutes
			$ts_hours = number_format($time_row['worked_time'] / 60, 2);
			$time_row['hours'] = $ts_hours;

			// append the timeslot data to the user group
			$ts_key = $time_row['start_time'] . "-" . $time_row['object_id'];
			$phase_groups[$phase_key]['labor'][$task_key]['users'][$user_key]['timeslots'][$ts_key] = $time_row;

			// build totals for user
			$phase_groups[$phase_key]['labor'][$task_key]['users'][$user_key]['totals']['hours'] += $ts_hours;
			$phase_groups[$phase_key]['labor'][$task_key]['users'][$user_key]['totals']['cost'] += $time_row['fixed_cost'];
			$phase_groups[$phase_key]['labor'][$task_key]['users'][$user_key]['totals']['billing'] += $time_row['fixed_billing'];
		}
	}

	// sort the groups by their keys and build total rows for each subgroup
	foreach ($phase_groups as &$phase_group) {

		foreach ($phase_group['labor'] as &$task_groups) {

			foreach ($task_groups['users'] as &$user_groups) {
				// sort timeslots by date
				ksort($user_groups['timeslots']);

				// build totals for task
				$task_groups['totals']['hours'] += $user_groups['totals']['hours'];
				$task_groups['totals']['cost'] += $user_groups['totals']['cost'];
				$task_groups['totals']['billing'] += $user_groups['totals']['billing'];
			}
			// sort user groups by user name
			ksort($task_groups['users']);

			// build totals for phase
			$phase_group['totals']['hours'] += $task_groups['totals']['hours'];
			$phase_group['totals']['cost'] += $task_groups['totals']['cost'];
			$phase_group['totals']['billing'] += $task_groups['totals']['billing'];

			// build labor totals for phase
			$phase_group['labor_totals']['hours'] += $task_groups['totals']['hours'];
			$phase_group['labor_totals']['cost'] += $task_groups['totals']['cost'];
			$phase_group['labor_totals']['billing'] += $task_groups['totals']['billing'];
		}
		// sort task groups by task name
		ksort($phase_group['labor']);
	}
	// sort phases by phase member name
	ksort($phase_groups);

	return array(
		'phases' => $phase_groups,
	);
}

/**
 * @return array The expense rows grouped by job phase and expense category
 */
function get_project_detail_jdt_report_expenses_data($project_member_id, $report_result = array()) {
	
	$phase_groups = array_var($report_result, 'phases', array());
	
	$job_phases_dim = Dimensions::findByCode('job_phases');
	$exp_categories_dim = Dimensions::findByCode('expense_categories');
	$default_currency = Currencies::getDefaultCurrencyInfo();
	$default_currency_id = array_var($default_currency, 'id');

	$phase_col_sql = "coalesce((select p.id from ".TABLE_PREFIX."members p inner join ".TABLE_PREFIX."object_members om on om.member_id=p.id 
		where om.object_id=pr.object_id and p.dimension_id = ".$job_phases_dim->getId()." limit 1), 0) as phase_id";
	$exp_cat_col_sql = "coalesce((select p.id from ".TABLE_PREFIX."members p inner join ".TABLE_PREFIX."object_members om on om.member_id=p.id 
		where om.object_id=pr.object_id and p.dimension_id = ".$exp_categories_dim->getId()." limit 1), 0) as exp_cat_id";

	$expense_sql = "
		select 
			pr.object_id,
			pr.date,
			pr.description,
			pr.paid_by_id, c.name as contact_name,
			pr.currency_id, pr.amount as total_cost, pr.total_price,
			$phase_col_sql,
			$exp_cat_col_sql
			
		from ".TABLE_PREFIX."payment_receipts pr
		inner join ".TABLE_PREFIX."objects o on o.id = pr.object_id
		inner join ".TABLE_PREFIX."object_members om on om.object_id = pr.object_id
		inner join ".TABLE_PREFIX."members m on m.id = om.member_id
		left join ".TABLE_PREFIX."objects c on c.id = pr.paid_by_id
		
		where m.id = $project_member_id
		and o.trashed_on = 0
	";

	$expense_rows = DB::executeAll($expense_sql);

	// group the plain expense rows by phase and expense category
	$phase_name_cache = array();
	$exp_cat_name_cache = array();

	if ($expense_rows) {
		foreach ($expense_rows as $expense_row) {

			// get the full name of the phase
			$expense_row['phase_name'] = '';
			if ($expense_row['phase_id'] > 0) {
				$expense_row['phase_name'] = array_var($phase_name_cache, $expense_row['phase_id']);
				if (!$expense_row['phase_name']) {
					$phase_member = Members::getMemberById($expense_row['phase_id']);
					if ($phase_member) {
						$expense_row['phase_name'] = $phase_member->getName();
						$phase_name_cache[$expense_row['phase_id']] = $expense_row['phase_name'];
					}
				}
			}
			// get the full name of the expense category
			$expense_row['exp_cat_name'] = '';
			if ($expense_row['exp_cat_id'] > 0) {
				$expense_row['exp_cat_name'] = array_var($exp_cat_name_cache, $expense_row['exp_cat_id']);
				if (!$expense_row['exp_cat_name']) {
					$cat_member = Members::getMemberById($expense_row['exp_cat_id']);
					if ($cat_member) {
						$expense_row['exp_cat_name'] = $cat_member->getName();
						$exp_cat_name_cache[$expense_row['exp_cat_id']] = $expense_row['exp_cat_name'];
					}
				}
			}

			// phase group
			$phase_key = $expense_row['phase_id'] > 0 ? $expense_row['phase_name'] : 'ZZZZZ';
			if (!isset($phase_groups[$phase_key])) {
				$phase_groups[$phase_key] = array(
					'group_name' => $expense_row['phase_id'] > 0 ? $expense_row['phase_name'] : 'Without phase',
					'labor' => array(),
					'expenses' => array(),
					'totals' => array('hours' => 0, 'cost' => 0, 'billing' => 0, 'currency_id' => array_var($expense_row, 'currency_id', $default_currency_id)),
					'labor_totals' =>   array('hours' => 0, 'cost' => 0, 'billing' => 0, 'currency_id' => array_var($expense_row, 'currency_id', $default_currency_id)), 
					'expense_totals' => array('hours' => 0, 'cost' => 0, 'billing' => 0, 'currency_id' => array_var($expense_row, 'currency_id', $default_currency_id)),
					// end
				);
			}

			// expense category group inside the phase group
			$exp_cat_key = $expense_row['exp_cat_id'] > 0 ? $expense_row['exp_cat_name'] : 'ZZZZZ';
			if (!isset($phase_groups[$phase_key]['expenses'][$exp_cat_key])) {
				$phase_groups[$phase_key]['expenses'][$exp_cat_key] = array(
					'actual_expenses' => array(),
					'group_name' => $expense_row['exp_cat_id'] > 0 ? $expense_row['exp_cat_name'] : 'Without expense category',
					'totals' => array('hours' => 0, 'cost' => 0, 'billing' => 0, 'currency_id' => $expense_row['currency_id']),
				);
			}

			// append the timeslot data to the user group
			$exp_key = $expense_row['date'] . "-" . $expense_row['object_id'];
			$phase_groups[$phase_key]['expenses'][$exp_cat_key]['actual_expenses'][$exp_key] = $expense_row;

			// build totals for expense category
			$phase_groups[$phase_key]['expenses'][$exp_cat_key]['totals']['cost'] += $expense_row['total_cost'];
			$phase_groups[$phase_key]['expenses'][$exp_cat_key]['totals']['billing'] += $expense_row['total_price'];
		}
	}
	// sort the groups by their keys and build total rows for each subgroup
	foreach ($phase_groups as &$phase_group) {

		foreach ($phase_group['expenses'] as &$exp_cat_group) {

			// sort user groups by user name
			ksort($exp_cat_group['actual_expenses']);

			// build totals for phase
			$phase_group['totals']['cost'] += $exp_cat_group['totals']['cost'];
			$phase_group['totals']['billing'] += $exp_cat_group['totals']['billing'];

			// build expense totals for phase
			$phase_group['expense_totals']['cost'] += $exp_cat_group['totals']['cost'];
			$phase_group['expense_totals']['billing'] += $exp_cat_group['totals']['billing'];
		}
		// sort task groups by task name
		ksort($phase_group['expenses']);
	}
	// sort phases by phase member name
	ksort($phase_groups);

	return array(
		'phases' => $phase_groups,
	);
}
