<?php

function build_project_earnings_report_excel($data) {
	$excel = array();

	// columns and report data
	$columns = array_var($data, 'columns');
	$project_earnings_report_data = array_var($data, 'report_data');

	// render data for each project
	foreach ($project_earnings_report_data['contacts'] as $pm_data) {
		$excel = array_merge($excel, build_project_earnings_pm_data_excel($pm_data, $columns));
	}
	
	// render totals table
	$excel[] = build_project_earnings_report_grand_totals_excel($project_earnings_report_data, $columns);

	return $excel;

}

function build_project_earnings_report_grand_totals_excel($data, $columns){
	$excel_table = array(
		'main_header' => array(),
		'headers' => array(), 
		'data' => array(),
		'totals' => array(),
		'totals_after_data' => true,
	);

	// main header = project name
	$excel_table['main_header'][] = lang('total');
	$i=1;
	while ($i < count($columns)) {
		$excel_table['main_header'][] = '';
		$i++;
	}

	// table headers
    foreach($columns as $c_name){
        $excel_table['headers'][] = $c_name == 'name' ? '' : lang($c_name);
    }

	$excel_table['totals'] = build_project_earnings_report_totals_excel($data, $columns, 'totals');

	return $excel_table;
}


function build_project_earnings_pm_data_excel($pm_data, $columns) {

	$excel_tables = array();

	$excel_table = array(
		'main_header' => array(),
		'headers' => array(), 
		'data' => array(),
		'totals' => array(),
		'totals_after_data' => true,
	);

	// main header = project name
	$excel_table['main_header'][] = $pm_data['name'];
	$i=1;
	while ($i < count($columns)) {
		$excel_table['main_header'][] = '';
		$i++;
	}

	// table headers
    foreach($columns as $c_name){
        $excel_table['headers'][] = $c_name == 'name' ? lang('project') : lang($c_name);
    }

	$excel_tables[] = $excel_table;

	
	// projects table
	if (count($pm_data['projects']) > 0) {
		$excel_tables[] = build_project_earnings_pm_projects_excel($pm_data, $columns);
	}

    // PM Totals
	$excel_tables[] = array('totals' => build_project_earnings_report_totals_excel($pm_data, $columns, 'totals'));

    return $excel_tables;
}

function build_project_earnings_pm_projects_excel($pm_data, $columns) {
	
	$data_rows = array();

	$excel_table = array(
		'main_header' => array(),
		'headers' => array(), 
		'data' => array(),
		'totals' => array(),
		'totals_after_data' => true,
	);

	foreach ($pm_data['projects'] as $project_data) {

		$data_row = array();
		$currency = Currencies::instance()->getCurrency($project_data['currency_id']);
		$currency_sym = $currency instanceof Currency ? $currency->getSymbol() : config_option('currency_code');
		
		foreach ($columns as $col) {
			$colspan = 1;
			$cell_class = '';
			$cell_value = '';
			switch ($col) {
				case 'name':
					$cell_class = 'name';
					$colspan = 2;
					$cell_value = $project_data['name'];
					break;
				case 'project_progress':
				case 'revenue_vs_budget':
				case 'remaining_budgeted_vs_contract':
					$cell_class = 'date';
					$cell_value = $project_data[$col]." %";
					break;
				case 'effective_multiplier':
					$cell_class = 'amount';
					$cell_value = format_amount($project_data[$col]);
					break;
				default: 
					$cell_class = 'amount';
					$cell_value = format_money_amount($project_data[$col], $currency_sym);
					break;
			}
	
			$data_row[] = $cell_value;
		}

		$data_rows[] =  $data_row;
		

	}

	$excel_table['data'] = $data_rows;

	return $excel_table;

}

function build_project_earnings_excel_title_row($group_name, $columns) {

	$row = array($group_name);
	$i=1;
	while ($i < count($columns)) {
		$row[] = '';
		$i++;
	}
	return $row;
}


function build_project_earnings_report_totals_excel($group_data, $columns, $totals_key='totals', $label=null, $prefix='') {
	$totals = $group_data[$totals_key];

	if (!$label) $label = $group_data['group_name'];

	$currency = Currencies::instance()->getCurrency($totals['currency_id']);
	$currency_sym = $currency instanceof Currency ? $currency->getSymbol() : config_option('currency_code');

	$total_row = array();
	foreach ($columns as $col) {
		$cell_value = '';
		switch ($col) {
			case 'name':
				$cell_class = 'item-name';
				$colspan = 2;
				$cell_value = lang('total') . " " . $group_data['name'];
				break;
			case 'project_progress':
			case 'revenue_vs_budget':
			case 'remaining_budgeted_vs_contract':
			case 'effective_multiplier':
				break;
			default: 
				$cell_class = 'amount';
				$cell_value = format_money_amount($totals[$col], $currency_sym);
				break;
		}

		$total_row[] = $cell_value;
	}

	return $total_row;
}
