<?php


function get_project_earnings_report_data($report_data) {

	/**
	 * Project manager X
	 *   Project A1
	 *   Project B1
	 *   ...
	 * Total project manager X
	 * Project manager Y
	 *   Project A2
	 *   Project B2
	 *   ...
	 * Total project manager Y
	 */

	$project_ot = ObjectTypes::findByName('project');
    // Get dim or cp ids for project grouping
	$group_member_ids = get_member_ids_for_project_grouping($report_data);

    $result = array(
        'contacts' => array(),
    );

	$grand_total = array();
	$multipliers = array(); 
	$project_member_ids_seen = array();
    foreach($group_member_ids as $member_id){
        $pm_project_earnings = get_project_manager_project_earnings($member_id, $report_data, $project_member_ids_seen);
        if(!empty($pm_project_earnings['projects'])){
			$result['contacts'][$member_id] = $pm_project_earnings;
			$grand_total = sum_project_earnings_totals($pm_project_earnings['totals'], $grand_total, $multipliers);
        }
    }

	// Get unclassified projects
	$dim_name = array_var($report_data, 'group_by');
	$selected_mem_id = json_decode($report_data[$dim_name.'_id']);
	if(empty($selected_mem_id)) {
		$unseen_project_earnings = get_project_manager_project_earnings(0, $report_data, $project_member_ids_seen, true);
		if(!empty($unseen_project_earnings['projects'])){
			$result['contacts']['unseen'] = $unseen_project_earnings;
			$grand_total = sum_project_earnings_totals($unseen_project_earnings['totals'], $grand_total, $multipliers);
		}
	}

	$grand_total['effective_multiplier'] = count($multipliers) ? array_sum($multipliers)/count($multipliers) : 0; // average effective multiplier

	$result['totals'] = $grand_total;

	$columns = array(
		'name', 'total_budget', 'revenue', 'revenue_vs_budget', 'project_progress', 'total_billed' ,'ar_amount',
		'unbilled','labor_cost','expense_cost','total_spent',
		'added_value','profit');
    
	// Add super profit loss column if exists
	$super_profit_cp = CustomProperties::instance()->getCustomPropertyByCode($project_ot->getId(), 'super_profit_loss');
	if ($super_profit_cp) {
		$columns[] = 'super_profit_loss';
	}
	
	$columns_part_2 = array('potential_provision',
		'effective_multiplier',
		'remaining_budgeted_vs_contract'
	);

	$columns = array_merge($columns, $columns_part_2);

	$data = array(
		'columns' => $columns,
		'report_data' => $result,
	); 

	return $data;
}


function get_project_manager_project_earnings($member_or_cp_id, $report_data, &$project_member_ids_seen, $get_unseen_projects = false) {
	$group_by = array_var($report_data, 'group_by');
	if($get_unseen_projects) {
		$all_project_member_ids = get_project_member_ids_for_project_manager_report(false);
		$member_project_ids = array_diff($all_project_member_ids, $project_member_ids_seen);
	} else {
		$member_project_ids = find_projects_for_earnings_reports($member_or_cp_id, $report_data);
	}
	
	if(empty($member_project_ids)) return array();
	if(!$get_unseen_projects) {
		$project_member_ids_seen = array_merge($project_member_ids_seen, $member_project_ids);
	}
	

	$manager_totals = array(
		'revenue' => 0,
		'total_billed' => 0,
		'ar_amount' => 0,
		'unbilled' => 0,
		'labor_cost' => 0,
		'expense_cost' => 0,
		'total_spent' => 0,
		'added_value' => 0,
		'profit' => 0,
		'super_profit_loss' => 0,
		'potential_provision' => 0,
		'effective_multiplier' => 0,
		'net_revenue' => 0,
		'total_budget' => 0
	);

	$all_project_earnigns = array();
	$multipliers = array();
	foreach($member_project_ids as $member_id){
		$project_earnings = get_project_earnings($member_id, $report_data);
		if ($project_earnings) {
			$all_project_earnigns[] = $project_earnings;
			$manager_totals = sum_project_earnings_totals($project_earnings, $manager_totals, $multipliers);
		}
	}

	$manager_totals['effective_multiplier'] = count($multipliers) ? array_sum($multipliers)/count($multipliers) : 0; // average effective multiplier


	if(strpos($group_by, '_dim')) {
		$mem = Members::instance()->findById($member_or_cp_id);
		$name = $mem instanceof Member ? $mem->getName() : lang('unclassified');
	}

	return array(
		'name' => $name,
		'projects' => $all_project_earnigns,
		'totals' => $manager_totals,
	);

}

function sum_project_earnings_totals($data, $totals, &$multipliers) {

	foreach ($data as $k => $v) {
		if (in_array($k, array('name', 'percent_complete', 'project_progress'))) continue;
		if ($k == 'effective_multiplier'){
			$multipliers[] = $v;
		} else {
			if( isset($totals[$k])) 
				$totals[$k] += $v;
			else
				$totals[$k] = $v;
		}
	}

	return $totals;
}

function get_project_earnings($member_id, $report_data) { 
	$allow_without_budget_total = array_var($report_data, 'include_unset_total_budget') == 'checked' ? true : false;
	$member = Members::getMemberById($member_id);
	if (!$member) return;

	$contract_phase_dim = Dimensions::findByCode('project_phases');
	$contract_phase_mem_id = get_associated_status_member_id($member, $contract_phase_dim);
	$is_active_project = false;
	$contract_phase_mem = Members::getMemberById($contract_phase_mem_id);

	// Check budget total and config option => exclude the project if necessary
	$contract_amount = 0;
	$contract_amount_cp = CustomProperties::instance()->getCustomPropertyByCode($member->getObjectTypeId(), 'budget_total');
	if ($contract_amount_cp) {
		$cp_val = CustomPropertyValues::instance()->getCustomPropertyValue($member->getObjectId(), $contract_amount_cp->getId());
		if ($cp_val) $contract_amount = $cp_val->getValue();
	}
	if ($contract_amount <= 0 && !$allow_without_budget_total) {
		return false;
	}

	$tmp_mem_arr = array($member);
	// get full name
	build_member_list_text_to_show_in_trees($tmp_mem_arr);

	$data = array(
		'name' => $member->getName(),
		'revenue' => 0, // total earned value
		'revenue_vs_budget' => 0, // Revenue/ContractAmount
		'percent_complete' => 0, // percent complete
		'total_billed' => 0, //Total amount invoiced (excl. VAT)
		'ar_amount' => 0, // pending invoices including vat
		'unbilled' => 0, // time and expenses not billed yet
		'labor_cost' => 0,
		'expense_cost' => 0,
		'total_spent' => 0, // labor cost + expense cost ***
		'added_value' => 0, // revenue - expense cost ***
		'profit' => 0, // revenue - total spent ***
		'super_profit_loss' => 0, // cp from project ***
		'potential_provision' => 0, // total billed - revenue ***
		'effective_multiplier' => 0, // net revenue / labor cost
		'net_revenue' => 0, // revenue - expense cost
		'total_budget' => 0,
		'remaining_budgeted_vs_contract' => 0,
	);

	// Get project's revenue, labor cost and expense cost from member_totals table
	get_project_revenue_labor_and_expense_cost($member_id, $data, $contract_amount);
	$revenue = $data['revenue'];

	// percent complete
	if ($contract_amount > 0) {
		$data['revenue_vs_budget'] = number_format(100 * $revenue / $contract_amount, 2);
	}

	// unbilled amount
	$data['unbilled'] = $data['revenue'] - $data['total_billed'];

	// Total budget
	$data['total_budget'] = $contract_amount;

	// Calculate profit, potential provision and net revenue
	$data['profit'] = $data['revenue'] - $data['total_spent'];
	$data['potential_provision'] = $data['total_billed'] - $data['revenue'];
	$data['net_revenue'] = $data['revenue'] - $data['expense_cost'];

	// Get super profit loss
	$super_profit_cp = CustomProperties::instance()->getCustomPropertyByCode($member->getObjectTypeId(), 'super_profit_loss');
	if ($super_profit_cp) {
		$cp_val = CustomPropertyValues::instance()->getCustomPropertyValue($member->getObjectId(), $super_profit_cp->getId());
		if ($cp_val) $data['super_profit_loss'] = $cp_val->getValue();
	}

	return $data;
}

function get_project_earnings_unbilled_amount($member) {

	$amount = 0;

	if (Plugins::instance()->isActivePlugin('income')) {
		$row = DB::executeOne("
			SELECT SUM(ts.fixed_billing) as total
			FROM ".TABLE_PREFIX."timeslots ts
			INNER JOIN ".TABLE_PREFIX."objects o on o.id=ts.object_id
			INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
			WHERE o.trashed_by_id=0
			AND ts.invoicing_status = 'pending'
			AND om.member_id=".$member->getId().";
		");
		if ($row) $amount += $row['total'];

		if (Plugins::instance()->isActivePlugin('expenses2')) {

			$exp_row = DB::executeOne("
				SELECT SUM(pr.total_price) as total
				FROM ".TABLE_PREFIX."payment_receipts pr
				INNER JOIN ".TABLE_PREFIX."objects o on o.id=pr.object_id
				INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
				WHERE o.trashed_by_id=0
				AND pr.invoicing_status = 'pending'
				AND om.member_id=".$member->getId().";
			");
			if ($exp_row) {
				$amount += $exp_row['total'];
			}
		}
	}
	return $amount;
}


function get_project_earnings_ar_amount($member) {

	$amount = 0;

	$now_str = DateTimeValueLib::now()->format(DATE_MYSQL);

	if (Plugins::instance()->isActivePlugin('income')) {
		$row = DB::executeOne("
			SELECT SUM(i.total) as total
			FROM ".TABLE_PREFIX."income_invoices i
			INNER JOIN ".TABLE_PREFIX."objects o on o.id=i.object_id
			INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
			WHERE o.trashed_by_id=0
			AND i.status != 'canceled' AND i.status != 'confirmed'
			AND i.expiration_date < '".$now_str."'
			AND om.member_id=".$member->getId().";
		");
		if ($row) $amount += $row['total'];
	}

	return $amount;
}

function get_project_earnings_total_billed($member) {

	$amount = 0;

	if (Plugins::instance()->isActivePlugin('income')) {
		$row = DB::executeOne("
			SELECT SUM(i.subtotal) as subtotal
			FROM ".TABLE_PREFIX."income_invoices i
			INNER JOIN ".TABLE_PREFIX."objects o on o.id=i.object_id
			INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
			WHERE o.trashed_by_id=0
			AND i.status != 'canceled'
			AND om.member_id=".$member->getId().";
		");
		if ($row) $amount += $row['subtotal'];
	}

	return $amount;
}

function find_projects_for_earnings_reports($member_or_cp_id, $report_data) {

	$group_by = array_var($report_data, 'group_by');
	$project_member_ids = array();
	if(strpos($group_by, '_dim')) {
		$report_data['use_other_associated_member_ids'] = true;
		$report_data['other_associated_member_ids'] = array($member_or_cp_id);
		$project_member_ids = get_project_member_ids_for_project_manager_report(false, $report_data);
	}

	return $project_member_ids;
	
}


// Get project's revenue, labor cost and expense cost from member_totals and store in $data
function get_project_revenue_labor_and_expense_cost($member_id, &$data, $contract_amount) {
	// Get member totals
	$member_totals = MemberTotals::instance()->findOne(array('conditions' => array('`member_id`="'.$member_id.'"')));
   
	// Get project's revenue, labor cost and expense cost
    if($member_totals instanceof MemberTotal) {
        $data['revenue'] = $member_totals->getColumnValue('executed_overall_total_price');
		$data['labor_cost'] = $member_totals->getColumnValue('executed_labor_total_cost');
		$data['expense_cost'] = $member_totals->getColumnValue('executed_expense_total_cost');
		$data['total_billed'] = $member_totals->getColumnValue('invoiced_subtotal');
		$data['ar_amount'] = $member_totals->getColumnValue('invoiced_due_amount');
		$data['total_spent'] = $member_totals->getColumnValue('executed_overall_total_cost');
		$data['added_value'] = $member_totals->getColumnValue('executed_labor_total_price');
		$data['project_progress'] = $member_totals->getColumnValue('percent_completed');
		$data['effective_multiplier'] = $data['labor_cost'] == 0 ? 0 : $data['added_value'] / $data['labor_cost'];
		$remaining_budgeted = $member_totals->getColumnValue('estimated_overall_total_price') - $member_totals->getColumnValue('executed_overall_total_price');

		if($remaining_budgeted > 0 && $contract_amount > 0) {
			$data['remaining_budgeted_vs_contract'] = number_format(100 * $remaining_budgeted / $contract_amount, 2);
		}
    }
}

function get_member_ids_for_project_grouping($report_data) {
	$group_by = array_var($report_data, 'group_by');
	$member_or_cp_ids = array();
    
	if(strpos($group_by, '_dim')) {
		$member_or_cp_ids = json_decode($report_data[$group_by.'_id']);
		if(empty($member_or_cp_ids)) {
			$dim_code = substr($group_by, 0, -4);
			$dimension = Dimensions::findByCode($dim_code);
			if(!$dimension instanceof Dimension) return array();
			$dim_id = $dimension->getId();
			$members_row = DB::execute('
				SELECT group_concat(id) as `member_ids`
				FROM '.TABLE_PREFIX.'members
				WHERE dimension_id='.$dim_id.' AND depth=1
			');
			$members_string = $members_row->fetchRow();
			$member_or_cp_ids = explode(',', $members_string['member_ids']);
		}
	}

	return $member_or_cp_ids;
}