<?php

function build_project_forecast_report_excel($data, $report_data){
    $excel = array();

    $projects_data = array_var($data, 'projects_data');
    $date_periods = array_var($data, 'date_periods');
    $projects_excel = array();
    foreach ($projects_data as $project_data) {
        $projects_excel[] = build_forecast_report_project_data($project_data, $report_data, $date_periods);	
    }
    $excel = array_merge($excel, $projects_excel);
    
    $use_expense_estimate_cost = $report_data['group_expenses_by'] == 'estimated_cost';
    $use_labor_estimate_cost = $report_data['group_labor_by'] == 'estimated_cost';
    if(count($projects_data) > 1 && ($use_expense_estimate_cost || $use_labor_estimate_cost)) {
        $excel[] = build_forecast_report_total_data($data['grand_totals'], $report_data, $date_periods);
    }

    return $excel;
}



function build_forecast_report_project_data($project_data, $report_data, $date_periods){

    
    $total_cols = count($date_periods) + 2;

    $excel_table = array(
		'main_header' => array(),
		'headers' => array(), 
		'data' => array(),
		'totals' => array(),
		'totals_after_data' => true,
	);

    // main header = project name
	$excel_table['main_header'][] = $project_data['name'];
	$i=1;
	while ($i < $total_cols) {
		$excel_table['main_header'][] = '';
		$i++;
	}
   
    $excel_table['headers'] = build_forecast_header_columns($date_periods);


    if(!empty($project_data['labor_data'])){
        $excel_table['data'] = build_forecast_project_labor_table($project_data, $report_data, $date_periods);
    }
	
	if(!empty($project_data['expenses_data'])){
        $excel_table['data'] = array_merge($excel_table['data'], build_forecast_project_expenses_table($project_data, $report_data, $date_periods));
    }

    $use_labor_estimate_cost = $report_data['group_labor_by'] == 'estimated_cost';
    $use_expense_estimate_cost = $report_data['group_expenses_by'] == 'estimated_cost';
    if($use_labor_estimate_cost && $use_expense_estimate_cost){
        $excel_table['totals'] = build_forecast_report_project_total($project_data['project_totals'], $report_data, $date_periods);
    }

    return $excel_table;
}

function build_forecast_project_labor_table($project_data, $report_data, $date_periods){
    $use_expense_estimate_cost = $report_data['group_expenses_by'] == 'estimated_cost';
    $use_labor_estimate_cost = $report_data['group_labor_by'] == 'estimated_cost';
    $currency_symbol = config_option('currency_code');
    $total_cols = count($date_periods) + 2;

    $data_rows = array();
    // Render Labor
    if (!empty($project_data['labor_data'])){
        $labor_header = array(lang('estimated labor'));
        $i = 1;
        while($i < $total_cols) {
            $labor_header[] = '';
            $i++;
        }
        $data_rows[] = $labor_header;
        foreach ($project_data['labor_data'] as $category_data) {
            $labor_row = array($category_data['name']);
            
            foreach($date_periods as $period){
                $value_per_period = $category_data['periods'][$period] == 0 ? '' : $category_data['periods'][$period];
                if($use_labor_estimate_cost && $value_per_period != ''){
                    $value_per_period = format_money_amount($value_per_period, $currency_symbol);
                }
                $labor_row[] = $value_per_period;
            }
            $category_total = $use_labor_estimate_cost ? format_money_amount($category_data['category_total'], $currency_symbol) : $category_data['category_total'];
            $labor_row[] = $category_total;
            $data_rows[] = $labor_row;
        }
        // Render labor totals if estimated cost is used
        if($use_labor_estimate_cost) {
            $data_rows[] = build_forecast_project_labor_totals($project_data, $date_periods);
        }
        if(($use_expense_estimate_cost && $use_labor_estimate_cost) || !empty($project_data['expenses_data'])) {
            $data_rows[] = array('');
        }
    }
    return $data_rows;
}

function build_forecast_project_labor_totals($project_data, $date_periods){
    $currency_symbol = config_option('currency_code');
    $style = "font-weight: 600;";
    $labor_totals = $project_data['labor_totals'];
    $labor_totals_row = array(lang('labor totals'));
    
    foreach($date_periods as $period){
        $value_per_period = format_money_amount($labor_totals['periods'][$period], $currency_symbol);
        $labor_totals_row[] = $value_per_period;
    }
    $labor_grand_total = format_money_amount($labor_totals['labor_total'], $currency_symbol);
    $labor_totals_row[] = $labor_grand_total;
    
    return $labor_totals_row;
}



function build_forecast_project_expenses_table($project_data, $report_data, $date_periods){
    $use_expense_estimate_cost = $report_data['group_expenses_by'] == 'estimated_cost';
    $use_labor_estimate_cost = $report_data['group_labor_by'] == 'estimated_cost';
    $currency_symbol = config_option('currency_code');
    $total_cols = count($date_periods) + 2;

    $data_rows = array();
    // Render expenses info
    if (!empty($project_data['expenses_data'])){
        $expense_header = array(lang('expenses'));
        $i = 1;
        while($i < $total_cols) {
            $expense_header[] = '';
            $i++;
        }
        $data_rows[] = $expense_header;

        foreach ($project_data['expenses_data'] as $product_type_data) {
            $expense_row = array($product_type_data['name']);
            foreach($date_periods as $period){
                $value_per_period = $product_type_data['periods'][$period] == 0 ? '' : $product_type_data['periods'][$period];
                if($use_expense_estimate_cost && $value_per_period != ''){
                    $value_per_period = format_money_amount($value_per_period, $currency_symbol);
                }
                $expense_row[] = $value_per_period;
            }
            $product_type_total = $use_expense_estimate_cost ? format_money_amount($product_type_data['product_type_total'], $currency_symbol) : $product_type_data['product_type_total'];
            $expense_row[] = $product_type_total;
            $data_rows[] = $expense_row;
        }
        // Render expenses totals if estimated cost is used
        if($use_expense_estimate_cost) {
            $data_rows[] = build_forecast_project_expenses_totals($project_data, $date_periods);
        }
        if($use_expense_estimate_cost && $use_labor_estimate_cost) {
            $data_rows[] = array('');
        }

        return $data_rows;
    }
}

function build_forecast_project_expenses_totals($project_data, $date_periods){
    $currency_symbol = config_option('currency_code');
    $style = "font-weight: 600;";
    $expense_totals = $project_data['expense_totals'];
    $expense_totals_row = array(lang('expense totals'));
    foreach($date_periods as $period){
        $value_per_period = format_money_amount($expense_totals['periods'][$period], $currency_symbol);
        $expense_totals_row[] = $value_per_period;
    }
    $expense_grand_total = format_money_amount($expense_totals['expenses_total'], $currency_symbol);
    $expense_totals_row[] = $expense_grand_total;
    return $expense_totals_row;
}

function build_forecast_report_project_total($totals, $report_data, $date_periods){
    $currency_symbol = config_option('currency_code');
    $style = "font-weight: 600;";
    $total_row = array(lang('totals'));
    foreach($date_periods as $period){
        $value_per_period = format_money_amount($totals['periods'][$period], $currency_symbol);
        $total_row[] = $value_per_period;
    }
    $project_grand_total = format_money_amount($totals['project_total'], $currency_symbol);
    $total_row[] = $project_grand_total;

    return $total_row;
}

function build_forecast_report_total_data($totals, $report_data, $date_periods){
    $use_labor_estimate_cost = $report_data['group_labor_by'] == 'estimated_cost';
    $use_expense_estimate_cost = $report_data['group_expenses_by'] == 'estimated_cost';
    $currency_symbol = config_option('currency_code');
    $total_cols = count($date_periods) + 2;

    $excel_table = array(
		'main_header' => array(),
		'headers' => array(), 
		'data' => array(),
		'totals' => array(),
		'totals_after_data' => true,
	);

    // main header = project name
	$excel_table['main_header'][] = lang('grand totals');
	$i=1;
	while ($i < $total_cols) {
		$excel_table['main_header'][] = '';
		$i++;
	}
   
    $excel_table['headers'] = build_forecast_header_columns($date_periods);


    // Render labor row
    if($use_labor_estimate_cost){
        $labor_row = array(lang('labor totals'));
        foreach($date_periods as $period){
            $value_per_period = format_money_amount($totals['labor_totals']['periods'][$period], $currency_symbol);
            $labor_row[] = $value_per_period;
        }
        $grand_labor_total = format_money_amount($totals['labor_totals']['labor_grand_total'], $currency_symbol);
        $labor_row[] = $grand_labor_total;
        $excel_table['data'][] = $labor_row;
    }

    // Render expenses row
    if($use_expense_estimate_cost){
        $expense_row = array(lang('expense totals'));
        foreach($date_periods as $period){
            $value_per_period = format_money_amount($totals['expenses_totals']['periods'][$period], $currency_symbol);
            $expense_row[] = $value_per_period;
        }
        $grand_expenses_total = format_money_amount($totals['expenses_totals']['expenses_grand_total'], $currency_symbol);
        $expense_row[] = $grand_expenses_total;
        $excel_table['data'][] = $expense_row;
    }

    // Render total row
    if($use_labor_estimate_cost && $use_expense_estimate_cost){
        $total_row = array(lang('grand totals'));
        foreach($date_periods as $period){
            $value_per_period = format_money_amount($totals['report_totals']['periods'][$period], $currency_symbol);
            $total_row[] = $value_per_period;
        }
        $grand_total = format_money_amount($totals['report_totals']['report_grand_total'], $currency_symbol);
        $total_row[] = $grand_total;
        $excel_table['data'][] = $total_row;
    }

    return $excel_table;
}


function build_forecast_header_columns($date_periods){
    // Get header column names
	$columns = array(' ');
    foreach($date_periods as $period){
        $period_dtv = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $period.'-5');
        $columns[] = $period_dtv->format('M') . ' ' . $period_dtv->format('Y');
    }
    $columns[] = lang('total'); 

    // Render header columns
    return $columns;
}
