<?php

function get_main_project_summary_report_data($report_data) {

    $project_ot = ObjectTypes::instance()->findOne(array('conditions' => "`name` = 'project'"));
    $project_ot_id = $project_ot->getId();
    $project_managers = get_project_managers_for_project_summary_report($report_data);

    if(count($project_managers) == 0) {
        return array();
    }

    $filtered_project_member_ids = get_summary_project_member_ids_filtered_by_date($report_data);

    $result_data = array(
        'project_managers' => array(),
        'totals' => array()
    );

    get_columns_for_project_summary_report($report_data, $result_data);

    foreach($project_managers as $pm_id) {
        $project_manager = Members::instance()->findOne(array('conditions' => '`id`='.$pm_id));
        if(!$project_manager instanceof Member) {
            continue;
        }
        $pm_id = $project_manager->getId();
        $pm_name = $project_manager->getDisplayName();
        $projects = get_projects_for_pm_for_project_summary_report($report_data, $pm_id, $filtered_project_member_ids);

        // if there are no projects for this project manager, skip it
        if(count($projects) == 0) {
            continue;
        }

        $result_data['project_managers'][$pm_id] = array(
            'project_manager_name' => $pm_name,
            'projects' => array(),
            'totals' => array()
        );

        foreach($projects as $project_id) {
            $project_data = collect_project_info_for_project_summary_report($report_data, $project_id);
            if(empty($project_data)) {
                continue;
            }
            $result_data['project_managers'][$pm_id]['projects'][$project_id] = $project_data;
        }
        // if there are no projects for this project manager, remove it
        if(count($result_data['project_managers'][$pm_id]['projects']) == 0) {
            unset($result_data['project_managers'][$pm_id]);
        } else {
            // Calculate totals
            $result_data['project_managers'][$pm_id]['totals'] = calculate_pm_totals_for_project_summary_report($result_data['project_managers'][$pm_id]['projects']);
        }
    }

    // Calculate overall totals
    $pm_totals = array_column($result_data['project_managers'], 'totals');
    $result_data['totals'] = calculate_pm_totals_for_project_summary_report($pm_totals);

    return $result_data;
}


function get_columns_for_project_summary_report(&$report_data, &$result_data) {
    $columns = array('name');
    $cp_object_type_id = ObjectTypes::instance()->findOne(array('conditions' => "`name` = 'project'"))->getId();
    $budget_total_cp = CustomProperties::instance()->findOne(array('conditions' => '`object_type_id`='.$cp_object_type_id. ' AND `code`="budget_total"'));
    if ($budget_total_cp instanceof CustomProperty) {
        $columns[] = 'budget_total';
        $report_data['budget_total_cp_id'] = $budget_total_cp->getId();
        $result_data['budget_total_cp_id'] = $budget_total_cp->getId();
        $result_data['budget_total_cp_name'] = $budget_total_cp->getName();
    }
    $revised_budget_total_cp = CustomProperties::instance()->findOne(array('conditions' => '`object_type_id`='.$cp_object_type_id. ' AND `code`="revised_budget_total"'));
    if ($revised_budget_total_cp instanceof CustomProperty) {
        $columns[] = 'revised_budget_total';
        $report_data['revised_budget_total_cp_id'] = $revised_budget_total_cp->getId();
        $result_data['revised_budget_total_cp_id'] = $revised_budget_total_cp->getId();
        $result_data['revised_budget_total_cp_name'] = $revised_budget_total_cp->getName();
    }
    $columns[] = 'executed_cost';
    if(Plugins::instance()->isActivePlugin('income')) {
        $columns[] = 'invoiced_subtotal';
    }
    if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
        $columns[] = 'invoiced_paid';
    }

    $report_data['columns'] = $columns;
    $result_data['columns'] = $columns;
}

function calculate_pm_totals_for_project_summary_report($pm_projects) {
    $totals = array();
    foreach($pm_projects as $project) {
        foreach($project as $column => $value) {
            if($column == 'name') {
                continue;
            }
            if(!isset($totals[$column])) {
                $totals[$column] = 0;
            }
            $totals[$column] += $value;
        }
    }
    return $totals;
}


function get_project_managers_for_project_summary_report($report_data) {
    $project_manager_dimension = Dimensions::instance()->findByCode('project_managers'); 
    $pm_dim_id = $project_manager_dimension instanceof Dimension ? $project_manager_dimension->getId() : 0;
    $enabled_dimensions = config_option('enabled_dimensions');

    $proj_proj_manager_association = DimensionMemberAssociations::instance()->getAssociationByCode('project_project_manager');

    $project_managers = array();

    if ($project_manager_dimension instanceof Dimension && $proj_proj_manager_association instanceof DimensionMemberAssociation && in_array($pm_dim_id, $enabled_dimensions)) {
        $pm_cp_values = Members::instance()->findAll((array('conditions' => '`dimension_id`='.$project_manager_dimension->getId())));
        $project_managers = array_map('get_member_id', $pm_cp_values);
    }

    return $project_managers;

}

function get_projects_for_pm_for_project_summary_report($report_data, $pm_id, $filtered_project_member_ids) {
    if($pm_id == 0) {
        return array();
    }

    $project_ids = array();

    // Get project and project manager association id
    $proj_proj_manager_association = DimensionMemberAssociations::instance()->getAssociationByCode('project_project_manager');

    // Get project ids for this project manager
    if($proj_proj_manager_association instanceof DimensionMemberAssociation) {
        $mpms = MemberPropertyMembers::instance()->findAll(array('conditions' => '`association_id`='.$proj_proj_manager_association->getId().' AND `property_member_id`='.$pm_id));
        $project_ids = array_map('get_project_member_ids', $mpms);
    }

    // Filter out projects that don't have any timeslots or expenses in the given date range
    $project_ids = array_intersect($project_ids, $filtered_project_member_ids);
   
    return $project_ids;
}

function get_summary_project_member_ids_filtered_by_date($report_data){
	$project_ot = ObjectTypes::findByName('project');

	// Get start and end date SQLs
	$start_date = array_var($report_data, 'start_date');
	$end_date = array_var($report_data, 'end_date');
	$sql_st_date = $start_date->toMySQL();
	$sql_end_date = $end_date->toMySQL();

	// Get project member ids assigned to timeslots in the given date range
	$time_sql = "SELECT DISTINCT(m.id)
			FROM ".TABLE_PREFIX."timeslots t INNER JOIN ".TABLE_PREFIX."object_members om ON t.object_id=om.object_id
            INNER JOIN ".TABLE_PREFIX."objects o ON t.object_id=o.id
			INNER JOIN ".TABLE_PREFIX."members m ON om.member_id=m.id
			WHERE o.trashed_by_id=0
            AND t.start_time BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
			AND m.object_type_id=".$project_ot->getId().";";
	$time_rows = DB::executeAll($time_sql);
	$time_project_member_ids = !empty($time_rows) ? array_column($time_rows, 'id') : array();


    // Rewrite start and end date SQLs to exclude timezone offset (the bug in our current system)
    $start_date = $start_date->add('s', logged_user()->getUserTimezoneValue());
    $sql_st_date = $start_date->toMySQL();
    $end_date = $end_date->add('s', logged_user()->getUserTimezoneValue());
    $sql_end_date = $end_date->toMySQL();

	// Get project member ids assigned to actual expenses in the given date range									
	$expense_sql = "SELECT DISTINCT(m.id)
				FROM ".TABLE_PREFIX."payment_receipts pr INNER JOIN ".TABLE_PREFIX."object_members om ON pr.object_id=om.object_id
                INNER JOIN ".TABLE_PREFIX."objects o ON pr.object_id=o.id
				INNER JOIN ".TABLE_PREFIX."members m ON om.member_id=m.id
				WHERE o.trashed_by_id=0
                AND pr.date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
				AND m.object_type_id=".$project_ot->getId().";";
	$expense_rows = DB::executeAll($expense_sql);
    $expense_project_member_ids = !empty($expense_rows) ? array_column($expense_rows, 'id') : array();

	// Merge time and expense project member ids										
	$combined_project_member_ids = array_unique(array_merge($time_project_member_ids, $expense_project_member_ids));

    // Return start and end date SQLs to include timezone offset (the bug in our current system)
    $start_date = $start_date->add('s', -1 *logged_user()->getUserTimezoneValue());
    $end_date = $end_date->add('s', -1 * logged_user()->getUserTimezoneValue());

	return $combined_project_member_ids;
}

function collect_project_info_for_project_summary_report($report_data, $project_id) {
    $project_member = Members::instance()->findOne(array('conditions' => '`id`='.$project_id));
    if(!$project_member instanceof Member || $project_member->getDepth() > 1) {
        return array(); 
    }
    $columns = $report_data['columns'];
    $project_data = array(
        'name' => $project_member->getDisplayName()
    );

    if(in_array('budget_total', $columns)) {
        $budget_total_cp_id = $report_data['budget_total_cp_id'];
        $cpv = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $budget_total_cp_id);
        if ($cpv instanceof CustomPropertyValue) {
            $project_data['budget_total'] = $cpv->getValue();
        }
    }
    if(in_array('revised_budget_total', $columns)) {
        $revised_budget_total_cp_id = $report_data['revised_budget_total_cp_id'];
        $cpv = CustomPropertyValues::getCustomPropertyValue($project_member->getObjectId(), $revised_budget_total_cp_id);
        if ($cpv instanceof CustomPropertyValue) {
            $project_data['revised_budget_total'] = $cpv->getValue();
        }
    }
    if(in_array('executed_cost', $columns)) {
        $project_data['executed_cost'] = get_executed_cost_for_project_summary_report($report_data, $project_id);
    }
    $invoice_ids = array();
    if(in_array('invoiced_subtotal', $columns)) {
        $invoices_data = get_invoiced_subtotal_for_project_summary_report($report_data, $project_id);
        $project_data['invoiced_subtotal'] = $invoices_data['invoiced_subtotal'];
        $invoice_ids = $invoices_data['invoice_ids'];
    }
    if(in_array('invoiced_paid', $columns)) {
        $project_data['invoiced_paid'] = get_invoiced_paid_for_project_summary_report($report_data, $invoice_ids);
    }

    return $project_data;
}

function get_executed_cost_for_project_summary_report($report_data, $project_id) {
    $start_date = array_var($report_data, 'start_date');
    $end_date = array_var($report_data, 'end_date');
    $sql_st_date = $start_date->toMySQL();
    $sql_end_date = $end_date->toMySQL();

    $executed_cost = 0;

    // Get timeslots for this project
    $time_row = DB::executeOne("
        SELECT SUM(ts.fixed_cost) as total_cost
        FROM ".TABLE_PREFIX."timeslots ts
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=ts.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        AND ts.start_time BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
        AND om.member_id=".$project_id.";
    ");
    if ($time_row) $executed_cost += $time_row['total_cost'];

    // Rewrite start and end date SQLs to exclude timezone offset (the bug in our current system)
    $start_date = $start_date->add('s', logged_user()->getUserTimezoneValue());
    $sql_st_date = $start_date->toMySQL();
    $end_date = $end_date->add('s', logged_user()->getUserTimezoneValue());
    $sql_end_date = $end_date->toMySQL();

    // Get expenses for this project
    $expense_row = DB::executeOne("
        SELECT SUM(pr.amount) as total_cost
        FROM ".TABLE_PREFIX."payment_receipts pr
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=pr.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        AND pr.date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
        AND om.member_id=".$project_id.";
    ");
    if ($expense_row) $executed_cost += $expense_row['total_cost'];

    // Return start and end date SQLs to include timezone offset (the bug in our current system)
    $start_date = $start_date->add('s', -1 *logged_user()->getUserTimezoneValue());
    $end_date = $end_date->add('s', -1 * logged_user()->getUserTimezoneValue());

    return $executed_cost;
}

function get_invoiced_subtotal_for_project_summary_report($report_data, $project_id) {
    $start_date = array_var($report_data, 'start_date');
    $end_date = array_var($report_data, 'end_date');
    $sql_st_date = $start_date->toMySQL();
    $sql_end_date = $end_date->toMySQL();

    $result = array(
        'invoiced_subtotal' => 0,
        'invoice_ids' => array()
    );

    // Get invoices for this project
    $invoice_row = DB::executeOne("
        SELECT GROUP_CONCAT(i.object_id) as invoice_ids, SUM(i.subtotal) as subtotal
        FROM ".TABLE_PREFIX."income_invoices i
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=i.object_id
        INNER JOIN ".TABLE_PREFIX."object_members om ON om.object_id=o.id
        WHERE o.trashed_by_id=0
        AND i.date BETWEEN '".$sql_st_date."' AND '".$sql_end_date."'
        AND om.member_id=".$project_id.";
    ");
    if ($invoice_row) {
        $result['invoiced_subtotal'] += $invoice_row['subtotal'];
        $result['invoice_ids'] = $invoice_row['invoice_ids'] == '' ? array() : explode(',', $invoice_row['invoice_ids']);
    } 

    return $result;
}

function get_invoiced_paid_for_project_summary_report($report_data, $invoice_ids) {

    if(count($invoice_ids) == 0) {
        return 0;
    }

    $invoice_ids_string = implode(',', $invoice_ids);
    $invoiced_paid = 0;

    $payment_row = DB::executeOne("
        SELECT SUM(ipri.amount) as total_paid
        FROM ".TABLE_PREFIX."invoice_payment_receipt_invoices ipri 
        INNER JOIN ".TABLE_PREFIX."objects o on o.id=ipri.payment_receipt_id
        WHERE o.trashed_by_id=0 
        AND ipri.invoice_id IN (".$invoice_ids_string.")
    ");
    if ($payment_row) $invoiced_paid += $payment_row['total_paid'];

    return $invoiced_paid;
}
