<?php


function get_utilization_report_data($report_data) {

	$labor_cat_dim = Dimensions::findByCode('hour_types');
	$labor_cat_ot = ObjectTypes::findByName('hour_type');
	$contract_hours_cp = CustomProperties::getCustomPropertyByCode(Contacts::instance()->getObjectTypeId(), 'contract_hours');
	$utilization_goal_cp = CustomProperties::getCustomPropertyByCode(Contacts::instance()->getObjectTypeId(), 'billable_utilization_goal');
	$exclude_users_without_utilization_goal = array_var($report_data, 'exclude_users_without_utilization_goal', false) == 'checked';

	$first_level_categories = Members::instance()->findAll(array("conditions" => array("dimension_id = ? AND depth = 1", $labor_cat_dim->getId())));

	$billable_labor_categories = find_billable_labor_categories_member_ids();
	$time_off_labor_categories = find_time_off_labor_categories_member_ids();
	$include_contract_hours = array_var($report_data, 'include_contract_hours') == 'checked';
	
	// Dates config
	Env::useHelper('advanced_reports', 'advanced_reports');
	$report_dates = parse_report_param_date_type($report_data, true, 'date_type', 'start_value', 'end_value', 'utilization_report');
	$start_date = array_var($report_dates, 'st', DateTimeValueLib::now());
	$end_date = array_var($report_dates, 'et', DateTimeValueLib::now());

	// Get weekday working days
	if(array_var($report_data, 'working_days_option') == 'general_option'){
		$weekday_working_days = explode(',', config_option('working_days'));
	} else {
		$weekday_working_days = array(0, 1, 2, 3, 4);
	}

	// Get working days in period to calculate contract hours per user
	$working_days_in_period = get_working_days_from_period($start_date, $end_date, $weekday_working_days);


	$start_date_str = $start_date->toMySQL();
	$end_date_str = $end_date->toMySQL();


	// query the sum of worked hours grouped by user and the first level of labor categories
	$sql = "
		select t.object_id, t.contact_id, c.name as contact_name,
		(sum(t.worked_time) / 60) as period_worked_hours,
		m.id as category_id, m.name
		from ".TABLE_PREFIX."timeslots t
		inner join ".TABLE_PREFIX."object_members om on om.object_id=t.object_id
		inner join ".TABLE_PREFIX."members m on m.id=om.member_id
		inner join ".TABLE_PREFIX."objects c on c.id=t.contact_id
		
		where
		m.depth = 1 and m.dimension_id=".$labor_cat_dim->getId()."
		and t.start_time >= '$start_date_str' and t.end_time <= '$end_date_str'
		and (SELECT trashed_by_id FROM ".TABLE_PREFIX."objects oc WHERE oc.id = t.object_id) = 0
		
		group by t.contact_id, om.member_id;
	";

	$raw_rows = DB::executeAll($sql);

	$timeslots_cache = array();
	$utilization_data = array();
	$utilization_hrs = array();
	$variation_prtg = array();

	// group the data by contact to send to the view
	if ($raw_rows) {

		// get involved timeslot objets and keep them in a cache array, so we can ask them if they are time off or not
		$all_timeslot_ids = array();
		foreach ($raw_rows as $row) {
			$all_timeslot_ids[] = $row['object_id'];
		}
		if (count($all_timeslot_ids) > 0) {
			$tmp_times = Timeslots::instance()->findAll(array("conditions" => "id IN (".implode(',',$all_timeslot_ids).")"));
			foreach ($tmp_times as $t) {
				$timeslots_cache[$t->getId()] = $t;
			}
		}

		foreach ($raw_rows as $row) {

			$timeslot_id = $row['object_id'];
			$contact_id = $row['contact_id'];
			$row_cat_id = $row['category_id'];

			if (!isset($utilization_data[$contact_id])) {
				// create the structure for the contact

				$contract_hours_cpval = CustomPropertyValues::getCustomPropertyValue($contact_id, $contract_hours_cp->getId());
				$contract_hours_per_day = $contract_hours_cpval instanceof CustomPropertyValue ? $contract_hours_cpval->getValue() : 8;
				if ($contract_hours_per_day == 0)
					$contract_hours_per_day = 8;
				$utilization_goal_cpval = CustomPropertyValues::getCustomPropertyValue($contact_id, $utilization_goal_cp->getId());
				$utilization_goal = $utilization_goal_cpval ? $utilization_goal_cpval->getValue() : 0;

				// calculate the contract hours using contract hours per day that assigned to user and working days in period
				$contract_hours = $working_days_in_period * (float) $contract_hours_per_day;
				$contract_hours = number_format($contract_hours, 2);

				$utilization_data[$contact_id] = array(
					'contact_name' => $row['contact_name'],
					'total_hours' => 0,
					'contract_hours' => $contract_hours,
					'utilization_goal' => $utilization_goal,
					'billable_hours' => 0,
					'non_billable_hours' => 0,
				);
				array_push($utilization_hrs, $utilization_goal);
				foreach ($first_level_categories as $cat) {
					$utilization_data[$contact_id]['registered_hours_' . $cat->getId()] = 0;
				}
			}

			// set the labor category hours for this contact
			$utilization_data[$contact_id]['registered_hours_' . $row_cat_id] = number_format($row['period_worked_hours'], 2);
			// accumulate total hours
			$utilization_data[$contact_id]['total_hours'] += $row['period_worked_hours'];

			// get the real labor category of the timeslot, so we can see if it is a time off or not
			$real_timeslot_cat_id = 0;
			$timeslot = array_var($timeslots_cache, $timeslot_id);
			if ($timeslot instanceof Timeslot) {
				$real_timeslot_cat = $timeslot->getMemberOfType($labor_cat_ot->getId());
				if ($real_timeslot_cat) {
					$real_timeslot_cat_id = $real_timeslot_cat->getId();
				}
			}
			// if real category or parent category is time off then the timeslot is a time off entry
			$is_time_off = in_array($real_timeslot_cat_id, $time_off_labor_categories) || in_array($row_cat_id, $time_off_labor_categories);

			// define if these labor category hours are billable or not, exclude from totals if category is for time off
			if (!$is_time_off) {
				if (in_array($row_cat_id, $billable_labor_categories)) {
					$utilization_data[$contact_id]['billable_hours'] += $row['period_worked_hours'];
				} else {
					$utilization_data[$contact_id]['non_billable_hours'] += $row['period_worked_hours'];
				}
			}

		}
	}
	$utilization_avg = array_sum($utilization_hrs)/count($utilization_hrs);

	if(count($utilization_data) > 0){
		$contact_names = array_column($utilization_data, 'contact_name');
		array_multisort($contact_names, SORT_ASC, $utilization_data);
	}

	// initialize grand totals row
	$totals = array(
		'contract_hours' => 0,
		'total_hours' => 0,
		'billable_hours' => 0,
		'non_billable_hours' => 0,
	);

	$total_contract_hours = 0;
	$total_contract_x_util_goal = 0;
	
	foreach ($utilization_data as $contact_id => &$contact_data) {
		if ($exclude_users_without_utilization_goal && $utilization_data[$contact_id]['utilization_goal'] == 0) {
			unset($utilization_data[$contact_id]);
			continue;
		}

		// calculate the billable utilization
		$billable_utilization = 0;
/*		if($include_contract_hours == 'contract'){
			if ($contact_data['contract_hours'] > 0) {
				$billable_utilization = number_format(100 * $contact_data['billable_hours'] / $contact_data['contract_hours'], 2);
			}
		} else { */
			$total_hours = $contact_data['billable_hours'] + $contact_data['non_billable_hours'];
			if($total_hours > 0){
				$billable_utilization = number_format(100 * $contact_data['billable_hours'] / $total_hours, 2);
			}
//		}
		$contact_data['billable_utilization'] = $billable_utilization;

		$total_contract_x_util_goal += (float) $contact_data['contract_hours'] * (float) $contact_data['utilization_goal'];
		$total_contract_hours += $contact_data['contract_hours'];

		// calculate the variation
		$variation = $billable_utilization - (float) $utilization_data[$contact_id]['utilization_goal'];
		$contact_data['variation'] = $variation;

		// format the other totals
		$contact_data['total_hours'] = number_format($contact_data['total_hours'], 2);
		$contact_data['billable_hours'] = number_format($contact_data['billable_hours'], 2);
		$contact_data['non_billable_hours'] = number_format($contact_data['non_billable_hours'], 2);
		$contact_data['variation'] =  number_format($contact_data['variation'], 2);
		array_push($variation_prtg, $contact_data['variation']);

		// accumulate grand totals
		$totals['contract_hours'] += $contact_data['contract_hours'];
		$totals['total_hours'] += $contact_data['total_hours'];
		$totals['billable_hours'] += $contact_data['billable_hours'];
		$totals['non_billable_hours'] += $contact_data['non_billable_hours'];

		// accumulate grand totals for labor category columns
		foreach ($contact_data as $data_key => $data_val) {
			if (str_starts_with($data_key, 'registered_hours_')) {
				if (!isset($totals[$data_key])) {
					$totals[$data_key] = 0;
				}
				$totals[$data_key] += $contact_data[$data_key];
			}
		}
		
	}

	// build the array of the report columns
	$columns = array();
	$columns['contact_name'] = lang('name');
	if($include_contract_hours){
		$columns['contract_hours'] = $contract_hours_cp->getName();
	}
	$columns['total_hours'] = lang('total');

	foreach ($first_level_categories as $labor_category) {
		$columns['registered_hours_'.$labor_category->getId()] = $labor_category->getName();
	}

	$columns['billable_hours'] = lang('billable');
	$columns['non_billable_hours'] = lang('non_billable');
	$columns['billable_utilization'] = lang('billable_utilization');
	$columns['utilization_goal'] = $utilization_goal_cp->getName();
	$columns['variation'] = lang('variation');


	// weighted utilization goal average total
	$utilization_avg_weighted = $total_contract_x_util_goal / $total_contract_hours;
	$totals['utilization_goal'] = $utilization_avg_weighted;

	// variation totals
	/*$variation_avg = array_sum($variation_prtg)/count($variation_prtg);
	$totals['variation'] = $variation_avg;*/

	// billable utilization goal
	if ($totals['total_hours'] > 0) {
		$billable_utilization = 100 * $totals['billable_hours'] / ($totals['billable_hours'] + $totals['non_billable_hours']);
	} else {
		$billable_utilization = 0;
	}
	$totals['billable_utilization'] = $billable_utilization;

	// variation totals
	$totals['variation'] = round($totals['billable_utilization'],2) - round($totals['utilization_goal'],2);

	// format grand totals row
	$totals = format_utilization_report_grand_totals_row($columns, $totals);

	$data = array(
		'columns' => $columns,
		'utilization_data' => $utilization_data,
		'totals' => $totals,
		'from_date' => $start_date,
		'to_date' => $end_date,
	);

	return $data;
}

function format_utilization_report_grand_totals_row($columns, $totals) {

	foreach ($columns as $key => $label) {
		$total_value = '';
		switch ($key) {
			case 'contact_name':
				$total_value = lang('total');
				break;
			case 'contract_hours':
			case 'total_hours':
			case 'billable_hours':
			case 'non_billable_hours':
				$total_value = number_format($totals[$key], 2);
				break;
			case 'billable_utilization':
				$total_value = number_format($totals[$key], 2);
				break;
			case 'variation';
				$total_value = number_format($totals[$key], 2);
			case 'utilization_goal';
				$total_value = number_format($totals[$key], 2);
			default:
				if (str_starts_with($key, 'registered_hours_')) {
					$total_value = $totals[$key] > 0 ? number_format($totals[$key], 2) : 0;
				}
				break;
		}
		$totals[$key] = $total_value;
	}

	return $totals;
}

function find_billable_labor_categories_member_ids(){
	$billable_labor_categories = array();
	 
	$labor_cat_ot = ObjectTypes::findByName('hour_type');
	if($labor_cat_ot instanceof ObjectType){
		$labor_cat_ot_id = $labor_cat_ot->getId();
		$billable_labot_cat_cp = MemberCustomProperties::instance()->findOne(array('conditions' => '`code`="is_billable" AND `object_type_id`='.$labor_cat_ot_id));
		if($billable_labot_cat_cp instanceof MemberCustomProperty){
			$billable_labot_cat_cp_id = $billable_labot_cat_cp->getId();
			$all_lab_cat_members = Members::instance()->findAll(array('conditions' => '`object_type_id`='.$labor_cat_ot_id));
			$non_billable_mcpvs = MemberCustomPropertyValues::instance()->findAll(array('conditions' => '`custom_property_id`='.$billable_labot_cat_cp_id. ' AND `value`="-1"'));
			$non_billable_mcpv_mem_ids = array();
			foreach($non_billable_mcpvs as $mcpv){
				$non_billable_mcpv_mem_ids[] = $mcpv->getMemberId();
			}
			foreach($all_lab_cat_members as $member){
				if(!in_array($member->getId(), $non_billable_mcpv_mem_ids)) $billable_labor_categories[] = $member->getId(); 
			}
		}
	}

	if(empty($billable_labor_categories)){
		$billable_labor_categories = config_option('billable_labor_categories');
	}

	return $billable_labor_categories;
}


function find_time_off_labor_categories_member_ids(){

	$labor_cat_ot_id = ObjectTypes::findByName('hour_type')->getId();

	$rows = DB::executeAll("
		SELECT m.id, m.name FROM ".TABLE_PREFIX."members m
		INNER JOIN ".TABLE_PREFIX."member_custom_properties cp ON cp.object_type_id = m.object_type_id
		LEFT JOIN ".TABLE_PREFIX."member_custom_property_values cpv ON cpv.custom_property_id = cp.id and cpv.member_id=m.id
		WHERE
		m.object_type_id = $labor_cat_ot_id
		AND cp.code = 'is_time_off'
		AND (cpv.value = 1)
	");

	if (!isset($rows)) $rows = array();

	return array_column($rows,'id');
}

function get_working_days_from_period($start_date, $end_date, $weekday_working_days){
	$working_days_in_period = 0;

	if (!$start_date instanceof DateTimeValue || !$end_date instanceof DateTimeValue) {
		return $working_days_in_period;
	}
	
	$start = $start_date->advance(logged_user()->getUserTimezoneValue(), false);
	$end = $end_date->advance(logged_user()->getUserTimezoneValue(), false);

	for($i = $start; $i <= $end; $i->add('d', 1)){
		$weekday = $i->format('N') - 1;
		if(in_array($weekday, $weekday_working_days)){
			$working_days_in_period++;
		}
	}

	return $working_days_in_period;
}
