<?php

Env::useHelper('view_functions', 'project_reports');

$report_data = array_var($_SESSION, 'actual_vs_forecast_report_data');

$from_date_str = format_date($report_data['start_date']);
$to_date_str = format_date($report_data['end_date']);
$report_data['is_pdf_export'] = isset($is_pdf_export) ? $is_pdf_export : false;
// Render report title
echo '<div class="project-report-title">'.$title.'</div>';

// Render date of the report
$today_dtv = DateTimeValueLib::now();
$today_date = format_date($today_dtv);
echo '<div class="project-report-info" style="float: right;">'.lang('report date').': '.$today_date.'</div>';

echo '<div class="project-report-info" style="margin-bottom: 20px;">'.lang('showing data from x to y', $from_date_str, $to_date_str).'</div>';

echo '<div class="clear"></div>';

function render_actual_vs_forecast_report($data, $report_data) {
    $revenue_types = array('forecasts', 'actuals');
    $use_proj_type = array_var($report_data, 'group_by') == 'project_type';
    $is_pdf_export = array_var($report_data, 'is_pdf_export', false);
    $table_zoom = $is_pdf_export ? 'zoom: 0.8;' : '';

    // Start table
    echo '<table class="budgetReport" style="width: 100%;'.$table_zoom.'">';
    foreach($revenue_types as $revenue_type) {
        render_revenue_type_name_actual_vs_forecast_report($data, $revenue_type, $report_data);
        render_main_header_actual_vs_forecast_report($data, $report_data);
        render_overall_totals_actual_vs_forecast_report($data, $report_data, $revenue_type);
        if($use_proj_type) {
            foreach($data['project_types'] as $proj_type_data) {
                render_project_type_header_actual_vs_forecast_report($proj_type_data, $data, $revenue_type);
                render_projects_for_actual_vs_forecast_report($proj_type_data['projects'], $data, $revenue_type);
            }
        } else {
            render_projects_for_actual_vs_forecast_report($data['projects'], $data, $revenue_type);
        }
        if($revenue_type == 'forecasts') {
            echo '<tr><td colspan="100" style="height: 30px;"></td></tr>';
        }

    }
    // Close table
    echo '</table>';
}

function render_revenue_type_name_actual_vs_forecast_report($data, $revenue_type, $report_data) {
    $columns = array_var($data, 'months_between_dates');
    $colspan = count($columns) * 2 + 5;
    $financial_type = array_var($report_data, 'financial_type', 'revenue');

    $revenue_type_name = strtoupper(lang($revenue_type . ' ' . $financial_type));
    echo '<thead class="tHeadTitles">
              <tr><th class="center bold" padding colspan="'.$colspan.'">'.$revenue_type_name.'</th></tr>
          </thead>';
}
function render_project_type_header_actual_vs_forecast_report($proj_type_data, $data, $revenue_type) {
    $columns = array_var($data, 'months_between_dates');
    $columns[] = 'total';
    $totals = $proj_type_data[$revenue_type];
    $currency_sym = config_option('currency_code');
    $pm_name = $proj_type_data['type_name'];

    // Start header
    echo '<thead class="titles"><tr>
            <th class="header_0" colspan="3">'.$pm_name.'</th>';
    foreach($columns as $column) {
        if($column == 'name') continue;
        $money_val = format_money_amount(array_var($totals, $column, 0), $currency_sym);
        echo '<th class="header_0 right" colspan="2">'.$money_val.'</th>';
    }

    // Close header
    echo '</tr></thead>';
}

function render_projects_for_actual_vs_forecast_report($projects, $data, $revenue_type) {
    $columns = array_var($data, 'months_between_dates');
    $columns[] = 'total';
    $currency_sym = config_option('currency_code');

    echo '<tbody>';
    foreach($projects as $project) {
        $totals = $project[$revenue_type];
        // Start project
        echo '<tr class="parentRow">
                <td class="left" colspan="3">'.$project['name'].'</td>';
        foreach($columns as $column) {
            if($column == 'name') continue;
            $money_val = format_money_amount(array_var($totals, $column, 0), $currency_sym);
            echo '<td class="right" colspan="2">'.$money_val.'</td>';
        }
        // Close project
        echo '</tr>';
    }
    echo '</tbody>';

}

function render_main_header_actual_vs_forecast_report($data, $report_data) {
    $columns = array_var($data, 'months_between_dates');
    // Start header

    echo '<thead class="tHeadTitles"><tr>
            <th class="left bold" colspan="3"></th>';
    foreach($columns as $column) {
        $column_arr = explode('-', $column);
        $year = $column_arr[0];
        $month = $column_arr[1];
        $month_str = ucfirst(lang('month '. intval($month)));
        $month_abbr = substr($month_str, 0, 3);

        echo '<th class="center bold" colspan="2">'.$month_abbr . ' ' . $year.'</th>';
    }
    echo '<th class="center bold" colspan="2">'.lang('totals').'</th>';

    // Close header
    echo '</tr></thead>';
}

function render_overall_totals_actual_vs_forecast_report($data, $report_data, $revenue_type) {
    $columns = array_var($data, 'months_between_dates');
    $columns[] = 'total';
    $totals = array_var($data, $revenue_type, array());

    $currency_sym = config_option('currency_code');

    // Start totals
    echo '<tbody><tr class="titles">
            <td class="left header_1" colspan="3">'.lang('totals').'</td>';
    foreach($columns as $column) {
        if($column == 'name') continue;
        $money_val = format_money_amount($totals[$column], $currency_sym);
        echo '<td class="right header_1" colspan="2">'.$money_val.'</td>';
    }

    // Close totals
    echo '</tr></tbody>';
}

render_actual_vs_forecast_report($data, $report_data);