<?php

// Render report title
echo '<div class="project-report-title">'.$title.'</div>';

// Render date of the report
$today_dtv = DateTimeValueLib::now();
$today_date = format_date($today_dtv);
echo '<div class="project-report-info" style="float: right;">'.lang('report date').': '.$today_date.'</div>';

echo render_all_aging_invoices_report($data, $report_data);

function render_all_aging_invoices_report($data, $report_data){
    // Get custom property 'project_manager'
    $project_ot = ObjectTypes::findByName('project');
    $cp_project_manager = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="project_manager"'));
    $contacts = $data['contacts'];
    foreach($contacts as $contact_id => $contact_info){
        echo render_contact_info($contact_info, $cp_project_manager, $report_data);
    }
    

    echo render_report_totals($data['totals'], $data['distribution'], $report_data);
}

function render_contact_info($contact_info, $cp_project_manager, $report_data){
    $group_by = array_var($report_data, 'group_by', 'project');
    $has_selected_pm_id = array_var($report_data, 'has_selected_pm_id');
    echo '<table class="aging-invoices-table">';

    // HEADER
    $header_columns_1 = array(' ', 'invoice');
    if(isset($report_data['show_invoice_issued_date'])) {
        $header_columns_1[] = 'date';
    }
    $header_columns_2 = array('due_date', 'balance');
    if(isset($report_data['include_current_invoices'])){
        $header_columns_2[] = 'current not due';
    }
    $header_columns_3 = array('1-30', '31-60', '61-90', '91-120', 'over 120');
    $header_columns = array_merge($header_columns_1, $header_columns_2, $header_columns_3);
    $header_colspan = count($header_columns);
    echo '<thead>';
    if($has_selected_pm_id || $group_by == 'pm') {
        echo '<tr class="table-name"><td colspan="' . $header_colspan . '"><span style="color: #0a85d8; padding-right: 10px;">'.$cp_project_manager->getName().'</span> '.$contact_info['contact_name'].'</td></tr>';
    }
    if(!$has_selected_pm_id && $group_by == 'report_sent') {
        $project_ot = ObjectTypes::findByName('project');
        $cp_report_sent = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="report_sent"'));
        $report_sent_date = $contact_info['contact_name'];
        if(strpos($report_sent_date, lang('without')) === false) {
            $dt = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $report_sent_date);
            $report_sent_date = format_date($dt);
        }
        
        echo '<tr class="table-name"><td colspan="' . $header_colspan . '"><span style="color: #0a85d8; padding-right: 10px;">'.$cp_report_sent->getName().'</span> '.$report_sent_date.'</td></tr>';
    }
    echo '<tr>';
    foreach($header_columns as $c_name){
        echo '<th>'.lang($c_name).'</th>';
    }
    echo '</tr></thead>';

    // BODY
    // Project Manager name
    echo '<tbody>';

    // Render projects
    $project_and_invoices_info = $contact_info['project_and_invoices_info'];
    foreach($project_and_invoices_info['projects'] as $project_id => $project_info){
        echo render_project_info($project_info, $report_data);
    }

    // Render Contact Totals
    $total_name = $contact_info['contact_name'];
    if(!$has_selected_pm_id && $group_by == 'report_sent') {
       $total_name = $report_sent_date;
    }

    echo render_contact_totals($total_name, $project_and_invoices_info['contact_totals'], $report_data);


    echo '</tbody></table>';

}

function render_project_info($project_info, $report_data){
    // Client and Project name row
    $project_name = $project_info['project_info']['full_project_name'];
    $billing_contact_data = array_var($project_info['project_info'], 'billing_contact_info', null);
    $has_selected_pm_id = array_var($report_data, 'has_selected_pm_id');
    $group_by = array_var($report_data, 'group_by', 'project');
    $header_colspan = isset($report_data['show_invoice_issued_date']) ? 10 : 9;
    if(isset($report_data['include_current_invoices'])) {
        $header_colspan++;
    }
    echo '<tr>';
	echo '<td class="bold" colspan="'.$header_colspan.'">';

	// Show client name if exist
	if(!empty($project_info['client_name'])) {
		echo lang('customer') . ': ' . $project_info['client_name'] . '<br>';
	}
	
	echo lang('project') . ': ' . $project_name;
    if(isset($report_data['show_billing_contact'])) {
       $cp_billing_contact_name = array_var($report_data, 'cp_billing_contact_name', 'billing contact');
       if(is_null($billing_contact_data)) {
            echo '<br>' . lang($cp_billing_contact_name) . ': ' . '-';
       } else {
            echo '<br>' . lang($cp_billing_contact_name) . ': ' . $billing_contact_data['name'];
            echo ' (' . lang('email') . ': ' . $billing_contact_data['email'] . '; ';
            echo lang('phone') . ': ' . $billing_contact_data['phone'] . ')';
       }
    }
    $groups_by_cp_report_sent = !$has_selected_pm_id && $group_by == 'report_sent';
    if(isset($report_data['show_report_sent']) && !$groups_by_cp_report_sent) {
        $cp_report_sent_name = array_var($report_data, 'cp_report_sent_name', 'Report sent');
        $report_sent_date = $project_info['project_info']['report_sent'];
        echo '<br>' . $cp_report_sent_name . ': ' . $report_sent_date;
    }
    if(isset($report_data['show_invoice_sent'])) {
        $cp_invoice_sent_name = array_var($report_data, 'cp_invoice_sent_name', 'Invoice sent');
        $invoice_sent_date = $project_info['project_info']['invoice_sent'];
        echo '<br>' . $cp_invoice_sent_name . ': ' . $invoice_sent_date;
    }
    if(isset($report_data['show_collection_notes'])) {
        $cp_collection_notes_name = array_var($report_data, 'cp_collection_notes_name', 'Collection notes');
        $collection_notes = $project_info['project_info']['collection_notes'];
        echo '<br>' . $cp_collection_notes_name . ': ' . $collection_notes;
    }

	echo '</td>';
	echo '</tr>';

	// Render invoices
    foreach($project_info['invoices'] as $invoice){
        echo render_invoice_info($invoice, $report_data);
    }
    // Project total row
    echo render_project_total($project_info, $report_data);
}

function render_invoice_info($invoice, $report_data){
    $currency = Currencies::instance()->getCurrency($invoice['currency_id']);
    $currency_sym = $currency instanceof Currency ? $currency->getSymbol() : config_option('currency_code');
    // Build invoice row
    echo '<tr><td></td>';
    echo '<td>'.$invoice['invoice_number'].'</td>';
    if(isset($report_data['show_invoice_issued_date'])) {
        echo '<td>'.$invoice['issued_date'].'</td>';
    }
    echo '<td>'.$invoice['date'].'</td>';
    echo '<td class="amount">'.format_money_amount($invoice['total'], $currency_sym).'</td>';
    if(isset($report_data['include_current_invoices'])) {
            echo '<td class="amount">'.format_money_amount($invoice['current_time'], $currency_sym).'</td>';
    }
    echo '<td class="amount">'.format_money_amount($invoice['first_month'], $currency_sym).'</td>';
    echo '<td class="amount">'.format_money_amount($invoice['second_month'], $currency_sym).'</td>';
    echo '<td class="amount">'.format_money_amount($invoice['third_month'], $currency_sym).'</td>';
    echo '<td class="amount">'.format_money_amount($invoice['fourth_month'], $currency_sym).'</td>';
    echo '<td class="amount">'.format_money_amount($invoice['greater_than_four_month'], $currency_sym).'</td>';
    echo '</tr>';
}

function render_project_total($project_info, $report_data){
    $project_totals = $project_info['project_totals'];
    $totals = build_totals_using_multicurrencies($project_totals);
    $total_colspan = isset($report_data['show_invoice_issued_date']) ? 4 : 3;
    
    // Build project totals row
    echo '<tr class="bold">';
    echo '<td colspan="' . $total_colspan . '">'.lang('total for').' '.$project_info['project_info']['project_id'].'</td>';
    echo '<td class="amount">'.$totals['total'].'</td>';
    if(isset($report_data['include_current_invoices'])) {
        echo '<td class="amount">'.$totals['current_time'].'</td>';
    }
    echo '<td class="amount">'.$totals['first_month'].'</td>';
    echo '<td class="amount">'.$totals['second_month'].'</td>';
    echo '<td class="amount">'.$totals['third_month'].'</td>';
    echo '<td class="amount">'.$totals['fourth_month'].'</td>';
    echo '<td class="amount">'.$totals['greater_than_four_month'].'</td>';
    echo '</tr>';   
}

function render_contact_totals($contact_name, $contact_totals, $report_data){
    $totals = build_totals_using_multicurrencies($contact_totals);
    $total_colspan = isset($report_data['show_invoice_issued_date']) ? 4 : 3;
    // Build project totals row
    echo '<tr class="contact-totals">';
    echo '<td colspan="' . $total_colspan . '">'.lang('total for').' '.$contact_name.'</td>';
    echo '<td class="amount">'.$totals['total'].'</td>';
    if(isset($report_data['include_current_invoices'])) {
        echo '<td class="amount">'.$totals['current_time'].'</td>';
    }
    echo '<td class="amount">'.$totals['first_month'].'</td>';
    echo '<td class="amount">'.$totals['second_month'].'</td>';
    echo '<td class="amount">'.$totals['third_month'].'</td>';
    echo '<td class="amount">'.$totals['fourth_month'].'</td>';
    echo '<td class="amount">'.$totals['greater_than_four_month'].'</td>';
    echo '</tr>';
}

function render_report_totals($totals, $distribution_totals, $report_data){
    $formatted_totals = build_totals_using_multicurrencies($totals);
    $distrib_totals = build_distribution_totals_using_multicurrencies($distribution_totals);
    $total_colspan = isset($report_data['show_invoice_issued_date']) ? 4 : 3;
    echo '<table class="aging-invoices-table">';

    // Header
    echo '<thead>';
    echo '<tr class="table-name--totals"><td colspan="9">'.lang('totals').'</td></tr>';
    echo '<tr><th colspan="' . $total_colspan . '"></th>';
    $header_columns_1 = array('balance');
    if(isset($report_data['include_current_invoices'])) {
        $header_columns_1[] = 'current not due';
    }
    $header_columns_2 = array('1-30', '31-60', '61-90', '91-120', 'over 120');
    $header_columns = array_merge($header_columns_1, $header_columns_2);
    foreach($header_columns as $c_name){
        echo '<th>'.lang($c_name).'</th>';
    }
    echo '</tr></thead>';

    // Body
    echo '<tbody>';

    // Build totals row
    echo '<tr>';
    echo '<td class="bold" colspan="' . $total_colspan . '">'.lang('final totals').'</td>';
    echo '<td class="amount">'.$formatted_totals['total'].'</td>';
    if(isset($report_data['include_current_invoices'])) {
        echo '<td class="amount">'.$formatted_totals['current_time'].'</td>';
    }
    echo '<td class="amount">'.$formatted_totals['first_month'].'</td>';
    echo '<td class="amount">'.$formatted_totals['second_month'].'</td>';
    echo '<td class="amount">'.$formatted_totals['third_month'].'</td>';
    echo '<td class="amount">'.$formatted_totals['fourth_month'].'</td>';
    echo '<td class="amount">'.$formatted_totals['greater_than_four_month'].'</td>';
    echo '</tr>';

      // Build distribution row
    if($distribution_totals['currencies']){
        echo '<tr>';
        echo '<td class="bold" colspan="' . $total_colspan . '">'.lang('distribution').'</td>';
        echo '<td class="amount">'.$distrib_totals['total'].'</td>';
        if(isset($report_data['include_current_invoices'])) {
            echo '<td class="amount">'.$distrib_totals['current_time'].'</td>';
        }
        echo '<td class="amount">'.$distrib_totals['first_month'].'</td>';
        echo '<td class="amount">'.$distrib_totals['second_month'].'</td>';
        echo '<td class="amount">'.$distrib_totals['third_month'].'</td>';
        echo '<td class="amount">'.$distrib_totals['fourth_month'].'</td>';
        echo '<td class="amount">'.$distrib_totals['greater_than_four_month'].'</td>';
        echo '</tr>';
        
    }


    echo '</table>';

}

function build_totals_using_multicurrencies($totals) {
     $total = '';
     $current_time = '';
     $first_month = '';
     $second_month = '';
     $third_month = '';
     $fourth_month = '';
     $greater_than_four_month = '';
     foreach($totals['currencies'] as $currency_id => $currency_totals){
         $currency = Currencies::instance()->getCurrency($currency_id);
         $currency_sym = $currency instanceof Currency ? $currency->getSymbol() : config_option('currency_code');
         $total .= '<div>' . format_money_amount($currency_totals['total'], $currency_sym) . '</div>';
         $current_time .= '<div>' . format_money_amount($currency_totals['current_time'], $currency_sym) . '</div>';
         $first_month .= '<div>' . format_money_amount($currency_totals['first_month'], $currency_sym) . '</div>';
         $second_month .= '<div>' . format_money_amount($currency_totals['second_month'], $currency_sym) . '</div>';
         $third_month .= '<div>' . format_money_amount($currency_totals['third_month'], $currency_sym) . '</div>';
         $fourth_month .= '<div>' . format_money_amount($currency_totals['fourth_month'], $currency_sym) . '</div>';
         $greater_than_four_month .= '<div>' . format_money_amount($currency_totals['greater_than_four_month'], $currency_sym) . '</div>';
     }

     $result = array(
        'total' => $total,
        'current_time' => $current_time,
        'first_month' => $first_month,
        'second_month' => $second_month,
        'third_month' => $third_month,
        'fourth_month' => $fourth_month,
        'greater_than_four_month' => $greater_than_four_month
     );

     return $result;
}

function build_distribution_totals_using_multicurrencies($totals){
    $total = '';
     $current_time = '';
     $first_month = '';
     $second_month = '';
     $third_month = '';
     $fourth_month = '';
     $greater_than_four_month = '';
     foreach($totals['currencies'] as $currency_id => $currency_totals){  
        $total .= '<div>' . $currency_totals['total'].' %' . '</div>';
        $current_time .= '<div>' . $currency_totals['current_time'].' %' . '</div>';
        $first_month .= '<div>' . $currency_totals['first_month'].' %' . '</div>';
        $second_month .= '<div>' . $currency_totals['second_month'].' %' . '</div>';
        $third_month .= '<div>' . $currency_totals['third_month'].' %' . '</div>';
        $fourth_month .= '<div>' . $currency_totals['fourth_month'].' %' . '</div>';
        $greater_than_four_month .= '<div>' . $currency_totals['greater_than_four_month'].' %' . '</div>';
     }

     $result = array(
        'total' => $total,
        'current_time' => $current_time,
        'first_month' => $first_month,
        'second_month' => $second_month,
        'third_month' => $third_month,
        'fourth_month' => $fourth_month,
        'greater_than_four_month' => $greater_than_four_month
     );

     return $result;
}
