<?php

$from_date_str = null;
if ($data['from_date']) {
	$from_date_str = format_date($data['from_date'], null, 0);
}
$to_date_str = null;
if ($data['to_date']) {
	$data['to_date']->add('d',-1);
	$to_date_str = format_date($data['to_date'], null, 0);
}

// Render report title
echo '<div class="project-report-title">'.$title.'</div>';

// Render date of the report
$today_dtv = DateTimeValueLib::now();
$today_date = format_date($today_dtv);
echo '<div class="project-report-info" style="float: right;">'.lang('report date').': '.$today_date.'</div>';

if ($from_date_str && $to_date_str) {
	echo '<div class="project-report-info">'.lang('showing data from x to y', $from_date_str, $to_date_str).'</div>';
}

?>
<div class="clear"></div>

<?php

render_all_projects_ar_ledger_data($data);

function render_all_projects_ar_ledger_data($data) {

	// columns and report data
	$columns = array_var($data, 'columns');
	$ar_ledger_data = array_var($data, 'ar_ledger_data');
	$ar_ledger_totals = array_var($data, 'ar_ledger_totals');

	// render data for each project
	foreach ($ar_ledger_data as $project_data) {
		render_ar_ledger_project_info($project_data, $columns);
	}
	
	// render totals table
    render_ar_ledger_report_totals($ar_ledger_totals, $columns);
}

function render_ar_ledger_project_info($project_data, $columns) {
	echo '<table class="ar-ledger-table">';

    // HEADER - project name
    echo '<thead>';
    echo '<tr class="table-name"><td colspan="'.count($columns).'">'.$project_data['project_name'].'</td></tr>';
	// column headers
    echo '<tr>';
    foreach($columns as $c_name){
        echo '<th>'.lang($c_name).'</th>';
    }
    echo '</tr></thead>';

    // BODY
    echo '<tbody>';

    // Render projects
    $invoices_info = $project_data['grouped_invoices'];
    foreach($invoices_info as $invoice_info){
        render_ar_ledger_invoice_info($invoice_info, $columns);
    }

    // Render Contact Totals
    render_ar_ledger_project_totals($project_data, $columns);


    echo '</tbody></table>';
}

function render_ar_ledger_invoice_info($invoice_info, $columns) {
	$invoice = $invoice_info['invoice'];
	$payments = $invoice_info['payments'];
	$currency = Currencies::instance()->getCurrency($invoice['currency_id']);
	$currency_sym = $currency instanceof Currency ? $currency->getSymbol() : '$';

	$inv_num = lang($invoice['invoice_type_name']) . " " .$invoice['invoice_number'];

	echo '<tr class="invoice-total"><td colspan="'.count($columns).'">'. $inv_num . " - " . clean($invoice['name']) . '</td></tr>';

	$amount_prefix = '';
	$amount_suffix = '';
	if (!$invoice['sum_in_total']) {
		// for credit and debit notes
		$amount_prefix = '(';
		$amount_suffix = ')';		
	}
    
    echo '<tr>';
	foreach ($columns as $col) {
		$cell_class = '';
		switch ($col) {
			case 'invoice':
				$cell_class = 'name indent-small';
				$cell_value = $inv_num;
				break;
			case 'receipt_detail':
				$cell_value = '';
				break;
			case 'date':
				$cell_value = $invoice['date'];
				break;
			case 'total':
				$cell_class = 'amount';
				$cell_value = $amount_prefix . format_money_amount($invoice['total'], $currency_sym) . $amount_suffix;
				break;
			case 'fees':
				$cell_class = 'amount';
				$cell_value = $amount_prefix . format_money_amount($invoice['subtotal'], $currency_sym) . $amount_suffix;
				break;
			case 'taxes':
				$cell_class = 'amount';
				$cell_value = $amount_prefix . format_money_amount($invoice['taxes'], $currency_sym) . $amount_suffix;
				break;
		}

		echo '<td class="'.$cell_class.'">'.$cell_value.'</td>';
	}
	echo '</tr>';

	if (is_array($payments) && count($payments) > 0) {
		foreach ($payments as $payment) {
			$p_currency = Currencies::instance()->getCurrency($payment['currency_id']);
			$p_currency_sym = $p_currency instanceof Currency ? $p_currency->getSymbol() : '$';

			echo '<tr>';
			foreach ($columns as $col) {
				$cell_class = '';
				switch ($col) {
					case 'invoice':
						$cell_class = 'name indent-small';
						$cell_value = lang('payment for')." ".$inv_num;
						break;
					case 'receipt_detail':
						$cell_class = 'name';
						$cell_value = $payment['ext_transaction_id'] ? $payment['ext_transaction_id'] : $payment['name'];
						break;
					case 'date':
						$cell_value = $payment['date'];
						break;
					case 'total':
						$cell_class = 'amount';
						$cell_value = '('.format_money_amount($payment['amount'], $p_currency_sym).')';
						break;
					case 'fees':
						$cell_class = 'amount';
						$cell_value = '';
						break;
					case 'taxes':
						$cell_class = 'amount';
						$cell_value = '';
						break;
				}

				echo '<td class="'.$cell_class.'">'.$cell_value.'</td>';
			}
			echo '</tr>';
		}
	}

	echo '<tr class="invoice-total">';
	foreach ($columns as $col) {
		$cell_class = '';
		$cell_value = '';
		switch ($col) {
			case 'invoice':
				$cell_value = lang('total')." ".$inv_num;
				break;
			case 'receipt_detail':
			case 'date':
				break;
			case 'total':
				$cell_class = 'amount';
				$cell_value = $amount_prefix . format_money_amount(isset($invoice['due_amount']) ? $invoice['due_amount'] : 0, $currency_sym) . $amount_suffix;
				break;
			case 'fees':
				$cell_class = 'amount';
				$cell_value = $amount_prefix . format_money_amount($invoice['subtotal'], $currency_sym) . $amount_suffix;
				break;
			case 'taxes':
				$cell_class = 'amount';
				$cell_value = $amount_prefix . format_money_amount($invoice['taxes'], $currency_sym) . $amount_suffix;
				break;
		}

		echo '<td class="'.$cell_class.'">'.$cell_value.'</td>';
	}
	echo '</tr>';

}

function render_ar_ledger_project_totals($project_data, $columns) {

	$totals = $project_data['totals'];
	$currency = Currencies::instance()->getCurrency($totals['currency_id']);
	$currency_sym = $currency instanceof Currency ? $currency->getSymbol() : '$';

	echo '<tr class="project-total">';
	foreach ($columns as $col) {
		$colspan = 1;
		$draw_cell = true;
		$cell_class = '';
		$cell_value = '';
		switch ($col) {
			case 'invoice':
				$colspan = 3;
				$cell_value = lang('total')." ".$project_data['project_name'];
				break;
			case 'receipt_detail':
			case 'date':
				$draw_cell = false;
				break;
			case 'total':
				$cell_class = 'amount';
				$cell_value = format_money_amount($totals['due_amount'], $currency_sym);
				break;
			case 'fees':
				$cell_class = 'amount';
				$cell_value = format_money_amount($totals['subtotal'], $currency_sym);
				break;
			case 'taxes':
				$cell_class = 'amount';
				$cell_value = format_money_amount($totals['taxes'], $currency_sym);
				break;
		}

		if ($draw_cell) echo '<td colspan="'.$colspan.'" class="'.$cell_class.'">'.$cell_value.'</td>';
	}
	echo '</tr>';
}

function render_ar_ledger_report_totals($ar_ledger_totals, $columns) {

	$currency = Currencies::instance()->getCurrency($ar_ledger_totals['currency_id']);
	$currency_sym = $currency instanceof Currency ? $currency->getSymbol() : '$';

	echo '<table class="ar-ledger-table">';

    // HEADER - project name
    echo '<thead>';
    echo '<tr class="table-name"><td colspan="'.count($columns).'">'.lang('total').'</td></tr>';
	// column headers
    echo '<tr>';
    foreach($columns as $c_name){
		if (in_array($c_name, array('invoice','receipt_detail','date'))) {
			echo '<th></th>';
		} else {
			echo '<th>'.lang($c_name).'</th>';
		}
    }
    echo '</tr></thead>';

    // BODY
    echo '<tbody><tr class="grand-total">';

    foreach ($columns as $col) {
		$colspan = 1;
		$cell_class = '';
		$cell_value = '';
		switch ($col) {
			case 'invoice':
			case 'receipt_detail':
			case 'date':
				break;
			case 'total':
				$cell_class = 'amount';
				$cell_value = format_money_amount($ar_ledger_totals['due_amount'], $currency_sym);
				break;
			case 'fees':
				$cell_class = 'amount';
				$cell_value = format_money_amount($ar_ledger_totals['subtotal'], $currency_sym);
				break;
			case 'taxes':
				$cell_class = 'amount';
				$cell_value = format_money_amount($ar_ledger_totals['taxes'], $currency_sym);
				break;
		}

		echo '<td colspan="'.$colspan.'" class="'.$cell_class.'">'.$cell_value.'</td>';
	}

    echo '</tr></tbody></table>';
}


?>