<?php

// Render report title
echo '<div class="project-report-title">'.$title.'</div>';

// Render date of the report
$today_dtv = DateTimeValueLib::now();
$today_date = format_date($today_dtv);
$report_data['is_pdf_export'] = isset($is_pdf_export) ? $is_pdf_export : false;
echo '<div class="project-report-info" style="float: right;">'.lang('report date').': '.$today_date.'</div>';

echo render_budget_by_tasks_report($data, $report_data);

function render_budget_by_tasks_report($data, $report_data) {
    $is_pdf_export = array_var($report_data, 'is_pdf_export', false);
    $table_zoom = $is_pdf_export ? 'zoom: 0.9;' : '';
    echo '<table class="budgetReport" style="'.$table_zoom.'">';
    render_header_for_budget_by_tasks($report_data);
    if($data['has_project_manager']) {
        render_project_manager_for_budget_by_tasks($data, $report_data);
        render_project_for_budget_by_tasks($data['project_data'], $report_data, $indentation, $is_parent);
    } else {
        $indentation = 1;
        $is_parent = true;
        render_project_for_budget_by_tasks($data['project_data'], $report_data, $indentation, $is_parent);
    }
    echo '</table>';
    require_javascript("og/reports/budget_by_tasks_report.js");
    render_script_for_budget_by_tasks_report();
}

function render_header_for_budget_by_tasks($report_data) {
    echo '<thead class="tHeadTitles">
            <tr>
            <th class="" colspan="4"></th>
            <th class="center groupTitle" colspan="3" scope="colgroup">'.lang('time').'</th>
            <th class="center groupTitle" colspan="3" scope="colgroup">'.lang('cost').'</th>
            <th class="center groupTitle" colspan="3" scope="colgroup">'.lang('price').'</th>
            </tr>
            <tr>
            <th class="center" colspan="2"></th>
            <th class="center">'.lang('percent completed').'</th>
            <th class="center">'.lang('status').'</th>
            <th class="right">'.lang('estimated (hrs)').'</th>
            <th class="right">'.lang('worked').'</th>
            <th class="right">'.lang('remaining budgeted').'</th>
            <th class="right">'.lang('estimated cost').'</th>
            <th class="right">'.lang('executed cost').'</th>
            <th class="right">'.lang('remaining budgeted').'</th>
            <th class="right">'.lang('estimated price').'</th>
            <th class="right">'.lang('earned value').'</th>
            <th class="right">'.lang('remaining budgeted').'</th>
            </tr>
        </thead>';

    
}

function render_project_manager_for_budget_by_tasks($data, $report_data) {

    $project_manager_name = $data['project_manager_name'];
    render_manager_name_for_budget_by_tasks($project_manager_name);
    $projects = $data['project_data'];
    foreach($projects as $project) {
        render_project_for_budget_by_tasks($project, $report_data, 1, true);
    }
    render_project_total_for_budget_by_tasks($data, $report_data, 0, true, true);
}

function render_manager_name_for_budget_by_tasks($project_manager_name) {
    echo '<tbody class="titles">
            <tr>
            <td class="header_0" colspan="13">'.$project_manager_name.'</td>
            </tr>
        </tbody>';
}

function render_project_for_budget_by_tasks($project, $report_data, $indentation, $is_parent) {
    $indent = $indentation * 20;
    $group_by_job_phase = array_var($report_data, 'group_by', false) == 1;
    $tasks = $project['tasks_data'];
    $subprojects = $project['subprojects_data'];

    if((($group_by_job_phase && count($tasks['phases']) == 0) || count($tasks) == 0) && count($subprojects) == 0) {
        return;
    }
 
    render_project_header_with_totals_for_budget_by_tasks($project, $report_data, $indentation, $is_parent);
    
    $indentation += 1;
    $data_level = 1;

    if($group_by_job_phase) {
        foreach($tasks['phases'] as $phase) {
            render_phase_for_budget_by_tasks($phase, $report_data, $indentation);
        }
    } else {
        foreach($tasks as $task) {
            render_task_for_budget_by_tasks($task, $report_data, $indentation, true, $data_level);
        }
    }

    if(!$group_by_job_phase) {
        render_unassigned_time_for_budget_by_tasks($project, $report_data, $indentation, $data_level);
        render_unassigned_actual_expenses_for_budget_by_tasks($project, $report_data, $indentation, $data_level);
        render_unassigned_budgeted_expenses_for_budget_by_tasks($project, $report_data, $indentation, $data_level);
    }

    foreach($subprojects as $subproject) {
        render_project_for_budget_by_tasks($subproject, $report_data, $indentation, false);
    }
}

function render_phase_for_budget_by_tasks($phase, $report_data, $indentation) { 
    $indent = $indentation * 20;
    $header_class = 'header_2';

    $tasks = $phase['tasks'];
    $has_tasks = count($tasks) > 0;
    $time_entries = $phase['time_data']['time_entries'];
    $has_time = count($time_entries) > 0;
    $budgeted_expenses = $phase['budgeted_expenses_data']['budgeted_expenses'];
    $has_budgeted_expenses = count($budgeted_expenses) > 0;
    $actual_expenses = $phase['actual_expenses_data']['actual_expenses'];
    $has_actual_expenses = count($actual_expenses) > 0;

    if(!$has_tasks && !$has_time && !$has_budgeted_expenses && !$has_actual_expenses) {
        return;
    }

    // build the phase name row
    $is_phase = true;
    render_project_header_with_totals_for_budget_by_tasks($phase, $report_data, $indentation, false, $is_phase);

    $indentation += 1;

    $data_level = 1;
    if(count($tasks) > 0) {
        foreach($tasks as $task) {
            render_task_for_budget_by_tasks($task, $report_data, $indentation, true, $data_level);
        }
    }
    render_unassigned_time_for_budget_by_tasks($phase, $report_data, $indentation, $data_level, false, true);
    render_unassigned_actual_expenses_for_budget_by_tasks($phase, $report_data, $indentation, $data_level, false, true);
    render_unassigned_budgeted_expenses_for_budget_by_tasks($phase, $report_data, $indentation, $data_level, false, true);

}

function render_task_for_budget_by_tasks($task, $report_data, $indentation, $is_parent, $data_level) {
    $is_pdf_export = array_var($report_data, 'exportPDF', false);
    $currency_sym = config_option('currency_code');
    $indent = $indentation * 20;
    $subtasks = $task['subtasks'];
    $has_subtasks = count($subtasks) > 0;
    $has_time = count(array_var($task['time_data'], 'time_entries', array())) > 0;
    $has_actual_expenses = count(array_var($task['actual_expenses_data'], 'actual_expenses', array())) > 0;
    $has_budgeted_expenses = count(array_var($task['budgeted_expenses_data'], 'budgeted_expenses', array())) > 0;
    $completed = $task['is_completed'] ? lang('completed') : lang('open');
    $negative_minutes = $task['remaining_minutes'] < 0 ? 'negative' : '';
    $remaining_minutes = get_formatted_remaining_minutes($task['remaining_minutes']);
    $negative_cost = $task['remaining_cost'] < 0 ? 'negative' : '';
    $remaining_cost = get_formatted_remaining_value($task['remaining_cost']);
    $negative_price = $task['remaining_price'] < 0 ? 'negative' : '';
    $remaining_price = get_formatted_remaining_value($task['remaining_price']);
    $parent_child_class = $is_parent ? 'parentRow' : 'childs';
    $percent_completed_class = $task['percent_completed'] == 100 ? 'positive' : '';

    $task_icon = '';
    $task_name_padding = '';
    $is_collapsable_row = $has_subtasks || $has_time || $has_actual_expenses || $has_budgeted_expenses;
    if($is_collapsable_row && !$is_pdf_export) {
        $task_icon = '<img class="parentSwitch" src="./plugins/project_reports/public/assets/images/open.svg" data-status="closed" alt="">';
        $indent = $indent - 23;
    }

    $collapsed = $data_level > 1 ? 'collapsed' : '';
    if($is_collapsable_row) {
        echo '<tr class="'.$parent_child_class .' '.$collapsed.'" data-level="'.$data_level.'">';
    } else {
        echo '<tr class="'.$parent_child_class .' '.$collapsed.'" data-level="'.$data_level.'">';
    }
    
    echo '<td style="padding-left: '.$indent.'px;" colspan="2">';
    echo $task_icon;
    echo '<span style="' .$task_name_padding . '">' . $task['task_name'] . '</span>';
    echo '</td>
        <td class="center '.$percent_completed_class.'">'.round($task['percent_completed']). ' %'.'</td>
        <td class="left '.$percent_completed_class.'">'.$completed.'</td>
        <td class="right">' . format_amount(round($task['estimated_minutes'] / 60, 2), 2) . '</td>
        <td class="right">' . format_amount(round($task['worked_minutes'] / 60, 2), 2). '</td>
        <td class="right '.$negative_minutes.'">' . $remaining_minutes . '</td>
        <td class="right">' . format_money_amount($task['estimated_cost'], $currency_sym) . '</td>
        <td class="right">' . format_money_amount($task['executed_cost'], $currency_sym) . '</td>
        <td class="right '.$negative_cost.'">' . $remaining_cost . '</td>
        <td class="right">' . format_money_amount($task['estimated_price'], $currency_sym) . '</td>
        <td class="right">' . format_money_amount($task['executed_price'], $currency_sym) . '</td>
        <td class="right '.$negative_price.'">' . $remaining_price . '</td>
        </tr>';
    
    $indentation += 1;

    $data_level += 1;
    foreach($subtasks as $subtask) {
        render_task_for_budget_by_tasks($subtask, $report_data, $indentation, false,  $data_level);
    }

    $is_task = true;
    render_unassigned_time_for_budget_by_tasks($task, $report_data, $indentation, $data_level, $is_task);
    render_unassigned_actual_expenses_for_budget_by_tasks($task, $report_data, $indentation, $data_level, $is_task);
    render_unassigned_budgeted_expenses_for_budget_by_tasks($task, $report_data, $indentation, $data_level, $is_task);

}

function render_unassigned_time_for_budget_by_tasks($project, $report_data, $indentation, $data_level, $is_task = false, $is_phase = false) {
    $is_pdf_export = array_var($report_data, 'exportPDF', false);
    if(!$is_phase) {
        $unassigned_time_data = !$is_task ? $project['unassigned_time_entries'] : $project['time_data'];
    } else {
        $unassigned_time_data = $project['time_data'];
    }
    $time_entries = array_var($unassigned_time_data, 'time_entries', array());
    if(count($time_entries) == 0) {
        return;
    }
   
    $row_title = !$is_task ? lang('time entries not linked to tasks') : lang('time entries linked to') . ' '. $project['task_name'];
    $currency_sym = config_option('currency_code');
    if(!$is_pdf_export) {
        $indent = $indentation * 20 - 23;
    }
    
    $negative_minutes = $unassigned_time_data['worked_minutes'] > 0 ? 'negative' : '';
    $remaining_minutes = $unassigned_time_data['worked_minutes'] > 0 ? $unassigned_time_data['worked_minutes'] * -1 : 0;
    $formatted_remaining_minutes = get_formatted_remaining_minutes($remaining_minutes);
    $negative_cost = $unassigned_time_data['executed_cost'] > 0 ? 'negative' : '';
    $remaining_cost = $unassigned_time_data['executed_cost'] > 0 ? $unassigned_time_data['executed_cost'] * -1 : 0;
    $formatted_remaining_cost = get_formatted_remaining_value($remaining_cost);
    $negative_price = $unassigned_time_data['executed_price'] > 0 ? 'negative' : '';
    $remaining_price = $unassigned_time_data['executed_price'] > 0 ? $unassigned_time_data['executed_price'] * -1 : 0;
    $formatted_remaining_price = get_formatted_remaining_value($remaining_price);
    $parent_child_class = !$is_task ? 'parentRow' : 'childs';

    // Render summary of unassigned time
    $collapsed = $data_level > 1 ? 'collapsed' : '';
    $open_icon = !$is_pdf_export ? '<img class="parentSwitch" src="./plugins/project_reports/public/assets/images/open.svg" alt="" data-status="closed">' : '';
    echo '<tr class="'.$parent_child_class.' '.$collapsed.'" data-level="'.$data_level.'"><td style="padding-left: '.$indent.'px;" colspan="4">';
        echo $open_icon;
        echo  $row_title;
    echo '</td>
        <td class="right"></td>
        <td class="right">' . format_amount(round($unassigned_time_data['worked_minutes'] / 60, 2), 2). '</td>
        <td class="right '.$negative_minutes.'">' . $formatted_remaining_minutes . '</td>
        <td class="right"></td>
        <td class="right">' . format_money_amount($unassigned_time_data['executed_cost'], $currency_sym) . '</td>
        <td class="right '.$negative_cost.'">' . $formatted_remaining_cost . '</td>
        <td class="right"></td>
        <td class="right">' . format_money_amount($unassigned_time_data['executed_price'], $currency_sym) . '</td>
        <td class="right '.$negative_price.'">' . $formatted_remaining_price . '</td>
        </tr>';
    
    // Render time entries
    $indentation += 1;
    $data_level += 1;
    foreach($time_entries as $time) {
        render_individual_time_entry_for_budget_by_tasks($time, $report_data, $indentation, $data_level, $is_task);
    }
}

function render_individual_time_entry_for_budget_by_tasks($time, $report_data, $indentation, $data_level, $is_task) {
    $currency_sym = config_option('currency_code');
    $indent = $indentation * 20;
    $negative_minutes = $time['worked_minutes'] > 0 ? 'negative' : '';
    $remaining_minutes = $time['worked_minutes'] > 0 ? $time['worked_minutes'] * -1 : 0;
    $formatted_remaining_minutes = get_formatted_remaining_minutes($remaining_minutes);
    $negative_cost = $time['executed_cost'] > 0 ? 'negative' : '';
    $remaining_cost = $time['executed_cost'] > 0 ? $time['executed_cost'] * -1 : 0;
    $formatted_remaining_cost = get_formatted_remaining_value($remaining_cost);
    $negative_price = $time['executed_price'] > 0 ? 'negative' : '';
    $remaining_price = $time['executed_price'] > 0 ? $time['executed_price'] * -1 : 0;
    $formatted_remaining_price = get_formatted_remaining_value($remaining_price);
    $collapsed = $data_level > 1 ? 'collapsed' : '';

    // Render summary of unassigned time
    echo '<tr class="childs '.$collapsed.'" data-level="'.$data_level.'"><td style="padding-left: '.$indent.'px;" class="right">'.format_date($time['date']).'</td>
        <td class="left">';
            echo $time['contact_name'];
    echo '</td>
        <td class="left descriptionWidth" colspan="2">'.$time['description'].'</td>
        <td class="right"></td>
        <td class="right">' . format_amount(round($time['worked_minutes'] / 60, 2), 2). '</td>
        <td class="right '.$negative_minutes.'">'.$formatted_remaining_minutes.'</td>
        <td class="right"></td>
        <td class="right">' . format_money_amount($time['executed_cost'], $currency_sym) . '</td>
        <td class="right '.$negative_cost.'">'.$formatted_remaining_cost.'</td>
        <td class="right"></td>
        <td class="right">' . format_money_amount($time['executed_price'], $currency_sym) . '</td>
        <td class="right '.$negative_price.'">'.$formatted_remaining_price.'</td>
    </tr>';
}

function render_unassigned_actual_expenses_for_budget_by_tasks($project, $report_data, $indentation, $data_level, $is_task = false, $is_phase = false) {
    $is_pdf_export = array_var($report_data, 'exportPDF', false);
    if(!$is_phase) {
        $unassigned_actual_expenses = !$is_task ? $project['unassigned_actual_expenses'] : $project['actual_expenses_data'];
    } else {
        $unassigned_actual_expenses = $project['actual_expenses_data'];
    }
    $actual_expenses = array_var($unassigned_actual_expenses, 'actual_expenses', array());
    if(count($actual_expenses) == 0) {
        return;
    }
   
    $currency_sym = config_option('currency_code');
    $row_title = !$is_task ? lang('actual expenses not budgeted and not linked to tasks') : lang('actual expenses not budgeted, linked to') . ' '. $project['task_name'];
    if(!$is_pdf_export) {
        $indent = $indentation * 20 - 23;
    }
    $negative_cost = $unassigned_actual_expenses['executed_cost'] > 0 ? 'negative' : '';
    $remaining_cost = $unassigned_actual_expenses['executed_cost'] > 0 ? $unassigned_actual_expenses['executed_cost'] * -1 : 0;
    $formatted_remaining_cost = get_formatted_remaining_value($remaining_cost);
    $negative_price = $unassigned_actual_expenses['executed_price'] > 0 ? 'negative' : '';
    $remaining_price = $unassigned_actual_expenses['executed_price'] > 0 ? $unassigned_actual_expenses['executed_price'] * -1 : 0;
    $formatted_remaining_price = get_formatted_remaining_value($remaining_price);
    $collapsed = $data_level > 1 ? 'collapsed' : '';
    $parent_child_class = !$is_task ? 'parentRow' : 'childs';
    $open_icon = !$is_pdf_export ? '<img class="parentSwitch" src="./plugins/project_reports/public/assets/images/open.svg" alt="" data-status="closed">' : '';

    // Render summary of unassigned actual expenses
    echo '<tr class="'.$parent_child_class.' '.$collapsed.'" data-level="'.$data_level.'"><td style="padding-left: '.$indent.'px;" colspan="4">';
        echo $open_icon;
        echo $row_title;
    echo '</td>
        <td class="right"></td>
        <td class="right"></td>
        <td class="right"></td>
        <td class="right"></td>
        <td class="right">' . format_money_amount($unassigned_actual_expenses['executed_cost'], $currency_sym) . '</td>
        <td class="right '.$negative_cost.'">' . $formatted_remaining_cost . '</td>
        <td class="right"></td>
        <td class="right">' . format_money_amount($unassigned_actual_expenses['executed_price'], $currency_sym) . '</td>
        <td class="right '.$negative_price.'">' . $formatted_remaining_price . '</td>
        </tr>';
    
    // Render individual actual expenses
    $indentation += 1;
    $data_level += 1;
    foreach($actual_expenses as $expense) {
        render_individual_actual_expense_for_budget_by_tasks($expense, $report_data, $indentation, $data_level, $is_task);
    }
}

function render_individual_actual_expense_for_budget_by_tasks($expense, $report_data, $indentation, $data_level, $is_task) {
    $currency_sym = config_option('currency_code');
    $indent = $indentation * 20;
    $negative_cost = $expense['executed_cost'] > 0 ? 'negative' : '';
    $remaining_cost = $expense['executed_cost'] > 0 ? $expense['executed_cost'] * -1 : 0;
    $formatted_remaining_cost = get_formatted_remaining_value($remaining_cost);
    $negative_price = $expense['executed_price'] > 0 ? 'negative' : '';
    $remaining_price = $expense['executed_price'] > 0 ? $expense['executed_price'] * -1 : 0;
    $formatted_remaining_price = get_formatted_remaining_value($remaining_price);
    $collapsed = $data_level > 1 ? 'collapsed' : '';

    echo '<tr class="childs '.$collapsed.'" data-level="'.$data_level.'"><td style="padding-left: '.$indent.'px;" class="right">'.format_date($expense['date']).'</td>
        <td class="left">';
            echo $expense['expense_name'];
    echo '</td>
        <td class="left descriptionWidth" colspan="2">'.$expense['description'].'</td>
        <td class="right"></td>
        <td class="right"></td>
        <td class="right"></td>
        <td class="right"></td>
        <td class="right">' . format_money_amount($expense['executed_cost'], $currency_sym) . '</td>
        <td class="right '.$negative_cost.'">' . $formatted_remaining_cost . '</td>
        <td class="right"></td>
        <td class="right">' . format_money_amount($expense['executed_price'], $currency_sym) . '</td>
        <td class="right '.$negative_price.'">' . $formatted_remaining_price . '</td>
    </tr>';
}

function render_unassigned_budgeted_expenses_for_budget_by_tasks($project, $report_data, $indentation, $data_level, $is_task = false, $is_phase = false) {
    $is_pdf_export = array_var($report_data, 'exportPDF', false);
    if(!$is_phase) {
        $unassigned_budgeted_expenses = !$is_task ? $project['unassigned_budgeted_expenses'] : $project['budgeted_expenses_data'];
    } else {
        $unassigned_budgeted_expenses = $project['budgeted_expenses_data'];
    }
    $budgeted_expenses = array_var($unassigned_budgeted_expenses, 'budgeted_expenses', array());
    if(count($budgeted_expenses) == 0) {
        return;
    }
   
    $currency_sym = config_option('currency_code');
    $row_title = !$is_task ? lang('budgeted expenses not linked to tasks') : lang('budgeted expenses linked to') . ' '. $project['task_name'];
    // *** LEC 2023-09-29
    # $negative_cost = $unassigned_budgeted_expenses['remaining_cost'] < 0 ? 'negative' : '';
    # $remaining_cost = get_formatted_remaining_value($unassigned_budgeted_expenses['remaining_cost']);
    # $negative_price = $unassigned_budgeted_expenses['remaining_price'] < 0 ? 'negative' : '';
    # $remaining_price = get_formatted_remaining_value($unassigned_budgeted_expenses['remaining_price']);
    $remaining_cost = isset( $unassigned_budgeted_expenses['remaining_cost']) ? get_formatted_remaining_value($unassigned_budgeted_expenses['remaining_cost']): 0;
    $negative_cost = $remaining_cost < 0 ? 'negative' : '';
    $remaining_price = isset( $unassigned_budgeted_expenses['remaining_price']) ? get_formatted_remaining_value($unassigned_budgeted_expenses['remaining_price']): 0;
    $negative_price = $remaining_price < 0 ? 'negative' : '';
    // end
    if(!$is_pdf_export) {
        $indent = $indentation * 20 - 23;
    }
    $collapsed = $data_level > 1 ? 'collapsed' : '';
    $parent_child_class = !$is_task ? 'parentRow' : 'childs';
    $open_icon = !$is_pdf_export ? '<img class="parentSwitch" src="./plugins/project_reports/public/assets/images/open.svg" alt="" data-status="closed">' : '';

    // Render summary of unassigned budgeted expenses
    echo '<tr class="'.$parent_child_class.' '.$collapsed.'" data-level="'.$data_level.'"><td style="padding-left: '.$indent.'px;" colspan="4">';
        echo $open_icon;
        echo $row_title;
    echo '</td>
        <td class="right"></td>
        <td class="right"></td>
        <td class="right"></td>
        <td class="right">' . format_money_amount($unassigned_budgeted_expenses['estimated_cost'], $currency_sym) . '</td>
        <td class="right">' . format_money_amount($unassigned_budgeted_expenses['executed_cost'], $currency_sym) . '</td>
        <td class="right '.$negative_cost.'">' . $remaining_cost . '</td>
        <td class="right">' . format_money_amount($unassigned_budgeted_expenses['estimated_price'], $currency_sym) . '</td>
        <td class="right">' . format_money_amount($unassigned_budgeted_expenses['executed_price'], $currency_sym) . '</td>
        <td class="right '.$negative_price.'">' . $remaining_price . '</td>
        </tr>';

    // Render individual budgeted expenses
    $indentation += 1;
    $data_level += 1;
    foreach($budgeted_expenses as $expense) {
        render_individual_budgeted_expense_for_budget_by_tasks($expense, $report_data, $indentation, $data_level, $is_task);
    }
}

function render_individual_budgeted_expense_for_budget_by_tasks($expense, $report_data, $indentation, $data_level, $is_task) {
    $is_pdf_export = array_var($report_data, 'exportPDF', false);
    $currency_sym = config_option('currency_code');
    $indent = $indentation * 20;
    $negative_cost = $expense['remaining_cost'] < 0 ? 'negative' : '';
    $remaining_cost = get_formatted_remaining_value($expense['remaining_cost']);
    $negative_price = $expense['remaining_price'] < 0 ? 'negative' : '';
    $remaining_price = get_formatted_remaining_value($expense['remaining_price']);
    $collapsed = $data_level > 1 ? 'collapsed' : '';
    $actual_expenses = array_var($expense, 'actual_expenses', array());

    if(count($actual_expenses) > 0 && !$is_pdf_export) {
        $exp_icon = '<img class="parentSwitch" src="./plugins/project_reports/public/assets/images/open.svg" data-status="closed" alt="">';
        $indent = $indent - 23;
    }

    echo '<tr class="childs '.$collapsed.'" data-level="'.$data_level.'"><td style="padding-left: '.$indent.'px;" class="left" colspan="1">'; 
        echo isset($exp_icon) ? $exp_icon : "";
    
    echo format_date($expense['date']).'</td>
            <td class="left" colspan="3">';
                echo $expense['expense_name'];
    echo '</td>
    <td class="right"></td>
    <td class="right"></td>
    <td class="right"></td>
    <td class="right">' . format_money_amount($expense['estimated_cost'], $currency_sym) . '</td>
    <td class="right">' . format_money_amount($expense['executed_cost'], $currency_sym) . '</td>
    <td class="right '.$negative_cost.'">' . $remaining_cost . '</td>
    <td class="right">' . format_money_amount($expense['estimated_price'], $currency_sym) . '</td>
    <td class="right">' . format_money_amount($expense['executed_price'], $currency_sym) . '</td>
    <td class="right '.$negative_price.'">' . $remaining_price . '</td>
    </tr>';

    // Render individual actual expenses
    $indentation += 1;
    $data_level += 1;
    foreach($actual_expenses as $act_expense) {
        render_individual_actual_expense_for_budget_by_tasks($act_expense, $report_data, $indentation, $data_level, $is_task);
    }
}


function render_script_for_budget_by_tasks_report() {
    echo '<script>
        og.render_script_for_budget_by_tasks_report();
    </script>';
}

function render_project_header_with_totals_for_budget_by_tasks($project, $report_data, $indentation, $is_parent, $is_phase = false) {
    $indent = $indentation * 20;
    $currency_sym = config_option('currency_code');
    if($is_phase) {
        $header_class = 'header_1';
    } else {
        $header_class = $is_parent ? 'header_0' : 'header_2';
    }   

    $title = $is_phase ? $project['phase_name'] : $project['project_name'];
    $project = $is_phase ? $project['totals'] : $project;
    $remaining_minutes = get_formatted_remaining_minutes($project['remaining_minutes']);
    $remaining_cost = get_formatted_remaining_value($project['remaining_cost']);
    $remaining_price = get_formatted_remaining_value($project['remaining_price']);
    

 
    // build the project name row
    echo '<tbody class="titles">
                <tr>
                    <td class="'.$header_class.'" colspan="2" style="padding-left: '.$indent.'px;">'.$title.'</td>
                    <td class="center '.$header_class.'">'.round($project['percent_completed']).' %</td>
                    <td class="center '.$header_class.'"></td>
                    <td class="right '.$header_class.'">' . format_amount(round($project['estimated_minutes'] / 60, 2), 2) . '</td>
                    <td class="right '.$header_class.'">' . format_amount(round($project['worked_minutes'] / 60, 2), 2) . '</td>
                    <td class="right '.$header_class.'">' . $remaining_minutes . '</td>
                    <td class="right '.$header_class.'">' . format_money_amount($project['estimated_cost'], $currency_sym) . '</td>
                    <td class="right '.$header_class.'">' . format_money_amount($project['executed_cost'], $currency_sym) . '</td>
                    <td class="right '.$header_class.'">' . $remaining_cost . '</td>
                    <td class="right '.$header_class.'">' . format_money_amount($project['estimated_price'], $currency_sym) . '</td>
                    <td class="right '.$header_class.'">' . format_money_amount($project['executed_price'], $currency_sym) . '</td>
                    <td class="right '.$header_class.'">' . $remaining_price . '</td>
                </tr>
            </tbody>';
}

function get_formatted_remaining_value($remaining_value) {
    $currency_sym = config_option('currency_code');
    $corrected_remaining_value = $remaining_value < 0 ? $remaining_value * -1 : $remaining_value;
    $formatted_value = $remaining_value < 0 ? '('.format_money_amount($corrected_remaining_value, $currency_sym).')' : format_money_amount($corrected_remaining_value, $currency_sym);

    return $formatted_value;
}

function get_formatted_remaining_minutes($remaining_minutes) {
    $corrected_remaining_minutes = $remaining_minutes < 0 ? $remaining_minutes * -1 : $remaining_minutes;
    $formatted_value = $remaining_minutes < 0 ? '('.format_amount(round($corrected_remaining_minutes / 60, 2), 2).')' : format_amount(round($corrected_remaining_minutes / 60, 2), 2);

    return $formatted_value;
}