<?php

// Render report title
echo '<div class="project-report-title">'.$title.'</div>';

// Render date of the report
$today_dtv = DateTimeValueLib::now();
$today_date = format_date($today_dtv);
echo '<div class="project-report-info" style="float: right;">'.lang('report date').': '.$today_date.'</div>';

echo render_all_collections_information_report($data, $report_data);

function render_all_collections_information_report($data, $report_data){
    // Get custom property 'project_manager'
    $project_ot = ObjectTypes::findByName('project');
    $cp_project_manager = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="project_manager"'));
    $contacts = $data['contacts'];
    foreach($contacts as $contact_id => $contact_info){
        echo render_contact_info($contact_info, $cp_project_manager, $report_data);
    }
    

    echo render_report_totals($data['totals'], $data['distribution'], $report_data);
}

function render_contact_info($contact_info, $cp_project_manager, $report_data){
    $group_by = array_var($report_data, 'group_by', 'project');
    $has_selected_pm_id = array_var($report_data, 'has_selected_pm_id');
    echo '<table class="aging-invoices-table" style="zoom: 0.9;">';
    $project_columns = array_var($report_data, 'project_columns', array('client_name', 'full_project_name'));
    $project_ot = ObjectTypes::findByName('project');

    // HEADER
    $header_columns_1 = array('invoice', 'client_name');
    foreach($project_columns as $c_name){
        $header_columns_1[] = $c_name;
    }

    $header_columns_2 = array('payment_received', 'due_date', 'paid_amount', 'balance');
    if(isset($report_data['include_current_invoices'])){
        $header_columns_2[] = 'current not due';
    }
    $header_columns_3 = array('1-30', '31-60', '61-90', '91-120', 'over 120');
    $header_columns = array_merge($header_columns_1, $header_columns_2, $header_columns_3);
    $header_colspan = count($header_columns);
    echo '<thead>';
    echo '<tr>';
    foreach($header_columns as $c_name){
        if($c_name == 'client_name') {
            $client_dim = Dimensions::instance()->findByCode('customers');
            $col_name = $client_dim->getName();
        } else if($c_name == 'full_project_name') {
            $project_dim = Dimensions::instance()->findByCode('customer_project');
            $col_name = $project_dim->getName();
        } else if($c_name == 'project_phase') {
            $project_phase_dim = Dimensions::instance()->findByCode('project_phases');
            $col_name = $project_phase_dim->getName();
        } else if($c_name == 'project_manager') {
            $col_name = $cp_project_manager->getName();
        } else if($c_name == 'date_received') {
            $cp_report_sent = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="date_received"'));
            $col_name = $cp_report_sent->getName();
        } else if($c_name == 'billing_contact') {
            $cp_billing_contact = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="billing_contact"'));
            $col_name = $cp_billing_contact->getName();
        } else if($c_name == 'referred_by') {
            $cp_invoice_sent = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="referred_by"'));
            $col_name = $cp_invoice_sent->getName();
        } else if($c_name == 'project_location') {
            $cp_invoice_sent = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="project_location"'));
            $col_name = $cp_invoice_sent->getName();
        } else if($c_name == 'last_follow_up_date') {
            $cp_invoice_sent = CustomProperties::instance()->findOne(array('conditions' => 'object_type_id='.$project_ot->getId().' AND `code`="last_follow_up_date"'));
            $col_name = $cp_invoice_sent->getName();
        } else {
            $col_name = lang($c_name);
        }
        echo '<th>'.$col_name.'</th>';
    }
    echo '</tr></thead>';

    // BODY
    // Project Manager name
    echo '<tbody>';

    // Render projects
    $project_and_invoices_info = $contact_info['project_and_invoices_info'];
    foreach($project_and_invoices_info['projects'] as $project_id => $project_info){
        echo render_project_info($project_info, $report_data);
    }

    // Render Contact Totals
    $total_name = $contact_info['contact_name'];
    if(!$has_selected_pm_id && $group_by == 'report_sent') {
       $total_name = $report_sent_date;
    }

    echo render_contact_totals($total_name, $project_and_invoices_info['contact_totals'], $report_data);


    echo '</tbody></table>';

}

function render_project_info($project_info, $report_data){
    // Client and Project name row
    $project_name = $project_info['project_info']['full_project_name'];
    $billing_contact_data = array_var($project_info['project_info'], 'billing_contact_info', null);
    $has_selected_pm_id = array_var($report_data, 'has_selected_pm_id');
    $group_by = array_var($report_data, 'group_by', 'project');
    $header_colspan = 9;
    if(isset($report_data['include_current_invoices'])) {
        $header_colspan++;
    }

	// Render invoices
    foreach($project_info['invoices'] as $invoice){
        echo render_invoice_info($invoice, $project_info, $report_data);
    }
}

function render_invoice_info($invoice, $project_info, $report_data){
    $project_columns = array_var($report_data, 'project_columns', array('full_project_name'));
    $currency = Currencies::instance()->getCurrency($invoice['currency_id']);
    $currency_sym = $currency instanceof Currency ? $currency->getSymbol() : config_option('currency_code');

    $current_time_val = $invoice['current_time'] > 0 ? format_money_amount($invoice['current_time'], $currency_sym) : '';
    $first_month_val = $invoice['first_month'] > 0 ? format_money_amount($invoice['first_month'], $currency_sym) : '';
    $second_month_val = $invoice['second_month'] > 0 ? format_money_amount($invoice['second_month'], $currency_sym) : '';
    $third_month_val = $invoice['third_month'] > 0 ? format_money_amount($invoice['third_month'], $currency_sym) : '';
    $fourth_month_val = $invoice['fourth_month'] > 0 ? format_money_amount($invoice['fourth_month'], $currency_sym) : '';
    $greater_than_four_month_val = $invoice['greater_than_four_month'] > 0 ? format_money_amount($invoice['greater_than_four_month'], $currency_sym) : '';

    // Build invoice row
    echo '<tr>';
    echo '<td>'.$invoice['invoice_number'].'</td>';
    echo '<td>'.$invoice['client_name'].'</td>';
    foreach($project_columns as $c_name){
        echo '<td>'.$project_info['project_info'][$c_name].'</td>';
    }
    if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
        echo '<td>'.$invoice['payment_received'].'</td>';
    }
    echo '<td>'.$invoice['date'].'</td>';
    echo '<td class="amount">'.format_money_amount($invoice['paid_amount'], $currency_sym).'</td>';
    echo '<td class="amount">'.format_money_amount($invoice['total'], $currency_sym).'</td>';
    if(isset($report_data['include_current_invoices'])) {
            echo '<td class="amount">'.$current_time_val.'</td>';
    }
    echo '<td class="amount">'. $first_month_val .'</td>';
    echo '<td class="amount">'. $second_month_val .'</td>';
    echo '<td class="amount">'. $third_month_val .'</td>';
    echo '<td class="amount">'. $fourth_month_val .'</td>';
    echo '<td class="amount">'. $greater_than_four_month_val .'</td>';
    echo '</tr>';
}

function render_project_total($project_info, $report_data){
    $project_columns = array_var($report_data, 'project_columns', array('full_project_name'));
    $project_totals = $project_info['project_totals'];
    $totals = build_totals_using_multicurrencies($project_totals);
    $total_colspan = 3;
    if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
        $total_colspan++;
    }
    $total_colspan += count($project_columns);
    
    // Build project totals row
    echo '<tr class="bold">';
    echo '<td colspan="' . $total_colspan . '">'.lang('total for').' '.$project_info['project_info']['full_project_name'].'</td>';
    echo '<td class="amount">'.$totals['paid_amount'].'</td>';
    echo '<td class="amount">'.$totals['total'].'</td>';
    if(isset($report_data['include_current_invoices'])) {
        echo '<td class="amount">'.$totals['current_time'].'</td>';
    }
    echo '<td class="amount">'.$totals['first_month'].'</td>';
    echo '<td class="amount">'.$totals['second_month'].'</td>';
    echo '<td class="amount">'.$totals['third_month'].'</td>';
    echo '<td class="amount">'.$totals['fourth_month'].'</td>';
    echo '<td class="amount">'.$totals['greater_than_four_month'].'</td>';
    echo '</tr>';   
}

function render_contact_totals($contact_name, $contact_totals, $report_data){
    $project_columns = array_var($report_data, 'project_columns', array('full_project_name'));
    $totals = build_totals_using_multicurrencies($contact_totals);
    $total_colspan = 3;
    if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
        $total_colspan++;
    }
    $total_colspan += count($project_columns);
    // Build project totals row
    echo '<tr class="contact-totals">';
    echo '<td colspan="' . $total_colspan . '">'.lang('totals').'</td>';
    echo '<td class="amount">'.$totals['paid_amount'].'</td>';
    echo '<td class="amount">'.$totals['total'].'</td>';
    if(isset($report_data['include_current_invoices'])) {
        echo '<td class="amount">'.$totals['current_time'].'</td>';
    }
    echo '<td class="amount">'.$totals['first_month'].'</td>';
    echo '<td class="amount">'.$totals['second_month'].'</td>';
    echo '<td class="amount">'.$totals['third_month'].'</td>';
    echo '<td class="amount">'.$totals['fourth_month'].'</td>';
    echo '<td class="amount">'.$totals['greater_than_four_month'].'</td>';
    echo '</tr>';
}

function render_report_totals($totals, $distribution_totals, $report_data){
    $project_columns = array_var($report_data, 'project_columns', array('full_project_name'));
    $formatted_totals = build_totals_using_multicurrencies($totals);
    $distrib_totals = build_distribution_totals_using_multicurrencies($distribution_totals);
    $total_colspan = 3;
    if(Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
        $total_colspan++;
    }
    $total_colspan += count($project_columns);
    echo '<table class="aging-invoices-table" style="zoom: 0.9;">';

    // Header
    echo '<thead>';
    echo '<tr class="table-name--totals"><td colspan="9">'.lang('totals').'</td></tr>';
    echo '<tr><th colspan="' . $total_colspan . '"></th>';
    $header_columns_1 = array('paid_amount', 'balance');
    if(isset($report_data['include_current_invoices'])) {
        $header_columns_1[] = 'current not due';
    }
    $header_columns_2 = array('1-30', '31-60', '61-90', '91-120', 'over 120');
    $header_columns = array_merge($header_columns_1, $header_columns_2);
    foreach($header_columns as $c_name){
        echo '<th>'.lang($c_name).'</th>';
    }
    echo '</tr></thead>';

    // Body
    echo '<tbody>';

    // Build totals row
    echo '<tr>';
    echo '<td class="bold" colspan="' . $total_colspan . '">'.lang('final totals').'</td>';
    echo '<td class="amount">'.$formatted_totals['paid_amount'].'</td>';
    echo '<td class="amount">'.$formatted_totals['total'].'</td>';
    if(isset($report_data['include_current_invoices'])) {
        echo '<td class="amount">'.$formatted_totals['current_time'].'</td>';
    }
    echo '<td class="amount">'.$formatted_totals['first_month'].'</td>';
    echo '<td class="amount">'.$formatted_totals['second_month'].'</td>';
    echo '<td class="amount">'.$formatted_totals['third_month'].'</td>';
    echo '<td class="amount">'.$formatted_totals['fourth_month'].'</td>';
    echo '<td class="amount">'.$formatted_totals['greater_than_four_month'].'</td>';
    echo '</tr>';

    // Build distribution row
    $distrib_colspan = $total_colspan + 1;
    if($distribution_totals['currencies']){
        echo '<tr>';
        echo '<td class="bold" colspan="' . $distrib_colspan . '">'.lang('distribution').'</td>';
        echo '<td class="amount">'.$distrib_totals['total'].'</td>';
        if(isset($report_data['include_current_invoices'])) {
            echo '<td class="amount">'.$distrib_totals['current_time'].'</td>';
        }
        echo '<td class="amount">'.$distrib_totals['first_month'].'</td>';
        echo '<td class="amount">'.$distrib_totals['second_month'].'</td>';
        echo '<td class="amount">'.$distrib_totals['third_month'].'</td>';
        echo '<td class="amount">'.$distrib_totals['fourth_month'].'</td>';
        echo '<td class="amount">'.$distrib_totals['greater_than_four_month'].'</td>';
        echo '</tr>';
        
    }


    echo '</table>';

}

function build_totals_using_multicurrencies($totals) {
     $total = '';
     $paid_amount = '';
     $current_time = '';
     $first_month = '';
     $second_month = '';
     $third_month = '';
     $fourth_month = '';
     $greater_than_four_month = '';
     foreach($totals['currencies'] as $currency_id => $currency_totals){
         $currency = Currencies::instance()->getCurrency($currency_id);
         $currency_sym = $currency instanceof Currency ? $currency->getSymbol() : config_option('currency_code');
         $total .= '<div>' . format_money_amount($currency_totals['total'], $currency_sym) . '</div>';
         $paid_amount .= '<div>' . format_money_amount($currency_totals['paid_amount'], $currency_sym) . '</div>';
         $current_time .= '<div>' . format_money_amount($currency_totals['current_time'], $currency_sym) . '</div>';
         $first_month .= '<div>' . format_money_amount($currency_totals['first_month'], $currency_sym) . '</div>';
         $second_month .= '<div>' . format_money_amount($currency_totals['second_month'], $currency_sym) . '</div>';
         $third_month .= '<div>' . format_money_amount($currency_totals['third_month'], $currency_sym) . '</div>';
         $fourth_month .= '<div>' . format_money_amount($currency_totals['fourth_month'], $currency_sym) . '</div>';
         $greater_than_four_month .= '<div>' . format_money_amount($currency_totals['greater_than_four_month'], $currency_sym) . '</div>';
     }

     $result = array(
        'total' => $total,
        'paid_amount' => $paid_amount,
        'current_time' => $current_time,
        'first_month' => $first_month,
        'second_month' => $second_month,
        'third_month' => $third_month,
        'fourth_month' => $fourth_month,
        'greater_than_four_month' => $greater_than_four_month
     );

     return $result;
}

function build_distribution_totals_using_multicurrencies($totals){
    $total = '';
     $current_time = '';
     $first_month = '';
     $second_month = '';
     $third_month = '';
     $fourth_month = '';
     $greater_than_four_month = '';
     foreach($totals['currencies'] as $currency_id => $currency_totals){  
        $total .= '<div>' . $currency_totals['total'].' %' . '</div>';
        $current_time .= '<div>' . $currency_totals['current_time'].' %' . '</div>';
        $first_month .= '<div>' . $currency_totals['first_month'].' %' . '</div>';
        $second_month .= '<div>' . $currency_totals['second_month'].' %' . '</div>';
        $third_month .= '<div>' . $currency_totals['third_month'].' %' . '</div>';
        $fourth_month .= '<div>' . $currency_totals['fourth_month'].' %' . '</div>';
        $greater_than_four_month .= '<div>' . $currency_totals['greater_than_four_month'].' %' . '</div>';
     }

     $result = array(
        'total' => $total,
        'current_time' => $current_time,
        'first_month' => $first_month,
        'second_month' => $second_month,
        'third_month' => $third_month,
        'fourth_month' => $fourth_month,
        'greater_than_four_month' => $greater_than_four_month
     );

     return $result;
}
