<?php

Env::useHelper('view_functions', 'project_reports');

$report_data = array_var($_SESSION, 'contract_status_report_data');
$report_data['is_pdf_export'] = isset($is_pdf_export) ? $is_pdf_export : false;

$from_date_str = format_date($report_data['start_date']);
$to_date_str = format_date($report_data['end_date']);

// Render report title
echo '<div class="project-report-title">'.$title.'</div>';

// Render date of the report
$today_dtv = DateTimeValueLib::now();
$today_date = format_date($today_dtv);
echo '<div class="project-report-info" style="float: right;">'.lang('report date').': '.$today_date.'</div>';

echo '<div class="project-report-info" style="margin-bottom: 20px;">'.lang('showing data from x to y', $from_date_str, $to_date_str).'</div>';

echo '<div class="clear"></div>';

function render_contract_status_report($data, $report_data) {
    $is_pdf_export = array_var($report_data, 'is_pdf_export', false);
    $table_zoom = $is_pdf_export ? 'zoom: 0.8;' : '';
    // Start table
    echo '<table class="budgetReport" style="width: 100%;'.$table_zoom.'">';
    render_main_header_contract_status_report($data, $report_data);
    render_overall_totals_contract_status_report($data, $report_data);
    $projects = array_var($data, 'projects', array());
    echo '<tbody>';
    foreach($projects as $proj_data) {
        render_project_contract_status_report($proj_data, $data);
    }   
    echo '</tbody>';
    // Close table
    echo '</table>';
}

function render_project_contract_status_report($proj_data, $data) {
    $columns = array_var($data, 'columns', array());
    $currency_sym = config_option('currency_code');
    $proj_name = $proj_data['name'];
    $name_columns = array('name', 'contract_type', 'project_manager', 'project_phase', 'project_type');
    $time_columns = array('executed_labor_minutes');

    // Start row
    echo '<tr class="parentRow">';
    echo '<td class="left" colspan="4">'.$proj_name.'</td>';
            
    foreach($columns as $column) {
        if($column == 'name') continue;
        $class = 'left';
        if(in_array($column, $time_columns)) {
            $class = 'right';
            $val = format_amount($proj_data[$column]/60);
        } else if(in_array($column, $name_columns)) {
            $val = $proj_data[$column];
        } else {
            $class = 'right';
            $val = format_money_amount($proj_data[$column], $currency_sym);
        }
        
        echo '<td class="'.$class.'" colspan="2">'.$val.'</td>';
    }
      
    // Close row
    echo '</tr>';
}

function render_main_header_contract_status_report($data, $report_data) {
    $columns = array_var($data, 'columns', array());

    // Start header
    
    echo '<thead class="titels"><tr>
            <th class="left bold header_1 min-160" colspan="4"></th>';
    foreach($columns as $column) {
        if($column == 'name') continue;
        if($column == 'contract_type') {
            $column_name = $data['contract_type_cp_name'];
        } else if($column == 'project_manager') {
            $column_name = $data['project_manager_dim_name'];
        } else if($column == 'project_phase') {
            $column_name = $data['project_phase_dim_name'];
        } else if($column == 'project_type') {
            $column_name = $data['project_type_dim_name'];
        } else if($column == 'budget_total') {
            $column_name = $data['budget_total_cp_name'];
        } else if ($column == 'revised_budget_total') {
            $column_name = $data['revised_budget_total_cp_name'];
        } else if ($column == 'executed_cost') {
            $column_name = lang('executed cost');
        } else if ($column == 'invoiced_subtotal') {
            $column_name = lang('invoiced');
        } else if ($column == 'invoiced_paid') {
            $column_name = lang('invoiced paid');
        } else if ($column == 'remaining_budgeted_vs_contract') {
            $column_name = lang('remaining budgeted vs contract contract_status_report');
        } else {
            $column_name = lang($column);
        }

        echo '<th class="center bold header_1 min-80" colspan="2">'.$column_name.'</th>';
    }

    // Close header
    echo '</tr></thead>';
}

function render_overall_totals_contract_status_report($data, $report_data) {
    $columns = array_var($data, 'columns', array());
    $totals = array_var($data, 'totals', array());
    $currency_sym = config_option('currency_code');
    $name_columns = array('name', 'contract_type', 'project_manager', 'project_phase', 'project_type');
    $time_columns = array('executed_labor_minutes');

    // Start totals
    echo '<tbody><tr class="titles">
            <td class="left header_0" colspan="4">'.lang('totals').'</td>';
    foreach($columns as $column) {
        if($column == 'name') continue;
        if(in_array($column, $time_columns)) {
            $val = format_amount($totals[$column]/60);
        } else if(in_array($column, $name_columns)) {
            $val = '';
        } else {
            $val = format_money_amount($totals[$column], $currency_sym);
        }
        echo '<td class="right header_0" colspan="2">'.$val.'</td>';
    }

    // Close totals
    echo '</tr></tbody>';
}

render_contract_status_report($data, $report_data);
