<?php

Env::useHelper('view_functions', 'project_reports');

$report_data = array_var($_SESSION, 'gross_margin_report_data');

$from_date_str = format_date($report_data['start_date']);
$to_date_str = format_date($report_data['end_date']);

// Render report title
echo '<div class="project-report-title">'.$title.'</div>';

// Render date of the report
$today_dtv = DateTimeValueLib::now();
$today_date = format_date($today_dtv);
echo '<div class="project-report-info" style="float: right;">'.lang('report date').': '.$today_date.'</div>';

echo '<div class="project-report-info" style="margin-bottom: 20px;">'.lang('showing data from x to y', $from_date_str, $to_date_str).'</div>';

echo '<div class="clear"></div>';

function render_gross_margin_report($data, $report_data) {
    echo '<table class="budgetReport" style="width: 100%">';
    render_header_for_gross_margin_report($data, $report_data);
    render_fin_data_for_gross_margin_report($data, $report_data);
    echo '</table>';
}

function render_header_for_gross_margin_report($data, $report_data) {
    $months_range = array_var($data, 'months_between_dates', array());
    $current_colspan = array_var($data, 'count_current_months', 0);
    $forecasted_colspan = array_var($data, 'count_forecasted_months', 0);
    $mapped_months = array_var($data, 'mapped_months', array());
    $columns = array_var($data, 'columns', array());

    echo '<thead>';
    // Main header
    echo '<tr class="header_table">
            <th colspan="1" scope="colgroup"></th>';
    if ($current_colspan > 0) {
        echo '<th colspan="'.$current_colspan.'" scope="colgroup"></th>';
    }
    if ($forecasted_colspan > 0) {
        echo '<th class="forecasted_level center" colspan="'.$forecasted_colspan.'" scope="colgroup">'.lang('forecasted').'</th>';
    }
    echo '<th class="total_level center" colspan="1" scope="colgroup">'.lang('total').'</th></tr>';

    // Sub header
    echo '<tr class="sub_header_table"><th></th>';
    foreach($months_range as $column) {
        $column_arr = explode('-', $column);
        $year = $column_arr[0];
        $month = $column_arr[1];
        $month_str = ucfirst(lang('month '. intval($month)));
        $month_abbr = substr($month_str, 0, 3);
        $class = $mapped_months[$column] == 'forecasted' ? 'forecasted_level_text' : '';

        echo '<th class="'.$class.'">'.$month_abbr . ' ' . $year.'</th>';
    }
    echo '<th></th></tr>';
    echo '</thead>';
}

function render_fin_data_for_gross_margin_report($data, $report_data) {
    $actual_revenue_type = array_var($report_data, 'revenue', 'earned_value');
    $mapped_months = array_var($data, 'mapped_months', array());
    $fin_types = array('estimated_revenue');
    $fin_types[] = $actual_revenue_type == 'earned_value' ? 'earned_value' : 'invoiced';
    $rest_fin_types = array('estimated_costs', 'actual_costs', 'gross_profit_estimated', 'gross_profit_actual', 'gross_margin_estimated', 'gross_margin_actual');
    $fin_types = array_merge($fin_types, $rest_fin_types);

    echo '<tbody>';
    $currency_sym = config_option('currency_code');
    foreach($fin_types as $fin_type) {
        $fin_type_class = get_fin_type_class($fin_type);
        echo '<tr class="parentRow">';
        echo '<td class="left '.$fin_type_class.'">'.lang($fin_type.' gross_margin').'</td>';
        $financial_val_class = 'right '.$fin_type_class;
        $columns = array_var($data, 'months_between_dates', array());
        foreach($columns as $column) {
            $is_forecasted = $mapped_months[$column] == 'forecasted';
            $bold_class = $is_forecasted ? 'bold_value' : '';
            $class = $financial_val_class . ' ' . $bold_class;
            $amount = array_var($data['fin_data'][$fin_type]['months'], $column, 0);
            if($fin_type == 'gross_margin_estimated' || $fin_type == 'gross_margin_actual') {
                $val = format_amount($amount) . ' %';
            } else {
                $val = $amount != 0 ? format_money_amount($amount, $currency_sym) : '';
            }
            //$val = $amount != 0 ? format_money_amount($amount, $currency_sym) : '';
            echo '<td class="'.$class.'">'.$val.'</td>';
        }
        $total_amount = array_var($data['fin_data'][$fin_type], 'total', 0);
        if($fin_type == 'gross_margin_estimated' || $fin_type == 'gross_margin_actual') {
            $total_val = format_amount($total_amount) . ' %';
        } else {
            $total_val = $total_amount != 0 ? format_money_amount($total_amount, $currency_sym) : '';
        }

        echo '<td class="'.$financial_val_class.'">'.$total_val.'</td>';
        echo '</tr>';
    }
    echo '</tbody>';

}

function get_fin_type_class($fin_type) {
    switch ($fin_type) {
        case 'estimated_revenue':
        case 'earned_value':
        case 'invoiced':
            return 'gross_level_1';
        case 'estimated_costs':
        case 'actual_costs':
            return 'gross_level_2';
        case 'gross_profit_estimated':
        case 'gross_profit_actual':
            return 'gross_level_3';
        case 'gross_margin_estimated':
        case 'gross_margin_actual':
            return 'gross_level_4';
        default:
            return '';
    }
}


echo '<link rel="stylesheet" href="./plugins/project_reports/public/assets/css/gross_margin.css">';
echo '<div class="grossMargin">';
render_gross_margin_report($data, $report_data);
echo '</div>';

