<?php
// date of the report
$today_dtv = DateTimeValueLib::now();
$today_date = format_date($today_dtv);
$is_pdf_export = $is_pdf_export ?? false;
?>

<div class="project-report-title"><?php echo $title ?></div>
<div class="project-report-info" style="float: right;"><?php echo lang('report date').': '.$today_date?></div>
<div class="clear"></div>

<?php

render_project_earnings_data($data, $is_pdf_export);

function render_project_earnings_data($data, $is_pdf_export) {
	$table_zoom = $is_pdf_export ? 'zoom: 0.8;' : '';
	// columns and report data
	$columns = array_var($data, 'columns');
	$project_earnings_report_data = array_var($data, 'report_data');

	echo '<div style="'.$table_zoom.'">';

	// render data for each project
	if( isset( $project_earnings_report_data['contacts'])){  // *** LC 2023-09-29
		foreach ($project_earnings_report_data['contacts'] as $pm_data) {
			render_project_earnings_pm_data($pm_data, $columns);
		}
		// render totals table
		render_project_earnings_report_grand_totals($project_earnings_report_data, $columns);
	}
	echo '</div>';
	
}

function render_project_earnings_report_grand_totals($project_earnings_report_data, $columns) {
	$project_ot_id = ObjectTypes::findByName('project')->getId();
	$contract_amount_cp = CustomProperties::instance()->getCustomPropertyByCode($project_ot_id, 'budget_total');

	echo '<table class="project-jdt-report-table project-earnings-table">';

    // HEADER - contact name
    echo '<tr class="table-name"><td colspan="'.(count($columns) + 1).'">'.lang('total').'</td></tr>';
	// column headers
    echo '<tr>';
    foreach($columns as $c_name){
		$colspan = 1;
		$col_text = lang($c_name);
		if ($c_name == 'name'){
			$colspan = 2;
			$col_text = lang('project');
		} else if($c_name == 'total_budget' && $contract_amount_cp instanceof CustomProperty){
			$col_text = $contract_amount_cp->getName();
		}
        echo '<th colspan="'.$colspan.'">'. $col_text .'</th>';
    }
    echo '</tr></thead>';

    // BODY
    echo '<tbody>';

	render_project_earnings_report_totals($project_earnings_report_data, $columns);

	echo '</tbody></table>';
}



function render_project_earnings_pm_data($pm_data, $columns) {
	
	echo '<table class="project-jdt-report-table project-earnings-table">';

	$project_ot_id = ObjectTypes::findByName('project')->getId();
	$contract_amount_cp = CustomProperties::instance()->getCustomPropertyByCode($project_ot_id, 'budget_total');
    // HEADER - contact name
    echo '<tr class="table-name"><td colspan="'.(count($columns) + 1).'">'.$pm_data['name'].'</td></tr>';
	// column headers
    echo '<tr>';
    foreach($columns as $c_name){
		$colspan = 1;
		$col_text = lang($c_name);
		if ($c_name == 'name'){
			$colspan = 2;
			$col_text = lang('project');
		} else if($c_name == 'total_budget' && $contract_amount_cp instanceof CustomProperty){
			$col_text = $contract_amount_cp->getName();
		}

        echo '<th colspan="'.$colspan.'">'. $col_text .'</th>';
    }
    echo '</tr></thead>';

    // BODY
    echo '<tbody>';

	// projects table
	if (count($pm_data['projects']) > 0) {
		render_project_earnings_pm_projects($pm_data, $columns);
	}

    // PM Totals
	render_project_earnings_report_totals($pm_data, $columns, 'table-name');

    echo '</tbody></table>';
}



function render_project_earnings_pm_projects($pm_data, $columns) {
	
	foreach ($pm_data['projects'] as $project_data) {

		$currency = isset($project_data['currency_id']) ? Currencies::instance()->getCurrency($project_data['currency_id']) : null;
		$currency_sym = $currency && $currency instanceof Currency ? $currency->getSymbol() : config_option('currency_code');
		echo '<tr>';
		foreach ($columns as $col) {
			$colspan = 1;
			$cell_class = '';
			$cell_value = '';
			switch ($col) {
				case 'name':
					$cell_class = 'name';
					$colspan = 2;
					$cell_value = $project_data['name'];
					break;
				case 'project_progress':
				case 'revenue_vs_budget':
				case 'remaining_budgeted_vs_contract':
					$cell_class = 'date';
					$cell_value = $project_data[$col]." %";
					break;
				case 'effective_multiplier':
					$cell_class = 'amount';
					$cell_value = format_amount($project_data[$col]);
					break;
				default: 
					$cell_class = 'amount';
					$cell_value = format_money_amount($project_data[$col], $currency_sym);
					break;
			}
	
			echo '<td colspan="'.$colspan.'" class="'.$cell_class.'">'.$cell_value.'</td>';
		}
		echo '</tr>';

	}

}



function render_project_earnings_report_totals($group_data, $columns, $row_cls='', $show_hours=true, $totals_key='totals', $label=null) {

	$totals = $group_data[$totals_key];

	$currency = isset($totals['currency_id']) && Currencies::instance()->getCurrency($totals['currency_id']);
	$currency_sym = $currency instanceof Currency ? $currency->getSymbol() : config_option('currency_code');

	if (!$label) $label = isset($group_data['name']) ? $group_data['name'] : "";

	echo '<tr class="total '.$row_cls.'">';
	foreach ($columns as $col) {
		$colspan = 1;
		$cell_class = '';
		$cell_value = '';
		switch ($col) {
			case 'name':
				$cell_class = 'item-name';
				$colspan = 2;
				$cell_value = lang('total') . " " . $label;
				break;
			case 'project_progress':
			case 'revenue_vs_budget':
			case 'remaining_budgeted_vs_contract':
			case 'effective_multiplier':
				break;
			default: 
				$cell_class = 'amount';
				$cell_value = format_money_amount($group_data['totals'][$col], $currency_sym);
				break;
		}

		echo '<td colspan="'.$colspan.'" class="'.$cell_class.'">'.$cell_value.'</td>';
	}
	echo '</tr>';

}


