<?php
echo stylesheet_tag("og/reporting.css");
require_javascript("og/ReportingFunctions.js");
$genid = gen_id();
if (!isset($conditions)) $conditions = array();

$report_data = array_var($_SESSION, 'project_earnings_report_data');
?>
<form style='height:100%;background-color:white' class="internalForm" action="<?php echo get_url('project_reports', 'project_earnings_report') ?>" method="post" enctype="multipart/form-data">
<div class="coInputHeader">
	<div class="coInputHeaderUpperRow">
		<div class="coInputTitle"><?php echo $title ?>
	</div>
</div>
<div class="coInputSeparator"></div>
<div class="coInputMainBlock">
<?php
    $project_ot = ObjectTypes::findByName('project');

    // Group by options
    $group_by_options = array();
    $used_dimensions = array();
    
    // Enabled object types
    $enabled_object_types = ObjectTypes::getAllObjectTypes();
    $enabled_object_type_ids = array();
    foreach ($enabled_object_types as $enabled_ot) {
        $enabled_object_type_ids[] = $enabled_ot->getId();
    }

    // Project types dimension
    $project_types_dim = Dimensions::findByCode('project_types');
    $project_type_ot_ids = DimensionObjectTypes::getObjectTypeIdsByDimension($project_types_dim->getId());
    if(count(array_intersect($project_type_ot_ids, $enabled_object_type_ids)) > 0) {
        $group_by_options[] = array('project_types_dim', $project_types_dim->getName());
        $used_dimensions['project_types_dim'] = $project_types_dim;
    }

    // Project managers dimension
    $project_manager_dim = Dimensions::findByCode('project_managers');
    $pm_dim_added = false;
    $project_manager_ot_ids = DimensionObjectTypes::getObjectTypeIdsByDimension($project_manager_dim->getId());
    if(count(array_intersect($project_manager_ot_ids, $enabled_object_type_ids)) > 0) {
        $group_by_options[] = array('project_managers_dim', $project_manager_dim->getName());
        $used_dimensions['project_managers_dim'] = $project_manager_dim;
        $pm_dim_added = true;
    }

    // Project phase dimension
    $project_phase_dim = Dimensions::findByCode('project_phases');
    $project_phase_ot_ids = DimensionObjectTypes::getObjectTypeIdsByDimension($project_phase_dim->getId());
    if(count(array_intersect($project_phase_ot_ids, $enabled_object_type_ids)) > 0) {
        $group_by_options[] = array('project_phases_dim', $project_phase_dim->getName());
        $used_dimensions['project_phases_dim'] = $project_phase_dim;
    }
    
    if(!$pm_dim_added && $cp_project_manager instanceof CustomProperty) {
        $group_by_options[] = array('project_manager_cp', $cp_project_manager->getName());
    }

    $gr_by_listeners = array('on_selection_change' => 'og.show_correct_dimension_selector()');

	?>

    <div class="dataBlock">
        <label><?php echo lang('group by') ?></label>
        <?php echo simple_select_box('report[group_by]', $group_by_options, array_var($report_data, 'group_by'), array('id' => 'group_by_selector', 'onchange' => 'og.show_correct_dimension_selector()'))?>
    </div>

    <?php foreach($group_by_options as $gr_option) {
        $dim_name = $gr_option[0];
        if(strpos($dim_name, '_dim')) {
            $dim = $used_dimensions[$dim_name];
            $selected_mem_id = json_decode(array_var($report_data, $dim_name.'_id')); ?>
            <div class="dataBlock" id="<?php echo $dim_name; ?>_container">  
                <?php
                    render_single_member_selector($dim, $genid . $dim_name, $selected_mem_id, array('is_multiple' => false, 'hidden_field_name' => 'report['.$dim_name.'_id]'));
                ?>
            </div>
            <?php
        }
    }
    ?> 

    <!-- Include projects with unset 'budget_total' custom property -->
    <?php
        $project_ot_id = ObjectTypes::findByName('project')->getId();
        $contract_amount_cp = CustomProperties::instance()->getCustomPropertyByCode($project_ot_id, 'budget_total');
        if($contract_amount_cp){
    ?>
    <div class="dataBlock">
		<label><?php echo lang('include projects with unset ') . '\'' . $contract_amount_cp->getName() . '\''; ?>:</label>
		<?php echo checkbox_field('report[include_unset_total_budget]', array_var($report_data, 'include_unset_total_budget'))?>
	</div>
    <?php } ?>

<?php echo submit_button(lang('generate report'),'s',array('style'=>'margin-top:0px;')) ?>
</div>

<script>
	og.loadFiltersReport("<?php echo $report_key; ?>");

	og.show_correct_dimension_selector = function () {
        $('*[id*=_container]').hide();
        $('#' + $('#group_by_selector').val() + '_container').show();
	}
	
	og.show_correct_dimension_selector();
</script>

</form>
