<?php
$report_data['is_pdf_export'] = isset($is_pdf_export) ? $is_pdf_export : false;

// Render report title
echo '<div class="project-report-title">'.$title.'</div>';
if(array_var($report_data, 'start_date') && array_var($report_data, 'end_date')){
	// Get Start Date value
	$date_format = user_config_option('date_format');
	$start_date = $report_data['start_date'];
	
	$start_date_string = $start_date->add('s',logged_user()->getUserTimezoneValue())->format($date_format);
	
	// Get End Date value
	$end_date = $report_data['end_date'];
	$end_date_string = $end_date->add('s',logged_user()->getUserTimezoneValue())->format($date_format);
	
	// Build html with Date filters
	echo "<div class='project-reports__filters'>";
	echo "<p>" . lang('start date') . ': ' . $start_date_string . "</p>";
	echo "<p>" . lang('end date') . ': ' . $end_date_string . "</p>";
	echo "</div>";
}
$projects_data = array_var($data, 'projects_data');
$date_periods = array_var($data, 'date_periods');

$is_pdf_export = array_var($report_data, 'is_pdf_export', false);
$table_zoom = $is_pdf_export ? 'zoom: 0.8;' : '';
echo '<div style="'.$table_zoom.'">';
foreach ($projects_data as $project_data) {
	render_forecast_report_project_table($project_data, $report_data, $date_periods);	
}

$use_expense_estimate_cost = $report_data['group_expenses_by'] == 'estimated_cost';
$use_labor_estimate_cost = $report_data['group_labor_by'] == 'estimated_cost';
if(count($projects_data) > 1 && ($use_expense_estimate_cost || $use_labor_estimate_cost)) {
    render_forecast_report_total_table($data['grand_totals'], $report_data, $date_periods);
}
echo '</div>';


function render_forecast_report_project_table($project_data, $report_data, $date_periods){
    // *** LC 2023-09-04 *** se define $padding
    $padding = "";

    $total_cols = count($date_periods) + 3;
    // Render table name
    echo '<div style="page-break-inside: avoid;">';
	echo '<table class="report custom-report project-mamagement-report" style="margin-top: 30px;">';
    echo '<thead><tr class="header-custom-report project-report-project-header"><th colspan="'.$total_cols.'" style='.$padding.'><span style="color: #0a85d8; margin-right: 20px;">'.lang('project').'</span>'.$project_data['name'].'</th></tr></thead>';
    echo '</table>';

    echo '<table class="report custom-report project-mamagement-report timeslots">';

    render_forecast_header_columns($date_periods);

    if(!empty($project_data['labor_data'])){
        render_forecast_project_labor_table($project_data, $report_data, $date_periods);
    }
	
	if(!empty($project_data['expenses_data'])){
        render_forecast_project_expenses_table($project_data, $report_data, $date_periods);
    }
	
    $use_labor_estimate_cost = $report_data['group_labor_by'] == 'estimated_cost';
    $use_expense_estimate_cost = $report_data['group_expenses_by'] == 'estimated_cost';
    if($use_labor_estimate_cost && $use_expense_estimate_cost){
        render_forecast_report_project_total($project_data['project_totals'], $report_data, $date_periods);
    }

}

function render_forecast_project_labor_table($project_data, $report_data, $date_periods){
    $use_expense_estimate_cost = $report_data['group_expenses_by'] == 'estimated_cost';
    $use_labor_estimate_cost = $report_data['group_labor_by'] == 'estimated_cost';
    $currency_symbol = config_option('currency_code');
    $total_cols = count($date_periods) + 2;

    // Render Labor
    if (!empty($project_data['labor_data'])){
        $header_colspan = $total_cols - 1;
        echo '<tr><td style="font-weight: 600;" colspan="2">'.lang('estimated labor').'</td><td colspan="'.$header_colspan.'"></td></tr>';
        foreach ($project_data['labor_data'] as $category_data) {
            echo '<tr>';
            echo '<td colspan="2">'. $category_data['name'] .'</td>';
            foreach($date_periods as $period){
                $value_per_period = $category_data['periods'][$period] == 0 ? '' : $category_data['periods'][$period];
                if($use_labor_estimate_cost && $value_per_period != ''){
                    $value_per_period = format_money_amount($value_per_period, $currency_symbol);
                }
                echo '<td class="right">'. $value_per_period .'</td>';
            }
            $category_total = $use_labor_estimate_cost ? format_money_amount($category_data['category_total'], $currency_symbol) : $category_data['category_total'];
            echo '<td class="right">'.$category_total .'</td>';
            echo '</tr>';
        }
        // Render labor totals if estimated cost is used
        if($use_labor_estimate_cost) {
            render_forecast_project_labor_totals($project_data, $date_periods);
        }
        if(($use_expense_estimate_cost && $use_labor_estimate_cost) || !empty($project_data['expenses_data'])) {
            echo '<tr><td colspan="2">&nbsp;</td></tr>';
        }
    }
}

function render_forecast_project_labor_totals($project_data, $date_periods){
    $currency_symbol = config_option('currency_code');
    $style = "font-weight: 600;";
    $labor_totals = $project_data['labor_totals'];
    echo '<tr style="'.$style.'">';
    echo '<td colspan="2">'. lang('labor totals') .'</td>';
    foreach($date_periods as $period){
        $value_per_period = format_money_amount($labor_totals['periods'][$period], $currency_symbol);
        echo '<td class="right">'. $value_per_period .'</td>';
    }
    $labor_grand_total = format_money_amount($labor_totals['labor_total'], $currency_symbol);
    echo '<td class="right">'.$labor_grand_total.'</td>';
    echo '</tr>';
}



function render_forecast_project_expenses_table($project_data, $report_data, $date_periods){
    $use_expense_estimate_cost = $report_data['group_expenses_by'] == 'estimated_cost';
    $use_labor_estimate_cost = $report_data['group_labor_by'] == 'estimated_cost';
    $currency_symbol = config_option('currency_code');
    $total_cols = count($date_periods) + 2;

    // Render expenses info
    if (!empty($project_data['expenses_data'])){
        $header_colspan = $total_cols - 1;
        echo '<tr><td style="font-weight: 600;" colspan="2">'.lang('expenses').'</td><td colspan="'.$header_colspan.'"></td></tr>';
        foreach ($project_data['expenses_data'] as $product_type_data) {
            echo '<tr>';
            echo '<td colspan="2">'. $product_type_data['name'] .'</td>';
            foreach($date_periods as $period){
                $value_per_period = $product_type_data['periods'][$period] == 0 ? '' : $product_type_data['periods'][$period];
                if($use_expense_estimate_cost && $value_per_period != ''){
                    $value_per_period = format_money_amount($value_per_period, $currency_symbol);
                }
                echo '<td class="right">'. $value_per_period .'</td>';
            }
            $product_type_total = $use_expense_estimate_cost ? format_money_amount($product_type_data['product_type_total'], $currency_symbol) : $product_type_data['product_type_total'];
            echo '<td class="right">'.$product_type_total .'</td>';
            echo '</tr>';
        }
        // Render expenses totals if estimated cost is used
        if($use_expense_estimate_cost) {
            render_forecast_project_expenses_totals($project_data, $date_periods);
        }
        if($use_expense_estimate_cost && $use_labor_estimate_cost) {
            echo '<tr><td colspan="2">&nbsp;</td></tr>';
        }
    }
}

function render_forecast_project_expenses_totals($project_data, $date_periods){
    $currency_symbol = config_option('currency_code');
    $style = "font-weight: 600;";
    $expense_totals = $project_data['expense_totals'];
    echo '<tr style="'.$style.'">';
    echo '<td colspan="2">'. lang('expense totals') .'</td>';
    foreach($date_periods as $period){
        $value_per_period = format_money_amount($expense_totals['periods'][$period], $currency_symbol);
        echo '<td class="right">'. $value_per_period .'</td>';
    }
    $expense_grand_total = format_money_amount($expense_totals['expenses_total'], $currency_symbol);
    echo '<td class="right">'.$expense_grand_total.'</td>';
    echo '</tr>';
}

function render_forecast_report_project_total($totals, $report_data, $date_periods){
    $currency_symbol = config_option('currency_code');
    $style = "font-weight: 600;";
    echo '<tr style="'.$style.'">';
    echo '<td colspan="2">'. lang('totals') .'</td>';
    foreach($date_periods as $period){
        $value_per_period = format_money_amount($totals['periods'][$period], $currency_symbol);
        echo '<td class="right">'. $value_per_period .'</td>';
    }
    $project_grand_total = format_money_amount($totals['project_total'], $currency_symbol);
    echo '<td class="right">'.$project_grand_total.'</td>';
    echo '</tr>';
}

function render_forecast_report_total_table($totals, $report_data, $date_periods){
    $use_labor_estimate_cost = $report_data['group_labor_by'] == 'estimated_cost';
    $use_expense_estimate_cost = $report_data['group_expenses_by'] == 'estimated_cost';
    $currency_symbol = config_option('currency_code');
    $total_cols = count($date_periods) + 2;

    // Render table name
    echo '<table class="report custom-report project-mamagement-report"><thead>';
    echo '<tr class="header-custom-report custom-report-table-heading custom-report-table-heading-capture-totals"><th colspan="'.$total_cols.'" style="padding-top: 30px;">'.lang('grand totals').'</th></tr></thead></table>';

    // Render table
    echo '<table class="report custom-report project-mamagement-report">';
    render_forecast_header_columns($date_periods);

    // Render labor row
    if($use_labor_estimate_cost){
        $style = "font-weight: 600;";
        echo '<tr style="'.$style.'">';
        echo '<td colspan="2">'. lang('labor totals') .'</td>';
        foreach($date_periods as $period){
            $value_per_period = format_money_amount($totals['labor_totals']['periods'][$period], $currency_symbol);
            echo '<td class="right">'. $value_per_period .'</td>';
        }
        $grand_labor_total = format_money_amount($totals['labor_totals']['labor_grand_total'], $currency_symbol);
        echo '<td class="right">'.$grand_labor_total.'</td>';
        echo '</tr>';
    }

    // Render expenses row
    if($use_expense_estimate_cost){
        $style = "font-weight: 600;";
        echo '<tr style="'.$style.'">';
        echo '<td colspan="2">'. lang('expense totals') .'</td>';
        foreach($date_periods as $period){
            $value_per_period = format_money_amount($totals['expenses_totals']['periods'][$period], $currency_symbol);
            echo '<td class="right">'. $value_per_period .'</td>';
        }
        $grand_expenses_total = format_money_amount($totals['expenses_totals']['expenses_grand_total'], $currency_symbol);
        echo '<td class="right">'.$grand_expenses_total.'</td>';
        echo '</tr>';
    }

    // Render total row
    if($use_labor_estimate_cost && $use_expense_estimate_cost){
        $style = "font-weight: 600;";
        echo '<tr style="'.$style.'">';
        echo '<td colspan="2">'. lang('grand totals') .'</td>';
        foreach($date_periods as $period){
            $value_per_period = format_money_amount($totals['report_totals']['periods'][$period], $currency_symbol);
            echo '<td class="right">'. $value_per_period .'</td>';
        }
        $grand_total = format_money_amount($totals['report_totals']['report_grand_total'], $currency_symbol);
        echo '<td class="right">'.$grand_total.'</td>';
        echo '</tr>';
    }

    echo "</table>";

}


function render_forecast_header_columns($date_periods){
    // Get header column names
	$columns = array(' ');
    foreach($date_periods as $period){
        $period_dtv = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $period.'-5');
        $columns[] = $period_dtv->format('M') . ' ' . $period_dtv->format('Y');
    }
    $columns[] = lang('total'); 

    // Render header columns
    echo '<thead>';
	echo '<tr class="header-custom-report custom-report-table-heading2">';
	foreach ($columns as $col) {
		if ($col == ' '){
			echo '<th colspan="2">'.$col.'</th>';
		} else {
			echo '<th class="right">'.$col.'</th>';
		}
	}
	echo '</tr></thead>';
}
