<?php 

Env::useHelper('functions', 'project_reports');

function render_project_timeslots_table($project_data, $report_data, $total_cols, $level = 0) {
	if (empty($project_data['timeslots_data'])) return;

	$fin_type = array_var($report_data, 'financial_type', 'price');
	$use_cost = in_array($fin_type, array('cost', 'price_and_cost'));
	$use_price = in_array($fin_type, array('price', 'price_and_cost'));

	$report_data = array_var($_SESSION, 'project_management_report_data');
	$show_invoice_detail = get_show_invoicing_detail($report_data);
	$padding = $level > 0 ? "padding-left:". ($level * 15) . "px" : "";
	$top_level = ($level > 1) ? $level+2 : $level+1;
	$default_currency_symbol = config_option('currency_code');;

	$padding_top = $level > 0 ? "padding-left:". ($top_level * 15) . "px" : "";
	$zero_padding_class = "";
	if($padding_top == ""){
		$zero_padding_class = "zero-padding";
	}

	?><thead><?php  
	echo '<tr class="header-custom-report custom-report-table-heading custom-report-table-heading-capture '.$zero_padding_class.'"><th colspan="'.$total_cols.'" style="'.$padding_top.'" class="'.$zero_padding_class.'">'/*.$project_data['project_name'].' -  '*/.lang('detail of labor').'<span class="small-project-name"> '.$project_data['project_name'].'</span></th></tr>';
	
	$columns = array('date', 'person', 'hour_type');
	$billing_columns = array('worked_time');
	if($use_cost) $billing_columns[] = 'total_cost';
	if($use_price) $billing_columns[] = 'totalbilling';
	

	if (array_var($report_data, 'include_time_description')) {
		$columns[] = 'description';
	}
	
	if (array_var($report_data, 'include_miles')) {
		$columns[] = 'miles';
	}
	if (array_var($report_data, 'include_billing_rate_name')) {
		$columns[] = 'billing_rate';
	}
	if ($use_cost) $columns[] = 'hourly_cost';
	if ($use_price) $columns[] = 'hourlybilling';

	if (array_var($report_data, 'include_estimated_time')) {
		$columns[] = ' ';
	}
	if (array_var($report_data, 'include_estimated_billing')) {
		$columns[] = ' ';
	}
	if ($show_invoice_detail) {
		$billing_columns[] = 'invoiced';
		$billing_columns[] = 'not_invoiced';
	}
	if (array_var($report_data, 'include_pending_time')) {
		$billing_columns[] = ' ';
	}
	if (array_var($report_data, 'include_pending_billing')) {
		$billing_columns[] = ' ';
	}
	if (array_var($report_data, 'include_rate_schedule')) {
		$billing_columns[] = 'rate_schedule';
	}
	if (array_var($report_data, 'include_approval_status')) {
		$billing_columns[] = 'approval_status';
	}
	if (array_var($report_data, 'include_modules_dim')) {
		$billing_columns[] = 'modules_dim';
	}
	
	$columns = array_merge($columns, $billing_columns);
	?>
	<tr class="header-custom-report custom-report-table-heading2">
	<?php 
	foreach ($columns as $col) {
		$class = '';
		if (in_array($col, array('miles'))) $class = 'center';
		else if (in_array($col, array('worked_time', 'hourlybilling','totalbilling', 'invoiced', 'not_invoiced', 'hourly_cost', 'total_cost'))) $class = 'right';
		if($col == 'date'){
			echo '<th style='.$padding.'>'.lang($col).'</th>';
		} else if ($col == 'description'){
			echo '<th colspan="2" class="'.$class.'">'.lang($col).'</th>';
		}  else if ($col == 'rate_schedule'){
			$rate_sched_dim = Dimensions::findByCode('rate_schedules');
			$rate_sched_name = $rate_sched_dim->getName();
			echo '<th>'.$rate_sched_name.'</th>';
		} else if ($col == 'modules_dim'){
			$modules_dim = Dimensions::findByCode('modules');
			$modules_name = $modules_dim->getName();
			echo '<th>'.$modules_name.'</th>';
		} else if ($col == 'approval_status'){
			$approval_status_dim = Dimensions::findByCode('status_timesheet');
			$approval_status_name = $approval_status_dim->getName();
			echo '<th>'.$approval_status_name.'</th>';
		} else {
			echo '<th class="'.$class.'">'.lang($col).'</th>';
		}
	}
	?>
	</tr></thead>
	<tbody>
	<?php 
	foreach ($project_data['timeslots_data'] as $timeslot) {
		echo '<tr>';
		foreach ($columns as $col) {
			switch ($col) {
				case 'date':
					$date = new DateTimeValue($timeslot['start_time_ts']);
					echo '<td style='.$padding.'>'. format_date($date, null, null) .'</td>';
					break;
				case 'person':
					echo '<td>'. $timeslot['uname'] .'</td>';
					break;
				case 'hour_type':
					echo '<td class="category-column">'. $timeslot['hour_types'][0] .'</td>';
					break;
				case 'worked_time':
					echo '<td class="right">'. number_format($timeslot['worked_hours'],  2, '.', ',') .'</td>';
					break;
				case ' ':
					echo '<td class="center"></td>';
					break;
				case 'billing_rate':
					echo '<td>'. $timeslot['billing_category_name'] .'</td>';
					break;
				case 'miles':
					echo '<td class="right">'. number_format($timeslot['miles'],  2, '.', ',') .'</td>';
					break;
				case 'hourlybilling':
					echo '<td class="right">'. format_money_amount($timeslot['hourlybilling'], isset($timeslot['rate_currency_sym']) ? $timeslot['rate_currency_sym'] : $default_currency_symbol).'</td>';
					break;
				case 'hourly_cost':
					echo '<td class="right">'. format_money_amount($timeslot['hourly_cost'], isset($timeslot['rate_currency_sym']) ? $timeslot['rate_currency_sym'] : $default_currency_symbol).'</td>';
					break;
				case 'total_cost':
					echo '<td class="right">'. format_money_amount($timeslot['total_cost'], isset($timeslot['rate_currency_sym']) ? $timeslot['rate_currency_sym'] : $default_currency_symbol).'</td>';
					break;
				case 'description':
					echo '<td colspan="2">'. $timeslot['description'].'</td>';
					break;
				case 'rate_schedule':
					$rate_schedule = array_var($timeslot, 'rate_schedules', array(''));
					echo '<td>'.$rate_schedule[0].'</td>';
					break;
				case 'modules_dim':
					$modules = array_var($timeslot, 'modules', array(''));
					echo '<td>'.$modules[0].'</td>';
					break;
				case 'approval_status':
					$approval_status = array_var($timeslot, 'status_timesheet', array(''));
					echo '<td>'.$approval_status[0].'</td>';
					break;
				case 'totalbilling':
					echo '<td class="right">'. format_money_amount($timeslot['totalbilling'], isset($timeslot['rate_currency_sym']) ? $timeslot['rate_currency_sym'] : $default_currency_symbol).'</td>';
					break;
				case 'invoiced':
					if($timeslot['invoicing_status'] == 'invoiced'){
						echo '<td class="right">'. format_money_amount($timeslot['totalbilling'], $timeslot['rate_currency_sym']).'</td>';
					} else {
						echo '<td class="center"></td>';
					}
					break;
				case 'not_invoiced':
					if($timeslot['invoicing_status'] == 'pending'){
						echo '<td class="right">'. format_money_amount($timeslot['totalbilling'], $timeslot['rate_currency_sym']).'</td>';
					} else {
						echo '<td class="center"></td>';
					}
					break;
				default:
					break;
			}
		}
		echo '</tr>';
	
	}
	?>
	</tbody><tbody style="border: 0px none;">
	<tr class="separator"><td colspan=<?php echo $total_cols?>>&nbsp;</td></tr>
	</tbody>
	<?php 
	
}


function render_project_expenses_table($project_data, $report_data, $total_cols, $level = 0) {
	if (empty($project_data['expenses_data'])) return;

	$fin_type = array_var($report_data, 'financial_type', 'price');
	$use_cost = in_array($fin_type, array('cost', 'price_and_cost'));
	$use_price = in_array($fin_type, array('price', 'price_and_cost'));
	
	$project_billing_alt_report = Reports::instance()->findOne(array("conditions" => "`code` = 'project_billing_alt_report'"));
	$project_billing_alt_report_id = $project_billing_alt_report instanceof Report ? $project_billing_alt_report->getId() : 0;
	if($report_data['report_id']==$project_billing_alt_report_id){
		$project_expenses_alt_table = build_project_expenses_table_for_alt_report($project_data, $report_data, $total_cols, $level);
		return;
	}
	$show_invoice_detail = get_show_invoicing_detail($report_data);
	$padding = $level > 0 ? "padding-left:". ($level * 20) . "px" : "";
	?><thead><?php 
	echo '<tr class="header-custom-report custom-report-table-heading custom-report-table-heading-capture"><th colspan="'.$total_cols.'" style='.$padding.'>'/*.$project_data['project_name'].' -  '*/.lang('detail of expenses').'<span class="small-project-name"> '.$project_data['project_name'].'</span></th></tr>';
	
	$columns = array('date', 'person', 'expense_category');
	$billing_columns = array('quantity');
	if($use_cost) $billing_columns[] = 'total_cost';
	if($use_price) $billing_columns[] = 'amount';
	?>
	
	<tr class="header-custom-report custom-report-table-heading2">
	<?php 

	if (array_var($report_data, 'include_time_description')) {
		$columns[] = 'description';
	}
	if (array_var($report_data, 'include_miles')) {
		$columns[] = 'miles';
	}
	if (array_var($report_data, 'include_billing_rate_name')) {
		$columns[] = ' ';
	}
	if ($use_cost) $columns[] = 'unit_cost';
	if ($use_price) $columns[] = 'unit_price';
	if (array_var($report_data, 'include_estimated_time')) {
		$columns[] = ' ';
	}
	if (array_var($report_data, 'include_estimated_billing')) {
		$columns[] = ' ';
	}
	if ($show_invoice_detail) {
		$billing_columns[] = 'invoiced';
		$billing_columns[] = 'not_invoiced';
	}
	if (array_var($report_data, 'include_pending_time')) {
		$billing_columns[] = ' ';
	}
	if (array_var($report_data, 'include_pending_billing')) {
		$billing_columns[] = ' ';
	}
	if (array_var($report_data, 'include_rate_schedule')) {
		$enabled_dimensions = config_option('enabled_dimensions');
		$expense_rate_schedule_dim = Dimensions::findByCode('expense_rate_schedules');

		if($expense_rate_schedule_dim instanceof Dimension && in_array($expense_rate_schedule_dim->getId(), $enabled_dimensions)){
			$billing_columns[] = 'expense_rate_schedule';
		} else {
			$billing_columns[] = 'rate_schedule';
		}
	}
	if (array_var($report_data, 'include_approval_status')) {
		$billing_columns[] = 'approval_status';
	}
	if (array_var($report_data, 'include_modules_dim')) {
		$billing_columns[] = ' ';
	}

	$columns = array_merge($columns, $billing_columns);


	foreach ($columns as $col) {
		$class = '';
		if ($col == 'worked_time') $class = 'center';
		else if (in_array($col, array('amount', 'invoiced', 'not_invoiced', 'unit_price', 'quantity', 'unit_cost', 'total_cost'))) $class = 'right';
		$colspan = 1;
		if ($col == 'description') $colspan = 2;

		if($col == 'date'){
			echo '<th style='.$padding.' colspan="'.$colspan.'">'.lang($col).'</th>';
		} else if($col == 'amount'){
			echo '<th class="'.$class.'" colspan="'.$colspan.'">'.lang($col.' project report').'</th>';
		} else if($col == 'name'){
			echo '<th colspan="'.$name_colspan.'">'.lang($col).'</th>';
		} else if($col == 'expense_rate_schedule'){
			$exp_rate_sched_dim = Dimensions::findByCode('expense_rate_schedules');
			$exp_rate_sched_name = $exp_rate_sched_dim->getName();
			echo '<th>'.$exp_rate_sched_name.'</th>';
		} else if($col == 'rate_schedule'){
			$rate_sched_dim = Dimensions::findByCode('rate_schedules');
			$rate_sched_name = $rate_sched_dim->getName();
			echo '<th>'.$rate_sched_name.'</th>';
		} else if ($col == 'approval_status'){
			$approval_status_dim = Dimensions::findByCode('status_timesheet');
			$approval_status_name = $approval_status_dim->getName();
			echo '<th>'.$approval_status_name.'</th>';
		}  else {
			echo '<th class="'.$class.'" colspan="'.$colspan.'">'.lang($col).'</th>';
		}
	}
	?>
	</tr></thead>
	<tbody>
	<?php 
	foreach ($project_data['expenses_data'] as $expense) {
		echo '<tr>';
		foreach ($columns as $col) {
			switch ($col) {
				case 'date':
					echo '<td style='.$padding.'>'. $expense['date'] .'</td>';
					break;
				case 'expense_category':
					echo '<td class="category-column">'. $expense['expense_categories'][0] .'</td>';
					break;
				case 'name':
					echo '<td colspan="'.$name_colspan.'">'. $expense['name'] .'</td>';
					break;
				case 'person':
					echo '<td>'. $expense['paid_by_name'] .'</td>';
					break;
				case 'quantity':
					echo '<td class="right">'. $expense['quantity'] .'</td>';
					break;
				case 'unit_price':
					echo '<td class="right">'. format_money_amount($expense['unit_price'], $expense['currency_sym']).'</td>';
					break;
				case 'unit_cost':
					echo '<td class="right">'. format_money_amount($expense['unit_cost'], $expense['currency_sym']).'</td>';
					break;
				case 'description':
					echo '<td colspan="2">'. $expense['description'] .'</td>';
					break;
				case 'amount':
					echo '<td class="right">'. format_money_amount($expense['total_price'], $expense['currency_sym']).'</td>';
					break;
				case 'total_cost':
					echo '<td class="right">'. format_money_amount($expense['amount'], $expense['currency_sym']).'</td>';
					break;
				case 'rate_schedule':
					$rate_schedule = array_var($expense, 'rate_schedules', array(''));
					echo '<td>'.$rate_schedule[0].'</td>';
					break;
				case 'expense_rate_schedule':
					$exp_rate_schedule = array_var($expense, 'expense_rate_schedules', array(''));
					echo '<td>'.$exp_rate_schedule[0].'</td>';
					break;
				case 'approval_status':
					$approval_status = array_var($expense, 'status_timesheet', array(''));
					echo '<td>'.$approval_status[0].'</td>';
					break;
				case 'invoiced':
					if($expense['invoicing_status'] == 'invoiced'){
						echo '<td class="right">'. format_money_amount($expense['total_price'], $expense['currency_sym']).'</td>';
					} else {
						echo '<td class="center"></td>';
					}
					break;
				case 'not_invoiced':
					if($expense['invoicing_status'] == 'pending'){
						echo '<td class="right">'. format_money_amount($expense['total_price'], $expense['currency_sym']).'</td>';
					} else {
						echo '<td class="center"></td>';
					}
					break;
				case ' ':
					echo '<td class="center"></td>';
					break;
				default:
					break;
			}
		}
		echo '</tr>';
	
	}
	?>
	</tbody><tbody style="border: 0px none;">
	<tr class="separator"><td colspan=<?php echo $total_cols?>>&nbsp;</td></tr>
	</tbody>
	<?php 
	
}

function build_project_expenses_table_for_alt_report($project_data, $total_cols, $level = 0) {
	if (empty($project_data['expenses_data'])) return;
	$report_data = array_var($_SESSION, 'project_management_report_data');

	$show_invoice_detail = get_show_invoicing_detail($report_data);
	$padding = $level > 0 ? "padding-left:". ($level * 20) . "px" : "";
	?><thead><?php 
	echo '<tr class="header-custom-report custom-report-table-heading custom-report-table-heading-capture"><th colspan="'.$total_cols.'" style='.$padding.'>'/*.$project_data['project_name'].' -  '*/.lang('detail of expenses').'<span class="small-project-name"> '.$project_data['project_name'].'</span></th></tr>';
	
	$columns = array('date','expense_category', 'name', 'description');
	$billing_columns = array('amount');
	?>
	
	<tr class="header-custom-report custom-report-table-heading2">
	<?php 

	if (array_var($report_data, 'include_miles')) {
		$columns[] = 'miles';
	}
	if (array_var($report_data, 'include_billing_rate_name')) {
		$columns[] = ' ';
	}
	//$columns[] = ' ';
	if (array_var($report_data, 'include_estimated_time')) {
		$columns[] = ' ';
	}
	if (array_var($report_data, 'include_estimated_billing')) {
		$columns[] = ' ';
	}
	if ($show_invoice_detail) {
		$billing_columns[] = 'invoiced';
		$billing_columns[] = 'not_invoiced';
	}
	if (array_var($report_data, 'include_pending_time')) {
		$billing_columns[] = ' ';
	}
	if (array_var($report_data, 'include_pending_billing')) {
		$billing_columns[] = ' ';
	}
	if (array_var($report_data, 'include_rate_schedule')) {
		$enabled_dimensions = config_option('enabled_dimensions');
		$expense_rate_schedule_dim = Dimensions::findByCode('expense_rate_schedules');

		if($expense_rate_schedule_dim instanceof Dimension && in_array($expense_rate_schedule_dim->getId(), $enabled_dimensions)){
			$billing_columns[] = 'expense_rate_schedule';
		} else {
			$billing_columns[] = 'rate_schedule';
		}
	}
	if (array_var($report_data, 'include_approval_status')) {
		$billing_columns[] = 'approval_status';
	}
	if (array_var($report_data, 'include_modules_dim')) {
		$billing_columns[] = ' ';
	}

	$columns = array_merge($columns, $billing_columns);


	foreach ($columns as $col) {
		$class = '';
		if ($col == 'worked_time') $class = 'center';
		else if (in_array($col, array('amount', 'invoiced', 'not_invoiced', 'unit_price', 'quantity'))) $class = 'right';
		$colspan = 1;
		if ($col == 'description') $colspan = 3;

		if($col == 'date'){
			echo '<th style='.$padding.' colspan="'.$colspan.'">'.lang($col).'</th>';
		} else if($col == 'amount'){
			echo '<th class="'.$class.'" colspan="'.$colspan.'">'.lang($col.' project report').'</th>';
		} else if($col == 'name'){
			echo '<th colspan="'.$name_colspan.'">'.lang($col).'</th>';
		} else if($col == 'expense_rate_schedule'){
			$exp_rate_sched_dim = Dimensions::findByCode('expense_rate_schedules');
			$exp_rate_sched_name = $exp_rate_sched_dim->getName();
			echo '<th>'.$exp_rate_sched_name.'</th>';
		} else if($col == 'rate_schedule'){
			$rate_sched_dim = Dimensions::findByCode('rate_schedules');
			$rate_sched_name = $rate_sched_dim->getName();
			echo '<th>'.$rate_sched_name.'</th>';
		} else if ($col == 'approval_status'){
			$approval_status_dim = Dimensions::findByCode('status_timesheet');
			$approval_status_name = $approval_status_dim->getName();
			echo '<th>'.$approval_status_name.'</th>';
		}  else {
			echo '<th class="'.$class.'" colspan="'.$colspan.'">'.lang($col).'</th>';
		}
	}
	?>
	</tr></thead>
	<tbody>
	<?php 
	foreach ($project_data['expenses_data'] as $expense) {
		echo '<tr>';
		foreach ($columns as $col) {
			switch ($col) {
				case 'date':
					echo '<td style='.$padding.'>'. $expense['date'] .'</td>';
					break;
				case 'expense_category':
					echo '<td class="category-column">'. $expense['expense_categories'][0] .'</td>';
					break;
				case 'name':
					echo '<td colspan="'.$name_colspan.'">'. $expense['name'] .'</td>';
					break;
				case 'person':
					echo '<td>'. $expense['paid_by_name'] .'</td>';
					break;
				case 'quantity':
					echo '<td class="right">'. $expense['quantity'] .'</td>';
					break;
				case 'unit_price':
					echo '<td class="right">'. format_money_amount($expense['unit_price'], $expense['currency_sym']).'</td>';
					break;
				case 'description':
					echo '<td colspan="3">'. $expense['description'] .'</td>';
					break;
				case 'amount':
					echo '<td class="right">'. format_money_amount($expense['total_price'], $expense['currency_sym']).'</td>';
					break;
				case 'rate_schedule':
					$rate_schedule = array_var($expense, 'rate_schedules', array(''));
					echo '<td>'.$rate_schedule[0].'</td>';
					break;
				case 'expense_rate_schedule':
					$exp_rate_schedule = array_var($expense, 'expense_rate_schedules', array(''));
					echo '<td>'.$exp_rate_schedule[0].'</td>';
					break;
				case 'approval_status':
					$approval_status = array_var($expense, 'status_timesheet', array(''));
					echo '<td>'.$approval_status[0].'</td>';
					break;
				case 'invoiced':
					if($expense['invoicing_status'] == 'invoiced'){
						echo '<td class="right">'. format_money_amount($expense['total_price'], $expense['currency_sym']).'</td>';
					} else {
						echo '<td class="center"></td>';
					}
					break;
				case 'not_invoiced':
					if($expense['invoicing_status'] == 'pending'){
						echo '<td class="right">'. format_money_amount($expense['total_price'], $expense['currency_sym']).'</td>';
					} else {
						echo '<td class="center"></td>';
					}
					break;
				case ' ':
					echo '<td class="center"></td>';
					break;
				default:
					break;
			}
		}
		echo '</tr>';
	
	}
	?>
	</tbody><tbody style="border: 0px none;">
	<tr class="separator"><td colspan=<?php echo $total_cols?>>&nbsp;</td></tr>
	</tbody>
	<?php 
	
}



function render_project_timeslots_summary($project_data, $report_data, $total_cols, $level = 0) {
	if (empty($project_data['timeslots_summary'])) return;

	$fin_type = array_var($report_data, 'financial_type', 'price');
	$use_cost = in_array($fin_type, array('cost', 'price_and_cost'));
	$use_price = in_array($fin_type, array('price', 'price_and_cost'));

	$padding = $level > 0 ? "padding-left:". ($level * 25) . "px" : "";
	?><thead><?php 
	echo '<tr class="header-custom-report custom-report-table-heading custom-report-table-heading-capture"><th colspan="'.$total_cols.'" style='.$padding.'>'/*.$project_data['project_name'].' -  '*/.lang('summary of labor').'<span class="small-project-name"> '.$project_data['project_name'].'</span></th></tr>';
	
	$columns = array('', '', 'labor_category_summary');
	$billing_columns = array('worked_time');
	if($use_cost) $billing_columns[] = 'total_cost';
	if($use_price) $billing_columns[] = 'totalbilling';
	
	$report_data = array_var($_SESSION, 'project_management_report_data');
	$show_invoice_detail = get_show_invoicing_detail($report_data);
	if (array_var($report_data, 'include_time_description')) {
		$columns[] = '';
		$columns[] = '';
	}
	if (array_var($report_data, 'include_miles')) {
		$columns[] = 'miles';
	}
	if (array_var($report_data, 'include_billing_rate_name')) {
		$columns[] = 'billing_rate';
	}

	if ($use_cost) $columns[] = 'hourly_cost';
	if ($use_price) $columns[] = 'hourlybilling';

	if (array_var($report_data, 'include_estimated_time')) {
		$columns[] = 'estimated_hours';
	}
	if (array_var($report_data, 'include_estimated_billing')) {
		$columns[] = 'estimated_billing';
	}
	if ($show_invoice_detail) {
		$billing_columns[] = 'invoiced';
		$billing_columns[] = 'not_invoiced';
	}
	if (array_var($report_data, 'include_pending_time')) {
		$billing_columns[] = 'pending_hours';
	}
	if (array_var($report_data, 'include_pending_billing')) {
		$billing_columns[] = 'pending_billing';
	}
	if (array_var($report_data, 'include_rate_schedule')) {
		$billing_columns[] = '';
	}
	if (array_var($report_data, 'include_approval_status')) {
		$billing_columns[] = '';
	}
	if (array_var($report_data, 'include_modules_dim')) {
		$billing_columns[] = '';
	}

	
	$columns = array_merge($columns, $billing_columns);
	?>
	<tr class="header-custom-report custom-report-table-heading2">
	<?php 
	foreach ($columns as $col) {
		$class = '';
		if (in_array($col, array('miles'))) $class = 'center';
		else if (in_array($col, array('worked_time', 'estimated_hours', 'pending_hours', 'estimated_billing', 'pending_billing', 'hourlybilling','totalbilling', 'invoiced', 'not_invoiced', 'hourly_cost', 'total_cost'))) $class = 'right';

		echo '<th class="'.$class.'">'.($col == '' ? '' : lang($col)).'</th>';
	}
	?>
	</tr></thead>
	<tbody>
	<?php 
	foreach ($project_data['timeslots_summary'] as $k => $summary) {
		if ($k == 'subtotal') {
			$row_class = "subtotal level-".$level;
			echo '<tr class="'.$row_class.'">';
		} else {
			echo '<tr>';
		}
		foreach ($columns as $col) {
			switch ($col) {
				case '':
					echo '<td>&nbsp;</td>';
					break;
				case 'labor_category_summary':
					echo '<td class="category-column">'. $summary['hour_type'] .'</td>';
					break;
				case 'worked_time':
					echo '<td class="right">'. number_format($summary['worked_hours'],  2, '.', ',') .'</td>';
					break;
				case 'estimated_hours':
					echo '<td class="right">'. number_format($summary['estimated_hours'],  2, '.', ',') .'</td>';
					break;
				case 'pending_hours':
					$pending_class = '';
					if($summary['pending_hours'] < 0){
						$pending_class = 'red-font';
					}
					echo '<td class="right '.$pending_class.'">'. number_format($summary['pending_hours'],  2, '.', ',') .'</td>';
					break;
				case 'hourlybilling':
					$hourlybilling = $summary['hour_type'] == 'Without labor category' ? 'N/A' : format_money_amount(array_var($summary, 'hourlybilling'), $summary['currency_sym']);
					echo '<td class="right">'. $hourlybilling .'</td>';
					break;
				case 'hourly_cost':
					$hourly_cost = $summary['hour_type'] == 'Without labor category' ? 'N/A' : format_money_amount(array_var($summary, 'hourly_cost'), $summary['currency_sym']);
					echo '<td class="right">'. $hourly_cost .'</td>';
					break;
				case 'total_cost':
					$total_cost = $summary['hour_type'] == 'Without labor category' ? 'N/A' : format_money_amount(array_var($summary, 'total_cost'), $summary['currency_sym']);
					echo '<td class="right">'. $total_cost .'</td>';
					break;
				case 'totalbilling':
					echo '<td class="right">'. format_money_amount($summary['totalbilling'], $summary['currency_sym']).'</td>';
					break;
				case 'estimated_billing':
					$estimated_billing = $summary['hour_type'] == 'Without labor category' ? 'N/A' : format_money_amount($summary['estimated_billing'], $summary['currency_sym']);
					if($summary['hour_type'] == 'Without labor category'){}
					echo '<td class="right">'.$estimated_billing.'</td>';
					break;
				case 'pending_billing':
					$pending_class = '';
					if($summary['pending_billing'] < 0){
						$pending_class = 'red-font';
					}
					$pending_billing = $summary['hour_type'] == 'Without labor category' ? 'N/A' : format_money_amount($summary['pending_billing'], $summary['currency_sym']);
					echo '<td class="right '.$pending_class.'">'. $pending_billing .'</td>';
					break;
				case 'miles':
					echo '<td class="right">'. number_format($summary['miles'],  2, '.', ',') .'</td>';
					break;
				case 'billing_rate':
					echo '<td></td>';
					break;
				case 'invoiced':
					echo '<td class="right">'. format_money_amount($summary['invoiced'], $summary['currency_sym']).'</td>';
					break;
				case 'not_invoiced':
					echo '<td class="right">'. format_money_amount($summary['not_invoiced'], $summary['currency_sym']).'</td>';
					break;
				default:
					break;
			}
		}
		echo '</tr>';
	
	}
	?>
	</tbody><tbody style="border: 0px none;">
	<tr class="separator"><td colspan=<?php echo count($columns)?>>&nbsp;</td></tr>
	</tbody>
	<?php 
}


function render_project_expenses_summary($project_data, $report_data, $total_cols, $level = 0) {
	if (empty($project_data['expenses_summary'])) return;

	$fin_type = array_var($report_data, 'financial_type', 'price');
	$use_cost = in_array($fin_type, array('cost', 'price_and_cost'));
	$use_price = in_array($fin_type, array('price', 'price_and_cost'));

	$padding = $level > 0 ? "padding-left:". ($level * 25) . "px" : "";
	?><thead><?php 
	echo '<tr class="header-custom-report custom-report-table-heading custom-report-table-heading-capture"><th colspan="'.$total_cols.'" style='.$padding.'>'/*.$project_data['project_name'].' - '*/.lang('summary of expenses').'<span class="small-project-name"> '.$project_data['project_name'].'</span></th></tr>';
	
	$columns = array('', '', 'expense_category_summary');

	
	$show_invoice_detail = get_show_invoicing_detail($report_data);

	// Calculate how many empty columns should be generated, based on configurations
	$empty_columns = 1;
	$empty_columns += array_var($report_data, 'include_time_description') ? 2 : 0;
	$empty_columns += array_var($report_data, 'include_estimated_time') ? 1 : 0;
	$empty_columns += array_var($report_data, 'include_billing_rate_name') ? 1 : 0;
	$empty_columns += array_var($report_data, 'include_miles') ? 1 : 0;
	//$empty_columns += array_var($report_data, 'include_pending_billing') ? 1 : 0;
	//$empty_columns += array_var($report_data, 'include_pending_time') ? 1 : 0;
	
	$name_colspan = 1;
	if ($empty_columns > 1) {
		$name_colspan++;
		$empty_columns--;
	}
	
	$empty_columns_arr = array_fill(0, $empty_columns, '');
	$columns = array_merge($columns, $empty_columns_arr);
	if (array_var($report_data, 'include_estimated_billing')) {
		$columns[] = 'estimated_expense_billing';
	}
	if ($use_cost) $columns[] = ' ';
	if ($use_price) $columns[] = ' ';
	if ($use_cost) $columns[] = 'total_cost';
	if ($use_price) $columns[] = 'totalbilling';
	if ($show_invoice_detail) {
		$columns[] = 'invoiced';
		$columns[] = 'not_invoiced';
	}
	//$empty_columns += array_var($report_data, 'include_estimated_billing') ? 1 : 0;
	if (array_var($report_data, 'include_pending_time')) {
		$columns[] = ' ';
	}
	if (array_var($report_data, 'include_pending_billing')) {
		$columns[] = 'pending_expense_billing';
	}
	if (array_var($report_data, 'include_rate_schedule')) {
		$columns[] = '';
	}
	if (array_var($report_data, 'include_approval_status')) {
		$columns[] = '';
	}
	if (array_var($report_data, 'include_modules_dim')) {
		$columns[] = '';
	}

	?>
	<tr class="header-custom-report custom-report-table-heading2">
	<?php 
	foreach ($columns as $col) {
		$class = '';
		if (in_array($col, array('totalbilling', 'estimated_expense_billing', 'pending_expense_billing', 'invoiced', 'not_invoiced', 'total_cost'))) $class = 'right';
		if($col == 'totalbilling'){
			$col = 'totalbilling expenses summary';
		}
		if ($col == 'expense_category_summary') {
			echo '<th colspan="'.$name_colspan.'">'.lang($col).'</th>';
		} else {
			echo '<th class="'.$class.'">'.($col == '' ? '' : lang($col)).'</th>';
		}
	}
	?>
	</tr></thead>
	<tbody>
	<?php 
	foreach ($project_data['expenses_summary'] as $k => $summary) {
		if ($k == 'subtotal') {
			$row_class = "subtotal level-".$level;
			echo '<tr class="'.$row_class.'">';
		} else {
			echo '<tr>';
		}
		foreach ($columns as $col) {
			switch ($col) {
				case '':
					echo '<td>&nbsp;</td>';
					break;
				case 'expense_category_summary':
					echo '<td class="category-column" colspan="'.$name_colspan.'">'. $summary['expense_category'] .'</td>';
					break;
				case 'estimated_expense_billing':
					echo '<td class="right">'. format_money_amount($summary['estimated_expense_billing'], $summary['currency_sym']).'</td>';
					break;
				case 'pending_expense_billing':
					$pending_class = '';
					if($summary['pending_expense_billing'] < 0){
						$pending_class = 'red-font';
					}
					echo '<td class="right '.$pending_class.'">'. format_money_amount($summary['pending_expense_billing'], $summary['currency_sym']).'</td>';
					break;
				case 'totalbilling':
					echo '<td class="right">'. format_money_amount($summary['amount'], $summary['currency_sym']).'</td>';
					break;
				case 'total_cost':
					echo '<td class="right">'. format_money_amount($summary['expenses_total_cost'], $summary['currency_sym']).'</td>';
					break;
				case 'invoiced':
					echo '<td class="right">'. format_money_amount($summary['invoiced'], $summary['currency_sym']).'</td>';
					break;
				case 'not_invoiced':
					echo '<td class="right">'. format_money_amount($summary['not_invoiced'], $summary['currency_sym']).'</td>';
					break;
				default:
					echo '<td></td>';
					break;
			}
		}
		echo '</tr>';
	
	}
	?>
	</tbody><tbody style="border: 0px none;">
	<tr class="separator"><td colspan=<?php echo count($columns)?>>&nbsp;</td></tr>
	</tbody>
	<?php 
}



function render_project_totals($project_data, $report_data, $total_cols, $scope) {
	// *** LC 2023-09-04 *** $padding no definido, se define vacío
	$padding = "";  // ***

	if (empty($project_data['totals'])) return;
	$fin_type = array_var($report_data, 'financial_type', 'price');
	$use_cost = in_array($fin_type, array('cost', 'price_and_cost'));
	$use_price = in_array($fin_type, array('price', 'price_and_cost'));
	$project_billing_report_id = Reports::instance()->findOne(array("conditions" => "`code` = 'project_billing_report'"))->getId();

	echo '<table class="report custom-report project-mamagement-report"><thead>';
	if($scope == 'grand'){
		if($report_data['report_id'] == $project_billing_report_id && ( ! isset($report_data['display_grand_total']) || $report_data['display_grand_total'] != 'checked' )) return;
		echo '<tr class="header-custom-report custom-report-table-heading custom-report-table-heading-capture-totals"><th colspan="'.$total_cols.'" style='.$padding.'>'/*.$project_data['project_name'].' - '*/.lang('grand totals').'</th></tr>';
	} else if($scope == 'project') {
		echo '<tr class="header-custom-report custom-report-table-heading custom-report-table-heading-capture"><th colspan="'.$total_cols.'">'/*.$project_data['project_name'].' - '*/.lang('totals').'<span class="small-project-name"> '.$project_data['project_name'].'</span></th></tr>';
	}
	
	$columns = array('', '', 'totals summary');
	if($fin_type == 'price_and_cost') $columns[] = '';
	$billing_columns = array('worked_time');
	
	if($use_cost) $billing_columns[] = 'total_cost';
	if($use_price) $billing_columns[] = 'totalbilling';

	
	
	$show_invoice_detail = get_show_invoicing_detail($report_data);
	if (array_var($report_data, 'include_time_description')) {
		$columns[] = '';
		$columns[] = '';
	}
	if (array_var($report_data, 'include_miles')) {
		$columns[] = 'miles';
	}
	if (array_var($report_data, 'include_billing_rate_name')) {
		$columns[] = '';
	}
	//$columns[] = '';
	if (array_var($report_data, 'include_estimated_time')) {
		$columns[] = 'estimated_hours';
	}
	if (array_var($report_data, 'include_estimated_billing')) {
		$columns[] = 'estimated_billing';
	}
	if ($show_invoice_detail) {
		$billing_columns[] = 'invoiced';
		$billing_columns[] = 'not_invoiced';
	}
	if (array_var($report_data, 'include_pending_time')) {
		$billing_columns[] = 'pending_hours';
	}
	if (array_var($report_data, 'include_pending_billing')) {
		$billing_columns[] = 'pending_billing';
	}
	if (array_var($report_data, 'include_rate_schedule')) {
		$billing_columns[] = '';
	}
	if (array_var($report_data, 'include_approval_status')) {
		$billing_columns[] = '';
	}
	if (array_var($report_data, 'include_modules_dim')) {
		$billing_columns[] = '';
	}
	
	$columns = array_merge($columns, $billing_columns);
	
	if($scope == 'grand'){
		echo '<tr class="header-custom-report custom-report-table-heading custom-report-table-heading-total">';
	} else if($scope == 'project') {
		echo '<tr class="header-custom-report custom-report-table-heading2">';
	}

	?>
	<?php 
	foreach ($columns as $col) {
		$class = '';
		if (in_array($col, array('worked_time', 'totalbilling', 'total_cost', 'estimated_hours', 'estimated_billing', 'pending_billing', 'pending_hours', 'invoiced', 'not_invoiced'))) $class = 'right';
		else if (in_array($col,  array('miles'))) $class = 'center';
		if($col == 'totalbilling'){
			$col = 'totalbilling total summary';
		}
		$colspan = 1;
		if ($col == 'totals summary') {
			$class = 'category-column';
			$colspan = 2;
		}
		echo '<th class="'.$class.'" colspan="'.$colspan.'">'.($col == '' ? '' : lang($col)).'</th>';
	}
	?>
	</tr></thead>
	<tbody>
		<!-- Labor Totals Row -->
		<tr>
			<td></td><td></td>
			<td colspan="2"><?php echo lang('labor total') ?></td>
			<?php if (array_var($report_data, 'include_time_description')) {
				echo '<td></td>';
				echo '<td></td>';
			} ?>
			<?php if ($fin_type == 'price_and_cost') {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_miles')) {
				echo '<td class="right">'.number_format($project_data['totals']['miles'],  2, '.', ',').'</td>';
			} ?>
			<?php if (array_var($report_data, 'include_billing_rate_name')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_estimated_time')) {
				//echo '<td></td>';
				echo '<td class="right">'.number_format($project_data['totals']['estimated_hours'],  2, '.', ',').'</td>';
			} ?>
			<?php if (array_var($report_data, 'include_estimated_billing')) {
				echo '<td class="right">'.format_money_amount($project_data['totals']['labor_estimated_billing'], $project_data['totals']['currency_sym']).'</td>';
			} ?>
			<td class="right"><?php 
				echo number_format($project_data['totals']['worked_hours'],  2, '.', ',') 
			?></td>
			<?php  if (in_array('total_cost', $columns)) {
				echo '<td class="right">'. format_money_amount($project_data['totals']['timeslots_total_cost'], $project_data['totals']['currency_sym']) .' </td>';
			} ?>
			<?php  if (in_array('totalbilling', $columns)) {
				echo '<td class="right">'. format_money_amount($project_data['totals']['timeslots_total'], $project_data['totals']['currency_sym']) .' </td>';
			} ?>
			<?php if ($show_invoice_detail) {
				echo '<td class="right">'.format_money_amount($project_data['totals']['labor_invoiced'], $project_data['totals']['currency_sym']).'</td>';
				echo '<td class="right">'.format_money_amount($project_data['totals']['labor_not_invoiced'], $project_data['totals']['currency_sym']).'</td>';
			} ?>	
			<?php if (array_var($report_data, 'include_pending_time')) {
				$pending_class = '';
				if($project_data['totals']['pending_hours'] < 0){
					$pending_class = 'red-font';
				}
				echo '<td class="right '.$pending_class.'">'.number_format($project_data['totals']['pending_hours'],  2, '.', ',').'</td>';
				//echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_pending_billing')) {
				$pending_class = '';
				if($project_data['totals']['labor_pending_billing'] < 0){
					$pending_class = 'red-font';
				}
				echo '<td class="right '.$pending_class.'">'.format_money_amount($project_data['totals']['labor_pending_billing'], $project_data['totals']['currency_sym']).'</td>';
			} ?>
			<?php if (array_var($report_data, 'include_rate_schedule')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_approval_status')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_modules_dim')) {
				echo '<td></td>';
			} ?>
		</tr>
		<!-- Expenses Totals Row -->
		<tr>
			<td></td><td></td>
			<td colspan="2"><?php echo lang('expenses total') ?></td>
			<?php if (array_var($report_data, 'include_time_description')) {
				echo '<td></td>';
				echo '<td></td>';
			} ?>
			<?php if ($fin_type == 'price_and_cost') {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_billing_rate_name')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_miles')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_estimated_time')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_estimated_billing')) {
				echo '<td class="right">'.format_money_amount($project_data['totals']['estimated_expense_billing'], $project_data['totals']['currency_sym']).'</td>';
			} ?>
			<td></td>
			<?php  if (in_array('total_cost', $columns)) {
				echo '<td class="right">'. format_money_amount($project_data['totals']['expenses_total_cost'], $project_data['totals']['currency_sym']) .' </td>';
			} ?>
			<?php  if (in_array('totalbilling', $columns)) {
				echo '<td class="right">' .format_money_amount($project_data['totals']['expenses_total'], $project_data['totals']['currency_sym']) .'</td>';
			} ?>
			<?php if ($show_invoice_detail) {
				echo '<td class="right">'.format_money_amount($project_data['totals']['expense_invoiced'], $project_data['totals']['currency_sym']).'</td>';
				echo '<td class="right">'.format_money_amount($project_data['totals']['expense_not_invoiced'], $project_data['totals']['currency_sym']).'</td>';
			} ?>
			<?php if (array_var($report_data, 'include_pending_time')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_pending_billing')) {
				$pending_class = '';
				if($project_data['totals']['pending_expense_billing'] < 0){
					$pending_class = 'red-font';
				}
				echo '<td class="right '.$pending_class.'">'.format_money_amount($project_data['totals']['pending_expense_billing'], $project_data['totals']['currency_sym']).'</td>';
			} ?>
			<?php if (array_var($report_data, 'include_rate_schedule')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_approval_status')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_modules_dim')) {
				echo '<td></td>';
			} ?>
		</tr>
		<!-- Labor and Expenses Totals Row -->
		<tr class="subtotal level-0">
			<td></td><td></td>
			<td colspan="2"><?php echo lang('labor and expenses total') ?></td>
			<?php if (array_var($report_data, 'include_time_description')) {
				echo '<td></td>';
				echo '<td></td>';
			} ?>
			<?php if ($fin_type == 'price_and_cost') {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_miles')) {
				echo '<td class="right">'.number_format($project_data['totals']['miles'],  2, '.', ',').'</td>';
			} ?>
			<?php if (array_var($report_data, 'include_billing_rate_name')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_estimated_time')) {
				//echo '<td class="right">'.number_format($project_data['totals']['estimated_hours'],  2, '.', ',').'</td>';
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_estimated_billing')) {
				echo '<td class="right">'.format_money_amount($project_data['totals']['total_estimated_billing'], $project_data['totals']['currency_sym']).'</td>';
			} ?>
			<td class="right"><?php 
				//echo number_format($project_data['totals']['worked_hours'],  2, '.', ',');
			?></td>
			<?php  if (in_array('total_cost', $columns)) {
				$overall_total_cost = $project_data['totals']['timeslots_total_cost'] + $project_data['totals']['expenses_total_cost'];
				echo '<td class="right">'. format_money_amount($overall_total_cost, $project_data['totals']['currency_sym']) .' </td>';
			} ?>
			<?php  if (in_array('totalbilling', $columns)) {
				$overall_total_billing = $project_data['totals']['timeslots_total'] + $project_data['totals']['expenses_total'];
				echo '<td class="right">' .format_money_amount($overall_total_billing, $project_data['totals']['currency_sym']) .'</td>';
			} ?>
			<?php if ($show_invoice_detail) {
				echo '<td class="right">'.format_money_amount($project_data['totals']['total_invoiced'], $project_data['totals']['currency_sym']).'</td>';
				echo '<td class="right">'.format_money_amount($project_data['totals']['total_not_invoiced'], $project_data['totals']['currency_sym']).'</td>';
			} ?>
			<?php if (array_var($report_data, 'include_pending_time')) {
				//echo '<td class="right">'.number_format($project_data['totals']['pending_hours'],  2, '.', ',').'</td>';
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_pending_billing')) {
				$pending_class = '';
				if($project_data['totals']['total_pending_billing'] < 0){
					$pending_class = 'red-font';
				}
				echo '<td class="right '.$pending_class.'">'.format_money_amount($project_data['totals']['total_pending_billing'], $project_data['totals']['currency_sym']).'</td>';
			} ?>
			<?php if (array_var($report_data, 'include_rate_schedule')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_approval_status')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_modules_dim')) {
				echo '<td></td>';
			} ?>
		</tr>
		<!-- Project Budget Total Row -->
		<?php if ((in_array(array_var($report_data, 'for_budget_use'), array('tasks_expenses', 'budget_total_cp')))) { ?>
		<tr class="subtotal level-0">
			<td></td><td></td>
			<?php if(array_var($report_data, 'for_budget_use') == 'tasks_expenses'){ ?>
				<td colspan="2"><?php echo lang('sum of budgeted tasks + expenses') ?></td>
			<?php } else if (array_var($report_data, 'for_budget_use') == 'budget_total_cp'){ 
				$project_object_type = ObjectTypes::instance()->findOne(array("conditions" => "`name`='project'"));
				$cp_object_type_id = $project_object_type->getId();
				$budget_custom_property = CustomProperties::instance()->findOne(array('conditions' => '`object_type_id`='.$cp_object_type_id. ' AND `code`="budget_total"')); ?>
				<td colspan="2"><?php echo $budget_custom_property->getName(); ?></td>	
			<?php } ?>
			<?php if (array_var($report_data, 'include_time_description')) {
				echo '<td></td>';
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_miles')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_billing_rate_name')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_estimated_time')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_estimated_billing')) {
				echo '<td></td>';
			} ?>
			<td class="right"><?php 
			?></td>
			<td class="right"><?php echo format_money_amount($project_data['totals']['project_total_budget'], $project_data['totals']['currency_sym']) ?></td>
			<?php if ($show_invoice_detail) {
				echo '<td></td>';
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_pending_time')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_pending_billing')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_rate_schedule')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_approval_status')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_modules_dim')) {
				echo '<td></td>';
			} ?>
		</tr>
		<?php }?>
		<!-- Project Budget Total Difference Row -->
		<?php if ((in_array(array_var($report_data, 'for_budget_use'), array('tasks_expenses', 'budget_total_cp')))) { ?>
		<tr class="subtotal level-0">
			<td></td><td></td>
			<td colspan="2"><?php echo lang('budget consumption') ?></td>
			<?php if (array_var($report_data, 'include_time_description')) {
				echo '<td></td>';
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_miles')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_billing_rate_name')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_estimated_time')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_estimated_billing')) {
				echo '<td></td>';
			} ?>
			<td class="right"><?php 
			?></td>
			<?php 
				$budget_difference = $project_data['totals']['project_total_budget'] == 0 ? 0 :($project_data['totals']['timeslots_total'] + $project_data['totals']['expenses_total'])/ $project_data['totals'] ['project_total_budget'] * 100;
				$diff_style = $budget_difference > 100 ? "color: red;" : "";
			?>
			<td class="right" style="<?php echo $diff_style; ?>"><?php echo round($budget_difference) . "%" ?></td>
			<?php if ($show_invoice_detail) {
				echo '<td></td>';
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_pending_time')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_pending_billing')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_rate_schedule')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_approval_status')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_modules_dim')) {
				echo '<td></td>';
			} ?>
		</tr>
		<?php }?>
		<!-- Project Contract Type -->
		<?php 
		$project_object_type = ObjectTypes::instance()->findOne(array("conditions" => "`name`='project'"));
		$cp_object_type_id = $project_object_type->getId();
		$contract_type_custom_property = CustomProperties::instance()->findOne(array('conditions' => '`object_type_id`='.$cp_object_type_id. ' AND `code`="contract_type"'));
		if ((in_array(array_var($report_data, 'for_budget_use'), array('tasks_expenses', 'budget_total_cp'))) && $contract_type_custom_property instanceof CustomProperty) { ?>
		<tr class="subtotal level-0">
			<td></td><td></td>
			<td colspan="2"><?php echo $contract_type_custom_property->getName() ?></td>
			<?php if (array_var($report_data, 'include_time_description')) {
				echo '<td></td>';
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_miles')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_billing_rate_name')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_estimated_time')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_estimated_billing')) {
				echo '<td></td>';
			} ?>
			<td class="right"><?php 
			?></td>
			<td class="right"><?php echo $project_data['totals']['contract_type']; ?></td>
			<?php if ($show_invoice_detail) {
				echo '<td></td>';
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_pending_time')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_pending_billing')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_rate_schedule')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_approval_status')) {
				echo '<td></td>';
			} ?>
			<?php if (array_var($report_data, 'include_modules_dim')) {
				echo '<td></td>';
			} ?>
		</tr>
		<?php }?>
	</tbody><tbody style="border: 0px none;">
		<tr class="separator"><td colspan=<?php echo count($columns)?>>&nbsp;</td></tr>
	</tbody>
	</table>
	<?php 
}


function render_project_report_main_information(Member $project_member, $properties_array, $total_cols, $params = array()) {
	
	$tmp_members = array($project_member);
	build_member_list_text_to_show_in_trees($tmp_members);
	tpl_assign('add_indentation',  array_var($params, 'add_indentation'));
	tpl_assign('project_member', $project_member);
	tpl_assign('properties_array', $properties_array);
	$project_data = array_var($params, 'project_data', array());
	
	$html = tpl_fetch(get_template_path('project_management_report_properties', 'project_reports', 'project_reports'));
	
	// dummy empty thead to avoid putting the properties in every page
	echo '<thead class="project-info dummy"><tr><td colspan="'.$total_cols.'"></td></tr></thead>';
	
	// project properties
	echo '<thead class="project-info">
			<tr>
				<td colspan="'.$total_cols.'" class="project-td" style="padding-left: 0px !important;">'.$html;
				if (array_var($report_data, 'show_client_and_start_date')) {
					echo '<div style="font-size: 15px; margin-top: 0px; margin-bottom: 1px; font-weight: bold;">'.lang('client').': '.$project_data['client_name'].'</div>
					<div style="font-size: 13px; margin-top: 1px; margin-bottom: 5px;">'.lang('start date').': '.$project_data['project_start_date'].'</div>';
				}
	echo '</td>
		</tr>
	</thead>';
	
}



function render_project_report_project_table($project_data, $report_data, $show_main_information_blocks, $total_cols, $level = 0, $insert_page_break=false) {
	
	if($insert_page_break){
		echo '<div style="page-break-inside: avoid;">';
	}
	
	echo '<table class="report custom-report project-mamagement-report">';
	
	//echo '<tbody style="display:block; page-break-before: always; border:0px none;"><tr><td colspan="'.$total_cols.'"></td></tr></tbody>';
	$padding = $level > 0 ? "padding-left:". ($level * 30) . "px" : "";
	// dummy empty thead to avoid putting the properties in every page
	//echo '<thead class="project-info dummy"><tr><td colspan="'.$total_cols.'"></td></tr></thead>';
	
	if ($show_main_information_blocks && !(empty($project_data['timeslots_summary']) && empty($project_data['expenses_summary']))) {
		$project_member = Members::instance()->findById($project_data['project_id']);
		if ($project_member instanceof Member) {
			$properties_array = '';
			$params =  array(
					'member' => $project_member,
					'visible_by_default' => true,
					'return_as_array' => true,
					'project_data' => $project_data
			);
			Hook::fire('override_render_properties_for_view', $params, $properties_array);
			
			render_project_report_main_information($project_member, $properties_array, $total_cols, $params);
		}
	}
	
	
	if (empty($project_data['timeslots_data']) 
		&& empty($project_data['expenses_data']) 
		&& count($project_data['subprojects']) == 0
		&& (!(array_var($report_data, 'include_estimated_time') || array_var($report_data, 'include_pending_time')) || empty($project_data['timeslots_summary']) ) ) {
				echo '</table>';
				return;
	}
	
	if (!$show_main_information_blocks && (!empty($project_data['timeslots_data']) || !empty($project_data['expenses_data']) || !empty($project_data['timeslots_summary']) || !empty($project_data['expenses_summary']))) {
		if ($level == 0){
			echo '<thead>
					<tr class="header-custom-report project-report-project-header">
						<th colspan="'.$total_cols.'" style='.$padding.'>
							<span style="color: #0a85d8; margin-right: 20px;">'.lang('project').'</span>'.$project_data['project_name'];

							if (array_var($report_data, 'show_client_and_start_date')) {
								echo '<div style="font-size: 15px; margin-top: 10px; margin-bottom: 1px; font-weight: bold;">'.lang('client').': '.$project_data['client_name'].'</div>
								<div style="font-size: 13px; margin-top: 1px; margin-bottom: 5px;">'.lang('start date').': '.$project_data['project_start_date'].'</div>';
							}
			echo '</th>
			</tr></thead>';
		} else {
			echo '<thead>
					<tr class="header-custom-report project-report-project-header'.$level.'">
						<th colspan="'.$total_cols.'" style='.$padding.'>
							<span style="color: #999; margin-right: 15px;">'.lang('project').'</span>'.$project_data['project_name'];
							if (array_var($report_data, 'show_client_and_start_date')) {
								echo '<div style="font-size: 13px; margin-top: 1px; margin-bottom: 5px;">'.lang('start date').': '.$project_data['project_start_date'].'</div>';
							}
			echo '</th>
				</tr>
			</thead>';
		}
	}
	
	echo '</table>';
	//render_project_report_timeslots_data($project_data, $total_cols, $level);
	
	//render_project_report_expenses_data($project_data, $total_cols, $level);
	
	$subprojects_data = array_var($project_data, 'subprojects');
	$sub_level = $level + 1;
	if (count($subprojects_data) > 0) {
		foreach ($subprojects_data as $i=>$subproject_data) {
			//$padding = $sub_level > 0 ? "padding-left:". ($sub_level * 30) . "px" : "";
			//echo '<thead><tr class="header-custom-report custom-report-table-heading"><th colspan="'.$total_cols.'" style='.$padding.'>'.$subproject_data['project_name'].'</th></tr></thead>';
			
			$insert_pb = array_var($report_data, 'project_page_break') && $i < count($subprojects_data)-1;
			
			render_project_report_project_table($subproject_data, $report_data, false, $total_cols, $sub_level, $insert_pb);
		}
	}
	
	echo '<table class="report custom-report project-mamagement-report timeslots">';
	render_project_timeslots_table($project_data, $report_data, $total_cols, $level);
	echo '</table>';
	
	$subprojects_have_timeslots = false;
	foreach ($subprojects_data as $subproject_data) {
		if (count($subproject_data['timeslots_data']) > 0) {
			$subprojects_have_timeslots = true;
			break;
		}
	}
	if (!$subprojects_have_timeslots && count($project_data['timeslots_data']) != 0) {
		echo '<table class="report custom-report project-mamagement-report summary">';
		render_project_timeslots_summary($project_data, $report_data, $total_cols, $level);
		echo '</table>';
	}
	
	echo '<table class="report custom-report project-mamagement-report expenses">';
	render_project_expenses_table($project_data, $report_data, $total_cols, $level);
	echo '</table>';	

	$subprojects_have_expenses = false;
	foreach ($subprojects_data as $subproject_data) {
		if (count($subproject_data['expenses_data']) > 0) {
			$subprojects_have_expenses = true;
			break;
		}
	}

	if (!$subprojects_have_expenses && count($project_data['expenses_data']) != 0) {
		echo '<table class="report custom-report project-mamagement-report summary">';
		render_project_expenses_summary($project_data, $report_data, $total_cols, $level);
		echo '</table>';
	}
	
	$project_member_ids = json_decode(array_var($report_data, 'project_member_ids', '[]'), true);
	if(count($subprojects_data) == 0  && count($project_member_ids) == 0){
		render_project_totals($project_data, $report_data, $total_cols, 'project');
	}
	
	echo '</div>';
	
	if ($insert_page_break) echo '<table class="report custom-report project-mamagement-report page-break"></table>';
}

/*
function render_project_report_timeslots_data($project_data, $total_cols, $level) {
	
	$subprojects_data = array_var($project_data, 'subprojects');
	
	render_project_timeslots_table($project_data, $total_cols, $level);
	
	$sub_level = $level + 1;
	
	if (count($subprojects_data) > 0) {
		foreach ($subprojects_data as $subproject_data) {
			
			render_project_report_timeslots_data($subproject_data, $total_cols, $sub_level);
		}
	}
	
	render_project_timeslots_summary($project_data, $level);
	
}

function render_project_report_expenses_data($project_data, $total_cols, $level) {
	
	$subprojects_data = array_var($project_data, 'subprojects');
	
	render_project_expenses_table($project_data, $total_cols, $level);
	
	$sub_level = $level + 1;
	
	if (count($subprojects_data) > 0) {
		foreach ($subprojects_data as $subproject_data) {
			
			render_project_report_expenses_data($subproject_data, $total_cols, $sub_level);
		}
	}
	
	render_project_expenses_summary($project_data, $level);
	
}
*/

?>


<?php

$total_cols = 6;
$report_data = array_var($_SESSION, 'project_management_report_data');
if (array_var($report_data, 'include_billing_rate_name')) {
	$total_cols++;
}
if (array_var($report_data, 'include_estimated_time')) {
	$total_cols++;
}
if (array_var($report_data, 'include_estimated_billing')) {
	$total_cols++;
}
if (array_var($report_data, 'include_pending_billing')) {
	$total_cols++;
}
if (array_var($report_data, 'include_pending_time')) {
	$total_cols++;
}
if (array_var($report_data, 'include_miles')) {
	$total_cols++;
}


$projects_data = array_var($data, 'projects_data');
if(array_var($report_data, 'start_date') && array_var($report_data, 'end_date')){
	// Get Start Date value
	$date_format = user_config_option('date_format');
	$start_date = $report_data['start_date'];
	$date_value_start_date = getDateValue($start_date);
	if (!$date_value_start_date instanceof DateTimeValue) {
		$start_date_string = lang('none selected');
	} else {
		$start_date_string = $date_value_start_date->add('s',logged_user()->getUserTimezoneValue())->format($date_format);
	}
	// Get End Date value
	$end_date = $report_data['end_date'];
	$date_value_end_date = getDateValue($end_date);
	if (!$date_value_end_date instanceof DateTimeValue) {
		$end_date_string = lang('none selected');
	} else {
		$end_date_string = $date_value_end_date->add('s',logged_user()->getUserTimezoneValue())->format($date_format);
	}
	if(array_var($report_data, 'show_billing_period')){
		// Build html with Date filters
		echo "<div class='project-reports__filters'>";
		echo "<p>" . lang('start date') . ': ' . $start_date_string . "</p>";
		echo "<p>" . lang('end date') . ': ' . $end_date_string . "</p>";
		echo "</div>";
	}	
}
foreach ($projects_data as $i => $project_data) {
	$insert_page_break = array_var($report_data, 'project_page_break') && $i < count($projects_data) - 1;
	render_project_report_project_table($project_data, $report_data, $show_main_information_blocks, $total_cols, 0, $insert_page_break);
	
}

render_project_totals($data, $report_data, $total_cols, 'grand');

?>
