<?php

Env::useHelper('view_functions', 'project_reports');

$report_data = array_var($_SESSION, 'project_summary_report_data');
$report_data['is_pdf_export'] = isset($is_pdf_export) ? $is_pdf_export : false;

$from_date_str = format_date($report_data['start_date']);
$to_date_str = format_date($report_data['end_date']);

// Render report title
echo '<div class="project-report-title">'.$title.'</div>';

// Render date of the report
$today_dtv = DateTimeValueLib::now();
$today_date = format_date($today_dtv);
echo '<div class="project-report-info" style="float: right;">'.lang('report date').': '.$today_date.'</div>';

echo '<div class="project-report-info" style="margin-bottom: 20px;">'.lang('showing data from x to y', $from_date_str, $to_date_str).'</div>';

echo '<div class="clear"></div>';

function render_project_summary_report($data, $report_data) {
    $is_pdf_export = array_var($report_data, 'is_pdf_export', false);
    $table_zoom = $is_pdf_export ? 'zoom: 0.8;' : '';
    // Start table
    echo '<table class="budgetReport" style="width: 100%;'.$table_zoom.'">';
    render_main_header_project_summary_report($data, $report_data);
    render_overall_totals_project_summary_report($data, $report_data);
    $project_managers = array_var($data, 'project_managers', array());
    foreach($project_managers as $pm_data) {
        render_project_manager_header_project_summary_report($pm_data, $data);
        render_project_manager_projects_project_summary_report($pm_data, $data, $report_data);
    }   
    // Close table
    echo '</table>';
}

function render_project_manager_header_project_summary_report($pm_data, $data) {
    $columns = array_var($data, 'columns', array());
    $totals = $pm_data['totals'];
    $currency_sym = config_option('currency_code');
    $pm_name = $pm_data['project_manager_name'];

    // Start header
    echo '<thead class="titles"><tr>
            <th class="header_0" colspan="4">'.$pm_name.'</th>';
    foreach($columns as $column) {
        if($column == 'name') continue;
        $money_val = format_money_amount($totals[$column], $currency_sym);
        echo '<th class="header_0 right" colspan="2">'.$money_val.'</th>';
    }
      
    // Close header
    echo '</tr></thead>';
}

function render_project_manager_projects_project_summary_report($pm_data, $data, $report_data) {
    $columns = array_var($data, 'columns', array());
    $currency_sym = config_option('currency_code');
    

    $projects = array_var($pm_data, 'projects', array());
    echo '<tbody>';
    foreach($projects as $project) {
        // Start project
        echo '<tr class="parentRow">
                <td class="left" colspan="4">'.$project['name'].'</td>';
        foreach($columns as $column) {
            if($column == 'name') continue;
            $money_val = format_money_amount($project[$column], $currency_sym);
            echo '<td class="right" colspan="2">'.$money_val.'</td>';
        }
        // Close project
        echo '</tr>';
    }
    echo '</tbody>';
            
}

function render_main_header_project_summary_report($data, $report_data) {
    $columns = array_var($data, 'columns', array());

    // Start header
    
    echo '<thead class="tHeadTitles"><tr>
            <th class="left bold" colspan="4"></th>';
    foreach($columns as $column) {
        if($column == 'name') continue;
        if($column == 'budget_total') {
            $column_name = $data['budget_total_cp_name'];
        } else if ($column == 'revised_budget_total') {
            $column_name = $data['revised_budget_total_cp_name'];
        } else if ($column == 'executed_cost') {
            $column_name = lang('executed cost');
        } else if ($column == 'invoiced_subtotal') {
            $column_name = lang('invoiced');
        } else if ($column == 'invoiced_paid') {
            $column_name = lang('invoiced paid');
        } else {
            $column_name = lang($column);
        }

        echo '<th class="center bold" colspan="2">'.$column_name.'</th>';
    }

    // Close header
    echo '</tr></thead>';
}

function render_overall_totals_project_summary_report($data, $report_data) {
    $columns = array_var($data, 'columns', array());
    $totals = array_var($data, 'totals', array());
    $currency_sym = config_option('currency_code');

    // Start totals
    echo '<tbody><tr class="titles">
            <td class="left header_1" colspan="4">'.lang('totals').'</td>';
    foreach($columns as $column) {
        if($column == 'name') continue;
        $money_val = format_money_amount($totals[$column], $currency_sym);
        echo '<td class="right header_1" colspan="2">'.$money_val.'</td>';
    }

    // Close totals
    echo '</tr></tbody>';
}

render_project_summary_report($data, $report_data);
