<?php
/**
 * QboViewer controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class QboViewerController extends ApplicationController {

	var $plugin_name = "quickbooks";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
		// Include Quickbooks Online SDK to use the API
		Env::useLibrary('qbo_sdk', $this->plugin_name);
		Env::useHelper('qbo_render_functions', $this->plugin_name);
		Env::useHelper('qbo_functions', $this->plugin_name);
		Env::useHelper('qbo_functions_incoming', $this->plugin_name);
		Env::useHelper('qbo_viewer', $this->plugin_name);
		
	}


	function index() {

		$entitites = array(
			'Clients' => get_url('qbo_viewer', 'view_clients', array('first'=>true)),
			'Invoices' => get_url('qbo_viewer', 'view_invoices', array('first'=>true)),
			'Accounts' => get_url('qbo_viewer', 'view_accounts', array('first'=>true)),
			'Items' => get_url('qbo_viewer', 'view_items', array('first'=>true)),
			'Company info' => get_url('qbo_viewer', 'view_company_info', array('first'=>true)),
			'System preferences' => get_url('qbo_viewer', 'view_preferences', array('first'=>true)),
		);

		tpl_assign('entities', $entitites);

	}


	function view_custom_query() {

		$query = array_var($_REQUEST, 'query', '');
		$start_pos = array_var($_REQUEST, 'start', 1);
		$limit = array_var($_REQUEST, 'limit', 1000);

		$_SESSION['qbo_viewer_last_query'] = $query;

		$data_service = get_qbo_data_service();

		$query_str = $query . " STARTPOSITION $start_pos MAXRESULTS $limit";
		$objects = $data_service->Query($query_str);

		tpl_assign('base_url', get_url('qbo_viewer', __FUNCTION__));
		tpl_assign('objects', $objects);
		tpl_assign('query', $query);

	}

	function view_invoice_by_number() {
		$number = array_var($_REQUEST, 'number', 1);
		
		$data_service = get_qbo_data_service();

		$objects = $data_service->Query("select * from Invoice where DocNumber='$number'");
		$object = $objects[0];
		
		tpl_assign('number', $number);
		tpl_assign('object', $object);
	}


	function view_invoices() {

		if (!array_var($_REQUEST, 'first')) {
			ajx_replace(true);
		}

		$start_pos = array_var($_REQUEST, 'start_pos', 1);
		$page_size = 50;

		$data_service = get_qbo_data_service();

		$objects = $data_service->Query("select * from Invoice STARTPOSITION $start_pos MAXRESULTS  $page_size");
		
		tpl_assign('start_pos', $start_pos);
		tpl_assign('next_start_pos', $start_pos + $page_size);
		tpl_assign('page_size', $page_size);
		tpl_assign('base_url', get_url('qbo_viewer', __FUNCTION__));
		tpl_assign('objects', $objects);

	}


	function view_client_invoices() {

		if (!array_var($_REQUEST, 'first')) {
			ajx_replace(true);
		}

		$client_id = array_var($_REQUEST, 'id');
		$start_pos = array_var($_REQUEST, 'start_pos', 1);
		$page_size = 50;

		$data_service = get_qbo_data_service();

		$objects = $data_service->Query("select * from Invoice WHERE CustomerRef = '$client_id' STARTPOSITION $start_pos MAXRESULTS  $page_size");
		
		tpl_assign('start_pos', $start_pos);
		tpl_assign('next_start_pos', $start_pos + $page_size);
		tpl_assign('page_size', $page_size);
		tpl_assign('base_url', get_url('qbo_viewer', __FUNCTION__));
		tpl_assign('objects', $objects);

		$this->setTemplate('view_invoices');
	}


	function view_clients() {

		if (!array_var($_REQUEST, 'first')) {
			ajx_replace(true);
		}

		$start_pos = array_var($_REQUEST, 'start_pos', 1);
		$page_size = 50;
		
		$text_filter = array_var($_REQUEST, 'text_filter', '');
		$where_clause = '';
		if ($text_filter != '') {
			$where_clause = "WHERE FullyQualifiedName LIKE '%$text_filter%'";
		}

		$data_service = get_qbo_data_service();

		$objects = $data_service->Query("select * from Customer $where_clause STARTPOSITION $start_pos MAXRESULTS  $page_size");

		foreach ($objects as $obj) {
			$view_invoices_link = get_url('qbo_viewer', 'view_client_invoices', array('id'=>$obj->Id, 'first'=>true));
			$obj->ViewInvoicesLink = '<a class="internalLink" href="'.$view_invoices_link.'">View Invoices</a>';
		}
		
		tpl_assign('start_pos', $start_pos);
		tpl_assign('next_start_pos', $start_pos + $page_size);
		tpl_assign('page_size', $page_size);
		tpl_assign('base_url', get_url('qbo_viewer', __FUNCTION__));
		tpl_assign('objects', $objects);

	}


	function view_accounts() {

		if (!array_var($_REQUEST, 'first')) {
			ajx_replace(true);
		}

		$start_pos = array_var($_REQUEST, 'start_pos', 1);
		$page_size = 50;
		
		$text_filter = array_var($_REQUEST, 'text_filter', '');
		$where_clause = '';
		if ($text_filter != '') {
			$where_clause = "WHERE FullyQualifiedName LIKE '%$text_filter%'";
		}

		$data_service = get_qbo_data_service();

		$objects = $data_service->Query("select * from Account $where_clause STARTPOSITION $start_pos MAXRESULTS  $page_size");
		
		tpl_assign('start_pos', $start_pos);
		tpl_assign('next_start_pos', $start_pos + $page_size);
		tpl_assign('page_size', $page_size);
		tpl_assign('base_url', get_url('qbo_viewer', __FUNCTION__));
		tpl_assign('objects', $objects);

	}


	function view_items() {

		if (!array_var($_REQUEST, 'first')) {
			ajx_replace(true);
		}

		$start_pos = array_var($_REQUEST, 'start_pos', 1);
		$page_size = 50;
		
		$text_filter = array_var($_REQUEST, 'text_filter', '');
		$where_clause = '';
		if ($text_filter != '') {
			$where_clause = "WHERE FullyQualifiedName LIKE '%$text_filter%'";
		}

		$data_service = get_qbo_data_service();

		$objects = $data_service->Query("select * from Item $where_clause STARTPOSITION $start_pos MAXRESULTS  $page_size");
		
		tpl_assign('start_pos', $start_pos);
		tpl_assign('next_start_pos', $start_pos + $page_size);
		tpl_assign('page_size', $page_size);
		tpl_assign('base_url', get_url('qbo_viewer', __FUNCTION__));
		tpl_assign('objects', $objects);

	}


	function view_company_info() {

		if (!array_var($_REQUEST, 'first')) {
			ajx_replace(true);
		}

		$start_pos = array_var($_REQUEST, 'start_pos', 1);
		$page_size = 50;

		$data_service = get_qbo_data_service();

		$object = $data_service->getCompanyInfo();
		
		tpl_assign('start_pos', $start_pos);
		tpl_assign('next_start_pos', $start_pos + $page_size);
		tpl_assign('page_size', $page_size);
		tpl_assign('base_url', get_url('qbo_viewer', __FUNCTION__));
		tpl_assign('object', $object);

	}


	function view_preferences() {

		if (!array_var($_REQUEST, 'first')) {
			ajx_replace(true);
		}

		$start_pos = array_var($_REQUEST, 'start_pos', 1);
		$page_size = 50;

		$data_service = get_qbo_data_service();

		$object = $data_service->getCompanyPreferences();
		
		tpl_assign('start_pos', $start_pos);
		tpl_assign('next_start_pos', $start_pos + $page_size);
		tpl_assign('page_size', $page_size);
		tpl_assign('base_url', get_url('qbo_viewer', __FUNCTION__));
		tpl_assign('object', $object);

	}
	
}
