<?php
/**
 * Quickbooks controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class QuickbooksController extends ApplicationController {

	var $plugin_name = "quickbooks";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		
		// Include Quickbooks Online SDK to use the API
		Env::useLibrary('qbo_sdk', $this->plugin_name);
		Env::useHelper('qbo_render_functions', $this->plugin_name);
		Env::useHelper('qbo_functions', $this->plugin_name);
		Env::useHelper('qbo_functions_incoming', $this->plugin_name);
		Env::useHelper('ts_functions', $this->plugin_name);
		
		// Include tsheets SDK to use the API
		Env::useLibrary('tsheets_sdk', $this->plugin_name);

	}
	
	function admin_connect() {
		evt_add("login_to_quickbooks", array(
				'need_qbo_login' => true,
				'action' => 'reload_current_panel',
				'id' => 'dummy',
				'auth_url' => get_qbo_auth_url(),
		));
		ajx_current("empty");
	}
	
	function reload_current_panel() {
		ajx_current("reload");
	}
	
	function admin_disconnect() {
		set_config_option('qbo_access_token', '');
		ajx_current("reload");
	}
	
	function edit_hour_services_match() {
		
		$accessToken = get_qbo_access_token();
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			evt_add("login_to_quickbooks", array(
					'need_qbo_login' => true,
					'action' => 'edit_hour_services_match',
					'id' => 'dummy',
					'auth_url' => get_qbo_auth_url(),
			));
			ajx_current("empty");
			return;
		}
		
		$items_match = array_var($_REQUEST, 'items_match');
		if (!$items_match) {
			// show the form
			
			$cf_matches_rows = DB::executeAll("SELECT * FROM ".TABLE_PREFIX."qbo_user_hour_items_match");
			$qbo_items_matches = array();
			foreach ($cf_matches_rows as $row) {
				$qbo_items_matches[$row['user_id']] = $row['qbo_item_id'];
			}
			$users = Contacts::getAllUsers();
			
			tpl_assign('users', $users);
			tpl_assign('qbo_items', get_qbo_items_definition(null, 'Service'));
			tpl_assign('qbo_items_matches', $qbo_items_matches);
			
		} else {
			// save the data
			
			foreach ($items_match as $user_id => $item_id) {
				DB::execute("
					INSERT INTO ".TABLE_PREFIX."qbo_user_hour_items_match (user_id, qbo_item_id) VALUES
					($user_id, '$item_id')
					ON DUPLICATE KEY UPDATE qbo_item_id='$item_id';
				");
			}
			
			flash_success(lang('properties saved successfully'));
			ajx_current("back");
		}
		
	}
	
	function edit_taxes_match() {
		
		$accessToken = get_qbo_access_token();
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			evt_add("login_to_quickbooks", array(
					'need_qbo_login' => true,
					'action' => 'edit_taxes_match',
					'id' => 'dummy',
					'auth_url' => get_qbo_auth_url(),
			));
			ajx_current("empty");
			return;
		}
		
		$taxes_match = array_var($_REQUEST, 'taxes_match');
		if (!$taxes_match) {
			// show the form
			
			$feng_taxes = IncomeTaxes::instance()->findAll();
			$qbo_taxes = get_all_qbo_taxes();
			
			tpl_assign('feng_taxes', $feng_taxes);
			tpl_assign('qbo_taxes', $qbo_taxes);
			
		} else {
			// save the data
			try {
				DB::beginWork();

				foreach ($taxes_match as $feng_tax_id => $qbo_tax_id) {
					$income_tax = IncomeTaxes::instance()->findById($feng_tax_id);
					if ($income_tax) {
						$income_tax->setColumnValue('quickbooks_online_id', $qbo_tax_id);
						$income_tax->save();
					}
				}

				DB::commit();
			} catch (Exception $e) {
				DB::rollback();
				ajx_current("empty");
				flash_error($e->getMessage());
				Logger::log_r($e);
			}
			
			flash_success(lang('properties saved successfully'));
			ajx_current("back");
		}
	}
	
	function edit_custom_field_match() {
		
		$accessToken = get_qbo_access_token();
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			evt_add("login_to_quickbooks", array(
					'need_qbo_login' => true,
					'action' => 'edit_custom_field_match',
					'id' => 'dummy',
					'auth_url' => get_qbo_auth_url(),
			));
			ajx_current("empty");
			return;
		}
		
		$custom_field_match = array_var($_REQUEST, 'custom_field_match');
		if (!$custom_field_match) {
			// show the form
			
			$cf_matches_rows = DB::executeAll("SELECT * FROM ".TABLE_PREFIX."qbo_custom_field_match");
			$cf_matches = array();
			foreach ($cf_matches_rows as $row) {
				$cf_matches[$row['quickbooks_id']] = $row;
			}
			
			tpl_assign('qbo_custom_fields', get_qbo_custom_fields_definition());
			tpl_assign('qbo_custom_field_matches', $cf_matches);
			
		} else {
			// save the data
			
			foreach ($custom_field_match as $qbo_id => $cf_match) {
				foreach ($cf_match as $order => $value) {
					if ($order == 1) $feng_prop1 = $value;
					if ($order == 2) $feng_prop2 = $value;
				}
				DB::execute("
					INSERT INTO ".TABLE_PREFIX."qbo_custom_field_match (quickbooks_id, feng_property1, feng_property2) VALUES
					($qbo_id, '$feng_prop1', '$feng_prop2')
					ON DUPLICATE KEY UPDATE feng_property1='$feng_prop1', feng_property2='$feng_prop2';
				");
			}
			
			flash_success(lang('properties saved successfully'));
			ajx_current("back");
		}
	}

	function edit_account_categories_match() {
		
		$accessToken = get_qbo_access_token();
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			evt_add("login_to_quickbooks", array(
					'need_qbo_login' => true,
					'action' => 'edit_account_categories_match',
					'id' => 'dummy',
					'auth_url' => get_qbo_auth_url(),
			));
			ajx_current("empty");
			return;
		}
		
		$category_match = array_var($_REQUEST, 'category_match');
		if (!$category_match) {
			// show the form
			
			$matches_rows = DB::executeAll("SELECT * FROM ".TABLE_PREFIX."qbo_accounts_category_match");
			$qbo_cat_account_matches = array();
			foreach ($matches_rows as $row) {
				$qbo_cat_account_matches[$row['category_id']] = $row['qbo_account_id'];
			}
			
			$labor_categories_dim = Dimensions::findByCode('hour_types');
			$expense_categories_dim = Dimensions::findByCode('expense_categories');

			$labor_categories_by_parent = array();
			if ($labor_categories_dim instanceof Dimension) {
				$labor_categories = Members::instance()->findAll(array(
					"conditions" => "dimension_id = ".$labor_categories_dim->getId(),
					"order" => "depth, name",
				));

				foreach ($labor_categories as $category) {
					if (!isset($labor_categories_by_parent[$category->getParentMemberId()])) {
						$labor_categories_by_parent[$category->getParentMemberId()] = array();
					}
					$labor_categories_by_parent[$category->getParentMemberId()][] = $category;
				}

			}

			$expense_categories_by_parent = array();
			if ($expense_categories_dim instanceof Dimension) {
				$expense_categories = Members::instance()->findAll(array(
					"conditions" => "dimension_id = ".$expense_categories_dim->getId(),
					"order" => "depth, name",
				));

				foreach ($expense_categories as $category) {
					if (!isset($expense_categories_by_parent[$category->getParentMemberId()])) {
						$expense_categories_by_parent[$category->getParentMemberId()] = array();
					}
					$expense_categories_by_parent[$category->getParentMemberId()][] = $category;
				}

			}

			tpl_assign('labor_categories_dim', $labor_categories_dim);
			tpl_assign('expense_categories_dim', $expense_categories_dim);
			tpl_assign('labor_categories_by_parent', $labor_categories_by_parent);
			tpl_assign('expense_categories_by_parent', $expense_categories_by_parent);
			tpl_assign('qbo_accounts', get_qbo_all_accounts());
			tpl_assign('qbo_cat_account_matches', $qbo_cat_account_matches);
			
		} else {
			// save the data
			
			foreach ($category_match as $category_id => $qbo_account_id) {
				DB::execute("
					INSERT INTO ".TABLE_PREFIX."qbo_accounts_category_match (category_id, qbo_account_id) VALUES
					($category_id, '$qbo_account_id')
					ON DUPLICATE KEY UPDATE qbo_account_id='$qbo_account_id';
				");
			}
			
			flash_success(lang('properties saved successfully'));
			ajx_current("back");
		}
		
	}

	/**
	 * If user has imported a new list of items and their ids have changed, then try to get them using the saved name and update the id
	 */
	private function check_and_fix_invalid_category_items($items_by_id, $qbo_cat_item_matches) {

		$item_ids = array_keys($items_by_id);

		$dataService = null;

		foreach ($qbo_cat_item_matches as $category_id => &$qbo_item_id) {

			if (!in_array($qbo_item_id, $item_ids)) {
				// item id does not exists anymore => get the item by name and update it

				$item_name = null;
				$row = DB::executeOne("SELECT qbo_item_name FROM ".TABLE_PREFIX."qbo_items_category_match WHERE category_id='$category_id'");
				if ($row) {
					$item_name = $row['qbo_item_name'];
				}

				if ($item_name) {
					$item_name = escape_character($item_name);

					if (!$dataService) $dataService = get_qbo_data_service();

					$items_result = $dataService->Query("select * from Item where FullyQualifiedName = '$item_name'");
					
					if (count($items_result) > 0) {
						$new_qbo_item = $items_result[0];

						// update matching id
						$qbo_item_id = $new_qbo_item->Id;

						// update database
						DB::execute("
							UPDATE ".TABLE_PREFIX."qbo_items_category_match 
							SET qbo_item_id = '$qbo_item_id'
							WHERE category_id='$category_id'
						");
					}

				}

			} else {
				// if item id exists, then update the saved name, in case it has changed in qbo

				$qbo_item = array_var($items_by_id, $qbo_item_id);
				if ($qbo_item) {
					$item_name = escape_character($qbo_item->FullyQualifiedName);

					DB::execute("
						UPDATE ".TABLE_PREFIX."qbo_items_category_match 
						SET qbo_item_name = '$item_name'
						WHERE category_id='$category_id'
					");
				}

			}
		}

		return $qbo_cat_item_matches;
	}

	function edit_items_and_categories_match() {
		
		$accessToken = get_qbo_access_token();
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			evt_add("login_to_quickbooks", array(
					'need_qbo_login' => true,
					'action' => 'edit_item_and_categories_match',
					'id' => 'dummy',
					'auth_url' => get_qbo_auth_url(),
			));
			ajx_current("empty");
			return;
		}
		
		$category_match = array_var($_REQUEST, 'category_match');
		if (!$category_match) {
			// show the form

			$qbo_items = get_qbo_items_definition();
			$items_by_id = array();
			foreach ($qbo_items as $qbo_item) {
				$items_by_id[$qbo_item->Id] = $qbo_item;
			}
			$_SESSION['qbo_items_by_id'] = $items_by_id; // they will be used when saving the submitted form
			
			$matches_rows = DB::executeAll("SELECT * FROM ".TABLE_PREFIX."qbo_items_category_match");
			$qbo_income_cat_item_matches = array();
			$qbo_expense_cat_item_matches = array();
			foreach ($matches_rows as $row) {
				$qbo_income_cat_item_matches[$row['category_id']] = $row['qbo_item_id'];
				$qbo_expense_cat_item_matches[$row['category_id']] = $row['qbo_expense_item_id'];
			}

			$qbo_income_cat_item_matches = $this->check_and_fix_invalid_category_items($items_by_id, $qbo_income_cat_item_matches);
			
			$labor_categories_dim = Dimensions::findByCode('hour_types');
			$expense_categories_dim = Dimensions::findByCode('expense_categories');

			$labor_categories_by_parent = array();
			if ($labor_categories_dim instanceof Dimension) {
				$labor_categories = Members::instance()->findAll(array(
					"conditions" => "dimension_id = ".$labor_categories_dim->getId(),
					"order" => "depth, name",
				));

				foreach ($labor_categories as $category) {
					if (!isset($labor_categories_by_parent[$category->getParentMemberId()])) {
						$labor_categories_by_parent[$category->getParentMemberId()] = array();
					}
					$labor_categories_by_parent[$category->getParentMemberId()][] = $category;
				}

			}

			$expense_categories_by_parent = array();
			if ($expense_categories_dim instanceof Dimension) {
				$expense_categories = Members::instance()->findAll(array(
					"conditions" => "dimension_id = ".$expense_categories_dim->getId(),
					"order" => "depth, name",
				));

				foreach ($expense_categories as $category) {
					if (!isset($expense_categories_by_parent[$category->getParentMemberId()])) {
						$expense_categories_by_parent[$category->getParentMemberId()] = array();
					}
					$expense_categories_by_parent[$category->getParentMemberId()][] = $category;
				}

			}

			$all_accounts = get_qbo_all_accounts();

			$qbo_income_items = array();
			$qbo_expense_items = array();

			$income_accounts_by_item_id = array();
			$expense_accounts_by_item_id = array();
			foreach ($qbo_items as $qbo_item) {
				
				$qbo_income_items[] = $qbo_item;

				if ($qbo_item->IncomeAccountRef) {

					foreach ($all_accounts as $acc) {
						if ($acc->Id == $qbo_item->IncomeAccountRef) {
							$income_accounts_by_item_id[$qbo_item->Id] = $acc;
							break;
						}
					}
				}

				$qbo_expense_items[] = $qbo_item;
				
				if ($qbo_item->ExpenseAccountRef) {

					foreach ($all_accounts as $acc) {
						if ($acc->Id == $qbo_item->ExpenseAccountRef) {
							$expense_accounts_by_item_id[$qbo_item->Id] = $acc;
							break;
						}
					}
				}
				
/*
				if (!isset($accounts_by_item_id[$qbo_item->Id])) {
					if ($qbo_item->AssetAccountRef) {
						foreach ($all_accounts as $acc) {
							if ($acc->Id == $qbo_item->AssetAccountRef) {
								$accounts_by_item_id[$qbo_item->Id] = $acc;
								break;
							}
						}
					}
				}*/
			}

			tpl_assign('labor_categories_dim', $labor_categories_dim);
			tpl_assign('expense_categories_dim', $expense_categories_dim);
			tpl_assign('labor_categories_by_parent', $labor_categories_by_parent);
			tpl_assign('expense_categories_by_parent', $expense_categories_by_parent);
			tpl_assign('qbo_items', $qbo_items);
			tpl_assign('qbo_income_items', $qbo_income_items);
			tpl_assign('qbo_expense_items', $qbo_expense_items);
			tpl_assign('qbo_income_cat_item_matches', $qbo_income_cat_item_matches);
			tpl_assign('qbo_expense_cat_item_matches', $qbo_expense_cat_item_matches);
			tpl_assign('income_accounts_by_item_id', $income_accounts_by_item_id);
			tpl_assign('expense_accounts_by_item_id', $expense_accounts_by_item_id);
			
		} else {
			// save the data

			$items_by_id = $_SESSION['qbo_items_by_id'];
			
			foreach ($category_match as $category_id => $qbo_items_data) {

				$qbo_item_id = array_var($qbo_items_data, 'income_item');
				$qbo_expense_item_id = array_var($qbo_items_data, 'expense_item');

				// also save the item name so if the internal id changes we can look for the item by name
				$item_name = '';
				$qbo_item = array_var($items_by_id, $qbo_item_id);
				if ($qbo_item) {
					$item_name = escape_character($qbo_item->FullyQualifiedName);
				}

				$expense_item_name = '';
				$qbo_exp_item = array_var($items_by_id, $qbo_expense_item_id);
				if ($qbo_exp_item) {
					$expense_item_name = escape_character($qbo_exp_item->FullyQualifiedName);
				}

				DB::execute("
					INSERT INTO ".TABLE_PREFIX."qbo_items_category_match (category_id, qbo_item_id, qbo_item_name, qbo_expense_item_id, qbo_expense_item_name) VALUES
					($category_id, '$qbo_item_id', '$item_name', '$qbo_expense_item_id', '$expense_item_name')
					ON DUPLICATE KEY UPDATE qbo_item_id='$qbo_item_id', qbo_item_name='$item_name',  qbo_expense_item_id='$qbo_expense_item_id', qbo_expense_item_name='$expense_item_name';
				");
			}

			unset($_SESSION['qbo_items_by_id']);
			
			flash_success(lang('properties saved successfully'));
			ajx_current("back");
		}
		
	}
	
	
	function sync_qbo_payment_methods() {
		
		$accessToken = get_qbo_access_token();
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			evt_add("login_to_quickbooks", array(
					'need_qbo_login' => true,
					'action' => 'sync_qbo_payment_methods',
					'auth_url' => get_qbo_auth_url(),
			));
			ajx_current("empty");
			return;
		}
		
		$transaction = false;
		try {
		
			$dataService = get_qbo_data_service();
			$qbo_payment_methods = get_qbo_all_payment_methods($dataService);
			
			$default_pms = array('','check','money_order','cash','credit_card','other');
			foreach ($qbo_payment_methods as $pm) {
				if (!in_array(strtolower($pm->Name), $default_pms)) {
					$default_pms[] = $pm->Name;
				}
			}
			
			DB::beginWork();
			$transaction = true;
			
			$invoice_pm_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'payment_method');
			$invoice_pm_cp->setValues(implode(',', $default_pms));
			$invoice_pm_cp->save();
			
			if (Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
				$payment_pm_cp = CustomProperties::getCustomPropertyByCode(InvoicePaymentReceipts::instance()->getObjectTypeId(), 'payment_method');
				$payment_pm_cp->setValues(implode(',', $default_pms));
				$payment_pm_cp->save();
			}
			
			DB::commit();
			
			flash_success(lang('payment methods synchronized successfully'));
			ajx_current("back");
			
		} catch (Exception $e) {
			if ($transaction) DB::rollback();
			flash_error($e->getMessage());
			ajx_current("empty");
		}
	}
	
	
	/************************************************/
	/******************* INVOICES *******************/
	/************************************************/
	
	function void_invoice_in_quickbooks($invoice = null) {
		ajx_current("empty");
		
		if ($invoice instanceof IncomeInvoice) {
			$invoice_id = $invoice->getId();
		} else {
			$invoice_id = get_id();
			$invoice = IncomeInvoices::instance()->findById($invoice_id);
		}
		
		$accessToken = get_qbo_access_token();
		
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			
			evt_add("login_to_quickbooks", array(
					'need_qbo_login' => true,
					'id' => $invoice_id,
					'action' => 'void_invoice_in_quickbooks',
					'auth_url' => get_qbo_auth_url(),
			));
			return;
		}
		
		try {
			
			if ($invoice instanceof IncomeInvoice) {
				
				void_invoice_in_qbo($invoice);
				
				flash_success(lang('invoice successfully voided in quickbooks'));
			}
			
		} catch (Error $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		} catch (Exception $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		}
		
	}
	
	
	function delete_invoice_from_quickbooks($invoice = null) {
		
		if (!AjaxResponse::instance()->hasCurrent()) {
			// only set this state if no state previously set by other controller
			ajx_current("empty");
		}
		
		if ($invoice instanceof IncomeInvoice) {
			$invoice_id = $invoice->getId();
		} else {
			$invoice_id = get_id();
			$invoice = IncomeInvoices::instance()->findById($invoice_id);
		}
		
		$accessToken = get_qbo_access_token();
		
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			
			evt_add("login_to_quickbooks", array(
					'need_qbo_login' => true,
					'id' => $invoice_id,
					'action' => 'delete_invoice_from_quickbooks',
					'auth_url' => get_qbo_auth_url(),
			));
			return;
		}
		
		try {
			
			if ($invoice instanceof IncomeInvoice) {
				
				if (substr(php_uname(), 0, 7) == "Windows" || !is_exec_available()){

					delete_invoice_from_qbo($invoice);

					flash_success(lang('invoice successfully deleted in quickbooks'));

				} else {

					// synchronize in background
					$user = logged_user();
					$command = "nice -n19 ".PHP_PATH." ". ROOT . "/plugins/quickbooks/application/helpers/delete_invoice_from_qbo_in_background.php ".ROOT." ".$user->getId()." ".$user->getTwistedToken()." ".$invoice->getId();
					exec("$command > /dev/null &");

				}
			}
			
		} catch (Error $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		} catch (Exception $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		}
		
	}


	function save_several_invoices_to_quickbooks($invoices = null) {
		
		$feng_invoices = array();

		if (!is_array($invoices)) {
			ajx_current("empty");

			$invoice_ids = get_id();
			$invoice_ids = array_filter(explode(',',$invoice_ids), 'is_numeric');
			$invoice_ids_csv = implode(',',$invoice_ids);

			if ($invoice_ids_csv != '') {
				$feng_invoices = IncomeInvoices::instance()->findAll(array("conditions"=>"id IN (".$invoice_ids_csv.")"));
			}
			
		} else {
			$feng_invoices = $invoices;
			$invoice_ids = array();
			foreach ($feng_invoices as $inv) $invoice_ids[] = $inv->getId();
			$invoice_ids_csv = implode(',',$invoice_ids);
		}

		$accessToken = get_qbo_access_token();
		
		// check if login to qb is needed
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			
			$login_event_data = array(
					'need_qbo_login' => true,
					'id' => $invoice_ids_csv,
					'action' => 'save_several_invoices_to_quickbooks',
					'auth_url' => get_qbo_auth_url(),
			);
			// call the login event and tell it to call this function again
			evt_add("login_to_quickbooks", $login_event_data);
			ajx_current("empty");
			
			return;
		}

		// send all the invoices to qb
		try {

			// if we can't sync in background then do it now
			if (substr(php_uname(), 0, 7) == "Windows" || !is_exec_available()){
				
				foreach ($feng_invoices as $feng_invoice) {
			
					if ($feng_invoice instanceof IncomeInvoice && check_if_object_can_be_sync($feng_invoice)) {
						
						save_invoice_to_qbo($feng_invoice);
						
					}
				}

			} else {
				// synchronize in background
				$user = logged_user();
				$ids = array();
				foreach ($feng_invoices as $feng_invoice) $ids[] = $feng_invoice->getId();
				$ids_csv = implode(',',$ids);

				$command = "nice -n19 ".PHP_PATH." ". ROOT . "/plugins/quickbooks/application/helpers/save_invoices_to_qbo_in_background.php ".ROOT." ".$user->getId()." ".$user->getTwistedToken()." ".$ids_csv;
				exec("$command > /dev/null &");
			}
			
		} catch (Error $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		} catch (Exception $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		}

	}


	function save_invoice_to_quickbooks($feng_invoice = null) {
		ajx_current("empty");
		
		if ($feng_invoice instanceof IncomeInvoice) {
			$invoice_id = $feng_invoice->getId();
			$trigger_login_event = true;
		} else {
			$invoice_id = get_id();
			$feng_invoice = IncomeInvoices::instance()->findById($invoice_id);
			$trigger_login_event = false;
		}
		//unset($_SESSION['QBOsessionAccessToken']); return;
		
		$accessToken = get_qbo_access_token();
		
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			
			$login_event_data = array(
					'need_qbo_login' => true,
					'id' => $invoice_id,
					'action' => 'save_invoice_to_quickbooks',
					'auth_url' => get_qbo_auth_url(),
			);
			
			if ($trigger_login_event) {
				// call the login event and tell it to call this function again
				evt_add("login_to_quickbooks", $login_event_data);
			} else {
				ajx_extra_data($login_event_data);
			}
			return;
		}
		
		try {
		
			if ($feng_invoice instanceof IncomeInvoice) {

				// will throw an exception if the there are fields incompatible with QBO
				$this->validate_invoice_fields_before_qbo_sync($feng_invoice);
				
				if (array_var($_REQUEST, 'sync_and_reload')) {
					save_invoice_to_qbo($feng_invoice);

					flash_success(lang('invoice successfully saved in quickbooks'));
					ajx_current("reload");
					return;

				} else {

					$invoices_to_sync = array($feng_invoice);

					// if this invoice has any related invoice (like the ones generated by the split invoice feature) then sync them too.
					$related_invoices = null;
					Hook::fire('qbo_related_invoices_to_sync', array('main_invoice'=>$feng_invoice), $related_invoices);
					if (isset($related_invoices) && count($related_invoices) > 0) {
						$invoices_to_sync = array_merge($invoices_to_sync, $related_invoices);
					}
					
					$this->save_several_invoices_to_quickbooks($invoices_to_sync);	
				}

			}
			
		} catch (Error $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		} catch (Exception $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		}
		
	}

	/**
	 * @params IncomeInvoice $feng_invoice the invoice to analyze
	 * @return boolean
	 * @throws Exception if some of the invoice fields are not compatible with QBO
	 */
	private function validate_invoice_fields_before_qbo_sync(IncomeInvoice $feng_invoice) {

		$ok = true;

		$max_qbo_invoice_number_length = 21;

		// validate invoice number length
		if (strlen($feng_invoice->getInvoiceNumber()) > $max_qbo_invoice_number_length) {
			$ok = false;
			throw new Exception(lang('qbo_sync invoice number cant have more than x chars', $max_qbo_invoice_number_length));
		}

		return $ok;
	}
	
	/************************************************/
	/******************* PAYMENTS *******************/
	/************************************************/
	
	function save_invoice_payment_to_quickbooks($feng_payment = null) {
		
		ajx_current("empty");
		
		if ($feng_payment instanceof InvoicePaymentReceipt) {
			$invoice_payment_id = $feng_payment->getId();
			$trigger_login_event = true;
		} else {
			$invoice_payment_id = get_id();
			$feng_payment = InvoicePaymentReceipts::instance()->findById($invoice_payment_id);
			$trigger_login_event = false;
		}
		
		$accessToken = get_qbo_access_token();
		
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			
			$login_event_data = array(
					'need_qbo_login' => true,
					'id' => $invoice_payment_id,
					'action' => 'save_invoice_payment_to_quickbooks',
					'auth_url' => get_qbo_auth_url(),
			);
			
			if ($trigger_login_event) {
				evt_add("login_to_quickbooks", $login_event_data);
			} else {
				ajx_extra_data($login_event_data);
			}
			return;
		}
		
		try {
			
			if ($feng_payment instanceof InvoicePaymentReceipt) {
				
				save_invoice_payment_to_qbo($feng_payment);
				
				flash_success(lang('payment successfully saved in quickbooks'));

				if (array_var($_REQUEST, 'sync_and_reload')) {
					ajx_current("reload");
				}
			}
			
		} catch (Error $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		} catch (Exception $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		}
		
	}
	
	function delete_invoice_payment_from_quickbooks($payment = null) {
		
		ajx_current("empty");
		
		if ($payment instanceof InvoicePaymentReceipt) {
			$payment_id = $payment->getId();
		} else {
			$payment_id = get_id();
			$payment = InvoicePaymentReceipts::instance()->findById($payment_id);
		}
		
		$accessToken = get_qbo_access_token();
		
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			
			evt_add("login_to_quickbooks", array(
					'need_qbo_login' => true,
					'id' => $payment_id,
					'action' => 'delete_invoice_payment_from_quickbooks',
					'auth_url' => get_qbo_auth_url(),
			));
			return;
		}
		
		try {
			
			if ($payment instanceof InvoicePaymentReceipt) {
				
				delete_invoice_payment_from_qbo($payment);

				flash_success(lang('payment successfully deleted in quickbooks'));
				
				ajx_current("back");
			}
			
		} catch (Error $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		} catch (Exception $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		}
		
	}



	/******************************************************************/
	/* TIME */
	/******************************************************************/
	
	function save_timeslots_to_quickbooks($feng_timeslots_array = null, $run_in_background = false) {
		ajx_current("empty");

		if($feng_timeslots_array == null){
			$timeslot_ids = array_var($_GET,'feng_timeslots_ids');
			$timeslot_ids_arr = explode(',',$timeslot_ids);
		}else{
			$timeslot_ids_arr = $feng_timeslots_array;
			$timeslot_ids = implode(',',$timeslot_ids_arr);
		}

		// synchronize in background
		if ($run_in_background && substr(php_uname(), 0, 7) != "Windows" && is_exec_available()) {

			$user = logged_user();
			$command = "nice -n19 ".PHP_PATH." ". ROOT . "/plugins/quickbooks/application/helpers/save_timeslots_to_qbo_in_background.php ".ROOT." ".$user->getId()." ".$user->getTwistedToken()." ".$timeslot_ids;
			exec("$command > /dev/null &");

			return;
		}

		$accessToken = get_qbo_access_token();
		
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			evt_add("login_to_quickbooks", array(
					'need_qbo_login' => true,
					'id' => $timeslot_ids,
					'action' => 'save_timeslots_to_quickbooks',
					'auth_url' => get_qbo_auth_url(),
			));
			return;
		}

		try {
			// Remove from cron queue so if cron executes at the same time they won't be synchronized two times
			set_if_timeslots_needs_to_sync($timeslot_ids_arr, false);
			
			foreach($timeslot_ids_arr as $timeslot_id) {
				$timeslot = Timeslots::instance()->findById($timeslot_id);
				
				if(config_option('have_time_sheets')){
					save_timeslot_to_ts($timeslot);
				}else{
					save_timeslot_to_qbo($timeslot);
				}
			}

			flash_success(lang('timeslots synchronized correctly'));
			ajx_current("reload");
		} catch (Error $e) {
			if (isset($timeslot)) {
				set_if_timeslots_needs_to_sync(array($timeslot->getId()), true);
			}
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		} catch (Exception $e) {
			if (isset($timeslot)) {
				set_if_timeslots_needs_to_sync(array($timeslot->getId()), true);
			}
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		}
	}

	function delete_timeslot_from_quickbooks($timeslot = null) {
		ajx_current("empty");

		if ($timeslot instanceof Timeslot) {
			$timeslot_id = $timeslot->getId();
		} else {
			$timeslot_id = get_id();
			$timeslot = Timeslots::instance()->findById($timeslot_id);
		}
		
		$accessToken = get_qbo_access_token();
		
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			
			evt_add("login_to_quickbooks", array(
					'need_qbo_login' => true,
					'id' => $timeslot_id,
					'action' => 'delete_invoice_from_quickbooks',
					'auth_url' => get_qbo_auth_url(),
			));
			return;
		}
		
		try {
			if ($timeslot instanceof Timeslot) {

				if(config_option('have_time_sheets')){
					delete_ts_timeslot($timeslot);
				}else{
					delete_time_from_qbo($timeslot);
				}

				flash_success(lang('timeslot successfully deleted in quickbooks'));
			}
			
		} catch (Error $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		} catch (Exception $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		}
	}

	function save_timeslots_to_qb_time($feng_timeslots = null) {
		ajx_current("empty");

		if ($feng_timeslots instanceof Timeslot) {
			$timelot_id = $feng_timeslots->getId();
			$trigger_login_event = true;
		}else{
			$timeslot_ids = array_var($_GET,'timeslot_ids');
			$timeslot_ids_arr = explode(',',$timeslot_ids);
		}

		$accessToken = get_qbo_access_token();
		
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			
			$login_event_data = array(
					'need_qbo_login' => true,
					'id' => $timelot_id,
					'action' => 'save_invoice_to_quickbooks',
					'auth_url' => get_qbo_auth_url(),
			);
			
			if ($trigger_login_event) {
				// call the login event and tell it to call this function again
				evt_add("login_to_quickbooks", $login_event_data);
			} else {
				ajx_extra_data($login_event_data);
			}
			return;
		}

		try {
			if ($feng_timeslots instanceof Timeslot) {
				$timeslot = Timeslots::instance()->findById($timelot_id);
				save_timeslot_to_qb_time($timeslot);
			}else{
				foreach($timeslot_ids_arr as $timeslot_id) {
					$timeslot = Timeslots::instance()->findById($timeslot_id);
					save_timeslot_to_qb_time($timeslot);
				}	
			}
		} catch (Error $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		} catch (Exception $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		}
	}

	function set_timeslot_as_unbilled($feng_timeslot = null) {
		ajx_current("empty");
		
		if ($feng_timeslot instanceof Timeslot) {
			$timeslot_id = $feng_timeslot->getId();
			$trigger_login_event = true;
		} else {
			$timeslot_id = get_id();
			$feng_timeslot = Timeslots::instance()->findById($timeslot_id);
			$trigger_login_event = false;
		}
		//unset($_SESSION['QBOsessionAccessToken']); return;
		
		$accessToken = get_qbo_access_token();
		
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			
			$login_event_data = array(
					'need_qbo_login' => true,
					'id' => $timeslot_id,
					'action' => 'set_timeslot_as_unbilled',
					'auth_url' => get_qbo_auth_url(),
			);
			
			if ($trigger_login_event) {
				// call the login event and tell it to call this function again
				evt_add("login_to_quickbooks", $login_event_data);
			} else {
				ajx_extra_data($login_event_data);
			}
			return;
		}
		
		try {
		
			if ($feng_timeslot instanceof Timeslot) {

				// this will update all the time entry data in quickbooks with the same data in feng/evx, including the status field
				save_timeslot_to_qbo($feng_timeslot);

			}
			
		} catch (Error $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		} catch (Exception $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		}
		
	}

	function send_timeslots() {
		ajx_current("empty");
		
		// get the selected time entries in the grid
		$timeslots = get_timeslots_from_grid_parameters();

		// if only one timeslot then sync it in real time
		if (count($timeslots) == 1) {

			$ts_id =  $timeslots[0] instanceof Timeslot ? $timeslots[0]->getId() : 0;
			
			$this->save_timeslots_to_quickbooks(array($ts_id));

		} else {
			
			// get the time entry ids that can be synchronized
			$timeslot_ids = array();
			foreach ($timeslots as $ts) {
				if ($ts instanceof Timeslot && check_if_object_can_be_sync($ts) && !$ts->isOpen()) {
					$timeslot_ids[] = $ts->getId();
				}
			}
	
			// queue the time entries to be sent in a background process
			set_if_timeslots_needs_to_sync($timeslot_ids, true);
			
			// tell the user that the time entries will be syncrhonized in a background process
			flash_success(lang('selected timeslots queued to be synchronized'));
			ajx_current("empty");
		}
	}
	

	function initial_setting_time_sheets() {

		$accessToken = get_qbo_access_token();
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			evt_add("login_to_quickbooks", array(
					'need_qbo_login' => true,
					'action' => 'initial_setting_time_sheets',
					'id' => 'dummy',
					'auth_url' => get_qbo_auth_url(),
			));
			ajx_current("empty");
			return;
		}

		if(config_option('time_sheets_token')){
			try{
				save_ts_custom_field("Customer type", SHORT_CODE_CUSTOMER_TYPE, "managed-list");
				$customer_type_id = get_id_custom_field_by_short_code(SHORT_CODE_CUSTOMER_TYPE);
				save_ts_custom_field_item($customer_type_id, "Customer");
				save_ts_custom_field_item($customer_type_id, "Project");
	
				// save_ts_custom_field("Billable", SHORT_CODE_BILLABLE, "managed-list");
				// $billable_id = get_id_custom_field_by_short_code(SHORT_CODE_BILLABLE);
				// save_ts_custom_field_item($billable_id, "Yes");
				// save_ts_custom_field_item($billable_id, "No");
				flash_success(lang('properties saved successfully'));
				ajx_current("back");
			}catch (Exception $e) {
				flash_success($e->http_body);
				ajx_current("back");
			}
		}else if(config_option('time_sheets_token')){
			flash_success(lang('timesheets token required'));
			ajx_current("back");
		}else{
			flash_success(lang('properties saved successfully'));
			ajx_current("back");			
		}
	}
	
	/****************************************************************************************************/
	/* INCOMING SYNCHRONIZATION */
	/****************************************************************************************************/
	
	function process_qbo_data_changes($entities) {
		if(!config_option('synchronize_incoming_changes')){
			return false;
		}
		
		// first save the entities in the database so if we don't have a valid token right now
		// we can try to execute the actions later when a valid token is available
		try {
			DB::beginWork();
			
			qbo_save_entites_to_process($entities);
			
			DB::commit();
		} catch (Exception $e) {
			DB::rollback();
			Logger::log_r("ERROR SAVING QBO ENTITIES TO PROCESS: ".$e->getMessage()."\n".print_r($entities)."\n".$e->getTraceAsString());
			return;
		}
		
		// process the data
		$this->qbo_process_saved_entities();
	}
	
	
	function qbo_process_saved_entities() {
		if(!config_option('synchronize_incoming_changes')){
			return false;
		}

		try {
			// try to process the changes, if valid token is not available then the changes will remain saved, to be processed in the future
			qbo_process_saved_entities();
			
		} catch (Exception $e) {
			Logger::log_r("ERROR PROCESSING QBO RECEIVED DATA:\n".$e->getMessage()."\n".$e->getTraceAsString());
		}
	}


	/****************************************************************************************************/
	/* IMPORTING FUNCTIONS */
	/****************************************************************************************************/

	/**
	 * Queries the QBO account to get a list of incvoices within a date range and imports the ones that are not present here
	 */
	function import_invoices_from_qbo() {

		$genid = gen_id();
		$max_objects_to_import = 100;

		tpl_assign('genid', $genid);
		tpl_assign('max_objects_to_import', $max_objects_to_import);

		if (array_var($_REQUEST, 'submit')) {

			if (array_var($_REQUEST, 'action') == 'do_query') {

				$from_date = getDateValue(array_var($_REQUEST, 'from_date'));
				$to_date = getDateValue(array_var($_REQUEST, 'to_date'));
		
				if (!$from_date instanceof DateTimeValue || !$to_date instanceof DateTimeValue) {
					flash_error(lang('parameters from date and to date must be specified'));
					ajx_current("empty");
					return;
				}

				// get the data service object to make queries
				$dataService = get_qbo_data_service();

				// get the invoices that are in qbo and not in feng/evx
				$qbo_invoices_filtered = $this->find_qbo_invoices_to_import($dataService, $from_date, $to_date);

				// get the payments that are in qbo and not in feng/evx
				$qbo_payments_filtered = $this->find_qbo_payments_to_import($dataService, $from_date, $to_date);
				

				// fills the $_SESSION variable with qbo entities so we don't need to make requests later
				$this->build_qbo_entities_cache_for_import($dataService, $qbo_invoices_filtered, $qbo_payments_filtered);

				// store data in the session to use it in the next step
				$_SESSION['qbo_invoices_to_import'] = $qbo_invoices_filtered;
				$_SESSION['qbo_payments_to_import'] = $qbo_payments_filtered;
				$_SESSION['qbo_invoices_import_from'] = $from_date;
				$_SESSION['qbo_invoices_import_to'] = $to_date;

				// send variables to the view
				tpl_assign('from_date', $from_date);
				tpl_assign('to_date', $to_date);
				tpl_assign('qbo_invoices', $qbo_invoices_filtered);
				tpl_assign('qbo_payments', $qbo_payments_filtered);
				tpl_assign('qbo_clients', $_SESSION['qbo_entities_cache']['clients']);
				tpl_assign('action', 'import_invoices');

			} else if (array_var($_REQUEST, 'action') == 'import_invoices') {

				// increase memory limit to prevent failure if there are too much invoices to import
				ini_set('memory_limit', '1G');
				@set_time_limit(0);

				// get the invoices to import from the session
				$invoices_to_import = $_SESSION['qbo_invoices_to_import'];
				$payments_to_import = $_SESSION['qbo_payments_to_import'];
				$new_feng_invoices = array();
				$new_feng_payments = array();

				try {

					$_SESSION['sync_incoming_objects'] = true;
					$selected_invoices = array_var($_REQUEST, 'selected_invoices', array());
					$selected_payments = array_var($_REQUEST, 'selected_payments', array());

					$dataService = get_qbo_data_service();

					$count = 0;

					// generate the invoices
					foreach ($invoices_to_import as $qbo_invoice) {

						// if not selected then ignore it
						if (!isset($selected_invoices[$qbo_invoice->Id])) {
							continue;
						}

						// verify if the invoice has not been added in a previous step or by a separate process 
						// this can happen when an invoice has a payment and the payment has 2 invoices so the other invoice will be created by when creating the paymtn
						// so we must check again in to avoid duplicating an invoice, if this invoice was created by a payment of a previous invoice.
						$invoice_already_exists = IncomeInvoices::instance()->findOne(array("conditions" => array("quickbooks_online_id=?", $qbo_invoice->Id)));
						if ($invoice_already_exists instanceof IncomeInvoice) {
							continue;
						}
						$err_messages = array();

						try {
							DB::beginWork();
						
							// create the invoice in Feng/EVX (don't save application log due to performance issue)
							$new_invoice = create_or_update_feng_invoice_from_qbo_object($dataService, $qbo_invoice, null, false);

							if ($new_invoice) {
								$new_feng_invoices[] = $new_invoice;
							}
							
							DB::commit();

						} catch (Exception $e) {
							DB::rollback();
							$err_msg = "Error importing invoice ".($qbo_invoice ? $qbo_invoice->DocNumber : '');
							$err_messages[] = $err_msg;
							Logger::log_r($err_msg . ":" . $e->getMessage() ."\n". $e->getTraceAsString());
						}


						// import no more than $max_invoices_to_import per run
						$count++;
						if ($count >= $max_objects_to_import) {
							break;
						}
					}

					// generate the payments
					foreach ($payments_to_import as $qbo_payment) {

						// if not selected then ignore it
						if (!isset($selected_payments[$qbo_payment->Id])) {
							continue;
						}

						// verify if the payment has not been added in a previous step or by a separate process 
						// this can happen when a payment has an invoice and the invoice has 2 payments so the other payment will be created by when creating the invoice
						// so we must check again in to avoid duplicating a payment, if this payment was created by an invoice of a previous payment.
						$payment_already_exists = InvoicePaymentReceipts::instance()->findOne(array("conditions" => array("quickbooks_online_id=?", $qbo_payment->Id)));
						if ($payment_already_exists instanceof InvoicePaymentReceipt) {
							continue;
						}
						$err_messages = array();

						try {
							DB::beginWork();
						
							// create the payment in Feng/EVX (don't save application log due to performance issue)
							$new_payment = create_or_update_feng_payment_from_qbo_object($dataService, $qbo_payment, null, false);

							if ($new_payment) {
								$new_feng_payments[] = $new_payment;
							}
							
							DB::commit();

						} catch (Exception $e) {
							DB::rollback();
							$err_msg = "Error importing payment ".($qbo_payment ? $qbo_payment->Id : '');
							$err_messages[] = $err_msg;
							Logger::log_r($err_msg . ":" . $e->getMessage() ."\n". $e->getTraceAsString());
						}


						// import no more than $max_invoices_to_import per run
						$count++;
						if ($count >= $max_objects_to_import) {
							break;
						}
					}


					unset($_SESSION['qbo_invoices_to_import']);
					unset($_SESSION['qbo_payments_to_import']);
					unset($_SESSION['sync_incoming_objects']);

					$err_msg_concat = "";
					if (count($err_messages) > 0) {
						$err_msg_concat .= '<br />' . implode('<br />', $err_messages);
					}

					$success_msg = count($new_feng_invoices) > 0 ? count($new_feng_invoices) . " Invoices imported" : "";
					$success_msg .= '<br />';
					$success_msg .= count($new_feng_payments) > 0 ? count($new_feng_payments) . " Payments imported" : "";

					flash_success($success_msg . $err_msg_concat);
					ajx_current("back");

				} catch (Exception $e) {

					$err_msg = count($new_feng_invoices) . " Invoices generated. ".(count($new_feng_payments) > 0 ? count($new_feng_payments) . " Payments generated. " : "")."Error importing invoice ".($qbo_invoice ? $qbo_invoice->DocNumber : '');
					flash_error($err_msg);

					Logger::log_r($e->getMessage()."\n".$e->getTraceAsString());
					ajx_current("back");
					
					unset($_SESSION['qbo_invoices_to_import']);
					unset($_SESSION['qbo_payments_to_import']);
					unset($_SESSION['sync_incoming_objects']);
				}

			}

		} else {
			
			unset($_SESSION['qbo_invoices_to_import']);
			unset($_SESSION['qbo_payments_to_import']);
			unset($_SESSION['sync_incoming_objects']);

			tpl_assign('action', 'do_query');

		}

	}


	private function find_qbo_invoices_to_import($dataService, $from_date, $to_date) {
		// max results from api is 1000 objects, so we have to iterate if there are more than 1000
		$qbo_invoices = array();
		$stop = false;
		$offset = 1;
		$limit = 1000;
		while (!$stop) {
			$query = "
				SELECT * FROM Invoice 
				WHERE TxnDate >= '".$from_date->format("Y-m-d")."' AND TxnDate <= '".$to_date->format("Y-m-d")."'
				STARTPOSITION $offset MAXRESULTS $limit
			";

			// make the query
			$qbo_invoices_temp = $dataService->Query($query);

			// add the found invoices to the final result var
			$qbo_invoices = array_merge($qbo_invoices, $qbo_invoices_temp);

			// increase the start position for next query
			$offset += $limit;

			// check if we have to stop the iteration
			if (count($qbo_invoices_temp) < $limit) {
				$stop = true;
			}
		}
		
		$qbo_invoices_filtered = array();

		// filter the invoices, we don't want to re-import existing invoices
		foreach ($qbo_invoices as $qbo_invoice) {
			$inv_count = IncomeInvoices::instance()->count("quickbooks_online_id=".$qbo_invoice->Id);
			if ($inv_count == 0) {
				$qbo_invoices_filtered[] = $qbo_invoice;
			}
		}

		return $qbo_invoices_filtered;
	}


	private function find_qbo_payments_to_import($dataService, $from_date, $to_date) {

		if (!Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
			return array();
		}

		// max results from api is 1000 objects, so we have to iterate if there are more than 1000
		$qbo_payments = array();
		$stop = false;
		$offset = 1;
		$limit = 1000;
		while (!$stop) {
			$query = "
				SELECT * FROM Payment 
				WHERE TxnDate >= '".$from_date->format("Y-m-d")."' AND TxnDate <= '".$to_date->format("Y-m-d")."'
				STARTPOSITION $offset MAXRESULTS $limit
			";

			// make the query
			$qbo_payments_temp = $dataService->Query($query);

			// add the found payments to the final result var
			$qbo_payments = array_merge($qbo_payments, $qbo_payments_temp);

			// increase the start position for next query
			$offset += $limit;

			// check if we have to stop the iteration
			if (count($qbo_payments_temp) < $limit) {
				$stop = true;
			}
		}
		
		$qbo_payments_filtered = array();

		// filter the invoices, we don't want to re-import existing invoices
		foreach ($qbo_payments as $qbo_payment) {
			$pay_count = InvoicePaymentReceipts::instance()->count("quickbooks_online_id=".$qbo_payment->Id);
			if ($pay_count == 0) {
				$qbo_payments_filtered[] = $qbo_payment;
			}
		}

		return $qbo_payments_filtered;
	}


	function build_qbo_entities_cache_for_import($dataService, $qbo_invoices, $qbo_payments) {

		$_SESSION['qbo_entities_cache'] = array(
			'clients' => array(),
			'items' => array(),
			'classes' => array(),
			'terms' => array(),
			'payment_methods' => array(),
		);

		$qbo_client_ids = array();

		foreach ($qbo_invoices as $qbo_invoice) {

			if (!in_array($qbo_invoice->CustomerRef, $qbo_client_ids)) {
				$qbo_client_ids[] = $qbo_invoice->CustomerRef;
			}
			
		}

		foreach ($qbo_payments as $qbo_payment) {

			if (!in_array($qbo_payment->CustomerRef, $qbo_client_ids)) {
				$qbo_client_ids[] = $qbo_payment->CustomerRef;
			}
			
		}

		// projects and clients cache (only the ones present in the invoices)
		if (count($qbo_client_ids) > 0) {
			$qbo_cli_result = $dataService->Query("SELECT * FROM Customer WHERE Id IN ('".implode("','",$qbo_client_ids)."') and Active IN (true,false) maxresults 1000");
			foreach ($qbo_cli_result as $qbo_cli) {
				$_SESSION['qbo_entities_cache']['clients'][$qbo_cli->Id] = $qbo_cli;
			}
		}

		// items cache
		$qbo_items = $dataService->Query("select * from Item maxresults 1000");
		foreach ($qbo_items as $item) {
			$_SESSION['qbo_entities_cache']['items'][$item->Id] = $item;
		}

		// classes cache
		$qbo_classes = $dataService->Query("select * from Class");
		foreach ($qbo_classes as $class) {
			$_SESSION['qbo_entities_cache']['classes'][$class->Id] = $class;
		}

		// payment terms cache
		$qbo_terms = $dataService->Query("select * from Term");
		foreach ($qbo_terms as $term) {
			$_SESSION['qbo_entities_cache']['terms'][$term->Id] = $term;
		}

		// payment payment methods cache
		$qbo_p_methods = get_qbo_all_payment_methods($dataService);
		foreach ($qbo_p_methods as $qbo_p_method) {
			$_SESSION['qbo_entities_cache']['payment_methods'][$qbo_p_method->Id] = $qbo_p_method;
		}

	}
	
	

	/****************************************************************************************************/
	/* EXPENSES */
	/****************************************************************************************************/

	function save_actual_expense_to_quickbooks($feng_payments_array = null, $run_in_background = false) {
		ajx_current("empty");

		if ($feng_payments_array == null) {
			$payment_ids = array_var($_GET,'feng_payments_ids');
			$payment_ids_arr = explode(',',$payment_ids);
		} else {
			$payment_ids_arr = $feng_payments_array;
			$payment_ids = implode(',',$payment_ids_arr);
		}

		// synchronize in background
		if ($run_in_background && substr(php_uname(), 0, 7) != "Windows" && is_exec_available()) {

			$user = logged_user();
			$command = "nice -n19 ".PHP_PATH." ". ROOT . "/plugins/quickbooks/application/helpers/save_expenses_to_qbo_in_background.php ".ROOT." ".$user->getId()." ".$user->getTwistedToken()." ".$payment_ids;
			exec("$command > /dev/null &");

			return;
		}

		$accessToken = get_qbo_access_token();
		
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			evt_add("login_to_quickbooks", array(
				'need_qbo_login' => true,
				'id' => $payment_ids,
				'action' => 'save_actual_expense_to_quickbooks',
				'auth_url' => get_qbo_auth_url(),
			));
			return;
		}

		try {

			foreach($payment_ids_arr as $payment_id) {
				$payment = PaymentReceipts::instance()->findById($payment_id);
				if($payment instanceof PaymentReceipt){
					save_actual_expense_to_qbo($payment);
				}
			}

			if (array_var($_REQUEST, 'sync_and_reload')) {
				ajx_current("reload");
			}

		} catch (Error $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		} catch (Exception $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		}
	}

	function delete_actual_expense_from_quickbooks($payment = null) {
		ajx_current("empty");

		if ($payment instanceof PaymentReceipt) {
			$payment_id = $payment->getId();
		} else {
			$payment_id = get_id();
			$payment = PaymentReceipts::instance()->findById($payment_id);
		}
		
		$accessToken = get_qbo_access_token();
		
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			
			evt_add("login_to_quickbooks", array(
					'need_qbo_login' => true,
					'id' => $payment_id,
					'action' => 'delete_actual_expense_from_quickbooks',
					'auth_url' => get_qbo_auth_url(),
			));
			return;
		}
		
		try {
			if ($payment instanceof PaymentReceipt) {
				delete_actual_expense_from_qbo($payment);
				flash_success(lang('payment receipt successfully deleted in quickbooks'));
			}
			
		} catch (Error $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			Logger::log_r($e->getMessage()."\n".$e->getTraceAsString());
		} catch (Exception $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			Logger::log_r($e->getMessage()."\n".$e->getTraceAsString());
		}
	}

	function sync_qbo_accounts_and_payment_methods() {
		ajx_current("empty");

		qbo_sync_accounts_and_payment_methods();

		flash_success(lang('qb expenses methods synchronized successfully'));
		ajx_current("back");
	}


    function sync_invoices_whit_quickbooks(){
		
		$accessToken = get_qbo_access_token();
		if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
			evt_add("login_to_quickbooks", array(
					'need_qbo_login' => true,
					'action' => 'sync_qbo_payment_methods',
					'auth_url' => get_qbo_auth_url(),
			));
			ajx_current("empty");
			return;
		}


		$from = config_option("qbo_sync_entities_from_date");
		$from_date = '';
		if ($from instanceof DateTimeValue) {
			$from_date = $from->toMySQL();
		}

		$ids_sel = array_var($_GET,'ids_sel');
		$ids_to_array = explode(',', $ids_sel);


		try {

			foreach($ids_to_array as $invoice_id){
				$invoice_data = IncomeInvoices::instance()->findOne(array('conditions' => " `object_id` =  '".$invoice_id."' and `quickbooks_online_id`='0' and `date` >= '".$from_date."' "));

				if($invoice_data){
					save_invoice_to_qbo($invoice_data);
				}
			}
			
			flash_success('Invoices sync');
			ajx_current("reload");
		} catch (Error $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		} catch (Exception $e) {
			Logger::log_r("ERROR AT ".__FUNCTION__." : ".$e->getMessage()."\n".$e->getTraceAsString());
			flash_error($e->getMessage());
		}
		
    }


	/****************************************************************************************************/
	/* TIMESHEETS */
	/****************************************************************************************************/

	// function save_timeslots_to_timesheet($feng_timeslots_array = null) {
	// 	ajx_current("empty");
	// 	Logger::log_r("OK");

	// 	if($feng_timeslots_array == null){
	// 		$timeslot_ids = array_var($_GET,'feng_timeslots_ids');
	// 		$timeslot_ids_arr = explode(',',$timeslot_ids);
	// 	}else{
	// 		$timeslot_ids_arr = $feng_timeslots_array;
	// 		$timeslot_ids = implode(',',$timeslot_ids_arr);
	// 	}

	// 	// sync_contact_with_ts(null,null);
		
	// 	// if (!isset($accessToken) || !is_valid_qbo_access_token($accessToken)) {
	// 	// 	evt_add("login_to_quickbooks", array(
	// 	// 			'need_qbo_login' => true,
	// 	// 			'id' => $timeslot_ids,
	// 	// 			'action' => 'save_timeslots_to_timesheet',
	// 	// 			'auth_url' => get_qbo_auth_url(),
	// 	// 	));
	// 	// 	return;
	// 	// }

	// 	try {
	// 		foreach($timeslot_ids_arr as $timeslot_id) {
	// 			$timeslot = Timeslots::instance()->findById($timeslot_id);
	// 			save_timeslot_to_ts($timeslot);
	// 		}
	// 	} catch (Error $e) {
	// 		flash_error($e->getMessage());
	// 	} catch (Exception $e) {
	// 		flash_error($e->getMessage());
	// 	}
	// }


	public function get_total_not_sync(){
		$from = config_option("qbo_sync_entities_from_date");
		$from_date = '';
		if ($from instanceof DateTimeValue) {
			$from_date = $from->toMySQL();
		}

		$invoices_not_sync = IncomeInvoices::instance()->findAll(array('conditions' => "`quickbooks_online_id`='0' and `trashed_on` ='0000-00-00 00:00:00' and `date` >= '".$from_date."' "));
		$total_invoices = count($invoices_not_sync);

        ajx_current("empty");
        ajx_extra_data(array('total'=> $total_invoices));
    }


	public function get_tsheets_time_off_jobcodes() {
		if (config_option('have_time_sheets')) {

			$tsheets_client = getInstanteTS();
	
			$filters = array(
				'type' => 'pto'
			);
			$result = $tsheets_client->get(TSheets\ObjectType::Jobcodes, $filters);
			
			$jobcodes = $result['results'][TSheets\ObjectType::Jobcodes];
	
			return $jobcodes;

		} else {
			return array();
		}
	}

}