<?php

function import_pending_entities_from_quickbooks() {
	_log("Import Quickbooks pending entities...");
	try {
		// try to process the changes, if valid token is not available then the changes will remain saved, to be processed in the future
		qbo_process_saved_entities();
		
	} catch (Exception $e) {
		Logger::log_r("ERROR PROCESSING QBO RECEIVED DATA:\n".$e->getMessage()."\n".$e->getTraceAsString());
	}
	_log("End Import Quickbooks pending entities.");
}


/**
 * This will send to quickbooks all the timeslots marked as "need to sync"
 */
function send_timeslots_to_quickbooks() {

	_log("Quickbooks timeslots sync start...");
	
	Env::useHelper('permissions');
	Env::useHelper('dimension');

	if (!logged_user() instanceof Contact) {
		$admin = Contacts::instance()->findOne(array("conditions" => "user_type=1"));
		if ($admin instanceof Contact) {
			CompanyWebsite::instance()->logUserIn($admin);
		}
	}
	
	$qbo_controller = new QuickbooksController();
	$data_service = get_qbo_data_service();

	$timeslots_to_sync = Timeslots::instance()->findAll(array(
		"conditions" => "needs_to_sync = 1",
		"limit" => 50,
	));
	$sync_count = 0;
	$timeslot_ids = array();

	foreach ($timeslots_to_sync as $timeslot) {
		try {
			$timeslot_ids[] = $timeslot->getId();
			DB::beginWork();

			if(config_option('have_time_sheets')){
				save_timeslot_to_ts($timeslot, $data_service);
			}else{
				save_timeslot_to_qbo($timeslot, $data_service);
			}
			$sync_count++;
			
			DB::commit();
			
		} catch (Exception $e) {
			DB::rollback();
			$timeslot_id = $timeslot ? $timeslot->getId() : "0";
			Logger::log("Error saving timeslot to QBO: timeslot_id: $timeslot_id - ".$e->getMessage()."\n".$e->getTraceAsString());
		}
	}

	// remove from queue
	set_if_timeslots_needs_to_sync($timeslot_ids, false);

	_log("Quickbooks timeslots sync end ($sync_count/".count($timeslots_to_sync).")");
}
