<?php
	chdir($argv[1]);
	define("CONSOLE_MODE", true);
	define('PUBLIC_FOLDER', 'public');
	include "init.php";

	session_commit(); // we don't need sessions
	@set_time_limit(0); // don't limit execution of cron, if possible
	ini_set('memory_limit', '1024M');

	Env::useHelper('permissions');
	Env::useHelper('dimension');

	$user_id = array_var($argv, 2);
	$token = array_var($argv, 3);

	// log user in
	$user = Contacts::instance()->findById($user_id);
	if(!($user instanceof Contact) || !$user->isValidToken($token)) {
		die();
	}
	CompanyWebsite::instance()->setLoggedUser($user, false, false, false);

	// get the invoice ids
	$object_id = array_var($argv, 4);

	$qbo_controller = new QuickbooksController();
	$data_service = get_qbo_data_service();

	$invoice = IncomeInvoices::instance()->findById($object_id);
	if ($invoice instanceof IncomeInvoice) {
		try {
			DB::beginWork();

			delete_invoice_from_qbo($invoice, $data_service);

			DB::commit();
		} catch (Exception $e) {
			DB::rollback();

			// prevent triggering other synchronizations
			$_SESSION['sync_incoming_objects'] = true;
			// rollback the trash action
			$invoice->untrash();

			// create a popup reminder for the user to let them know that there was an error
			$reminder = new ObjectReminder();
			$reminder->setType('reminder_popup');
			$reminder->setContext('error_delete_qbo');
			$reminder->setUser(logged_user());
			$reminder->setObject($invoice);
			$reminder->setDate(DateTimeValueLib::now());
			$reminder->save();

			Logger::log("Error saving invoive to QBO: invoice_id: $id - ".$e->getMessage()."\n".$e->getTraceAsString());
		}
	}
