<?php

/**************************************************************************************/
/************************************* AUX FUNCTIONS **********************************/
/**************************************************************************************/

function create_client_member_from_qbo_customer($qbo_customer) {
	Env::useHelper('dimension');
	Env::useHelper('form');
	Env::useHelper('functions', 'crpm');
	
	$company_ot_id = ObjectTypes::findByName('company')->getId();
	$customers_dim = get_customers_dimension();

	// remove multiple spaces between name parts, it can lead to duplicates
	$client_name_exploded = array_filter(explode(' ',$qbo_customer->DisplayName));
	foreach ($client_name_exploded as &$str) $str = trim($str);
	$client_name = trim(implode(' ', $client_name_exploded));
	// --
	
	$data = array();
	$data['customer'] = array(
			'contact_data_type' => $company_ot_id,
			'name' => $client_name,
			'status_id' => 6,
			'parent' => 0,
			'automatic_status' => '1',
			'customer_name' => '',
			'customer_address' => '',
			'customer_rut' => '',
			'customer_number' => '',
	);
	
	$_POST = $data;
	$_REQUEST = $data;
	$_SESSION['dont_check_required_cps'] = true;
	
	Env::useController('customer');
	$ccontroller = new CustomerController();
	$ccontroller->add();
	
	unset($_REQUEST);
	unset($_POST);
	unset($_SESSION['dont_check_required_cps']);
	
	$member = Members::instance()->findOne(array(
			"conditions" => "dimension_id=".$customers_dim->getId()." AND name LIKE '".escape_character($client_name)."'"
	));

	// set qbo id
	$member->setColumnValue('quickbooks_online_id', $qbo_customer->Id);
	$member->setColumnValue('quickbooks_online_sync_token', $qbo_customer->SyncToken);
	$member->save();

	return $member;
}


function create_project_member_from_qbo_customer($qbo_project, $client_mem_id=0) {
	Env::useHelper('dimension');
	Env::useHelper('form');
	Env::useHelper('functions', 'crpm');
	
	$customers_dim = get_customers_dimension();
	$projects_dim = Dimensions::findByCode('customer_project');
	$client_ot_id = ObjectTypes::findByName('customer')->getId();
	
	$parent_mem_id = 0;
	if ($customers_dim->getId() == $projects_dim->getId()) {
		$parent_mem_id = $client_mem_id;
	}
	
	$data = array();
	$data['project'] = array(
			'name' => $qbo_project->DisplayName,
			'parent' => $parent_mem_id,
	);
	
	$_POST = $data;
	$_REQUEST = $data;
	$_SESSION['dont_check_required_cps'] = true;
	$_SESSION['dont_check_required_associations'] = true;
	
	Env::useController('project');
	$pcontroller = new ProjectController();
	$pcontroller->add();
	
	unset($_REQUEST);
	unset($_POST);
	unset($_SESSION['dont_check_required_cps']);
	unset($_SESSION['dont_check_required_associations']);
	
	$member = Members::instance()->findOne(array(
			"conditions" => "dimension_id=".$projects_dim->getId()." AND name LIKE '".escape_character($qbo_project->DisplayName)."'"
	));

	// set qbo id
	$member->setColumnValue('quickbooks_online_id', $qbo_project->Id);
	$member->setColumnValue('quickbooks_online_sync_token', $qbo_project->SyncToken);
	$member->save();
	
	// associate client with project
	if ($customers_dim->getId() != $projects_dim->getId()) {
		$assoc = DimensionMemberAssociations::instance()->getAssociationObject($projects_dim->getId(), $member->getObjectTypeId(), $customers_dim->getId(), $client_ot_id);
		if ($assoc) {
			$mpm = new MemberPropertyMember();
			$mpm->setAssociationId($assoc->getId());
			$mpm->setMemberId($member->getId());
			$mpm->setPropertyMemberId($client_mem_id);
			$mpm->setIsActive(true);
			$mpm->setCreatedById(logged_user()->getId());
			$mpm->setCreatedOn(DateTimeValueLib::now());
			$mpm->save();
		}
	}
	
	return $member;
}


/**************************************************************************************/
/************************************* MAIN FUNCTIONS *********************************/
/**************************************************************************************/

function qbo_save_entites_to_process($entities) {
	
	if ($entities && count($entities) > 0) {
		$values_array = array();
		
		foreach ($entities as $e) {
			$dt = DateTimeValueLib::makeFromString($e['lastUpdated']);
			$values_array[] = "('".$e['name']."', '".$e['id']."', '".$e['operation']."', '".$dt->toMySQL()."')";
		}
		
		if (count($values_array) > 0) {
			$sql = "INSERT INTO ".TABLE_PREFIX."qbo_entities_to_process (qbo_type, qbo_id, operation, last_updated) 
					VALUES ".implode(',', $values_array).";";
			
			DB::execute($sql);
		}
	}
}


function qbo_do_process_saved_entities($access_token) {
	
	$data_service = get_qbo_data_service($access_token);
	
	$entities = DB::executeAll("SELECT * FROM ".TABLE_PREFIX."qbo_entities_to_process");
	
	foreach ($entities as $entity) {
		$ok = false;
		
		try {
			switch ($entity['qbo_type']) {
				case 'Invoice':
					$ok = process_qbo_invoice($data_service, $entity);
					break;
				case 'Payment':
					$ok = process_qbo_payment($data_service, $entity);
					break;
			}
		} catch (Exception $e) {
			Logger::log_r("QBO SYNC: Error processing ".$entity['qbo_type']." (".$entity['qbo_id']."): \n".$e->getMessage()."\n".$e->getTraceAsString());
		}
		
		if ($ok) {
			try {
				DB::beginWork();
				// remove from table after processing the data
				DB::executeAll("DELETE FROM ".TABLE_PREFIX."qbo_entities_to_process WHERE id=".$entity['id']);
				DB::commit();
			} catch (Exception $e) {
				DB::rollback();
				Logger::log_r("QBO SYNC: Error deleting pending entity action: ".$entity['qbo_type']." (".$entity['qbo_id']."): \n".$e->getMessage()."\n".$e->getTraceAsString());
			}
		}
	}
}

function qbo_process_saved_entities() {
	
	if (array_var($_SESSION, 'sync_incoming_objects')) return; // prevent cron event overlap
	
	$access_token = get_qbo_access_token();
	
	if ($access_token && is_valid_qbo_access_token($access_token)) {
		
		$_SESSION['sync_incoming_objects'] = true; // set this flag to true to prevent sync loops
		
		qbo_do_process_saved_entities($access_token);
		
		unset($_SESSION['sync_incoming_objects']);
		
	} else {
		Logger::log_r("QBO SYNC: COULD NOT PROCESS PENDING QBO ENTITIES DUE TO INVALID TOKEN");
	}
}


/**************************************************************************************/
/********************************** INVOICE FUNCTIONS *********************************/
/**************************************************************************************/

function process_qbo_invoice($data_service, $entity) {
	
	sleep(1); // wait to prevent duplicates if the qbo id was not commited yet
	// get the invoice using qbo id
	$feng_invoice = IncomeInvoices::instance()->findOne(array("conditions" => array("quickbooks_online_id=?", $entity['qbo_id'])));
	
	if ($entity['operation'] == 'Create' && $feng_invoice instanceof IncomeInvoice) {
		// it was created by evx
		return true;
	}

	$qbo_invoice = get_qbo_invoice($data_service, $entity['qbo_id']);
	
	// Process delete function
	if ($entity['operation'] == 'Delete') {
		if ($feng_invoice instanceof IncomeInvoice) {
			
			try {
				DB::beginWork();
				
				$feng_invoice->setColumnValue("quickbooks_online_id", 0); // this is to avoid sending the changes again to quickbooks
				$feng_invoice->save();
				$feng_invoice->trash();
				
				DB::commit();
				
				ApplicationLogs::createLog($feng_invoice, ApplicationLogs::ACTION_TRASH, false, null, true, 'Deleted from Quickbooks Online');
				
			} catch(Exception $e) {
				DB::rollback();
				Logger::log_r("QBO SYNC: COULD NOT DELETE INVOICE: qbo_id: ".$entity['qbo_id']. " - id:".$feng_invoice->getId());
				return false;
			}
		}
		return true;
	}
	
	// Process void function
	if ($entity['operation'] == 'Void') {
		if ($feng_invoice instanceof IncomeInvoice) {
			try {
				DB::beginWork();
				
				$feng_invoice->void();
				
				DB::commit();
				
			} catch(Exception $e) {
				DB::rollback();
				Logger::log_r("QBO SYNC: COULD NOT VOID INVOICE: qbo_id: ".$entity['qbo_id']. " - id:".$feng_invoice->getId());
				return false;
			}
		}
		return true;
	}
	
	// Create or Update
	$new_invoice = create_or_update_feng_invoice_from_qbo_object($data_service, $qbo_invoice, $feng_invoice);
	return $new_invoice instanceof IncomeInvoice;
}


function create_or_update_feng_invoice_from_qbo_object($data_service, $qbo_invoice, $feng_invoice = null, $save_app_log = true, $payments_to_ignore = array()) {
	
	if (!$feng_invoice instanceof IncomeInvoice) {
		$feng_invoice = new IncomeInvoice();
		$feng_invoice->setObjectTypeId(IncomeInvoices::instance()->getObjectTypeId());
		$member_ids = array();
		$log_action = ApplicationLogs::ACTION_ADD;
	} else {
		$member_ids = $feng_invoice->getMemberIds();
		$log_action = ApplicationLogs::ACTION_EDIT;
	}
	
	// check sync tokens, to see if we have a new version of the object
	if ($qbo_invoice->SyncToken <= $feng_invoice->getColumnValue('quickbooks_online_sync_token')) {
		// we already have the latest version
		return $feng_invoice;
	}

	$invoice_data = qbo_build_invoice_data_to_save($data_service, $qbo_invoice, $feng_invoice);

	$member_ids = array_var($invoice_data, 'member_ids');
	$_POST['object_custom_properties'] = array_var($invoice_data, 'object_custom_properties');
	
	$controller = new InvoiceController();
	$new_invoice = $controller->save_invoice($feng_invoice, $invoice_data, $member_ids);
	
	
	if ($save_app_log) {
		ApplicationLogs::createLog($feng_invoice, $log_action, false, null, true, ($log_action==ApplicationLogs::ACTION_ADD ? 'Added' : 'Edited').' from Quickbooks Online');
	}
	
	// save associated payments if they don't exist
	if (Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
		/* @var $qbo_invoice QuickBooksOnline\API\Data\IPPInvoice */
		
		$linked_transactions = is_array($qbo_invoice->LinkedTxn) ? $qbo_invoice->LinkedTxn : array($qbo_invoice->LinkedTxn);

		foreach ($linked_transactions as $linked_transaction) {
			
			if ($linked_transaction && $linked_transaction->TxnType == 'Payment' && !in_array($linked_transaction->TxnId, $payments_to_ignore)) {
			
				$feng_payment = InvoicePaymentReceipts::instance()->findOne(array(
						"conditions" => "quickbooks_online_id=".$linked_transaction->TxnId,
				));
				
				if (!$feng_payment) {
					// get the invoice and save it into feng
					$qbo_payment = get_qbo_payment($data_service, $linked_transaction->TxnId);
					if ($qbo_payment) {
						$feng_payment = create_or_update_feng_payment_from_qbo_object($data_service, $qbo_payment, null, $save_app_log);
					}
					if (!$feng_payment instanceof InvoicePaymentReceipt) {
						Logger::log_r("QBO SYNC: COULD NOT ADD NEW PAYMENT FROM INVOICE: ".print_r($linked_transaction,1));
					}
				}
			}
		}

	}
	
	return $new_invoice;
}


function qbo_get_client_and_project_member_ids_from_qbo_ref($data_service, $qbo_object_customer_ref) {
	$project_ot_id = Projects::instance()->getObjectTypeId();
	$client_ot_id = Customers::instance()->getObjectTypeId();

	$member_ids = array();
	// find customer or project
	if ($qbo_object_customer_ref > 0) {

		// check if we already have the client/project mapped in our database
		$mapped_member = Members::instance()->findOne(array("conditions" => array("quickbooks_online_id = ?", $qbo_object_customer_ref)));
		if ($mapped_member instanceof Member) {

			// add the client/project to the members to add
			$member_ids[] = $mapped_member->getId();

			// if it is a project, then get the associated client
			if ($mapped_member->getObjectTypeId() == $project_ot_id) {
				Env::useHelper('functions', 'crpm');
				$clients_dim = get_customers_dimension();
				$client_member_id = get_associated_status_member_id($mapped_member, $clients_dim);
				if ($client_member_id > 0) {
					$member_ids[] = $client_member_id;
				}
			}

		} else {

			// if not mapped then query it in qbo and add it
			$qbo_project = null;
			$qbo_customer = get_qbo_customer_object_by_id($data_service, $qbo_object_customer_ref);
			if ($qbo_customer && $qbo_customer->ParentRef > 0) {
				// $qbo_customer object is a project
				$qbo_project = $qbo_customer;
				$qbo_customer = get_qbo_customer_object_by_id($data_service, $qbo_project->ParentRef);
			}
			
			$client_mem_id = 0;
			if ($qbo_customer) {
				// search by qbo_id, if not found then search by name and then assign qbo_id
				$client_mem = Members::instance()->findOne(array("conditions" => array("quickbooks_online_id=? AND object_type_id=?", $qbo_customer->Id, $client_ot_id)));
				if (!$client_mem) {
					$client_mem = Members::instance()->findOne(array("conditions" => array("name=? AND object_type_id=?", $qbo_customer->DisplayName, $client_ot_id)));
					if ($client_mem) {
						$client_mem->setColumnValue('quickbooks_online_id', $qbo_customer->Id);
						$client_mem->save();
					}
				}
				if (!$client_mem) {
					$client_mem = create_client_member_from_qbo_customer($qbo_customer);
				}
				if ($client_mem) {
					$member_ids[]=$client_mem->getId();
					$client_mem_id = $client_mem->getId();;
				}
			}
			if ($qbo_project) {
				// search by qbo_id, if not found then search by project_number pc or by name and then assign qbo_id
				$m = Members::instance()->findOne(array("conditions" => array("quickbooks_online_id=? AND object_type_id=?", $qbo_project->Id, $project_ot_id)));
	
				if (!$m) {
					// search by project number
					$project_id_cp = CustomProperties::getCustomPropertyByCode($project_ot_id, 'project_id');
					if ($project_id_cp) {
	
						$exploded_name = explode(' ', $qbo_project->DisplayName);
						$project_number_str = trim(array_var($exploded_name, 0));

						// remove possible trailing chars from project number so we can query with a clean proj num and we can avoid generating duplicates
						if (str_ends_with($project_number_str, '-') || str_ends_with($project_number_str, ':')) {
							$project_number_str = substr($project_number_str, 0, strlen($project_number_str) - 1);
						}
	
						$cp_val = CustomPropertyValues::instance()->findOne(array("conditions" => array("custom_property_id=? AND value=?", $project_id_cp->getId(), $project_number_str)));
						if ($cp_val instanceof CustomPropertyValue) {
							$m = Members::instance()->findOne(array("conditions" => array("object_id=? AND object_type_id=?", $cp_val->getObjectId(), $project_ot_id)));
						}
					}
				}
				if (!$m) {
					// search by project name
					$m = Members::instance()->findOne(array("conditions" => array("name=? AND object_type_id=?", $qbo_project->DisplayName, $project_ot_id)));
				}
	
				// if not found by qbo_id, or project_number cp or by name then create it
				if (!$m) {
					$m = create_project_member_from_qbo_customer($qbo_project, $client_mem_id);
				}
	
				if ($m) $member_ids[]=$m->getId();
			}

		}


	}

	return $member_ids;
}


function qbo_build_invoice_data_to_save($data_service, QuickBooksOnline\API\Data\IPPInvoice $qbo_invoice, IncomeInvoice $feng_invoice) {

	$invoice_ot_id = IncomeInvoices::instance()->getObjectTypeId();
	
	$currency = Currencies::instance()->findOne(array("conditions" => "short_name='".$qbo_invoice->CurrencyRef."'"));
	
	if ($currency) {
		$currency_id = $currency->getId();
	} else {
		$currency_info = Currencies::getDefaultCurrencyInfo();
		$currency_id = $currency_info['id'];
	}

	// $currency_id = $currency ? $currency->getId() : Currencies::instance()->getDefaultCurrency(true);
	
	$invoice_subtype = ObjectSubtypes::instance()->findOne(array('conditions' => array("object_type_id=? AND `name`='invoice'", $invoice_ot_id)));
	$invoice_subty_id = $invoice_subtype ? $invoice_subtype->getId() : 0;
	
	if ($qbo_invoice->Balance > 0) {
		if ($qbo_invoice->Balance < $qbo_invoice->TotalAmt) {
			$status = 'part_confirmed';
		} else {
			$status = 'pending';
		}
	} else {
		$status = 'confirmed';
	}
	
	$member_ids = qbo_get_client_and_project_member_ids_from_qbo_ref($data_service, $qbo_invoice->CustomerRef);
	
	if ($qbo_invoice->ClassRef > 0) {
		$class_member = get_member_for_qbo_class($data_service, $qbo_invoice->ClassRef);
		if ($class_member instanceof Member && !in_array($class_member->getId(), $member_ids)) {
			$member_ids[]=$class_member->getId();
		}
	}
	
	
	// build lines
	$lines = $qbo_invoice->Line;
	if ($lines instanceof QuickBooksOnline\API\Data\IPPLine) {
		$lines = array($lines);
	}
	$invoice_lines = array();
	foreach ($lines as $qbo_line) {
		/* @var $qbo_line QuickBooksOnline\API\Data\IPPLine */
		if ($qbo_line->DetailType == 'SalesItemLineDetail') {
			
			$quantity = $qbo_line->SalesItemLineDetail->Qty;
			$unit_value = $qbo_line->SalesItemLineDetail->UnitPrice;

			// if a line comes with quantity = 0, then put quantity=1 to avoid lines with 0 amount in feng/evx
			if ($quantity == 0 && $qbo_line->Amount > 0) {
				$unit_value = $qbo_line->Amount;
				$quantity = 1;
			}
			
			$line_detail = '';
			$user_id = 0;
			if ($qbo_line->SalesItemLineDetail->ItemRef && $qbo_line->SalesItemLineDetail->ItemRef > 0) {
				$qbo_item = get_qbo_item_definition($data_service, $qbo_line->SalesItemLineDetail->ItemRef);
				if ($qbo_item) {
					$line_detail = escape_character($qbo_item->Name);
				}
				
				$row = DB::executeOne("SELECT * FROM ".TABLE_PREFIX."qbo_user_hour_items_match WHERE qbo_item_id=".$qbo_line->SalesItemLineDetail->ItemRef);
				if ($row && $row['user_id'] > 0) {
					$user_id = $row['user_id'];
				}
			}

			// if we couldn't find item name then use the line description
			if ($line_detail == '' && trim($qbo_line->Description) != '') {
				$line_detail = trim($qbo_line->Description);
			}
			
			// get line class and use the mapped member
			if ($qbo_line->SalesItemLineDetail->ClassRef > 0) {
				$class_member = get_member_for_qbo_class($data_service, $qbo_line->SalesItemLineDetail->ClassRef);
				if ($class_member instanceof Member && !in_array($class_member->getId(), $member_ids)) {
					$member_ids[]=$class_member->getId();
				}
			}

			$line_total = $qbo_line->Amount;
			if (!$line_total) {
				$line_total = number_format($quantity * $unit_value, 2);
			}

			// @TODO get the labor/expense category from the received line
			// ensure that rel_object_id and object_type_id and all the properties are not lost
			
			
			$line = array(
					"amount" => $quantity,
					"user_id" => $user_id,
					"category_id" => '[]',
					"unit_value" => $unit_value,
					"currency_id" => $currency_id,
					"exchange_rate" => 1,
					"total" => $line_total,
					"detail" => $line_detail,
			);
			
			$invoice_lines[] = $line;
		}
	}
	
	// find cp values for 
	$object_custom_properties = array();
	$invoice_cps = CustomProperties::getAllCustomPropertiesByObjectType($invoice_ot_id);
	foreach ($invoice_cps as $invoice_cp) {/* @var $invoice_cp CustomProperty */
		
		$cp_val = null; 
		
		$cf_row = DB::executeOne("SELECT * FROM ".TABLE_PREFIX."qbo_custom_field_match WHERE feng_property1='cp_".$invoice_cp->getId()."' OR feng_property2='cp_".$invoice_cp->getId()."'");
		if ($cf_row) {
			// if this custom property is matching a qbo custom field then get the value from the field
			$custom_field_id = $cf_row['quickbooks_id'];
			foreach ($qbo_invoice->CustomField as $customField) {
				if ($customField->DefinitionId == $custom_field_id) {
					
					$custom_field_val = '';
					if ($customField->Type == 'StringType') $custom_field_val = $customField->StringValue;
					else if ($customField->Type == 'BooleanType') $custom_field_val = $customField->BooleanValue;
					else if ($customField->Type == 'DateType') $custom_field_val = $customField->DateValue;
					else if ($customField->Type == 'NumberType') $custom_field_val = $customField->NumberValue;
					
					if ($custom_field_val != '') {
						$fp1 = str_replace("cp_", "", $cf_row['feng_property1']);
						$fp2 = str_replace("cp_", "", $cf_row['feng_property2']);
						if ($fp1 > 0 && $fp2 > 0) {
							$exploded = explode('-', $custom_field_val);
							if ($invoice_cp->getId() == $fp2) $cp_val = trim(end($exploded));
							else $cp_val = trim($exploded[0]);
						} else {
							$cp_val = $custom_field_val;
						}
					}
					
					if ($cp_val && ($invoice_cp->getType() == 'date' || $invoice_cp->getType() == 'datetime')) {
						try {
							$dt = DateTimeValueLib::dateFromFormatAndString(user_config_option('date_format'), $cp_val);
							if ($dt instanceof DateTimeValue) {
								$cp_val = format_date($dt, null, 0);
							}
						} catch (Exception $e) {
							$cp_val = null;
						}
					}
					
					break;
				}
			}
		
		} else {
		
			switch ($invoice_cp->getCode()) {
				case 'invoice_payment_period':
					if ($qbo_invoice->SalesTermRef > 0) {
						$qbo_term = get_qbo_payment_term($data_service, $qbo_invoice->SalesTermRef);
						$cp_val = $qbo_term ? $qbo_term->Name : null;
						if ($cp_val) {
							$possible_values = explode(',', $invoice_cp->getValues());
							if (!in_array($cp_val, $possible_values)) {
								$possible_values[] = $cp_val;
								$invoice_cp->setValues(implode(',', $possible_values));
								$invoice_cp->save();
							}
						}
					}
					break;
				case 'payment_method':
					break;
				case 'external_transaction_id':
					break;
				case 'external_transaction_desc':
					break;
				case 'billing_period_from':
					break;
				case 'billing_period_to':
					break;
				case 'po_number':
					$cp_val = $qbo_invoice->PONumber;
					break;
				case 'shipping_address':
					$cp_val = array(
						'type' => 1,
						'street' => trim($qbo_invoice->ShipAddr->Line2." ".$qbo_invoice->ShipAddr->Line3." ".$qbo_invoice->ShipAddr->Line4." ".$qbo_invoice->ShipAddr->Line5),
						'city' => $qbo_invoice->ShipAddr->City,
						'state' => $qbo_invoice->ShipAddr->CountrySubDivisionCode,
						'zip_code' => $qbo_invoice->ShipAddr->PostalCode,
						'country' => strtolower($qbo_invoice->ShipAddr->CountryCode)
					);
					break;
				case 'shipping_date':
					$cp_val = $qbo_invoice->ShipDate;
					if ($cp_val) {
						try {
							$dt = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $cp_val);
							if ($dt instanceof DateTimeValue) {
								$cp_val = format_date($dt, null, 0);
							} else {
								$cp_val = null;
							}
						} catch (Exception $e) {
							$cp_val = null;
						}
					}
					break;
				case 'shipping_method':
					$cp_val = $qbo_invoice->ShipMethodRef;
					break;
				case 'shipping_tracking_num':
					$cp_val = $qbo_invoice->TrackingNum;
					break;
				
			}
		}
		
		if ($cp_val) {
			$object_custom_properties[$invoice_cp->getId()] = $cp_val;
		}
	}
	
	$data = array(
			'name' => $feng_invoice->getObjectName(),
			'quickbooks_online_id' => $qbo_invoice->Id,
			'quickbooks_online_sync_token' => $qbo_invoice->SyncToken,
			'object_subtype_id' => $invoice_subty_id,
			'use_mysql_dates' => true,
			'date' => $qbo_invoice->TxnDate, //DateTimeValueLib::dateFromFormatAndString('Y-m-d', $qbo_invoice->TxnDate)
			'expiration_date' => $qbo_invoice->DueDate,
			'status' => $status,
			'override_invoice_number' => true,
			'invoice_number' => $qbo_invoice->DocNumber,
			'currency_id' => $currency_id,
			'additional_information' => $qbo_invoice->CustomerMemo,
			'object_custom_properties' => $object_custom_properties,
			'invoice_lines' => $invoice_lines,
			'member_ids' => $member_ids,
			'customer_email' => $qbo_invoice->BillEmail ? $qbo_invoice->BillEmail->Address : '',
			'customer_name' => null, // will be filled by the controller
			'customer_address' => trim($qbo_invoice->BillAddr->Line2." ".$qbo_invoice->BillAddr->Line3." ".$qbo_invoice->BillAddr->Line4." ".$qbo_invoice->BillAddr->Line5),
			'customer_city' => $qbo_invoice->BillAddr->City,
			'customer_state' => $qbo_invoice->BillAddr->CountrySubDivisionCode,
			'customer_country' => $qbo_invoice->BillAddr->CountryCode,
			'customer_zipcode' => $qbo_invoice->BillAddr->PostalCode,
	);
	return $data;
}





/**************************************************************************************/
/********************************** PAYMENT FUNCTIONS *********************************/
/**************************************************************************************/




function process_qbo_payment($data_service, $entity) {
	
	if (!Plugins::instance()->isActivePlugin('invoice_payment_receipts')) {
		Logger::log_r("QBO SYNC: PAYMENT RECEIPTS PLUGIN NOT INSTALLED");
		return true;
	}
	
	$qbo_payment = get_qbo_payment($data_service, $entity['qbo_id']);
	
	$feng_payment = InvoicePaymentReceipts::instance()->findOne(array("conditions" => array("quickbooks_online_id=?", $entity['qbo_id'])));
	
	if ($entity['operation'] == 'Create' && $feng_payment instanceof InvoicePaymentReceipt) {
		// it was created by evx
		return true;
	}
	
	// Process delete function
	if ($entity['operation'] == 'Delete') {
		if ($feng_payment instanceof InvoicePaymentReceipt) {
			
			try {
				DB::beginWork();
				
				$feng_payment->setColumnValue("quickbooks_online_id", 0); // this is to avoid sending the changes again to quickbooks
				$feng_payment->save();
				$feng_payment->trash();
				
				DB::commit();
				
				ApplicationLogs::createLog($feng_payment, ApplicationLogs::ACTION_TRASH, false, null, true, 'Deleted from Quickbooks Online');
				
			} catch(Exception $e) {
				DB::rollback();
				Logger::log_r("QBO SYNC: COULD NOT DELETE PAYMENT: qbo_id: ".$entity['qbo_id']. " - id:".$feng_payment->getId());
				return false;
			}
		}
		return true;
	}
	
	// Create or Update
	$new_payment = create_or_update_feng_payment_from_qbo_object($data_service, $qbo_payment, $feng_payment);
	
	return $new_payment instanceof InvoicePaymentReceipt;
}



function create_or_update_feng_payment_from_qbo_object($data_service, $qbo_payment, $feng_payment = null, $save_app_log = true) {
	
	if (!$feng_payment instanceof InvoicePaymentReceipt) {
		$feng_payment = new InvoicePaymentReceipt();
		$feng_payment->setObjectTypeId(InvoicePaymentReceipts::instance()->getObjectTypeId());
		$member_ids = array();
		$log_action = ApplicationLogs::ACTION_ADD;
	} else {
		$member_ids = $feng_payment->getMemberIds();
		$log_action = ApplicationLogs::ACTION_EDIT;
	}
	
	// check sync tokens, to see if we have a new version of the object
	if ($qbo_payment->SyncToken <= $feng_payment->getColumnValue('quickbooks_online_sync_token')) {
		// we already have the latest version
		return $feng_payment;
	}
	
	$payment_data = qbo_build_payment_data_to_save($data_service, $qbo_payment, $feng_payment);
	
	$member_ids = array_var($payment_data, 'member_ids');
	$_POST['object_custom_properties'] = array_var($payment_data, 'object_custom_properties');
	$_REQUEST['invoices'] = array_var($payment_data, 'invoices');
	
	$controller = new InvoicePaymentReceiptController();
	$new_payment = $controller->save_invoice_payment_receipt($feng_payment, $payment_data, $member_ids);
	
	if ($save_app_log) {
		ApplicationLogs::createLog($feng_payment, $log_action, false, null, true, ($log_action==ApplicationLogs::ACTION_ADD ? 'Added' : 'Edited').' from Quickbooks Online');
	}
	
	return $new_payment;
}


function qbo_build_payment_data_to_save($data_service, QuickBooksOnline\API\Data\IPPPayment $qbo_payment, InvoicePaymentReceipt $feng_payment) {
	
	$payemnt_ot_id = InvoicePaymentReceipts::instance()->getObjectTypeId();
	
	$currency = Currencies::instance()->findOne(array("conditions" => "short_name='".$qbo_payment->CurrencyRef."'"));
	$currency_id = $currency ? $currency->getId() : Currencies::instance()->getDefaultCurrency(true);
	
	
	$member_ids = qbo_get_client_and_project_member_ids_from_qbo_ref($data_service, $qbo_payment->CustomerRef);
	
	// find cp values for
	$object_custom_properties = array();
	$payment_cps = CustomProperties::getAllCustomPropertiesByObjectType($payemnt_ot_id);
	foreach ($payment_cps as $payment_cp) {/* @var $invoice_cp CustomProperty */
		
		$cp_val = null;
		switch ($payment_cp->getCode()) {
			case 'payment_method':
				$pm_ref_id = $qbo_payment->PaymentMethodRef;
				$qbo_pmethod = get_qbo_payment_method($data_service, null, $pm_ref_id);
				if ($qbo_pmethod) {
					$cp_val = $qbo_pmethod->Name;
					
					$possible_values = explode(',', $payment_cp->getValues());
					if (!in_array($cp_val, $possible_values)) {
						$possible_values[] = $cp_val;
						$payment_cp->setValues(implode(',', $possible_values));
						$payment_cp->save();
					}
				}
				break;
			case 'external_transaction_id':
				$cp_val = $qbo_payment->PaymentRefNum;
				break;
			case 'description':
				$cp_val = $qbo_payment->PrivateNote;
				break;
			case 'date':
				$cp_val = $qbo_payment->TxnDate;
				if ($cp_val) {
					try {
						$dt = DateTimeValueLib::dateFromFormatAndString('Y-m-d', $cp_val);
						if ($dt instanceof DateTimeValue) {
							$cp_val = format_date($dt, null, 0);
						} else {
							$cp_val = null;
						}
					} catch (Exception $e) {
						$cp_val = null;
					}
				}
				break;
			case 'paid_by':
				break;
		}
		
		if ($cp_val) {
			$object_custom_properties[$payment_cp->getId()] = $cp_val;
		}
	}
	
	// build affected invoices lines
	$lines = $qbo_payment->Line;
	if ($lines instanceof QuickBooksOnline\API\Data\IPPLine) {
		$lines = array($lines);
	}
	
	$invoices = array();
	foreach ($lines as $line) {
		
		$ln_transaction_obj = $line->LinkedTxn;
		$amount = $line->Amount;
		
		if ($ln_transaction_obj->TxnType == 'Invoice') {
			
			$feng_invoice = IncomeInvoices::instance()->findOne(array(
					"conditions" => "quickbooks_online_id=".$ln_transaction_obj->TxnId,
			));
			
			if (!$feng_invoice) {
				// get the invoice and save it into feng
				$qbo_invoice = get_qbo_invoice($data_service, $ln_transaction_obj->TxnId);
				if ($qbo_invoice) {
					$feng_invoice = create_or_update_feng_invoice_from_qbo_object($data_service, $qbo_invoice, null, true, array($qbo_payment->Id));
				}
				if (!$feng_invoice instanceof Incomeinvoice) {
					Logger::log_r("QBO SYNC: COULD NOT ADD NEW INVOICE FROM PAYMENT: ".print_r($line,1));
					continue;
				}
			}
			$feng_invoice_id = $feng_invoice->getId();
			
			if ($feng_invoice_id > 0) {
				$affected_invoice_line = array(
						'to_pay' => $amount,
						'id' => $feng_invoice_id,
						'currency_id' => $currency_id,
				);
				$invoices[] = $affected_invoice_line;
			}
		}
	}
	
	$data = array(
			'name' => $feng_payment->getObjectName(),
			'quickbooks_online_id' => $qbo_payment->Id,
			'quickbooks_online_sync_token' => $qbo_payment->SyncToken,
			'allow_zero_amount' => true,
			'currency_id' => $currency_id,
			'amount' => $qbo_payment->TotalAmt,
			'balance' => $qbo_payment->UnappliedAmt,
			'object_custom_properties' => $object_custom_properties,
			'invoices' => $invoices,
			'member_ids' => $member_ids,
	);
	
	return $data;
}

