<?php
	chdir($argv[1]);
	define("CONSOLE_MODE", true);
	define('PUBLIC_FOLDER', 'public');
	include "init.php";

	session_commit(); // we don't need sessions
	@set_time_limit(0); // don't limit execution of cron, if possible
	ini_set('memory_limit', '1024M');

	Env::useHelper('permissions');
	Env::useHelper('dimension');

	$user_id = array_var($argv, 2);
	$token = array_var($argv, 3);

	// log user in
	$user = Contacts::instance()->findById($user_id);
	if(!($user instanceof Contact) || !$user->isValidToken($token)) {
		die();
	}
	CompanyWebsite::instance()->setLoggedUser($user, false, false, false);

	$ids = array();

	// get the invoice ids
	$object_id = array_var($argv, 4);
	if (is_numeric($object_id)) {
		$ids[] = $object_id;
	} else {
		$ids = explode(',', $object_id);
	}

	$qbo_controller = new QuickbooksController();
	$data_service = get_qbo_data_service();

	// Remove from cron queue so if cron executes at the same time they won't be synchronized two times
	set_if_timeslots_needs_to_sync($ids, false);


	foreach ($ids as $timeslot_id) {

		$timeslot = Timeslots::instance()->findById($timeslot_id);
		if ($timeslot instanceof Timeslot) {
			try {
				DB::beginWork();

				if(config_option('have_time_sheets')){
					save_timeslot_to_ts($timeslot, $data_service);
				}else{
					save_timeslot_to_qbo($timeslot, $data_service);
				}
				sleep(1); // wait to send the next timeslot

				DB::commit();
			} catch (Exception $e) {
				DB::rollback();
				Logger::log("Error saving timeslot to QBO: timeslot_id: $timeslot_id - ".$e->getMessage()."\n".$e->getTraceAsString());
				
				set_if_timeslots_needs_to_sync(array($timeslot->getId()), true);
			}
		}
	}
	