<?php

use org\apache\maven\POM\_4_0_0\project;
use TSheets\TSheetsRestClient;


define('SHORT_CODE_SERVICE_ITEM','si');
define('SHORT_CODE_CUSTOMER_TYPE','ctype');
define('SHORT_CODE_BILLABLE','b');
define('TOKEN_TIME_SHEETS', config_option('time_sheets_token')); // S.17__b8288fdbb98c3ac6f450b554b6d1b0795dadfbcc
$tsheets_client = null;

function getInstanteTS(){
    global $tsheets_client;

    if(!$tsheets_client){
        $tsheets_client = new TSheetsRestClient(1,TOKEN_TIME_SHEETS);
    }

    return $tsheets_client;
}

function get_id_custom_field_by_short_code($short_code){
    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->get(TSheets\ObjectType::CustomFields);
    $response = $items['results']['customfields'];

    foreach($response as $item){
        if($item['short_code'] == $short_code){
            return $item['id'];
        }
    }
}

function getIdItemByName($name){
    $tsheets_client = getInstanteTS();
    $service_item_id = get_id_custom_field_by_short_code(SHORT_CODE_SERVICE_ITEM);

    if(!$service_item_id){
        $error = 'Not found custom field';
        // Logger::log_r("Error getting in getIdCustomFieldByShortCode");
        throw new Exception(build_qbo_error_message($error));
    }

    $items = $tsheets_client->get(TSheets\ObjectType::CustomFieldItems, ['customfield_id'=>$service_item_id]);
    $response = $items['results']['customfielditems'];
    foreach($response as $item){
        if($item['name'] == $name){
            return $item['id'];
        }
    }
}

// for development purposes
// function get_ts_employees_by_name(){
//     $tsheets_client = getInstanteTS();
//     $items = $tsheets_client->get(ObjectType::Users,['usernames'=>'rtaylor@quinnesi.com']);
//     $response = reset($items['results']);
//     Logger::log_r($response);
// }

/*
    Currently the API does not allow to bring the user information by email
*/
function get_ts_employee_by_name_and_lastname($first_name, $last_name){
    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->get(TSheets\ObjectType::Users,['first_name'=>$first_name,'last_name'=>$last_name]);
    $response = reset($items['results']['users']);
    return $response;
}

function get_ts_employee_id_by_name($name){
    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->get(TSheets\ObjectType::Users,['usernames'=>$name]);
    $response = reset($items['results']['users']);

    return $response['id'];
}

function get_ts_employee_information_by_name($name){
    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->get(TSheets\ObjectType::Users,['usernames'=>$name]);
    $response = reset($items['results']['users']);
    
    return $response;
}

function get_ts_customer_id_by_name($name){
    if(empty($name)){ 
		return 0;
	}

	// HERE IS THE BUG OF THE WRONG CUSTOMER 
	// WE NEED TO CALCULATE THE PARENT ID AND INCLUDE IT INT THE REQUEST

    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->get(TSheets\ObjectType::Jobcodes,['name'=>$name]);
    $response = reset($items['results']['jobcodes']);
    return $response['id'] ?? 0;
}

function save_ts_custom_field($name,$short_code,$type){
    $request[] = [
        "name" => $name,
        "required" => false,
        "active" => true,
        "applies_to" => "timesheet",
        "type" => $type,
        "show_to_all" => true,
        "short_code" => $short_code
    ];

    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->add(TSheets\ObjectType::CustomFields, $request);

    if(reset($items['results']['customfields'])['_status_code'] == 200){
        return reset($items['results']['customfields']);     
    }else{
        $error = 'Custom field not saved';
        // Logger::log_r("Error to save custom field");
        // Logger::log_r($items);
        throw new Exception(build_qbo_error_message($error));
    }   
}

function save_ts_custom_field_item($customfield_id, $name){
    $request[] = [
        "name" => $name,
        "customfield_id" => $customfield_id
    ];

    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->add(TSheets\ObjectType::CustomFieldItems, $request);

    if(reset($items['results']['customfielditems'])['_status_code'] == 200){
        return reset($items['results']['customfielditems']);     
    }else{
        $error = 'Custom field Item id not found';
        // Logger::log_r("Error in save  custom field item");
        throw new Exception(build_qbo_error_message($error));
    }
}

function get_ts_custom_field_item($customfield_id, $name){
    $request = [
        "customfield_id" => $customfield_id,
        "name" => $name
    ];

    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->get(TSheets\ObjectType::CustomFieldItems, $request);

    return reset($items['results']['customfielditems']) ?? [];
}

// function save_ts_project($name){
//     $item_information = get_ts_custom_field_item(1555648, $name);

//     if(!isset($item_information['name'])){
//         $item_information = save_ts_custom_field_item(1555648, $name);
//     }

//     return $item_information;
// }

function apply_permission_to_user($ts_user_id){
    $request[] = [
        "id" => $ts_user_id,
        "permissions" => [
            "time_tracking" => 1
        ]
    ];

    
    $tsheets_client = getInstanteTS();
    $items = $tsheets_client->put(TSheets\ObjectType::Users, $request);
    if(reset($items['results']['users'])['_status_code'] == 200){
        return reset($items['results']['users']);     
    }

    return false;
}

function ts_get_labor_category($object) {
	$categories_dim = Dimensions::findByCode('hour_types');
	if (!$categories_dim) return;
	$cat_member = null;

	// get the labor category member of the object
	$members = $object->getMembers();
	foreach ($members as $m) {
		if ($m->getDimensionId() == $categories_dim->getId()) {
			$cat_member = $m;
			break;
		}
	}
    
    return $cat_member;
}

function save_timeslot_to_ts(Timeslot $timeslot){    
	
	if (!check_if_object_can_be_sync($timeslot)) return;

    $service_item_id = get_id_custom_field_by_short_code(SHORT_CODE_SERVICE_ITEM);
    $customer_type_id = get_id_custom_field_by_short_code(SHORT_CODE_CUSTOMER_TYPE);
    $billable_id = get_id_custom_field_by_short_code(SHORT_CODE_BILLABLE);
    $tsheets_client = getInstanteTS();
    $dataService = get_qbo_data_service();
    $ts_time_activity_id = $timeslot->getColumnValue('tsheets_online_id');
    $sync_error = '';


	// check if the time entry is a time off request 
	$time_off_code = get_time_entry_time_off_code($timeslot);
	$is_time_off_entry = is_numeric($time_off_code) && $time_off_code > 0;

	if (!$is_time_off_entry) {

		$labor_information = ts_get_labor_category($timeslot);
		if($labor_information){
			// get the time entry item depending on the labor category
			$hours_item_ref = get_object_qbo_labor_item($timeslot);
			$name_item = null;
			if ($hours_item_ref) {
				$name_item = $hours_item_ref->FullyQualifiedName;
			}
			// $item_id = getIdItemByName($name_item);
			if(!$name_item){
				$sync_error = 'Not found item name';
			}
		}else{
			$name_item = 'Hours';
		}
	}

    $qbo_employee_obj = sync_contact_with_qbo($timeslot->getContactId());
    if(!isset($qbo_employee_obj->PrimaryEmailAddr->Address)){
        $sync_error = 'Employee not synced with quickbooks';
        throw new Exception($sync_error);
    }

    $ts_employee_data = get_ts_employee_by_name_and_lastname($qbo_employee_obj->GivenName, $qbo_employee_obj->FamilyName);
    $ts_employee_id = $ts_employee_data['id'];
    if(!empty($ts_employee_id)){
        if(empty($ts_employee_data['permissions']['time_tracking'])){
            $result_permission = apply_permission_to_user($ts_employee_id);
            if($result_permission){
                $sync_error = 'Problems to apply permission to employee';
            }
        }
    }else{
        $sync_error = 'Employee id not found';
    }

	if (!$is_time_off_entry) {

		$client_ot = ObjectTypes::findByName('customer');
		$project_ot = ObjectTypes::findByName('project');

		// find client and project members
		$feng_timeslot_members = $timeslot->getMembers();
		$client_member = null;
		$project_member = null;
		$job_id = null;
		$job_type = '';

		foreach ($feng_timeslot_members as $member) {

			if ($member->getObjectTypeId() == $client_ot->getId()) {
				$client_member = $member;
			}
			if ($member->getObjectTypeId() == $project_ot->getId()) {
				$project_member = $member;
			}
		}
		
		// get qbo customer for the evx client
		if($client_member){
			$customerRef = get_qbo_customer_object($dataService, $client_member);
			if(!empty($customerRef->DisplayName)){
				$custumer_id = get_ts_customer_id_by_name($customerRef->DisplayName);
				$job_id = $custumer_id;
				$job_type = 'Customer';

				if(empty($custumer_id)){
					$sync_error = 'Customer not found';
				}
			}
		}

		// get qbo job/project for the evx project
		if ($customerRef && $project_member instanceof Member) {
			$project_data = get_qbo_project_object($dataService, $project_member, $customerRef);

			if(!empty($project_data->DisplayName)){
				$project_id = get_ts_customer_id_by_name($project_data->DisplayName);
				$job_id = $project_id;
				$job_type = 'Project';

				if(empty($project_id)){
					$sync_error = 'Project not found';
				}
			}
		}

		if(!empty($sync_error)){
			$sync_error = "Error only when synchronizing: ". $sync_error; 
			throw new Exception($sync_error);
		}
	}

    $type_entry = config_option('have_time_sheets') ? 'MANUAL' : 'REGULAR';

    if($type_entry == 'MANUAL'){
        $tz_offset = $timeslot->getTimezoneValue();
        $start_time = new DateTimeValue($timeslot->getStartTime()->getTimestamp());
        $start_time->add('s', $tz_offset);
        $start_time_str = $start_time->toMySQL();
        $start_time_str = str_replace(' ', 'T', $start_time_str);
        $start_time_str = substr($start_time_str,0,10);
        $time_seconds = $timeslot->getColumnValue("worked_time") * 60;
    }else{
        $tz_offset = $timeslot->getTimezoneValue();
        $start_time = new DateTimeValue($timeslot->getStartTime()->getTimestamp());
        $start_time->add('s', $tz_offset);
        $start_time_str = $start_time->toMySQL();
        $start_time_str = str_replace(' ', 'T', $start_time_str);
        $start_time_str = $start_time_str.'-00:00';
        $end_time = new DateTimeValue($timeslot->getEndTime()->getTimestamp());
        $end_time->add('s', $tz_offset);
        $end_time_str = $end_time->toMySQL();
        $end_time_str = str_replace(' ', 'T', $end_time_str);
        $end_time_str = $end_time_str.'-00:00';
    }

    $is_billable = "Yes";
    if($timeslot->getColumnValue("invoicing_status") == "non_billable"){
        $is_billable = 'No';
    }


	if ($is_time_off_entry) {

		// calculate the status to send in the request depending on the timeslot's approval status
		$status = get_tsheets_time_off_status_from_approval_status($timeslot);

		// syncronize a time off request
		$time_off_request_data = array(
			"user_id" => $ts_employee_id,
			"time_off_request_notes" => array(
				array(
					"note" => $timeslot->getDescription()
				)
			),
			"time_off_request_entries" => array(
				array(
					"date" => $start_time_str,
					"entry_method" => "manual",
					"duration" => $time_seconds,
					"jobcode_id" => $time_off_code,
					"status" => $status,
				)
			)
		);

		$ts_time_off_entry_id = $timeslot->getColumnValue('tsheets_online_id');
		$ts_time_off_request_id = $timeslot->getColumnValue('tsheets_time_off_req_id');
		
		if ($ts_time_off_request_id == 0) {
			// is a new time off request
			
			$time_off_request = array($time_off_request_data);
			$result = $tsheets_client->add('time_off_requests', $time_off_request);			

		} else {

			// check if this time-off is not approved, denied or canceled, only update pending time-off
			$ts_object = get_tsheets_time_off_object($tsheets_client, $ts_time_off_request_id);
			if ($ts_object && $ts_object['status'] != 'pending') {
				update_feng_timeslot_status_using_tsheets_object($timeslot, $ts_object);
				return;
			}


			// edit exisiting time off request

			/* // THIS DOES NOT WORK, TSHEETS DOESN'T ALLOW TO EDIT A PENDING TIME OFF ENTRY => so we have to archive the current and create a new one
			$time_off_entry_request_data = $time_off_request_data["time_off_request_entries"][0];
			unset($time_off_entry_request_data['status']);
			$time_off_entry_request_data['id'] = $ts_time_off_entry_id;
			$result = $tsheets_client->put('time_off_request_entries', [$time_off_entry_request_data]);
			*/

			// due to tsheets restrictions we have to deny and archive the time off entry and add a new one to the time off request
			// delete (deny and archive)
			delete_ts_timeslot($timeslot);

			// set time off id
//			$time_off_request_data["id"] = $ts_time_off_request_id;

			// use api to modify the time off request with the new data
//			$time_off_request = array($time_off_request_data);
//			$result = $tsheets_client->put('time_off_requests', $time_off_request);

			$time_off_request = array($time_off_request_data);
			$result = $tsheets_client->add('time_off_requests', $time_off_request);
			
		}
		//Logger::log_r($result);

		$saved_time_off_request = reset($result['results']['time_off_requests']);
		if($saved_time_off_request['_status_code'] == 200){
			
			$timeslot->setColumnValue('tsheets_online_id', $saved_time_off_request['time_off_request_entries'][0]);
			$timeslot->setColumnValue('tsheets_time_off_req_id', $saved_time_off_request['id']);
			$timeslot->setColumnValue('needs_to_sync', false);
			$timeslot->save();

		} else {

			Logger::log_r("ERROR SAVING TIME OFF REQUEST:\n".$saved_time_off_request['_status_code']." - ".$saved_time_off_request['_status_message']."\n".print_r($saved_time_off_request, 1));

			set_if_timeslots_needs_to_sync(array($timeslot->getId()), true);

			// show received error to the user
			throw new Exception($saved_time_off_request['_status_message']);
		}

	} else {
		// synchronize a normal time activity

		if(!empty($job_id)){
			$request_data = array(
				"user_id"=> $ts_employee_id,
				"duration"=> $time_seconds,
				"jobcode_id"=> $job_id,
				"notes"=> $timeslot->getDescription(),
				"customfields"=> [
					$billable_id => $is_billable,
					$service_item_id => $name_item,
				]
			);
			if ($customer_type_id) {
				$request_data['customfields'][$customer_type_id] = $job_type;
			}
	
			if($type_entry == 'REGULAR'){
				$request_data['type'] = 'regular';
				$request_data['start'] = $start_time_str;
				$request_data['end'] = $end_time_str;
			}else{
				$request_data['type'] = 'manual';
				$request_data['date'] = $start_time_str;
			}
	
			if ($ts_time_activity_id == 0) {
				// new time entry
				$request[] = $request_data;
				
				$result = $tsheets_client->add(TSheets\ObjectType::Timesheets, $request);
				$result_save = reset($result['results']['timesheets']);
	
				if($result_save['_status_code'] == 200){
					$timeslot->setColumnValue('tsheets_online_id', $result_save['id']);
					$timeslot->setColumnValue('needs_to_sync', false);
					$timeslot->save();
				} else {
					set_if_timeslots_needs_to_sync(array($timeslot->getId()), true);
					
					$error = $result_save['_status_message'].": ".$result_save['_status_extra'];
					throw new Exception(build_qbo_error_message($error));
				}
				
			} else {
				// edit time entry
				$request_data["id"] = $ts_time_activity_id;
				$request[] = $request_data;
				$result_update = $tsheets_client->put(TSheets\ObjectType::Timesheets, $request);

			}
	
		} else {
			$error = lang('custom property value required', lang('project'));
			throw new Exception(build_qbo_error_message($error));    
		}
	}

}

function delete_ts_timeslot(Timeslot $timeslot){
	$tsheets_client = getInstanteTS();

	$ts_timeslot_id = $timeslot->getColumnValue("tsheets_online_id");
    
    if($ts_timeslot_id){

		$time_off_code = get_time_entry_time_off_code($timeslot);
		$is_time_off_entry = is_numeric($time_off_code) && $time_off_code > 0;

		if ($is_time_off_entry) {
			// this request sets as inactive the time off entry and the related time off request if it has only one entry

			// first deny it so we can set as inactive later
			$entry_request = array(array( 'id' => $ts_timeslot_id, 'status' => 'denied' ));
			$result = $tsheets_client->put('time_off_request_entries', $entry_request);
			
			// set as inactive
			$entry_request = array(array( 'id' => $ts_timeslot_id, 'active' => false ));
			$result = $tsheets_client->put('time_off_request_entries', $entry_request);

			$result_obj = reset($result['results']['time_off_request_entries']);

		} else {

			$result = $tsheets_client->delete(TSheets\ObjectType::Timesheets, [$ts_timeslot_id]);
			$result_obj = reset($result['results']['timesheets']);
			
		}
		
		$timeslot->setColumnValue("tsheets_online_id", 0);
		$timeslot->save();

        return $result_obj;
    }
}


function get_tsheets_time_off_object($tsheets_client, $ts_time_off_req_id) {

	$items = $tsheets_client->get('time_off_requests', ['ids' => $ts_time_off_req_id]);
	$get_ts_response = $items['results']['time_off_requests'];
	/* 
	Logger::log_r($items);
	$items['results'] has the result of the query
	$items['supplemental_data'] has the objects referenced by the result of the query, like users, time_off_request_notes, etc
	*/

	$ts_object = reset($get_ts_response); // get the first object ogj 

	return $ts_object;

}


/**
 * Calculate the status to send in the request depending on the timeslot's approval status 
 * We can only send "approved" or "pending" status in tsheets requests
 * If time is approved here we send it as approved, otherwise is pending
 * */
function get_tsheets_time_off_status_from_approval_status(Timeslot $timeslot) {

	$status = "pending";

	if (Plugins::instance()->isActivePlugin('timesheet_approval')) {
		$status_type = ObjectTypes::findByName('status_timesheet');
		if ($status_type instanceof ObjectType) {
			$time_status = $timeslot->getMemberOfType($status_type->getId());
			
			if ($time_status instanceof Member && $time_status->getId() == config_option('time_bulk_action_approved_member_id')) {
				$status = "approved";
			}
		}
	}

	return $status;
}

/**
 * When trying to sync a time entry and it is approved or denied then approve it or deny it in feng/evx
 */
function update_feng_timeslot_status_using_tsheets_object(Timeslot $timeslot, $ts_object) {
	if (Plugins::instance()->isActivePlugin('timesheet_approval')) {
		$member_to_add = null;

		// if time off is already approved, denied or canceled in Tsheets then update the status in EVX
		if ($ts_object['status'] == 'approved') {
			$member_to_add = config_option('time_bulk_action_approved_member_id'); // approved
		} else if ($ts_object['status'] == 'denied') {
			$member_to_add = config_option('time_bulk_action_review_member_id'); // rejected
		}
		if ($member_to_add) {

			$members_to_remove = array();
			$p_members = $timeslot->getMembers();
			$dim = Dimensions::findByCode('status_timesheet');
			foreach ($p_members as $m) {
				if ($m->getDimensionId() == $dim->getId()) {
					$members_to_remove[] = $m->getId();
				}
			}

			if (count($members_to_remove) > 0) {
				ObjectMembers::removeObjectFromMembers($timeslot, logged_user(), $members_to_remove, $members_to_remove, false);
			}
			
			$m = Members::getMemberById($member_to_add);
			if ($m instanceof Member) {
				ObjectMembers::addObjectToMembers($timeslot->getId(), array($m));

				// reload the view if the classification changed
				if (count($members_to_remove) > 0 && $members_to_remove[0] != $member_to_add) {
					evt_add("reload current panel");
				}
			}
		}
	}
}

function get_time_entry_time_off_code(Timeslot $timeslot) {

	$time_off_code = null;

	if (Plugins::instance()->isActivePlugin('hour_types')) {
		$labor_cat_ot_id = ObjectTypes::findByName('hour_type')->getId();
		$labor_cat = $timeslot->getMemberOfType($labor_cat_ot_id);

		$time_off_code = get_labor_category_time_off_code($labor_cat);
	}

	return $time_off_code;
}


function get_labor_category_time_off_code($labor_cat) {
	$time_off_code = null;

	if ($labor_cat instanceof Member) {
		$time_off_code = $labor_cat->getCustomPropertyValueByCode('tsheets_time_off_code');
	}

	return $time_off_code;
}


function validate_if_time_switched_from_time_off_to_normal_category($timeslot, $members) {

	$labor_cat_dim = Dimensions::findByCode('hour_types');
	if (!$labor_cat_dim) return;

	// check if the time entry is a time off request 
	$time_off_code = null;
	foreach ($members as $m) {
		if ($m->getDimensionId() == $labor_cat_dim->getId()) {
			$time_off_code = get_labor_category_time_off_code($m);
			break;
		}
	}
	$is_time_off_entry = is_numeric($time_off_code) && $time_off_code > 0;

	// if we have the previous data to compare then analize the previous labor category
	if ($timeslot->old_content_object instanceof Timeslot 
	&& isset($timeslot->old_content_object->member_ids) && count($timeslot->old_content_object->member_ids) > 0) {

		$old_members = Members::instance()->findAll(array(
			"conditions" => "id IN (".implode(',', $timeslot->old_content_object->member_ids).")"
		));

		// check if the time entry was a time off request 
		$was_time_off_entry_code = $is_time_off_entry; // start with equal values to prevent execption if time didn't have a labor category
		foreach ($old_members as $old_m) {
			if ($old_m->getDimensionId() == $labor_cat_dim->getId()) {
				$was_time_off_entry_code = get_labor_category_time_off_code($old_m);
				break;
			}
		}
		$was_time_off_entry = is_numeric($was_time_off_entry_code) && $was_time_off_entry_code > 0;

		// if time was switched from time-off to normal or from normal to time-off then throw an exception to rollback all changes and show error message to user
		if ($was_time_off_entry != $is_time_off_entry) {
			$continue = false;
			if ($was_time_off_entry) {
				$err_message = lang('time off entries cant be changed to normal time entries');
			} else {
				$err_message = lang('normal time entries cant be changed to time off entries');
			}
			throw new Exception($err_message);
		}
	}
	
}
