og.quickbooks = {
	
	init: function() {
		// event triggered after an invoice is added or edited insied feng
		og.eventManager.addListener('save_invoice_to_quickbooks', function(data) {
			if (data && data.id > 0) {
				og.quickbooks.save_invoice_to_quickbooks(data.id);
			}
		});
		// event triggered after an invoice payment is added or edited insied feng
		og.eventManager.addListener('save_invoice_payment_to_quickbooks', function(data) {
			if (data && data.id > 0) {
				og.quickbooks.save_invoice_payment_to_quickbooks(data.id);
			}
		});
		// event triggered after an invoice is added or edited insied feng
		og.eventManager.addListener('login_to_quickbooks', function(data) {
			if (data) {
				og.quickbooks.qbo_need_to_login(data);
			}
		});

		// event triggered after an timeslot is added or edited insied feng
		og.eventManager.addListener('save_timeslot_to_quickbook', function(data) {
			if (data.length) {
				og.quickbooks.save_timeslot_to_quickbook(data);
			}
		});
		
		// event triggered after an expense is added or edited insied feng
		og.eventManager.addListener('save_actual_expense_to_quickbooks', function(data) {
			if (data && data.id > 0) {
				og.quickbooks.save_actual_expense_to_quickbooks(data.id);
			}
		});
		
		
		// new column to show qbo sync status at invoices list
		if (!og.invoices.additional_list_columns) og.invoices.additional_list_columns = [];
		og.invoices.additional_list_columns.push({name:'qbo_sync_status', text:lang('quickbooks'), align:'left', sortable: true, renderer: og.quickbooks.render_grid_qbo_status});
		
		// new column to show qbo sync status at invoice payments list
		if (og.invoice_payment_receipts) {
			if (!og.invoice_payment_receipts.additional_list_columns) og.invoice_payment_receipts.additional_list_columns = [];
			og.invoice_payment_receipts.additional_list_columns.push({name:'qbo_sync_status', text:lang('quickbooks'), align:'left', sortable: true, renderer: og.quickbooks.render_grid_qbo_status});
		}


		og.eventManager.addListener('save_actual_expense_to_quickbooks', function(data) {
			if (data && data.id > 0) {
				og.quickbooks.save_actual_expense_to_quickbooks(data.id);
			}
		});

		// INVOICE MANAGER

		og.eventManager.addListener('invoice_append_actions', function(actions) {
			actions.sync_qb =  new Ext.Action({
				id: 'sync_qb',
				text: 'Sync QB',
				tooltip: '',
				iconCls: 'ico-invoice',
				//disabled: true,
				handler: function() {
					var ids_sel = actions.CONTEXT.getSelectedIds();
					og.income.sync_qb(ids_sel);
				}
			});

			actions.label_pendding =  new Ext.Action({
				id: 'label-qb',
				text: lang('you still have n'),
				tooltip: '',
				iconCls: 'ico-invoice',
				//disabled: true,
				handler: function() {
				}
			});

			actions.sync_qb.setDisabled(true);
		});

		og.eventManager.addListener('invoice_start_manager', function() {
			og.income.reload_not_sync();
		});

		og.eventManager.addListener('invoice_append_tbar', function(data) {
			data[0].push(data[1].sync_qb);
        	data[0].push(data[1].label_pendding);
		});

		og.eventManager.addListener('invoice_selectionchange_count_off', function(actions) {
			actions.archive.setDisabled(true);
            actions.sync_qb.setDisabled(true);
		});

		og.eventManager.addListener('invoice_selectionchange_count_on', function(actions) {
			actions.sync_qb.setDisabled(false);
		});

		// -- INVOICE MANAGER
	},

	render_grid_qbo_status: function(value, p, r) {
		if (r.data.id == '__total_row__') return value;
		
		if(value =='noApply'){
			return '<div>Not synchronized</div>';
		}else if (value > 0) {
			return '<div>' + lang('synchronized') + '</div>';
		} else {
			return '<div class="error">' + lang('not synchronized') + '</div>';
		}
	},
	
	/****************************************************************************************/
	// New QBO API integration functions
	/****************************************************************************************/
	
    OAuthCode : function(url) {

        this.loginPopup = function (parameter) {
            this.loginPopupUri(parameter);
        }

        this.loginPopupUri = function (parameter) {

            // Launch Popup
            var parameters = "location=1,width=800,height=650";
            parameters += ",left=" + (screen.width - 800) / 2 + ",top=" + (screen.height - 650) / 2;

            var win = window.open(url, 'connectPopup', parameters);
            var pollOAuth = window.setInterval(function () {
                try {

                    if (win.document.URL.indexOf("code") != -1) {
                        window.clearInterval(pollOAuth);
                        win.close();
                        
                        // after login -> save the invoice
                        if (parameter.action && parameter.id) {
                        	og.openLink(og.getUrl('quickbooks', parameter.action, {id: parameter.id}));
                        }
                        //location.reload();
                    }
                } catch (e) {
                    console.log(e)
                }
            }, 100);
        }
    },
  //  var oauth = new og.quickbooks.OAuthCode(url);
  	
	
    save_invoice_to_quickbooks: function(feng_invoice_id, sync_and_reload) {

		if (typeof(sync_and_reload) == 'undefined') sync_and_reload = 0;

		og.quickbooks.next_invoice_to_generate = {
			id: feng_invoice_id,
			sync_and_reload: sync_and_reload
		};

		// wait a bit to send the syncroniation request so the list can reload first and user doesnt have to wait for qb to see the invoices list
		setTimeout(function() {
			og.openLink(og.getUrl('quickbooks', 'save_invoice_to_quickbooks', og.quickbooks.next_invoice_to_generate), {
				callback: function(success, data) {
					if (data && data.need_qbo_login && data.auth_url) {
						var oauth = new og.quickbooks.OAuthCode(data.auth_url);
						oauth.loginPopup(data);
					}
				}
			});

			og.quickbooks.next_invoice_to_generate = null;
			
		}, 500);
    },

	save_actual_expense_to_quickbooks: function(feng_payment_id, sync_and_reload) {

		if (typeof(sync_and_reload) == 'undefined') sync_and_reload = 0;

		og.quickbooks.next_expense_to_generate = {
			feng_payments_ids: feng_payment_id,
			sync_and_reload: sync_and_reload
		};

		// wait a bit to send the syncroniation request so the list can reload first and user doesnt have to wait for qb to see the invoices list
		setTimeout(function() {
			og.openLink(og.getUrl('quickbooks', 'save_actual_expense_to_quickbooks', og.quickbooks.next_expense_to_generate), {
				callback: function(success, data) {
					if (data && data.need_qbo_login && data.auth_url) {
						var oauth = new og.quickbooks.OAuthCode(data.auth_url);
						oauth.loginPopup(data);
					}
				}
			});		
			
			og.quickbooks.next_expense_to_generate = null;

		}, 500);
    },

    save_timeslot_to_quickbook: function(feng_timeslots_ids, sync_and_reload) {
		var params = {
			feng_timeslots_ids: feng_timeslots_ids.join(',')
		};

		og.openLink(og.getUrl('quickbooks', 'save_timeslots_to_quickbooks', params), {
			callback: function(success, data) {
				if (data && data.need_qbo_login && data.auth_url) {
					var oauth = new og.quickbooks.OAuthCode(data.auth_url);
					oauth.loginPopup(data);
				}
			}
		});
    },
    
    save_invoice_payment_to_quickbooks: function(feng_payment_id, sync_and_reload) {

		if (typeof(sync_and_reload) == 'undefined') sync_and_reload = 0;

		og.quickbooks.next_payment_to_generate = {
			id: feng_payment_id,
			sync_and_reload: sync_and_reload
		};
    	
		// wait a bit to send the syncroniation request so the list can reload first and user doesnt have to wait for qb to see the invoices list
		setTimeout(function() {
			
			og.openLink(og.getUrl('quickbooks', 'save_invoice_payment_to_quickbooks', og.quickbooks.next_payment_to_generate), {
				callback: function(success, data) {
					if (data && data.need_qbo_login && data.auth_url) {
						var oauth = new og.quickbooks.OAuthCode(data.auth_url);
						oauth.loginPopup(data);
					}
				}
			});

			og.quickbooks.next_payment_to_generate = null;
			
		}, 500);
    	
    },
    
    qbo_need_to_login: function(data) {
    	if (data && data.need_qbo_login && data.auth_url) {
			var oauth = new og.quickbooks.OAuthCode(data.auth_url);
			oauth.loginPopup(data);
		}
    },

	send_to_tsheets: function(grid_id){
		var ids = [];
        var grid = Ext.getCmp(grid_id);
        if (grid) {
            ids = grid.getSelectedIds();
        }

        var params = {};
        if (ids.length > 0) {
        	params.timeslot_ids = ids;
        } else {
        	params.all_timeslots = true;
        	if (grid) {
        		Ext.apply(params, grid.store_params);
        	}
        }
        
    	og.openLink(og.getUrl('quickbooks', 'send_timeslots', params));
	}
}