<?php 

function quickbooks_update_1_2() {
	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."qbo_custom_field_match` (
		  `quickbooks_id` int(11) NOT NULL,
		  `feng_property1` varchar(100) COLLATE utf8_unicode_ci NOT NULL,
		  `feng_property2` varchar(100) COLLATE utf8_unicode_ci NOT NULL,
		  PRIMARY KEY (`quickbooks_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
	");
	
	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."qbo_user_hour_items_match` (
		  `user_id` int(11) NOT NULL,
		  `qbo_item_id` varchar(100) COLLATE utf8_unicode_ci NOT NULL,
		  PRIMARY KEY (`user_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
	");
}

function quickbooks_update_2_3() {
	$config_option_exists = ConfigOptions::getByName("qbo_dimension_for_classes");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`) VALUES
			 ('quickbooks_online', 'qbo_dimension_for_classes', (SELECT id FROM ".TABLE_PREFIX."dimensions WHERE code='project_types'), 'SingleDimensionConfigHandler', 0, 50, '')
			ON DUPLICATE KEY UPDATE `category_name`=`category_name`;
		");
	}
	
	// Add "shipping" custom properties to invoices
	$ship_addr_cp = CustomProperties::getCustomPropertyByCode(IncomeInvoices::instance()->getObjectTypeId(), 'shipping_address');
	if (!$ship_addr_cp) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."custom_properties` (`object_type_id`, `name`, `type`, `description`, `values`, `default_value`, `code`, `is_required`, `is_multiple_values`, `property_order`, `visible_by_default`, `show_in_lists`, `is_special`) VALUES
				((SELECT id FROM `".TABLE_PREFIX."object_types` WHERE name = 'invoice'),'Shipping address', 'address', '', '', '', 'shipping_address', 0, 0, 35, 1, 1, 1),
				((SELECT id FROM `".TABLE_PREFIX."object_types` WHERE name = 'invoice'),'Shipping date', 'date', '', '', '', 'shipping_date', 0, 0, 36, 1, 1, 1),
				((SELECT id FROM `".TABLE_PREFIX."object_types` WHERE name = 'invoice'),'Shipping via', 'text', '', '', '', 'shipping_method', 0, 0, 37, 1, 1, 1),
				((SELECT id FROM `".TABLE_PREFIX."object_types` WHERE name = 'invoice'),'Tracking number', 'text', '', '', '', 'shipping_tracking_num', 0, 0, 38, 1, 1, 1)
			ON DUPLICATE KEY UPDATE name = name;
		");
		
		if (Plugins::instance()->isActivePlugin('advanced_core')) {
			Env::useHelper('functions', 'advanced_core');
			fill_property_groups_default_data();
		}
	}
}

function quickbooks_update_3_4() {
	$config_option_exists = ConfigOptions::getByName("qbo_access_token");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`) VALUES
			 ('system', 'qbo_access_token', '', 'StringConfigHandler', 1, 0, '')
			ON DUPLICATE KEY UPDATE `category_name`=`category_name`;
		");
	}
	
	DB::execute("
		UPDATE `".TABLE_PREFIX."config_options`
		SET `value`='https://quickbooks.evxsoftware.com/callback.php'
		WHERE `name`='qbo_oauth_redirect_uri';
	");
	
	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."qbo_entities_to_process` (
		  `id` INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
		  `qbo_type` varchar(100) collate utf8_unicode_ci NOT NULL,
		  `qbo_id` INTEGER UNSIGNED NOT NULL DEFAULT 0,
		  `operation` varchar(100) collate utf8_unicode_ci NOT NULL,
		  `last_updated` DATETIME NOT NULL,
		  PRIMARY KEY (`id`),
		  KEY (qbo_type, qbo_id, last_updated)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
	");
	
	DB::execute("
		INSERT INTO `".TABLE_PREFIX."cron_events` (name, recursive, delay, is_system, enabled) VALUES 
		 ('import_pending_entities_from_quickbooks', 1, 120, 0, 1)
		ON DUPLICATE KEY UPDATE name=name;
	");
	
	// ensure that callback files are placed in the root folder
	$files = array('qbo_callback.php','qbo_webhook_callback.php');
	foreach ($files as $file) {
		@copy(ROOT . "/plugins/quickbooks/application/helpers/$file", ROOT . "/$file");
	}
}

function quickbooks_update_4_5() {
	
	if (!check_column_exists(TABLE_PREFIX."income_invoices", "quickbooks_online_sync_token")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."income_invoices`
			ADD `quickbooks_online_sync_token` int(10) unsigned NOT NULL DEFAULT '0';
		");
	}
	
	if (Plugins::instance()->isInstalledPlugin("invoice_payment_receipts")) {
		if (!check_column_exists(TABLE_PREFIX."invoice_payment_receipts", "quickbooks_online_sync_token")) {
			DB::execute("
				ALTER TABLE `".TABLE_PREFIX."invoice_payment_receipts`
				ADD `quickbooks_online_sync_token` int(10) unsigned NOT NULL DEFAULT '0';
			");
		}
	}
}

function quickbooks_update_5_6() {
	$config_option_exists = ConfigOptions::getByName("qbo_sync_only_first_level_classes");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`) VALUES
			 ('quickbooks_online', 'qbo_sync_only_first_level_classes', 1, 'BoolConfigHandler', 0, 0, '')
			ON DUPLICATE KEY UPDATE `category_name`=`category_name`;
		");
	}
}

function quickbooks_update_6_7() {
	if (!check_column_exists(TABLE_PREFIX."members", "quickbooks_online_id")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."members`
			ADD `quickbooks_online_id` int(10) unsigned NOT NULL DEFAULT '0';
		");
	}
	if (!check_column_exists(TABLE_PREFIX."members", "quickbooks_online_sync_token")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."members`
			ADD `quickbooks_online_sync_token` int(10) unsigned NOT NULL DEFAULT '0';
		");
	}
}

function quickbooks_update_7_8() {

	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."qbo_accounts_category_match` (
			`category_id` int(11) NOT NULL DEFAULT 0,
			`qbo_account_id` int(11) NOT NULL DEFAULT 0,
			PRIMARY KEY (`category_id`),
			KEY (`qbo_account_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
	");
}


function quickbooks_update_8_9() {

	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."qbo_items_category_match` (
			`category_id` int(11) NOT NULL DEFAULT 0,
			`qbo_item_id` int(11) NOT NULL DEFAULT 0,
			PRIMARY KEY (`category_id`),
			KEY (`qbo_item_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
	");
}


function quickbooks_update_9_10() {

	if (!check_column_exists(TABLE_PREFIX."qbo_items_category_match", "qbo_item_name")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."qbo_items_category_match`
			ADD `qbo_item_name` text COLLATE 'utf8_unicode_ci' NOT NULL;
		");
	}
}


function quickbooks_update_10_11() {

	if (!check_column_exists(TABLE_PREFIX."income_taxes", "quickbooks_online_id")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."income_taxes`
			ADD `quickbooks_online_id` varchar(100) COLLATE 'utf8_unicode_ci' NOT NULL DEFAULT '';
		");
	}
	if (!check_column_exists(TABLE_PREFIX."income_taxes", "quickbooks_online_sync_token")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."income_taxes`
			ADD `quickbooks_online_sync_token` int(10) unsigned NOT NULL DEFAULT '0';
		");
	}
}


function quickbooks_update_11_12() {

	// add quickbooks id to contacts table to map users
	if (!check_column_exists(TABLE_PREFIX."contacts", "quickbooks_online_id")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."contacts`
			ADD `quickbooks_online_id` varchar(100) COLLATE 'utf8_unicode_ci' NOT NULL DEFAULT '';
		");
	}
	if (!check_column_exists(TABLE_PREFIX."contacts", "quickbooks_online_sync_token")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."contacts`
			ADD `quickbooks_online_sync_token` int(10) unsigned NOT NULL DEFAULT '0';
		");
	}

	// add quickbooks id to timeslots table to synchronize time entries
	if (!check_column_exists(TABLE_PREFIX."timeslots", "quickbooks_online_id")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."timeslots`
			ADD `quickbooks_online_id` varchar(100) COLLATE 'utf8_unicode_ci' NOT NULL DEFAULT '';
		");
	}
	if (!check_column_exists(TABLE_PREFIX."timeslots", "quickbooks_online_sync_token")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."timeslots`
			ADD `quickbooks_online_sync_token` int(10) unsigned NOT NULL DEFAULT '0';
		");
	}

	if(Plugins::instance()->isActivePlugin('expenses2')){
		if (!check_column_exists(TABLE_PREFIX."payment_receipts", "quickbooks_online_id")) {
			DB::execute("
				ALTER TABLE `".TABLE_PREFIX."payment_receipts`
				ADD `quickbooks_online_id` varchar(100) NOT NULL;
			");
		}

		if (!check_column_exists(TABLE_PREFIX."payment_receipts", "quickbooks_online_sync_token")) {
			DB::execute("
				ALTER TABLE `".TABLE_PREFIX."payment_receipts`
				ADD `quickbooks_online_sync_token` int(10) NOT NULL DEFAULT '0';
			");
		}
	}

	if (!check_column_exists(TABLE_PREFIX."qbo_items_category_match", "qbo_expense_item_id")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."qbo_items_category_match`
			ADD `qbo_expense_item_id` int(11) NOT NULL DEFAULT '0',
			ADD `qbo_expense_item_name` text COLLATE 'utf8_unicode_ci' NOT NULL AFTER `qbo_expense_item_id`;
		");
	}



	// $qb_pay_method_cp = CustomProperties::getCustomPropertyByCode(PaymentReceipts::instance()->getObjectTypeId(), 'quickbooks_payment_method');
	// if (!$qb_pay_method_cp) {
	// 	DB::execute("
	// 		INSERT INTO `".TABLE_PREFIX."custom_properties` (`object_type_id`, `name`, `type`, `description`, `values`, `default_value`, `code`, `is_required`, `is_multiple_values`, `property_order`, `visible_by_default`, `show_in_lists`, `is_special`) VALUES
	// 			((SELECT id FROM `".TABLE_PREFIX."object_types` WHERE name = 'payment_receipt'),'Payment method', 'list', '', '', '', 'quickbooks_payment_method', 1, 0, 10, 1, 1, 1)
	// 		ON DUPLICATE KEY UPDATE name = name;
	// 	");
	// }
	// $qb_account_cp = CustomProperties::getCustomPropertyByCode(PaymentReceipts::instance()->getObjectTypeId(), 'quickbooks_payment_account');
	// if (!$qb_account_cp) {
	// 	DB::execute("
	// 		INSERT INTO `".TABLE_PREFIX."custom_properties` (`object_type_id`, `name`, `type`, `description`, `values`, `default_value`, `code`, `is_required`, `is_multiple_values`, `property_order`, `visible_by_default`, `show_in_lists`, `is_special`) VALUES
	// 			((SELECT id FROM `".TABLE_PREFIX."object_types` WHERE name = 'payment_receipt'),'Payment account', 'list', '', '', '', 'quickbooks_payment_account', 1, 0, 11, 1, 1, 1)
	// 		ON DUPLICATE KEY UPDATE name = name;
	// 	");
	// }

	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."quickbooks_account_types` (
			`account_type_id` int(11) NOT NULL AUTO_INCREMENT,
			`quickbooks_id` int(11) NOT NULL,
			`name` varchar(32) NOT NULL,
			`is_payment_account` tinyint(4) NOT NULL DEFAULT 0,
			PRIMARY KEY (`account_type_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;
	");

	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."quickbooks_accounts` (
			`account_id` int(11) NOT NULL AUTO_INCREMENT,
			`account_type_id` int(11) NOT NULL,
			`quickbooks_id` int(11) NOT NULL,
			`name` varchar(255) NOT NULL,
			`visible` int(11) NOT NULL DEFAULT 1,
			`is_default` int(11) NOT NULL DEFAULT 0,
			PRIMARY KEY (`account_id`),
			KEY `account_type_id` (`quickbooks_id`),
			KEY `account_type_id_2` (`account_type_id`),
			CONSTRAINT `".TABLE_PREFIX."quickbooks_accounts_ibfk_1` FOREIGN KEY (`account_type_id`) REFERENCES `".TABLE_PREFIX."quickbooks_account_types` (`account_type_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;
	");

	DB::execute("
		CREATE TABLE IF NOT EXISTS `".TABLE_PREFIX."quickbooks_payment_methods` (
			`payment_method_id` int(11) NOT NULL AUTO_INCREMENT,
			`quickbooks_id` int(11) NOT NULL,
			`name` varchar(255) NOT NULL,
			`type` varchar(255) NOT NULL,
			`visible` int(11) NOT NULL DEFAULT 1,
			`is_default` int(11) NOT NULL DEFAULT 0,
			PRIMARY KEY (`payment_method_id`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;
	");

	DB::execute("
		INSERT INTO `".TABLE_PREFIX."quickbooks_account_types` (`account_type_id`, `quickbooks_id`, `name`, `is_payment_account`) VALUES
			(1,	0,	'Bank',	1),
			(2,	0,	'Other Current Asset',	1),
			(3,	0,	'Fixed Asset',	0),
			(4,	0,	'Other Asset',	0),
			(5,	0,	'Accounts Receivable',	0),
			(6,	0,	'Equity',	0),
			(7,	0,	'Expense',	0),
			(8,	0,	'Other Expense',	0),
			(9,	0,	'Cost of Goods Sold',	0),
			(10,	0,	'Credit Card',	1),
			(11,	0,	'Long Term Liability',	0),
			(12,	0,	'Other Current Liability',	0),
			(13,	0,	'Income',	0),
			(14,	0,	'Other Income',	0);
	");

	if (Plugins::instance()->isActivePlugin('advanced_core')) {
		Env::useHelper('functions', 'advanced_core');
		fill_property_groups_default_data();
	}
}


function quickbooks_update_12_13() {
	// add quickbooks vendor id to contacts table to map employee vendor entity
	if (!check_column_exists(TABLE_PREFIX."contacts", "quickbooks_online_vendor_id")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."contacts`
			ADD `quickbooks_online_vendor_id` int(10) NOT NULL DEFAULT '0';
		");
	}
}

function quickbooks_update_13_14() {

	if (Plugins::instance()->isActivePlugin('member_custom_properties')) {

		$qb_class_cp = MemberCustomProperties::getCustomPropertyByCode(PaymentReceipts::instance()->getObjectTypeId(), 'qbo_class');
		if (!$qb_class_cp) {
			DB::execute("
				INSERT INTO `".TABLE_PREFIX."member_custom_properties` (`object_type_id`, `name`, `type`, `description`, `values`, `default_value`, `code`, `is_required`, `is_multiple_values`, `property_order`, `visible_by_default`, `show_in_lists`, `is_special`) VALUES
					((SELECT id FROM `".TABLE_PREFIX."object_types` WHERE name = 'project_type'),'QB Class', 'text', '', '', '', 'qbo_class', 0, 0, 10, 1, 1, 1)
				ON DUPLICATE KEY UPDATE name = name;
			");
		}

		if (Plugins::instance()->isActivePlugin('advanced_core')) {
			Env::useHelper('functions', 'advanced_core');
			fill_property_groups_default_data();
		}
	}

	// new config option to specify which types of expenses are going to be sync
	$config_option_exists = ConfigOptions::getByName("expense_types_to_sync");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`) VALUES
			('quickbooks_online', 'expense_types_to_sync', '', 'MultipleListConfigHandler', 0, 0, '', 'reimbursable,direct_to_vendor,rental_consumable,field_lab_testing')
			ON DUPLICATE KEY UPDATE `category_name`=`category_name`;
		");
	}

	// hide system config options
	DB::execute("
		UPDATE `".TABLE_PREFIX."config_options` SET `is_system`=1
		WHERE `name` IN ('qbo_authorization_request_url','qbo_token_endpoint_url','qbo_client_id','qbo_client_secret','qbo_oauth_scope','qbo_oauth_redirect_uri','qbo_base_url')
	");

}


function quickbooks_update_14_15() {
	if (!check_column_exists(TABLE_PREFIX."qbo_items_category_match", "qbo_expense_item_id")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."qbo_items_category_match`
			ADD `qbo_expense_item_id` int(11) NOT NULL DEFAULT '0',
			ADD `qbo_expense_item_name` text COLLATE 'utf8_unicode_ci' NOT NULL AFTER `qbo_expense_item_id`;
		");
	}
}


function quickbooks_update_16_17() {
	if (!check_column_exists(TABLE_PREFIX."timeslots", "tsheets_online_id")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."timeslots`
			ADD `tsheets_online_id` varchar(100) NOT NULL DEFAULT '0';
		");
	}

	$config_option_exists = ConfigOptions::getByName("have_time_sheets");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`) VALUES
			('quickbooks_online', 'have_time_sheets', 0, 'ListConfigHandler', 0, 0, '', '{\"option\":[{\"value\":0,\"text\":\"Quickbooks Time\"},{\"value\":1,\"text\":\"Tsheets\"}]}');
		");
	}

	$config_option_exists = ConfigOptions::getByName("time_sheets_token");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`) VALUES
			('quickbooks_online', 'time_sheets_token', '', 'StringConfigHandler', 0, 0, '', '');
		");
	}

	$config_option_exists = ConfigOptions::getByName("initial_setting_time_sheets");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`) VALUES
			('quickbooks_online', 'initial_setting_time_sheets', 0, 'BoolConfigHandler', 1, 0, '', '');
		");
	}

	// new config option to prevent synchronization of old entities
	$config_option_exists = ConfigOptions::getByName("qbo_sync_entities_from_date");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`) VALUES
			('quickbooks_online', 'qbo_sync_entities_from_date', '2022-01-01', 'DateTimeConfigHandler', 0, 0, '', '')
			ON DUPLICATE KEY UPDATE `category_name`=`category_name`;
		");
	}
}

function quickbooks_update_17_18() {
	if(Plugins::instance()->isActivePlugin('expenses2')){
		if (!check_column_exists(TABLE_PREFIX."payment_receipts", "qbo_reimburse_charge_id")) {
			DB::execute("
				ALTER TABLE `".TABLE_PREFIX."payment_receipts`
				ADD `qbo_reimburse_charge_id` int(10) NOT NULL DEFAULT 0;
			");
		}

		DB::execute("
			UPDATE ".TABLE_PREFIX."payment_receipts
			SET qbo_reimburse_charge_id = quickbooks_online_id + 1
			WHERE quickbooks_online_id > 0;
		");
	}
}


function quickbooks_update_18_19() {

	if (Plugins::instance()->isActivePlugin('member_custom_properties') && Plugins::instance()->isActivePlugin('expense_categories')) {

		$exp_cat_ot_id = ObjectTypes::findByName('expense_category')->getId();

		$qb_class_cp = MemberCustomProperties::getCustomPropertyByCode($exp_cat_ot_id, 'qbo_class');
		if (!$qb_class_cp) {
			DB::execute("
				INSERT INTO `".TABLE_PREFIX."member_custom_properties` (`object_type_id`, `name`, `type`, `description`, `values`, `default_value`, `code`, `is_required`, `is_multiple_values`, `property_order`, `visible_by_default`, `show_in_lists`, `is_special`) VALUES
					($exp_cat_ot_id, 'QB Class', 'text', '', '', '', 'qbo_class', 0, 0, 10, 1, 1, 1)
				ON DUPLICATE KEY UPDATE name = name;
			");
		}

		if (Plugins::instance()->isActivePlugin('advanced_core')) {
			Env::useHelper('functions', 'advanced_core');
			fill_property_groups_default_data();
		}
	}
}

function quickbooks_update_19_20() {

	$config_option_exists = ConfigOptions::getByName("qbo_sync_only_in_first_level_projects");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`) VALUES
			 ('quickbooks_online', 'qbo_sync_only_in_first_level_projects', 0, 'BoolConfigHandler', 0, 0, '')
			ON DUPLICATE KEY UPDATE `category_name`=`category_name`;
		");
	}
}


function quickbooks_update_20_21() {
	DB::execute("
		INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`) VALUES
		('quickbooks_online', 'synchronize_incoming_changes', 0, 'BoolConfigHandler', 0, 99, '')
		ON DUPLICATE KEY UPDATE name = name;
	");
}


function quickbooks_update_21_22() {

	if (!check_column_exists(TABLE_PREFIX."timeslots", "tsheets_time_off_req_id")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."timeslots`
			ADD `tsheets_time_off_req_id` int(10) NOT NULL DEFAULT 0;
		");
	}

	if (Plugins::instance()->isActivePlugin('member_custom_properties') && Plugins::instance()->isActivePlugin('hour_types')) {

		$hour_type_ot = ObjectTypes::findByName('hour_type');
		$time_off_code_cp = MemberCustomProperties::getCustomPropertyByCode($hour_type_ot->getId(), 'tsheets_time_off_code');
		if (!$time_off_code_cp) {
			$disabled = config_option('have_time_sheets') ? '0' : '1';
			DB::execute("
				INSERT INTO `".TABLE_PREFIX."member_custom_properties` (`object_type_id`, `name`, `type`, `description`, `values`, `default_value`, `code`, `is_required`, `is_multiple_values`, `property_order`, `visible_by_default`, `show_in_lists`, `is_special`, `is_disabled`) VALUES
					((SELECT id FROM `".TABLE_PREFIX."object_types` WHERE name = 'hour_type'),'Time Off Code', 'list', '', '', '', 'tsheets_time_off_code', 0, 0, 10, 1, 1, 1, $disabled)
				ON DUPLICATE KEY UPDATE name = name;
			");
		}

		if (Plugins::instance()->isActivePlugin('advanced_core')) {
			Env::useHelper('functions', 'advanced_core');
			fill_property_groups_default_data();
		}
	}
}

function quickbooks_update_22_23() {
	// disable deprecated custom properties for expenses 
	DB::execute("
		UPDATE `".TABLE_PREFIX."custom_properties` 
		SET is_disabled = 1
		WHERE code in ('quickbooks_payment_method', 'quickbooks_payment_account')
		AND object_type_id = (SELECT id FROM `".TABLE_PREFIX."object_types` WHERE name = 'payment_receipt');
	");
}


function quickbooks_update_23_24() {
	// use the values of the config option "set_actual_expense_type" in the option "expense_types_to_sync" so if any other type is added then it will be used automatically in this option
	DB::execute("
		UPDATE `".TABLE_PREFIX."config_options` 
		SET `options` = '<config_option:set_actual_expense_type>' 
		WHERE `name` = 'expense_types_to_sync';
	");
}


function quickbooks_update_24_25() {
	// enable payment method and payment account custom properties for expenses, so we can sync CC expenses with purchase entity
	DB::execute("
		UPDATE `".TABLE_PREFIX."custom_properties` 
		SET is_disabled = 0, is_required = 0
		WHERE code in ('quickbooks_payment_method', 'quickbooks_payment_account')
		AND object_type_id = (SELECT id FROM `".TABLE_PREFIX."object_types` WHERE name = 'payment_receipt');
	");
}


function quickbooks_update_25_26() {

	$config_option_exists = ConfigOptions::getByName("qbo_sync_only_approved_objects");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`) VALUES
			 ('quickbooks_online', 'qbo_sync_only_approved_objects', 0, 'BoolConfigHandler', 0, 0, '', '')
			ON DUPLICATE KEY UPDATE `category_name`=`category_name`;
		");
	}
}



function quickbooks_update_26_27() {

	if (Plugins::instance()->isActivePlugin('expenses2')) {
		$config_option_exists = ConfigOptions::getByName("qbo_dimension_for_classes_expenses");
		if (!$config_option_exists instanceof ConfigOption) {
			DB::execute("
				INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`) VALUES
				('quickbooks_online', 'qbo_dimension_for_classes_expenses', COALESCE((SELECT id FROM ". TABLE_PREFIX ."dimensions WHERE code='expense_categories'),0), 'SingleDimensionConfigHandler', 0, 0, '', '')
				ON DUPLICATE KEY UPDATE `category_name`=`category_name`;
			");
		}
	}
}


function quickbooks_update_27_28() {

	// add need to sync column to timeslots table to synchronize time entries using cron
	if (!check_column_exists(TABLE_PREFIX."timeslots", "needs_to_sync")) {
		DB::execute("
			ALTER TABLE `".TABLE_PREFIX."timeslots`
			ADD `needs_to_sync` tinyint(1) unsigned NOT NULL DEFAULT '0';
		");
	}

	// Add the cron event to save timeslots in QBO
	$sync_cron_event = CronEvents::instance()->getByName('send_timeslots_to_quickbooks');
	if (!$sync_cron_event instanceof CronEvent) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."cron_events` (`name`, `recursive`, `delay`, `is_system`, `enabled`) VALUES 
			 ('send_timeslots_to_quickbooks', 1, 1, 1, 1)
			ON DUPLICATE KEY UPDATE name=name;
		");
	}
}

function quickbooks_update_28_29() {

	$config_option_exists = ConfigOptions::getByName("override_expense_class_with_project");
	if (!$config_option_exists instanceof ConfigOption) {
		DB::execute("
			INSERT INTO `".TABLE_PREFIX."config_options` (`category_name`, `name`, `value`, `config_handler_class`, `is_system`, `option_order`, `dev_comment`, `options`) VALUES
			('quickbooks_online', 'override_expense_class_with_project', 0, 'BoolConfigHandler', 0, 0, '', '')
			ON DUPLICATE KEY UPDATE `category_name`=`category_name`;
		");
	}

}

