<?php
/**
 * Quickbooks controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class QuickbooksDesktopController extends ApplicationController {

	var $plugin_name = "quickbooks_desktop";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('constants', $this->plugin_name);
		Env::useHelper('functions', $this->plugin_name);
		
	}
	
	
	/****************************************************************************************************/
	/* QB DESKTOP FUNCTIONS */
	/****************************************************************************************************/
	
	function export_timeslots() {
		ajx_current("empty");
		
		try {
			// get the timeslots list
			$_REQUEST['limit'] = PHP_INT_MAX;
			$time_controller = new TimeController();
			$timeslots = $time_controller->list_all(true);
			
			
			$all_data = array();

			// get the timeslot individual header
			$line_headers = get_qb_export_timeslots_line_headers();
			$line = $line_headers;
			$line[0] = "!" . $line[0];
			$all_data[] = $line;
			
			// build data for each timeslot
			foreach ($timeslots as $timeslot) {
				$all_data[] = get_qb_export_timeslots_line_values($line_headers, $timeslot);
			}
			
			// concatenate all the generated data to save in a file
			$export_file_content = build_export_file_contents_from_arrays($all_data);
	
			// generate the filename and its metadata
			$now = DateTimeValueLib::now();
			$now->advance(logged_user()->getUserTimezoneValue());
			$now_str = $now->format("Y_m_d_h_i_s");
			$filename = "qb_timeslots_$now_str.iif";
			$filetype = "text/iif";
			
			// save the generated content in the file that will be downloaded
			file_put_contents(ROOT . "/tmp/$filename", $export_file_content);
			$filesize = filesize(ROOT . "/tmp/$filename");
			
			// send the file data to the view
			ajx_extra_data(array("filename" => $filename, 'size' => $filesize, 'type' => $filetype));
			
		} catch (Exception $e) {
			flash_error($e->getMessage());
			ajx_current("empty");
			return;
		}
	}


	function export_invoices() {
		ajx_current("empty");

		try {

			// get the invoices list
			$_GET['start'] = 0;
			$_GET['limit'] = PHP_INT_MAX;
			$invoice_controller = new InvoiceController();
			$invoices = $invoice_controller->list_all(true);
			
			$all_data = array();
			
			// get the definition headers
			$header_lines = get_qb_export_invoices_headers();
			foreach ($header_lines as &$header_line) {
				$header_line[0] = "!" . $header_line[0];
			}

			$all_data = array_merge($all_data, $header_lines);

			$end_transaction_line = get_qb_export_invoices_end_headers();
			
			// build data for each invoice
			foreach ($invoices as $invoice) {
				$invoice_main_data = get_qb_export_invoices_main_values($invoice);
				$invoice_lines_data = get_qb_export_invoices_line_values($invoice);
				
				$all_data[] = $invoice_main_data;
				$all_data = array_merge($all_data, $invoice_lines_data);
				$all_data[] = $end_transaction_line;
			}
			
			// concatenate all the generated data to save in a file
			$export_file_content = build_export_file_contents_from_arrays($all_data);

			// generate the filename and its metadata
			$now = DateTimeValueLib::now();
			$now->advance(logged_user()->getUserTimezoneValue());
			$now_str = $now->format("Y_m_d_h_i_s");
			$filename = "qb_invoices_$now_str.iif";
			$filetype = "text/iif";
			
			// save the generated content in the file that will be downloaded
			file_put_contents(ROOT . "/tmp/$filename", $export_file_content);
			$filesize = filesize(ROOT . "/tmp/$filename");
			
			// send the file data to the view
			ajx_extra_data(array("filename" => $filename, 'size' => $filesize, 'type' => $filetype));

		} catch (Exception $e) {
			flash_error($e->getMessage());
			ajx_current("empty");
			return;
		}

	}


	function export_actual_expenses() {
		ajx_current("empty");
		
		try {

			// get the expenses list
			$_GET['start'] = 0;
			$_GET['limit'] = PHP_INT_MAX;
			$controller = new PaymentReceiptController();
			$actual_expenses = $controller->list_all(true);
			$all_data = array();
			
			// // get the definition headers
			$header_lines = get_qb_export_actual_expenses_headers();
			foreach ($header_lines as &$header_line) {
				$header_line[0] = "!" . $header_line[0];
			}
			
			$all_data = array_merge($all_data, $header_lines);

			$end_transaction_line = get_qb_export_actual_expenses_end_headers();
			
			// build data for each actual_expense
			foreach ($actual_expenses as $actual_expense) {
				$actual_expense_main_data = get_qb_export_actual_expenses_main_values($actual_expense);
				$actual_expense_lines_data = get_qb_export_actual_expenses_line_values($actual_expense);
				
				if(!empty($actual_expense_lines_data[0])){
					$all_data[] = $actual_expense_main_data;
					$all_data = array_merge($all_data, $actual_expense_lines_data);
					$all_data[] = $end_transaction_line;
				}
			}
			
			// concatenate all the generated data to save in a file
			$export_file_content = build_export_file_contents_from_arrays($all_data);
			
			// generate the filename and its metadata
			$now = DateTimeValueLib::now();
			$now->advance(logged_user()->getUserTimezoneValue());
			$now_str = $now->format("Y_m_d_h_i_s");
			$filename = "qb_bills_$now_str.iif";
			$filetype = "text/iif";
			
			// save the generated content in the file that will be downloaded
			file_put_contents(ROOT . "/tmp/$filename", $export_file_content);
			$filesize = filesize(ROOT . "/tmp/$filename");
			
			// send the file data to the view
			ajx_extra_data(array("filename" => $filename, 'size' => $filesize, 'type' => $filetype));

		} catch (Exception $e) {
			flash_error($e->getMessage());
			ajx_current("empty");
			return;
		}

	}

	function map_employees_vendor(){

		$vendor_name = array_var($_REQUEST, 'vendor_name');
		if (!$vendor_name) {

			$cf_matches_rows = DB::executeAll("SELECT * FROM ".TABLE_PREFIX."qbd_vendor_by_employee");
			$qbo_employee_matches = array();
			foreach ($cf_matches_rows as $row) {
				$qbo_employee_matches[$row['eployee_id']] = $row['vendor_name'];
			}

			$users = Contacts::getAllUsers();
			tpl_assign('users', $users);
			tpl_assign('qbo_employee_matches', $qbo_employee_matches);
		}else{
			if(is_array($vendor_name)) {
				foreach ($vendor_name as $eployee_id => $vendor_name_text) {
					$vendor_name_text = DB::escape($vendor_name_text);
					DB::execute("
						INSERT INTO ".TABLE_PREFIX."qbd_vendor_by_employee (eployee_id, vendor_name) VALUES
						($eployee_id, $vendor_name_text)
						ON DUPLICATE KEY UPDATE vendor_name=$vendor_name_text;
					");
				}
			}

			flash_success(lang('map saved successfully'));
			ajx_current("back");
		}
	}

	function map_default_export_type_transaction(){
		$types = array(TYPE_BILL, TYPE_CREDIT_CARD);

		$type_data = array_var($_REQUEST, 'type_name');
		if (!$type_data) {
			$cf_matches_rows = DB::executeAll("SELECT * FROM ".TABLE_PREFIX."qbd_default_account_by_type");
			$qbo_type_matches = array();

			foreach ($cf_matches_rows as $row) {
				$qbo_type_matches[qbd_format_key($row['transaction_type'])] = $row['account'];
			}

			tpl_assign('types', $types);
			tpl_assign('qbo_type_matches', $qbo_type_matches);
		}else{
			if(is_array($type_data)) {
				foreach ($type_data as $name => $account) {
					$name = DB::escape($name);
					$account = DB::escape($account);
					DB::execute("
						INSERT INTO `".TABLE_PREFIX."qbd_default_account_by_type` (transaction_type, account) VALUES
						($name, $account)
						ON DUPLICATE KEY UPDATE account=$account;
					");
				}
			}

			flash_success(lang('map default export successfully'));
			ajx_current("back");
		}

	}
}
