<?php
/**
 * Risk controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class RiskController extends ApplicationController {

	var $plugin_name = "risks";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
	//	Env::useHelper('functions', $this->plugin_name);
	}
	
	function init() {
		require_javascript("RiskManager.js", $this->plugin_name);
		ajx_current("panel", "risks-containerpanel", null, null, true);
		ajx_replace(true);
	}
	
	function list_all() {
		ajx_current("empty");
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		$dim_order = null;
		$cp_order = null;
		$attributes = array("ids" => explode(',', array_var($_GET,'ids')));
		
		if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		} else if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		}
		
		$extra_conditions = "";
		
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		$only_count_result = array_var($_GET, 'only_result',false);
		
		$result = Risks::instance()->listing(array(
			"order" => $order,
			"order_dir" => $order_dir,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"extra_conditions" => $extra_conditions,
			"count_results" => true,
			"only_count_results" => $only_count_result,
			"start" => $start,
			"limit" => $limit,
		));
		$risks = $result->objects;
		$object = $this->prepareObject($risks, $start, $limit, $result);
		
		ajx_extra_data($object);
	}

	
	private function prepareObject($risks, $start, $limit, $res_obj) {
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"risks" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
		
		$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType(Risks::instance()->getObjectTypeId());
		$count = min(array($limit, count($risks)));
		for ($i = 0; $i < $count; $i++){
			if (isset($risks[$i])){
				$obj = $risks[$i];
				if ($obj instanceof Risk){
					$info = $obj->getArrayInfo();
					$info['memPath'] = json_encode($obj->getMembersIdsToDisplayPath());
					$object["risks"][] = $info;

					foreach ($custom_properties as $cp) {
						$object["risks"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $obj);
					}
    			}
			}
		}
		return $object;
	}

	
	function add() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$risk_data = array_var($_POST, 'risk');
		
		if(!is_array($risk_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$obj = new Risk();
			$obj->setObjectTypeId(Risks::instance()->getObjectTypeId());
			tpl_assign('risk', $obj);
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$risk = $this->save_risk($risk_data, $member_ids);
				
				DB::commit();

				ApplicationLogs::createLog($risk, ApplicationLogs::ACTION_ADD);
				flash_success(lang('success add risk', $risk->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	function edit() {
		$this->setTemplate('add');
		
		$risk = Risks::instance()->findById(get_id());
		if(!($risk instanceof Risk)) {
			flash_error(lang('risk dnx'));
			ajx_current("empty");
			return;
		}
		
		if (logged_user()->isGuest() || !$risk->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$notAllowedMember = '';
		if(!Risk::canAdd(logged_user(), active_context(), $notAllowedMember )) {
			if (str_starts_with($notAllowedMember, '-- req dim --')) flash_error(lang('must choose at least one member of', str_replace_first('-- req dim --', '', $notAllowedMember, $in)));
			else flash_error(lang('no context permissions to add',lang("Risks"),$notAllowedMember ));
			ajx_current("empty");
			return;
		}
		
		$risk_data = array_var($_POST, 'risk');
		
		if(!is_array($risk_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			tpl_assign('risk', $risk);
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$risk = $this->save_risk($risk_data, $member_ids, $risk);
				
				DB::commit();

				ApplicationLogs::createLog($risk, ApplicationLogs::ACTION_EDIT);
				flash_success(lang('success edit risk', $risk->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	private function save_risk($risk_data, $member_ids, $risk = null) {
		// Save Risk object
		$is_new = false;
		if (is_null($risk) || !$risk instanceof Risk) {
			$risk = new Risk();
			$is_new = true;
		}
		
		$risk->setFromAttributes($risk_data);
		
		$risk->save();
		
		$object_controller = new ObjectController();
		if (!is_null($member_ids)) {
			$object_controller->add_to_members($risk, $member_ids);
		}
		$object_controller->add_subscribers($risk);
		$object_controller->link_to_new_object($risk);
		$object_controller->add_custom_properties($risk);
		
		return $risk;
	}
	
	
	
	
	
	function view() {
		ajx_set_no_toolbar();
		$risk = Risks::instance()->findById(get_id());
		if(!($risk instanceof Risk)) {
			flash_error(lang('risk dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$risk->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		tpl_assign('risk', $risk);
	}
	
	
	
	private function resolveAction($action, $attributes){
				 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				$messages = array();
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$risk = Risks::instance()->findById($id);
					if (isset($risk) && $risk->canDelete(logged_user())){
						try{
							DB::beginWork();
							$risk->trash();
							DB::commit();
							ApplicationLogs::createLog($risk, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$messages[] = $e->getMessage();
							$err++;
						}
					} else {
						$err++;
						$messages[] = lang('cannot delete risk', $risk instanceof Risk ? clean($risk->getName()) : '');
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) .("\n - ".implode("\n - ",$messages)). ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			case "archive":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$risk = Risks::instance()->findById($id);
					if (isset($risk) && $risk->canEdit(logged_user())){
						try{
							DB::beginWork();
							$risk->archive();
							DB::commit();
							ApplicationLogs::createLog($risk, ApplicationLogs::ACTION_ARCHIVE);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error archive objects", $err) . "<br />" . ($succ > 0 ? lang("success archive objects", $succ) : "");
				} else {
					$resultMessage = lang("success archive objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;		
		} // switch
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
	}
}