<?php
Hook::register('roles_dimension');

function roles_dimension_object_definition($type, &$ret) {
	if ($type == 'Contact') {
		$ret['default_role_id'] = DATA_TYPE_INTEGER;
	}
}


function roles_dimension_additional_system_columns($manager, &$columns) {
	if ($manager instanceof Contacts) {
		$columns[] = 'default_role_id';
	}
}


function roles_dimension_before_render_main_custom_properties($params, &$categories) {

	$object = array_var($params, 'object');
	//$genid = array_var($params, 'genid');
	$genid = gen_id();

	if ($object instanceof Contact) {
		
		if ($object->getUserType() > 0 || ($object->isNew() && array_var($_REQUEST, 'is_user') > 0)) {
			
			$dimension = Dimensions::findByCode('roles');
			$default_member_id = $object->getColumnValue('default_role_id');
			
			echo '<div class="dataBlock">';
			render_single_member_selector($dimension, $genid, array($default_member_id), array(
					'hidden_field_name' => 'contact[default_role_id]',
					'label' => lang('role'),
					'is_multiple' => false,
			));
			echo '</div>';
		}
		
	}
}

function roles_dimension_before_object_save($object, &$ret) {
	if ($object instanceof Contact) {
		
		$default_role_id_json = array_var($_REQUEST['contact'], 'default_role_id');
		$decoded = json_decode($default_role_id_json, true);
		$default_role_id = count($decoded) > 0 ? $decoded[0] : 0;
		
		$object->setColumnValue('default_role_id', $default_role_id);
	}
}


function roles_dimension_after_contact_view($contact, &$ret) {
	
	if ($contact->getUserType() > 0 && $contact->getColumnValue('default_role_id') > 0) {
		
		$mem = Members::instance()->findById($contact->getColumnValue('default_role_id'));
		if ($mem instanceof Member) {
			
			$member_breadcrumb .= '<span class="member-path real-breadcrumb og-wsname-color-'.$mem->getColor().'">'.$mem->getName().'</span>';
			
			$html = '<div class="basic-info">
						<div class="info-type">'. lang('role') .'</div>
						<div class="info-content">
							<div class="info-content-item">'.$member_breadcrumb.'</div>
						</div>
						<div class="clear"></div>
					</div>';
			echo $html;
		}
	}
	
}


function roles_dimension_preselected_time_form_member_ids($params, &$pre_selected_member_ids) {
	$timeslot = array_var($params, 'object');
	
	if ($timeslot instanceof Timeslot) {
		
		$ts_contact = Contacts::instance()->findById($timeslot->getContactId());
		if ($ts_contact instanceof Contact && $ts_contact->getColumnValue('default_role_id') > 0) {
			
			if (!is_array($pre_selected_member_ids)) $pre_selected_member_ids = array();
			
			$role_set = false;
			if (count($pre_selected_member_ids) > 0) {
				$roles_dimension = Dimensions::findByCode('roles');
				foreach ($pre_selected_member_ids as $mid) {
					$m = Members::instance()->findById($mid);
					if ($m instanceof Member && $m->getDimensionId() == $roles_dimension->getId()) {
						$role_set = true;
						break;
					}
				}
			}
			if (!$role_set) {
				$pre_selected_member_ids[] = $ts_contact->getColumnValue('default_role_id');
			}
		}
	}
}

function roles_dimension_after_api_add_timeslot($params, &$modified) {
	$timeslot = array_var($params, 'timeslot');
	
	if ($timeslot instanceof Timeslot) {
	
		$ts_contact = Contacts::instance()->findById($timeslot->getContactId());
		if ($ts_contact instanceof Contact && $ts_contact->getColumnValue('default_role_id') > 0) {
			$role_set = false;
			$roles_dimension = Dimensions::findByCode('roles');
			
			$ts_members = $timeslot->getMembers();
			foreach ($ts_members as $m) {
				if ($m instanceof Member && $m->getDimensionId() == $roles_dimension->getId()) {
					$role_set = true;
					break;
				}
			}
			if (!$role_set) {
				$role_member = Members::getMemberById($ts_contact->getColumnValue('default_role_id'));
				if ($role_member instanceof Member) {
					ObjectMembers::addObjectToMembers($timeslot->getId(), array($role_member));
					$modified = true;
				}
			}
		}
	}
}


/**
 * This function is used when grouping timeslots by person for invoicing
 * hour types plugin is used then the hour type (labor category) member id will be appended to the grouping key
 */
function roles_dimension_override_timeslot_person_key_for_grouping($t, &$key) {
	$dim = Dimensions::findByCode('roles');
	$members = $t->getMembers();
	foreach ($members as $m) {
		if ($m instanceof Member && $m->getDimensionId() == $dim->getId()) {
			$key .= "-".$m->getId();
			break;
		}
	}
}

/**
 * Classify task in the role member of the assigned user after instantiating the task from a template
 * @param array $parameters
 * @param null $ignored
 */
function roles_dimension_after_template_object_instantiation_and_commit($parameters, &$ignored) {
	$object = array_var($parameters, 'object');
	
	if ($object instanceof ProjectTask) {
		
		$assigned_user = $object->getAssignedToContact();
		if ($assigned_user instanceof Contact && $assigned_user->getColumnValue('default_role_id') > 0) {
			
			$role_set = false;
			$roles_dimension = Dimensions::findByCode('roles');
			
			$task_members = $object->getMembers();
			foreach ($task_members as $m) {
				if ($m instanceof Member && $m->getDimensionId() == $roles_dimension->getId()) {
					$role_set = true;
					break;
				}
			}
			
			if (!$role_set) {
				$role_member = Members::getMemberById($assigned_user->getColumnValue('default_role_id'));
				if ($role_member instanceof Member) {
					ObjectMembers::addObjectToMembers($object->getId(), array($role_member));
				}
			}
			
			// apply the role member to the next repetitions
			$repetition_ids = DB::executeAll("SELECT object_id from ".TABLE_PREFIX."project_tasks WHERE original_task_id=".$object->getId());
			if ($repetition_ids) {
				if (!$role_member) {
					$role_member = Members::getMemberById($assigned_user->getColumnValue('default_role_id'));
				}
				if ($role_member) {
					$repetition_ids = array_flat($repetition_ids);
					foreach ($repetition_ids as $task_id) {
						ObjectMembers::addObjectToMembers($task_id, array($role_member));
					}
				}
			}
		}
	}
}


