<?php
use Office365\SharePoint\Folder;
use Office365\SharePoint\SPList;
use Office365\SharePoint\ClientContext;
use Office365\Runtime\Auth\AuthenticationContext;

/**
 * Create folder in sharepoint
 * 
 * @param mixed $folder_name
 * @param array $subfolders
 * 
 * @return Office365\SharePoint\Folder
 */
function create_project_folder_in_sharepoint($folder_name, array $subfolders = []): Folder
{
	set_time_limit(0);
	ini_set('memory_limit', '1G');
	Env::useLibrary('phpSPO', 'sharepoint_integration');
	$settings = array(
			'BaseUrl' => config_option('sp_base_url'),
			'SitePath' => config_option('sp_site_path'),
			'UserName' => config_option('sp_user'),
			'Password' => config_option('sp_password'),
	);
	// authenticate with the general url, only the domain, without path
	$authCtx = new AuthenticationContext($settings['BaseUrl']);
	$authCtx->acquireTokenForUser($settings['UserName'], $settings['Password']);
	// set the context using the url containing the site path
	$context = new ClientContext($settings['BaseUrl'] . $settings['SitePath'], $authCtx);
	
	$site = $context->getSite();
	$context->load($site); //load site settings
	$context->executeQuery();
	
	$list = $context->getWeb()->getLists()->getByTitle("Documents");
	
	return do_create_project_folder_in_sharepoint($context, $list, $folder_name);
}

/**
 * Do create folder in sharepoint
 * 
 * @param Office365\SharePoint\ClientContext $context
 * @param Office365\SharePoint\SPList $targetList
 * @param mixed $folderName
 * @param array $subfolders
 * 
 * @return Office365\SharePoint\Folder
 */
function do_create_project_folder_in_sharepoint(ClientContext $context, SPList $targetList, $folderName, array $subfolders = []): Folder 
{
	$root_parent_folder = config_option('sp_projects_dir');
	
	// add the folder for the project
	$projects_root_folder = $targetList->getRootFolder()->getFolders()->getByUrl($root_parent_folder);
	$folder = $projects_root_folder->getFolders()->add($folderName);
	$context->executeQuery();
	
	// add the project folder childs
	foreach ($subfolders as $child_folder) {
		$folder->getFolders()->add($child_folder);
		$context->executeQuery();
	}
	
	return $folder;
}