<?php
/**
 * Ticket controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class TicketController extends ApplicationController {

	var $plugin_name = "tickets";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
	//	Env::useHelper('functions', $this->plugin_name);
	}
	
	function init() {
		require_javascript("TicketManager.js", $this->plugin_name);
		ajx_current("panel", "tickets-containerpanel", null, null, true);
		ajx_replace(true);
	}
	
	function list_all() {
		ajx_current("empty");
		$start = array_var($_GET,'start', 0);
		$limit = array_var($_GET,'limit', config_option('files_per_page'));
		$order = array_var($_GET,'sort');
		$order_dir = array_var($_GET,'dir');
		$action = array_var($_GET,'action');
		$dim_order = null;
		$join_params = null;
		$cp_order = null;
		$attributes = array("ids" => explode(',', array_var($_GET,'ids')));
		$select_columns = array();
		
		if (str_starts_with($order, "cp_")) {
			$cp_order = substr($order, 3);
			$order = 'customProp';
		} else if (str_starts_with($order, "dim_")) {
			$dim_order = substr($order, 4);
			$order = 'dimensionOrder';
		} 
		/*else if ($order == "dateUpdated") {
			$order = 'updated_on';
		} else if ($order == 'dateCreated') {
			$order = 'created_on';
		} else if ($order == 'updatedBy') {
			$order = 'updated_by';
		}
		*/
		switch ($order){
			case 'dateUpdated':
				$order = '`updated_on`';
				break;
			case 'dateCreated':
				$order = '`created_on`';
				break;
			case 'name':
				$order = '`name`';
				break;
			case 'updatedBy':
				$order = '`jt`.`name`';
				$join_params = array(
					'join_type' => 'INNER',
					'table' => Objects::instance()->getTableName(),
					'jt_field' => 'id',
					'e_field' => 'updated_by_id',
					'get_object_data' => true
				);
				break;
			case 'createdBy':
				$order = '`jt`.`name`';
				$join_params = array(
					'join_type' => 'INNER',
					'table' => Objects::instance()->getTableName(),
					'jt_field' => 'id',
					'e_field' => 'created_by_id',
					'get_object_data' => true
				);
				break;
			default:
				$order = '`updated_on`';  
				break;
		}

		// Check if get_object_data exist in $join_params, assign $select_columns
	    if(array_var($join_params, 'get_object_data')){
			$select_columns = array('DISTINCT(o.id), o.*, e.*, jt.name jt_name');
		}
		
		$extra_conditions = "";
		/*$status_filter = array_var($_GET,'status_filter');
		if ($status_filter != -1) {
			$extra_conditions .= " AND `status`=$status_filter";
		}*/
		
		$actionMessage = array();
		if (isset($action)) {
			$actionMessage = $this->resolveAction($action, $attributes);
			if ($actionMessage["errorCode"] == 0) {
				flash_success($actionMessage["errorMessage"]);
			} else {
				flash_error($actionMessage["errorMessage"]);
			}
		}
		
		$only_count_result = array_var($_GET, 'only_result',false);
		
		Hook::fire("listing_extra_conditions", null, $extra_conditions);
		
		$result = Tickets::instance()->listing(array(
			"order" => $order,
			"order_dir" => $order_dir,
			"join_params"=> $join_params,
			"dim_order" => $dim_order,
			"cp_order" => $cp_order,
			"extra_conditions" => $extra_conditions,
			"count_results" => true,
			"only_count_results" => $only_count_result,
			"start" => $start,
			"limit" => $limit,
			"select_columns" => $select_columns
		));
		$tickets = $result->objects;
		$object = $this->prepareObject($tickets, $start, $limit, $result);
		
		/*$object['filters'] = array('status' => array(
			array('id' => 0, 'name' => lang('incomplete')),
			array('id' => 1, 'name' => lang('in progress')),
			array('id' => 2, 'name' => lang('complete')),
		));*/
		
		ajx_extra_data($object);
	}

	
	private function prepareObject($tickets, $start, $limit, $res_obj) {
		$object = array(
			"totalCount" => $res_obj->total,
			"start" => $start,
			"Tickets" => array()
		);
		foreach ($res_obj as $k => $v) {
			if ($k != 'total' && $k != 'objects') $object[$k] = $v;
		}
		
		$custom_properties = CustomProperties::getAllCustomPropertiesByObjectType(Tickets::instance()->getObjectTypeId());
		$count = min(array($limit, count($tickets)));
		for ($i = 0; $i < $count; $i++){
			if (isset($tickets[$i])){
				$obj = $tickets[$i];
				if ($obj instanceof Ticket){
					$info = $obj->getArrayInfo();
					$info['memPath'] = json_encode($obj->getMembersIdsToDisplayPath());
					$object["tickets"][] = $info;

					foreach ($custom_properties as $cp) {
						$object["tickets"][$i]['cp_'.$cp->getId()] = get_custom_property_value_for_listing($cp, $obj);
					}
    			}
			}
		}
		return $object;
	}

	
	function add() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$ticket_data = array_var($_POST, 'ticket');
		
		if(!is_array($ticket_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			$obj = new Ticket();
			$obj->setObjectTypeId(Tickets::instance()->getObjectTypeId());

			$selected_members_ids = null;
			$pre_linked_objects = array();

			if (Plugins::instance()->isActivePlugin('mail')) {
				$from_email = array_var($_GET, 'from_email');
				$email = MailContents::instance()->findById($from_email);
				if ($email instanceof MailContent) {
					$obj->setObjectName($email->getSubject());
					$obj->setText(lang('create task from email description', $email->getSubject(), $email->getFrom(), $email->getTextBody()));

					$selected_members_ids = $email->getMemberIds();

					$pre_linked_objects = array($email);
				}
			}

			tpl_assign('ticket', $obj);
			tpl_assign('selected_members_ids', $selected_members_ids);
			tpl_assign('pre_linked_objects', $pre_linked_objects);
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$ticket = $this->save_ticket($ticket_data, $member_ids);
				
				DB::commit();

				ApplicationLogs::createLog($ticket, ApplicationLogs::ACTION_ADD);
				flash_success(lang('success add ticket', $ticket->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	function edit() {
		$this->setTemplate('add');
		
		$ticket = Tickets::instance()->findById(get_id());
		if(!($ticket instanceof Ticket)) {
			flash_error(lang('ticket dnx'));
			ajx_current("empty");
			return;
		}
		
		if (logged_user()->isGuest() || !$ticket->canEdit(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$notAllowedMember = '';
		if(!Ticket::canAdd(logged_user(), active_context(), $notAllowedMember )) {
			if (str_starts_with($notAllowedMember, '-- req dim --')) flash_error(lang('must choose at least one member of', str_replace_first('-- req dim --', '', $notAllowedMember, $in)));
			else flash_error(lang('no context permissions to add',lang("Tickets"),$notAllowedMember ));
			ajx_current("empty");
			return;
		}
		
		$ticket_data = array_var($_POST, 'ticket');
		
		if(!is_array($ticket_data)) {
			// set layout for modal form
			if (array_var($_REQUEST, 'modal')) {
				$this->setLayout("json");
				tpl_assign('modal', true);
			}
			tpl_assign('ticket', $ticket);
		} else {
			// form submitted
			try {
				DB::beginWork();
				
				$member_ids = json_decode(array_var($_POST, 'members'));
				
				$ticket = $this->save_ticket($ticket_data, $member_ids, $ticket);
				
				DB::commit();

				ApplicationLogs::createLog($ticket, ApplicationLogs::ACTION_EDIT);
				flash_success(lang('success edit ticket', $ticket->getObjectName()));
				ajx_current("back");
	          	
				if (array_var($_REQUEST, 'modal')) {
					evt_add("reload current panel");
				}
				
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				Logger::log($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	private function save_ticket($ticket_data, $member_ids, $ticket = null) {
		// Save Ticket object
		$is_new = false;
		if (is_null($ticket) || !$ticket instanceof Ticket) {
			$ticket = new Ticket();
			$is_new = true;
		}
				
		$ticket_data['text'] = preg_replace("/[\n|\r|\n\r]/", '', array_var($ticket_data, 'text'));
		
		$ticket->setFromAttributes($ticket_data);
		
		$ticket->save();
		
		$object_controller = new ObjectController();
		if (!is_null($member_ids)) {
			$object_controller->add_to_members($ticket, $member_ids);
		}
		$object_controller->add_subscribers($ticket);
		$object_controller->link_to_new_object($ticket);
		$object_controller->add_custom_properties($ticket);
		
		return $ticket;
	}
	
	
	
	
	
	function view() {
		ajx_set_no_toolbar();
		$ticket = Tickets::instance()->findById(get_id());
		if(!($ticket instanceof Ticket)) {
			flash_error(lang('ticket dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$ticket->canView(logged_user())) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		tpl_assign('ticket', $ticket);
	}
	
	
	
	private function resolveAction($action, $attributes){
				 
		$resultMessage = "";
		$resultCode = 0;
		switch ($action){
			case "delete":
				$succ = 0; $err = 0;
				$messages = array();
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$ticket = Tickets::instance()->findById($id);
					if ($ticket instanceof Ticket && $ticket->canDelete(logged_user())){
						try{
							DB::beginWork();
							$ticket->trash();
							DB::commit();
							ApplicationLogs::createLog($ticket, ApplicationLogs::ACTION_TRASH);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$messages[] = $e->getMessage();
							$err++;
						}
					} else {
						$err++;
						$messages[] = lang('cannot delete ticket', $ticket instanceof Ticket ? clean($ticket->getName()) : '');
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error delete objects", $err) .("\n - ".implode("\n - ",$messages)). ($succ > 0 ? "\n".lang("success delete objects", $succ) : "");
				} else {
					$resultMessage = lang("success delete objects", $succ);
				}
				break;
			case "archive":
				$succ = 0; $err = 0;
				for($i = 0; $i < count($attributes["ids"]); $i++){
					$id = $attributes["ids"][$i];
					$ticket = Tickets::instance()->findById($id);
					if ($ticket instanceof Ticket && $ticket->canEdit(logged_user())){
						try{
							DB::beginWork();
							$ticket->archive();
							DB::commit();
							ApplicationLogs::createLog($ticket, ApplicationLogs::ACTION_ARCHIVE);
							$succ++;
						} catch(Exception $e){
							DB::rollback();
							$err++;
						}
					} else {
						$err++;
					}
				}; // for
				if ($err > 0) {
					$resultCode = 2;
					$resultMessage = lang("error archive objects", $err) . "<br />" . ($succ > 0 ? lang("success archive objects", $succ) : "");
				} else {
					$resultMessage = lang("success archive objects", $succ);
				}
				break;
			default:
				$resultMessage = lang("Unimplemented action: '" . $action . "'");// if 
				$resultCode = 2;	
				break;		
		} // switch
		return array("errorMessage" => $resultMessage, "errorCode" => $resultCode);
	}
}