<?php

function get_tickets_request_types() {
	$values = array();
	
	$cp = CustomProperties::instance()->findOne(array('conditions' => "code='request_type_special'"));
	if ($cp instanceof CustomProperty) {
		$values = explode(',', $cp->getValues());
	}
	
	return $values;
}


function save_external_ticket($data) {
	$ticket_ot = ObjectTypes::findByName('ticket');
	
	// prepare ticket data
	$ticket_data = array('name' => array_var($data, 'name'));
	$custom_properties_data = array();
	
	// ticket type and description
	$cp_desc = CustomProperties::instance()->findOne(array('conditions' => "code='description_special' AND object_type_id=".$ticket_ot->getId()));
	if ($cp_desc instanceof CustomProperty) {
		$custom_properties_data[$cp_desc->getId()] = array_var($data, 'description');
	}
	$cp_rt = CustomProperties::instance()->findOne(array('conditions' => "code='request_type_special' AND object_type_id=".$ticket_ot->getId()));
	if ($cp_rt instanceof CustomProperty) {
		$custom_properties_data[$cp_rt->getId()] = array_var($data, 'request_type');
	}
	
	// fixed statuses
	$cp_is = CustomProperties::instance()->findOne(array('conditions' => "code='internal_status_special' AND object_type_id=".$ticket_ot->getId()));
	if ($cp_is instanceof CustomProperty) {
		$custom_properties_data[$cp_is->getId()] = 'New';
	}
	$cp_cs = CustomProperties::instance()->findOne(array('conditions' => "code='client_status_special' AND object_type_id=".$ticket_ot->getId()));
	if ($cp_cs instanceof CustomProperty) {
		$custom_properties_data[$cp_cs->getId()] = 'New';
	}
	
	// requested by
	$contact_id = 0;
	$email_address = trim(array_var($data, 'email', ''));
	if ($email_address == "") {
		throw new Exception("Reporter email address missing.");
	}
	$ce = ContactEmails::instance()->findOne(array('conditions' => "email_address='$email_address'"));
	if ($ce instanceof ContactEmail) {
		$contact_id = $ce->getContactId();
	}
	
	$contact = Contacts::instance()->findById($contact_id);
	if (!$contact instanceof Contact) {
		$contact = new Contact();
		$contact->setFirstName(array_var($data, 'user_name'));
		$contact->setObjectName();
		$contact->save();
		$contact->addEmail(array_var($data, 'email'), 'personal', true);
	}
	$cp_req_by = CustomProperties::instance()->findOne(array('conditions' => "code='requested_by_special' AND object_type_id=".$ticket_ot->getId()));
	if ($cp_req_by instanceof CustomProperty && $contact instanceof Contact) {
		$custom_properties_data[$cp_req_by->getId()] = $contact->getId();
	}
	
	// save ticket
	$ticket = new Ticket();
	
	$ticket->setFromAttributes($ticket_data);
	
	$ticket->save();
	
	$member_ids = array();
	if (Plugins::instance()->isActivePlugin('crpm')) {
		// classify ticket in client if reporter is associated to a client
		$customer = Customers::instance()->findOne(array('conditions' => "contact_id=".$contact->getId()));
		if (!$customer instanceof Customer && $contact->getCompanyId() > 0) {
			$customer = Customers::instance()->findOne(array('conditions' => "contact_id=".$contact->getCompanyId()));
		}
		if ($customer instanceof Customer) {
			Env::useHelper('functions', 'crpm');
			$mem_rows = DB::executeAll("select id from ".TABLE_PREFIX."members where object_id=".$customer->getId()." and dimension_id=".get_customers_dimension()->getId());
			if (is_array($mem_rows) && count($mem_rows) > 0) {
				$row = $mem_rows[0];
				$member_ids[] = array_var($row, 'id');
			}
		}
	}

	$user = Contacts::instance()->findOne(array('conditions' => "user_type IN (SELECT id FROM ".TABLE_PREFIX."permission_groups WHERE name IN ('Super Administrator', 'Administrator'))"));
	
	$object_controller = new ObjectController();
	$object_controller->add_to_members($ticket, $member_ids, $user);
	$object_controller->add_subscribers($ticket);
//	$object_controller->link_to_new_object($ticket);
	$object_controller->add_custom_properties($ticket, $custom_properties_data);
	
	return $ticket;
}


function get_external_tickets_list($user_email, $order_by=null, $order_dir=null) {
	Env::useHelper('format');
	$ticket_ot = ObjectTypes::findByName('ticket');
	
	$list = array();
	$extra_conditions = "";
	
	$contact_id = 0;
	$ce = ContactEmails::instance()->findOne(array('conditions' => "email_address='$user_email'"));
	if ($ce instanceof ContactEmail) {
		$contact_id = $ce->getContactId();
	}
	$contact = Contacts::instance()->findById($contact_id);
	
	$cp_desc = CustomProperties::instance()->findOne(array('conditions' => "code='description_special' AND object_type_id=".$ticket_ot->getId()));
	$cp_cs = CustomProperties::instance()->findOne(array('conditions' => "code='client_status_special' AND object_type_id=".$ticket_ot->getId()));
	
	if (!$order_by) $order_by = 'updated_on';
	if (!$order_dir) $order_dir = 'DESC';
	
	$cp_req_by = CustomProperties::instance()->findOne(array('conditions' => "code='requested_by_special' AND object_type_id=".$ticket_ot->getId()));
	
	if ($cp_req_by instanceof CustomProperty && $contact instanceof Contact) {
		$extra_conditions .= " AND EXISTS (SELECT cpv.id FROM ".TABLE_PREFIX."custom_property_values cpv 
			WHERE cpv.object_id=o.id AND cpv.custom_property_id=".$cp_req_by->getId()." AND cpv.value='".$contact->getId()."')";
		
		$tickets = Tickets::instance()->findAll(array(
			'conditions' => "trashed_by_id=0 $extra_conditions",
			'order' => "$order_by $order_dir"
		));
		
		foreach ($tickets as $ticket) {
			$cp_desc_val = CustomPropertyValues::getCustomPropertyValue($ticket->getId(), $cp_desc->getId());
			$cp_cs_val = CustomPropertyValues::getCustomPropertyValue($ticket->getId(), $cp_cs->getId());
			
			$info = array(
				'name' => $ticket->getObjectName(),
				'description' => $cp_desc_val instanceof CustomPropertyValue ? $cp_desc_val->getValue() : "",
				'date' => format_datetime($ticket->getCreatedOn()),
				'status' => $cp_cs_val instanceof CustomPropertyValue ? $cp_cs_val->getValue() : "",
			);
			$list[] = $info;
		}
	}
	
	return $list;
}