<?php
/**
 * Timesheet Approvals controller
 *
 * @version 1.0
 */
class TimesheetApprovalController extends ApplicationController {

	var $plugin_name = "timesheet_approval";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
	}
	
	
	private function change_timesheet_status_member($timeslots, $member, $action) {
		$modified_objects = array();

		// If action is 'process' check if all timeslots are non-billable, throm exception otherwise
		if($action == 'process'){
			$all_non_billable = $this->check_timeslots_non_billable($timeslots);
			if(!$all_non_billable){
				throw new Exception(lang('you can process only non-billable time entries'));
				return;
			}
		}
		$processed_member_id = config_option("time_bulk_action_process_member_id");

		foreach ($timeslots as $timeslot) {
			// to use when saving the application log
			$old_content_object = $timeslot->generateOldContentObjectData();
			
			// check if timeslot is classified in any timesheet status member, and mark it to remove
			$members_to_remove = array();
			$ts_members = $timeslot->getMembers();
			foreach ($ts_members as $m) {
				if ($m->getDimensionId() == $member->getDimensionId()) {
					
					// if user can't edit the timeslot depemding on the current classification, then this function throws an exception (don't check if changing from 'processed' member id)
					if (Plugins::instance()->isActivePlugin('workflow_permissions') && $m->getId() != $processed_member_id) {
						WorkflowPermissions::checkIfUserCanEdit(logged_user(), $m, $timeslot);
					}
					$members_to_remove[] = $m->getId();
				}
			}
			
			// if user can't classify the timeslot depemding on the next status member, then this function throws an exception
			if (Plugins::instance()->isActivePlugin('workflow_permissions') && $action != 'process') {
				WorkflowPermissions::checkIfUserCanChangeStatus(logged_user(), $member, $timeslot); 
			}
			
			// remove from previous timesheet status member (if any)
			if (count($members_to_remove) > 0) {
				ObjectMembers::removeObjectFromMembers($timeslot, logged_user(), $members_to_remove, $members_to_remove);
			}
			
			// classify in the new timesheet status member
			ObjectMembers::addObjectToMembers($timeslot->getId(), array($member));
			
			$modified_objects[] = $timeslot;
		}

		$null = null;
		Hook::fire('after_time_bulk_action', array('modified_objects' => $modified_objects, 'action' => $action, 'member' => $member), $null);
		
		return $modified_objects;
	}
	
	
	private function get_timeslots_from_parameters() {
		return get_timeslots_from_grid_parameters();
	}
	
	
	private function bulk_action_timeslots($member_id, $action) {
		ajx_current("emtpy");
		
		$timeslots = $this->get_timeslots_from_parameters();
		if (is_array($timeslots)) {
			
			$member = Members::getMemberById($member_id);
			if ($member instanceof Member) {
				
				// If action is 'invoice' include timeslots with 'approved' status only
				if($action == 'invoice'){
					$timeslots = $this->get_timeslots_with_approved_status($timeslots);
					
					// Create a warning if user didn't select any approved actual expense
					if(empty($timeslots)){
						flash_error(lang('you must select at least one approved time entry to perform this action'));
						return;
					}
				}

				$timeslot_ids = array();
				foreach ($timeslots as $t) $timeslot_ids[] = $t->getId();
				
				$prev_classifications = array();
				if (count($timeslot_ids) > 0) {
					$prev_classifications_rows = DB::executeAll("
						SELECT group_concat(om.member_id) as old_members, om.object_id
						FROM ".TABLE_PREFIX."object_members om 
						INNER JOIN ".TABLE_PREFIX."members m ON om.member_id=m.id 
						WHERE m.dimension_id=".$member->getDimensionId()." AND om.object_id IN (".implode(',', $timeslot_ids).")
						GROUP BY om.object_id
					");
					foreach ($prev_classifications_rows as $row) {
						if (array_var($row, 'old_members') != "") $prev_classifications[$row['object_id']] = $row['old_members'];
					}
				}
				
				try {
					DB::beginWork();
					
					$modified_objects = $this->change_timesheet_status_member($timeslots, $member, $action);
					
					// generate the invoice
					if ($action == 'invoice') {
						$project_ot = ObjectTypes::findByName('project');
						$client_ot = ObjectTypes::findByName('customer');
						$project_member = null;
						foreach (active_context() as $selection) {
							if ($selection instanceof Member && in_array($selection->getObjectTypeId(), array($project_ot->getId(), $client_ot->getId()))) {
								$project_member = $selection;
								break;
							}
						}
						// don't invoice if not in a project or client
						if (!$project_member instanceof Member) {
							throw new Exception(lang('you need to be in a client or a project in order to perform this action'));
						}
						
						$template_id = IncomeInvoiceTemplates::getMemberInvoiceTemplateId($project_member);
						$invoice_template = IncomeInvoiceTemplates::instance()->findById($template_id);
						
						$generated_invoices = null;
						$hook_params = array('member'=>$project_member, 'invoice_template'=>$invoice_template, 'timeslots'=>$modified_objects, 'payment_receipts'=>array(), 'param_billing_period_from'=>'', 'param_billing_period_to'=>'');
						Hook::fire('override_invoice_generation', $hook_params, $generated_invoices);
						
						if (!$generated_invoices) {
							$invoice = IncomeInvoiceTemplates::generateInvoiceFromTemplateAndObjects($project_member, $invoice_template, $modified_objects, array(), '', '');
							$generated_invoices = array($invoice);
						}
						foreach ($generated_invoices as $invoice) {
							if ($invoice instanceof IncomeInvoice) {
								flash_success(lang('success add invoice', $invoice->getObjectName()));
							}
						}
						evt_add('reload tab panel', 'income-panel');
					}
					
					DB::commit();
					//ajx_current("reload");
					evt_add("reload current panel");
					
					foreach ($modified_objects as $object) {
						$log_action = array_var($prev_classifications, $object->getId()) ? ApplicationLogs::ACTION_MOVE : ApplicationLogs::ACTION_COPY;
						$log_data = (array_var($prev_classifications, $object->getId()) ? "from:" . array_var($prev_classifications, $object->getId()) . ";" : "") . "to:" . $member->getId();
						ApplicationLogs::createLog($object, $log_action, false, true, true, $log_data);
					}
					
				} catch (Exception $e) {
					DB::rollback();
					flash_error($e->getMessage());
					ajx_current("empty");
				}
				
			}
		} else {
			flash_error(lang('you must select at least one timeslot or project to perform this action'));
		}
	}

	function get_timeslots_with_approved_status($all_timeslots){
		$timeslots = array();
		$approved_member_id = config_option("time_bulk_action_approved_member_id");
		$aproval_status_dim_id = Dimensions::findByCode('status_timesheet')->getId();
		foreach ($all_timeslots as $t){
			$ts_members = $t->getMembers();
			foreach ($ts_members as $m) {
				if ($m->getDimensionId() == $aproval_status_dim_id && $m->getId() == $approved_member_id) {
					$timeslots[] = $t;
				}
			}
		} 

		return $timeslots;
	}

	private function check_timeslots_non_billable($timeslots){
		foreach ($timeslots as $t){
			if($t->getColumnValue('invoicing_status') != 'non_billable') return false;
		}
		return true;
	}
	
	
	function put_pending_timeslots() {
		$this->bulk_action_timeslots(config_option("time_bulk_action_pending_member_id"), 'pending');
	}
	
	
	function lock_timeslots() {
		$this->bulk_action_timeslots(config_option("time_bulk_action_locked_member_id"), 'lock');
	}
	
	
	function approve_timeslots() {
		$this->bulk_action_timeslots(config_option("time_bulk_action_approved_member_id"), 'approve');
	}
	
	
	function invoice_timeslots() {
		$this->bulk_action_timeslots(config_option("time_bulk_action_invoiced_member_id"), 'invoice');
	}

	function process_timeslots() {
		$this->bulk_action_timeslots(config_option("time_bulk_action_processed_member_id"), 'process');
	}
	
	
	function put_under_review_timeslots() {
		$this->bulk_action_timeslots(config_option("time_bulk_action_review_member_id"), 'review');
	}
		
}

