<?php
Hook::register('timesheet_approval');

function timesheet_approval_timeslot_list_additional_actions($params, &$actions) {
	$bulk_actions = array();
	$timeslots_ot = ObjectTypes::findByName('timeslot');
	
	$member = Members::getMemberById(config_option("time_bulk_action_pending_member_id"));
	if ($member instanceof Member && can_write(logged_user(), array($member), $timeslots_ot->getId())) {
		$button_text = clean(str_replace("'", "", config_option('time_bulk_action_pending_btn_text')));
		$bulk_actions[] = array(
			'url' => '#',
			'onclick' => "og.timesheet_approval.bulk_actions.pending('".array_var($params, 'panel_id')."', '$button_text');",
			'cls' => 'ico-unlocked',
			'text' => $button_text,
		);
	}
	
	$member = Members::getMemberById(config_option("time_bulk_action_locked_member_id"));
	if ($member instanceof Member && can_write(logged_user(), array($member), $timeslots_ot->getId())) {
		$button_text = clean(str_replace("'", "", config_option('time_bulk_action_lock_btn_text')));
		$bulk_actions[] = array(
			'url' => '#',
			'onclick' => "og.timesheet_approval.bulk_actions.lock('".array_var($params, 'panel_id')."', '$button_text');",
			'cls' => 'ico-locked',
			'text' => $button_text,
		);
	}
	
	$member = Members::getMemberById(config_option("time_bulk_action_approved_member_id"));
	if ($member instanceof Member && can_write(logged_user(), array($member), $timeslots_ot->getId())) {
		$button_text = clean(str_replace("'", "", config_option('time_bulk_action_approve_btn_text')));
		$bulk_actions[] = array(
			'url' => '#',
	 		'onclick' => "og.timesheet_approval.bulk_actions.approve('".array_var($params, 'panel_id')."', '$button_text');",
	 		'cls' => 'ico-complete-grey',
			'text' => $button_text,
		);
	}
	
	$member = Members::getMemberById(config_option("time_bulk_action_invoiced_member_id"));
	if ($member instanceof Member && can_write(logged_user(), array($member), $timeslots_ot->getId())) {
		$button_text = clean(str_replace("'", "", config_option('time_bulk_action_invoice_btn_text')));
		$bulk_actions[] = array(
			'url' => '#',
	 		'onclick' => "og.timesheet_approval.bulk_actions.invoice('".array_var($params, 'panel_id')."', '$button_text');",
			'cls' => 'ico-invoice',
			'text' => $button_text,
		);
	}

	$processed_member = Members::getMemberById(config_option("time_bulk_action_processed_member_id"));
	if ($processed_member instanceof Member && can_write(logged_user(), array($processed_member), $timeslots_ot->getId())) {
		$button_text = clean(str_replace("'", "", config_option('time_bulk_action_processed_btn_text')));
		$bulk_actions[] = array(
			'url' => '#',
	 		'onclick' => "og.timesheet_approval.bulk_actions.process('".array_var($params, 'panel_id')."', '$button_text');",
			'cls' => 'ico-milestone',
			'text' => $button_text,
		);
	}
	
	$member = Members::getMemberById(config_option("time_bulk_action_review_member_id"));
	if ($member instanceof Member && can_write(logged_user(), array($member), $timeslots_ot->getId())) {
		$button_text = clean(str_replace("'", "", config_option('time_bulk_action_review_btn_text')));
		$bulk_actions[] = array(
			'url' => '#',
	 		'onclick' => "og.timesheet_approval.bulk_actions.review('".array_var($params, 'panel_id')."', '$button_text');",
			'cls' => 'ico-undo',
			'text' => $button_text,
		);
	}
	
	if (count($bulk_actions) > 2) {
		// make a dropdown menu with the actions
		$action = array(
			'type' => 'menu',
			'text' => lang('bulk actions'),
			'items' => $bulk_actions,
		);
		$actions[] = $action;
	
	} else if (count($bulk_actions) > 0) {
		// add the actions as single buttons
		$actions = array_merge($actions, $bulk_actions);
	
	}
	
	
}

function timesheet_approval_config_option_changed($params, &$ignored) {
	
	$config_option = array_var($params, 'option'); /* @var $config_option ConfigOption */
	$old_value = array_var($params, 'old_value');
	$new_value = array_var($params, 'new_value');
	
	if ($config_option->getCategoryName() == 'timesheets_config' && str_ends_with($config_option->getName(), "_member_id")) {
		
		$new_value = str_replace(array("[","]"), "", $new_value);
		$old_value = str_replace(array("[","]"), "", $old_value);
		
		if ($old_value != $new_value) {
			
			if ($old_value != '') {
				$sql = "UPDATE ".TABLE_PREFIX."workflow_permissions SET member_id='$new_value' WHERE member_id='$old_value'";
				DB::execute($sql);
				
			} else {
				$key = str_replace("_member_id", "", str_replace("time_bulk_action_", "", $config_option->getName()));
				if ($key == 'approved') $key = 'approve';
				else if ($key == 'invoiced') $key = 'invoice';
				else if ($key == 'locked') $key = 'lock';
				
				$sql = "UPDATE ".TABLE_PREFIX."workflow_permissions SET member_id='$new_value' WHERE name like '%$key%'";
				DB::execute($sql);
			}
		}
		
	}
}


function timesheet_approval_after_user_add($object, $ignored) {
	if (!class_exists('WorkflowPermissions')) return; // the installer can't include this class so the first user can't execute this code
	
	// set workflow permissions to every administrator over the new user
	$administrators = Contacts::instance()->findAll(array("conditions" => "user_type IN (1,2)"));
	
	$w_permissions = WorkflowPermissions::instance()->findAll();
	
	foreach ($administrators as $admin) {
		/* @var $admin Contact */
		$permission_group_id = $admin->getPermissionGroupId();
		foreach ($w_permissions as $wp) {
			$wp_val = WorkflowPermissionValues::getValueForPermissionGroup($wp->getId(), $permission_group_id);
			/* @var $wp_val WorkflowPermissionValue */
			if ($wp_val && $wp_val->getBoolValue() == 1) {
				$wp_pg = WorkflowPermissionValuePgs::instance()->findOne(array(
						"conditions" => array("workflow_permission_value_id=? AND affected_permission_group_id=?", $wp_val->getId(), $object->getPermissionGroupId())
				));
				if (!$wp_pg) {
					$wp_pg = new WorkflowPermissionValuePg();
					$wp_pg->setWorkflowPermissionValueId($wp_val->getId());
					$wp_pg->setAffectedPermissionGroupId($object->getPermissionGroupId());
					$wp_pg->save();
				}
			}
		}
	}
}


function timesheet_approval_before_add_to_members($params, &$continue) {
	
	$timeslot = array_var($params, 'object');
	$members = array_var($params, 'members');
	
	if ($timeslot instanceof Timeslot) {
		Env::useHelper('functions', 'timesheet_approval');
		check_timeslot_permissions_before_classifying($timeslot, $members);
	}
}

function timesheet_approval_before_classify_additional_verifications($params, &$ignored) {
	$timeslot = array_var($params, 'object');
	$member_ids = array_var($params, 'member_ids');
	
	if ($timeslot instanceof Timeslot && count($member_ids) > 0) {
		Env::useHelper('functions', 'timesheet_approval');
		$members = Members::instance()->findAll(array("conditions" => "id IN (".implode(',', $member_ids).")"));
		check_timeslot_permissions_before_classifying($timeslot, $members);
	}
}




function timesheet_approval_override_action_view($controller, &$results) {
	if ($controller->getControllerName() == 'config' && $controller->getAction() == 'update_category') {
		$cat = ConfigCategories::instance()->findById(get_id());
		if ($cat instanceof ConfigCategory && $cat->getName() == 'timesheets_config') {
			$controller->setTemplate(get_template_path('config_category', 'timesheet_approval', 'timesheet_approval'));
		}
	}
}


