<?php
chdir(dirname(__FILE__)."/../../");
define("CONSOLE_MODE", true);
define('PUBLIC_FOLDER', 'public');
include "init.php";

Env::useHelper('page');
Env::useHelper('permissions');

$field_rows = DB::executeAll("SELECT `name` FROM ".TABLE_PREFIX."user_registration_fields WHERE required=1 ORDER BY `order`");
$required_fields = array_flat($field_rows);

$guest = PermissionGroups::instance()->findOne(array("conditions" => "name='Guest'"));
// form data
$user_data = array_var($_POST, 'user');

// save parameters in session
$_SESSION['new_user_registration_data'] = $user_data;
file_put_contents(ROOT.'/cache/lala.txt', print_r($user_data,1), FILE_APPEND);

// fixed data to create user
$user_data['password_generator'] = 'link';
$user_data['send_email_notification'] = true;
$user_data['type'] = $guest->getId();
$user_data['user_type'] = $guest->getId();

// login administrator
$logged_user = Contacts::instance()->findOne(array('conditions' => "user_type IN (SELECT id FROM ".TABLE_PREFIX."permission_groups WHERE name IN ('Super Administrator', 'Administrator'))"));
CompanyWebsite::instance()->logUserIn($logged_user);

try {
	// check for unique email
	if (is_valid_email(array_var($user_data, 'email'))) {
		$user_tmp = Contacts::getByEmail(array_var($user_data, 'email'), null, true);
		if($user_tmp instanceof Contact && $user_tmp->isUser()) {
			throw new Exception(lang('user with provided email already exists'));
		}
	}
	
	// check for required fields
	$req_fields_errors = array();
	foreach ($required_fields as $req_field) {
		if (!isset($user_data[$req_field])) {
			file_put_contents(ROOT.'/cache/lala.txt', "$req_field not set\n", FILE_APPEND);
			$req_fields_errors[] = ' - ' . lang('field '.$req_field) . ' ' . lang('is required');
		}
		if ($user_data[$req_field] == "") {
			file_put_contents(ROOT.'/cache/lala.txt', "$req_field =''\n", FILE_APPEND);
			$req_fields_errors[] = ' - ' . lang('field '.$req_field) . ' ' . lang('is required');
		}
	}
	if (count($req_fields_errors) > 0) {
		throw new Exception('The following fields are required: <br />'.implode('<br />', $req_fields_errors));
	}
	
	// check email = email_confirmation
	if (array_var($user_data, 'email') != array_var($user_data, 'email_confirmation')) {
		throw new Exception(' - Email and email confirmation does not match.');
	}
	
	// search the company, if no company exists => create a new one
	$company = null;
	if ($user_data['company_name'] != '') {
		$company = Contacts::instance()->findOne(array('conditions' => array("name = ? AND is_company=1", $user_data['company_name'])));
		if (!$company instanceof Contact) {
			$company = new Contact();
			$company->setIsCompany(1);
			$company->setFirstName($user_data['company_name']);
			$company->getObject()->setName($user_data['company_name']);
			$company->save();
			
			$comp_member_dim_id = Dimensions::findByCode("feng_persons")->getId();
			$company_ot = ObjectTypes::instance()->findOne(array("conditions" => "`name` = 'company'"));
			
			$comp_member = new Member();
			$comp_member->setObjectTypeId($company_ot->getId());
			$comp_member->setName($user_data['company_name']);
			$comp_member->setDimensionId($comp_member_dim_id);
			$comp_member->setObjectId($company->getId());
			$comp_member->save();
			
			$new_company = true;
		}
		$user_data['company_id'] = $company->getId();
	}
	
	// create contact
	$user = new Contact();
	$user->setFromAttributes($user_data);
	$user->setObjectName(trim($user->getFirstName().' '.$user->getSurname()));
	$user->save();

	$user_data['contact_id'] = $user->getId();
	$user_data['username'] = strtolower(trim($user->getFirstName() . $user->getSurname()));
	
	// add contact data
	if($user_data['w_address'] != "" || $user_data['w_city'] != "" || $user_data['w_state'] != "" || $user_data['w_country'] != "" || $user_data['w_zipcode'] != ""){
		$user->addAddress($user_data['w_address'], $user_data['w_city'], $user_data['w_state'], $user_data['w_country'], $user_data['w_zipcode'], 'work');
	}
	if($user_data['w_phone_number'] != "") $user->addPhone($user_data['w_phone_number'], 'work', true);
	if($user_data['h_mobile_number'] != "") $user->addPhone($user_data['h_mobile_number'], 'mobile');
	if($user_data['w_assistant_number'] != "") $user->addPhone($user_data['w_assistant_number'], 'assistant', null, $user_data['w_assistant_name']);
	if($user_data['w_fax_number'] != "") $user->addPhone($user_data['w_fax_number'], 'fax', true);
	if($user_data['w_web_page'] != "") $user->addWebpage($user_data['w_web_page'], 'work');
	
	// assign user to default group if configured
	$group_pg_id = config_option('default_guest_user_group');
	if ($group_pg_id > 0) {
		$cpg = ContactPermissionGroups::instance()->findOne(array('conditions' => "permission_group_id = $group_pg_id AND contact_id = ".$user->getId()));
		if (!$cpg instanceof ContactPermissionGroup) {
			$cpg = new ContactPermissionGroup();
			$cpg->setPermissionGroupId($group_pg_id);
			$cpg->setContactId($user->getId());
			$cpg->save();
		}
	}
	
	// create user
	$user = create_user($user_data, '');

	// dont let user see other users assinged tasks
	$sys_perm = SystemPermissions::instance()->findOne(array('conditions' => 'permission_group_id = '.$user->getPermissionGroupId()));
	if ($sys_perm instanceof SystemPermission) {
		$sys_perm->setColumnValue('can_see_assigned_to_other_tasks', false);
		$sys_perm->save();
	}
	
	// associate user member (in feng_persons dimension) with members in which user has permissions
	$user_member = Members::findOneByObjectId($user->getId(), Dimensions::findByCode("feng_persons")->getId());
	if ($user_member instanceof Member) {
		$user_pgs = $user->getPermissionGroupIds();
		$members = Members::instance()->findAll(array('conditions' => 'id <> '.$user_member->getId().' AND id in (select cmp.member_id from '.TABLE_PREFIX.'contact_member_permissions cmp where cmp.permission_group_id in ('.implode(',',$user_pgs).'))'));
		core_dim_create_member_associations($user, $user_member, $members, false);
	}
	// give permissions to company member
	if ($new_company && $company instanceof Contact) {
		$members = Members::instance()->findAll(array('conditions' => 'id <> '.$user_member->getId().' AND id in (select cmp.member_id from '.TABLE_PREFIX.'contact_member_permissions cmp where cmp.permission_group_id in ('.implode(',',$user_pgs).'))'));
		DB::execute("INSERT INTO ".TABLE_PREFIX."contact_member_permissions (permission_group_id, member_id) VALUES (".$user->getPermissionGroupId().",".$comp_member->getId().")");
		core_dim_create_member_associations($user, $comp_member, $members, false);
	}

	$ws_dim = Dimensions::findByCode('workspaces');
	set_user_config_option('root_dimensions', $ws_dim->getId(), $user->getId());
	set_user_config_option('can_modify_navigation_panel', false, $user->getId());
	
	// Send notification
	send_notification($user_data, $user->getId());
	
	$_SESSION['new_user_registration_success_msg'] = lang('user successfully registered');
	redirect_to('success.php');
	
} catch (Exception $e) {
	$_SESSION['new_user_registration_error_msg'] = lang('an error occurred while performing the registration', str_replace("\n","<br />",$e->getMessage()));
	redirect_to('register.php');
}

// logout administrator
CompanyWebsite::instance()->logUserOut();
