<?php
/**
 * Returns the Member object result of following the $path given.
 * @param string $path - The webdav filesystem path to the member (folder)
 * @param Contact $user - the logged user
 * @return Member
 */
function get_member_from_path($path, Contact $user) {
	$dimension_id = config_option('webdav_dimension');
	
	// explode parent path
	$parent_names = array_filter(explode (WEBDAV_PATH_SEPARATOR, $path));
	
	$member = null;
	$parent_id = 0;
	
	foreach ($parent_names as $pname) {
		// get parent member
		$member = Members::instance()->findOne(array('conditions' => array("name=? AND dimension_id=? AND parent_member_id=?", $pname, $dimension_id, $parent_id)));
		
		// set parent id for next iteration
		if ($member instanceof Member) {
			$parent_id = $member->getId();
		} else {
			break;
		}
	}
	
	$pg_ids_csv = implode(',', $user->getPermissionGroupIds());
	
	// return the last member found
	if ($member && $member->canBeReadByContact($pg_ids_csv, $user)) {
		return $member;
	} else {
		return null;
	}
}

/**
 * Creates a member with name=$name under the member $current_member, if $current_member is null the new member is created in root.
 * @param string $name - Member name
 * @param Member $current_member - parent member
 */
function create_webdav_member($name, $current_member) {
	$dimension_id = config_option('webdav_dimension');
	
	// get the first member type available to be child of $current member
	if ($current_member instanceof Member) {
		$object_type_ids = $current_member->getAllChildrenObjectTypeIds();
		foreach ($object_type_ids as $ot_id) {
			$member_type_id = $ot_id;
			break;
		}
	} else {
		// if current member is null then get the first member type that can be created in the root of this dimension
		$dim_object_types = DimensionObjectTypes::instance()->findAll(array("conditions"=>"enabled=1 AND dimension_id = $dimension_id AND is_root = 1 
				AND object_type_id<>(SELECT id from ".TABLE_PREFIX."object_types WHERE name='company')"));
		foreach ($dim_object_types as $dot) {
			$member_type_id = $dot->getObjectTypeId();
			break;
		}
	}
	
	$member_controller = new MemberController();
	// build data to create the member
	$member_data = array(
			'parent_member_id' => ($current_member instanceof Member ? $current_member->getId() : 0),
			'name' => $name,
			'dimension_id' => $dimension_id,
			'object_type_id' => $member_type_id,
	);
	$_POST['member'] = $member_data;
	// create the member
	$member_controller->add();
	
	return true;
}

/**
 * Returns the member list under $parent_id
 * @param integer $parent_id
 * @param Contact $user
 * @return array of Member
 */
function get_member_list($parent_id, $user) {
	$dimension_id = config_option('webdav_dimension');
	
	// listing parameters
	$parameters = array(
		'limit' => 1000,
		'dim_id' => $dimension_id,
		'extra_conditions' => " AND mem.parent_member_id='$parent_id' ",
	);
	// execute listing
	$member_controller = new MemberController();
	$listing_result = $member_controller->listing($parameters);
	
	// foreach member data build the member object
	$members = array();
	$members_info = array_var($listing_result, 'members');
	foreach ($members_info as $mem_info) {
		$mem = new Member();
		$mem->setFromAttributes($mem_info);
		$mem->setId($mem_info['member_id']);
		$members[] = $mem;
	}
	
	return $members;
}

/**
 * Get the last update date of the member from application logs
 * @param Member $member
 * @return integer - timestamp of the date
 */
function get_member_last_update_time($member) {
	if (!$member instanceof Member) return 0;
	
	$app_log = ApplicationLogs::instance()->findOne(array('conditions' => "member_id=".$member->getId(), 'order' => "created_on DESC"));
	if ($app_log instanceof ApplicationLog) {
		return $app_log->getCreatedOn()->getTimestamp();
	}
	return 0;
}


