<?php 

class WebDAVFile extends Sabre\DAV\FS\File {

 private $myPath;
 private $myIde;
 
 private $size;
 private $mime_type;
 private $last_modified;
 

  function __construct($myPath, $file_id, $attributes=array()) {
  	
  	if (!logged_user()) {
  		throw new Sabre\DAV\Exception\NotAuthenticated("No user logged in");
  	}
  	
  	$myFilename = basename_safe($myPath);
  	$myDir = dirname($myPath);
  	
  	$this->myPath = $myPath;
  	$this->myIde = $file_id;
  	
  	$this->size = $attributes['size'];
  	$this->mime_type = $attributes['mime_type'];
  	$this->last_modified = $attributes['last_modified'];
  }
  
  /**
   * Returns the ProjectFile object associated to this WebDAVFile
   * @return ProjectFile
   */
  private function getFile(){
  	
  	$file = ProjectFiles::instance()->findById($this->myIde);
  	if (!$file instanceof ProjectFile) {
  		$file = get_file_from_path($this->myPath, logged_user());
  		$this->myIde = $file->getId();
  	}
  	return $file;
  	
  }
  

  /**
   * get the file name
   * {@inheritDoc}
   * @see \Sabre\DAV\FS\Node::getName()
   * @return string
   */
  function getName() {
  	return basename_safe($this->myPath);
  }

  /**
   * Get the file contents
   * {@inheritDoc}
   * @see \Sabre\DAV\FS\File::get()
   * @return string
   */
  function get() {
	$file = $this->getFile();
	if ($file instanceof ProjectFile) {
		return $file->getFileContent();	
	}
	return '';
  }

  /**
   * Return file size in bytes
   * {@inheritDoc}
   * @see \Sabre\DAV\FS\File::getSize()
   * @return integer
   */
  function getSize() {
  	return $this->size;
  }

  /**
   * Override delete funcion to prevent deletion.
   * {@inheritDoc}
   * @see \Sabre\DAV\FS\File::instance()->delete()
   */
  function delete(){
  	log_wd("Trying to delete webdav file '".$this->myPath."' - user: ".(logged_user() instanceof Contact ? logged_user()->getId() : ''));
  	die();
  }
  
  /**
   * {@inheritDoc}
   * @see \Sabre\DAV\FS\Node::getLastModified()
   */
  function getLastModified(){
  	return $this->last_modified;
  }  
  
  /**
   * Changes the file name
   * {@inheritDoc}
   * @see \Sabre\DAV\FS\Node::setName()
   */
  function setName($newName){
  	if($newName!= ""){
	  	$file = $this->getFile();
  		$logged_user = logged_user();
		if ($file instanceof ProjectFile && $file->canEdit($logged_user) &&
				(!$file->isCheckedOut() || $file->getCheckedOutById() == logged_user()->getId())) {
			
			$file->setFilename($newName);
			$file->save();
		}
  	}
  	return;
  }
  
  /**
   * Get the file type
   * {@inheritDoc}
   * @see \Sabre\DAV\FS\File::getContentType()
   * @return string
   */
  function getContentType(){
  	return $this->mime_type;
  }
  
}

