<?php
/**
 * Weill Cornell Reports controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class WeillCornellReportsController extends ApplicationController {

	var $plugin_name = "weill_cornell_reports";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', 'expenses');
		Env::useHelper('grouping');
		Env::useHelper('format');
	}
	
	function sponsors_report_parameters() {
		
		$report_data = array_var($_SESSION, 'sponsors_report_data', array());
		
		$report_data['column_list'] = $this->sponsors_report_report_columns($report_data);
		$report_data['ordered_columns'] = $this->sponsors_report_report_columns_ordered($report_data);
		
		$_SESSION['sponsors_report_data'] = $report_data;
		
	}
	
	private function sponsors_report_report_columns($report_data) {
		$columns = array_var($report_data, 'show_columns');
		$col_list = array();
		
		$col_list[] = array('id' => 'name', 'text' => lang('name'), 'selected' => array_var($columns, 'name', 1) > 0);
		$col_list[] = array('id' => 'related_to', 'text' => lang('related to'), 'selected' => array_var($columns, 'related_to', 2) > 0);
		$col_list[] = array('id' => 'payment_type', 'text' => lang('payment type'), 'selected' => array_var($columns, 'payment_type', 3) > 0);
		$col_list[] = array('id' => 'position', 'text' => lang('position'), 'selected' => array_var($columns, 'position', 3) > 0);
		$col_list[] = array('id' => 'destination', 'text' => lang('destination'), 'selected' => array_var($columns, 'destination', 4) > 0);
		
		$expense_ot = ObjectTypes::findByName('expense');
		$proj_year_cp = CustomProperties::instance()->findOne(array('conditions' => "name='Project Year' AND object_type_id=".$expense_ot->getId()));
		
		$col_list[] = array('id' => 'cp_'.$proj_year_cp->getId(), 'text' => $proj_year_cp->getName(), 'selected' => array_var($columns, 'cp_'.$proj_year_cp->getId(), 4) > 0);

		$i = count($col_list);
		
		$def_payment_status = config_option('default_payment_status');
		$statuses = ExpensesStatuses::instance()->getAllStatusInfo();
		foreach ($statuses as $st) {
			if ($st['only_for_expenses'] || $st['id'] != $def_payment_status) continue;
			$i++;
			$col_list[] = array('id' => $st['code'], 'text' => lang($st['code']), 'selected' => array_var($columns, $st['code'], $i) > 0);
		}
		
		return $col_list;
	}
	
	private function sponsors_report_report_columns_ordered($report_data) {
		$ordered = array();
		foreach ($report_data['show_columns'] as $key => $order) {
			if ($order > 0) {
				$name = lang($key);
				foreach ($report_data['column_list'] as $col) {
					if ($col['id'] == $key) {
						$name = $col['text'];
						break;
					}
				}
				$ordered[$order] = array('id' => $key, 'name' => $name);
			} 
		}
		ksort($ordered);
		
		return $ordered;
	}
	
	
	function sponsors_report() {
		
		$csv_export = array_var($_POST, 'csv_export');
		
		if ($csv_export) {
			$report_data = $_SESSION['sponsors_report_data'];
		} else {
			$report_data = array_var($_POST, 'report', array());
			$report_data['column_list'] = $this->sponsors_report_report_columns($report_data);
			$report_data['ordered_columns'] = $this->sponsors_report_report_columns_ordered($report_data);
			$_SESSION['sponsors_report_data'] = $report_data;
		}
		
		$extra_conditions = '';
		
		$payment_type = array_var($report_data, 'payment_type');
		if (is_numeric($payment_type) && $payment_type > 0) {
			$extra_conditions .= " AND EXISTS (SELECT pp.expense_id FROM ".TABLE_PREFIX."payments pp WHERE pp.expense_id=e.object_id AND pp.type_id=$payment_type) ";
		}
		
		$cp_filters = array_var($report_data, 'cp_filters');
		foreach ($cp_filters as $cp_id => $cp_val) {
			if (trim($cp_val) && is_numeric($cp_id) && $cp_id > 0) {
				$cp_value = DB::escape(trim($cp_val));
				$extra_conditions .= " AND EXISTS (SELECT cppvv.object_id FROM ".TABLE_PREFIX."custom_property_values cppvv 
					WHERE cppvv.object_id=e.object_id AND cppvv.custom_property_id=$cp_id AND cppvv.value=$cp_value) ";
			}
		}
		
		$now = DateTimeValueLib::now();
		$now->advance(logged_user()->getUserTimezoneValue(), true);
		switch (array_var($report_data, 'date_type')){
			case -1: //None
				$st = null;
				$et = null;
				break;
			case 1: //Today
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),$now->getDay(),$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),$now->getDay(),$now->getYear());break;
			case 2: //This week
				$monday = $now->getMondayOfWeek();
				$nextMonday = $now->getMondayOfWeek()->add('w',1)->add('d',-1);
				$st = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
				$et = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
			case 3: //Next week
				$monday = $now->getMondayOfWeek()->add('w',1);
				$nextMonday = $now->getMondayOfWeek()->add('d',-1);
				$st = DateTimeValueLib::make(0,0,0,$monday->getMonth(),$monday->getDay(),$monday->getYear());
				$et = DateTimeValueLib::make(23,59,59,$nextMonday->getMonth(),$nextMonday->getDay(),$nextMonday->getYear());break;
			case 4: //This month
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
			case 5: //Next month
				$now->add('M',1);
				$st = DateTimeValueLib::make(0,0,0,$now->getMonth(),1,$now->getYear());
				$et = DateTimeValueLib::make(23,59,59,$now->getMonth(),1,$now->getYear())->add('M',1)->add('d',-1);break;
			case 6: //Date interval
				$st = getDateValue(array_var($report_data, 'date_start_value'));
				$st = $st->beginningOfDay();
				
				$et = getDateValue(array_var($report_data, 'date_end_value'));
				$et = $et->endOfDay();
				break;
		}
		if ($et != null && $st != null) {
			tpl_assign('start_time', $st);
			tpl_assign('end_time', $et);
			$extra_conditions .= " AND EXISTS (SELECT p.expense_id FROM ".TABLE_PREFIX."payments p WHERE p.expense_id=object_id AND p.date <= '".$et->toMySQL()."' AND p.date >= '".$st->toMySQL()."')";
		}
		
		$expenses = Expenses::instance()->listing(array(
			'extra_conditions' => $extra_conditions,
			'count_results' => false,
		))->objects;
		
		
		$group_by = array();
		for ($i = 1; $i <= 3; $i++){
			$gb = array_var($report_data, 'group_by_' . $i);
			if ($gb != '0' && !in_array($gb, $group_by)) $group_by[] = $gb;
		}
		$gb_criterias = array();
		foreach ($group_by as $text) {
			if (str_starts_with($text, 'dim_')) $gb_criterias[] = array('type' => 'dimension', 'value' => str_replace_first('dim_', '', $text));
			else if (str_starts_with($text, 'cp_')) $gb_criterias[] = array('type' => 'custom_prop', 'value' => str_replace_first('cp_', '', $text));
		}
		$grouped_expenses = groupObjects($gb_criterias, $expenses);
		
		$expense_ids = array();
		foreach ($expenses as $expense) $expense_ids[] = $expense->getId();
		if (count($expense_ids) == 0) $expense_ids[] = 0;
		
		$statuses = ExpensesStatuses::instance()->getAllStatusInfo();
		
		$payments = array();
		$totals = array();
		
		if (array_var($report_data, 'aggregate_payment_types_of_the_same_key')) {
			
			$processed_payments = array();
			$tmp_payments = array();
			foreach ($grouped_expenses['grouped_objects'] as $gkey => $g_expenses) {

				$tmp_payments[$gkey] = array();
				
				foreach ($g_expenses as $exp) {
					$p_objs = $exp->getPaymentsFromHistory();
					
					foreach ($statuses as $st) {
						if ($st['only_for_expenses']) continue;
							
						if (!isset($payments[$exp->getId()][$st['code']])) $payments[$exp->getId()][$st['code']] = array();
						if (!isset($totals[$exp->getId()])) $totals[$exp->getId()] = array();
						if (!isset($totals[$exp->getId()][$st['code']])) $totals[$exp->getId()][$st['code']] = 0;
					}
				
					foreach ($p_objs as $p) {
						$st = ExpensesStatuses::instance()->getStatus($p->getStatusId());
						$p_key = $p->getExpenseId()."_".$p->getPaymentNum();
						
						if (in_array($p_key, $processed_payments) || is_numeric($payment_type) && $payment_type > 0 && $p->getTypeId() != $payment_type) {
							continue;
						}
						$totals[$p->getExpenseId()][$st->getCode()] += $p->getAmount();
						$tmp_payments[$gkey][] = $p;
						
						$processed_payments[] = $p_key;
					}
				}
			}
			
			foreach ($tmp_payments as $key => &$payment_objs) {
				$aggregated = $this->aggregate_payment_types_of_the_same_key($payment_objs);
				$tmp_payments[$key] = $aggregated;
			}
			
			$payments = $tmp_payments;
			
		} else {
		
			foreach ($expenses as $exp) {
				$p_objs = $exp->getPaymentsFromHistory();
				$payments[$exp->getId()] = array();
				
				foreach ($statuses as $st) {
					if ($st['only_for_expenses']) continue;
					
					if (!isset($payments[$exp->getId()][$st['code']])) $payments[$exp->getId()][$st['code']] = array();
					if (!isset($totals[$exp->getId()])) $totals[$exp->getId()] = array();
					if (!isset($totals[$exp->getId()][$st['code']])) $totals[$exp->getId()][$st['code']] = 0;
				}
				
				foreach ($p_objs as $p) {
					if (is_numeric($payment_type) && $payment_type > 0 && $p->getTypeId() != $payment_type) {
						continue;
					}
					$st = ExpensesStatuses::instance()->getStatus($p->getStatusId());
					$payments[$p->getExpenseId()][$st->getCode()][] = $p;
					$totals[$p->getExpenseId()][$st->getCode()] += $p->getAmount();
				}
			}
			
		}

		tpl_assign('grouped_expenses', $grouped_expenses);
		tpl_assign('payments', $payments);
		tpl_assign('totals', $totals);
		tpl_assign('status_info', $statuses);
		
		tpl_assign('post', $report_data);
		tpl_assign('title', lang('project expenses by year report'));
		tpl_assign('allow_export', false);
		
		if ($csv_export) {
			$this->setLayout('empty');
			//ajx_current('empty');
			
			$_SESSION['expense_report_export_tmp_filename'] = gen_id().".csv";
			$_SESSION['expense_report_export_filename'] = lang('payments report').".csv";
			
			tpl_assign('tmp_filename', ROOT."/tmp/".$_SESSION['expense_report_export_tmp_filename']);
			$this->setTemplate('sponsors_report_csv');
			
		} else {
			tpl_assign('template_name', 'sponsors_report');
			$this->setTemplate('report_wrapper');
		}
	}
	
	
	
	private function aggregate_payment_types_of_the_same_key($payments) {
		
		$pos_attributes = array();
		$pos_attributes_tmp = PaymentTypeAttributes::instance()->findAll(array('conditions' => "name='Position'"));
		foreach ($pos_attributes_tmp as $pos_att) {
			$pos_attributes[$pos_att->getPaymentTypeId()] = $pos_att;
		}
		$dest_attributes = array();
		$dest_attributes_tmp = PaymentTypeAttributes::instance()->findAll(array('conditions' => "name='Destination'"));
		foreach ($dest_attributes_tmp as $dest_att) {
			$dest_attributes[$dest_att->getPaymentTypeId()] = $dest_att;
		}
		
		$aggregated_payments = array();
		
		$status = ExpensesStatuses::instance()->findById(config_option('default_payment_status'));
		
		foreach ($payments as $p) {
			/* @var $p Payment */
			$pos_attr_val = "";
			$pos_attr = array_var($pos_attributes, $p->getTypeId());
			if ($pos_attr instanceof PaymentTypeAttribute) {
				$pos_value_tmp = PaymentAttributeValues::instance()->findOne(array(
					'conditions' => "expense_id=".$p->getExpenseId()." AND attribute_id=".$pos_attr->getAttributeId()." AND payment_num=".$p->getPaymentNum()." AND type_id=".$p->getTypeId()
				));
				if ($pos_value_tmp instanceof PaymentAttributeValue) {
					$pos_attr_val = trim($pos_value_tmp->getValue());
				}
				
			}
			
			$dest_attr_val = "";
			$dest_attr = array_var($dest_attributes, $p->getTypeId());
			if ($dest_attr instanceof PaymentTypeAttribute) {
				$dest_value_tmp = PaymentAttributeValues::instance()->findOne(array(
						'conditions' => "expense_id=".$p->getExpenseId()." AND attribute_id=".$dest_attr->getAttributeId()." AND payment_num=".$p->getPaymentNum()." AND type_id=".$p->getTypeId()
				));
				if ($dest_value_tmp instanceof PaymentAttributeValue) {
					$dest_attr_val = trim($dest_value_tmp->getValue());
				}
			}
			
			$key = $p->getTypeId() . "-" . trim($p->getDescription()) . "-" . $pos_attr_val . "-" . $dest_attr_val;
			
			if (!isset($aggregated_payments[$key])) {
				$aggregated_payments[$key] = array(
					'name' => $p->getDescription(),
					'position' => $pos_attr_val,
					'expense_id' => $p->getExpenseId(),
					'type_id' => $p->getTypeId(),
					'payment_num' => $p->getPaymentNum(),
					'currency' => $p->getCurrency(),
					$status->getCode() => 0,
				);
			}
			$aggregated_payments[$key][$status->getCode()] = $aggregated_payments[$key][$status->getCode()] + $p->getAmount();
		}
		
		return $aggregated_payments;
	}
	
}