<?php
	Env::useHelper('format');

	function sponsors_report_print_group($group_obj, $grouped_objects, $options, $skip_groups = array(), $level = 0, $more_info, $prev = "", &$crumbJs = "", &$totals=array()) {
		
		$margin_left = 15 * $level;
		$cls_suffix = $level > 2 ? "all" : $level;
		$next_level = $level + 1;
		
		$group_name = $group_obj['group']['name'];
		//echo '<div style="margin-left:' . $margin_left.'px;" class="report-group-heading-'.$cls_suffix.'">' . $group_name . '</div>';
		
		echo '<tr><td colspan="'.count($options['ordered_columns']).'" style="padding-left:' . $margin_left.'px;" class="report-group-heading-'.$cls_suffix.'">' . $group_name . '</td></tr>';
		
		$mem_index = $prev . $group_obj['group']['id'];
		
		
		// draw the table for the values
		if (isset($grouped_objects[$mem_index]) && count($grouped_objects[$mem_index]) > 0) {
			$tmp_totals = null;
			sponsors_report_print_table($grouped_objects[$mem_index], $margin_left, $options, $group_name, $more_info, $crumbJs, $tmp_totals, $prev.$group_obj['group']['id']);
			
			foreach ($more_info['status_info'] as $st) {
				if ($st['only_for_expenses']) continue;
				if (!isset($totals[$st['code']])) $totals[$st['code']] = 0;
				$totals[$st['code']] += array_var($tmp_totals, $st['code']);
			}
		}
		
		if (!is_array($group_obj['subgroups'])) return;
		
		$subgroups = order_groups_by_name($group_obj['subgroups']);
		foreach ($subgroups as $subgroup) {
			$tmp_totals = null;
			sponsors_report_print_group($subgroup, $grouped_objects, $options, array(), $next_level, $more_info, $prev . $group_obj['group']['id'] . "_", $crumbJs, $tmp_totals);
			
			foreach ($more_info['status_info'] as $st) {
				if ($st['only_for_expenses']) continue;
				if (!isset($totals[$st['code']])) $totals[$st['code']] = 0;
				$totals[$st['code']] += array_var($tmp_totals, $st['code']);
			}
		}
		
		// TODO: Fix this using the expense currency, for now only one currency will be used
		$c = Currencies::instance()->findOne();
		$c_sym = $c instanceof Currency ? $c->getSymbol() : '$';
		
		$status_codes = array();
		foreach ($more_info['status_info'] as $st) {
			if ($st['only_for_expenses']) continue;
			$status_codes[] = $st['code'];
		}
		
		echo '<tr>';
		
		$first = true;
		foreach ($options['ordered_columns'] as $colinfo) {
			if ($first) {
				echo '<td style="padding-left:'.$margin_left.'px; font-size:13px;" class="bold">'.lang('total').' '.$group_name.'</td>';
				
			} else {
				if (in_array($colinfo['id'], $status_codes)) {
					echo '<td class="right bold" style="min-width:100px;max-width:100px;padding:2px 4px; font-size:13px;">'.format_money_amount($totals[$colinfo['id']], $c_sym).'</td>';
				} else {
					echo '<td></td>';
				}
			}
			$first = false;
		}
		
		echo '</tr>';
	}
	
	
	function sponsors_report_print_table($objects, $left, $options, $group_name, $more_info, &$crumbJs = "", &$table_totals, $group_obj_id) {
		
		$alt_cls = "";
		$table_totals = array();
		
		$status_codes = array();
		foreach ($more_info['status_info'] as $st) {
			if ($st['only_for_expenses']) continue;
			$status_codes[] = $st['code'];
		}
		$status = ExpensesStatuses::instance()->findById(config_option('default_payment_status'));
		$st = array('id' => $status->getId(), 'code' => $status->getCode());
		
		$payments = $more_info['payments'];
		$ordered_payments = array();
		
		$type_order = $more_info['type_order'];
		
		if (array_var($options, 'aggregate_payment_types_of_the_same_key')) {
			foreach ($payments[$group_obj_id] as $key => $p) {
				$t_order = array_search($p['type_id'], $type_order);
				$ordered_payments[$t_order ."_". $key] = $p;
				
				if (!isset($table_totals[$status->getCode()])) $table_totals[$status->getCode()] = 0;
				$table_totals[$status->getCode()] += $p[$status->getCode()];
			}
		} else {
			foreach ($objects as $exp) {
				foreach ($payments[$exp->getId()][$st['code']] as $p) {/* @var $p PaymentHistory */
					$k = $p->getTypeId()."_".strtolower($p->getDescription())."_".$p->getExpenseId()."_".$p->getPaymentNum()."_".rand(0, 1000);
					$t_order = array_search($p->getTypeId(), $type_order);
					
					$ordered_payments[$t_order ."_". $k] = array(
						'name' => $p->getDescription(),
						'expense_id' => $p->getExpenseId(),
						'type_id' => $p->getTypeId(),
						'payment_num' => $p->getPaymentNum(),
						'currency' => $p->getCurrency(),
						$status->getCode() => $p->getAmount(),
					);
					
					if (!isset($table_totals[$status->getCode()])) $table_totals[$status->getCode()] = 0;
					$table_totals[$status->getCode()] += $p->getAmount();
				}
			}
		}
		
		ksort($ordered_payments);
		
		sponsors_report_print_all_payments($ordered_payments, $left, $options, $more_info, $alt_cls, $crumbJs, $group_obj_id);
		
	}
	
	
	function sponsors_report_print_all_payments($payments, $margin_left, $options, $more_info, &$alt_cls='', &$crumbJs='', $group_obj_id) {
		$status_codes = array();
		foreach ($more_info['status_info'] as $st) {
			if ($st['only_for_expenses']) continue;
			$status_codes[] = $st['code'];
		}
		
		$this_expense_related_to = "";
		$this_expense_cps = array();
		
		$status = ExpensesStatuses::instance()->findById(config_option('default_payment_status'));
		$st = array('id' => $status->getId(), 'code' => $status->getCode());
		$p_type_id = array_var($options, 'payment_type', 0);
		
		foreach ($payments as $key => $p) {
	
			if ($p_type_id > 0 && $p['type_id'] != $p_type_id) {
				continue;
			}
			$exp = Expenses::instance()->findById($p['expense_id']);
				
			$currency = Currencies::instance()->getCurrency($p['currency']);
			echo '<tr '.$alt_cls.' style="border-top:1px dotted #ddd;">';
			
			foreach ($options['ordered_columns'] as $colinfo) {
				switch ($colinfo['id']) {
	
					case 'name':
						echo '<td>'. $p['name'] .'</td>';
						break;
	
					case 'related_to':
						$gid = gen_id();
						echo '<td style="width:200px;"><span id="t-'.$exp->getId().'-'. $gid.'" style="font-weight:normal;"></span></td>';
						$crumbOptions = $exp->getMembersToDisplayPath();
						$crumbJs .= escape_character("var crumbHtml = og.getEmptyCrumbHtml(".json_encode($crumbOptions).");");
						$crumbJs .= 'document.getElementById("t-'. $exp->getId() .'-'. $gid .'").innerHTML = crumbHtml;';
	
						break;
	
					case 'payment_type':
						$pt = PaymentTypes::getById($p['type_id']);
						echo '<td>'. ($pt instanceof PaymentType ? $pt->getName() : '') .'</td>';
						break;
	
					case 'position':
						$pt = PaymentTypes::getById($p['type_id']);
						if ($pt instanceof PaymentType) {
							$pt_att = PaymentTypeAttributes::instance()->findOne(array('conditions' => "payment_type_id=".$pt->getId()." AND name='Position'"));
							if ($pt_att instanceof PaymentTypeAttribute) {
	
								$pt_att_val = PaymentAttributeValues::instance()->findOne(array(
										'conditions' => "expense_id=".$exp->getId()." AND attribute_id=".$pt_att->getAttributeId()." AND payment_num=".$p['payment_num']." AND type_id=".$pt->getId()
								));
	
								echo '<td>'. ($pt_att_val instanceof PaymentAttributeValue ? $pt_att_val->getValue() : '') .'</td>';
	
							} else {
								echo '<td></td>';
							}
						} else {
							echo '<td></td>';
						}
						break;
	
	
					case 'destination':
						$pt = PaymentTypes::getById($p['type_id']);
						if ($pt instanceof PaymentType) {
							$pt_att = PaymentTypeAttributes::instance()->findOne(array('conditions' => "payment_type_id=".$pt->getId()." AND name='Destination'"));
							if ($pt_att instanceof PaymentTypeAttribute) {
	
								$pt_att_val = PaymentAttributeValues::instance()->findOne(array(
										'conditions' => "expense_id=".$exp->getId()." AND attribute_id=".$pt_att->getAttributeId()." AND payment_num=".$p['payment_num']." AND type_id=".$pt->getId()
								));
	
								echo '<td>'. ($pt_att_val instanceof PaymentAttributeValue ? $pt_att_val->getValue() : '') .'</td>';
	
							} else {
								echo '<td></td>';
							}
						} else {
							echo '<td></td>';
						}
						break;
	
	
					default:
						if (in_array($colinfo['id'], $status_codes)) {
	
							if ($colinfo['id'] == $st['code']) {
								echo '<td class="right" style="width:100px;">'.format_money_amount($p[$st['code']], $currency->getSymbol()) .'</td>';
							} else {
								echo '<td class="right" style="width:100px;">'.format_money_amount(0, $currency->getSymbol()) .'</td>';
							}
	
						} else if (str_starts_with($colinfo['id'], "cp_")) {
	
							$cp_id = str_replace("cp_", "", $colinfo['id']);
							$cp_val = "";
							$cp = CustomProperties::instance()->findById($cp_id);
							if ($cp instanceof CustomProperty) {
								$cp_val = get_custom_property_value_for_listing($cp, $exp);
							}
							echo "<td>".$cp_val.'</td>';
	
							$this_expense_cps[$colinfo['id']] = $cp_val;
	
						}
						break;
				}
			}
				
			echo "</tr>";
			$alt_cls = $alt_cls == "" ? 'class="alt-row"' : "";
		}
	}
	
	

	$date_format = user_config_option('date_format');

	if (array_var($post, 'due_date_type') == 6) {
		if ($start_time instanceof DateTimeValue) $start_time->advance(-1 * logged_user()->getUserTimezoneValue(), true);
		if ($end_time instanceof DateTimeValue) $end_time->advance(-1 * logged_user()->getUserTimezoneValue(), true);
	}
	
	if (isset($start_time) && $start_time instanceof DateTimeValue) { ?>
		<span class="bold"><?php echo lang('from')?></span>:&nbsp;<?php echo $start_time->format($date_format) ?>
	<?php }
	if (isset($end_time) && $end_time instanceof DateTimeValue) { ?>
		<span class="bold" style="padding-left:10px"><?php echo lang('to date')?></span>:&nbsp;<?php echo $end_time->format($date_format) ?>
	<?php } ?>
	
	<?php if (count(active_context_members(false)) > 0) : ?>
	<div class="clear"></div>
	<div style="margin-bottom: 10px; padding-bottom: 5px; float:left;">
		<h5><?php echo lang('showing information for')?>:</h5>
		<ul>
		<?php
			$context = active_context();
			foreach ($context as $selection) :
				if ($selection instanceof Member) : ?>
					<li><span class="coViewAction <?php echo $selection->getIconClass()?>"><?php echo $selection->getName()?></span></li>
		<?php 	endif;
			endforeach;
		?>
		</ul>
	</div>
	<div class="clear"></div>
	<?php endif; ?>
	
	<div class="tasks-report-container expenses-report-container">
	<table style="width:100%; margin-top:5px; border-top: 1px dotted #ccc;" class="reporting-table">
	<?php
	$crumbJs = "";
	
	$options = array_var($_SESSION, 'sponsors_report_data', array());
	
	$type_order_array = array(2,1,6,3,5,4);
	
	$more_info = array(
		'status_info' => $status_info,
		'payments' => $payments,
		'totals' => $totals,
		'type_order' => $type_order_array,
	);
	
	$all_totals = array('budgeted' => 0, 'executed' => 0);
	
	// headers
	echo '<tr class="reporting-table-heading">';
	$status_codes = array();
	foreach ($more_info['status_info'] as $st) {
		if ($st['only_for_expenses']) continue;
		$status_codes[] = $st['code'];
	}
	
	foreach ($options['ordered_columns'] as $colinfo) {
		if (in_array($colinfo['id'], $status_codes)) {
			echo '<th class="right">' . $colinfo['name'] . '</th>';
		} else {
			echo '<th>' . $colinfo['name'] . '</th>';
		}
	}
	echo '</tr>';
	
	$groups = order_groups_by_name($grouped_expenses['groups']);
	foreach ($groups as $gid => $group_obj) {
		$tmp_tot = array();
		foreach ($status_info as $st) {
			if ($st['only_for_expenses']) continue;
			$tmp_tot[$st['code']] = 0;
		}
		
		sponsors_report_print_group($group_obj, $grouped_expenses['grouped_objects'], $options, array(), 0, $more_info, "", $crumbJs, $tmp_tot);
		foreach ($status_info as $st) {
			if ($st['only_for_expenses']) continue;
			if (!isset($all_totals[$st['code']])) $all_totals[$st['code']] = 0;
			$all_totals[$st['code']] += array_var($tmp_tot, $st['code']);
		}
	}
	
	if (count($groups) > 1) {
		// TODO: Fix this using the expense currency, for now only one currency will be used
		$c = Currencies::instance()->findOne();
		$c_sym = $c instanceof Currency ? $c->getSymbol() : '$';
		
		echo '<tr>';
		
		$first = true;
		foreach ($options['ordered_columns'] as $colinfo) {
			if ($first) {
				echo '<td style="padding-left:0px; border-top:1px solid #999; font-size:15px;" class="bold">'.lang('total').'</td>';
		
			} else {
				if (in_array($colinfo['id'], $status_codes)) {
					echo '<td class="right bold" style="min-width:100px;max-width:100px;padding:2px 4px; border-top:1px solid #999; font-size:15px;">'. format_money_amount(array_var($all_totals, $colinfo['id']), $c_sym) .'</td>';
				} else {
					echo '<td id="'.$colinfo['id'].'" style="border-top:1px solid #999;"></td>';
				}
			}
			$first = false;
		}
		
		echo '</tr>';
	}
	?>
	</table>
	</div>
	<script>
	$(function() {
		eval('<?php echo $crumbJs; ?>');
		og.eventManager.fireEvent('replace all empty breadcrumb');
	});
	</script>