<?php


	Env::useHelper('format');

	function payments_report_print_group_csv($group_obj, $grouped_objects, $options, $skip_groups = array(), $level = 0, $more_info, $prev = "", &$crumbJs = "", &$totals=array()) {
		$text = "";
		
		$margin_left = 15 * $level;
		$cls_suffix = $level > 2 ? "all" : $level;
		$next_level = $level + 1;
		
		$group_name = $group_obj['group']['name'];
		
		$text .= $group_name ."\n";
		
		$mem_index = $prev . $group_obj['group']['id'];
		
		// draw the table for the values
		if (isset($grouped_objects[$mem_index]) && count($grouped_objects[$mem_index]) > 0) {
			$tmp_totals = null;
			$text .= payments_report_print_table_csv($grouped_objects[$mem_index], $margin_left, $options, $group_name, $more_info, $crumbJs, $tmp_totals);
			
			foreach ($more_info['status_info'] as $st) {
				if ($st['only_for_expenses']) continue;
				$totals[$st['code']] += $tmp_totals[$st['code']];
			}
		}
		
		if (!is_array($group_obj['subgroups'])) return;
		
		$subgroups = order_groups_by_name($group_obj['subgroups']);
		foreach ($subgroups as $subgroup) {
			$tmp_totals = null;
			$text .= payments_report_print_group_csv($subgroup, $grouped_objects, $options, array(), $next_level, $more_info, $prev . $group_obj['group']['id'] . "_", $crumbJs, $tmp_totals);
			
			foreach ($more_info['status_info'] as $st) {
				if ($st['only_for_expenses']) continue;
				$totals[$st['code']] += $tmp_totals[$st['code']];
			}
		}
		
		// TODO: Fix this using the expense currency, for now only one currency will be used
		$c = Currencies::instance()->findOne();
		$c_sym = $c instanceof Currency ? $c->getSymbol() : '$';
		
		$status_codes = array();
		foreach ($more_info['status_info'] as $st) {
			if ($st['only_for_expenses']) continue;
			$status_codes[] = $st['code'];
		}
		
		$first = true;
		foreach ($options['ordered_columns'] as $colinfo) {
			if ($first) {
				$text .= lang('total').' '.$group_name.";";
			} else {
				if (in_array($colinfo['id'], $status_codes)) {
					$text .= format_money_amount($totals[$colinfo['id']], $c_sym).";";
				} else {
					$text .= ";";
				}
			}
			$first = false;
		}
		
		$text .= "\n\n";
		
		return $text;
	}
	
	
	function payments_report_print_table_csv($objects, $left, $options, $group_name, $more_info, &$crumbJs = "", &$table_totals) {
		$text = "";
		
		$alt_cls = "";
		$table_totals = array();
		foreach ($objects as $e) {
			$text .= payments_report_print_expense_csv($e, $left, $options, $more_info, $alt_cls, $crumbJs);
			foreach ($more_info['status_info'] as $st) {
				if ($st['only_for_expenses']) continue;
				if (!isset($table_totals[$st['code']])) $table_totals[$st['code']] = 0;
				$table_totals[$st['code']] += array_var($more_info['totals'][$e->getId()], $st['code']);
			}
		}
		
		return $text;
	}
	
	
	function payments_report_print_expense_csv($exp, $margin_left, $options, $more_info, &$alt_cls='', &$crumbJs='') {
		
		$text = "";
		
		$status_codes = array();
		foreach ($more_info['status_info'] as $st) {
			if ($st['only_for_expenses']) continue;
			$status_codes[] = $st['code'];
		}
		
		$this_expense_related_to = "";
		$this_expense_cps = array();
		
		foreach ($options['ordered_columns'] as $colinfo) {
			switch ($colinfo['id']) {
				
				case 'name':
					$text .= clean($exp->getObjectName()) . ";";
					break;
					
				case 'related_to':
					$mem_text = "";
					$members = $exp->getMembers();
					foreach ($members as $m) {
						if (!$m->getDimension()->getIsManageable()) continue;
						$sep = ' / ';
						$path = $m->getPath($sep);
						$path .= ($path == "" ? "" : $sep) . $m->getName();
						$mem_text .= ($mem_text == "" ? "" : ", ") . $path;
					}
					$this_expense_related_to = $mem_text;
					$text .= $mem_text . ";";
					break;
				
				case 'payment_type':
				case 'payment_type_details':
					$text .= ';';
					break;
					
				case 'date':
					$text .= (config_option("use_date_in_expense_level") ? format_date($exp->getDate()) : ""). ";";
					break;
					
				case 'description':
					$text .= clean(html_to_text($exp->getDescription())) . ";";
					break;
					
				case 'status':
					$st = null;
					$status_info = $more_info['status_info'];
					foreach ($status_info as $status) {
						if ($status['id'] == $exp->getStatus()) {
							$st = $status;
							break;
						}
					}
					$text .= $st['name'] . ";";
					break;
					
				default:
					if (in_array($colinfo['id'], $status_codes)) {
						
						// TODO: Fix this using the expense currency, for now only one currency will be used
						$c = Currencies::instance()->findOne();
						$c_sym = $c instanceof Currency ? $c->getSymbol() : '$';
						
						$totals = $more_info['totals'];
						$text .= format_money_amount(array_var($totals[$exp->getId()], $colinfo['id'], 0), $c_sym) . ";";
						
					} else if (str_starts_with($colinfo['id'], "cp_")) {
						// Custom properties
						
						$cp_id = str_replace("cp_", "", $colinfo['id']);
						$cp_val = "";
						$cp = CustomProperties::instance()->findById($cp_id);
						if ($cp instanceof CustomProperty) {
							$cp_val = str_replace(array("\r\n", "\n"), " ", get_custom_property_value_for_listing($cp, $exp));
						}
						
						$this_expense_cps[$colinfo['id']] = $cp_val;
						
						$text .= $cp_val . ";";
						
					}
					break;
			}
		}
		$text .= "\n";
		
		$payments = $more_info['payments'];
		
		foreach ($payments[$exp->getId()][$st['code']] as $p) {
			/* @var $p Payment */
			$currency = Currencies::instance()->getCurrency($p->getCurrency());
			
			foreach ($options['ordered_columns'] as $colinfo) {
				switch ($colinfo['id']) {
		
				case 'name':
					if ($options['print_expense_props_in_each_payment']) {
						$text .= clean($exp->getObjectName()) . ";";
					} else {
						$text .= ";";
					}
					break;
						
				case 'related_to':
					if ($options['print_expense_props_in_each_payment']) {
						$text .= $this_expense_related_to;
					}
					$text .= ";";
					break;
				
				case 'date':
					$text .= (config_option("use_date_in_expense_level") ? "" : format_date($p->getDate())). ";";
					break;
				
				case 'description':
					
					$text .= str_replace(array("\r\n", "\n"), " ", $p->getDescription()) . ";";
					break;
				
				case 'payment_type':
					$pt = PaymentTypes::getById($p->getTypeId());
					$text .= ($pt instanceof PaymentType ? $pt->getName() : '').";";
					break;
						
				case 'payment_type_details':
					$pt = PaymentTypes::getById($p->getTypeId());
					$text .= ($pt instanceof PaymentType ? $p->getAttributesString(", ", 'text') : '') .';';
					break;
				
				case 'status':
					$st = null;
					$status_info = $more_info['status_info'];
					foreach ($status_info as $status) {
						if ($status['id'] == $exp->getStatus()) {
							$st = $status;
							break;
						}
					}
					$text .= $st['name'] . ";";
					break;
						
				default:
					if (in_array($colinfo['id'], $status_codes)) {

						if ($colinfo['id'] == $st['code']) {
							$text .= format_money_amount($p->getAmount(), $currency->getSymbol()) . ";";
						} else {
							$text .= format_money_amount(0, $currency->getSymbol()) . ";";
						}
						
					} else if (str_starts_with($colinfo['id'], "cp_")) {
						if ($options['print_expense_props_in_each_payment']) {
							$text .= array_var($this_expense_cps, $colinfo['id'], '');
						}
						$text .= ";";
					}
					break;
				}
			}

			$text .= "\n";
		}
		
		return $text;
		
	}
	

	$text = "";
	$options = array_var($_SESSION, 'payments_by_state_data', array());
	
	$more_info = array(
		'status_info' => $status_info,
		'payments' => $payments,
		'totals' => $totals,
	);
	
	$all_totals = array('budgeted' => 0, 'executed' => 0);
	
	
	// headers
	$status_codes = array();
	foreach ($more_info['status_info'] as $st) {
		if ($st['only_for_expenses']) continue;
		$status_codes[] = $st['code'];
	}
	
	foreach ($options['ordered_columns'] as $colinfo) {
		$text .= $colinfo['name'] . ";";
	}
	$text .= "\n";
	
	$groups = order_groups_by_name($grouped_expenses['groups']);
	foreach ($groups as $gid => $group_obj) {
		$tmp_tot = array();
		foreach ($status_info as $st) {
			if ($st['only_for_expenses']) continue;
			$tmp_tot[$st['code']] = 0;
		}
		
		$text .= payments_report_print_group_csv($group_obj, $grouped_expenses['grouped_objects'], $options, array(), 0, $more_info, "", $crumbJs, $tmp_tot);
		foreach ($status_info as $st) {
			if ($st['only_for_expenses']) continue;
			if (!isset($all_totals[$st['code']])) $all_totals[$st['code']] = 0;
			$all_totals[$st['code']] += array_var($tmp_tot, $st['code']);
		}
	}
	
	if (count($groups) > 1) {
		// TODO: Fix this using the expense currency, for now only one currency will be used
		$c = Currencies::instance()->findOne();
		$c_sym = $c instanceof Currency ? $c->getSymbol() : '$';
		
		$first = true;
		foreach ($options['ordered_columns'] as $colinfo) {
			if ($first) {
				$text .= lang('total') . ";";
			} else {
				if (in_array($colinfo['id'], $status_codes)) {
					$text .= format_money_amount(array_var($all_totals, $colinfo['id']), $c_sym) .";";
				} else {
					$text .= ";";
				}
			}
			$first = false;
		}
		
		$text .= "\n";
	}
	
	file_put_contents($tmp_filename, $text);
	die();
	
	//return $text;
