<?php
/**
 * NewsTypeController controller
 *
 * @version 1.0
 * @author Alvaro Torterola <alvaro.torterola@fengoffice.com>
 */
class NewsTypeController extends ApplicationController {

	var $plugin_name = "work_time_registration";
	
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');
		Env::useHelper('functions', $this->plugin_name);
	}
	
	
	function list_all() {
		
		$groups = NewsTypes::instance()->findAll(array('order' => 'description'));
		
		tpl_assign("groups", $groups);
		
	}
	
	
	
	private function save_news_type($group_data, $group = null) {
		$is_new = false;
		if (is_null($group) || !$group instanceof NewsType) {
			$group = new NewsType();
			$is_new = true;
		}
		
		$group->setFromAttributes($group_data);
		$group->save();
		
		return $group;
	}
	
	
	function add() {
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		
		$group_data = array_var($_POST, 'type');
		
		if(!is_array($group_data)) {
			$g = new NewsType();
			$g->setEnabled(true);
			$g->setDaysToGenerateAbscence(0);
			$g->setGeneratesAbscence(false);
			tpl_assign('object', $g);
		} else {
			// form submitted
			try {
				DB::beginWork();
				$g = $this->save_news_type($group_data);
				DB::commit();

				ApplicationLogs::createLog($g, ApplicationLogs::ACTION_ADD);
				flash_success(lang('success add new news type', $g->getDescription()));
				ajx_current("back");
	          	
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				//flash_error($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	function edit() {
		$this->setTemplate('add');
		
		$group = NewsTypes::instance()->findById(get_id());
		if(!($group instanceof NewsType)) {
			flash_error(lang('news type dnx'));
			ajx_current("empty");
			return;
		}
		
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		$group_data = array_var($_POST, 'type');
		
		if(!is_array($group_data)) {
			tpl_assign('object', $group);
		} else {
			// form submitted
			try {
				DB::beginWork();
				$g = $this->save_news_type($group_data, $group);
				DB::commit();

				ApplicationLogs::createLog($g, ApplicationLogs::ACTION_ADD);
				flash_success(lang('success edit news type', $g->getDescription()));
				ajx_current("back");
	          	
			} catch(Exception $e) {
				DB::rollback();
				flash_error($e->getMessage());
				//flash_error($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
				ajx_current("empty");
			}
		}
	}
	
	
	function delete() {
		
		$group = NewsTypes::instance()->findById(get_id());
		if(!($group instanceof NewsType)) {
			flash_error(lang('news type dnx'));
			ajx_current("empty");
			return;
		}
		
		if (!$group->canBeDeleted()) {
			flash_error(lang('news type cannot be deleted'));
			ajx_current("empty");
			return;
		}
	
		if (logged_user()->isGuest()) {
			flash_error(lang('no access permissions'));
			ajx_current('empty');
			return;
		}
		
		try {
			DB::beginWork();
			$group->delete();
			DB::commit();

			ApplicationLogs::createLog($g, ApplicationLogs::ACTION_ADD);
			flash_success(lang('success delete news type', $group->getDescription()));
			ajx_current("reload");
          	
		} catch(Exception $e) {
			DB::rollback();
			flash_error($e->getMessage());
			//flash_error($e->getMessage()."\n".get_back_trace()."\n".$e->getTraceAsString());
			ajx_current("empty");
		}
	}
}