<?php
	class NewsDetail extends BaseNewsDetail {
		
		protected $searchable_columns = array('name', 'date');
		
		function buildObjectName() {
			$c = Contacts::instance()->findById($this->getPersonId());
			if ($c instanceof Contact) {
				$this->getObject()->setName(lang('news detail for x in y', str_replace("'", "\'", $c->getSurname().", ".$c->getFirstName()), format_date($this->getDate(), null, 0)));
			}
		}
		
		function getViewUrl() {
			return get_url('time_registration', 'view', array('id' => $this->getId()));
		}
		
		function getArrayInfo($clean = false) {
			$info = array();
			
			$columns = $this->getColumns();
			foreach ($columns as $col) {
				$val = $this->getColumnValue($col);
				if ($clean && is_string($val)) $val = clean($val);
				
				$info[$col] = $val;
			}
		
			return $info;
		}
		
		
		function addToHistory($user, $action=null, $log_data='') {
			
			$history = new NewsDetailsHistory();
			
			$cols = $this->getColumns();
			
			foreach ($cols as $col) {
				if ($history->columnExists($col)) $history->setColumnValue($col, $this->getColumnValue($col));
			}
			
			if (is_null($action)) $action = 'edit';
			
			$history->setModificatedById($user->getId());
			$history->setModificatedOn(DateTimeValueLib::now());
			$history->setAction($action);
			$history->setModificationLog($log_data);
			
			$history->save();
		}
		
		
		function copy() {
			$new_det = new NewsDetail();
			
			$cols = $this->getColumns();
			foreach ($cols as $col) {
				if ($col == 'object_id') continue;
				$new_det->setColumnValue($col, $this->getColumnValue($col));
			}
			
			return $new_det;
		}
		
		
		
		static function canAdd(Contact $user, $context, &$notAllowedMember = ''){
			return can_add($user, $context, NewsDetails::instance()->getObjectTypeId(), $notAllowedMember);
		}
		
		/**
		 * Returns true if $user can access this message
		 *
		 * @param Contact $user
		 * @return boolean
		 */
		function canView(Contact $user) {
			return can_read($user, $this->getMembers(), $this->getObjectTypeId());
		} // canView
	
		/**
		 * Check if specific user can edit this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canEdit(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canEdit
	
	
		/**
		 * Check if specific user can delete this messages
		 *
		 * @access public
		 * @param Contact $user
		 * @return boolean
		 */
		function canDelete(Contact $user) {
			return can_delete($user,$this->getMembers(), $this->getObjectTypeId());
		} // canDelete
	
		/**
		 * Check if specific user can comment this message
		 *
		 * @access public
		 * @param void
		 * @return boolean
		 */
		function canAddComment(Contact $user) {
			return can_write($user, $this->getMembers(), $this->getObjectTypeId());
		} // canAddComment
	}